/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: bmfpga.h,v 1.4 2002/11/05 00:20:36 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _BMFPGA_H
#define _BMFPGA_H

#define BMFPGA_NIM_SLOTS	10

#ifdef LANGUAGE_C
typedef struct bmfpga_slot_struct {
	uint32	control;
	uint32	control_exp;
	uchar8		res1[0x10-0x08];
	uint32	status;
	uint32	status_exp;
	uchar8		res2[0x20-0x18];
	uint32	power_out;
	uint32	power_in;
	uchar8		res3[0x30-0x28];
	uint32	message_out;
	uint32	message_in;
	uchar8		res4[0x40-0x38];
} bmfpga_slot_t;

typedef struct bmfpga_struct {
	uint32			board_status;
	uint32			module_presence;
	uchar8				res1[0x10-0x08];
	uint32			gpp;
	uint32			gpp_direction;
	uchar8				res2[0x20-0x18];
	bmfpga_slot_t	nim_slot[BMFPGA_NIM_SLOTS];
	uchar8				res3[0x200-0x150];
	uint32			test_register;
	uchar8				res4[0x300-0x204];
	uint32			warm_reboot_register;
} bmfpga_master_t;

#ifdef SIBYTE
/*
 * 0x0 = board status
 * 0xc = version register
 * 0x10 = status to SSC
 * 0x14 = expansion status
 * 0x18 = control from SSC
 * 0x1C = expansion control
 * 0x20 = power to SSC
 * 0x24 = power from SSC
 * 0x28 = message out
 * 0x2C = message in
 */
typedef struct bmfpga_slave_struct {
	uint32		board_status;
	uchar8			res1[0xC-0x04];
	uint32		version;
	uint32		status;
	uint32		status_exp;
	uint32		control;
	uint32		control_exp;
	uint32		power_out;
	uint32		power_in;
	uint32		message_out;
	uint32		message_in;
} bmfpga_slave_t;
#else
typedef struct bmfpga_slave_struct {
	uint32		board_status;
	uchar8			res1[0x10-0x04];
	uint32		gpp;
	uint32		gpp_direction;
	uchar8			res2[0x20-0x18];
	uint32		status;
	uint32		status_exp;
	uchar8			res3[0x30-0x28];
	uint32		control;
	uint32		control_exp;
	uchar8			res4[0x40-0x38];
	uint32		power_out;
	uint32		power_in;
	uchar8			res5[0x50-0x48];
	uint32		message_out;
	uint32		message_in;
} bmfpga_slave_t;
#endif

#endif

#define BMFPGA_REG_BOARD_STATUS_OFF	0x00
#define BMFPGA_REG_GPP_OFF			0x14


/* for FC cards */
#define BMFPGA_REG_GPP_CTL_VAL_OFF		0x18
#define BMFPGA_REG_GPP_CTL_DIR_OFF		0x1C

/*------------------------------------------------------------------
 * Board Status
 *------------------------------------------------------------------*/
#ifndef R9000
#define BMFPGA_BOARD_STATUS_SLOT_MASK				0x0000000F
#define BMFPGA_BOARD_STATUS_SLOT_SHIFT				0
#define BMFPGA_BOARD_STATUS_CHASSIS_ID_MASK			0x00000070
#define BMFPGA_BOARD_STATUS_CHASSIS_ID_SHIT			4
#define BMFPGA_BOARD_STATUS_POWER_STATUS_MASK		0x00000100
#define BMFPGA_BOARD_STATUS_POWER_STATUS_SHIFT		8
#define BMFPGA_BOARD_STATUS_SSC_ACTIVE_MASK			0x00000200
#define BMFPGA_BOARD_STATUS_SSC_ACTIVE_SHIFT		9
#define BMFPGA_BOARD_STATUS_BACKUP_SSC_ACTIVE_MASK	0x00000400
#define BMFPGA_BOARD_STATUS_BACKUP_SSC_ACTIVE_SHIFT	10
#define BMFPGA_BOARD_STATUS_PROM_SEL_MASK			0x00001000
#define BMFPGA_BOARD_STATUS_PROM_SEL_SHIFT			12
#define BMFPGA_BOARD_STATUS_DIAG_SEL_MASK			0x00002000
#define BMFPGA_BOARD_STATUS_DIAG_SEL_SHIFT			13
#define BMFPGA_BOARD_STATUS_MEM_TYPE_MASK			0x00004000
#define BMFPGA_BOARD_STATUS_MEM_TYPE_SHIFT			14
#define BMFPGA_BOARD_STATUS_MEM_TYPE_UNREG			0
#define BMFPGA_BOARD_STATUS_MEM_TYPE_REG			1
#define BMFPGA_BOARD_STATUS_CLK_MODE_MASK			0x00008000
#define BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT			15
#define BMFPGA_BOARD_STATUS_CLK_MODE_100MHz			0
#define BMFPGA_BOARD_STATUS_CLK_MODE_125MHz			1
#define BMFPGA_BOARD_STATUS_VERSION_MASK			0xFFFF0000
#define BMFPGA_BOARD_STATUS_VERSION_SHIFT			16
#else
/*
 * BMFPGA Register Map on Bobcat
 */
#define BMFPGA_REG_BOARD_STATUS						0x00
#define BMFPGA_REG_STATUS_CTRL						0x04
#define BMFPGA_REG_GPR								0x08
#define BMFPGA_REG_TEST								0x0C
#define BMFPGA_REG_FP_CTRL							0x10
#define BMFPGA_REG_FP_CTRL_EXP						0x14
#define BMFPGA_REG_FP_STATUS						0x18
#define BMFPGA_REG_FP_STATUS_EXP					0x1c
#define BMFPGA_REG_FP_PWR_OUT						0x20
#define BMFPGA_REG_FP_PWR_IN						0x24
#define BMFPGA_REG_FP_MBOX_MSG_OUT					0x28
#define BMFPGA_REG_FP_MBOX_MSG_IN					0x2C

#define BMFPGA_REG_FC0_CTRL							0x30
#define BMFPGA_REG_FC0_CTRL_EXP						0x34
#define BMFPGA_REG_FC0_STATUS						0x38
#define BMFPGA_REG_FC0_STATUS_EXP					0x3c
#define BMFPGA_REG_FC0_PWR_OUT						0x40
#define BMFPGA_REG_FC0_PWR_IN						0x44
#define BMFPGA_REG_FC0_MBOX_MSG_OUT					0x48
#define BMFPGA_REG_FC0_MBOX_MSG_IN					0x4C

#define BMFPGA_REG_FC1_CTRL							0x50
#define BMFPGA_REG_FC1_CTRL_EXP						0x54
#define BMFPGA_REG_FC1_STATUS						0x58
#define BMFPGA_REG_FC1_STATUS_EXP					0x5c
#define BMFPGA_REG_FC1_PWR_OUT						0x60
#define BMFPGA_REG_FC1_PWR_IN						0x64
#define BMFPGA_REG_FC1_MBOX_MSG_OUT					0x68
#define BMFPGA_REG_FC1_MBOX_MSG_IN					0x6C

/*
 * BMFPGA Board Status Register Bits
 */
#define BMFPGA_BOARD_STATUS_DIAG_SEL_MASK			0x00000001
#define BMFPGA_BOARD_STATUS_DIAG_SEL_SHIFT			0
#define BMFPGA_BOARD_STATUS_PROM_SEL_MASK			0x00000002
#define BMFPGA_BOARD_STATUS_PROM_SEL_SHIFT			1
#define BMFPGA_BOARD_STATUS_BOARD_REV				0x00004000
#define BMFPGA_BOARD_STATUS_BOARD_REV_SHIFT			14
#define BMFPGA_BOARD_STATUS_CLK_MODE_MASK			0x00008000
#define BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT			15
#define BMFPGA_BOARD_STATUS_CLK_MODE_100MHz			0
#define BMFPGA_BOARD_STATUS_CLK_MODE_133MHz			1
#define BMFPGA_BOARD_STATUS_VERSION_MASK			0xFFFF0000
#define BMFPGA_BOARD_STATUS_VERSION_SHIFT			16

/*
 * BMFPGA Status & Control Register Bits
 */
#define BMFPGA_STATUS_CTRL_FP_STAT_INT				0x00000001
#define BMFPGA_STATUS_CTRL_FP_MSG_INT				0x00000002
#define BMFPGA_STATUS_CTRL_FC_STAT_INT				0x00000004
#define BMFPGA_STATUS_CTRL_FC_MSG_INT				0x00000008
#define BMFPGA_STATUS_CTRL_PS0_OK					0x00000010
#define BMFPGA_STATUS_CTRL_PS0_GOOD					0x00000020
#define BMFPGA_STATUS_CTRL_PS1_OK					0x00000040
#define BMFPGA_STATUS_CTRL_PS1_GOOD					0x00000080
#define BMFPGA_STATUS_CTRL_FAN0_STATUS				0x00000100
#define BMFPGA_STATUS_CTRL_FAN1_STATUS				0x00000200
#define BMFPGA_STATUS_CTRL_FAN_CTRL_ON 				0x00010000

/*
 * BMFPGA General Purpose Register Bits
 */
#define BMFPGA_GPR_TXRX_COLD_RESET					0x00000001
#define BMFPGA_GPR_TXRX_WARM_RESET					0x00000002
#define BMFPGA_GPR_FP_COLD_RESET					0x00000004
#define BMFPGA_GPR_FP_WARM_RESET					0x00000008
#define BMFPGA_GPR_LSI_MAC_RESET					0x00000010
#define BMFPGA_GPR_LUC_RESET						0x00000020
#define BMFPGA_GPR_PCI0_RESET						0x00000040
#define BMFPGA_GPR_PCI1_RESET						0x00000080
#define BMFPGA_GPR_SEC_PCI1_RESET					0x00000100
#define BMFPGA_GPR_SERDES_RESET						0x00000200
#define BMFPGA_GPR_FP_PORT_MASK						0x0000F000
#define BMFPGA_GPR_FP_PORT0_ENA						0x00001000
#define BMFPGA_GPR_FP_PORT1_ENA						0x00002000
#define BMFPGA_GPR_FP_PORT2_ENA						0x00004000
#define BMFPGA_GPR_FP_PORT3_ENA						0x00008000


#endif

/*------------------------------------------------------------------
 * Module Presence and Interrupt
 *------------------------------------------------------------------*/
#define BMFPGA_MOD_PRESENT_BIT						0x00000004
#define BMFPGA_STAT_INT_PENDING_MASK				0x00001000
#define BMFPGA_MSG_INT_PENDING_MASK					0x00002000

/*------------------------------------------------------------------
 * Master GPP bits
 *------------------------------------------------------------------*/
#define BMFPGA_MASTER_GPP_FAN0_ALARM_MASK			0x00000003
#define BMFPGA_MASTER_GPP_FAN1_ALARM_MASK			0x00000018
#define BMFPGA_MASTER_GPP_FAN0_PRESENT_MASK			0x00004000
#define BMFPGA_MASTER_GPP_FAN1_PRESENT_MASK			0x00008000

/*------------------------------------------------------------------
 * Slot Control
 *------------------------------------------------------------------*/
#define BMFPGA_SLOT_CONTROL_SLICE_RESET_MASK		0x00000003
#define BMFPGA_SLOT_CONTROL_SLICE_COLD_START_MASK	0x00000001
#define BMFPGA_SLOT_CONTROL_SLICE_WARM_START_MASK	0x00000002
#define BMFPGA_SLOT_CONTROL_SLOT_COLD_START_MASK	0x00000055
#define BMFPGA_SLOT_CONTROL_SLOT_WARM_START_MASK	0x000000FF
#define BMFPGA_SLOT_CONTROL_GLOBAL_ENABLE			0x80000000

/*------------------------------------------------------------------
 * Slot Status
 *------------------------------------------------------------------*/
#define BMFPGA_SLOT_STATUS_SLICE_STATE_MASK			0x0000000F
#define BMFPGA_SLOT_STATUS_SLICE0123_STATE_MASK		0x0000FFFF
#define BMFPGA_SLOT_STATUS_SLICE_STATE_PROM_INIT	\
												CM_SLICE_STATE_PROM_INIT
#define BMFPGA_SLOT_STATUS_SLICE_STATE_PROM_NO_AUTOBOOT	\
												CM_SLICE_STATE_PROM_NO_AUTOBOOT		
#define BMFPGA_SLOT_STATUS_SLICE_STATE_PROM_CRASHED	\
												CM_SLICE_STATE_PROM_CRASHED			
#define BMFPGA_SLOT_STATUS_SLICE_STATE_PROM_LOADING	\
												CM_SLICE_STATE_PROM_LOADING			
#define BMFPGA_SLOT_STATUS_SLICE_STATE_SWITCH_TO_RUNTIME	\
												CM_SLICE_STATE_SWITCH_TO_RUNTIME	
#define BMFPGA_SLOT_STATUS_SLICE_STATE_RUNTIME_INIT	\
												CM_SLICE_STATE_RUNTIME_INIT			
#define BMFPGA_SLOT_STATUS_SLICE_STATE_RUNTIME	CM_SLICE_STATE_RUNTIME
#define BMFPGA_SLOT_STATUS_SLICE_STATE_RUNTIME_CRASHED	\
												CM_SLICE_STATE_RUNTIME_CRASHED
#define BMFPGA_SLOT_STATUS_SLICE_STATE_RUNTIME_REBOOT	\
												CM_SLICE_STATE_RUNTIME_REBOOT
#define BMFPGA_SLOT_STATUS_BOARD_TYPE_MASK			0x000F0000
#define BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT			16
#define BMFPGA_SLOT_STATUS_BOARD_TYPE_SSC			CM_CARD_TYPE_SSC
#define BMFPGA_SLOT_STATUS_BOARD_TYPE_GARFIELD		CM_CARD_TYPE_GARFIELD
#define BMFPGA_SLOT_STATUS_BOARD_TYPE_CHEETAH		CM_CARD_TYPE_CHEETAH
#define BMFPGA_SLOT_STATUS_BOARD_TYPE_FCNIM			CM_CARD_TYPE_FCNIM
#define BMFPGA_SLOT_STATUS_TEMP_MASK				0x0FF00000
#define BMFPGA_SLOT_STATUS_TEMP_SHIFT				20
#if 0
#define BMFPGA_SLOT_STATUS_TEMP_NORMAL				0
#define BMFPGA_SLOT_STATUS_TEMP_BELOW_NORMAL		1
#define BMFPGA_SLOT_STATUS_TEMP_ABOVE_NORMAL		2
#define BMFPGA_SLOT_STATUS_TEMP_CRITICAL			3
#endif

/*------------------------------------------------------------------
 * Message
 *------------------------------------------------------------------*/
#define BMFPGA_OWN	0x80000000

#ifdef R7000
/*------------------------------------------------------------------
 *
 * CTRL
 * 12:11	Controls frontpanel Fault LED (?sl0_fp_led[3:2]")
 *          00 = off, 01 = Red on, 10 = Green on, 11 = both Red & Green on
 * 10:9	    Controls frontpanel OK LED (?sl0_fp_led[1:0]")
 *          00 = off, 01 = Red on, 10 = Green on, 11 = both Red & Green on
 *------------------------------------------------------------------*/
#define BMFPGA_CTRL_FP_LED_FLT_MASK         0x00001800 
#define BMFPGA_CTRL_FP_LED_FLT_OFF          0x00000000 
#define BMFPGA_CTRL_FP_LED_FLT_RED          0x00000800 
#define BMFPGA_CTRL_FP_LED_FLT_GREEN        0x00001000 
#define BMFPGA_CTRL_FP_LED_OK_MASK          0x00000600 
#define BMFPGA_CTRL_FP_LED_OK_OFF           0x00000000 
#define BMFPGA_CTRL_FP_LED_OK_RED           0x00000200 
#define BMFPGA_CTRL_FP_LED_OK_GREEN         0x00000400 

#endif /* R7000 */

/*------------------------------------------------------------------
 * Function Prototypes
 *------------------------------------------------------------------*/
#ifdef LANGUAGE_C
#ifndef SSC
void    bmfpga_init(void);
void    bmfpga_setCardTemp(uint32 temp);
void    bmfpga_setCardPower(uint32 power);
void    bmfpga_setCardType(uint32	type);
void    bmfpga_initCardType(uint32	type);
void    bmfpgaSetCPUState(uint32	state);
void    bmfpga_nmi_clear(uint32 val);
void    bmfpga_sendMessage(
	uint32	dest_slot,
	uint32	dest_cpu,
	uint32	* msg,
	uint32	num_bytes
);
uint32  bmfpga_get_busclock(void);
#ifdef SSC_MGMT
uint32  bmfpgaGetNimSlot(uint32	slot);
void    bmfpgaResetCard(uint32 slot);
void    bmfpgaResetCPU(uint32 slot, uint32 cpu);
int     bmfpgaIsModulePresent(int32	slot);
void    bmfpga_clearPendingInts(void);
#endif  /* SSC_MGMT */
#endif  /* !SSC */
#endif  /* LANGUAGE_C */
#endif	/* _BMFPGA_H */

