/*
 *-----------------------------------------------------------------
 *
 * Name:        gt64440r.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Internal register definition file for GT64440 from 
 *              Galileo, based on mv64340reg.h which is supposed to
 *              be register compatible.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 03/18/04
 *
 *-----------------------------------------------------------------
 */

/*
 * Copyright (c) 2001-2002 Galileo Technology.
 * Copyright (c) 2002 Momentum Computer.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Galileo Technology.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _GT64440REG_H_
#define _GT64440REG_H_

#ifdef __MIPSEB__
#define HTOLE32(v)      ((((v) & 0xff) << 24) | (((v) & 0xff00) << 8) | \
                        (((v) >> 24) & 0xff) | (((v) >> 8) & 0xff00))
#else
#ifdef __MIPSEL__
#define HTOLE32(v)      (v)
#else
#error ENDIAN NOT DEFINED!
#endif
#endif

#ifdef PROM_SIBYTE_BC	
#define GT_WRITE(ofs, data) \
    *(volatile uint32 *)(GT_BASE_ADDR_FROM_TXRX+ofs) = HTOLE32(data)

#define GT_READ(ofs) \
    HTOLE32(*(volatile uint32 *)(GT_BASE_ADDR_FROM_TXRX+ofs))

#else

#define GT_WRITE(ofs, data) \
    *(volatile uint32 *)(GT_BASE_ADDR+ofs) = HTOLE32(data)

#define GT_READ(ofs) \
    HTOLE32(*(volatile uint32 *)(GT_BASE_ADDR+ofs))
#endif

#ifdef EVAL_BOARD
#define GT_BASE_ADDR			PHYS_TO_IOSPACE(0x14000000)
#else
#define GT_BASE_ADDR			PHYS_TO_K1(HW_GALILEO_BASE)
#endif
#define GT_BASE_ADDR_DEFAULT		PHYS_TO_K1(0x14000000)


/************************************************/
#define GT_DEVPAR_TurnOff(x)            ((((x) & 7) << 0) | (((x) & 8) << 19))
#define GT_DEVPAR_AccToFirst(x)         ((((x) & 15) << 3) | (((x) & 16) << 19))
#define GT_DEVPAR_AccToNext(x)          ((((x) & 15) << 7) | (((x) & 16) << 20))
#define GT_DEVPAR_ALEtoWr(x)            ((((x) & 7) << 11) | (((x) & 8) << 22))
#define GT_DEVPAR_WrActive(x)           ((((x) & 7) << 14) | (((x) & 8) << 23))
#define GT_DEVPAR_WrHigh(x)             ((((x) & 7) << 17) | (((x) & 8) << 24))
#define GT_DEVPAR_DevWidth8             (0<<20)
#define GT_DEVPAR_DevWidth16            (1<<20)
#define GT_DEVPAR_DevWidth32            (2<<20)
#define	GT_DEVPAR_DevWidthMASK		(3<<20)
#define GT_DEVPAR_BaddrSkew0            (0<<28)
#define GT_DEVPAR_BaddrSkew1            (1<<28)
#define GT_DEVPAR_BaddrSkew2            (2<<28)
#define GT_DEVPAR_BaddrSkewMASK         (3<<28)
#define GT_DEVPAR_DPEnDisable           (0<<30)
#define GT_DEVPAR_DPEnEnable            (1<<30)
#define GT_DEVPAR_ReservedMASK          0x80000000
#define GT_DEVPAR_Reserved              0x80000000 


/************************************************/
#define GT_PCI0_MAP10                   0x0010
#define GT_PCI0_MAP14                   0x0014
#define GT_PCI1_MAP10                   0x0090
#define GT_PCI1_MAP14                   0x0094
/************************************************/
#define GT_IPCI_CFGADDR_ConfigEn        (1<<31)
/************************************************/
#define GT_SMIR_REG			0x2010
#define CPU_CONF        		0x0000
#define SDRAM_CNFG      		0x0448
#define SDRAM_PARA0     		0x044c
#define SDRAM_PARA1     		0x0450
#define SDRAM_PARA2     		0x0454
#define SDRAM_PARA3     		0x0458
#define GT_DEV0_PAR     		0x045c
#define GT_DEV1_PAR     		0x0460
#define GT_DEV2_PAR     		0x0464
#define GT_DEV3_PAR     		0x0468
#define GT_BOOT_PAR     		0x046c
#define PCI0_COMMAND    		0x0c00
#define PCI1_COMMAND    		0x0c80
#define PCI0_IO_LO      		0x0048
#define PCI0_IO_HI      		0x0050
#define PCI0_MEM0_LO    		0x0058
#define PCI0_MEM0_HI    		0x0060
#define PCI1_IO_LO     			0x0090
#define PCI1_IO_HI      		0x0098
#define PCI1_MEM0_LO    		0x00a0
#define PCI1_MEM0_HI    		0x00a8
#define CPU_CONFIG      		0x0000
#define SCS0_LOW 	     		0x0008
#define SCS0_HIGH 	    		0x0010
#define SCS2_LOW 	     		0x0018
#define SCS2_HIGH 	    		0x0020
#define SCS1_LOW        		0x0208
#define SCS1_HIGH       		0x0210
#define SCS3_LOW        		0x0218
#define SCS3_HIGH       		0x0220
#define PCI_1_IO_LOW    		0x0090
#define PCI_1_IO_HIGH   		0x0098
#define PCI1_MEM0_LOW   		0x00a0
#define PCI1_MEM0_HIGH  		0x00a8
#define PCI_1_MEM1_LOW  		0x00b0
#define PCI_1_MEM1_HIGH 		0x00b8
#define PCI_0_TIME_OUT  		0x0c04
#define PCI_1_TIME_OUT  		0x0c84
#define PCI_0_BAR_EN    		0x0c3c
#define PCI_1_BAR_EN    		0x0cbc
#define PCI_0_ARBITER   		0x1d00
#define PCI_1_ARBITER   		0x1d80
#define MPP_CNTRL0      		0xf000
#define MPP_CNTRL1      		0xf004
#define MPP_CNTRL2      		0xf008
#define MPP_CNTRL3      		0xf00c
#define SER_PORTS_MUX   		0xf010

/****************************************/
/* Processor Address Space		*/
/****************************************/


/* BAR Enables */
#define CPU_BASE_ADDRESS_ENABLE				0x278

/* Sdram's BAR'S */
#define SCS_0_BASE_ADDRESS				0x008
#define SCS_0_SIZE					0x010
#define SCS_1_BASE_ADDRESS				0x208
#define SCS_1_SIZE					0x210
#define SCS_2_BASE_ADDRESS				0x018
#define SCS_2_SIZE					0x020
#define SCS_3_BASE_ADDRESS				0x218
#define SCS_3_SIZE					0x220

/* Devices BAR'S */
#define CS_0_BASE_ADDRESS				0x028
#define CS_0_SIZE					0x030
#define CS_1_BASE_ADDRESS			    	0x228
#define CS_1_SIZE					0x230
#define CS_2_BASE_ADDRESS			    	0x248
#define CS_2_SIZE					0x250
#define CS_3_BASE_ADDRESS				0x038
#define CS_3_SIZE					0x040
#define BOOTCS_BASE_ADDRESS				0x238
#define BOOTCS_SIZE					0x240

#define PCI_0I_O_BASE_ADDRESS				0x048
#define PCI_0I_O_SIZE					0x050
#define PCI_0MEMORY0_BASE_ADDRESS			0x058
#define PCI_0MEMORY0_SIZE				0x060
#define PCI_0MEMORY1_BASE_ADDRESS			0x080
#define PCI_0MEMORY1_SIZE				0x088
#define PCI_0MEMORY2_BASE_ADDRESS			0x258
#define PCI_0MEMORY2_SIZE				0x260
#define PCI_0MEMORY3_BASE_ADDRESS			0x280
#define PCI_0MEMORY3_SIZE				0x288

#define PCI_1I_O_BASE_ADDRESS				0x090
#define PCI_1I_O_SIZE					0x098
#define PCI_1MEMORY0_BASE_ADDRESS			0x0a0
#define PCI_1MEMORY0_SIZE				0x0a8
#define PCI_1MEMORY1_BASE_ADDRESS			0x0b0
#define PCI_1MEMORY1_SIZE				0x0b8
#define PCI_1MEMORY2_BASE_ADDRESS			0x2a0
#define PCI_1MEMORY2_SIZE				0x2a8
#define PCI_1MEMORY3_BASE_ADDRESS			0x2b0
#define PCI_1MEMORY3_SIZE				0x2b8

#define INTERNAL_SPACE_BASE				0x068
#define INTERNAL_SRAM_BASE				0x268

#define PCI_0I_O_ADDRESS_REMAP				0x0f0
#define PCI_0MEMORY0_ADDRESS_REMAP  			0x0f8
#define PCI_0MEMORY0_HIGH_ADDRESS_REMAP			0x320
#define PCI_0MEMORY1_ADDRESS_REMAP  			0x100
#define PCI_0MEMORY1_HIGH_ADDRESS_REMAP			0x328
#define PCI_0MEMORY2_ADDRESS_REMAP  			0x2f8
#define PCI_0MEMORY2_HIGH_ADDRESS_REMAP			0x330
#define PCI_0MEMORY3_ADDRESS_REMAP  			0x300
#define PCI_0MEMORY3_HIGH_ADDRESS_REMAP			0x338

#define PCI_1I_O_ADDRESS_REMAP				0x108
#define PCI_1MEMORY0_ADDRESS_REMAP  			0x110
#define PCI_1MEMORY0_HIGH_ADDRESS_REMAP			0x340
#define PCI_1MEMORY1_ADDRESS_REMAP  			0x118
#define PCI_1MEMORY1_HIGH_ADDRESS_REMAP			0x348
#define PCI_1MEMORY2_ADDRESS_REMAP  			0x310
#define PCI_1MEMORY2_HIGH_ADDRESS_REMAP			0x350
#define PCI_1MEMORY3_ADDRESS_REMAP  			0x318
#define PCI_1MEMORY3_HIGH_ADDRESS_REMAP			0x358


/****************************************/
/* CPU Sync Barrier             	*/
/****************************************/

#define PCI_0SYNC_BARIER_VIRTUAL_REGISTER		0x0c0
#define PCI_1SYNC_BARIER_VIRTUAL_REGISTER		0x0c8


/****************************************/
/* CPU Access Protect             	*/
/****************************************/

#define CPU_LOW_PROTECT_ADDRESS_0                     	0x180
#define CPU_HIGH_PROTECT_ADDRESS_0                      0x188
#define CPU_LOW_PROTECT_ADDRESS_1                       0x190
#define CPU_HIGH_PROTECT_ADDRESS_1                      0x198
#define CPU_LOW_PROTECT_ADDRESS_2                       0x1a0
#define CPU_HIGH_PROTECT_ADDRESS_2                      0x1a8
#define CPU_LOW_PROTECT_ADDRESS_3                       0x1b0
#define CPU_HIGH_PROTECT_ADDRESS_3                      0x1b8

/****************************************/
/*          CPU Error Report       	*/
/****************************************/

#define CPU_BUS_ERROR_LOW_ADDRESS 			0x070
#define CPU_BUS_ERROR_HIGH_ADDRESS 			0x078
#define CPU_BUS_ERROR_LOW_DATA                          0x128
#define CPU_BUS_ERROR_HIGH_DATA                         0x130
#define CPU_BUS_ERROR_LOW_PARITY                        0x138
#define CPU_BUS_ERROR_HIGH_PARITY                       0x140
#define CPU_BUS_ERROR_MASK                              0x148

/****************************************/
/*          Pslave Debug           	*/
/****************************************/

#define X_0_ADDRESS                                   	0x360
#define X_0_COMMAND_ID                                  0x368
#define X_1_ADDRESS                                     0x370
#define X_1_COMMAND_ID                                  0x378
#define WRITE_DATA_LOW                                  0x3c0
#define WRITE_DATA_HIGH                                 0x3c8
#define WRITE_BYTE_ENABLE                               0X3e0
#define READ_DATA_LOW                                   0x3d0
#define READ_DATA_HIGH                                  0x3d8
#define READ_ID                                         0x3e8


/****************************************/
/* Integrated SRAM Register Map         */
/****************************************/
#define SRAM_CONFIGURATION                              0x380
#define SRAM_ERROR_CAUSE                                0x388
#define SRAM_ERROR_MASK                                 0x2d8
#define SRAM_ERROR_ADDRESS_LOW                          0x390
#define SRAM_ERROR_ADDRESS_HIGH                         0x3f8
#define SRAM_ERROR_DATA_LOW                             0x398
#define SRAM_ERROR_DATA_HIGH                            0x3a0
#define SRAM_ERROR_ECC                                  0x3a8
#define SRAM_CALCULATED_ECC                             0x308
#define SRAM_ECC_SINGLE_BIT_ERR_COUNTER                 0x2e0
#define SRAM_ECC_DOUBLE_BIT_ERR_COUNTER                 0x2e8
	
/****************************************/
/* SDRAM and Device Address Space	*/
/****************************************/
	

/****************************************/
/* SDRAM Configuration			*/
/****************************************/

#define SDRAM_CONFIGURATION	 			0x1400

#define SDRAM_CFG_REFRESH_INTERVAL			(0x0400 << 0)
#define SDRAM_CFG_FBSPLIT_DISABLE			(1 << 14)
#define SDRAM_CFG_FBVM_DISABLE				(1 << 15)
#define SDRAM_CFG_TYPE_SDRAM				(0 << 16)
#define SDRAM_CFG_TYPE_FCRAM				(1 << 16)
#define SDRAM_CFG_REGISTERED				(1 << 17)
#define SDRAM_CFG_ECC_DISABLED				(0 << 18)
#define SDRAM_CFG_ECC_ENABLED				(1 << 18)
#define SDRAM_CFG_ECC_REPORTED				(0 << 19)
#define SDRAM_CFG_ECC_IGNORED				(1 << 19)
#define SDRAM_CFG_DQS_X32_DEV				(0 << 20)
#define SDRAM_CFG_DQS_X8_DEV				(2 << 20)
#define SDRAM_CFG_DQS_X4_DEV 				(3 << 20)
#define SDRAM_CFG_RFMODE_1 				(0 << 22)
#define SDRAM_CFG_RFMODE_4 				(1 << 22)
#define SDRAM_CFG_RFMODE_8 				(2 << 22)
#define SDRAM_CFG_SRMODE_EXIT_ON_RST			(0 << 24)
#define SDRAM_CFG_SRMODE_EXIT_ON_ACCESS			(1 << 24)
#define SDRAM_CFG_RDBUFF_0_PCI0				(0 << 27)
#define SDRAM_CFG_RDBUFF_1_PCI0				(1 << 27)
#define SDRAM_CFG_RDBUFF_0_PCI1				(0 << 28)
#define SDRAM_CFG_RDBUFF_1_PCI1				(1 << 28)
#define SDRAM_CFG_RDBUFF_0_MPSC				(0 << 29)
#define SDRAM_CFG_RDBUFF_1_MPSC				(1 << 29)
#define SDRAM_CFG_RDBUFF_0_IDMA				(0 << 30)
#define SDRAM_CFG_RDBUFF_1_IDMA				(1 << 30)
#define SDRAM_CFG_RDBUFF_0_GBE				(0 << 31)
#define SDRAM_CFG_RDBUFF_1_GBE				(1 << 31)

/****************************************/
/* SDRAM Mode         			*/
/****************************************/

#define SDRAM_MODE					0x141c

#define SDRAM_MODE_BL_4					(2 << 0)
#define SDRAM_MODE_CL_2 				(2 << 4)
#define SDRAM_MODE_CL_3 				(3 << 4)
#define SDRAM_MODE_CL_4 				(4 << 4)
#define SDRAM_MODE_CL_15				(5 << 4)
#define SDRAM_MODE_CL_25				(6 << 4)

#define DUNIT_CONTROL_LOW				0x1404
#define DUNIT_CONTROL_HIGH				0x1424
#define SDRAM_TIMING_LOW				0x1408
#define SDRAM_TIMING_HIGH				0x140c
#define SDRAM_OPEN_PAGES_CONTROL			0x1414


/****************************************/
/* SDRAM Address Control                */
/****************************************/

#define SDRAM_ADDRESS_CONTROL				0x1410

#define SDRAM_ADDR_CTRL_ADDR_SEL_2			(2 << 0)
#define SDRAM_ADDR_CTRL_DCFG_256M  			(0 << 4)
#define SDRAM_ADDR_CTRL_DCFG_512M_OR_1G 		(1 << 4)
#define SDRAM_ADDR_CTRL_DCFG_2G   			(2 << 4)

/****************************************/
/* SDRAM Operation    			*/
/****************************************/

#define SDRAM_OPERATION					0x1418

#define SDRAM_OP_CMD_NORMAL				0
#define SDRAM_OP_CMD_PRECHARGE				1
#define SDRAM_OP_CMD_REFRESH				2
#define SDRAM_OP_CMD_SET_MODE				3
#define SDRAM_OP_CMD_SET_XMODE				4
#define SDRAM_OP_CMD_NOP				5
#define SDRAM_OP_CMD_SELF_REFRESH			7


#define SDRAM_XTENDED_MODE				0x1420
#define SDRAM_CROSSBAR_CONTROL_LOW			0x1430
#define SDRAM_CROSSBAR_CONTROL_HIGH			0x1434
#define SDRAM_CROSSBAR_TIMEOUT				0x1438
#define SDRAM_ADDRESS_PADS_CALIBRATION			0x14c0
#define SDRAM_DATA_PADS_CALIBRATION			0x14c4

#define DFCDL_CONFIGURATION0				0x1480
#define DFCDL_CONFIGURATION1				0x1484
#define SRAM_ADDRESS					0x1490
#define SRAM_DATA0					0x1494
#define DFCDL_PROBE					0x14a0

/****************************************/
/* SDRAM Error Report 			*/
/****************************************/
			
#define DUNIT_INTERRUPT_CAUSE                           0x14D0
#define DUNIT_INTERRUPT_MASK                            0x14D4
#define SDRAM_ERROR_DATA_LOW                            0x1444
#define SDRAM_ERROR_DATA_HIGH                           0x1440
#define SDRAM_RECEIVED_ECC                              0x1448
#define SDRAM_CALCULATED_ECC                            0x144c
#define SDRAM_AND_DEVICE_ERROR_ADDRESS                  0x1450
#define SDRAM_ECC_CONTROL                               0x1454
#define SDRAM_ECC_ERROR_COUNTER                         0x1458
#define SDRAM_CS0_SINGLE_BIT_ERROR_COUNTER              0x1458
#define SDRAM_CS0_DOUBLE_BIT_ERROR_COUNTER              0x145c
#define SDRAM_CS1_SINGLE_BIT_ERROR_COUNTER              0x1460
#define SDRAM_CS1_DOUBLE_BIT_ERROR_COUNTER              0x1464
#define SDRAM_CS2_SINGLE_BIT_ERROR_COUNTER              0x1468
#define SDRAM_CS2_DOUBLE_BIT_ERROR_COUNTER              0x146c
#define SDRAM_CS3_SINGLE_BIT_ERROR_COUNTER              0x1470
#define SDRAM_CS3_DOUBLE_BIT_ERROR_COUNTER              0x1474

/****************************************/
/* DUNIT interrupt bit definitions      */
/****************************************/

#define DUNIT_INTERRUPT_SBIT_ERR                        0x00000001
#define DUNIT_INTERRUPT_DBIT_ERR                        0x00000002
#define DUNIT_INTERRUPT_DP_ERR                          0x00000004


/****************************************/
/* Device Parameters			*/
/****************************************/


#define DEVICE_BANK0PARAMETERS				0x45c
#define DEVICE_BANK1PARAMETERS				0x460
#define DEVICE_BANK2PARAMETERS				0x464
#define DEVICE_BANK3PARAMETERS				0x468
#define DEVICE_BOOT_BANK_PARAMETERS			0x46c
#define DEVICE_CONTROL                                  0x4c0
#define DEVICE_CROSS_BAR_CONTROL_LOW                    0x4c8
#define DEVICE_CROSS_BAR_CONTROL_HIGH                   0x4cc
#define DEVICE_CROSS_BAR_TIMEOUT                        0x4c4

/****************************************/
/* Device Interrupt 			*/
/****************************************/

#define DEVICE_INTERRUPT_CAUSE                          0x4d0
#define DEVICE_INTERRUPT_MASK                           0x4d4
#define DEVICE_ERROR_ADDRESS                            0x4d8
#define DEVICE_ERROR_DATA                               0x4dc
#define DEVICE_ERROR_PARITY                             0x4e0

/****************************************/
/* DMA Record				*/
/****************************************/

#define CHANNEL0_DMA_BYTE_COUNT				0x800
#define CHANNEL1_DMA_BYTE_COUNT	 			0x804
#define CHANNEL2_DMA_BYTE_COUNT	 			0x808
#define CHANNEL3_DMA_BYTE_COUNT	 			0x80C
#define CHANNEL0_DMA_SOURCE_ADDRESS			0x810
#define CHANNEL1_DMA_SOURCE_ADDRESS			0x814
#define CHANNEL2_DMA_SOURCE_ADDRESS			0x818
#define CHANNEL3_DMA_SOURCE_ADDRESS			0x81C
#define CHANNEL0_DMA_DESTINATION_ADDRESS		0x820
#define CHANNEL1_DMA_DESTINATION_ADDRESS		0x824
#define CHANNEL2_DMA_DESTINATION_ADDRESS		0x828
#define CHANNEL3_DMA_DESTINATION_ADDRESS		0x82C
#define CHANNEL0NEXT_RECORD_POINTER			0x830
#define CHANNEL1NEXT_RECORD_POINTER			0x834
#define CHANNEL2NEXT_RECORD_POINTER			0x838
#define CHANNEL3NEXT_RECORD_POINTER			0x83C
#define CHANNEL0CURRENT_DESCRIPTOR_POINTER		0x870
#define CHANNEL1CURRENT_DESCRIPTOR_POINTER		0x874
#define CHANNEL2CURRENT_DESCRIPTOR_POINTER		0x878
#define CHANNEL3CURRENT_DESCRIPTOR_POINTER		0x87C

/****************************************/
/* DMA Channel Control			*/
/****************************************/

#define CHANNEL0CONTROL 				0x840
#define CHANNEL0CONTROL_HIGH				0x880
#define CHANNEL1CONTROL 				0x844
#define CHANNEL1CONTROL_HIGH				0x884
#define CHANNEL2CONTROL 				0x848
#define CHANNEL2CONTROL_HIGH				0x888
#define CHANNEL3CONTROL 				0x84C
#define CHANNEL3CONTROL_HIGH				0x88C


/****************************************/
/* DMA Interrupt			*/
/****************************************/

#define CHANELS0_3_INTERRUPT_CAUSE                      0x8c0
#define CHANELS0_3_INTERRUPT_MASK                       0x8c4
#define CHANELS0_3_ERROR_ADDRESS                        0x8c8
#define CHANELS0_3_ERROR_SELECT                         0x8cc


/****************************************/
/* DMA Debug (for internal use)         */
/****************************************/

#define DMA_X0_ADDRESS                                  0x8e0
#define DMA_X0_COMMAND_AND_ID                           0x8e4
#define DMA_X0_WRITE_DATA_LOW                           0x8e8
#define DMA_X0_WRITE_DATA_HIGH                          0x8ec
#define DMA_X0_WRITE_BYTE_ENABLE                        0x8f8
#define DMA_X0_READ_DATA_LOW                            0x8f0
#define DMA_X0_READ_DATA_HIGH                           0x8f4
#define DMA_X0_READ_ID                                  0x8fc
#define DMA_X1_ADDRESS                                  0x9e0
#define DMA_X1_COMMAND_AND_ID                           0x9e4
#define DMA_X1_WRITE_DATA_LOW                           0x9e8
#define DMA_X1_WRITE_DATA_HIGH                          0x9ec
#define DMA_X1_WRITE_BYTE_ENABLE                        0x9f8
#define DMA_X1_READ_DATA_LOW                            0x9f0
#define DMA_X1_READ_DATA_HIGH                           0x9f4
#define DMA_X1_READ_ID                                  0x9fc

/****************************************/
/* Timer_Counter 			*/
/****************************************/

#define TIMER_COUNTER0					0x850
#define TIMER_COUNTER1					0x854
#define TIMER_COUNTER2					0x858
#define TIMER_COUNTER3					0x85C
#define TIMER_COUNTER_0_3_CONTROL			0x864
#define TIMER_COUNTER_0_3_INTERRUPT_CAUSE		0x868
#define TIMER_COUNTER_0_3_INTERRUPT_MASK      		0x86c

/****************************************/
/* PCI Slave Address Decoding           */
/****************************************/

        
#define PCI_0SCS_0_BANK_SIZE				0xc08
#define PCI_1SCS_0_BANK_SIZE				0xc88
#define PCI_0SCS_1_BANK_SIZE				0xd08
#define PCI_1SCS_1_BANK_SIZE				0xd88
#define PCI_0SCS_2_BANK_SIZE				0xc0c
#define PCI_1SCS_2_BANK_SIZE				0xc8c
#define PCI_0SCS_3_BANK_SIZE				0xd0c
#define PCI_1SCS_3_BANK_SIZE				0xd8c
#define PCI_0CS_0_BANK_SIZE				0xc10
#define PCI_1CS_0_BANK_SIZE				0xc90
#define PCI_0CS_1_BANK_SIZE				0xd10
#define PCI_1CS_1_BANK_SIZE				0xd90
#define PCI_0CS_2_BANK_SIZE				0xd18
#define PCI_1CS_2_BANK_SIZE				0xd98
#define PCI_0CS_3_BANK_SIZE				0xc14
#define PCI_1CS_3_BANK_SIZE				0xc94
#define PCI_0CS_BOOT_BANK_SIZE				0xd14
#define PCI_1CS_BOOT_BANK_SIZE				0xd94
#define PCI_0P2P_MEM0_BAR_SIZE                          0xd1c
#define PCI_1P2P_MEM0_BAR_SIZE                          0xd9c
#define PCI_0P2P_MEM1_BAR_SIZE                          0xd20
#define PCI_1P2P_MEM1_BAR_SIZE                          0xda0
#define PCI_0P2P_I_O_BAR_SIZE                           0xd24
#define PCI_1P2P_I_O_BAR_SIZE                           0xda4
#define PCI_0INTERNAL_SRAM_BAR_SIZE                     0xe00
#define PCI_1INTERNAL_SRAM_BAR_SIZE                     0xe80
#define PCI_0EXPANSION_ROM_BAR_SIZE                     0xd2c
#define PCI_1EXPANSION_ROM_BAR_SIZE                     0xdac
#define PCI_0BASE_ADDRESS_REGISTERS_ENABLE 		0xc3c
#define PCI_1BASE_ADDRESS_REGISTERS_ENABLE 		0xcbc
#define PCI_0SCS_0_BASE_ADDRESS_REMAP			0xc48
#define PCI_1SCS_0_BASE_ADDRESS_REMAP			0xcc8
#define PCI_0SCS_1_BASE_ADDRESS_REMAP			0xd48
#define PCI_1SCS_1_BASE_ADDRESS_REMAP			0xdc8
#define PCI_0SCS_2_BASE_ADDRESS_REMAP			0xc4c
#define PCI_1SCS_2_BASE_ADDRESS_REMAP			0xccc
#define PCI_0SCS_3_BASE_ADDRESS_REMAP			0xd4c
#define PCI_1SCS_3_BASE_ADDRESS_REMAP			0xdcc
#define PCI_0CS_0_BASE_ADDRESS_REMAP			0xc50
#define PCI_1CS_0_BASE_ADDRESS_REMAP			0xcd0
#define PCI_0CS_1_BASE_ADDRESS_REMAP			0xd50
#define PCI_1CS_1_BASE_ADDRESS_REMAP			0xdd0
#define PCI_0CS_2_BASE_ADDRESS_REMAP			0xd58
#define PCI_1CS_2_BASE_ADDRESS_REMAP			0xdd8
#define PCI_0CS_3_BASE_ADDRESS_REMAP           		0xc54
#define PCI_1CS_3_BASE_ADDRESS_REMAP           		0xcd4
#define PCI_0CS_BOOTCS_BASE_ADDRESS_REMAP      		0xd54
#define PCI_1CS_BOOTCS_BASE_ADDRESS_REMAP      		0xdd4
#define PCI_0P2P_MEM0_BASE_ADDRESS_REMAP_LOW            0xd5c
#define PCI_1P2P_MEM0_BASE_ADDRESS_REMAP_LOW            0xddc
#define PCI_0P2P_MEM0_BASE_ADDRESS_REMAP_HIGH           0xd60
#define PCI_1P2P_MEM0_BASE_ADDRESS_REMAP_HIGH           0xde0
#define PCI_0P2P_MEM1_BASE_ADDRESS_REMAP_LOW            0xd64
#define PCI_1P2P_MEM1_BASE_ADDRESS_REMAP_LOW            0xde4
#define PCI_0P2P_MEM1_BASE_ADDRESS_REMAP_HIGH           0xd68
#define PCI_1P2P_MEM1_BASE_ADDRESS_REMAP_HIGH           0xde8
#define PCI_0P2P_I_O_BASE_ADDRESS_REMAP                 0xd6c
#define PCI_1P2P_I_O_BASE_ADDRESS_REMAP                 0xdec
#define PCI_0INTERNAL_SRAM_BASE_ADDRESS_REMAP		    0xf00
#define PCI_1INTERNAL_SRAM_BASE_ADDRESS_REMAP		    0xf80
#define PCI_0EXPANSION_ROM_BASE_ADDRESS_REMAP               0xf38
#define PCI_1EXPANSION_ROM_BASE_ADDRESS_REMAP               0xfb8
#define PCI_0ADDRESS_DECODE_CONTROL                         0xd3c
#define PCI_1ADDRESS_DECODE_CONTROL                         0xdbc

/****************************************/
/* PCI Control                          */
/****************************************/

#define PCI0_DLL_STATUS_CONTROL					0x1d20
#define PCI1_DLL_STATUS_CONTROL					0x1da0
#define PCI0_PADS_CALIBRATION					0x1d1c
#define PCI1_PADS_CALIBRATION					0x1d9c

#define PCI_0COMMAND					    0xc00
#define PCI_1COMMAND					    0xc80
#define PCI_0MODE                                           0xd00
#define PCI_1MODE                                           0xd80
#define PCI_0TIMEOUT_RETRY				    0xc04
#define PCI_1TIMEOUT_RETRY				    0xc84
#define PCI_0READ_BUFFER_DISCARD_TIMER                      0xd04
#define PCI_1READ_BUFFER_DISCARD_TIMER                      0xd84
#define MSI_0TRIGGER_TIMER                                  0xc38
#define MSI_1TRIGGER_TIMER                                  0xcb8
#define PCI_0ARBITER_CONTROL                                0x1d00
#define PCI_1ARBITER_CONTROL                                0x1d80

/* changed until here */
#define PCI_0CROSS_BAR_CONTROL_LOW                           0x1d08
#define PCI_0CROSS_BAR_CONTROL_HIGH                          0x1d0c
#define PCI_0CROSS_BAR_TIMEOUT                               0x1d04
#define PCI_0SYNC_BARRIER_TRIGGER                            0x1d18
#define PCI_0SYNC_BARRIER_VIRTUAL_REGISTER                   0x1d10
#define PCI_0P2P_CONFIGURATION                               0x1d14
#define PCI_0ACCESS_CONTROL_BASE_0_LOW                       0x1e00
#define PCI_0ACCESS_CONTROL_BASE_0_HIGH                      0x1e04
#define PCI_0ACCESS_CONTROL_SIZE_0                           0x1e08
#define PCI_0ACCESS_CONTROL_BASE_1_LOW                       0c1e10
#define PCI_0ACCESS_CONTROL_BASE_1_HIGH                      0x1e14
#define PCI_0ACCESS_CONTROL_SIZE_1                           0x1e18
#define PCI_0ACCESS_CONTROL_BASE_2_LOW                       0c1e20
#define PCI_0ACCESS_CONTROL_BASE_2_HIGH                      0x1e24
#define PCI_0ACCESS_CONTROL_SIZE_2                           0x1e28
#define PCI_0ACCESS_CONTROL_BASE_3_LOW                       0c1e30
#define PCI_0ACCESS_CONTROL_BASE_3_HIGH                      0x1e34
#define PCI_0ACCESS_CONTROL_SIZE_3                           0x1e38
#define PCI_0ACCESS_CONTROL_BASE_4_LOW                       0c1e40
#define PCI_0ACCESS_CONTROL_BASE_4_HIGH                      0x1e44
#define PCI_0ACCESS_CONTROL_SIZE_4                           0x1e48
#define PCI_0ACCESS_CONTROL_BASE_5_LOW                       0c1e50
#define PCI_0ACCESS_CONTROL_BASE_5_HIGH                      0x1e54
#define PCI_0ACCESS_CONTROL_SIZE_5                           0x1e58
#define PCI_1CROSS_BAR_CONTROL_LOW                           0x1d88
#define PCI_1CROSS_BAR_CONTROL_HIGH                          0x1d8c
#define PCI_1CROSS_BAR_TIMEOUT                               0x1d84
#define PCI_1SYNC_BARRIER_TRIGGER                            0x1d98
#define PCI_1SYNC_BARRIER_VIRTUAL_REGISTER                   0x1d90
#define PCI_1P2P_CONFIGURATION                               0x1d94
#define PCI_1ACCESS_CONTROL_BASE_0_LOW                       0x1e80
#define PCI_1ACCESS_CONTROL_BASE_0_HIGH                      0x1e84
#define PCI_1ACCESS_CONTROL_SIZE_0                           0x1e88
#define PCI_1ACCESS_CONTROL_BASE_1_LOW                       0c1e90
#define PCI_1ACCESS_CONTROL_BASE_1_HIGH                      0x1e94
#define PCI_1ACCESS_CONTROL_SIZE_1                           0x1e98
#define PCI_1ACCESS_CONTROL_BASE_2_LOW                       0c1ea0
#define PCI_1ACCESS_CONTROL_BASE_2_HIGH                      0x1ea4
#define PCI_1ACCESS_CONTROL_SIZE_2                           0x1ea8
#define PCI_1ACCESS_CONTROL_BASE_3_LOW                       0c1eb0
#define PCI_1ACCESS_CONTROL_BASE_3_HIGH                      0x1eb4
#define PCI_1ACCESS_CONTROL_SIZE_3                           0x1eb8
#define PCI_1ACCESS_CONTROL_BASE_4_LOW                       0c1ec0
#define PCI_1ACCESS_CONTROL_BASE_4_HIGH                      0x1ec4
#define PCI_1ACCESS_CONTROL_SIZE_4                           0x1ec8
#define PCI_1ACCESS_CONTROL_BASE_5_LOW                       0c1ed0
#define PCI_1ACCESS_CONTROL_BASE_5_HIGH                      0x1ed4
#define PCI_1ACCESS_CONTROL_SIZE_5                           0x1ed8

/****************************************/
/* PCI Configuration Address            */
/****************************************/


#define PCI_0CONFIGURATION_ADDRESS 				0xcf8
#define PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER           	0xcfc
#define PCI_1CONFIGURATION_ADDRESS 				0xc78
#define PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER           	0xc7c
#define PCI_0INTERRUPT_ACKNOWLEDGE_VIRTUAL_REGISTER		0xc34
#define PCI_1INTERRUPT_ACKNOWLEDGE_VIRTUAL_REGISTER		0xcb4

/****************************************/
/* PCI Error Report                     */
/****************************************/


#define PCI_0SERR_MASK						0xc28
#define PCI_0ERROR_ADDRESS_LOW					0x1d40
#define PCI_0ERROR_ADDRESS_HIGH					0x1d44
#define PCI_0ERROR_ATTRIBUTE					0x1d48
#define PCI_0ERROR_COMMAND					0x1d50
#define PCI_0ERROR_CAUSE					0x1d58
#define PCI_0ERROR_MASK						0x1d5c

#define PCI_1SERR_MASK						0xca8
#define PCI_1ERROR_ADDRESS_LOW					0x1dc0
#define PCI_1ERROR_ADDRESS_HIGH					0x1dc4
#define PCI_1ERROR_ATTRIBUTE					0x1dc8
#define PCI_1ERROR_COMMAND					0x1dd0
#define PCI_1ERROR_CAUSE					0x1dd8
#define PCI_1ERROR_MASK						0x1ddc

/****************************************/
/* PCI Configuration Function 0         */
/****************************************/


#define PCI_DEVICE_AND_VENDOR_ID 				0x000
#define PCI_STATUS_AND_COMMAND					0x004
#define PCI0_COMMAND_STATUS_REG        				0x004
#define PCI1_COMMAND_STATUS_REG        				0x084
#define PCI_CLASS_CODE_AND_REVISION_ID			        0x008
#define PCI_BIST_HEADER_TYPE_LATENCY_TIMER_CACHE_LINE 		0x00C
#define PCI_SCS_0_BASE_ADDRESS_LOW    				0x010
#define PCI_SCS_0_BASE_ADDRESS_HIGH			    	0x014
#define PCI_SCS_1_BASE_ADDRESS_LOW				0x018
#define PCI_SCS_1_BASE_ADDRESS_HIGH   				0x01C
#define PCI_INTERNAL_REGISTERS_MEMORY_MAPPED_BASE_ADDRESS	0x020
#define PCI_INTERNAL_REGISTERS_I_OMAPPED_BASE_ADDRESS		0x024
#define PCI_SUBSYSTEM_ID_AND_SUBSYSTEM_VENDOR_ID		0x02C
#define PCI_EXPANSION_ROM_BASE_ADDRESS_REGISTER			0x030
#define PCI_CAPABILTY_LIST_POINTER                          	0x034
#define PCI_INTERRUPT_PIN_AND_LINE 			    	0x03C
#define PCI_POWER_MANAGEMENT_CAPABILITY                     0x040
#define PCI_POWER_MANAGEMENT_STATUS_AND_CONTROL             0x044
#define PCI_VPD_ADDRESS                                     0x048
#define PCI_VPD_DATA                                        0X04c
#define PCI_MSI_MESSAGE_CONTROL                             0x050
#define PCI_MSI_MESSAGE_ADDRESS                             0x054
#define PCI_MSI_MESSAGE_UPPER_ADDRESS                       0x058
#define PCI_MSI_MESSAGE_DATA                                0x05c
#define PCI_X_COMMAND					    0x060
#define PCI_X_STATUS					    0x064
#define PCI_COMPACT_PCI_HOT_SWAP_CAPABILITY                 0x068

/****************************************/
/* PCI Configuration Function 1         */
/****************************************/


#define PCI_SCS_2_BASE_ADDRESS_LOW    				0x110
#define PCI_SCS_2_BASE_ADDRESS_HIGH    				0x114
#define PCI_SCS_3_BASE_ADDRESS_LOW    				0x118
#define PCI_SCS_3_BASE_ADDRESS_HIGH    				0x11c
#define PCI_INTEGRATED_SRAM_BASE_ADDRESS_LOW   			0x120
#define PCI_INTEGRATED_SRAM_BASE_ADDRESS_HIGH			0x124

/****************************************/
/* PCI Configuration Function 2         */
/****************************************/


#define PCI_CS_0_BASE_ADDRESS_LOW	    			0x210
#define PCI_CS_0_BASE_ADDRESS_HIGH 				0x214
#define PCI_CS_1_BASE_ADDRESS_LOW	    			0x218
#define PCI_CS_1_BASE_ADDRESS_HIGH 				0x21c
#define PCI_CS_2_BASE_ADDRESS_LOW	    			0x220
#define PCI_CS_2_BASE_ADDRESS_HIGH 				0x224

/****************************************/
/* PCI Configuration Function 3         */
/****************************************/


#define PCI_CS_3_BASE_ADDRESS_LOW	    			0x310
#define PCI_CS_3_BASE_ADDRESS_HIGH 				0x314
#define PCI_BOOTCS_BASE_ADDRESS_LOW	    			0x318
#define PCI_BOOTCS_BASE_ADDRESS_HIGH 				0x31c

/****************************************/
/* PCI Configuration Function 4         */
/****************************************/


#define PCI_P2P_MEM0_BASE_ADDRESS_LOW 				0x410
#define PCI_P2P_MEM0_BASE_ADDRESS_HIGH			    	0x414
#define PCI_P2P_MEM1_BASE_ADDRESS_LOW 				0x418
#define PCI_P2P_MEM1_BASE_ADDRESS_HIGH			    	0x41c
#define PCI_P2P_I_O_BASE_ADDRESS 				0x420
#define PCI_INTERNAL_REGS_ADDRESS_BASE			    	0x424

/****************************************/
/* Interrupts	  			*/
/****************************************/
			
#define LOW_INTERRUPT_CAUSE_REGISTER   				0x004
#define HIGH_INTERRUPT_CAUSE_REGISTER				0x00c
#define CPU_INT_0_MASK_LOW                                     	0x014
#define CPU_INT_0_MASK_HIGH                                    	0x01c
#define CPU_INT_0_SELECT_CAUSE                                 	0x024
#define CPU_INT_1_MASK_LOW                                     	0x034
#define CPU_INT_1_MASK_HIGH                                    	0x03c
#define CPU_INT_1_SELECT_CAUSE                                 	0x044
#define INT_0_MASK_LOW                                     	0x054
#define INT_0_MASK_HIGH                                    	0x05c
#define INT_0_SELECT_CAUSE                                 	0x064
#define INT_1_MASK_LOW                                     	0x074
#define INT_1_MASK_HIGH                                    	0x07c
#define INT_1_SELECT_CAUSE                                 	0x084

/****************************************/
/* Interrupt Mask Bits                  */
/****************************************/

#define INT_MASK_LOW_DEV_ERR					(1 << 1)
#define INT_MASK_LOW_DMA_ERR					(1 << 2)
#define INT_MASK_LOW_CPU_ERR					(1 << 3)
#define INT_MASK_LOW_IDMA0					(1 << 4)
#define INT_MASK_LOW_IDMA1					(1 << 5)
#define INT_MASK_LOW_IDMA2					(1 << 6)
#define INT_MASK_LOW_IDMA3					(1 << 7)
#define INT_MASK_LOW_TIMER0					(1 << 8)
#define INT_MASK_LOW_TIMER1					(1 << 9)
#define INT_MASK_LOW_TIMER2					(1 << 10)
#define INT_MASK_LOW_TIMER3					(1 << 11)
#define INT_MASK_LOW_PCI_0					(1 << 12)
#define INT_MASK_LOW_SRAM_ERR					(1 << 13)
#define INT_MASK_LOW_GBE_ERR					(1 << 14)
#define INT_MASK_LOW_SERIAL_ERR					(1 << 15)
#define INT_MASK_LOW_PCI_1					(1 << 16)
#define INT_MASK_LOW_DRAM_ERR					(1 << 17)
#define INT_MASK_LOW_WDNMI					(1 << 18)
#define INT_MASK_LOW_WDE					(1 << 19)
#define INT_MASK_LOW_PCI_0_IN					(1 << 20)
#define INT_MASK_LOW_PCI_0_OUT					(1 << 21)
#define INT_MASK_LOW_PCI_1_IN					(1 << 22)
#define INT_MASK_LOW_PCI_1_OUT					(1 << 23)
#define INT_MASK_LOW_XOR_ERR					(1 << 29)

/****************************************/
/* I20 Support registers		*/
/****************************************/

#define INBOUND_MESSAGE_REGISTER0_PCI_SIDE			0x010
#define INBOUND_MESSAGE_REGISTER1_PCI_SIDE  			0x014
#define OUTBOUND_MESSAGE_REGISTER0_PCI_SIDE 			0x018
#define OUTBOUND_MESSAGE_REGISTER1_PCI_SIDE  			0x01C
#define INBOUND_DOORBELL_REGISTER_PCI_SIDE  			0x020
#define INBOUND_INTERRUPT_CAUSE_REGISTER_PCI_SIDE  		0x024
#define INBOUND_INTERRUPT_MASK_REGISTER_PCI_SIDE		0x028
#define OUTBOUND_DOORBELL_REGISTER_PCI_SIDE 			0x02C
#define OUTBOUND_INTERRUPT_CAUSE_REGISTER_PCI_SIDE   		0x030
#define OUTBOUND_INTERRUPT_MASK_REGISTER_PCI_SIDE   		0x034
#define INBOUND_QUEUE_PORT_VIRTUAL_REGISTER_PCI_SIDE  		0x040
#define OUTBOUND_QUEUE_PORT_VIRTUAL_REGISTER_PCI_SIDE   	0x044
#define QUEUE_CONTROL_REGISTER_PCI_SIDE 			0x050
#define QUEUE_BASE_ADDRESS_REGISTER_PCI_SIDE 			0x054
#define INBOUND_FREE_HEAD_POINTER_REGISTER_PCI_SIDE		0x060
#define INBOUND_FREE_TAIL_POINTER_REGISTER_PCI_SIDE  		0x064
#define INBOUND_POST_HEAD_POINTER_REGISTER_PCI_SIDE 		0x068
#define INBOUND_POST_TAIL_POINTER_REGISTER_PCI_SIDE 		0x06C
#define OUTBOUND_FREE_HEAD_POINTER_REGISTER_PCI_SIDE		0x070
#define OUTBOUND_FREE_TAIL_POINTER_REGISTER_PCI_SIDE		0x074
#define OUTBOUND_POST_HEAD_POINTER_REGISTER_PCI_SIDE		0x078
#define OUTBOUND_POST_TAIL_POINTER_REGISTER_PCI_SIDE		0x07C

#define INBOUND_MESSAGE_REGISTER0_CPU_SIDE			0X1C10
#define INBOUND_MESSAGE_REGISTER1_CPU_SIDE  			0X1C14
#define OUTBOUND_MESSAGE_REGISTER0_CPU_SIDE 			0X1C18
#define OUTBOUND_MESSAGE_REGISTER1_CPU_SIDE  			0X1C1C
#define INBOUND_DOORBELL_REGISTER_CPU_SIDE  			0X1C20
#define INBOUND_INTERRUPT_CAUSE_REGISTER_CPU_SIDE  		0X1C24
#define INBOUND_INTERRUPT_MASK_REGISTER_CPU_SIDE		0X1C28
#define OUTBOUND_DOORBELL_REGISTER_CPU_SIDE 			0X1C2C
#define OUTBOUND_INTERRUPT_CAUSE_REGISTER_CPU_SIDE   		0X1C30
#define OUTBOUND_INTERRUPT_MASK_REGISTER_CPU_SIDE   		0X1C34
#define INBOUND_QUEUE_PORT_VIRTUAL_REGISTER_CPU_SIDE  		0X1C40
#define OUTBOUND_QUEUE_PORT_VIRTUAL_REGISTER_CPU_SIDE   	0X1C44
#define QUEUE_CONTROL_REGISTER_CPU_SIDE 			0X1C50
#define QUEUE_BASE_ADDRESS_REGISTER_CPU_SIDE 			0X1C54
#define INBOUND_FREE_HEAD_POINTER_REGISTER_CPU_SIDE		0X1C60
#define INBOUND_FREE_TAIL_POINTER_REGISTER_CPU_SIDE  		0X1C64
#define INBOUND_POST_HEAD_POINTER_REGISTER_CPU_SIDE 		0X1C68
#define INBOUND_POST_TAIL_POINTER_REGISTER_CPU_SIDE 		0X1C6C
#define OUTBOUND_FREE_HEAD_POINTER_REGISTER_CPU_SIDE		0X1C70
#define OUTBOUND_FREE_TAIL_POINTER_REGISTER_CPU_SIDE		0X1C74
#define OUTBOUND_POST_HEAD_POINTER_REGISTER_CPU_SIDE		0X1C78
#define OUTBOUND_POST_TAIL_POINTER_REGISTER_CPU_SIDE		0X1C7C

/****************************************/
/*        Ethernet Unit Registers  		*/
/****************************************/

#define ETH_PHY_ADDR_REG                                     0x2000
#define ETH_SMI_REG                                          0x2004
#define ETH_UNIT_DEFAULT_ADDR_REG                            0x2008
#define ETH_UNIT_DEFAULTID_REG                               0x200c
#define ETH_UNIT_INTERRUPT_CAUSE_REG                         0x2080
#define ETH_UNIT_INTERRUPT_MASK_REG                          0x2084
#define ETH_UNIT_INTERNAL_USE_REG                            0x24fc
#define ETH_UNIT_ERROR_ADDR_REG                              0x2094
#define ETH_UNIT_INTERNAL_ADDR_ERROR                         0x2098
#define ETH_UNIT_CONTROL                                     0x20B0
#define ETH_BAR_0                                            0x2200
#define ETH_BAR_1                                            0x2208
#define ETH_BAR_2                                            0x2210
#define ETH_BAR_3                                            0x2218
#define ETH_BAR_4                                            0x2220
#define ETH_BAR_5                                            0x2228
#define ETH_SIZE_REG_0                                       0x2204
#define ETH_SIZE_REG_1                                       0x220c
#define ETH_SIZE_REG_2                                       0x2214
#define ETH_SIZE_REG_3                                       0x221c
#define ETH_SIZE_REG_4                                       0x2224
#define ETH_SIZE_REG_5                                       0x222c
#define ETH_HEADERS_RETARGET_BASE_REG                        0x2230
#define ETH_HEADERS_RETARGET_CONTROL_REG                     0x2234
#define ETH_HIGH_ADDR_REMAP_REG_0                            0x2280
#define ETH_HIGH_ADDR_REMAP_REG_1                            0x2284
#define ETH_HIGH_ADDR_REMAP_REG_2                            0x2288
#define ETH_HIGH_ADDR_REMAP_REG_3                            0x228c
#define ETH_BASE_ADDR_ENABLE_REG                             0x2290
#define ETH_ACCESS_PROTECTION_REG(port)                    (0x2294 + (port<<2))
#define ETH_MIB_COUNTERS_BASE(port)                        (0x3000 + (port<<7))
#define ETH_PORT_CONFIG_REG(port)                          (0x2400 + (port<<10))
#define ETH_PORT_CONFIG_EXTEND_REG(port)                   (0x2404 + (port<<10))
#define ETH_MII_SERIAL_PARAMETRS_REG(port)                 (0x2408 + (port<<10))
#define ETH_GMII_SERIAL_PARAMETRS_REG(port)                (0x240c + (port<<10))
#define ETH_VLAN_ETHERTYPE_REG(port)                       (0x2410 + (port<<10))
#define ETH_MAC_ADDR_LOW(port)                             (0x2414 + (port<<10))
#define ETH_MAC_ADDR_HIGH(port)                            (0x2418 + (port<<10))
#define ETH_SDMA_CONFIG_REG(port)                          (0x241c + (port<<10))
#define ETH_DSCP_0(port)                                   (0x2420 + (port<<10))
#define ETH_DSCP_1(port)                                   (0x2424 + (port<<10))
#define ETH_DSCP_2(port)                                   (0x2428 + (port<<10))
#define ETH_DSCP_3(port)                                   (0x242c + (port<<10))
#define ETH_DSCP_4(port)                                   (0x2430 + (port<<10))
#define ETH_DSCP_5(port)                                   (0x2434 + (port<<10))
#define ETH_DSCP_6(port)                                   (0x2438 + (port<<10))
#define ETH_PORT_SERIAL_CONTROL_REG(port)                  (0x243c + (port<<10))
#define ETH_VLAN_PRIORITY_TAG_TO_PRIORITY(port)            (0x2440 + (port<<10))
#define ETH_PORT_STATUS_REG(port)                          (0x2444 + (port<<10))
#define ETH_TRANSMIT_QUEUE_COMMAND_REG(port)               (0x2448 + (port<<10))
#define ETH_TX_QUEUE_FIXED_PRIORITY(port)                  (0x244c + (port<<10))
#define ETH_PORT_TX_TOKEN_BUCKET_RATE_CONFIG(port)         (0x2450 + (port<<10))
#define ETH_MAXIMUM_TRANSMIT_UNIT(port)                    (0x2458 + (port<<10))
#define ETH_PORT_MAXIMUM_TOKEN_BUCKET_SIZE(port)           (0x245c + (port<<10))
#define ETH_INTERRUPT_CAUSE_REG(port)                      (0x2460 + (port<<10))
#define ETH_INTERRUPT_CAUSE_EXTEND_REG(port)               (0x2464 + (port<<10))
#define ETH_INTERRUPT_MASK_REG(port)                       (0x2468 + (port<<10))
#define ETH_INTERRUPT_EXTEND_MASK_REG(port)                (0x246c + (port<<10))
#define ETH_RX_FIFO_URGENT_THRESHOLD_REG(port)             (0x2470 + (port<<10))
#define ETH_TX_FIFO_URGENT_THRESHOLD_REG(port)             (0x2474 + (port<<10))
#define ETH_RX_MINIMAL_FRAME_SIZE_REG(port)                (0x247c + (port<<10))
#define ETH_RX_DISCARDED_FRAMES_COUNTER(port)              (0x2484 + (port<<10))
#define ETH_PORT_DEBUG_0_REG(port)                         (0x248c + (port<<10))
#define ETH_PORT_DEBUG_1_REG(port)                         (0x2490 + (port<<10))
#define ETH_PORT_INTERNAL_ADDR_ERROR_REG(port)             (0x2494 + (port<<10))
#define ETH_INTERNAL_USE_REG(port)                         (0x24fc + (port<<10))
#define ETH_RECEIVE_QUEUE_COMMAND_REG(port)                (0x2680 + (port<<10))
#define ETH_CURRENT_SERVED_TX_DESC_PTR(port)               (0x2684 + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_0(port)              (0x260c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_1(port)              (0x261c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_2(port)              (0x262c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_3(port)              (0x263c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_4(port)              (0x264c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_5(port)              (0x265c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_6(port)              (0x266c + (port<<10))
#define ETH_RX_CURRENT_QUEUE_DESC_PTR_7(port)              (0x267c + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_0(port)              (0x26c0 + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_1(port)              (0x26c4 + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_2(port)              (0x26c8 + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_3(port)              (0x26cc + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_4(port)              (0x26d0 + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_5(port)              (0x26d4 + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_6(port)              (0x26d8 + (port<<10))
#define ETH_TX_CURRENT_QUEUE_DESC_PTR_7(port)              (0x26dc + (port<<10))
#define ETH_TX_QUEUE_0_TOKEN_BUCKET_COUNT(port)            (0x2700 + (port<<10))
#define ETH_TX_QUEUE_1_TOKEN_BUCKET_COUNT(port)            (0x2710 + (port<<10))
#define ETH_TX_QUEUE_2_TOKEN_BUCKET_COUNT(port)            (0x2720 + (port<<10))
#define ETH_TX_QUEUE_3_TOKEN_BUCKET_COUNT(port)            (0x2730 + (port<<10))
#define ETH_TX_QUEUE_4_TOKEN_BUCKET_COUNT(port)            (0x2740 + (port<<10))
#define ETH_TX_QUEUE_5_TOKEN_BUCKET_COUNT(port)            (0x2750 + (port<<10))
#define ETH_TX_QUEUE_6_TOKEN_BUCKET_COUNT(port)            (0x2760 + (port<<10))
#define ETH_TX_QUEUE_7_TOKEN_BUCKET_COUNT(port)            (0x2770 + (port<<10))
#define ETH_TX_QUEUE_0_TOKEN_BUCKET_CONFIG(port)           (0x2704 + (port<<10))
#define ETH_TX_QUEUE_1_TOKEN_BUCKET_CONFIG(port)           (0x2714 + (port<<10))
#define ETH_TX_QUEUE_2_TOKEN_BUCKET_CONFIG(port)           (0x2724 + (port<<10))
#define ETH_TX_QUEUE_3_TOKEN_BUCKET_CONFIG(port)           (0x2734 + (port<<10))
#define ETH_TX_QUEUE_4_TOKEN_BUCKET_CONFIG(port)           (0x2744 + (port<<10))
#define ETH_TX_QUEUE_5_TOKEN_BUCKET_CONFIG(port)           (0x2754 + (port<<10))
#define ETH_TX_QUEUE_6_TOKEN_BUCKET_CONFIG(port)           (0x2764 + (port<<10))
#define ETH_TX_QUEUE_7_TOKEN_BUCKET_CONFIG(port)           (0x2774 + (port<<10))
#define ETH_TX_QUEUE_0_ARBITER_CONFIG(port)                (0x2708 + (port<<10))
#define ETH_TX_QUEUE_1_ARBITER_CONFIG(port)                (0x2718 + (port<<10))
#define ETH_TX_QUEUE_2_ARBITER_CONFIG(port)                (0x2728 + (port<<10))
#define ETH_TX_QUEUE_3_ARBITER_CONFIG(port)                (0x2738 + (port<<10))
#define ETH_TX_QUEUE_4_ARBITER_CONFIG(port)                (0x2748 + (port<<10))
#define ETH_TX_QUEUE_5_ARBITER_CONFIG(port)                (0x2758 + (port<<10))
#define ETH_TX_QUEUE_6_ARBITER_CONFIG(port)                (0x2768 + (port<<10))
#define ETH_TX_QUEUE_7_ARBITER_CONFIG(port)                (0x2778 + (port<<10))
#define ETH_PORT_TX_TOKEN_BUCKET_COUNT(port)               (0x2780 + (port<<10))
#define ETH_DA_FILTER_SPECIAL_MULTICAST_TABLE_BASE(port)   (0x3400 + (port<<10))
#define ETH_DA_FILTER_OTHER_MULTICAST_TABLE_BASE(port)     (0x3500 + (port<<10))
#define ETH_DA_FILTER_UNICAST_TABLE_BASE(port)             (0x3600 + (port<<10))
#define GT_MII_MAX_PORTS        3
                                                                                
/****************************************/
/* Ethernet Port MIB Register Offsets   */
/****************************************/
                                                                                
#define ETH_MIB_GOOD_OCTETS_RCVD_LO     0x00
#define ETH_MIB_GOOD_OCTETS_RCVD_HI     0x04
#define ETH_MIB_BAD_OCTETS_RCVD         0x08
#define ETH_MIB_MAC_TRANS_ERR           0x0c
#define ETH_MIB_GOOD_FRAMES_RCVD        0x10
#define ETH_MIB_BAD_FRAMES_RCVD         0x14
#define ETH_MIB_BCAST_FRAMES_RCVD       0x18
#define ETH_MIB_MCAST_FRAMES_RCVD       0x1c
#define ETH_MIB_FRAMES_64               0x20
#define ETH_MIB_FRAMES_65_TO_127        0x24
#define ETH_MIB_FRAMES_128_TO_255       0x28
#define ETH_MIB_FRAMES_256_TO_511       0x2c
#define ETH_MIB_FRAMES_512_TO_1023      0x30
#define ETH_MIB_FRAMES_1024_TO_MAX      0x34
#define ETH_MIB_GOOD_OCTETS_SENT_LO     0x38
#define ETH_MIB_GOOD_OCTETS_SENT_HI     0x3c
#define ETH_MIB_GOOD_FRAMES_SENT        0x40
#define ETH_MIB_EXCESS_COLLISION        0x44
#define ETH_MIB_MCAST_FRAMES_SENT       0x48
#define ETH_MIB_BCAST_FRAMES_SENT       0x4c
#define ETH_MIB_UNKNOWN_MAC_CTRL_RCVD   0x50
#define ETH_MIB_GOOD_FC_RCVD            0x58
#define ETH_MIB_BAD_FC_RCVD             0x5c
#define ETH_MIB_UNDERSIZE_RCVD          0x60
#define ETH_MIB_FRAGMENTS_RCVD          0x64
#define ETH_MIB_OVERSIZE_RCVD           0x68
#define ETH_MIB_JABBER_RCVD             0x6c
#define ETH_MIB_MAC_RCV_ERR             0x70
#define ETH_MIB_BAD_CRC                 0x74
#define ETH_MIB_COLLISIONS              0x78
#define ETH_MIB_LATE_COLLISIONS         0x7c
                                                                                
#define ETH_MIB_MAX_IDX ((ETH_MIB_LATE_COLLISIONS >> 2) + 1)

#define GT_READ_MIB( port, offset ) \
    (*(volatile uint32 *)(GT_BASE_ADDR+ETH_MIB_COUNTERS_BASE(port)+offset))
                                                                                


/****************************************/
/* Communication Unit Registers         */
/****************************************/

#define COMM_UNIT_BASE_ADDRESS_0                            0xf200
#define COMM_UNIT_BASE_ADDRESS_1                            0xf208
#define COMM_UNIT_BASE_ADDRESS_2                            0xf210
#define COMM_UNIT_BASE_ADDRESS_3                            0xf218

#define COMM_UNIT_SIZE_REGISTER_0                           0xf204
#define COMM_UNIT_SIZE_REGISTER_1                           0xf20c
#define COMM_UNIT_SIZE_REGISTER_2                           0xf214
#define COMM_UNIT_SIZE_REGISTER_3                           0xf21c

#define COMM_UNIT_BASE_ADDR_ENABLE                          0xf250

#define COMM_UNIT_ARBITER_CONTROL                           0xf300
#define COMM_UNIT_CROSS_BAR_TIMEOUT                         0xf304
#define COMM_UNIT_INTERRUPT_CAUSE                           0xf310
#define COMM_UNIT_INTERRUPT_MASK                            0xf314
#define COMM_UNIT_ERROR_ADDRESS                             0xf318
           
/****************************************/
/* Cunit Debug  (for internal use)     */
/****************************************/

#define CUNIT_ADDRESS                                       0xf340
#define CUNIT_COMMAND_AND_ID                                0xf344
#define CUNIT_WRITE_DATA_LOW                                0xf348
#define CUNIT_WRITE_DATA_HIGH                               0xf34c
#define CUNIT_WRITE_BYTE_ENABLE                             0xf358
#define CUNIT_READ_DATA_LOW                                 0xf350
#define CUNIT_READ_DATA_HIGH                                0xf354
#define CUNIT_READ_ID                                       0xf35c

/****************************************/
/* SDMA Registers                       */
/****************************************/

#define SDMA_GROUP_CONFIGURATION_REGISTER                   0xb1f0
#define CHANNEL0_CONFIGURATION_REGISTER                     0x4000
#define CHANNEL0_COMMAND_REGISTER                           0x4008
#define CHANNEL0_RX_CMD_STATUS                              0x4800
#define CHANNEL0_RX_PACKET_AND_BUFFER_SIZES                 0x4804
#define CHANNEL0_RX_BUFFER_POINTER                          0x4808
#define CHANNEL0_RX_NEXT_POINTER                            0x480c
#define CHANNEL0_CURRENT_RX_DESCRIPTOR_POINTER              0x4810
#define CHANNEL0_TX_CMD_STATUS                              0x4C00
#define CHANNEL0_TX_PACKET_SIZE                             0x4C04
#define CHANNEL0_TX_BUFFER_POINTER                          0x4C08
#define CHANNEL0_TX_NEXT_POINTER                            0x4C0c
#define CHANNEL0_CURRENT_TX_DESCRIPTOR_POINTER              0x4c10
#define CHANNEL0_FIRST_TX_DESCRIPTOR_POINTER                0x4c14
#define CHANNEL1_CONFIGURATION_REGISTER                     0x6000
#define CHANNEL1_COMMAND_REGISTER                           0x6008
#define CHANNEL1_RX_CMD_STATUS                              0x6800
#define CHANNEL1_RX_PACKET_AND_BUFFER_SIZES                 0x6804
#define CHANNEL1_RX_BUFFER_POINTER                          0x6808
#define CHANNEL1_RX_NEXT_POINTER                            0x680c
#define CHANNEL1_CURRENT_RX_DESCRIPTOR_POINTER              0x6810
#define CHANNEL1_TX_CMD_STATUS                              0x6C00
#define CHANNEL1_TX_PACKET_SIZE                             0x6C04
#define CHANNEL1_TX_BUFFER_POINTER                          0x6C08
#define CHANNEL1_TX_NEXT_POINTER                            0x6C0c
#define CHANNEL1_CURRENT_RX_DESCRIPTOR_POINTER              0x6810
#define CHANNEL1_CURRENT_TX_DESCRIPTOR_POINTER              0x6c10
#define CHANNEL1_FIRST_TX_DESCRIPTOR_POINTER                0x6c14

/* SDMA Interrupt */

#define SDMA_CAUSE                                          0xb800
#define SDMA_MASK                                           0xb880


/****************************************/
/* Baude Rate Generators Registers      */
/****************************************/

/* BRG 0 */

#define BRG0_CFG_REG                         		    0xb200
#define BRG0_BAUD_TUNING_REG                                0xb204

/* BRG 1 */

#define BRG1_CFG_REG                                        0xb208
#define BRG1_BAUD_TUNING_REG                                0xb20c

/* BRG Interrupts */

#define BRG_CAUSE_REGISTER                                  0xb834
#define BRG_MASK_REGISTER                                   0xb8b4

/* MISC */

#define MAIN_ROUTING_REGISTER                               0xb400
#define RECEIVE_CLOCK_ROUTING_REGISTER                      0xb404
#define TRANSMIT_CLOCK_ROUTING_REGISTER                     0xb408
#define COMM_UNIT_ARBITER_CONFIGURATION_REGISTER            0xb40c

/****************************************/
/* Watchdog Registers                   */
/****************************************/

#define WATCHDOG_CONFIGURATION_REGISTER                     0xb410
#define WATCHDOG_VALUE_REGISTER                             0xb414


/****************************************/
/* Flex TDM Registers                   */
/****************************************/

/* FTDM Port */

#define FLEXTDM_TRANSMIT_READ_POINTER                       0xa800
#define FLEXTDM_RECEIVE_READ_POINTER                        0xa804
#define FLEXTDM_CONFIGURATION_REGISTER                      0xa808
#define FLEXTDM_AUX_CHANNELA_TX_REGISTER                    0xa80c
#define FLEXTDM_AUX_CHANNELA_RX_REGISTER                    0xa810
#define FLEXTDM_AUX_CHANNELB_TX_REGISTER                    0xa814
#define FLEXTDM_AUX_CHANNELB_RX_REGISTER                    0xa818

/* FTDM Interrupts */

#define FTDM_CAUSE_REGISTER                                 0xb830
#define FTDM_MASK_REGISTER                                  0xb8b0


/****************************************/
/* GPP Interface Registers              */
/****************************************/


#define GPP_IO_CONTROL                                      0xf100
#define GPP_LEVEL_CONTROL                                   0xf110
#define GPP_VALUE                                           0xf104
#define GPP_INTERRUPT_CAUSE                                 0xf108
#define GPP_INTERRUPT_MASK                                  0xf10c

#define MPP_CONTROL0                                        0xf000
#define MPP_CONTROL1                                        0xf004
#define MPP_CONTROL2                                        0xf008
#define MPP_CONTROL3                                        0xf00c

/****************************************/
/* I2C Registers                        */
/****************************************/

#define I2C_SLAVE_ADDRESS                                   0xc000
#define I2C_EXTENDED_SLAVE_ADDRESS                          0xc010
#define I2C_DATA                                            0xc004
#define I2C_CONTROL                                         0xc008
#define I2C_STATUS_BAUDE_RATE                               0xc00C
#define I2C_SOFT_RESET                                      0xc01c

/****************************************/ 
/* MPSC Registers                       */ 
/****************************************/ 

/* MPSC0  */

#define MPSC0_MAIN_CONFIGURATION_LOW                        0x8000
#define MPSC0_MAIN_CONFIGURATION_HIGH                       0x8004
#define MPSC0_PROTOCOL_CONFIGURATION                        0x8008
#define CHANNEL0_REGISTER1                                  0x800c
#define CHANNEL0_REGISTER2                                  0x8010 
#define CHANNEL0_REGISTER3                                  0x8014 
#define CHANNEL0_REGISTER4                                  0x8018 
#define CHANNEL0_REGISTER5                                  0x801c 
#define CHANNEL0_REGISTER6                                  0x8020 
#define CHANNEL0_REGISTER7                                  0x8024 
#define CHANNEL0_REGISTER8                                  0x8028 
#define CHANNEL0_REGISTER9                                  0x802c 
#define CHANNEL0_REGISTER10                                 0x8030 

/* MPSC1  */

#define MPSC1_MAIN_CONFIGURATION_LOW                        0x9000
#define MPSC1_MAIN_CONFIGURATION_HIGH                       0x9004
#define MPSC1_PROTOCOL_CONFIGURATION                        0x9008
#define CHANNEL1_REGISTER1                                  0x900c
#define CHANNEL1_REGISTER2                                  0x9010 
#define CHANNEL1_REGISTER3                                  0x9014 
#define CHANNEL1_REGISTER4                                  0x9018 
#define CHANNEL1_REGISTER5                                  0x901c 
#define CHANNEL1_REGISTER6                                  0x9020 
#define CHANNEL1_REGISTER7                                  0x9024 
#define CHANNEL1_REGISTER8                                  0x9028 
#define CHANNEL1_REGISTER9                                  0x902c 
#define CHANNEL1_REGISTER10                                 0x9030 

/* MPSCs Interupts  */

#define MPSC0_CAUSE                                         0xb804
#define MPSC0_MASK                                          0xb884
#define MPSC1_CAUSE                                         0xb80c
#define MPSC1_MASK                                          0xb88c

#endif /* _GT64440REG_H_ */
