/*
 *-----------------------------------------------------------------
 *
 * Name:	nsdb.S
 *
 * RCS:  $Id: nsdb.S,v 1.18 2002/10/17 21:32:42 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: NoStackDeBugger (nsdb), for initial RAM bringup, and other
 *				debug without the use of memory.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/2/01
 *
 *-----------------------------------------------------------------
 */
#ifdef __mips

#include <nfx-incl.h>
#include <sys/mips.h>
#ifndef TX3927
#include <exports/hwMap.h>
#endif
#include <exports/promio-api.h>

#ifdef USE_NSDB

	.globl	nsdb

	.text

	.set	noreorder

/*-----------------------------------------------------------------
 * Name:	nsdb
 *
 * Description: Top level NSDB menu display
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/2/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb
nsdb:
#if RAL_XXX
#else
	move	gp,ra
#endif

nsdb_loop:
	la		ARG1,main_menu
	JMP_SUB4(promio_prs)

	JMP_SUB5(promio_get_char)

	beq		RET,'0',nsdb_gt_init_ssc
	nop
	beq		RET,'1',nsdb_gt_init_fc
	nop
	beq		RET,'2',nsdb_read_mem
	nop
	beq		RET,'3',nsdb_write_mem
	nop
	beq		RET,'4',nsdb_read_loop
	nop
	beq		RET,'5',nsdb_write_loop
	nop
	beq		RET,'6',nsdb_sizeram
	nop
	beq		RET,'7',nsdb_init_cache
	nop
	beq		RET,'8',nsdb_init_tlb
	nop
	beq		RET,'9',nsdb_init_mem
	nop
	beq		RET,'a',nsdb_continue
	nop
#ifdef TX3927
	beq		RET,'b',nsdb_jump
	nop
#endif
	beq		RET,'c',nsdb_mem_addr_test
	nop
	beq		RET,'d',nsdb_mem_data_test
	nop

	la		ARG1,invalid_opt_msg
	JMP_SUB4(promio_prs)

	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_gt_init_ssc:
	la		ARG1,gt_init_ssc_msg
	JMP_SUB4(promio_prs)
	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_gt_init_fc:
	la		ARG1,gt_init_fc_msg
	JMP_SUB4(promio_prs)

	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_read_mem:

	JMP_SUB1(nsdb_get_size)			/* LEN valid */

	JMP_SUB1(nsdb_get_addr)			/* ADDR valid */

read_addr:
	JMP_SUB1(nsdb_read)				/* RET valid */

	move	ARG1,ADDR
	li		ARG2,0x8
	JMP_SUB3(promio_prx)

	la		ARG1,equal_msg
	JMP_SUB4(promio_prs)

	move	ARG1,RET
	move	ARG2,LEN
	sll		ARG2,1
	JMP_SUB3(promio_prx)

	la		ARG1,continue_msg
	JMP_SUB4(promio_prs)

	JMP_SUB5(promio_get_char)		/* RET valid */

	la		ARG1,nl_msg
	JMP_SUB4(promio_prs)

	beq		RET,ESC,nsdb_loop
	nop
	beq		RET,EQUAL,read_addr
	nop
	beq		RET,MINUS,back_one
	nop

	JMP_SUB1(nsdb_addr_inc)			/* ADDR incremented, depending on LEN */
	b		read_addr
	nop

back_one:
	JMP_SUB1(nsdb_addr_dec)			/* ADDR decremented */
	b		read_addr
	nop

/*------------------------------------------------------------------*/
nsdb_write_mem:

	JMP_SUB1(nsdb_get_size)			/* LEN valid */

	JMP_SUB1(nsdb_get_addr)			/* ADDR valid */

write_addr:
	JMP_SUB1(nsdb_read)				/* RET valid */

	move	SAVE_VAL,RET

	move	ARG1,ADDR
	li		ARG2,0x8
	JMP_SUB3(promio_prx)

	la		ARG1,equal_msg
	JMP_SUB4(promio_prs)

	move	ARG1,RET
	move	ARG2,LEN
	sll		ARG2,1
	JMP_SUB3(promio_prx)

	la		ARG1,write_opt_msg
	JMP_SUB4(promio_prs)

	move	ARG1,LEN
	sll		ARG1,1
	JMP_SUB2(promio_readx)
	move	WRITE_VAL,RET

	JMP_SUB1(nsdb_write)			/* RET valid */

	JMP_SUB1(nsdb_addr_inc)
	b		write_addr
	nop

/*------------------------------------------------------------------*/
nsdb_read_loop:
	la		ARG1,read_loop_msg
	JMP_SUB4(promio_prs)
	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_write_loop:
	la		ARG1,write_loop_msg
	JMP_SUB4(promio_prs)
	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_sizeram:
#ifndef TX3927
	la		ARG1,sizeram_msg
	JMP_SUB4(promio_prs)

/*
	jal		sizeram
	nop
*/

#endif
	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_init_cache:
#ifndef TX3927
	la		ARG1,init_cache_msg
	JMP_SUB4(promio_prs)

	la		ARG1,zero_bottom_256KB_msg
	JMP_SUB4(promio_prs)

	la		a0,PHYS_TO_K1(HW_RAM_BASE)
	addu	a1,a0,0x40000
#if __mips >= 3
1:
	sd		zero,0(a0)
	sd		zero,8(a0)
	sd		zero,16(a0)
	addu	a0,32
	bne		a0,a1,1b
	sd		zero,-8(a0)
#else
1:
	sw		zero,0(a0)
	sw		zero,4(a0)
	sw		zero,8(a0)
	addu	a0,16
	bne		a0,a1,1b
	sw		zero,-4(a0)
#endif

	jal		mips_init_cache
	nop

	la		ARG1,init_cache_msg
	JMP_SUB4(promio_prs)

#endif
	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_init_tlb:
	la		ARG1,init_tlb_msg
	JMP_SUB4(promio_prs)

/*
	jal		init_tlb
	nop
*/

	b		nsdb_loop
	nop

/*------------------------------------------------------------------*/
nsdb_init_mem:
#ifndef TX3927
	la		ARG1,init_mem_msg
	JMP_SUB4(promio_prs)

/*
	jal		meminit_data
	nop

	jal		meminit_bss
	nop
*/

	jal		rm7k_size_cache			/* restore cache sizes */
	nop

#endif
	b		nsdb_loop
	nop
	
/*------------------------------------------------------------------*/
nsdb_mem_addr_test:
    la      ARG1,addr_test_msg_all
    JMP_SUB4(promio_prs)

	JMP_SUB1(nsdb_get_addr)			/* ADDR valid */
    move    a3,ADDR                 /* start addr */
	JMP_SUB1(nsdb_get_addr)			/* ADDR valid */
    move    t6,ADDR                 /* end   addr */

    move    a0,a3                   /* start addr */
    move    a1,t6                   /* end addr */

    /*
     * Write address to address
     */
addr_test_write:
    beq     a0,a1,addr_test_check
    nop
    li      t8,0x000FFFFF           /* interval for printing status */
    and     t8,a0,t8
    beqz    t8,addr_print_status
    nop
    b       addr_no_print
    nop
addr_print_status:
    la      t9,write_addr_msg
    jal     addr_test_status
    nop
addr_no_print:
    sw      a0,0(a0)
    b       addr_test_write
    addu    a0,4

addr_test_status:
    move    t7,a0
    move    t8,a1
    move    ARG1,t9
    JMP_SUB4(promio_prs)
    move    a0,t7
    li      a1,8
    JMP_SUB3(promio_prx)
    la      ARG1,nl_msg
    JMP_SUB4(promio_prs)
    move    a0,t7
    j       ra
    move    a1,t8

addr_test_check:
    /*
     * Read address, compare for good data
     */
    la      ARG1,mem_test_msg_all_check
    JMP_SUB4(promio_prs)

    move    a0,a3                   /* saved start addr */
    move    a1,t6                   /* saved end   addr */
addr_test_read:
    beq     a0,a1,addr_test_done
    nop
    li      t8,0x000FFFFF           /* interval for printing status */
    and     t8,a0,t8
    beqz    t8,addr_read_print_status
    nop
    b       addr_read_no_print
    nop
addr_read_print_status:
    la      t9,read_addr_msg
    jal     addr_test_status
    nop
addr_read_no_print:
    lw      t6,0(a0)
    bne     t6,a0,addr_test_fail
    nop
    b       addr_test_read
    addu    a0,4

addr_test_fail:
    move    t7,a0

    la      ARG1,mem_test_error_msg
    JMP_SUB4(promio_prs)

    la      ARG1,read_addr_msg
    JMP_SUB4(promio_prs)

    move    a0,t7
    li      a1,8
    JMP_SUB3(promio_prx)

    la      ARG1,expected_msg
    JMP_SUB4(promio_prs)

    move    a0,t7
    li      a1,8
    JMP_SUB3(promio_prx)

    la      ARG1,read_msg
    JMP_SUB4(promio_prs)

    move    a0,t6
    li      a1,8
    JMP_SUB3(promio_prx)

addr_test_done:
    b       nsdb_loop
    nop

/*------------------------------------------------------------------*/
nsdb_mem_data_test:
    la      ARG1,data_test_msg_all
    JMP_SUB4(promio_prs)

	JMP_SUB1(nsdb_get_addr)			/* ADDR valid */
    move    a3,ADDR                 /* start addr */
	JMP_SUB1(nsdb_get_addr)			/* ADDR valid */
    move    t6,ADDR                 /* end   addr */
	JMP_SUB1(nsdb_get_data)			/* ADDR valid */
    move    WRITE_VAL,ADDR          /* data pattern in ADDR */

    move    a0,a3                   /* start addr */
    move    a1,t6                   /* end addr */

    /*
     * Write address to address
     */
data_test_write:
    beq     a0,a1,data_test_check
    nop
    li      t8,0x000FFFFF           /* interval for printing status */
    and     t8,a0,t8
    beqz    t8,data_print_status
    nop
    b       data_no_print
    nop
data_print_status:
    la      t9,write_addr_msg
    jal     data_test_status
    nop
data_no_print:
    sw      WRITE_VAL,0(a0)
    b       data_test_write
    addu    a0,4

data_test_status:
    move    t7,a0
    move    t8,a1
    move    ARG1,t9
    JMP_SUB4(promio_prs)
    move    a0,t7
    li      a1,8
    JMP_SUB3(promio_prx)
    la      ARG1,nl_msg
    JMP_SUB4(promio_prs)
    move    a0,t7
    j       ra
    move    a1,t8

data_test_check:
    /*
     * Read address, compare for good data
     */
    la      ARG1,mem_test_msg_all_check
    JMP_SUB4(promio_prs)

    move    a0,a3                   /* saved start addr */
    move    a1,t6                   /* saved end   addr */
data_test_read:
    beq     a0,a1,data_test_done
    nop
    li      t8,0x000FFFFF           /* interval for printing status */
    and     t8,a0,t8
    beqz    t8,data_read_print_status
    nop
    b       data_read_no_print
    nop
data_read_print_status:
    la      t9,read_addr_msg
    jal     data_test_status
    nop
data_read_no_print:
    lw      t6,0(a0)
    bne     t6,WRITE_VAL,data_test_fail
    nop
    b       data_test_read
    addu    a0,4

data_test_fail:
    move    t7,a0

    la      ARG1,mem_test_error_msg
    JMP_SUB4(promio_prs)

    la      ARG1,read_addr_msg
    JMP_SUB4(promio_prs)

    move    a0,t7
    li      a1,8
    JMP_SUB3(promio_prx)

    la      ARG1,expected_msg
    JMP_SUB4(promio_prs)

    move    a0,t7
    li      a1,8
    JMP_SUB3(promio_prx)

    la      ARG1,read_msg
    JMP_SUB4(promio_prs)

    move    a0,t6
    li      a1,8
    JMP_SUB3(promio_prx)

data_test_done:
    b       nsdb_loop
    nop

/*------------------------------------------------------------------*/
nsdb_continue:
#if RAL_XXX
	j		nsdb_done
#else
	move	ARG1,gp
	li		ARG2,0x8
	JMP_SUB3(promio_prx)
	move	ra,gp
	j		ra
#endif
	nop

#ifdef TX3927
/*------------------------------------------------------------------*/
nsdb_jump:

	la		a0,0x80008000
	jalr	a0
	nop

	b		nsdb_loop
	nop
#endif

	.end	nsdb

/*-----------------------------------------------------------------
 * Name:	nsdb_read
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/16/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_read
nsdb_read:
#if __mips >= 3
	beq		LEN,8,read_double
	nop
#endif
	beq		LEN,4,read_word
	nop
	beq		LEN,2,read_short
	nop
read_byte:
	lb		RET,0(ADDR)
	b		read_it
	nop
read_short:
	lh		RET,0(ADDR)
	b		read_it
	nop
#if __mips >= 3
read_double:
	ld		RET,0(ADDR)
	b		read_it
	nop
#endif
read_word:
	lw		RET,0(ADDR)

read_it:
	RET_SUB

	.end	nsdb_read

/*-----------------------------------------------------------------
 * Name:	nsdb_write
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/16/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_write
nsdb_write:
#if __mips >= 3
	beq		LEN,8,write_double
	nop
#endif
	beq		LEN,4,write_word
	nop
	beq		LEN,2,write_short
	nop
write_byte:
	sb		WRITE_VAL,0(ADDR)
	b		wrote_it
	nop
write_short:
	sh		WRITE_VAL,0(ADDR)
	b		wrote_it
	nop
#if __mips >= 3
write_double:
	sd		WRITE_VAL,0(ADDR)
	b		wrote_it
	nop
#endif
write_word:
	sw		WRITE_VAL,0(ADDR)
	
wrote_it:
	RET_SUB

	.end	nsdb_write

/*-----------------------------------------------------------------
 * Name:	nsdb_addr_inc
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/16/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_addr_inc
nsdb_addr_inc:
	addu	ADDR,LEN

	RET_SUB

	.end	nsdb_addr_inc

/*-----------------------------------------------------------------
 * Name:	nsdb_addr_dec
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/16/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_addr_dec
nsdb_addr_dec:
	subu	ADDR,LEN

	RET_SUB

	.end	nsdb_addr_dec

/*-----------------------------------------------------------------
 * Name:	nsdb_get_addr
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_get_addr
nsdb_get_addr:
	la		ARG1,addr_menu
	JMP_SUB4(promio_prs)

	li		ARG1,8
	JMP_SUB2(promio_readx)

	move	ADDR,RET

#if __mips >= 3
	li		RET,-1
	dsll	RET,32
	or		ADDR,RET
#endif

	RET_SUB

	.end	nsdb_get_addr

/*-----------------------------------------------------------------
 * Name:	nsdb_get_data
 *
 * Description:
 *
 * Created by:	John Hahn
 *
 * Date Created:	10/2/02
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_get_data
nsdb_get_data:
	la		ARG1,data_menu
	JMP_SUB4(promio_prs)

	li		ARG1,8
	JMP_SUB2(promio_readx)

	move	ADDR,RET

#if __mips >= 3
	li		RET,-1
	dsll	RET,32
	or		ADDR,RET
#endif

	RET_SUB

	.end	nsdb_get_data

/*-----------------------------------------------------------------
 * Name:	nsdb_get_size
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nsdb_get_size
nsdb_get_size:
	move	LEN,zero

	la		ARG1,size_menu
	JMP_SUB4(promio_prs)

	JMP_SUB5(promio_get_char)

	beq		RET,ESC,nsdb_loop
	nop
	bne		RET,'1',1f
	nop
	li		LEN,1
	b		got_it
	nop
1:
	bne		RET,'2',1f
	nop
	li		LEN,2
	b		got_it
	nop
1:
	bne		RET,'4',1f
	nop
	li		LEN,4
	b		got_it
	nop
1:
#if __mips >= 3
	bne		RET,'8',1f
	nop
	li		LEN,8
	b		got_it
	nop
1:
#endif
	la		ARG1,invalid_opt_msg
	JMP_SUB4(promio_prs)
	b		nsdb_get_size
	nop

got_it:
	RET_SUB

	.end	nsdb_get_size


	.set	reorder
/*------------------------------------------------------------------*/
/*
 *
 * Display Strings
 *
 */

main_menu:
	.ascii	"\r\n\r\nNSDB Main Menu\r\n"
	.ascii	"0) Init Galileo for SSC\r\n"
	.ascii	"1) Init Galileo for FC\r\n"
	.ascii	"2) Read Memory Location\r\n"
	.ascii	"3) Write Memory Location\r\n"
	.ascii	"4) Read Memory Loop\r\n"
	.ascii	"5) Write Memory Loop\r\n"
	.ascii	"6) Size RAM\r\n"
	.ascii	"7) Init Cache\r\n"
	.ascii	"8) Init TLB\r\n"
	.ascii	"9) Init DATA and BSS\r\n"
	.ascii	"a) Continue Normal Boot Sequence\r\n"
#ifdef TX3927
	.ascii	"b) Move execution to 0x80008000\r\n"
#endif
	.ascii	"c) Run Memory Address Test\r\n"
	.ascii	"d) Run Memory Data Pattern Test\r\n"
	.asciiz	"Selection > "

gt_init_ssc_msg:
	.asciiz	"Init GT for SSC\r\n"
gt_init_fc_msg:
	.asciiz	"Init GT for FC\r\n"
read_mem_msg:
	.asciiz	"Read memory address\r\n"
write_mem_msg:
	.asciiz	"Write memory address\r\n"
read_loop_msg:
	.asciiz	"Read Loop\r\n"
write_loop_msg:
	.asciiz	"Write Loop\r\n"
sizeram_msg:
	.asciiz	"Size RAM\r\n"
init_cache_msg:
	.asciiz	"Init Cache\r\n"
init_tlb_msg:
	.asciiz	"Init TLB\r\n"
init_mem_msg:
	.asciiz	"Init DATA and BSS\r\n"

invalid_opt_msg:
	.asciiz	"Invalid selection\r\n"

write_opt_msg:
	.asciiz	"   Enter new value, or ESC > "

size_menu:
	.ascii	"\r\n\r\nSize Menu\r\n"
	.ascii	"1) Byte (8 bits)\r\n"
	.ascii	"2) HalfWord (16 bits)\r\n"
	.ascii	"4) Word (32 bits)\r\n"
#if __mips >= 3
	.ascii	"8) DoubleWord (64 bits)\r\n"
#endif
	.asciiz	"Selection > "

data_menu:
	.asciiz	"\r\n\r\nEnter Hex Data (no '0x' needed) > "

addr_menu:
	.asciiz	"\r\n\r\nEnter Hex Address (no '0x' needed) > "

nl_msg:
	.asciiz	"\r\n"

equal_msg:
	.asciiz	" = "

continue_msg:
	.asciiz	" ['=' (same), '-' (prev), ESC (break), *(next)] > "

zero_bottom_256KB_msg:
	.asciiz	"zeroing bottom 256KB for cache init"

#if 1

addr_test_msg_all:
    .asciiz "Memory (address) test enter start and end address\r\n"

mem_test_msg_all_check:
    .asciiz "Memory written to now read and verify\r\n"

data_test_msg_all:
    .asciiz "Memory (data) test enter start, end, addrs and data pattern\r\n"

read_addr_msg:
    .asciiz "Rd Address = "

write_addr_msg:
    .asciiz "Wr Address = "

read_msg:
    .asciiz "Read = "

expected_msg:
    .asciiz "Expected = "

mem_test_error_msg:
    .asciiz "Memory Test Error: "

#else
memtest_msg:
	.asciiz "Running Memory Test from 0xa3000000 to 0xb0000000"

memtest_err_msg:
	.asciiz "Mem Test Error, addr/data = "
#endif

#endif  /* USE_NSDB */

#endif	/* __mips */
