/*
 *-----------------------------------------------------------------
 *
 * Name:	tx3927-promio.sx
 *
 * RCS:  $Id: tx3927-promio.S,v 1.2 2001/12/08 22:14:20 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: PROM serial driver for Toshiba uarts
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/13/01
 *
 *-----------------------------------------------------------------
 */

#if defined(TX3927)

#include <sys/tx3927.h>
#include <sys/mips.h>
#include <exports/promio-api.h>

 
	.set	noreorder

	.text

/*-----------------------------------------------------------------
 * Name:	promio_init()
 *
 * Description:
 *			registers used: t1 = UART
 *							t0 = TEMP
 *							t2 = TEMP
 *							t3 = TEMP
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_init
	.ent	promio_init
promio_init:

	la		UART,TX3927_SIO0_BASE

ser_init_channel:
	
	li		t0,0x00000024				/* IMCLK | 8-N-1 */
	sw		t0,TX3927_SIO_LCR_OFF(UART)

	li		t0,0
	sw		t0,TX3927_SIO_DICR_OFF(UART)

	li		t0,0
	sw		t0,TX3927_SIO_DISR_OFF(UART)

	li		t0,0
	sw		t0,TX3927_SIO_CISR_OFF(UART)

	li		t0,0
	sw		t0,TX3927_SIO_FCR_OFF(UART)

	li		t0,0
	sw		t0,TX3927_SIO_FLCR_OFF(UART)

	li		t0,14
	sw		t0,TX3927_SIO_BGR_OFF(UART)

	move	t2,UART
	la		UART,TX3927_SIO1_BASE
	bne		t2,UART,ser_init_channel
	nop

	j		ra
	nop

	.end	promio_init

/*-----------------------------------------------------------------
 * Name:	promio_put_char()
 *
 * Description:
 *			registers used: t1 = UART
 *							t0 = TEMP
 *							a0 = ARG1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_put_char
	.ent	promio_put_char
promio_put_char:
	la		UART,TX3927_SIO0_BASE
1:	lw		TEMP,TX3927_SIO_CISR_OFF(UART)
	nop
	and		TEMP,TEMP,0x04
	beqz	TEMP,1b
	nop
	sw		ARG1,TX3927_SIO_TFIFO_OFF(UART)
	nop
	j		ra
	nop

	.end	promio_put_char

/*-----------------------------------------------------------------
 * Name:	promio_get_char()
 *
 * Description:
 *			registers used: t1 = UART
 *							t0 = TEMP
 *							v0 = RET
 *							a0 = ARG1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_get_char
	.ent	promio_get_char
promio_get_char:
	la		UART,TX3927_SIO0_BASE
1:
	lw		TEMP,TX3927_SIO_DISR_OFF(UART)
	and		TEMP,TEMP,0x00004000
	bnez	TEMP,1b
	nop
	lw		RET,TX3927_SIO_RFIFO_OFF(UART)
	nop

1:
	lw		TEMP,TX3927_SIO_CISR_OFF(UART)
	nop
	and		TEMP,TEMP,0x04
	beqz	TEMP,1b
	nop
	sw		RET,TX3927_SIO_TFIFO_OFF(UART)
	nop
	j		ra
	nop

	.end	promio_get_char

	.globl	promio_get_char_no_echo
	.ent	promio_get_char_no_echo
promio_get_char_no_echo:
	la		UART,TX3927_SIO0_BASE
1:
	lw		TEMP,TX3927_SIO_DISR_OFF(UART)
	and		TEMP,TEMP,0x00004000
	bnez	TEMP,1b
	nop
	lw		RET,TX3927_SIO_RFIFO_OFF(UART)
	nop
	j		ra
	nop

	.end	promio_get_char_no_echo

/*-----------------------------------------------------------------
 * Name:	promio_char_avail()
 *
 * Description:
 *			registers used: t1 = UART
 *							v0 = RET
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_char_avail
	.ent	promio_char_avail
promio_char_avail:
	la		UART,TX3927_SIO0_BASE
	lw		RET,TX3927_SIO_DISR_OFF(UART)
	and		RET,RET,0x00004000
	beqz	RET,1f
	nop
	li		RET,0
	j		ra
	nop
1:
	li		RET,1
	j		ra
	nop

	.end	promio_char_avail

/*-----------------------------------------------------------------
 * Name:	promio_tx_ready()
 *
 * Description:
 *			registers used:	t1 = UART
 *							v0 = RET
 *
 *			RET will be 0 or != 0 (NOT == 1, if transmitter is ready)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_tx_ready
	.ent	promio_tx_ready
promio_tx_ready:
	la		UART,TX3927_SIO0_BASE
	lw		RET,TX3927_SIO_CISR_OFF(UART)
	nop
	and		RET,RET,0x04
	j		ra
	nop

	.end	promio_tx_ready

	.set	reorder

#endif	/* TX3927 */

