/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	commands.c
 *
 * RCS:  $Id: commands.c,v 1.36 2002/10/10 18:36:05 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/6/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-target.h>
#include <nfx-defs.h>
#include <sys/mips.h>
#include "pmon.h"
#include <pmon/stdio.h>
#include <pmon/termio.h>
#include "set.h"
#include <pmon/string.h>
#include <exports/i2c-api.h>
#include <exports/hwMap.h>
#include <exports/prom-part-api.h>

char           *searching = "searching..  ";

char           *badhexsym = "%s: neither hex value nor symbol\n";
unsigned short xlnx_lmux_csum = 0;

#if RAL_XXX
/** transp(ac,av), the 'tr' command */
transp (ac, av)
     int             ac;
     char           *av[];
{
    int             fd, n, i;
    jmp_buf         intrsave;
    char           *hostport, buf[80], abortch, *trabort;
    struct termio   tbuf, consave, hostsave;

    trabort = getenv ("trabort");
    abortch = str2cc (trabort);
    if (abortch == 0) {
	printf ("tr: error: trabort not set\n");
	return 1;
    }
    hostport = getenv ("hostport");
    if (strequ (hostport, "tty0")) {
	printf ("can't use tty0 as hostport in transparent mode\n");
	return 1;
    }
    fd = open (hostport, 0);
    if (fd == -1) {
	printf ("can't open %s\n", hostport);
	return 1;
    }
    printf ("Entering transparent mode, %s to abort\n", trabort);

    ioctl (fd, TCGETA, &tbuf);
    hostsave = tbuf;
    tbuf.c_lflag &= ~(ICANON | ECHO | ECHOE);
    tbuf.c_iflag &= ~(ICRNL);
    tbuf.c_iflag |= IXOFF;	/* enable tandem mode */
    tbuf.c_oflag &= ~ONLCR;
    tbuf.c_cc[4] = 1;
    ioctl (fd, TCSETAF, &tbuf);

    ioctl (STDIN, TCGETA, &tbuf);
    consave = tbuf;
    tbuf.c_lflag &= ~(ICANON | ECHO | ECHOE);
    tbuf.c_iflag &= ~(ICRNL | IXON);
    tbuf.c_oflag &= ~ONLCR;
    tbuf.c_cc[4] = 1;
    ioctl (STDIN, TCSETAF, &tbuf);

/* disable INTR char */
    ioctl (STDIN, GETINTR, intrsave);
    ioctl (STDIN, SETINTR, 0);

    for (;;) {
	ioctl (STDIN, FIONREAD, &n);
	if (n > 0) {
	    if (n > sizeof(buf) - 1)
		n = sizeof(buf) - 1;
	    n = read (STDIN, buf, n);
	    buf[n] = '\0';
	    if (strchr (buf, abortch))
		break;
	    write (fd, buf, n);
	}
	ioctl (fd, FIONREAD, &n);
	if (n > 0) {
	    if (n > sizeof(buf))
		n = sizeof(buf);
	    n = read (fd, buf, n);
	    write (STDOUT, buf, n);
	}
    }
    ioctl (STDIN, TCSETAF, &consave);
    ioctl (fd, TCSETAF, &hostsave);
    ioctl (STDIN, SETINTR, intrsave);
    return 0;
}
#endif

const Optdesc         m_opts[] =
{
    {"-b", "access bytes"},
    {"-h", "access half-words"},
    {"-w", "access words"},
#ifdef R4000
    {"-d", "access double-words"},
#endif
    {"-n", "non-interactive (no write)"},
    {"", "Interactive Options"},
    {"<hexval>", "set memory, forward one"},
    {"CR", "forward one, no change"},
    {"=", "re-read"},
    {"^|-", "back one"},
    {".", "quit"},
    {0}};

/** modify(ac,av), the 'm' command */
modify (ac, av)
     int             ac;
     char           *av[];
{
    word            adr, i;
    ureg_t	    v;
    char           *p;
    int		    datasz = -2;
    int		    nowrite = 0;
    extern int      optind;
    extern char    *optarg;
    int 	    c;

    optind = 0;
    while ((c = getopt (ac, av, "dwhbn")) != EOF) {
	if (datasz != -2) {
	    printf ("multiple data types specified\n");
	    return (-1);
	}
	switch (c) {
#ifdef R4000
	case 'd':
	    datasz = 8;
	    break;
#endif
	case 'w':
	    datasz = 4;
	    break;
	case 'h':
	    datasz = 2;
	    break;
	case 'b':
	    datasz = 1;
	    break;
	case 'n':
	    nowrite = 1;
	    break;
	default:
	    return (-1);
	}
    }

    if (optind >= ac)
      return (-1);

    if (!get_rsa (&adr, av[optind++]))
      return (-1);

#if __long64
	signextend(&adr);
#endif

    if (datasz == -2)
      datasz = 1 << matchenv ("datasz");

    if (optind < ac) {
	/* command mode */
	for (; optind < ac; optind++) {
	    if (strequ (av[optind], "-s")) {
		if (++optind >= ac) {
		    printf ("bad arg count\n");
		    return (-1);
		}
		for (p = av[optind]; *p; p++)
		  store_byte (adr++, *p);
	    } else {
		if (!get_rsa_reg (&v, av[optind]))
		    return (-1);
		if (adr & (datasz - 1)) {
		    printf ("%08x: unaligned address\n", adr);
		    return (1);
		}
		switch (datasz) {
		case 1:
		    store_byte (adr, v);
		    break;
		case 2:
		    store_half (adr, v);
		    break;
		case 4:
		    store_int (adr, v);
		    break;
#ifdef R4000
		case 8:
		    store_dword (adr, v);
		    break;
#endif
		}
		adr += datasz;
	    }
	}
    } else {
	/* interactive mode */
	if (adr & (datasz - 1)) {
	    printf ("%08x: unaligned address\n", adr);
	    return (1);
	}
	for (;;) {
	    switch (datasz) {
	    case 1:
		v = *(unsigned char *)adr;
		break;
	    case 2:
		v = *(unsigned short *)adr;
		break;
	    case 4:
		v = *(unsigned int *)adr;
		break;
#ifdef R4000
	    case 8:
		v = *(unsigned long long *)adr;
		break;
#endif
	    }
#if __mips >= 3
	    printf ("%08x %0*llx ", adr, 2 * datasz, v);
#else
	    printf ("%08x %0*lx ", adr, 2 * datasz, v);
#endif
	    if (nowrite) {
		printf ("\n");
		break;
	    }
	    line[0] = '\0'; get_line (line, 0);
	    for (p = line; *p == ' '; p++);
	    if (*p == '.')
	      break;
	    else if (*p == '\0')
	      adr += datasz;
	    else if (*p == '^' || *p == '-')
	      adr -= datasz;
	    else if (*p == '=')	
	      /* reread */;
	    else if (get_rsa_reg (&v, p)) {
		switch (datasz) {
		case 1:
		    store_byte (adr, v);
		    break;
		case 2:
		    store_half (adr, v);
		    break;
		case 4:
		    store_int (adr, v);
		    break;
#ifdef R4000
		case 8:
		    store_dword (adr, v);
		    break;
#endif
		}
		adr += datasz;
	    }
	}
    }
    return (0);
}


/** search(ac,av), the search command */
search (ac, av)
     int             ac;
     char           *av[];
{
    uword           from, to, adr, i, a;
    char           *s, *d, c;
    char            pat[PATSZ];
    int             siz, ln;

    ln = siz = moresz;
    ioctl (STDIN, CBREAK, NULL);

    if (!get_rsa (&from, av[1]) || !get_rsa (&to, av[2]))
	return (-1);
    for (d = pat, i = 3; i < ac; i++) {
	if (strequ (av[i], "-s")) {
	    if (++i >= ac) {
		printf ("bad arg count\n");
		return (-1);
	    }
	    for (s = av[i]; *s; s++)
	      *d++ = *s;
	} else {
	    if (!get_rsa (&a, av[i]))
		return (-1);
	    c = a;
	    *d++ = c;
	}
    }

    if (to <= from) {
	printf ("'to' address too small\n");
	return (1);
    }
    printf ("%s", searching);
    if (from <= to) {		/* forward search */
	to -= d - pat - 1;
	while (from <= to) {
	    s = pat;
	    adr = from++;
	    while (s < d) {
		if (*s != load_byte (adr))
		    break;
		s++;
		adr++;
	    }
	    if (d <= s) {
		era_line (searching);
		dispmem (prnbuf, from - 1, 4);
		if (more (prnbuf, &ln, siz))
		    break;
		printf ("%s", searching);
	    } else
		dotik (1, 0);
	}
	if (from > to)
	    era_line (searching);
    } else {			/* backward search */
	from -= d - pat - 1;
	while (to <= from) {
	    s = pat;
	    adr = from--;
	    while (s < d) {
		if (*s != load_byte (adr))
		    break;
		s++;
		adr++;
	    }
	    if (d <= s) {
		era_line (searching);
		dispmem (prnbuf, from - 1, 4);
		if (more (prnbuf, &ln, siz))
		    break;
		printf ("%s", searching);
	    }
	}
	if (to > from)
	    era_line (searching);
    }
    return (0);
}

/** call(ac,av), the call command */
call (ac, av)
     int             ac;
     char           *av[];
{
    int             i, j, k;
    char           *arg[10];

    arg[0] = 0;
    k = 0;
    for (i = 1; i < ac; i++) {
	if (av[i][0] == '-') {
	    j = 1;
	    while (av[i][j] != 0) {
		if (av[i][j] == 's') {
		    if (++i >= ac) {
			printf ("bad arg count\n");
			return (-1);
		    }
		    arg[k++] = av[i];
		    break;
		} else {
		    printf ("%c: unknown option\n", av[i][j]);
		    return (-1);
		}
		j++;
	    }
	} else {
	    if (!get_rsa ((uword *)&arg[k], av[i]))
		return (-1);
	    k++;
	}
    }
    if (arg[0] == 0) {
	printf ("Function address not specified\n");
	return (-1);
    }
    i = (((Func *) arg[0])) (arg[1], arg[2], arg[3], arg[4], arg[5]);
    printf ("Function returns: 0x%x (%d)\n", i, i);
    return (0);
}


#ifdef E2PROM
e2program (ac, av)
     int             ac;
     char           *av[];
{
    uword	addr, size, offset;

    if (!get_rsa (&addr, av[1])) {
	printf ("bad address\n");
	return -1;
    }

    if (!get_rsa (&size, av[2])) {
	printf ("bad size\n");
	return -1;
    }

    if (!get_rsa (&offset, av[3])) {
	printf ("bad offset\n");
	return -1;
    }

    printf ("Address = %x, Size = %x, Offset = %x\n",
	    addr, size, offset);

    sbd_e2program (addr, size, offset);
    return (0);
}
#endif


#ifdef FLASH
flashprogram (ac, av)
     int             ac;
     char           *av[];
{
    uword	addr, size, offset;
#if #endian(big)
    int		bigend = 1;
#else
    int		bigend = 0;
#endif

    void *fp; int fsize;
    sbd_flashinfo (&fp, &fsize);

    if (fsize == 0) {
	printf ("No flash found\n");
	return (1);
    }

    if (striequ (av[1], "-eb")) {
	if (ac < 5)
	    return (1);
	bigend = 1;
	av++;
    }
    else if (striequ (av[1], "-el")) {
	if (ac < 5)
	    return (1);
	bigend = 0;
	av++;
    }
    else if (striequ (av[1], "-w")) {
	if (ac < 5)
	    return (1);
	bigend = !bigend;
	av++;
    }
	
    if (!get_rsa (&addr, av[1])) {
	printf ("bad address\n");
	return -1;
    }

    if (!get_rsa (&size, av[2])) {
	printf ("bad size\n");
	return -1;
    }

    if (!get_rsa (&offset, av[3])) {
	printf ("bad offset\n");
	return -1;
    }

    printf ("Address = %x, Size = %x, Offset = %x\n",
	    addr, size, offset);

    sbd_flashprogram (addr, size, offset, bigend);
    return (0);
}
#endif


flush_cache (type, adr)
     int             type;
     word            adr;
{

    switch (type) {
    case ICACHE:
#if defined(R3000)
	r3k_iflush ();
#elif defined(R4000)
	mips_flush_cache ();	/* flush (and writeback) everything! */
#endif
	break;

    case DCACHE:
#if defined(R3000)
	r3k_dflush ();
#elif defined(R4000)
	mips_flush_dcache ();
#endif
	break;

    case IADDR:
#if defined(R3000)
	r3k_iaflush (adr);
#elif defined(R4000)
	mips_clean_cache (adr, 1);
#endif
	break;
    }
}

const Optdesc         flush_opts[] =
{
    {"-i", "flush I-cache"},
    {"-d", "flush D-cache"},
    {0}};

/** flush(ac,av), the 'flush' command */
flush (ac, av)
     int             ac;
     char           *av[];
{
    extern int      optind;
    extern char    *optarg;
    int 	    c;
    int             flags = 0;

    optind = 0;
    while ((c = getopt (ac, av, "id")) != EOF) 
      switch (c) {
      case 'd':
	  flags |= 1;
	  break;
      case 'i':
	  flags |= 2;
	  break;
      default:
	  return (-1);
      }

    if (!flags)
	flags = 3;

    if (flags & 2)
	flush_cache (ICACHE);

    if (flags & 1)
	flush_cache (DCACHE);

    return (0);
}

void
store_dword (adr, v)
    word            adr;
    dword	    v;
{
    *(dword *) adr = v;
    flush_cache (IADDR, adr);
}

void
store_word (adr, v)
     word            adr;
     word	     v;
{
    *(word *) adr = v;
    flush_cache (IADDR, adr);
}

void
store_int (adr, v)
     word            adr;
     int	     v;
{
    *(int *) adr = v;
    flush_cache (IADDR, adr);
}

void
store_half (adr, v)
     word            adr;
     half            v;
{
    *(half *) adr = v;
    flush_cache (IADDR, adr);
}

void
store_byte (adr, v)
     word            adr;
//     byte            v;
     char            v;
{
//    *(byte *)adr = v;
    *(char *)adr = v;
    flush_cache (IADDR, adr);
}

#if RAL_XXX
#ifndef NO_SERIAL
const Optdesc         dump_opts[] =
{
    {"-B", "dump binary image"},
    {"-h<port>", "send dump to host <port>"},
    {0}};

/** sdump(ac,av), the 'dump' command */
sdump (ac, av)
     int             ac;
     char           *av[];
{
    uword           adr, siz, len, i, a;
    char            *tmp;
    char           *uleof, *ulcr, *hostport = 0, *eol;
    int             fd, cs, v, binary = 0;
    struct termio   tbuf;
    extern int      optind;
    extern char    *optarg;
    int 	    c;

    optind = 0;
    while ((c = getopt (ac, av, "Bh:")) != EOF) 
      switch (c) {
      case 'B':
	  binary = 1;
	  break;
      case 'h':
	  hostport = optarg;
	  break;
      default:
	  return (-1);
      }

    if (optind + 2 > ac)
      return (-1);
    if (!get_rsa (&adr, av[optind++]))
      return (-1);
    if (!get_rsa (&siz, av[optind++]))
      return (-1);

    if (!hostport)
      hostport = (optind < ac) ? av[optind++] : getenv ("hostport");

    if (optind < ac)
      return (-1);

    fd = open (hostport, 1);
    if (fd == -1) {
	printf ("can't open %s\n", hostport);
	return (1);
    }

    if (binary) {
	if (ioctl (fd, TCGETA, &tbuf) >= 0) {
	    printf ("can't dump binary to tty\n");
	    return (1);
	}
	write (fd, adr, siz);
    } else {
	ioctl (fd, TCGETA, &tbuf);
	tbuf.c_iflag &= ~IXANY;
	tbuf.c_oflag &= ~ONLCR;
	ioctl (fd, TCSETAF, &tbuf);
	
	uleof = getenv ("uleof");
	ulcr = getenv ("ulcr");
	if (striequ (ulcr, "cr"))
	  eol = "\r";
	else if (striequ (ulcr, "lf"))
	  eol = "\n";
	else /* crlf */
	  eol = "\r\n";
	
	while (siz > 0) {
	    if (siz < 32)
	      len = siz;
	    else
	      len = 32;
	    cs = len + 5;
	    for (i = 0; i < 4; i++)
	      cs += (adr >> (i * 8)) & 0xff;
	    sprintf (line, "S3%02X%08X", len + 5, adr);
	    for (a = adr, tmp = line + 12, i = 0; i < len; a++, i++) {
		v = load_byte (a);
		cs += v;
		sprintf (tmp, "%02X", v & 0xff);
		tmp += 2;
	    }
	    sprintf (tmp, "%02X%s", (~cs) & 0xff, eol);
	    tmp += 2 + strlen (eol);
	    write (fd, line, tmp - line);
	    adr += len;
	    siz -= len;
	}
	sprintf (line, "S70500000000FA%s", eol);
	write (fd, line, strlen (line));
	write (fd, uleof, strlen (uleof));
    }

    close (fd);
    return (0);
}
#endif
#endif

/** copy(ac,av), the 'copy' command */
copy (ac, av)
     int             ac;
     char           *av[];
{
    uword           from, to, n;

    if (!get_rsa (&from, av[1]) || !get_rsa (&to, av[2]) || !get_rsa (&n, av[3]))
	return (-1);
    if (to < from)
	while (n-- > 0)
	    store_byte (to++, load_byte (from++));
    else
	for (from += n, to += n; n-- > 0;)
	    store_byte (--to, load_byte (--from));
    return (0);
}

const Optdesc         d_opts[] =
{
    {"-b", "display as bytes"},
    {"-h", "display as half-words"},
    {"-w", "display as words"},
#ifdef R4000
    {"-d", "display as double-words"},
#endif
    {"-s", "display a null terminated string"},
    {"-r<reg>", "display as register"},
    {0}};

/** dump(ac,av), the 'd' command */
dump (ac, av)
     int             ac;
     char           *av[];
{
    word            adr, x;
    char           *reg;
    int             siz, ln, i;
    int		    datasz = -2;
    extern int      optind;
    extern char    *optarg;
    int 	    c;
    static word     last_adr;

    optind = 0;
    while ((c = getopt (ac, av, "dwhbsr:")) != EOF) {
	if (datasz != -2) {
	    printf ("multiple data types specified\n");
	    return (-1);
	}
	switch (c) {
#ifdef R4000
	case 'd':
	    datasz = 8;
	    break;
#endif
	case 'w':
	    datasz = 4;
	    break;
	case 'h':
	    datasz = 2;
	    break;
	case 'b':
	    datasz = 1;
	    break;
	case 's':
	    datasz = 0;
	    break;
	case 'r':
	    reg = optarg;
	    datasz = -1;
	    break;
	default:
	    return (-1);
	}
    }

    /* get <addr> */
    if (optind >= ac || !get_rsa (&adr, av[optind++]))
      return (-1);

    if( repeating_cmd )
      adr = last_adr;
    
#if __long64
	signextend(&adr);
#endif

    /* get [<size>|<reg>] */
    if (optind < ac) {
	if (get_rsa (&x, av[optind])) {
	    /* size */
	    optind++;
	    ln = x; siz = 0;
	} else if (getregadr (0, av[optind])) {
	    if (datasz != -2) {
		printf ("multiple data types specified\n");
		return (-1);
	    }
	    reg = av[optind++];
	    datasz = -1;
	} else
	  return (-1);
    } else {
	ln = siz = moresz;
    }

    if (optind != ac) 
      return (-1);

    if (datasz == 0) {		/* -s, print string */
	strncpy (prnbuf, adr, 70);
	prnbuf[70] = '\0';
	for (i = 0; prnbuf[i] != 0; i++)
	    if (!isprint (prnbuf[i]))
		prnbuf[i] = '.';
	printf ("%s\n", prnbuf);
	return (0);
    }

    ioctl (STDIN, CBREAK, NULL);
    if (datasz == -1) {
	/* -r<reg> print as register */
	if (!disp_as_reg ((sreg_t *)adr, reg, &ln)) {
	    printf ("%s: bad register name\n", reg);
	    return (-1);
	}
    } else {
	if (datasz < 0)
	  datasz = 1 << matchenv ("datasz");
	while (1) {
	    last_adr = adr;
	    adr = dispmem (prnbuf, adr, datasz);
	    if (more (prnbuf, &ln, siz))
	      break;
	}
    }

    return (0);
}

word 
dispmem (p, adr, siz)
     char           *p;
     word            adr;
     int             siz;
{
    int             i;
    char            v;
    char            tmp[18];
    union {
	double		dummy;	/* ensure 8-byte alignment */
	unsigned char   b[16];
    } buf;
    word            w;

    w = adr;
    for (i = 0; i < 16; i++) {
	v = load_byte (w++);
	buf.b[i] = v;
    }

    sprintf (p, "%08x  ", adr);
    for (i = 0; i < 16; i += siz) {
	if (i == 8)
	    strccat (p, ' ');
	switch (siz) {
	case 1:
	    sprintf (tmp, "%02x ", *(unsigned char *)&buf.b[i]);
	    break;
	case 2:
	    sprintf (tmp, "%04x ", *(unsigned short *)&buf.b[i]);
	    break;
	case 4:
	    sprintf (tmp, "%08x ", *(unsigned int *)&buf.b[i]);
	    break;
#ifdef R4000
	case 8:
	    sprintf (tmp, "%016llx ", *(unsigned long long *)&buf.b[i]);
	    break;
#endif
	}
	strcat (p, tmp);
    }
    strcat (p, "  ");
    for (i = 0; i < 16; i++) {
	v = buf.b[i];
	strccat (p, isprint (v) ? v : '.');
    }
    return (adr + 16);
}

/** fill(ac,av) the fill command */
fill (ac, av)
     int             ac;
     char           *av[];
{
    uword           from, to, i, a, w;
    ubyte	    *p, *d;
    union {
	uword	w;
	uhalf	h;
	ubyte	b[PATSZ];
    } pat;
    int             len;

    if (!get_rsa (&from, av[1]) || !get_rsa (&to, av[2]))
	return (-1);
    if (to < from)
	return (1);

    for (d = p = pat.b, i = 3; i < ac; i++) {
	if (strequ (av[i], "-s")) {
	    if (++i >= ac) {
		printf ("bad arg count\n");
		return (-1);
	    }
	    else {
		char *s;
		for (s = av[i]; *s; s++) {
		    if (d >= &pat.b[PATSZ]) {
			printf ("pattern too long\n");
			return (-1);
		    }
		    *d++ = *s;
		}
	    }
	} else {
	    if (!get_rsa (&a, av[i]))
		return (-1);
	    if (d >= &pat.b[PATSZ]) {
		printf ("pattern too long\n");
		return (-1);
	    }
	    *d++ = a;
	}
    }

    len = d - p;
    if ((len == 1 || len == 2 || len == 4)
	&& (from & 3) == 0 && (to & 3) == 3) {
	/* special case using word writes */
	switch (len) {
	case 1:
	    w = pat.b[0];
	    w |= (w << 8);
	    w |= (w << 16);
	    break;
	case 2:
	    w = pat.h;
	    w |= (w << 16);
	    break;
	case 4:
	    w = pat.w;
	    break;
	}
	for (; from <= to; from += sizeof (uword))
	    *(uword *)from = w;
    } else {
	/* all other cases: byte by byte */
	for (; from <= to; from += sizeof (ubyte)) {
	    *(ubyte *)from = *p;
	    if (++p >= d)
	      p = pat.b;
	}
    }

    flush_cache (ICACHE, 0);
    return (0);
}

get_rsa (vp, p)
     uword	    *vp;
     char	    *p;
{
    ureg_t	    val;
    
    if (get_rsa_reg (&val, p)) {
	*vp = (uword) val;
	return (1);
    }
    return (0);
}


get_rsa_reg (vp, p)
     ureg_t	    *vp;
     char           *p;
{
    int             r, inbase, inalpha;
    word	    adr;
    ureg_t   	    v1, v2;
    char           *q, subexpr[LINESZ];

/* strip enclosing parens */
    while (*p == '(' && strbalp (p) == p + strlen (p) - 1) {
	strdchr (p);
	p[strlen (p) - 1] = 0;
    }

    if (q = strrpset (p, "+-")) {	/* is compound */
	strncpy (subexpr, p, q - p);
	subexpr[q - p] = '\0';
	r = get_rsa_reg (&v1, subexpr);
	if (r == 0)
	    return (r);
	r = get_rsa_reg (&v2, q + 1);
	if (r == 0)
	    return (r);
	if (*q == '+')
	    *vp = v1 + v2;
	else
	    *vp = v1 - v2;
	return (1);
    }
    if (q = strrpset (p, "*/")) {
	strncpy (subexpr, p, q - p);
	subexpr[q - p] = '\0';
	r = get_rsa_reg (&v1, subexpr);
	if (r == 0)
	    return (r);
	r = get_rsa_reg (&v2, q + 1);
	if (r == 0)
	    return (r);
	if (*q == '*')
	    *vp = v1 * v2;
	else {
	    if (v2 == 0) {
		printf ("divide by zero\n");
		return (0);
	    }
	    *vp = v1 / v2;
	}
	return (1);
    }
    if (*p == '^') {
	r = get_rsa_reg (&v2, &p[1]);
	if (r == 0)
	    printf ("%s: bad indirect address\n", &p[1]);
	else
	  *vp = load_word ((word)v2);
    }
    else if (*p == '@') {
	r = getreg (vp, &p[1]);
	if (r == 0)
	    printf ("%s: bad register name\n", &p[1]);
    } else if (strequ (p, ".")) {
	r = getreg (vp, "epc");
    } else if (*p == '&') {
	r = sym2adr (&adr, &p[1]);
	if (r == 0)
	    printf ("%s: bad symbol name\n", &p[1]);
	else
	    *vp = adr;
    } else if (isdigit (*p)) {
	inbase = matchenv ("inbase");
	switch (inbase) {
	case TEN:
	    r = ator (vp, p, 10);
	    break;
	case SIXTEEN:
	    r = ator (vp, p, 16);
	    break;
	case EIGHT:
	    r = ator (vp, p, 8);
	    break;
	case AUTO:
	    r = ator (vp, p, 0);
	    break;
	default:
	    printf ("%s: bad inbase value\n", getenv ("inbase"));
	    return (0);
	}
	if (r == 0) {
	    r = ator (vp, p, 0);
	    if (r == 0)
		printf ("%s: bad base %s value\n",
			p, getenv ("inbase"));
	}
    } else if (isxdigit (*p)) {
	inalpha = matchenv ("inalpha");
	if (inalpha == HEX) {
	    r = ator (vp, p, 16);
	    if (r == 0) {
		r = sym2adr (&adr, p);
		if (r == 0)
		    printf (badhexsym, p);
		else
		    *vp = adr;
	    }
	} else if (inalpha == SYMBOL) {
	    r = sym2adr (&adr, p);
	    if (r == 0) {
		r = ator (vp, p, 16);
		if (r == 0)
		    printf (badhexsym, p);
		else
		    *vp = adr;
	    }
	} else {
	    printf ("%s: bad inalpha value\n", getenv ("inalpha"));
	    return (0);
	}
    } else {
	r = sym2adr (&adr, p);
	if (r == 0)
	    printf ("%s: bad symbol name\n", p);
	else
	  *vp = adr;
    }
    return (r);
}


int
reboot_cmd (ac, av)
    int             ac;
    char           *av[];
{
#ifdef EVAL_BOARD
    volatile void (*rebootaddr)() = (void (*))0xbfc00000;


    printf ("Rebooting...\n");

    (*rebootaddr)();
#endif

#ifdef AGILE_HARDWARE
    printf ("Rebooting...\n");
    watchdog_disable();
	
	if (isSSC()) {
		*(volatile char *)0xbf000068 = 0x1;		/* write to RTC (1 second) */
		if (ac == 2) {
			if ((strncmp(av[1], "r7k", 3) == 0) || (strncmp(av[1], "soft", 3) == 0))
				*(volatile char *)0xbf000078 = 0xb2; /* cause R7000 warm start */
			else if(!strncmp(av[1], "module", 3))
				*(volatile char *)0xbf000078 = 0xb3; /* cause cold start */
			else {
				printf("usage: reboot [module|r7k|soft]\n");	
				return 0;
			}	
		}
		else
			*(volatile char *)0xbf000078 = 0xb3;	 /* cause cold start */
        while(1);       /* spin, waiting for reboot from BMFPGA */
	}
    else if (isFC()) {
        extern unsigned int nfxMyCPUId;	    /* one of the CPU identifiers in nfx-defs.h */

        if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU) { // control cpuid is 3(fc slice 0)
            pmon_cmSetCPUStateReboot();
            delay_us(1000);
            *(volatile char *)0xbf4001f4 = 0;			/* Brian's hack */
        }
        else {
            // FC slices 1,2,3, nfxCPUId 2,1,0
            printf ("wait for reset from SSC-Mgmt\n");
            delay_us(1000);
            pmon_cmSetCPUStateResetWait();
        }
	}
    else if (isTXRX()) {
		pmon_cmSetCPUStateReboot();
		delay_us(1000);
		*(volatile char *)(int)0xbf4000fc = 0;      /* Mehdi's hack*/
	}
    else if (isFP()) {
        printf ("wait for reset from SSC-Mgmt\n");
		delay_us(1000);
		pmon_cmSetCPUStateResetWait();
    }
    else
        return 0;

#endif
    return 0;
}

#ifdef SROM
const Optdesc         srom_opts[] =
{
    {"-n", "don't execute softROM code"},
    {0}};

sbd_srom (ac, av)
    int             ac;
    char           *av[];
{
    uword           from, to, n;
    int		    dogo, docopy;
    extern int      optind;
    int		    c;

    dogo = 1;
    docopy = 0;

    optind = 0;
    while ((c = getopt (ac, av, "n")) != EOF) {
	switch (c) {
	case 'n':
	    dogo = 0;
	    break;
	default:
	    return (-1);
	}
    }

    from = to = 0;
    if (optind+3 <= ac) {
	if (!get_rsa (&from, av[optind++]) ||
	    !get_rsa (&to, av[optind++]) ||
	    !get_rsa (&n, av[optind++]))
	    return (-1);
    }
    if (optind != ac)
	return (-1);

    /* copy softrom code */
    sbd_softromcopy (from, to, n);
    if (dogo)
	sbd_softromgo ();
    return (0);
}
#endif

/*-----------------------------------------------------------------
 * Name:	do_quit
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/8/01
 *
 *-----------------------------------------------------------------
 */
do_quit(ac, av)
	int		ac;
	char	* av[];
{
	int	(*fn)();

	fn = 0xBFC00520;

	(*fn)();
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/10/01
 *
 *-----------------------------------------------------------------
 */
do_pci_read(ac, av)
	int		ac;
	char	* av[];
{
	uword			bus, dev, func, off;
	unsigned int	i, data;
	int				rc;

	if ((ac != 4) && (ac != 5))
		return -1;

	if (!get_rsa (&bus, av[1]))
		return 1;
	if (!get_rsa (&dev, av[2]))
		return 1;
	if (!get_rsa (&func, av[3]))
		return 1;
	if (ac == 5) {
		if (!get_rsa (&off, av[4]))
			return 1;
	}
	else
		off = 0xFF;

	if (off == 0xFF) {
		for (i=0; i<0x40; i+= 4) {
			if (rc = pci_cfg_read(bus, dev, func, i, &data))
				printf("pci_cfg_read returned error, 0x%x\n", rc);
			else
				printf("pci reg 0x%02x = 0x%x\n", i, data);
		}
	}
	else {
		if (rc = pci_cfg_read(bus, dev, func, off, &data))
			printf("pci_cfg_read returned error, 0x%x\n", rc);
		else
			printf("pci reg = 0x%x\n", data);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/10/01
 *
 *-----------------------------------------------------------------
 */
do_pci_write(ac, av)
	int		ac;
	char	* av[];
{
	uword	bus, dev, func, off, data;
	int		rc;

	if (ac != 6)
		return -1;

	if (!get_rsa (&bus, av[1]))
		return 1;
	if (!get_rsa (&dev, av[2]))
		return 1;
	if (!get_rsa (&func, av[3]))
		return 1;
	if (!get_rsa (&off, av[4]))
		return 1;
	if (!get_rsa (&data, av[5]))
		return 1;

	if (rc = pci_cfg_write(bus, dev, func, off, data))
		printf("pci_cfg_write returned error, 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/10/01
 *
 *-----------------------------------------------------------------
 */
do_pci_clear(ac, av)
	int		ac;
	char	* av[];
{
	uword	bus, dev;

	if (ac != 3)
		return -1;

	if (!get_rsa (&bus, av[1]))
		return 1;
	if (!get_rsa (&dev, av[2]))
		return 1;

	pci_cfg_clear_status(bus, dev);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/10/01
 *
 *-----------------------------------------------------------------
 */
do_pci_stat(ac, av)
	int		ac;
	char	* av[];
{
    uword	bus, dev;
    int		rc;

	if (ac != 3)
		return -1;

	if (!get_rsa (&bus, av[1]))
		return 1;
	if (!get_rsa (&dev, av[2]))
		return 1;

	if (rc = pci_cfg_check_status(bus, dev))
		printf("ERROR : rc = 0x%x\n", rc);
	else
		printf("no error status bits\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/10/01
 *
 *-----------------------------------------------------------------
 */
do_pci_scan(ac, av)
	int		ac;
	char	* av[];
{
	uword	bus;

	printf("PCI bus scan");

	if (ac == 1)
		printf("\n");
	else {
		if (!get_rsa(&bus, av[1]))
			return 1;
		printf(", bus = %d\n", bus);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/10/01
 *
 *-----------------------------------------------------------------
 */
do_pci_init(ac, av)
	int		ac;
	char	* av[];
{

	pci_init();

	return 0;
}
int
do_break(ac, av)
	int		ac;
	char	* av[];
{
	dummy();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 * 
 * Description:
 * 
 * Created by:  Rick Lund
 * 
 * Date Created:
 * 
 *-----------------------------------------------------------------
 */
int
do_showEEEconfig(
	int		argc,
	char	* argv[]
)
{
	eee_displayEEEconfig();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:  Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int
do_showEEEError(
	int		argc,
	char	* argv[]
)
{
	eee_displayErrorStats();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */
int
do_i2c(
	int		argc,
	char	* argv[]
)
{
	unsigned long arg, data;
	unsigned char	i2c_addr, byte_addr;
	unsigned int	i, len;
	unsigned char	buf[256];
	int				rc;
	char *		dptr;
	unsigned char bus=0;

	if (strncmp(argv[1], "read", 4) == 0) {
#ifdef SIBYTE		
		if (argc != 6)
#else				
		if (argc != 5)
#endif				
			return -1;

#ifdef SIBYTE		
		if (!get_rsa (&arg, argv[2]))
			return (-1);
		bus = arg;
		if (!get_rsa (&arg, argv[3]))
			return (-1);
		i2c_addr = arg;
		if (!get_rsa (&arg, argv[4]))
			return (-1);
		byte_addr = arg;
		if (!get_rsa (&arg, argv[5]))
			return (-1);
		len = arg;
#else
		if (!get_rsa (&arg, argv[2]))
			return (-1);
		i2c_addr = arg;
		if (!get_rsa (&arg, argv[3]))
			return (-1);
		byte_addr = arg;
		if (!get_rsa (&arg, argv[4]))
			return (-1);
		len = arg;
#endif		
		if (byte_addr + len > 256) {
			printf("%d + %d > 256\n", byte_addr, len);
			return 1;
		}

		rc = i2c(I2C_READ, bus, i2c_addr, byte_addr, len, buf);
		if (rc < 0) {
			printf("Error in I2C_READ, rc = 0x%x\n", rc);
			return 0;
		}

		for (i=0; i<len; i++)
			printf("%d: read data 0x%x\n", i, buf[i]);
	}
	else if (strncmp(argv[1], "write", 5) == 0) {
#ifdef SIBYTE		
		if (argc < 7)
#else				
		if (argc < 6)
#endif				
			return -1;

#ifdef SIBYTE		
		if (!get_rsa (&arg, argv[2]))
			return (-1);
		bus = arg;
		if (!get_rsa (&arg, argv[3]))
			return (-1);
		i2c_addr = arg;
		if (!get_rsa (&arg, argv[4]))
			return (-1);
		byte_addr = arg;
		if (!get_rsa (&arg, argv[5]))
			return (-1);
		len = arg;
		if (!get_rsa (&arg, argv[6]))
			return (-1);
		data = arg;
		dptr = &data;
#else
		if (!get_rsa (&arg, argv[2]))
			return (-1);
		i2c_addr = arg;
		if (!get_rsa (&arg, argv[3]))
			return (-1);
		byte_addr = arg;
		if (!get_rsa (&arg, argv[4]))
			return (-1);
		len = arg;
		if (!get_rsa (&arg, argv[5]))
			return (-1);
		data = arg;
		dptr = &data;
#endif		
		if (byte_addr + len > 256) {
			printf("%d + %d > 256\n", byte_addr, len);
			return 1;
		}
#ifdef PROM_SIBYTE
		if(isFP()) {
			if((i2c_addr == 0x48) || (i2c_addr == 0x49)) { //lmux i2c address 
				/* Check for Valid register range */
				/* Valid range: register address + len to be read
				 * must not exceed the max register value */
				if(byte_addr + len > LMUX_VALID_REG_MAX) {
					printf("%d + %d > 0x%x\n", byte_addr, len, LMUX_VALID_REG_MAX);
					return 1;
				}
				if((((byte_addr + len) > 3) && ((byte_addr + len) < 0x58))
							   || ((byte_addr + len) > 0x60))	
				{
					printf("Cannot write to read-only register %d + %d \n", byte_addr, len);
					return 1;
				}
			}		
		}	
#endif		
		for (i=0; i<len; i++)
			buf[i] = *((char *)dptr+i);

		rc = i2c(I2C_WRITE, bus, i2c_addr, byte_addr, len, buf);
		if (rc < 0) {
			printf("Error in I2C_WRITE, rc = 0x%x\n", rc);
			return 0;
		}

		printf("Success in i2c write\n");

	}
	else
		return -1;

	return 0;

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */
int
do_wait(
	int		argc,
	char	* argv[]
)
{
	unsigned int	sec, us;

	if (argc != 2)
		return -1;

	sec = atoi(argv[1]);

	us = sec * 1000 * 1000;

	printf("us = %d\n", us);
	delay_us(us);

	printf("done\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/10/01
 *
 *-----------------------------------------------------------------
 */
int
do_ver(
	int		argc,
	char	* argv[]
)
{
	extern char		ImageVerString[];

	printf("\n%s\n\n", ImageVerString);

	return 0;

}

#if defined(SIBYTE) && (AGILE_HARDWARE)

typedef struct {
    int             luc;
    int             bootload;
} XLNX_DATA_DESC_S, * XLNX_DATA_DESC_P;

XLNX_DATA_DESC_S xdesc;

int
xlnx_download_fpga(void * uhdl, char * buf_p, int dlen)
{
    XLNX_DATA_DESC_P xdesc_p;
    volatile unsigned char *dest_addr;
    unsigned char *src_addr;
    volatile unsigned int *start_addr;
    unsigned int i, count;
    volatile unsigned int *status;
    volatile unsigned int *luc_reset;
    volatile unsigned int *deprogram;
    unsigned char hex_val, tmp;
    int luc;
    int times;
	unsigned short csum;

    xdesc_p = (XLNX_DATA_DESC_P) uhdl;
    luc = xdesc_p->luc;

    printf("xlnx_download_fpga: addr = %p, len = 0x%x\n", buf_p, dlen);

    for (times = 0; times < 2; times++) {

        if (luc) {
/*
            start_addr = (unsigned int*)PHYS_TO_K1(HW_BMFPGA_BASE+0x5c);
*/
            start_addr = (volatile unsigned int*)0xffffffffbf40005cll;


            /* write anything to bf400140 to deprogram the LUC */
            deprogram = (volatile unsigned int*)0xffffffffbf400140ll;
            *deprogram = 0xffffffff;
            delay_us(3000);
        }
        else /* lmux 0 and 1 */
        {
            if (times == 0) {
/*
                start_addr = (unsigned int*)PHYS_TO_K1(HW_BMFPGA_BASE+0x6c);
*/
                start_addr = (volatile unsigned int*)0xffffffffbf40006cll;

                /* write anything to bf40012c to deprogram the LMUX */
                deprogram = (volatile unsigned int*)0xffffffffbf40012cll;
                *deprogram = 0xffffffff;
                delay_us(3000);
            }
            else {
/*
                start_addr = (unsigned int*)PHYS_TO_K1(HW_BMFPGA_BASE+0x7c);
*/
                start_addr = (volatile unsigned int*)0xffffffffbf40007cll;
            }
        }
        printf("Programming FPGA...\n");
        count = 0;
		csum = 0;
        *start_addr = 0;  /* trigger start signal */
/*
        dest_addr = (unsigned char *)PHYS_TO_K1(HW_XILINX_PROGRAM_BASE);
*/
        dest_addr = (volatile unsigned char *)0xffffffffbf200000ll;
        //src_addr = (unsigned char *)0x40000000;
        src_addr = buf_p;
        for (i = 0; i < dlen-2; i+=2) {
            tmp = *(src_addr+i);
			csum += tmp;
            if ((tmp >= '0') && (tmp <= '9'))
                tmp = tmp - '0';
            else
                if ((tmp >= 'a') && (tmp <= 'f'))
                    tmp = tmp - 'a' + 0x0a;
            else
                if ((tmp >= 'A') && (tmp <= 'F'))
                    tmp = tmp - 'A' + 0x0a;
            else
            {
                break; /* ignore trailing invalid bytes */
            }
            hex_val = (tmp << 4); /* store upper nibble */

            tmp = *(src_addr+i+1);
			csum += tmp;
            if ((tmp >= '0') && (tmp <= '9'))
                tmp = tmp - '0';
            else
                if ((tmp >= 'a') && (tmp <= 'f'))
                    tmp = tmp - 'a' + 0x0a;
            else
                if ((tmp >= 'A') && (tmp <= 'F'))
                    tmp = tmp - 'A' + 0x0a;
            else
            {
                break; /* ignore trailing invalid bytes */
            }
            hex_val += tmp;     /* add lower nibble */

            *dest_addr = hex_val;
            count++;
        }
        *start_addr = 0xffffffff; /* trigger end signal */
        printf("Done writing %d bytes!\n", count);
/*
        status = (unsigned int*)PHYS_TO_K1(HW_BMFPGA_BASE);
*/
        status = (volatile unsigned int*)0xffffffffbf400000ll;
        printf("Program status: %x\n", *status);
		printf("checksum = %04x\n", csum);
        if (luc)
            break;
		else
			xlnx_lmux_csum = csum;
    }


	if (luc) {
/*
        luc_reset = (unsigned int*)PHYS_TO_K1(HW_BMFPGA_BASE+0xac);
*/
        delay_us(500000);   //XXX get rid of after production xlnx parts
        luc_reset = (volatile unsigned int*)0xffffffffbf4000acll;
        *luc_reset = 0xffffffff;
        delay_us(1);
        *luc_reset = 0;
        delay_us(100);
#if 0
        status = (unsigned int*)0xffffffffbf000100ll;
        *status = 1;
        delay_us(1);
        if (*status == 1)
            printf("LUC is up!\n");
        else
            printf("LUC is not responding!\n");
#endif
        // LUC_GIO_CFG_REG: make little endian
        luc_reset = (volatile unsigned int*)0xffffffffbf000154ll;
        *luc_reset = 0;         
        delay_us(1);
        luc_reset = (volatile unsigned int*)0xffffffffbf000000ll;
        *luc_reset = 0x61; /* bypass LUC CAM, since it doesn't work */
        if (*luc_reset == 0x61)
            printf("LUC is up!\n");
        else
            printf("LUC is not responding!\n");
    }
	else {
		if (*status & 0x800)
			printf("LMUX is programmed!\n");
		else
			printf("LMUX is not programmed!\n");
	}
    return 0;
}

/*
 * what:
 * 0 is all
 * 1 is lmux
 * 2 is luc
 */
void
xlnx_download_sel(int bootload, int what)
{
    int rc0;
    int rc1;
    int rv;
    unsigned short part_flags = 0;

    if (!isTXRX() || !isCheetah())
        return;

    xdesc.bootload = bootload;
    prom_part_get_cb_flags("lmux_hex", &part_flags);
    if (bootload && (part_flags & PROM_PART_BOOT_USE_DISABLE)) {
        printf("LMUX download on boot disabled\n");
    }
    else if (what == 0 || what == 1) {
        xdesc.luc = 0;
        rc0 = prom_part_proc_data(
                PROM_PART_LMUX_HEX,
                xlnx_download_fpga,
                (void *)&xdesc);
        if (rc0 != 0)
            printf("LMUX download/init failed\n");
    }

    prom_part_get_cb_flags("luc_hex", &part_flags);
    if (bootload && (part_flags & PROM_PART_BOOT_USE_DISABLE)) {
        printf("LUC download on boot disabled\n");
    }
    else if (what == 0 || what == 2) {
        xdesc.luc = 1;
        rc1 = prom_part_proc_data(
                PROM_PART_LUC_HEX,
                xlnx_download_fpga,
                (void *)&xdesc);
        if (rc1 != 0)
            printf("LUC download/init failed\n");
    }

}

/*-----------------------------------------------------------------
 * Name: do_xlnx
 *
 * Description: Program Xilinx FPGA
 *
 * Created by:	Tony Wong
 *
 * Date Created:03-22-02
 *
 *-----------------------------------------------------------------
 */
int
do_xlnx(int argc, char *argv[])
{
    extern unsigned long xlnx_load_size;
    int rc;

    if (argc < 2)
        return 0;

    if (strncmp(argv[1], "luc", 3) == 0) {
        xdesc.luc = 1;
    }
    else
    if (strncmp(argv[1], "lmux", 4) == 0) {
        xdesc.luc = 0;
    }
    else
    if (strncmp(argv[1], "all", 3) == 0) {
        xlnx_download_sel(0, 0);
        return 0;
    }
    else
        return 0;


    if ((argc == 3) && (strncmp(argv[2], "flash", 2) == 0)) {
        rc = prom_part_proc_data(
                (xdesc.luc) ? PROM_PART_LUC_HEX : PROM_PART_LMUX_HEX,
                xlnx_download_fpga,
                (void *)&xdesc);
    }
    else {
        rc = xlnx_download_fpga(
                (void *) &xdesc,
                (unsigned char *)0x40000000,
                xlnx_load_size);
    }

    if (rc != 0) {
        printf("Error: failed to download xlnx\n");
        return 0;
    }
    else
        lsi_init(); /* must initialize LSI MACs after LUC or LMUX is up */
    return 0;
}
#endif
