/*-----------------------------------------------------------------
 *
 * Name: ds1511.h
 *
 * RCS:  $Id $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: DS1511 registers definitions.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created: 02/28/2002
 *
 *-----------------------------------------------------------------
 */

#ifndef __ds1511_h
#define __ds1511_h

enum ds1511_regs {
	DS1511_SEC,
	DS1511_MIN,
	DS1511_HOUR,
	DS1511_DOW,
	DS1511_DOM,
	DS1511_MONTH,
	DS1511_YEAR,
	DS1511_CENTURY,
	DS1511_AM1_SEC,
	DS1511_AM2_MIN,
	DS1511_AM3_HOUR,
	DS1511_AM4_DATE,
	DS1511_WATCHDOG_MSEC,
	DS1511_WATCHDOG_SEC,
	DS1511_CONTROL_A,
	DS1511_CONTROL_B,
	DS1511_RAMADDR_LSB,
	DS1511_RAMDATA = 0x13
};

#define DS1511_CONTROL_A_BLF1 0x80
#define DS1511_CONTROL_A_BLF2 0x40
#define DS1511_CONTROL_A_PRS 0x20
#define DS1511_CONTROL_A_PAB 0x10
#define DS1511_CONTROL_A_TDF 0x08
#define DS1511_CONTROL_A_KSF 0x04
#define DS1511_CONTROL_A_WDF 0x02
#define DS1511_CONTROL_A_IRQF 0x01
#define DS1511_CONTROL_B_TE 0x80
#define DS1511_CONTROL_B_CS 0x40
#define DS1511_CONTROL_B_BME 0x20
#define DS1511_CONTROL_B_TPE 0x10
#define DS1511_CONTROL_B_TIE 0x08
#define DS1511_CONTROL_B_KIE 0x04
#define DS1511_CONTROL_B_WDE 0x02
#define DS1511_CONTROL_B_WDS 0x01


#define DS1511_BASE 0xbf000000
#define DS1511_BUS_SPARSE 8

void ds1511_init_watchdog(int dsec);
void ds1511_disable_watchdog(void);
void ds1511_do_reset(void);

static inline volatile unsigned char *
ds1511_regptr(int reg)
{
	return (unsigned char *)((unsigned)DS1511_BASE
							 + reg * DS1511_BUS_SPARSE);
}

static inline void
ds1511_write_reg(int reg, unsigned char val)
{
	*ds1511_regptr(reg) = val;
}


static inline unsigned char
ds1511_read_reg(int reg)
{
	return *ds1511_regptr(reg);
}



#endif
