/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	main.c
 *
 * RCS:  $Id: main.c,v 1.41 2002/12/07 00:01:04 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/6/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-target.h>
#include <sys/mips.h>
#include "pmon.h"
#include <pmon/stdio.h>
#include <pmon/termio.h>
#include <pmon/string.h>
#include <pmon/signal.h>
#include "ds1511.h"
#include <sys/led-api.h>

#ifdef DIAG_SIBYTE
#undef SBD_DISPLAY(msg, n)
#define SBD_DISPLAY(msg, n)     printf("%s\n", msg)
#endif

#ifdef MIPSEB
char            endian[] = "EB";
#else
char            endian[] = "EL";
#endif


Bps             Bpt[MAX_AC];    /* user break points                    */
Bps             BptTmp;         /* tmp bpt used for continue cmd        */
Bps             BptTrc;         /* bpt for tracing                      */
Bps             BptTrcb;        /* bpt for tracing through branches     */

reg_t           DBGREG[NREGS];	/* debugger's register value holder */

int             machtype;
jmp_buf         jmpb;           /* non-local goto jump buffer */
char            line[LINESZ + 1];	/* input line */
int             pmlst, clilst;	/* list of files opened by: PMON & client */
int             *curlst;		/* list of files opened by: current context */
struct termio   clitbuf;        /* client terminal mode */
unsigned long   initial_sr;
unsigned long long memorysize;
int             warm_start;
char            prnbuf[LINESZ + 8];	/* commonly used print buffer */

int             _pmon_in_ram;
int             repeating_cmd;

int             busclock_mhz;

int             init_done = 0;

#if defined(FLOATINGPT) && !defined(NEWFP)
extern struct c1state *c1dat;
#endif

extern int		nfxMySlotId, nfxMyCPUId;

const Optdesc   sh_opts[] =
{
    {"^P", "recall previous cmd"},
    {"^N", "recall next cmd"},
    {"^F", "move cursor right"},
    {"^B", "move cursor left"},
    {"^A", "move cursor far left"},
    {"^E", "move cursor far right"},
    {"^D", "delete char at cursor"},
    {"^H", "delete char before cursor"},
    {"^K", "delete line from cursor"},
    {"", ""},
    {"!!", "repeat last cmd"},
    {"!str", "recall and execute cmd str"},
    {"!num", "recall and execute cmd num"},
    {"", ""},
    {"+-*/()", "operators"},
    {"^addr", "contents of address"},
    {"@name", "contents of register"},
    {"&name", "value of symbol"},
    {"0xnum", "hex number"},
    {"0onum", "octal number"},
    {"0tnum", "decimal number"},
    {0}};


static char *expand(char *cmdline);
void    close (int);
void    get_cmd (char *p);
void    do_cmd (char *p);
int     help (int ac, char *av[]);
int     prhelp (int n, int *lnp, int siz, int namemax, int optsmax);
void    shrc (unsigned char *adr);
int     getln (unsigned char **adr, unsigned char *p);
word    no_cmd (int ac, char *av[]);
void    closelst (int lst);
void    swlst (int lst);
int     setjmp (jmp_buf);
int     ioctl (int, int, void *);
void    rm_bpts ( void );
char    *cmGetCpuString ( int, int );
void    watchdog_init (void);
void    watchdog_disable (void);
void    bmfpga_nmi_clear (int);
int     check_env_autoreboot_flag(void);
void    delay_us(unsigned int);
void    pmon_cmSetCPUStateResetWait(void);
void    pmon_cmSetCPUStatePromNoAutoboot(void);
void    zero_lomem( void );
void    envinit( void );
void    devinit( void );
void    histinit( void );
void    syminit( void );
void    sbddevinit( void );
void    serio_init( void );
void    sbdmachinit( void );
void    sbdmachinfo( void );
int     getmachtype( void );
char    *sbdgetname( void );
int     _mips_watchpoint_init (void);
int     isalpha(int c); 
int     isdigit(int c); 
int     strempty(char *);
int     matchenv(char *);
int     wordsz(char *);
int     argvize (char  *av[], char *);
int     more (char  *, int *, int);
unsigned int    sbdenable( int );
unsigned int    get_busclock( void );
unsigned long   sbdcpufreq (void);
unsigned long   sbdpipefreq (void);
unsigned long   enableCU1 ( void );
unsigned long   set_sr ( unsigned long );

#ifdef INET
static void
pmon_intr (int dummy)
{
    sigsetmask (0);
    longjmp (jmpb, 1);
}
#endif

#ifdef PROM_SIBYTE
unsigned int (*diag_jump)();	
#endif

/*************************************************************
 *  main()
 */
void
_main ()
{
    char            prompt[20];
	extern int		rcon;


    if (setjmp (jmpb))
	printf (" break!\r\n");
#ifdef INET
    signal (SIGINT, pmon_intr);
#else
    ioctl (STDIN, SETINTR, jmpb);
#endif

    rm_bpts ();
    set_sr (initial_sr);

#ifdef DIAG_SIBYTE
	do_mdk();
#endif

    while (1) {
		closelst (0);
		swlst (1);
		ioctl (STDIN, SETSANE, NULL);
#if 0
		memset(prompt, 0, 20);
		if (rcon)
			strcpy(prompt, "RCON-");
		if (isSSC())
			strcat(prompt, "SSC-");
		else if (isFC())
			strcat(prompt, "FC-");
		else if (isTXRX())
			strcat(prompt, "TXRX-");
		else if (isFP())
			strcat(prompt, "FP-");
		else
			strcat(prompt, "???-");
		if (_pmon_in_ram)
		    strcat (prompt, "PRAM> ");
		else
		    strcat (prompt, getenv ("prompt"));
		if (p = strchr (prompt, '!')) {
		    strdchr (p);	/* delete the bang */
		    sprintf (tmp, "%d", histno);
		    stristr (p, tmp);
		}
#endif
	    sprintf(prompt, "%s%s-PROM> ", (rcon ? "RCON-" : ""),
								cmGetCpuString(nfxMySlotId, nfxMyCPUId));
		printf ("%s", prompt);
		get_cmd (line);
		do_cmd (line);
    }
}

/*------------------------------------------------------------------
 *                        Panic prompt
 *------------------------------------------------------------------*/
void
panic_prompt()
{
	extern int	dont_poll;
	char		prompt[30];

	dont_poll = 1;

    watchdog_disable();
    bmfpga_nmi_clear(1);
	if (check_env_autoreboot_flag() == 0) {
		printf("Autoreboot set \"off\", stopping in debug mode\n");
	}
    else {
        delay_us(1000);
#ifdef R9000
		// ssc mgmt is now a SSC runtime process in Bobcat.
		// When in SSC Prom, reset system if auto reboot is on.
		printf("Autoreboot set \"on\", system reset\n");
		reboot_cmd(1, 0);
#else
		printf("Autoreboot set \"on\", waiting to be reset from ssc-mgmt\n");
#endif
        pmon_cmSetCPUStateResetWait();
    }

    while (1) {
		sprintf(prompt, "DEBUG-PROM-%s:%d",
							cmGetCpuString(nfxMySlotId, nfxMyCPUId), histno);
		printf("%s > ", prompt);
		get_cmd(line);
		do_cmd(line);
	}
}

void
autorun (char *s)
{
    char buf[LINESZ];
    char *d;
    unsigned long delay, lastt;
#if defined(RTC) || defined(INET)
    unsigned long t;
#endif
    unsigned int cnt;
    struct termio sav;
    int i;

    if (s && strlen(s)) {
	d = getenv ("bootdelay");
	if (!d || !atob (&delay, d, 10) || delay < 0 || delay > 99)
	    delay = 20;

	SBD_DISPLAY ("AUTO", CHKPNT_AUTO);
	printf ("Autoboot command: \"%.55s\"\nPress any key to cancel.\n", s);
	ioctl (STDIN, CBREAK, &sav);
	lastt = 0;
	delay++;
	do {
#if defined(RTC) || defined(INET)
	    t = __time (0);
	    if (t != lastt) {
		printf ("\r%2d", --delay);
		lastt = t;
	    }
#else
	    for (i = 10000000; i != 0; i--) continue;
	    printf ("\r%2ld", --delay);
#endif
	    ioctl (STDIN, FIONREAD, &cnt);
	} while (delay != 0 && cnt == 0);
	ioctl (STDIN, TCSETAF, &sav);
	putchar ('\n');
	if (cnt == 0) {
	    strcpy (buf, s);
	    do_cmd (buf);
	}
    }
}


#if RAL_XXX
#pragma weak sbdmachinfo
void sbdmachinfo()
{
}
#endif

void
bmfpga_aknowledge_up(void)
{
#ifdef AGILE_HARDWARE
#ifdef R7000
    bmfpga_nmi_clear(0);
#endif
#endif
}

/*************************************************************
 *  dbginit(adr)
 */
void
dbginit (adr)
     char           *adr;
{
    static const int fraction[] = {1000000, 100000, 10000, 1000, 100, 10, 1};
    unsigned int             i, freq, frac;
    unsigned int             runAutoDiagsFlag=0, diag_status=0, myModId=0, doAutoLoadAfterDiags=0;
    char	    *failure = 0;
	char        *s;
//#ifndef PROM_SIBYTE_BC // 06/01/04:This part will be enabled later.
	unsigned int qt_read=0, ft_read=0, diag_read=0, diag_jumper=0, run_from_rec_prom=0;
    unsigned int shortMem;  // WTG for debugging
#ifdef PROM_SIBYTE	
    unsigned long long config = 0;
#endif	
//#endif	
#ifdef INET
    int		    neterr = 0;
#endif
	extern char	ImageVerString[], _decompression_text_start[];
	unsigned long long	memsize;

	busclock_mhz = get_busclock();

    DIAG_LED(LED_DIAG_11);
	printf("\n\n\nPowerOn Self Test........OK\n\n");
    zero_lomem();
	printf("Initializing System......please wait\n\n");

	if (isSSC()) {
		ds1511_write_reg(DS1511_CONTROL_B, 0x30);
		ds1511_write_reg(DS1511_WATCHDOG_SEC, 0);
		ds1511_write_reg(DS1511_WATCHDOG_MSEC, 0);
/*
		warm_start = ds1511_read_reg(DS1511_CONTROL_A) & DS1511_CONTROL_A_WDF;
*/


		ds1511_write_reg(DS1511_CONTROL_A,
									 ds1511_read_reg(DS1511_CONTROL_A) & 0xf0);

		ds1511_write_reg(DS1511_CONTROL_B, 0xb0);
	}

	SBD_DISPLAY(" top",0);
	serio_init();

	SBD_DISPLAY("sbdm",0);
	sbdmachinit();

	SBD_DISPLAY("mllc",0);
	mallocInit(0x81000000, 0x1000000);

    from_init();

	crashdump_init();

#ifdef USE_WATCHDOG
	watchdog_init();
#endif

	cm_init(1);

    SBD_DISPLAY ("ENVI", CHKPNT_ENVI);
    envinit ();

    SBD_DISPLAY ("SBDD", CHKPNT_SBDD);
    sbddevinit ();

    SBD_DISPLAY ("DEVI", CHKPNT_DEVI);
    devinit ();

#ifndef PROM_R9000
	cm_init(2);
#endif

	upgrade_init();

	init_done = 1;

	SBD_DISPLAY ("BACK", CHKPNT_DEVI);
    failure = getenv ("itfailure");

#ifdef INET
    if (failure) {
	s = getenv ("itfailcodes");
	if (s) {
	    /* check for an ethernet test error */
	    char buf[128];
	    strncpy (buf, s, sizeof(buf)-1);
	    for (s = strtok (buf, " "); s; s = strtok (NULL, " ")) {
		unsigned long code;
		if (atob (&code, s, 10) && code == 0x15) /* XXX ethernet err */
		    neterr = 1;
	    }
	}
	else
	    /* cannot distinguish test failures, assume the worst */
	    neterr = 1;
    }

    SBD_DISPLAY ("NETI", CHKPNT_NETI);
    init_net (!neterr);
#endif

    SBD_DISPLAY ("HSTI", CHKPNT_HSTI);
    histinit ();

    SBD_DISPLAY ("SYMI", CHKPNT_SYMI);
    syminit ();

#ifdef DEMO
    SBD_DISPLAY ("DEMO", CHKPNT_DEMO);
    demoinit ();
#endif

    SBD_DISPLAY ("MACH", CHKPNT_MACH);
#ifdef SR_FR
    initial_sr = enableCU1 () & (SR_FR | SR_CU1);
#else
    initial_sr = enableCU1 () & SR_CU1;
#endif
    machtype = getmachtype ();

    SBD_DISPLAY ("SBDE", CHKPNT_SBDE);
    initial_sr |= sbdenable (machtype);
#if LOCAL_MEM != 0
    initial_sr |= SR_BEV;
#else
    initial_sr |= SR_BEV;
#endif
#ifdef __long64
	initial_sr |= SR_KX;
#endif
#ifdef SR_FR
    Status = initial_sr & ~SR_FR; /* don't confuse naive clients */
#endif

#if RAL_XXX
    SBD_DISPLAY ("LOGO", CHKPNT_LOGO);
    printf ("\n%s version %s [%s,%s", 
	    _pmon_in_ram ? "PRAM" : "PMON",
	    ImageVerString, sbdgetname (), endian);

#ifdef FLOATINGPT
    if (initial_sr & SR_CU1)
        printf (",FP");
#endif
#ifdef INET
    printf (",NET");
#endif

#if 0
    printf ("], LSI LOGIC Corp. %s\n", date);
#else
    printf ("]\nAlgorithmics Ltd. %s\n", date);
#endif
#if defined(P4000) || defined(P4032)  || defined(P5064) || defined(P6032) || defined(P6064) || defined(MIDAS)
    /* Standard Algorithmics warranty applies to supported boards */
    printf ("This software is not subject to copyright and may be freely copied.\n");
#else
    printf ("This is free software, and comes with ABSOLUTELY NO WARRANTY,\n");
    printf ("you are welcome to redistribute it without restriction.\n");
#endif
#else	/* RAL_XXX */

    printf ("\n\n\n\n%s [%s,%s", 
	    _pmon_in_ram ? "PRAM" : "PMON", sbdgetname (), endian);
#ifdef FLOATINGPT
    if (initial_sr & SR_CU1)
      printf (",FP");
#endif
#ifdef INET
    printf (",NET");
#endif

#ifdef SR_KX
	if (initial_sr & SR_KX)
		printf(",64");
#endif
	printf("]\n");
	printf("ONStor Inc. %s\n", ImageVerString);
#endif	/* RAL_XXX */

    sbdmachinfo ();

#ifdef SIBYTE
	printf ("CPU type %s%d.", "SB", machtype);
    printf ("  Rev %d", (*(volatile long long *)(int)0xb0020000 & 0x0FF00) >> 8);
#else
    if (machtype)
        printf ("CPU type %s%d.", (machtype/1000 == 33) ? "LR" : "R", machtype);
    else
		printf ("CPU type UNKNOWN.");
    printf ("  Rev %d.%d.", (int)((word)Prid >> 4) & 0x0f, (int)(word)Prid & 0x0f);
#endif


#define ONEMHZ		(1000 * 1000)
#define DP		2    		/* 2 decimal places for user */

    freq = sbdpipefreq (); /* get the CPU pipeline clock */
    freq += fraction[DP] / 2;
    frac = (freq % ONEMHZ) / fraction[DP];
    if (frac != 0) 
        printf ("  %d.%0*d", freq / ONEMHZ,  DP, frac);
    else if (freq)
        printf ("  %d", freq / ONEMHZ);
    else
        printf ("  ??");
    printf (" MHz");

#ifndef SIBYTE
    freq = sbdcpufreq (); /* get the CPU external clock */
    freq += fraction[DP] / 2;
    frac = (freq % ONEMHZ) / fraction[DP];
    if (frac != 0) 
        printf ("/%d.%0*d", freq / ONEMHZ,  DP, frac);
    else if (freq)
        printf ("/%d", freq / ONEMHZ);
    else
        printf ("/??");
    printf (" MHz.\n");
#else
	printf("\n");
#endif

	printf("module: %s, Slot %d, CPU %d\n", sbdgetname(), nfxMySlotId, nfxMyCPUId);
#define MEG	(1024 * 1024)
    memsize = memorysize;
#if defined(LOCAL_MEM) && (LOCAL_MEM!=0)
    memsize -= LOCAL_MEM;
#endif

    if (memsize % MEG != 0)
        printf ("Memory size %3lld KB.\n", memsize / 1024);
    else
        printf ("Memory size %3lld MB.\n", memsize / MEG);

#ifdef R4000
    {
	extern int mips_icache_size, mips_icache_linesize;
	extern int mips_dcache_size, mips_dcache_linesize;
	extern int mips_scache_size, mips_scache_linesize;
	extern int mips_tcache_size, mips_tcache_linesize;
	extern int mips_icache_ways, mips_dcache_ways;
	extern int mips_scache_ways, mips_tcache_ways;
	extern int mips_scache_split, mips_scache_discontig;

	printf ("Icache size %3d KB, %2d/line",
		mips_icache_size / 1024, mips_icache_linesize);
	if (mips_icache_ways > 1)
	    printf (" (%d way)", mips_icache_ways);
	printf ("\n");

	printf ("Dcache size %3d KB, %2d/line",
		mips_dcache_size / 1024, mips_dcache_linesize);
	if (mips_dcache_ways > 1)
	    printf (" (%d way)", mips_dcache_ways);
	printf ("\n");

    if (mips_scache_size > 0) {
        unsigned int size = mips_scache_size;
        if (mips_scache_split || mips_scache_discontig)
        size *= 2;


        if (size % MEG != 0)
          printf ("Scache size %3d KB", size / 1024);
        else
          printf ("Scache size %3d MB", size / MEG);
        printf (", %2d/line %s",
            mips_scache_linesize,
            mips_scache_split ? ", split" : 
            mips_scache_discontig ? ", discontiguous" :
            "");
        if (mips_scache_ways > 1)
        printf (" (%d way)", mips_scache_ways);
        printf ("\n");
    }

    if (mips_tcache_size > 0) {
        if (mips_tcache_size % MEG != 0)
          printf ("Tcache size %3d KB", mips_tcache_size / 1024);
        else
          printf ("Tcache size %3d MB", mips_tcache_size / MEG);
        printf (", %2d/line", mips_tcache_linesize);
        if (mips_tcache_ways > 1)
        printf (" (%d way)", mips_tcache_ways);
        printf ("\n");
    }

    }
#else
    {
	extern int icache_size;
	extern int dcache_size;
	if (icache_size >= 1024)
	  printf ("Icache size %3d KB.\n", icache_size / 1024);
	else
	  printf ("Icache size %3d.\n", icache_size);
	if (dcache_size >= 1024)
	  printf ("Dcache size %3d KB.\n", dcache_size / 1024);
	else
	  printf ("Dcache size %3d.\n", dcache_size);
    }
#endif
#ifndef PROM_R9000
	{
        char	ipaddr_str[20];
        char	ipmask_str[20];
        seepGetIpAddr(ipaddr_str);
        seepGetIpMask(ipmask_str);
        printf("debug IP addr = %s\n", ipaddr_str);
        printf("debug IP mask = %s\n", ipmask_str);
	}
    printf ("\n\n");

	/* Tell BM-FPGA that we are up and running */
	bmfpga_aknowledge_up();
#endif

    for (i = 0; i < MAX_BPT; i++)
        Bpt[i].addr = NO_BPT;

    BptTmp.addr = NO_BPT;
    BptTrc.addr = NO_BPT;
    BptTrcb.addr = NO_BPT;
    trace_mode = TRACE_NO;

    _mips_watchpoint_init ();

    Epc = (word) CLIENTPC;
/*
    Gpr[29] = clienttos ();
*/
    Gpr[29] = (reg_t) _decompression_text_start;

#if defined(R4000) && defined(FLOATINGPT)
    if (machtype == 4640 || machtype == 4650) {
        int r;
#if __mips >= 3
        /* set m.s. half of float registers to d.p. NaN */
        for (r = R_F0; r <= R_F31; r++)
            DBGREG[r] = 0x7ff00bad00000000LL;
#else
        /* set odd float registers to d.p. NaN */
        for (r = R_F1; r <= R_F31; r+=2)
            DBGREG[r] = 0x7ff00bad;
#endif
    }
#endif

#if defined(FLOATINGPT) && !defined(NEWFP)
    c1dat = (struct c1state *)malloc (_fpstatesz ());
    _fpinit (c1dat);
#endif

#ifdef SIBYTE
	/* clear cache error counter at well-known location */
	*(volatile int *)(int)0x80700000 = 0;
#endif

/* wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww */
/* WTG  Move check for jumper here  */
/* wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww */

    /* This gets the env info */
    qt_read = get_hw_qt_info();
    ft_read = get_hw_ft_info();
    diag_read = get_hw_diag_info();
                                                                                                                                    
#ifdef PROM_SIBYTE_BC
/* This has been implemented to jump directly to the
 * diag partition in the big flash if the hw_qt/hw_ft/hw_diag
 * env fields are set to "on".
 */
    diag_jump = 0xffffffffbfcc0000;
                                                                                                                                    
    if(isTXRX()) {
extern int primary_is_rprom;
        config = ((*(volatile unsigned long long *)0xffffffffbf400000) & 0x0000000000F00000ll) >> 21;
        if ((config & 0x1) == 1)  // Bit 13 (1 = diag jumper )
            diag_jumper = 1;
// WTG Recovery Jumper doens't show up ..  command "prom view" works cuz it looks at PROM BASE and gets
//      the DevID from the part and sets "primary_is_rprom" to correct value......[ from_init() ]
      //  if ((config & 0x4) == 4)  // Bit 15 ( 1 = recovery prom   0 = Normal (Std))
        if (primary_is_rprom == 1)
            run_from_rec_prom = 1;
    }
                                                                                                                                    
    if(isFP()) {
        config = (*(volatile unsigned long long *)0x900000f81f400000) & 0x000000000000000Fll;
        if ((config & 0x1) == 1)  // Bit 0  (1 = diag jumper )
            diag_jumper = 1;
        if ((config & 0x2) == 2)  // Bit 1 ( 1 = recovery prom   0 = Normal (Std))
            run_from_rec_prom = 1;
    }

#endif
    if(isSSC()) {
extern int primary_is_rprom;
        if (primary_is_rprom == 1)
            run_from_rec_prom = 1;
    }

    if((qt_read) || (ft_read) || (diag_read) || (diag_jumper)) {
        if( diag_read && ft_read) // if both  set run short memory tests
            shortMem = 1;
        else
            shortMem = 0;
        /* Need to check to see if we are running from Std PROM else NO Jump */
        printf(" - debug - Got here cuz Diag Environments set or Diag Jumper installed \n");
        printf(" - debug - got qt = %x ft = %x diag = %x diag_Jumper = %x\n",qt_read,ft_read,diag_read,diag_jumper);
        printf(" - debug - We are running from %s PROM \n",(run_from_rec_prom == 1)? "Recovery":"Stardard");

//        if(!run_from_rec_prom) {
//           if(!diag_jumper) {   // TXRX and FP don't work if the Diag Jumper is set!!! weird
//              printf(" Jump to DIAG Partition \n");
//              if(isTXRX() || isFP()) {
//                if(isTXRX())
//                  delay_us(500); // seems that if both start at the same time I get the hung condition
//                (*diag_jump)();
//              }
//           }
//        }
    }

    s = getenv ("autoboot");
    if (failure) {
        printf ("NOTE: power-on tests reported: %s\n", failure);
        if (s)
          printf ("      autoboot skipped\n");
#ifdef INET
        if (neterr)
          printf ("      network disabled\n");
#endif
        return;
    }
    autorun (s);

// WTG  Set OK LED = AMBER and FLT LED to OFF = No tests, and No Autoload
    if (isSSC()) {
        led_set(LED_OK_AMBER);
        led_set(LED_FAULT_OFF);
    }

// Enter Diag Menu
    if( (!(qt_read)) && (!(ft_read)) && (diag_read) ) {
        // WTG  Set  OK red, and FLT Led's to amber = running menu
        if (isSSC()) {
            led_set(LED_OK_RED);
            led_set(LED_FAULT_AMBER);
        }
		printf("To abort running Diags --, hit control-E to bypass\n");
		for (i=0; i<80; i++) {
			eee_poll(1);
			if (char_avail()) {
				if (getchar() == CNTRL('e')) {
					printf("\nDiag User intervention, continue to PROM monitor\n");
					diag_set_mod_state(myModId, 7); // LOAD_BC_STATE_DIAG_STOPPED "sm-loader/load-bobcat.h"
                    pmon_cmSetCPUStatePromNoAutoboot();
					return;
				}
			}
			printf(".");
			delay_us(50000);
		}
		printf("\n\n");
        
        // start up menu
        ad_menu();

        // when done just exit to prom monitor
        pmon_cmSetCPUStatePromNoAutoboot();
        if (isSSC()) {
            led_set(LED_OK_AMBER); // signal going to Prom monitor
            led_set(LED_FAULT_OFF); 
        }
        return;
    }
/* wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww */
    //  WTG  -------------------------------------------------------
    // Here we do the same type of checking and run diags passing messages (States) to 
    // the SSC and do just like the autoload function.. Not the best but it's a start
    if((qt_read) || (ft_read) || (diag_read) || (diag_jumper)) {
		/*
		 * if the user hits Cntrl-E, and breaks out of auto daigs we need to 
		 * tell someone that diags was STOPPED on a module and Not Pass Go(Autoload)
		 * Need the ID of who is running and the set status for all to read
		 */

        // save a copy of the autoload env. Diags will set autoload if all tests Pass
	    if (env_autoload()) 
            doAutoLoadAfterDiags = 1;
        else 
            doAutoLoadAfterDiags = 0;
            
        // WTG  Set Both OK and FLT Led's to amber = running tests
        if (isSSC()) {
            myModId = 0; //LOAD_BC_SSC_ID in " sm-laoder/load-bobcat.h "
            led_set(LED_OK_AMBER);
            led_set(LED_FAULT_AMBER);
        }
        else if (isTXRX())
            myModId = 1; //LOAD_BC_TXRX_ID in " sm-laoder/load-bobcat.h "
        else if (isFP())
            myModId = 2; //LOAD_BC_RF_ID in " sm-laoder/load-bobcat.h "
        else
			myModId = 0; // This will stop all Diags from passing if we don't know who we are.

		/*
		 * See if the user hits Cntrl-E, and breaks out of autoloading
		 */
        runAutoDiagsFlag = 1;  // We are ready but see if user wants to get out.
		printf("To abort running Diags --, hit control-E to bypass\n");
		for (i=0; i<80; i++) {
			eee_poll(1);
			if (char_avail()) {
				if (getchar() == CNTRL('e')) {
					printf("\nDiag User intervention, continue to PROM monitor\n");
					diag_set_mod_state(myModId, 7); // LOAD_BC_STATE_DIAG_STOPPED "sm-loader/load-bobcat.h"
					runAutoDiagsFlag =0;  //signal to bypass
					break;
				}
			}
			printf(".");
			delay_us(50000);
		}

		printf("\n\n");

		/*
		 *  run diags automatically
		 */
        if(runAutoDiagsFlag)
		    diag_status = autodiag_bc(shortMem, 1); // 1 = Always sync up (wait for all)
        else {
		    printf("Skipping Diags Auto run, Skipping Autoloader check, jump to PROM monitor\n");
			pmon_cmSetCPUStatePromNoAutoboot();
            if (isSSC()) {
                led_set(LED_OK_OFF); // signal Diags Stopped
            }
			return;
        }

    // we don't return .. we just fall through to check for autoload 
    //  WTG  -------------------------------------------------------

		if(diag_status) {   // If we got errors don't autoload return (PROM monitor)
            if (isSSC()) {
                led_set(LED_FAULT_ON); // red
                led_set(LED_OK_RED);
            }
			pmon_cmSetCPUStatePromNoAutoboot();
		    printf(" Diags Dectected Errors - Skipping Autoloader check, jump to PROM monitor\n");
			return;
        }
        else {  //  We got Good status all passed let em know
            if (isSSC()) {
                led_set(LED_FAULT_GREEN);
                led_set(LED_OK_ON); // green
            }
 
            seepenv_reset();  // set environments to default state

            // When we are done will try to run autoload if all tests pass and the
            // environment was set prior to running diags.... since we just reset it :)
            if( !(doAutoLoadAfterDiags)) {
			    pmon_cmSetCPUStatePromNoAutoboot();
			    return;
            }
        }
    }
    // If diags are run  we will try to run autoload if all tests pass


	if (env_autoload()) {
// WTG Move around LED's states so can tell what's up...
/*  * - FLT RED 
**  * - OK  off     = Autoload mode (as was before)
*/
        if (isSSC()) {
            led_set(LED_OK_OFF);
            led_set(LED_FAULT_ON);
        }
		/*
		 * See if the user hits Cntrl-E, and breaks out of autoloading
		 */
		printf("Initializing Autoloader, hit control-E to bypass\n");
		for (i=0; i<80; i++) {
			eee_poll(1);
			if (char_avail()) {
				if (getchar() == CNTRL('e')) {
					printf("User intervention, continue to PROM monitor\n");
					pmon_cmSetCPUStatePromNoAutoboot();
					return;
				}
			}
			printf(".");
			delay_us(50000);
		}

		printf("\n\n");

		/*
		 * load runtime images from backplane automatically
		 */
		autoload();
	}
	else {
		printf("Skipping Autoloader, continue to PROM monitor\n");
	}

	pmon_cmSetCPUStatePromNoAutoboot();
}

/*
 * expand(cmdline) - expand environment variables
 * entry:
 *	char *cmdline pointer to input command line
 * returns:
 *	pointer to static buffer containing expanded line.
 */
static char *expand(cmdline)
char *cmdline;
{
  char *ip, *op, *v;
  char var[256];
  static char expline[LINESZ + 8];
  extern char *getenv ();

  if (!strchr (cmdline, '$'))
      return cmdline;

  ip = cmdline;
  op = expline;
  while (*ip) {
    if (op >= &expline[sizeof(expline) - 1]) {
        printf ("Line too long after expansion\n");
        return (0);
    }

    if (*ip != '$') {
        *op++ = *ip++;
        continue;
    }

    ip++;
    if (*ip == '$') {
        *op++ = '$';
        continue;
    }

    /* get variable name */
    v = var;
    if (*ip == '{') {
        /* allow ${xxx} */
        ip++;
        while (*ip && *ip != '}') {
            *v++ = *ip++;
        }
        if (*ip && *ip != '}') {
            printf ("Variable syntax\n");
            return (0);
        }
        ip++;
    }
    else {
        /* look for $[A-Za-z0-9]* */
        while (isalpha(*ip) || isdigit(*ip))
        *v++ = *ip++;
    }

    *v = 0;
    if (!(v = getenv (var))) {
        printf ("Undefined variable %s\n", var);
        return (0);
    }

    if (op + strlen(v) >= &expline[sizeof(expline) - 1]) {
        printf ("Line too long after expansion\n");
        return (0);
    }

    while (*v)
      *op++ = *v++;
  }
  *op = '\0';
  return (expline);
}

/*************************************************************
 *  do_cmd(p)
 *      execute a command, the string pointed to by p.
 *      warning: p does get written to
 */
void
do_cmd (p)
     char           *p;
{
    char           *av[MAX_AC];	/* argument holder */
    word            ac;		/* # of arguments */
    char           *cmdlist[20], *t, tmp[11];
    int             i, nc, j, c, rptcmd;

    repeating_cmd = 0;
    if (!*p || strempty (p)) {	/* blank */
        rptcmd = matchenv ("rptcmd");
        if (rptcmd) {		/* repeat requested */
            repeating_cmd = 1;
            t = gethistn (histno - 1);
            if (rptcmd == 1)
                strcpy (p, t);	/* all cmds */
            else if (rptcmd == 2) {	/* trace only */
                if (wordsz (t) > 10)
                    return;
                getword (tmp, t);
                if (strequ (tmp, "t") || strequ (tmp, "to"))
                    strcpy (p, tmp);
                else
                    return;
            } else {
                printf ("bad rptcmd value [%s]\n", getenv ("rptcmd"));
                return;
            }
        } else
            return;
    }

    if (!(p = expand (p)))
      return;

    nc = 0;
    cmdlist[nc++] = p;
    for (; *p;) {
        c = *p;
        if (c == '\'' || c == '"') {
            p++;
            while (*p && *p != c)
                ++p;
            if (*p)
                p++;
        } else if (c == ';') {
            *p++ = 0;
            cmdlist[nc++] = p;
        } else
            p++;
    }

    for (j = 0; j < nc; j++) {
        ac = argvize (av, cmdlist[j]);
        if (ac > 0) {
            for (i = 0; CmdTable[i].name != 0; i++)
                if (strequ (CmdTable[i].name, av[0]))
                    break;

            if (CmdTable[i].name != 0) {
                int stat = -1;

                if( repeating_cmd && !CmdTable[i].repeat )
                {
                  /*
                   * This command isn't supposed to repeat.  Just return.
                   */

                  repeating_cmd = 0;
                  return;
                }
                
                if (ac < CmdTable[i].minac)
                    printf ("%s: not enough arguments\n", CmdTable[i].name);
                else if (ac > CmdTable[i].maxac)
                    printf ("%s: too many arguments\n", CmdTable[i].name);
                else
                    stat = (CmdTable[i].func) (ac, av);
                if (stat < 0)
                    printf ("usage: %s %s\n", CmdTable[i].name, CmdTable[i].opts);
                if (stat != 0)
                    break;	/* skip commands following ';' */
            } else {
                printf ("%s: Command not found.\n", av[0]);
                break;
            }
        }
    }
}

/*************************************************************
 *  help(ac,av)
 */
int
help (ac, av)
     int             ac;
     char           *av[];
{
    int             i, j, namemax, optsmax, descmax, len;
    int             ln, siz;

    namemax = optsmax = descmax = 0;
    for (i = 0; CmdTable[i].name != 0; i++) {
        len = strlen (CmdTable[i].name);
        if (len > namemax)
            namemax = len;
        len = strlen (CmdTable[i].opts);
        if (len > optsmax)
            optsmax = len;
        len = strlen (CmdTable[i].desc);
        if (len > descmax)
            descmax = len;
    }

    siz = moresz;
    ioctl (STDIN, CBREAK, NULL);

    ln = siz;
    if (ac >= 2) {		/* extended help */
        if (strequ (av[1], "*")) {	/* all commands */
            for (i = 0; CmdTable[i].name != 0; i++) {
                if (prhelp (i, &ln, siz, namemax, optsmax))
                    break;
            }
        } else {		/* specific commands */
            for (j = 1; j < ac; j++) {
                for (i = 0; CmdTable[i].name != 0; i++) {
                    if (strequ (CmdTable[i].name, av[j]))
                    break;
                }
                if (CmdTable[i].name) {
                    if (prhelp (i, &ln, siz, namemax, optsmax))
                        break;
                } else
                    printf ("%s: command not found\n", av[j]);
            }
        }
    } else {			/* general help only */
        for (i = 0; CmdTable[i].name != 0; i++) {
            printf ("%*s  %-*s", namemax, CmdTable[i].name, descmax,
                CmdTable[i].desc);
            if (i % 2 != 0)
                printf ("\n");
            else
                printf ("   ");
        }
        if (i % 2 != 0)
            printf ("\n");
    }
    return (0);
}

/*************************************************************
 *  prhelp(n,lnp,siz,namemax,optsmax)
 */
int
prhelp (n, lnp, siz, namemax, optsmax)
    int             n, *lnp, siz, namemax, optsmax;
{
    const Optdesc   *p;
    int             i;

    sprintf (prnbuf, "%*s  %-*s    %s", namemax, CmdTable[n].name, optsmax,
	     CmdTable[n].opts, CmdTable[n].desc);
    if (more (prnbuf, lnp, siz))
        return (1);

    p = CmdTable[n].optdesc;
    if (p != 0) {
        for (i = 0; p[i].name; i++) {
            sprintf (prnbuf, "%*s  %15s    %s", namemax, "",
                 p[i].name, p[i].desc);
            if (more (prnbuf, lnp, siz))
                return (1);
        }
    }
    return (0);
}

/*************************************************************
 *  shrc(adr)
 */
void
shrc (adr)
     unsigned char *adr;
{
    int             c;
    char            buf[LINESZ];

    for (;;) {
        c = getln (&adr, buf);
        if (c == 0)
            break;
        do_cmd (buf);
    }
}

/*************************************************************
 *  getln(adr,p)
 */
int
getln (adr, p)
     unsigned char **adr, *p;
{
    unsigned int    c;

    for (;;) {
        c = **adr;
        if (c == 0xff)
            c = 0;
        if (c == 0)
            break;
        (*adr)++;
        if (c == '\n')
            break;
        *p++ = c;
    }
    *p = 0;
    return (c);
}

/*************************************************************
 *  word no_cmd(ac,av)
 */
word 
no_cmd (ac, av)
     int             ac;
     char           *av[];
{

    printf ("The %s command cannot be invoked by name.\n", av[0]);
    return (1);
}

/*************************************************************
 *  closelst(lst)
 */
void
closelst (lst)
     int             lst;
{
    int             i, x;
    struct termio   tbuf;

	x=0;

    switch (lst) {
    case 0:
	if (curlst == 0)
	    return;
	x = *curlst;
	*curlst = 0;
	break;
    case 1:
	x = pmlst;
	pmlst = 0;
	break;
    case 2:
	x = clilst;
	clilst = 0;
	/* reset client terminal state to "sane" value */
	ioctl (STDIN, TCGETA, &tbuf);
	ioctl (STDIN, SETSANE, NULL);
	ioctl (STDIN, TCGETA, &clitbuf);
	ioctl (STDIN, TCSETAW, &tbuf);
	break;
    }

    for (i = 0; i < OPEN_MAX; i++) {
	if (x & 1 && i > 2)
	    close (i);
	x >>= 1;
    }
}

/*************************************************************
 *  swlst(lst)
 */
void
swlst (lst)
     int             lst;
{

    switch (lst) {
    case 1:
	if (curlst != &pmlst) {
	    if (_pmon_in_ram) {
		SBD_DISPLAY ("PRAM", CHKPNT_PRAM);
	    }
	    else {
		SBD_DISPLAY ("PMON", CHKPNT_PMON);
	    }
	    /* save client terminal state and set PMON default */
	    ioctl (STDIN, TCGETA, &clitbuf);
	    ioctl (STDIN, SETSANE, NULL);
	    curlst = &pmlst;
	}
	break;
    case 2:

	if (curlst != &clilst) {
	    SBD_DISPLAY ("RUN ", CHKPNT_RUN);
	    /* restore client terminal state */
	    ioctl (STDIN, TCSETAF, &clitbuf);
	    curlst = &clilst;
	}
	break;
    }
}


#if RAL_XXX
#ifdef R4000
struct cerrfifo	cerrfifo;

void
cerrpoll ()
{
    static const char * const msg ="\r\nWarning: correctable cache error, CacheErr=0x";
    static int incerrpoll = 0;
    volatile struct cerrfifo *ce;
    char xbuf[10];

    if (incerrpoll)
	return;
    incerrpoll = 1;

    ce = (struct cerrfifo *)((unsigned int)&cerrfifo | K1BASE);
    while (ce->ce_out != ce->ce_in) {
	unsigned int val = ce->ce_buf[ce->ce_out];
	ce->ce_out = (ce->ce_out + 1) & CERR_FIFO_MASK;
	write (2, msg, strlen (msg));
	(void) btoa (xbuf, val, 16);
	write (2, xbuf, strlen (xbuf));
	write (2, "\r\n", 2);
    }

    incerrpoll = 0;
}
#endif
#endif

