/*
 *-----------------------------------------------------------------
 *
 * Name:	memtst.c
 *
 * RCS:  $Id: memtst.c,v 1.8 2002/09/23 18:08:17 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/6/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-types.h>
#include <nfx-target.h>
#include <sys/mips.h>
#include "pmon.h"
#include <pmon/stdio.h>
#include <pmon/termio.h>
/* 
 * Unable to include libc.h  too many conflicts
 * #include <exports/libc.h>
 * Pulling in the implicit declarations
 */
extern uint64   atollx(uint64 *value, char8 *p);
extern int      random(void);
extern void     srandom(uint32 seed);
extern int32    strncmp(const char8 *str1, const char8 *str2, int32 n);

extern void ecc_view(uint32 cpu);

/* in           "code/prom-lib/termio.c" */
extern int      ioctl(int fd, int op, int * argp);
/* in           "code/prom-pmon/more-commands.c" */
extern void     signextend( address_t * addr);
/* in           "code/prom-pmon/more.c" */
extern int      dotik (int rate, int use_ret);

/* Forward refs */
int domt (unsigned int *adr, int siz);
int docmt (unsigned int *adr, int siz);
int do_brian_memtest( int argc, char *argv[]);


#define MT_PRINT 1
#define MT_NOPRINT 0

#define MB (1024 * 1024)

#ifdef PROM_SIBYTE

void ecc_exception_init(void);
void ecc_exception_stop(unsigned int cpu);
void ecc_exception_start(unsigned int cpu);

void set_tlb();
void init_tlb();
int do_dram_mt (unsigned long *adr, unsigned long siz);
extern int tlb_set(int index, unsigned long va,unsigned long pa, unsigned int size, unsigned int calg);
#endif


static unsigned int	 round_cnt=0; 
const Optdesc         mt_opts[] = {
    {"-c", "continuous test"},
    {0}};



typedef struct {
        char  *str0;
        char  *str1;
        char  *str2;
        char  *str3;
        char  *str4;
} memtest_set_t;

extern uint32 admenu_running_mtest;


int 
memtst (ac, av)
    int             ac;
    char           *av[];
{
    int             i, j, cnt, cflag, err, cachetest;
    unsigned long   adr, siz;

    cflag = 0;
    cnt = 0;
    cachetest = 0;

    /* XXX hardcode default mem range K1 48MB - 256MB */
    adr = PHYS_TO_K1 (48 * MB);
    siz = (256 - 48) * MB;

    for (i = 1; i < ac; i++) {
        if (av[i][0] == '-') {
            for (j = 1; av[i][j]; j++) {
                if (av[i][j] == 'C')
                    cachetest = 1;
                else if (av[i][j] == 'c')
                    cflag = 1;
                else {
                    printf ("%c: bad option\n", av[i][j]);
                    return (-1);
                }
            }
        } else {
            switch (cnt) {
            case 0:
                if (!get_rsa (&siz, av[i]))
                    return (-1);
                cnt++;
                break;
            case 1:
                adr = siz;
                if (!get_rsa (&siz, av[i]))
                    return (-1);
                cnt++;
                break;
            default:
                printf ("%s: Unknown argument\n", av[i]);
                return (-1);
            }
        }
    }

    /* make sure the address is word aligned */
    adr = adr & ~0x3;

    /* make sure it's an exact number of words */
    siz &= ~0x3;

#if __long64
    signextend(&adr);
#endif

    ioctl (STDIN, CBREAK, NULL);

    printf ("Testing %08lx to %08lx %s\n", adr, adr + siz, (cflag) ? "continuous" : "");
    printf ("Memory test running..  ");

    round_cnt = 0;
    if(cachetest) 
        err = docmt ((unsigned int *)adr, siz);
    else if (cflag)
        while (!(err = domt ((unsigned int *)adr, siz)));
    else
        err = domt ((unsigned int *)adr, siz);

    if (err) {
        printf ("\b\nThere were %d errors.\n", err);
        return (1);
    }
    printf ("\b\nTest passed with no errors.\n");
    return (0);
}


int
domt (adr, siz)
     unsigned int   *adr;
     int             siz;
{
    int             i, j, err, c;
    volatile unsigned int    w, *p, r, val;
    volatile unsigned long *test_adr;
    volatile unsigned long test_size;

    err = 0;
    round_cnt ++;

    /* walking ones test doing 1Meg at 128Meg intervals */
    test_adr = adr;
    if(siz < 1*MB)
        test_size = siz;  // Don't test more that what was requested
    else 
        test_size = 1*MB; // 1 Meg chunks..

    while ((unsigned long)test_adr < (unsigned long)adr+siz) {
        for (p = test_adr, i = 0; i < test_size; i += sizeof(long), p++) {
            w = 1;
            for (j = 0; j < 32; j++) {
                *p = w;
                r = *p;
                if (r != w) {
                    err++;
                    printf ("\b\nerror: addr=%08x read=%08x expected=%08x  ",p, r, w);
                    if ((c = getchar()) == CNTRL('e'))
                        return(err);
                }
                w <<= 1;
                dotik (1, 0);
            }
        }
        /* test 1MB of mem at 128M intervals */
        (unsigned long)test_adr += 128 * MB;
    }

    /* walking zero's test doing 1Meg at 128Meg intervals */
    test_adr = adr;
    if(siz < 1*MB)
        test_size = siz;  // Don't test more that what was requested
    else 
        test_size = 1*MB; // 1 Meg chunks..

    while ((unsigned long)test_adr < (unsigned long)adr+siz) {
        for (p = test_adr, i = 0; i < test_size; i += sizeof(long), p++) {
            w = 1;
            for (j = 0; j < 32; j++) {
                val = ~w;
                *p = val;
                r = *p;
                if (r != val) {
                    err++;
                    printf ("\b\nerror: addr=%08x read=%08x expected=%08x  ", p, r, val);
                    if ((c = getchar()) == CNTRL('e'))
                        return(err);
                }
                w <<= 1;
                dotik (1, 0);
            }
        }
        /* test 1MB of mem at 128M intervals */
        (unsigned long)test_adr += 128 * MB;
    }

    /* store the address in each address */
    for (p = adr, i = 0; i < siz; i += 4, p++) {
        *p = (unsigned int)p;
        dotik (1, 0);
    }

    /* check that each address contains its address */
    for (p = adr, i = 0; i < siz; i += 4, p++) {
        r = *p;
        if (r != (unsigned int)p) {
            err++;
            printf ("\b\nerror: adr=%08x read=%08x expected=%08x  ",p, r, p);
            if ((c = getchar()) == CNTRL('e'))
                return(err);
        }
        dotik (1, 0);
    }
    if (!err)
	    printf(" Test passed round %d adr 0x%08x size 0x%08x \n", round_cnt, adr, siz);
    return (err);
}


/*********************************************************************
 *  docmt do cache miss memory test
 *   This will do:
 *      write 8300.0000   8300.0200  8300.0400 ...
        read  all  8300.0000 - end address
        write 8300.0004   8300.0204  8300.0404 ...
        read all
 *  start address  + 200 (512KB) writes
 *  start address + 4 reads 
 **********************************************************************/
int
docmt (adr, siz)
     unsigned int   *adr;
     int             siz;
{
    int             i, j, err, junk, readCnt;
    volatile unsigned int    *p;
    volatile unsigned long *test_adr;
    volatile unsigned long test_end_adr;

    err = 0;
    round_cnt ++;
    test_adr = adr;
    test_end_adr = (unsigned long)adr + siz;
    readCnt, junk = 0;

    for ( (unsigned long)test_adr = (unsigned long)adr, i = 0; 
          (unsigned long)test_adr < test_end_adr;
          (unsigned long)test_adr += sizeof(unsigned long)) {

        // Write all cache blocks between start - end
        for (p = test_adr, j = 0; (unsigned long)p < test_end_adr; (unsigned long)p+=0x200, j++) {
            *p = (unsigned int)p;
            //if(!(j % 32)) {
                dotik (1, 0);
            //}
            //if( j < 10) {printf(" -Debug- write all cache %x \n",p);};
        }
        // read all mem locations from start - end
        for (p = adr, j=0; (unsigned long)p < test_end_adr; p++,j++ ) {
            readCnt++;
            //if (j < 2) { printf(" -Debug- read all from %x to %x \n", p, test_end_adr); };
            junk = *p;  // read all throw it away
            if(!(j % 16)) {
                dotik (1, 0);
            }
        } 
        if( i++ > (512 / sizeof(unsigned long))) {  // gone through all address with in a cache block..
            printf(" \n\n -- i -- %d -- break  read cnt %d -- \n\n",i,readCnt);
            break;
        }
    }

    printf("\n\n");
    printf(" Cache Miss Test round %d adr 0x%08x size 0x%08x \n", round_cnt, adr, siz);
    return (err);
}

/***********************************************************
* Pulled in do_brain_memtest() from runtime code
*
***********************************************************/


/******************************************************************
 * 
 * Name:   do_bmt_rw_local_cached 
 *
 * Description:  This is called from admenu. It sets up the parameters
 *               required to run "memtest readwrite 
 *                 local / remote   cached / uncached 
 *               tests .. 
 * Created by:   Warren Gale 
 *
 * Date Created: 4/03/07
 *
 ******************************************************************/

int
do_bmt_rw_local_cached(int local, int cached, int gtmem)
{
    uint32 rc = 0;
    memtest_set_t mstr;
    mstr.str0 = "";
    mstr.str4 = "\n";

    if( !gtmem) {
        mstr.str1 = "readwrite";

        if( local == 1)
            mstr.str2 = "local";
        else 
            mstr.str2 = "remote";
        if( cached == 1)
            mstr.str3 = "cached";
        else 
            mstr.str3 = "uncached";
    }
    else {
        mstr.str1 = "gtmem";
        mstr.str2 = "readwrite";

        if( local == 1)
            mstr.str3 = "local";
        else 
            mstr.str3 = "remote";
        if( cached == 1)
            mstr.str4 = "cached";
        else 
            mstr.str4 = "uncached";
    }
      
    rc = do_brian_memtest(5,(char **)&mstr);
    return (rc);
}


#ifdef R9000
#include <sys/gt64440r.h>
#include <exports/hwMap.h>
#include <sys/r9k-regs.h>
typedef unsigned int u_int32_t;
extern void ecc_exception_init();

/*-----------------------------------------------------------------
 * Name:    addrAdjustR9K
 *
 * Description:  This returns an address with in the R9K memory space
 *               to test .. 
 *                Lower 256 Mb of 512 is at 8000.0000 to 9000.0000
 *                Upper 256 Mb of 512 is at 1000.0000 to 2000.0000
 *               Random test will generate an address and this puts
 *               in a 256 Mb region
 *
 * Created by:   Warren Gale 
 *
 * Date Created: 4/03/07
 *
 *-----------------------------------------------------------------
 */
uint32
addrAdjustR9K(
    uint32    a
)
{

    if( a < 0x0D000000)
        return (0x83000000 + a);
    return ( 0x10000000 | a);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:   do_brian_memtest on RM9K and Marvell memory
 *
 * Created by:    Rick Lund 
 * Modified by:   Warren Gale
 *
 * Date Created:    1/09/03
 * Date Modified:   4/09/07
 *
 *-----------------------------------------------------------------
 */

int
do_brian_memtest( int argc, char *argv[])
{
    uint32    n = 0, cached = 0, use_random = 0, verify = 0, readwrite = 0, remote = 0, gtmem = 0;
    uint32    read = 0, write = 0;
    uint32    data,rc, read_data, read2_data;
    uint32    addr_base, test_addr, addr_offset;
    uint32    testMax, testMin, testMaxData, testMinData;
    uint32    i;
    uint64 readwrite_write = 0;
    uint64 readwrite_read = 0;
    uint32 pass_cnt = 0;
    uint32 mterror_cnt = 0;

    uint32 tmemorysize =0;

    uint32  seed = _mips_mfc0(C0_COUNT);
    printf("random seed = 0x%x\n", seed);
    srandom(seed);


    printf(" Got Command line args of %s %s %s %s\n",argv[1],argv[2],argv[3],argv[4]);

    if (strncmp(argv[1], "gtmem", 2) == 0) {
        gtmem= 1;
        n = 1;    // arg incrementer
    }
    if (strncmp(argv[(1+n)], "readwrite", 6) == 0) {
        readwrite = 1;
    }
    if (strncmp(argv[(1+n)], "verify", 2) == 0) {
        verify = 1;
    }
    if( (readwrite ==0) && (verify ==0))
        return -1;

    if (strncmp(argv[(2+n)], "remote", 3) == 0) {
        remote = 1;
        return -1;
    }
    else
        remote = 0;

    if (remote) {
        return -1;
    }
    else {
        if (strncmp(argv[(3+n)], "cached", 2) == 0) {
            cached = 1;
            if(gtmem) {
                addr_base = 0x40400000;  // length of 3fc0.0000
                tmemorysize = 0x3fc00000;
            }
            else {
                /* 
                 * 8000.0000 - 9000.0000 lower 256 MB of 512
                 * 1000.0000 - 2000.0000 upper 256 MB of 512
                 */ 
                addr_base = 0x83000000;  // length of 1D00.0000
                tmemorysize = 0x1D000000;
            }

        }
        else if (strncmp(argv[(3+n)], "uncached", 2) == 0) {
            cached = 0;
            if(gtmem) {
                addr_base = 0xC0400000;
                tmemorysize = 0x3fc00000;
            }
            else {
                addr_base = 0xA3000000; // length 0f 0d00.0000
                tmemorysize = 0x0D000000;
            }
        }
        else
            return -1;
    }

    use_random = 1;
    if(!admenu_running_mtest) {
        if (argc == (5+n)) {
            use_random = 0;
            rc = atollx(&data, argv[(4+n)]);
        }
    }

    /*
     * Clear Error conditions, counters, and Enable ECC
     */
    RM9K_WRITE( R9K_XECC, (R9K_XECC_SECC | R9K_XECC_MECC)); 
    ecc_exception_init();   /* clears counters and sets up interrupt handler on GT*/
    watchdog_disable();
    printf(" Disable Watchdog\n");

    if (use_random)
        printf("memtest: %s  %s, %s, %s, RANDOM ADDR, RANDOM DATA\n", (gtmem) ? "GT-MEM" : "R9K-MEM" ,
                    (readwrite) ? "READ/WRITE" : ((verify) ? "VERIFY" : ((read) ? "READ" : "WRITE")),
                    (remote) ? "REMOTE" : "LOCAL", 
                    (cached) ? "CACHED" : "UNCACHED");
    else
        printf("memtest: %s  %s, %s, %s, RANDOM ADDR, DATA = 0x%llx\n", (gtmem) ? "GT-MEM" : "R9K-MEM" ,
                    (readwrite) ? "READ/WRITE" : ((verify) ? "VERIFY" : ((read) ? "READ" : "WRITE")),
                    (remote) ? "REMOTE" : "LOCAL", 
                    (cached) ? "CACHED" : "UNCACHED",
                    data);
    testMax = 0;
    testMin = 0xFFFFFFFF;
    testMaxData = 0;
    testMinData = 0xFFFFFFFF;

    printf("testing from %p, %lld MB\n", (void *)addr_base, tmemorysize/0x00100000);

    if (verify || readwrite) {
    /*
     *  In the case of RM9K Prom clears out memory during boot up.
     *  No need to clear all of memory before starting test.
     */

        while (!char_avail()) {

            printf("pass_cnt = %d\n", pass_cnt++);

            for (i=0; i<0x100000; i++) {
                int32    ran = random();
                uint32    uran = (uint32)ran;
                if (uran >= tmemorysize)
                    continue;

                if( (!gtmem) && (cached)){
                    // R9K Memory not contigious ..
                    addr_offset = addrAdjustR9K(uran & ~0x3);
                    test_addr = addr_offset;
                }
                else {
                    addr_offset = (uran & ~0x3);
                    test_addr = addr_base | addr_offset;
                }

                if (use_random) {
                    uint32   random_low;
                    random_low = random();
                    data = random_low;
                }

                if (verify || (ran & 2)) {
                    if (test_addr > testMax) {
                        testMax = test_addr;
                        printf("testMax = 0x%lx\n", testMax);
                    }
                    if (test_addr < testMin) {
                        testMin = test_addr;
                        printf("testMin = 0x%lx\n", testMin);
                    }
                    if (data > testMaxData) {
                        testMaxData = data;
                        printf("testMaxData = 0x%lx\n", testMaxData);
                    }
                    if (data < testMinData) {
                        testMinData = data;
                        printf("testMinData = 0x%lx\n", testMinData);
                    }
                    *(volatile uint32 *)test_addr = data;
                    readwrite_write++;
                }
                else {
                    data = *(volatile uint32 *)test_addr;
                    readwrite_read++;
                }
                if (verify) {
                    read_data = *(volatile uint32 *)test_addr;
                    if (read_data != data) {
                        printf("Verify Error at %p, wrote 0x%lx, read 0x%lx, ",
                                                    (void *)test_addr, data, read_data);
                        read2_data = *(volatile uint32 *)test_addr;
                        if (read_data == read2_data) {
                            uint32    bit;
                            printf("WRITE ERROR, diff = 0x%08lx\n", read_data ^ data);
                            for (bit=0; bit<31; bit++) {
                                if ((read_data ^ data) & (uint32)(1<<bit))
                                    printf("BIT %d\n", bit);
                            }
                        }
                        else {
                            printf("READ ERROR, diff = 0x%08lx\n", read_data ^ data);
                        }
                        mterror_cnt++;
                    }
                }
            }
            if (readwrite) {
                volatile uint32    * taddr;
                uint32        offset;
    
                printf("read top %d MB memory\n",
                                    (tmemorysize - 0x00100000)/0x00100000);
                for (offset = 0; offset < tmemorysize; offset+=4) {
                    if (offset < (uint32)0x03000000)
                        continue;
                    if((!gtmem) && (cached)){
                        // R9K Memory not contigious ..
                        addr_offset = addrAdjustR9K( offset);
                        taddr = addr_offset;
                    }
                    else {
                        addr_offset = offset;
                        taddr = (uint32 *)(addr_base | addr_offset);
                    }
                    *taddr;
                }
            }

            /*
             *  Need to check Marvell Ecc error counters 
             */
            if(gtmem) {
              {
                uint32 reg;
                if( reg = GT_READ(SDRAM_ECC_ERROR_COUNTER) ) {
                    printf("GT SDRAM Err Count   = 0x%08x\n",
                    GT_READ(SDRAM_ECC_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS0_DOUBLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM DBit Err Count CS0   = 0x%08x\n",
                    GT_READ(SDRAM_CS0_DOUBLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS1_SINGLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM SBit Err Count CS1   = 0x%08x\n",
                    GT_READ(SDRAM_CS1_SINGLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS1_DOUBLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM DBit Err Count CS1   = 0x%08x\n",
                    GT_READ(SDRAM_CS1_DOUBLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS2_SINGLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM SBit Err Count CS2   = 0x%08x\n",
                    GT_READ(SDRAM_CS2_SINGLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS2_DOUBLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM DBit Err Count CS2   = 0x%08x\n",
                    GT_READ(SDRAM_CS2_DOUBLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS3_SINGLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM SBit Err Count CS3   = 0x%08x\n",
                    GT_READ(SDRAM_CS3_SINGLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
                if( reg = GT_READ(SDRAM_CS3_DOUBLE_BIT_ERROR_COUNTER) ) {
                    printf("GT SDRAM DBit Err Count CS3   = 0x%08x\n",
                    GT_READ(SDRAM_CS3_DOUBLE_BIT_ERROR_COUNTER));
                     mterror_cnt++;
                }
              }
            }
            else {
                if ( RM9K_READ(R9K_XECC) & (R9K_XECC_SECC | R9K_XECC_MECC)) {
                     printf(" RM9K ECC %s Error\n", 
                      (RM9K_READ(R9K_XECC) & R9K_XECC_SECC) ? "Single-Bit" : "Multi-Bit");
                     mterror_cnt++;
                }
            }
            /*
             * this is where we break out if admenu is running the test.
             *  We want to run it a few times since it's readom.. 
             * 5 loops if local cached, 2 for local uncached, and 1 for remote 
             * limint passes so admenu doesn't get a timeout(8 Min).
             */
            if( admenu_running_mtest) {
                if( remote == 1) {
                    if(pass_cnt >= 1)
                        break; 
                }
                else if( cached == 0) {
                    if(pass_cnt > 2)
                        break; 
                }
                else {
                    if( pass_cnt > 5) {
                        break; 
                    }
                }
            }
        }
        if(!admenu_running_mtest) {
            getchar();
        }
        if (readwrite)
            printf("passes %d: wrote %lld, read %lld\n",
                                    pass_cnt, readwrite_write, readwrite_read);
    }

    if(mterror_cnt)
        printf(" -Error! Test Failed.  got %d errors\n",mterror_cnt); 
    return(mterror_cnt);
}
#endif

/*------------------------------------------------------------------*/
#ifdef PROM_SIBYTE
// In sb1250regs.h
#define A_BUS_L2_ERRORS             0x00100208C0
#define A_BUS_MEM_IO_ERRORS         0x00100208C8

#include <exports/load-bobcat-diag.h>

unsigned long long diagForceBadAddr = 0;
unsigned long long diagForceBadECC = 0;
unsigned char diagEccBits = 0;
 
dram_memtst (ac, av)
    int             ac;
    char           *av[];
{
    int             i, j, cnt, cflag = 0, cachetest = 0, err;
    unsigned long   adr, siz;
    int             mem;

    unsigned long long sTicCount, eTicCount;
    unsigned long testtimeSec;
    // from ecc.c 
    volatile unsigned long long * berr_l2_cnts_p;
    volatile unsigned long long * berr_mie_cnts_p;
    berr_l2_cnts_p = ((volatile unsigned long long *)PHYS_TO_K1(A_BUS_L2_ERRORS));
    berr_mie_cnts_p = ((volatile unsigned long long *)PHYS_TO_K1(A_BUS_MEM_IO_ERRORS));
    signextend(&berr_l2_cnts_p);
    signextend(&berr_mie_cnts_p);

    // Enable ECC
    ecc_exception_init();   /* clears counters and sets up interrupt handler*/
    //ecc_exception_disable_intr(0);   /* clears interrupt handler (cpu) */


	mem = memorysize/MB;

	adr = 0x1000000000 + 48*MB;
    siz = (memorysize - 48*MB);

    for (i = 1; i < ac; i++) {
        if (av[i][0] == '-') {
            for (j = 1; av[i][j]; j++) {
                if (av[i][j] == 'C')
                    cachetest = 1;
                else if (av[i][j] == 'c')
                    cflag = 1;
                else {
                    printf ("%c: bad option\n", av[i][j]);
                    return (-1);
                }
            }
        } else {
            switch (cnt) {
            case 0:
                if (!get_rsa (&siz, av[i]))
                    return (-1);
                cnt++;
                break;
            case 1:
                adr = siz;
                if (!get_rsa (&siz, av[i]))
                    return (-1);
                cnt++;
                break;
            default:
                printf ("%s: Unknown argument\n", av[i]);
                return (-1);
            }
        }
    }

    /* make sure the address is word aligned */
    adr = adr & ~0x3;

    /* make sure it's an exact number of words */
    siz &= ~0x3;

#if __long64
    signextend(&adr);
#endif

    ioctl (STDIN, CBREAK, NULL);

    printf ("Testing %p to %p %s\n", adr, adr + siz, (cflag) ? "continuous" : "");
    printf ("Memory test running..  ");

    // read the zcycle_cnt register
    sTicCount = *((volatile uint64 *)PHYS_TO_K1(0x10030000));

    round_cnt = 0;
    if(cachetest)
        err = do_dram_cmt (adr, siz);
    else if (cflag) {
        while (!(err = do_dram_mt (adr, siz))) {
			while (char_avail()) {
				int	c;
				if ((c = getchar()) == CNTRL('c'))
					goto dram_test_done;
			}
		}
	}
    else
        err = do_dram_mt (adr, siz);

dram_test_done:
    //  Go read zbbus cycle count register
    eTicCount = *((volatile uint64 *)PHYS_TO_K1(0x10030000));
    //printf (" TIC Counts -Start %llx -End %llx -Total %llx \n",sTicCount,eTicCount,(eTicCount-sTicCount)); 
    testtimeSec = ((eTicCount-sTicCount) / 393216000);
    if( testtimeSec < 60)    
        printf(" Test Time = %d Seconds\n",testtimeSec); 
    else
        printf(" Test Time = %d Minutes\n",(testtimeSec / 60));


    // Go check the ECC error counters to see if there were any errors
    printf("\n Check for ECC errors: \n");
    printf(" L2 Correctable Errors = %x \n",(*berr_l2_cnts_p & 0x000000FF));
    printf(" L2 Un-Correctable Errors = %x \n",((*berr_l2_cnts_p & 0x0000FF00) > 8));
    printf(" Mem/IO Correctable Errors = %x \n",(*berr_mie_cnts_p & 0x000000FF));
    printf(" Mem/IO Un-Correctable Errors = %x \n",((*berr_mie_cnts_p & 0x0000FF00) > 8));

    if( *berr_l2_cnts_p != 0) {
        //printf ("\b\nThere were %d L2 Un Correctable ECC Errors. \n", ((*berr_l2_cnts_p & 0x0000FF00) > 8));
        //printf ("\b\nThere were %d L2 Correctable ECC Errors. \n", (*berr_l2_cnts_p & 0x000000FF) );
        dramEccErr = DRAM_ECC_L2_ERR;  // 2
        err++;
    }

    if( *berr_mie_cnts_p != 0) {
        //printf ("\b\nThere were %d Mem/IO Un Correctable ECC Errors. \n", ((*berr_mie_cnts_p & 0x0000FF00) > 8));
        //printf ("\b\nThere were %d Mem/IO Correctable ECC Errors. \n", (*berr_mie_cnts_p & 0x000000FF) );
        dramEccErr = DRAM_ECC_MEM_ERR;  // 3
        err++;
    }
    if (err) {
        printf ("\b\nThere were errors.\n");
        return (1);
    }

    // ecc_exception_stop(uint32 cpu)
    ecc_exception_stop(0);
    printf ("\b\nTest passed with no errors.\n");

    /* reset the original tlb */
    init_tlb();
    mem_map_tlb_init();
    return (0);
}

/*********************************************************************
 *  docmt do cache miss memory test
 *   This will do:
 *      write 8300.0000   8300.0200  8300.0400 ...
        read  all  8300.0000 - end address
        write 8300.0004   8300.0204  8300.0404 ...
        read all
 *  start address  + 200 (512KB) writes
 *  start address + 4 reads
 **********************************************************************/
int
do_dram_cmt (adr, siz)
     unsigned long   *adr;
     unsigned long   siz;
{
    int             i, j, err, junk, readCnt;
    int             tlbset = 0;

    unsigned long diagForceBadEcc = 0;

    volatile unsigned long *p;
    volatile unsigned long *test_adr;
    volatile unsigned long test_end_adr;

    err = 0;
    round_cnt ++;
    test_adr = adr;
    test_end_adr = (unsigned long)adr + siz;
    readCnt, junk = 0;

    if( diagForceBadAddr != 0)
        diagForceBadEcc = 1;

    // WTG  add a srub memory before runnung cache miss test
    for ( (unsigned long)test_adr = (unsigned long)adr, j = 0;
          (unsigned long)test_adr < test_end_adr;
          (unsigned long)test_adr += sizeof(unsigned long), j++) {

        if ((tlbset != 1) && ((unsigned int)test_adr < (unsigned int)2048*MB)) {
            set_tlb1();
            tlbset = 1;
        }
        if ((tlbset != 2) && ((unsigned int)test_adr >= (unsigned int)2048*MB)) {
            set_tlb2();
            tlbset = 2;
        }
        *test_adr = 0;
        if(!(j % 8)) {
            dotik (1, 0);
        }
    }

    watchdog_disable();
    printf(" Disable Watchdog\n");

    //  Start Cache miss part
    for ( (unsigned long)test_adr = (unsigned long)adr, i = 0;
          (unsigned long)test_adr < test_end_adr;
          (unsigned long)test_adr += sizeof(unsigned long)) {

        // give some one a chance to break out of test.. It's  Very long
        if (char_avail()) {
            int     c;
            if ((c = getchar()) == CNTRL('c'))
                break;
            }

        // Write all cache blocks between start - end
        for (p = test_adr; (unsigned long)p < test_end_adr; (unsigned long)p+=0x200) {
            if ((tlbset != 1) && ((unsigned int)p < (unsigned int)2048*MB)) {
                set_tlb1_tp(MT_NOPRINT);
                tlbset = 1;
            }
            if ((tlbset != 2) && ((unsigned int)p >= (unsigned int)2048*MB)) {
                set_tlb2_tp(MT_NOPRINT);
                tlbset = 2;
            }

            *p = (unsigned int)p;
////////////////
                if( diagForceBadEcc ) {
                    if ( p == (unsigned long long *)diagForceBadAddr) {
                        if((unsigned long long)p & 0x20)   //which memory controller based on bit 5 (interleave)
                            // *(volatile unsigned long long *)PHYS_TO_K1(0x10052400) = diagForceBadECC;
                            *(volatile unsigned long long *)PHYS_TO_K1(0x10052420) = diagEccBits;
                        else
                            // *(volatile unsigned long long *)PHYS_TO_K1(0x10051400) = diagForceBadECC;
                            *(volatile unsigned long long *)PHYS_TO_K1(0x10051420) = diagEccBits;
                        // Write data with BAD ECC and then clear the ECC Test Registers
                        *p = (unsigned long)p;
                        diagForceBadEcc = 0;
                        printf("\n -- force bad ECC at = 0x%llx  diagEccBits = 0x%x \n",
                                p, (unsigned long )diagEccBits );
                        *(volatile unsigned long long *)PHYS_TO_K1(0x10052420) = 0;
                        *(volatile unsigned long long *)PHYS_TO_K1(0x10051420) = 0;
                        flush_cache (DCACHE);
                        // read it to give me the error
                        diagForceBadECC = *p;
                    }
                }
//////////////
        }

        // read all mem locations from start - end
        for (p = adr; (unsigned long)p < test_end_adr; p++ ) {
            if ((tlbset != 1) && ((unsigned int)p < (unsigned int)2048*MB)) {
                set_tlb1_tp(MT_NOPRINT);
                tlbset = 1;
            }
            if ((tlbset != 2) && ((unsigned int)p >= (unsigned int)2048*MB)) {
                set_tlb2_tp(MT_NOPRINT);
                tlbset = 2;
            }

            readCnt++;
            junk = *p;  // read all throw it away
        }

        if( i++ > (512 / sizeof(unsigned long))) {  // gone through all address with in a cache block..
            //printf(" \n\n -- i -- %d -- break  read cnt %d -- \n\n",i,readCnt);
            break;
        }
    }

    printf("\n\n");
    printf(" Cache Miss Test round %d adr 0x%08x size 0x%08x \n", round_cnt, adr, siz);
    return (err);
}

/*------------------------------------------------------------------*/
int 
do_dram_mt (adr, siz)
     unsigned long   *adr;
     unsigned long    siz;
{
    int             j, err, temp;
    volatile unsigned long	val, r, w;
    volatile unsigned long	* test_adr, *p;
	unsigned long i;
	int			tlbset = 0;
    unsigned long diagForceBadEcc = 0;

    err = 0;
    round_cnt ++;

    /* walking ones test */
	test_adr = adr;
	while ((unsigned long)test_adr < (unsigned long)adr+siz) {

		if ((tlbset != 1) && ((unsigned int)test_adr < (unsigned int)2048*MB)) {
			set_tlb1();
			tlbset = 1;
		}
		if ((tlbset != 2) && ((unsigned int)test_adr >= (unsigned int)2048*MB)) {
			set_tlb2();
			tlbset = 2;
		}

//		printf("   1's: test_adr = %p\n", test_adr);
		for (p = test_adr, i = 0; i < 1*MB; i += sizeof(long), p++) {
			w = 1;
			for (j = 0; j < sizeof(long)*8; j++) {
				*p = w;
				r = *p;
				if (r != w) {
					err++;
                                        if( err == 1) { diagDramErrorAddr = p; };
					printf ("\b\nerror: addr=%p read=%lx expected=%lx  ",
						p, r, w);
					return(err);
				}
				w <<= 1;
				dotik (1, 0);
			}
		}

		/* test 1MB of mem at 128M intervals */
		(unsigned long)test_adr += 128 * MB;
	}
	
    /* walking zeros test */
	test_adr = adr;
	while ((unsigned long)test_adr < (unsigned long)adr+siz) {

		if ((tlbset != 1) && ((unsigned int)test_adr < (unsigned int)2048*MB)) {
			set_tlb1();
			tlbset = 1;
		}
		if ((tlbset != 2) && ((unsigned int)test_adr >= (unsigned int)2048*MB)) {
			set_tlb2();
			tlbset = 2;
		}

//		printf("   0's: test_adr = %p\n", test_adr);
		for (p = test_adr, i = 0; i < 1*MB; i += sizeof(long), p++) {
			w = 1;
			for (j = 0; j < sizeof(long)*8; j++) {
				val = ~w;
				*p = val;
				r = *p;
				if (r != val) {
					err++;
                                        if( err == 1) { diagDramErrorAddr = p; };
					printf ("\b\nerror: addr=%p read=%lx expected=%lx  ",
						p, r, val);
					return(err);
				}
				w <<= 1;
				dotik (1, 0);
			}
		}

		/* test 1MB of mem at 128M intervals */
		(unsigned long)test_adr += 128 * MB;
	}

    /* store the address in each address */
    if( diagForceBadAddr != 0)
        diagForceBadEcc = 1;

    for (p = adr, i = 0; i < siz; i += sizeof(long), p++) {
		if ((tlbset != 1) && ((unsigned int)p < (unsigned int)2048*MB)) {
			set_tlb1();
			tlbset = 1;
		}
		if ((tlbset != 2) && ((unsigned int)p >= (unsigned int)2048*MB)) {
			set_tlb2();
			tlbset = 2;
		}

// Adding a way to test ECC and Diag Error reporting
//   diagForceBadEcc   =1 diagForceBadAddr = 10-X000-0000 X = Dram dimm  Correctable or Uncorrectable
        if( diagForceBadEcc ) {
            if ( p == (unsigned long long *)diagForceBadAddr) {
                if((unsigned long long)p & 0x20)   //which memory controller based on bit 5 (interleave)
                    // *(volatile unsigned long long *)PHYS_TO_K1(0x10052400) = diagForceBadECC;
                    *(volatile unsigned long long *)PHYS_TO_K1(0x10052420) = diagEccBits;
                else
                    // *(volatile unsigned long long *)PHYS_TO_K1(0x10051400) = diagForceBadECC;
                    *(volatile unsigned long long *)PHYS_TO_K1(0x10051420) = diagEccBits;
                // Write data with BAD ECC and then clear the ECC Test Registers 
                *p = (unsigned long)p;
                printf("\n -- force bad ECC at 0x%llx diagEccBits = 0x%x \n",
                        p, (unsigned long )diagEccBits ); 
                *(volatile unsigned long long *)PHYS_TO_K1(0x10052420) = 0;
                *(volatile unsigned long long *)PHYS_TO_K1(0x10051420) = 0;
                flush_cache (DCACHE);
                // read it to give me the error
                diagForceBadECC = *p;
            }
            else {
                *p = (unsigned long)p;
            }
        }
        else {
            *p = (unsigned long)p;
        }
        dotik (1, 0);
    }

    /* check that each address contains its address */
    for (p = adr, i = 0; i < siz; i += sizeof(long), p++) {
		if ((tlbset != 1) && ((unsigned int)p < (unsigned int)2048*MB)) {
			set_tlb1();
			tlbset = 1;
		}
		if ((tlbset != 2) && ((unsigned int)p >= (unsigned int)2048*MB)) {
			set_tlb2();
			tlbset = 2;
		}
        r = *p;
        if (r != (unsigned long)p) {
            err++;
            if( err == 1) { diagDramErrorAddr = p; };
            printf ("\b\nerror: adr=%p read=%lx expected=%lx  ", p, r, p);
            if( err > 100) {
                return(err);
            }
        }
        dotik (1, 0);
    }
    if (!err)
        printf(" Test passed round %d adr %p size 0x%08x \n", round_cnt, adr, siz);

    return (err);
}

/*------------------------------------------------------------------*/
void 
set_tlb1_tp(int print)
{
	int	index = 0;
	int	mem;
	unsigned long start_Vadd = 0x1000000000;
   	unsigned long start_Padd = 0x0000000000;
	unsigned int size = 0x1000000; /* 16M */
	unsigned int numBlk;
	int i;
	int	sr;
	/* Need to clear old tlb */

	/* Same memory configured for both TXRX and FP */
	mem = memorysize/MB;
        if(print) {
	    printf("(Memsize %dM)", mem);
	    printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
        }
	numBlk = 64;
	
	init_tlb();

	sr = _mips_mfc0(C0_SR);
	sr |= SR_UX;
	_mips_mtc0(C0_SR, sr);

	for(i=0;i<numBlk;i++) {
			if(i == 8)
				start_Padd = 0x80000000;
			if(i == 16)
				start_Padd = 0x90000000;
			if(i == 24)
				start_Padd = 0xC0000000;
			if(i == 32)
				start_Padd = 0x0100000000;
			tlb_set(index++, start_Vadd, start_Padd, size, CFG_C_UNCACHED);
			start_Vadd = start_Vadd + 2 * size;
			start_Padd = start_Padd + 2 * size;
	}
}

void 
set_tlb1()
{
    return(set_tlb1_tp(MT_PRINT));
}

void 
set_tlb2_tp(int print)
{
	int	index = 0;
	int	mem;
	unsigned long start_Vadd = 0x1080000000;
   	unsigned long start_Padd = 0x0140000000;
	unsigned int size = 0x1000000; /* 16M */
	unsigned int numBlk;
	int i;
	int	sr;
	/* Need to clear old tlb */

	/* Same memory configured for both TXRX and FP */
	mem = memorysize/MB;
        if( print) {
	    printf("(Memsize %dM)", mem);
    	    printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
        }
	numBlk = 64;
	
	init_tlb();

	sr = _mips_mfc0(C0_SR);
	sr |= SR_UX;
	_mips_mtc0(C0_SR, sr);

	for(i=0;i<numBlk;i++) {
			tlb_set(index++, start_Vadd, start_Padd, size, CFG_C_UNCACHED);
			start_Vadd = start_Vadd + 2 * size;
			start_Padd = start_Padd + 2 * size;
	}
}

void
set_tlb2()
{
    return( set_tlb2_tp(MT_PRINT));
}




#ifdef SIBYTE
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:    Rick Lund
 *
 * Date Created:    1/09/03
 *
 *-----------------------------------------------------------------
 */
uint64
addr_adjust(
    uint32    a
)
{
    if ((uint64)a < 0x10000000LL)
        return (uint64)a;
    if ((uint64)a < 0x30000000LL)
        return (0x70000000LL + (uint64)a);
    if ((uint64)a < 0x40000000LL)
        return (0x90000000LL + (uint64)a);
    return (0xc0000000LL + (uint64)a);
}

int
do_brian_memtest( int argc, char *argv[])
{
    uint32    read = 0, cached = 0, use_random = 0, verify = 0, readwrite = 0, remote, uart = 0, gtmem = 0;
    uint64    data,rc, read_data, read2_data;
    uint64    addr_base, test_addr, addr_offset;
    uint64    testMax, testMin, testMaxData, testMinData;
    uint32    i;
    uint64 readwrite_write = 0;
    uint64 readwrite_read = 0;
    uint32 pass_cnt = 0;
    uint32 mterror_cnt = 0;

    uint32 gtmemorysize = 0x40000000; // GT mem is 1GB

    uint32  seed = _mips_mfc0(C0_COUNT);
    printf("random seed = 0x%x\n", seed);
    srandom(seed);

    printf(" Got Command line args of %s %s %s\n",argv[1],argv[2],argv[3]);

    if (strncmp(argv[1], "readwrite", 6) == 0) {
        readwrite = 1;
    }
    else if (strncmp(argv[1], "read", 2) == 0) {
        read = 1;
    }
    else if (strncmp(argv[1], "write", 2) == 0) {
        read = 0;
    }
    else if (strncmp(argv[1], "verify", 2) == 0) {
        verify = 1;
    }
    else if (strncmp(argv[1], "uart", 2) == 0) {
        uart = 1;
    }
    else if (strncmp(argv[1], "gtmem", 2) == 0) {
        gtmem= 1;
    }
    else
        return -1;

    if (strncmp(argv[2], "remote", 3) == 0)
        remote = 1;
    else
        remote = 0;

    if (uart) {
        volatile uint64    * u1, * u2;

        if (remote)
            addr_base = 0x900000E000000000LL;
        else
            addr_base = 0x9000000000000000LL;
        
        u1 = (volatile uint64 *)(addr_base + 0x10060120);
        u2 = (volatile uint64 *)(addr_base + 0x10060220);

printf("reading uart status regs from %p and %p\n", u1, u2);

        while (!char_avail()) {
            printf("pass_cnt = %d\n", pass_cnt++);
            for (i=0; i<0x1000000; i++) {
                *u1;
                *u2;
            }
        }

        getchar();

        return (mterror_cnt);
    }

    if( !gtmem) { // not GT MEM
        if (remote) {
            if (strncmp(argv[3], "cached", 2) == 0) {
                cached = 1;
                addr_base = 0x900000E000000000LL;
            }
            else if (strncmp(argv[3], "uncached", 2) == 0) {
                cached = 0;
                addr_base = 0x900000E000000000LL;
            }
            else
                return -1;
        }
        else {
            if (strncmp(argv[3], "cached", 2) == 0) {
                cached = 1;
                addr_base = 0xA800000000000000LL;
            }
            else if (strncmp(argv[3], "uncached", 2) == 0) {
                cached = 0;
                addr_base = 0x9000000000000000LL;
            }
            else
                return -1;
        }
        // when called from admenu always do random
        // parameter passing is off by 1 when called from admemu
        use_random = 1;
        if( !admenu_running_mtest) {
            if (argc == 5) {
                use_random = 0;
                rc = atollx(&data, argv[4]);
            }
        }
    }
    else { // we are doing GT Memory over the PCI Bus
        gtmem = 1;
        remote = 0;
        cached = 0;
        addr_base = 0x900000F841000000LL;
        if (strncmp(argv[2], "readwrite", 6) == 0) {
            readwrite = 1;
        }
        else if (strncmp(argv[2], "read", 2) == 0) {
            read = 1;
        }
        else if (strncmp(argv[2], "write", 2) == 0) {
            read = 0;
        }
        else if (strncmp(argv[2], "verify", 2) == 0) {
            verify = 1;
        }
        use_random = 1;
        if (argc == 6) {
            use_random = 0;
            rc = atollx(&data, argv[5]);
        }
    }

    // Enable ECC
    ecc_exception_init();   /* clears counters and sets up interrupt handler*/
    // ecc_exception_disable_intr(0);   /* clears interrupt handler (cpu) */
    watchdog_disable();
    printf(" Disable Watchdog\n");

    if (use_random)
        printf("memtest: %s  %s, %s, %s, RANDOM ADDR, RANDOM DATA\n", (gtmem) ? "GT-MEM" : NULL ,
                    (readwrite) ? "READ/WRITE" : ((verify) ? "VERIFY" : ((read) ? "READ" : "WRITE")),
                    (remote) ? "REMOTE" : "LOCAL", 
                    (cached) ? "CACHED" : "UNCACHED");
    else
        printf("memtest: %s  %s, %s, %s, RANDOM ADDR, DATA = 0x%llx\n", (gtmem) ? "GT-MEM" : NULL ,
                    (readwrite) ? "READ/WRITE" : ((verify) ? "VERIFY" : ((read) ? "READ" : "WRITE")),
                    (remote) ? "REMOTE" : "LOCAL", 
                    (cached) ? "CACHED" : "UNCACHED",
                    data);
    testMax = 0;
    testMin = 0xFFFFFFFFFFFFFFFFLL;
    testMaxData = 0;
    testMinData = 0xFFFFFFFFFFFFFFFFLL;

    if(!gtmem)
        printf("testing from %p, %lld GB\n", (void *)0x08000000, memorysize/0x40000000);
    else
        printf("testing from %p, %d GB\n", (void *)addr_base, gtmemorysize/0x40000000);



    if (verify || readwrite) {

        volatile uint64    * taddr;
        uint64        offset;
        printf("write top %lld GB memory\n",
            (memorysize - 0x40000000LL)/0x40000000LL);
        for (offset = 0; offset < memorysize; offset+=8) {
            if (offset < (uint64)0x03000000) // 48 MB
                continue;
            addr_offset = addr_adjust((uint32)offset);
            taddr = (uint64 *)(0x9000000000000000LL | addr_offset);
            *taddr = 0;
        }

        while (!char_avail()) {

            printf("pass_cnt = %d\n", pass_cnt++);

            for (i=0; i<0x100000; i++) {
                int32    ran = random();
                uint32    uran = (uint32)ran;
                if(! gtmem) {
                    /* reorder the checks her so the 2Gig to 2.25Gig memory
                     * doesn't get skipped.
                     */
                    if ((memorysize > 0x80000000) && (ran & 1))
                        uran |= 0x80000000;
                    if (uran < 0x03000000) // 48MB
                        continue;
                    if (uran >= memorysize)
                        continue;
                    addr_offset = addr_adjust(uran & ~0x7);
                    test_addr = addr_base | addr_offset;
                    if (use_random) {
                        uint64    random_high, random_low;
                        random_high = random();
                        random_low = random();
                        data = (random_high << 32) | random_low;
                    }
                }
                else {
                    if (uran >= gtmemorysize)
                        continue;
                    addr_offset = (uran & ~0x7); // make it a 64 bit address
                    test_addr = addr_base | addr_offset;
                    if (use_random) {
                        uint64    random_low;
                        random_low = random();
                        data = random_low;
                    }
                }
    

                if (verify || (ran & 2)) {
                    if (test_addr > testMax) {
                        testMax = test_addr;
                        printf("testMax = 0x%llx\n", testMax);
                    }
                    if (test_addr < testMin) {
                        testMin = test_addr;
                        printf("testMin = 0x%llx\n", testMin);
                    }
                    if (data > testMaxData) {
                        testMaxData = data;
                        printf("testMaxData = 0x%llx\n", testMaxData);
                    }
                    if (data < testMinData) {
                        testMinData = data;
                        printf("testMinData = 0x%llx\n", testMinData);
                    }
                    *(volatile uint64 *)test_addr = data;
                    readwrite_write++;
                }
                else {
                    data = *(volatile uint64 *)test_addr;
                    readwrite_read++;
                }
                if (verify) {
                    read_data = *(volatile uint64 *)test_addr;
                    if (read_data != data) {
                        printf("Verify Error at %p, wrote 0x%llx, read 0x%llx, ",
                                                    (void *)test_addr, data, read_data);
                        read2_data = *(volatile uint64 *)test_addr;
                        if (read_data == read2_data) {
                            uint32    bit;
                            printf("WRITE ERROR, diff = 0x%016llx\n", read_data ^ data);
                            for (bit=0; bit<63; bit++) {
                                if ((read_data ^ data) & (uint64)(1LL<<bit))
                                    printf("BIT %d\n", bit);
                            }
                        }
                        else {
                            printf("READ ERROR, diff = 0x%016llx\n", read_data ^ data);
                        }
                        mterror_cnt++;
                    }
                }
            }
            if (readwrite) {
                volatile uint64    * taddr;
                uint64        offset;
    
                if(!gtmem) {
                    printf("read top %lld GB memory\n",
                                    (memorysize - 0x40000000LL)/0x40000000LL);
                    for (offset = 0; offset < memorysize; offset+=8) {
                        if (offset < (uint64)0x10000000)
                            continue;
                        addr_offset = addr_adjust((uint32)offset);
                        taddr = (uint64 *)(addr_base | addr_offset);
                        *taddr;
                    }
                }
                else { // doing GT-MEM read all memory
                    for (offset = 0; offset < gtmemorysize; offset+=8) {
                        addr_offset = (uint32)offset;
                        taddr = (uint64 *)(addr_base | addr_offset);
                        *taddr;
                    }
                }
            }

            // want to check the Error counters 
            {
            uint64    reg;
            if ((reg = *(volatile uint64 *)PHYS_TO_K1(0x100208c8)) &
                                                            0x0000000000FFFFFF) {
                printf("Bus Watcher Memory and I/O Error Counter = 0x%016llx\n",
                                                                            reg);
                mterror_cnt++;
            }
            }
            /*
             * this is where we break out if admenu is running the test.
             *  We want to run it a few times since it's readom.. 
             * 5 loops if local cached, 2 for local uncached, and 1 for remote 
             * limint passes so admenu doesn't get a timeout(8 Min).
             */
            if( admenu_running_mtest) {
                if( remote == 1) {
                    if(pass_cnt >= 1)
                        break; 
                }
                else if( cached == 0) {
                    if(pass_cnt > 2)
                        break;
                }
                else {
                    if( pass_cnt > 5) {
                        break;
                    }
                }
            }
        }
        if(!admenu_running_mtest) {
            getchar();
        }
        if (readwrite)
            printf("passes %d: wrote %lld, read %lld\n",
                                    pass_cnt, readwrite_write, readwrite_read);
    }
    else if (read) {
        while (!char_avail()) {
            for (i=0; i<10000; i++) {
                int32    ran = random();
                uint32    uran = (uint32)ran;
                if(!gtmem) {
                    if ((memorysize > 0x80000000) && (ran & 1))
                        uran |= 0x80000000;
                    if (uran < 0x40000000)
                        continue;
                    if (uran >= memorysize)
                        continue;
                    addr_offset = addr_adjust(uran & ~0x7);
                    test_addr = addr_base | addr_offset;
                }
                else {
                    if (uran >= gtmemorysize)
                        continue;
                    addr_offset = (uran & ~0x7);
                    test_addr = addr_base | addr_offset;
    
                }
                if (test_addr > testMax) {
                    testMax = test_addr;
                    printf("testMax = 0x%llx\n", testMax);
                }
                if (test_addr < testMin) {
                    testMin = test_addr;
                    printf("testMin = 0x%llx\n", testMin);
                }
                *(volatile uint64 *)test_addr;
            }
        }
        getchar();
    }
    else {
        while (!char_avail()) {
            for (i=0; i<10000; i++) {
                int32    ran = random();

                if(!gtmem) {
                    if ((memorysize > 0x80000000) && (ran & 1))
                        ran |= 0x80000000;
                    if (ran < 0x03000000)
                        continue;
                    if (ran >= memorysize)
                        continue;
                    addr_offset = addr_adjust(ran & ~0x7);
                    test_addr = addr_base | addr_offset;
                    if (use_random) {
                        uint64    random_high, random_low;
                        random_high = random();
                        random_low = random();
                        data = (random_high << 32) | random_low;
                    }
                }
                else {
                    if (ran >= gtmemorysize)
                        continue;
                    addr_offset = (ran & ~0x7);
                    test_addr = addr_base | addr_offset;
                }
                if (test_addr > testMax) {
                    testMax = test_addr;
                    printf("testMax = 0x%llx\n", testMax);
                }
                if (test_addr < testMin) {
                    testMin = test_addr;
                    printf("testMin = 0x%llx\n", testMin);
                }
                if (data > testMaxData) {
                    testMaxData = data;
                    printf("testMaxData = 0x%llx\n", testMaxData);
                }
                if (data < testMinData) {
                    testMinData = data;
                    printf("testMinData = 0x%llx\n", testMinData);
                }
                *(volatile uint64 *)test_addr = data;
            }
        }
        getchar();
    }

    if(mterror_cnt)
        printf(" -Error! Test Failed.  got %d errors\n",mterror_cnt); 

    return(mterror_cnt);
}
#endif

#endif

