/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: boot_block.S,v 1.3 2002/05/23 18:54:51 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/tx3927.h>

#define BB_SIG_OFF				0x0
#define BB_VER_OFF				0x4
#define BB_JB_OFFSET_OFF		0x8
#define BB_TOSH_MEMSIZE_OFF		0xc
#define BB_TOSH_NUM_BANKS_OFF	0x10
#define BB_TOSH_SDCCR0_OFF		0x14
#define BB_TOSH_SDCCR1_OFF		0x18

#define JB_SIG_OFF				0x0
#define JB_SSC_MEMSIZE_OFF		0x4
#define JB_SSC_REV_OFF			0x8
#define JB_SSC_MSG_BASE_OFF		0xc
#define JB_SSC_MSG_LEN_OFF		0x10
#define JB_TOSH_ENTRY_OFFSET_OFF	0x14
#define JB_TOSH_MEMSIZE_OFF		0x18

#define TOSHIBA_BB_VER  1
#define TOSHIBA_BB_SIG  0x19283746
#define TOSHIBA_JB_SIG  0x46372819

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *			Reconfigures memory and
 *			Returns jump_block_addr in a0
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
	.globl	get_boot_block
	.ent	get_boot_block
get_boot_block:
	la		a0,0xa0000000

	/*
	 * Wait for signature
	 */
	lw		t0,BB_SIG_OFF(a0)
	sw		zero,BB_SIG_OFF(a0)

1:
	lw		t0,BB_SIG_OFF(a0)
	bne		t0,TOSHIBA_BB_SIG,1b

	/*
	 * Get memory config and reconfigure
	 */
	lw		t1,BB_TOSH_SDCCR0_OFF(a0)
	la		t0,TX3927_SDCCR0
	sw		t1,0(t0)

	lw		t1,BB_TOSH_SDCCR1_OFF(a0)
	la		t0,TX3927_SDCCR1
	sw		t1,0(t0)

	/*
	 * Get jump block address
	 */
	lw		t0,BB_JB_OFFSET_OFF(a0)
	add		a0,t0,a0

	j		ra

	.end	get_boot_block

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
	.globl	get_jump_block
	.ent	get_jump_block
get_jump_block:
1:
	lw		t0,JB_SIG_OFF(a0)
	bne		t0,TOSHIBA_JB_SIG,1b

	lw		v0,JB_TOSH_ENTRY_OFFSET_OFF(a0)
	add		v0,0x80000000

	j		ra

	.end	get_jump_block

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
LEAF(get_memorysize)
	la		v0,0xa0000000
	lw		v0,BB_TOSH_MEMSIZE_OFF(v0)
	jr		ra
END(get_memorysize)

