/*
 *------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Name         : am-agent-api.h
 *
 * Description  : Interface and API for the Agile Mirror Agent (AM agent).
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 25-SEP-2001
 *
 * Modified     : 1/23/02    Nitin Jain      New messages based on design
 *
 *------------------------------------------------------------------------------
 */
#ifndef _AM_AGENT_API_H_
#define _AM_AGENT_API_H_

/*
 *------------------------------------------------------------------------------
 * Include files
 *------------------------------------------------------------------------------
 */

#include "exports/evm-api.h"
#include "exports/elog-api.h"


#define AM_DBG_PRINT(_par_) E_LOG(class_1, LOG_INFO, amAppId, 0, 0, 0, _par_)
#define AM_ERR_PRINT(_par_) E_LOG(class_1, LOG_ERR, amAppId, 0, 0, 0, _par_)
#define AM_FCT_PRINT(_par_) E_LOG(class_1, LOG_INFO, amAppId, 0, 0, 0, _par_)

/*
 * Commun type definition.
 */

#define 	AM_MAX_AUTH_STRING    		32
#define 	MAX_SRC_MIRROR_ENTRY 		64
#define 	MAX_TGT_HOST_ENTRY 			32
#define 	AM_MAX_NAME					256

typedef char8   am_rc_t;    // NFX_OK, NFX_ERR

typedef char    am_snapName_t[FS_SNAPNAME_MAX];

typedef struct
{
	char 			amName[AM_MAX_NAME];		// Mirror name
	uint32			adminState;		  			// 1 = enabled; 0 = disabled

	uint32			operState;		 			// 0 = Idle; 1 = InProgress		
#define AM_OPER_STATE_IDLE			1
#define AM_OPER_STATE_INPROGRESS	2
	uint64			amApplHdl;					// AM appl handle
	uint32			ctrlHdl;					// controller handle 
												// table entry index

	uint32			numMirrorStarted;			// stats
	uint32			numMirrorCompleted;			// stats
	uint32			numMirrorAborted;			// stats


	uint64			minBitMap;					// frequency of mirroring
	uint32			hourBitMap;
	uint32			dateBitMap;
	uint32			monthBitMap; 
	uint32			dayBitMap;

#define SET_BIT(var, bit) (var |= (1ull << bit))
#define GET_BIT(var, bit) (var & (1ull << bit))

    uint32          srcAddr;      				// local IP @ (source)
    uint32          tgtAddr;      				// remote IP @ (target)

	char            srcFsysName[VOL_MAX_NAME];	// source volume
	char            tgtFsysName[VOL_MAX_NAME];	// target volume

	evm_volId_t		srcVolId;
	evm_volId_t		tgtVolId;

    uint32          maxDataRate;  				// max data transfer rate
} am_srcConfig_t;

typedef struct
{
    uint32     allowedHost[MAX_TGT_HOST_ENTRY];
} am_tgtConfig_t;

/*
 * Message type for interface with other modules.
 */
typedef enum
{
    // 1. Interface between the AM controller and the AM agent.
    //    AM_MSG_BASE ----> 9376
    AM_AGENT_LISTEN_REQ = AM_MSG_BASE,   // listen on well-known port to DM
    AM_AGENT_LISTEN_RSP,    // Generated by AM agent when DM returns success

	AM_CREATE_MIRROR_REQ,
	AM_CREATE_MIRROR_RSP,

	AM_DELETE_MIRROR_REQ, //80
	AM_DELETE_MIRROR_RSP,

	AM_ENABLE_MIRROR_REQ, //82
	AM_ENABLE_MIRROR_RSP,

	AM_DISABLE_MIRROR_REQ,
	AM_DISABLE_MIRROR_RSP,

	AM_ADD_HOST_REQ, //86
	AM_ADD_HOST_RSP,

	AM_DELETE_HOST_REQ,
	AM_DELETE_HOST_RSP,

	AM_SHOW_MIRROR_REQ, //90
	AM_SHOW_MIRROR_RSP,

    AM_USER_START_MIRROR_REQ,
    AM_USER_START_MIRROR_RSP,  

	AM_USER_STOP_MIRROR_REQ, //94
	AM_USER_STOP_MIRROR_RSP, //95

    AM_START_MIRROR_REQ,
    AM_START_MIRROR_RSP,

    AM_AGENT_ABORT_REQ,
    AM_AGENT_ABORT_RSP,

    AM_MIRROR_STATUS_IND,

    // 2. Interface between data mover and AM agent.
    // Look at datamover-api.h

    // 3. Interface with EFS
    // Look at fs-am-api.h and fs-api.h

    // 4. Interface with EVM
    // Look at evm-api.h

    // 5. Interface between AM-agents on target and source
	AM_GET_VOLID_REQ,
	AM_GET_VOLID_RSP,

    AM_LAST_SNAP_REQ,       // This also sends the auth info
    AM_LAST_SNAP_RSP,

    AM_SET_SNAP_REQ,
    AM_SET_SNAP_OK,
    AM_SET_SNAP_NOK,

    AM_SEND_DATA_IND,

    AM_SEND_DONE_IND,
    AM_RCVD_DONE_OK,
    AM_RCVD_DONE_NOK
} am_intfMsg_t;

/*
 * Header of the interface message between modules
 */
typedef struct
{
    am_intfMsg_t  msgType;    // the message type SHALL always be first
    uint32        ctrlHdl;    // handle to AM controller instance
							  // index for table entry
    uint64        amApplHdl;  // handle to AM agent instance
} am_intfHdr_t;

/*
 * Data structure for UI/conf related messages
 */

typedef struct
{
	am_intfMsg_t	msgType;    // the message type SHALL always be first
	int64			numBlk;		// number of entries returned for show request
	int64			result;		// result code used by response messages
	char			output[256];
	am_tgtConfig_t  tgtCfg;
	am_srcConfig_t	srcCfg[1];
} am_msg_cmd_t;

/*
 * Data structures for messages between source and target agents
 */

typedef struct
{
    am_intfMsg_t    am_msgType;
    evm_volId_t   volId;        // volume to mirror
    // char            passcode[AM_MAX_AUTH_STRING + 1];
    // Could add some more parameters to authenticate
} am_msg_hdr_t;

typedef struct
{
    am_intfMsg_t    am_msgType;
    evm_volId_t     volId;        // volume to mirror
    am_rc_t         result;
    uint32          snapNum;
    am_snapName_t   snapName;
    // char            passcode[AM_MAX_AUTH_STRING + 1];
    // Could add some more parameters to authenticate
} am_snap_msg_hdr_t;

typedef struct
{
    am_intfMsg_t    am_msgType;
    evm_volId_t     volId;
    evm_volOffset_t blkOffSet;
    evm_volSize_t   blkCnt;
} am_data_msg_hdr_t;


/*
 * Data structures for messages between AM controller and agent
 */

typedef struct
{
    am_intfHdr_t    hdr;
    uint64          lclAddr;      // Local IP address
    uint64        	lclPort;      // Well-known port
} am_listenReq_t;

typedef struct
{
    am_intfHdr_t    hdr;
    uint64         	result;   // NFX_OK if the datamover daemon was installed
} am_listenRsp_t;

typedef struct
{
    am_intfHdr_t    hdr;
	char 			amName[AM_MAX_NAME];	// Mirror name
    uint64          lclAddr;      			// local IP @ (source)
    uint64          rmtAddr;      			// remote IP @ (target)
    uint64        	rmtPort;      			// remote port (target)
    evm_volId_t     amVolId;      			// source volume
	char            tgtFsysName[VOL_MAX_NAME];  // target volume Name
    uint32          maxDataRate;  				// max data transfer rate
	uint32			snapSeqSuffix;				// Snap number suffix embedded
												// in snap name
} am_startMirrorReq_t;

typedef struct
{
    am_intfHdr_t    hdr;
	char 			amName[AM_MAX_NAME];		// Mirror name
    uint64         	result; // NFX_OK - target accepted the Snap Mirror req
} am_startMirrorRsp_t;

typedef struct
{
	am_intfHdr_t    hdr;
	char            amName[AM_MAX_NAME];        // Mirror name
	char            tgtFsysName[VOL_MAX_NAME];  // target volume Name
} am_getVolIdReq_t;

typedef struct
{
	am_intfHdr_t    hdr;
	char            amName[AM_MAX_NAME];        // Mirror name
	char            tgtFsysName[VOL_MAX_NAME];  // target volume Name
	evm_volId_t     tgtVolId;                   // target vol id
} am_getVolIdRsp_t;

typedef struct
{
    am_intfHdr_t    hdr;
	char 			amName[AM_MAX_NAME];		// Mirror name
    evm_volId_t     amVolId;      // volume to mirror
} am_abortAgentReq_t;

typedef struct
{
    am_intfHdr_t    hdr;
	char 			amName[AM_MAX_NAME];		// Mirror name
    evm_volId_t     amVolId;  // volume to mirror
    uint64         	result;   // NFX_OK - if aborted and rollbacked
                              // NFX_ERR - if EFS/EVM/Other errors
} am_abortAgentRsp_t;

typedef struct
{
    am_intfHdr_t    hdr;
	uint64			status;
#define		AM_MIRROR_COMPLETED		1
#define 	AM_MIRROR_ABORTED		2

} am_mirrorStatusInd_t;

#endif  // _AM_AGENT_API_H_
