/*  
 *=============================================================================
nn * File Name	: bp-mac-addr.c
 *
 *  Description	:    
 *
 *  Created		: 04 January 2002
 *
 *  Author		: Frank Salzmann
 *
 *  RCS			: $Id: bp-mac-addr.c,v 1.5 2002/11/07 01:23:36 rick Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2002
 *=============================================================================
 */
#include <nfx-incl.h>
#include <exports/backplane-api.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>


#define BP_DTYPE_DATA		EEE_USER_DATA
#define BP_DTYPE_CONTROL	EEE_CONTROL
#define BP_DTYPE_MGMT		EEE_MGMT	

typedef union _bp_mac_addr {
	char s[6];
	struct {
		uchar8 resv1;
		uchar8 resv2;
		uchar8 slot;
		uchar8 cpu;
		uchar8 port;
		uchar8 dtype;
	} f; 
} bp_macAddr_t;

/*
 *------------------------------------------------------------------------------
 * Function Name:	bp_getMyMacAddr
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 04, 2002
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
bp_getMyMacAddr(uint32 slot, 
				uint32 cpu, 
				uint32 port, 
				uint32 dtype, 
				char *addrStr)
{
	bp_macAddr_t *addr;

	addr = (bp_macAddr_t *)addrStr;

	addr->s[0] = 0x8;
	addr->s[1] = 0x8;
	addr->s[2] = slot;
	addr->s[3] = cpu;
	addr->s[4] = port;
	addr->s[5] = dtype;

	//printf("port: %d\n", addr->f.port);
	//printf("cpu: %d\n", addr->f.cpu);
	//printf("slot: %d\n", addr->f.slot);
	//printf("dtype: %d\n", addr->f.dtype);

	return 0;
}	// bp_get_my_mac_addr

/*
 *------------------------------------------------------------------------------
 * Function Name:	bp_getDestMacAddr
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 04, 2002
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
bp_setDestMacAddr( uint32 destPort, uint32 remotePortNum, char *buf )
{
	uint32 cpu;
	bp_macAddr_t *addr;

	addr = (bp_macAddr_t *)buf;


	switch(EEE_GET_DTYPE(destPort)) {
		case EEE_USER_DATA:
		case EEE_SAC:
		case EEE_METADATA:
			addr->s[5] = BP_DTYPE_DATA;
			break;

		case EEE_CONTROL:
			addr->s[5] = BP_DTYPE_CONTROL;
			break; 

		case EEE_MGMT:
		default:
			addr->s[5] = BP_DTYPE_MGMT;
			break;
	}

	cpu = EEE_GET_DEST_CPU(destPort);
	switch( cpu ) {

		case NFX_NFP_TXRX_ANY: 
		case NFX_NFP_FP_ANY: 
			addr->s[3] = NFX_NFP_FP2 & 0xff; 
			break;

		default:
			addr->s[3] = cpu & 0xff;
	}

	addr->s[0] = 0x8;
	addr->s[1] = 0x8;
	addr->s[2] = EEE_GET_SLOT_NUM(destPort) & 0xff;
	addr->s[4] = remotePortNum & 0xff;

	//printf("port: %d\n", addr->f.port);
	//printf("cpu: %d\n", addr->f.cpu);
	//printf("slot: %d\n", addr->f.slot);
	//printf("dtype: %d\n", addr->f.dtype);
	return 0;
}	// bp_getDestMacAddr

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
uint32
bp_getIpAddr()
{
	uint32	ip;

	ip =(192 << 24) |
		(168 << 16) |
		(192 << 8) |
		((nfxMySlotId+1) << 4) | ((nfxMyCPUId+1));

	return ip;
}

