/*
 *-----------------------------------------------------------------
 *
 * Name:	bp_init_ssc.c
 *
 * RCS:  $Id: bp_init_ssc.c,v 1.14 2002/11/19 20:02:19 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:		Tony Wong
 *
 * Date Created:	01/04/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/backplane-api.h>

#ifdef AGILE_HARDWARE
#ifndef SIBYTE
#include <sys/mips.h>
#include <exports/hwMap.h>
#endif
#endif

#ifdef SSC_MGMT
typedef int *common_func_mii(uint32, uint32, uint32, uint32);
extern uint32 cm_ssc_rev;
extern void common_init_phy8353(common_func_mii *fp, uint32 unit, uint32 dev_id);
extern void common_init_phy8354(common_func_mii *fp, uint32 unit, uint32 dev_id);
extern void common_init_phy5228(common_func_mii *fp, uint32 unit, uint32 dev_id);
extern int gt_mii_readSMI(uint32, uint32, uint32 *);
extern int bcm_init_5600(int);
extern int bcm_init_5632(int);
extern void seepInit(void);
#endif
void bp_init_ssc(void);

void
bp_init_ssc()
{
#ifdef AGILE_HARDWARE
#ifndef SIBYTE
#ifdef SSC_MGMT
	uint32 i, data;
#endif
	extern void gt_mii_writeSMI_common(uint32, uint32, uint32, uint32);
	extern int cmic_write_miim(int, uint8, uint8, uint16);

#ifdef PROM
#ifdef PROM_R9000
	extern int gt64440mii_initModule(void);

	/* initialize Galileo MII ports */
    gt64440mii_initModule();
#else
	extern int gt64240mii_initModule(void);

	/* initialize Galileo MII ports */
    gt64240mii_initModule();
#endif

#elif defined(SSC_MGMT)

	seepInit();

	/* For REV 1 */
	if (cm_ssc_rev == 0) {

    	/* initialize 5600's 5228 phy to management plane */
        for (i = 9; i <= 16; i++) {
            common_init_phy5228((common_func_mii *)cmic_write_miim, 0, i);
        }

    	/* initialize 5632's 8353 phy to control/data plane */
        for (i = 0; i <= 7; i++) {
            common_init_phy8353((common_func_mii *)cmic_write_miim, 0, i);
        }
        for (i = 0x1c; i <= 0x1f; i++) {
            common_init_phy8353((common_func_mii *)cmic_write_miim, 0, i);
        }
	}
	else
	/* For REV 2 or higher */
	{
    	/* initialize 5600's two 5228 phys to management plane */
        for (i = 1; i <= 16; i++) {
            common_init_phy5228((common_func_mii *)cmic_write_miim, 0, i);
        }

    	/* initialize 5632's six 8353/8354 phys to control/data plane */
        for (i = 0; i < 4; i++) {
            if (gt_mii_readSMI(0x04+i, 0x03, &data) == NFX_OK) {
                if (((data & 0x0000ffff) == 0x8050) || 
                    ((data & 0x0000ffff) == 0x8051))
                {
                    common_init_phy8354((common_func_mii *)
								gt_mii_writeSMI_common, 0xff, 0x04+i);
                }
                else {
                    common_init_phy8353((common_func_mii *)
								gt_mii_writeSMI_common, 0xff, 0x04+i);
                }
            }
            if (gt_mii_readSMI(0x08+i, 0x03, &data) == NFX_OK) {
                if (((data & 0x0000ffff) == 0x8050) || 
                    ((data & 0x0000ffff) == 0x8051))
                {
                    common_init_phy8354((common_func_mii *)
								gt_mii_writeSMI_common, 0xff, 0x08+i);
                }
                else {
                    common_init_phy8353((common_func_mii *)
								gt_mii_writeSMI_common, 0xff, 0x08+i);
                }
            }
            if (gt_mii_readSMI(0x0c+i, 0x03, &data) == NFX_OK) {
                if (((data & 0x0000ffff) == 0x8050) || 
                    ((data & 0x0000ffff) == 0x8051))
                {
                    common_init_phy8354((common_func_mii *)
								gt_mii_writeSMI_common, 0xff, 0x0c+i);
                }
                else {
                    common_init_phy8353((common_func_mii *)
								gt_mii_writeSMI_common, 0xff, 0x0c+i);
                }
            }
        }
	}
	bcm_init_5600(0);
	bcm_init_5632(0);
#endif
#endif // SIBYTE
#endif	/* AGILE_HARDWARE */
}
