/*
 *=============================================================================
 * File Name   :    bcm5600-l2.c
 *
 *  RCS: $Id: bcm5600-l2.c,v 1.5 2002/10/09 01:02:40 tonyw Exp $
 *
 *  Description:	Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <nfx-incl.h>

#include "bcm5600-soctypes.h"
#include "bcm5600-port.h"
#include "bcm5600-ll.h"
#include "bcm5600-cmic.h"
#include "bcm5600-schanmsg.h"
#include "bcm5600-error.h"
#include "bcm5600-l2.h"

static arl_entry_t sme_arl;
void bcm5600_l2_fix_addr();

extern int arl_insert_5600(int, arl_entry_t *);

/*
 * Function: bcm_l2_init_5600
 *
 * Description:
 *	Perform required initializations to L2 table.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *
 * Returns:
 *	BCM_E_XXX
 *
 * Notes:
 *	None.
 */
int bcm_l2_init_5600(int unit)
{
    /*
     * Insert a static ARL table entry via VLAN ID 0xfff and MAC address
     * 0xffffffffffff.  This is an errata that is required for proper
     * operation to continue when the ARL gets full.
     */
    arl_entry_t		ae;
    int			rv;

    sme_arl.mac_addr_b0 = ~0;
    sme_arl.mac_addr_b1 = ~0;
    sme_arl.mac_addr_b2 = ~0;
    sme_arl.mac_addr_b3 = ~0;
    sme_arl.mac_addr_b4 = ~0;
    sme_arl.mac_addr_b5 = ~0;
    sme_arl.vlan_tag = ~0;
	sme_arl.port = 0;

    memcpy(&ae, &sme_arl, sizeof (ae));

    ae.st = 1;
    rv = arl_insert_5600(unit, &ae);

    /* 
     * Add more calls to arl_insert_5600() here to initialize ARL 
     * table with meaningful MAC addresses, VLAN tags and ports.
     */

    bcm5600_l2_fix_addr();

    return rv; 
}

/*
 * Internal functions to translate between arl_entry_t and bcm_l2_addr_t
 */

void _bcm_l2_from_arl(bcm_l2_addr_t *l2addr, arl_entry_t *arl_entry)
{
    
    l2addr->flags = 0;
    ARL_MAC_GET(*arl_entry, l2addr->mac);

    l2addr->vid = arl_entry->vlan_tag;
    l2addr->cos_dst = arl_entry->cos_dst;

    if (arl_entry->cpu)
	l2addr->flags |= BCM_L2_COPY_TO_CPU;

    if (arl_entry->l3)
	l2addr->flags |= BCM_L2_L3LOOKUP;

    if ((arl_entry->sd_dis & 1) != 0)
	l2addr->flags |= BCM_L2_DISCARD_SRC;

    if ((arl_entry->sd_dis & 2) != 0)
	l2addr->flags |= BCM_L2_DISCARD_DST;

    if (arl_entry->st)
	l2addr->flags |= BCM_L2_STATIC;

    if (arl_entry->hit)
	l2addr->flags |= BCM_L2_HIT;

    l2addr->cos_src = arl_entry->cos_src;

    if (arl_entry->trunk)
	l2addr->flags |= BCM_L2_TRUNK_MEMBER;

    l2addr->tgid = arl_entry->tgid;
    l2addr->rtag = arl_entry->rtag;
    l2addr->port = arl_entry->port;

    if (arl_entry->scp)
	l2addr->flags |= BCM_L2_COS_SRC_PRI;
}

void _bcm_l2_to_arl(arl_entry_t *arl_entry, bcm_l2_addr_t *l2addr)
{

    memset(arl_entry, 0, sizeof (*arl_entry));

    ARL_MAC_SET(*arl_entry, l2addr->mac);

    arl_entry->vlan_tag = l2addr->vid;
    arl_entry->cos_dst = l2addr->cos_dst;

    if (l2addr->flags & BCM_L2_COPY_TO_CPU)
	arl_entry->cpu = 1;

    if (l2addr->flags & BCM_L2_L3LOOKUP)
	arl_entry->l3 = 1;

    if (l2addr->flags & BCM_L2_DISCARD_SRC)
	arl_entry->sd_dis |= 1;

    if (l2addr->flags & BCM_L2_DISCARD_DST)
	arl_entry->sd_dis |= 2;

    if (l2addr->flags & BCM_L2_STATIC)
	arl_entry->st = 1;

    if (l2addr->flags & BCM_L2_HIT)
	arl_entry->hit = 1;

    arl_entry->cos_src = l2addr->cos_src;

    if (l2addr->flags & BCM_L2_TRUNK_MEMBER)
	arl_entry->trunk = 1;

    arl_entry->tgid = l2addr->tgid;
    arl_entry->rtag = l2addr->rtag;
    arl_entry->port = l2addr->port;

    if (l2addr->flags & BCM_L2_COS_SRC_PRI)
	arl_entry->scp = 1;
}

void bcm5600_l2_fix_addr()
{
	bcm_l2_addr_t l2addr;
	mac_addr_t mac;

	mac[0] = 0x00;
	mac[1] = 0x07;
	mac[2] = 0x34;
	mac[3] = 0x00;
	mac[4] = 0xba;
	mac[5] = 0xdf;
	bcm_l2_addr_init_5600(&l2addr, mac, 1);
	if (isFC()) {
		l2addr.port = 15; /* goes to debug port on FC-NIM */
	}
	else
	if (isSSC()) {
		l2addr.port = 23; /* goes to Galileo port on SSC */
	}
	l2addr.flags = BCM_L2_STATIC;
	bcm_l2_addr_add_5600(0, &l2addr);
}


/*
 * Function: bcm_l2_addr_init_5600
 *
 * Description:
 *	Initialize a bcm_l2_addr_t to a specified MAC address and VLAN,
 *	while zeroing all other fields.
 *
 * Parameters:
 *	l2addr - Pointer to bcm_l2_addr_t
 *
 * Returns:
 *	Nothing.
 *
 * Notes:
 *	None.
 */

void bcm_l2_addr_init_5600(bcm_l2_addr_t *l2addr, mac_addr_t mac, vlan_id_t vid)
{
    memset(l2addr, 0, sizeof (*l2addr));
    memcpy(l2addr->mac, mac, sizeof (mac_addr_t));
    l2addr->vid = vid;
}

/*
 * Function: bcm_l2_addr_add_5600
 *
 * Description:
 *	Add a MAC address to the Switch Address Resolution Logic (ARL)
 *	port with the given VLAN ID and parameters.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	l2addr - Pointer to bcm_l2_addr_t containing all valid fields
 *
 * Returns:
 *	BCM_E_NONE		Success
 *	BCM_E_INTERNAL		Chip access failure
 *
 * Notes:
 */

int 
bcm_l2_addr_add_5600(int unit, bcm_l2_addr_t *l2addr)
{
    arl_entry_t		ae;
    int 		rv;
	
    /*
     * We don't check for valid port number because certain applications
     * require us to add entries with invalid port numbers.
     */

    _bcm_l2_to_arl(&ae, l2addr);
	
    /* Add the entry to the ARL */
    rv = arl_insert_5600(unit, &ae);

    return rv;
}

#if 0
/*
 * Function: bcm_l2_age_timer_set
 *
 * Description:
 *	Set the age timer for all blocks.
 *	Setting the value to 0 disables the age timer.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	age_seconds - Age timer value in seconds
 *
 * Returns:
 *	BCM_E_NONE		Success
 *	BCM_E_INTERNAL		Chip access failure
 *
 * Notes:
 *	None.
 */

int
bcm_l2_age_timer_set(int unit, int age_seconds)
{
    uint32		max_value;
    uint32		value = 0;

    max_value = GET_AGE_TIMERr_AGE_VAL(0xffffffff);

    if (age_seconds < 0 || age_seconds > max_value) {
	return BCM_E_PARAM;
    }

    if (age_seconds > 0) {
	SET_AGE_TIMERr_AGE_ENA(value, 1);
    } else {
	SET_AGE_TIMERr_AGE_ENA(value, 0);
    }

    SET_AGE_TIMERr_AGE_VAL(value, age_seconds);

    /*
     * For chips with both EPICs and GPICs, the AGE_TIMER register
     * exists only in the EPICs.
     */

#if (NUM_EPIC_5600 > 0)
    return (cmic_write_reg_blocks(unit, AGE_TIMERr, ANY_EPIC, value));
#else
    return (cmic_write_reg_blocks(unit, AGE_TIMERr, ANY_GPIC, value));
#endif
}

/*
 * Function: bcm_l2_age_timer_get
 *
 * Description:
 *	Returns the current age timer value.
 *	The value is 0 if aging is not enabled.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	age_seconds - Place to store returned age timer value in seconds
 *
 * Returns:
 *	BCM_E_NONE		Success
 *	BCM_E_INTERNAL		Chip access failure
 *
 * Notes:
 *	None.
 */

int
bcm_l2_age_timer_get(int unit, int *age_seconds)
{
    uint32		value;

    SOC_IF_ERROR_RETURN(READ_AGE_TIMERr(unit, 0, &value));

    if (GET_AGE_TIMERr_AGE_ENA(value)) {
	*age_seconds = GET_AGE_TIMERr_AGE_VAL(value);
    }
    else {
	*age_seconds = 0;
    }
    return BCM_E_NONE;
}

#endif
