/*
 *=============================================================================
 * File Name   :    bcm5600-ll.h
 *
 *  RCS: $Id: bcm5600-ll.h,v 1.1.1.1 2001/12/20 04:53:01 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef	BCM5600_LL_H
#define	BCM5600_LL_H

/*
 * Typedef:	mac_driver_t
 * Purpose:	Maps out entry points into MAC drivers.
 *
 * Notes: Supplied functions (ALL REQUIRED) are:
 *
 *	md_init	      - Initialize the MAC to a known good state.
 *	md_duplex_set - Set the current operating duplex mode for the MAC.
 *	md_duplex_get - Return the current operating duplex mode for the MAC.
 *	md_speed_set  - Set the current operating speed for the MAC
 *	md_speed_get  - Return the current operating speed for the MAC.
 *	md_pause_set  - Set current flow control mode.
 *	md_pause_get  - Return current flow control mode.
 *	md_pause_addr_set - Set current flow control address.
 *	md_pause_addr_get - Return current flow control address. 
 *	md_lb_set     - Set current Loopback mode (enable/disable) for MAC.
 *	md_lb_get     - Return current Loopback mode for MAC.
 *	md_interface_set - set MII/GMII or 10B mode.
 *	md_interface_get - retrieve MII/10B mode setting.
 *	md_ability    - Return mask of MAC abilities (from portmode.h)
 *
 * All routines return SOC_E_XXXX values.
 *
 * Macros are provided to access the fields.
 */
typedef struct mac_driver_s {
    char        *drv_name;
    int		(*md_init)(int, soc_port_t);
    int		(*md_duplex_set)(int, soc_port_t, int);
    int		(*md_duplex_get)(int, soc_port_t, int *);
    int		(*md_speed_set)(int, soc_port_t, int);
    int		(*md_speed_get)(int, soc_port_t, int *);
    int		(*md_pause_set)(int, soc_port_t, int, int);
    int		(*md_pause_get)(int, soc_port_t, int *, int *);
    int		(*md_pause_addr_set)(int, soc_port_t, mac_addr_t);
    int		(*md_pause_addr_get)(int, soc_port_t, mac_addr_t);
    int		(*md_lb_set)(int, soc_port_t, int);
    int		(*md_lb_get)(int, soc_port_t, int *);
    int		(*md_interface_set)(int, soc_port_t, int);
    int		(*md_interface_get)(int, soc_port_t, int *);
    int		(*md_ability_get)(int, soc_port_t, port_mode_t *);
} mac_driver_t;

#define	MAC_INIT(_md, _u, _p)		(_md)->md_init((_u), (_p))
#define	MAC_DUPLEX_SET(_md, _u, _p, _d)	(_md)->md_duplex_set((_u), (_p), (_d))
#define	MAC_DUPLEX_GET(_md, _u, _p, _d) (_md)->md_duplex_get((_u), (_p), (_d))
#define	MAC_SPEED_SET(_md, _u, _p, _s)  (_md)->md_speed_set((_u), (_p), (_s))
#define	MAC_SPEED_GET(_md, _u, _p, _s)  (_md)->md_speed_get((_u), (_p), (_s))
#define	MAC_PAUSE_SET(_md, _u, _p, _tx, _rx)	\
    (_md)->md_pause_set((_u), (_p), (_tx), (_rx))
#define	MAC_PAUSE_GET(_md, _u, _p, _tx, _rx)	\
    (_md)->md_pause_get((_u), (_p), (_tx), (_rx))
#define	MAC_PAUSE_ADDR_SET(_md, _u, _p, _m)	\
    (_md)->md_pause_addr_set((_u), (_p), (_m))
#define	MAC_PAUSE_ADDR_GET(_md, _u, _p, _m)	\
    (_md)->md_pause_addr_get((_u), (_p), (_m))
#define	MAC_LOOPBACK_SET(_md, _u, _p, _l) (_md)->md_lb_set((_u), (_p), (_l))
#define	MAC_LOOPBACK_GET(_md, _u, _p, _l) (_md)->md_lb_get((_u), (_p), (_l))
#define MAC_INTERFACE_SET(_md, _u, _p, _i) (_md)->md_interface_set((_u), (_p), \
								   (_i))
#define MAC_INTERFACE_GET(_md, _u, _p, _i) (_md)->md_interface_get((_u_), (_p), \
								   (_i))

#define	MAC_ABILITY_GET(_md, _u, _p, _a)	\
    (_md)->md_ability_get((_u), (_p), (_a))

extern	mac_driver_t	*soc_mac_probe(int unit, soc_port_t port);

/*
 * Typedef:	phy_driver_t
 * Purpose:	Maps out entry points into PHY drivers.
 *
 * Notes: Supplied functions (ALL REQUIRED) are:
 *
 *	pd_init		- Initialize the PHY to a known good state.
 *	pd_link_get	- Retrieve current link status from PHY.
 *	pd_enable_set	- Enable/Disable the PHY physical interface.
 *	pd_enable_get	- Retrieve current PHY Enable/disable mode.
 *	pd_duplex_set	- Set the non-negotiated duplex mode (does not
 *			  disable autonegotiation at this layer).
 *	pd_duplex_get	- Get the current operating duplicity (if 
 *			  auto-negotiation is enabled, negotiated speed
 *			  returned, otherwise fixed speed returned).
 *	pd_speed_set	- Set the non-negiotiated speed in the PHY (does
 *			  not disable auto-negotiation at this layer).
 *	pd_speed_get	- Get the current operating speed (if 
 *			  auto-negotiation is enabled, negotiated speed
 *			  returned, otherwise fixed speed returned).
 *	pd_an_set	- Enable/Disable autonegotiation
 *	pd_an_get	- Retrieve current autonegotiation settings.
 *	pd_adv_local_set- Set local advertisment (does not enable 
 *			  autonegotiation).
 *	pd_adv_local_get- Get local advertisment for auto-negotiation.
 *	pd_adv_remote_get- Retrieve remote advertisment if auto-negotiation
 *			  is enabled.
 *	pd_lb_set	- Enable/Disable PHY loopback.
 *	pd_lb_get	- Retrieve current PHY loopback mode.
 *	pd_interface_set- Set MII/10B mode
 *	pd_interface_get- Get MII/10B mode
 *	pd_ability	- Retrieve PHY abilities.
 *      pd_linkup_evt   - Call back for link up event
 *      pd_linkdn_evt   - Call back for link down event
 */
typedef struct phy_driver_s {
    char        *drv_name;
    int		(*pd_init)(int, soc_port_t);
    int		(*pd_link_get)(int, soc_port_t, int *);
    int		(*pd_enable_set)(int, soc_port_t, int);
    int		(*pd_enable_get)(int, soc_port_t, int *);
    int		(*pd_duplex_set)(int, soc_port_t, int);
    int		(*pd_duplex_get)(int, soc_port_t, int *);
    int		(*pd_speed_set)(int, soc_port_t, int);
    int		(*pd_speed_get)(int, soc_port_t, int *);
    int		(*pd_an_set)(int, soc_port_t, int);
    int		(*pd_an_get)(int, soc_port_t, int *, int *);
    int		(*pd_adv_local_set)(int, soc_port_t, port_mode_t);
    int		(*pd_adv_local_get)(int, soc_port_t, port_mode_t *);
    int		(*pd_adv_remote_get)(int, soc_port_t, port_mode_t *);
    int		(*pd_lb_set)(int, soc_port_t, int);
    int		(*pd_lb_get)(int, soc_port_t, int *);
    int		(*pd_interface_set)(int, soc_port_t, int);
    int		(*pd_interface_get)(int, soc_port_t, int *);
    int		(*pd_ability)(int, soc_port_t, port_mode_t *);
    int		(*pd_linkup_evt)(int, soc_port_t);
    int		(*pd_linkdn_evt)(int, soc_port_t);
} phy_driver_t;

#define	PHY_INIT(_pd, _u, _p)		(_pd)->pd_init((_u), (_p))
#define	PHY_LINK_GET(_pd, _u, _p, _l)	(_pd)->pd_link_get((_u), (_p), (_l))
#define	PHY_ENABLE_SET(_pd, _u, _p, _l)	\
    (_pd)->pd_enable_set((_u), (_p), (_l))
#define	PHY_ENABLE_GET(_pd, _u, _p, _l)	\
    (_pd)->pd_enable_get((_u), (_p), (_l))
#define	PHY_DUPLEX_SET(_pd, _u, _p, _d)	\
    (_pd)->pd_duplex_set((_u), (_p), (_d))
#define	PHY_DUPLEX_GET(_pd, _u, _p, _d)	\
    (_pd)->pd_duplex_get((_u), (_p), (_d))
#define	PHY_SPEED_SET(_pd, _u, _p, _s)	(_pd)->pd_speed_set((_u), (_p), (_s))
#define	PHY_SPEED_GET(_pd, _u, _p, _s)	(_pd)->pd_speed_get((_u), (_p), (_s))
#define	PHY_AUTO_NEGOTIATE_SET(_pd, _u, _p, _a)	\
    (_pd)->pd_an_set((_u), (_p), (_a))
#define	PHY_AUTO_NEGOTIATE_GET(_pd, _u, _p, _a, _b) \
    (_pd)->pd_an_get((_u), (_p), (_a), (_b))
#define	PHY_ADV_SET(_pd, _u, _p, _a)		\
    (_pd)->pd_adv_local_set((_u), (_p), (_a))
#define	PHY_ADV_GET(_pd, _u, _p, _a)		\
    (_pd)->pd_adv_local_get((_u), (_p), (_a))
#define	PHY_LOOPBACK_SET(_pd, _u, _p, _l) (_pd)->pd_lb_set((_u), (_p), (_l))
#define	PHY_LOOPBACK_GET(_pd, _u, _p, _l) (_pd)->pd_lb_get((_u), (_p), (_l))
#define	PHY_INTERFACE_SET(_pd, _u, _p, _i) \
    (_pd)->pd_interface_set((_u), (_p), (_i))
#define	PHY_INTERFACE_GET(_pd, _u, _p, _i) \
    (_pd)->pd_interface_get((_u), (_p), (_i))

#define	PHY_ABILITY_GET(_pd, _u, _p, _m) 	\
    (_pd)->pd_ability((_u), (_p), (_m))
#define	PHY_ADV_REMOTE_GET(_pd, _u, _p, _m) 	\
    (_pd)->pd_adv_remote_get((_u), (_p), (_m))

extern	phy_driver_t	*soc_phy_probe(int unit, soc_port_t port);
extern	char		*soc_phy_name(int unit, soc_port_t port);
extern  int mac_set_autoz(int unit, soc_port_t port, int enable);
extern  int soc_gmac_in_fe_mode(int unit, soc_port_t port);
extern  mac_driver_t mac_fe_5600;
extern  mac_driver_t mac_ge_5600;
#endif /* BCM5600_LL_H */
