/*
 *=============================================================================
 * File Name   :    bcm5632-cmic.c
 *
 *  RCS: $Id: bcm5632-cmic.c,v 1.3 2002/10/18 02:52:36 johnh Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>

#include "bcm5632-soctypes.h"
#include "bcm5632-cmic.h"
#include "bcm5632-error.h"

extern uint32 reg_addr_5632(soc_reg_t reg, ...);

#define DBG_PRINTF printf

char *TABLE_CMD_TEXT[] = {
	"No OP",
	"Read",
	"Write",
	"DMA Read",
	"Learn",
	"Search",
	"Delete",
	"DMA Write"
};

int table_id(soc_mem_t mem)
{
    soc_meminfo_t *m = &soc_memories_5632[mem];

    /* compare table name to get table id; default to L2 table */
    if (strcmp(m->name, "L4_") == 0)  return (TABLE_ID)TABLE_L2_L7;
    if (strcmp(m->name, "VLAN_") == 0) return (TABLE_ID)TABLE_VLAN;
    if (strcmp(m->name, "ARL_5632_") == 0) return (TABLE_ID)TABLE_L2;
    if (strcmp(m->name, "EEPROM_") == 0) return (TABLE_ID)TABLE_EEPROM;
    return (TABLE_ID)TABLE_L2;
}
void swap_u32(uint32 *addr, int count);
int _BCM5632_L2_management(int unit, soc_mem_t mem, void *key_data, void *entry_data, int index, int cmd);
int _BCM5632_table_management(int unit, soc_mem_t mem, void *key_data, int index, int cmd);
void Struct2Buf(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
void Buf2Struct(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
void dummy_DMA_5632(int unit);
void clear_table_status(int unit);

extern void delay_us(uint32);

extern uint32 BCM5632_PCI_BASE_ADDR[BCM5632_NUM_DEV];

#if defined(PCI_SWAP_REQUIRED)
uint32 pci_swap_5632(uint32 val)
{
    val = val << 16 | val >> 16;
    return (val & 0xff00ffff) >> 8 | (val & 0xffff00ff) << 8;
}
#endif 

/*
 * Enable (unmask) or disable (mask) a set of CMIC interrupts.  These
 * routines should be used instead of manipulating CMIC_IRQ_MASK
 * directly, since a read-modify-write is required.  The return value is
 * the previous mask (can pass mask of 0 to just get the current mask).
 */

uint32 cmic_irq_enable_5632(int unit, uint32 mask)
{
    uint32 	oldMask;

    oldMask = cmic_pci_getreg_5632(unit, CMIC_IRQ_MASK);
    cmic_pci_setreg_5632(unit, CMIC_IRQ_MASK, oldMask & (~mask & IRQ_ALL));

    return oldMask;
}

uint32 cmic_irq_disable_5632(int unit, uint32 mask)
{
    uint32 	oldMask;

    oldMask = cmic_pci_getreg_5632(unit, CMIC_IRQ_MASK);
    cmic_pci_setreg_5632(unit, CMIC_IRQ_MASK, oldMask | mask);

    return oldMask;
}

/*
 * Get a CMIC register in PCI space.
 * Input address is relative to the base of CMIC registers.
 */

uint32 cmic_pci_getreg_5632(int unit, uint32 addr)
{
    uint32 pci_addr, data;

    pci_addr = BCM5632_PCI_BASE_ADDR[unit] + (addr << 2);

# if defined(PCI_SWAP_REQUIRED)
    data = pci_swap_5632(*(uint32 *) pci_addr);
# else /* !PCI_SWAP_REQUIRED */
    data = *(uint32 *) pci_addr;
# endif /* !PCI_SWAP_REQUIRED */
#if 0
    DBG_PRINTF("PCI%d memR(0x%x)=0x%x\n", unit, pci_addr, data);
#endif
	delay_us(1000);
    return data;
}

/*
 * Set a CMIC register in PCI space.
 * Input address is relative to the base of CMIC registers.
 */

int cmic_pci_setreg_5632(int unit, uint32 addr, uint32 data)
{
    uint32 pci_addr;

    pci_addr = BCM5632_PCI_BASE_ADDR[unit] + (addr << 2);

# if defined(PCI_SWAP_REQUIRED)
    *(uint32 *) pci_addr = pci_swap_5632(data);
# else /* !PCI_SWAP_REQUIRED */
    *(uint32 *) pci_addr = data;
# endif /* !PCI_SWAP_REQUIRED */
#if 0
    DBG_PRINTF("PCI%d memW(0x%x)=0x%x\n", unit, pci_addr, data);
#endif
	delay_us(1000);
    return 0;
}

/************************************************************************/
/* Routines for reading/writing SOC internal registers			*/
/************************************************************************/

/*
 * Read an internal SOC register through PCI Memory Mapping.
 */
int cmic_read_reg_5632(int unit, uint32 addr, uint32 *data)
{
    *data = (uint32) cmic_pci_getreg_5632(unit, (uint32) addr);

    return SOC_E_NONE;
}

/*
 * Write an internal SOC register through PCI Memory Mapping.
 */
int cmic_write_reg_5632(int unit, uint32 addr, uint32 data)
{
    cmic_pci_setreg_5632(unit, (uint32) addr, data);

    return SOC_E_NONE;
}

/************************************************************************/
/* Routines for reading/writing all tables (except no write to ARL)	*/
/************************************************************************/

int cmic_read_memory_5632(int unit,
		     soc_mem_t mem,
		     int copyno,
		     int index,
		     void *entry_data)
{
    uint32  *addr;
    uint32 cnt;		
    uint32 cnt_idx;	
    int entry_size;	
    uint32 buf_size;
    uint32 cmd, status;		
    int polls = 0;
    int rv = SOC_E_NONE;

/*
    soc_meminfo_t *meminfo = &soc_memories_5632[mem];
*/
    int entry_dw = BYTES2WORDS(get_mem_numbytes_5632(mem));

    /* for BCM5632,  "index" is an aggregate of (cnt, idx) */

    cnt_idx = index;
    index = GET_LO_WORD(index);

    cnt = GET_HI_WORD(cnt_idx);		/* (((uint32)index >> 16) & 0xFFFF); */
    cnt = (cnt) ? cnt : 1;		/* default to get one entry */
    index = GET_LO_WORD(cnt_idx);	/* ((uint32)index & 0xFFFF); */
    cnt_idx = MAKE_UINT32(cnt, index);
    entry_size = get_mem_numbytes_5632(mem);
    buf_size = entry_dw * cnt * sizeof(uint32);	/* assure 4-byte boundary */

    /* allocate buffer for receiving data */
    if ((addr = (uint32 *)eee_ramAlloc(buf_size)) == 0) {
	printf("Not enough DMA memory\n");
        return SOC_E_MEMORY;
    }

    /* dummy DMA read of 1 entry from VLAN, the smallest, table */
    if (cnt > 1 ) {
		dummy_DMA_5632(unit);
    }

    /* command for Table DMA Read */
    cmd = ((table_id(mem) & TABLE_ID_MASK) << TABLE_ID_POS) | (TABLE_DMA_READ);

    /* program registers */
    cmic_write_reg_5632(unit, CMIC_REG(TBL_SXADDRr), (uint32)addr);
    cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), cnt_idx);
    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), cmd); 

    /* Poll for completion of Table command */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
        if (++polls >= MIIM_TIMEOUT) {
            printf("cmic_read_memory_5632: timeout (addr=0x%08x)\n", (uint32)addr);
            rv = SOC_E_MII_TIMEOUT;
            goto done;
        }

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);
#if 0
    DBG_PRINTF("cmic_read_memory_5632: polls=%d\n", polls + 1);
#endif
    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("cmic_read_memory_5632: table command failed\n");
        rv = SOC_E_INTERNAL;
		clear_table_status(unit);  /* chip workaround - clear failed status */
        goto done;
    }

    /* swap byte order */
    swap_u32(addr, entry_dw * cnt);
    
    /* copy data to buffer provided by calling routine */
    memcpy((unsigned char *)entry_data, (unsigned char *)addr, buf_size);
/* 
    if (strcmp(meminfo->name, "ARL_5632_") == 0) {
      UnpackTableEntry(mem, (unsigned char *)addr, (unsigned char *)entry_data);
    }
    else if (strcmp(meminfo->name, "L4_") == 0) {
          memcpy((unsigned char *)entry_data, (unsigned char *)addr, buf_size);
    } else if (strcmp(meminfo->name, "VLAN_") == 0) {
          memcpy((unsigned char *)entry_data, (unsigned char *)addr, buf_size);
	}
*/
done:
    eee_ramDealloc(addr); 

    return rv;
}

/*
 * Write a memory internal to the SOC.
 */

int cmic_write_memory_5632(int unit,
		      soc_mem_t mem,
		      int copyno,	/* Use COPYNO_ALL for all */
		      int index,
		      void *entry_data)
{
    int entry_dw = BYTES2WORDS(get_mem_numbytes_5632(mem));
/*
    soc_meminfo_t *meminfo = &soc_memories_5632[mem];
*/
    uint32 cnt, cnt_idx;
    uint32 entry_size;
    uint32 cmd, status;
    uint32 buf_size;
    uint32 *addr;
    int polls = 0;
    int rv = SOC_E_NONE;

    cnt = GET_HI_WORD(index);		/* get entry count */
    cnt = (cnt) ? cnt : 1;		/* at least 1 */	
    index = GET_LO_WORD(index);		/* strip entry count from index */
    cnt_idx = MAKE_UINT32(cnt, index);	/* make cnt_idx for 5632 table access */

    entry_size = get_mem_numbytes_5632(mem);	/* get size of table entry */
    buf_size = entry_dw * cnt * sizeof(uint32);	/* assure 4-byte boundary */

    DBG_PRINTF("cmic_write_memory_5632: cnt=%d, index=%d, cnt_idx=%d, ", 
              cnt, index, cnt_idx);
    /* socmem_dump_entry_5632(mem, entry_data); */

    /* command for Table DMA write */
    cmd = ((table_id(mem) & TABLE_ID_MASK) << TABLE_ID_POS) | 
	  (TABLE_DMA_WRITE & TABLE_CMD_MASK);

    /* allocate DMA buffer */
    if ((addr = (uint32 *)eee_ramAlloc(buf_size)) == 0) {
	printf("Not enough DMA memory\n");
	return SOC_E_MEMORY;
    }
		
    /* copy date from user provided buffer to DMA buffer */
    memset((unsigned char *)addr, 0x0 , buf_size);
    memcpy((unsigned char *)addr, (unsigned char *)entry_data, buf_size);
/*
    if (strcmp(meminfo->name, "ARL_5632_") == 0) {
        PackTableEntry(mem,(unsigned char *)entry_data,(unsigned char *)addr,0);
    }
    else 
    if (strcmp(meminfo->name, "L4_") == 0) {
        memset((unsigned char *)addr, 0 , buf_size);
        memcpy((unsigned char *)addr, (unsigned char *)entry_data, buf_size);
    }
    else 
    if (strcmp(meminfo->name, "VLAN_") == 0) {
        memset((unsigned char *)addr, 0 , buf_size);
        memcpy((unsigned char *)addr, (unsigned char *)entry_data, buf_size);
    }
*/
    /* swap byte order */
    swap_u32(addr, entry_dw * cnt);

    /* start command ... */
    cmic_write_reg_5632(unit, CMIC_REG(TBL_SXADDRr), (uint32)addr);
    cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), cnt_idx);
    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), cmd);	/* table write */

    /* show debug message */
    DBG_PRINTF("cmic_write_memory_5632: cmd=0x%X, index=%d\n", cmd, cnt_idx);

    /* Poll for completion of Table command */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
        if (++polls >= MIIM_TIMEOUT) {
            printf("cmic_write_memory_5632: timeout (addr=0x%08x)\n",(uint32)addr);
            rv = SOC_E_MII_TIMEOUT;
            goto done;
        }

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);
#if 0
    DBG_PRINTF("cmic_write_memory_5632: polls=%d\n", polls + 1);
#endif
    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("cmic_write_memory_5632: table command failed\n");
		clear_table_status(unit);  /* chip workaround - clear failed status */
        rv = SOC_E_INTERNAL;
    }
done:
    eee_ramDealloc(addr);

    return rv;
}

/*
 * cmic_search_memory_5632
 *
 *   For the ARL table, all entries are searched.  On revision GSL and
 *	higher, a hardware lookup feature is used.  For earlier revisions,
 *	a software search is used, which is not recommended since the ARL
 *	table may be under constant update by the hardware.  For hardware
 *	lookup, the index_ptr is written with -1.
 */
int cmic_search_memory_5632(int unit, 
		       soc_mem_t mem,
		       void *key_data,
		       void *entry_data)
{
    int rv = 0;

    /* search is available for ARL only */
    /* if necessary, search for other table need to implemented seperately */
    if (mem == ARL_5632_m ) {	
	    rv = _BCM5632_L2_management(unit, mem, key_data, entry_data, 0, 
                                    TABLE_SEARCH);
    }
    else {
	/* do nothing for now */
	return SOC_E_NONE;			/* No error; not found */
    }

    return rv; /* rv = 1 --> found; 0 --> Not found */
}

/*
 * cmic_insert_memory_5632
 *
 *   Insert an ARL entry in hash table.
 *
 *
 *   Returns  -- SOC_E_NONE
 */
int cmic_insert_memory_5632(int unit,
		       soc_mem_t mem,
		       int copyno,
		       void *entry_data)
{
    int rv = SOC_E_NONE;

    /* L2 of BCM5632 is hashed.  Use learn instead ... */
    if (mem == ARL_5632_m) {
        _BCM5632_L2_management(unit, mem, entry_data, (void *)0,0, TABLE_LEARN);
    }
    /* show debug data */
    /* socmem_dump_entry_5632(mem, entry_data); */

    /* since L2 is hashed.  No index to return ... */
    return rv;
}

/*
 * Function:
 *	cmic_write_miim_5632
 * Purpose:
 *	Write a value to a MIIM register.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	phy_id - Phy ID to write (MIIM address)
 *	phy_reg_addr - PHY register to write
 *	phy_wr_date - Data to write.
 * Returns:
 *	SOC_E_XXX
 * Notes:
 */
int 
cmic_write_miim_5632(int unit, uint8 phy_id,
		uint8 phy_reg_addr, uint16 phy_wr_data)
{
    int rv;
    uint32 rd_data, status;
    int polls = 0;

    DBG_PRINTF("cmic_write_miim_5632: addr=0x%02x id=0x%02x data=0x%04x\n",
	       phy_reg_addr, phy_id, phy_wr_data);

    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &rd_data);
    if( rd_data & 0x80) {
	printf("cmic_write_miim_5632 ERROR:table management cmd in execution\n");
	rv = SOC_E_INTERNAL;
	goto done;
    }

    /* Write parameter register and tell CMIC to start */

    cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr),
		       (uint32) 1 << 10 |
		       (uint32) phy_reg_addr |
		       (uint32) phy_id << 5);

    cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_0r), phy_wr_data);

    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), 
                             (TABLE_MDIO << TABLE_ID_POS) | TABLE_WRITE);

    /* Poll for completion of MIIM operation */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
	if (++polls >= MIIM_TIMEOUT) {
	    printf("cmic_write_miim_5632: timeout (addr=0x%02x id=0x%02x data=0x%04x)\n",
		   phy_reg_addr, phy_id, phy_wr_data);
	    rv = SOC_E_MII_TIMEOUT;
	    goto done;
	}

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);

    DBG_PRINTF("cmic_write_miim_5632: polls=%d\n", polls + 1);

    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("cmic_write_miim_5632: table command failed\n");
        rv = SOC_E_INTERNAL;
    }
    else
       rv = SOC_E_NONE;

done:
    return rv;
}

/*
 * cmic_read_miim_5632
 *
 * Reads a value from a PHY register.
 * Stores the 16-bit read data in the phy_rd_data pointer.
 * Temporarily disables auto link scan if it was enabled.
 *
 * On platforms where it is appropriate, the MIIM registers are locked
 * during the operation to prevent multiple tasks trying to access PHY
 * registers simultaneously.
 *
 * Return value is negative on error, 0 on success.
 */
int cmic_read_miim_5632(int unit, uint8 phy_id,
		   uint8 phy_reg_addr, uint16 *phy_rd_data)
{
    int rv;
    uint32 rd_data, status;
    int polls = 0;

    DBG_PRINTF("cmic_read_miim_5632: addr=0x%02x id=0x%02x\n", phy_reg_addr, 
               phy_id);

    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &rd_data);
    if( rd_data & 0x80) {
	printf("cmic_read_miim_5632 ERROR:table management cmd in execution\n");
	rv = SOC_E_INTERNAL;
	goto done;
    }

    /* Write parameter register and tell CMIC to start */

    cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr),
		       (uint32) 1 << 10 |
		       (uint32) phy_reg_addr |
		       (uint32) phy_id << 5);

    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), 
                             (TABLE_MDIO << TABLE_ID_POS) | TABLE_READ);

    /* Poll for completion of MIIM operation */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
	if (++polls >= MIIM_TIMEOUT) {
	    printf("cmic_read_miim_5632 : timeout (addr=0x%02x id=0x%02x)\n",
		   phy_reg_addr, phy_id);
	    rv = SOC_E_MII_TIMEOUT;
	    goto done;
	}

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);

    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("cmic_read_miim_5632: table command failed\n");
        rv = SOC_E_INTERNAL;
        goto done;
    }

    cmic_read_reg_5632(unit, CMIC_REG(TBL_DBUF_0r), &rd_data);
    *phy_rd_data = (uint16) rd_data;

    DBG_PRINTF("cmic_read_miim_5632: polls=%d data=0x%04x\n", polls+1,
               *phy_rd_data);

    rv = SOC_E_NONE;

done:
    return rv;
}

/*
 * Function:
 *  cmic_write_xmiim_5632
 * Purpose:
 *  Write a value to a MIIM register using Clause 22 MDIO Format.
 * Parameters:
 *  unit - StrataSwitch Unit #.
 *  phy_id - Phy ID to write (MIIM address)
 *      phy_bank - Phy Bank to support non-IEEE registers
 *  phy_reg_addr - PHY register to write
 *  phy_wr_date - Data to write.
 * Returns:
 *  SOC_E_XXX
 */
int
cmic_write_xmiim_5632(int unit, uint8 phy_id, uint8 phy_bank,
        uint16 phy_reg_addr, uint16 phy_wr_data)
{
    int rv;

# if defined(BCM5632_A0) | defined(BCM5633_A0)
    /* support MDIO-22 only */
    if(1) {
# else
    if( cmic_pci_getreg_5632(unit, CMIC_MDIO_STANDARD)&0x1) {
# endif
        if( (rv=cmic_write_miim_5632( unit, phy_id, 0x1f, (uint16)phy_bank << 4))
                != SOC_E_NONE) {
        printf("cmic_write_xmiim_5632 ERROR: cannot set bank ID\n");
        return rv;
    }
        if( (rv=cmic_write_miim_5632( unit, phy_id, phy_reg_addr, phy_wr_data))
            != SOC_E_NONE) {
        printf("cmic_write_xmiim_5632 ERROR: cannot write to PHY register\n");
        return rv;
        }
    } else {
        if( (rv=cmic_write_miim_5632( unit, phy_id,
            (uint16) ((phy_reg_addr&0x10)<<11) | (uint16) (phy_bank << 4) | (uint16) (phy_reg_addr & 0xf),
        phy_wr_data)) != SOC_E_NONE) {
            printf("cmic_read_xmiim_5632 ERROR: cannot set bank ID\n");
        return rv;
    }
    }

    return rv;
}

/*
 * cmic_read_xmiim_5632
 *
 * Reads a value from a PHY register using Clause 22 MDIO Format.
 * Stores the 16-bit read data in the phy_rd_data pointer.
 *
 * Return value is negative on error, 0 on success.
 */
int cmic_read_xmiim_5632(int unit, uint8 phy_id, uint8 phy_bank,
           uint16 phy_reg_addr, uint16 *phy_rd_data)
{
    int rv;

# if defined(BCM5632_A0) | defined(BCM5633_A0)
    /* support MDIO-22 only */
    if(1) {
# else
    if( cmic_pci_getreg_5632(unit, CMIC_MDIO_STANDARD)&0x1) {
# endif
        if( (rv=cmic_write_miim_5632( unit, phy_id, 0x1f, (uint16)phy_bank << 4))
                != SOC_E_NONE) {
        printf("cmic_read_xmiim_5632 ERROR: cannot set bank ID\n");
        return rv;
    }
        if( (rv=cmic_read_miim_5632( unit, phy_id, phy_reg_addr, phy_rd_data))
            != SOC_E_NONE) {
        printf("cmic_read_xmiim_5632 ERROR: cannot read PHY register\n");
        return rv;
    }

    } else {
        if( (rv=cmic_read_miim_5632( unit, phy_id,
            (uint16) ((phy_reg_addr&0x10)<<11) | (uint16) (phy_bank << 4) | (uint16) (phy_reg_addr & 0xf),
        phy_rd_data)) != SOC_E_NONE) {
            printf("cmic_read_xmiim_5632 ERROR: cannot set bank ID\n");
        return rv;
    }
    }
    return rv;
}

/*
 * cmic_read_mib_5632
 *
 * Reads a value from a MIB table.
 *
 * Return value is negative on error, 0 on success.
 */

int cmic_read_mib_5632(int unit, uint32 mib_entry, uint32 *mib_rd_data)
{
    int rv;
    uint32 rd_data, status;
    int polls = 0;
#if 0
    DBG_PRINTF("cmic_read_mib_5632: entry=0x%04x\n", mib_entry);
#endif
    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &rd_data);
    if( rd_data & 0x80) {
	printf("cmic_read_mib_5632 ERROR:table management cmd in execution\n");
	rv = SOC_E_INTERNAL;
	goto done;
    }

    /* Write parameter register and tell CMIC to start */
    cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), mib_entry);
    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), 
                             (TABLE_MIB << TABLE_ID_POS) | TABLE_READ);

    /* Poll for completion of MIIM operation */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
	if (++polls >= MIIM_TIMEOUT) {
	    printf("cmic_read_mib_5632 ERROR: timeout (entry=0x%04x)\n",
		   mib_entry);
	    rv = SOC_E_MII_TIMEOUT;
	    goto done;
	}

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);

    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("cmic_read_mib_5632: table command failed\n");
		clear_table_status(unit);  /* chip workaround - clear failed status */
        rv = SOC_E_INTERNAL;
        goto done;
    }

    cmic_read_reg_5632(unit, CMIC_REG(TBL_DBUF_0r), mib_rd_data);
#if 0
    DBG_PRINTF("cmic_read_mib_5632: polls=%d data=0x%08x\n", polls+1, 
              *mib_rd_data);
#endif
    rv = SOC_E_NONE;

done:
    return rv;
}

/***************************************************************************
int _BCM5632_L2_management()

Descriptions:	Instruct BCM5632 to learn/search/delete the key_data.
				Only MAC address is refered in hash.  

***************************************************************************/
int _BCM5632_L2_management(int unit, 
		       soc_mem_t mem,
		       void *key_data,
			   void *entry_data,
			   int index,
			   int mgmt_cmd)
{
    uint32 cmd;
    uint32 status;
    uint32 data = 0;
    uint32 *raw= (uint32 *)key_data;
    uint32 *buf = (uint32 *)entry_data;
    uint32 cnt_idx = GET_LO_WORD(index);
    int polls = 0;
    int rv = SOC_E_NONE;

	if (!key_data) {
		printf("Null entry for key_data!\n");
		return(SOC_E_INTERNAL);
	}

/*
printf("Input: raw[0]=%0x, raw[1]=%0x, raw[2]=%0x\n", raw[0], raw[1], raw[2]); 	
*/
    raw[2] |= 0x2;	/* set valid bit = 1 */
    if (mgmt_cmd == TABLE_READ) {
	/* for read */
	cmd = ((table_id(mem) & TABLE_ID_MASK) << TABLE_ID_POS) | 
				(mgmt_cmd & TABLE_CMD_MASK);
	cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), cnt_idx);
    }
    else if (mgmt_cmd == TABLE_WRITE) {
	/* for write */
	cmd = ((table_id(mem) & TABLE_ID_MASK) << TABLE_ID_POS) | 
				(mgmt_cmd & TABLE_CMD_MASK);
	cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), cnt_idx);

	cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_0r), raw[2]);
	cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_1r), raw[1]);
	cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_2r), raw[0]);
		
    }
    else {
	/* for learn/search/delete */
	cmd = (((raw[0] & 0x7ff) << 21) | (raw[1] & 0xf8000000) >> 11) |
		  ((table_id(mem) & TABLE_ID_MASK) << TABLE_ID_POS) | 
			(mgmt_cmd & TABLE_CMD_MASK);
	data = ((raw[1] & 0x7ffffff) << 5) | ((raw[2] & 0xf8000000) >> 27);

        if (cmd & (1 << 24))
            raw[2] |= 1;  /* Static is used as Valid by multicast mac entry */

	cmic_write_reg_5632(unit, CMIC_REG(TBL_SXADDRr), data);	 
	cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_0r), raw[2]);
	cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_1r), raw[1]);
	cmic_write_reg_5632(unit, CMIC_REG(TBL_DBUF_2r), raw[0]);

    }
    /* show debug message */
#if 0
    DBG_PRINTF("L2 table mgmt: cmd=%s, (cmd, data)=(0x%X, 0x%X), index=%d, 
        raw=[0x%X, 0x%X, 0x%X]\n", TABLE_CMD_TEXT[mgmt_cmd], cmd, data, cnt_idx,
        raw[0], raw[1], raw[2]);
#endif
    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), cmd);

    /* Poll for completion of Table command */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
        if (++polls >= MIIM_TIMEOUT) {
            printf("_BCM5632_L2_management: ERROR: timeout\n");
            rv = SOC_E_MII_TIMEOUT;
            goto done;
        }

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);
#if 0
    DBG_PRINTF("_BCM5632_L2_management: polls=%d\n", polls + 1);
#endif
    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   
		if (mgmt_cmd != TABLE_SEARCH)
			printf("_BCM5632_L2_management_5632: table command failed\n");
		clear_table_status(unit);  /* chip workaround - clear failed status */
        rv = SOC_E_INTERNAL;
        goto done;
    }

    if ((mgmt_cmd == TABLE_READ) || (mgmt_cmd == TABLE_SEARCH)) {
	cmic_read_reg_5632(unit, CMIC_REG(TBL_DBUF_0r), &buf[2]);
	cmic_read_reg_5632(unit, CMIC_REG(TBL_DBUF_1r), &buf[1]);
	cmic_read_reg_5632(unit, CMIC_REG(TBL_DBUF_2r), &buf[0]);

	/* show debug message */
#if 0
	DBG_PRINTF("L2 table mgmt result: cmd=%s, raw=[0x%X, 0x%X, 0x%X] %X\n",
	TABLE_CMD_TEXT[mgmt_cmd], buf[0], buf[1], buf[2], CMIC_REG(TBL_DBUF_2r));
#endif
    }
done:
    return rv;
}

unsigned char _table_buf[4096 * 2 * 4];

/*
 * Single entry VLAN/L4/... Table read/write, including delete.
 */
int _BCM5632_table_management(int unit, 
		       soc_mem_t mem,
		       void *key_data,
		       int index, 
		       int mgmt_cmd)
{
	uint32 cmd;
	uint32 status;
	int i;
	uint32 *raw= (uint32 *)key_data;
	int word_count = BYTES2WORDS(get_mem_numbytes_5632(mem));
	int last_word_index = word_count - 1;
	int cnt_idx = GET_LO_WORD(index);
    int polls = 0;

	/* map TABLE_DELETE to TABLE_WRITE with all 0 data */
	if (mgmt_cmd == TABLE_DELETE) {
		memset(_table_buf, 0, sizeof(_table_buf));
		mgmt_cmd = TABLE_WRITE;
	}

	/* set command register */
	cmd = ((table_id(mem) & TABLE_ID_MASK) << TABLE_ID_POS) | 
			(mgmt_cmd & TABLE_CMD_MASK);

	/* set Table_Access_Entry_ID register */
	cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), cnt_idx);

	/* set Table_Data_Buffer registers for TABLE_WRITE command */
	if (mgmt_cmd == TABLE_WRITE) {
	    for (i = (TBL_DBUF_0r + last_word_index); i >= TBL_DBUF_0r ; i--) {
		cmic_write_reg_5632(unit, CMIC_REG(i), 
                                    raw[last_word_index - (i - TBL_DBUF_0r)]);
	    }
	}
	
	/* start command */
	cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), cmd); 
        /* CPU_CMDr = 0x152 table write */

    /* Poll for completion of Table command */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
        if (++polls >= MIIM_TIMEOUT) {
            printf("_BCM5632_table_management: ERROR: timeout\n");
            return SOC_E_MII_TIMEOUT;
        }

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE);
#if 0
    DBG_PRINTF("_BCM5632_table_management: polls=%d\n", polls + 1);
#endif
    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("_BCM5632_table_management: command failed\n");
		clear_table_status(unit);  /* chip workaround - clear failed status */
        return SOC_E_INTERNAL;
    }

    /* read data from registers TBL_DBUF_0r... */
    if (mgmt_cmd == TABLE_READ) {
	for (i = (TBL_DBUF_0r + last_word_index); i >= TBL_DBUF_0r ; i--) {
	    cmic_read_reg_5632(unit, CMIC_REG(i), 
                          &raw[last_word_index - (i - TBL_DBUF_0r)]);
	}
    }

    return SOC_E_NONE;
}

_u32 FIELD_WIDTH_MASK[] = {
	0,
	0x1, 0x3, 0x7, 0xF, 
	0x1F, 0x3F, 0x7F, 0xFF,
	0x1FF, 0x3FF, 0x7FF, 0xFFF, 
	0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF,
	0x1FFFF, 0x3FFFF, 0x7FFFF, 0xFFFFF, 
	0x1FFFFF, 0x3FFFFF, 0x7FFFFF, 0xFFFFFF,
	0x1FFFFFF, 0x3FFFFFF, 0x7FFFFFF, 0xFFFFFFF, 
	0x1FFFFFFF, 0x3FFFFFFF, 0x7FFFFFFF, 0xFFFFFFFF
};

/*
 * Swap the contents of the uint32.
 */
void swap_u32(uint32 *addr, int count)
{
    uint32 int_buf, i;
    unsigned char *pdest = (unsigned char *)&int_buf, *psrc;

    for (i = 0; i < count; i++) {
        psrc = (unsigned char *)&addr[i];
        /* swap the contents */
        pdest[3] = psrc[0];
        pdest[2] = psrc[1];
        pdest[1] = psrc[2];
        pdest[0] = psrc[3];
        /* put it back */
        addr[i] = int_buf;
    }
}
 
/*
 * Pack a struct buffer into a raw buffer
 */
void Struct2Buf(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32)
{
    int i;	
    int bits_left_in_src_word;
    int bits_left_in_dest_word;
    int width;
    int dest_word, dest_bit;
    int src_word;
    int lf_width, hf_width;
    _u32 val;
    soc_meminfo_t *meminfo = &soc_memories_5632[mem];
    soc_fieldinfo_t *fields = meminfo->fields; 
    int nField = meminfo->nFields;
    int entry_bytes = get_mem_numbytes_5632(mem);

    _u32 *src_buf = (_u32 *)from;
    _u32 *dest_buf = (_u32 *)to;
    int word_width = 32;
    int last_dest_words = ((entry_bytes + 3) / 4) - 1;	/* starts from 0 */
	
    memset(to, 0, (last_dest_words + 1) * sizeof(_u32));
    bits_left_in_dest_word = word_width;
    bits_left_in_src_word = word_width;

    /* initialize table specific parameters ... */
    if (strcmp(meminfo->name, "L4_") == 0) {
	/* for L4 */
        bits_left_in_dest_word = word_width - 4;  /* 4 is hard coded for L4 */
    }
    else if (strcmp(meminfo->name, "VLAN_") == 0) {
        bits_left_in_dest_word = word_width - 2;  /* 2 is hard coded for VLAN */
	if (pack_to_32) bits_left_in_dest_word -= 8;
    }
    else if (mem == ARL_5632_m) {
        bits_left_in_dest_word = word_width - 5;  /* 2 is hard coded for VLAN */
	if (pack_to_32) bits_left_in_dest_word -= 16;
    }

    for (i = 0, src_word = 0; i < nField; i++) {
    /* get next field width */
         width = fields[i].len;

	 /* get field value from src buffer */
	 if (width != word_width) {
             if (width > bits_left_in_src_word) {
	         src_word++;
	         bits_left_in_src_word = 32;
	     }
	     val = (src_buf[src_word] >> (bits_left_in_src_word - width)) & 
                   FIELD_WIDTH_MASK[width];
	     bits_left_in_src_word -= width;
	 }
	 else 
         {
	     hf_width = bits_left_in_src_word;
	     lf_width = word_width - hf_width;
	     val = ((src_buf[src_word] & FIELD_WIDTH_MASK[hf_width]) << 
                   lf_width) | ((src_buf[src_word+1] >> hf_width) & 
                   FIELD_WIDTH_MASK[lf_width]);
	     src_word++;
         }

	 /* find field anchor in dest buffer */
	 dest_word = last_dest_words - ( fields[i].bp / word_width);
	 dest_bit = fields[i].bp % word_width;

	 /* set field value in dest buffer */
	 if (bits_left_in_dest_word < width) {
	    
             /* the field is splited into two words */
	     hf_width = bits_left_in_dest_word;
	     lf_width = width - hf_width;
	     dest_buf[dest_word - 1] |= (val >> lf_width) & 
                                        FIELD_WIDTH_MASK[hf_width];

	     bits_left_in_dest_word = 32 - lf_width;
	     dest_buf[dest_word] |= ((val & FIELD_WIDTH_MASK[lf_width]) << 
                                    dest_bit);
	 }
	 else 
         {
	     /* all bits of the field are in the same word */
	     dest_buf[dest_word] |= (val << dest_bit);
	     bits_left_in_dest_word -= width;
	 }
    }
}

/*
 * Unpack a raw buffer into a struct buffer
 */
void Buf2Struct(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32)
{
    int i;
    int bits_left_in_src_word;
    int bits_left_in_dest_word;
    int width;
    int dest_word;
    int src_word, src_bit;
    int lf_width, hf_width;
    _u32 val;
    soc_meminfo_t *meminfo = &soc_memories_5632[mem];
    soc_fieldinfo_t *fields = meminfo->fields; 
    int nField = meminfo->nFields;
    int entry_bytes = get_mem_numbytes_5632(mem);

    _u32 *src_buf = (_u32 *)from;
    _u32 *dest_buf = (_u32 *)to;
    int word_width = 32;
    int last_src_words = (entry_bytes + 3) / 4 - 1;	/* starts from 0 */

    /* initialize ... */
    bits_left_in_dest_word = word_width;
    bits_left_in_src_word = word_width;

    /* initialize table specific parameters ... */
    if (strcmp(meminfo->name, "L4_") == 0) {
	/* L4 ... */
        memset(to, 0, sizeof(L2_L7_m_entry_t));
        bits_left_in_src_word = word_width - 4; /* the 4 is hard coded for L4 */
	}
    else if (strcmp(meminfo->name, "VLAN_") == 0) {
        memset(to, 0, sizeof(VLAN_m_entry_t));
        bits_left_in_src_word = word_width - 2;/* the 2 is hard coded for VLAN*/
	if (pack_to_32) bits_left_in_src_word -= 8;
    }
    else if (mem == ARL_5632_m) {
        memset(to, 0, sizeof(ARL_5632_m_entry_t));
        bits_left_in_src_word = word_width - 5;/* the 5 is hard coded for ARL */
	if (pack_to_32) bits_left_in_src_word -= 16;
    }

    for (i = 0, dest_word = 0; i < nField; i++) {
	/* get next field width */
	width = fields[i].len;

	/* find field anchor in dest buffer */
	src_word = last_src_words - (fields[i].bp / word_width);
	src_bit = fields[i].bp % word_width;

	/* get field value from buffer */
	if (bits_left_in_src_word < width) {
	    /* the field is splited into two words */
	    hf_width = bits_left_in_src_word;
	    lf_width = width - hf_width;
	    val = (src_buf[src_word - 1] & FIELD_WIDTH_MASK[hf_width]) << 
                  lf_width;
	    bits_left_in_src_word = 32 - lf_width;
	    val |= (src_buf[src_word] >> bits_left_in_src_word) & 
	    FIELD_WIDTH_MASK[lf_width];
	}
	else 
        {
	    /* all bits of the field are in the same word */
	    val = (src_buf[src_word] >> src_bit) & FIELD_WIDTH_MASK[width];
	    bits_left_in_src_word -= width;
	}

	/* set field value into dest buffer */
	if (width != word_width) {
	    if (width > bits_left_in_dest_word) {
		dest_word++;
		bits_left_in_dest_word = 32;
	    }
	    dest_buf[dest_word] |= (val & FIELD_WIDTH_MASK[width]) << 
                                   (bits_left_in_dest_word - width);
	    bits_left_in_dest_word -= width;
	}
	else 
        {
	    hf_width = bits_left_in_dest_word;
	    lf_width = word_width - hf_width;
	    dest_buf[dest_word] |= ((val >> lf_width) & 
                                    FIELD_WIDTH_MASK[hf_width]);
	    dest_word++;
	    dest_buf[dest_word] |= ((val & FIELD_WIDTH_MASK[lf_width]) << 
                                    hf_width);
	}
    }	
}

void dummy_DMA_5632(int unit)
{
    int polls = 0;
    uint32 cmd, status;
	uint32 *addr;

    if ((addr = (uint32 *)eee_ramAlloc(2*sizeof(uint32))) == 0) {
		printf("dummy_DMA_5632: couldn't allocate memory\n");
		return;
	}
	cmd = ((table_id(VLAN_m) & TABLE_ID_MASK) << TABLE_ID_POS) | TABLE_DMA_READ;
    /* program registers */
    cmic_write_reg_5632(unit, CMIC_REG(TBL_SXADDRr), (uint32)addr);
    cmic_write_reg_5632(unit, CMIC_REG(TBL_ENTIDr), MAKE_UINT32(1, 0));
    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), cmd); 

    /* Poll for completion of Table command */

    while (!(cmic_pci_getreg_5632(unit, CMIC_IRQ_STAT) & IRQ_TBLCMDDONE))
        if (++polls >= MIIM_TIMEOUT) {
            printf("dummy_DMA_5632: ERROR: timeout\n");
			break;
        }

    cmic_pci_setreg_5632(unit, CMIC_IRQ_STAT, IRQ_TBLCMDDONE); 
#if 0
    DBG_PRINTF("dummy_DMA_5632: polls=%d\n", polls + 1);
#endif
    cmic_read_reg_5632(unit, CMIC_REG(SYS_STATUSr), &status);
    if (GET_SYS_STATUSr_LastTblCmdX(status))
    {   printf("dummy_DMA_5632: table command failed\n");
		clear_table_status(unit);  /* chip workaround - clear failed status */
    }
    eee_ramDealloc(addr); 
    return;
}

void clear_table_status(int unit)
{
    cmic_write_reg_5632(unit, CMIC_REG(CPU_CMDr), 0x2a); 
}
