/*
 *=============================================================================
 * File Name   :    bcm5632-cmic.h
 *
 *  RCS: $Id: bcm5632-cmic.h,v 1.1.1.1 2001/12/22 04:14:58 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef _BCM5632_CMIC_H
#define _BCM5632_CMIC_H

#define BCM5632_PCI_BASE_ADDR0  0x38000000
#define BCM5632_PCI_BASE_ADDR1  0x39000000
#define BCM5632_BUS_ID          8
#define BCM5632_DEV_ID0         6
#define BCM5632_DEV_ID1         7

#define BCM5632_NUM_DEV			2

#define COPYNO_ALL          	-1
#define COPYNO_NONE         	-2

#define N_DMA_CHAN				2

/* CMIC DMA Memory Address Registers */
#define CMIC_DMA_BASE_CHAN(ch)		((ch)==1 ? 0x442:0x462)

/* IRQ Register (RO) */
#define CMIC_IRQ_STAT			0x43f  

/* IRQ Mask Register (R/W) */
#define CMIC_IRQ_MASK			0x440  

/* IRQ Mask bits */
#define IRQ_PKT(ch)			(0x00000001 << (2 * (ch)))	/* ch=0 for TX, ch=1 for Rx */
#define IRQ_PKTLAST(ch)			(0x00000002 << (2 * (ch)))	/* ch=0 for Tx, ch=1 for Rx */
#define IRQ_TBLCMDDONE			0x00000010 /* Bit 4 */
#define IRQ_MIBCNTOF			0x00000020
#define IRQ_BISTDONE			0x00000040
#define IRQ_MEMINITDONE			0x00000080
#define IRQ_PCIERR			0x00000100 /* Bit 8 */
#define IRQ_CPUINITSTART		0x00000200
#define IRQ_ALL				0x000003ff
#define SOC_NUM_IRQ			10

#define MIIM_TIMEOUT	10000
#define CMIC_MDIO_STANDARD  0x98

#define TABLE_ID_POS    0x3
#define TABLE_ID_MASK   0x7
#define TABLE_CMD_MASK  0x7

typedef enum _TABLE_CMD {
    TABLE_NO_OPERATION = 0,
    TABLE_READ = 0x1,
    TABLE_WRITE = 0x2,
    TABLE_DMA_READ = 0x3,
    TABLE_LEARN = 0x4,
    TABLE_SEARCH = 0x5,
    TABLE_DELETE = 0x6,
    TABLE_DMA_WRITE = 0x7
} TABLE_CMD;

typedef enum _TABLE_ID {
    TABLE_L2 = 0,
    TABLE_VLAN = 0x1,
    TABLE_L2_L7 = 0x2,
    TABLE_MIB = 0x3,
    TABLE_EEPROM = 0x4,
    TABLE_MDIO = 0x5,
    TABLE_SRAM = 0x6,
    TABLE_TEST = 0x7
} TABLE_ID;

/*
   Definition Tx (CPU to CHIP) FIFO controller 0
 */
typedef union {
    uint32	dmat0_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5632_LE_HOST
	uint32	_length:14,	/* Packet length */
	        _modified:1,	/* Packet modified */
	        _drop:1,	/* Conditional drop */
	        _forward:1,	/* Uncnditional forward */
	        :15;		/* Reserved (15) */
#else /* BCM5632_BE_HOST */
	uint32	:15,
		_forward:1,
		_drop:1,
		_modified:1,
		_length:14;
#endif /* BCM5632_BE_HOST */
    } _dmat0;
} dmat0_t;			/* Control bit definition */

#define	dmat0_length		_dmat0._length
#define dmat0_modified		_dmat0._modified
#define dmat0_drop		_dmat0._drop
#define dmat0_forward		_dmat0._forward

#define DMAT0_INIT(dp, len)	((dp).dmat0_val = 0)

/*
   Definition Tx (CPU to CHIP) FIFO controller 1
 */
typedef union {
    uint32	dmat1_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5632_LE_HOST
	uint32	_decode:3,	/* Unicast/Multicast decode */
	        _queue:2,	/* Queue spec */
	        _bitmap:14,	/* Destination bitmap (up, p12, p11,..., p0) */
	        _id:6,		/* Destination ID */
	        :7;		/* Reserved (7) */
#else /* BCM5632_BE_HOST */
	uint32	:7,
		_id:6,
		_bitmap:14,
		_queue:2,
		_decode:3;
#endif /* BCM5632_BE_HOST */
    } _dmat1;
} dmat1_t;			/* Control bit definition */

#define dmat1_decode		_dmat1._decode
#define dmat1_queue		_dmat1._queue
#define dmat1_bitmap		_dmat1._bitmap
#define dmat1_id		_dmat1._id

#define DMAT1_DECODE_UNICAST	0
#define DMAT1_DECODE_UNKNOWN	4
#define DMAT1_DECODE_BROADCAST	5
#define DMAT1_DECODE_MULTICAST	6
#define DMAT1_DECODE_IPMC	7

/*
   Definition Tx (CPU to CHIP) FIFO controller 2
 */
typedef union {
    uint32	dmat2_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5632_LE_HOST
	uint32	_otag:14,	/* Output tag/untag */
	        _ptag:3,	/* Priority tag */
	        _vtag:12,	/* VLAN tag */
	        _itag:1,	/* Input tag/untag */
	        :2;		/* Reserved (2) */
#else /* BCM5632_BE_HOST */
	uint32	:2,
		_itag:1,
		_vtag:12,
		_ptag:3,
		_otag:14;
#endif /* BCM5632_BE_HOST */
    } _dmat2;
} dmat2_t;			/* Control bit definition */

#define dmat2_otag		_dmat2._otag
#define dmat2_ptag		_dmat2._ptag
#define dmat2_vtag		_dmat2._vtag
#define dmat2_itag		_dmat2._itag

/*
   Definition Rx (CHIP to CPU) FIFO controller 0
 */
typedef union {
    uint32	dmar0_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5632_LE_HOST
	uint32	_length:14,	/* Packet length */
	        :18;		/* Reserved (18) */
#else /* BCM5632_BE_HOST */
	uint32	:18,
		_length:14;
#endif /* BCM5632_BE_HOST */
    } _dmar0;
} dmar0_t;			/* Control bit definition */

/*
   Definition Rx (CHIP to CPU) FIFO controller 1
 */
typedef union {
    uint32	dmar1_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5632_LE_HOST
	uint32	_ptag:3,	/* Priority tag */
	        _vtag:12,	/* VLAN tag */
	        _otag:1,	/* Output tag */
	        _itag:1,	/* Input tag/untag */
		_id:6,		/* soruce ID */
		_class:4,	/* CPU class spec */
	        :5;		/* Reserved (5) */
#else /* BCM5632_BE_HOST */
	uint32	:5,
		_class:4,
		_id:6,
		_itag:1,
		_otag:1,
		_vtag:12,
		_ptag:3;
#endif /* BCM5632_BE_HOST */
    } _dmar1;
} dmar1_t;			/* Control bit definition */

typedef union {
    uint32	dmac_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5632_LE_HOST
	uint32	_cnt:16,	/* Byte count to transfer */
	        :1,		/* Reserved (0) */
	        _rld:1,		/* Reload */
	        _sg:1,		/* Scatter Gather */
	        :2,		/* Reserved (0) */
		:1,		/* Unused (IPX data) */
		_ip:1,		/* Indicates IP switched packet */
		_obit:2,	/* DMA Rd: Ingress optimization bits (below) */
		_crc:2,		/* If 0, packet is assumed to have valid */
				/*   CRC and is sent out as is */
				/* If 1, packet is assumed to have no CRC */
				/*   dest. egress will append it unless */
				/*   egress is CPU itself (MMU loopback). */
				/* If 2. packet is assumed to have CRC */
				/*   which is ignored and regenerated by */
				/*   egress unless egress is CPU itself. */
		_jbit:1,	/* DMA Rd: Ingress indicates jumbo packet */
		_cos:3,		/* DMA Rd: Class of service from packet */
		_chain:1;	/* DMA Chaining */
#else /* BCM5632_BE_HOST */
	uint32	_chain:1,
		_cos:3,
		_jbit:1,
		_crc:2,
		_obit:2,
		_ip:1,
		:1,
	        :2,		/* Reserved (0) */
	        _sg:1,		/* Scatter Gather */
	        _rld:1,		/* Reload */
	        :1,		/* Reserved (0) */
	        _cnt:16;	/* Byte Count */
#endif /* BCM5632_BE_HOST */
    } _dmac;
} dmac_t;			/* Control bit definition */

#define	dmac_cnt	_dmac._cnt
#define	dmac_ipx	_dmac._ipx
#define	dmac_ip		_dmac._ip
#define	dmac_obit	_dmac._obit
#define	dmac_crc	_dmac._crc
#define	dmac_jbit	_dmac._jbit
#define	dmac_cos	_dmac._cos
#define	dmac_chain	_dmac._chain
#define	dmac_sg		_dmac._sg
#define	dmac_rld	_dmac._rld

#define DMAC_INIT(dp, len)	((dp).dmac_val = 0, (dp).dmac_cnt = (len))

#define DMAC_OBIT_NONE		0	/* Normal packet */
#define DMAC_OBIT_CBIT		1	/* Packet was sent to CPU as a */
					/*   result of the C bit being set */
					/*   in the default router table */
#define DMAC_OBIT_FTYPE		2	/* Frame type mismatch (unused) */
#define DMAC_OBIT_PURGE		3	/* Packet from GARBAGE_COLLECT */

#define	DMAC_CRC_LEAVE	0		/* Egress should leave packet alone */
#define	DMAC_CRC_REGEN	2		/* Egress should regenerate CRC */


/*
 * DMA status word #0: updated by DMA controller on SOC -> memory
 * master write over PCI.
 */
typedef union {
    uint32	dmas0_val;	/* Raw 32-bit value */
    struct {
#ifdef BCM5632_LE_HOST
	uint32	_cnt:16,/* Transferred byte count (SOC -> MEM) */
		:4,	/* Reserved */
		_sw:1,	/* Start of packet transferred from SOC ->MEM */
		_ow:2,	/* 0:unused, 1:C bit set, 2:Frame Mismatch,3:rsvp */
		_crc:2,	/* If 0, packet contains valid CRC. */
			/* If 1, packet has no CRC */
			/* If 2, packet has wrong CRC */
			/* The CPU is actually being asked to perform a */
			/*   CRC function as if it were any other egress */
		:1,	/* Reserved */
		_cosw:3,/* COS value for packet transferred from SOC ->MEM */
		_utw:1,	/* If 1, CMIC was asked to remove tag from packet */
			/*    (as if it were any other egress) */
		_ew:1,	/* Last packet or end of packet */
		_v:1;	/* Status word valid */
#else
	uint32	_v:1,
		_ew:1,
		_utw:1,
		_cosw:3,
		:1,
		_crc:2,
		_ow:2,
		_sw:1,
		:4,
		_cnt:16;
#endif
    } _dmas0;
} dmas0_t;

#define	dmas0_cnt	_dmas0._cnt
#define	dmas0_sw	_dmas0._sw
#define	dmas0_ow	_dmas0._ow
#define	dmas0_crc	_dmas0._crc
#define	dmas0_cosw	_dmas0._cosw
#define	dmas0_utw	_dmas0._utw
#define	dmas0_ew	_dmas0._ew
#define	dmas0_v		_dmas0._v

typedef union {
    uint32	dmas1_val;	/* Full Raw value */
    struct {
#ifdef BCM5632_LE_HOST
	uint32	_sp:6,	/* Source port (Ingress-Origin) for this packet */
		:2,	/* Reserved */
		_d:1,	/* DMA Done */
		:5,	/* Reserved */
		_op:18;	/* CPU opcode (tells what to do with packet) */
#else
	uint32	_op:18,
		:5,
		_d:1,
		:2,
		_sp:6;
#endif
    } _dmas1;
} dmas1_t;

#define	dmas1_sp	_dmas1._sp
#define	dmas1_op	_dmas1._op
#define	dmas1_d		_dmas1._d

/*
 * CPU Opcodes
 */

#define	DMAS1_OP_FM	0x00001		/* Filter match and one of the
					 * action bits is send to CPU */
#define	DMAS1_OP_VSL	0x00002		/* PTABLE cpu-learn bit=1 and src
					 * MAC is learned in ARL; or CML=1
					 * and src MAC lookup failure; or
					 * or VLAN ID not found in VTABLE;
					 * or VLAN ID = 0xfff */
#define	DMAS1_OP_SRS	0x00004		/* Source routing bit is set in
					 * source MAC address (bit 40) */
#define	DMAS1_OP_DLF	0x00008		/* DLF or L3 station movement */
#define DMAS1_OP_CF	0x00010		/* Control frame: packet is BPDU,
					 * GVRP, GMRP, or reserved addr */
#define DMAS1_OP_IPSW	0x00020		/* Packet needs to be IP-switched */
#define DMAS1_OP_IPINF	0x00080		/* IP Info bit: set if IP options
					 * are present or IP version not 4 */
#define DMAS1_OP_IPMC	0x00100		/* Set for class D multicast pkt
					 * with DA = 01 00 5e xx yy zz, where
					 * bit 7 of xx is also zero */
#define DMAS1_OP_TTL	0x00200		/* Set if TTL <= 0 after decrement,
					   or less than threshold for IPMC */
#define DMAS1_OP_BCA	0x00400		/* Set if broadcast packet */
#define DMAS1_OP_MCA	0x00800		/* Set if (non-IP) multicast packet */
#define DMAS1_OP_IGMP	0x01000		/* IGMP packet */

/*
 * Typedef: DMA Control Block (dcb)
 * Purpose: Maps the CMIC DMA Control block area.
 *
 * Note:  Byte-swapping is required when reading or writing the 3 words
 * found in this structure.  For this reason it is illegal to access the
 * structure members directly.  Instead, the DCB_SET_* and DCB_GET_*
 * macros defined below should be used.
 */
typedef struct dcbt_s {
    dmat0_t	_dcb_dmat0;		/* Tx DMA control word 0 */
    dmat1_t	_dcb_dmat1;		/* Tx DMA control word 1 */
    dmat2_t	_dcb_dmat2;		/* Tx DMA control word 2 */
} dcbt_t;

#define DCB_SET_DMAT0(dcb, dmat0) \
    ((dcb)->_dcb_dmat0.dmat0_val = pci_swap_5632(dmat0.dmat0_val))
#define DCB_SET_DMAT1(dcb, dmat1) \
    ((dcb)->_dcb_dmat1.dmat1_val = pci_swap_5632(dmat1.dmat1_val))
#define DCB_SET_DMAT2(dcb, dmat2) \
    ((dcb)->_dcb_dmat2.dmat2_val = pci_swap_5632(dmat2.dmat2_val))

#define DCB_CLR_DMAT0(dcb) \
    ((dcb)->_dcb_dmat0.dmat0_val = 0)
#define DCB_CLR_DMAT1(dcb) \
    ((dcb)->_dcb_dmat1.dmat1_val = 0)
#define DCB_CLR_DMAT2(dcb) \
    ((dcb)->_dcb_dmat2.dmat2_val = 0)

#define DCB_GET_DMAT0(dcb, dmat0) \
    ((dmat0).dmat0_val = pci_swap_5632((dcb)->_dcb_dmat0.dmat0_val))
#define DCB_GET_DMAT1(dcb, dmat1) \
    ((dmat1).dmat1_val = pci_swap_5632((dcb)->_dcb_dmat1.dmat1_val))
#define DCB_GET_DMAT2(dcb, dmat2) \
    ((dmat2).dmat2_val = pci_swap_5632((dcb)->_dcb_dmat2.dmat2_val))

typedef struct dcbr_s {
    dmar0_t	_dcb_dmar0;		/* Rx DMA control word 0 */
    dmar1_t	_dcb_dmar1;		/* Rx DMA control word 1 */
} dcbr_t;

#define DCB_SET_DMAR0(dcb, dmar0) \
    ((dcb)->_dcb_dmar0.dmar0_val = pci_swap_5632(dmar0.dmar0_val))
#define DCB_SET_DMAR1(dcb, dmar1) \
    ((dcb)->_dcb_dmar1.dmar1_val = pci_swap_5632(dmar1.dmar1_val))

#define DCB_CLR_DMAR0(dcb) \
    ((dcb)->_dcb_dmar0.dmar0_val = 0)
#define DCB_CLR_DMAR1(dcb) \
    ((dcb)->_dcb_dmar1.dmar1_val = 0)

#define DCB_GET_DMAR0(dcb, dmar0) \
    ((dmar0).dmar0_val = pci_swap_5632((dcb)->_dcb_dmar0.dmar0_val))
#define DCB_GET_DMAR1(dcb, dmar1) \
    ((dmar1).dmar1_val = pci_swap_5632((dcb)->_dcb_dmar1.dmar1_val))

/*
 * Typedef: DMA Control Block (dcb)
 * Purpose: Maps the CMIC DMA Control block area.
 *
 * Note:  Byte-swapping is required when reading or writing the 6 words
 * found in this structure.  For this reason it is illegal to access the
 * structure members directly.  Instead, the DCB_SET_* and DCB_GET_*
 * macros defined below should be used.
 */
typedef struct dcb_s {
    physaddr_t	_dcb_addr;		/* 32-bit physical address */
    dmac_t	_dcb_dmac;		/* DMA control word */
    pbmp_t	_dcb_portbmp;		/* Port map definitions */
    pbmp_t	_dcb_utagbmp;		/* CP Channel transfers */
    dmas0_t	_dcb_dmas0;		/* DMA Status 0 */
    dmas1_t	_dcb_dmas1;		/* DMA Status 1 */
} dcb_t;

#define DCB_SET_ADDR(dcb, addr) \
    ((dcb)->_dcb_addr = pci_swap_5632(addr))
#define DCB_SET_DMAC(dcb, dmac) \
    ((dcb)->_dcb_dmac.dmac_val = pci_swap_5632(dmac.dmac_val))
#define DCB_SET_PORTBMP(dcb, pbmp) \
    ((dcb)->_dcb_portbmp = pci_swap_5632(pbmp))
#define DCB_SET_UTAGBMP(dcb, ubmp) \
    ((dcb)->_dcb_utagbmp = pci_swap_5632(ubmp))
#define DCB_SET_DMAS0(dcb, dmas0) \
    ((dcb)->_dcb_dmas0.dmas0_val = pci_swap_5632(dmas0.dmas0_val))
#define DCB_SET_DMAS1(dcb, dmas1) \
    ((dcb)->_dcb_dmas1.dmas1_val = pci_swap_5632(dmas1.dmas1_val))

#define DCB_CLR_DMAS0(dcb) \
    ((dcb)->_dcb_dmas0.dmas0_val = 0)
#define DCB_CLR_DMAS1(dcb) \
    ((dcb)->_dcb_dmas1.dmas1_val = 0)

#define DCB_ADDR(dcb) \
    (pci_swap_5632((dcb)->_dcb_addr))
#define DCB_GET_DMAC(dcb, dmac) \
    ((dmac).dmac_val = pci_swap_5632((dcb)->_dcb_dmac.dmac_val))
#define DCB_PORTBMP(dcb) \
    (pci_swap_5632((dcb)->_dcb_portbmp))
#define DCB_UTAGBMP(dcb) \
    (pci_swap_5632((dcb)->_dcb_utagbmp))
#define DCB_GET_DMAS0(dcb, dmas0) \
    ((dmas0).dmas0_val = pci_swap_5632((dcb)->_dcb_dmas0.dmas0_val))
#define DCB_GET_DMAS1(dcb, dmas1) \
    ((dmas1).dmas1_val = pci_swap_5632((dcb)->_dcb_dmas1.dmas1_val))

#define DCB_DONE(dcb) \
    ((pci_swap_5632((dcb)->_dcb_dmas1.dmas1_val) & 0x100) != 0)

/*
 * CMIC I/O API
 */

/* Get CMIC PCI register */
int cmic_pci_setreg_5632(int unit, uint32 addr, uint32 data);

/* Set CMIC PCI register */
uint32 cmic_pci_getreg_5632(int unit, uint32 addr);

/* Read internal register */
int cmic_read_reg_5632(int unit, uint32 addr, uint32 *data);

/* Write internal register */
int cmic_write_reg_5632(int unit, uint32 addr, uint32 data);

int cmic_write_memory_5632(int unit,
		      soc_mem_t mem,
		      int copyno,		/* COPYNO_ALL allowed */
		      int index,
		      void *entry_data);

int cmic_read_memory_5632(int unit,
		     soc_mem_t mem,
		     int copyno,
		     int index,
		     void *entry_data);

int cmic_search_memory_5632(int unit,
		       soc_mem_t mem,
		       void *key_data,
		       void *entry_data);

int cmic_insert_memory_5632(int unit,
		       soc_mem_t mem,
		       int copyno,		/* COPYNO_ALL allowed */
		       void *entry_data);

/* Write to PHY registers */
int cmic_write_miim_5632(int unit, uint8 phy_id,
		         uint8 phy_reg_addr, uint16 phy_wr_data);

/* Read from PHY registers */
int cmic_read_miim_5632(int unit, uint8 phy_id,
		        uint8 phy_reg_addr, uint16 *phy_rd_data);

/* Read from MIB tables */
int cmic_read_mib_5632(int unit, uint32 mib_entry, uint32 *mib_rd_data);

int cmic_write_xmiim_5632(int unit, uint8 phy_id, uint8 phy_bank,
        uint16 phy_reg_addr, uint16 phy_wr_data);

int cmic_read_xmiim_5632(int unit, uint8 phy_id, uint8 phy_bank,
           uint16 phy_reg_addr, uint16 *phy_rd_data);

int PackTableEntry(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
int UnpackTableEntry(soc_mem_t mem, unsigned char *from, unsigned char *to);
void Struct2Buf(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
void Buf2Struct(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
void struct2buf_ARL_5632(ARL_5632_m_entry_t *ae, uint32 *raw);
void buf2struct_ARL_5632(ARL_5632_m_entry_t *ae, uint32 *raw);
void struct2buf_VLAN_5632(VLAN_m_entry_t *ve, uint32 *raw);
void buf2struct_VLAN_5632(VLAN_m_entry_t *ve, uint32 *raw);

#ifndef CMIC_REG	/* ktsai (022201) */
/* Get register address for PCI mapping space */
#define CMIC_REG(r)	(reg_addr_5632(r,0))
#endif	/* CMIC_REG */

/*
 * Enable (unmask) or disable (mask) a set of CMIC interrupts.
 * These routines should be used instead of manipulating CMIC_IRQ_MASK
 * directly, since a shadow value is kept.  The return value is the
 * previous mask (can pass mask of 0 to just get the current mask).
 */
uint32 cmic_irq_enable_5632(int unit, uint32 mask);
uint32 cmic_irq_disable_5632(int unit, uint32 mask);

#endif /* BCM5632_CMIC_H */
