/*  
 *=============================================================================
 * File Name	: bmc12500-eth-mii.c
 *
 *  Description	: Defines and functions used to access the phy layer device
 *				  using the MII interface of the BMC12500 Ethernet MAC.
 *				  Code was ported from Broadcom. Their copyright appears below.
 *
 *  Created		: 10 October 2001
 *
 *  Author		: Frank Salzmann
 *
 *  RCS			: $Id: bmc12500-eth-mii.c,v 1.20 2002/10/08 23:45:45 tonyw Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

/////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2000,2001
//  Broadcom Corporation, Irvine, CA
// 
//  BROADCOM PROPRIETARY AND CONFIDENTIAL
//    
//  This software is furnished under a Broadcom license agreement
//  and may only be used in accordance with and subject to the terms 
//  and conditions of such license. 
/////////////////////////////////////////////////////////////////////////

#include "bmc12500-eth.h"

#define _BMC_MAKE64(x) ((uint64)(x))
#define _BMC_MAKE32(x) ((uint32)(x))

#define _BMC_MAKEMASK1(n) (_BMC_MAKE64(1) << _BMC_MAKE64(n))
#define _BMC_MAKEMASK1_32(n) (_BMC_MAKE32(1) << _BMC_MAKE32(n))

#define SBETH_MIIPOLL_TIMER	(4*CFE_HZ)

#define S_MAC_MDC           	0
#define S_MAC_MDIO_DIR      	1
#define S_MAC_MDIO_OUT      	2
#define S_MAC_GENC          	3
#define S_MAC_MDIO_IN       	4

#define M_MAC_MDC       		_BMC_MAKEMASK1(S_MAC_MDC)
#define M_MAC_MDIO_DIR      	_BMC_MAKEMASK1(S_MAC_MDIO_DIR)
#define M_MAC_MDIO_DIR_INPUT    _BMC_MAKEMASK1(S_MAC_MDIO_DIR)
#define M_MAC_MDIO_OUT      	_BMC_MAKEMASK1(S_MAC_MDIO_OUT)
#define M_MAC_GENC      		_BMC_MAKEMASK1(S_MAC_GENC)
#define M_MAC_MDIO_IN   	    _BMC_MAKEMASK1(S_MAC_MDIO_IN)
#define M_MAC_MDIO_DIR_OUTPUT	0

unsigned int bmc12500Eth_mii_read(bc1250Eth_regs_t *, int, int);
void bmc12500Eth_mii_write(bc1250Eth_regs_t *r,
	 					  int phyaddr, 
						  int regidx, 
						  unsigned int regval);
void bmc12500_mii_write_common(int, int, int, int);

extern int lsi_get_linkstatus(uint16);
extern int lsi_set_autoneg(uint16, uint16);
extern int bmc12500Eth_setMacSpeed(bc1250EthCfg_t *);
extern int bmc12500Eth_startMacFifo(bc1250EthCfg_t *);
extern int bmc12500Eth_startMac(bc1250EthCfg_t *);
extern int bmc12500Eth_stopMac(bc1250EthCfg_t *);
extern int bmc12500Eth_uiGetInput(char *);
#ifndef PROM_SIBYTE_BC	
extern void dp83820_writeMII_common(uint32, uint32, uint32, uint32);
extern uint32 dp83820_readMII_common(uint32, uint32);
#endif
extern void delay_us(uint32);

#ifdef PROM_SIBYTE_BC
int32 gt_mii_readSMI(uint32 phyAddr, uint32 regAddr, uint32 *value);
int32 gt_mii_writeSMI(uint32 phyAddr, uint32 regAddr, uint32 value, uint32 mask);
#endif

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_findPhy
 *
 * Description: 	Locates the first available phy.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		NONE
 *------------------------------------------------------------------------------
 */
void 
bmc12500Eth_mii_findPhy(bc1250EthCfg_t *pbmcEthCfg)
{
    int    phy;
    uint16 bmsr;
	bc1250Eth_regs_t *r;

#ifdef AGILE_HARDWARE
	pbmcEthCfg->phy.addr = 0x10;
	return;
#endif

	r = &pbmcEthCfg->regs;

    for (phy = 0; phy < 31; phy++) {
		bmsr = bmc12500Eth_mii_read(r, phy, MII_BMSR);
		if (bmsr != 0) {
			pbmcEthCfg->phy.addr = phy;
			return;
	    }
	}
    pbmcEthCfg->phy.addr = 0;

	return;
} // bmc12500Eth_mii_findPhy

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_initPhy
 *
 * Description: 	Initializes phy registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		NONE
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_mii_initPhy(bc1250EthCfg_t *cfg)
{
	int    i,j;
	uint32 phyAddr;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;
	phyAddr = cfg->phy.addr;


#ifdef AGILE_HARDWARE
	if( isTXRX() ) {
		//printf("[bmc12500Eth_mii_initPhy] : r = 0x%x\n", r);
		//printf("[bmc12500Eth_mii_initPhy] : phyAddr = 0x%x\n", phyAddr);

#ifndef PROM_SIBYTE_BC
		//-- RESET DP83820, PHY == BC5208 
		bmc12500Eth_mii_write(r, 2, 0, 0x00008000);
		bmc12500Eth_mii_write(r, 3, 0, 0x00008000);
		bmc12500Eth_mii_read(r, 2, 0);
		bmc12500Eth_mii_read(r, 3, 0);
		//-- Now setup the txrx and fp sibyte mac phys.
		for(j=0; j<2; j++) { 
			if( j== 0)
				phyAddr = 16;
			else
				phyAddr = 8;

			for(i=0; i < 4; i++, phyAddr++) {
    			bmc12500Eth_mii_write(r, phyAddr, 0x1e, 0x4090);
				delay_us(1000);
    			bmc12500Eth_mii_write(r, phyAddr, 0x1e, 0x0090);
				delay_us(1000);
    			bmc12500Eth_mii_write(r, phyAddr, 0x11, 0x6308);
				delay_us(1000);
    			bmc12500Eth_mii_write(r, phyAddr, 0x11, 0xe384);
				delay_us(1000);
			}

			// don't Set Loopback, backplane phys (bit 14)
			bmc12500Eth_mii_write(r, 8, 0, 0x00001140);
			bmc12500Eth_mii_write(r, 9, 0, 0x00001140);
			bmc12500Eth_mii_write(r, 10, 0, 0x00001140);
			bmc12500Eth_mii_write(r, 11, 0, 0x00001140);

			// don't set DP83820, PHY == BC5208 into loopback (bit 14)
			bmc12500Eth_mii_write(r, 0, 0, 0x00002500);
			bmc12500Eth_mii_write(r, 1, 0, 0x00002500);
			bmc12500Eth_mii_write(r, 2, 0, 0x00002500);
			bmc12500Eth_mii_write(r, 3, 0, 0x00002100);
		}
#else /* BOBCAT */		
               phyAddr = HW_PMC8354_ADDR;

               for(i=0; i < 4; i++, phyAddr++) {
                   gt_mii_writeSMI( phyAddr, 0x1e, 0x4090, 0xffff);
                   delay_us(1000);
                   gt_mii_writeSMI( phyAddr, 0x11, 0x6308, 0xffff);
                   delay_us(1000);
                   gt_mii_writeSMI(phyAddr, 0x11, 0xe384, 0xffff);
                   delay_us(1000);
                   gt_mii_writeSMI(phyAddr, 0, 0x00001140, 0xffff);
                   delay_us(1000);
               }

#endif

		//-- Isolate 17, 18, and 19
#if 0
		bmc12500Eth_mii_write(r, 17, 0x0, 0x540);
		bmc12500Eth_mii_write(r, 18, 0x0, 0x540);
		bmc12500Eth_mii_write(r, 19, 0x0, 0x540);
#endif
	}
	else
		return 0;
#endif
	return 0;
} // bmc12500Eth_mii_initPhy

int
bmc12500Eth_mii_initPhyFifo()
{
#ifndef NFP_FP

	uint32 i, data;
	if (!isTXRX()) /* init on TXRX only */
		return NFX_OK;

	if (isCheetah()) {
		/* initialize all 8353/8354 PHYs for Fifo mode */
#ifdef PROM_SIBYTE_BC	// for compilation
            for (i = 0; i < 4; i++) {
		gt_mii_writeSMI(HW_PMC8354_ADDR+i, 0x11, 0x6308, 0xffff);
		delay_us(1000);
		gt_mii_writeSMI(HW_PMC8354_ADDR+i, 0x11, 0xe300, 0xffff);
		delay_us(1000);
		gt_mii_writeSMI(HW_PMC8354_ADDR+i, 0x1d, 0x0ff0, 0xffff);
		delay_us(1000);
		gt_mii_writeSMI(HW_PMC8354_ADDR+i, 0x10, 0xaae8, 0xffff);
		delay_us(1000);

                /* if PHY-8354, don't write to register 0x1f 
                 */
                gt_mii_readSMI(HW_PMC8354_ADDR+i, 0x03, &data);
                delay_us(1000);
                if (((data & 0x0000ffff) != 0x8050) && 
                    ((data & 0x0000ffff) != 0x8051))
                {
		    gt_mii_writeSMI(HW_PMC8354_ADDR+i, 0x1f, 0xaa28, 0xffff);
		    delay_us(1000);
                }
                /* auto neg on, full-duplex, 1000Mbps */
                gt_mii_writeSMI(HW_PMC8354_ADDR+i, 0x00, 0x1140, 0xffff);
                delay_us(1000);
	}
#else	    
		/* initialize all 8353/8354 PHYs for Fifo mode */
		for (i = 0; i < 4; i++) {
			dp83820_writeMII_common(0, 0x08+i, 0x1e, 0x4090);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x1e, 0x4090);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x1e, 0x4090);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x08+i, 0x1e, 0x0090);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x1e, 0x0090);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x1e, 0x0090);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x08+i, 0x11, 0x6308);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x11, 0x6308);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x11, 0x6308);
			delay_us(1000);
#if 0
			dp83820_writeMII_common(0, 0x08+i, 0x11, 0xe302);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x11, 0xe302);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x11, 0xe302);
			delay_us(1000);
#else
			dp83820_writeMII_common(0, 0x08+i, 0x11, 0xe300);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x11, 0xe300);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x11, 0xe300);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x08+i, 0x1d, 0x0ff0);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x1d, 0x0ff0);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x1d, 0x0ff0);
			delay_us(1000);
#endif
			dp83820_writeMII_common(0, 0x08+i, 0x10, 0xaae8);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x10+i, 0x10, 0xaae8);
			delay_us(1000);
			dp83820_writeMII_common(0, 0x18+i, 0x10, 0xaae8);
			delay_us(1000);

            /* if PHY-8354, don't write to register 0x1f 
             */
    		data = dp83820_readMII_common(0x08+i, 0x03);
			delay_us(1000);
            if (((data & 0x0000ffff) != 0x8050) && 
                ((data & 0x0000ffff) != 0x8051))
            {
			    dp83820_writeMII_common(0, 0x08+i, 0x1f, 0xaa28);
			    delay_us(1000);
            }
    		data = dp83820_readMII_common(0x10+i, 0x03);
			delay_us(1000);
            if (((data & 0x0000ffff) != 0x8050) && 
                ((data & 0x0000ffff) != 0x8051))
            {
			    dp83820_writeMII_common(0, 0x10+i, 0x1f, 0xaa28);
			    delay_us(1000);
            }
    		data = dp83820_readMII_common(0x18+i, 0x03);
			delay_us(1000);
            if (((data & 0x0000ffff) != 0x8050) && 
                ((data & 0x0000ffff) != 0x8051))
            {
			    dp83820_writeMII_common(0, 0x18+i, 0x1f, 0xaa28);
			    delay_us(1000);
            }
            /* auto neg on, full-duplex, 1000Mbps */
            dp83820_writeMII_common(0, 0x08+i, 0x00, 0x1140);
            delay_us(1000);
            dp83820_writeMII_common(0, 0x10+i, 0x00, 0x1140);
            delay_us(1000);
            dp83820_writeMII_common(0, 0x18+i, 0x00, 0x1140);
            delay_us(1000);
		}
		/* initialize 5208 PHY */
		dp83820_writeMII_common(0, 0, 0, 0x00008000); /* reset */
		delay_us(1000);
		dp83820_writeMII_common(0, 1, 0, 0x00008000); /* reset */
		delay_us(1000);
		dp83820_writeMII_common(0, 2, 0, 0x00008000); /* reset */
		delay_us(1000);
		dp83820_writeMII_common(0, 3, 0, 0x00008000); /* reset */
		delay_us(1000);
		dp83820_readMII_common(0, 0);
		delay_us(1000);
		dp83820_readMII_common(1, 0);
		delay_us(1000);
		dp83820_readMII_common(2, 0);
		delay_us(1000);
		dp83820_readMII_common(3, 0);
		delay_us(1000);
		dp83820_writeMII_common(0, 0, 0, 0x00000400); /* isolate */
		delay_us(1000);
		dp83820_writeMII_common(0, 1, 0, 0x00000400); /* isolate */
		delay_us(1000);
		dp83820_writeMII_common(0, 2, 0, 0x00000400); /* isolate */
		delay_us(1000);
		dp83820_writeMII_common(0, 3, 0, 0x00002100); /* normal */
#endif	
	}
#endif
	return NFX_OK;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   bmc12500Eth_mii_readPhyReg
 *
 * Description:     Reads a 16-bit value phy register.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    November 06, 2001
 *                  
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_mii_readPhyReg( bc1250EthCfg_t *cfg )
{
    uint32 regAddr;

#ifndef NFP_FP
    uint32 value;
#endif
	uint32 phyAddr;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;

    phyAddr = bmc12500Eth_uiGetInput("Phy Address: ");
    regAddr = bmc12500Eth_uiGetInput("Reg Address: ");
#ifndef NFP_FP
	if (isCheetah())
#ifdef PROM_SIBYTE_BC
                (uint32)gt_mii_readSMI(phyAddr, regAddr, &value);
#else
                value = (uint32)dp83820_readMII_common(phyAddr, regAddr);
#endif
	else
		value = (uint32)bmc12500Eth_mii_read(0, phyAddr,regAddr);

	value &= 0xFFFF;
	printf("Value: %04x\n", value);
#endif
    return(NFX_OK);
} /* bmc12500Eth_mii_readPhyReg */

/*
 *------------------------------------------------------------------------------
 * Function Name:   bmc12500Eth_mii_writePhyRegs
 *
 * Description:     Write a 16-bit value into phy register.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    November 06, 2001
 *                  
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_mii_writePhyRegs( bc1250EthCfg_t *cfg )
{
    uint32 regAddr;
    uint32 value;
	uint32 phyAddr;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;

    phyAddr = bmc12500Eth_uiGetInput("Phy Address: ");
    regAddr = bmc12500Eth_uiGetInput("Reg Address: ");
    value   = bmc12500Eth_uiGetInput("Value: ");
    value &= 0xFFFF;
#ifndef NFP_FP
	if (isCheetah())
#ifdef PROM_SIBYTE_BC		
		gt_mii_writeSMI(phyAddr, regAddr, value, 0xffff);
#else		
        	dp83820_writeMII_common(0, phyAddr, regAddr, value);
#endif	
	else
		bmc12500Eth_mii_write(r,phyAddr,regAddr,value);
#endif
    return(NFX_OK);
} /* bmc12500Eth_mii_writePhyRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_sendData
 *
 * Description: 	Send data to the MII port. 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		None
 *------------------------------------------------------------------------------
 */
void 
bmc12500Eth_mii_sendData(bc1250Eth_regs_t *r, unsigned int data, int bitcnt)
{
    int 			i;
    unsigned int 	curmask;
    uint64 			bits;

    bits = M_MAC_MDIO_DIR_OUTPUT;
    bmc12500Eth_writeReg(r->mac_mdio, bits);

    curmask = 1 << (bitcnt - 1);
    for (i = 0; i < bitcnt; i++) {

		if (data & curmask) 
			bits |= M_MAC_MDIO_OUT;
		else 
			bits &= ~M_MAC_MDIO_OUT;

		bmc12500Eth_writeReg(r->mac_mdio,bits);
		bmc12500Eth_writeReg(r->mac_mdio,bits | M_MAC_MDC);
		bmc12500Eth_writeReg(r->mac_mdio,bits);
		curmask >>= 1;
	}
	return;
} // bmc12500Eth_mii_sendData

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_sync
 *
 * Description: 	Synchronize with the MII - send a pattern of bits to the 
 *					MII that will guarantee that it is ready to accept a 
 *					command.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		None
 *------------------------------------------------------------------------------
 */
void 
bmc12500Eth_mii_sync(bc1250Eth_regs_t *r)
{
    int cnt;
    uint64 bits;

    bits = M_MAC_MDIO_DIR_OUTPUT | M_MAC_MDIO_OUT;

    bmc12500Eth_writeReg(r->mac_mdio,bits);

    for (cnt = 0; cnt < 32; cnt++) {
		bmc12500Eth_writeReg(r->mac_mdio,bits | M_MAC_MDC);
		bmc12500Eth_writeReg(r->mac_mdio,bits);
	}
} // bmc12500Eth_mii_sync

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_read
 *
 * Description: 	Reads a phy register using the MII port.	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		Register value.	
 *------------------------------------------------------------------------------
 */
unsigned int 
bmc12500Eth_mii_read(bc1250Eth_regs_t *r, int phyaddr, int regidx)
{
    int idx;
    int error;
    int regval;

    // Synchronize ourselves so that the PHY knows the next
    // thing coming down is a command
    bmc12500Eth_mii_sync(r);

    // Send the data to the PHY.  The sequence is
    // a "start" command (2 bits)
    // a "read" command (2 bits)
    // the PHY addr (5 bits)
    // the register index (5 bits)
    bmc12500Eth_mii_sendData(r,MII_COMMAND_START, 2);
    bmc12500Eth_mii_sendData(r,MII_COMMAND_READ, 2);
    bmc12500Eth_mii_sendData(r,phyaddr, 5);
    bmc12500Eth_mii_sendData(r,regidx, 5);

	// Switch the port around without a clock transition.
    bmc12500Eth_writeReg(r->mac_mdio, M_MAC_MDIO_DIR_INPUT);

    // Send out a clock pulse to signal we want the status
    bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_INPUT | M_MAC_MDC);
    bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_INPUT);

    // If an error occured, the PHY will signal '1' back
    error = bmc12500Eth_readReg(r->mac_mdio) & M_MAC_MDIO_IN;

    //  Issue an 'idle' clock pulse, but keep the direction
	// the same.
    bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_INPUT | M_MAC_MDC);
    bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_INPUT);

    regval = 0;

    for(idx = 0; idx < 16; idx++) {
		regval <<= 1;

		if(error == 0) {
			if (bmc12500Eth_readReg(r->mac_mdio) & M_MAC_MDIO_IN) 
				regval |= 1;
	    }
		bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_INPUT | M_MAC_MDC);
		bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_INPUT);
	}

    // Switch back to output 
    bmc12500Eth_writeReg(r->mac_mdio,M_MAC_MDIO_DIR_OUTPUT);

    if (error == 0) 
		return regval;
	else
		return 0;

} // bmc12500Eth_mii_read

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_write
 *
 * Description: 	Writes a value into a phy register using MII interface. 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		None 
 *------------------------------------------------------------------------------
 */
void 
bmc12500Eth_mii_write(bc1250Eth_regs_t *r,
					  int phyaddr, 
					  int regidx, 
					  unsigned int regval)
{
    bmc12500Eth_mii_sync(r);

    bmc12500Eth_mii_sendData(r, MII_COMMAND_START, 2);
    bmc12500Eth_mii_sendData(r, MII_COMMAND_WRITE, 2);
    bmc12500Eth_mii_sendData(r, phyaddr, 5);
    bmc12500Eth_mii_sendData(r, regidx, 5);
    bmc12500Eth_mii_sendData(r, MII_COMMAND_ACK, 2);
    bmc12500Eth_mii_sendData(r, regval, 16);

    bmc12500Eth_writeReg(r->mac_mdio, M_MAC_MDIO_DIR_OUTPUT);

	return;
} // bmc12500Eth_mii_write

/* general MII write function for common PHY init */
void
bmc12500_mii_write_common(int mac_id, int phyaddr, int reg, int reg_val)
{
	bc1250EthCfg_t *cfg;
	bc1250Eth_regs_t *r;

	cfg = bcEthCfg[mac_id];
	r = &cfg->regs;
	bmc12500Eth_mii_write(r, phyaddr, reg, reg_val);
}

//#if defined(EVAL_BOARD)

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_cswarm_hack
 *
 * Description:  	The CSWARM boards leave the PHYs in JTAG mode.  The sequence
 *  				below turns off JTAG mode and puts the PHYs back
 *  				into their regular reset state.  This is only used with the 
 *					BCM5411
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		1 if 5411 detected.
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_mii_song_and_dance(bc1250Eth_regs_t *r)
{
    int phyid1,phyid2;

    phyid1 = bmc12500Eth_mii_read(r,1,MII_PHYIDR1);
    phyid2 = bmc12500Eth_mii_read(r,1,MII_PHYIDR2);

    // Check for the 5611.  Don't do this unless it is a 5611. 
    if ((phyid1 == 0x0020) && ((phyid2 & 0xFFF0) == 0x6070)) {

		printf("**** BMC12500 MII MSG: FOUND BMC5411 PHY ****\n");
		// It's a BCM5411 

		// clear ext loopback 
		bmc12500Eth_mii_write(r,1,MII_AUXCTL,0x0420);

		// clear swap rx MDIX/TXHalfOut bits 
		bmc12500Eth_mii_write(r,1,MII_AUXCTL,0x0004);

		// set up 10/100 advertisement 
		bmc12500Eth_mii_write(r,1,MII_ANAR,0x01E1);

		// set up 1000 advertisement 
		bmc12500Eth_mii_write(r,1,MII_K1CTL,0x0300);

		// set autonegotiate bit and restart autoneg 
		bmc12500Eth_mii_write(r,1,MII_BMCR,0x1340);
		return 1;
	}
    return 0;
}

//#endif  // EVAL_BOARD

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_poll
 *
 * Description:     Ask the PHY what is going on, and configure speed 
 *					appropriately. For the moment, we only support automatic 
 *					configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 10, 2001
 * 					
 * Returns: 		1 if changed else 0
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_mii_poll(bc1250EthCfg_t *pbmcEthCfg)
{
    uint16 bmsr, bmcr, k1stsr, anlpar;
	uint32 phyAddr;
#if 0
    int chg;
#endif
    char buffer[100];
    char *p = buffer;
	bc1250Eth_regs_t *r;
	bc1250Eth_phy_t  *phy;

	r = &pbmcEthCfg->regs;
	phyAddr = pbmcEthCfg->phy.addr;
	phy     = &pbmcEthCfg->phy;

    // Read the mode status and mode control registers. 
    bmsr = bmc12500Eth_mii_read(r, phyAddr, MII_BMSR);
    bmcr = bmc12500Eth_mii_read(r, phyAddr, MII_BMCR);

    // get the link partner status 
    anlpar = bmc12500Eth_mii_read(r, phyAddr, MII_ANLPAR);

    // If supported, read the 1000baseT register 
    if (bmsr & BMSR_1000BT_XSR) {
		k1stsr = bmc12500Eth_mii_read(r, phyAddr, MII_K1STSR);
	}
    else {
		k1stsr = 0;
	}


#if 0 // HACK
    chg = 0;
    if( (phy->p.bmc5411.bmsr != bmsr)	 ||
		(phy->p.bmc5411.bmcr != bmcr) 	 ||
		(phy->p.bmc5411.anlpar != anlpar) ||
		(phy->p.bmc5411.k1stsr != k1stsr) ) {

		phy->p.bmc5411.bmsr = bmsr;
		phy->p.bmc5411.bmcr = bmcr;
		phy->p.bmc5411.anlpar = anlpar;
		phy->p.bmc5411.k1stsr = k1stsr;
		chg = 1;
	}

    if (chg == 0) 
		return 0;
#endif // HACK

    p += sprintf(p,"Link speed: ");

	//---- HACK
	pbmcEthCfg->link.speed    = speed_1000;
	pbmcEthCfg->link.duplex   = full_duplex;
	pbmcEthCfg->link.fc		  = fc_frame; 
	pbmcEthCfg->link.linkstat = ETHER_SPEED_1000FDX;
	p += sprintf(p,"1000BaseT FDX");
	//---- HACK

#if 0
    if (k1stsr & K1STSR_LP1KFD) {
		pbmcEthCfg->link.speed    = speed_1000;
		pbmcEthCfg->link.duplex   = full_duplex;
		pbmcEthCfg->link.fc		  = fc_frame; 
		pbmcEthCfg->link.linkstat = ETHER_SPEED_1000FDX;
		p += sprintf(p,"1000BaseT FDX");
	}
    else if (k1stsr & K1STSR_LP1KHD) {
		pbmcEthCfg->link.speed    = speed_1000;
		pbmcEthCfg->link.duplex   = half_duplex;
		pbmcEthCfg->link.fc		  = fc_disabled; 
		pbmcEthCfg->link.linkstat = ETHER_SPEED_1000HDX;
		p += sprintf(p,"1000BaseT HDX");
	}
    else if (anlpar & ANLPAR_TXFD) {
		pbmcEthCfg->link.speed    = speed_100;
		pbmcEthCfg->link.duplex   = full_duplex;
		pbmcEthCfg->link.fc		  = (anlpar & ANLPAR_PAUSE) ? fc_frame : fc_disabled;
		pbmcEthCfg->link.linkstat = ETHER_SPEED_100FDX;
		p += sprintf(p,"100BaseT FDX");
	}
    else if (anlpar & ANLPAR_TXHD) {
		pbmcEthCfg->link.speed    = speed_100;
		pbmcEthCfg->link.duplex   = half_duplex;
		pbmcEthCfg->link.fc		  = fc_disabled; 
		pbmcEthCfg->link.linkstat = ETHER_SPEED_100HDX;
		p += sprintf(p,"100BaseT HDX");
	}
    else if (anlpar & ANLPAR_10FD) {
		pbmcEthCfg->link.speed    = speed_10;
		pbmcEthCfg->link.duplex   = full_duplex;
		pbmcEthCfg->link.fc		  = fc_frame; 
		pbmcEthCfg->link.linkstat = ETHER_SPEED_10FDX;
		p += sprintf(p,"10BaseT FDX");
	}
    else if (anlpar & ANLPAR_10HD) {
		pbmcEthCfg->link.speed    = speed_10;
		pbmcEthCfg->link.duplex   = half_duplex;
		pbmcEthCfg->link.fc		  = fc_collision; 
		pbmcEthCfg->link.linkstat = ETHER_SPEED_10HDX;
		p += sprintf(p,"10BaseT HDX");
	}
    else {
		pbmcEthCfg->link.linkstat = ETHER_SPEED_UNKNOWN;
		pbmcEthCfg->link.speed    = speed_unknown;
		pbmcEthCfg->link.duplex   = unknown_duplex;
		pbmcEthCfg->link.fc		  = fc_unknown; 
		p += sprintf(p,"Unknown");
	}
#endif

	//-- printf("\n%s: %s\n","BMC12500 Eth",buffer);

    return 1;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_phyLoopback
 *
 * Description:     Sets/Clears internal loopback mode in the phy.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 05, 2001
 * 					
 * Returns: 		1 if changed else 0
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_mii_phyLoopback(bc1250EthCfg_t *cfg, int mode)
{
    uint16 val;
	uint32 phyAddr;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;
	phyAddr = cfg->phy.addr;

	if( mode == BC1250_SET ) {
		val = bmc12500Eth_mii_read(r, phyAddr, MII_BMCR);
		val |= 0x4000;
		bmc12500Eth_mii_write(r, phyAddr, MII_BMCR, val);
	}
	else {
		bmc12500Eth_mii_write(r, phyAddr, MII_BMCR, 0x8000); // Just reset the phy
	}
	return 0;
} // bmc12500Eth_mii_phyLoopback

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_viewPhyRegs
 *
 * Description:     Displays PHY registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 06, 2001
 * 					
 * Returns: 		
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_mii_viewPhyRegs(bc1250EthCfg_t *cfg)
{
	uint32 i;
	uint32 phyAddr;
	bc1250Eth_regs_t *r;

#ifndef NFP_FP
    uint16 val;

	if (isCheetah()) {
#ifdef PROM_SIBYTE_BC
		for (i = HW_PMC8354_ADDR; i <= 0x13; i++) {
			(uint32)gt_mii_readSMI(i, 0, &val);
			printf("8354 Phy Id 0x%x: %x\n", i, val);
		}
#else		
		for (i = 0x08; i <= 0x0b; i++) {
			val = (uint32)dp83820_readMII_common(i, 0);
			printf("8353 Phy Id 0x%x: %x\n", i, val);
		}
		printf("\n");
		for (i = 0x10; i <= 0x13; i++) {
			val = (uint32)dp83820_readMII_common(i, 0);
			printf("8353 Phy Id 0x%x: %x\n", i, val);
		}
		printf("\n");
		for (i = 0x18; i <= 0x1b; i++) {
			val = (uint32)dp83820_readMII_common(i, 0);
			printf("8353 Phy Id 0x%x: %x\n", i, val);
		}
		printf("\n");
		val = (uint32)dp83820_readMII_common(3, 0);
		printf("5208 Phy Id 0x%x: %x\n", i, val);
#endif		
		return 0;
	}
#endif
	r = &cfg->regs;
	phyAddr = cfg->phy.addr;

	if( isTXRX() )
		phyAddr = 8;
	else {
		printf("Can only see on TXRX!!\n");
		return 0;
	}

	phyAddr = 8;
	for(i=0; i < 2; i++, phyAddr++) {
	printf("\nPHY REGISTER DUMP (FP Backplane Phy Addr = 0x%x):\n", phyAddr);
	printf("============================\n");
	printf("MII_BMCR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_BMCR));
	printf("MII_BMSR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_BMSR));
	printf("MII_PHYDR1:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_PHYIDR1));
	printf("MII_PHYDR2:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_PHYIDR2));
	printf("MII_ANAR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANAR));
	printf("MII_ANLPAR:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANLPAR));
	printf("MII_ANER  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANER));
	printf("MII_K1CTL :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_K1CTL));
	printf("MII_K1STSR:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_K1STSR));
	printf("MII_AUXCTL:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_AUXCTL));
	printf("TEST1     :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, 0x1e));
	printf("TEST2     :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, 0x1f));
	printf("\n");
	}

	phyAddr = 16;
	for(i=0; i < 2; i++, phyAddr++) {
	printf("\nPHY REGISTER DUMP (TxRx External Phy Addr = 0x%x):\n", phyAddr);
	printf("============================\n");
	printf("MII_BMCR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_BMCR));
	printf("MII_BMSR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_BMSR));
	printf("MII_PHYDR1:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_PHYIDR1));
	printf("MII_PHYDR2:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_PHYIDR2));
	printf("MII_ANAR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANAR));
	printf("MII_ANLPAR:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANLPAR));
	printf("MII_ANER  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANER));
	printf("MII_K1CTL :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_K1CTL));
	printf("MII_K1STSR:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_K1STSR));
	printf("MII_AUXCTL:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_AUXCTL));
	printf("TEST1     :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, 0x1e));
	printf("TEST2     :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, 0x1f));
	printf("\n");
	}

	phyAddr = 3;
	for(i=0; i < 1; i++, phyAddr++) {
	printf("\nPHY REGISTER DUMP (DP83820 Backplane Phy Addr = 0x%x):\n", phyAddr);
	printf("============================\n");
	printf("MII_BMCR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_BMCR));
	printf("MII_BMSR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_BMSR));
	printf("MII_PHYDR1:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_PHYIDR1));
	printf("MII_PHYDR2:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_PHYIDR2));
	printf("MII_ANAR  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANAR));
	printf("MII_ANLPAR:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANLPAR));
	printf("MII_ANER  :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_ANER));
	printf("MII_K1CTL :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_K1CTL));
	printf("MII_K1STSR:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_K1STSR));
	printf("MII_AUXCTL:   %04x\n", bmc12500Eth_mii_read(r, phyAddr, MII_AUXCTL));
	printf("TEST1     :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, 0x1e));
	printf("TEST2     :   %04x\n", bmc12500Eth_mii_read(r, phyAddr, 0x1f));
	printf("\n");
	}
	return 0;
} // bmc12500Eth_mii_viewPhyRegs

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_mii_phyBackToBack
 *
 * Description:     Sets/Clears internal loopback mode in the phy.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 05, 2001
 * 					
 * Returns: 		1 if changed else 0
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_mii_phyBackToBack(bc1250EthCfg_t *cfg, int mode)
{
    uint16 val;
	uint32 phyAddr;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;
	phyAddr = cfg->phy.addr;

	if( mode == BC1250_SET ) {
		val = bmc12500Eth_mii_read(r, phyAddr, MII_BMCR);
		val |= 0x4000;
		bmc12500Eth_mii_write(r, phyAddr, MII_BMCR, val);
	}
	else {
		bmc12500Eth_mii_write(r, phyAddr, MII_BMCR, 0x8000); // Just reset the phy
	}
	return 0;
} // bmc12500Eth_mii_phyBackToBack

int bmc12500Eth_set_autoneg(uint16 port, uint16 mode)
{
#ifndef NFP_FP
    bc1250EthCfg_t *cfg;
    bc1250Eth_regs_t *r;
    uint16 reg_val;
    int phy_id;

    if (!isTXRX()) {
        printf("bmc12500Eth_set_autoneg: available on TXRX only!\n");
        return NFX_ERR;
    }
    if (isCheetah()) {
        switch (port) {
            case 0:
            case 1:
                cfg = bcEthCfg[0];
                break;
            case 2:
            case 3:
                cfg = bcEthCfg[2];
                break;
            default:
                printf("bmc12500Eth_set_autoneg: invalid port=%d\n", port);
                return NFX_ERR;
        }      
        phy_id = HW_PMC8354_ADDR + port;
#ifdef PROM_SIBYTE_BC	
        gt_mii_readSMI(phy_id, MII_BMCR, &reg_val);
#else	
        reg_val = dp83820_readMII_common(phy_id, MII_BMCR);
#endif	
        if (mode == 1) {
            reg_val |= 0x1200;
        }
        else
        if (mode == 0) {
            reg_val &= 0xedff;
        }
        else {
            printf("bmc12500Eth_set_autoneg: invalid mode=%d\n", mode);
            return NFX_ERR;
        }
        bmc12500Eth_stopMac(cfg);
#ifdef PROM_SIBYTE_BC	
        gt_mii_writeSMI(phy_id, MII_BMCR, reg_val, 0xffff);
#else	
        dp83820_writeMII_common(0, phy_id, MII_BMCR, reg_val);
#endif	
        lsi_set_autoneg(port, mode);
        bmc12500Eth_startMacFifo(cfg);
    }
    else
    {
        if (port == 0 || port == 1)
            cfg = bcEthCfg[port];
        else {
            printf("bmc12500Eth_set_autoneg: invalid port=%d\n", port);
            return NFX_ERR;
        }
        r = &cfg->regs;
        bmc12500Eth_stopMac(cfg);
        phy_id = HW_PMC8354_ADDR + port;
        reg_val = bmc12500Eth_mii_read(r, phy_id, MII_BMCR);
        if (mode == 1) {
            reg_val |= 0x1200;
            bmc12500Eth_mii_write(r, phy_id, MII_BMCR, reg_val);
        }
        else
        if (mode == 0) {
            reg_val &= 0xedff;
            bmc12500Eth_mii_write(r, phy_id, MII_BMCR, reg_val);
        }
        bmc12500Eth_startMac(cfg);
    }
#endif
    return NFX_OK;
}

int bmc12500Eth_get_autoneg(uint16 port)
{
#ifndef NFP_FP
    bc1250EthCfg_t *cfg;
    bc1250Eth_regs_t *r;
    uint16 reg_val=0;
    int phy_id;

    if (!isTXRX()) {
        printf("bmc12500Eth_get_autoneg: available on TXRX only!\n");
        return NFX_ERR;
    }
    if (isCheetah()) {
        switch (port) {
            case 0:
            case 1:
                cfg = bcEthCfg[0];
                break;
            case 2:
            case 3:
                cfg = bcEthCfg[2];
                break;
            default:
                printf("bmc12500Eth_get_autoneg: invalid port=%d\n", port);
                return NFX_ERR;
        }      
        phy_id = HW_PMC8354_ADDR + port;
#ifdef PROM_SIBYTE_BC	
        gt_mii_readSMI(phy_id, MII_BMCR, &reg_val);
#else	
        reg_val = dp83820_readMII_common(phy_id, MII_BMCR);
#endif	
    }
    else /* Garfield */
    {
        if (port == 0 || port == 1)
            cfg = bcEthCfg[port];
        else {
            printf("bmc12500Eth_get_autoneg: invalid port=%d\n", port);
            return NFX_ERR;
        }
        r = &cfg->regs;
        phy_id = HW_PMC8354_ADDR + port;
        reg_val = bmc12500Eth_mii_read(r, phy_id, MII_BMCR);
    }
    if (reg_val & 0x1000)
        return 1;
    else
        return 0;
#endif
	return 0;
}

int bmc12500Eth_set_maxPktSize(uint16 port, uint16 size)
{
#ifndef NFP_FP
    bc1250EthCfg_t *cfg;
    bc1250Eth_regs_t *r;
    bmc12500_macFrmCfgr_t macFrameCfg;

    if (!isTXRX()) {
        printf("bmc12500Eth_set_maxPktSize: available on TXRX only!\n");
        return NFX_ERR;
    }
    if (isCheetah()) {
        switch (port) {
            case 0:
            case 1:
                cfg = bcEthCfg[0];
                break;
            case 2:
            case 3:
                cfg = bcEthCfg[2];
                break;
            default:
                printf("bmc12500Eth_set_maxPktSize: invalid port=%d\n", port);
                return NFX_ERR;
        }      
    }
    else
    {
        if (port == 0 || port == 1)
            cfg = bcEthCfg[port];
        else {
            printf("bmc12500Eth_set_maxPktSize: invalid port=%d\n", port);
            return NFX_ERR;
        }
    }
    if (size < 64) {
        printf("bmc12500Eth_set_maxPktSize: invalid size=%d\n", size);
        return NFX_ERR;
    }
    r = &cfg->regs;
    bmc12500Eth_stopMac(cfg);
    macFrameCfg.wd = bmc12500Eth_readReg(r->mac_frame_cfg);
    macFrameCfg.bits.max_framesz = size;
    bmc12500Eth_writeReg(r->mac_frame_cfg, macFrameCfg.wd);
    if (isCheetah())
        bmc12500Eth_startMacFifo(cfg);
    else
        bmc12500Eth_startMac(cfg);
#endif
    return NFX_OK;
}

int bmc12500Eth_get_maxPktSize(uint16 port)
{
#ifndef NFP_FP
    bc1250EthCfg_t *cfg;
    bc1250Eth_regs_t *r;
    bmc12500_macFrmCfgr_t macFrameCfg;
    uint16 size;

    if (!isTXRX()) {
        printf("bmc12500Eth_get_maxPktSize: available on TXRX only!\n");
        return NFX_ERR;
    }
    if (isCheetah()) {
        switch (port) {
            case 0:
            case 1:
                cfg = bcEthCfg[0];
                break;
            case 2:
            case 3:
                cfg = bcEthCfg[2];
                break;
            default:
                printf("bmc12500Eth_get_maxPktSize: invalid port=%d\n", port);
                return NFX_ERR;
        }      
    }
    else
    {
        if (port == 0 || port == 1)
            cfg = bcEthCfg[port];
        else {
#ifndef PROM
            printf("bmc12500Eth_get_maxPktSize: invalid port=%d\n", port);
#endif
            return NFX_ERR;
        }
    }
    r = &cfg->regs;
    macFrameCfg.wd = bmc12500Eth_readReg(r->mac_frame_cfg);
    size = macFrameCfg.bits.max_framesz;
    return (size);
#endif
	return 0;
}

int bmc12500Eth_set_speed(uint16 port, uint16 speed)
{
    bc1250EthCfg_t *cfg;

    return NFX_OK;
    /* phy ONLY supports 1000 full-duplex, so this is not useful for now */

    if (!isTXRX()) {
        printf("bmc12500Eth_set_speed: available on TXRX only!\n");
        return NFX_ERR;
    }
    if (port == 0 || port == 1)
        cfg = bcEthCfg[port];
    else {
        printf("bmc12500Eth_set_speed: invalid port=%d\n", port);
        return NFX_ERR;
    }
    switch (speed) {
        case 0: cfg->link.speed = speed_10;
                break;
        case 1: cfg->link.speed = speed_100;
                break;
        case 2: cfg->link.speed = speed_1000;
                break;
        default:printf("bmc12500Eth_set_speed: invalid speed=%d\n", speed);
                return NFX_ERR;
    }
    bmc12500Eth_stopMac(cfg);
    bmc12500Eth_setMacSpeed(cfg);
    bmc12500Eth_startMac(cfg);
    return NFX_OK;
}

int bmc12500Eth_get_speed(uint16 port)
{
    if (!isTXRX()) {
        printf("bmc12500Eth_get_speed: available on TXRX only!\n");
        return NFX_ERR;
    }
    return (speed_1000); /* only speed that is supported */
}

int bmc12500Eth_get_linkstatus(uint16 port)
{
    int status = 0;

    if (isCheetah()) {
        if (port > 4) {
            printf("bmc12500Eth_get_linkstatus: invalid port=%d\n", port);
            return NFX_ERR;
        }
        status = lsi_get_linkstatus(port);
    }
    else {
	/* need special code for Garfield because PHY status is not yet available */
        status = 0; 
    }
    return (status);
}


