/*  
 *=============================================================================
 * File Name  :    bmc12500-eth-rx.c
 *
 * Description:    Receive driver code for the BMC12500 Ethernet Mac.    
 *
 * Created    :    18 October 2001
 *
 * Author     :    Frank Salzmann
 *
 * RCS        :    $Id: bmc12500-eth-rx.c,v 1.62 2002/12/07 01:50:42 tonyw Exp $
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include "bmc12500-eth.h"
#include <exports/mem-api.h>
#include "../sm-eee/eee-test.h"

int bmc12500Eth_rx_pkt_trace = 0;
#ifndef PROM
extern uint32 sysDVTrunning;
extern uint32 txrxdebug; /* This flag is used for txrx pkt dump.*/
#endif
#ifndef SIBYTE_BC
#define LMUX_DEBUG
#endif
#define LUC_DEBUG
#ifdef LMUX_DEBUG
uint32 lmux_rxPkts[6] = { 0,0,0,0,0,0 };
uint32 lmux_rxBytes[6] = { 0,0,0,0,0,0 };
#endif
#ifdef LUC_DEBUG
uint32 luc_rxPkts[4] = { 0,0,0,0 };
uint32 luc_rxBytes[4] = { 0,0,0,0 };
#endif
extern uint32 lmux_loopback_test;
extern uint32 luc_loopback_test;
extern uint32 bc1250_diagCPU;
extern uint32 bc1250_FP2CPU_active;
extern uint32 bc1250_FP1CPU_active;
extern dvt_front_cfg dvtFrontCfg;
extern eee_test_hdr_t  * test_hdr;

uint32 get_word(char *buf)
{
   int i;
   uint32 tmp = 0;
   for(i=0;i<4;i++)
   {
      tmp |= ((*(buf + i)) << (i*8));
   }
   return tmp;
}
void bmc12500Eth_decodeHdr(char*);
int bmc1250_rxPktCapture(char*, uint32, int32);
int bmc12500Eth_printBuf(char*, int, int);
int bmc12500Eth_decodeScsi(char *); 

extern void luc_rcvPkt(eee_desc_t *);

#define EEE_MAC_HDR_LEN		16
#define EEE_AGILE_HDR_LEN	8

uint32 lmux_get_rev = 1;
uint32 lmux_rev[3] = {0,0,0};

int verify_test_packet(eee_desc_t 	*edesc);
int bobcat_test_pkt=0;

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_rxPkt_net
 *
 * Description: 	Receives a unicast packet.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_rxPkt_net( bc1250Eth_rxCtrl_t *rxc, 
					   bc1250EthCfg_t     *cfg, 
					   uint32             loopCnt )
{
	uint32			procCnt;		// Keep track of number of desc processed.
	uint32 			pktLen;
	uint32			bufCnt;
	uchar8 			*newBuf;
	uint64 			status;
	eee_desc_t     	*edesc;
	volatile eee_descBuf_t	*bd;
	volatile bc1250_rxdesc_t*rxCur;
#ifndef NFP_TXRX
	volatile agile_hdr_t	*eeeHdr;
	uchar8 *lmux_hdr = NULL;
	int hdrlen;
#endif
	int	err_flag;
#ifndef NFP_FP
	uchar8 *luc_hdr;
	uchar8 idx;
#endif
uint64 tmp;

	procCnt = 0;
	rxCur = rxc->rxCur;
	
	while( procCnt < loopCnt ) {

		// Make sure that the descriptor we are looking at indicates
		// that the MAC	has completed receiving the entire packet.
		status = rxCur->dw1;
		if( !(status & BC1250_DESC_STATUS_SOP) ) 
			goto bmc12500_rx_finish;

		err_flag = 0;
		if (status & (BC1250_DESC_STATUS_BAD | BC1250_DESC_STATUS_ERR)) {
		//	printf("RX Error, status = 0x%lx\n", status);
		//	printf("MAC Status = 0x%lx\n", *(cfg->regs.mac_status));
			rxc->rxBadErr++;
			err_flag = 1;
		}

		//-- Allocate EEE packet descriptor.
		edesc = eee_allocateDesc(rxc->bufPoolId, NFX_COS_HIGH);
		if( !edesc ) {
			rxc->noDescErr++;
			goto bmc12500_rx_finish;
		}
        eee_attrAssignDesc(edesc, EEE_ATTR_MAC_RX);

		// Get packet len from 2nd descriptor word.
		pktLen = BC1250_GET_PKT_SIZE(rxCur);

		/* Update pkt Stats */
		rxc->pkts++;
		rxc->octets += pktLen;

		// Calculate the buffer count for this packet.
		bufCnt = (pktLen + rxc->bufOffset) / rxc->descBufSize;
		if( (pktLen + rxc->bufOffset) % rxc->descBufSize )
			bufCnt++;
		//-- Strip CRC bytes 
		if (!isCheetah())
			pktLen -= 4;

#ifdef SIBYTE_BC
			pktLen -= 4;
#endif			

		//-- Mark first buffer as unused.
		edesc->bd[0].control = 0;
		bd = &(edesc->bd[1]);

		if( bmc12500Eth_rx_pkt_trace) {
			printf("BMC Rx: pktLen: %d, BufCnt: %d, Status: %016llx\n", 
					pktLen, bufCnt, status & BC1250_DESC_STATUS );
		}

		if( bufCnt == 1 ) {
			//-- Allocate a new receive buffer and add to rx queue.
			newBuf = eee_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
			if( !newBuf ) {
				rxc->noBufErr++;
				eee_deallocateDesc(edesc);
				goto bmc12500_rx_finish;      // Wait for more bufs.
			}

            eee_attrAssignBuf(newBuf, EEE_ATTR_MAC_RX);

			//-- Grab buffer pointer from rx desc.
			BC1250_DESC_GET_A_ADDR(rxCur, (address_t)bd->buf);
			bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
			
			//-- Data len equal to packet len minus the 4-byte CRC.
			bd->len = pktLen;

			if( bmc12500Eth_rx_pkt_trace) {
				printf("BMC Rx: bufp: %lx, bufLen: %d\n", 
						(address_t)bd->buf, pktLen);
				bmc12500Eth_printBuf( bd->buf, pktLen, 1);  
			}
#ifndef NFP_FP
#ifdef LUC_DEBUG 
		if (isCheetah() && !err_flag) {
			luc_hdr = bd->buf;
			idx = (luc_hdr[4] >> 2) & 0x03;
			luc_rxPkts[idx] += 1;
			luc_rxBytes[idx] += pktLen;
		}
#endif
#endif
			//-- Replenish buffer
			BC1250_DESC_SET_A_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
											rxc->bufOffset, rxc->descBufSize);

			//-- FSDEBUG: Clear pkt size
			tmp = rxCur->dw2;
			tmp &= ~BC1250_DESC_PKT_SIZE;
			rxCur->dw2 = tmp;

			//-- Keep track of number of rx descriptors processed.
			procCnt++;

			if( ++rxCur >= rxc->rxqEnd )
				rxCur = rxc->rxqStart;
		}
		else //-- Each rx desc can hold 2 buf ptrs.
		if( bufCnt == 2 ) {
			int32 buf2Len;
			char *newBuf2 = NULL;

			//-- Alloc first new receive buffer.
			newBuf = eee_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
			if( !newBuf ) {
				rxc->noBufErr++;
				eee_deallocateDesc(edesc);
				goto bmc12500_rx_finish;      // Wait for more bufs.
			}

            eee_attrAssignBuf(newBuf, EEE_ATTR_MAC_RX);

			//-- Calculate number of data bytes in second buffer.
			buf2Len = pktLen - (rxc->descBufSize - rxc->bufOffset);

			if( bmc12500Eth_rx_pkt_trace) {
				printf("buf2Len: %d\n", buf2Len);
			}

			//-- Alloc second new receive buffer.
			if(buf2Len > 0 ) {
				newBuf2 = eee_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
				if( !newBuf2 ) {
					rxc->noBufErr++;
					eee_deallocateDesc(edesc);
					eee_deallocateBuffer(newBuf); // Release first buffer.
					goto bmc12500_rx_finish;      // Wait for more bufs.
				}
                eee_attrAssignBuf(newBuf2, EEE_ATTR_MAC_RX);
			}
			else {
				bufCnt=1;
			}

			//-- Get first buffer pointer and len.
			BC1250_DESC_GET_A_ADDR(rxCur, (address_t)bd->buf);
			bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
			bd->len     = (rxc->descBufSize - rxc->bufOffset);


			if( bmc12500Eth_rx_pkt_trace) {
				printf("BMC Rx: bufp: %lx, bufLen: %d\n", 
						(address_t)bd->buf, bd->len);
				bmc12500Eth_printBuf( bd->buf, bd->len, 1);   
			}
#ifndef NFP_FP
#ifdef LUC_DEBUG 
			if (isCheetah() && !err_flag) {
				luc_hdr = bd->buf;
				idx = (luc_hdr[4] >> 2) & 0x03;
				luc_rxPkts[idx] += 1;
				luc_rxBytes[idx] += pktLen;
			}
#endif
#endif
			//-- Replenish first buffer
			BC1250_DESC_SET_A_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
											rxc->bufOffset, rxc->descBufSize);
			if( buf2Len > 0 ) {
				//-- Get second buffer pointer and len.
				bd++;
				BC1250_DESC_GET_B_ADDR(rxCur, (address_t)bd->buf);
				bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
				bd->len     = buf2Len;
							  
				if( bmc12500Eth_rx_pkt_trace) {
					printf("BMC Rx: bufp: %lx, bufLen: %d\n", 
							(address_t)bd->buf, bd->len);
					bmc12500Eth_printBuf( bd->buf, bd->len, 0);   
				}
				//-- Replenish second buffer
				BC1250_DESC_SET_B_ADDR_AND_SIZE(rxCur, (address_t)newBuf2, 
												rxc->descBufSize);
			}
			//-- Keep track of number of rx descriptors processed.
			procCnt++;

			if( ++rxCur >= rxc->rxqEnd )
				rxCur = rxc->rxqStart;
		}
		else { //-- Packet is contained in multiple descriptors.
			int32 remain = pktLen;
			uint32 i     = bufCnt;
			uint32 buf_a = 1;
			bc1250_rxdesc_t *rxq_save = (bc1250_rxdesc_t *)rxCur;

			while( i-- ) {
				
				//-- Allocate a new receive buffer and add to rx queue.
				newBuf = eee_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
				if( !newBuf ) {
					rxc->noBufErr++;

					//-- Release bufs and edesc.
					edesc->hdr.control = (pktLen & EDESC_TOTAL_LEN) |
			          (((bufCnt-(i-1))<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
					eee_freePacket( edesc );	

					//-- Put rxq pointer back to where it was.
					rxCur = rxq_save;
					goto bmc12500_rx_finish;	// Wait for more bufs.
				}

                eee_attrAssignBuf(newBuf, EEE_ATTR_MAC_RX);

				if( bmc12500Eth_rx_pkt_trace) {
					printf("BMC RxCur: %lx  ", (address_t)rxCur);
				}
				//-- Get and replenish data buffer.
				if( buf_a ) {
					BC1250_DESC_GET_A_ADDR(rxCur, (address_t)bd->buf);

					//-- Replenish buffer A.
					BC1250_DESC_SET_A_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
											rxc->bufOffset, rxc->descBufSize);
					buf_a = 0;
				}
				else {
					BC1250_DESC_GET_B_ADDR(rxCur, (address_t)bd->buf);

					//-- Replenish buffer B.
					BC1250_DESC_SET_B_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
													rxc->descBufSize);

					//-- Keep track of number of rx descriptors processed.
					procCnt++;

					//-- Get next descriptor.
					if( ++rxCur >= rxc->rxqEnd )
						rxCur = rxc->rxqStart;
					buf_a = 1;
				}
				//-- Process the last buffer.
				if( i == 0 ) {
					if( remain > 0 ) {
					    bd->len = remain; 
					    bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
					}
					else { // Last buffer only contains CRC bytes, so free it.
						eee_deallocateBuffer(bd->buf);
						bd->buf = 0;
						bufCnt--;
					}

					//-- Make sure procCnt gets updated if only first
					//-- buffer is used.
					if( buf_a == 0 )  {
						procCnt++;
						if( ++rxCur >= rxc->rxqEnd )
							rxCur = rxc->rxqStart;
					}
				}
				else
			    if( i == bufCnt-1 ) // First buffer
				{
					bd->len = rxc->descBufSize - rxc->bufOffset;
					bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
					remain -= bd->len;
#ifndef NFP_FP
#ifdef LUC_DEBUG 
					if (isCheetah() && !err_flag) {
						luc_hdr = bd->buf;
						idx = (luc_hdr[4] >> 2) & 0x03;
						luc_rxPkts[idx] += 1;
						luc_rxBytes[idx] += pktLen;
					}
#endif
#endif
				}
				else {
					remain -= rxc->descBufSize;
					bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
					if( remain > 0 )
						bd->len = rxc->descBufSize;
					else
					    bd->len = (remain + rxc->descBufSize);
				}
				if( bmc12500Eth_rx_pkt_trace) {
					printf("Rx:%d bufp: %lx, bufLen: %d\n", i,
							(address_t)bd->buf, bd->len);
					if( i== bufCnt-1 )
						bmc12500Eth_printBuf( bd->buf, bd->len, 1);   
					else
						bmc12500Eth_printBuf( bd->buf, bd->len, 0);   
				}
				//-- Next eee buf desc.
				bd++;
			} 
		}
		//-- Setup dest and src port information.
#if defined(PROM)
		if( isTXRX() ) {
			if(cfg->link.loopback == lb_none) {
				//-- packet to the LUC application.
				edesc->hdr.src_port  = cfg->myPortAddr;		// BMC12500 App Id.
				edesc->hdr.dest_port = cfg->lucRcvAddr; 	// LUC application 
			}
			else {
				edesc->hdr.src_port  = cfg->myPortAddr;		// BMC12500 App Id.
				edesc->hdr.dest_port = cfg->myPortAddr; 	// BMC12500 App Id.
			}
		}
		else {
			/* Used as a backplane driver */
		if (isCheetah()) {
#ifdef LMUX_DEBUG
			if (!err_flag) {
				lmux_hdr = edesc->bd[1].buf;
				if (cfg->macId == 0) {
#ifdef NFX_SMP
					rxc->lmux_rxPkts[lmux_hdr[7] & 0x03]++;
					rxc->lmux_rxBytes[lmux_hdr[7] & 0x03] += pktLen;
#else
					lmux_rxPkts[lmux_hdr[7] & 0x03]++;
					lmux_rxBytes[lmux_hdr[7] & 0x03] += pktLen;
#endif
				}
				else {
#ifdef NFX_SMP
					rxc->lmux_rxPkts[(lmux_hdr[7] & 0x03)+3]++;
					rxc->lmux_rxBytes[(lmux_hdr[7] & 0x03)+3] += pktLen;
#else
					lmux_rxPkts[(lmux_hdr[7] & 0x03)+3]++;
					lmux_rxBytes[(lmux_hdr[7] & 0x03)+3] += pktLen;
#endif
				}
			}
#endif
#ifndef SIBYTE_BC			
			if (lmux_get_rev == 1) { // get lmux revision from header
				lmux_get_rev = 0;
				lmux_rev[0] = lmux_hdr[4];					
				lmux_rev[1] = lmux_hdr[5];					
				lmux_rev[2] = lmux_hdr[6];					
			}
#endif			
		if(!cfg->link.loopback)	{
		eeeHdr = (agile_hdr_t *)(edesc->bd[1].buf + sizeof(enet_hdr_t) + 8);
    		hdrlen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + eeeHdr->pad + 8;
    		edesc->hdr.dest_port = eeeHdr->dest_port;
		edesc->hdr.src_port = eeeHdr->src_port;
#if defined(NETEEE_FRAGMENT)
    		edesc->hdr.offset = eeeHdr->offset;
#endif     
    		edesc->bd[1].buf += hdrlen;
		    edesc->bd[1].len -= hdrlen;
			pktLen -= hdrlen;
		}	
		}
		else {
			pktLen -= eee_removeHeader(edesc);
		}
		}
#endif
		//printf("sp %08x dp %08x\n",edesc->hdr.src_port, edesc->hdr.dest_port);
		//-- Finish setting up the packet.
		edesc->hdr.control = (pktLen & EDESC_TOTAL_LEN) |
			                 ((bufCnt<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

		//-- Drop pkt if rx error detected.
		if( err_flag ) {
#if defined(PROM_SIBYTE)
            // XXX for FP in prom we can get huged (>25KB) garbage pkts on
            //     SSC/chassis powerup. So Ignore any bogus packets received,
            //     this is a temp workaround until hw power-up un-reset sequence
            //     is properly done in HW
            if(isTXRX())
                eee_freePacket(edesc);
            else
                printf("bmc12500Eth_rxPkt_net: bad pkt, ignore desc %p\n", edesc);
#else
            eee_freePacket(edesc);
#endif
			goto bmc12500_rx_next_pkt;
		}

		//-- Send packet up to the E-Layer for further processing.
			if( isTXRX() ) {
				if( cfg->link.loopback == lb_none ) 
					luc_rcvPkt(edesc);
				else {
					if (luc_loopback_test)
						eee_freePacket(edesc);
					else
						eee_receivePacket(edesc);
				}
			}
			else { // NFP_FP
#ifndef SIBYTE_BC					
                if (lmux_loopback_test)
                    eee_freePacket(edesc);
                else
#endif						
#ifdef SIBYTE_BC					
		if((isFP()) && (cfg->link.loopback) && (verify_test_packet(edesc))) 
		{
                    eee_spin_packet(edesc);	
                    goto bmc12500_rx_finish;
		}		
		else
#endif			
                    eee_receivePacket(edesc);
		}

bmc12500_rx_next_pkt:
#ifdef SIBYTE_BC					
		if((isFP()) && (cfg->link.loopback) && (verify_test_packet(edesc))) 
			eee_spin_packet(edesc);	
#endif		
#if 0
#if defined(NFX_SMP) && defined(NFP_FP)
		if (!NFX_SPIN_TRYLOCK(&rxc->lock))
			return(procCnt);
//		rxc->lock_cnt[cmGetMyCPUId() - 2]++;
		rxCur = rxc->rxCur;	// get latest queue ptr
#endif
#endif
	; // avoids compiler warning
	}

bmc12500_rx_finish:
	rxc->rxCur = (bc1250_rxdesc_t *)rxCur;
	return(procCnt);

} /* bmc12500Eth_rxPkt_net */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_rxPktPoll
 *
 * Description: 	Called by the eee layer and in turn calls dp83820_rxPacket
 *					for each receive priority queue to collect packet buffers
 *					and pass them up to the eee layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 18, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int32 bmc_rxHWM CPU_PRIVATE;
#ifdef NFP_FP
int32 bmc_rxMax=256;
#else
int32 bmc_rxMax=8;
#endif

void
bmc12500Eth_rxPktPoll( void *cb, uint32 tlimit )
{
	short16 i;
	uint64 mac_descCnt;
	uint32 dirty_descCnt;
	uint32 descProcCnt;
	bc1250EthCfg_t *cfg;
	
	volatile uint64 *cntReg;
	bc1250Eth_rxCtrl_t *rxc;

#if defined(NFX_SMP) 
#if defined(NFP_FP)
	// check for disabled mode 
	if (!bc1250_FP2CPU_active) {
		if (cmGetMyCPUId() == NFX_NFP_FP2)
			return;
	}
	if (!bc1250_FP1CPU_active) {
		if (cmGetMyCPUId() == NFX_NFP_FP1)
			return;
	}
#else 
	if (!CM_IS_TXRX_NCPU)
		return;
#endif /* NFP_FP */	
#endif /* NFX_SMP */

	cfg = (bc1250EthCfg_t *)cb;
	
	for(i=0; i < BC1250_MAC_RX_Q_CNT; i++) {
		
		cntReg = cfg->regs.dma_dscr_cnt_rx[i];

		// Get the driver rx control block.
		rxc = &(cfg->rxc[i]);

		mac_descCnt = *cntReg;

#if defined(NFX_SMP) && defined(NFP_FP)
		if (!NFX_SPIN_TRYLOCK(&rxc->lock))
			continue;
//		rxc->lock_cnt[cmGetMyCPUId() - 2]++;
#endif
		// Get the MAC rx desc count.
		if (mac_descCnt  <= 4) // check for low descriptor queue
			rxc->lowDesc++;

		// The difference should give the count of descriptors
		// ready to be processed. Note: mac_descCnt should never be
		// greater than BMC12500_MAC_RX_Q_LEN.
		dirty_descCnt = BC1250_MAC_RX_Q_LEN - (uint32)mac_descCnt;
		if( !dirty_descCnt ) {
#if defined(NFX_SMP) && defined(NFP_FP)
			NFX_SPIN_UNLOCK(&rxc->lock);
//			rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
			continue;
		}

		if( dirty_descCnt > bmc_rxMax )
			dirty_descCnt = bmc_rxMax;
			
		// Call the receive function to collect the completed packets.
		// The function will return the number of descriptors actually
		// processed. 
		descProcCnt = bmc12500Eth_rxPkt_net(rxc, cfg, dirty_descCnt);

#if defined(NFX_SMP) && defined(NFP_FP)
		NFX_SPIN_UNLOCK(&rxc->lock); // preserve sequentiality
//		rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
		if( descProcCnt > bmc_rxHWM )
			bmc_rxHWM = descProcCnt;

		// Give the MAC back the number of descriptors processed by 
		// the receive function.
		*cntReg = descProcCnt;
	}
	return;
} /* bmc12500Eth_rxPktPoll */

int
bmc12500Eth_printBuf( char *buf, int len, int fDecode )
{
    int i, j;
    char ch;
	char *data;

	data = buf;
	if( !data || !len )
		return 0;

	if( fDecode ) {
		bmc12500Eth_decodeHdr(data);
	}

#if 0
	if(len > 64)
		len = 64;
#endif
	
    for(i=0; i < len; i+=16) {
        printf("\n%04x:\t", i);
        for( j=i; j < (i+16); j++ )
            printf("%02x ", data[j] & 0xff);
        printf("    ");
        for( j=i; j < (i+16); j++ ) {
            ch = data[j] & 0xff;
            if( (ch > 0x20) && (ch < 0x7F))
                printf("%c", ch);
            else
            if( ch == 0 )
                printf("%c",'.');
            else
                printf("%c",'_');
        }
    }
    printf("\n\n");
	return 0;
} // bmc12500Eth_printBuf

void
bmc12500Eth_decodeHdr( char *data )
{
	uint32 tmp;

	printf("\n");
	printf("MAC DA  : %02x:%02x:%02x:%02x:%02x:%02x\n", 
		   data[0]&0xff, data[1]&0xff, data[2]&0xff, 
		   data[3]&0xff, data[4]&0xff, data[5]&0xff);
	printf("MAC SA  : %02x:%02x:%02x:%02x:%02x:%02x\n", 
		   data[6]&0xff, data[7]&0xff, data[8]&0xff, 
		   data[9]&0xff, data[10]&0xff, data[11]&0xff);

	
#ifndef PROM
	if(txrxdebug)
		tmp = get_word(data+12) & 0xFFFF;
	else
#endif
		tmp = get_word(data+12) & 0xFFFF;
	printf("TYPE    : %04x\n", tmp);

	if((tmp == EEE_ETH_TYPE) || (tmp==EEE_ETH_TYPE_SAC)) {		// Only if Agile Protocol type.
#ifndef PROM
	if(txrxdebug)
		tmp = get_word(data+16);
	else
#endif
		tmp = get_word(data+16);
		printf("EEE DEST: %08x, App: %d:%s, CPU: %d, Slot: %d, Dtype: %d\n", 
			   tmp,
			   EEE_GET_APP_ID(tmp),
			   eee_getAppNameFromId(EEE_GET_APP_ID(tmp)),
			   EEE_GET_DEST_CPU(tmp),
			   EEE_GET_SLOT_NUM(tmp),
			   EEE_GET_DTYPE(tmp));
#ifndef PROM
	if(txrxdebug)
		tmp = get_word(data+20);
	else
#endif
		tmp = get_word(data+20);
		printf("EEE SRC : %08x, App: %d:%s, CPU: %d, Slot: %d, Dtype: %d\n", 
			   tmp,
			   EEE_GET_APP_ID(tmp),
			   eee_getAppNameFromId(EEE_GET_APP_ID(tmp)),
			   EEE_GET_DEST_CPU(tmp),
			   EEE_GET_SLOT_NUM(tmp),
			   EEE_GET_DTYPE(tmp));

#ifdef NFP_FP
		bmc12500Eth_decodeScsi(data);
#endif
	}
	printf("\n............HEX DUMP.................\n");
} // bmc12500Eth_decodeHdr

int
bmc12500Eth_decodeScsi(char *data) 
{
	uint32 tmp;
	uint32 *msg;

	msg = (uint32 *)(data+32);

	tmp = *msg;
	if( tmp == 0x0401 || tmp == 0x0403 ) {
		printf("\n............SCSI HDR.................\n\n");
		if( tmp == 0x0401 )
			printf("SCSI READ RSP : ");
		else
			printf("SCSI WRITE RSP: ");

		printf("CHANDLE: %x, LBA: %d, LBCNT: %d\n",  
			   *(msg+1), *(msg+5), *(msg+6));
	} 
	return 0;
} // bmc12500Eth_decodeScsi

int
bmc1250_rxPktCapture(  char *buf, uint32 len, int32 macId )
{
#ifdef BC1250_RX_PKT_CAPTURE  
	uint32 capLen;

	if( len > BC1250_BYTES_CAPTURE_MAX )
		capLen = BC1250_BYTES_CAPTURE_MAX;
	else
		capLen = len;

	memcpy(bc1250_pcb[macId].pcbe_rx[bc1250_pcb[macId].curEntry_rx].bytes, buf, capLen); 

	bc1250_pcb[macId].pcbe_rx[bc1250_pcb[macId].curEntry_rx].pktLen = len;
	bc1250_pcb[macId].pcbe_rx[bc1250_pcb[macId].curEntry_rx].capLen = capLen;
	bc1250_pcb[macId].pcbe_rx[bc1250_pcb[macId].curEntry_rx].bufPtr = buf;
	eee_rtc(&(bc1250_pcb[macId].pcbe_rx[bc1250_pcb[macId].curEntry_rx].t));

	if( ++bc1250_pcb[macId].curEntry_rx >= BC1250_CAPTURE_ENTRIES_MAX )
		bc1250_pcb[macId].curEntry_rx = 0;
#endif

	return 0;
} // bmc1250_rxPktCapture

int
verify_test_packet(eee_desc_t 	*edesc)
{		
    volatile eee_descBuf_t	*bd;
    uint32 j=0;
	uchar8 *pktPtr;
	uint32 pktLen=0, remain=0;
	uint32 pktCompLen = 0;
	uint32 i = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);
	uint32 bufCnt = i;

    bd = &(edesc->bd[1]);
	while (i--) {
		pktLen += bd->len;
		bd++;
	}	
	i = 0;
    bd = &(edesc->bd[1]);
	pktLen -= 32; //replace with the actual hdrLen 
	remain = pktLen;

	while (i++ < bufCnt) {
	    pktPtr = bd->buf;
        /* Skip 32 bytes of header */
		if( i== 1 ) {
	        pktPtr += 32;
			pktCompLen = bd->len - 32;	
	    }
		else
			pktCompLen = bd->len;	

        for(j=0;j<pktCompLen;j++) {
            if(*pktPtr == ((remain-j) & 0xff))
                pktPtr++;	
            else {
                printf("Verify_test_packet ERR. buf %d; pktLen %d; at %x expected %x got %x\n",i,pktLen,j,(remain-j)&0xff, *pktPtr);	
 	            bmc12500Eth_printBuf( bd->buf, bd->len, 0);   
                return 0;
            }	
		}	
		remain = remain - pktCompLen;
		bd++;
    }	
    bobcat_test_pkt = 1;
    return 1;
}		

void
eee_spin_packet(eee_desc_t 	*edesc)
{		
    volatile eee_descBuf_t	*bd;
	uint32 pktLen=0;
	int rc=0;
	uint32 src_port;
	uint32 dest_port;
	volatile agile_hdr_t    *eeeHdr;
	uint32 i = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);
	uint32 tmp=0;

    bd = &(edesc->bd[1]);
	while (i--) {
		pktLen += bd->len;
		bd++;
	}	
    bd = &(edesc->bd[1]);

    eeeHdr = (agile_hdr_t *)(bd + sizeof(enet_hdr_t) + 2); // 6+6+2 bytes total. need 2 bytes padding

	tmp = edesc->hdr.dest_port;
    edesc->hdr.dest_port = edesc->hdr.src_port;
    edesc->hdr.src_port = tmp;

    tmp = eeeHdr->dest_port;
    eeeHdr->dest_port = edesc->hdr.src_port;
    eeeHdr->src_port = tmp;

/*	
	edesc->hdr.dest_port = eeeHdr->src_port;
	edesc->hdr.src_port = eeeHdr->dest_port;
	eeeHdr->src_port = edesc->hdr.src_port;
	eeeHdr->dest_port = edesc->hdr.dest_port;
*/
//bmc12500Eth_printBuf( bd->buf, bd->len, 0);

	edesc->hdr.control &= ~EDESC_TOTAL_LEN;
	edesc->hdr.control |= (pktLen & EDESC_TOTAL_LEN);
	rc = bmc12500Eth_txUnicast( edesc->hdr.dest_port,edesc );
	if(rc != EEE_FORWARDED_PACKET)
		   printf("Could not spin packet around. rc %x\n",rc);	
				
	return;	
}		
