/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: bsd-vif.h
 *
 * RCS:  $Id: bsd-vif.h
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: virtual interface functions on txrx.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/10/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _BSD_VIF_H
#define _BSD_VIF_H

#if !defined(PROM)
#include "../sm-ipmd-lib/ipmd-lib.h"
#include "exports/bsd-api.h"

/* mirrors ipm_vif_t, except linkNames are replaced
 * by ptr to the link, and all vifs are in chain
 * rather than array.
 */ 


typedef struct bsd_vif 
{
	char			bsd_vifName[IPM_STR_SIZE];
	char			bsd_vifMac[8];
	struct ifnet*	bsd_vifLinks[IPM_MAXLINK_VIF];
	
	/* fail-over only, user specified link */
	struct ifnet*	bsd_vifPrefLink; 
	/* where IP and routes are assigned to,
	 * fail-over  -- actLink changes depend on link state,
	 * link-aggre -- actLink stays constant
	 */
	struct ifnet*	bsd_vifActLink; 
	/* link-aggre only, outgoing link for slowpath traffic
	 * note that upLink and actLink can be different.
	 */
	struct ifnet*	bsd_vifUpLink;
	
	uint32			bsd_vifLinkCnt;
	uint32			bsd_vifFlags;
	uint32			bsd_vifSlot;			
	uint32			bsd_vifIpCnt;
	struct bsd_vif	*bsd_vifNxt;
} bsd_vif_t;

/* debugging stats */
typedef struct bsd_vifStats
{
	uint32			vif_actLinkChange;
	uint32			vif_upLinkChange;
	uint32			vif_alloc;
	uint32			vif_dealloc;
	uint32			vif_allocErr;
	uint32			vif_addErr;
	uint32			vif_delErr;
	uint32			vif_prefErr;
	uint32			vif_addLinkErr;
	uint32			vif_delLinkErr;
	uint32			vif_addIpErr;
	uint32			vif_delIpErr;
	uint32			vif_cnt;
} bsd_vifStats_t;

/* define an API ctrl block, this should contain 
 * global varibles for iprelay, arpsniff, and vif,
 * currently only vif is here.
 */
typedef	struct bsd_apiCb
{
	bsd_vif_t*			bsd_vifHead;
	uint32				bsd_vifCnt;
	bsd_vifStats_t*		bsd_vifStats;
} bsd_apiCb_t;

extern bsd_apiCb_t*	bsd_apiCbPtr;

#define VifHead		bsd_apiCbPtr->bsd_vifHead
#define VifStats	bsd_apiCbPtr->bsd_vifStats


#define IPM_VIF_MSGS(_mtype)		((_mtype == IPM_MSG_VIF_CREAT)	||	\
									(_mtype == IPM_MSG_VIF_DEL)		||	\
									(_mtype == IPM_MSG_VIF_ADDLNK)	||	\
									(_mtype == IPM_MSG_VIF_RMLNK)	||	\
									(_mtype == IPM_MSG_VIP_ADD)		||	\
									(_mtype == IPM_MSG_VIP_DEL)		||	\
									(_mtype == IPM_MSG_VIF_PREF))


/* function declarations */
bsd_vif_t*	bsd_FindVifByName(char	*vifName);
bsd_vif_t*	bsd_FindVifByLink(void	*in_ifp);
void*		bsd_VifLAGetActLink(void	*in_ifp);
void*		bsd_VifLAGetUpLink(void	*in_ifp);
int32		bsd_IsLinkVifLA (void	*in_ifp);
void*		bsd_VifHasIP(uint32 ip, bsd_vif_t	*vifp);
void		bsd_SwitchLinkIP(void *in_srcLink,  void *in_dstLink);
int32		bsd_AddLinkToVif(void* in_ifp, bsd_vif_t *vifp);
int32		bsd_RmLinkFromVif(void	*in_ifp, bsd_vif_t *vifp);
int32		bsd_AddVif(char	*vifName, uint32 flags, void *linkList, int count);
int32		bsd_DelVif(char *vifName);
int32		bsd_PrefLinkVif(char *vifName, void *in_ifp);
int32		bsd_AddVifLink(char	*vifName, void*	 linkStart, int count);
int32		bsd_RmVifLink(char *vifName, void*	 linkStart, int count);
int32		bsd_AddVifIP(char *vifName, uint32 ip, uint32 mask);
int32		bsd_DelVifIP(char *vifName, uint32 ip);
void		bsd_VifLinkChange(void*	in_ifp);
void		bsd_PrintVif(void*	in_vif);
#endif /* PROM */
#endif /* _BSD_VIF_H */
