/* vi:ts=4
*-----------------------------------------------------------------
*
* Name:	bsdnet-api.h
*
* RCS:  $Id: bsdnet-api.h,v 1.15 2002/10/29 01:23:58 swang Exp $
*
* Copyright (C) 2001, Agile Storage, Inc.
*
* Description:	API functions to bsd TCP/IP
*
* Created by:	Su Wang
*
* Date Created:	7/25/01
*
*-----------------------------------------------------------------
*/

#ifndef _BSDNET_API_H
#define _BSDNET_API_H


#include "exports/bsd-api.h"
#include "exports/pkt.h"

#define MASK_HAS_HOLE(mask)		((mask | (~mask +1)) != mask) 
#define BSD_GET_ADDR(gw)		((gw->sa_family == AF_INET) ?			\
								(char*)&((struct sockaddr_in*)gw)->sin_addr : \
								LLADDR((struct sockaddr_dl*)gw))		
#define BSD_GET_ADDR_LEN(gw)	((gw->sa_family == AF_INET) ? 4 :		\
								((struct sockaddr_dl*)gw)->sdl_alen)	


/* fast path return value by tcp_input and tcp_output */
#define BSD_FAST_ERR		-1	/* bad pkt, free pkt */
#define BSD_FAST_FWD		-2	/* forward pkt to appl/or luc */
#define BSD_FAST_QUEUED		-3	/* pkt in reass/send queue, hold pkt */
#define BSD_FAST_ACK		-4	/* pure ack pkt(TCP rcv only), free pkt */
#define BSD_FAST_FULL		-5	/* queue is full, appl should know this */
#define BSD_FAST_MORE		-6	/* more in TCP send queue and are ready to send */

int32
bsd_SetMtuIntf(void* device, int	  mtu);

/*-----------------------------------------------------------------
 * Name: bsd_AttachIf
 *
 * Description: adding a device interface
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsd_AttachIf(void *ipm_mp);

/*
*-----------------------------------------------------------------
*
* Name : bsd_CreateIpIntf
*
* Description: create an IP interface 
*
* Created by: Su Wang
*
* Data Created: 7/25/01
*
*-----------------------------------------------------------------
*/
int32							/* Nfx_Err Fail, Nfx_Ok ok */
bsd_CreateIpIntf(
				 uint32 ipaddr,	/* in network order */ 
				 uint32 ipmask,	/* in network order */
				 void* device,	/* media port id */
				 void* param1,
				 void* param2
				 );


int32  
bsd_AddIpByIntf(uint32 ipAddr, 
				uint32	mask,
				void*   in_ifp);

/*
 *-----------------------------------------------------------------
 *
 * Name : bsd_ChangeStateIntf
 *
 * Description: change the administrative state of the device.
 *              all ip interfaces associated with this device
 *              will be affected.    
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *-----------------------------------------------------------------
 */
int32  /* NFX_ERR fail, NFX_OK ok */
bsd_ChangeStateIntf(
					void* device,/* in network order */ 
					int32 state, /* new state */
					void* param1,
					void* param2
					);

/*
 *-----------------------------------------------------------------
 *
 * Name : bsd_ChangeLinkStatus
 *
 * Description: change the operational state of the device. Called 
 *				driver.
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *-----------------------------------------------------------------
 */
int32  /* NFX_ERR fail, NFX_OK ok */
bsd_ChangeLinkStatus(int link,	/* 0, 1, .. */
					int state	/* 1 up, 2 down */
					);


/*
 *-----------------------------------------------------------------
 *
 * Name : bsd_DeleteIpIntf
 *
 * Description: delete an IP interface 
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_ERR fail, NFX_OK ok */
bsd_DeleteIpIntf(uint32 ipaddr,	/* in network order */
				 void* device,	/* device to which IP interface ass. with */
				 void* param1,
				 void* param2
				 );

/*
 *-----------------------------------------------------------------
 *
 * Name : bsd_DeleteIpAll
 *
 * Description: delete all ip address 
 *
 * Created by: Su Wang
 *
 * Data Created: 08/19/02
 *
 *-----------------------------------------------------------------
 */
void
bsd_DeleteIpAll(void	*in_ifp);

/*----------------------------------------------------------------
 * 
 * Name : bsd_GetNextIntf
 *
 * Description: get next intf
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *---------------------------------------------------------------
 */
void*
bsd_GetNextIntf(void *ifp);

/*
 *----------------------------------------------------------------
 * 
 * Name : bsd_ShowIntf
 *
 * Description: display a summary of all IP interfaces or one IP 
 *              interface in detail.
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *---------------------------------------------------------------
 */
void
bsd_ShowIntf(
			 int32 how,			/* summary or a single interface */
			 void* device,		/* device need to to displayed */
			 void* param1,
			 void* param2
			 );


/* route type flags */
#define  IPRT_LOCAL    0x00000001
#define  IPRT_STATIC   0x00000002
#define  IPRT_RIP      0x00000004
#define  IPRT_OSPF     0x00000008
#define  IPRT_HOST     0x00000010
#define  IPRT_ALL	   0xffffffff



#define BSD_RTMISC_NO_STATIC	1	/* dont add to static rt list */
/*
 *------------------------------------------------------------------
 *
 * Name : bsd_ModRoute
 *
 * Description: Add/Delete a route to bsd route domain 
 *
 * Created by : Su Wang
 *
 * Date Created: 7/27/01
 *-----------------------------------------------------------------
 */	     
int32							/* NFX_OK, NFX_ERR */
bsd_ModRoute(
			 uint32 ipaddr,		/* in network order */
			 uint32 ipmask,   
			 uint32 gateway,
			 uint32 type,		/* see route type definition */
			 int    action,
			 void*  misc,		/* misc. actions need to be performed */
			 void*  param2
			 );

/*
 *------------------------------------------------------------------
 *
 * Name : bsd_ShowRoute
 *
 * Description: Display route table 
 *
 * Created by : Su Wang
 *
 * Date Created: 7/27/01
 *-----------------------------------------------------------------
 */	  
void
bsd_ShowRoute(
			  int	 how,		/* single route or multiple routes */
			  uint32 type,		/* show what route type */ 
			  uint32 ipaddr,	/* given if want to display a single route */
			  uint32 ipmask,	/* given if want to display a single route */
			  void*  misc,
			  void*  param2
			  );


/*
 *------------------------------------------------------------------
 *
 * Name : bsd_ModARP
 *
 * Description: Add/Delete an ARP entry to bsd route domain 
 *
 * Created by : Su Wang
 *
 * Date Created: 7/27/01
 *-----------------------------------------------------------------
 */	     
int32							/* NFX_OK, NFX_ERR */
bsd_ModARP (uint32 ipaddr,		/* in network order */
			char*  mac,			/* mac address in network order */
			int    action,
			void*  param1,
			void*  param2
			);

/*
 *------------------------------------------------------------------
 *
 * Name : bsd_ShowARP
 *
 * Description: Show ARP table. 
 *
 * Created by : Su Wang
 *
 * Date Created: 7/27/01
 *-----------------------------------------------------------------
 */	  
void
bsd_ShowARP (  uint32 ipaddr,	/* network range  */
			   uint32 ipmask,	/* network range */
			   void*  param1,
			   void*  param2
			   );



/*
 *-----------------------------------------------------------------------
 *
 * Name:   bsd_XmtPkt
 *
 * Description: convert IP/ARP in mbuf to pkt descriptor. It also initiate an
 *              arp request if dl addr not avaiable. Any IP packet without
 *              valid dl addr is droped by this function. 
 *              It is called by ip_output from bsd TCP/IP stack, and in 
 *              turn calls pkt_xmtSlowPath to add layer2 header. Pkt descriptor
 *              has only hdrsize setup.
 *              Note: make sure to BSD compatible error code.
 * 
 * Created by : Su Wang
 *
 * Date Created:  8/6/01
 *------------------------------------------------------------------------
 */
int32							/* bsd compatible err code, 0 success */
bsd_XmtPkt(
		   struct ifnet	*ifp,	/* ptr to interface */
		   struct mbuf	*m,		/* ip packet */
		   struct sockaddr* dst,/* nexthop address, is alway an IP */
		   struct rtentry* rt0	/* route entry */

		   ); 


/*
 *-----------------------------------------------------------------------
 *
 * Name:   bsd_IpRcvPkt
 *
 * Description: Called by pkt_pollRcvQueue to convert IP in pkt-descr to mbuf. 
 *              It send packet to BSD TCP/IP stack by calling ip_input(). 
 *              packets are assumed to have layer2 header processed already.
 *
 *
 * Created by : Su Wang
 *
 * Date Created:  8/6/01
 *------------------------------------------------------------------------
 */
int32							/* PKT_FREE_PKT, PKT_HOLD_PKT */
bsd_RcvIpPkt(void*	pktDesc,
			 struct ifnet *ifp,	/* rcv interface */
			 void*	param1,
			 void*  param2);

 
/*
 *-----------------------------------------------------------------------
 *
 * Name:   bsd_RcvArpPkt
 *
 * Description: Called by pkt_pollRcvQueue to convert ARP in pkt-descr to mbuf. 
 *              It send packet to BSD TCP/IP stack by calling arp_input(). 
 *              packets are assumed to have layer2 header processed already.
 *
 *
 * Created by : Su Wang
 *
 * Date Created:  8/6/01
 *------------------------------------------------------------------------
 */
int32							/* PKT_FREE_PKT, PKT_HOLD_PKT */
bsd_RcvArpPkt(
			  void* pktDesc,
			  struct ifnet* ifp, /* rcv interface */
			  void* param1,
			  void* param2
			  );


/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_UpdtPcbRt
 *
 * Descriptions: called when route info. in each PCB need to updated. 
 *               This happens when ARP changes, route changes, or
 *				 interface state changes.
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/22/01
 *------------------------------------------------------------------------------
 */
void
bsd_UpdtPcbRt(
#define			BSD_UPDT_ARP	1
#define			BSD_UPDT_RT		2
#define			BSD_UPDT_INTF	3			  
			  int		type,	/* types of info changed */
			  int		action,	/* what change? add/del/mod */
			  void		*info,	/* ptr to info, *rt or *ifaddr */
			  void*		param1,
			  void*		param2
			  );

/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_ChkPcbRt
 *
 * Descriptions: called to retrive next hop info when a socket is first
 *               connected. This function will try to establish a route 
 *				 to destination specified. 1) it check to see if 
 *				 the route is no longer up. If so, free the route, and then
 *				 follows, 2) if the route is empty, if so type to allocate
 *				 an route. If route not found, exit; if found, 3) if the 
 *				 the route is gateway route, if so, 4) if the gateway route
 *				 is up, if not, allocate a new gateway route.5) if 		 
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/15/01
 *------------------------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsd_ChkPcbRt(
			 struct inpcb* inp
			 );

/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_Ping
 *
 * Descriptions: Ping program 
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/15/01
 *------------------------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsd_Ping(
		 ipa_t da,				/* destination address */
		 int size,				/* ip datagram size */
		 int repeat				/* numb. of ping send */
		 );

/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_RestoreStaticRoute
 *
 * Descriptions: called when an interface just come up or during config.
 *				 restore process to add static route back to route table.
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/22/01
 *------------------------------------------------------------------------------
 */
void
bsd_RestoreStaticRoute(
					   void		*ifa /* ptr to intf addr, 0 restore all routes */
					   );

/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_PurgeStaticRoute
 *
 * Descriptions: remove related static route and arp entries.
 *				 called when an IP interface is to be removed. 
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/22/01
 *------------------------------------------------------------------------------
 */
void
bsd_PurgeStaticRoute(
					 void		*ifa /* ptr to intf addr, 0 purge all routes */
					 );


/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_ShowStaticRoute
 *
 * Descriptions: show current static route and static arp.
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/23/01
 *------------------------------------------------------------------------------
 */
void
bsd_ShowStaticRoute();

/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_ShowMbufStats
 *
 * Descriptions: show mbuf stats.
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/23/01
 *------------------------------------------------------------------------------
 */
void
bsd_ShowMbufStats();

void* bsd_FindIntfByName(char  *name);
void* bsd_FindIntfByIp(uint32 ipAddr);
int32 bsd_DeleteIpByIntf(uint32 ipAddr, void*   in_ifp);
void  bsd_ChangeStateIntfAll(int32);
void* bsd_FindRt(uint32 ipaddr, uint32 ipmask, uint32 gw, uint32 flags);
void* bsd_FindIpByIntf(void*	in_ifp);
byte* bsd_GetIntfMacVif(void	*in_ifp);
byte* bsd_GetIntfMac(void	*in_ifp);

extern int32 tcp_inputC(void	*ptr,
                        int		iphlen,
						void	*pconn,
						void	*incb,
						void	*ip,
						pkt_desc_t	**ret,
						void	*parm);

extern int tcp_outputC(void			*tp,
					   void			*pconn,		
					   pkt_desc_t	*pdesc,
					   pkt_desc_t	**ret,
					   void			*parm);
#endif /* _BSDNET_API_H */
