/* syscfg.c */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <time.h>
#include <pwd.h>

#include <ipmd-lib.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <nfxnis-api.h>

#include "httpdagent.h"
#include "aghelper.h"
#include "bsdhelper.h"

#define CFGFILE_TEMP	"/tmp/av_syscfg_temp"

#define TIMEZONEPATH "/usr/share/zoneinfo"
#define TIMEZONELINK "/etc/localtime"

#define DEFAULTROUTE "/etc/mygate"


char *groupNames[] =
{
	"nfxview", 
	"nfxadd", 
	"nfxchang", 
	"nfxfull", 
	"nfxdiags"
};

char *autoSupport[] =
{
	"options.autosupport.enable",
	"options.autosupport.to",
	"options.autosupport.noteto",
};

int WriteCommas(int num, char *buf);
int AppendStep1Config(char *buf);
int AppendStep2Config(char *buf);
int AppendStep3Config(char *buf);
int AppendStep4Config(char *buf);
int CreateTmpCfgFile(u_int32_t id);
int RemoveTmpCfgFile(u_int32_t id);

int getTmpSystemConfig(u_int32_t id, char* file, char* buf);
int getSystemConfig(int from, int step, u_int32_t id, haSystemConfig_t* cfg);

int setSystemConfig1(char* cfg);
int setSystemConfig2(char* cfg);
int setSystemConfig3(char* cfg);
int setSystemConfig4(char* cfg);
int SaveSystemConfig(char* file, char* cfg);
int setSystemConfig(int from, int step, u_int32_t id, char* cfg);



int WriteCommas(int num, char *buf)
{
int i;

	for (i = 0; i < num; i++)
		strcat(buf, ",");

	return 1;
}


int AppendStep1Config(char *buf)
{
char rspbuf[2048];
struct timeval tval;
struct timezone tzval;
struct tm tmval;

	/* append timezone, language, and time information */

	/* timezone */
	if (readlink(TIMEZONELINK, rspbuf, 2048) > 0)
	{
		/* remove the prefix path "/usr/share/zoneinfo/" */
		strcpy(rspbuf, &rspbuf[strlen(TIMEZONEPATH)+1]);
		strcat(buf, rspbuf);
	}
	strcat(buf, ",");

	/* language */
	strcat(buf, "English,");

	/* time */
	gettimeofday(&tval, &tzval);
	localtime_r((time_t *) &(tval.tv_sec), &tmval);
	sprintf(rspbuf, "%d,%d,%d,%d,%d",
			tmval.tm_mon, tmval.tm_mday, tmval.tm_year+1900,
			tmval.tm_hour, tmval.tm_min);
	strcat(buf, rspbuf);

	return 1;
}


int AppendStep2Config(char *buf)
{
autoSupport_t ast;

	/* append AgileSupport information */
	/* autosupport, to email, noteto email */
	if (getAutoSupportConfig(&ast))
	{
		if (ast.enabled)
			sprintf(buf, "on,%s,%s", ast.to, ast.noteto);
		else
			sprintf(buf, "off,%s,%s", ast.to, ast.noteto);
	}
	else
		WriteCommas(2, buf);

	return 1;
}



int AppendStep3Config(char *buf)
{
	nfxnis_cfg_t **list;
	nfxnis_cfg_t *entry;
	int i, j, count;

	list = nfxnis_get_cfg(0, 0);
	if (list == NULL || *list == NULL)
	{
		/* no default yet, fill in blank */
		WriteCommas(5, buf);
		return 1;
	}

	/* find default DNS entry */
	count = 0;
	for (i = 0; list[i] != NULL; i++)
	{
		entry = list[i];

		if (!entry->net && !entry->mask && (entry->type == dns_name))
		{
			strcat(buf, entry->domain);
			strcat(buf, ",");

			if (entry->num_srvrs > 3)
				count = 3;
			else
				count = entry->num_srvrs;

			for (j = 0; j < count; j++)
			{
				strcat(buf, GetIpAddressStr(entry->srvrs[j]));
				strcat(buf, ",");
			}
			for (j = count; j < 3; j++)
				strcat(buf, ",");

			/* done default DNS, exit loop... */
			break;
		}
	}
	if (count == 0)		/* no default DNS? fill in blank */
		strcat(buf, ",,,,");

	/* find default NIS entry */
	count = 0;
	for (i = 0; list[i] != NULL; i++)
	{
		entry = list[i];

		if (!entry->net && !entry->mask && (entry->type == nis_name))
		{
			strcat(buf, entry->domain);
			strcat(buf, ",");

			strcat(buf, GetIpAddressStr(entry->srvrs[0]));
			count = 1;

			/* done default NIS, exit loop... */
			break;
		}
	}
	if (count == 0)
		strcat(buf, ",");

	return 1;
}


int AppendStep4Config(char *buf)
{
u_int32_t defroute;

	defroute = GetDefaultRoute();
	/* only append if there is a default route */
	if (defroute)
		strcat(buf, GetIpAddressStr(defroute));
	return 1;
}


int CreateTmpCfgFile(u_int32_t id)
{
	char fname[128];
	char buf[1024];
	int step;
	FILE *f;

	for (step = 1; step <= 4; step++)
	{
		buf[0] = '\0';
		switch (step)
		{
		case 1:
			/* for adding new user only, so no previous value */
			/* include admin name, passwd, and privilege      */
			WriteCommas(3, buf);
			/* now append timezone, language, and time info */
			AppendStep1Config(buf);
			break;
		case 2:
			AppendStep2Config(buf);
			break;
		case 3:
			AppendStep3Config(buf);
			break;
		case 4:
			/* for adding new interface IP only, so no previous value */
			/* include mgmt if, ip, ipmask, 4 client if, ip, ipmask   */
			WriteCommas(15, buf);
			/* now append default route info */
			AppendStep4Config(buf);
			break;
		}

		sprintf(fname, "%s_%d_%08x", CFGFILE_TEMP, step, id);
		f = fopen(fname, "w+");
		if (f == NULL)
			continue;	/* can't create temp cfg file... */

		fwrite(buf, sizeof(char), strlen(buf), f);
		fclose(f);
	}

	return 1;
}


int RemoveTmpCfgFile(u_int32_t id)
{
int step;
char buf[128];

	for (step = 1; step <= 4; step++)
	{
		sprintf(buf, "%s_%d_%08x", CFGFILE_TEMP, step, id);
		unlink(buf);
	}

	return 1;
}




/******************************************************************
               GET SYSTEM CONFIGURATION ROUTINES
******************************************************************/

int getTmpSystemConfig(u_int32_t id, char* file, char* buf)
{
	FILE *f;
	int numread;
	char readbuf[1024];

	f = fopen(file, "r+t");
	if (f == NULL)
	{
		/* file doesn't exist, need to create them */
		CreateTmpCfgFile(id);

		/* open it again */
		f = fopen(file, "r+t");
		if (f == NULL)
			return 0;
	}

	numread = fread(readbuf, sizeof(char), 1024, f);
	readbuf[numread] = '\0';
	fclose(f);
	if (numread > 0)
	{
		strcpy(buf, readbuf);
		return 1;
	}
	return 0;
}

int getSystemConfig(int from, int step, u_int32_t id, haSystemConfig_t* cfg)
{
	char file[128];

	if (from == 1)	/* get from permanent, remove tmp files if any */
		RemoveTmpCfgFile(id);

	/* get system config will always get from temporary files */
	strcpy(file, CFGFILE_TEMP);
	sprintf(file, "%s_%d_%08x", CFGFILE_TEMP, step, id);
	return getTmpSystemConfig(id, file, cfg->configs);
}




/******************************************************************
               SET SYSTEM CONFIGURATION ROUTINES
******************************************************************/

int setSystemConfig1(char* cfg)
{
char buf[1024], buf1[128], buf2[128];
int prvg, start = 0;
struct tm tmval;
pid_t pid;
int status;

	memset(buf1, 0, sizeof(buf1));
	memset(buf2, 0, sizeof(buf2));

	start = GetDelimiteredParam(start, cfg, buf1);		/* hostname */
	gethostname(buf2, sizeof(buf2));
	if (buf1[0] && strcmp(buf1, buf2))
		sethostname(buf1, strlen(buf1));

	start = GetDelimiteredParam(start, cfg, buf1);		/* adm name */
	start = GetDelimiteredParam(start, cfg, buf2);		/* adm passwd */
	start = GetDelimiteredParam(start, cfg, buf);		/* adm privilege */
	prvg = atoi(buf);

	if (buf1[0] && buf2[0])
	{
		/* now fork and execute useradd BSD command */
		pid = fork();
		if (!pid)		/* this is child process */
		{
			int ret;
			struct passwd *pwd = getpwnam("root");

			ret = execlp("/usr/sbin/useradd", "/usr/sbin/useradd",
					"-d", "/usr/local/agile/root",
					"-s", "/usr/local/agile/bin/nfxsh",
					"-g", groupNames[prvg],
					"-p", crypt(buf2, pwd->pw_passwd),
					buf1, NULL);

			/* shouldn't return from above call, if it return */
			/* failed to execute command...                   */
			exit(1);
		}

		/* the original process continue here... */
		status = 0;
		if (pid > 0)
		{
			int i;
			int options = WNOHANG | WUNTRACED;
			pid_t ret_pid;
			struct timespec tout = {1, 0};

			for (i = 0; i < 10; i++)
			{
				ret_pid = waitpid(pid, &status, options);
				if (ret_pid == 0)
					nanosleep(&tout, NULL);
				else
					break;
			}
		}
		else
			return 0;
	}

	start = GetDelimiteredParam(start, cfg, buf1);		/* timezone */
	sprintf(buf, "%s/%s", TIMEZONEPATH, buf1);
	unlink(TIMEZONELINK);
	symlink(buf, TIMEZONELINK);

	start = GetDelimiteredParam(start, cfg, buf1);		/* language */
	/* do nothing for language for now... */

	start = GetDelimiteredParam(start, cfg, buf);		/* month */
	tmval.tm_mon  = atoi(buf);
	start = GetDelimiteredParam(start, cfg, buf);		/* day */
	tmval.tm_mday = atoi(buf);
	start = GetDelimiteredParam(start, cfg, buf);		/* year */
	tmval.tm_year = atoi(buf) - 1900;
	start = GetDelimiteredParam(start, cfg, buf);		/* hour */
	tmval.tm_hour = atoi(buf);
	start = GetDelimiteredParam(start, cfg, buf);		/* minute */
	tmval.tm_min  = atoi(buf);

	SetSystemTime(&tmval);

	return 1;
}

int setSystemConfig2(char* cfg)
{
int start = 0;
char buf[256];
autoSupport_t ast;

	start = GetDelimiteredParam(start, cfg, buf)	;		/* autosupport */
	if (!strcmp(buf, "on"))
		ast.enabled = 1;
	else
		ast.enabled = 0;
		
	start = GetDelimiteredParam(start, cfg, ast.to);		/* TO email */
	start = GetDelimiteredParam(start, cfg, ast.noteto);	/* NOTETO email */

	return setAutoSupportConfig(&ast);
}

int setSystemConfig3(char* cfg)
{
char domain[128], server[32];
u_int32_t serverip;
int i, start = 0;

	/* remove the default service first */
	nfxnis_delservice(0, 0);

	/* DNS domain */
	start = GetDelimiteredParam(start, cfg, domain);		/* dns domain */
	if (strlen(domain))
	{
		for (i = 0; i < 3; i++)		/* expecting 3 DNS servers */
		{
			start = GetDelimiteredParam(start, cfg, server);	/* dns server */
			if (strlen(server))
			{
				serverip = GetIpAddressFromStr(server);
				nfxnis_addservice(dns_name, 0, 0, serverip, domain);
			}
		}
	}
	else
	{
		/* skip all the 3 dns servers */
		for (i = 0; i < 3; i++)		/* expecting 3 DNS servers */
			start = GetDelimiteredParam(start, cfg, server);	/* dns server */
	}

	/* NIS domain */
	start = GetDelimiteredParam(start, cfg, domain);		/* nis domain */
	if (strlen(domain))
	{
		start = GetDelimiteredParam(start, cfg, server);	/* nis server */
		if (strlen(server))
		{
			serverip = GetIpAddressFromStr(server);
			nfxnis_addservice(nis_name, 0, 0, serverip, domain);
		}
	}
	else
	{
		/* skip 1 nis server */
		start = GetDelimiteredParam(start, cfg, server);	/* nis server */
	}

	return 1;
}

int setSystemConfig4(char* cfg)
{
int i, res, start = 0;
char ifname[IPM_STR_SIZE];
char ipstr[32], maskstr[32];
u_int32_t ipaddr, ipmask;

	/* MANAGEMENT INTERFACE */
	start = GetDelimiteredParam(start, cfg, ifname);		/* mgmt interface */
	if (strcmp(ifname, "no"))
	{
		start  = GetDelimiteredParam(start, cfg, ipstr);	/* ip */
		ipaddr = GetIpAddressFromStr(ipstr);
		start  = GetDelimiteredParam(start, cfg, maskstr);	/* mask */
		ipmask = GetIpAddressFromStr(maskstr);

		res = ipmd_AddIp(ifname, ipaddr, ipmask, NULL, 0);
	}
	else
	{
		/* skip mgmt ip and mask */
		start = GetDelimiteredParam(start, cfg, ipstr);		/* ip */
		start = GetDelimiteredParam(start, cfg, maskstr);	/* mask */
	}

	/* CLIENT INTERFACES */
	for (i = 0; i < 4; i++)
	{
		start = GetDelimiteredParam(start, cfg, ifname);		/* client if */
		if (strcmp(ifname, "no"))
		{
			start = GetDelimiteredParam(start, cfg, ipstr);		/* ip */
			ipaddr = GetIpAddressFromStr(ipstr);
			start = GetDelimiteredParam(start, cfg, maskstr);	/* mask */
			ipmask = GetIpAddressFromStr(maskstr);

			res = ipmd_AddIp(ifname, ipaddr, ipmask, NULL, 0);
		}
		else
		{
			/* skip ip and mask */
			start = GetDelimiteredParam(start, cfg, ipstr);		/* ip */
			start = GetDelimiteredParam(start, cfg, maskstr);	/* mask */
		}
	}

	/* DEFAULT ROUTE */
	{
	FILE *f;
	char tmpbuf[40];

		start = GetDelimiteredParam(start, cfg, ipstr);			/* default route */

		f = fopen(DEFAULTROUTE, "w");
		if (f)
		{
			sprintf(tmpbuf, "%s\n", ipstr);
			fwrite(tmpbuf, sizeof(char), strlen(tmpbuf), f);
			fclose(f);
		}

		/* also need to add to current routing table */
		ipaddr = GetIpAddressFromStr(ipstr);
		SetDefaultRoute(ipaddr);
	}

	return 1;
}


int SaveSystemConfig(char* file, char* buf)
{
	FILE *f;
	int numwrite;

	f = fopen(file, "w+t");
	if (f == NULL)
		return 0;
	numwrite = fwrite(buf, sizeof(char), strlen(buf), f);
	fclose(f);
	if (numwrite > 0)
		return 1;
	return 0;
}


int setSystemConfig(int from, int step, u_int32_t id, char* cfg)
{
	char file[128];

	/* get file name first */
	if (from == 1)		/* request for perm cfg file */
	{
		int i;

		/* remove all the temp files as this is */
		/* the APPLY button pressed...         */
		for (i = 1; i <= 4; i++)
		{
			sprintf(file, "%s_%d_%08x", CFGFILE_TEMP, i, id);
			unlink(file);
		}

		/* set system config */
		switch (step)
		{
		case 1:
			return setSystemConfig1(cfg);
		case 2:
			return setSystemConfig2(cfg);
		case 3:
			return setSystemConfig3(cfg);
		case 4:
			return setSystemConfig4(cfg);
		}
	}
	else				/* temp file */
	{
		sprintf(file, "%s_%d_%08x", CFGFILE_TEMP, step, id);
		return SaveSystemConfig(file, cfg);
	}

	return 0;
}

