/*  File Name  :    agilefilermain.c 
 *
 *  Description:    HTTPD action functions for filer system information
 *
 *  Created    :    5/15/2002
 *
 *  Author     :    Shyh-Pei Yen
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <cm-api.h>

#include "agilepost.h"
#include "agileutil.h"
#include "../agent/httpdagent.h"

#define INVALID_TAG			0
#define FILER_UPTIME		1
#define FILER_SOFTWARE		2
#define FILER_DATE			3
#define FILER_SLOT			4
#define FILER_MODULE		5
#define FILER_POWER			6
#define FILER_TEMP			7
#define FILER_SOFTVER		8
#define NUM_OF_TAGS			8

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{FILER_UPTIME,		"uptime"},
	{FILER_SOFTWARE,	"software"},
	{FILER_DATE,		"date"},
	{FILER_SLOT,		"slot"},
	{FILER_MODULE,		"module"},
	{FILER_POWER,		"power"},
	{FILER_TEMP,		"temperature"},
	{FILER_SOFTVER,		"softver"},
};


#define PARAM_SLOT			1
#define PARAM_ACT			2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_SLOT,	"slot"},
	{PARAM_ACT,		"act"},
};


/* HERE ARE VARIABLES FOR FILER SYSTEM INFORMATION */

typedef struct _filer_config_t
{
	int valid;
	int	boardType;
	int power;
	int temp;
	char bmfpga_ver[CM_VER_STRING_LEN];
	char luc_ver[CM_VER_STRING_LEN];
	char lmux_ver[CM_VER_STRING_LEN];
} filer_config_t;

static int32 inSlot;
static char inAct[10];
static int dataCount;
static int dataCurr;
static filer_config_t filerConfig[CM_NUM_SLOTS];
static filer_config_t *pstData;

char *moduleName[CM_NUM_CARDTYPES] =
{
	CM_CARD_TYPE_SSC_STR,
	CM_CARD_TYPE_GARFIELD_STR,		/* garfield */
	CM_CARD_TYPE_CHEETAH_STR,		/* cheetah */
	CM_CARD_TYPE_FCNIM_STR,
};

/***********************************************************
 *  Main routines to handle Reset/Enable/Disable Port actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_SLOT:
			sscanf(argval[argind], "%d", &inSlot);
			break;
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		}
	}

	return 0;
}

static int DoRebootSlot(int slot)
{
int stat;
char buf[512];
struct timeval to = {5, 0};

	if (slot >= CM_NUM_SLOTS)
		return -1;

	memset(buf, 0, sizeof(buf));

	if (slot == 0)		/* SSC slot */
	{
	haMessage_t req;
	haMessage_t *rsp = (haMessage_t *) buf;

		req.type = HA_REBOOT_SSC;
		req.datalen = 0;

		stat = sendAgileMsg(-1, &req, sizeof(req),
					"httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
					&to, rsp, sizeof(haMessage_t));
		if ((stat <= 0) || (rsp->rspstatus != HARSP_OK))
			return -1;
	}
	else
	{
	chassisdMsgRebootReq_t req;
	chassisdMsgRebootResp_t *rsp;

		req.hdr.mid = CHASSISD_REBOOT_REQ;
		req.reboot_type = CM_REBOOT_TYPE_SLOT;
		req.slot = slot;
		req.slice = 0;		/* unused */

		rsp = (chassisdMsgRebootResp_t *) buf;

		stat = sendAgileMsg(-1, &req, sizeof(req),
					"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
					&to, rsp, sizeof(chassisdMsgRebootResp_t));
		if ((stat <= 0) || (rsp->rc != CM_REBOOT_OK))
			return -1;
	}

	return 1;
}

static int HandlePortAction(char *newfname, int count, char **argname, char **argval)
{
int res;
char reqbuf[128];
char rspbuf[128];
haMessage_t *req = (haMessage_t *) reqbuf;
haMessage_t *rsp = (haMessage_t *) rspbuf;
struct timeval to = {5, 0};

	memset(reqbuf, 0, sizeof(reqbuf));
	memset(rspbuf, 0, sizeof(rspbuf));

	getInputParams(count, argname, argval);

	if (!strcmp(inAct, "reboot"))
	{
		res = DoRebootSlot(inSlot);
		if (res <= 0)
			strcpy(newfname, "error_rebootslot.html");
	}
	else if (!strcmp(inAct, "sysReboot"))
	{
		req->type = HA_SYSTEM_OP;
		req->datalen = sizeof(u_int32_t);
		*((u_int32_t *) req->data) = HA_SYS_OP_REBOOT;

		res = sendAgileMsg(-1, req,
				HAMESSAGE_HEADER_SIZE + sizeof(u_int32_t),
				"httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&to, rsp, sizeof(haMessage_t));
		if ((res <= 0) || (rsp->rspstatus != HARSP_OK))
			return -1;
	}
	else if (!strcmp(inAct, "sysHalt"))
	{
		req->type = HA_SYSTEM_OP;
		req->datalen = sizeof(u_int32_t);
		*((u_int32_t *) req->data) = HA_SYS_OP_HALT;

		res = sendAgileMsg(-1, req,
				HAMESSAGE_HEADER_SIZE + sizeof(u_int32_t),
				"httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&to, rsp, sizeof(haMessage_t));
		if ((res <= 0) || (rsp->rspstatus != HARSP_OK))
			return -1;
	}
	return 1;
}


/***********************************************************
 * routines to retrieve filer information thru anpssc
 **********************************************************/

static int GetFilerConfigInfo()
{
chassisdMsgInfoReq_t req;
chassisdMsgInfoResp_t resp;
chassisdMsgPowerReq_t pwr_req;
chassisdMsgPowerResp_t pwr_resp;
chassisdMsgTempReq_t temp_req;
chassisdMsgTempResp_t temp_resp;
chassisdMsgVersionReq_t ver_req;
chassisdMsgVersionResp_t ver_resp;
struct timeval to = { 2, 0 };
int stat, i;

	memset(filerConfig, 0, sizeof(filerConfig));

	memset(&req, 0, sizeof(req));
	memset(&resp, 0, sizeof(resp));
	memset(&pwr_req, 0, sizeof(pwr_req));
	memset(&pwr_resp, 0, sizeof(pwr_resp));
	memset(&temp_req, 0, sizeof(temp_req));
	memset(&temp_resp, 0, sizeof(temp_resp));
	memset(&ver_req, 0, sizeof(ver_req));
	memset(&ver_resp, 0, sizeof(ver_resp));

	req.hdr.mid = CHASSISD_GET_CHASSIS_INFO_REQ;
	stat = sendAgileMsg(-1, &req, sizeof(req),
						"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&to, &resp, sizeof(resp));
	if (stat <= 0)
		return stat;

	/* for the next three requests, we don't quit */
	/* even if we don't get a response...       */
	pwr_req.hdr.mid = CHASSISD_GET_POWER_REQ;
	stat = sendAgileMsg(-1, &pwr_req, sizeof(pwr_req),
						"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&to, &pwr_resp, sizeof(pwr_resp));

	temp_req.hdr.mid = CHASSISD_GET_TEMP_REQ;
	stat = sendAgileMsg(-1, &temp_req, sizeof(temp_req),
						"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&to, &temp_resp, sizeof(temp_resp));

	ver_req.hdr.mid = CHASSISD_GET_VERSION_REQ;
	stat = sendAgileMsg(-1, &ver_req, sizeof(ver_req),
						"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&to, &ver_resp, sizeof(ver_resp));

	/* populate our data structure */
	for (i = 0; i < CM_NUM_SLOTS; i++)
	{
		if (resp.present[i] && (resp.num_cpus[i] > 0))
		{
			filerConfig[i].valid = 1;
			filerConfig[i].boardType = resp.board_type[i];

			if (pwr_resp.valid[i])
				filerConfig[i].power = pwr_resp.power[i];
			else
				filerConfig[i].power = -1;

			if (temp_resp.valid[i])
				filerConfig[i].temp = temp_resp.temp[i];
			else
				filerConfig[i].temp = -1;

			strcpy(filerConfig[i].bmfpga_ver, ver_resp.bmfpga_ver[i]);
			strcpy(filerConfig[i].luc_ver, ver_resp.luc_ver[i]);
			strcpy(filerConfig[i].lmux_ver, ver_resp.lmux_ver[i]);
		}
		else
			filerConfig[i].valid = 0;
	}

	return 1;
}

static int getTableRowCount(void)
{
int count = 0;

	if (GetFilerConfigInfo() > 0)
		count = CM_NUM_SLOTS;

	return count;
}

#if 0
static int getTableRowData(int index, filer_config_t *cfgdata)
{
	if (index < 1 || index > dataCount)
		return -1;
	if (cfgdata == NULL)
		return -1;

	memcpy(cfgdata, &filerConfig[index-1], sizeof(filer_config_t));
	return 1;
}
#endif


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	char tmpbuf[1024];
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	switch (tagType) 
	{
	case FILER_UPTIME:
	case FILER_DATE:
		break;
	case FILER_SOFTWARE:
		{
			FILE *f;
			char *s;

			strcpy(value, "UNKNOWN");

			f = fopen("/version", "r");
			if (f != NULL)
			{
				size_t size;
				s = fgetln(f, &size);
				if (s != NULL)
				{
					s[size - 1] = 0;
					strcpy(value, s);
				}
				fclose(f);
			}
		}
		break;
	case FILER_SLOT:
		sprintf(value, "%d", dataCurr-1);
		break;
	case FILER_MODULE:
		if (!pstData->valid ||
			(pstData->boardType < 0) ||
			(pstData->boardType >= CM_NUM_CARDTYPES))
			strcpy(value, "Unknown");
		else
			strcpy(value, moduleName[pstData->boardType]);
		break;
	case FILER_POWER:
		if (pstData->valid && (pstData->power >= 0))
			sprintf(value, "%d", pstData->power);
		else
			strcpy(value, "&nbsp");
		break;
	case FILER_TEMP:
		if (pstData->valid && (pstData->temp >= 0))
			sprintf(value, "%d", pstData->temp);
		else
			strcpy(value, "&nbsp");
		break;
	case FILER_SOFTVER:
		value[0] = '\0';
		if (strlen(pstData->bmfpga_ver)) {
			sprintf(tmpbuf, "%s<br>", pstData->bmfpga_ver);
			strcat(value, tmpbuf);
		}
		if (strlen(pstData->luc_ver)) {
			sprintf(tmpbuf, "%s<br>", pstData->luc_ver);
			strcat(value, tmpbuf);
		}
		if (strlen(pstData->lmux_ver)) {
			sprintf(tmpbuf, "%s<br>", pstData->lmux_ver);
			strcat(value, tmpbuf);
		}
		if (!value[0])
			strcpy(value, "&nbsp");
		break;

	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			pstData = &filerConfig[0];
			return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		pstData = &filerConfig[dataCurr-1];
		return 1;
	}
	else {
		/* other tags */
		if (dataCount <= 0) {
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount)) {
			dataCurr = 1;
			pstData = &filerConfig[0];
		}

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionFilerMain(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	memset(newfname, 0, sizeof(newfname));
	inSlot = 0;
	inAct[0] = '\0';

	/* Handle reboot slot actions */
	if (count > 0)
		HandlePortAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
