/*  File Name  :    agilelunstat.c 
 *
 *  Description:    HTTPD action functions for displaying lun stats.
 *
 *  Created    :    2/14/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <eee-desc.h>
#include <scsi-api.h>
#include <sdm-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define STGDEV_RDREQ		1
#define STGDEV_RDRSP		2
#define STGDEV_WRREQ		3
#define STGDEV_WRRSP		4
#define STGDEV_RLREQ		5
#define STGDEV_RLRSP		6
#define STGDEV_ERROR		7
#define STGDEV_WWNSTR		8
#define STGDEV_ARGWWN		9
#define STGDEV_ARGLUN		10
#define STGDEV_ARGTYPE		11
#define STGDEV_TITLE		12
#define NUM_OF_TAGS			12

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{STGDEV_RDREQ,	"sdm_devStatsRspMsg_t.readReqCnt"},
	{STGDEV_RDRSP,	"sdm_devStatsRspMsg_t.readRspCnt"},
	{STGDEV_WRREQ,	"sdm_devStatsRspMsg_t.writeReqCnt"},
	{STGDEV_WRRSP,	"sdm_devStatsRspMsg_t.writeRspCnt"},
	{STGDEV_RLREQ,	"sdm_devStatsRspMsg_t.relayReqCnt"},
	{STGDEV_RLRSP,	"sdm_devStatsRspMsg_t.relayRspCnt"},
	{STGDEV_ERROR,	"sdm_devStatsRspMsg_t.devErrCnt"},
	{STGDEV_WWNSTR,	"args.wwnstr"},
	{STGDEV_ARGWWN,	"args.wwn"},
	{STGDEV_ARGLUN,	"args.lun"},
	{STGDEV_ARGTYPE,"args.type"},
	{STGDEV_TITLE,	"title.path"},
};


#define PARAM_WWN			1
#define PARAM_LUN			2
#define PARAM_TYPE			3
#define NUM_OF_PARAMS		3

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_WWN,		"wwn"},
	{PARAM_LUN,		"lun"},
	{PARAM_TYPE,	"type"},
};


/* HERE ARE VARIABLES FOR STORAGE DEVICE DETAIL INFORMATION */
static int64 inWwn;
static int32 inLun;
static char inType[10];
static int res;
static sdm_getStatsRspMsg_t stData;


/***********************************************************
 * routines to retrieve storage information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_WWN:
			// inWwn = atol(argval[argind]);
			sscanf(argval[argind], "%lld", &inWwn);
			break;
		case PARAM_LUN:
			inLun = atoi(argval[argind]);
			break;
		case PARAM_TYPE:
			strcpy(inType, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetDeviceStatsData()
{
    int			stat;
    sdm_getStatsReqMsg_t	msg;
	struct timeval	timeout = {5, 0};

	res = -1;

	/* get device stats data */
    msg.hdr.mid = SDM_MID_GET_STATS_REQ;
	msg.type	= SDM_STATS_TYPE_DEV;
	msg.subtype = SDM_STATS_SUBTYPE_USE_WWN;
    msg.wwn     = inWwn;
    msg.lun     = inLun;

    stat = sendAgileMsg(-1, &msg, sizeof(msg),
						"sdm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&timeout, &stData, sizeof(sdm_getStatsRspMsg_t));

	if (stat < 0) 		
		return;
	else if (!stat)
		return;

    if (stData.hdr.mid != SDM_MID_GET_STATS_RSP)
		return;
    else if (stData.eCode == SDM_FAIL)
		return;
    else if (stData.eCode == SDM_NOT_OPEN)
		return;

	res = 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;
	else if (tagType == STGDEV_TITLE) {
		if (inType[0] == 'v')
			strcpy(value, "Storage -> Volume -> LUN Detail -> LUN Stats");
		else
			strcpy(value, "Storage -> Disk -> LUN Detail -> LUN Stats");
		return 1;
	}

	if (res <= 0) {
		strcpy(value, "&nbsp");
		return -1;
	}

	switch (tagType) 
	{
	case STGDEV_RDREQ:
		sprintf(value, "%d", stData.stats.dev_stats.readReqCnt);
		break;
	case STGDEV_RDRSP:
		sprintf(value, "%d", stData.stats.dev_stats.readRspCnt);
		break;
	case STGDEV_WRREQ:
		sprintf(value, "%d", stData.stats.dev_stats.writeReqCnt);
		break;
	case STGDEV_WRRSP:
		sprintf(value, "%d", stData.stats.dev_stats.writeRspCnt);
		break;
	case STGDEV_RLREQ:
		sprintf(value, "%d", stData.stats.dev_stats.relayReqCnt);
		break;
	case STGDEV_RLRSP:
		sprintf(value, "%d", stData.stats.dev_stats.relayRspCnt);
		break;
	case STGDEV_ERROR:
		sprintf(value, "%d", stData.stats.dev_stats.devErrCnt);
		break;
	case STGDEV_WWNSTR:
		GetWwnStr(inWwn, value);
		break;
	case STGDEV_ARGWWN:
		sprintf(value, "%lld", inWwn);
		break;
	case STGDEV_ARGLUN:
		sprintf(value, "%d", inLun);
		break;
	case STGDEV_ARGTYPE:
		strcpy(value, inType);
		break;
	case STGDEV_TITLE:
		if (inType[0] == 'v')
			strcpy(value, "Storage -> Volume -> LUN Detail -> LUN Stats");
		else
			strcpy(value, "Storage -> Disk -> LUN Detail -> LUN Stats");
		break;
	default:
		return -1;
	}

	if (value && value[0] == '\0')
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionLunStat(void *r, int count, char **argname, char **argval)
{
	/* initialize storage information variables */
	getInputParams(count, argname, argval);
	GetDeviceStatsData();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
