/*  File Name  :    agilenfsadd.c
 *
 *  Description:    HTTPD action functions for add export/share.
 *
 *  Created    :    3/18/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>
#include <anpssc-api.h>

#include <evm_com_defs.h>
#include <evm-api.h>
#include <fs-api.h>
#include <rpc.h>
#include <mount.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_ID				1
#define VOL_NAME			2
#define PATH_NAME			3
#define PATH_IP				4
#define PATH_HOST			5
#define PATH_MASK			6
#define PATH_HOSTMASK		7
#define PATH_PERM			8
#define PATH_RS				9
#define NUM_OF_TAGS			9

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_ID,		"args.vol_id"},
	{VOL_NAME,		"args.vol_name"},
	{PATH_NAME,		"args.path"},
	{PATH_IP,		"args.ip"},
	{PATH_HOST,		"share_path.host"},
	{PATH_MASK,		"share_path.mask"},
	{PATH_HOSTMASK,	"share_path.hostip_subnet"},
	{PATH_PERM,		"share_path.permission"},
	{PATH_RS,		"share_path.root_squash"},
};


#define PARAM_ACT			1
#define PARAM_VOLID			2
#define PARAM_VOLNAME		3
#define PARAM_PATH			4
#define PARAM_IP			5
#define PARAM_ORGIP			6
#define PARAM_HOST			7
#define PARAM_MASK			8
#define PARAM_ROOT			9
#define PARAM_ACCESS		10
#define PARAM_RWN			11
#define PARAM_RS			12
#define NUM_OF_PARAMS		12

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_VOLID,	"id"},
	{PARAM_VOLNAME,	"vn"},
	{PARAM_PATH,	"path"},
	{PARAM_IP,		"ip"},
	{PARAM_ORGIP,	"orgip"},
	{PARAM_HOST,	"host"},
	{PARAM_MASK,	"mask"},
	{PARAM_ROOT,	"pr"},
	{PARAM_ACCESS,	"pa"},
	{PARAM_RWN,		"prw"},
	{PARAM_RS,		"rs"},
};


/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static int64 inId;
static char inAct[10];
static char bufname[VOL_MAX_NAME+1];
static char bufpath[MAX_NAME_LEN+1];
#if 0
static int32 inIp;
#endif
static char inIp[32];
static char inOrgIp[32];
static char bufhost[32]; // host ip
static char bufmask[32];
static char perm_root[5], perm_access[5];
static char perm_rw[5], opt_rs[5];


typedef struct hostInfo {
	uint32	hostIP;
	uint32	hostMask;
	char	hostIPStr[32];
	char	hostName[128];
	int		rootsquash;
	int		readwrite;
} hostInfo_t;

static int dataCount;
static int dataCurr;
static hostInfo_t *hostInfo = NULL;
static hostInfo_t stData;


/***********************************************************
 * routines to retrieve export/share information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_VOLID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_VOLNAME:
			strcpy(bufname, argval[argind]);
			break;
		case PARAM_PATH:
			strcpy(bufpath, argval[argind]);
			// convertUrlCharToSpecChar(bufpath);
			break;
		case PARAM_IP:
			strcpy(inIp, argval[argind]);
#if 0
			sscanf(argval[argind], "%u", &inIp);
#endif
			break;
		case PARAM_ORGIP:
			strcpy(inOrgIp, argval[argind]);
			break;
		case PARAM_HOST:
			strcpy(bufhost, argval[argind]);
			break;
		case PARAM_MASK:
			strcpy(bufmask, argval[argind]);
			break;
		case PARAM_ROOT:
			strcpy(perm_root, argval[argind]);
			break;
		case PARAM_ACCESS:
			strcpy(perm_access, argval[argind]);
			break;
		case PARAM_RWN:
			strcpy(perm_rw, argval[argind]);
			break;
		case PARAM_RS:
			strcpy(opt_rs, argval[argind]);
			break;
		}
	}

	return 0;
}

static int getTableRowCount(int64 volID,
			char *path, char *localIP, hostInfo_t **hostList)
{
	int					stat, i, count;
	char 				tmpbuf[2048];
	char 				rspbuf[4096];
	mnt_infoReq_t   	req;
	mnt_infoResp_t		*rsp;
	hostInfo_t			*p1;
	mnt_infoEntry_t 	*p2;
	uint32				inIpNum;
	struct timeval		timeout = {5, 0};

	/* no need to retrieve mnt info for add new share */
	if (!bufpath[0])
		return -1;

    /* Fill the request. */
	req.volId.whole_thing = volID;
	req.msgHdr.msgType = MNT_EXPORT_SHOW;
	req.nameLen = 0;		// All Paths for the given volume
	req.cookie = 0;	/* this is the starting index */
	//req.msgHdr.private = 0;	/* this is the starting index */

#if 0
/* waiting for Joseph's API to handle volume... */
	req->nameLen = strlen(path);
	strcpy(req.pathName, path);
	req.ipAddr = 0;	// All Hosts/Clients
#endif

	count = 0;
	while (1)
	{
		memset(rspbuf, 0, sizeof(rspbuf));

		/* Send message to FS. */
		stat = sendAgileMsg(-1, &req, sizeof(mnt_infoReq_t),
					"mountd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &timeout,
					rspbuf, sizeof(rspbuf));

		/* Check response from FS. */
		if (stat <= 0)
			return count;

		rsp = (mnt_infoResp_t *)rspbuf;

		if ((rsp->numOfEntries == 0) ||
			(req.cookie == rsp->cookie))
			break;
#if 0
		if ((rsp->numOfEntries == 0) ||
			(req.msgHdr.private == rsp->msgHdr.private))
			break;
#endif

		if (*hostList)
		{
			void *tp;
			int size;

			size = (count + rsp->numOfEntries) * sizeof(hostInfo_t);
			tp = realloc(*hostList, size);
			if (tp == NULL)
				return count;
			count += rsp->numOfEntries;
			*hostList = tp;
		}
		else
		{
			*hostList = malloc(rsp->numOfEntries * sizeof(hostInfo_t));
			if (*hostList == NULL)
				return -1;
			count = rsp->numOfEntries;
		}

			/* need to filter out those hosts belong to other shares... */
		inIpNum = GetIpAddressFromStr(localIP);
		p1 = (*hostList) + (count - rsp->numOfEntries);
		p2 = rsp->mtnInfoEntry;

		for (i = 0; i < rsp->numOfEntries; i++)
		{
			strncpy(tmpbuf, p2->pathName, p2->pathNameLen);
			tmpbuf[p2->pathNameLen] = '\0';
			if (!strcmp(tmpbuf, &path[1]) && (p2->localIpAddr == inIpNum))
			{
				char *p;

				memset(p1, 0, sizeof(hostInfo_t));

				p = p2->pathName + p2->pathNameLen;
				strncpy(tmpbuf, p, p2->exportInfoLen);
				tmpbuf[p2->exportInfoLen]='\0';
				p = tmpbuf;

				if (isdigit(*p))		/* start with digit, an IP addr */
				{
					while (isdigit(*p) || (*p == '.'))
						p++;
					if (*p == '(')
					{
						*p++ = '\0';
						strcpy(p1->hostName, tmpbuf);
						strcpy(p1->hostIPStr, tmpbuf);
						p1->hostIP = GetIpAddressFromStr(tmpbuf);
					}
					else
					{
						/* something wrong with the data */
						p = tmpbuf;
					}
				}
				else if (*p == '(')
				{
					p1->hostName[0] = '\0';
					strcpy(p1->hostIPStr, "0.0.0.0");
					p1->hostIP = 0;
				}
					
				p1->rootsquash = GetNfsRootSquash(p);
				p1->readwrite = GetNfsPermission(p);
				p1->hostMask = 0;			/* not currently supported */

				p1++;
			}
			MNT_INFO_RESP_ADV_ENTRY_PTR(p2);
		}

		req.cookie = rsp->cookie;
		//req.msgHdr.private = rsp->msgHdr.private;
	}
	return count;
}

static int getTableRowData(int *index, hostInfo_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if ((hostInfo == NULL) || (data == NULL))
		return -1;

	memcpy(data, &hostInfo[(*index)-1], sizeof(hostInfo_t));
	// memcpy(data, &hostInfo[(*index)-1], sizeof(mnt_infoEntry_t));

	return 0;
}


/***********************************************************
 *  Main routines to handle Add/Del/Mod actions
 **********************************************************/

static void GetOptionStr(char* buf)
{
	if (perm_rw[0] == 'w')
		strcpy(buf, "rw");
	else 
		strcpy(buf, "ro");
	if (opt_rs[0] == 'y')
		strcat(buf, ",root_squash");
	else
		strcat(buf, ",no_root_squash");
}

static int DoAddHost(int64 volID,
	char *path, char *localIP, char *clientIP, char *opts)
{
	int					stat;
	char 				reqbuff[4096];
	mnt_infoReq_t   	*req;
	mnt_infoResp_t		rsp;
	char				*p;
	struct timeval		timeout = {60, 0};

    /* Fill the request. */
	memset(reqbuff, 0, sizeof(reqbuff));
    req = (mnt_infoReq_t *)reqbuff;

	req->msgHdr.msgType = MNT_EXPORT_REQ;
//	req->volId.whole_thing = volID;

	/* prepare path name first. If there is a non-zero */
	/* host, it should be prepend to the path name...  */
	// req->ipAddr = htonl(GetIpAddressFromStr(clientIP));

	p = req->pathName;
	if (clientIP && clientIP[0] && strcmp(clientIP, "0.0.0.0"))
		sprintf(p, "%s:%s", clientIP, path);
	else
		strcpy(p, path);
	req->nameLen = strlen(p);

	/* now prepare options string. If there is a non-zero */
	/* shared local IP interface, it sould be add here... */
	p = req->pathName + req->nameLen;

	if (localIP && localIP[0] && strcmp(localIP, "0.0.0.0"))
		sprintf(p, "%s,laddr=%s", opts, localIP);
	else
		strcpy(p, opts);
	req->optionsLen = strlen(p);

	req->localIpAddr = GetIpAddressFromStr(localIP);
    /* Send message to FS. */
    stat = sendAgileMsg(-1, reqbuff,
						sizeof(mnt_infoReq_t)+req->nameLen+req->optionsLen+2,
                        "mountd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &timeout,
                        &rsp, sizeof(rsp));

    /* Check response from FS. */
	if (stat <= 0) 
		return -1;
	if (rsp.msgHdr.status != NFX_OK)
		return -1;

	return 1;
}

static int DoDelHost(int64 volID, char *path, char *localIP, char *host)
{
	int					stat;
	char 				*p, reqbuff[4096];
	mnt_infoReq_t   	*req;
	mnt_infoResp_t		rsp;
	struct timeval		timeout = {5, 0};

    /* Fill the request. */
	memset(reqbuff, 0, sizeof(reqbuff));
    req = (mnt_infoReq_t *)reqbuff;

	req->msgHdr.msgType = MNT_UNEXPORT_REQ;
	req->volId.whole_thing = volID;

	p = req->pathName;
	if (host[0] && strcmp(host, "0.0.0.0"))
		sprintf(p, "%s:%s", host, path);
	else
		strcpy(p, path);
	req->nameLen = strlen(p);

#if 0
/* not how they are used in Joseph's implementation */
	req->nameLen = strlen(path);
	strcpy(req->pathName, path);
	req->ipAddr = GetIpAddressFromStr(host);
#endif

	if (localIP && localIP[0] && strcmp(localIP, "0.0.0.0"))
		req->localIpAddr = GetIpAddressFromStr(localIP);
#if 0
	if (localIP && localIP[0] && strcmp(localIP, "0.0.0.0"))
		req->localIpAddr = htonl(GetIpAddressFromStr(localIP));
#endif

    /* Send message to FS. */
    stat = sendAgileMsg(-1, reqbuff, sizeof(mnt_infoReq_t)+req->nameLen,
                        "mountd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &timeout,
                        &rsp, sizeof(rsp));

    /* Check response from FS. */
	if (stat <= 0) 
		return -1;
	if (rsp.msgHdr.status != NFX_OK)
		return -1;

	return 1;
}

static int DoModHost(int64 volID,
		char *path, char *localIP, char *host, char *opts)
{
	/* since currently, mountd API doesn't support update  */
	/* properly, we'll need to delete entry first then add */
	/* the updated entry to mountd...                      */

	DoDelHost(volID, path, localIP, host);
	return DoAddHost(volID, path, localIP, host, opts);

#if 0
	int					stat;
	char 				reqbuff[4096];
	char				tempbuf[50];
	mnt_infoReq_t   	*req;
	mnt_infoResp_t		rsp;
	struct timeval		timeout = {5, 0};

    /* Fill the request. */
	memset(reqbuff, 0, sizeof(reqbuff));
    req = (mnt_infoReq_t *)reqbuff;
	req->msgHdr.msgType = MNT_EXPORT_UPDATE;
	req->volId.whole_thing = inId;
	req->nameLen = strlen(bufpath);
	strcpy(req->pathName, bufpath);
	req->ipAddr = GetIpAddressFromStr(bufhost);
	GetOptionStr(tempbuf);
	req->optionsLen = strlen(tempbuf);
	strcpy(req->options, tempbuf);

    /* Send message to FS. */
    stat = sendAgileMsg(-1, reqbuff, sizeof(reqbuff),
                        "mountd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &timeout,
                        &rsp, sizeof(rsp));

    /* Check response from FS. */
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;
    if (rsp.msgHdr.msgType != MNT_EXPORT_UPDATE)
		return -1;
	if (rsp.msgHdr.status != NFX_OK)
		return -1;

	return 1;
#endif
}

static int DoSaveShare()
{
	int					stat, msgLen;
	char 				reqbuff[4096];
	mnt_infoReq_t   	*req;
	mnt_infoResp_t		rsp;
	struct timeval		timeout = {5, 0};
	char				*p, tmppermission[16] = "rw,root_squash";

    /* Fill the request. */
	memset(reqbuff, 0, sizeof(reqbuff));
    req = (mnt_infoReq_t *)reqbuff;

	req->msgHdr.msgType = MNT_EXPORT_REQ;
	req->volId.whole_thing = inId;

	/* path name... */
	req->nameLen = strlen(bufpath);
	strcpy(req->pathName, bufpath);

#if 0
/* Joseph's API doesn't support this yet... */
	req->ipAddr = 0; // ??? not to modify host here (just the local ip)
#endif

	req->localIpAddr = GetIpAddressFromStr(inIp);

	/* options */
	p = req->pathName + req->nameLen;
	strcpy(p, tmppermission);
#if 0
	req->optionsLen = strlen(p);
#endif
	if (inIp[0] && strcmp(inIp, "0.0.0.0"))
		sprintf(p, "%s,laddr=%s", tmppermission, inIp);
	else
		strcpy(p, tmppermission);
	req->optionsLen = strlen(p);

	msgLen = sizeof(mnt_infoReq_t) + req->nameLen + req->optionsLen;

    /* Send message to FS. */
    stat = sendAgileMsg(-1, reqbuff, msgLen,
                        "mountd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &timeout,
                        &rsp, sizeof(rsp));

    /* Check response from FS. */
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;
	if (rsp.msgHdr.status != NFX_OK)
		return -1;

	return 1;
}

static int DoModShare()
{
	int		stat, i, count;
	char	tmpbuf[64];

	/* if the IP address is not changed, don't do anything */
	if (!strcmp(inIp, inOrgIp))
		return 1;

	count = getTableRowCount(inId, bufpath, inOrgIp, &hostInfo);
	if (count == 0)
	{
		/* weird, we should get at least one host */
		/* Anyway, we'll pretend adding new entry */
		return DoSaveShare();
	}

	/* now del original share and add new share for each host */
	for (i = 0; i < count; i++)
	{
		DoDelHost(inId, bufpath, inOrgIp, hostInfo[i].hostIPStr);

		if (hostInfo[i].readwrite)
			strcpy(tmpbuf, "rw");
		else
			strcpy(tmpbuf, "ro");
		if (hostInfo[i].rootsquash)
			strcat(tmpbuf, ",root_squash");
		else
			strcat(tmpbuf, ",no_root_squash");

		stat = DoAddHost(inId, bufpath, inIp,
						hostInfo[i].hostIPStr, tmpbuf);
		/* we don't check status as we need to continue */
		/* to add all the clients in the host list      */
	}

	return 1;
}

static int HandleNfsAction(char *newfname)
{
	int res = 1;
	char tmpbuf[64];

	// Add Host
	if (!strcmp(inAct, "add"))
	{
		GetOptionStr(tmpbuf);
		res = DoAddHost(inId, bufpath, inIp, bufhost, tmpbuf);
		if (res <= 0)
			strcpy(newfname, "error_addhost.html");
	}
	// Del Host
	else if (!strcmp(inAct, "del"))
	{
		res = DoDelHost(inId, bufpath, inIp, bufhost);
		if (res <= 0)
			strcpy(newfname, "error_delhost.html");
	}
	// Modify Host
	else if (!strcmp(inAct, "mod"))
	{
		GetOptionStr(tmpbuf);
		res = DoModHost(inId, bufpath, inIp, bufhost, tmpbuf);
		if (res <= 0)
			strcpy(newfname, "error_modhost.html");
	}
	// Add Share
	else if (!strcmp(inAct, "asave"))
	{
		res = DoSaveShare();
		if (res <= 0)
			strcpy(newfname, "error_savenfs.html");
	}
	// Mod Share
	else if (!strcmp(inAct, "msave"))
	{
		res = DoModShare();
		if (res <= 0)
			strcpy(newfname, "error_savenfs.html");
	}
	return res;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	if (tagType == VOL_NAME)
		strcpy(value, bufname);
	else if (tagType == VOL_ID)
		sprintf(value, "%lld", inId);
	else if (tagType == PATH_NAME)
		strcpy(value, bufpath);
	else if (tagType == PATH_IP)
	{
		strcpy(value, inIp);
#if 0
		GetIpAddressStr(inIp, value);
#endif
	}
	else {
		if ((dataCurr < 1) || (dataCurr > dataCount))
			return -1;

		switch (tagType) 
		{
		case VOL_ID:
			sprintf(value, "%lld", inId);
			break;
		case VOL_NAME:
			strcpy(value, bufname);
			break;
		case PATH_NAME:
			strcpy(value, bufpath);
			break;
		case PATH_IP:
			strcpy(value, inIp);
			break;
		case PATH_HOST:
			strcpy(value, stData.hostIPStr);
			break;
		case PATH_MASK:
			//GetIpAddressStr(stData.subnet_mask, value);
			strcpy(value, "0");
			break;
		case PATH_HOSTMASK:
			strcpy(value, stData.hostIPStr);
			/*
			if (stData.subnet_mask != NULL)
			{
				GetIpAddressStr(stData.subnet_mask, tempbuf);
				strcat(value, "/");
				strcat(value, tempbuf);
			}
			*/
			break;
		case PATH_PERM:
			if (stData.readwrite)
				strcpy(value, "RW");
			else
				strcpy(value, "RO");
			break;
		case PATH_RS:
			if (stData.rootsquash)
				strcpy(value, "Yes");
			else
				strcpy(value, "No");
			break;
		default:
			return -1;
		}
	}

	if (value && !value[0])
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount(inId, bufpath, inIp, &hostInfo);
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNfsAdd(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));
	bufname[0] = '\0';
	bufpath[0] = '\0';
	bufhost[0] = '\0';
	inAct[0] = '\0';
	inIp[0] = '\0';
	hostInfo = NULL;

	getInputParams(count, argname, argval);

	/* Handle Save Share, Add/Del/Mod Host actions */
	if (count > 0)
		HandleNfsAction(newfname);

	prepare_agile_template(r, newfname);

	/* initialize nfs information variables */
	dataCount = 0;
	dataCurr = 0;

	agile_scan_and_send(r, actRptTagFunc);

	if (hostInfo != NULL)
	{
		free(hostInfo);
		hostInfo = NULL;
	}

	return 1;
}
