/*  File Name  :    agilenwmanarp.c
 *
 *  Description:    HTTPD action functions for managing network arp table.
 *
 *  Created    :    2/6/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#undef MIN
#undef MAX
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ipmd-lib.h>
#include "../agent/httpdagent.h"
#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define NWARP_ADDR			1
#define NWARP_MAC			2
#define NWARP_AGE			3
#define NWARP_IFNAME		4
#define NUM_OF_TAGS			4

#define NUM_ARP_ENTRIES		20

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NWARP_ADDR,	"ipm_arp_t.arp_addr"},
	{NWARP_MAC,		"ipm_arp_t.arp_mac"},
	{NWARP_AGE,		"ipm_arp_t.arp_age"},
	{NWARP_IFNAME,	"ipm_arp_t.arp_ifname"}
};

#define PARAM_ACT			1
#define PARAM_IP			2
#define PARAM_MAC			3
#define NUM_OF_PARAMS		3

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_IP,		"ip"},
	{PARAM_MAC,		"mac"}
};

static char bufact[5];
static u_int32_t arpip;
static char bufmac[20];


/* HERE ARE VARIABLES FOR STORAGE DEVICE LIST INFORMATION */
static int dataCount;
static int dataCurr;
static ipm_arp_t *stArpTable;
static ipm_arp_t stData;


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getTableRowCount(void)
{
	int32	res;
	int count, need;

	/* Try to get ARP Table with maxi. 20 entries */
	stArpTable = (ipm_arp_t *) malloc(NUM_ARP_ENTRIES * sizeof(ipm_arp_t));
	if (stArpTable == NULL)
		return -1;
	res = ipmd_GetArpTbl(stArpTable, NUM_ARP_ENTRIES*sizeof(ipm_arp_t), 
							&count, &need);
	if (res == IPM_GET_ERR)
		return -1;
	else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
	{
		// Need a larger buffer
		free(stArpTable);
		stArpTable = (ipm_arp_t *) malloc(need);
		if (stArpTable == NULL)
			return -1;

		res = ipmd_GetArpTbl(stArpTable, need, &count, &need);
		if (res == IPM_GET_ERR)
			return -1;
		else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
			return -1;
	}

	if (count < 0)
		return -1;
	return count;
}

static int getTableRowData(int *index, ipm_arp_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (stArpTable == NULL)
		return -1;

	memcpy(data, &stArpTable[(*index)-1], sizeof(ipm_arp_t));
	return 0;
}


/***********************************************************
 *  Main routines to handle Add, Del actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
int argind, paramType;
char bufip[20];
struct in_addr ipaddr;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(bufact, argval[argind]);
			break;
		case PARAM_IP:
			strcpy(bufip, argval[argind]);
			inet_aton(bufip, &ipaddr);
			arpip = ipaddr.s_addr;
			break;
		case PARAM_MAC:
			strcpy(bufmac, argval[argind]);
			break;
		}
	}

	return 0;
}

static int HandleArpAction(char *newfname, int count, char **argname, char **argval)
{
haMessage_t *msg;
haArpEntry_t *pArpData;
char buf[256];
int n;
struct timeval tout;

	msg = (haMessage_t *) buf;
	msg->clientData = getpid();
	msg->rspstatus = 0;
	msg->datalen = sizeof(haArpEntry_t);
	pArpData = (haArpEntry_t *) msg->data;
	pArpData->ip = arpip;

	getInputParams(count, argname, argval);

	if (!strcmp(bufact, strAdd))
	{
		msg->type = HA_ADDARP;
		memcpy(pArpData->bufmac, bufmac, 20);

	}
	else if (!strcmp(bufact, strDel))
	{
		msg->type = HA_DELARP;
	}

	tout.tv_sec = 5;
	tout.tv_usec = 0;

	n = sendAgileMsg(-1, buf, HAMESSAGE_HEADER_SIZE + sizeof(haArpEntry_t),
						"httpdagent", NFX_SSC_UNIX, SSC_SLOT, 0,
						&tout, buf, sizeof(buf));
	if (n < 0) {
		return -1;
	}
	else if (!n) {
		return -1;
	}
	else {
		/* received response from httpdagent */
		msg = (haMessage_t *) buf;
		if (msg->rspstatus != HARSP_OK) {
			if (msg->type == HA_ADDARP)
				strcat(newfname, "error_addarp.html");
			else if (msg->type == HA_DELARP)
				strcat(newfname, "error_delarp.html");
			return -1;
		}
	}

	return 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NWARP_ADDR:
		GetIpAddressStr(stData.arp_addr, value);
		break;
	case NWARP_MAC:
		GetMacAddressStr(stData.arp_mac, value);
		break;
	case NWARP_AGE:
		sprintf(value, "%d", stData.arp_age);
		break;
	case NWARP_IFNAME:
		strcpy(value, stData.arp_ifname);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		if (dataCount <= 0) {
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount)) {
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) < 0) {
				return -1;
			}
		}

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNwManARP(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	/* Handle Add, Del actions */
	memset(newfname, 0, sizeof(newfname));
	if (count > 0)
		HandleArpAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;
	stArpTable = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (stArpTable != NULL)
		free(stArpTable);
	return 1;
}
