/*  File Name  :    actionNwStat.c
 *
 *  Description:    HTTPD action functions for displaying all interface stats.
 *
 *  Created    :    2/28/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <ipmd-lib.h>

#include "agilepost.h"
#include "agileutil.h"
#include "../agent/httpdagent.h"

#define INVALID_TAG			0
#define NWIF_TYPE			1
#define NWIF_SLOT			2
#define NWIF_LINK			3
#define NWIF_NAME			4
#define NWIF_NTYPE			5
#define NWIF_SPEED			6
#define NWIF_QLEN			7
#define NWIF_PHYS			8
#define NWIF_ADDRL			9
#define NWIF_HEADL			10
#define NWIF_INQUO			11
#define NWIF_OUTQUO			12
#define NWIF_MTU			13
#define NWIF_BRATE			14
#define NWIF_INPKT			15
#define NWIF_INERR			16
#define NWIF_OUTPKT			17
#define NWIF_OUTERR			18
#define NWIF_COLL			19
#define NWIF_INOCT			20
#define NWIF_OUTOCT			21
#define NWIF_INMCST			22
#define NWIF_OUTMCST		23
#define NWIF_INDROP			24
#define NWIF_UNSUPP			25
#define NWIF_INTIME			26
#define NWIF_OUTTIME		27
#define NWIF_MODTIME		28
#define NWIF_MODMTIME		29
#define NUM_OF_TAGS			29

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NWIF_TYPE,		"args.if_type"},
	{NWIF_SLOT,		"args.if_slot"},
	{NWIF_LINK,		"args.if_link"},
	{NWIF_NAME,		"ipm_ifstat_t.ifi_ifname"},
	{NWIF_NTYPE,	"ipm_ifstat_t.ifi_type"},
	{NWIF_SPEED,	"ipm_ifstat_t.ifi_speed"},
	{NWIF_QLEN,		"ipm_ifstat_t.ifi_oQlen"},
	{NWIF_PHYS,		"ipm_ifstat_t.ifi_physical"},
	{NWIF_ADDRL,	"ipm_ifstat_t.ifi_addrlen"},
	{NWIF_HEADL,	"ipm_ifstat_t.ifi_hdrlen"},
	{NWIF_INQUO,	"ipm_ifstat_t.ifi_recvquota"},
	{NWIF_OUTQUO,	"ipm_ifstat_t.ifi_xmitquota"},
	{NWIF_MTU,		"ipm_ifstat_t.ifi_mtu"},
	{NWIF_BRATE,	"ipm_ifstat_t.ifi_baudrate"},
	{NWIF_INPKT,	"ipm_ifstat_t.ifi_ipackets"},
	{NWIF_INERR,	"ipm_ifstat_t.ifi_ierrors"},
	{NWIF_OUTPKT,	"ipm_ifstat_t.ifi_opackets"},
	{NWIF_OUTERR,	"ipm_ifstat_t.ifi_oerrors"},
	{NWIF_COLL,		"ipm_ifstat_t.ifi_collisions"},
	{NWIF_INOCT,	"ipm_ifstat_t.ifi_ibytes"},
	{NWIF_OUTOCT,	"ipm_ifstat_t.ifi_obytes"},
	{NWIF_INMCST,	"ipm_ifstat_t.ifi_imcasts"},
	{NWIF_OUTMCST,	"ipm_ifstat_t.ifi_omcasts"},
	{NWIF_INDROP,	"ipm_ifstat_t.ifi_iqdrops"},
	{NWIF_UNSUPP,	"ipm_ifstat_t.ifi_noproto"},
	{NWIF_INTIME,	"ipm_ifstat_t.ifi_recvtiming"},
	{NWIF_OUTTIME,	"ipm_ifstat_t.ifi_xmittiming"},
	{NWIF_MODTIME,	"ipm_ifstat_t.ifi_lastchange_sec"},
	{NWIF_MODMTIME,	"ipm_ifstat_t.ifi_lastchange_usec"},
};


/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static ipm_ifstat_t stData;
static int getres;


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/


static void GetStatData()
{
int n;
haMessage_t *msg;
haIfStats_t *phaifmsg;
char buf[1024];
struct timeval tout;
ipm_ifstat_t *pifstat;

	msg = (haMessage_t *) buf;
	msg->clientData = getpid();
	msg->type = HA_GETIFSTATS;
	msg->rspstatus = 0;
	msg->datalen = sizeof(haIfStats_t);
	strcpy(msg->data, "\0");

	tout.tv_sec = 5;
	tout.tv_usec = 0;

	n = sendAgileMsg(-1, buf, HAMESSAGE_HEADER_SIZE + sizeof(haIfStats_t),
					"httpdagent", NFX_SSC_UNIX, SSC_SLOT, 0,
					&tout, buf, sizeof(buf));
	if (n < 0) {
		getres = -1;
	}
	else if (!n) {
		getres = -1;
	}
	else {
		/* received response from httpdagent */
		msg = (haMessage_t *) buf;
		if (msg->rspstatus != HARSP_OK)
			return;
		phaifmsg = (haIfStats_t *) msg->data;
		pifstat = (ipm_ifstat_t *) phaifmsg->stats;
		memcpy(&stData, pifstat, sizeof(ipm_ifstat_t));
		getres = 1;
	}
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (getres < 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
#if 0
	case NWIF_TYPE:
		strcpy(value, buftype);
		break;
	case NWIF_SLOT:
		strcpy(value, bufslot);
		break;
	case NWIF_LINK:
		strcpy(value, buflink);
		break;
#endif
	case NWIF_NAME:
		strcpy(value, stData.ifi_ifname);
		break;
	case NWIF_NTYPE:
		sprintf(value, "%d", stData.ifi_type);	/* need a GetString fct from ipmd */
		break;
	case NWIF_SPEED:
		sprintf(value, "%d", stData.ifi_speed);
		break;
	case NWIF_QLEN:
		sprintf(value, "%d", stData.ifi_oQlen);
		break;
	case NWIF_PHYS:
		sprintf(value, "%d", stData.ifi_physical);	/* need a GetString fct from ipmd */
		break;
	case NWIF_ADDRL:
		sprintf(value, "%d", stData.ifi_addrlen);
		break;
	case NWIF_HEADL:
		sprintf(value, "%d", stData.ifi_hdrlen);
		break;
	case NWIF_INQUO:
		sprintf(value, "%d", stData.ifi_recvquota);
		break;
	case NWIF_OUTQUO:
		sprintf(value, "%d", stData.ifi_xmitquota);
		break;
	case NWIF_MTU:
		sprintf(value, "%d", stData.ifi_mtu);
		break;
	case NWIF_BRATE:
		sprintf(value, "%d", stData.ifi_baudrate);
		break;
	case NWIF_INPKT:
		sprintf(value, "%d", stData.ifi_ipackets);
		break;
	case NWIF_INERR:
		sprintf(value, "%d", stData.ifi_ierrors);
		break;
	case NWIF_OUTPKT:
		sprintf(value, "%d", stData.ifi_opackets);
		break;
	case NWIF_OUTERR:
		sprintf(value, "%d", stData.ifi_oerrors);
		break;
	case NWIF_COLL:
		sprintf(value, "%d", stData.ifi_collisions);
		break;
	case NWIF_INOCT:
		sprintf(value, "%d", stData.ifi_ibytes);
		break;
	case NWIF_OUTOCT:
		sprintf(value, "%d", stData.ifi_obytes);
		break;
	case NWIF_INMCST:
		sprintf(value, "%d", stData.ifi_imcasts);
		break;
	case NWIF_OUTMCST:
		sprintf(value, "%d", stData.ifi_omcasts);
		break;
	case NWIF_INDROP:
		sprintf(value, "%d", stData.ifi_iqdrops);
		break;
	case NWIF_UNSUPP:
		sprintf(value, "%d", stData.ifi_noproto);
		break;
	case NWIF_INTIME:
		sprintf(value, "%d", stData.ifi_recvtiming);
		break;
	case NWIF_OUTTIME:
		sprintf(value, "%d", stData.ifi_xmittiming);
		break;
	case NWIF_MODTIME:
		sprintf(value, "%d", stData.ifi_lastchange_sec);
		break;
	case NWIF_MODMTIME:
		sprintf(value, "%d", stData.ifi_lastchange_usec);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	return getTagValue(tag, value);
}

int actionNwStat(void *r)
{
	/* initialize network information variables */
	GetStatData();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
