/*  File Name  :    agilesshotmain.c
 *
 *  Description:    HTTPD action functions for main Snapshot Page.
 *
 *  Created    :    3/15/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
// #include <sys/types.h>
// #include <unistd.h>
#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <evm_com_defs.h>
#include <evm-api.h>
#include <fs-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define SS_VOLID			1
#define SS_VOLNAME			2
#define SS_RESV				3
#define SS_WEEK				4
#define SS_DAILY			5
#define SS_HOUR				6
#define SS_NAME				7
#define SS_USAGE			8
#define SS_BLOCK			9
#define SS_TIME				10
#define SS_H1_STS			11
#define SS_H2_STS			12
#define SS_H3_STS			13
#define SS_H4_STS			14
#define SS_H5_STS			15
#define SS_H6_STS			16
#define SS_H7_STS			17
#define SS_H8_STS			18
#define SS_H9_STS			19
#define SS_H10_STS			20
#define SS_H11_STS			21
#define SS_H12_STS			22
#define SS_H13_STS			23
#define SS_H14_STS			24
#define SS_H15_STS			25
#define SS_H16_STS			26
#define SS_H17_STS			27
#define SS_H18_STS			28
#define SS_H19_STS			29
#define SS_H20_STS			30
#define SS_H21_STS			31
#define SS_H22_STS			32
#define SS_H23_STS			33
#define SS_H24_STS			34
#define NUM_OF_TAGS			34

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SS_VOLID,		"args.volId"},
	{SS_VOLNAME,	"args.vol_name"},
	{SS_RESV,		"snap_sch.snap_reserve"},
	{SS_WEEK,		"snap_sch.snap_weekly"},
	{SS_DAILY,		"snap_sch.snap_daily"},
	{SS_HOUR,		"snap_sch.snap_hourly"},
	{SS_NAME,		"snap_usg.snapshot_name"},
	{SS_USAGE,		"snap_usg.snapshot_usage"},
	{SS_BLOCK,		"snap_usg.snapshot_block"},
	{SS_TIME,		"snap_usg.snapshot_time"},
	{SS_H1_STS,		"snap_sch.h1_status"},
	{SS_H2_STS,		"snap_sch.h2_status"},
	{SS_H3_STS,		"snap_sch.h3_status"},
	{SS_H4_STS,		"snap_sch.h4_status"},
	{SS_H5_STS,		"snap_sch.h5_status"},
	{SS_H6_STS,		"snap_sch.h6_status"},
	{SS_H7_STS,		"snap_sch.h7_status"},
	{SS_H8_STS,		"snap_sch.h8_status"},
	{SS_H9_STS,		"snap_sch.h9_status"},
	{SS_H10_STS,	"snap_sch.h10_status"},
	{SS_H11_STS,	"snap_sch.h11_status"},
	{SS_H12_STS,	"snap_sch.h12_status"},
	{SS_H13_STS,	"snap_sch.h13_status"},
	{SS_H14_STS,	"snap_sch.h14_status"},
	{SS_H15_STS,	"snap_sch.h15_status"},
	{SS_H16_STS,	"snap_sch.h16_status"},
	{SS_H17_STS,	"snap_sch.h17_status"},
	{SS_H18_STS,	"snap_sch.h18_status"},
	{SS_H19_STS,	"snap_sch.h19_status"},
	{SS_H20_STS,	"snap_sch.h20_status"},
	{SS_H21_STS,	"snap_sch.h21_status"},
	{SS_H22_STS,	"snap_sch.h22_status"},
	{SS_H23_STS,	"snap_sch.h23_status"},
	{SS_H24_STS,	"snap_sch.h24_status"},
};


#define PARAM_ACT			1
#define PARAM_ID			2
#define PARAM_RESV			3
#define PARAM_WEEK			4
#define PARAM_DAY			5
#define PARAM_HOUR			6
#define PARAM_H1			7
#define PARAM_H2			8
#define PARAM_H3			9
#define PARAM_H4			10
#define PARAM_H5			11
#define PARAM_H6			12
#define PARAM_H7			13
#define PARAM_H8			14
#define PARAM_H9			15
#define PARAM_H10			16
#define PARAM_H11			17
#define PARAM_H12			18
#define PARAM_H13			19
#define PARAM_H14			20
#define PARAM_H15			21
#define PARAM_H16			22
#define PARAM_H17			23
#define PARAM_H18			24
#define PARAM_H19			25
#define PARAM_H20			26
#define PARAM_H21			27
#define PARAM_H22			28
#define PARAM_H23			29
#define PARAM_H24			30
#define PARAM_VN			31
#define PARAM_SN			32
#define NUM_OF_PARAMS		32

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_ID,		"id"},
	{PARAM_RESV,	"rv"},
	{PARAM_WEEK,	"wk"},
	{PARAM_DAY,		"day"},
	{PARAM_HOUR,	"hr"},
	{PARAM_H1,		"h1"},
	{PARAM_H2,		"h2"},
	{PARAM_H3,		"h3"},
	{PARAM_H4,		"h4"},
	{PARAM_H5,		"h5"},
	{PARAM_H6,		"h6"},
	{PARAM_H7,		"h7"},
	{PARAM_H8,		"h8"},
	{PARAM_H9,		"h9"},
	{PARAM_H10,		"h10"},
	{PARAM_H11,		"h11"},
	{PARAM_H12,		"h12"},
	{PARAM_H13,		"h13"},
	{PARAM_H14,		"h14"},
	{PARAM_H15,		"h15"},
	{PARAM_H16,		"h16"},
	{PARAM_H17,		"h17"},
	{PARAM_H18,		"h18"},
	{PARAM_H19,		"h19"},
	{PARAM_H20,		"h20"},
	{PARAM_H21,		"h21"},
	{PARAM_H22,		"h22"},
	{PARAM_H23,		"h23"},
	{PARAM_H24,		"h24"},
	{PARAM_VN,		"vn"},
	{PARAM_SN,		"sn"},
};


/* HERE ARE VARIABLES FOR VOLUME INFORMATION */
static int64 inId;
static char inAct[10];
static char inVolName[VOL_MAX_NAME+1];
static char inSnapName[FS_SNAPNAME_MAX+1];
static int inResv;
static int inWeek, inDay, inHour;
static int inHrs[FS_SNAP_SCHED_MAX_HOURS];
static uint64 totalBlks;
static uint64 blksUsed;
static int dataCount = 0;
static int dataCurr = 0;
static fs_SnapUsage stData;
static fs_SnapUsage *snapshotList;


/***********************************************************
 *  Main routines to handle Add/Delete/Schedule Snapshot actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType)
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_ID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_RESV:
			sscanf(argval[argind], "%d", &inResv);
			break;
		case PARAM_WEEK:
			sscanf(argval[argind], "%d", &inWeek);
			break;
		case PARAM_DAY:
			sscanf(argval[argind], "%d", &inDay);
			break;
		case PARAM_HOUR:
			sscanf(argval[argind], "%d", &inHour);
			break;
		case PARAM_H1:
		case PARAM_H2:
		case PARAM_H3:
		case PARAM_H4:
		case PARAM_H5:
		case PARAM_H6:
		case PARAM_H7:
		case PARAM_H8:
		case PARAM_H9:
		case PARAM_H10:
		case PARAM_H11:
		case PARAM_H12:
		case PARAM_H13:
		case PARAM_H14:
		case PARAM_H15:
		case PARAM_H16:
		case PARAM_H17:
		case PARAM_H18:
		case PARAM_H19:
		case PARAM_H20:
		case PARAM_H21:
		case PARAM_H22:
		case PARAM_H23:
		case PARAM_H24:
			if (!strcmp(argval[argind], "on"))
				inHrs[paramType-PARAM_H1] = 1;
			else
				inHrs[paramType-PARAM_H1] = 0;
			// sscanf(argval[argind], "%d", &inHrs[(paramType-PARAM_H1)]);
			break;
		case PARAM_VN:
			strcpy(inVolName, argval[argind]);
			break;
		case PARAM_SN:
			strcpy(inSnapName, argval[argind]);
			break;
		}
	}

	return 0;
}

static int DoSnapshotSchedule(char *newfname)
{
    int						stat, res, i, count;
    fs_MsgSnapSchedReq	    req;
    fs_MsgSnapSchedResp	    rsp;
	struct timeval			timeout = {5, 0};

	res = 1;

    //Fill the request.
    memset(&req, 0, sizeof(req));
    req.hdr.requestType = FS_REQ_SNAP_SCHED;
    req.hdr.len = sizeof(fs_MsgSnapSchedReq);
	req.volId.whole_thing = inId;
	req.sched.maxWeekly = inWeek;
	req.sched.maxDaily  = inDay;
	req.sched.maxHourly = inHour;

	count = 0;
	for (i = 1; i <= FS_SNAP_SCHED_MAX_HOURS; i++)
	{
		if (inHrs[i-1])
			req.sched.hours[count++] = i;
	}

    // Send message to FS.
    stat = sendAgileMsg(-1, &req, req.hdr.len,
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE, &timeout,
                        &rsp, sizeof(fs_MsgSnapSchedResp));

    // Check response from FS.
	if (stat < 0)
		res = -1;
	else if (!stat)
		res = -1;
    if (rsp.hdr.requestType != FS_REQ_SNAP_SCHED)
		res = -1;
	if (rsp.hdr.fsStatus != FS_ERR_NONE)
		res = -1;

	if (res <= 0)
	{
		strcpy(newfname, "error_snapsch.html");
		return -1;
	}
#if 0
	else
		strcpy(newfname, "result_snapsch.html");
#endif

	return 1;
}

static int DoSnapshotAdd()
{
    int						stat;
	char					reqbuff[512];
    fs_MsgSnapCreateReq	    *req;
    fs_MsgSnapCreateResp    rsp;
	struct timeval			timeout = {5, 0};
 
    //Fill the request.
    memset(reqbuff, 0, sizeof(reqbuff));
	req = (fs_MsgSnapCreateReq *) reqbuff;
    req->hdr.requestType = FS_REQ_SNAP_CREATE;
    req->hdr.len = sizeof(fs_MsgSnapCreateReq) + strlen(inSnapName);
	req->volId.whole_thing = inId;
	req->nameLen = strlen(inSnapName);
	strcpy(req->snapDirName, inSnapName);

    // Send message to FS.
    stat = sendAgileMsg(-1, req, req->hdr.len,
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE, &timeout,
                        &rsp, sizeof(fs_MsgSnapCreateResp));

    // Check response from FS.
	if (stat < 0)
		return -1;
	else if (!stat)
		return -1;

    if (rsp.hdr.requestType != FS_REQ_SNAP_CREATE)
		return -1;
	if (rsp.hdr.fsStatus != FS_ERR_NONE)
		return -1;

	return 1;
}

static int DoSnapshotDel()
{
    int						stat;
	char					reqbuff[512];
    fs_MsgSnapRemoveReq	    *req;
    fs_MsgSnapRemoveResp    rsp;
	struct timeval			timeout = {5, 0};
 
    //Fill the request.
    memset(reqbuff, 0, sizeof(reqbuff));
	req = (fs_MsgSnapRemoveReq *) reqbuff;
    req->hdr.requestType = FS_REQ_SNAP_REMOVE;
    req->hdr.len = sizeof(fs_MsgSnapRemoveReq) + strlen(inSnapName);
	req->volId.whole_thing = inId;
	req->nameLen = strlen(inSnapName);
	strcpy(req->snapDirName, inSnapName);

    // Send message to FS.
    stat = sendAgileMsg(-1, req, req->hdr.len,
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE, &timeout,
                        &rsp, sizeof(fs_MsgSnapRemoveResp));

    // Check response from FS.
	if (stat < 0)
		return -1;
	else if (!stat)
		return -1;
    if (rsp.hdr.requestType != FS_REQ_SNAP_REMOVE)
		return -1;
	if (rsp.hdr.fsStatus != FS_ERR_NONE)
		return -1;

	return 1;
}

static int HandleSnapshotAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);

	if (!strcmp(inAct, "sch"))
	{
		res = DoSnapshotSchedule(newfname);
	}
	else if (!strcmp(inAct, "add"))
	{
		res = DoSnapshotAdd();
		if (res <= 0)
			strcpy(newfname, "error_snapadd.html");
	}
	else if (!strcmp(inAct, "del"))
	{
		res = DoSnapshotDel();
		if (res <= 0)
			strcpy(newfname, "error_snapdel.html");
	}
	return res;
}


/***********************************************************
 * routines to get snapshot information thru anpssc
 **********************************************************/

static void GetSnapshotSchedule()
{
    int						stat, i;
    fs_MsgSnapShowReq	    req;
    fs_MsgSnapShowResp	    rsp;
	struct timeval			timeout = {5, 0};

    //Fill the request.
    memset(&req, 0, sizeof(fs_MsgSnapShowReq));
    req.hdr.requestType = FS_REQ_SNAP_SHOW;
    req.hdr.len = sizeof(fs_MsgSnapShowReq);
	req.type = FS_SNAP_SHOW_SCHED;
	req.volId.whole_thing = inId;

    // Send message to FS.
    stat = sendAgileMsg(-1, &req, req.hdr.len,
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE, &timeout,
                        &rsp, sizeof(fs_MsgSnapShowResp));

    // Check response from FS.
	if (stat < 0)
		return;
	else if (!stat)
		return;

    if (rsp.hdr.requestType != FS_REQ_SNAP_SHOW)
		return;
	if (rsp.hdr.fsStatus != FS_ERR_NONE)
		return;

	inWeek = rsp.sched.maxWeekly;
	inDay  = rsp.sched.maxDaily;
	inHour = rsp.sched.maxHourly;

	memset(inHrs, 0, sizeof(inHrs));
	for (i = 0; i < FS_SNAP_SCHED_MAX_HOURS; i++)
	{
		if ((rsp.sched.hours[i] <= FS_SNAP_SCHED_MAX_HOURS) &&
			(rsp.sched.hours[i] > 0))
		{
			inHrs[rsp.sched.hours[i]-1] = 1;
		}
	}
}

static int getTableRowCount(void)
{
    int						stat, i;
    fs_MsgSnapShowReq	    req;
    fs_MsgSnapShowResp	    rspname;
	struct timeval			timeout = {60, 0};
 
    //Fill the request.
    memset(&req, 0, sizeof(fs_MsgSnapShowReq));
    req.hdr.requestType = FS_REQ_SNAP_SHOW;
    req.hdr.len = sizeof(fs_MsgSnapShowReq);
	req.type = FS_SNAP_SHOW_USAGE;
	req.volId.whole_thing = inId;

    // Send message to FS.
    stat = sendAgileMsg(-1, &req, req.hdr.len,
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE, &timeout,
                        &rspname, sizeof(fs_MsgSnapShowResp));

	if (stat <= 0)
		return -1;		/* error sending or timeout */

	if (rspname.hdr.fsStatus != NFX_OK)
		return -1;

	/* lc: do we care about internal snapshots ?? */
	snapshotList = malloc(rspname.nUsers * sizeof(fs_SnapUsage));
	if (snapshotList == NULL)
		return -1;
	memset(snapshotList, 0, rspname.nUsers * sizeof(fs_SnapUsage));

	/* now get the snapshot usage for each snapshot... */
	for (i = 0; i < rspname.nUsers; i++)
		memcpy(snapshotList+i, &rspname.user[i], sizeof(fs_SnapUsage));

	totalBlks = rspname.totalBlks;
	blksUsed = rspname.blksUsed;

	return rspname.nUsers;
}

static int getTableRowData(int *index, fs_SnapUsage *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;

	memcpy(data, &snapshotList[(*index)-1], sizeof(fs_SnapUsage));
	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType, priv;
	uint64 size;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType)
	{
	case SS_VOLID:
		sprintf(value, "%lld", inId);
		break;
	case SS_VOLNAME:
		strcpy(value, inVolName);
		break;
	case SS_RESV:
		strcpy(value, "0");
		break;
	case SS_WEEK:
		sprintf(value, "%d", inWeek);
		break;
	case SS_DAILY:
		sprintf(value, "%d", inDay);
		break;
	case SS_HOUR:
		sprintf(value, "%d", inHour);
		break;
	case SS_NAME:
		if ((dataCurr <= 0) || (dataCurr > dataCount))
			return -1;
		strcpy(value, stData.name);
		break;
	case SS_USAGE:
		if ((dataCurr <= 0) || (dataCurr > dataCount))
			return -1;
		if (stData.size + totalBlks)
			size = (100 * stData.size) / (stData.size + totalBlks);
		else
			size = 0;
		sprintf(value, "%lld%%", size);
		break;
	case SS_BLOCK:
		if ((dataCurr <= 0) || (dataCurr > dataCount))
			return -1;
		sprintf(value, "%lld", stData.size);
		break;
	case SS_TIME:
		if ((dataCurr <= 0) || (dataCurr > dataCount))
			return -1;
		sprintf(value, "%s", ctime((time_t *) &stData.ctime.secs));
		// sprintf(value, "%d secs, %d u secs", stData.ctime.secs, stData.ctime.usecs);
		break;
	case SS_H1_STS:
	case SS_H2_STS:
	case SS_H3_STS:
	case SS_H4_STS:
	case SS_H5_STS:
	case SS_H6_STS:
	case SS_H7_STS:
	case SS_H8_STS:
	case SS_H9_STS:
	case SS_H10_STS:
	case SS_H11_STS:
	case SS_H12_STS:
	case SS_H13_STS:
	case SS_H14_STS:
	case SS_H15_STS:
	case SS_H16_STS:
	case SS_H17_STS:
	case SS_H18_STS:
	case SS_H19_STS:
	case SS_H20_STS:
	case SS_H21_STS:
	case SS_H22_STS:
	case SS_H23_STS:
	case SS_H24_STS:
		priv = GetUserPrivilege();
		if (inHrs[tagType - SS_H1_STS] != 0)
		{
			if (priv > PRIV_VIEW)
				strcpy(value, "checked");
			else
				strcpy(value, "<b>x</b>");
		}
		else
		{
			if (priv > PRIV_VIEW)
				strcpy(value, "");
			else
				strcpy(value, "&nbsp;");
		}
		break;
	default:
		return -1;
	}

	if (value && !value[0])
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSshotMain(void *r, int count, char **argname, char **argval)
{
	int res = 1;
	char newfname[128];

	inResv = 0;
	inWeek = 0;
	inDay = 0;
	inHour = 0;
	totalBlks = 0;
	blksUsed = 0;
	memset(inAct, 0, sizeof(inAct));
	memset(inVolName,0, sizeof(inVolName));
	memset(inSnapName, 0, sizeof(inSnapName));
	memset(inHrs, 0, sizeof(inHrs));
	memset(newfname, 0, sizeof(newfname));

	/* Handle Add/Delete/Schedule actions */
	if (count > 0)
		res = HandleSnapshotAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize snapshot information variables */
	dataCount = 0;
	dataCurr = 0;
	snapshotList = NULL;
	if (res > 0)
		GetSnapshotSchedule();

	agile_scan_and_send(r, actRptTagFunc);

	if (snapshotList != NULL)
	{
		free(snapshotList);
		snapshotList = NULL;
	}
	return 1;
}
