/*  File Name  :    agilesyswz2.c 
 *
 *  Description:    HTTPD action functions for System Setup Wizard step 2.
 *
 *  Created    :    4/8/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <anpssc-api.h>
#include "../agent/httpdagent.h"

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define SYS_AUTOSUPPON		1
#define SYS_AUTOSUPPOFF		2
#define SYS_TOEMAIL			3
#define SYS_NOTEEMAIL		4
#define NUM_OF_TAGS			4

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SYS_AUTOSUPPON,	"autosupporton"},
	{SYS_AUTOSUPPOFF,	"autosupportoff"},
	{SYS_TOEMAIL,		"TOemail"},
	{SYS_NOTEEMAIL,		"NOTETOemail"},
};


#define PARAM_ACT			1
#define PARAM_HNAME			2
#define PARAM_ADMIN			3
#define PARAM_PWD			4
#define PARAM_ADMPRVG		5
#define PARAM_TIMEZ			6
#define PARAM_LANG			7
#define PARAM_MON			8
#define PARAM_DAY			9
#define PARAM_YEAR			10
#define PARAM_HOUR			11
#define PARAM_MIN			12
#define NUM_OF_PARAMS		12

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_HNAME,	"hname"},
	{PARAM_ADMIN,	"admn"},
	{PARAM_PWD,		"pwd1"},
	{PARAM_ADMPRVG,	"prvg"},
	{PARAM_TIMEZ,	"tz"},
	{PARAM_LANG,	"lang"},
	{PARAM_MON,		"mon"},
	{PARAM_DAY,		"day"},
	{PARAM_YEAR,	"year"},
	{PARAM_HOUR,	"hour"},
	{PARAM_MIN,		"min"},
};


/* HERE ARE VARIABLES FOR SYSTEM INFORMATION */
typedef struct _system_config_t
{
    int			autosupport;
    char		TOEMail[128];
    char		NOTETOEMail[128];
} system_config_t;


static char inAct[10];
static char inHname[128];
static char inAdmin[128];
static char inPwd[128];
static int inAdminPrvg;
static char inTimeZone[256];
static char inLanguage[64];
static int inMon, inDay, inYear, inHour, inMin;
static int	res;
static system_config_t stData;


/***********************************************************
 * routines to retrieve system information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	memset(inAct, 0, 10);
	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_HNAME:
			strcpy(inHname, argval[argind]);
			break;
		case PARAM_ADMIN:
			strcpy(inAdmin, argval[argind]);
			break;
		case PARAM_PWD:
			strcpy(inPwd, argval[argind]);
			break;
		case PARAM_ADMPRVG:
			sscanf(argval[argind], "%d", &inAdminPrvg);
			break;
		case PARAM_TIMEZ:
			strcpy(inTimeZone, argval[argind]);
			break;
		case PARAM_LANG:
			strcpy(inLanguage, argval[argind]);
			break;
		case PARAM_MON:
			sscanf(argval[argind], "%d", &inMon);
			break;
		case PARAM_DAY:
			sscanf(argval[argind], "%d", &inDay);
			break;
		case PARAM_YEAR:
			sscanf(argval[argind], "%d", &inYear);
			break;
		case PARAM_HOUR:
			sscanf(argval[argind], "%d", &inHour);
			break;
		case PARAM_MIN:
			sscanf(argval[argind], "%d", &inMin);
			break;
		}
	}

	return 0;
}

static void GetSupportConfigs(char* buf)
{
	int from = 0;
	char tmpbuf[16];

	from = GetDelimiteredParam(from, buf, tmpbuf);
	if (!strcmp(tmpbuf, "on"))
		stData.autosupport = 1;
	else
		stData.autosupport = 0;
	from = GetDelimiteredParam(from, buf, stData.TOEMail);
	from = GetDelimiteredParam(from, buf, stData.NOTETOEMail);
}

static void GetSystemConfig(void *r, int from)
{
	int rc;
	char reqbuf[256];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

	res = -1;

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Get Agile contact and local e-mail info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	cfgtype = (haSystemConfigType_t *) req->data;

	cfgtype->from = from;
	cfgtype->step = 2;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if ((rsp->rspstatus != HARSP_OK) || (rsp->datalen != sizeof(haSystemConfig_t)))
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetSupportConfigs(config->configs);

	res = 1;
}

static void SaveSystemConfig(void *r)
{
	int rc;
	char reqbuf[2048];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Save data from prev page - admin and pwd (save to temp)
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	config = (haSystemConfig_t *) req->data;

    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);

	config->type.from = 0;
	config->type.step = 1;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s,%d,%s,%s,%d,%d,%d,%d,%d",
			inHname, inAdmin, inPwd, inAdminPrvg,
			inTimeZone, inLanguage,
			inMon, inDay, inYear, inHour, inMin);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));

	cfgtype = (haSystemConfigType_t *) rsp->data;
}

static void GetAndSetSystemData(void *r)
{
	int from;

	// prev action, get data from temp file that we just created
	if (!strcmp(inAct, "prev"))
		from = 0;		
	// next action, get data from perminate file, also save data for prev page
	// SY: changed to always get data from temp file
	else
	{
		// from = 1;
		from = 0;	// always get from temp
		SaveSystemConfig(r);
	}

	// Get System Configuration
	GetSystemConfig(r, from);
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (res <= 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case SYS_AUTOSUPPON:
		if (stData.autosupport)
			strcpy(value, " checked");
		else
			value[0] = '\0';
		break;
	case SYS_AUTOSUPPOFF:
		if (!stData.autosupport)
			strcpy(value, " checked");
		else
			value[0] = '\0';
		break;
	case SYS_TOEMAIL:
		strcpy(value, stData.TOEMail);
		break;
	case SYS_NOTEEMAIL:
		strcpy(value, stData.NOTETOEMail);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSysWizard2(void *r, int count, char **argname, char **argval)
{
	/* initialize system information variables */
	getInputParams(count, argname, argval);
	GetAndSetSystemData(r);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
