/*  File Name  :    agilesyswz5.c 
 *
 *  Description:    HTTPD action functions for System Setup Wizard step 5 - confirmation.
 *
 *  Created    :    4/9/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <anpssc-api.h>
#include "../agent/httpdagent.h"
#include <nfxnis-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define SYS_HOST			1
#define SYS_ADMIN			2
#define SYS_PWDHIDDEN		3
#define SYS_ADMPRVG			4
#define SYS_TZONE			5
#define SYS_CURRTIME		6
#define SYS_LANG			7
#define SYS_AUTOSUPP		8
#define SYS_TOEMAIL			9
#define SYS_NOTEEMAIL		10
#define SYS_DNSDOMAIN		11
#define SYS_DNSSVRLIST		12
#define SYS_NISDOMAIN		13
#define SYS_NISSVR			14
#define SYS_MIFINFO			15
#define SYS_CIFLIST			16
#define SYS_DEFROUTE		17
#define NUM_OF_TAGS			17

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SYS_HOST,		"system.hostname"},
	{SYS_ADMIN,		"system.adminname"},
	{SYS_PWDHIDDEN,	"system.adminpwd.hidden"},
	{SYS_ADMPRVG,	"system.adminprvg"},
	{SYS_TZONE,		"system.timezone"},
	{SYS_CURRTIME,	"system.currtime"},
	{SYS_LANG,		"system.language"},
	{SYS_AUTOSUPP,	"system.autosupport"},
	{SYS_TOEMAIL,	"system.toemail"},
	{SYS_NOTEEMAIL,	"system.noteemail"},
	{SYS_DNSDOMAIN,	"system.dnsdomain"},
	{SYS_DNSSVRLIST,"system.dnssvr.list"},
	{SYS_NISDOMAIN,	"system.nisdomain"},
	{SYS_NISSVR,	"system.nissvr"},
	{SYS_MIFINFO,	"system.mifipmask"},
	{SYS_CIFLIST,	"system.cifipmask.list"},
	{SYS_DEFROUTE,	"system.defroute"},
};


#define PARAM_ACT			1
#define PARAM_MANIF			2
#define PARAM_MANIP			3
#define PARAM_MANMASK		4
#define PARAM_CL1IF			5
#define PARAM_CL2IF			6
#define PARAM_CL3IF			7
#define PARAM_CL4IF			8
#define PARAM_CL1IP			9
#define PARAM_CL2IP			10
#define PARAM_CL3IP			11
#define PARAM_CL4IP			12
#define PARAM_CL1MASK		13
#define PARAM_CL2MASK		14
#define PARAM_CL3MASK		15
#define PARAM_CL4MASK		16
#define PARAM_DEFRT			17
#define NUM_OF_PARAMS		17

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_MANIF,	"mif"},
	{PARAM_MANIP,	"mip"},
	{PARAM_MANMASK,	"mmask"},
	{PARAM_CL1IF,	"cif1"},
	{PARAM_CL1IP,	"ip1"},
	{PARAM_CL1MASK,	"mask1"},
	{PARAM_CL2IF,	"cif2"},
	{PARAM_CL2IP,	"ip2"},
	{PARAM_CL2MASK,	"mask2"},
	{PARAM_CL3IF,	"cif3"},
	{PARAM_CL3IP,	"ip3"},
	{PARAM_CL3MASK,	"mask3"},
	{PARAM_CL4IF,	"cif4"},
	{PARAM_CL4IP,	"ip4"},
	{PARAM_CL4MASK,	"mask4"},
	{PARAM_DEFRT,	"drt"},
};

static char *groupNames[] =
{
	"View", 
	"Add", 
	"Change", 
	"Full", 
	"Diags"
};

/* HERE ARE VARIABLES FOR SYSTEM INFORMATION */
typedef struct _system_config_t
{
    char		name[128];
    char		adminName[128];
    char		adminPwd[128];
    int			adminPrvg;
	char		timeZone[256];
	char		language[64];
	int			tm_mon;
	int			tm_mday;
	int			tm_year;
	int			tm_hour;
	int			tm_min;
    int			autoSupport;
    char		toEMail[128];
    char		noteEMail[128];
    char		dnsDomain[128];
    char		dnsServer1[16];
    char		dnsServer2[16];
    char		dnsServer3[16];
    char		nisDomain[128];
    char		nisServer[16];
    char		mif[20];
    char		mifip[16];
    char		mifmask[16];
    char		cif[4][20];
    char		cifip[4][16];
    char		cifmask[4][16];
    char		defroute[16];
} system_config_t;


static char inAct[10];
static char inMif[20];
static char inMifip[16];
static char inMifmask[16];
static char inCif[4][20];
static char inCifip[4][16];
static char inCifmask[4][16];
static char inDefroute[16];
static int	res;
static system_config_t stData;


/***********************************************************
 * routines to retrieve system information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	memset(inAct, 0, 10);
	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_MANIF:
			strcpy(inMif, argval[argind]);
			break;
		case PARAM_MANIP:
			strcpy(inMifip, argval[argind]);
			break;
		case PARAM_MANMASK:
			strcpy(inMifmask, argval[argind]);
			break;
		case PARAM_CL1IF:
		case PARAM_CL2IF:
		case PARAM_CL3IF:
		case PARAM_CL4IF:
			strcpy(inCif[paramType-PARAM_CL1IF], argval[argind]);
			break;
		case PARAM_CL1IP:
		case PARAM_CL2IP:
		case PARAM_CL3IP:
		case PARAM_CL4IP:
			strcpy(inCifip[paramType-PARAM_CL1IP], argval[argind]);
			break;
		case PARAM_CL1MASK:
		case PARAM_CL2MASK:
		case PARAM_CL3MASK:
		case PARAM_CL4MASK:
			strcpy(inCifmask[paramType-PARAM_CL1MASK], argval[argind]);
			break;
		case PARAM_DEFRT:
			strcpy(inDefroute, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetAdminConfigs(void* req, char* buf)
{
	char tmpbuf[64];
	int from = 0;

	// Get System Name
	// GetSystemHostInfo(req, tmpbuf, stData.name);

	from = GetDelimiteredParam(from, buf, stData.name);
	from = GetDelimiteredParam(from, buf, stData.adminName);
	from = GetDelimiteredParam(from, buf, stData.adminPwd);
	from = GetDelimiteredParam(from, buf, tmpbuf);			/* adm privilege */
	stData.adminPrvg = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, stData.timeZone);
	from = GetDelimiteredParam(from, buf, stData.language);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* month */
	stData.tm_mon = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* day */
	stData.tm_mday = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* year */
	stData.tm_year = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* hour */
	stData.tm_hour = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* minute */
	stData.tm_min = atoi(tmpbuf);
}

static void GetSupportConfigs(char* buf)
{
	int from = 0;
	char tmpbuf[16];

	from = GetDelimiteredParam(from, buf, tmpbuf);
	if (!strcmp(tmpbuf, "on"))
		stData.autoSupport = 1;
	else
		stData.autoSupport = 0;
	from = GetDelimiteredParam(from, buf, stData.toEMail);
	from = GetDelimiteredParam(from, buf, stData.noteEMail);
}

static void GetDnsNisConfigs(char* buf)
{
	int from = 0;

	from = GetDelimiteredParam(from, buf, stData.dnsDomain);
	from = GetDelimiteredParam(from, buf, stData.dnsServer1);
	from = GetDelimiteredParam(from, buf, stData.dnsServer2);
	from = GetDelimiteredParam(from, buf, stData.dnsServer3);
	from = GetDelimiteredParam(from, buf, stData.nisDomain);
	from = GetDelimiteredParam(from, buf, stData.nisServer);
}

static void GetIpIfConfigs(char* buf)
{
	int i, from = 0;

	from = GetDelimiteredParam(from, buf, stData.mif);
	from = GetDelimiteredParam(from, buf, stData.mifip);
	from = GetDelimiteredParam(from, buf, stData.mifmask);
	for (i = 0; i < 4; i++)
	{
		from = GetDelimiteredParam(from, buf, stData.cif[i]);
		from = GetDelimiteredParam(from, buf, stData.cifip[i]);
		from = GetDelimiteredParam(from, buf, stData.cifmask[i]);
	}
	from = GetDelimiteredParam(from, buf, stData.defroute);
}

static void GetSystemConfig(void* r, int from)
{
	int rc;
	char reqbuf[2048];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

	res = -1;

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Get admin info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	cfgtype = (haSystemConfigType_t *) req->data;

	cfgtype->from = from;
	cfgtype->step = 1;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if (rsp->rspstatus != HARSP_OK)
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetAdminConfigs(r, config->configs);

	// Get contact & email info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	cfgtype->from = from;
	cfgtype->step = 2;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if (rsp->rspstatus != HARSP_OK)
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetSupportConfigs(config->configs);

	// Get dns, nis info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	cfgtype->from = from;
	cfgtype->step = 3;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if (rsp->rspstatus != HARSP_OK)
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetDnsNisConfigs(config->configs);

	// Get if info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	cfgtype->from = from;
	cfgtype->step = 4;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if (rsp->rspstatus != HARSP_OK)
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetIpIfConfigs(config->configs);

	res = 1;
}

static void SaveSystemConfig(void *r)
{
	int rc, i;
	char reqbuf[2048];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Save data from prev page - ip info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	config = (haSystemConfig_t *) req->data;

    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);
	config->type.from = 0;
	config->type.step = 4;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s,", inMif, inMifip, inMifmask);
	for (i = 0; i < 4; i ++)
	{
		strcat(config->configs, inCif[i]);
		strcat(config->configs, ",");
		strcat(config->configs, inCifip[i]);
		strcat(config->configs, ",");
		strcat(config->configs, inCifmask[i]);
		strcat(config->configs, ",");
	}
	strcat(config->configs, inDefroute);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));

	cfgtype = (haSystemConfigType_t *) rsp->data;
}

static int SaveConfig(void *r, char* newfname)
{
	int rc, i;
	char reqbuf[2048];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {180, 0};

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Save data to perm file - admin info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	config = (haSystemConfig_t *) req->data;

    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);
	config->type.from = 1;
	config->type.step = 1;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s,%d,%s,%s,%d,%d,%d,%d,%d",
			stData.name,
			stData.adminName, stData.adminPwd, stData.adminPrvg,
			stData.timeZone, stData.language,
			stData.tm_mon, stData.tm_mday, stData.tm_year,
			stData.tm_hour, stData.tm_min);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0 || rsp->rspstatus != HARSP_OK)
	{
		fprintf(stderr, "System wizard: unable to save admin info (rc=%d).\n", rc);
		return 0;
	}

	// Save Time Zone and Language
	//..........

	// Save data to perm file - support info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);
	config->type.from = 1;
	config->type.step = 2;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%d,%s,%s",
			stData.autoSupport, stData.toEMail, stData.noteEMail);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if ((rc <= 0) || (rsp->rspstatus != HARSP_OK))
	{
		fprintf(stderr,
			"System wizard: unable to save timezone and language info (rc=%d).\n",
			rc);
		return 0;
	}

	// Save data to perm file - name services
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);
	config->type.from = 1;
	config->type.step = 3;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s,%s,%s,%s",
				stData.dnsDomain, stData.dnsServer1,
				stData.dnsServer2, stData.dnsServer3,
				stData.nisDomain, stData.nisServer);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
						"httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, rspbuf, sizeof(rspbuf));
	if ((rc <= 0) || (rsp->rspstatus != HARSP_OK))
	{
		fprintf(stderr,
			"System wizard: unable to save name service info (rc=%d).\n",
			rc);
		return 0;
	}

	// Save data to perm file - ip info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);
	config->type.from = 1;
	config->type.step = 4;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s,",
					stData.mif, stData.mifip, stData.mifmask);
	for (i = 0; i < 4; i ++)
	{
		strcat(config->configs, stData.cif[i]);
		strcat(config->configs, ",");
		strcat(config->configs, stData.cifip[i]);
		strcat(config->configs, ",");
		strcat(config->configs, stData.cifmask[i]);
		strcat(config->configs, ",");
	}
	strcat(config->configs, stData.defroute);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
						"httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, rspbuf, sizeof(rspbuf));
	if ((rc <= 0) || (rsp->rspstatus != HARSP_OK))
	{
		fprintf(stderr,
			"System wizard: unable to save ip info (rc=%d).\n",
			rc);
		return 0;
	}

	cfgtype = (haSystemConfigType_t *) rsp->data;

	// Set default route
	//res = ipmd_EnableIntf(ifname, NULL, 0);
	//	if (res == NFX_ERR)

	return 1;
}

static void GetAndSetSystemData(void* r, char* newfname)
{
	int from, res;
	memset(&stData, 0, sizeof(system_config_t));

	// Save Action - Save System Config to Perm file
	if (!strcmp(inAct, "save"))
	{
		// Get System Configuration from temp file
		from = 0;
		GetSystemConfig(r, from);

		// Save System Configuration to perm file
		res = SaveConfig(r, newfname);
		if (res <= 0)
			strcpy(newfname, "error_syswzsave.html");
		else
			strcpy(newfname, "SysSetWizSaveRes.html");
	}
	// Next Action
	else
	{
		// Save prev page to temp file
		SaveSystemConfig(r);

		// Get System Configuration from temp file
		from = 0;
		GetSystemConfig(r, from);
	}
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	char tempbuf[128];
	
	if (res <= 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case SYS_HOST:
		strcpy(value, stData.name);
		break;
	case SYS_ADMIN:
		strcpy(value, stData.adminName);
		break;
	case SYS_PWDHIDDEN:
		memset(tempbuf, '*', 128);
		tempbuf[strlen(stData.adminPwd)] = 0;
		strcpy(value, tempbuf);
		break;
	case SYS_ADMPRVG:
		strcpy(value, groupNames[stData.adminPrvg]);
		break;
	case SYS_TZONE:
		strcpy(value, stData.timeZone);
		break;
	case SYS_CURRTIME:
		sprintf(value, "%02d/%02d/%04d, %02d:%02d",
					stData.tm_mon, stData.tm_mday, stData.tm_year,
					stData.tm_hour, stData.tm_min);
		break;
	case SYS_LANG:
		strcpy(value, stData.language);
		break;
	case SYS_AUTOSUPP:
		if (stData.autoSupport)
			strcpy(value, "yes");
		else
			strcpy(value, "no");
		break;
	case SYS_TOEMAIL:
		strcpy(value, stData.toEMail);
		break;
	case SYS_NOTEEMAIL:
		strcpy(value, stData.noteEMail);
		break;
	case SYS_DNSDOMAIN:
		strcpy(value, stData.dnsDomain);
		break;
	case SYS_DNSSVRLIST:
		strcpy(value, stData.dnsServer1);
		if (strlen(stData.dnsServer2) > 0)
		{
			strcat(value, "<br>");
			strcat(value, stData.dnsServer2);
		}
		if (strlen(stData.dnsServer3) > 0)
		{
			strcat(value, "<br>");
			strcat(value, stData.dnsServer3);
		}
		break;
	case SYS_NISDOMAIN:
		strcpy(value, stData.nisDomain);
		break;
	case SYS_NISSVR:
		strcpy(value, stData.nisServer);
		break;
	case SYS_MIFINFO:
		if (strcmp(stData.mif, "no"))
		{
			strcpy(value, stData.mif);
			strcat(value, ": ");
			strcat(value, stData.mifip);
			strcat(value, "/");
			strcat(value, stData.mifmask);
		}
		else
			strcpy(value, "");
		break;
	case SYS_CIFLIST:
		strcpy(value, "");
		for (i = 0; i < 4; i++)
		{
			if (strcmp(stData.cif[i], "no"))
			{
				strcat(value, stData.cif[i]);
				strcat(value, ": ");
				strcat(value, stData.cifip[i]);
				strcat(value, "/");
				strcat(value, stData.cifmask[i]);
				strcat(value, "<br>");
			}
		}
		break;
	case SYS_DEFROUTE:
		strcpy(value, stData.defroute);
		break;
	default:
		return -1;
	}

	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSysWizard5(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* initialize system information variables */
	getInputParams(count, argname, argval);
	GetAndSetSystemData(r, newfname);

	prepare_agile_template(r, newfname);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
