/*  File Name  :    agilevoldetail.c 
 *
 *  Description:    HTTPD action functions for detail Volume Page.
 *
 *  Created    :    3/7/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
// #include <sys/types.h>
// #include <unistd.h>
#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <evm_com_defs.h>
#include <ea-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define VOL_VOLNAME			1
#define VOL_STATUS			2
#define VOL_FSYS			3
#define VOL_FSSTATUS		4
#define VOL_USED			5
#define VOL_ALLOC			6
#define VOL_SFQUOTA			7
#define VOL_HRQUOTA			8
#define VOL_MINAG			9
#define VOL_MAKE			10
#define VOL_MODEL			11
#define VOL_RAIDLVL			12
#define VOL_MINLUN			13
#define VOL_MAXLUN			14
#define VOL_NUMSTRP			15
#define VOL_STRPSIZE		16
#define VOL_NUMMIRR			17
#define VOL_LOC				18
#define VOL_NUMLUN			19
#define VOL_SCSIID			20
#define VOL_LUNSIZE			21
#define NUM_OF_TAGS			23

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_VOLNAME,	"ea_fsysDetail_t.volCfg.name"},
	{VOL_STATUS,	"ea_fsysDetail_t.volCfg.onLine"},
	{VOL_FSYS,		"ea_fsysDetail_t.fsStatus"},
	{VOL_FSSTATUS,	"ea_fsysDetail_t.mountStatus"},
	{VOL_USED,		"ea_fsysDetail_t.usedSize"},
	{VOL_ALLOC,		"ea_fsysDetail_t.volCfg.currSize"},
	{VOL_SFQUOTA,	"ea_fsysDetail_t.softQuota"},
	{VOL_HRQUOTA,	"ea_fsysDetail_t.volCfg.maxSize"},
	{VOL_MINAG,		"ea_fsysDetail_t.volCfg.minAutoGrow"},
	{VOL_MAKE,		"ea_fsysDetail_t.volCfg.attrs.vendorInfo.vendor"},
	{VOL_MODEL,		"ea_fsysDetail_t.volCfg.attrs.vendorInfo.model"},
	{VOL_RAIDLVL,	"ea_fsysDetail_t.volCfg.attrs.raidLevel"},
	{VOL_MINLUN,	"ea_fsysDetail_t.volCfg.attrs.minLunSize"},
	{VOL_MAXLUN,	"ea_fsysDetail_t.volCfg.attrs.maxLunSize"},
	{VOL_NUMSTRP,	"ea_fsysDetail_t.volCfg.numStripes"},
	{VOL_STRPSIZE,	"ea_fsysDetail_t.volCfg.stripeSize"},
	{VOL_NUMMIRR,	"ea_fsysDetail_t.volCfg.numMirrors"},
	{VOL_LOC,		"ea_fsysDetail_t.location"},
	{VOL_NUMLUN,	"ea_fsysDetail_t.strg.num"},
	{VOL_SCSIID,	"ea_fsysDetail_t.strg.au.strgId"},
	{VOL_LUNSIZE,	"ea_fsysDetail_t.strg.au.auSize"},
};


#define PARAM_NAME			1
#define NUM_OF_PARAMS		1

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_NAME,	"name"},
};


/* HERE ARE VARIABLES FOR VOLUME INFORMATION */
typedef struct _volume_detail_t
{
    char		name[VOL_MAX_NAME];
    uint32		onLine;
    uint32		fsStatus;
    uint32		mountStatus;
    uint64		usedSize;
    uint64		currSize;
    uint64		softQuota;
    uint64		hardQuota;
    uint64		minAutoGrow;
    char		vendor[EVM_VENDOR_NAME_LEN];
    char		model[EVM_MODEL_NAME_LEN];
    evm_raid_t	raidLevel;
    uint32		minLunSize; // blocks
    uint32		maxLunSize; // blocks
    uint32		numStripes; // Stripes (valid 0, 1+0, 5)
    uint32		stripeSize;
    uint32		numMirrors; // Mirrors (valid 1, 1+0)
	char		location[256];
} volume_detail_t;

static char inName[VOL_MAX_NAME];
static int dataCount;
static int dataCurr;
static volume_detail_t stVolCfg;
static int numLun;
#if 0
static evm_strgConfig_t *lunList;
static evm_strgConfig_t stData;
#endif
static evm_lvStrg_t *lunList;
static evm_lvStrg_t stData;
static int getRes;

static char *volState_str[] = {"Offline", "Online", "Unknown" };
static char *fs_str[] = {"No", "Yes" };
static char *fsState_str[] = {"Unmounted", "Mounted" };
static char *stripe_str[] = {"0", "1+0", "5" };
static char *mirror_str[] = {"1", "1+0" };


/***********************************************************
 *  Main routines to handle input parameters.
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_NAME:
			strcpy(inName, argval[argind]);
			break;
		}
	}

	return 0;
}


/***********************************************************
 * routines to volume information thru anpssc
 **********************************************************/

static void GetVolumeDetailData()
{
    int						stat;
    char					respbuff[4096];
    ea_getDetailFsysReq_t	req;
    ea_getDetailFsysRsp_t	*rsp;
	numLun = -1;
            
    //Fill the request. 
    bzero((void *) &req, sizeof(ea_getDetailFsysReq_t));
    req.hdr.msgType = EA_GET_DETAIL_FSYS_REQ;
    strcpy(req.hdr.node, "\0");
	strcpy(req.fsysId.fsysName, inName);

    // Send message to EA. 
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        NULL,
                        &respbuff, sizeof(respbuff));

    // Check response from EA. 
	if (stat < 0) 
		return;
	else if (!stat)
		return;
    rsp = (ea_getDetailFsysRsp_t *)respbuff;
    if (rsp->hdr.msgType != EA_GET_DETAIL_FSYS_RSP)
		return;
	if (rsp->result != NFX_OK)
		return;

	// Save Volume Detail Data
	strcpy(stVolCfg.name, rsp->fsysDetail.volCfg.name);
    stVolCfg.onLine = rsp->fsysDetail.volCfg.onLine;
    stVolCfg.fsStatus = rsp->fsysDetail.fsState;
    stVolCfg.mountStatus = rsp->fsysDetail.mountStatus;
    stVolCfg.usedSize = rsp->fsysDetail.usedSize;
    stVolCfg.currSize = rsp->fsysDetail.volCfg.currSize;
    stVolCfg.softQuota = rsp->fsysDetail.volCfg.softQuota;
    stVolCfg.hardQuota = rsp->fsysDetail.volCfg.maxSize;
    stVolCfg.minAutoGrow = rsp->fsysDetail.volCfg.minAutoGrow;
    strcpy(stVolCfg.vendor, rsp->fsysDetail.volCfg.attrs.vendorInfo[0].vendor);
    strcpy(stVolCfg.model, rsp->fsysDetail.volCfg.attrs.vendorInfo[0].model);
    stVolCfg.raidLevel  = rsp->fsysDetail.volCfg.attrs.raidLevel;
    stVolCfg.minLunSize = rsp->fsysDetail.volCfg.attrs.minLunSize;
    stVolCfg.maxLunSize = rsp->fsysDetail.volCfg.attrs.maxLunSize;
#ifdef ISABELLE // code to delete
    stVolCfg.numStripes = rsp->fsysDetail.volCfg.numStripes;
    stVolCfg.stripeSize = rsp->fsysDetail.volCfg.stripeSize;
    stVolCfg.numMirrors = rsp->fsysDetail.volCfg.numMirrors;
#endif // ISABELLE
	strcpy(stVolCfg.location, rsp->hdr.node);

	// Get LUN List Table
#if 0
	numLun = rsp->fsysDetail.strg.num;
	lunList = malloc(numLun * sizeof(evm_strgConfig_t));
	if (lunList == NULL)
		numLun = -1;
	memcpy(lunList, rsp->fsysDetail.strg.au, numLun * sizeof(evm_strgConfig_t));
#endif
	numLun = rsp->fsysDetail.nbStrg;
	lunList = malloc(numLun * sizeof(evm_lvStrg_t));
	if (lunList == NULL)
		numLun = -1;
	memcpy(lunList, rsp->fsysDetail.strg, numLun * sizeof(evm_lvStrg_t));

	return;
}

static int getTableRowCount(void)
{
	return numLun;
}

static int getTableRowData(int *index, evm_lvStrg_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (lunList == NULL)
		return -1;

	memcpy(data, &lunList[(*index)-1], sizeof(evm_lvStrg_t));
	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	uint64 size;
	
	if (getRes < 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_VOLNAME:
		strcpy(value, stVolCfg.name);
		break;
	case VOL_STATUS:
		if (stVolCfg.onLine)
			strcpy(value, volState_str[1]);
		else
			strcpy(value, volState_str[0]);
		break;
	case VOL_FSYS:
		strcpy(value, fs_str[stVolCfg.fsStatus]);
		break;
	case VOL_FSSTATUS:
		strcpy(value, fsState_str[stVolCfg.mountStatus]);
		break;
	case VOL_USED:
		size = GetVolumeDispUsedSize(stVolCfg.usedSize);
		sprintf(value, "%lld", size);
		break;
	case VOL_ALLOC:
		sprintf(value, "%lld", GetMBfromBlock(stVolCfg.currSize));
		break;
	case VOL_SFQUOTA:
		if (stVolCfg.softQuota)
			sprintf(value, "%lld%%", stVolCfg.softQuota);
		else
			strcpy(value, "N/A");
		break;
	case VOL_HRQUOTA:
		if (stVolCfg.hardQuota)
			sprintf(value, "%lld", GetMBfromBlock(stVolCfg.hardQuota));
		else
			strcpy(value, "No limit");
		break;
	case VOL_MINAG:
		sprintf(value, "%lld", GetMBfromBlock(stVolCfg.minAutoGrow));
		break;
	case VOL_MAKE:
		strcpy(value, stVolCfg.vendor);
		if (!value[0])
			strcpy(value, "Any");
		break;
	case VOL_MODEL:
		strcpy(value, stVolCfg.model);
		if (!value[0])
			strcpy(value, "Any");
		break;
	case VOL_RAIDLVL:
		GetRaidLevelStr(stVolCfg.raidLevel, value);
		break;
	case VOL_MINLUN:
		if (stVolCfg.minLunSize != EVM_DONT_CARE)
			sprintf(value, "%lld", GetMBfromBlock(stVolCfg.minLunSize));
		else
			strcpy(value, "No minimum");
		break;
	case VOL_MAXLUN:
		if (stVolCfg.maxLunSize != EVM_DONT_CARE)
			sprintf(value, "%lld", GetMBfromBlock(stVolCfg.maxLunSize));
		else
			strcpy(value, "No maximum");
		break;
	case VOL_NUMSTRP:
		strcpy(value, stripe_str[stVolCfg.numStripes]);
		break;
	case VOL_STRPSIZE:
		sprintf(value, "%lld", GetMBfromBlock(stVolCfg.stripeSize));
		break;
	case VOL_NUMMIRR:
		strcpy(value, mirror_str[stVolCfg.numMirrors]);
		break;
	case VOL_LOC:
		strcpy(value, stVolCfg.location);
		break;
	case VOL_NUMLUN:
		sprintf(value, "%d", numLun);
		break;
	case VOL_SCSIID:
		sprintf(value, "%lld", stData.au.strgId);
		break;
	case VOL_LUNSIZE:
		sprintf(value, "%lld", GetMBfromBlock(stData.au.auSize));
		break;
	default:
		return -1;
	}

	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionVolDetail(void *r, int count, char **argname, char **argval)
{
	getInputParams(count, argname, argval);
	prepare_agile_template(r, NULL);

	/* initialize volume information variables */
	dataCount = 0;
	dataCurr = 0;
	lunList = NULL;

	GetVolumeDetailData();

	agile_scan_and_send(r, actRptTagFunc);
	
	if (lunList != NULL)
		free(lunList);

	return 1;
}
