/*  File Name  :    agilevolmod.c
 *
 *  Description:    HTTPD action functions for modify volume.
 *
 *  Created    :    3/6/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <ea-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_NAME			1
#define VOL_HARDQ			2
#define VOL_HARDQ_VAL		3
#define VOL_HARDQ_DEF		4
#define VOL_SOFTQ			5
#define VOL_SOFTQ_VAL		6
#define VOL_SOFTQ_DEF		7
#define VOL_MINAG			8
#define VOL_MINAG_VAL		9
#define VOL_MINAG_DEF		10
#define VOL_HIGHWATER		11
#define NUM_OF_TAGS			11

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_NAME,		"fsysDetail.volCfg.name"},
	{VOL_HARDQ,		"fsysDetail.volCfg.maxSize"},
	{VOL_HARDQ_VAL,	"fsysDetail.volCfg.maxSize.sel"},
	{VOL_HARDQ_DEF,	"fsysDetail.volCfg.maxSize.def"},
	{VOL_SOFTQ,		"fsysDetail.volCfg.softQuota"},
	{VOL_SOFTQ_VAL,	"fsysDetail.volCfg.softQuota.sel"},
	{VOL_SOFTQ_DEF,	"fsysDetail.volCfg.softQuota.def"},
	{VOL_MINAG,		"fsysDetail.volCfg.minAutoGrow"},
	{VOL_MINAG_VAL,	"fsysDetail.volCfg.minAutoGrow.sel"},
	{VOL_MINAG_DEF,	"fsysDetail.volCfg.minAutoGrow.def"},
	{VOL_HIGHWATER,	"fsysDetail.volCfg.highWaterMark"},
};


#define PARAM_NAME			1
#define NUM_OF_PARAMS		1

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_NAME,	"name"},
};


/* HERE ARE VARIABLES FOR VOLUME INFORMATION */
typedef struct _volume_detail_t
{
    char		name[VOL_MAX_NAME];
    uint64		softQuota;
    uint64		hardQuota;
    uint64		minAutoGrow;
    uint64		highWaterMark;
} volume_detail_t;

static volume_detail_t stVolCfg;
static char bufname[VOL_MAX_NAME];
static int getres;


/***********************************************************
 * routines to retrieve volume information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_NAME:
			strcpy(bufname, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetVolumeInfo()
{
    int						stat;
    char					respbuff[4096];
    ea_getDetailFsysReq_t	req;
    ea_getDetailFsysRsp_t	*rsp;
	struct timeval			timeout = {90, 0};
	getres = -1;
            
    //Fill the request. 
    memset((void *) &req, 0, sizeof(ea_getDetailFsysReq_t));
    req.hdr.msgType = EA_GET_DETAIL_FSYS_REQ;
    strcpy(req.hdr.node, "local");
	strcpy(req.fsysId.fsysName, bufname);

    // Send message to EA. 
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, &respbuff, sizeof(respbuff));

    // Check response from EA. 
	if (stat < 0) 
		return;
	else if (!stat)
		return;

    rsp = (ea_getDetailFsysRsp_t *)respbuff;
	if (rsp->result != NFX_OK)
		return;

	// Save Volume Detail Data
	strcpy(stVolCfg.name, rsp->fsysDetail.volCfg.name);
    stVolCfg.softQuota = rsp->fsysDetail.volCfg.softQuota;
    stVolCfg.hardQuota = rsp->fsysDetail.volCfg.maxSize;
    stVolCfg.minAutoGrow = rsp->fsysDetail.volCfg.minAutoGrow;
    stVolCfg.highWaterMark = rsp->fsysDetail.highWaterMark;

	getres = 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	uint64 size;
	
	if (getres < 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_NAME:
		strcpy(value, stVolCfg.name);
		break;
	case VOL_HARDQ:
		if (stVolCfg.hardQuota > 0)
		{
			size = GetMBfromBlock(stVolCfg.hardQuota);
			sprintf(value, "%lld", size);
		}
		else
			strcpy(value, "");
		break;
	case VOL_HARDQ_VAL:
		if (stVolCfg.hardQuota > 0)
			strcpy(value, "checked");
		else
			strcpy(value, "");
		break;
	case VOL_HARDQ_DEF:
		if (stVolCfg.hardQuota == 0)
			strcpy(value, "checked");
		else
			strcpy(value, "");
		break;
	case VOL_SOFTQ:
		sprintf(value, "%lld", stVolCfg.softQuota);
		break;
	case VOL_SOFTQ_DEF:
		if (stVolCfg.softQuota == 0)
			strcpy(value, "checked");
		else
			strcpy(value, "");
		break;
	case VOL_MINAG:
		if (stVolCfg.minAutoGrow > 0)
			sprintf(value, "%lld", GetMBfromBlock(stVolCfg.minAutoGrow));
		else
			strcpy(value, "");
		break;
	case VOL_MINAG_VAL:
		if (stVolCfg.minAutoGrow > 0)
			strcpy(value, "checked");
		else
			strcpy(value, "");
		break;
	case VOL_MINAG_DEF:
		if (stVolCfg.minAutoGrow == 0)
			strcpy(value, "checked");
		else
			strcpy(value, "");
		break;
	case VOL_HIGHWATER:
		sprintf(value, "%lld", stVolCfg.highWaterMark);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionVolModify(void *r, int count, char **argname, char **argval)
{
	getInputParams(count, argname, argval);
	GetVolumeInfo();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
