/* vi:ts=4 
 *-----------------------------------------------------------------
 *
 * Name: tcp_output.c
 *
 * RCS:  $Id: tcp_output.c,v 1.14 2002/10/04 01:29:14 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 08/09/01
 *
 *-----------------------------------------------------------------
 */

/*
 * Copyright (c) 1982, 1986, 1988, 1990, 1993, 1995
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifdef FREE_BSD
#include "opt_tcpdebug.h"

#include <stddef.h>

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>

#include <net/route.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/in_pcb.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>
#define	TCPOUTFLAGS
#include <netinet/tcp_fsm.h>
#include <netinet/tcp_seq.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>
#include <netinet/tcpip.h>
#ifdef TCPDEBUG
#include <netinet/tcp_debug.h>
#endif

#ifdef notyet
extern struct mbuf *m_copypack();
#endif

#else

#include <nfx-incl.h>
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/macros.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/protosw.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/socketvar.h"

#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/route.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/in_pcb.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/ip_var.h"
#include "../sm-bsd-netinet/tcp.h"
#define	TCPOUTFLAGS
#include "../sm-bsd-netinet/tcp_fsm.h"
#include "../sm-bsd-netinet/tcp_seq.h"
#include "../sm-bsd-netinet/tcp_timer.h"
#include "../sm-bsd-netinet/tcp_var.h"
#include "../sm-bsd-netinet/tcpip.h"
#include "../sm-bsd-netinet/tcp_debug.h"
#include "exports/bsdnet-api.h"
#include "exports/tpl-api.h"
#include "../sm-tpl/tpl.h"
#endif

/*
 * Tcp output routine: figure out what should be sent and send it.
 */
#if defined(AG_EXTENSIONS)
/*
 *-----------------------------------------------------------------
 *
 * Name: tcp_output
 *
 * Description: This is wrapper for core tcp function. This function
 *				determines the fast or slow path of output segment.
 *
 * Created by:	Su Wang
 *
 * Date Created: 09/28/01
 *
 *-----------------------------------------------------------------
 */
int
tcp_output(tp)
	 register struct tcpcb *tp;
{
  register struct socket *so = tp->t_inpcb->inp_socket;
  int err = 0;
  
  /* we are to use fast path if */
  /* 1. connection established */
  /* 2. so has fastpath option */
  /* 3. send queue is pkt queue */
  
  if ((so->so_state & SS_FASTPATHDONE)   &&
	  (so->so_snd.sb_flags & SB_PKT)     &&
	  (so->so_handle))
	{
#if defined(NFX_SMP)
	  tpl_xmtPktCore(so->so_handle, 0);
#else
	  tpl_xmtPkt(so->so_handle, 0);
#endif /* NFX_SMP */
	}
  else
	{							/* slowpath */
	  err = tcp_outputC(tp, 0, 0,0, 0);
	  if (err < 0)
		bsd_log(BSD_ERR_TCP_TX, "tcp_output: bad return code(%d), slowpath\n",
				err); 
	}

  return err;
}

#define GETQSIZE(sb) \
(((sb)->sb_flags & SB_PKT)?(sb)->sb_pktcc:(sb)->sb_cc) \


int 
/*
 *-----------------------------------------------------------------
 *
 * Name: tcp_outputC
 *
 * Description: slowpath: called by BSD code to establish a tcp
 *				socket connection, as well as all tcp operations
 *				if socket is not fastpath socket. Also, IP reassem
 *				output.
 *				return: 0 OK, or bsd err num. 

 *				fastpath: called by tpl_xmt as well as BSD code
 *				once tcp connection is established. Do not process
 *				ip reassem output.
 *				return: BSD_FAST_ERR, err occured, del pkt
 *						BSD_FAST_QUEUED, in send queue, hold pkt
 *						BSD_FAST_FWD,	 fwd pkt, pkt ptr in *ret
 *						BSD_FAST_FULL,   send queue is full.
 *						BSD_FAST_MORE,   more pkt need to send.
 *				This values are meaningful if called from tpl.
 *
 * Created by:	Su Wang
 *
 * Date Created: 09/28/01
 *
 *-----------------------------------------------------------------
 */
tcp_outputC(intp,					/* tcp ctrl blk, */
			pconn,				/* tpl ctrl blk, fastpath */
			pdesc,				/* pkt desc ptr, fastpath */
								/* pHdr: start of TCP segment(excl tcp hdr) */
								/* hdrSize: size of TCP segment in hdr buffer */
			ret,				/* pkt desc to be fwded if any, fastpath*/
			parm)
	 void						*intp;
	 void						*pconn;		
	 pkt_desc_t					*pdesc;
	 pkt_desc_t					**ret;
	 void						*parm;
#else
int
tcp_output(tp)
	register struct tcpcb *tp;
#endif /* AG_EXTENSIONS */

{
	register struct tcpcb		*tp = (struct tcpcb*)intp;
	register struct socket *so = tp->t_inpcb->inp_socket;
	register long len, win;
	int off, flags, error=0;
	register struct mbuf *m = NULL;
	register struct tcpiphdr *ti;
	u_char opt[TCP_MAXOLEN];
	unsigned ipoptlen, optlen, hdrlen;
	int idle, sendalot;
	struct rmxp_tao *taop;
	struct rmxp_tao tao_noncached;
#if defined(AG_EXTENSIONS)
	int fastpath = ret ? 1:0;
#endif /* AG_EXTENSIONS */


	/*
	 * Determine length of data that should be transmitted,
	 * and flags that will be used.
	 * If there is some data or critical controls (SYN, RST)
	 * to send, then transmit; otherwise, investigate further.
	 */
	idle = (tp->snd_max == tp->snd_una);
	if (idle && tp->t_idle >= tp->t_rxtcur)
		/*
		 * We have been idle for "a while" and no acks are
		 * expected to clock out any data we send --
		 * slow start to get ack "clock" running again.
		 */
		tp->snd_cwnd = tp->t_maxseg;

again:
	sendalot = 0;
	off = tp->snd_nxt - tp->snd_una;
	win = min(tp->snd_wnd, tp->snd_cwnd);

	flags = tcp_outflags[tp->t_state];
	/*
	 * Get standard flags, and add SYN or FIN if requested by 'hidden'
	 * state flags.
	 */
	if (tp->t_flags & TF_NEEDFIN)
		flags |= TH_FIN;
	if (tp->t_flags & TF_NEEDSYN)
		flags |= TH_SYN;

	/*
	 * If in persist timeout with window of 0, send 1 byte.
	 * Otherwise, if window is small but nonzero
	 * and timer expired, we will send what we can
	 * and go to transmit state.
	 */
	if (tp->t_force) {
		if (win == 0) {
			/*
			 * If we still have some data to send, then
			 * clear the FIN bit.  Usually this would
			 * happen below when it realizes that we
			 * aren't sending all the data.  However,
			 * if we have exactly 1 byte of unsent data,
			 * then it won't clear the FIN bit below,
			 * and if we are in persist state, we wind
			 * up sending the packet without recording
			 * that we sent the FIN bit.
			 *
			 * We can't just blindly clear the FIN bit,
			 * because if we don't have any more data
			 * to send then the probe will be the FIN
			 * itself.
			 */
#if defined(AG_EXTENSIONS)
		  if (off < GETQSIZE(&so->so_snd))
#else			
		  if (off < so->so_snd.sb_cc)
#endif /* AG_EXTENSIONS */
			flags &= ~TH_FIN;
		  win = 1;
		} else {
			tp->t_timer[TCPT_PERSIST] = 0;
			tp->t_rxtshift = 0;
		}
	}

#if defined(AG_EXTENSIONS)
	/* add pkt to send queue */
	if (fastpath && pdesc)
	  {
		/* fastpath, need to check send queue */
#ifdef xxxx
		  /* no need to chekc send buf queue size */
		  /* it is determined by application */
		if (sbspace(&so->so_snd) < pdesc->dataSize + pdesc->hdrSize)
		  {
			*ret = 0;	
		    return BSD_FAST_FULL;
		  }
#endif /* xxxx */

		if ((m = m_devget((char*)pdesc->pIpHdr, 
						  MHLEN, 0, 0, 0)) == 0)
		  {
			*ret = 0;
			return BSD_FAST_ERR;
		  }
		
		/* setup flag, so tpl would not delete it after transmission */
		//printf("tcp_output: add pdesc %x to queue \n", pdesc);
		pdesc->flags |= PKT_USR_TCP_TX_Q;
		m->m_pktdesc = pdesc;
		sbappend(&so->so_snd, m);
		pTplCfg->tplStats.cntXmtTcpQueue++;
	  }
#endif /* AG_EXTENSIONS */

#if defined(AG_EXTENSIONS)
	len = (long)ulmin(GETQSIZE(&so->so_snd), win) - off;
#else
	len = (long)ulmin(so->so_snd.sb_cc, win) - off;
#endif /* AG_EXTENSIONS */

	if ((taop = tcp_gettaocache(tp->t_inpcb)) == NULL) {
		taop = &tao_noncached;
		bzero(taop, sizeof(*taop));
	}

	/*
	 * Lop off SYN bit if it has already been sent.  However, if this
	 * is SYN-SENT state and if segment contains data and if we don't
	 * know that foreign host supports TAO, suppress sending segment.
	 */
	if ((flags & TH_SYN) && SEQ_GT(tp->snd_nxt, tp->snd_una)) {
		flags &= ~TH_SYN;
		off--, len++;
		if (len > 0 && tp->t_state == TCPS_SYN_SENT &&
		    taop->tao_ccsent == 0)
		  {
			// printf("tcp_output return 1\n");
			return 0;
		  }
	}

	/*
	 * Be careful not to send data and/or FIN on SYN segments
	 * in cases when no CC option will be sent.
	 * This measure is needed to prevent interoperability problems
	 * with not fully conformant TCP implementations.
	 */
	if ((flags & TH_SYN) &&
	    ((tp->t_flags & TF_NOOPT) || !(tp->t_flags & TF_REQ_CC) ||
	     ((flags & TH_ACK) && !(tp->t_flags & TF_RCVD_CC)))) {
		len = 0;
		flags &= ~TH_FIN;
	}

	if (len < 0) {
		/*
		 * If FIN has been sent but not acked,
		 * but we haven't been called to retransmit,
		 * len will be -1.  Otherwise, window shrank
		 * after we sent into it.  If window shrank to 0,
		 * cancel pending retransmit, pull snd_nxt back
		 * to (closed) window, and set the persist timer
		 * if it isn't already going.  If the window didn't
		 * close completely, just wait for an ACK.
		 */
		len = 0;
		if (win == 0) {
			tp->t_timer[TCPT_REXMT] = 0;
			tp->t_rxtshift = 0;
			tp->snd_nxt = tp->snd_una;
			if (tp->t_timer[TCPT_PERSIST] == 0)
				tcp_setpersist(tp);
		}
	}

	
	if (len > tp->t_maxseg) {
		len = tp->t_maxseg;
		sendalot = 1;
	}

#if defined(AG_EXTENSIONS)
	if (SEQ_LT(tp->snd_nxt + len, tp->snd_una + GETQSIZE(&so->so_snd)))
#else
	if (SEQ_LT(tp->snd_nxt + len, tp->snd_una + so->so_snd.sb_cc))
#endif /* AG_EXTENSIONS */
		flags &= ~TH_FIN;

	win = sbspace(&so->so_rcv);

	/*
	 * Sender silly window avoidance.  If connection is idle
	 * and can send all data, a maximum segment,
	 * at least a maximum default-size segment do it,
	 * or are forced, do it; otherwise don't bother.
	 * If peer's buffer is tiny, then send
	 * when window is at least half open.
	 * If retransmitting (possibly after persist timer forced us
	 * to send into a small window), then must resend.
	 */
	if (len) {
#if defined(AG_EXTENSIONS)
	  /* we want to send pkt out whenever there are queued */
	  if (so->so_snd.sb_flags & SB_PKT)
		goto send;

#endif /* AG_EXTENSIONS */	    
		if (len == tp->t_maxseg)
			goto send;
		if (!(tp->t_flags & TF_MORETOCOME) &&
		    (idle || tp->t_flags & TF_NODELAY) &&
		    (tp->t_flags & TF_NOPUSH) == 0 &&
#if defined(AG_EXTENSIONS)
			len + off >= GETQSIZE(&so->so_snd)
#else
		     len + off >= so->so_snd.sb_cc
#endif /* AG_EXTENSIONS */
			   )
			goto send;
		if (tp->t_force)
			goto send;
		if (len >= tp->max_sndwnd / 2 && tp->max_sndwnd > 0)
			goto send;
		if (SEQ_LT(tp->snd_nxt, tp->snd_max))
			goto send;
	}


check_ack:
	/*
	 * Compare available window to amount of window
	 * known to peer (as advertised window less
	 * next expected input).  If the difference is at least two
	 * max size segments, or at least 50% of the maximum possible
	 * window, then want to send a window update to peer.
	 */
	if (win > 0) {
		/*
		 * "adv" is the amount we can increase the window,
		 * taking into account that we are limited by
		 * TCP_MAXWIN << tp->rcv_scale.
		 */
		long adv = min(win, (long)TCP_MAXWIN << tp->rcv_scale) -
			(tp->rcv_adv - tp->rcv_nxt);

		if (adv >= (long) (2 * tp->t_maxseg))
			goto send;
		if (2 * adv >= (long) so->so_rcv.sb_hiwat)
			goto send;
	}

	/*
	 * Send if we owe peer an ACK.
	 */
	if (tp->t_flags & TF_ACKNOW)
		goto send;
	if ((flags & TH_RST) ||
	    ((flags & TH_SYN) && (tp->t_flags & TF_NEEDSYN) == 0))
		goto send;
	if (SEQ_GT(tp->snd_up, tp->snd_una))
		goto send;
	/*
	 * If our state indicates that FIN should be sent
	 * and we have not yet done so, or we're retransmitting the FIN,
	 * then we need to send.
	 */
	if (flags & TH_FIN &&
	    ((tp->t_flags & TF_SENTFIN) == 0 || tp->snd_nxt == tp->snd_una))
		goto send;
	
		
	/*
	 * TCP window updates are not reliable, rather a polling protocol
	 * using ``persist'' packets is used to insure receipt of window
	 * updates.  The three ``states'' for the output side are:
	 *	idle			not doing retransmits or persists
	 *	persisting		to move a small or zero window
	 *	(re)transmitting	and thereby not persisting
	 *
	 * tp->t_timer[TCPT_PERSIST]
	 *	is set when we are in persist state.
	 * tp->t_force
	 *	is set when we are called to send a persist packet.
	 * tp->t_timer[TCPT_REXMT]
	 *	is set when we are retransmitting
	 * The output side is idle when both timers are zero.
	 *
	 * If send window is too small, there is data to transmit, and no
	 * retransmit or persist is pending, then go to persist state.
	 * If nothing happens soon, send when timer expires:
	 * if window is nonzero, transmit what we can,
	 * otherwise force out a byte.
	 */
	if (so->so_snd.sb_cc && tp->t_timer[TCPT_REXMT] == 0 &&
	    tp->t_timer[TCPT_PERSIST] == 0) {
		tp->t_rxtshift = 0;
		tcp_setpersist(tp);
	}



	/*
	 * No reason to send a segment, just return.
	 */
#if defined(AG_EXTENSIONS)
	if (fastpath)
	  {
		*ret = 0;
		return BSD_FAST_QUEUED;
	  }
	else
#endif /* AG_EXTENSIONS */		
	  {
		// printf("tcp_output return 2\n");
		return (0);
	  }

send:
	/*
	 * Before ESTABLISHED, force sending of initial options
	 * unless TCP set not to do any options.
	 * NOTE: we assume that the IP/TCP header plus TCP options
	 * always fit in a single mbuf, leaving room for a maximum
	 * link header, i.e.
	 *	max_linkhdr + sizeof (struct tcpiphdr) + optlen <= MHLEN
	 */
	optlen = 0;
	hdrlen = sizeof (struct tcpiphdr);
	if (flags & TH_SYN) {
		tp->snd_nxt = tp->iss;
		if ((tp->t_flags & TF_NOOPT) == 0) {
			u_short mss;

			opt[0] = TCPOPT_MAXSEG;
			opt[1] = TCPOLEN_MAXSEG;

			mss = htons((u_short) tcp_mssopt(tp));
			(void)memcpy(opt + 2, &mss, sizeof(mss));
			optlen = TCPOLEN_MAXSEG;

			if ((tp->t_flags & TF_REQ_SCALE) &&
			    ((flags & TH_ACK) == 0 ||
			    (tp->t_flags & TF_RCVD_SCALE))) {
				*((u_int32_t *)(opt + optlen)) = htonl(
					(TCPOPT_NOP << 24 |
					TCPOPT_WINDOW << 16 |
					TCPOLEN_WINDOW << 8 |
					tp->request_r_scale));
				optlen += 4;
			}
		}
 	}

 	/*
	 * Send a timestamp and echo-reply if this is a SYN and our side
	 * wants to use timestamps (TF_REQ_TSTMP is set) or both our side
	 * and our peer have sent timestamps in our SYN's.
 	 */
 	if ((tp->t_flags & (TF_REQ_TSTMP|TF_NOOPT)) == TF_REQ_TSTMP &&
 	    (flags & TH_RST) == 0 &&
	    ((flags & TH_ACK) == 0 ||
	     (tp->t_flags & TF_RCVD_TSTMP))) {
		u_int32_t *lp = (u_int32_t *)(opt + optlen);

 		/* Form timestamp option as shown in appendix A of RFC 1323. */
 		*lp++ = htonl(TCPOPT_TSTAMP_HDR);
 		*lp++ = htonl(tcp_now);
 		*lp   = htonl(tp->ts_recent);
 		optlen += TCPOLEN_TSTAMP_APPA;
 	}

 	/*
	 * Send `CC-family' options if our side wants to use them (TF_REQ_CC),
	 * options are allowed (!TF_NOOPT) and it's not a RST.
 	 */
 	if ((tp->t_flags & (TF_REQ_CC|TF_NOOPT)) == TF_REQ_CC &&
 	     (flags & TH_RST) == 0) {
		switch (flags & (TH_SYN|TH_ACK)) {
		/*
		 * This is a normal ACK, send CC if we received CC before
		 * from our peer.
		 */
		case TH_ACK:
			if (!(tp->t_flags & TF_RCVD_CC))
				break;
			/*FALLTHROUGH*/

		/*
		 * We can only get here in T/TCP's SYN_SENT* state, when
		 * we're a sending a non-SYN segment without waiting for
		 * the ACK of our SYN.  A check above assures that we only
		 * do this if our peer understands T/TCP.
		 */
		case 0:
			opt[optlen++] = TCPOPT_NOP;
			opt[optlen++] = TCPOPT_NOP;
			opt[optlen++] = TCPOPT_CC;
			opt[optlen++] = TCPOLEN_CC;
			*(u_int32_t *)&opt[optlen] = htonl(tp->cc_send);

			optlen += 4;
			break;

		/*
		 * This is our initial SYN, check whether we have to use
		 * CC or CC.new.
		 */
		case TH_SYN:
			opt[optlen++] = TCPOPT_NOP;
			opt[optlen++] = TCPOPT_NOP;
			opt[optlen++] = tp->t_flags & TF_SENDCCNEW ?
						TCPOPT_CCNEW : TCPOPT_CC;
			opt[optlen++] = TCPOLEN_CC;
			*(u_int32_t *)&opt[optlen] = htonl(tp->cc_send);
 			optlen += 4;
			break;

		/*
		 * This is a SYN,ACK; send CC and CC.echo if we received
		 * CC from our peer.
		 */
		case (TH_SYN|TH_ACK):
			if (tp->t_flags & TF_RCVD_CC) {
				opt[optlen++] = TCPOPT_NOP;
				opt[optlen++] = TCPOPT_NOP;
				opt[optlen++] = TCPOPT_CC;
				opt[optlen++] = TCPOLEN_CC;
				*(u_int32_t *)&opt[optlen] =
					htonl(tp->cc_send);
				optlen += 4;
				opt[optlen++] = TCPOPT_NOP;
				opt[optlen++] = TCPOPT_NOP;
				opt[optlen++] = TCPOPT_CCECHO;
				opt[optlen++] = TCPOLEN_CC;
				*(u_int32_t *)&opt[optlen] =
					htonl(tp->cc_recv);
				optlen += 4;
			}
			break;
		}
 	}

 	hdrlen += optlen;

	if (tp->t_inpcb->inp_options) {
		ipoptlen = tp->t_inpcb->inp_options->m_len -
				offsetof(struct ipoption, ipopt_list);
	} else {
		ipoptlen = 0;
	}

	/*
	 * Adjust data length if insertion of options will
	 * bump the packet length beyond the t_maxopd length.
	 * Clear the FIN bit because we cut off the tail of
	 * the segment.
	 */
	if (len + optlen + ipoptlen > tp->t_maxopd) {
		/*
		 * If there is still more to send, don't close the connection.
		 */
		flags &= ~TH_FIN;
		len = tp->t_maxopd - optlen - ipoptlen;
		sendalot = 1;
	}

/*#ifdef DIAGNOSTIC*/
 	if (max_linkhdr + hdrlen > MHLEN)
		panic("tcphdr too big");
/*#endif*/

	/*
	 * Grab a header mbuf, attaching a copy of data to
	 * be transmitted, and initialize the header from
	 * the template for sends on this connection.
	 */
	if (len) {
#if defined(AG_EXTENSIONS)
	  if (fastpath)
		{
		  int t_off = off; /* need to init pdesc */

		  pdesc = 0;
		  /* which pkt to send */
		  for (m = so->so_snd.sb_mb; m; m = m->m_next)
			{ 

			  if (!m->m_pktdesc)
				panic("tcp_output: bad snd queue\n");

			  if (t_off == 0)
				{ /* matches OK */
				  pdesc = (pkt_desc_t*)m->m_pktdesc;
				  break;
				}
			  else if (t_off < 0) 
				{	/* cant advance pkt ptr */
				  break;		
				}
			  else
				pdesc = (pkt_desc_t*)m->m_pktdesc;
			  
			  t_off -= pdesc->dataSize + pdesc->hdrSize;
			}
		  
		  /* snd_una snd_nxt offset larger than data queued */
		  if (t_off > 0)
			{
			  bsd_log(BSD_ERR_TCP_TX, 
					  "tcp_input: tcp offset too large %d\n", t_off);
			  off = 0;
			  tp->snd_nxt = tp->snd_una;
			  pdesc = (so->so_snd.sb_mb)->m_pktdesc;
			}
		  else if (t_off < 0)
			{
			  /* offset does not match pkt boundry, */
			  /* we need to realign snd_una snd_nxt */
			  bsd_log(BSD_ERR_TCP_TX, 
					  "tcp_input: snd_una %d snd_nxt %d not aligned\n", 
					  tp->snd_nxt,tp->snd_una);		
			  off -= (pdesc->hdrSize+pdesc->dataSize)+t_off;
			  tp->snd_nxt -= tp->snd_una + off;
			}
		  
		  /* no support for forced out yet */
		  if (tp->t_force && len == 1)
			{
				panic("tcp_output: forced detected\n");
				*ret = 0;
				return BSD_FAST_ERR;
			}


		  if (len < pdesc->dataSize + pdesc->hdrSize)
			{ 
			  /* we cannot send because win size is too small */ 
			  /* go back to see any other reason to send  */
			  len = 0;
			  goto check_ack;
			}
		  else if (len > pdesc->dataSize + pdesc->hdrSize)
			{
			  /* there are more data can be sent, note that */
			  /* len <= t_maxseg which is larger than */
			  /* pdesc->dataSize + pdesc->hdrSize */
			  sendalot = 1;
			}

		  /* now we have the pkt, */ 
		  /* len will be pkt lenth */
		  len = pdesc->dataSize + pdesc->hdrSize;

		  /* need to check fin again, because len is smaller */
		  if (SEQ_LT(tp->snd_nxt + len, tp->snd_una + GETQSIZE(&so->so_snd)))
			  flags &= ~TH_FIN;
		}
	  else
#endif /* AG_EXTENSIONS */
	  {
		MGETHDR(m, M_DONTWAIT, MT_HEADER);
		if (m == NULL) {
			error = ENOBUFS;
			goto out;
		}
		m->m_data += max_linkhdr;
		m->m_len = hdrlen;
		if (len <= MHLEN - hdrlen - max_linkhdr) {
			m_copydata(so->so_snd.sb_mb, off, (int) len,
			    mtod(m, caddr_t) + hdrlen);
			m->m_len += len;
		} else {
			m->m_next = m_copy(so->so_snd.sb_mb, off, (int) len);
			if (m->m_next == 0) {
				(void) m_free(m);
				error = ENOBUFS;
				goto out;
			}
		}
	  }

		if (tp->t_force && len == 1)
			tcpstat.tcps_sndprobe++;
		else if (SEQ_LT(tp->snd_nxt, tp->snd_max)) {
			tcpstat.tcps_sndrexmitpack++;
			tcpstat.tcps_sndrexmitbyte += len;
		} else {
			tcpstat.tcps_sndpack++;
			tcpstat.tcps_sndbyte += len;
		}
	
		/*
		 * If we're sending everything we've got, set PUSH.
		 * (This will keep happy those implementations which only
		 * give data to the user when a buffer fills or
		 * a PUSH comes in.)
		 */
#if defined(AG_EXTENSIONS)
		if (off + len == GETQSIZE(&so->so_snd))
#else
		if (off + len == so->so_snd.sb_cc)
#endif /* AG_EXTENSIONS */
			flags |= TH_PUSH;
	} else {
		if (tp->t_flags & TF_ACKNOW)
			tcpstat.tcps_sndacks++;
		else if (flags & (TH_SYN|TH_FIN|TH_RST))
			tcpstat.tcps_sndctrl++;
		else if (SEQ_GT(tp->snd_up, tp->snd_una))
			tcpstat.tcps_sndurg++;
		else
			tcpstat.tcps_sndwinup++;

#if defined(AG_EXTENSIONS)
		pdesc = 0;
		/* fast path for pure ack */
		if (fastpath)
		  {
			if (!(pdesc = TPL_ALLOC_PACKET(pconn, 0)))
			{
//			  BSD_LOGERR(BSD_ERR_GEN_PANIC);
			  *ret = 0;
			  return BSD_FAST_ERR;
			}
			/* set size to 0, since no data */
			// printf("tcp_output: pure ack pDesc %x allocated\n", pdesc);
			pdesc->hdrSize = 0;
			pdesc->flags  |= PKT_USR_TCP_TX_ACK;

			/* move pHdr ahead by 4 bytes, no RPC header for pure ack */
			if (((tpl_connCb_t*)pconn)->flags &  TPL_CONN_USE_RPC)
				pdesc->pHdr -= 4;

			/* pkt should be returned to connection immediately */
			/* after transmision */
	           pdesc->appInfo = NULL;
		  }
		else
#endif /* AG_EXTENSIONS */
		  {
			MGETHDR(m, M_DONTWAIT, MT_HEADER);
			if (m == NULL) {
			  error = ENOBUFS;
			  goto out;
			}
			m->m_data += max_linkhdr;
			m->m_len = hdrlen;
		  }
	}

#if defined(AG_EXTENSIONS)
	if (fastpath)
	  {
		ti = (struct tcpiphdr*)(pdesc->pIpHdr);
//		ti = (struct tcpiphdr*)(pdesc->pHdr - sizeof(struct tcpiphdr));
		ti->ti_off = 5;			/* tcp hdr is fixed for fastpath */
	  }
	else
#endif /* AG_EXTENSIONS */	
	  {
		m->m_pkthdr.rcvif = (struct ifnet *)0;
		ti = mtod(m, struct tcpiphdr *);

		if (tp->t_template == 0)
		  panic("tcp_output");
		(void)memcpy(ti, tp->t_template, sizeof (struct tcpiphdr));
	  }

	/*
	 * Fill in fields, remembering maximum advertised
	 * window for use in delaying messages about window sizes.
	 * If resending a FIN, be sure not to use a new sequence number.
	 */
	if (flags & TH_FIN && tp->t_flags & TF_SENTFIN &&
	    tp->snd_nxt == tp->snd_max)
		tp->snd_nxt--;
	/*
	 * If we are doing retransmissions, then snd_nxt will
	 * not reflect the first unsent octet.  For ACK only
	 * packets, we do not want the sequence number of the
	 * retransmitted packet, we want the sequence number
	 * of the next unsent octet.  So, if there is no data
	 * (and no SYN or FIN), use snd_max instead of snd_nxt
	 * when filling in ti_seq.  But if we are in persist
	 * state, snd_max might reflect one byte beyond the
	 * right edge of the window, so use snd_nxt in that
	 * case, since we know we aren't doing a retransmission.
	 * (retransmit and persist are mutually exclusive...)
	 */
	if (len || (flags & (TH_SYN|TH_FIN)) || tp->t_timer[TCPT_PERSIST])
		ti->ti_seq = htonl(tp->snd_nxt);
	else
		ti->ti_seq = htonl(tp->snd_max);
	ti->ti_ack = htonl(tp->rcv_nxt);
	if (optlen) {
		bcopy(opt, (char*)(ti + 1), optlen);
		ti->ti_off = (sizeof (struct tcphdr) + optlen) >> 2;
	}
	ti->ti_flags = flags;
	/*
	 * Calculate receive window.  Don't shrink window,
	 * but avoid silly window syndrome.
	 */
	if (win < (long)(so->so_rcv.sb_hiwat / 4) && win < (long)tp->t_maxseg)
		win = 0;
	if (win < (long)(tp->rcv_adv - tp->rcv_nxt))
		win = (long)(tp->rcv_adv - tp->rcv_nxt);
	if (win > (long)TCP_MAXWIN << tp->rcv_scale)
		win = (long)TCP_MAXWIN << tp->rcv_scale;
	ti->ti_win = htons((u_short) (win>>tp->rcv_scale));
	if (SEQ_GT(tp->snd_up, tp->snd_nxt)) {
		ti->ti_urp = htons((u_short)(tp->snd_up - tp->snd_nxt));
		ti->ti_flags |= TH_URG;
	} else
		/*
		 * If no urgent pointer to send, then we pull
		 * the urgent pointer to the left edge of the send window
		 * so that it doesn't drift into the send window on sequence
		 * number wraparound.
		 */
		tp->snd_up = tp->snd_una;		/* drag it along */


#if defined(AG_EXTENSIONS)
	/* fastpath, set ip pck lenth */
	if (fastpath)
	  {
		((struct ip *)ti)->ip_len = htons(len+sizeof(struct tcpiphdr));
		/* setup ti->ti_len for trace purpose, can be removed later */
		ti->ti_len = htons(len+sizeof(struct tcphdr));
	  }
	else
#endif /* AG_EXTENSIONS */
	  {
		/*
		 * Put TCP length in extended header, and then
		 * checksum extended header and data.
		 */
		if (len + optlen)
		  ti->ti_len = htons((u_short)(sizeof (struct tcphdr) +
									   optlen + len));
		ti->ti_sum = in_cksum(m, (int)(hdrlen + len));
	  }


	
	/*
	 * In transmit state, time the transmission and arrange for
	 * the retransmit.  In persist state, just set snd_max.
	 */
	if (tp->t_force == 0 || tp->t_timer[TCPT_PERSIST] == 0) {
		tcp_seq startseq = tp->snd_nxt;

		/*
		 * Advance snd_nxt over sequence space of this segment.
		 */
		if (flags & (TH_SYN|TH_FIN)) {
			if (flags & TH_SYN)
				tp->snd_nxt++;
			if (flags & TH_FIN) {
				tp->snd_nxt++;
				tp->t_flags |= TF_SENTFIN;
			}
		}
		tp->snd_nxt += len;
		if (SEQ_GT(tp->snd_nxt, tp->snd_max)) {
			tp->snd_max = tp->snd_nxt;
			/*
			 * Time this transmission if not a retransmission and
			 * not currently timing anything.
			 */
			if (tp->t_rtt == 0) {
				tp->t_rtt = 1;
				tp->t_rtseq = startseq;
				tcpstat.tcps_segstimed++;
			}
		}

		/*
		 * Set retransmit timer if not currently set,
		 * and not doing an ack or a keep-alive probe.
		 * Initial value for retransmit timer is smoothed
		 * round-trip time + 2 * round-trip time variance.
		 * Initialize shift counter which is used for backoff
		 * of retransmit time.
		 */
		if (tp->t_timer[TCPT_REXMT] == 0 &&
		    tp->snd_nxt != tp->snd_una) {
			tp->t_timer[TCPT_REXMT] = tp->t_rxtcur;
			if (tp->t_timer[TCPT_PERSIST]) {
				tp->t_timer[TCPT_PERSIST] = 0;
				tp->t_rxtshift = 0;
			}
		}
	} else
		if (SEQ_GT(tp->snd_nxt + len, tp->snd_max))
			tp->snd_max = tp->snd_nxt + len;

#ifdef TCPDEBUG
	/*
	 * Trace.
	 */
	if (so->so_options & SO_DEBUG)
		tcp_trace(TA_OUTPUT, tp->t_state, tp, ti, 0);
#endif

	/* we are ready to send output now */
#if defined(AG_EXTENSIONS)
	if (fastpath)					/* fastpath */
	  {
		*ret = pdesc;
	  }
	else
#endif /* AG_EXTENSIONS */	
	{
	  /*
	   * Fill in IP length and desired time to live and
	   * send to IP level.  There should be a better way
	   * to handle ttl and tos; we could keep them in
	   * the template, but need a way to checksum without them.
	   */
	  m->m_pkthdr.len = hdrlen + len;
	  {
#if 1
		struct rtentry *rt;
#endif
		((struct ip *)ti)->ip_len = m->m_pkthdr.len;
		((struct ip *)ti)->ip_ttl = tp->t_inpcb->inp_ip_ttl;	/* XXX */
		((struct ip *)ti)->ip_tos = tp->t_inpcb->inp_ip_tos;	/* XXX */
#if 1
		/*
		 * See if we should do MTU discovery.  We do it only if the following
		 * are true:
		 *	1) we have a valid route to the destination
		 *	2) the MTU is not locked (if it is, then discovery has been
		 *	   disabled)
		 */
		if ((rt = tp->t_inpcb->inp_route.ro_rt)
			&& rt->rt_flags & RTF_UP
			&& !(rt->rt_rmx.rmx_locks & RTV_MTU)) {
		  ((struct ip *)ti)->ip_off |= IP_DF;
		}
#endif
		error = ip_output(m, tp->t_inpcb->inp_options, &tp->t_inpcb->inp_route,
						  so->so_options & SO_DONTROUTE, 0);
	  }
	  
	}

	if (error) {
out:
		if (error == ENOBUFS) {
			tcp_quench(tp->t_inpcb, 0);
			// printf("tcp_output return 3\n");
			return (0);
		}
#if 1
		if (error == EMSGSIZE) {
			/*
			 * ip_output() will have already fixed the route
			 * for us.  tcp_mtudisc() will, as its last action,
			 * initiate retransmission, so it is important to
			 * not do so here.
			 */
			tcp_mtudisc(tp->t_inpcb, 0);
			// printf("tcp_output return 4\n");
			return 0;
		}
#endif
		if ((error == EHOSTUNREACH || error == ENETDOWN)
		    && TCPS_HAVERCVDSYN(tp->t_state)) {
			tp->t_softerror = error;
			// printf("tcp_output return 5\n"); 
			return (0);
		}
		// printf("tcp_output return 6\n");
		return (error);
	}
	tcpstat.tcps_sndtotal++;

	/*
	 * Data sent (as far as we can tell).
	 * If this advertises a larger window than any other segment,
	 * then remember the size of the advertised window.
	 * Any pending ACK has now been sent.
	 */
	if (win > 0 && SEQ_GT(tp->rcv_nxt+win, tp->rcv_adv))
		tp->rcv_adv = tp->rcv_nxt + win;
	tp->last_ack_sent = tp->rcv_nxt;
	tp->t_flags &= ~(TF_ACKNOW|TF_DELACK);

#if defined(AG_EXTENSIONS)
	if (fastpath)
	  {
	  if (sendalot)
		return BSD_FAST_MORE;
	  else
		return BSD_FAST_FWD;
	  }
	else  
#endif /* AG_EXTENSIONS */
	  if (sendalot)
		goto again;

	//printf("tcp_output return 7\n");
	return (0);
}

void
tcp_setpersist(tp)
	register struct tcpcb *tp;
{
	register int t = ((tp->t_srtt >> 2) + tp->t_rttvar) >> 1;

	if (tp->t_timer[TCPT_REXMT])
		panic("tcp_output REXMT");
	/*
	 * Start/restart persistance timer.
	 */
	TCPT_RANGESET(tp->t_timer[TCPT_PERSIST],
	    t * tcp_backoff[tp->t_rxtshift],
	    TCPTV_PERSMIN, TCPTV_PERSMAX);
	if (tp->t_rxtshift < TCP_MAXRXTSHIFT)
		tp->t_rxtshift++;
}


