
/*
 *  TCP MIB group implementation - tcp.c
 *
 */

#include <config.h>

#include <unistd.h>
#include <string.h>
#include <sys/param.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <net/route.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <sys/queue.h>
#include <netinet/ip_var.h>
#include <sys/socketvar.h>
#include <netinet/in_pcb.h>

#include "tools.h"
#include "kernel.h"

#include "mibincl.h"
#include "util_funcs.h"

#include "system.h"
#include <sys/sysctl.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <netinet/tcpip.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>
#include <netinet/tcp_fsm.h>

#include "auto_nlist.h"
#include "mibincl.h"

#include "tcp.h"
#include "tcpTable.h"

#include <ipmd-lib.h>
#include "ipifhelp.h"


void swapconn(ipm_conn_t *p1, ipm_conn_t *p2);
int conncmp(ipm_conn_t *p1, ipm_conn_t *p2);
int myGetTcpConn(ipm_conn_t *connentry, int *datafrom);
int myGetTcpConnSSC(ipm_conn_t *pipm_conn, int *pcount);
int myFindNextTcpConn(ipm_conn_t *centry, int *datafrom);

#define TCPCONN_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
marker_t tcpconn_cache_marker = NULL;


	/*********************
	 *
	 *  Kernel & interface information,
	 *   and internal forward declarations
	 *
	 *********************/


	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/


	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/

u_char *
var_tcpEntry(struct variable *vp,
		 oid *name,
		 size_t *length,
		 int exact,
		 size_t *var_len,
		 WriteMethod **write_method)
{
	static int datafrom = 0;	/* 0=invalid, 1=SSC, 2=FP NIM */
	static ipm_conn_t connent;
	int i;
	uint32_t localIP, remoteIP;
	unsigned short localport, remoteport;
	unsigned char *ipaddr;

/* we dropped support for TCP Connection table for now */
return NULL;

	/* retrieve local IP index */
	ipaddr = (unsigned char *) &localIP;
	for (i = 0; i < 4; i++)
		ipaddr[i] = (unsigned char) name[10+i];

	/* retrieve remote IP index */
	ipaddr = (unsigned char *) &remoteIP;
	for (i = 0; i < 4; i++)
		ipaddr[i] = (unsigned char) name[15+i];

	localport  = name[14];
	remoteport = name[19];

	if (exact) {
		/* A Get request? */
		connent.conn_lAddr = localIP;
		connent.conn_lPort = localport;
		connent.conn_rAddr = remoteIP;
		connent.conn_rPort = remoteport;

		if (myGetTcpConn(&connent, &datafrom) != IPM_GET_OK)
			return NULL;
	}
	else {
		/* this is a GetNext request */
		/* need to find out whether we can use the data */
		/* stored in connent variable...                */
		if (myFindNextTcpConn(&connent, &datafrom) != IPM_GET_OK)
			return NULL;
	}

	localIP = connent.conn_lAddr;
	ipaddr = (unsigned char *) &localIP;
	for (i = 0; i < 4; i++)
		name[10+i] = (oid) ipaddr[i];
	name[14] = (oid) connent.conn_lPort;

	remoteIP = connent.conn_rAddr;
	ipaddr = (unsigned char *) &remoteIP;
	for (i = 0; i < 4; i++)
		name[15+i] = (oid) ipaddr[i];
	name[19] = (oid) connent.conn_rPort;

	*length = vp->namelen + 10;
	*write_method = 0;
	*var_len = sizeof(long);

	switch (vp->magic) {
	case TCPCONNSTATE:
		long_return = connent.conn_state;
		break;
	case TCPCONNLOCALADDRESS:
		long_return = connent.conn_lAddr;
		break;
	case TCPCONNLOCALPORT:
		long_return = connent.conn_lPort;
		break;
	case TCPCONNREMADDRESS:
		long_return = connent.conn_rAddr;
		break;
	case TCPCONNREMPORT:
		long_return = connent.conn_rPort;
		break;
	default:
		return NULL;
	}

	return (u_char *) &long_return;
}


	/*********************
	 *
	 *  Internal implementation functions
	 *
	 *********************/

void swapconn(ipm_conn_t *p1, ipm_conn_t *p2)
{
ipm_conn_t t;

	if ((p1 == NULL) || (p2 == NULL))
		return;

	memcpy(&t, p1, sizeof(ipm_conn_t));
	memcpy(p1, p2, sizeof(ipm_conn_t));
	memcpy(p2, &t, sizeof(ipm_conn_t));
}

int conncmp(ipm_conn_t *p1, ipm_conn_t *p2)
{
	if ((p1 == NULL) || (p2 == NULL))
		return 0;

	if (ntohl(p1->conn_lAddr) > ntohl(p2->conn_lAddr))
		return 1;
	else if (ntohl(p1->conn_lAddr) < ntohl(p2->conn_lAddr))
		return -1;

	if (ntohs(p1->conn_lPort) > ntohs(p2->conn_lPort))
		return 1;
	else if (ntohs(p1->conn_lPort) < ntohs(p2->conn_lPort))
		return -1;

	if (ntohl(p1->conn_rAddr) > ntohl(p2->conn_rAddr))
		return 1;
	else if (ntohl(p1->conn_rAddr) < ntohl(p2->conn_rAddr))
		return -1;

	if (ntohs(p1->conn_rPort) > ntohs(p2->conn_rPort))
		return 1;
	else if (ntohs(p1->conn_rPort) < ntohs(p2->conn_rPort))
		return -1;

	return 0;
}

int myGetTcpConnSSC(ipm_conn_t *pipm_conn, int *pcount)
{
static ipm_conn_t *pSSCConn = NULL;
int i, j, rval;
int size, count, needed;

	pipm_conn = NULL;

	if (tcpconn_cache_marker &&
		(!atime_ready( tcpconn_cache_marker, TCPCONN_CACHE_TIMEOUT*1000 ))) {
		pipm_conn = pSSCConn;
		return IPM_GET_OK;
	}

	if (pSSCConn)
		free(pSSCConn);

	count = 40;
	size = count * sizeof(ipm_conn_t);
	pSSCConn = (ipm_conn_t *) malloc(size);
	if (pSSCConn == NULL)
		return IPM_GET_ERR;

	rval = ipmd_GetTcpConnSSC(pSSCConn, size, &count, &needed);
	if (rval == IPM_GET_ERR) {
		free(pSSCConn);
		return rval;
	}
	else if ((rval == IPM_GET_NO_BUF) || (rval == IPM_CMDERR_APPNOBUF)) {
		free(pSSCConn);
		size = needed;
		count = size / sizeof(ipm_conn_t);
		pSSCConn = (ipm_conn_t *) malloc(size);
		if (pSSCConn == NULL)
			return IPM_GET_ERR;
		rval = ipmd_GetTcpConnSSC(pSSCConn, size, &count, &needed);
		if (rval != IPM_GET_OK) {
			free(pSSCConn);
			return IPM_GET_ERR;
		}
	}

	/* now bubble sort it... */
	for (i = 0; i < count; i++) {
		for (j = i; j < count; j++) {
			if (ntohl(pSSCConn[i].conn_lAddr) > ntohl(pSSCConn[j].conn_lAddr))
				swapconn(&pSSCConn[i], &pSSCConn[j]);
			else if (ntohl(pSSCConn[i].conn_lAddr) < ntohl(pSSCConn[j].conn_lAddr))
				continue;

			if (ntohs(pSSCConn[i].conn_lPort) > ntohs(pSSCConn[j].conn_lPort))
				swapconn(&pSSCConn[i], &pSSCConn[j]);
			else if (ntohs(pSSCConn[i].conn_lPort) < ntohs(pSSCConn[j].conn_lPort))
				continue;

			if (ntohl(pSSCConn[i].conn_rAddr) > ntohl(pSSCConn[j].conn_rAddr))
				swapconn(&pSSCConn[i], &pSSCConn[j]);
			else if (ntohl(pSSCConn[i].conn_rAddr) < ntohl(pSSCConn[j].conn_rAddr))
				continue;

			if (ntohs(pSSCConn[i].conn_rPort) > ntohs(pSSCConn[j].conn_rPort))
				swapconn(&pSSCConn[i], &pSSCConn[j]);
		}
	}
	pipm_conn = pSSCConn;
	*pcount = count;

	if (tcpconn_cache_marker)
		atime_setMarker(tcpconn_cache_marker);
	else
		tcpconn_cache_marker = atime_newMarker();

	return IPM_GET_OK;
}

int myFindNextTcpConn(ipm_conn_t *centry, int *datafrom)
{
int rval, i, count;
ipm_conn_t *pentrySSC = NULL, *pentryFP = NULL;
ipm_conn_t entryFP, nextFP;

	if ((centry == NULL) || (datafrom == NULL))
		return IPM_GET_ERR;

	/* datafrom: 0-invalid, 1-SSC, 2-FPNIM */
	if (*datafrom == 0) {

		rval = myGetTcpConnSSC(pentrySSC, &count);
		if ((rval != IPM_GET_OK) || (count <= 0))
			pentrySSC = NULL;
		else {
			for (i = 0; i < count; i++) {
				if (conncmp(&pentrySSC[i], centry) > 0)
					break;
			}
			if (i >= count)
				pentrySSC = NULL;	/* no next in SSC */
			else
				pentrySSC = &pentrySSC[i];
		}

		if (ipmd_GetTcpNxtConn(NULL, &entryFP) != IPM_GET_OK)
			pentryFP = NULL;
		else {
			pentryFP = &entryFP;
			while (conncmp(pentryFP, centry) <= 0) {
				if (ipmd_GetTcpNxtConn(pentryFP, &nextFP) != IPM_GET_OK) {
					pentryFP = NULL;
					break;
				}
				memcpy(pentryFP, &nextFP, sizeof(ipm_conn_t));
			}
		}
	}

	if (*datafrom == 1) {
		rval = myGetTcpConnSSC(pentrySSC, &count);
		if ((rval != IPM_GET_OK) || (count <= 0))
			pentrySSC = NULL;
		else {
			for (i = 0; i < count; i++) {
				if (conncmp(&pentrySSC[i], centry) > 0)
					break;
			}
			if (i < count)
				pentrySSC = &pentrySSC[i];
			else
				pentrySSC = NULL;	/* no next in SSC */
		}

		/* if local IP address stay the same, then we  */
		/* are safe to assume this next entry is valid */
		/* and we don't need to look into FP side for  */
		/* possible other next IP                      */
		pentryFP = NULL;
		if (!pentrySSC || (pentrySSC->conn_lAddr != centry->conn_lAddr)) {
			if (ipmd_GetTcpNxtConn(NULL, &entryFP) == IPM_GET_OK) {
				pentryFP = &entryFP;
				while (conncmp(pentryFP, centry) <= 0) {
					if (ipmd_GetTcpNxtConn(pentryFP, &nextFP) != IPM_GET_OK) {
						pentryFP = NULL;
						break;
					}
					memcpy(pentryFP, &nextFP, sizeof(ipm_conn_t));
				}
			}
		}
	}

	if (*datafrom == 2) {
		if (ipmd_GetTcpNxtConn(centry, &nextFP) != IPM_GET_OK)
			pentryFP = NULL;

		if (!pentryFP || (centry->conn_lAddr != nextFP.conn_lAddr)) {
			rval = myGetTcpConnSSC(pentrySSC, &count);
			if ((rval != IPM_GET_OK) || (count <= 0))
				pentrySSC = NULL;
			else {
				for (i = 0; i < count; i++) {
					if (conncmp(&pentrySSC[i], centry) > 0)
						break;
				}
				if (i < count)
					pentrySSC = &pentrySSC[i];
				else
					pentrySSC = NULL;	/* no next in SSC */
			}
		}
	}

	/* now we found the next entry for 'centry' */
	/* in both SSC and FP. We need to return    */
	/* the lesser one of these two...           */
	if (pentrySSC) {
		if (pentryFP) {
			/* in duplicated entries case (which should be */
			/* occuring in the first place but we need to  */
			/* handle anyway), we'll use FP's              */
			if (conncmp(pentryFP, pentrySSC) <= 0)
				*datafrom = 2; /* both exists, FP <= SSC, use FP */
			else
				*datafrom = 1; /* both exists, FP > SSC, use SSC */
		}
		else
			*datafrom = 1; /* only SSC exists */
	}
	else {
		if (pentryFP)
			*datafrom = 2; /* only FP exists */
		else
			*datafrom = 0; /* NONE exists!!! */
	}

	if (*datafrom == 1) {
		memcpy(centry, pentrySSC, sizeof(ipm_conn_t));
		return IPM_GET_OK;
	}
	else if (*datafrom == 2) {
		memcpy(centry, pentryFP, sizeof(ipm_conn_t));
		return IPM_GET_OK;
	}
	*datafrom = 0;
	return IPM_GET_ERR;
}


int myGetTcpConn(ipm_conn_t *connentry, int *datafrom)
{
int rval, count, i;
ipm_conn_t tmpent, *pentry = NULL;

	/* check FP NIM first to see if this entry's there */
	rval = ipmd_GetTcpConn(connentry, &tmpent);
	if (rval == IPM_GET_OK) {
		memcpy(connentry, &tmpent, sizeof(ipm_conn_t));
		*datafrom = 2; /* entry is from FP NIM */
		return rval;
	}

	/* now check SSC */
	rval = myGetTcpConnSSC(pentry, &count);
	if (rval == IPM_GET_ERR)
		return rval;

	for (i = 0; i < count; i++) {
		if ((pentry[i].conn_lAddr == connentry->conn_lAddr) &&
			(pentry[i].conn_lPort == connentry->conn_lPort) &&
			(pentry[i].conn_rAddr == connentry->conn_rAddr) &&
			(pentry[i].conn_rPort == connentry->conn_rPort)) {
			/* found it */
			connentry->conn_state = pentry[i].conn_state;
			*datafrom = 1; /* entry is from SSC */
			return IPM_GET_OK;
		}
	}

	/* exhausted, but still not found... */
	return IPM_GET_ERR;
}
