/*
 *-----------------------------------------------------------------
 *
 * Name:	cache.S
 *
 * RCS:  $Id: cache.S,v 1.3 2002/01/03 20:13:27 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/6/01
 *
 *-----------------------------------------------------------------
 */
/*
 * cache.s: switchable cache support for SDE-MIPS kit
 * Copyright (c) 1998 Algorithmics Ltd
 *
 * The function of this module is to support boards which can handle
 * multiple MIPS cpus with different cache architectures.
 */	
	
#include <nfx-target.h>
#include <sys/mips.h>

#if defined(IN_PMON) || defined(ITROM)
#define DECL(x, val)		\
	BSS(x,4)
#else	
#define DECL(x, val)		\
	.sdata;			\
	EXPORTS(x, 4);		\
	.word	val
#endif	

DECL(mips_icache_size,-1)
DECL(mips_icache_linesize,-1)
DECL(mips_icache_ways,1)
	
DECL(mips_dcache_size,-1)
DECL(mips_dcache_linesize,-1)
DECL(mips_dcache_ways,1)
	
DECL(mips_scache_size,-1)
DECL(mips_scache_linesize,-1)
DECL(mips_scache_ways,1)
	
/* R4000 specials */	
DECL(mips_scache_split,0)
DECL(mips_scache_discontig,0)
	
DECL(mips_tcache_size,-1)
DECL(mips_tcache_linesize,-1)
DECL(mips_tcache_ways,1)

#define GETPRID 		\
	mfc0	t0,$15;		\
	srl	t0,8;			\
	and	t0,0xff;

#define SWTEST(prid,dest) 	\
	beq	t0,prid,dest;
	
#if RAL_XXX

#if RAL_XXX
#define R5KSWITCH(x)	SWTEST(PRID_R5000,r5k_/**/x) \
			SWTEST(PRID_RM52XX,r5k_/**/x)
#else
#define R5KSWITCH(x)
#endif
	
#define RM7KSWITCH(x)	SWTEST(PRID_RM7000,rm7k_/**/x)
	
#if RAL_XXX
#if #cache(lr30) + #cache(cw) + #cache(r4k) + #cache(r3k) > 1
#error Only one of lr30, cw, r4k or r3k should be asserted for -Acache
#endif	
#endif
	
/* Vanilla R4000 is a default fall-through if enabled */
#if #cache(r4k)
#define R4KSWITCH(x)	b	r4k_/**/x;
#else
#define R4KSWITCH(x)
#endif
	
/* Vanilla R3000 is a default fall-through if enabled */
#define R3KSWITCH(x)	b	r3k_/**/x;
	
#define SWCACHE(name) 		\
LEAF(mips_/**/name); 		\
	GETPRID			\
	R5KSWITCH(name)		\
	RM7KSWITCH(name)	\
	R4KSWITCH(name)		\
	R3KSWITCH(name)		\
	j	$ra;		\
END(mips_/**/name)
	
#else

#define SWCACHE(name)

/* 
 * void mips_init_cache (void)
 *
 * Size and (re)initialise caches from scratch; usually only called
 * once during power on reset.
 *
 * NOTE: loses any dirty lines in writeback caches
 */
LEAF(mips_init_cache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_init_cache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_init_cache
2:	j	r3k_init_cache
#endif
END(mips_init_cache)
SWCACHE(init_cache)
	
/* 
 * void mips_size_cache (void)
 *
 * Size caches without reinitialising and losing dirty cache lines.
 */
LEAF(mips_size_cache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_size_cache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_size_cache
2:	j	r3k_size_cache
#endif
END(mips_size_cache)
SWCACHE(size_cache)
	
/* 
 * void mips_flush_cache (void)
 *
 * Writeback and invalidate all caches in the quickest way possible.
 */
LEAF(mips_flush_cache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_flush_cache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_flush_cache
2:	j	r3k_flush_cache
#endif
END(mips_flush_cache)
SWCACHE(flush_cache)
	
/* 
 * void mips_flush_dcache (void)
 *
 * Writeback and invalidate only the data caches.  
 * Joint caches (i.e. combined I & D) will be flushed too. 
 */
LEAF(mips_flush_dcache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_flush_dcache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_flush_dcache
2:	j	r3k_flush_dcache
#endif
END(mips_flush_dcache)
SWCACHE(flush_dcache)
	
/* 
 * void mips_flush_icache (void)
 *
 * Writeback and invalidate only the instruction caches.  
 * Joint caches (i.e. combined I & D) will be flushed too. 
 */
LEAF(mips_flush_icache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_flush_icache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_flush_icache
2:	j	r3k_flush_icache
#endif
END(mips_flush_icache)
SWCACHE(flush_icache)
	
/* 
 * void mips_clean_cache (vaddr_t va, unsigned int size)
 *
 * Writeback and invalidate a virtual address range in all caches.
 */
LEAF(mips_clean_cache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_clean_cache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_clean_cache
2:	j	r3k_clean_cache
#endif
END(mips_clean_cache)
SWCACHE(clean_cache)
	
/* 
 * void mips_clean_dcache (vaddr_t va, unsigned int size)
 *
 * Writeback and invalidate a virtual address range in data caches.
 * Joint caches (i.e. combined I & D) will be cleaned too. 
 */
LEAF(mips_clean_dcache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_clean_dcache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_clean_dcache
2:	j	r3k_clean_dcache
#endif
END(mips_clean_dcache)
SWCACHE(clean_dcache)
	
/* 
 * void mips_clean_icache (vaddr_t va, unsigned int size)
 *
 * Writeback and invalidate a virtual address range in instruction caches.
 * Joint caches (i.e. combined I & D) will be cleaned too. 
 */
LEAF(mips_clean_icache)
	GETPRID
#ifdef	R9000
	beq	t0,PRID_RM9000,1f
	j	ra
1:	j	rm7k_clean_icache
#else
	beq	t0,PRID_RM7000,1f
	beq	t0,PRID_R3900,2f
	j	ra
1:	j	rm7k_clean_icache
2:	j	r3k_clean_icache
#endif
END(mips_clean_icache)
SWCACHE(clean_icache)

#endif
