/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cm-api.c,v 1.25 2002/11/19 04:43:32 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

#ifndef SSC

#include <nfx-incl.h>
#include <sys/gt64240r.h>
#include <exports/hwMap.h>
#include <exports/cm-api.h>
#include <exports/gpp-defs.h>
#include "cm.h"
#include <sys/bmfpga.h>
#include <sys/led-api.h>

void cmGetCardTemp_bmfpga();

uchar8	* cmCPUStrings[4][NFX_MAX_CPUS_PER_SLOT] = {
    {
        CM_CPU_TYPE_SSC_STR,
        CM_CPU_TYPE_SSC_MGMT_STR,
        0, 0,
        0, 0, 0, 0, 
        0, 0, 0, 0, 
        0, 0, 0, 0 
    },
    {
        CM_CPU_TYPE_TXRX0_STR,
        CM_CPU_TYPE_TXRX1_STR,
        CM_CPU_TYPE_FP0_STR,
        CM_CPU_TYPE_FP1_STR,
        0, 0, 0, 0, 
        0, 0, 0, 0, 
        0, 0, 0, 0 
    },
    {
        CM_CPU_TYPE_TXRX0_STR,
        CM_CPU_TYPE_TXRX1_STR,
        CM_CPU_TYPE_FP0_STR,
        CM_CPU_TYPE_FP1_STR,
        0, 0, 0, 0, 
        0, 0, 0, 0, 
        0, 0, 0, 0 
    },
    {
        CM_CPU_TYPE_FC0_STR,
        CM_CPU_TYPE_FC1_STR,
        CM_CPU_TYPE_FC2_STR,
        CM_CPU_TYPE_FC3_STR,
        0, 0, 0, 0, 
        0, 0, 0, 0, 
        0, 0, 0, 0 
    }
};

cm_t   cm;


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
cmSetCardType(
	uint32	type
)
{
	cm.slot[nfxMySlotId].type = type;

	cm.slot[nfxMySlotId].num_cpus = cmGetNumCpus(cm.slot[nfxMySlotId].type);

#ifdef FCNIM
    // control cpu is 3 (fc slice 0)
	if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU)
		bmfpga_setCardType(type);
#endif

#ifdef NFP_TXRX
	bmfpga_setCardType(type);
#endif

#ifdef PROM
	if (CM_IS_FC_CTRL)
		bmfpga_setCardType(type);
	else if (isTXRX())
		bmfpga_setCardType(type);
#endif
}


void
cmInitCardType(uint32 type)
{
	extern void bmfpga_initCardType(uint32);

	cm.slot[nfxMySlotId].type = type;

	cm.slot[nfxMySlotId].num_cpus = cmGetNumCpus(cm.slot[nfxMySlotId].type);

#ifdef FCNIM
    // control cpu is 3 (fc slice 0)
	if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU)
		bmfpga_initCardType(type);
#endif

#ifdef NFP_TXRX
	bmfpga_initCardType(type);
#endif

#ifdef PROM
	if (CM_IS_FC_CTRL)
		bmfpga_initCardType(type);
	else if (isTXRX())
		bmfpga_initCardType(type);
#endif
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
cmSetCPUState(
	uint32	state
)
{
#ifdef FCNIM
    // control cpu is 3 (fc slice 0)
	if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU) {
		cm_fc_slice0_state = state;
		cmGpioCardStatePoll(0, 0);
	}
	else {
		volatile uint32	*   gpio_reg;
		uint32			    t;

		/*
		 * write state to GPIO bits
		 */
		gpio_reg = (volatile uint32 *)PHYS_TO_K1(HW_GALILEO_BASE+GPP_VALUE);
		t = *gpio_reg;
		t &= ~GPP_FC123_SLICE_STATE_MASK;
		t |= (state << GPP_FC123_SLICE_STATE_SHIFT);
		t &= GPP_FC123_SLICE_STATE_MASK;
		*gpio_reg = t;
	}
#endif

#ifdef NFP_TXRX
	cm_slot_status &= ~( BMFPGA_SLOT_STATUS_SLICE_STATE_MASK << (nfxMyCPUId*4));
	cm_slot_status |= (state << (nfxMyCPUId*4));
	bmfpgaSetCPUState(cm_slot_status);
    if (((cm_slot_status & 0xf) != CM_SLICE_STATE_RUNTIME) ||
       (((cm_slot_status >> 4) & 0xf) != CM_SLICE_STATE_RUNTIME) ||
       (((cm_slot_status >> 8) & 0xf) != CM_SLICE_STATE_RUNTIME) ||
       (((cm_slot_status >> 12) & 0xf) != CM_SLICE_STATE_RUNTIME))
    {
        led_set(LED_FAULT_ON);
        led_set(LED_OK_OFF);
    }
#endif

#ifdef NFP_FP
	cmCpuStateMsg_t	msg;

	msg.mid = CM_SET_INFO_RESP;
	msg.cpu = nfxMyCPUId;
	msg.state = state;
	if (cm_inited)
		eee_sendMessage(&msg, sizeof(cmCpuStateMsg_t), cm_txrx_port, cm_fp_port);
#endif

#ifdef PROM_R7000
	if (isFC()) {
        // control cpu is 3 (fc slice 0)
        if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU) {
            cm_fc_slice0_state = state;
            cmGpioCardStatePoll(0, 0);
        }
        else {
            volatile uint32	* gpio_reg;
            uint32			t;

            /*
             * write state to GPIO bits
             */
            gpio_reg = (volatile uint32 *)PHYS_TO_K1(HW_GALILEO_BASE+GPP_VALUE);
            t = *gpio_reg;
            t &= ~GPP_FC123_SLICE_STATE_MASK;
            t |= (state << GPP_FC123_SLICE_STATE_SHIFT);
            t &= GPP_FC123_SLICE_STATE_MASK;
            *gpio_reg = t;
        }
	}
#endif

#ifdef PROM_SIBYTE
	if (isTXRX())
		bmfpgaSetCPUState(state);
	else {
		cmCpuStateMsg_t	msg;

		msg.mid = CM_SET_INFO_RESP;
		msg.state = state;
/*
		if (cm_inited)
			eee_sendMessage(&msg, sizeof(cmCpuStateMsg_t), cm_txrx_port, cm_fp_port);
*/
	}
#endif

	cm.slot[nfxMySlotId].cpu[nfxMyCPUId].state = state;
}

/*-----------------------------------------------------------------
 * Name: cmGetCardTemp
 *
 * Description:
 *		 Read card temperature.
 *
 * Created by:	Tony Wong
 *
 * Date Created:	7/16/03
 *
 *-----------------------------------------------------------------
 */
void
cmGetCardTemp()
{
#ifdef SSC_MGMT
	extern int ssc_getCardTemp(uint32 *);
	uint32 temp = 0;

	if (ssc_getCardTemp(&temp) == 0) {
		cm.slot[nfxMySlotId].temp = temp;
		cm.slot[nfxMySlotId].temp_valid = 1;
	}
	else {
		cm.slot[nfxMySlotId].temp_valid = 0;
	}
#endif
#if defined(NFP_TXRX) || defined(FCNIM)
	cmGetCardTemp_bmfpga();
#endif
}

#if defined(NFP_TXRX) || defined(FCNIM)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmSetCardTemp(
	uint32	temp
)
{
	cm.slot[nfxMySlotId].temp = temp;

	bmfpga_setCardTemp(temp);

	cmGetCardTemp_bmfpga();

}

/*-----------------------------------------------------------------
 * Name: cmGetCardTemp_bmfpga
 *
 * Description:
 *
 *   Read card temperature.
 *
 * Created by:	Tony Wong
 *
 * Date Created:	7/18/03
 *
 *-----------------------------------------------------------------
 */
void
cmGetCardTemp_bmfpga()
{
	extern int bmfpga_getCardTemp(uint32 *);
	uint32 temp = 0;

	if (bmfpga_getCardTemp(&temp) == 0) {
		cm.slot[nfxMySlotId].temp = temp;
		cm.slot[nfxMySlotId].temp_valid = 1;
		bmfpga_setCardTemp(temp);
	}
	else {
		cm.slot[nfxMySlotId].temp_valid = 0;
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmSetCardPower(
	uint32	power
)
{
	cm.slot[nfxMySlotId].power = power;

	bmfpga_setCardPower(power);
}
#endif	/* NFP_TXRX || FCNIM */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
int32
cmGetCardType(
	uint32	slot
)
{
	if (slot >= CM_NUM_SLOTS)
		return -1;

	return (cm.slot[slot].type);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int32
cmGetCPUState(
	uint32	slot,
	uint32	cpu
)
{
	if (slot >= CM_NUM_SLOTS)
		return -1;

	if (cpu >= cm.slot[slot].num_cpus)
		return -1;

	return (cm.slot[slot].cpu[cpu].state);
}

/*-----------------------------------------------------------------
 * Name:    cmGetCpuString()
 *
 * Description:
 *
 * Created by:  Rick Lund
 *
 * Date Created:    6/20/01
 *
 *-----------------------------------------------------------------
 */
uchar8  *
cmGetCpuString(
	uint32	slot_num,
	uint32	cpu_num
)
{
	uint32  card_type;

	card_type = cmGetCardType(slot_num);
	return (cmCPUStrings[card_type][cpu_num]);

}	/* cmGetCpuString() */

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmRebootCard(
	uint32	slot_num
)
{
	uint32	cpu;

	if (isSSCmgmt()) {
		bmfpgaResetCard(slot_num);
		cm.slot[slot_num].power_valid = 0;
		cm.slot[slot_num].temp_valid = 0;
		for (cpu = 0; cpu < CM_NUM_CPUS; cpu++) {
			cm.slot[slot_num].cpu[cpu].state = 0;
		}
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmRebootCPU(
	uint32	slot_num,
	uint32	cpu_num
)
{
	if (isSSCmgmt()) {
		if (cm.slot[slot_num].type == CM_CARD_TYPE_FCNIM)
			bmfpgaResetCPU(slot_num, cpu_num);
		else
			bmfpgaResetCard(slot_num);
	}
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int32
cmGetNumCpus(
	uint32	board_type
)
{
	if (board_type == BMFPGA_SLOT_STATUS_BOARD_TYPE_SSC)
		return 2;
	else if (board_type == BMFPGA_SLOT_STATUS_BOARD_TYPE_GARFIELD)
		return 4;
	else if (board_type == BMFPGA_SLOT_STATUS_BOARD_TYPE_CHEETAH)
		return 4;
	else if (board_type == BMFPGA_SLOT_STATUS_BOARD_TYPE_FCNIM)
		return 4;
	else
		return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
bmfpgaGetVersion()
{
	extern uchar8	bmfpga_str[];

	return &(bmfpga_str[0]);
}

#ifdef PROM
void
pmon_cmSetCPUStatePromNoAutoboot()
{
	cmSetCPUState(CM_SLICE_STATE_PROM_NO_AUTOBOOT);
}

void
pmon_cmSetCPUStateCrashed()
{
	cmSetCPUState(CM_SLICE_STATE_PROM_CRASHED);
}

void
pmon_cmSetCPUStateReboot()
{
	bmfpgaSetCPUState(	(CM_SLICE_STATE_RUNTIME_REBOOT << 0) |
						(CM_SLICE_STATE_RUNTIME_REBOOT << 4) |
						(CM_SLICE_STATE_RUNTIME_REBOOT << 8) |
						(CM_SLICE_STATE_RUNTIME_REBOOT << 12));
}

void
pmon_cmSetCPUStateResetWait()
{
	cmSetCPUState(CM_SLICE_STATE_RESET_WAIT);
}
#endif

#ifdef SSC_MGMT
/* 
 * Poll this function to check if chassis fan failed. 
 * return 1 if fan failed else return 0 
 */
int
cm_checkFanFail()
{
    uint32  gpp;

    gpp = *(volatile uint32 *)(HW_BMFPGA_BASE + 0x10);

    if (gpp & 0x00004003) // fan 0 absent, temp alarm, rotor locked
		return 1;
    else
		return 0;
}

/* 
 * Poll this function to check if power supply failed. 
 * return 1 if fan failed else return 0 
 */
int
cm_checkPowerFail()
{
    uint32  gpp;

    gpp = *(volatile uint32 *)(HW_BMFPGA_BASE + 0x10);

    if (gpp & 0x00000080)  // check for fault if power 0 present 
		;
	else
		if (gpp & 0x00000100)  // fault ?
			return 1;
	
	if (gpp & 0x00000400)  // check for fault if power 1 present
		;
	else
		if (gpp & 0x00000800)  // fault ?
			return 1;
	return 0;
}

/*
 * Set LEDs from ssc-mgmt
 */
void sscm_led_set(int led_state)
{
	volatile uint32 gpp_value;

    gpp_value = *(volatile uint32 *)(HW_GALILEO_BASE + GPP_VALUE);
    if (led_state == LED_OK_ON) {
        gpp_value &= ~0x30000000;
        gpp_value |= 0x20000000;
    }
    if (led_state == LED_OK_OFF) {
        gpp_value &= ~0x30000000;
    }
    if (led_state == LED_FAULT_ON) {
        gpp_value &= ~0xC0000000;
        gpp_value |= 0x40000000;
    }
    if (led_state == LED_FAULT_OFF) {
        gpp_value &= ~0xC0000000;
    }
    if (led_state == LED_FAN_GREEN) {
        gpp_value &= ~0x03000000;
        gpp_value |= 0x02000000;
    }
    if (led_state == LED_FAN_RED) {
        gpp_value &= ~0x03000000;
        gpp_value |= 0x01000000;
    }
    *(volatile uint32 *)(HW_GALILEO_BASE + GPP_VALUE) = gpp_value;
}

#endif

#endif
