/*
 *-----------------------------------------------------------------
 *
 * Name:	cm-init.c
 *
 * RCS:  $Id: cm-init.c,v 1.30 2002/12/04 01:46:14 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

#ifndef SSC

#include <nfx-incl.h>
#if !defined(R9000)
#include <sys/gt64240r.h>
#else
#include <sys/gt64440r.h>
#endif
#include <sys/bmfpga.h>
#include <exports/eee-api.h>
#include <exports/hwMap.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include <exports/gpp-defs.h>
#include "cm.h"
#if !defined(PROM) && !defined(FCNIM_TM)
#include "exports/event-api.h"
#endif  /* !PROM && !FCNIM_TM */
#ifdef SIBYTE
#include <exports/smp-api.h>
#endif

#ifdef SSC_MGMT
#include <sys/tx3927.h>
static void cmGetInitInfo(void);
void systemUptimeTimer();
extern chassis_heartbeat_t hb;
#endif

int32	cm_appid;
uint32	cm_txrx_port;
uint32	cm_fp_port;
uint32	cm_fc0_port;
uint32	cm_fc1_port;
uint32	cm_fc2_port;
uint32	cm_fc3_port;
uint32	cm_ssc_port;
uint32	cm_src_port;

#ifdef NFP_TXRX
#define HEARTBEAT_TIMEOUT_MAX   20 // 20sec: each count corresponds to 1 timeout of a 1s timer 
uint32 heartbeatTimeout_counter;
int32	do_reboot();
#endif

#if defined (NFP_TXRX) || defined (SSC_MGMT)
int32 heartbeat_timer;
#if defined (SSC_MGMT)
uint32 start_heartbeat=1;
#else
uint32 start_heartbeat=0;
#endif
#endif

/*-----------------------------------------------------------------
 * Name:	cm_init()
 *
 * Description:
 *				Toshiba	: send version strings to chassisd
 *							send info req message to TXRX and FC0 (via bmfpga)
 *				TXRX	: send version strings to chassisd
 *							write cpu state, temp, power, and board type to
 *							bmfpga
 *				FP		: send version strings to chassisd
 *							send cpu state to TXRX
 *				FCNIM	: send version strings to chassisd
 *							FC0 : write cpu state, temp, power, and board type to
 *									bmfpga
 *							FC1,2,3 : write cpu state to gpio bits (wired to FC0)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
cm_init(
	int32	pass
)
{
	uint32	card_type;
#ifdef SSC_MGMT
	int32	fan_poll_timer;
#endif

	if (pass == 0) {
		nfxMyCPUId = cmGetMyCPUId();
#ifdef SIBYTE
		nfxMyCoreId = sb1250_getCoreId();
#else
		nfxMyCoreId = nfxMyCPUId;
#endif
		nfxMySlotId = cmGetMySlotId();
		return;
	}

	card_type = cmGetMyCardType();

	if (pass == 1) {

		nfxSscSlotId = 0;

		nfxMyCPUId = cmGetMyCPUId();
#ifdef SIBYTE
		nfxMyCoreId = sb1250_getCoreId();
#else
		nfxMyCoreId = nfxMyCPUId;
#endif
		nfxMySlotId = cmGetMySlotId();

#ifdef PROM_R7000
		if (CM_IS_FC_CTRL) {        // control cpu is 3 (fc slice 0)
			watchdog_kick();
			delay_us(1000000*5);
			watchdog_kick();
		}
#endif

		bmfpga_init();

		cmInitCardType(card_type);

#ifdef PROM_R9000
		// Disable all-on bit in BM-FPGA
		BMFPGA_WRITE(BMFPGA_REG_STATUS_CTRL, 0);

		// Init fan & temp controllers
		cmInitFan(I2C_ADDR_FAN_CTRL_0);
		cmInitFan(I2C_ADDR_FAN_CTRL_1);
		cmTempEna(I2C_ADDR_TEMP_SENSOR);
#endif

#ifdef PROM
		cmSetCPUState(CM_SLICE_STATE_PROM_INIT);
#else
		cmSetCPUState(CM_SLICE_STATE_RUNTIME_INIT);
#endif

		cm_appid = eee_allocateApplicationId("cm");

		/*
		 * create dest port words
		 */
		cm_txrx_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_NFP, NFX_NFP_TXRX1, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);
		cm_fp_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_NFP, NFX_NFP_FP2, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);
		cm_fc0_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID0, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);
		cm_fc1_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID1, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);
		cm_fc2_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID2, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);
		cm_fc3_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID3, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);
		cm_ssc_port = PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_SSC, NFX_SSC_UNIX, cm_appid,
												EEE_MGMT, NFX_COS_HIGH,
												PORT_BEST_FIT);

		if (isTXRX())
			cm_src_port = cm_txrx_port;
		else if (isFP())
			cm_src_port = cm_fp_port;
		else if (isFC()) {
			if (nfxMyCPUId == NFX_FCNIM_CPUID3)
				cm_src_port = cm_fc0_port;      // cpuid3 is slice fc0
			else if (nfxMyCPUId == NFX_FCNIM_CPUID2)
				cm_src_port = cm_fc1_port;      // cpuid2 is slice fc1
			else if (nfxMyCPUId == NFX_FCNIM_CPUID1)
				cm_src_port = cm_fc2_port;      // cpuid1 is slice fc2
			else if (nfxMyCPUId == NFX_FCNIM_CPUID0)
				cm_src_port = cm_fc3_port;      // cpuid0 is slice fc3
		}
		else if (isSSC())
			cm_src_port = cm_ssc_port;
	}
	else {
		cmSendVersionResp(0, 0);

#ifdef SSC_MGMT
		cmGetInitInfo();
#ifndef PROM
		cmSendCpuEvent(nfxMySlotId, NFX_SSC_UNIX, EVENT_STATE_UP);
		cmSendCpuEvent(nfxMySlotId, NFX_SSC_MGMT, EVENT_STATE_UP);
#endif  /* !PROM */
		cmSendInfoResp(0, 0);

		cmSendInfoReq();

		bmfpga_clearPendingInts();

		eee_registerPollingFunc(EEE_HIGH_PRIORITY, cm_poll_bmfpga_intr, NULL);


#if defined(SSC_MGMT)		
		eee_registerPollingFunc(EEE_HIGH_PRIORITY, cm_poll_module_present,
								NULL);
		{
		volatile tx3927_intcntrl_t *int_cntrl =
							(tx3927_intcntrl_t *)TX3927_INT_CONTROL_BASE;
		int_cntrl->der = 0x1;
		}
#endif		
		
		fan_poll_timer = eee_timerCreate(10000, EEE_TIMER_REPEAT);
		eee_timerStart(fan_poll_timer, (void *)cm_checkFanStatus, 0, 0, 0, 0);
#endif

#ifdef NFP_TXRX
		cmSetCardTemp(0x11);
		cmSetCardPower(1);

		eee_registerPollingFunc(EEE_LOW_PRIORITY, cmGpioCardStatePoll,NULL);

//		intr_registerHandler(0, cm_intrHandleStatusChange);			/* TODO */
//		intr_registerHandler(1, cm_intrHandleMessageChange);		/* TODO */
		

		/* 
		 * Heartbeat Feature 
		 */ 

		heartbeatTimeout_counter = 0; /* Initialise the counter to 0 */
		cm_heartbeatInit();	
#endif

#ifdef FCNIM
		if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU) {    // cpu 3 is fc slice 0
			cmSetCardTemp(0x11);
			cmSetCardPower(1);

			eee_registerPollingFunc(EEE_LOW_PRIORITY, cmGpioCardStatePoll,NULL);

//			intr_registerHandler(0, cm_intrHandleStatusChange);		/* TODO */
//			intr_registerHandler(1, cm_intrHandleMessageChange);	/* TODO */
		}
#endif

#ifdef PROM_R7000
		if (CM_IS_FC_CTRL) {        // control cpu is 3 (fc slice 0)
			eee_registerPollingFunc(EEE_LOW_PRIORITY, cmGpioCardStatePoll,NULL);
		}
#endif

#ifdef PROM_SIBYTE
		if (isTXRX()) {
            if (nfxMyCPUId == NFX_NFP_TXRX1) {
                eee_registerPollingFunc(EEE_LOW_PRIORITY, cmGpioCardStatePoll,NULL);
            }
		}
#endif

		/*
		 * Register CM Application Receive Function
		 */
		eee_registerReceiveFunc(cm_appid, cm_rcvMsg, cm_rcvMsg);

		cm_inited = 1;
#if defined(SSC_MGMT)		
		/* Start the timers that keep track of board life */
        systemUptimeTimer();

		/* 
		 * Heartbeat Feature 
		 */ 
		cm_heartbeatInit();	
#endif			
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
uint32
cmGetMySlotId(void)
{
	if (isSSC() || isSSCmgmt())
		return NFX_SLOTTYPE_SSC;
	if (isTXRX() || isFP())
		return NFX_SLOTTYPE_NFP;
	if (isFC())
		return NFX_SLOTTYPE_FC;

    panic("cmGetMySlotId: Unknown\n");
    return NFX_SLOTTYPE_UNKNOWN;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
uint32
cmGetMyCPUId(void)
{
#ifdef R7000
	if (isSSC())
		return NFX_SSC_UNIX;

    if (isFC()) {
        volatile uint32 id;

        id = ((*(volatile uint32 *)
                PHYS_TO_K1(HW_GALILEO_BASE + GPP_VALUE) >> GPP_FC_SLICE_SHIFT) & 0x3);
        return(CM_FC_SLICE2CPUID(id));
    }
#endif

#ifdef SIBYTE
	if (isTXRX()) {
		uint32 prid;
		prid = _mips_mfc0(C0_PRID);
		return (NFX_NFP_TXRX1 + ((prid >> 25) & 0x7));
	}
	if (isFP()) {
		uint32 prid;
		prid = _mips_mfc0(C0_PRID);
		if (((prid >> 25) & 0x7) == 0)
			return NFX_NFP_FP1;
		else
			return NFX_NFP_FP2;
	}
#endif

#ifdef SSC_MGMT
	return (NFX_SSC_MGMT);
#endif

    panic("cmGetMyCPUId: Unknown\n");
    return NFX_CPUID_UNKNOWN;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
uint32
cmGetMyCardType(void)
{
#ifdef NFX_MOD_SSC
	return (CM_CARD_TYPE_SSC);
#endif

#ifdef NFX_MOD_NFP
	if (isCheetah())
		return (CM_CARD_TYPE_CHEETAH);
	else
		return (CM_CARD_TYPE_GARFIELD);
#endif

#ifdef NFX_MOD_FCNIM
	return (CM_CARD_TYPE_FCNIM);
#endif

#ifdef PROM
#ifdef SIBYTE
	if (isCheetah())
		return (CM_CARD_TYPE_CHEETAH);
	else
		return (CM_CARD_TYPE_GARFIELD);
#else
	if (isSSC())
		return (CM_CARD_TYPE_SSC);
	else
		return (CM_CARD_TYPE_FCNIM);
#endif
#endif
}

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
static void
cmGetInitInfo(void)
{
	int32	slot;

	cm.slot[0].present = 1;		/* SSC_SLOT */
	cm.slot[0].type = NFX_CARDTYPE_SSC;
	cm.slot[0].temp_valid = 0;
	cm.slot[0].power_valid = 0;
	cm.slot[0].num_cpus = 2;
	cm.slot[0].cpu[0].state = CM_SLICE_STATE_RUNTIME;
	cm.slot[0].cpu[1].state = CM_SLICE_STATE_RUNTIME;

	for (slot=1;slot<CM_NUM_SLOTS;slot++) {
		if (bmfpgaIsModulePresent(slot)) {
			cm.slot[slot].present = 1;
		}
		else
			cm.slot[slot].present = 0;
		cm.slot[slot].control = 0xFF;
	}
}
#endif	/* SSC_MGMT */

#if defined (NFP_TXRX) || defined (SSC_MGMT)
/*-----------------------------------------------------------------
 * Name: cm_heartbeatInit()
 *
 * Description: Creates the timer. Every timeout will result
 *              TOSHIBA: in a chassis status to be sent to the TXRX.
 *              TXXR: Process the chassis info
 *
 * Created by:	lipika
 *
 * Date Created: 6/12/03
 *
 *-----------------------------------------------------------------
 */
void
cm_heartbeatInit()
{	
	/*
	 * Create the heartbeat timer
	 */
	heartbeat_timer = eee_timerCreate(MSEC_PER_SEC, EEE_TIMER_ONCE);
	if(heartbeat_timer == TIMER_ERR_NO_TIMER_AVAILABLE)
		printf("ERROR: Could not create heartbeat timer.\n");

	/*
	 * Start the heartbeat timer
	 */
	cm_Heartbeat(heartbeat_timer);
}	

/*-----------------------------------------------------------------
 * Name: cm_Heartbeat()
 *
 * Description: When chassisd starts, sends chassis status message to TXRX 
 *
 * Created by:	lipika
 *
 * Date Created: 6/12/03
 *
 *-----------------------------------------------------------------
 */
void
cm_Heartbeat(int timer)
{
	uint32 rct=0;
//printf("cm_Heartbeat: starting timer \n");
	/* Every timeout, send a msg to the TXRX with the cpu status
	 * of each card
	 */ 
	rct = eee_timerStart(timer, (void *)cm_heartbeat_timeout,0,0,0,0);

	if(rct != NFX_OK)
		printf("Heartbeat:Could not start timer %d. Error %08x \n",timer, rct);
}	

/*-----------------------------------------------------------------
 * Name: cm_heartbeat_timeout()
 *
 * Description: every timeout, send msg to FP with cpu status
 *
 * Created by:	lipika
 *
 * Date Created: 6/12/03
 *
 *-----------------------------------------------------------------
 */
void
cm_heartbeat_timeout(int32 tmr_handle,
         void * usr_param0,
         void * usr_param1,
         void * usr_param2,
         void * usr_param3)
{
//printf("start_heartbeat %d , cm_heartbeat_timeout\n", start_heartbeat);	

#ifdef SSC_MGMT		

	/* If feature is enabled */
	if(start_heartbeat) {
		/* Check if the SSC has completed its initialisation */
		if(hb.chassisdUp) 
		{
			cm_heartbeatSendInfo();
		}
	}	
#endif	

#ifdef NFP_TXRX

	/* If feature is enabled */
	if(start_heartbeat) {
		/* Increment the count for not receiving the Heartbeat 
	 	 * Once count surpasses the limit, send a msg to the module
	 	 * to start filer takeover.
	 	 */ 
		heartbeatTimeout_counter++;
		if(heartbeatTimeout_counter > HEARTBEAT_TIMEOUT_MAX)
		{
			/* Reboot the FP */
			printf("cm_heartbeat_timeout: Did not rcv heartbeat from Toshiba for %d sec\n",HEARTBEAT_TIMEOUT_MAX);
			printf("Rebooting self......\n");
			do_reboot();
		}		
	}	
#endif	

	/*
	 * Restart timer
	 */  	
	cm_Heartbeat(tmr_handle);
}	

/*-----------------------------------------------------------------
 * Name: heartbeat_cmd()
 *
 * Description: to allow user to enable/disable feature
 *
 * Created by:	lipika
 *
 * Date Created: 6/12/03
 *
 *-----------------------------------------------------------------
 */
int32
heartbeat_cmd(
	int32	argc,
	uchar8	* argv[]
)
{
	if (argc != 2)
		return -1;

	if (!strcmp(argv[1], "on"))
		start_heartbeat = 1;
	else if (!strcmp(argv[1], "off"))
		start_heartbeat = 0;
	else
		return -1;
	return 0;
}
#endif

#ifdef PROM
uchar8 *
getPromImageVerString(void)
{
	extern uchar8	ImageVerString[];

	return ImageVerString;
}
#endif
#endif	/* !SSC */

