/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-rpc-ed-srvsvc.c,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 *
 * Name			: cifs-rpc-ed-srvsvc.c
 *
 * Description	: CIFS DCE/RPC server services encode & decode functions. 
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/14/2002
 *
 *-----------------------------------------------------------------
 */

#ifdef FS_STANDALONE
#include "../sm-cifs/cifs-api.h"
#else
#include <exports/cifs-api.h>
#endif

#include "cifs-rpc.x"
#include "cifs-rpc-srvsvc.h"
#include "cifs-rpc-ed-srvsvc.h"
#include "cifs-rpc-ed-sec.h"


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi1()
 *
 * Description  : SRVSVC share info level 1 integers encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi1(cifsrpc_ed *edp, shareInfoLv1 *i1p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi1", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuint32  (edp, &i1p->netNamep,	"netNamep", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &i1p->type,		"type    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &i1p->remarkp,	"remarkp ", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi1Strings()
 *
 * Description  : SRVSVC share info level 1 strings encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi1Strings(cifsrpc_ed *edp, shareInfoLv1 *i1p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi1Strings", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &i1p->netName,	"netName",  lv) != NFX_OK
	 || cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &i1p->remark,	"remark ",  lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi2()
 *
 * Description  : SRVSVC share info level 2 integers encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi2(cifsrpc_ed *edp, shareInfoLv2 *i2p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi2", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign (edp)										!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->netNamep,	"netNamep ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->type,		"type     ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->remarkp,		"remarkp  ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->perms,		"perms    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->nbUsesMax,	"nbUsesMax", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->nbUses,		"nbUses   ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->pathp,		"pathp    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i2p->passwordp,	"passwordp", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi2Strings()
 *
 * Description  : SRVSVC share info level 2 strings encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi2Strings(cifsrpc_ed *edp, shareInfoLv2 *i2p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi2Strings", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &i2p->netName,	"netName",	lv) != NFX_OK
	 || cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &i2p->remark,	"remark ",	lv) != NFX_OK
	 || cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &i2p->path,		"path   ",	lv) != NFX_OK
	 || cifsrpcEDalign   (edp)									!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &i2p->password,	"passwd ",	lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi502()
 *
 * Description  : SRVSVC share info level 2 integers encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi502(cifsrpc_ed *edp, shareInfoLv502 *i502p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi502", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign (edp)										!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->netNamep,	 "netNamep ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->type,		 "type     ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->remarkp,	 "remarkp  ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->perms,		 "perms    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->nbUsesMax,	 "nbUsesMax", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->nbUses,	 "nbUses   ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->pathp,		 "pathp    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->passwordp,	 "passwordp", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->secDescSize,"sDescSize", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i502p->secDescp,	 "secDescp ", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi502Strings()
 *
 * Description  : SRVSVC share info level 502 strings encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi502Strings(cifsrpc_ed *edp, shareInfoLv502 *i502p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi502Strings", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign(edp) 									!= NFX_OK
	 || (i502p->netNamep										!= 0 
	  && cifsrpcEDuniStruc(edp, &i502p->netName, "netName",	lv) != NFX_OK)
	 || cifsrpcEDalign(edp)										!= NFX_OK
	 || (i502p->remarkp											!= 0
	  && cifsrpcEDuniStruc(edp, &i502p->remark,	 "remark ",	lv)	!= NFX_OK)
	 || cifsrpcEDalign(edp)										!= NFX_OK
	 || (i502p->pathp											!= 0
      && cifsrpcEDuniStruc(edp, &i502p->path,	 "path   ", lv)	!= NFX_OK)
	 || cifsrpcEDalign(edp)										!= NFX_OK
	 || (i502p->passwordp										!= 0
      && cifsrpcEDuniStruc(edp, &i502p->password,"passwd ", lv)	!= NFX_OK)
	 || cifsrpcEDalign(edp)										!= NFX_OK
	 || (i502p->secDescp										!= 0
	  && (cifsrpcEDuint32(edp, &i502p->sdSize2,	 "sdSize2", lv)	!= NFX_OK
	   || (CIFSRPC_ENCODING(edp)
		&& cifsrpcEDsecDesc(edp, i502p->secDesc.sdp,"share",lv) != NFX_OK)
	   || (CIFSRPC_DECODING(edp)
		&& cifsrpcEDsecDesc(edp, &i502p->secDesc.sd,"share",lv) != NFX_OK))))
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi1501()
 *
 * Description  : SRVSVC share info level 1501 integers encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/22/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi1501(cifsrpc_ed *edp, shareInfoLv1501 *i1501p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi1501", cifs_spaces(lv)));
	lv++;
	return (cifsrpcEDsdBuf(edp, i1501p, "share", lv));
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshiContr()
 *
 * Description  : SRVSVC share info container encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshiContr(cifsrpc_ed *edp, shareEnumInfoContainer *p, uchar8 lv)
{
	shareInfoLv1	*i1p;
	shareInfoLv2	*i2p;
	shareInfoLv502	*i502p;
	uint32 i, n;

	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshiContr", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"info level ", lv)	!= NFX_OK)
		return (NFX_ERR);
	
	if (p->level == 0) return (NFX_OK);
	
	if (cifsrpcEDuint32  (edp, &p->switchValue,	"switchValue", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->shareInfop,	"shareInfop ", lv)	!= NFX_OK)
		return (NFX_ERR);
	
	if (p->shareInfop  == 0) return (NFX_OK);

	if (cifsrpcEDuint32  (edp, &p->entriesRead,	"entriesRead", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->entriesp,	"entriesp   ", lv)	!= NFX_OK)
		return (NFX_ERR);
	
	if (p->entriesp  == 0) 
	{
		if (p->entriesRead == 0)
			return (NFX_OK);
		else
			return (NFX_ERR);
	}

	if (cifsrpcEDuint32  (edp, &p->nbEntries,	"nbEntries  ", lv)	!= NFX_OK
	 || p->entriesRead != p->nbEntries)
		return (NFX_ERR);

	n = p->nbEntries;
	switch (p->switchValue)
	{
		case 1:
			if (CIFSRPC_DECODING(edp))
	        {
				i1p = (shareInfoLv1*)
					cifsrpc_allocMemItem(edp->memLp, n * sizeof(shareInfoLv1));
				if (i1p == NULL)
					return (NFX_ERR);
				p->info.lv1p = i1p;
			} 
			else i1p = p->info.lv1p;
			
			for (i=0; i<n; i++)
				if (cifsrpcEDshi1(edp, i1p + i, lv) != NFX_OK)
					return (NFX_ERR);
			
			for (i=0; i<n; i++)
				if (cifsrpcEDshi1Strings(edp, i1p + i, lv) != NFX_OK) 
					return (NFX_ERR);
			
			break;
		
		case 2:
			if (CIFSRPC_DECODING(edp))
	        {
				i2p = (shareInfoLv2*)
					cifsrpc_allocMemItem(edp->memLp, n * sizeof(shareInfoLv2));
				if (i2p == NULL)
					return (NFX_ERR);
				p->info.lv2p = i2p;
			} 
			else i2p = p->info.lv2p;
			
			for (i=0; i<n; i++)
				if (cifsrpcEDshi2(edp, i2p + i, lv) != NFX_OK)
					return (NFX_ERR);
			
			for (i=0; i<n; i++)
				if (cifsrpcEDshi2Strings(edp, i2p + i, lv) != NFX_OK) 
					return (NFX_ERR);
			
			break;
		
		case 502:
			if (CIFSRPC_DECODING(edp))
	        {
				i502p = (shareInfoLv502*)cifsrpc_allocMemItem(edp->memLp,
													n * sizeof(shareInfoLv502));
				if (i502p == NULL)
					return (NFX_ERR);
				p->info.lv502p = i502p;
			} 
			else i502p = p->info.lv502p;
			
			for (i=0; i<n; i++)
				if (cifsrpcEDshi502(edp, i502p + i, lv) != NFX_OK)
					return (NFX_ERR);
			
			for (i=0; i<n; i++)
				if (cifsrpcEDshi502Strings(edp, i502p + i, lv) != NFX_OK) 
					return (NFX_ERR);
			
			break;
		

	}
			

	return (NFX_OK);
}



/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareEnumCall()
 *
 * Description  : SRVSVC NetShareEnum function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareEnumCall(cifsrpc_ed *edp, netShareEnumCall *p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareEnumCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->serverNamep,	"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->serverName,	"srvName",		lv) != NFX_OK
	 || cifsrpcEDshiContr(edp, &p->shiContr, lv)					!= NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->prefMaxLen,	"prefMaxLen ",	lv)	!= NFX_OK
	 || cifsrpcEDenumHd  (edp, &p->resumeHandle,"resumeHandle", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareEnumResp()
 *
 * Description  : SRVSVC NetShareEnum function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareEnumResp(cifsrpc_ed *edp, netShareEnumResp *p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareEnumResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDshiContr(edp, &p->shiContr, lv)					!= NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->totalEntries,"totalEntries", lv) != NFX_OK
	 || cifsrpcEDenumHd  (edp, &p->resumeHandle,"resumeHandle", lv) != NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue ", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDshi()
 *
 * Description  : SRVSVC share info encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDshi(cifsrpc_ed *edp, shareInfo *p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->switchValue,	"switchValue", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->shareInfop,	"shareInfop ", lv)	!= NFX_OK)
		return (NFX_ERR);
	
	if (p->shareInfop  == 0) return (NFX_OK);

	switch (p->switchValue)
	{
		case 1:
			if (cifsrpcEDshi1(edp, &p->info.lv1, lv) != NFX_OK
			 || cifsrpcEDshi1Strings(edp, &p->info.lv1, lv) != NFX_OK) 
				return (NFX_ERR);
			break;
		
		case 2:
			if (cifsrpcEDshi2(edp, &p->info.lv2, lv) != NFX_OK
			 || cifsrpcEDshi2Strings(edp, &p->info.lv2, lv) != NFX_OK) 
				return (NFX_ERR);
			break;
		
		case 502:
			if (cifsrpcEDshi502(edp, &p->info.lv502, lv) != NFX_OK
			 || cifsrpcEDshi502Strings(edp, &p->info.lv502, lv) != NFX_OK) 
				return (NFX_ERR);
			break;
		case 1501:
			if (cifsrpcEDshi1501(edp, &p->info.lv1501, lv) != NFX_OK)
				return (NFX_ERR);
			break;
	}

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareGetInfoCall()
 *
 * Description  : SRVSVC NetShareGetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareGetInfoCall(	cifsrpc_ed *edp,
								netShareGetInfoCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareGetInfoCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->svNamep,		"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->svName,		"serverName",	lv) != NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->shareName,	"shareName",	lv) != NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"level      ",	lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareGetInfoResp()
 *
 * Description  : SRVSVC NetShareGetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareGetInfoResp(	cifsrpc_ed *edp,
								netShareGetInfoResp *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareGetInfoResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign	(edp)										!= NFX_OK
	 || cifsrpcEDshi	(edp, &p->shareInfo, lv)					!= NFX_OK
	 || cifsrpcEDuint32	(edp, &p->returnValue,	"returnValue ", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareSetInfoCall()
 *
 * Description  : SRVSVC NetShareSetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareSetInfoCall(	cifsrpc_ed *edp,
								netShareSetInfoCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareSetInfoCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->svNamep,		"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->svName,		"serverName",	lv) != NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->shareName,	"shareName",	lv) != NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"level      ",	lv)	!= NFX_OK
	 || cifsrpcEDshi	 (edp, &p->shareInfo, lv)					!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareSetInfoResp()
 *
 * Description  : SRVSVC NetShareSetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareSetInfoResp(	cifsrpc_ed *edp,
								netShareSetInfoResp *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareSetInfoResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->unknown,		"unknown    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareAddCall()
 *
 * Description  : SRVSVC NetShareAdd function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareAddCall(	cifsrpc_ed *edp,
								netShareAddCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareAddCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->svNamep,		"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->svName,		"serverName",	lv) != NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"level      ",	lv)	!= NFX_OK
	 || cifsrpcEDshi	 (edp, &p->shareInfo, lv)					!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareAddResp()
 *
 * Description  : SRVSVC NetShareAdd function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareAddResp(	cifsrpc_ed *edp,
							netShareAddResp *p, 
							uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareAddResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->unknown,		"unknown    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareDelCall()
 *
 * Description  : SRVSVC NetShareDel function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareDelCall(	cifsrpc_ed *edp,
								netShareDelCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareDelCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->svNamep,		"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->svName,		"serverName",	lv) != NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->shareName,	"shareName",	lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareDelResp()
 *
 * Description  : SRVSVC NetShareDel function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareDelResp(	cifsrpc_ed *edp,
								netShareDelResp *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetShareDelResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign	(edp)										!= NFX_OK
	 || cifsrpcEDuint32	(edp, &p->returnValue,	"returnValue ", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}






/*-----------------------------------------------------------------
 * Name         : cifsrpcEDsviContr()
 *
 * Description  : SRVSVC server info encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDsviContr(cifsrpc_ed *edp, serverInfoContainer *p, uchar8 lv)
{
	serverInfoLv100 *i100p;
	serverInfoLv101 *i101p;
	serverInfoLv102 *i102p;
	uint32 i, n;

	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%scifsrpcEDshi", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign (edp)										!= NFX_OK
	 || cifsrpcEDuint32(edp, &p->switchValue,	"switchValue", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &p->serverInfop,	"serverInfop", lv)	!= NFX_OK)
		return (NFX_ERR);

	if (p->serverInfop == 0 || p->switchValue == 0)
		return (NFX_OK);
	
	CIFS_PRINT(CIFS_DBGLV_DBG3, 
				("%scifsrpcEDsvi%u", cifs_spaces(lv), p->switchValue));
	lv++;
	//Treat the common part of info level 100, 101 and 102
	i100p = &p->info.lv100;
	if (cifsrpcEDuint32(edp, &i100p->platformId,"platformId", lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &i100p->namep,		"namep     ", lv)	!= NFX_OK)
		return (NFX_ERR);
	
	//Treat the specific part of the info levels
	switch (p->switchValue)	{
	case 100:
		if (cifsrpcEDuniStruc(edp, &i100p->name, "name   ", lv) != NFX_OK)
			return (NFX_ERR);
		break;
	case 101:
		i101p = &p->info.lv101;
		if (cifsrpcEDuint32  (edp, &i101p->verMajor,"verMajor  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i101p->verMinor,"verMinor  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i101p->type,	"type      ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i101p->commentp,"commentp  ", lv) != NFX_OK
		 || cifsrpcEDuniStruc(edp, &i101p->name,	"name   ",	  lv) != NFX_OK
		 || cifsrpcEDuniStruc(edp, &i101p->comment,	"comment",	  lv) != NFX_OK)
			return (NFX_ERR);
		break;
	case 102:
		i102p = &p->info.lv102;
		if (cifsrpcEDuint32  (edp, &i102p->verMajor,"verMajor  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->verMinor,"verMinor  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->type,	"type      ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->commentp,"commentp  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->users,	"users     ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->disc,	"disc      ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->hidden,	"hidden    ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->announce,"announce  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->annDelta,"annDelta  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->licenses,"licenses  ", lv) != NFX_OK
		 || cifsrpcEDuint32	 (edp, &i102p->usrPathp,"usrPathp  ", lv) != NFX_OK
		 || cifsrpcEDuniStruc(edp, &i102p->name,	"name   ",	  lv) != NFX_OK
		 || cifsrpcEDuniStruc(edp, &i102p->comment,	"comment",	  lv) != NFX_OK
		 || cifsrpcEDuniStruc(edp, &i102p->usrPath,	"usrPath",	  lv) != NFX_OK)
			return (NFX_ERR);
		break;

	}
			
	if (cifsrpcEDalign(edp) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerGetInfoCall()
 *
 * Description  : SRVSVC NetServerGetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerGetInfoCall(	cifsrpc_ed *edp,
								netServerGetInfoCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetServerGetInfoCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->serverNamep,	"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->serverName,	"srvName",		lv) != NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"level      ", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerGetInfoResp()
 *
 * Description  : SRVSVC NetServerGetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerGetInfoResp(	cifsrpc_ed *edp,
								netServerGetInfoResp *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetServerGetInfoResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDsviContr(edp, &p->sviContr, lv)					!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue ", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerSetInfoCall()
 *
 * Description  : SRVSVC NetServerSetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerSetInfoCall(	cifsrpc_ed *edp,
								netServerSetInfoCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetServerSetInfoCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->serverNamep,	"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->serverName,	"srvName",		lv) != NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"level      ", lv)	!= NFX_OK
	 || cifsrpcEDsviContr(edp, &p->sviContr, lv)					!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerSetInfoResp()
 *
 * Description  : SRVSVC NetServerSetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerSetInfoResp(	cifsrpc_ed *edp,
								netServerSetInfoResp *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%snetServerGetInfoResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->unknown,		"unknown    ", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}



