/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-rpc-err.h,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 * 
 * Name			: cifs-rpc-err.h
 *
 * Description	: CIFS DCE/RPC error codes (From Win32)
 * 
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/11/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_RPC_ERR_H
#define _CIFS_RPC_ERR_H

//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      C - is the Customer code flag
//
//      R - is a reserved bit
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//
//
// Define the facility codes
//
#define FACILITY_WINDOWS                 8
#define FACILITY_STORAGE                 3
#define FACILITY_SSPI                    9
#define FACILITY_SETUPAPI                15
#define FACILITY_RPC                     1
#define FACILITY_WIN32                   7
#define FACILITY_CONTROL                 10
#define FACILITY_NULL                    0
#define FACILITY_MSMQ                    14
#define FACILITY_MEDIASERVER             13
#define FACILITY_INTERNET                12
#define FACILITY_ITF                     4
#define FACILITY_DISPATCH                2
#define FACILITY_CERT                    11


//Severity codes are defined in cifs-err.h



///////////////////////////
//                       //
//   RPC Status Codes    //
//                       //
///////////////////////////


//
// MessageId: CIFSRPC_STA_INVALID_STRING_BINDING
//
// MessageText:
//
//  The string binding is invalid.
//
#define CIFSRPC_STA_INVALID_STRING_BINDING     1700L

//
// MessageId: CIFSRPC_STA_WRONG_KIND_OF_BINDING
//
// MessageText:
//
//  The binding handle is not the correct type.
//
#define CIFSRPC_STA_WRONG_KIND_OF_BINDING      1701L

//
// MessageId: CIFSRPC_STA_INVALID_BINDING
//
// MessageText:
//
//  The binding handle is invalid.
//
#define CIFSRPC_STA_INVALID_BINDING            1702L

//
// MessageId: CIFSRPC_STA_PROTSEQ_NOT_SUPPORTED
//
// MessageText:
//
//  The RPC protocol sequence is not supported.
//
#define CIFSRPC_STA_PROTSEQ_NOT_SUPPORTED      1703L

//
// MessageId: CIFSRPC_STA_INVALID_RPC_PROTSEQ
//
// MessageText:
//
//  The RPC protocol sequence is invalid.
//
#define CIFSRPC_STA_INVALID_RPC_PROTSEQ        1704L

//
// MessageId: CIFSRPC_STA_INVALID_STRING_UUID
//
// MessageText:
//
//  The string universal unique identifier (UUID) is invalid.
//
#define CIFSRPC_STA_INVALID_STRING_UUID        1705L

//
// MessageId: CIFSRPC_STA_INVALID_ENDPOINT_FORMAT
//
// MessageText:
//
//  The endpoint format is invalid.
//
#define CIFSRPC_STA_INVALID_ENDPOINT_FORMAT    1706L

//
// MessageId: CIFSRPC_STA_INVALID_NET_ADDR
//
// MessageText:
//
//  The network address is invalid.
//
#define CIFSRPC_STA_INVALID_NET_ADDR           1707L

//
// MessageId: CIFSRPC_STA_NO_ENDPOINT_FOUND
//
// MessageText:
//
//  No endpoint was found.
//
#define CIFSRPC_STA_NO_ENDPOINT_FOUND          1708L

//
// MessageId: CIFSRPC_STA_INVALID_TIMEOUT
//
// MessageText:
//
//  The timeout value is invalid.
//
#define CIFSRPC_STA_INVALID_TIMEOUT            1709L

//
// MessageId: CIFSRPC_STA_OBJECT_NOT_FOUND
//
// MessageText:
//
//  The object universal unique identifier (UUID) was not found.
//
#define CIFSRPC_STA_OBJECT_NOT_FOUND           1710L

//
// MessageId: CIFSRPC_STA_ALREADY_REGISTERED
//
// MessageText:
//
//  The object universal unique identifier (UUID) has already been registered.
//
#define CIFSRPC_STA_ALREADY_REGISTERED         1711L

//
// MessageId: CIFSRPC_STA_TYPE_ALREADY_REGISTERED
//
// MessageText:
//
//  The type universal unique identifier (UUID) has already been registered.
//
#define CIFSRPC_STA_TYPE_ALREADY_REGISTERED    1712L

//
// MessageId: CIFSRPC_STA_ALREADY_LISTENING
//
// MessageText:
//
//  The RPC server is already listening.
//
#define CIFSRPC_STA_ALREADY_LISTENING          1713L

//
// MessageId: CIFSRPC_STA_NO_PROTSEQS_REGISTERED
//
// MessageText:
//
//  No protocol sequences have been registered.
//
#define CIFSRPC_STA_NO_PROTSEQS_REGISTERED     1714L

//
// MessageId: CIFSRPC_STA_NOT_LISTENING
//
// MessageText:
//
//  The RPC server is not listening.
//
#define CIFSRPC_STA_NOT_LISTENING              1715L

//
// MessageId: CIFSRPC_STA_UNKNOWN_MGR_TYPE
//
// MessageText:
//
//  The manager type is unknown.
//
#define CIFSRPC_STA_UNKNOWN_MGR_TYPE           1716L

//
// MessageId: CIFSRPC_STA_UNKNOWN_IF
//
// MessageText:
//
//  The interface is unknown.
//
#define CIFSRPC_STA_UNKNOWN_IF                 1717L

//
// MessageId: CIFSRPC_STA_NO_BINDINGS
//
// MessageText:
//
//  There are no bindings.
//
#define CIFSRPC_STA_NO_BINDINGS                1718L

//
// MessageId: CIFSRPC_STA_NO_PROTSEQS
//
// MessageText:
//
//  There are no protocol sequences.
//
#define CIFSRPC_STA_NO_PROTSEQS                1719L

//
// MessageId: CIFSRPC_STA_CANT_CREATE_ENDPOINT
//
// MessageText:
//
//  The endpoint cannot be created.
//
#define CIFSRPC_STA_CANT_CREATE_ENDPOINT       1720L

//
// MessageId: CIFSRPC_STA_OUT_OF_RESOURCES
//
// MessageText:
//
//  Not enough resources are available to complete this operation.
//
#define CIFSRPC_STA_OUT_OF_RESOURCES           1721L

//
// MessageId: CIFSRPC_STA_SERVER_UNAVAILABLE
//
// MessageText:
//
//  The RPC server is unavailable.
//
#define CIFSRPC_STA_SERVER_UNAVAILABLE         1722L

//
// MessageId: CIFSRPC_STA_SERVER_TOO_BUSY
//
// MessageText:
//
//  The RPC server is too busy to complete this operation.
//
#define CIFSRPC_STA_SERVER_TOO_BUSY            1723L

//
// MessageId: CIFSRPC_STA_INVALID_NETWORK_OPTIONS
//
// MessageText:
//
//  The network options are invalid.
//
#define CIFSRPC_STA_INVALID_NETWORK_OPTIONS    1724L

//
// MessageId: CIFSRPC_STA_NO_CALL_ACTIVE
//
// MessageText:
//
//  There are no remote procedure calls active on this thread.
//
#define CIFSRPC_STA_NO_CALL_ACTIVE             1725L

//
// MessageId: CIFSRPC_STA_CALL_FAILED
//
// MessageText:
//
//  The remote procedure call failed.
//
#define CIFSRPC_STA_CALL_FAILED                1726L

//
// MessageId: CIFSRPC_STA_CALL_FAILED_DNE
//
// MessageText:
//
//  The remote procedure call failed and did not execute.
//
#define CIFSRPC_STA_CALL_FAILED_DNE            1727L

//
// MessageId: CIFSRPC_STA_PROTOCOL_ERROR
//
// MessageText:
//
//  A remote procedure call (RPC) protocol error occurred.
//
#define CIFSRPC_STA_PROTOCOL_ERROR             1728L

//
// MessageId: CIFSRPC_STA_UNSUPPORTED_TRANS_SYN
//
// MessageText:
//
//  The transfer syntax is not supported by the RPC server.
//
#define CIFSRPC_STA_UNSUPPORTED_TRANS_SYN      1730L

//
// MessageId: CIFSRPC_STA_UNSUPPORTED_TYPE
//
// MessageText:
//
//  The universal unique identifier (UUID) type is not supported.
//
#define CIFSRPC_STA_UNSUPPORTED_TYPE           1732L

//
// MessageId: CIFSRPC_STA_INVALID_TAG
//
// MessageText:
//
//  The tag is invalid.
//
#define CIFSRPC_STA_INVALID_TAG                1733L

//
// MessageId: CIFSRPC_STA_INVALID_BOUND
//
// MessageText:
//
//  The array bounds are invalid.
//
#define CIFSRPC_STA_INVALID_BOUND              1734L

//
// MessageId: CIFSRPC_STA_NO_ENTRY_NAME
//
// MessageText:
//
//  The binding does not contain an entry name.
//
#define CIFSRPC_STA_NO_ENTRY_NAME              1735L

//
// MessageId: CIFSRPC_STA_INVALID_NAME_SYNTAX
//
// MessageText:
//
//  The name syntax is invalid.
//
#define CIFSRPC_STA_INVALID_NAME_SYNTAX        1736L

//
// MessageId: CIFSRPC_STA_UNSUPPORTED_NAME_SYNTAX
//
// MessageText:
//
//  The name syntax is not supported.
//
#define CIFSRPC_STA_UNSUPPORTED_NAME_SYNTAX    1737L

//
// MessageId: CIFSRPC_STA_UUID_NO_ADDRESS
//
// MessageText:
//
//  No network address is available to use to construct a universal
//  unique identifier (UUID).
//
#define CIFSRPC_STA_UUID_NO_ADDRESS            1739L

//
// MessageId: CIFSRPC_STA_DUPLICATE_ENDPOINT
//
// MessageText:
//
//  The endpoint is a duplicate.
//
#define CIFSRPC_STA_DUPLICATE_ENDPOINT         1740L

//
// MessageId: CIFSRPC_STA_UNKNOWN_AUTHN_TYPE
//
// MessageText:
//
//  The authentication type is unknown.
//
#define CIFSRPC_STA_UNKNOWN_AUTHN_TYPE         1741L

//
// MessageId: CIFSRPC_STA_MAX_CALLS_TOO_SMALL
//
// MessageText:
//
//  The maximum number of calls is too small.
//
#define CIFSRPC_STA_MAX_CALLS_TOO_SMALL        1742L

//
// MessageId: CIFSRPC_STA_STRING_TOO_LONG
//
// MessageText:
//
//  The string is too long.
//
#define CIFSRPC_STA_STRING_TOO_LONG            1743L

//
// MessageId: CIFSRPC_STA_PROTSEQ_NOT_FOUND
//
// MessageText:
//
//  The RPC protocol sequence was not found.
//
#define CIFSRPC_STA_PROTSEQ_NOT_FOUND          1744L

//
// MessageId: CIFSRPC_STA_PROCNUM_OUT_OF_RANGE
//
// MessageText:
//
//  The procedure number is out of range.
//
#define CIFSRPC_STA_PROCNUM_OUT_OF_RANGE       1745L

//
// MessageId: CIFSRPC_STA_BINDING_HAS_NO_AUTH
//
// MessageText:
//
//  The binding does not contain any authentication information.
//
#define CIFSRPC_STA_BINDING_HAS_NO_AUTH        1746L

//
// MessageId: CIFSRPC_STA_UNKNOWN_AUTHN_SERVICE
//
// MessageText:
//
//  The authentication service is unknown.
//
#define CIFSRPC_STA_UNKNOWN_AUTHN_SERVICE      1747L

//
// MessageId: CIFSRPC_STA_UNKNOWN_AUTHN_LEVEL
//
// MessageText:
//
//  The authentication level is unknown.
//
#define CIFSRPC_STA_UNKNOWN_AUTHN_LEVEL        1748L

//
// MessageId: CIFSRPC_STA_INVALID_AUTH_IDENTITY
//
// MessageText:
//
//  The security context is invalid.
//
#define CIFSRPC_STA_INVALID_AUTH_IDENTITY      1749L

//
// MessageId: CIFSRPC_STA_UNKNOWN_AUTHZ_SERVICE
//
// MessageText:
//
//  The authorization service is unknown.
//
#define CIFSRPC_STA_UNKNOWN_AUTHZ_SERVICE      1750L

//
// MessageId: EPT_S_INVALID_ENTRY
//
// MessageText:
//
//  The entry is invalid.
//
#define EPT_S_INVALID_ENTRY              1751L

//
// MessageId: EPT_S_CANT_PERFORM_OP
//
// MessageText:
//
//  The server endpoint cannot perform the operation.
//
#define EPT_S_CANT_PERFORM_OP            1752L

//
// MessageId: EPT_S_NOT_REGISTERED
//
// MessageText:
//
//  There are no more endpoints available from the endpoint mapper.
//
#define EPT_S_NOT_REGISTERED             1753L

//
// MessageId: CIFSRPC_STA_NOTHING_TO_EXPORT
//
// MessageText:
//
//  No interfaces have been exported.
//
#define CIFSRPC_STA_NOTHING_TO_EXPORT          1754L

//
// MessageId: CIFSRPC_STA_INCOMPLETE_NAME
//
// MessageText:
//
//  The entry name is incomplete.
//
#define CIFSRPC_STA_INCOMPLETE_NAME            1755L

//
// MessageId: CIFSRPC_STA_INVALID_VERS_OPTION
//
// MessageText:
//
//  The version option is invalid.
//
#define CIFSRPC_STA_INVALID_VERS_OPTION        1756L

//
// MessageId: CIFSRPC_STA_NO_MORE_MEMBERS
//
// MessageText:
//
//  There are no more members.
//
#define CIFSRPC_STA_NO_MORE_MEMBERS            1757L

//
// MessageId: CIFSRPC_STA_NOT_ALL_OBJS_UNEXPORTED
//
// MessageText:
//
//  There is nothing to unexport.
//
#define CIFSRPC_STA_NOT_ALL_OBJS_UNEXPORTED    1758L

//
// MessageId: CIFSRPC_STA_INTERFACE_NOT_FOUND
//
// MessageText:
//
//  The interface was not found.
//
#define CIFSRPC_STA_INTERFACE_NOT_FOUND        1759L

//
// MessageId: CIFSRPC_STA_ENTRY_ALREADY_EXISTS
//
// MessageText:
//
//  The entry already exists.
//
#define CIFSRPC_STA_ENTRY_ALREADY_EXISTS       1760L

//
// MessageId: CIFSRPC_STA_ENTRY_NOT_FOUND
//
// MessageText:
//
//  The entry is not found.
//
#define CIFSRPC_STA_ENTRY_NOT_FOUND            1761L

//
// MessageId: CIFSRPC_STA_NAME_SERVICE_UNAVAILABLE
//
// MessageText:
//
//  The name service is unavailable.
//
#define CIFSRPC_STA_NAME_SERVICE_UNAVAILABLE   1762L

//
// MessageId: CIFSRPC_STA_INVALID_NAF_ID
//
// MessageText:
//
//  The network address family is invalid.
//
#define CIFSRPC_STA_INVALID_NAF_ID             1763L

//
// MessageId: CIFSRPC_STA_CANNOT_SUPPORT
//
// MessageText:
//
//  The requested operation is not supported.
//
#define CIFSRPC_STA_CANNOT_SUPPORT             1764L

//
// MessageId: CIFSRPC_STA_NO_CONTEXT_AVAILABLE
//
// MessageText:
//
//  No security context is available to allow impersonation.
//
#define CIFSRPC_STA_NO_CONTEXT_AVAILABLE       1765L

//
// MessageId: CIFSRPC_STA_INTERNAL_ERROR
//
// MessageText:
//
//  An internal error occurred in a remote procedure call (RPC).
//
#define CIFSRPC_STA_INTERNAL_ERROR             1766L

//
// MessageId: CIFSRPC_STA_ZERO_DIVIDE
//
// MessageText:
//
//  The RPC server attempted an integer division by zero.
//
#define CIFSRPC_STA_ZERO_DIVIDE                1767L

//
// MessageId: CIFSRPC_STA_ADDRESS_ERROR
//
// MessageText:
//
//  An addressing error occurred in the RPC server.
//
#define CIFSRPC_STA_ADDRESS_ERROR              1768L

//
// MessageId: CIFSRPC_STA_FP_DIV_ZERO
//
// MessageText:
//
//  A floating-point operation at the RPC server caused a division by zero.
//
#define CIFSRPC_STA_FP_DIV_ZERO                1769L

//
// MessageId: CIFSRPC_STA_FP_UNDERFLOW
//
// MessageText:
//
//  A floating-point underflow occurred at the RPC server.
//
#define CIFSRPC_STA_FP_UNDERFLOW               1770L

//
// MessageId: CIFSRPC_STA_FP_OVERFLOW
//
// MessageText:
//
//  A floating-point overflow occurred at the RPC server.
//
#define CIFSRPC_STA_FP_OVERFLOW                1771L

//
// MessageId: CIFSRPC_X_NO_MORE_ENTRIES
//
// MessageText:
//
//  The list of RPC servers available for the binding of auto handles
//  has been exhausted.
//
#define CIFSRPC_X_NO_MORE_ENTRIES            1772L

//
// MessageId: CIFSRPC_X_SS_CHAR_TRANS_OPEN_FAIL
//
// MessageText:
//
//  Unable to open the character translation table file.
//
#define CIFSRPC_X_SS_CHAR_TRANS_OPEN_FAIL    1773L

//
// MessageId: CIFSRPC_X_SS_CHAR_TRANS_SHORT_FILE
//
// MessageText:
//
//  The file containing the character translation table has fewer than
//  512 bytes.
//
#define CIFSRPC_X_SS_CHAR_TRANS_SHORT_FILE   1774L

//
// MessageId: CIFSRPC_X_SS_IN_NULL_CONTEXT
//
// MessageText:
//
//  A null context handle was passed from the client to the host during
//  a remote procedure call.
//
#define CIFSRPC_X_SS_IN_NULL_CONTEXT         1775L

//
// MessageId: CIFSRPC_X_SS_CONTEXT_DAMAGED
//
// MessageText:
//
//  The context handle changed during a remote procedure call.
//
#define CIFSRPC_X_SS_CONTEXT_DAMAGED         1777L

//
// MessageId: CIFSRPC_X_SS_HANDLES_MISMATCH
//
// MessageText:
//
//  The binding handles passed to a remote procedure call do not match.
//
#define CIFSRPC_X_SS_HANDLES_MISMATCH        1778L

//
// MessageId: CIFSRPC_X_SS_CANNOT_GET_CALL_HANDLE
//
// MessageText:
//
//  The stub is unable to get the remote procedure call handle.
//
#define CIFSRPC_X_SS_CANNOT_GET_CALL_HANDLE  1779L

//
// MessageId: CIFSRPC_X_NULL_REF_POINTER
//
// MessageText:
//
//  A null reference pointer was passed to the stub.
//
#define CIFSRPC_X_NULL_REF_POINTER           1780L

//
// MessageId: CIFSRPC_X_ENUM_VALUE_OUT_OF_RANGE
//
// MessageText:
//
//  The enumeration value is out of range.
//
#define CIFSRPC_X_ENUM_VALUE_OUT_OF_RANGE    1781L

//
// MessageId: CIFSRPC_X_BYTE_COUNT_TOO_SMALL
//
// MessageText:
//
//  The byte count is too small.
//
#define CIFSRPC_X_BYTE_COUNT_TOO_SMALL       1782L

//
// MessageId: CIFSRPC_X_BAD_STUB_DATA
//
// MessageText:
//
//  The stub received bad data.
//
#define CIFSRPC_X_BAD_STUB_DATA              1783L

//
// MessageId: ERROR_INVALID_USER_BUFFER
//
// MessageText:
//
//  The supplied user buffer is not valid for the requested operation.
//
#define ERROR_INVALID_USER_BUFFER        1784L

//
// MessageId: ERROR_UNRECOGNIZED_MEDIA
//
// MessageText:
//
//  The disk media is not recognized.  It may not be formatted.
//
#define ERROR_UNRECOGNIZED_MEDIA         1785L

//
// MessageId: ERROR_NO_TRUST_LSA_SECRET
//
// MessageText:
//
//  The workstation does not have a trust secret.
//
#define ERROR_NO_TRUST_LSA_SECRET        1786L

//
// MessageId: ERROR_NO_TRUST_SAM_ACCOUNT
//
// MessageText:
//
//  The SAM database on the Windows NT Server does not have a computer
//  account for this workstation trust relationship.
//
#define ERROR_NO_TRUST_SAM_ACCOUNT       1787L

//
// MessageId: ERROR_TRUSTED_DOMAIN_FAILURE
//
// MessageText:
//
//  The trust relationship between the primary domain and the trusted
//  domain failed.
//
#define ERROR_TRUSTED_DOMAIN_FAILURE     1788L

//
// MessageId: ERROR_TRUSTED_RELATIONSHIP_FAILURE
//
// MessageText:
//
//  The trust relationship between this workstation and the primary
//  domain failed.
//
#define ERROR_TRUSTED_RELATIONSHIP_FAILURE 1789L

//
// MessageId: ERROR_TRUST_FAILURE
//
// MessageText:
//
//  The network logon failed.
//
#define ERROR_TRUST_FAILURE              1790L

//
// MessageId: CIFSRPC_STA_CALL_IN_PROGRESS
//
// MessageText:
//
//  A remote procedure call is already in progress for this thread.
//
#define CIFSRPC_STA_CALL_IN_PROGRESS           1791L

//
// MessageId: ERROR_NETLOGON_NOT_STARTED
//
// MessageText:
//
//  An attempt was made to logon, but the network logon service was not started.
//
#define ERROR_NETLOGON_NOT_STARTED       1792L

//
// MessageId: ERROR_ACCOUNT_EXPIRED
//
// MessageText:
//
//  The user's account has expired.
//
#define ERROR_ACCOUNT_EXPIRED            1793L

//
// MessageId: ERROR_REDIRECTOR_HAS_OPEN_HANDLES
//
// MessageText:
//
//  The redirector is in use and cannot be unloaded.
//
#define ERROR_REDIRECTOR_HAS_OPEN_HANDLES 1794L

//
// MessageId: ERROR_PRINTER_DRIVER_ALREADY_INSTALLED
//
// MessageText:
//
//  The specified printer driver is already installed.
//
#define ERROR_PRINTER_DRIVER_ALREADY_INSTALLED 1795L

//
// MessageId: ERROR_UNKNOWN_PORT
//
// MessageText:
//
//  The specified port is unknown.
//
#define ERROR_UNKNOWN_PORT               1796L

//
// MessageId: ERROR_UNKNOWN_PRINTER_DRIVER
//
// MessageText:
//
//  The printer driver is unknown.
//
#define ERROR_UNKNOWN_PRINTER_DRIVER     1797L

//
// MessageId: ERROR_UNKNOWN_PRINTPROCESSOR
//
// MessageText:
//
//  The print processor is unknown.
//
#define ERROR_UNKNOWN_PRINTPROCESSOR     1798L

//
// MessageId: ERROR_INVALID_SEPARATOR_FILE
//
// MessageText:
//
//  The specified separator file is invalid.
//
#define ERROR_INVALID_SEPARATOR_FILE     1799L

//
// MessageId: ERROR_INVALID_PRIORITY
//
// MessageText:
//
//  The specified priority is invalid.
//
#define ERROR_INVALID_PRIORITY           1800L

//
// MessageId: ERROR_INVALID_PRINTER_NAME
//
// MessageText:
//
//  The printer name is invalid.
//
#define ERROR_INVALID_PRINTER_NAME       1801L

//
// MessageId: ERROR_PRINTER_ALREADY_EXISTS
//
// MessageText:
//
//  The printer already exists.
//
#define ERROR_PRINTER_ALREADY_EXISTS     1802L

//
// MessageId: ERROR_INVALID_PRINTER_COMMAND
//
// MessageText:
//
//  The printer command is invalid.
//
#define ERROR_INVALID_PRINTER_COMMAND    1803L

//
// MessageId: ERROR_INVALID_DATATYPE
//
// MessageText:
//
//  The specified datatype is invalid.
//
#define ERROR_INVALID_DATATYPE           1804L

//
// MessageId: ERROR_INVALID_ENVIRONMENT
//
// MessageText:
//
//  The environment specified is invalid.
//
#define ERROR_INVALID_ENVIRONMENT        1805L

//
// MessageId: CIFSRPC_STA_NO_MORE_BINDINGS
//
// MessageText:
//
//  There are no more bindings.
//
#define CIFSRPC_STA_NO_MORE_BINDINGS           1806L

//
// MessageId: ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT
//
// MessageText:
//
//  The account used is an interdomain trust account.  Use your global user account or local user account to access this server.
//
#define ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT 1807L

//
// MessageId: ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT
//
// MessageText:
//
//  The account used is a computer account.  Use your global user account or local user account to access this server.
//
#define ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT 1808L

//
// MessageId: ERROR_NOLOGON_SERVER_TRUST_ACCOUNT
//
// MessageText:
//
//  The account used is a server trust account. Use your global user account or local user account to access this server.
//
#define ERROR_NOLOGON_SERVER_TRUST_ACCOUNT 1809L

//
// MessageId: ERROR_DOMAIN_TRUST_INCONSISTENT
//
// MessageText:
//
//  The name or security ID (SID) of the domain specified is inconsistent
//  with the trust information for that domain.
//
#define ERROR_DOMAIN_TRUST_INCONSISTENT  1810L

//
// MessageId: ERROR_SERVER_HAS_OPEN_HANDLES
//
// MessageText:
//
//  The server is in use and cannot be unloaded.
//
#define ERROR_SERVER_HAS_OPEN_HANDLES    1811L

//
// MessageId: ERROR_RESOURCE_DATA_NOT_FOUND
//
// MessageText:
//
//  The specified image file did not contain a resource section.
//
#define ERROR_RESOURCE_DATA_NOT_FOUND    1812L

//
// MessageId: ERROR_RESOURCE_TYPE_NOT_FOUND
//
// MessageText:
//
//  The specified resource type cannot be found in the image file.
//
#define ERROR_RESOURCE_TYPE_NOT_FOUND    1813L

//
// MessageId: ERROR_RESOURCE_NAME_NOT_FOUND
//
// MessageText:
//
//  The specified resource name cannot be found in the image file.
//
#define ERROR_RESOURCE_NAME_NOT_FOUND    1814L

//
// MessageId: ERROR_RESOURCE_LANG_NOT_FOUND
//
// MessageText:
//
//  The specified resource language ID cannot be found in the image file.
//
#define ERROR_RESOURCE_LANG_NOT_FOUND    1815L

//
// MessageId: ERROR_NOT_ENOUGH_QUOTA
//
// MessageText:
//
//  Not enough quota is available to process this command.
//
#define ERROR_NOT_ENOUGH_QUOTA           1816L

//
// MessageId: CIFSRPC_STA_NO_INTERFACES
//
// MessageText:
//
//  No interfaces have been registered.
//
#define CIFSRPC_STA_NO_INTERFACES              1817L

//
// MessageId: CIFSRPC_STA_CALL_CANCELLED
//
// MessageText:
//
//  The remote procedure call was cancelled.
//
#define CIFSRPC_STA_CALL_CANCELLED             1818L

//
// MessageId: CIFSRPC_STA_BINDING_INCOMPLETE
//
// MessageText:
//
//  The binding handle does not contain all required information.
//
#define CIFSRPC_STA_BINDING_INCOMPLETE         1819L

//
// MessageId: CIFSRPC_STA_COMM_FAILURE
//
// MessageText:
//
//  A communications failure occurred during a remote procedure call.
//
#define CIFSRPC_STA_COMM_FAILURE               1820L

//
// MessageId: CIFSRPC_STA_UNSUPPORTED_AUTHN_LEVEL
//
// MessageText:
//
//  The requested authentication level is not supported.
//
#define CIFSRPC_STA_UNSUPPORTED_AUTHN_LEVEL    1821L

//
// MessageId: CIFSRPC_STA_NO_PRINC_NAME
//
// MessageText:
//
//  No principal name registered.
//
#define CIFSRPC_STA_NO_PRINC_NAME              1822L

//
// MessageId: CIFSRPC_STA_NOT_RPC_ERROR
//
// MessageText:
//
//  The error specified is not a valid Windows RPC error code.
//
#define CIFSRPC_STA_NOT_RPC_ERROR              1823L

//
// MessageId: CIFSRPC_STA_UUID_LOCAL_ONLY
//
// MessageText:
//
//  A UUID that is valid only on this computer has been allocated.
//
#define CIFSRPC_STA_UUID_LOCAL_ONLY            1824L

//
// MessageId: CIFSRPC_STA_SEC_PKG_ERROR
//
// MessageText:
//
//  A security package specific error occurred.
//
#define CIFSRPC_STA_SEC_PKG_ERROR              1825L

//
// MessageId: CIFSRPC_STA_NOT_CANCELLED
//
// MessageText:
//
//  Thread is not canceled.
//
#define CIFSRPC_STA_NOT_CANCELLED              1826L

//
// MessageId: CIFSRPC_X_INVALID_ES_ACTION
//
// MessageText:
//
//  Invalid operation on the encoding/decoding handle.
//
#define CIFSRPC_X_INVALID_ES_ACTION          1827L

//
// MessageId: CIFSRPC_X_WRONG_ES_VERSION
//
// MessageText:
//
//  Incompatible version of the serializing package.
//
#define CIFSRPC_X_WRONG_ES_VERSION           1828L

//
// MessageId: CIFSRPC_X_WRONG_STUB_VERSION
//
// MessageText:
//
//  Incompatible version of the RPC stub.
//
#define CIFSRPC_X_WRONG_STUB_VERSION         1829L

//
// MessageId: CIFSRPC_X_INVALID_PIPE_OBJECT
//
// MessageText:
//
//  The RPC pipe object is invalid or corrupted.
//
#define CIFSRPC_X_INVALID_PIPE_OBJECT        1830L

//
// MessageId: CIFSRPC_X_WRONG_PIPE_ORDER
//
// MessageText:
//
//  An invalid operation was attempted on an RPC pipe object.
//
#define CIFSRPC_X_WRONG_PIPE_ORDER           1831L

//
// MessageId: CIFSRPC_X_WRONG_PIPE_VERSION
//
// MessageText:
//
//  Unsupported RPC pipe version.
//
#define CIFSRPC_X_WRONG_PIPE_VERSION         1832L

//
// MessageId: CIFSRPC_STA_GROUP_MEMBER_NOT_FOUND
//
// MessageText:
//
//  The group member was not found.
//
#define CIFSRPC_STA_GROUP_MEMBER_NOT_FOUND     1898L

//
// MessageId: EPT_S_CANT_CREATE
//
// MessageText:
//
//  The endpoint mapper database entry could not be created.
//
#define EPT_S_CANT_CREATE                1899L

//
// MessageId: CIFSRPC_STA_INVALID_OBJECT
//
// MessageText:
//
//  The object universal unique identifier (UUID) is the nil UUID.
//
#define CIFSRPC_STA_INVALID_OBJECT             1900L

//
// MessageId: ERROR_INVALID_TIME
//
// MessageText:
//
//  The specified time is invalid.
//
#define ERROR_INVALID_TIME               1901L

//
// MessageId: ERROR_INVALID_FORM_NAME
//
// MessageText:
//
//  The specified form name is invalid.
//
#define ERROR_INVALID_FORM_NAME          1902L

//
// MessageId: ERROR_INVALID_FORM_SIZE
//
// MessageText:
//
//  The specified form size is invalid.
//
#define ERROR_INVALID_FORM_SIZE          1903L

//
// MessageId: ERROR_ALREADY_WAITING
//
// MessageText:
//
//  The specified printer handle is already being waited on
//
#define ERROR_ALREADY_WAITING            1904L

//
// MessageId: ERROR_PRINTER_DELETED
//
// MessageText:
//
//  The specified printer has been deleted.
//
#define ERROR_PRINTER_DELETED            1905L

//
// MessageId: ERROR_INVALID_PRINTER_STATE
//
// MessageText:
//
//  The state of the printer is invalid.
//
#define ERROR_INVALID_PRINTER_STATE      1906L

//
// MessageId: ERROR_PASSWORD_MUST_CHANGE
//
// MessageText:
//
//  The user must change his password before he logs on the first time.
//
#define ERROR_PASSWORD_MUST_CHANGE       1907L

//
// MessageId: ERROR_DOMAIN_CONTROLLER_NOT_FOUND
//
// MessageText:
//
//  Could not find the domain controller for this domain.
//
#define ERROR_DOMAIN_CONTROLLER_NOT_FOUND 1908L

//
// MessageId: ERROR_ACCOUNT_LOCKED_OUT
//
// MessageText:
//
//  The referenced account is currently locked out and may not be logged on to.
//
#define ERROR_ACCOUNT_LOCKED_OUT         1909L

//
// MessageId: OR_INVALID_OXID
//
// MessageText:
//
//  The object exporter specified was not found.
//
#define OR_INVALID_OXID                  1910L

//
// MessageId: OR_INVALID_OID
//
// MessageText:
//
//  The object specified was not found.
//
#define OR_INVALID_OID                   1911L

//
// MessageId: OR_INVALID_SET
//
// MessageText:
//
//  The object resolver set specified was not found.
//
#define OR_INVALID_SET                   1912L

//
// MessageId: CIFSRPC_STA_SEND_INCOMPLETE
//
// MessageText:
//
//  Some data remains to be sent in the request buffer.
//
#define CIFSRPC_STA_SEND_INCOMPLETE            1913L

//
// MessageId: CIFSRPC_STA_INVALID_ASYNC_HANDLE
//
// MessageText:
//
//  Invalid asynchronous remote procedure call handle.
//
#define CIFSRPC_STA_INVALID_ASYNC_HANDLE       1914L

//
// MessageId: CIFSRPC_STA_INVALID_ASYNC_CALL
//
// MessageText:
//
//  Invalid asynchronous RPC call handle for this operation.
//
#define CIFSRPC_STA_INVALID_ASYNC_CALL         1915L

//
// MessageId: CIFSRPC_X_PIPE_CLOSED
//
// MessageText:
//
//  The RPC pipe object has already been closed.
//
#define CIFSRPC_X_PIPE_CLOSED                1916L

//
// MessageId: CIFSRPC_X_PIPE_DISCIPLINE_ERROR
//
// MessageText:
//
//  The RPC call completed before all pipes were processed.
//
#define CIFSRPC_X_PIPE_DISCIPLINE_ERROR      1917L

//
// MessageId: CIFSRPC_X_PIPE_EMPTY
//
// MessageText:
//
//  No more data is available from the RPC pipe.
//
#define CIFSRPC_X_PIPE_EMPTY                 1918L

//
// MessageId: ERROR_NO_SITENAME
//
// MessageText:
//
//  No site name is available for this machine.
//
#define ERROR_NO_SITENAME                1919L

//
// MessageId: ERROR_CANT_ACCESS_FILE
//
// MessageText:
//
//  The file can not be accessed by the system.
//
#define ERROR_CANT_ACCESS_FILE           1920L

//
// MessageId: ERROR_CANT_RESOLVE_FILENAME
//
// MessageText:
//
//  The name of the file cannot be resolved by the system.
//
#define ERROR_CANT_RESOLVE_FILENAME      1921L

//
// MessageId: ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY
//
// MessageText:
//
//  The directory service evaluated group memberships locally.
//
#define ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY 1922L

//
// MessageId: ERROR_DS_NO_ATTRIBUTE_OR_VALUE
//
// MessageText:
//
//  The specified directory service attribute or value does not exist.
//
#define ERROR_DS_NO_ATTRIBUTE_OR_VALUE   1923L

//
// MessageId: ERROR_DS_INVALID_ATTRIBUTE_SYNTAX
//
// MessageText:
//
//  The attribute syntax specified to the directory service is invalid.
//
#define ERROR_DS_INVALID_ATTRIBUTE_SYNTAX 1924L

//
// MessageId: ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED
//
// MessageText:
//
//  The attribute type specified to the directory service is not defined.
//
#define ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED 1925L

//
// MessageId: ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS
//
// MessageText:
//
//  The specified directory service attribute or value already exists.
//
#define ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS 1926L

//
// MessageId: ERROR_DS_BUSY
//
// MessageText:
//
//  The directory service is busy.
//
#define ERROR_DS_BUSY                    1927L

//
// MessageId: ERROR_DS_UNAVAILABLE
//
// MessageText:
//
//  The directory service is unavailable.
//
#define ERROR_DS_UNAVAILABLE             1928L

//
// MessageId: ERROR_DS_NO_RIDS_ALLOCATED
//
// MessageText:
//
//  The directory service was unable to allocate a relative identifier.
//
#define ERROR_DS_NO_RIDS_ALLOCATED       1929L

//
// MessageId: ERROR_DS_NO_MORE_RIDS
//
// MessageText:
//
//  The directory service has exhausted the pool of relative identifiers.
//
#define ERROR_DS_NO_MORE_RIDS            1930L

//
// MessageId: ERROR_DS_INCORRECT_ROLE_OWNER
//
// MessageText:
//
//  The requested operation could not be performed because the directory service is not the
//  master for that type of operation.
//
#define ERROR_DS_INCORRECT_ROLE_OWNER    1931L

//
// MessageId: ERROR_DS_RIDMGR_INIT_ERROR
//
// MessageText:
//
//  The directory service was unable to initialize the subsystem that allocates relative identifiers.
//
#define ERROR_DS_RIDMGR_INIT_ERROR       1932L

//
// MessageId: ERROR_DS_OBJ_CLASS_VIOLATION
//
// MessageText:
//
//  The requested operation did not satisfy one or more constraints associated with the class of the object.
//
#define ERROR_DS_OBJ_CLASS_VIOLATION     1933L

//
// MessageId: ERROR_DS_CANT_ON_NON_LEAF
//
// MessageText:
//
//  The directory service can perform the requested operation only on a leaf object.
//
#define ERROR_DS_CANT_ON_NON_LEAF        1934L

//
// MessageId: ERROR_DS_CANT_ON_RDN
//
// MessageText:
//
//  The directory service cannot perform the requested operation on the RDN attribute of an object.
//
#define ERROR_DS_CANT_ON_RDN             1935L

//
// MessageId: ERROR_DS_CANT_MOD_OBJ_CLASS
//
// MessageText:
//
//  The directory service detected an attempt to modify the object class of an object.
//
#define ERROR_DS_CANT_MOD_OBJ_CLASS      1936L

//
// MessageId: ERROR_DS_CROSS_DOM_MOVE_ERROR
//
// MessageText:
//
//  The requested cross domain move operation could not be performed.
//
#define ERROR_DS_CROSS_DOM_MOVE_ERROR    1937L

//
// MessageId: ERROR_DS_GC_NOT_AVAILABLE
//
// MessageText:
//
//  Unable to contact the global catalog server.
//
#define ERROR_DS_GC_NOT_AVAILABLE        1938L

//
// MessageId: ERROR_NO_BROWSER_SERVERS_FOUND
//
// MessageText:
//
//  The list of servers for this workgroup is not currently available
//
#define ERROR_NO_BROWSER_SERVERS_FOUND   6118L




// ***************************************************************************
// ******************************* FACILITY_RPC ******************************
// ***************************************************************************



//
// MessageId: CIFSRPC_ERR_CALL_REJECTED
//
// MessageText:
//
//  Call was rejected by callee.
//
#define CIFSRPC_ERR_CALL_REJECTED              (0x80010001L)

//
// MessageId: CIFSRPC_ERR_CALL_CANCELED
//
// MessageText:
//
//  Call was canceled by the message filter.
//
#define CIFSRPC_ERR_CALL_CANCELED              (0x80010002L)

//
// MessageId: CIFSRPC_ERR_CANTPOST_INSENDCALL
//
// MessageText:
//
//  The caller is dispatching an intertask SendMessage call and
//  cannot call out via PostMessage.
//
#define CIFSRPC_ERR_CANTPOST_INSENDCALL        (0x80010003L)

//
// MessageId: CIFSRPC_ERR_CANTCALLOUT_INASYNCCALL
//
// MessageText:
//
//  The caller is dispatching an asynchronous call and cannot
//  make an outgoing call on behalf of this call.
//
#define CIFSRPC_ERR_CANTCALLOUT_INASYNCCALL    (0x80010004L)

//
// MessageId: CIFSRPC_ERR_CANTCALLOUT_INEXTERNALCALL
//
// MessageText:
//
//  It is illegal to call out while inside message filter.
//
#define CIFSRPC_ERR_CANTCALLOUT_INEXTERNALCALL (0x80010005L)

//
// MessageId: CIFSRPC_ERR_CONNECTION_TERMINATED
//
// MessageText:
//
//  The connection terminated or is in a bogus state
//  and cannot be used any more. Other connections
//  are still valid.
//
#define CIFSRPC_ERR_CONNECTION_TERMINATED      (0x80010006L)

//
// MessageId: CIFSRPC_ERR_SERVER_DIED
//
// MessageText:
//
//  The callee (server [not server application]) is not available
//  and disappeared; all connections are invalid.  The call may
//  have executed.
//
#define CIFSRPC_ERR_SERVER_DIED                (0x80010007L)

//
// MessageId: CIFSRPC_ERR_CLIENT_DIED
//
// MessageText:
//
//  The caller (client) disappeared while the callee (server) was
//  processing a call.
//
#define CIFSRPC_ERR_CLIENT_DIED                (0x80010008L)

//
// MessageId: CIFSRPC_ERR_INVALID_DATAPACKET
//
// MessageText:
//
//  The data packet with the marshalled parameter data is incorrect.
//
#define CIFSRPC_ERR_INVALID_DATAPACKET         (0x80010009L)

//
// MessageId: CIFSRPC_ERR_CANTTRANSMIT_CALL
//
// MessageText:
//
//  The call was not transmitted properly; the message queue
//  was full and was not emptied after yielding.
//
#define CIFSRPC_ERR_CANTTRANSMIT_CALL          (0x8001000AL)

//
// MessageId: CIFSRPC_ERR_CLIENT_CANTMARSHAL_DATA
//
// MessageText:
//
//  The client (caller) cannot marshall the parameter data - low memory, etc.
//
#define CIFSRPC_ERR_CLIENT_CANTMARSHAL_DATA    (0x8001000BL)

//
// MessageId: CIFSRPC_ERR_CLIENT_CANTUNMARSHAL_DATA
//
// MessageText:
//
//  The client (caller) cannot unmarshall the return data - low memory, etc.
//
#define CIFSRPC_ERR_CLIENT_CANTUNMARSHAL_DATA  (0x8001000CL)

//
// MessageId: CIFSRPC_ERR_SERVER_CANTMARSHAL_DATA
//
// MessageText:
//
//  The server (callee) cannot marshall the return data - low memory, etc.
//
#define CIFSRPC_ERR_SERVER_CANTMARSHAL_DATA    (0x8001000DL)

//
// MessageId: CIFSRPC_ERR_SERVER_CANTUNMARSHAL_DATA
//
// MessageText:
//
//  The server (callee) cannot unmarshall the parameter data - low memory, etc.
//
#define CIFSRPC_ERR_SERVER_CANTUNMARSHAL_DATA  (0x8001000EL)

//
// MessageId: CIFSRPC_ERR_INVALID_DATA
//
// MessageText:
//
//  Received data is invalid; could be server or client data.
//
#define CIFSRPC_ERR_INVALID_DATA               (0x8001000FL)

//
// MessageId: CIFSRPC_ERR_INVALID_PARAMETER
//
// MessageText:
//
//  A particular parameter is invalid and cannot be (un)marshalled.
//
#define CIFSRPC_ERR_INVALID_PARAMETER          (0x80010010L)

//
// MessageId: CIFSRPC_ERR_CANTCALLOUT_AGAIN
//
// MessageText:
//
//  There is no second outgoing call on same channel in DDE conversation.
//
#define CIFSRPC_ERR_CANTCALLOUT_AGAIN          (0x80010011L)

//
// MessageId: CIFSRPC_ERR_SERVER_DIED_DNE
//
// MessageText:
//
//  The callee (server [not server application]) is not available
//  and disappeared; all connections are invalid.  The call did not execute.
//
#define CIFSRPC_ERR_SERVER_DIED_DNE            (0x80010012L)

//
// MessageId: CIFSRPC_ERR_SYS_CALL_FAILED
//
// MessageText:
//
//  System call failed.
//
#define CIFSRPC_ERR_SYS_CALL_FAILED            (0x80010100L)

//
// MessageId: CIFSRPC_ERR_OUT_OF_RESOURCES
//
// MessageText:
//
//  Could not allocate some required resource (memory, events, ...)
//
#define CIFSRPC_ERR_OUT_OF_RESOURCES           (0x80010101L)

//
// MessageId: CIFSRPC_ERR_ATTEMPTED_MULTITHREAD
//
// MessageText:
//
//  Attempted to make calls on more than one thread in single threaded mode.
//
#define CIFSRPC_ERR_ATTEMPTED_MULTITHREAD      (0x80010102L)

//
// MessageId: CIFSRPC_ERR_NOT_REGISTERED
//
// MessageText:
//
//  The requested interface is not registered on the server object.
//
#define CIFSRPC_ERR_NOT_REGISTERED             (0x80010103L)

//
// MessageId: CIFSRPC_ERR_FAULT
//
// MessageText:
//
//  RPC could not call the server or could not return the results of calling the server.
//
#define CIFSRPC_ERR_FAULT                      (0x80010104L)

//
// MessageId: CIFSRPC_ERR_SERVERFAULT
//
// MessageText:
//
//  The server threw an exception.
//
#define CIFSRPC_ERR_SERVERFAULT                (0x80010105L)

//
// MessageId: CIFSRPC_ERR_CHANGED_MODE
//
// MessageText:
//
//  Cannot change thread mode after it is set.
//
#define CIFSRPC_ERR_CHANGED_MODE               (0x80010106L)

//
// MessageId: CIFSRPC_ERR_INVALIDMETHOD
//
// MessageText:
//
//  The method called does not exist on the server.
//
#define CIFSRPC_ERR_INVALIDMETHOD              (0x80010107L)

//
// MessageId: CIFSRPC_ERR_DISCONNECTED
//
// MessageText:
//
//  The object invoked has disconnected from its clients.
//
#define CIFSRPC_ERR_DISCONNECTED               (0x80010108L)

//
// MessageId: CIFSRPC_ERR_RETRY
//
// MessageText:
//
//  The object invoked chose not to process the call now.  Try again later.
//
#define CIFSRPC_ERR_RETRY                      (0x80010109L)

//
// MessageId: CIFSRPC_ERR_SERVERCALL_RETRYLATER
//
// MessageText:
//
//  The message filter indicated that the application is busy.
//
#define CIFSRPC_ERR_SERVERCALL_RETRYLATER      (0x8001010AL)

//
// MessageId: CIFSRPC_ERR_SERVERCALL_REJECTED
//
// MessageText:
//
//  The message filter rejected the call.
//
#define CIFSRPC_ERR_SERVERCALL_REJECTED        (0x8001010BL)

//
// MessageId: CIFSRPC_ERR_INVALID_CALLDATA
//
// MessageText:
//
//  A call control interfaces was called with invalid data.
//
#define CIFSRPC_ERR_INVALID_CALLDATA           (0x8001010CL)

//
// MessageId: CIFSRPC_ERR_CANTCALLOUT_ININPUTSYNCCALL
//
// MessageText:
//
//  An outgoing call cannot be made since the application is dispatching an input-synchronous call.
//
#define CIFSRPC_ERR_CANTCALLOUT_ININPUTSYNCCALL (0x8001010DL)

//
// MessageId: CIFSRPC_ERR_WRONG_THREAD
//
// MessageText:
//
//  The application called an interface that was marshalled for a different thread.
//
#define CIFSRPC_ERR_WRONG_THREAD               (0x8001010EL)

//
// MessageId: CIFSRPC_ERR_THREAD_NOT_INIT
//
// MessageText:
//
//  CoInitialize has not been called on the current thread.
//
#define CIFSRPC_ERR_THREAD_NOT_INIT            (0x8001010FL)

//
// MessageId: CIFSRPC_ERR_VERSION_MISMATCH
//
// MessageText:
//
//  The version of OLE on the client and server machines does not match.
//
#define CIFSRPC_ERR_VERSION_MISMATCH           (0x80010110L)

//
// MessageId: CIFSRPC_ERR_INVALID_HEADER
//
// MessageText:
//
//  OLE received a packet with an invalid header.
//
#define CIFSRPC_ERR_INVALID_HEADER             (0x80010111L)

//
// MessageId: CIFSRPC_ERR_INVALID_EXTENSION
//
// MessageText:
//
//  OLE received a packet with an invalid extension.
//
#define CIFSRPC_ERR_INVALID_EXTENSION          (0x80010112L)

//
// MessageId: CIFSRPC_ERR_INVALID_IPID
//
// MessageText:
//
//  The requested object or interface does not exist.
//
#define CIFSRPC_ERR_INVALID_IPID               (0x80010113L)

//
// MessageId: CIFSRPC_ERR_INVALID_OBJECT
//
// MessageText:
//
//  The requested object does not exist.
//
#define CIFSRPC_ERR_INVALID_OBJECT             (0x80010114L)

//
// MessageId: CIFSRPC_STA_CALLPENDING
//
// MessageText:
//
//  OLE has sent a request and is waiting for a reply.
//
#define CIFSRPC_STA_CALLPENDING                (0x80010115L)

//
// MessageId: CIFSRPC_STA_WAITONTIMER
//
// MessageText:
//
//  OLE is waiting before retrying a request.
//
#define CIFSRPC_STA_WAITONTIMER                (0x80010116L)

//
// MessageId: CIFSRPC_ERR_CALL_COMPLETE
//
// MessageText:
//
//  Call context cannot be accessed after call completed.
//
#define CIFSRPC_ERR_CALL_COMPLETE              (0x80010117L)

//
// MessageId: CIFSRPC_ERR_UNSECURE_CALL
//
// MessageText:
//
//  Impersonate on unsecure calls is not supported.
//
#define CIFSRPC_ERR_UNSECURE_CALL              (0x80010118L)

//
// MessageId: CIFSRPC_ERR_TOO_LATE
//
// MessageText:
//
//  Security must be initialized before any interfaces are marshalled or
//  unmarshalled.  It cannot be changed once initialized.
//
#define CIFSRPC_ERR_TOO_LATE                   (0x80010119L)

//
// MessageId: CIFSRPC_ERR_NO_GOOD_SECURITY_PACKAGES
//
// MessageText:
//
//  No security packages are installed on this machine or the user is not logged
//  on or there are no compatible security packages between the client and server.
//
#define CIFSRPC_ERR_NO_GOOD_SECURITY_PACKAGES  (0x8001011AL)

//
// MessageId: CIFSRPC_ERR_ACCESS_DENIED
//
// MessageText:
//
//  Access is denied.
//
#define CIFSRPC_ERR_ACCESS_DENIED              (0x8001011BL)

//
// MessageId: CIFSRPC_ERR_REMOTE_DISABLED
//
// MessageText:
//
//  Remote calls are not allowed for this process.
//
#define CIFSRPC_ERR_REMOTE_DISABLED            (0x8001011CL)

//
// MessageId: CIFSRPC_ERR_INVALID_OBJREF
//
// MessageText:
//
//  The marshaled interface data packet (OBJREF) has an invalid or unknown format.
//
#define CIFSRPC_ERR_INVALID_OBJREF             (0x8001011DL)

//
// MessageId: CIFSRPC_ERR_NO_CONTEXT
//
// MessageText:
//
//  No context is associated with this call.  This happens for some custom
//  marshalled calls and on the client side of the call.
//
#define CIFSRPC_ERR_NO_CONTEXT                 (0x8001011EL)

//
// MessageId: CIFSRPC_ERR_TIMEOUT
//
// MessageText:
//
//  This operation returned because the timeout period expired.
//
#define CIFSRPC_ERR_TIMEOUT                    (0x8001011FL)

//
// MessageId: CIFSRPC_ERR_NO_SYNC
//
// MessageText:
//
//  There are no synchronize objects to wait on.
//
#define CIFSRPC_ERR_NO_SYNC                    (0x80010120L)

//
// MessageId: CIFSRPC_ERR_UNEXPECTED
//
// MessageText:
//
//  An internal error occurred.
//
#define CIFSRPC_ERR_UNEXPECTED                 (0x8001FFFFL)


#endif //_CIFS_RPC_ERR_H
