/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-rpc-mem.c,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 * 
 * Name			: cifs-rpc-mem.c
 *
 * Description	: Contains CIFS DCE/RPC memory management functions. 
 *				  Keeps track of all the memory used for a particular data
 *				  processing and allows to free it in a single operation
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/04/2002
 *
 *-----------------------------------------------------------------
 */

#ifdef FS_STANDALONE
#include "../sm-cifs/cifs-api.h"
#else
#include <exports/cifs-api.h>
#endif

#include "cifs-rpc-mem.h"
	
/*
void *realloc(void *ptr, uint32 size)
{
	void *newMemp;
	
	if (size == 0) 
	{
		if (ptr) CIFS_RAM_DEALLOC(ptr);
		return NULL;
	}
	if (!ptr)
	{
		CIFS_RAM_ALLOC(newMemp, void*, size);
		return (newMemp);
	}
	else {
		CIFS_RAM_ALLOC(newMemp, void*, size);
		memcpy(newMemp, ptr, size);	//not nice but all I can do, hope it will
									//not crash...
		CIFS_RAM_DEALLOC(ptr);
		return (newMemp);
	}
		
}
*/



/*-----------------------------------------------------------------
 * Name         : cifsrpc_initMemList
 *
 * Description  : Initialize a memory items list 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void
cifsrpc_initMemList(cifsrpc_memList *ml)
{
	ml->items = NULL;
	ml->size  = 0;
}


/*-----------------------------------------------------------------
 * Name         : cifsrpc_allocMemItem
 *
 * Description  : Allocate a memory item and add it to the memory list
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void *cifsrpc_allocMemItem(cifsrpc_memList *ml, uint32 size)
{
	cifsrpc_memItem   *item;

	CIFS_RAM_ALLOC(item, cifsrpc_memItem, sizeof(cifsrpc_memItem) - 1 + size);
	if (item == NULL) 
		return (NULL);
	item->size	= size;
	item->nextp	= ml->items;
	ml->items	= item;
	ml->size   += size;
	
	return (item->data);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpc_reallocMemItem
 *
 * Description  : Reallocate a memory item
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
/*
void *cifsrpc_reallocMemItem(cifsrpc_memList *ml, void *datap, uint32 size)
{
	cifsrpc_memItem	*itemToFind, *item, *prevItem;
	uint32			oldSize;

	if (size == 0)		//We don't bother to free the memory
		return (NULL);

	if (datap == NULL)
		return cifsrpc_allocMemItem(ml, size);
	
	itemToFind	= ((cifsrpc_memItem*)(((uchar8*)datap) + 1)) - 1; 
	item		= ml->items;
	prevItem	= NULL; 
	while (item)
	{
		if (item == itemToFind)
		{
			itemToFind = realloc(itemToFind, sizeof(cifsrpc_memItem)-1 + size);
			if (item)
			{
				ml->size += size - itemToFind->size;
				itemToFind->size = size;
				if (prevItem != NULL)
					prevItem->nextp = itemToFind;
			}
			return (itemToFind->data);
		}
		prevItem = item;
		item = item->nextp;
	}

	return (NULL);
}
*/

/*-----------------------------------------------------------------
 * Name         : cifsrpc_freeMemList
 *
 * Description  : free all the memory allocated in the memory list
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void cifsrpc_freeMemList(cifsrpc_memList *ml)
{
	cifsrpc_memItem   *nextItemp;
	
	while (ml->items)
	{
		nextItemp = ml->items->nextp;
		CIFS_RAM_DEALLOC(ml->items);
		ml->items = nextItemp;
	}
    ml->size  = 0;
}


/*-----------------------------------------------------------------
 * Name         : cifsrpc_mem
 *
 * Description  :  
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */



	



