/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-rpc-utils.c,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 *
 * Name			: cifs-rpc-srvsvc.c
 *
 * Description	: Contains CIFS DCE/RPC utility functions. 
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/16/2002
 *
 *-----------------------------------------------------------------
 */

#ifdef FS_STANDALONE
#include "../sm-cifs/cifs-api.h"
#include "../sm-cifs/cifs-unicode.h"
#include "../sm-cifs/cifs-utils.h"
#include "../sm-cifs/cifs-err.h"
#else
#include <exports/cifs-api.h>
#include <exports/cifs-utils.h>
#include <exports/cifs-unicode.h>
#include <exports/cifs-err.h>
#endif

#include "cifs-rpc.x"


/*-----------------------------------------------------------------
 * Name			: fillUniStrucWithoutAlloc
 *
 * Description	: fill a unicode structure without allocating memory
 * 					for the string
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/16/2002
 *
 *-----------------------------------------------------------------
 */
void
fillUniStrucWithoutAlloc(cifsrpc_uniStruc *usp, uni16 *str)
{
	uint32 length = unistrlen(str) + 1;
	
	memset (usp, 0, sizeof(cifsrpc_uniStruc));	
	usp->maxLen = length;
	usp->len	= length;
	usp->str	= str;
}


/*-----------------------------------------------------------------
 * Name			: fillUniStrucWithAlloc
 *
 * Description	: fill a unicode structure with allocating memory
 * 					for the string
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/16/2002
 *
 *-----------------------------------------------------------------
 */
void
fillUniStrucWithAlloc(cifsrpc_pipe *rpip, cifsrpc_uniStruc *usp, uni16 *str)
{
	uint32 len = unistrlen(str) + 1;
	
	memset (usp, 0, sizeof(cifsrpc_uniStruc));	
	usp->maxLen = len;
	usp->len = len;
	usp->str = (uni16*)cifsrpc_allocMemItem(&rpip->memList, len*sizeof(uni16));
	unistrcpy(usp->str, str);
}


/*-----------------------------------------------------------------
 * Name			: strncpyFromUniStruc
 *
 * Description	: strncpy from an uniStruct structure
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK if fits, NFX_ERR if truncated
strncpyFromUniStruc(uni16 *dstp, cifsrpc_uniStruc *usp, uint32 maxLen)
{
	uni16 *srcp = usp->str;
	uint32 len  = usp->len;
	int32 result= NFX_OK;
	
	if (len > maxLen)
	{	//Truncate the source string
		len = maxLen;
		result = NFX_ERR;
	}
	unistrncpy(dstp, srcp, len);
	
	return (result);
}

