/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-cfg.c,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-cfg.c
 *
 * Description	: Contain CIFS configuration functions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 10/31/2001
 *
 *-----------------------------------------------------------------
 */

#include "cifs-api.h"
#include "cifs-utils.h"
#include "cifs-unicode.h"
#include "cifs-cfg.h"



/*-----------------------------------------------------------------
 * Name			: cifs_loadCfg 
 *
 * Description	: CIFS load Configuration function.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/07/2001
 *
 *-----------------------------------------------------------------
 */
int32 	//NFX_OK or NFX_ERR
cifs_loadCfg(void)
{
	cifs_server *svp;
	cifs_share	*shp;		
	cifs_serverCfg svCfg;
	cifs_shareCfg  shCfg;

	/* For now we fake the configuration coming from the SSC */
	
	memset(&svCfg, 0, sizeof(cifs_serverCfg));
	svCfg.ipAddr		= htonl(CIFSTEST_IPADDR);
	svCfg.subnetMask	= CIFSTEST_NETMASK;
	svCfg.options		= CIFSTEST_OPTIONS;
	svCfg.capabilities	= CIFSTEST_CAPAS;
	strcpy(svCfg.domainName, CIFSTEST_DOMAIN);
	strDos2Uni(svCfg.uniDomainName, CIFSTEST_DOMAIN,
					unisizeof(svCfg.uniDomainName), TRUE);
	strcpy(svCfg.name, CIFSTEST_SVNAME);
	strDos2Uni(svCfg.uniName, CIFSTEST_SVNAME, unisizeof(svCfg.uniName), TRUE);
	strDos2Uni(svCfg.comment, CIFSTEST_SVCOMMENT,
					unisizeof(svCfg.comment), TRUE);
	strcpy(svCfg.trustAccountPwd, CIFSTEST_SVNAME);
	svp = cifs_createServer(&svCfg);
	NFX_ASSERT(svp != NULL);

	memset(&shCfg, 0, sizeof(shCfg));
	strDos2Uni(shCfg.name, "VOL1-SHARE", unisizeof(shCfg.name), TRUE);
	strDos2Uni(shCfg.comment, "VOL1-COMMENT", unisizeof(shCfg.comment), TRUE);
	strDos2Uni(shCfg.pathName, "/vol1/SHARE", unisizeof(shCfg.pathName),TRUE);
	shp = cifs_addShare(svp, &shCfg);
	NFX_ASSERT(shp != NULL);
/*
	memset(&shCfg, 0, sizeof(shCfg));
	strDos2Uni(shCfg.name, "VOL2-SHARE", unisizeof(shCfg.name), TRUE);
	strDos2Uni(shCfg.comment, "VOL2-COMMENT", unisizeof(shCfg.comment), TRUE);
	strDos2Uni(shCfg.pathName, "/vol2/SHARE", unisizeof(shCfg.pathName),TRUE);
	shp = cifs_addShare(svp, &shCfg);
	NFX_ASSERT(shp != NULL);
*/
	return (NFX_OK);
}


