/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-filetools.h,v 1.2 2002/12/10 18:13:18 laurentb Exp $
 *
 * Name			: cifs-filetools.h
 *
 * Description	: Contains CIFS file tools header.
 * 				  NOTE: Internal path refers to the file system path
 * 				  format which is encoded in utf-8.
 *
 * Created by	: Laurent Brard
 * 				  Xiaobing Zhang
 *
 * Date Created	: 03/19/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_FILETOOLS_H
#define _CIFS_FILETOOLS_H


#define CIFS_MAX_NUM_OF_ATTR	10


#define cifs_convertToUnixPath(path)	uniCharReplace(path, UNICHR_BACKSLASH,\
														UNICHR_SLASH)
#define cifs_convertToDosPath(path)		uniCharReplace(path, UNICHR_SLASH, \
														UNICHR_BACKSLASH)

#define cifs_initFsAttr(fsattrp)	memset(fsattrp, 0xFF, sizeof(fs_Attr))
 
#define cifs_initFssfAttr(fssfatp)	{}

#define cifs_waitFsResp(evt, parpp, epp)	\
		cifs_waitResp(evt, parpp, epp, TRUE, CIFS_INTMSG_TIMEOUT_FS)

#define cifs_waitFsRespAndKill(evt, parpp, epp)	\
		cifs_waitResp(evt, parpp, epp, FALSE, CIFS_INTMSG_TIMEOUT_FS)

#define cifs_waitFssfResp(evt, parpp, epp)	\
		cifs_waitResp(evt, parpp, epp, TRUE, CIFS_INTMSG_TIMEOUT_FSSF)

#define cifs_waitFssfRespAndKill(evt, parpp, epp)	\
		cifs_waitResp(evt, parpp, epp, FALSE, CIFS_INTMSG_TIMEOUT_FSSF)
		
#define cifs_testWaitResp_ifErrReturn(opp, result)					\
		if (result != CIFSTATUS_OK) 								\
		{   cifs_errResp(opp, 0, 0, CIFSTATUS_INTERNAL_ERROR);		\
			return (CIFS_SENDRESPNOW);								\
		}
	
#define cifs_buildFssfReqAccessFromDesiredAccess(fssfRAcc, desirAcc)		\
		{																	\
			fssfRAcc = 0;													\
			if (desirAcc & (SEC_STDR_READ_ACCESS | SEC_FILE_READ_DATA))	\
				fssfRAcc |= FSSF_ACCESS_READ;								\
			if (desirAcc & SEC_GENR_FILE_WRITE & (~SEC_STDR_SYNCH_ACCESS))\
				fssfRAcc |= FSSF_ACCESS_WRITE;								\
			if (desirAcc & SEC_STDR_DELETE_ACCESS) 						\
				fssfRAcc |= FSSF_ACCESS_DELETE;								\
		}

#define cifs_getDesiredAccessFromOpenMode(oMode)							\
		(((oMode & 0x7F) == 0)	? SEC_GENR_FILE_READ|SEC_GENR_FILE_WRITE 	\
								: openMode2desiredAccess[oMode & 0xF])
		
#define cifs_getFssfShareAccessFromOpenMode(oMode)							\
		(fssfAction2createAction[(oMode >> 4) & 0x7])	
			
#define cifs_getOpenActionIfExistsFromOFunc(oFunc)							\
		(openFunc2openActionIfExists[oFunc & 0x3])
		
#define cifs_getOpenActionIfNotExistsFromOFunc(oFunc)						\
		((oFunc & 0x10) ? FSSF_IFNOTEXIST_CREATE : FSSF_IFNOTEXIST_FAIL)
		
extern fssf_action_t createDispo2fssfAction[SEC_FILE_CREATEDISPO_MAX];
extern sec_accessMask openMode2desiredAccess[16];
extern fssf_shareMode_t openMode2shareAccess[8];
extern uint16 openFunc2openActionIfExists[4];
extern uint32 fssfAction2createAction[8];



void cifs_decodeFileHandle(fs_Handle *fsHandle);

void cifs_getUserAccessToFile(cifs_operation *opp, uchar8 *access, fs_Attr *fsAttrp);

int32 cifs_getFileNameByHandle(cifs_operation *opp, ushort16 fid, uni16 *fileName);

fs_Attr *cifs_fsGetAttrByType(fs_Attr *attrp, int32 numAttrs, int32 searchAttrType);

int32	//Number of bytes filled, or -1 if there is error
cifs_putShortName(uchar8	*dstp,	//Destination string in UNICODE or ASCII
				  uni16		*srcp,	//Source string in UNICODE
				  int32		maxLen);//Max len in bytes of destination string

fs_WriteStableHow cifs_writeMode2StableHow(ushort16   writeMode);



int64
cifs_fs2LongTime(fs_Time *fsTimep);


void
cifs_fillLongTime(	uchar8	*smbTimep,
					fs_Time	*fsTimep);

void
cifs_fs2SmbDateTime(uchar8  *msgBuf,
					fs_Time	*fsTimep);


void
cifs_fillSmbFileAttr(uchar8		*smbAttrsp,
					fs_Attr		*fsAttrp,
					boolean		extAttributes);


uint32  //CIFSTATUS
cifs_extractPathLastElement(uni16 *path,
							uni16 *lastElem,
							uint32 lastElemSize,
							boolean removeIt);


boolean
cifs_containsWildcard(char *s);


boolean
cifs_uniContainsWildcard(uni16 *s);


boolean
cifs_uniIsExecutable(uni16 *file);


int32
cifs_checkPathSyntax(uni16 *s);


uint32	//CIFSTATUS
cifs_Smb2UniPath(	uchar8 *smbp,		//source SMB message pointer
					uchar8 *spath,		//source path
					uni16  *dpath);		//destination path


uint32	//CIFSTATUS
cifs_Smb2InternalPath(	uchar8	*smbp,		//source SMB message pointer
						uchar8	*spath,		//source path
						uchar8	*dpath,		//destination path
						uni16	*delem,		//destination uni16 last elem
						uint32	*dpathLenp);//destination internal path len


uint32  //CIFSTATUS
cifs_waitResp(uint32	 	expectedEvent,
				void		**parpp,	//pointer on the msg response pointer
				eee_desc_t	**eDescpp,	//pointer on the eee desc pointer
				boolean 	takeCareOfKillEvent,
				uint32		timeout);


uint32	//CIFSTATUS
cifs_fsToCifsErrMapping(fs_Status fsStatus, uchar8 *classp, ushort16 *errp);

void
cifs_returnFsError(cifs_operation *opp, fs_Status fsStatus);

fssf_clientid_t
cifs_buildFssfClientId(cifs_operation *opp);
		
		


#endif // #ifndef _CIFS_FILETOOLS_H 

