/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-msg.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-msg.h
 *
 * Description	: Contains CIFS messages functions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/07/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_MSG_H
#define _CIFS_MSG_H


#define	cifsm_testCIFSTATUS_ifErrReturn(opp, result)					\
			if (result != CIFSTATUS_OK) 								\
			{	cifs_errResp(opp, 0, 0, result); return (CIFS_SENDRESPNOW); }
				
#ifndef FS_STANDALONE
#define	cifsm_testCIFSTATUS_ifErrFreeDescAndReturn(opp, result, eDescp)	\
			if (result != CIFSTATUS_OK) {								\
				cifs_errResp(opp, 0, 0, result);						\
				eee_freePacket(eDescp);									\
				return (CIFS_SENDRESPNOW);								\
			}

#define	cifsm_testFsStatus_ifErrReturn(opp, status, eDescp)				\
			if (status != FS_ERR_NONE)	{ 								\
				cifs_returnFsError(opp, status);						\
				eee_freePacket(eDescp);									\
				return (CIFS_SENDRESPNOW);								\
			}
#else
// FS standalone simulation will free the eee descriptor pointed by eDescp
#define	cifsm_testCIFSTATUS_ifErrFreeDescAndReturn(opp, result, eDescp)	\
			if (result != CIFSTATUS_OK) {								\
				cifs_errResp(opp, 0, 0, result);						\
				return (CIFS_SENDRESPNOW);								\
			}

#define	cifsm_testFsStatus_ifErrReturn(opp, status, eDescp)				\
			if (status != FS_ERR_NONE)	{ 								\
				cifs_returnFsError(opp, status);						\
				return (CIFS_SENDRESPNOW);								\
			}
#endif // FS_STANDALONE

//Returns CIFS_SENDRESPNOW or CIFS_DONTSENDRESP

int32 cifsm_createDir(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_delDir(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_open(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_create(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_close(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_flush(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_delete(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_rename(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_queryInfo(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_setInfo(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_read(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_write(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_lockByte(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ulockByte(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_createTmp(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_createNew(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_checkPath(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_exit(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_seek(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_lockRead(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_wrUnlock(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_setInf2(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_queryInf2(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_lockNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_transac(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_transacS(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ioctl(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_copy(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_move(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_echo(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_wrClose(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_openNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_readNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_writeNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_transac2(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_transac2S(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_findClos2(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_findNClos(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_treeCon(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_treeDisc(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_negociate(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ssSetupNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_logoffNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_treeConNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_infoDisk(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_search(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_findClose(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ntTrans(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ntTransS(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ntCreatNX(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);
int32 cifsm_ntCancel(cifs_client *clp, cifs_session *ssp, 
					cifs_shareInUse *siup, cifs_operation *opp,
					uchar8 *icmdp, uchar8 *ocmdp);

void cifs_msgRcv(eee_desc_t *eDescp);

#endif /* #ifndef _CIFS_MSG_H */

