/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: cifs.x,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs.x
 *
 * Description	: Contains CIFS utility macros
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/07/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_X
#define _CIFS_X


// SMB header fields offsets
#define SMBO_CMD	4
#define SMBO_ERCL	(SMBO_CMD+1)
#define SMBO_ERRS	(SMBO_CMD+2)
#define SMBO_ERR	(SMBO_CMD+3)
#define SMBO_FLGS	(SMBO_CMD+5)
#define SMBO_FLG2	(SMBO_CMD+6)
#define SMBO_PIDH	(SMBO_CMD+8)
#define SMBO_TID	(SMBO_CMD+20)
#define SMBO_PID	(SMBO_CMD+22)
#define SMBO_UID	(SMBO_CMD+24)
#define SMBO_MID	(SMBO_CMD+26)
#define SMBO_WCNT	(SMBO_CMD+28)

// Command fields offsets
#define SMBCO_WP0	(1)
#define SMBCO_WP1	(3)
#define SMBCO_WP2	(5)
#define SMBCO_WP3	(7)
#define SMBCO_WP4	(9)
#define SMBCO_WP5	(11)
#define SMBCO_WP6	(13)
#define SMBCO_WP7	(15)
#define SMBCO_WP8	(17)
#define SMBCO_WP9	(19)
#define SMBCO_WP10	(21)
#define SMBCO_WP11	(23)
#define SMBCO_WP12	(25)
#define SMBCO_WP13	(27)
#define SMBCO_WP14	(29)
#define SMBCO_WP15	(31)
#define SMBCO_WP16	(33)
#define SMBCO_WP17	(35)
#define SMBCO_WP19	(39)
#define SMBCO_WP20	(41)
#define SMBCO_WP21	(43)
#define SMBCO_WP22	(45)
#define SMBCO_WP23	(47)
#define SMBCO_WP24	(49)
#define SMBCO_WP25	(51)
#define SMBCO_WP26	(53)
#define SMBCO_WP27	(55)
#define SMBCO_WP28	(57)
#define SMBCO_WP29	(59)
#define SMBCO_WP30	(61)
#define SMBCO_WP31	(63)
#define SMBCO_WP32	(65)
#define SMBCO_WP33	(67)
#define SMBCO_WP34	(69)
#define SMBCO_WP35	(71)
#define SMBCO_WP36	(73)
#define SMBCO_BCNT(cmd)	(((GCAR(cmd,0))*2) + SMBCO_WP0)
#define SMBCO_BP0(cmd)	(SMBCO_BCNT(cmd) + 2)
		
//CIFS commands fields pointers
//Pointer on byte parameters
#define SMBCP_BPAR(cmd) (((char*)cmd) + SMBCO_BP0(cmd))

//SMB header size
#define SMB_S_HEADER	(SMBO_WCNT)


//Command specific offsets

//NT_CREATE_NX / NT_TRANSAC_CREATE
#define SMBCO_NTCREATE_NAMELENGTH	(SMBCO_WP2+1)
#define SMBCO_NTCREATE_CREATEFLAGS	(SMBCO_WP3+1)
#define SMBCO_NTCREATE_ROOTDIRFID	(SMBCO_WP5+1)
#define SMBCO_NTCREATE_DESIREDACC	(SMBCO_WP7+1)
#define SMBCO_NTCREATE_ALLOCSIZE	(SMBCO_WP9+1)
#define SMBCO_NTCREATE_FILEATTR		(SMBCO_WP13+1)
#define SMBCO_NTCREATE_SHAREACC		(SMBCO_WP15+1)
#define SMBCO_NTCREATE_CREATEDISPO	(SMBCO_WP17+1)
#define SMBCO_NTCREATE_CREATEOPTS	(SMBCO_WP19+1)
#define SMBCO_NTCREATE_IMPERSOLVL	(SMBCO_WP21+1)
#define SMBCO_NTCREATE_SECURITYFLGS	(SMBCO_WP23+1)

//READ_NX
#define SMBCO_READNX_FID			(SMBCO_WP2)
#define SMBCO_READNX_OFFSET_LOW		(SMBCO_WP3)
#define SMBCO_READNX_MAX_CNT_LOW	(SMBCO_WP5)
#define SMBCO_READNX_MIN_CNT		(SMBCO_WP6)
#define SMBCO_READNX_MAX_CNT_HIGH	(SMBCO_WP7)
#define SMBCO_READNX_OFFSET_HIGH	(SMBCO_WP10)

//WRITE_NX
#define SMBCO_WRITENX_FID			(SMBCO_WP2)
#define SMBCO_WRITENX_OFFSET_LOW	(SMBCO_WP3)
#define SMBCO_WRITENX_WRITE_MODE	(SMBCO_WP7)
#define SMBCO_WRITENX_RAMAIN		(SMBCO_WP8)
#define SMBCO_WRITENX_DATA_LEN_HIGH	(SMBCO_WP9)
#define SMBCO_WRITENX_DATA_LEN_LOW	(SMBCO_WP10)
#define SMBCO_WRITENX_DATA_OFFSET	(SMBCO_WP11)
#define SMBCO_WRITENX_OFFSET_HIGH	(SMBCO_WP12)
#define SMBCO_WRITENX_BYTE_CNT		(SMBCO_WP14)

//Number of bytes between 2 addresses
#define SMB_DIFF(a1,a2) ((address_t)(((const char*)(a1)) - (const char*)(a2)))

//Test if a batched command follows the current one
#define SMB_BATCHEDCMDFOLLOWS(cmdp)	(GCAR(cmdp, SMBCO_WP0) != 0xFF)

//Test bit in SMB little endian fields
#define SMB_TESTBIT(ptr,off,bit) ((GCAR(ptr,off) & (1<<(bit))) != 0)
#define SMB_TESTBIT2(ptr,off,bit) ((GETLE2(ptr,off) & (1<<(bit))) != 0)

//Test flags bits
#define SMB_TESTFLAGSBIT(smbp, bit)		(SMB_TESTBIT(smbp, SMBO_FLGS, bit))
#define SMB_TESTFLAGS2BIT(smbp, bit)	(SMB_TESTBIT2(smbp, SMBO_FLG2, bit))


// Macros for 16 bits, 32 bits and 64 bits manipulation without alignment issue

#define GCAR(msg,off) (((uchar8 *)(msg))[off])
#define PCAR(msg,off,val) (GCAR(msg,off)=(val))
#define GCAR2(msg,off) ((uint32)GCAR(msg,off))
#define PUTLE2X(msg,off,val) (GCAR(msg,off)=(val)&0xFF,GCAR(msg,off+1)=(val)>>8)
#define PUTLE4X(msg,off,val) (PUTLE2X(msg,off,val&0xFFFF), \
							  PUTLE2X(msg,off+2,val>>16))
#define PUTLE8X(msg,off,val) (PUTLE4X(msg,off,val&0xFFFFFFFF), \
							  PUTLE4X(msg,off+4,val>>32))
#define PUTBE2X(msg,off,val) (GCAR(msg,off)=(val)>>8,GCAR(msg,off+1)=(val)&0xFF)
#define PUTBE4X(msg,off,val) (PUTBE2X(msg,off,val>>16), \
                              PUTBE2X(msg,off+2,val&0xFFFF))
#define PUTBE8X(msg,off,val) (PUTBE4X(msg,off,val>>32), \
                              PUTBE4X(msg,off+4,val&0xFFFFFFFF))

/* Little Indian operations */		
// Get a 2 byte little endian value 
#define GETLE2(msg,off) (GCAR2(msg,off) | GCAR2(msg,(off)+1)<<8)
// Get a 4 byte little endian value 
#define GETLE4(msg,off) (GETLE2(msg,off) | GETLE2(msg,(off)+2)<<16)
// Get a 8 byte little endian value 
#define GETLE8(msg,off) (GETLE4(msg,off) | GETLE4(msg,(off)+4)<<32)
// Signed GETLE2()
#define GETLE2S(msg,off) ((short16)GETLE2(msg,off))
// Signed GETLE4()
#define GETLE4S(msg,off) ((int32)GETLE4(msg,off))
// Signed GETLE8()
#define GETLE8S(msg,off) ((int64)GETLE8(msg,off))
		
// Put a 2 byte little endian value
#define PUTLE2(msg,off,val) PUTLE2X((msg),(off),((ushort16)(val)))
// Put a 4 byte little endian value		
#define PUTLE4(msg,off,val) PUTLE4X((msg),(off),((uint32)(val)))
// Put a 8 byte little endian value		
#define PUTLE8(msg,off,val) PUTLE8X((msg),(off),((uint64)(val)))
// Signed PUTLE2()		
#define PUTLE2S(msg,off,val) PUTLE2X((msg),(off),((short16)(val)))
// Signed PUTLE4()		
#define PUTLE4S(msg,off,val) PUTLE4X((msg),(off),((int32)(val)))
// Signed PUTLE8()		
#define PUTLE8S(msg,off,val) PUTLE8X((msg),(off),((int64)(val)))

/* Big Endian operations */
// Get a 2 byte big endian value		
#define GETBE2(msg,off)	(GCAR2(msg,off)<<8 | GCAR2(msg,(off)+1))
// Get a 4 byte big endian value		
#define GETBE4(msg,off) (GETBE2(msg,off)<<16 | GETBE2(msg,(off)+2))
// Get a 8 byte big endian value		
#define GETBE8(msg,off) (GETBE4(msg,off)<<32 | GETBE4(msg,(off)+4))
// Signed GETBE2()		
#define GETBE2S(msg,off) ((short16)GETBE2(msg,off))
// Signed GETBE4()		
#define GETBE4S(msg,off) ((int32)GETBE4(msg,off))
// Signed GETBE8()		
#define GETBE8S(msg,off) ((int64)GETBE8(msg,off))
		
// Put a 2 byte big endian value		
#define PUTBE2(msg,off,val) PUTBE2X((msg),(off),((ushort16)(val)))
// Put a 4 byte big endian value		
#define PUTBE4(msg,off,val) PUTBE4X((msg),(off),((uint32)(val)))
// Put a 8 byte big endian value		
#define PUTBE8(msg,off,val) PUTBE8X((msg),(off),((uint64)(val)))
// Signed PUTBE2()		
#define PUTBE2S(msg,off,val) PUTBE2X((msg),(off),((short16)(val)))
// Signed PUTBE4()		
#define PUTBE4S(msg,off,val) PUTBE4X((msg),(off),((int32)(val)))
// Signed PUTBE8()		
#define PUTBE8S(msg,off,val) PUTBE8X((msg),(off),((int64)(val)))





#endif /* #ifndef _CIFS_X */

