/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: nfp-objects-api.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: nfp-objects.h
 *
 * Description	: NFP objects management functions
 * 				  This is the generic version of NFPv1 done by Joseph Vu
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/09/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFP_OBJECTS_H
#define _NFP_OBJECTS_H

#define NFP_S_OBJECTNAME		31	


// NFP object list
typedef struct nfp_objList {
  void		*listBasep;						// Pointer on array of objects
  void		*listFreep;						// Pointer on free objects
  uint32	freeObjCnt;						// Free objects counters
  uint32	objSize;						// Object size
  uint32	nbOfObj;						// Number of objects
  uint32	pad;
  char		objName[NFP_S_OBJECTNAME+1];	// Object name
} nfp_objList;


/*-----------------------------------------------------------------
 * Name: nfp_objInit 
 *
 * Description: Initialize a NFP object list
 *
 * Created by:  Laurent Brard
 *
 * Date Created:        11/09/2001
 *
 *-----------------------------------------------------------------
 */
int32 	//NFX_OK or NFX_ERR
nfp_objInit(nfp_objList* list, char* objName, uint32 objSize, uint32 nbOfObj);


/*-----------------------------------------------------------------
 * Name: nfp_allocObject 
 *
 * Description: Allocate NFP object. 
 * 				Fill the object with 0 if setToZero=TRUE
 *
 * Created by:  Laurent Brard
 *
 * Date Created:        11/09/2001
 *
 *-----------------------------------------------------------------
 */
void *	//Allocated object, NULL if none available
nfp_allocObject(nfp_objList* list, boolean setToZero);


/*-----------------------------------------------------------------
 * Name: nfp_deallocObject 
 *
 * Description: Deallocate a NFP object
 *
 * Created by:  Laurent Brard
 *
 * Date Created:        11/09/2001
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR 
nfp_deallocObject(nfp_objList* list, void *object);
	

/*-----------------------------------------------------------------
 * Name: nfp_validateObject 
 *
 * Description: Validate a NFP object
 *
 * Created by:  Laurent Brard
 *
 * Date Created:        11/09/2001
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR 
nfp_validateObject(nfp_objList* list, void *objp);

#endif

