/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : crypt-api.h
 *
 * Description  : Interface of "encrytion" library.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 06-SEP-2001
 *
 *------------------------------------------------------------------------------
 */
#ifndef _CRYPT_API_H_
#define _CRYPT_API_H_

/*------------------------------------------------------------------------------
 * Include files.
 *------------------------------------------------------------------------------
 */

#include <openssl/evp.h>

/*------------------------------------------------------------------------------
 * Constants and macros definition.
 *------------------------------------------------------------------------------
 */

/* Digest algorithms
 */
#define MD5         "md5"
#define SHA         "sha"
#define SHA1        "sha1"
#define RIPEMD160   "ripemd160"
//#define MD2         "md2"     // not supported
//#define MDC2        "mdc2"    // not supported

/* Maximun size of a password.
 */
#define CRYPT_MAX_PASSWD_LEN    256

/*------------------------------------------------------------------------------
 * Types definition.
 *------------------------------------------------------------------------------
 */

/* Message digest.
 */
typedef struct
{
    int             len;
    unsigned char   value[EVP_MAX_MD_SIZE];
} crypt_md_t;

/*------------------------------------------------------------------------------
 * Prototype of global functions.
 *------------------------------------------------------------------------------
 */

int                                 // NFX_OK, NFX_ERR      - OUPUT
crypt_encode(char * plainStr,       // string to encode     - INPUT
             char * encodStr);      // encoded string       - OUPUT

int                                 // NFX_OK, NFX_ERR      - OUPUT
crypt_decode(char * encodStr,       // string to decode     - INPUT
             char * decodStr);      // decoded string       - OUPUT

int                                 // NFX_OK, NFX_ERR              - OUPUT
crypt_digest(char *         algo,   // MD5, SHA, SHA1...            - INPUT
             char *         msg,    // message to digest            - INPUT
             size_t         msgLen, // length do message to digest  - INPUT
             crypt_md_t *   md);    // message digest               - OUPUT

int                                 // NFX_OK, NFX_ERR      - OUPUT
crypt_getPasswd(char * newPasswd);  // new password         - OUPUT

/*----------------------------------------------------------------------------*/

#endif // _CRYPT_API_H_
