/*  
 *=============================================================================
 * File Name   :    dp83820-tx.c
 *
 *  Description:    
 *
 *  Created    :    30 April 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Version    :    $Id: dp83820-tx.c,v 1.47 2002/11/07 01:21:00 rick Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/mem-api.h>
#include <sys/cache.h>
#include "exports/debug.h"
#include "dp83820.h"
#include "dp83820-ip.h"

uint32 *dp83820_cr;

uint32 dp83820_tx_pkt_trace = 0;

int dp83820_txBufCapture(char *, uint32);
int dp83820_txBufWatchRem(char *); 
int dp83820_txBufWatchAdd(char *); 

extern int dp83820_printBuf(char *, int, int);
extern void *eee_getBufBase(void *);
extern int32 eee_getBufSize(int32);
extern int32 eee_getBufType(void *);

#ifdef FCNIM
#define DP83820_SCSI_HISTORY	1 
//-- SCSI Req/Rsp History
typedef struct _scsi_history {
    uint64    id;
    uint32    op;
    uint32    lba;
    uint32    lbCnt;
    uint32    cHandle;
} scsi_history_t;
#endif

extern char *dp83820_txBufWatch[DP83820_TX_Q_LEN];

extern int bp_setDestMacAddr( uint32 destPort, uint32 remotePortNum,
							  char *buf );

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_txUnicast
 *
 * Description: 	Transmits a unicast packet to the backplane.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_txUnicast( uint32 dest, eee_desc_t *pktDesc )
{
	int i, priQ;
	int bufCnt;
	int pktLen;
	uint32 cmdsts, dest_slot, dest_cpu;
	volatile dp83820_txCtrl_t *txc;
	volatile dp83820_desc_t *txCur;
	int remotePort;
	int dType;

	// Do this to send packet across exteral ethernet network.
	dest_slot = EEE_GET_SLOT_NUM(dest);
	dest_cpu = EEE_GET_DEST_CPU(dest);
	dType = EEE_GET_DTYPE(dest);

#if defined(NFX_SMP) && defined(NFP_TXRX) && defined(SMP_TXRX_VERIFY)
	if (!CM_IS_TXRX_NCPU)
		panic("%s: called in acpu \n", __FUNCTION__);
#endif /* NFX_SMP */
	
#ifdef FCNIM
	//-- Pick the remote NFP_FP mac port to send to for 
	//-- load balancing purposes.
	// Although Cheetah FP has 6 backplane ports and Garfield only has 2 for 
	// control/data planes, both uses this addressing scheme, so a check does
	// not have to be made for which system type to conform to.
    //
    // src code as of oct 6 2002 has the FC slice #'s (physical)
    // remaped in inverse order with the FC cpuids (logical),
    // for the direct-connect backplanes (ie. no switch) the destination
    // port has to be selected correctly.
    // FC CPUID FC SLICE    FP LSI DATA PORT    FP LSI CONTROL PORT
    // 3        0           1                   3            
    // 2        1           2                   3
    // 1        2           4                   6
    // 0        3           5                   6
    if (dest_slot == 1) {
        switch (nfxMyCPUId) {
            case NFX_FCNIM_CPUID3:
                if (dType == EEE_CONTROL)
                    remotePort = 3;
                else
                    remotePort = 1;
                break;
            case NFX_FCNIM_CPUID2:
                if (dType == EEE_CONTROL)
                    remotePort = 3;
                else
                    remotePort = 2;
                break;
            case NFX_FCNIM_CPUID1:
                if (dType == EEE_CONTROL)
                    remotePort = 6;
                else
                    remotePort = 4;
                break;
            case NFX_FCNIM_CPUID0:
                if (dType == EEE_CONTROL)
                    remotePort = 6;
                else
                    remotePort = 5;
                break;
            default:
				printf("dp83820_txUnicast: invalid nfxMyCPUId=%d\n",nfxMyCPUId);
				remotePort = 0;
                break;
        }
    }
    else {
		remotePort = 1;
	}
#else
	remotePort = 1;
#endif

#ifdef PROM
	{
	uchar8	* bptr;
	bptr = pktDesc->bd[1].buf;
	//-- Look at the ethernet type field to see
	//-- if this is an IP packet. If not then add EEE header.
	if( (*(ushort16 *)(bptr+12) != 0x0008) && 
									(*(ushort16 *)(bptr+12) != 0x0608) )
		eee_addHeader(pktDesc, remotePort, dp83820->macAddr);
	}
#else
	if (pktDesc->hdr.control & EDESC_COMMAND_REUSE_HEADER) // retransmit case
		;
	else
	if (eee_addHeader(pktDesc, remotePort, dp83820->macAddr) == NFX_ERR) {
		return EEE_DROP_PACKET;
	}
#endif

	priQ = 0;		/* TODO: Get this value from the descriptor */
	txc = &dp83820->txc[priQ];
	txCur = (volatile dp83820_desc_t *)txc->txCur;

	pktLen = pktDesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = (pktDesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	

	/* 
	 * Check for room on the transmit queue.
	 */
	if( bufCnt > txc->avail-1 ) {
		printf("tx: no room\n");
		/*
		 * Call transmit cleanup to try and free up some tx descritors.
		 */
		//dp83820_txCleanupQ( txc );
		if( bufCnt > txc->avail-1 ) {
			txc->qFull++;
			CACHE_FLUSH_PACKET(pktDesc);
			return( EEE_DROP_PACKET );
		}
	}

	if( dp83820_tx_pkt_trace ) {
		printf("bufCnt: %d, txCur: %lx\n", bufCnt, (address_t)txCur);
	}

	i = 1;
	if( bufCnt == 1 ) {
		uint32 pktLen;
#if DP83820_TX_SAFEMODE
		/* Make sure driver owns descriptor. */
		if( txCur->cmdsts & DP83820_CMDSTS_OWN ) {
			printf("tx: q full\n");
			dp83820->txc[priQ].qFull++;
			CACHE_FLUSH_PACKET(pktDesc);
			return( EEE_DROP_PACKET );
		}
#endif /* DP83820_TX_SAFEMODE */
		txCur->cmdsts  = 0;
		txCur->pktDesc = pktDesc;

		if( dp83820_tx_pkt_trace ) {
			CACHE_FLUSH_PACKET(pktDesc);
			printf("pktDesc: %lx, tx buf cnt: %d, buf ptr: %lx, buf len: %d\n", 
			   	(address_t)pktDesc, bufCnt, (address_t)pktDesc->bd[1].buf, 
				pktDesc->bd[1].len);

#ifdef SIBYTE
			dp83820_printBuf( (char *)(address_t)pktDesc->bd[1].buf, pktDesc->bd[1].len, 1); 
#else
			dp83820_printBuf( (char *)((address_t)pktDesc->bd[1].buf | 0x20000000)
								, pktDesc->bd[1].len, 1); 
#endif

		}

		if( dp83820_pc.txEna ) {
			dp83820_txBufWatchAdd(pktDesc->bd[1].buf);
		}

#ifdef LARGE_MEMORY
		if (((address_t)pktDesc->bd[1].buf < 0x2000000000) || ((address_t)pktDesc->bd[1].buf >= 0x2000700000))
			panic("dp83820 single buffer: invalid mgmt buffer on transmit, buf = %p, desc = %p\n", pktDesc->bd[1].buf, pktDesc);
		txCur->bufptr = (uchar8 *)VIRT_TO_PHYS((address_t)pktDesc->bd[1].buf);
#else
#ifdef FC_USE_NEW_MEM_MAP
		txCur->bufptr = (uchar8 *)virt_to_phys((address_t)pktDesc->bd[1].buf);
#else
		txCur->bufptr = (uchar8 *)((address_t)(pktDesc->bd[1].buf) & ~0xA0000000);
#endif
#endif
		pktLen = pktDesc->bd[1].len;

		cmdsts = (DP83820_CMDSTS_OWN   |
				  DP83820_CMDSTS_INTR  |
				  (DP83820_CMDSTS_LEN & pktLen));

		//-- FSDEBUG CACHE_FLUSH_PACKET(pktDesc);
		CACHE_FLUSH(pktDesc->bd[1].buf, pktDesc->bd[1].buf + 256);    	

		txCur->cmdsts  = cmdsts;

		txc->avail--;
		txc->qd++;
		if( ++txCur >= txc->txqEnd )
			txCur = txc->txqStart;
	}
	else {
		if( pktDesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
		    i=0;
    	else
		    i=1;  

		while( bufCnt-- ) {
#if DP83820_TX_SAFEMODE
			/* Make sure driver owns descriptor. */
			if( txCur->cmdsts & DP83820_CMDSTS_OWN ) {
				printf("tx: q full\n");
				txc->qFull++;
				return( EEE_DROP_PACKET );
			}
#endif /* DP83820_TX_SAFEMODE */

			txCur->cmdsts  = 0;

			if( dp83820_tx_pkt_trace ) {
				CACHE_FLUSH_PACKET(pktDesc);
				printf("pktDesc: %lx, tx buf cnt: %d, buf ptr: %lx, buf len: %d\n", 
					   (address_t)pktDesc, bufCnt, (address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len);
#if defined(SIBYTE)
				if(bufCnt==
						(pktDesc->hdr.control&EDESC_BUFCNT_MASK)>>EDESC_BUFCNT_SHIFT)
					dp83820_printBuf(
							pktDesc->bd[i].buf,
							pktDesc->bd[i].len, 1);
				else
					dp83820_printBuf( 
							pktDesc->bd[i].buf,
							pktDesc->bd[i].len, 0);

#else
				if(bufCnt==
						(pktDesc->hdr.control&EDESC_BUFCNT_MASK)>>EDESC_BUFCNT_SHIFT)
					dp83820_printBuf(
							(char *)((uint32)pktDesc->bd[i].buf | 0x20000000),
							pktDesc->bd[i].len, 1);
				else
					dp83820_printBuf( 
							(char *)((uint32)pktDesc->bd[i].buf | 0x20000000),
							pktDesc->bd[i].len, 0);
#endif
			}

#ifdef LARGE_MEMORY
			if (((address_t)pktDesc->bd[i].buf < 0x2000000000) || ((address_t)pktDesc->bd[i].buf >= 0x2000700000))
				panic("dp83820 multi buffer: invalid mgmt buffer on transmit, buf = %p, desc = %p, i = %d\n", pktDesc->bd[i].buf, pktDesc, i);
			txCur->bufptr = (uchar8 *)VIRT_TO_PHYS((address_t)pktDesc->bd[i].buf);
#else
#ifdef FC_USE_NEW_MEM_MAP
			txCur->bufptr = (uchar8 *)virt_to_phys((address_t)pktDesc->bd[i].buf);
#else
			txCur->bufptr = (uchar8 *)((address_t)(pktDesc->bd[i].buf) & ~0xA0000000);
#endif
#endif

			if( dp83820_pc.txEna ) {
				dp83820_txBufWatchAdd(pktDesc->bd[i].buf);
			}

			if( bufCnt ) {
			    cmdsts = DP83820_CMDSTS_OWN	| DP83820_CMDSTS_MORE	|
					    (DP83820_CMDSTS_LEN & pktDesc->bd[i].len);	
			}
			else {
				txCur->pktDesc = pktDesc;
				cmdsts = DP83820_CMDSTS_OWN	| 
					(DP83820_CMDSTS_LEN & pktDesc->bd[i].len);	
				CACHE_FLUSH_PACKET(pktDesc);
			}

			//-- Pass ownership to the DP83820
			txCur->cmdsts = cmdsts;

			i++;
			txc->avail--;
			txc->qd++;

			if( ++txCur >= txc->txqEnd )
				txCur = txc->txqStart;
		}
	}
	// Strobe the tx enable bit.
	dp83820_writeReg(dp83820_cr, 0x00000001);
#if 1
	txc->txpkts[dest_slot][dest_cpu]++;
#endif
	txc->txCur  = (dp83820_desc_t *)txCur;
	txc->pkts++;
	txc->octets += pktLen;

	return(EEE_FORWARDED_PACKET);
} /* dp83820_txUnicast */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_txCleanupQ
 *
 * Description: 	Cleans up descriptors on a transmit queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_txCleanupQ( dp83820_txCtrl_t *txc )
{
	uint32			        cmdsts;
	uint32				    bufCnt;
	uint32					f_noPkt = 1;
	volatile dp83820_desc_t *ccur;
	volatile dp83820_desc_t *tcur;
	eee_desc_t *edesc = NULL;
	uint32 retransmit = 0;

	ccur = txc->txCln;
	tcur = txc->txCur;


	cmdsts = ccur->cmdsts;
	while( ccur != tcur && ((cmdsts & DP83820_CMDSTS_OWN) == 0) ) {

		f_noPkt = 0;

		/*
		 * Look for the last descriptor in a chain. The cmdsts word
		 * in this descriptor contains the transmit completion
		 * status for the packet.
		 */
		if( (cmdsts & DP83820_CMDSTS_MORE) == 0 ) {
			// printf("txCleanupQ: cmdsts = 0x%08x\n", cmdsts);
			edesc = ccur->pktDesc;

			/* debug 
			printf("txClean: bufptr: %x, ", edesc->bd[1].buf);
			printf("buflen: %x, ", edesc->bd[1].len);
			printf("ctrl:   %x\n", edesc->hdr.control);
			debug */

			bufCnt = (edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;

#if 0
#ifdef DP83820_SCSI_HISTORY	//-- Log SCSI Response Messages, only done on FCNIM
#define SCSI_MAX_HISTORY 1024
			{
				extern uint32 scsiHistoryCurEntry;
				extern scsi_history_t scsiHistory[SCSI_MAX_HISTORY]; 
				uint32 *ptr;
				uint32 mid;

				ptr = ((uint32 *)(edesc->bd[1].buf));
				mid = *ptr;
				//-- Check for scsi message type.
				if( mid == 0x0403 ) { // SCSI_MID_WRITE_RSP
					//-- Update the history buffer.
					scsiHistory[scsiHistoryCurEntry].id      = 0x55555555;
					scsiHistory[scsiHistoryCurEntry].op      = mid;
					scsiHistory[scsiHistoryCurEntry].cHandle = *(ptr+1);
					scsiHistory[scsiHistoryCurEntry].lba     = *(ptr+5);
					scsiHistory[scsiHistoryCurEntry].lbCnt   = *(ptr+6);
					if( ++scsiHistoryCurEntry >= SCSI_MAX_HISTORY )
						scsiHistoryCurEntry = 0;

				}
				else 	// SCSI_MID_READ_RSP
				if( mid == 0x0401 ) {
					scsiHistory[scsiHistoryCurEntry].id      = 0xAAAAAAAA;
					scsiHistory[scsiHistoryCurEntry].op      = mid;
					scsiHistory[scsiHistoryCurEntry].cHandle = *(ptr+1);
					scsiHistory[scsiHistoryCurEntry].lba     = *(ptr+5);
					scsiHistory[scsiHistoryCurEntry].lbCnt   = *(ptr+6);
					if( ++scsiHistoryCurEntry >= SCSI_MAX_HISTORY )
						scsiHistoryCurEntry = 0;
				}
			}
#endif	//-- End SCSI Log
#endif

#ifdef DP83820_TX_PKT_CAPTURE  
			//-- Keep trace buffer of all buffers that have been transmitted.
			if( dp83820_pc.txEna ) {
				dp83820_txBufCapture(ccur->bufptr, cmdsts);
				dp83820_txBufWatchRem(ccur->bufptr);
			}
#endif
			if( cmdsts & DP83820_CMDSTS_OK ) {

			}
			else {
				if( cmdsts & DP83820_CMDSTS_TXA ) {
					txc->txaErr++;
					retransmit = 1;
				}
				if( cmdsts & DP83820_CMDSTS_TFU )
					txc->tfuErr++;
				if( cmdsts & DP83820_CMDSTS_CRS )
					txc->crsErr++;
				if( cmdsts & DP83820_CMDSTS_TD )
					txc->tdErr++;
				if( cmdsts & DP83820_CMDSTS_ED )
					txc->edErr++;
				if( cmdsts & DP83820_CMDSTS_OWC )
					txc->owcErr++;
				if( cmdsts & DP83820_CMDSTS_EC )
					txc->ecErr++;
			}

			if (retransmit == 0)
				eee_freePacket(edesc);     

			ccur->pktDesc = 0;		

			txc->avail+=bufCnt;
			txc->unqd+=bufCnt;
		}
		else {

#ifdef DP83820_TX_PKT_CAPTURE  
			//-- Keep trace buffer of all pkts that have been transmitted.
			if( dp83820_pc.txEna ) {
				dp83820_txBufCapture(ccur->bufptr, cmdsts);
				dp83820_txBufWatchRem(ccur->bufptr);
			}
#endif
		}

		// printf("TxClean: Ptr: %x\n", ccur);		 MYDEBUG
		ccur->cmdsts = 0;
		ccur->bufptr = 0;			

		if( ++ccur >= txc->txqEnd )
			ccur = txc->txqStart;

		if (retransmit) // resume with clean-up next time
			break;

		cmdsts = ccur->cmdsts;
	}

	txc->txCln = (dp83820_desc_t *)ccur;

#if 0
	// Strobe the tx enable bit.
	if( f_noPkt ) {
		if( ccur != tcur )
			dp83820_writeReg(dp83820_cr, 0x00000001);
	}
#endif
	if (retransmit) {
		edesc->hdr.control |= EDESC_COMMAND_REUSE_HEADER;
		dp83820_txUnicast( edesc->hdr.dest_port, edesc );
	}

	return(NFX_OK);
} /* dp83820_txCleanupQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_txCleanup
 *
 * Description: 	Polling function registered with the EEE layer and 
 * 					called to cleanup the transmit queues.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
void
dp83820_txCleanPoll(void *cb, uint32 tlimit)
{
	int i;
	
#if defined(NFX_SMP) && defined(NFP_TXRX)
	if (!CM_IS_TXRX_NCPU)
		return;
#endif /* NFX_SMP, NFP_TXRX */
	
	for(i=0; i < DP83820_TX_Q_CNT; i++)
		dp83820_txCleanupQ( &dp83820->txc[i] );
	return;
} /* dp83820_txCleanup */

int
dp83820_txPktCapture(eee_desc_t *edesc, uint32 cmdsts)
{
#ifdef DP83820_TX_PKT_CAPTURE  
	int32 remain;
	char   *capBuf;
	uint32 cpyLen;
	uint32 capLen;
	uint32 pktLen;
	eee_descBuf_t *bd;

	pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;

	//-- Figure out how many bytes to copy from pkt.
	if( pktLen > DP83820_BYTES_TRACE_MAX )
		capLen = DP83820_BYTES_TRACE_MAX;
	else
		capLen = pktLen;

	//-- Save len information and timestamp pkt
	dp83820_tx_ptb[dp83820_curTxTraceEntry].capLen = capLen;
	dp83820_tx_ptb[dp83820_curTxTraceEntry].pktLen = pktLen;
	dp83820_tx_ptb[dp83820_curTxTraceEntry].cmdsts = cmdsts;
	eee_rtc(&(dp83820_tx_ptb[dp83820_curTxTraceEntry].t));

	//-- Get first buffer of the packet.
	if( edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
		    bd = &edesc->bd[0];
    	else
		    bd = &edesc->bd[1];

	//-- Figure out how many bytes to capture from the first buffer.
	if( bd->len < capLen )
		cpyLen = bd->len;
	else
		cpyLen = capLen;

	//-- Copy data from first buffer
	capBuf = dp83820_tx_ptb[dp83820_curTxTraceEntry].bytes; 
	memcpy(capBuf, bd->buf, cpyLen);

	//-- See if anymore bytes need to come from second buffer.
	remain = capLen - cpyLen; 
	if( remain > 0 ) {
		bd++;
		if( bd->control & EDESC_BD_CMD_IN_USE ) {
			capBuf += cpyLen;

			if( bd->len < remain )
				cpyLen = bd->len;
			else
				cpyLen = remain;

			memcpy(capBuf, bd->buf, cpyLen);
		}
	}

	if( ++dp83820_curTxTraceEntry >= DP83820_TRACE_ENTRIES_MAX )
		dp83820_curTxTraceEntry = 0;
#endif

	return 0;
} // dp83820_txPktCapture

int
dp83820_txBufCapture(char *buf, uint32 cmdsts)
{
#ifdef DP83820_TX_PKT_CAPTURE  
	char   *capBuf;
	char   *cpyBuf;
	uint32 capLen;
	uint32 bufLen;

	if( !buf ) {
		printf("dp83820:  Error capture null buf ptr.\n");
		//-- Save len information and timestamp pkt
		dp83820_tx_ptb[dp83820_curTxTraceEntry].capLen = 0;
		dp83820_tx_ptb[dp83820_curTxTraceEntry].pktLen = 0;
		dp83820_tx_ptb[dp83820_curTxTraceEntry].cmdsts = cmdsts;
		dp83820_tx_ptb[dp83820_curTxTraceEntry].orgBuf = 0;
		eee_rtc(&(dp83820_tx_ptb[dp83820_curTxTraceEntry].t));
	}
	else {
		bufLen = cmdsts & DP83820_CMDSTS_LEN;
		cpyBuf = (char *)(((address_t)(buf)) | 0x80000000);

		//-- Figure out how many bytes to copy from pkt.
		if( bufLen > DP83820_BYTES_TRACE_MAX )
			capLen = DP83820_BYTES_TRACE_MAX;
		else
			capLen = bufLen;

		//-- Save len information and timestamp pkt
		dp83820_tx_ptb[dp83820_curTxTraceEntry].capLen = capLen;
		dp83820_tx_ptb[dp83820_curTxTraceEntry].pktLen = bufLen;
		dp83820_tx_ptb[dp83820_curTxTraceEntry].cmdsts = cmdsts;
		dp83820_tx_ptb[dp83820_curTxTraceEntry].orgBuf = buf;
		eee_rtc(&(dp83820_tx_ptb[dp83820_curTxTraceEntry].t));

		//-- Copy data from buffer
		capBuf = dp83820_tx_ptb[dp83820_curTxTraceEntry].bytes; 
		memcpy(capBuf, cpyBuf, capLen);
	}

	if( ++dp83820_curTxTraceEntry >= DP83820_TRACE_ENTRIES_MAX )
		dp83820_curTxTraceEntry = 0;
#endif

	return 0;
} // dp83820_txBufCapture

int
dp83820_txBufWatchAdd(char *buf) 
{
	uint32 i,j = 0;

	for(i=0; i < DP83820_TX_Q_LEN; i++) {
		if( dp83820_txBufWatch[i] == buf ) {
			printf("Buffer ptr already on tx desc q! %lx\n", (address_t)buf);
			VERIFY(FALSE);
		}
		else
		if( dp83820_txBufWatch[i] == 0 ) {
			j=i;
		}
	}
	dp83820_txBufWatch[j] = buf;
	return 0;
} // dp83820_txBufWatchAdd

int
dp83820_txBufWatchRem(char *buf) 
{
	uint32 i;
	char *remBuf;

	remBuf = (char *)(((address_t)(buf)) | 0x80000000);
	for(i=0; i < DP83820_TX_Q_LEN; i++) {
		if( dp83820_txBufWatch[i] == remBuf ) {
			 dp83820_txBufWatch[i] = 0;
			 return 0;
		}
	}
	return 0;
} // dp83820_txBufWatchRem
