/*  
 *=============================================================================
 * File Name   :    dp83820.h
 *
 *  Description:    
 *
 *  Created    :    26 April 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#define NFX_DROP_PACKET			-1

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include "dp83820-regs.h"
#ifdef SIBYTE
#include <exports/hwMap.h>
#endif

#ifndef DP83820_H_INCLUDED
#define DP83820_H_INCLUDED

#ifdef AGILE_HARDWARE

#if defined(SIBYTE)
#define DP83820_BASE_ADDR		0x50000000		
#define DP83820_BUSID			0
#define DP83820_DEVID			4
#else
#ifdef FC_USE_NEW_MEM_MAP
#define DP83820_BASE_ADDR		0x3a000000		
#else
#define DP83820_BASE_ADDR		0x72000000		
#endif
#define DP83820_BUSID			8
#define DP83820_DEVID			7
#endif

#endif // AGILE_HARDWARE

#ifdef EVAL_BOARD
#ifdef SIBYTE
#define DP83820_BASE_ADDR		0x41200000		
#define DP83820_BUSID			2
#define DP83820_DEVID			1

#else

// #define DP83820_BASE_ADDR		0x12000000		
// #define DP83820_BUSID			0

#define DP83820_BASE_ADDR		0x16000000		
#define DP83820_BUSID			8
//#define DP83820_DEVID			7
#define DP83820_DEVID			8

#endif // SIBYTE
#endif // EVAL_BOARD 

#ifdef LINUX_TEST
#define DP83820_BASE_ADDR		0		/* TODO	*/
#endif /* LINUX_TEST */

#define DP83820_TX_SAFEMODE		1
#define DP83820_RX_SAFEMODE		1


#ifdef FCNIM
#define DP83820_TX_Q_CNT	  	1
#define DP83820_TX_Q_LEN		4096

#define DP83820_RX_Q_CNT	  	1
#define DP83820_RX_Q_LEN		4096

#else

#define DP83820_TX_Q_CNT	  	1
#define DP83820_TX_Q_LEN		2048

#ifdef EEE_BUF_MGMT
#define DP83820_RX_Q_CNT	  	1
#define DP83820_RX_Q_LEN		2048
#else
#define DP83820_RX_Q_CNT	  	1
#define DP83820_RX_Q_LEN		2048
#endif

#endif



#ifdef FCNIM
#define DP83820_BUFFER_SIZE		EEE_BUF_ULTRA
#define DP83820_RX_BUF_SIZE		EEE_BUFSIZE_ULTRA
#else
#ifdef EEE_BUF_MGMT
#define DP83820_BUFFER_SIZE		EEE_BUF_MGMT	/* from special pool on TXRX */
#else
#define DP83820_BUFFER_SIZE		EEE_BUF_LARGE
#endif
#define DP83820_RX_BUF_SIZE		2048
#endif

#define DP83820_LB_ENABLE		1
#define DP83820_LB_DISABLE		2

#define DP83820_LINK_TIMER		3000		// 3 second link poll timer.

typedef unsigned long long dword;

/*
 *----------------------------
 * Descriptor Format
 *----------------------------
 */
#if defined (FCNIM) || defined (DIAG_R7000)	
typedef struct _dp83820_desc {
	struct _dp83820_desc *link;
	uchar8 				 *bufptr;
	uint32 				 cmdsts;
	eee_desc_t 			 *pktDesc;
} dp83820_desc_t;

#else // TXRX

typedef struct _dp83820_desc {
	struct _dp83820_desc *link;
	uchar8 				 *bufptr;
	uint32 				 cmdsts;
	uint32               dummy;
	eee_desc_t 			 *pktDesc;
} dp83820_desc_t;
#endif

/*
 *--------------------------------
 * cmdsts Common Bit definitions 
 *--------------------------------
 */
#define DP83820_CMDSTS_OWN	0x80000000
#define DP83820_CMDSTS_MORE	0x40000000
#define DP83820_CMDSTS_INTR	0x20000000
#define DP83820_CMDSTS_ECRC	0x10000000
#define DP83820_CMDSTS_OK	0x08000000
#define DP83820_CMDSTS_LEN	0x0000FFFF

/*
 *--------------------------------
 * cmdsts Transmit Bit definitions 
 *--------------------------------
 */
#define DP83820_CMDSTS_TXA	0x04000000
#define DP83820_CMDSTS_TFU	0x02000000
#define DP83820_CMDSTS_CRS	0x01000000
#define DP83820_CMDSTS_TD	0x00800000
#define DP83820_CMDSTS_ED	0x00400000
#define DP83820_CMDSTS_OWC	0x00200000
#define DP83820_CMDSTS_EC	0x00100000
#define DP83820_CMDSTS_CCNT	0x000F0000

/*
 *--------------------------------
 * cmdsts Receive Bit definitions 
 *--------------------------------
 */
#define DP83820_CMDSTS_RXA	0x04000000
#define DP83820_CMDSTS_RXO	0x02000000
#define DP83820_CMDSTS_DEST	0x01800000
#define DP83820_CMDSTS_LONG	0x00400000
#define DP83820_CMDSTS_RUNT	0x00200000
#define DP83820_CMDSTS_ISE	0x00100000
#define DP83820_CMDSTS_CRCE	0x00080000
#define DP83820_CMDSTS_FAE	0x00040000
#define DP83820_CMDSTS_LBP	0x00020000
#define DP83820_CMDSTS_IRL	0x00010000

#define DP83820_CMDSTS_RX_ERRBITS (DP83820_CMDSTS_RXA  | \
								   DP83820_CMDSTS_RXO  | \
								   DP83820_CMDSTS_LONG | \
								   DP83820_CMDSTS_RUNT | \
								   DP83820_CMDSTS_ISE  | \
								   DP83820_CMDSTS_CRCE | \
								   DP83820_CMDSTS_FAE  | \
								   DP83820_CMDSTS_IRL )
									

typedef struct _dp83820_stats {
	uint32 rxPkts;
	uint32 rxOctets;
	uint32 txPkts;
	uint32 txOctets;
	uint32 rxCrc;
	uint32 rxNoBuf;
	uint32 txQFull;
} dp83820_stats_t;

typedef struct _dp83820_txCtrl {
	dp83820_desc_t	*txqStart;
	dp83820_desc_t	*txqEnd;
	dp83820_desc_t	*txCur;
	dp83820_desc_t	*txCln;
	int32			avail;
	uint32			qd;
	uint32          unqd;
	uint32			pkts;
	uint32			octets;
	uint32			qFull;
	uint32			txaErr;
	uint32			tfuErr;
	uint32			crsErr;
	uint32			tdErr;
	uint32			edErr;
	uint32			owcErr;
	uint32			ecErr;
	uint32 txpkts[3][5];
} dp83820_txCtrl_t;

typedef struct _dp83820_rxBuf_cache {
	uchar8 *ptr;
	uint32 len;
} dp83820_bc_t;

typedef struct _dp83820_rxCtrl {
	dp83820_desc_t	*rxqStart;
	dp83820_desc_t	*rxqEnd;
	dp83820_desc_t	*rxCur;
	dp83820_bc_t	bc[EEE_MAX_BUFS];
	uint32			bufCnt;
	uint32          avail;
	uint32			pkts;
	uint32			octets;
	uint32			rxoErr;
	uint32			rxaErr;
	uint32 			noBufErr;
	uint32 			noDescErr;
	uint32			tooLongErr;
	uint32			runtErr;
	uint32			isErr;
	uint32			crcErr;
	uint32			faErr;
	uint32			lbpErr;
	uint32			irlErr;
	uint32 rxpkts[3][5];
} dp83820_rxCtrl_t;

#define ETHER_SPEED_UNKNOWN	0	
#define ETHER_SPEED_10HDX	1	/* 10MB hdx and fdx */
#define ETHER_SPEED_10FDX	2
#define ETHER_SPEED_100HDX	3	/* 100MB hdx and fdx */
#define ETHER_SPEED_100FDX	4
#define ETHER_SPEED_1000HDX	5	/* 1000MB hdx and fdx */
#define ETHER_SPEED_1000FDX	6

typedef enum { 
	speed_10, 
	speed_100, 
	speed_1000,
   	speed_unknown,	
} dp_speed_t;

typedef enum { 
	half_duplex,
	full_duplex,
	unknown_duplex,
} dp_duplex_t;

typedef enum { 
	fc_disabled, 
	fc_frame,
	fc_collision, 
	fc_carrier,
   	fc_unknown,	
} dp_fc_t;

typedef enum { 
	lb_none, 
	lb_int,
	lb_phy,
	lb_ext
} dp_loop_t;

typedef struct _dp83820_link {
	dp_fc_t     fc;
	uint32	 	timer;
	dp_speed_t  speed;
	dp_duplex_t duplex;
	uint32   	linkstat;
	dp_loop_t	loopback;
} dp83820_link_t;

typedef struct _dp83820_phy {

	uint32	addr;
	union {
		struct {
			uint16 bmsr;
			uint16 bmcr;
			uint16 k1stsr;
			uint16 anlpar;
		}bmc5411;
	}p;
} dp83820_phy_t;

// DP83820 Configuration Structure
typedef struct _dp83820_cfg {
	char			    macAddr[6];
	char			    macAddr_fp[6];
	uint32				appId;
	uint32				macId;
	uint32				baseAddr;
	uint32				phyAddr;
	uint32				ipAppPort;
	uint32				loopAddr;		/* System address of loopback app */
	dp83820_regs_t		*regs;
	dp83820_desc_t		*rxQ[DP83820_RX_Q_CNT];

	dp83820_txCtrl_t	txc[DP83820_TX_Q_CNT];
	dp83820_rxCtrl_t	rxc[DP83820_RX_Q_CNT];

	dp83820_stats_t		stats;
	dp83820_link_t		link;
	dp83820_phy_t		phy;
} dp83820_cfg_t;

//-- Special build of SP_NIM to allow it to act as
//-- a packet sniffer on the control and data planes.

#ifdef FCNIM
// #define DP83820_SNIFFER		1
#endif

#define DP83820_BYTES_SNIF_MAX		9000
#define DP83820_SNIF_ENTRIES_MAX	2048
typedef struct _dp83820_pkt_snifbuf {
	eeeRTC_t t;
	uint32   bufLen;
	uint32   capLen;
	uint32   cmdsts;
	uchar8   *buf;
} dp83820_psb_t;

extern dp83820_psb_t dp83820_rx_psb[DP83820_SNIF_ENTRIES_MAX];
extern uint32 dp83820_curRxSnifEntry;
extern uint32 dp83820_snifferOn;

//-- Pkt tracing built into the runtime 
#ifdef FCNIM
#define DP83820_TX_PKT_CAPTURE      1
#define DP83820_RX_PKT_CAPTURE      1
#endif

#define DP83820_TRACE_ENTRIES_MAX	1024	
#define DP83820_BYTES_TRACE_MAX     128

typedef struct _dp83820_pkt_tracebuf {
	eeeRTC_t t;
	uint32   pktLen;
	uint32   capLen;
	uint32   cmdsts;
	char    *orgBuf;
	uchar8   bytes[DP83820_BYTES_TRACE_MAX];
} dp83820_ptb_t;


typedef struct _dp83820_pkt_capture {
	uint32 txEna;
	uint32 maxTxBytesCap;
	uint32 txCapFilter;

	uint32 rxEna;
	uint32 maxRxBytesCap;
	uint32 rxCapFilter;
} dp83820_pc_t;

extern dp83820_ptb_t dp83820_tx_ptb[DP83820_TRACE_ENTRIES_MAX];
extern dp83820_ptb_t dp83820_rx_ptb[DP83820_TRACE_ENTRIES_MAX];
extern uint32 dp83820_curTxTraceEntry;
extern uint32 dp83820_curRxTraceEntry;
extern dp83820_pc_t dp83820_pc;

/*
 * Macros to convert virtual address to phy and back again.
 */
#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#ifndef FC_USE_NEW_MEM_MAP
#define V2PHYS_ADDR_MASK	0xA0000000
#define DP83820_2UNCHACHED(_x_)	((_x_) |= 0x20000000)
#endif
#else
#define V2PHYS_ADDR_MASK	0x00000000
#define DP83820_2UNCHACHED(_x_)	((_x_)) 
#endif	/* EVAL_BOARD || AGILE_HARDWARE */

/*
 * External Variable Declarations
 */
dp83820_cfg_t *dp83820;

/*
 * Function Prototypes
 */
void dp83820_rxPktPoll(void *, uint32);
void dp83820_txCleanPoll(void *, uint32);
void dp83820_rxAppPkt(eee_desc_t *);
int dp83820_writeReg(volatile uint32 *, uint32);


#endif /* DP83820_H_INCLUDED */
