/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	eee-dvt.c
 *
 * RCS:  $Id: eee-dvt.c,v 1.5 2002/11/07 01:20:03 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Test routines for the dvt test.
 *
 * Created by: lipika	
 *
 * Date Created:	6/04/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include "eee-test.h"
#if defined (NFP_TXRX) || defined (NFP_FP)
#include "../sm-bmc12500-eth/bmc12500-eth.h"
#endif

#define DVT_NFP_PORT_TXRX_MAX	4
#define BC1250_MAC_CNT		2
#define BC1250_MAX_PKT_SIZE	9800

#ifndef PROM
dvt_front_cfg dvtFrontCfg;
#endif
eee_test_hdr_t	* test_hdr;
uint32  incoming_appid;
#ifndef PROM
extern uint32  sentFromSSC[4][3]; /* [cpu][plane]  Now max cpu on NFP/NFC is 4 per card */
uchar8  static_hdr[DVT_NFP_PORT_TXRX_MAX][70];
#ifdef NFP_TXRX
uint32	front_seq[2][DVT_NFP_PORT_TXRX_MAX] = {{0,0},{0,0},{0,0},{0,0}}; /* cpu 0, 1 for TXRX */
uint32	next_exp_front_seq[2][DVT_NFP_PORT_TXRX_MAX] = {{0,0},{0,0},{0,0},{0,0}};
#endif
#endif
extern uint32 nfxMySlotId;
extern uint32 nfxMyCPUId;

#ifndef PROM
#ifdef NFP_TXRX
extern bc1250EthCfg_t *bcEthCfg[BC1250_MAC_CNT+1];
int dvt_bmc12500Eth_diagTxLuc(uint32 port, uint32 size, uint32 seq);
#endif
#endif

int32  pktSize=0;

#ifndef PROM
void eee_reinit_validity_table();
void eee_systemDVTSend(uint32 slot, uint32 cpu, uchar8 * bfr, uint32 size);
void dvt_rcvd_txrx_front_pkt(eee_descPtr_t edesc);
int dvt_bmc12500_spin_pkt(eee_descPtr_t edesc,uint32 port);
void eee_dvttestFrontEnd(uint32 plane);
void dvt_front_loopback(eee_test_hdr_t	*test_hdr);
eee_descPtr_t eee_testCreateDVTTestPkt(uint32 slot,uint32 cpu,uint32 plane,uint32 spin,uint32 seq);
void * eee_ramAlloc(uint32 num_bytes);
void dvtRemake_front_pkt(uint32 seq, uint32 link);

extern volatile uint32 dvtStop;
extern uint32	next_exp_seq_num[NFX_NUM_SLOTS_PER_CHASSIS][NFX_MAX_CPUS_PER_SLOT][3];
#ifdef FCNIM
int gt64240_pollTxQ(int bufCnt, int port);
#endif
int dp83820_pollTxQ(int bufCnt);

sysdvt_qlogic_t  *qdvt;

/*-----------------------------------------------------------------
 * Name:	eee_systemDVTinit()
 *
 * Description: Initialises some dvt related params.	
 *
 * Created by: lipika	
 *
 * Date Created:	9/26/02
 *
 *-----------------------------------------------------------------
 */
void
eee_systemDVTinit()
{
		qdvt = (sysdvt_qlogic_t  *)eee_ramAlloc(sizeof(sysdvt_qlogic_t));
		memset(qdvt,0,sizeof(sysdvt_qlogic_t));
		return;
}

/*-----------------------------------------------------------------
 * Name:	eee_systemDVTSend()
 *
 * Description:	Sends a msg to the SSC application
 *
 * Created by: lipika	
 *
 * Date Created:	5/26/02
 *
 *-----------------------------------------------------------------
 */
void
eee_systemDVTSend(uint32 slot, uint32 cpu, uchar8 * bfr, uint32 size)
{
	uint32                  dp;
	uint32                  sp;
	uint32 rc;
	uchar8 *tmp;
	uint32 j;
	
	dp = PACK_UNICAST_PORT_ADDRESS(slot,
						cpu,
						incoming_appid,
						EEE_MGMT,
						NFX_COS_HIGH,
						PORT_BEST_FIT);

	sp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
						nfxMyCPUId,
						eee.dvt_test_app_id,
						EEE_MGMT,
						NFX_COS_HIGH,
						PORT_BEST_FIT);

	
      tmp = bfr;
      for (j=0; j<64; j++)
         printf("%0x ", *tmp++);

	rc = eee_sendMessage(bfr, size, dp, sp);
	if (rc == EEE_FORWARDED_PACKET) {
		printf("systemDVTSend dp %08x sp %08x, msg %02x%02x%02x%02x\n",dp,sp,bfr[0],bfr[1],bfr[2],bfr[3]);
	}			
	else 
		printf("PKT NOT fwd!!\n");					
}

/*-----------------------------------------------------------------
 * Name:	eee_reinit_validity_table()
 *
 * Description:	Re-initialize receive functions for test application
 *
 * Created by: 	lipika	
 *
 * Date Created:	06/03/02	
 *
 *-----------------------------------------------------------------
 */
void
eee_reinit_validity_table()
{
	/* data */
	eee.test_dvt_valid[0][0][EEE_MGMT] = 0;
	eee.test_dvt_valid[0][1][EEE_MGMT] = 0;
	eee.test_dvt_valid[1][0][EEE_MGMT] = 1;
	eee.test_dvt_valid[1][1][EEE_MGMT] = 0;
	eee.test_dvt_valid[1][2][EEE_MGMT] = 0;
	eee.test_dvt_valid[1][3][EEE_MGMT] = 0;
	eee.test_dvt_valid[2][0][EEE_MGMT] = 1;
	eee.test_dvt_valid[2][1][EEE_MGMT] = 1;
	eee.test_dvt_valid[2][2][EEE_MGMT] = 1;
	eee.test_dvt_valid[2][3][EEE_MGMT] = 1;

	eee.test_dvt_valid[0][0][EEE_CONTROL] = 0;
	eee.test_dvt_valid[0][1][EEE_CONTROL] = 0;
	eee.test_dvt_valid[1][0][EEE_CONTROL] = 0;
	eee.test_dvt_valid[1][1][EEE_CONTROL] = 0;
	eee.test_dvt_valid[1][2][EEE_CONTROL] = 1;
	eee.test_dvt_valid[1][3][EEE_CONTROL] = 0;
	eee.test_dvt_valid[2][0][EEE_CONTROL] = 1;
	eee.test_dvt_valid[2][1][EEE_CONTROL] = 1;
	eee.test_dvt_valid[2][2][EEE_CONTROL] = 1;
	eee.test_dvt_valid[2][3][EEE_CONTROL] = 1;

	eee.test_dvt_valid[0][0][EEE_USER_DATA] = 0;
	eee.test_dvt_valid[0][1][EEE_USER_DATA] = 0;
	eee.test_dvt_valid[1][0][EEE_USER_DATA] = 0;
	eee.test_dvt_valid[1][1][EEE_USER_DATA] = 0;
	eee.test_dvt_valid[1][2][EEE_USER_DATA] = 1;
	eee.test_dvt_valid[1][3][EEE_USER_DATA] = 1;
	eee.test_dvt_valid[2][0][EEE_USER_DATA] = 1;
	eee.test_dvt_valid[2][1][EEE_USER_DATA] = 1;
	eee.test_dvt_valid[2][2][EEE_USER_DATA] = 1;
	eee.test_dvt_valid[2][3][EEE_USER_DATA] = 1;

}		

/*-----------------------------------------------------------------
 * Name:	dvtRemake_pkt(slot, cpu, seq, plane, spin)
 *
 * Description: Recreates a missing pkt with the missed seq no for 
 *				that slot, cpu, plane.	
 *
 * Created by: lipika	
 *
 * Date Created:	6/12/02
 *
 *-----------------------------------------------------------------
 */
void
dvtRemake_pkt(uint32 slot, uint32 cpu, uint32 seq, uint32 plane, uint32 spin)
{
	eee_descPtr_t	edesc;
	uint32			rc;

//printf("remake_pkt : slot %d cpu %d plane %d seq %d\n",slot,cpu,plane,seq);
	edesc = (eee_descPtr_t)eee_testCreateDVTTestPkt(slot, cpu, plane, spin, seq);
	if (edesc) {
		if ((rc = eee_forwardPacket(edesc)) != EEE_FORWARDED_PACKET)
			eee.test_dvt_send_err[plane]++;
		else
			eee.test_dvt_my_sent[slot][cpu][plane]++;
			
	}	
}

#ifdef NFP_TXRX
/*-----------------------------------------------------------------
 * Name:	dvt_rcvd_txrx_front_pkt(edesc)
 *
 * Description: Processes DATA pkts that are rcvd on the front ports
 * 				of the txrx.
 *
 * Created by: 	lipika	
 *
 * Date Created:	06/03/02	
 *
 *-----------------------------------------------------------------
 */
void
dvt_rcvd_txrx_front_pkt(eee_descPtr_t edesc)
{
	uint32	seq;
	uint32  src_slot, src_cpu, src_plane;
	eee_test_hdr_t	* test_hdr;
	eee_descBuf_t	* bd;
	uint32  port;
	uint32  rc;

    bd = &(edesc->bd[0]);
    if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
        bd++;
	test_hdr = (eee_test_hdr_t *)(bd->buf);

	seq = test_hdr->seq;
	/* We use the spin variable to track which port the pkt is destined for */
	port = test_hdr->spin;

	/* Sanity check: Port within range? */

	src_slot = EEE_GET_SLOT_NUM(edesc->hdr.src_port);
	src_cpu = EEE_GET_DEST_CPU(edesc->hdr.src_port);
	src_plane = EEE_GET_DTYPE(edesc->hdr.src_port);
	
	if (seq != next_exp_front_seq[nfxMyCPUId][port]) {
		printf("%s test error, expected seq %d, got %d, slot %d, cpu %d port %d\n",
				"DATA",
				next_exp_front_seq[nfxMyCPUId][port], seq,
				src_slot, src_cpu, port);
		dvtRemake_front_pkt(next_exp_front_seq[nfxMyCPUId][port], port);
		next_exp_front_seq[nfxMyCPUId][port] = (seq + 1) % EEE_TEST_NUM_PKT_SPEC;
		eee.test_dvt_rcv_err[src_plane]++; /* DATA */

		/* For now we don't END the test, but record how many out of
		 * sequence packets we've received
		 */ 
#if 0						
printf("got test data pkt:\n");
printf("edesc = 0x%08x  0x%08x  0x%08x  0x%lx\n", edesc->hdr.control, edesc->hdr.dest_port, edesc->hdr.src_port, edesc->hdr.attr);
printf("      = 0x%04x%04x  %p  0x%04x%04x  %p\n", edesc->bd[0].control, edesc->bd[0].len, edesc->bd[0].buf, edesc->bd[1].control, edesc->bd[1].len, edesc->bd[1].buf);
printf("test_pkt =  cmd       = 0x%08x\n", test_hdr->cmd);
printf("            seq       = 0x%08x\n", test_hdr->seq);
printf("            size      = 0x%08x\n", test_hdr->size);
printf("            orig_slot = 0x%08x\n", test_hdr->orig_slot);
printf("            orig_cpu  = 0x%08x\n", test_hdr->orig_cpu);
printf("            plane     = 0x%08x\n", test_hdr->plane);
eee_dvttestFrontEnd(src_plane);
#endif
	}
	else {
		next_exp_front_seq[nfxMyCPUId][port] = (next_exp_front_seq[nfxMyCPUId][port] + 1) % EEE_TEST_NUM_PKT_SPEC;
		rc = dvt_bmc12500_spin_pkt(edesc, port);
	}
	if (rc != EEE_FORWARDED_PACKET)
		eee.test_dvt_send_err[src_plane]++;
	else
		eee.test_dvt_my_sent[src_slot][src_cpu][src_plane]++;
}

/*-----------------------------------------------------------------
 * Name:	dvt_bmc12500_spin_pkt(edesc, port)
 *
 * Description: Formats the spinning pkt with appropriate hdrs 
 * 				for Cheetah.	
 *
 * Created by: 	lipika	
 *
 * Date Created:	06/03/02	
 *
 *-----------------------------------------------------------------
 */
int
dvt_bmc12500_spin_pkt(eee_descPtr_t edesc,uint32 port)
{
		uint32 pktLen;
		uint32 rc;
		uchar8 *newBufp;
		uchar8 *tmp;
		uchar8 *bptr;

 		bptr  = (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
		                 (edesc->bd[0].buf) : (edesc->bd[1].buf);
 
         newBufp = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
         if( !newBufp ) {
             printf("Test data buffer allocation failed.\n");
             return(NFX_OK);
         }

		memcpy(newBufp,&static_hdr[port],64);
		tmp = newBufp;
		tmp+=64;
		pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
		memcpy(tmp,bptr,36);

		eee_deallocateBuffer(bptr);
		(uchar8 *)edesc->bd[1].buf = newBufp; 

		edesc->bd[1].len+=64;
		pktLen+=64;
		edesc->hdr.control &= ~EDESC_TOTAL_LEN;
		edesc->hdr.control |= (pktLen & EDESC_TOTAL_LEN);
		edesc->hdr.dest_port = dvtFrontCfg.sp[port];
        rc = bmc12500Eth_txUnicast( edesc->hdr.dest_port,edesc );
		return rc;
}		

/*-----------------------------------------------------------------
 * Name:	eee_dvttestFrontEnd(plane)
 *
 * Description:	Stops traffic on the front plane of the txrx.
 *
 * Created by: 	lipika	
 *
 * Date Created:	06/03/02	
 *
 *-----------------------------------------------------------------
 */
void
eee_dvttestFrontEnd(uint32 plane)
{

	if (!eee.test_dvt_running[plane]) {
		printf("EEE test not running\n");
		return;
	}

	eee.test_dvt_running[plane] = 0;
	//memset(&dvtFrontCfg,0,sizeof(dvt_front_cfg));
	dvtFrontCfg.test_stopped = 1;

	if(sentFromSSC[nfxMyCPUId][plane])
		eee_testInit();
	sentFromSSC[nfxMyCPUId][plane] = 0;
}

/*-----------------------------------------------------------------
 * Name:	dvtRemake_front_pkt(uint32 seq, uint32 link)
 *
 * Description: Recreates a missing pkt with missed seq no for that link.	
 *
 * Created by: lipika	
 *
 * Date Created:	6/12/02
 *
 *-----------------------------------------------------------------
 */
void
dvtRemake_front_pkt(uint32 seq, uint32 link)
{
//printf("remake_front_pkt : link %d seq %d\n",link, seq);
	dvt_bmc12500Eth_diagTxLuc(link, pktSize, seq);
}


/*-----------------------------------------------------------------
 * Name:	dvt_front_loopback(plane)
 *
 * Description: Starts the front plane loopback test.	
 *
 * Created by: 	lipika	
 *
 * Date Created:	06/03/02	
 *
 *-----------------------------------------------------------------
 */
void
dvt_front_loopback(eee_test_hdr_t	*test_hdr)
{
		uint32 mac;
		uint32 port, sp;
		uint32 i, plane;

		/* do not allow test to proceed on second core */
		if((nfxMySlotId == NFX_SLOTTYPE_NFP) && (nfxMyCPUId == NFX_NFP_TXRX2))
			return;

		plane = test_hdr->plane;
		if(plane == SYS_DVT_FRONT)
				plane = EEE_USER_DATA;
		memset(&dvtFrontCfg,0,sizeof(dvt_front_cfg));

		for(port = 0; port < DVT_NFP_PORT_TXRX_MAX ; port++)
       	{
			if((port == 0) || (port == 1))
					mac = NFP_PORT_TXRX_ETH0;
			else if ((port == 2) || (port == 3)) {
				if(isCheetah())
					mac = NFP_PORT_TXRX_ETH2;
				else	
					mac = NFP_PORT_TXRX_ETH1;
			}	
			sp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
											eee.dvt_test_app_id,
											plane, NFX_COS_HIGH, mac);
			dvtFrontCfg.sp[port]	= sp;

			for(i=0;i<EEE_TEST_NUM_PKT_SPEC;i++) 
				dvt_bmc12500Eth_diagTxLuc(port, test_hdr->size, 0xFFFF);
			
			dvtFrontCfg.test_started = 1;
			next_exp_front_seq[nfxMyCPUId][port] = 0;
		}
		
	eee.test_dvt_running[plane] = 1;
}

/*-----------------------------------------------------------------
 * Name:	dvt_bmc12500Eth_diagTxLuc()
 *
 * Description: This is a rehash of the transmit fn by Tony Wong in the
 *				bmc12500Eth-mii-diag.c. We add some bogus hdrs before
 *				directly calling the txUnicast fn.	
 *
 * Created by: 	lipika	
 *
 * Date Created:	06/03/02	
 *
 *-----------------------------------------------------------------
 */
int
dvt_bmc12500Eth_diagTxLuc(uint32 port, uint32 size, uint32 seq)
{
	bc1250EthCfg_t *cfg;
	uchar8 *buf;
	uint32 mac=0;
    int32  i=0,j=0,k=0;
    int32  bufCnt = 0;
    int32  bufLen = 0;
    int32  lastBufLen = 0;
    uchar8 *bufptr;
    uchar8 *p_uncached;
    eee_desc_t *edesc;
    eee_descHdr_t *hdr;
    int32   out_port = 0;
	uchar8 junkbuf[BC1250_MAX_PKT_SIZE];
	uint32 appId = 0;
	uint32 rc = 0;

    uchar8 fill_byte = 0;
    luc_tx_hdr_t luc_hdr;
    uint16 udp_len = 0;
    uint16 ip_len = 0;
    uint32 junk_idx = 0;
    uint32 index = 0;
    uchar8 *bufp;
    int linkid = 0;
	extern uint32 luc_loopback_test;

	if((port == 0) || (port == 1))
		mac = 0;
	if((port == 2) || (port == 3)) {
		if(isCheetah())
			mac = 2;
		else
			mac = 1;
	}
    if ((mac == 2) && (!isCheetah())) {
        printf("Only MAC 0 and 1 available!\n");
        return 0;
    }
	
    if ((mac == 1) && (isCheetah())) {
        printf("Only MAC 0 and 2 available!\n");
        return 0;
    }
	cfg =  bcEthCfg[mac] ;
	cfg->macId = mac;
	luc_loopback_test = 0;
	

	if((size < 100) || (size > BC1250_MAX_PKT_SIZE))
		pktSize = 1024;	
	else 
		pktSize = size;
    fill_byte = 0;
    linkid = port; 

    for (i = 0; i < BC1250_MAX_PKT_SIZE; i++)
      junkbuf[i] = fill_byte++;

    bufp = &junkbuf;

    /* fill LUC header = 20 bytes */
    luc_hdr.field.seq_num = 0;
    luc_hdr.field.pkt_len = 0;
    luc_hdr.field.link_id = linkid;
    luc_hdr.field.control = 3; /* generate both checksums */
    luc_hdr.field.protocol = 0x11;
    luc_hdr.field.tcp_udp_len = pktSize - 20 - 14 - 20;
    luc_hdr.field.src_ip = 0x99999999;
    luc_hdr.field.dest_ip = 0x66666666;
    luc_hdr.field.l3_offset = 14;
    luc_hdr.field.l4_offset = 34;
    luc_hdr.field.tcp_udp_csum = 40;
    luc_hdr.field.tcp_udp_csum = 40;
    luc_hdr.field.ip_csum = 24;

    for (i = 19; i >= 0; i--)
        junkbuf[index++] = luc_hdr.data[i];

    /* fill MAC header = 14 bytes */
    for (i = 0; i < 6; i++)
        junkbuf[index++] = 0xff;
    for (i = 6; i < 12; i++)
        junkbuf[index++] = 0x00;
    junkbuf[index++] = 0x08;
    junkbuf[index++] = 0x00;

    ip_len = pktSize - 14 - 20;

    /* fill IP header = 20 bytes */
    junkbuf[index++] = 0x45;
    junkbuf[index++] = 0x00;
    junkbuf[index++] = (ip_len >> 8) & 0xff ; /* length */
    junkbuf[index++] = (ip_len & 0xff);
    junkbuf[index++] = 0x00; /* id */
    junkbuf[index++] = 0x00; /* id */
    junkbuf[index++] = 0x40; /* flag */
    junkbuf[index++] = 0x00;
    junkbuf[index++] = 0xff;
    junkbuf[index++] = 0x11;
    junkbuf[index++] = 0x00; /* checksum */
    junkbuf[index++] = 0x00; /* checksum */
    junkbuf[index++] = 0x99; /* src ip */
    junkbuf[index++] = 0x99;
    junkbuf[index++] = 0x99;
    junkbuf[index++] = 0x99;
    junkbuf[index++] = 0x66; /* dest ip */
    junkbuf[index++] = 0x66;
    junkbuf[index++] = 0x66;
    junkbuf[index++] = 0x66;

    /* fill UDP header = 8 bytes */
    junkbuf[index++] = 0x11; /* src port */
    junkbuf[index++] = 0x11;
    junkbuf[index++] = 0x22; /* dest port */
    junkbuf[index++] = 0x22;
    udp_len = pktSize - 14 - 20 -20 - 8;
    junkbuf[index++] = (udp_len >> 8) & 0xff;
    junkbuf[index++] = (udp_len & 0xff);
    junkbuf[index++] = 0x00; /* checksum */
    junkbuf[index++] = 0x00; /* checksum */

    junkbuf[index++];
    junkbuf[index++];

    memcpy(&static_hdr[port], bufp, 64);
    buf = &(junkbuf[index++]);
    test_hdr = (eee_test_hdr_t *)buf;
    test_hdr->cmd = EEE_MGMT_TEST_CMD_DATA;
	if(seq == 0xFFFF)
   		test_hdr->seq = front_seq[nfxMyCPUId][port];
	else
   		test_hdr->seq = seq;
    test_hdr->size = pktSize;
    test_hdr->orig_slot = nfxMySlotId;
    test_hdr->orig_cpu = nfxMyCPUId;
    test_hdr->plane = EEE_USER_DATA;
    test_hdr->spin = port;
	
    if( !(pktSize % 2048) ) {
        bufCnt = (pktSize / 2048);
        lastBufLen = 2048;
    }
    else {
        bufCnt = (pktSize / 2048) + 1;
        lastBufLen = pktSize % 2048;
    }

    edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
    memset(edesc,0, sizeof(eee_desc_t));
    hdr = (eee_descHdr_t *)edesc;
    hdr->control = (bufCnt << EDESC_BUFCNT_SHIFT) | pktSize;
    junk_idx = 0;

    for(k=1; k <= bufCnt; k++) {
        if(k == bufCnt)
            bufLen = lastBufLen;
        else
            bufLen = 2048;

        bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
        if( !bufptr ) {
            printf("Test data buffer allocation failed.\n");
            return(NFX_OK);
        }
        p_uncached = (uchar8 *)((address_t)bufptr | 0x00000000);
        for (j=0; j < bufLen; j++)
            p_uncached[j] = junkbuf[junk_idx++];
            edesc->hdr.src_port =  0xbbccddee;
#ifdef NFP_TXRX
		if (cfg->macId == 0)
			out_port = NFP_PORT_TXRX_ETH0;
		else
			out_port = NFP_PORT_TXRX_ETH2;
#elif defined(NFP_FP)
		if (cfg->macId == 0)
			out_port = NFP_PORT_FP_ETH0;
		else
			out_port = NFP_PORT_FP_ETH2;
#elif defined(PROM)
        if( isTXRX() ) {
            if( cfg->macId == 0 )
                   out_port = PROM_PORT_TXRX0;
            else 
                   out_port = PROM_PORT_TXRX2;
        }
        else {
            if( cfg->macId == 0 )
                    out_port = PROM_PORT_FP0;
            else
                    out_port = PROM_PORT_FP2;
        }
#endif
		appId = eee_getApplicationId("eee_test_complex");	
       	edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
					 nfxMySlotId,
					 nfxMyCPUId,
					 appId,
					 EEE_USER_DATA,
					 NFX_COS_HIGH,
					 out_port);
		edesc->hdr.src_port = edesc->hdr.dest_port;
        edesc->bd[k].buf = bufptr;
        edesc->bd[k].len = bufLen;
        edesc->bd[k].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
        }
	luc_loopback_test = 1;

	if((rc = bmc12500Eth_txUnicast(edesc->hdr.dest_port,edesc)) != EEE_FORWARDED_PACKET)
		eee.test_dvt_send_err[EEE_USER_DATA]++;
	else {
		eee.test_dvt_my_sent[nfxMySlotId][nfxMyCPUId][EEE_USER_DATA]++;
	}
	if(seq == 0xFFFF)	
    	front_seq[nfxMyCPUId][port] = (front_seq[nfxMyCPUId][port] + 1) % EEE_TEST_NUM_PKT_SPEC;
    return(NFX_OK);
}
#endif
#endif
