/*
 *-----------------------------------------------------------------
 *
 * Name:	eee-init.c
 *
 * RCS:  $Id: eee-init.c,v 1.35 2002/12/07 00:02:01 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	EEE Layer initialization functions
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/10/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>

eee_config_t	eee;

nfx_spinlock_t eee_descpool_spin;

eee_stats_t		* eee_stats;

#ifdef EEE_VALIDATE
eee_validate_t	* eee_desc_validate[2];
eee_validate_t	* eee_buf_validate[2][EEE_NUM_BUF_SIZES];
#endif

uchar8	* eee_sm_prom_upgrade;
extern uint32	dont_poll;

address_t	stack_addr CPU_PRIVATE;
#ifndef NFX_SMP
uchar8	stack[STKSIZE];
#endif

nfx_spinlock_t eee_bufpool_spin;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:  Rick Lund
 *
 * Date Created:    8/23/02
 *
 *-----------------------------------------------------------------
 */
void
eee_initSync(
	uint32  val
)
{
#if defined(NFP_FP) || defined(NFP_TXRX)
	volatile uint32 * shmem_ptr;

	watchdog_kick();

#ifdef NFP_FP
	shmem_ptr = (volatile uint32 *)0x900000e000001000ll;
	while (*shmem_ptr != val);
	*shmem_ptr = 0;
	*(shmem_ptr+4) = val+1;
#endif

#ifdef NFP_TXRX
	shmem_ptr = (volatile uint32 *)(int32)0x80001000;
	*shmem_ptr = val;
	while (*(shmem_ptr+4) != (val+1));
	*(shmem_ptr+4) = 0;
#endif

#endif
}

/*-----------------------------------------------------------------
 * Name:	eee_init(uint32 pass)
 *
 * Description:	EEE layer init function
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/10/01
 *
 *-----------------------------------------------------------------
 */

void
eee_init(
	uint32	pass
)
{
	if (pass == 1) {
		EEE_PRINTF(("[eee_init]:pass 1\n"));
		bzero(&eee, sizeof(eee_config_t));
		eee_stats = &(eee.stats);

		NFX_SPIN_INIT(&eee_fwdqueue_spin, SPINLOCK_LEVEL_UNKNOWN);
		NFX_SPIN_INIT(&eee_bufpool_spin, SPINLOCK_LEVEL_UNKNOWN);
		NFX_SPIN_INIT(&eee_descpool_spin, SPINLOCK_LEVEL_UNKNOWN);

		eee_shmemGetSharedMemoryPools();

		eee.my_pool_id = eee_platGetMyPoolId();
		eee.my_fwd_id = eee_platGetMyFwdId();

		eee.sm_size = eee_platGetSharedMemorySize();

		eee.buf_size[EEE_BUF_SMALL] = EEE_BUFSIZE_SMALL;
		eee.buf_size[EEE_BUF_LARGE] = EEE_BUFSIZE_LARGE;
		eee.buf_size[EEE_BUF_ULTRA] = EEE_BUFSIZE_ULTRA;
#ifdef EEE_BUF_MGMT
		eee.buf_size[EEE_BUF_MGMT] = EEE_BUFSIZE_MGMT;
#endif

		eee.buf_mask[EEE_BUF_SMALL] = ~(EEE_BUFSIZE_SMALL-1);
		eee.buf_mask[EEE_BUF_LARGE] = 0;		/* N/A, since not aligned */
		eee.buf_mask[EEE_BUF_ULTRA] = 0;		/* N/A, since not aligned */
#ifdef EEE_BUF_MGMT
		eee.buf_mask[EEE_BUF_MGMT] = 0;
#endif

#ifdef EEE_BUF_MGMT
		/*
		 * Allocate shared memory space for accesses from PCI to memory on Sibyte
		 * This should be first memory allocated, so that it's guaranteed to
		 * be aligned on nice MB boundary (needed for PCI BAR0 setup).
		 */
#ifdef NFP_TXRX
		eee.mgmt_size_buf = 0x700000;
		eee.mgmt_size_cont = 0x100000;
#else
		eee.mgmt_size_buf = 0;
		eee.mgmt_size_cont = 0;
#endif
		eee.mgmt_stack_size = (eee.mgmt_size_buf/eee.buf_size[EEE_BUF_MGMT]) *
															sizeof(address_t);
#else
#ifdef	PROM_R9000
		eee.mgmt_stack_size = 0x100000;
#else
		eee.mgmt_stack_size = 0;
#endif
#endif

		eee_memInit(1);

#ifdef EEE_BUF_MGMT
		eee.mgmt_space = (address_t)eee_allocateBlock(eee.mgmt_size_buf + eee.mgmt_size_cont);
		eee_memInit(2);
#endif

		eee.wka = eee_platGetWKA();

		/*
		 * Application Ids
 		 */
		eee.app_id = eee_allocateApplicationId("eee");
		eee.simple_test_app_id = eee_allocateApplicationId("eee_test_simple");
		eee.dvt_test_app_id = eee_allocateApplicationId("eee_test_complex");
		eee.anp_test_app_id = eee_allocateApplicationId("eee_anp_test");

		/*
		 * Drivers can now allocate their shared memory as needed
		 */

/*
		if (isTXRX())
			eee_sm_prom_upgrade = eee_allocateBlock(0x80000);
*/

	}
	else if (pass == 2) {
#if (EEE_NUM_SM_POOLS > 1)
		int32	rc;
#endif
		uint32	i;
		EEE_PRINTF(("[eee_init]:pass 2\n"));
#if (EEE_NUM_IPC_QUEUES > 0)
		eee_initIPCQueues();
#endif	/* EEE_NUM_IPC_QUEUES > 0 */

#if (EEE_NUM_SM_POOLS > 0)
#ifndef EEE_NEW_RATIO
		eee.bytes_per_small_buf = (EEE_BUFSIZE_SMALL +
							((EEE_BUFSIZE_LARGE * SMALL_TO_LARGE_RATIO)/8) +
							((EEE_BUFSIZE_ULTRA * SMALL_TO_ULTRA_RATIO)/8) +
							((sizeof(eee_desc_t) * DESC_TO_SMALL_RATIO)/8) +
							((sizeof(long)* FWD_TO_SMALL_RATIO)/8) +
							((sizeof(long) * (8+
												SMALL_TO_LARGE_RATIO+
												SMALL_TO_ULTRA_RATIO+
												DESC_TO_SMALL_RATIO))/8)
#ifdef EEE_ATTR
							+ ((sizeof(eee_attr_t) * (8 +
												SMALL_TO_LARGE_RATIO+
												SMALL_TO_ULTRA_RATIO+
												DESC_TO_SMALL_RATIO))/8)
#endif

							);
#endif	/* !EEE_NEW_RATIO */

#if (EEE_NUM_SM_POOLS > 1)
		eee_initSharedDescRQ(1);		/* init RQ */
		eee_initSharedBufsRQ(1);		/* init RQ */

		eee_initAllocateTailPtrs();
#endif	/* EEE_NUM_SM_POOLS > 1 */

		/*
		 * After this point, only descriptors and buffers can be
		 * allocated from shared memory.
		 */
/*
 *
 * HACK HACK HACK
 *
 *
 */
#ifdef EEE_NEW_RATIO
		eee.num_alloc_pages = (eee_remainBytes()-eee.mgmt_stack_size) /
																ALLOC_DIVISOR;
#else
		eee.sm_for_desc_and_buf = eee_remainBytes() - 0x110000;
#endif

		for (i=0; i<EEE_NUM_BUF_SIZES; i++) {
			eee.sm_num_buf[i] = eee_platGetNumSharedBuffers(i);
			eee_stats->low_watermark_buf[eee.my_pool_id - SHARED_POOL_OFFSET][i]
														= eee.sm_num_buf[i];
		}
		eee_platAdjustNumSharedBuffers();
	
		eee.sm_num_desc = eee_platGetNumSharedDesc();
		eee_stats->low_watermark_desc[eee.my_pool_id - SHARED_POOL_OFFSET] =
															eee.sm_num_desc;
		eee.sm_num_fwd = eee_platGetNumSharedFwd();

#ifdef EEE_VALIDATE
		eee_desc_validate[1] = eee_ramAllocZero(eee.sm_num_desc *
													sizeof(eee_validate_t));
		for (i=0; i<EEE_NUM_BUF_SIZES; i++) {
			if (eee.sm_num_buf[i])
				eee_buf_validate[1][i] = eee_ramAllocZero(eee.sm_num_buf[i] *
													sizeof(eee_validate_t));
		}
#endif

		eee_initSharedDescRQ(2);		/* init shared pool for self */
		eee_initSharedBufsRQ(2);		/* init shared pool for self */

#if 0
		eee_displayRQ();
#endif

#if (EEE_NUM_FWD_QUEUES > 0)
		eee_initFWDQueues();

/*
		eee_displayFWD();
*/
#endif	/* EEE_NUM_FWD_QUEUES > 0 */

#ifdef EEE_ATTR
		eee_attrInit();
#endif

		eee_initDeallocateSharedDesc(0);
		eee_initDeallocateSharedBuffers(0);

#if (EEE_NUM_SM_POOLS > 1)
		if ((rc = eee_initPutSharedMemAddrsToOtherCPUs()) != NFX_OK) {
			printf("Failed to store RQ addr in shared memory, rc = 0x%x\n", rc);
		}

		if ((rc = eee_initGetSharedMemAddrsFromOtherCPUs()) != NFX_OK) {
			printf("Failed to get Shared Memory addresses, rc = 0x%x\n", rc);
		}

		eee_initGetTailPointers();
		eee_initTailPointers();

		eee_initSync(0x12345678);

		bzero((void *)eee.wka, 0x100000);

#endif	/* EEE_NUM_SM_POOLS > 1 */

		eee_initDeallocateSharedDesc(1);
		eee_initDeallocateSharedBuffers(1);

#endif	/* EEE_NUM_SM_POOLS > 0 */

		EEE_PRINTF(("Shared Memory remain = %d bytes\n", eee_remainBytes()));

/*
		eee_initLocalDesc();
		eee_initLocalBufs();
*/
		dont_poll = 0;
		eee.init_done = 1;

#if 0
		eee_displayDescQueues(0);
		eee_displayDescQueues(1);
		eee_displayBufQueues(0);
		eee_displayBufQueues(1);
		eee_displayIPCQueues(0);
		eee_displayIPCQueues(1);
		eee_displayFWDQueues(0);
		eee_displayFWDQueues(1);
#endif

#if (EEE_NUM_SM_POOLS > 1)
		eee_rqInit();
#endif

		eee_fwdInit();

		eee_testInit();

#if (EEE_NUM_SM_POOLS > 1)
#if defined(NFP_TXRX) || defined(NFP_FP)
		eee_initSync(0xabcdef01);

		eee_rqResupply(0, 0);

		eee_initSync(0x01020344);
#endif
#endif

	}

}	/* eee_init() */

