/*
 * Name:  elog-ssc.c
 *
 * RCS:  $Id: elog-ssc.c,v 1.2 2002/03/27 00:50:15 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:  Eagle Logging
 *               Provides the function call to log message on the SCC
 *               Use sendAgileMsg to send messages to "elog"
 *
 * Created By:  Jan Vanhoutte
 *              Pankaj Gupta
 *
 * Date Created:  8/20/2001
 */

#if defined (SSC)


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"
#include "elog-def.h"

#define ELOG_STRING_MAX    1024

char *
e_log_var_pa (char *format, ...)
{
    int index = 0 ;
    char *the_msg ;
    va_list ap;

    the_msg =  (char *)malloc(ELOG_STRING_MAX);

    va_start(ap, format);
    if (format != NULL && format[0] != '\0')
      index += vsnprintf (the_msg, ELOG_STRING_MAX, format, ap);
    va_end(ap);

	return (the_msg);
}


int
e_log (log_class_t class,
       log_severity_t severity,
       uint32 app_id,
       uint32 app_type,
       uint32 cookie,
       uint32 code,
       int line,
       char* file,
       char *string)
{
    int rc;
    elog_eventNtfy_t *elog_msg ;
    uint32 msg_size = 0;
    uint32 str_size = 0;

    str_size = strlen(string) ;
    msg_size = sizeof (elog_eventNtfy_t) + str_size ;
    
    elog_msg =  (elog_eventNtfy_t *)malloc(msg_size);
    
    elog_msg->msgType = ELOG_MSG_NOTIFY;
    
    elog_msg->class = class;
    elog_msg->severity = severity;
    elog_msg->appId = app_id;
    elog_msg->appType = app_type;
    elog_msg->cookie = cookie;
    elog_msg->code = code;
    elog_msg->amsg[0] = 0 ;

    if (string)
        strcpy(elog_msg->amsg, string);

    rc = sendAgileMsg (-1, elog_msg, sizeof (elog_eventNtfy_t) + str_size,
                       "elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                       NULL, NULL, 0);
    
    free (elog_msg);
    
    return (rc);
}



#endif // SSC
