/*
 *-----------------------------------------------------------------
 *
 * Name:    esm-nfx-thread.S
 *
 * RCS:  $Id: esm-nfx-thread.S,v 1.5 2002/04/23 02:38:15 bobonfrogger Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:   nfx thread assembly routines
 *
 * Created by:  Bob Fozard
 *
 * Date Created:    3/20/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-target.h>

#if !defined(LINUX_TEST)

#include <nfx-incl.h>
#include <sys/mips.h>

.set noreorder
.set noat

/**
 * void
 * _esm_switchThread(uint64 *oldStackPtrPtr, uint64 newStackPtr)
 *
 * Resumes execution of the thread using newStackPtr
 * to restore context from.  Saves context from current
 * thread and returns the value of its stack pointer.
 */
LEAF(_esm_switchThread)
    addiu	sp,-80        # keep this value in sync with esm_gdbGetRegs

#ifdef __long64
    sd      sp,0(a0)      # store stack pointer in location pointed by $a0
#else
	sw      sp,0(a0)
#endif				

	sd      s0,0(sp)      # store saved registers for old context
    sd      s1,8(sp)
    sd      s2,16(sp)
    sd      s3,24(sp)
    sd      s4,32(sp)
    sd      s5,40(sp)
    sd      s6,48(sp)
    sd      s7,56(sp)
    sd      s8,64(sp)
    sd      ra,72(sp)

    move    sp,a1

    ld      s0,0(sp)         # load saved registers for new context
    ld	    s1,8(sp)
    ld	    s2,16(sp)
    ld	    s3,24(sp)
    ld	    s4,32(sp)
    ld	    s5,40(sp)
    ld	    s6,48(sp)
    ld	    s7,56(sp)
    ld	    s8,64(sp)
    ld	    ra,72(sp)

    j	    ra
    addiu	sp,80
END(_esm_switchThread)



/**
 * void
 * _esm_startThread(uint64 *oldStackPtrPtr, uint64 newStackPtr,
 *                  void *runAddr, void *arg)
 *
 * Starts a new thread by using newStackPtr as the base
 * for the new threads stack and jumping to runAddr with
 * arg as its parameter. Saves context from current
 * thread.
 */
LEAF(_esm_startThread)
    addiu	sp,-80

#ifdef __long64
    sd      sp,0(a0)      # store stack pointer in location pointed by $a0
#else
	sw		sp,0(a0)	
#endif				

    sd      s0,0(sp)      # store saved registers from old context
    sd      s1,8(sp)
    sd      s2,16(sp)
    sd      s3,24(sp)
    sd      s4,32(sp)
    sd      s5,40(sp)
    sd      s6,48(sp)
    sd      s7,56(sp)
    sd      s8,64(sp)
    sd      ra,72(sp)

    # call "runAddr(arg);"

     
    move    sp,a1         # new SP comes from second arg
    addiu	sp,-32        # make space on stack for arguments
    move    ra,a2         # new run address comes from third arg
    move    a0,a3         # new arg comes from fourth arg

    j       ra
    nop
END(_esm_startThread)

#endif // LINUX_TEST
