/*
 *-----------------------------------------------------------------
 *
 * Name:	nfx-init.S
 *
 * RCS:  $Id: nfx-init.S,v 1.3 2001/11/20 00:18:04 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	low level board initialization - PROM code
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/13/01
 *
 *-----------------------------------------------------------------
 */

#ifdef EVAL_BOARD

#include <nfx-target.h>
#include <sys/mips.h>
#include "../prom-pmon/pmon.h"
#include <exports/hwMap.h>
#include <sys/gt64240r.h>
#include <exports/promio-api.h>

	.text

	.globl	nfx_init

	.extern	nfx_init_done

	.set	noreorder

/*-----------------------------------------------------------------
 * Name:	nfx_init
 *
 * Description:
 *				minimal galileo init
 *				uart init
 *				nsdb
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/13/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nfx_init
nfx_init:

	GT_WRITE_REG_ASS(DEVICE_BANK0PARAMETERS, 0x144FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK1PARAMETERS, 0x146FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK2PARAMETERS, 0x144FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK3PARAMETERS, 0x167FFFFF)

	GT_WRITE_REG_ASS(PCI_0TIMEOUT_RETRY, 0x0000FFFF)
	GT_WRITE_REG_ASS(PCI_1TIMEOUT_RETRY, 0x0000FFFF)

	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)

	la		a0,0xBC808000		/* LED 0 */
	sb		zero,0(a0)

	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)

	lw		t0,DEVICE_BOOT_BANK_PARAMETERS(a0)
	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(4<<17)|0x14400000
	and		t0,(3<<20)
	or		t1,t0
	li		t2,(1<<20)
	beqz	t0,bootis8
	sw		t1,DEVICE_BOOT_BANK_PARAMETERS(a0)

	li		t2,(0<<20)
bootis8:
	lw		t0,DEVICE_BANK3PARAMETERS(a0)
	and		t0,0x3d400000
	or		t0,t2
	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(7<<17)
	or		t0,t1
	lw		t0,DEVICE_BANK3PARAMETERS(a0)


	la		a0,0xBC80C000		/* LED 1 */
	sb		zero,0(a0)

	jal		promio_init
	nop

	DEBUG_STR("nfx-init\r\n")

    /* Turn off PCI_0 I/O space */
	GT_WRITE_REG_ASS(PCI_0I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 I/O space */
	GT_WRITE_REG_ASS(PCI_1I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 1 */
	GT_WRITE_REG_ASS(PCI_0MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 2 */
	GT_WRITE_REG_ASS(PCI_0MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 3 */
	GT_WRITE_REG_ASS(PCI_0MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 1 */
	GT_WRITE_REG_ASS(PCI_1MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 2 */
	GT_WRITE_REG_ASS(PCI_1MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 3 */
	GT_WRITE_REG_ASS(PCI_1MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn on PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000120)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0x0000013f)

    /* Turn on PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000160)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0x0000017f)

	/* Turn off PCI CS access */
	GT_WRITE_REG_ASS(PCI_0CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_BOOT_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_BOOT_BANK_SIZE, 0)

	/* Enable PCI access to SCS0 and SCS1 */
	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFC)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFC)
	
	/* Set up slave window sizes */
	GT_WRITE_REG_ASS(PCI_0SCS_0_BANK_SIZE, 0x01FFF000);
	GT_WRITE_REG_ASS(PCI_1SCS_0_BANK_SIZE, 0x01FFF000);
	
	/* Set up PCI_0 CFG space */
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	/* Set up PCI_1 CFG command register */
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	j		nfx_init_done
	nop

	.end	nfx_init


#endif	/* EVAL_BOARD */
