/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	event-api.h
 *
 * RCS:  $Id: event-api.h,v 1.21 2002/09/18 23:41:28 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Exported API for the Event Mgr module
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	11/05/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _EVENT_API_H
#define _EVENT_API_H

#define EVENT_SERVICE_NAME          "eventd"

#define MAX_CLIENT_PER_EVENT		16
#define MAX_EVENT_BUF_SIZE			32768

#define	EVENT_MAX_NODE_NAME			256
#define	EVENT_MAX_APP_NAME			256
#define	EVENT_MAX_MAIL_SUB			100


#define EVENT_DBG_PRINT(_par_) E_LOG(class_1, LOG_DEBUG, myappid,0,0,0,_par_)
#define EVENT_ERR_PRINT(_par_) E_LOG(class_1, LOG_ERR,   myappid,0,0,0,_par_)
#define EVENT_FCT_PRINT(_par_) E_LOG(class_1, LOG_INFO,  myappid,0,0,0,_par_)

#include "exports/evm-api.h"	// For evm_volId_t

/*
 * Event Mgr messages
 */
typedef enum {
	NFX_EVENT_REGISTER = EVENT_MSG_BASE,
	NFX_EVENT_UNREGISTER,
	NFX_EVENT_POST,
	NFX_EVENT_FORWARD,
	NFX_EVENT_STATUS,
	NFX_EVENT_AS_CONFIG_SET_REQ,
	NFX_EVENT_AS_CONFIG_VIEW_REQ,
	NFX_EVENT_AS_STATS_VIEW_REQ,
	NFX_EVENT_AS_STATS_CLEAR_REQ,
	NFX_EVENT_AS_GENERATE_REPORT
} nfx_eventMsgType_t;

typedef enum {
		NFX_EVENT_NODE,		/* nodeEvent   */
		NFX_EVENT_PCC,		/* nodeEvent   */
		NFX_EVENT_SCC,		/* nodeEvent   */
		NFX_EVENT_CARD,		/* cardEvent   */
		NFX_EVENT_CPU,		/* cpuEvent    */
		NFX_EVENT_VOLUME,	/* volEvent    */
		NFX_EVENT_IP_IF,	/* ipIfEvent   */
		NFX_EVENT_PORT,		/* portEvent   */
		NFX_EVENT_NFS,		/* exportEvent */
		NFX_EVENT_ROUTE,	/* ipRteEvent  */
        NFX_EVENT_QUOTA,    /* quotaEvent  */
        NFX_EVENT_MKFS
} nfx_eventType_t;

#define		NFX_EVENT_MIN		NFX_EVENT_NODE
#define		NFX_EVENT_MAX		NFX_EVENT_MKFS
#define		NUM_NFX_EVENTS		(NFX_EVENT_MAX + 1)

/*
 * Event client entry
 */
typedef struct {
	int		appId;
	char 	appName[EVENT_MAX_APP_NAME];
	int		slot;
	int		cpu;
	int 	flag;
} nfx_event_client_t;

/*------------------------------------------------------------------------------
 * Event statistics block
 *------------------------------------------------------------------------------
 */

typedef struct {
	int		eventRcvd[NUM_NFX_EVENTS]; 		// number of each event rcvd
	uint32	lastEventTime[NUM_NFX_EVENTS]; 	// time last event of 
} event_stats_t;


/*------------------------------------------------------------------------------
 * The CONFIG NTFY message.
 *------------------------------------------------------------------------------
 */

typedef struct {
    nfx_eventMsgType_t  	msgType;
	int     				status;
    struct
    { 
        char        option[OPT_MAX_OPTION_LEN];
        char        value[MAX_VALUE_LEN];
    } entry;
} event_configMsg_t;

/*------------------------------------------------------------------------------
 * EVENT message.
 *------------------------------------------------------------------------------
 */

/*
 * Msg Structure for Reporting events 
 * to/from Event Manager
 */
typedef struct {

	nfx_eventMsgType_t	msgType;

	char 	nodeName[EVENT_MAX_NODE_NAME];

	int		needAck;		/* if set the event is acknowledged with 
	                           the status appropriately set */
	int 	status;

#define NFX_EVENT_STATUS_OK			1
#define NFX_EVENT_STATUS_ERR		-1

	int 	eventScope;		

#define 	NFX_EVENT_LOCAL		1
#define 	NFX_EVENT_GLOBAL	2

	nfx_eventType_t		eventType;
/*
 * Event state definitions
 */
#define		EVENT_STATE_NONE        0
#define		EVENT_STATE_UP			1
#define		EVENT_STATE_DOWN		2
#define		EVENT_STATE_OFFLINE		3
#define		EVENT_STATE_FAILED		4
#define		EVENT_STATE_ACCESSIBLE	5
#define		EVENT_STATE_REMOVE		6
#define		EVENT_STATE_NOCHANGE    7 // the state after the event is
                                      // the same as before the event

	char	reason[EVENT_MAX_MAIL_SUB];	// Event mail subject line

	union {

	struct nodeEvent {
		/*
		 * Node/PCC/SCC State Event
		 */
		int		state; 				/* UP, DOWN */
		uint32	mgmt_ip[2];			/* Management 10/100 port IP addresses */
	} nodeEvent;

	struct cardEvent {
		/*
		 * Card State Event
		 */
		int		slot;
		int		cardType;
		int		state;				/* UP, DOWN */
	} cardEvent;

	struct cpuEvent {
		/*
		 * CPU State Event
		 */
		int		slot;
		int		cpu;
		int		state;				/* UP, DOWN */
	} cpuEvent;

    struct volEvent {
// generated by : EVM
// when         : - volume is created
//                - volume is discovered
// volume state : DOWN
#define EVENT_VOL_CREATE        1
// generated by : EVM
// when         : volume is destroyed
// volume state : N/A (NONE)
#define EVENT_VOL_DELETE        2
// generated by : EVM
// when         : - volume is created (1st growth)
//                - autogrow (on filesystem request)
// volume state : DOWN or UP (NOCHANGE)
#define EVENT_VOL_GROW          3
// generated by : EVM
// when         : volume softquota is reached
//                - volume is created (1st growth)
//                - autogrow (on filesystem request)
// volume state : DOWN or UP (NOCHANGE)
#define EVENT_VOL_SOFTQUOTA     4
// generated by : EVM
// when         : volume has reached its hardquota
//                - volume is created (1st growth)
//                - autogrow (on filesystem request)
// volume state : DOWN or UP    (NOCHANGE)     
#define EVENT_VOL_FULL          5
// generated by : EVM
// when         : one or more volume attributes has been modified (eg. name)
// volume state : DOWN or UP (NOCHANGE)    
#define EVENT_VOL_MODIFY        6
// generated by : EVM
// when         : one or more LUNs of the volume has "disappeared".
// volume state : FAILED
#define EVENT_VOL_BROKEN        7
// generated by : EVM server
// when         : IO errors occured on the volume
// volume state : FAILED
#define EVENT_VOL_IOERROR       8
// generated by : EVM server
// when         : a conflict in the volume management has been detected
// volume state : FAILED
#define EVENT_VOL_CONFLICT      9
// generated by : EVM server
// when         : another is taking over the volume
// volume state : DOWN
#define EVENT_VOL_TAKEOVER     10
// generated by : EVM server
// when         : corruption on the LUN private areas have been detected
// volume state : FAILED
#define EVENT_VOL_CORRUPT      11       
// generated by : FS
// when         : filesystem is successfully mounted => volume online
// volume state : UP
#define EVENT_VOL_ONLINE       12
// generated by : FS
// when         : filesystem is not mounted => volume offline
// volume state : DOWN
#define EVENT_VOL_OFFLINE      13
// generated by : FS
// when         : something wrong happened on the filesystem
// volume state : DOWN
#define EVENT_VOL_FSYSABORT    14
        uint32      event;          // event that just occured
		evm_volId_t volId;          // volume concerned by the event
        char        volName[VOL_MAX_NAME];  // volume name
		int			state;			// current volume state:
                                    // UP (online), DOWN (offline),
                                    // FAILED (something went wrong)
	} volEvent;

	struct exportEvent {
		/*
		 * Export State Event
		 */
		evm_volId_t	volId;
		uint32		expAddr;		/* Exporting IP address */
		char		expPath[256];	/* Export path */
		char		expOptions[256];/* Export options/rules */
		int			state;			/* UP (add), REMOVE */
	} exportEvent;

	struct ipIfEvent {
		/*
		 * IP i/f State Event
		 */
		uint32	ipa;				/* Interface IP address */
		uint32	mask;				/* Network mask */
		char	portName[256];		/* Fully constructed port name */
		int		state;				/* Up (add), Remove (delete) */
	} ipIfEvent;

	struct ipRteEvent {
		uint32	destNet;			/* to this network */
		uint32	destMask;			/* (default is 0) */
		uint32	gateWay;			/* via this gateway (!delete) */
		int		state;				/* Up (add), Remove (delete) */
	} ipRteEvent;

	struct portEvent {
		/*
		 * Physical port State Event
		 */
		int		portAddr;
		int		state; 				/* UP, DOWN */
	} portEvent;

    struct quotaEvent {
        char        volName[VOL_MAX_NAME];  // volume name
		evm_volId_t volId;                  // volume concerned by the event
        uint32      fileSetId;              // fileset id
        uint32      quotaType;              // FS_QUOTA_USER / FS_QUOTA_GROUP /
                                            // FS_QUOTA_AGGREGATE
        uint32      id;                     // user/group id
        int         notify;
    } quotaEvent;
    
    struct mkfsEvent {
        evm_volId_t volId;          /* Volume that has a new mkfs done */
    } mkfsEvent;

	} param;

} nfx_eventMsg_t;

#ifndef SSC
void event_registerClient(int32 eventId, void (*rcv_func)(nfx_eventMsg_t  * mptr));
void event_postEvent(nfx_eventMsg_t * mptr);

typedef struct event_st {
	uint32	posted;
	uint32	rcvd;
} event_mgr_stats_t;

extern event_mgr_stats_t	event_stats[NFX_EVENT_MAX+1];
extern uint32		event_client_stats[NFX_EVENT_MAX+1][MAX_CLIENT_PER_EVENT];

extern int32	event_mgr_debug;

#define EVENT_MGR_DEBUG(x)	if (event_mgr_debug) printf x
#endif

#endif /* _EVENT_API_H */

