/*
 *-----------------------------------------------------------------
 *
 * Name:	event-config.c
 *
 * RCS:  $Id: event-config.c,v 1.7 2002/09/18 23:41:28 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Event config
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	11/05/01
 *
 *-----------------------------------------------------------------
 */

#ifdef SSC

#include "nfx-incl.h"

#include <stdio.h>      // fprintf(), stderr
#include <stdlib.h>     // exit()
#include <string.h>     // memset()
#include <unistd.h>     // close()

#include <fcntl.h>
#include <netinet/in.h>

#include <sys/time.h>
#include <sys/socket.h>
#include <signal.h>
#include <errno.h>

#include "exports/anpssc-api.h"
#include "exports/event-api.h"
#include "exports/ncm-api.h"
#include "exports/opt-api.h"
#include "exports/elog-api.h"
#include "event-as.h"

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

struct timeval waitRegmTo = {10, 0};
extern as_config_t   asCfg;
extern int myappid;

/*------------------------------------------------------------------------------
 * Prototypes of functions.
 *------------------------------------------------------------------------------
 */
int event_setConfig(char * option, char * value);
int event_getConfig(char * option);
int event_updateConfig(opt_entry_t * entry);
void as_addCrontabEntry(void);
void as_deleteCrontabEntry(void);

/*------------------------------------------------------------------------------
 * Name         : event_setConfig()
 *
 * Description  : Set the option in the registry.
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 13-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

int
event_setConfig(char * option,
              char * value)
{
    char                 req[sizeof(opt_msgHdr_t) + sizeof(opt_setOptionReq_t)];
    opt_msgHdr_t *       reqHdr;
    opt_setOptionReq_t * reqBody;
    char                 rsp[sizeof(opt_msgHdr_t) + sizeof(opt_setOptionRsp_t)];
    opt_msgHdr_t *       rspHdr;
    opt_setOptionRsp_t * rspBody;
    
    EVENT_DBG_PRINT(("> event_setConfig()\n"));

    /*
     */
    bzero((void *) &req, sizeof(req));
    reqHdr = (opt_msgHdr_t *) &req;
    reqBody = (opt_setOptionReq_t *) (req + sizeof(opt_msgHdr_t));
    bzero((void *) &rsp, sizeof(rsp));
    rspHdr = (opt_msgHdr_t *) &rsp;
    rspBody = (opt_setOptionRsp_t *) (rsp + sizeof(opt_msgHdr_t));

    /* Fill the request.
     */
    reqHdr->msgType = OPT_SET_OPTION_REQ;
    reqHdr->userInfo = 0; // not used    
    reqBody->del = FALSE;
    strcpy(reqBody->entry.option, option);
    strcpy(reqBody->entry.value, value);
    
    /* Send the request to the registry manager located on the SSC.
     * Receive response from the registry manager.
     */
    if (sendAgileMsg(-1,
                     req, sizeof(req),
                     "regm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                     &waitRegmTo,
                     rsp, sizeof(rsp)) == -1)
    {
		perror("sendAgileMsg() failed");
		return(NFX_ERR);
    }
    
    /* Read the message header and check if it's expected response.
     */
    rspHdr = (opt_msgHdr_t *) rsp;
    if (rspHdr->msgType != OPT_SET_OPTION_RSP)
    {
        EVENT_ERR_PRINT(("Message received from REGM is not the expected one!\n"));
		return(NFX_ERR);
    }
    
    /* Read result.
     */
    rspBody = (opt_setOptionRsp_t *) (rsp + sizeof(opt_msgHdr_t));
    EVENT_DBG_PRINT(("result returned = %d\n", rspBody->result));
    
    EVENT_DBG_PRINT(("< event_setConfig()\n"));
    return(rspBody->result);
}

/*------------------------------------------------------------------------------
 * Name         : event_getConfig()
 *
 * Description  : Get configuration from registry manager.
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 11-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

int
event_getConfig(char * option)
{
    opt_msgType_t   reqType = OPT_GET_OPTION_REQ;
    elog_msg_t       req;
    elog_msg_t       rsp;
    opt_msgHdr_t *  hdr;
    void *          bufp = NULL;
    int             nbEntry = 0;
    int             i;

    EVENT_DBG_PRINT(("> event_getConfig\n"));

    /* Fill -a %sthe request.
     */
    bufp = &req.buf;
    bzero(bufp,
          (sizeof(opt_msgHdr_t) +
           sizeof(opt_getOptionReq_t)));
    
    memcpy(bufp, (void *) &reqType, sizeof(opt_msgType_t));
    bufp += sizeof(opt_msgType_t);
    // userInfo is not used
    bufp += sizeof(opt_userInfo_t);
    memcpy(bufp, (void *) option, sizeof(opt_option_t));
    bufp += sizeof(opt_option_t);

    req.len = bufp - (void *) req.buf;

    /* Send the request to the registry manager located on the SSC.
     */
    rsp.len = sendAgileMsg(-1,
                           req.buf, req.len,
                           "regm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                           &waitRegmTo,
                           rsp.buf, sizeof(rsp.buf));
    
    /* Receive response from the registry manager.
     */
    EVENT_DBG_PRINT(("nb bytes received = %d\n", rsp.len));
    if (rsp.len == -1)
    {
		perror("sendAgileMsg failed");
		return(NFX_ERR);
    }

    /* Read the message header and check if it's expected response.
     */
    bufp = &rsp.buf;
    hdr = (opt_msgHdr_t *) rsp.buf;
    bufp += sizeof(opt_msgHdr_t);
    if (hdr->msgType != OPT_GET_OPTION_RSP)
    {
        EVENT_ERR_PRINT(("Message received from REGM is not the expected one!\n"));
		return(NFX_ERR);
    }
    
    /* Read the response and print its content.
     */
    memcpy((void *) &nbEntry, bufp, sizeof(int));
    bufp += sizeof(int);

    for (i = 0; i < nbEntry; i++)
    {
        opt_entry_t * entry;

        entry = (opt_entry_t *) bufp;
        bufp += sizeof(opt_entry_t);

        event_updateConfig(entry);
    } // for
    
    EVENT_DBG_PRINT(("< event_getConfig()\n"));
    return(NFX_OK);
}

/*------------------------------------------------------------------------------
 * Name         : event_updateConfig()
 *
 * Description  :
 * Update local configuration with info received from the registry manager.
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 11-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

int
event_updateConfig(opt_entry_t * entry)
    
{
    EVENT_DBG_PRINT(("> event_updateConfig\n"));

	if (!strcmp(entry->option, OPTIONS_AUTOSUPPORT_ENABLE))
	{
		asCfg.enable = (!strcmp(entry->value, "on")? TRUE : FALSE);
	}
	else if (!strcmp(entry->option, OPTIONS_AUTOSUPPORT_TO))
	{
		strcpy(asCfg.to, entry->value);
	}
	else if (!strcmp(entry->option, OPTIONS_AUTOSUPPORT_NOTETO))
	{
		strcpy(asCfg.noteto, entry->value);
	}
	else if (!strcmp(entry->option, OPTIONS_AUTOSUPPORT_SCHEDULE))
	{
		sscanf(entry->value, "0x%llx:0x%x:0x%x:0x%x:0x%x",
				&asCfg.minBitMap, &asCfg.hourBitMap, &asCfg.dateBitMap, 
				&asCfg.monthBitMap, &asCfg.dayBitMap);

		EVENT_DBG_PRINT((
			"event_updateConfig: 0x%llx, 0x%x, 0x%x, 0x%x, 0x%x from %s\n",
				asCfg.minBitMap, asCfg.hourBitMap, asCfg.dateBitMap,
				asCfg.monthBitMap, asCfg.dayBitMap, entry->value));

		as_deleteCrontabEntry();
		as_addCrontabEntry();
	}
	else
	{
		printf("event_updateConfig has unrecognized option\n");
	}

    EVENT_DBG_PRINT(("< event_updateConfig\n"));
	return(NFX_OK);
}

#endif	/* SSC */

