/*------------------------------------------------------------------------------
 * RCS:  $Id: evm-srvr-structs.h,v 1.27 2002/10/22 01:30:32 jong Exp $
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Name         : evm-srvr-structs.h
 *
 * Description  : private data structures used by this module
 *
 * Created by   : Greg Seibert
 *                Lan Chin
 *                Isabelle Cnudde
 *
 *------------------------------------------------------------------------------
 */

#ifndef _EVM_SRVR_STRUCTS_H_
#define _EVM_SRVR_STRUCTS_H_

#include <exports/evm-api.h>

/* EVM tunables.
 */
typedef struct
{
    eeeRTC_t	ioTimeout;  // Time after which to timeout an IO request
    uint32      maxIoRetry; // max IO retries
    uint32      maxIo;      // max IOs to be sent in a loop
} evm_param_t;

/*
 */
typedef union
{
    uint32              whole;
    struct {
        unsigned        idx:16;            // index in the queue
        unsigned        genNb:15;          // generation number
        unsigned        dataType:1;        // meta or user data
    } parts;
} evm_qhandle_t;

typedef enum {
    m_read = 0,
    u_read = 1,
    m_write = 2,
    u_write = 3,
} evm_pReq_t;

/* The list of served Logical Volumes
 */
typedef struct
{
    boolean     isError;
    uint64      nStaleRsp;
    uint64      nOffline;
    uint64      nQueueFull;
} evm_volErr_t;

/* a evm_codLogCb_t struct is created to log COD arrivals
 */
typedef struct
{
    nfx_cod_t       cod;        // COD handle
    uint32          len;        // COD length
    evm_qhandle_t   pioHandle;  // to find the corresponding evm_pioCb_t
    uint64          timestamp;  // used to timeout a COD in the hash table
} evm_codLogCb_t;

/* LUN.
 */
typedef struct evm_lun_T evm_lun_t;
struct evm_lun_T
{
    uint32                  idx;                // position in the volume
    scsi_id_t               id;                 // SCSI id
    int32                   openHandle;         // SCSI open handle
    uint32                  physBlkOffset;      // starting where on it
    uint64                  volOffsetStart;     // logical vol offset start
    uint64                  volOffsetEnd;       // logical vol offset end
    uint64                  nBlks;              // num of blocks (end-start)
    uint32                  nTotalBlks;         // total nb blocks of the LUN
    DLL_ENTRY(evm_lun_t)    entry;
};

/* Logical volume.
 */
typedef struct evm_vol_T evm_vol_t;
struct evm_vol_T
{
    evm_volId_t             id;         // volume id
    uint64                  size;       // volume/filesystem size 
    uint64                  totalSize;  // total volume size (incl. labels...)
    uint64                  pwr2AvgLunSize;
    uint64                  pwr2AvgTotalLunSize;
    uint64                  nbLun;      // use to calculate the avgLunSize
    DLL_HDR(evm_lun_t)      lunList;    // list of LUNs
    address_t               lunHash;    // LUNs' table indexed by their position
    int32                   lastTakeoverTstamp;
    evm_nfxId_t             nfxId;      // NFX id
    uint64                  nRetries;   // number of IOs retried
    DLL_ENTRY(evm_vol_t)    entry;
};

#ifdef DEBUG
typedef enum
{
    EVM_DROPDATA_NONE   = 0,
    EVM_DROPDATA_USER   = 1,
    EVM_DROPDATA_META   = 2,
    EVM_DROPDATA_BOTH   = 3
} evm_dropData_t;

typedef enum
{
    EVM_DROPIO_NONE     = 0,
    EVM_DROPIO_READ     = 1,
    EVM_DROPIO_WRITE    = 2,
    EVM_DROPIO_BOTH     = 3
} evm_dropIo_t;
#endif // DEBUG
/*----------------------------------------------------------------------------*/

#endif // _EVM_SRVR_STRUCTS_H_
