/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: crashdump.h,v 1.11 2002/10/16 03:03:14 johnh Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _CRASHDUMP_H
#define _CRASHDUMP_H

#include "except-api.h"
#include "except.h"
#include "exports/from-api.h"
#include "exports/prom-part-api.h"

/* XXX CRASHDUMP_MAX_SIZE must be >= crashdump_t */
#define CRASHDUMP_MAX_SIZE      (16 * 1024 - PROM_PART_CB_SIZE)

#define CRASHDUMP_STACK_LEN		2048
#define CRASHDUMP_STRING_LEN	80
#define CRASHDUMP_TIME_LEN		80

#define CRASHDUMP_SIGNATURE		0xdeaddead
#define CRASHDUMP_FORMAT		1

typedef struct cdump {
	uint32		signature;
	uint32		format;
	uint32		flags;
	uchar8		prom_version[CRASHDUMP_STRING_LEN];
	uchar8		rt_version[CRASHDUMP_STRING_LEN];
	uchar8		start_time[CRASHDUMP_TIME_LEN];
	uchar8		crash_time[CRASHDUMP_TIME_LEN];
	uchar8		crash_string[CRASHDUMP_STRING_LEN];
	cpureg_t	regs[NREGS];
	address_t	sp;
	address_t	stacktop;
	uint32		stacksize;
	uint32		stack[CRASHDUMP_STACK_LEN];
	uint32		checksum;
} crashdump_t;

#define CRASHDUMP_FLAGS_PROM_VER		0x00000001
#define CRASHDUMP_FLAGS_RT_VER			0x00000002
#define CRASHDUMP_FLAGS_START_TIME		0x00000004
#define CRASHDUMP_FLAGS_CRASH_TIME		0x00000008
#define CRASHDUMP_FLAGS_EXCEPT_STRING	0x00000010
#define CRASHDUMP_FLAGS_PANIC_STRING	0x00000020
#define CRASHDUMP_FLAGS_NMI_STRING		0x00000040
#define CRASHDUMP_FLAGS_SOURCE_PROM		0x00000080
#define CRASHDUMP_FLAGS_SOURCE_RT		0x00000100
#define CRASHDUMP_FLAGS_REGS			0x00000200
#define CRASHDUMP_FLAGS_STACK			0x00000400
#define CRASHDUMP_FLAGS_CAUSE_MASK		0xFF000000
#define CRASHDUMP_FLAGS_CAUSE_UNKNOWN	0x01000000
#define CRASHDUMP_FLAGS_CAUSE_EXCEPT	0x02000000
#define CRASHDUMP_FLAGS_CAUSE_PANIC		0x04000000
#define CRASHDUMP_FLAGS_CAUSE_NMI		0x08000000

extern crashdump_t	saved_cd;
extern crashdump_t	* new_cd;

uint32  crashdump_checksum(crashdump_t * cptr);
void    crashdump_init(void);
int32   crashdump_get(crashdump_t * cptr);
int32   crashdump_valid(crashdump_t	* cptr);
void    crashdump_save(crashdump_t * cptr);
void    crashdump_save_regs(crashdump_t	* cptr);
void    crashdump_save_stack(crashdump_t * cptr);
int32   crashdump_getTimeStamp( uchar8	* time_str);
int32   crashdump_more(void);
uint32  crashdump_displayRegs(crashdump_t * cptr, uint8 * buf_p);
uint32  crashdump_displayRawStack(crashdump_t * cptr, uint8 * buf_p, boolean iflag);
uint32  crashdump_backtrace(crashdump_t * cptr, uint8 * buf_p);

void crashdump_test_core(void);


/* these fcts are directly accessed if PROM and SCC_MGMT, else thru jmptbl */
crashdump_t * get_pending_crashdump(void);
void    set_nmi_exception_handler(address_t addr);


/* in stacktrace.c */
uint32  format_stacktrace(crashdump_t * cptr, uint8 * buf_p);

#endif	/* _CRASHDUMP_H */
