/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: debug-util.c,v 1.6 2002/11/05 00:44:05 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */

#ifndef PROM

#include <nfx-incl.h>
#include "debug.h"
#include "except-api.h"
#include "except.h"

#define _go	exception_return

extern int gdb_init;
extern volatile uint32 inDebugger;

/*************************************************************
 *  clrbpts()
 */
void
clrbpts ()
{
	int             i;

	for (i = 0; i < MAX_BPT; i++) {
#if RAL_XXX
		if (Bpt[i].addr != NO_BPT && Bpt[i].wpt != 0)
			_mips_watchpoint_clear (Bpt[i].wpt, -1, Bpt[i].addr, Bpt[i].len);
#endif
		Bpt[i].addr = NO_BPT;
		Bpt[i].wpt = 0;
#if RAL_XXX
		if (Bpt[i].cmdstr)
			free (Bpt[i].cmdstr);
#endif
	}
}
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
/* Remove a breakpoint */
int
clrbpt (wpt, adr, len)
	int wpt;
	long adr;
	long len;
{
    int j;

//printf("clrbpt, adr = 0x%x\n", adr);
	for (j = 0; j < MAX_BPT; j++) {
		if (Bpt[j].addr == adr && Bpt[j].len == len && Bpt[j].wpt == wpt) {
#if RAL_XXX
			if (wpt)
				_mips_watchpoint_clear (wpt, -1, adr, len);
#endif
//printf("Bpt[%d] = 0x%x, setting .addr = NO_BPT\n", j, Bpt[j].addr);
			Bpt[j].addr = NO_BPT;
#if RAL_XXX
			if (Bpt[j].cmdstr)
				free (Bpt[j].cmdstr);
#endif
			return (1);
		}
	}
	return (0);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
/* Insert a breakpoint */
int
setbpt (wpt, adr, len)
	int wpt;
	long adr;
	long len;
{
	int j;

	/* remove any duplicates */
	clrbpt (wpt, adr, len);

	for (j = 0; j < MAX_BPT && Bpt[j].addr != NO_BPT; j++);
	if (MAX_BPT <= j)
		/* too many breakpoints */
		return (0);

#if RAL_XXX
    if (wpt) {
	    int res;
		res = _mips_watchpoint_set (wpt, -1, adr, 0, len);
		if (res != MIPS_WP_OK) {
			switch (res) {
				case MIPS_WP_NONE:
					printf ("h/w watchpoints not available\n");
					break;
				case MIPS_WP_NOTSUP:
					printf ("cannot handle this watchpoint type\n");
					break;
				case MIPS_WP_INUSE:
					printf ("out of h/w watchpoint resources\n");
					break;
				case MIPS_WP_OVERLAP:
					printf ("watchpoint overlaps PMON memory\n");
					break;
				case MIPS_WP_BADADDR:
					printf ("unsupported watchpoint virtual address\n");
					break;
				default:
					printf ("internal watchpoint problem\n");
					break;
			}
			return (0);
		}
    }
#endif

	Bpt[j].addr = adr;
	Bpt[j].len  = len;
	Bpt[j].wpt  = wpt;
	return (1);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
store_trace_breakpoint ()
{
	if (BptTrc.addr != NO_BPT) {
		BptTrc.value = load_int (BptTrc.addr);
//printf("store_trace_breakpoint BptTrc.addr = %p\n", BptTrc.addr);
		RAL_store_int (BptTrc.addr, BPT_CODE);
    }
	if (BptTrcb.addr != NO_BPT) {
		BptTrcb.value = load_int (BptTrcb.addr);
//printf("store_trace_breakpoint BptTrcb.addr = 0x%x\n", BptTrcb.addr);
		RAL_store_int (BptTrcb.addr, BPT_CODE);
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
store_breakpoint ()
{
	int             i;

	for (i = 0; i < MAX_BPT; i++) {
		if (BptTmp.addr == Bpt[i].addr)
			BptTmp.addr = NO_BPT;
		if (BptTrc.addr == Bpt[i].addr)
			BptTrc.addr = NO_BPT;
		if (BptTrcb.addr == Bpt[i].addr)
			BptTrcb.addr = NO_BPT;
	}

	if (BptTrc.addr == BptTmp.addr)
		BptTrc.addr = NO_BPT;
	if (BptTrcb.addr == BptTmp.addr || BptTrcb.addr == BptTrc.addr)
		BptTrcb.addr = NO_BPT;

	for (i = 0; i < MAX_BPT; i++)
		if (Bpt[i].addr != NO_BPT && Bpt[i].wpt == 0) {
			Bpt[i].value = load_int (Bpt[i].addr);
//printf("store_breakpoint, %d, addr = 0x%x, value = 0x%x\n", i, Bpt[i].addr, Bpt[i].value);
			RAL_store_int (Bpt[i].addr, BPT_CODE);
		}

	if (BptTmp.addr != NO_BPT) {
		BptTmp.value = load_int (BptTmp.addr);
//printf("store_breakpoint, Tmp, addr = 0x%x, value = 0x%x\n", i, Bpt[i].addr, Bpt[i].value);
		RAL_store_int (BptTmp.addr, BPT_CODE);
	}
	store_trace_breakpoint ();
#if RAL_XXX
	_mips_watchpoint_insert ();
#endif
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
static int
setTrcbp (adr, stepover)
	address_t       adr;
	int             stepover;
{
	unsigned long   target;

	BptTrc.addr = NO_BPT;
	BptTrcb.addr = NO_BPT;
	if (is_branch (adr)) {
#if 0
		if (is_branch (adr + 4)) {
			printf ("branch in delay slot\n");
			return (1);
		}
#endif
		target = branch_target_address (adr);
		if (target == adr)
			target = adr + 8;   /* skip self branches */
		if (is_conditional_branch (adr) && target != adr + 8)
			BptTrc.addr = adr + 8;
		if (is_jal (adr) && stepover)
			BptTrc.addr = adr + 8;
		else if (is_jr (adr) && !is_writeable (target))
			BptTrc.addr = EXC_DBGREG[R_RA];
		else
			BptTrcb.addr = target;
	}
	else
		BptTrc.addr = adr + 4;

	return (0);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
godebug ()
{
extern int32 gdb_init_continue;
	/* remote debugger has responsibility for issuing a single-step
	   and reinserting breakpoints before continuing, but we must
	   insert any breakpoints or watchpoints that it has set up
	   using the 'B' request. */
	trace_mode = TRACE_DC;
	store_breakpoint ();
if (gdb_init_continue)
return;
	_go ();
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
int
sstep ()
{

	if (setTrcbp (EXC_DBGREG[R_EPC], 0))
		return (1);
	trace_mode = TRACE_DS;
	store_trace_breakpoint ();
	_go ();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/22/02
 *
 *-----------------------------------------------------------------
 */
void
debug_exit()
{
	rm_bpts();

	trace_mode = TRACE_NO;

#if defined(SSC_MGMT)
	printf("exit_debug, EPC = 0x%lx\n", EXC_DBGREG[R_EPC]);
#else
	printf("exit_debug, EPC = 0x%llx\n", EXC_DBGREG[R_EPC]);
#endif

	inDebugger = 0;

	if (runtime_exception != EXCEPTION_NONE) {
		printf("runtime exception\n");
		panic_prompt();
	}
	else
		printf("no runtime exception\n");

	if (!gdb_init)
		_go();

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
remove_trace_breakpoint ()
{
	if (BptTrc.addr != NO_BPT && load_int (BptTrc.addr) == BPT_CODE)
{
//printf("remove_trace_breakpoint, BptTrc.addr = 0x%x\n", BptTrc.addr);
//printf("writing 0x%x to %p\n", BptTrc.value, BptTrc.addr);
		RAL_store_int (BptTrc.addr, BptTrc.value);
}
	BptTrc.addr = NO_BPT;

	if (BptTrcb.addr != NO_BPT && load_int (BptTrcb.addr) == BPT_CODE)
{
//printf("remove_trace_breakpoint, BptTrcb.addr = 0x%x\n", BptTrcb.addr);
		RAL_store_int (BptTrcb.addr, BptTrcb.value);
}
	BptTrcb.addr = NO_BPT;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
rm_bpts ()
{
	int             i;

//printf("rm_bpts: ");
	if (BptTmp.addr != NO_BPT && load_int (BptTmp.addr) == BPT_CODE)
{
//printf("Tmp, addr = 0x%x, value = 0x%x\n", BptTmp.addr, BptTmp.value);
	RAL_store_int (BptTmp.addr, BptTmp.value);
}
	BptTmp.addr = NO_BPT;

	for (i = 0; i < MAX_BPT; i++) {
	if (Bpt[i].addr != NO_BPT && Bpt[i].wpt == 0
		&& load_int (Bpt[i].addr) == BPT_CODE)
{
//printf("Bpt %d, addr = 0x%x, value = 0x%x\n", i, Bpt[i].addr, Bpt[i].value);
		RAL_store_int (Bpt[i].addr, Bpt[i].value);
}
	}
	remove_trace_breakpoint ();
}

#endif
