/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: exc-exception.c,v 1.27 2002/12/03 22:00:11 maximk Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/cm-api.h>
#include "except-api.h"
#include "except.h"
#include "debug.h"
#include "crashdump-api.h"

#if !defined(PROM)
#include <exports/pte-api.h>

#include <exports/esm-api.h>
#include <exports/queue-api.h>
#include "../sm-esm/esm.h"
#include "cm-api.h"
#include <sys/led-api.h>
#endif

int coreauto;

cpureg_t	EXC_DBGREG[NREGS] CPU_PRIVATE;

char exc_stack[0x3000] CPU_PRIVATE;

extern void serio_reinit(void);
extern void write_core(void);
extern void panic_prompt(void);
int ecc_bus_error_check(uint32 sr, uint32 ca, address_t epc);

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/17/02
 *
 *-----------------------------------------------------------------
 */
void
exc_install_handler(
	uchar8	* dest,
	uchar8	* src,
	uchar8	* src_end,
	uint32	max_len
)
{
	uint32	len = (address_t)src_end - (address_t)src;
//	printf("exc_install_handler(0x%x, 0x%x, 0x%x, %d)\n",
//												dest, src, src_end, max_len);

	if (len > max_len) {
		panic("Invalid exception length, src_end - src = 0x%x\n", len);
	}
	memcpy(dest, src, len);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
exc_getname(
	uint32	cause
)
{
	return (excodes[cause >> CAUSE_EXCSHIFT]);

}

#ifdef R4000

#ifndef PROM

#if defined(USE_THREAD_STACKS) 

#define OVERFLOW_LIMIT (32 * 1024)

uint64 crashdump_reserved_stack;

void
fixup_stack_overflow(void)
{
	int pte_index =
		(crashdump_reserved_stack
		 - get_memory_virt_base(MMU_THREADSTACK_REGION))
		/ (uint64)PTE_ENTRY_SIZE;
	pte_t* pte = &mmu_tptr[MMU_THREADSTACK_REGION][pte_index];

	__asm__("dmtc0 %0, $%1;nop;nop;nop;nop" : :
			"r"(pte->tlb_lo0_phys),"I"(C0_TLBLO0));
	__asm__("dmtc0 %0, $%1" : :	"r"(pte->tlb_lo1_phys),"I"(C0_TLBLO1));
	__asm__("mtc0 %0, $%1;nop;nop;nop;nop;tlbwr;nop;nop" : :
			"r"(pte->pgsize),"I"(C0_PGMASK));
}


static void
check_stack_overflow(uint64 badvaddr)
{
	esm_handle_t handle = esm_getCurrHandle();
	if (handle) {
		esm_instance_t *esm = esm_getInstFromHandle(handle);
		if (esm && (esm->flags & ESM_CAN_YIELD)
			&& (badvaddr < esm->stackBase)
			&& badvaddr >= esm->stackBase - OVERFLOW_LIMIT) {
			printf("got stack overflow: base=%llx badvaddr=%llx\n",
				   (uint64)esm->stackBase, badvaddr);
			fixup_stack_overflow();
		}
	}
}
#else

static void
check_stack_overflow(uint64 badvaddr)
{
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
exception(
	uint32		cause,
	address_t	epc,
	address_t	ra
)
{
	address_t	badvaddr, errepc;
	uint32		cacheerr, sr;
    int         skip_dumps;
	uchar8		except_str[80];

	if ((trace_mode == TRACE_NO) || ((cause & CAUSE_EXCMASK) != CEXC_BP))
		runtime_exception = EXCEPTION_GEN_EXCEPT;

	if ((trace_mode == TRACE_DC || trace_mode == TRACE_DS)) {
		/*
		 * Pass exceptions/breakpoints onto debug stub
		 */
		rm_bpts ();
		dbgmode();
	}

	/*
	 * General Exception, display info, save crashdump
	 */
	badvaddr = EXC_DBGREG[R_BADVA];
	sr = EXC_DBGREG[R_STATUS];

    skip_dumps = 0;

	switch (cause & CAUSE_EXCMASK) {
	case CEXC_TLBS:
	case CEXC_TLBL:
		check_stack_overflow(badvaddr);
		break;
	default:
		break;
	}

	printf("\n\nException Cause = %s (0x%08x)\n",
									exc_getname (cause & CAUSE_EXCMASK), cause);
	printf ("EPC:      %p\n", (void *)epc);
	printf ("RA:       %p\n", (void *)ra);
	printf ("SR:       0x%x\n", sr);
	switch (cause & CAUSE_EXCMASK) {
	    case CEXC_MOD:
	    case CEXC_TLBL:
	    case CEXC_TLBS:
	    case CEXC_ADEL:
	    case CEXC_ADES:
            printf ("BadVaddr: %p\n", (void *)badvaddr);
            break;
		case CEXC_CACHE:
        {
            errepc = EXC_DBGREG[R_ERREPC];
            cacheerr = EXC_DBGREG[R_CACHERR];
            printf ("ERREPC:   %p\n", (void *)errepc);
            printf ("CacheErr: 0x%x\n", cacheerr);
            skip_dumps = 1;
        }
        break;

	    case CEXC_INT:
        {
            errepc = EXC_DBGREG[R_ERREPC];
            cacheerr = EXC_DBGREG[R_CACHERR];
            printf ("ERREPC:   %p\n", (void *)errepc);
            printf ("BadVaddr: %p\n", (void *)badvaddr);
        }
	}

#ifdef SIBYTE
    /*
     * due to the way sibyte uncorrectable ECC errors generate different
     * exceptions. Some times bus errors get in mix. So check for ecc and
     * bus errors which are programed to Int3 and Int2 respectively and
     * call the error handler to deal with both
     */
    skip_dumps = ecc_bus_error_check(sr, cause, epc);
#endif

    if (!skip_dumps) {
        sprintf(except_str, "Exception Cause = %s",
                                            exc_getname (cause & CAUSE_EXCMASK));
        exc_crashdump_except_save(except_str);
    }

	cmSetCPUState(CM_SLICE_STATE_RUNTIME_CRASHED);

#ifndef NFX_SMP
	if (!skip_dumps && coreauto) {
		write_core();
	}
#endif

	panic_prompt();
}

#endif	/* !PROM */

#else

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
void
exception_tx3927()
{
	address_t	epc, ra, badvaddr;
	uint32		cause, sr;
	uchar8		except_str[80];

	if ((trace_mode == TRACE_DC || trace_mode == TRACE_DS)) {
		/*
		 * Pass exceptions/breakpoints onto debug stub
		 */
		rm_bpts ();
		dbgmode();
	}

	/*
	 * General Exception, display info, save crashdump
	 */
	cause = EXC_DBGREG[R_CAUSE];
	epc = EXC_DBGREG[R_EPC];
	ra = EXC_DBGREG[R_RA];
	sr = EXC_DBGREG[R_STATUS];
	badvaddr = EXC_DBGREG[R_BADVA];
	printf("\n\nException Cause = %s (0x%08x)\n",
									exc_getname (cause & CAUSE_EXCMASK), cause);
	printf ("EPC:      %p\n", (void *)epc);
	printf ("RA:       %p\n", (void *)ra);
	printf ("SR:       0x%x\n", sr);
	switch (cause & CAUSE_EXCMASK) {
	    case CEXC_MOD:
	    case CEXC_TLBL:
	    case CEXC_TLBS:
	    case CEXC_ADEL:
	    case CEXC_ADES:
	            printf ("BadVaddr: %p\n", (void *)badvaddr);
	}

	sprintf(except_str, "Exception Cause = %s",
										exc_getname (cause & CAUSE_EXCMASK));
	exc_crashdump_except_save(except_str);
    sscm_led_set(LED_FAULT_ON);
    sscm_led_set(LED_OK_OFF);
	panic_prompt();
}

#endif	/* R4000 */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
exception_nmi(
	uint32		cause,
	address_t	errepc,
	address_t	ra
)
{
	uint32	sr;

#ifndef SIBYTE_BC
extern void mallocInit(uchar8 *, uint32);
    mallocInit((uchar8 *)0x81000000, 0x1000000);
#endif

#ifndef PROM
	runtime_exception = EXCEPTION_NMI;
#endif


#ifdef	PROM_R9000
	// Re-init GT UART to debug mode.
	// Can't use SDMA since BSD runtime 
	// has changed the descriptor pointers.
	serio_reinit();
#endif

// WTG test SSC crash Dump stuff
//#ifndef PROM_R9000
	exc_crashdump_nmi_save();
//#endif

	sr = EXC_DBGREG[R_STATUS];
	printf("\n\nException Cause = Watchdog Timeout/NMI\n");
	printf ("ERREPC:   %p\n", (void *)errepc);
	printf ("RA:       %p\n", (void *)ra);
	printf ("SR:       0x%x\n", sr);

#ifndef PROM
	cmSetCPUState(CM_SLICE_STATE_RUNTIME_CRASHED);

#ifndef NFX_SMP
	if (coreauto) {
		write_core();
	}
#endif
#endif

	panic_prompt();
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
exception_panic(
	uchar8	* str
)
{
#ifndef PROM
	runtime_exception = EXCEPTION_GEN_EXCEPT;
	cmSetCPUState(CM_SLICE_STATE_RUNTIME_CRASHED);
#endif

	exc_crashdump_panic_save(str);

#ifndef NFX_SMP
	if (coreauto) {
		write_core();
	}
#endif
	
//#else
//    printf("PROM panic crashdump not saved in PROM!\n");

	panic_prompt();
}

