
/* emaproot.c */
void fs_emaproot_init(fs_Context *context, fs_Inode *inode);
void fs_emap_add(fs_Context *context, fs_Inode *inode, uint64 offset, fs_Extent extent);
void fs_emap_add_block(fs_Context *context, fs_Inode *inode, uint64 offset, fs_Block block);
fs_Emappair fs_emap_lookup(fs_Context *context, fs_Inode *inode, uint64 offset);
void fs_emap_remove(fs_Context *context, fs_Inode *inode, uint64 offset);
void fs_emap_remove_block(fs_Context *context, fs_Inode *inode, uint64 page);
void fs_emap_replace_block(fs_Context *context, fs_Inode *inode, uint64 page, fs_Block block);

/**
 * Look up a single block in an extent map.  Returns FS_INVALID_BLOCK if
 * the block is not mapped.
 */
extern fs_Block fs_emap_lookup_block(
    fs_Context *context,
    fs_Inode *inode,
    uint64 offset);

/*
 * Split an inode in two at the given offset, return a new inode pointing to the
 * remaining data, to be put into the trashcan.
 */
extern fs_Inode *fs_emap_cleave(
	fs_Context *context,
	fs_Inode *trashcan_inode,
	fs_Inode *inode,
	uint64 offset);

typedef enum fs_WalkBlockType {
    FS_WALK_INNER_NODE_TYPE = 1,
    FS_WALK_LEAF_NODE_TYPE,
    FS_WALK_EXTENT_TYPE,
} fs_WalkBlockType;

typedef struct fs_WalkResult {
    fs_WalkBlockType type;
    fs_Block block;     // FS_WALK_INNER_NODE_TYPE, FS_WALK_LEAF_NODE_TYPE
    fs_Extent extent;   // FS_WALK_EXTENT_TYPE
    uint64 offset;      // FS_WALK_EXTENT_TYPE
} fs_WalkResult;

/*
 * Walk an extent map, including the inner nodes, calling the worker
 * function for each block in the file. The worker function returns
 * FALSE when an inconsistency is discovered in the filesystem; this
 * is used by eek. When using emap_walk for other jobs (snap create/
 * delete, etc.) the worker function should always return TRUE.
 */
void
fs_emap_walk(
	fs_Context *context,
	fs_Inode *inode,
	boolean (*worker)(fs_Context *context, fs_Inode *inode,
                   fs_WalkResult *result, void *arg0),
	void *arg0);


/* emap-debug.c */
void fs_emapdebug(fs_Context *context);

/* btree.c */
extern void fs_bt_dispatch_init(void);

/**
 * Shrink an extent map by up to one page of extents, and return
 * zero when the entire file has been shrunk to zero size.
 */
uint64 fs_emap_shrink(fs_Context *context, fs_Inode *inode);

/**
 * Split an inode into two extent maps, one to the left of the given file
 * offset, and one to the right. The original inode contains the extents
 * to the left of the offset, and a new inode with the extents to the right
 * of the offset is returned.
 *
 * If the cleave operation results in an empty right-hand extent map (for
 * example, there is a hole in the file) then a NULL inode pointer might be
 * returned.
 */
fs_Inode *fs_emap_cleave(
	fs_Context *context,
	fs_Inode *trashcan_inode,		// trashcan directory
	fs_Inode *inode,			// file to cleave
	uint64 offset);

/*
 * Functions for EEK. (filesystem checker)
 *
 * Scans an extent map and marks used blocks in the given bitmap (one bit per
 * block), scans a directory and marks used inodes in the bitmap (one bit per
 * inode).
 */

boolean fs_emap_fix_used_blocks(fs_Context *context, uint8 *bitmap, fs_Inode *inode, fs_Eekmark type);


/*
 * Btree locking functions
 */
void *fs_getBtreeExclusive(fs_Context *context, fs_Inode *inode);
void *fs_getBtreeShared(fs_Context *context, fs_Inode *inode);
void fs_releaseBtreeShared(fs_Context *context, fs_Inode *inode);
void fs_releaseBtreeExclusive(fs_Context *context, fs_Inode *inode);
void fs_demoteBtreeLock(fs_Context *context, fs_Inode *inode);
