/*
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * $Id: bt_dispatch.h,v 1.10 2002/06/19 23:05:29 lanc Exp $
 */

/*
 * Dispatch functions for B-tree operations.
 */

#define FS_BT_MAXLEVEL 8

/* BT_SEARCH */

typedef struct fs_Bt_search_dispatch fs_Bt_search_dispatch;

typedef uint64 (*fs_Bt_search_func)(
    fs_Context *,
    fs_Inode *,
    fs_Buf *,
    int,
    uint64,
    void *,
    fs_Bt_search_dispatch *,
    boolean exclusive);

struct fs_Bt_search_dispatch {
	fs_Bt_search_func func[FS_BT_MAXLEVEL];
};

extern uint64
fs_bt_search(fs_Context *context,
	     fs_Inode *inode,
	     fs_Buf *buf,
	     int level,
	     uint64 key,
	     void *data,
	     fs_Bt_search_dispatch *search,
             boolean exclusive);

/* BT_INSERT */

typedef struct fs_Bt_insert_dispatch fs_Bt_insert_dispatch;

typedef fs_Bt_insert (*fs_Bt_insert_func)(
	fs_Context *,
	fs_Inode *,
	fs_Buf *,
	int,
	uint64,
	void *data,
	boolean,
	fs_Bt_insert_dispatch *);

struct fs_Bt_insert_dispatch {
	fs_Bt_insert_func func[FS_BT_MAXLEVEL];
};

extern fs_Bt_insert
fs_bt_insert(fs_Context *context,
	     fs_Inode *inode,
	     fs_Buf *buf,
	     int level,
	     uint64 key,
	     void *data,
	     boolean split_allowed,
	     fs_Bt_insert_dispatch *insert);

/* BT_REMOVE */

typedef struct fs_Bt_remove_dispatch fs_Bt_remove_dispatch;

typedef fs_Bt_remove (*fs_Bt_remove_func)(
	fs_Context *,
	fs_Inode *,
	fs_Buf *,
	int,
	uint64,
	void *,
	boolean,
	fs_Bt_remove_dispatch *);

struct fs_Bt_remove_dispatch {
	fs_Bt_remove_func func[FS_BT_MAXLEVEL];
};

extern fs_Bt_remove
fs_bt_remove(fs_Context *context,
	     fs_Inode *inode,
	     fs_Buf *buf,
	     int level,
	     uint64 key,
	     void *data,
	     boolean dispose_allowed,
	     fs_Bt_remove_dispatch *remove);

/* BT_CLEAVE */

typedef struct fs_Bt_cleave_dispatch fs_Bt_cleave_dispatch;

typedef fs_Bufpair (*fs_Bt_cleave_func)(
	fs_Context *,
	fs_Inode *,
	fs_Buf *,
	int,
	uint64,
	void *,
	fs_Bt_cleave_dispatch *);

struct fs_Bt_cleave_dispatch {
	fs_Bt_cleave_func func[FS_BT_MAXLEVEL];
};

extern fs_Bufpair
fs_bt_cleave(fs_Context *context,
	     fs_Inode *inode,
	     fs_Buf *buf,
	     int level,
	     uint64 key,
	     void *data,
	     fs_Bt_cleave_dispatch *cleave);

/* BT_EEK */

typedef struct fs_Bt_eek_dispatch fs_Bt_eek_dispatch;

typedef fs_Bt_eeksearch (*fs_Bt_eek_func)(
	fs_Context *,
	fs_Inode *,
	fs_Buf *,
	int,
	uint64,
	void *,
	fs_Bt_eek_dispatch *);

struct fs_Bt_eek_dispatch {
	fs_Bt_eek_func func[FS_BT_MAXLEVEL];
};

extern fs_Bt_eeksearch
fs_bt_eek(fs_Context *context,
	     fs_Inode *inode,
	     fs_Buf *buf,
	     int level,
	     uint64 key,
	     void *data,
	     fs_Bt_eek_dispatch *eek);

typedef struct fs_Bt_dispatch fs_Bt_dispatch;
typedef struct fs_Bt_funcs fs_Bt_funcs;

struct fs_Bt_funcs {
	fs_Bt_insert_dispatch insert;	/* insert extent/directory entry */
	fs_Bt_remove_dispatch remove;	/* remove extent/directory entry */
	fs_Bt_remove_dispatch shrink;	/* remove entire extentmap page */
	fs_Bt_search_dispatch search;	/* look up extent/directory entry */
	fs_Bt_search_dispatch read;	    /* readdir / search free tree sequentially */
	fs_Bt_eek_dispatch eek_blocks;	/* mark used blocks for eek */
	fs_Bt_eek_dispatch eek_inodes;  /* mark used inodes for eek (dir only) */
	fs_Bt_eek_dispatch eek_scan;    /* scan hash to rebuild indexer (dir only) */
	fs_Bt_search_dispatch walk;	    /* walk an extent map (now used by snapshots) */
	fs_Bt_cleave_dispatch cleave;	/* cleave an extent map (for ftrunc) */
};

struct fs_Bt_dispatch {
	fs_Bt_funcs dir, emap;
};

extern fs_Bt_dispatch bt_funcs;
