/*
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * $Id: emap.h,v 1.12 2002/11/26 22:54:07 bob Exp $
 */


/*
 * Emapleaf access "macros"
 */

#define EMAP_IS_COMPACT(_inode)                                  \
    ((_inode)->dinode.attrs & FS_IFLG_COMPACT_EMAP)

static inline uint32
get_index(fs_Inode *inode, fs_Buf *buf, int i)
{
	uint32 index;

	if (EMAP_IS_COMPACT(inode)) {
		fs_EmapleafCompact *l = buf->u.emapleafCompact;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_COMPACT_EXTENTS_MAX);
	} else {
		fs_Emapleaf *l = buf->u.emapleaf;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_EXTENTS_MAX);
	}

	return index;
}

static inline uint64
get_offset(fs_Inode *inode, fs_Buf *buf, int i)
{
	uint32 index;
	uint64 offset;

	if (EMAP_IS_COMPACT(inode)) {
		fs_EmapleafCompact *l = buf->u.emapleafCompact;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_COMPACT_EXTENTS_MAX);
		offset = l->offsets[index];
	} else {
		fs_Emapleaf *l = buf->u.emapleaf;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_EXTENTS_MAX);
		offset = l->offsets[index];
	}

	return offset;
}

static inline fs_Extent
get_extent(fs_Inode *inode, fs_Buf *buf, int i)
{
	uint32 index;
	fs_Extent extent;

	if (EMAP_IS_COMPACT(inode)) {
		fs_EmapleafCompact *l = buf->u.emapleafCompact;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_COMPACT_EXTENTS_MAX);
		extent = l->extents[index];
	} else {
		fs_Emapleaf *l = buf->u.emapleaf;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_EXTENTS_MAX);
		extent = l->extents[index];
	}

	return extent;
}

static inline uint64 *
get_offset_ref(fs_Inode *inode, fs_Buf *buf, int i)
{
	uint32 index;
	uint64 *offsetp;

	if (EMAP_IS_COMPACT(inode)) {
		fs_EmapleafCompact *l = buf->u.emapleafCompact;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_COMPACT_EXTENTS_MAX);
		offsetp = &l->offsets[index];
	} else {
		fs_Emapleaf *l = buf->u.emapleaf;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_EXTENTS_MAX);
		offsetp = &l->offsets[index];
	}

	return offsetp;
}

static inline fs_Extent *
get_extent_ref(fs_Inode *inode, fs_Buf *buf, int i)
{
	uint32 index;
	fs_Extent *extentp;

	if (EMAP_IS_COMPACT(inode)) {
		fs_EmapleafCompact *l = buf->u.emapleafCompact;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_COMPACT_EXTENTS_MAX);
		extentp = &l->extents[index];
	} else {
		fs_Emapleaf *l = buf->u.emapleaf;
		index = l->index[i];
		FS_ASSERT(index < FS_EMAP_EXTENTS_MAX);
		extentp = &l->extents[index];
	}

	return extentp;
}

/*
 * XXX: hack: nkeys is stored in the 0th slot in both emapleaf and emapleafCompact
 */
static inline int
get_nkeys(fs_Buf *buf)
{
	fs_Emapleaf *l = buf->u.emapleaf;
	return l->nkeys;
}
