/*
 * Name:  fs_cod.h
 *
 * RCS:  $Id: fs-cod.h,v 1.14 2002/08/20 15:51:54 bob Exp $
 *
 * Description:  Definitions for the MCPU <-> filesystem interfase
 *               Synchronisation of the COD for write and read requests
 * 
 *
 * Date Created:  7/22/2001
 */

#if !defined(FS_COD_H)
#define FS_COD_H

/*
 * A special pre-initialized COD, 
 * filled with '0's.
 * This COD should never be freed.
 * needed for read requests, in spare files, with only '0's
 * Avantage: no free cod is needed.
 */

extern nfx_cod_t fs_nullCOD ;


/*
 * message types
 */

typedef struct mcpu_confirm_cod
{
#if defined(FS_STANDALONE)
    int32       src;    // match fs_simMsgHdr
    int32       dest;   // match fs_simMsgHdr
    int32       nfsReqNum;
    int32       sockIndex;
#endif
    uint32      requestType;            // MCPU_CONFIRM_COD
    int32       len;
    nfx_cod_t   cod;
} mcpu_confirm_cod_t;


/*
 * API for fs-cod.c
 */

uint32
hash_cod (const void *key) ;

int32
cmp_cod (const void *key1, const void *key2) ;

void
destruct_fs_cod (void *key, void *obj);



int32
fs_cod_init ();

int32
fs_cod_remove ();

int32
fs_cod_write_confirm (
    nfx_cod_t cod);

int32
fs_cod_write_check (
    fs_Context *context,
    nfx_cod_t cod, 
    esm_handle_t smHandle);

int32
fs_cod_write_remove (
    nfx_cod_t cod);


int32
fs_cod_split (
    nfx_cod_t   src_COD,
    uint32      src_offset,
    nfx_cod_t   dest_COD);


int32
fs_cod_merge (
    nfx_cod_t   src_COD,
    uint32      src_len,
    uint32      src_offset,
    nfx_cod_t   dst_COD,
    uint32      dst_offset);

int32
fs_cod_copy (
    nfx_cod_t   src_COD,
    uint32      src_len,
    uint32      src_offset,
    nfx_cod_t   dst_COD,
    uint32      dst_offset);


int32
fs_cod_create_null (
    nfx_cod_t   COD,
    uint32      len,
    uint32      offset);

int32
fs_cod_fill_null (
    nfx_cod_t   *COD,
    uint32      len,
    uint32      offset);

nfx_cod_t 
fs_cod_get_new_cod(
    void);


void
fs_cod_free_cod(
    nfx_cod_t COD);


#endif // FS_COD_H







