/*
 * Name:  fs-debug.c
 *
 * RCS:  $Id: fs-debug.c,v 1.101 2002/12/10 18:42:46 bob Exp $
 *
 * Description: Command handlers for debugging commands.
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

#if !defined(SSC)

#include "fs-includes.h"

extern void fs_memCheckLeaks(void); // TODO

#ifdef FS_DEBUG_SNAP_CRASH
int fs_snapPhaseToCrash = -1;
#endif

/** 
 * fs_viewFsThread
 */
int32
fs_viewFsThread(
    fs_Context      *context)
{
    fs_MsgViewFsResp  *resp = (fs_MsgViewFsResp *) context->respHdr;
    fs_MsgViewFsReq   *req = (fs_MsgViewFsReq *) context->reqHdr;
    fs_FSInfo         *fsInfo = &(resp->u.fsInfo[0]);
    evm_volId_t       *volId = &(resp->u.volId[0]);
    uint32            size = sizeof(fs_MsgViewFsResp) ;
    uint32            length;
    uint32            loop, loop_1;
    
    resp->totalNbrFs = 0;
    resp->nbrFs = 0;
    resp->type = req->type;
    resp->clientprivate = req->clientprivate;

    NFX_SPIN_LOCK(&fs_fileSystemTableLock);

    if (req->type == FS_VIEW_LIST_OF_VOLUMES)
    {
        for (loop = 0; loop < FS_FILE_SYSTEM_HASH_MAX; loop++)
        {
            fs_FileSystem   *fileSystem;

            for (fileSystem = fs_fileSystems[loop]; fileSystem != NULL; fileSystem = fileSystem->next)
            {
                length = sizeof(evm_volId_t);

                if ((size + length) < 1490)
                {
                    volId->whole_thing = fileSystem->volId.whole_thing;
                    volId++;

                    size +=  length;

                    resp->nbrFs++;
                }
                resp->totalNbrFs++;
            }
        }
        
        NFX_SPIN_UNLOCK(&fs_fileSystemTableLock);

        context->respSize = size;

        return NFX_OK;
    }
    else
    {
        FS_ASSERT(req->type == FS_VIEW_ONE_VOLUME);
    }


    for (loop = 0; loop < FS_FILE_SYSTEM_HASH_MAX; loop++)
    {
        fs_FileSystem   *fileSystem;

        for (fileSystem = fs_fileSystems[loop]; fileSystem != NULL; fileSystem = fileSystem->next)
        {
            if (fileSystem->super &&
                (req->volId.whole_thing == fileSystem->volId.whole_thing))
            {
                length = alignto (sizeof(fs_FSInfo) + strlen(fileSystem->super->mountName) + 1, sizeof(uint64));

                if ((size + length) < 1490)
                {
                    bzero (fsInfo, sizeof(fs_FSInfo));
                    
                    fsInfo->volId.whole_thing = fileSystem->volId.whole_thing;
                    fsInfo->state = fileSystem->state;
                    context->fileSystem = fileSystem;
					if (fs_isBackupRestoreInProgress(context)) {
							fsInfo->state = FS_VOL_STATE_BACKUP_RESTORE;
					}

                    fsInfo->stat.fsBlockSize = fileSystem->super->blkSize;
                    fsInfo->stat.numBlks = fileSystem->super->numBlks;
                    fsInfo->stat.numBlksAvailUser =
                        fsInfo->stat.numBlksAvail = fileSystem->super->numBlks -
                        fileSystem->super->blksUsed;
                    // Use numBlks here, since maxInodeNumber will expand up
                    // to that limit.
                    fsInfo->stat.numInodes = fs_numInodesUserMax(fileSystem);
                    fsInfo->stat.numInodesAvail =
                        fsInfo->stat.numInodesAvailUser = fs_numInodesAvail(fileSystem);
                    //fsInfo->stat.readsPerSec =
                    //fsInfo->stat.writesPerSec =
                    //fsInfo->stat.avgReadSize = FS_PREF;
                    //fsInfo->stat.avgWriteSize = FS_PREF;

                    fsInfo->mkfsTime = fileSystem->super->mkfsTime;
                    for (loop_1 = 0 ; loop_1 < FS_SUPER_NBR_MOUNTTIME; loop_1++)
                    {
                        fsInfo->mountTime[loop_1] = fileSystem->super->mountTime[loop_1];
                        fsInfo->mountState[loop_1] = fileSystem->super->mountState[loop_1];
                        fsInfo->mountFail[loop_1] = fileSystem->super->mountFail[loop_1];
                        fsInfo->umountTime[loop_1] = fileSystem->super->umountTime[loop_1];
                        fsInfo->umountState[loop_1] = fileSystem->super->umountState[loop_1];
                    }
                    fsInfo->nbrMount = fileSystem->super->nbrMount;
                    fsInfo->nbrUmount = fileSystem->super->nbrUmount;
                    fsInfo->nbrDirtyMount = fileSystem->super->nbrDirtyMount;
                    fsInfo->nbrMountFailedLogDev = fileSystem->super->nbrMountFailedLogDev;
                    fsInfo->nbrMountFailedReplay = fileSystem->super->nbrMountFailedReplay;
                    fsInfo->nbrMountFailedEek = fileSystem->super->nbrMountFailedEek;
                    fsInfo->nbrMountFailedSnap = fileSystem->super->nbrMountFailedSnap;

                    fsInfo->percentMinFree = 100 - fileSystem->percentMinFree;
                    fsInfo->nameLength = strlen(fileSystem->super->mountName);
                    strcpy(&fsInfo->mountName[0], fileSystem->super->mountName);
                    fsInfo->length = length;

                    size +=  fsInfo->length;
                    resp->nbrFs++;

                    fsInfo = (fs_FSInfo*)(((char*)fsInfo) + length);
                }
            }
            resp->totalNbrFs++;
        }
    }
    NFX_SPIN_UNLOCK(&fs_fileSystemTableLock);

    context->respSize = size;

    return NFX_OK;

} /* fs_viewFsThread */


/*-----------------------------------------------------------------------------
 * fs_cachestatsThread
 */
int32
fs_cachestatsThread(
    fs_Context      *context)
{
    fs_MsgCachestatsResp  *resp = (fs_MsgCachestatsResp *) context->respHdr;
    fs_MsgCachestatsReq   *req = (fs_MsgCachestatsReq *) context->reqHdr;
    fs_FileSystem         *fileSystem;
    int32                 i,j;

    resp->volId = req->volId;
    resp->isVerbose = req->isVerbose;

    if (req->volId.whole_thing != 0)
    {
        fileSystem = fs_fileSystemForVolume(req->volId);
        if (!fileSystem)
        {
            printf("fileSystem not found for vol %llx\n",
                req->volId.whole_thing);
            return NFX_ERR;
        }

        for (i = 0; i < FS_BUFLIST_MAX; i++)
        {
            if (i == FS_BUFLIST_USER)
            {
                j = FS_BUF_CACHE_ITYPE_NONDIR;
                resp->bufCacheStats[i][j].size = fs_bufListGlobal[i].count.usedBufLimit[j];
                resp->bufCacheStats[i][j].used = fileSystem->bufList[i].count.usedBufs[j];
                resp->bufCacheStats[i][j].dirty = fileSystem->bufList[i].count.dirtyBufs[j];
                resp->bufCacheStats[i][j].misses = fileSystem->bufList[i].count.missCount[j];
                resp->bufCacheStats[i][j].hits = fileSystem->bufList[i].count.hitCount[j];
                resp->bufCacheStats[i][j].cows = fileSystem->bufList[i].count.cowCount[j];
#ifdef FS_DYN_CACHE_SIZE
                resp->bufCacheStats[i][j].LWM = fileSystem->bufList[i].count.usedBufLimitLWM[j];
#endif
            }
            else
            {
                for (j = 0; j < FS_BUF_CACHE_ITYPE_TOTAL; j++)
                {
                    resp->bufCacheStats[i][j].size = fs_bufListGlobal[i].count.usedBufLimit[j];
                    resp->bufCacheStats[i][j].used = fileSystem->bufList[i].count.usedBufs[j];
                    resp->bufCacheStats[i][j].dirty = fileSystem->bufList[i].count.dirtyBufs[j];
                    resp->bufCacheStats[i][j].misses = fileSystem->bufList[i].count.missCount[j];
                    resp->bufCacheStats[i][j].hits = fileSystem->bufList[i].count.hitCount[j];
                    resp->bufCacheStats[i][j].cows = fileSystem->bufList[i].count.cowCount[j];
#ifdef FS_DYN_CACHE_SIZE
                    resp->bufCacheStats[i][j].LWM = fileSystem->bufList[i].count.usedBufLimitLWM[j];
#endif
                }
#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
                for (j = 0; j < FS_MAX_META_INODE; j++)
                {
                    resp->bufMissCntType[i][j] = fileSystem->bufList[i].count.missCntType[j];
                    resp->bufUseCntType[i][j] = fileSystem->bufList[i].count.useCntType[j];
                }
#endif
            }
        }
        
        resp->inodeCacheStats[0].size = fs_inodeCache->count.numCachedLimit;
        resp->inodeCacheStats[0].used = fileSystem->inodeCacheCount.numCached;
        resp->inodeCacheStats[0].dirty = fileSystem->inodeCacheCount.numDirty;
        resp->inodeCacheStats[0].hits = fileSystem->inodeCacheCount.hitCount;
        resp->inodeCacheStats[0].misses = fileSystem->inodeCacheCount.missCount;
#ifdef FS_DYN_CACHE_SIZE
        resp->inodeCacheStats[0].LWM = fileSystem->inodeCacheCount.numCachedLimitLWM;
#endif
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
        for (j = 0; j < FS_MAX_META_INODE; j++)
        {
            resp->missCntType[j] = fileSystem->inodeCacheCount.missCntType[j];
        }
#endif
    
        resp->maxReadsOutstanding = fileSystem->ioMaxReadsPending;
        resp->maxWritesOutstanding = fileSystem->ioMaxWritesPending;
        resp->currReadsOutstanding = fileSystem->ioReadsPending;
        resp->currWritesOutstanding = fileSystem->ioWritesPending;
        resp->totalMetaReads = fileSystem->ioTotalMetaReads;
        resp->totalMetaReadAheads = fileSystem->ioTotalMetaReadAheads;
        resp->totalMetaWrites = fileSystem->ioTotalMetaWrites;
        resp->totalUserReads = fileSystem->ioTotalUserReads;
        resp->totalUserReadAheads = fileSystem->ioTotalUserReadAheads;
        resp->totalUserWrites = fileSystem->ioTotalUserWrites;
        resp->totalLogReads = fileSystem->ioTotalLogReads;
        resp->totalLogWrites = fileSystem->ioTotalLogWrites;

        if (req->isClear)
        {
            for (i = 0; i < FS_BUFLIST_MAX; i++)
            {
                NFX_SPIN_LOCK(&fs_bufListGlobal[i].spin);
                for (j = 0; j < FS_BUF_CACHE_ITYPE_MAX; j++)
                {
                    fileSystem->bufList[i].count.missCount[j] = 0;
                    fileSystem->bufList[i].count.hitCount[j] = 0;
                    fileSystem->bufList[i].count.cowCount[j] = 0;
                }
#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
                bzero(fileSystem->bufList[i].count.missCntType, sizeof(fileSystem->bufList[i].count.missCntType));
#endif
                NFX_SPIN_UNLOCK(&fs_bufListGlobal[i].spin);
            }
    
            NFX_SPIN_LOCK(&fs_inodeCache->spin);
            fileSystem->inodeCacheCount.hitCount = 0;
            fileSystem->inodeCacheCount.missCount = 0;
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
            bzero(fileSystem->inodeCacheCount.missCntType, sizeof(fileSystem->inodeCacheCount.missCntType));
#endif
            NFX_SPIN_UNLOCK(&fs_inodeCache->spin);
    
            NFX_SPIN_LOCK(&fileSystem->spin);
            fileSystem->ioMaxReadsPending = 0;
            fileSystem->ioMaxWritesPending = 0;
            fileSystem->ioTotalMetaReads = 0;
            fileSystem->ioTotalMetaReadAheads = 0;
            fileSystem->ioTotalMetaWrites = 0;
            fileSystem->ioTotalUserReads = 0;
            fileSystem->ioTotalUserReadAheads = 0;
            fileSystem->ioTotalUserWrites = 0;
            fileSystem->ioTotalLogReads = 0;
            fileSystem->ioTotalLogWrites = 0;
            NFX_SPIN_UNLOCK(&fileSystem->spin);
        }
    }
    else
    {
        for (i = 0; i < FS_BUFLIST_MAX; i++)
        {
            if (i == FS_BUFLIST_USER)
            {
                j = FS_BUF_CACHE_ITYPE_NONDIR;
                resp->bufCacheStats[i][j].size = fs_bufListGlobal[i].count.usedBufLimit[j];
                resp->bufCacheStats[i][j].used = fs_bufListGlobal[i].count.usedBufs[j];
                resp->bufCacheStats[i][j].dirty = fs_bufListGlobal[i].count.dirtyBufs[j];
                resp->bufCacheStats[i][j].misses = fs_bufListGlobal[i].count.missCount[j];
                resp->bufCacheStats[i][j].hits = fs_bufListGlobal[i].count.hitCount[j];
                resp->bufCacheStats[i][j].cows = fs_bufListGlobal[i].count.cowCount[j];
#ifdef FS_DYN_CACHE_SIZE
                resp->bufCacheStats[i][j].LWM = fs_bufListGlobal[i].count.usedBufLimitLWM[j];
#endif
            }
            else
            {
                for (j = 0; j < FS_BUF_CACHE_ITYPE_TOTAL; j++)
                {
                    resp->bufCacheStats[i][j].size = fs_bufListGlobal[i].count.usedBufLimit[j];
                    resp->bufCacheStats[i][j].used = fs_bufListGlobal[i].count.usedBufs[j];
                    resp->bufCacheStats[i][j].dirty = fs_bufListGlobal[i].count.dirtyBufs[j];
                    resp->bufCacheStats[i][j].misses = fs_bufListGlobal[i].count.missCount[j];
                    resp->bufCacheStats[i][j].hits = fs_bufListGlobal[i].count.hitCount[j];
                    resp->bufCacheStats[i][j].cows = fs_bufListGlobal[i].count.cowCount[j];
#ifdef FS_DYN_CACHE_SIZE
                    resp->bufCacheStats[i][j].LWM = fs_bufListGlobal[i].count.usedBufLimitLWM[j];
#endif
                }
            }
#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
            for (j = 0; j < FS_MAX_META_INODE; j++)
            {
                resp->bufMissCntType[i][j] = fs_bufListGlobal[i].count.missCntType[j];
                resp->bufUseCntType[i][j] = fs_bufListGlobal[i].count.useCntType[j];
            }
#endif
            for (j = 0; j < FS_WB_WAKEUP_MAX; j++)
            {
            	resp->bufWbCount[i][j] = fs_bufWbWakeupCount[i][j];
            }
            resp->userBufWbMaxPerLoop = fs_userBufWbMaxPerLoop;
            resp->metaBufWbMaxPerLoop = fs_metaBufWbMaxPerLoop;
        }
        for (j = 0; j < FS_INODE_CLEANER_MAX; j++)
        {
            resp->inodeCleanerCount[j] = fs_inodeCleanerCount[j];
        }
        resp->inodeCleanerMaxPerLoop = fs_inodeCleanerMaxPerLoop;

        resp->inodeCacheStats[0].size = fs_inodeCache->count.numCachedLimit;
        resp->inodeCacheStats[0].used = fs_inodeCache->count.numCached;
        resp->inodeCacheStats[0].dirty = fs_inodeCache->count.numDirty;
        resp->inodeCacheStats[0].hits = fs_inodeCache->count.hitCount;
        resp->inodeCacheStats[0].misses = fs_inodeCache->count.missCount;
#ifdef FS_DYN_CACHE_SIZE
        resp->inodeCacheStats[0].LWM = fs_inodeCache->count.numCachedLimitLWM;
#endif
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
        for (j = 0; j < FS_MAX_META_INODE; j++)
        {
            resp->missCntType[j] = fs_inodeCache->count.missCntType[j];
        }
#endif
        for (j = 0; j < FS_MAX_META_INODE; j++)
        {
            resp->diskWritebackType[j] = fs_diskWritebackType[j];
            resp->transWriteType[j] = fs_transWriteType[j];
        }
    
        resp->maxReadsOutstanding = fs_ioMaxReadsPending;
        resp->maxWritesOutstanding = fs_ioMaxWritesPending;
        resp->currReadsOutstanding = fs_ioReadsPending;
        resp->currWritesOutstanding = fs_ioWritesPending;
        resp->totalMetaReads = fs_ioTotalMetaReads;
        resp->totalMetaReadAheads = fs_ioTotalMetaReadAheads;
        resp->totalMetaWrites = fs_ioTotalMetaWrites;
        resp->totalUserReads = fs_ioTotalUserReads;
        resp->totalUserReadAheads = fs_ioTotalUserReadAheads;
        resp->totalUserWrites = fs_ioTotalUserWrites;
        resp->totalLogReads = fs_ioTotalLogReads;
        resp->totalLogWrites = fs_ioTotalLogWrites;

#ifdef FS_INODE_CHECK_HASH_DEPTH
        resp->inodeHashMax = FS_ICACHE_HASH_MAX;
        resp->inodeHashUsedEntries = 0;
        resp->inodeHashTotalEntries = 0;
        resp->inodeHashMaxDepth = 0;
        for (i = 0 ; i < FS_ICACHE_HASH_MAX; i++)
        {
            if (fs_inodeCache->depth[i])
            {
                resp->inodeHashTotalEntries += fs_inodeCache->depth[i];
                resp->inodeHashUsedEntries += 1;
                if (resp->inodeHashMaxDepth < fs_inodeCache->depth[i])
                {
                    resp->inodeHashMaxDepth = fs_inodeCache->depth[i];
                }
            }
        }
        resp->nbrLookups = fs_inodeCache->nbrLookups;
        resp->nbrScanned = fs_inodeCache->nbrScanned;
#endif

#ifdef FS_BUF_BLK_CHECK_HASH_DEPTH
        resp->UserBufBlkHashMax = FS_BUFLIST_HASH_MAX;
        resp->UserBufBlkHashUsedEntries = 0;
        resp->UserBufBlkHashTotalEntries = 0;
        resp->UserBufBlkHashMaxDepth = 0;
        for (i = 0 ; i < FS_BUFLIST_HASH_MAX; i++)
        {
            if (fs_bufListGlobal[FS_BUFLIST_USER].blkDepth[i])
            {
                resp->UserBufBlkHashTotalEntries += fs_bufListGlobal[FS_BUFLIST_USER].blkDepth[i];
                resp->UserBufBlkHashUsedEntries += 1;
                if (resp->UserBufBlkHashMaxDepth < fs_bufListGlobal[FS_BUFLIST_USER].blkDepth[i])
                {
                    resp->UserBufBlkHashMaxDepth = fs_bufListGlobal[FS_BUFLIST_USER].blkDepth[i];
                }
            }
        }
        resp->MetaBufBlkHashMax = FS_BUFLIST_HASH_MAX;
        resp->MetaBufBlkHashUsedEntries = 0;
        resp->MetaBufBlkHashTotalEntries = 0;
        resp->MetaBufBlkHashMaxDepth = 0;
        for (i = 0 ; i < FS_BUFLIST_HASH_MAX; i++)
        {
            if (fs_bufListGlobal[FS_BUFLIST_META].blkDepth[i])
            {
                resp->MetaBufBlkHashTotalEntries += fs_bufListGlobal[FS_BUFLIST_META].blkDepth[i];
                resp->MetaBufBlkHashUsedEntries += 1;
                if (resp->MetaBufBlkHashMaxDepth < fs_bufListGlobal[FS_BUFLIST_META].blkDepth[i])
                {
                    resp->MetaBufBlkHashMaxDepth = fs_bufListGlobal[FS_BUFLIST_META].blkDepth[i];
                }
            }
        }
#endif

#ifdef FS_BUF_PAGE_CHECK_HASH_DEPTH
        resp->UserBufPageHashMax = FS_BUFLIST_HASH_MAX;
        resp->UserBufPageHashUsedEntries = 0;
        resp->UserBufPageHashTotalEntries = 0;
        resp->UserBufPageHashMaxDepth = 0;
        for (i = 0 ; i < FS_BUFLIST_HASH_MAX; i++)
        {
            if (fs_bufListGlobal[FS_BUFLIST_USER].pageDepth[i])
            {
                resp->UserBufPageHashTotalEntries += fs_bufListGlobal[FS_BUFLIST_USER].pageDepth[i];
                resp->UserBufPageHashUsedEntries += 1;
                if (resp->UserBufPageHashMaxDepth < fs_bufListGlobal[FS_BUFLIST_USER].pageDepth[i])
                {
                    resp->UserBufPageHashMaxDepth = fs_bufListGlobal[FS_BUFLIST_USER].pageDepth[i];
                }
            }
        }
        resp->MetaBufPageHashMax = FS_BUFLIST_HASH_MAX;
        resp->MetaBufPageHashUsedEntries = 0;
        resp->MetaBufPageHashTotalEntries = 0;
        resp->MetaBufPageHashMaxDepth = 0;
        for (i = 0 ; i < FS_BUFLIST_HASH_MAX; i++)
        {
            if (fs_bufListGlobal[FS_BUFLIST_META].pageDepth[i])
            {
                resp->MetaBufPageHashTotalEntries += fs_bufListGlobal[FS_BUFLIST_META].pageDepth[i];
                resp->MetaBufPageHashUsedEntries += 1;
                if (resp->MetaBufPageHashMaxDepth < fs_bufListGlobal[FS_BUFLIST_META].pageDepth[i])
                {
                    resp->MetaBufPageHashMaxDepth = fs_bufListGlobal[FS_BUFLIST_META].pageDepth[i];
                }
            }
        }
#endif

#ifdef FS_BUF_INUM_CHECK_HASH_DEPTH
        resp->UserBufInumHashMax = FS_BUFLIST_HASH_MAX;
        resp->UserBufInumHashUsedEntries = 0;
        resp->UserBufInumHashTotalEntries = 0;
        resp->UserBufInumHashMaxDepth = 0;
        for (i = 0 ; i < FS_BUFLIST_HASH_MAX; i++)
        {
            if (fs_bufListGlobal[FS_BUFLIST_USER].inumDepth[i])
            {
                resp->UserBufInumHashTotalEntries += fs_bufListGlobal[FS_BUFLIST_USER].inumDepth[i];
                resp->UserBufInumHashUsedEntries += 1;
                if (resp->UserBufInumHashMaxDepth < fs_bufListGlobal[FS_BUFLIST_USER].inumDepth[i])
                {
                    resp->UserBufInumHashMaxDepth = fs_bufListGlobal[FS_BUFLIST_USER].inumDepth[i];
                }
            }
        }
        resp->MetaBufInumHashMax = FS_BUFLIST_HASH_MAX;
        resp->MetaBufInumHashUsedEntries = 0;
        resp->MetaBufInumHashTotalEntries = 0;
        resp->MetaBufInumHashMaxDepth = 0;
        for (i = 0 ; i < FS_BUFLIST_HASH_MAX; i++)
        {
            if (fs_bufListGlobal[FS_BUFLIST_META].inumDepth[i])
            {
                resp->MetaBufInumHashTotalEntries += fs_bufListGlobal[FS_BUFLIST_META].inumDepth[i];
                resp->MetaBufInumHashUsedEntries += 1;
                if (resp->MetaBufInumHashMaxDepth < fs_bufListGlobal[FS_BUFLIST_META].inumDepth[i])
                {
                    resp->MetaBufInumHashMaxDepth = fs_bufListGlobal[FS_BUFLIST_META].inumDepth[i];
                }
            }
        }
#endif

        if (req->isClear)
        {
            for (i = 0; i < FS_BUFLIST_MAX; i++)
            {
                NFX_SPIN_LOCK(&fs_bufListGlobal[i].spin);
                for (j = 0; j < FS_BUF_CACHE_ITYPE_MAX; j++)
                {
                    fs_bufListGlobal[i].count.missCount[j] = 0;
                    fs_bufListGlobal[i].count.hitCount[j] = 0;
                    fs_bufListGlobal[i].count.cowCount[j] = 0;
                }
#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
                bzero(fs_bufListGlobal[i].count.missCntType, sizeof(fs_bufListGlobal[i].count.missCntType));
#endif
        
                NFX_SPIN_UNLOCK(&fs_bufListGlobal[i].spin);
            }
    
            NFX_SPIN_LOCK(&fs_bufListGlobal[FS_BUFLIST_META].spin);
            NFX_SPIN_LOCK(&fs_bufListGlobal[FS_BUFLIST_USER].spin);
            bzero(fs_bufWbWakeupCount, sizeof(fs_bufWbWakeupCount));
            fs_userBufWbMaxPerLoop = 0;
            fs_metaBufWbMaxPerLoop = 0;
            NFX_SPIN_UNLOCK(&fs_bufListGlobal[FS_BUFLIST_META].spin);
            NFX_SPIN_UNLOCK(&fs_bufListGlobal[FS_BUFLIST_USER].spin);

            NFX_SPIN_LOCK(&fs_inodeCache->spin);
            fs_inodeCache->count.hitCount = 0;
            fs_inodeCache->count.missCount = 0;
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
            bzero(fs_inodeCache->count.missCntType, sizeof(fs_inodeCache->count.missCntType));
#endif
            bzero(fs_diskWritebackType, sizeof(fs_diskWritebackType));
            bzero(fs_transWriteType, sizeof(fs_transWriteType));
            bzero(fs_inodeCleanerCount, sizeof(fs_inodeCleanerCount));
            fs_inodeCleanerMaxPerLoop = 0;
#ifdef FS_INODE_CHECK_HASH_DEPTH
            fs_inodeCache->nbrLookups = 0;
            fs_inodeCache->nbrScanned = 0;
#endif
            NFX_SPIN_UNLOCK(&fs_inodeCache->spin);
    
            NFX_SPIN_LOCK(&fs_ioSpin);
            fs_ioMaxReadsPending = 0;
            fs_ioMaxWritesPending = 0;
            fs_ioTotalMetaReads = 0;
            fs_ioTotalMetaReadAheads = 0;
            fs_ioTotalMetaWrites = 0;
            fs_ioTotalUserReads = 0;
            fs_ioTotalUserReadAheads = 0;
            fs_ioTotalUserWrites = 0;
            fs_ioTotalLogReads = 0;
            fs_ioTotalLogWrites = 0;
            NFX_SPIN_UNLOCK(&fs_ioSpin);
        }
    }

    context->respSize = sizeof(fs_MsgCachestatsResp);

    return NFX_OK;

} /* fs_cachestatsThread */

/*-----------------------------------------------------------------------------
 * fs_dnlcstatsThread
 */
int32
fs_dnlcstatsThread(
    fs_Context      *context)
{
    fs_MsgDnlcstatsResp  *resp = (fs_MsgDnlcstatsResp *) context->respHdr;
    fs_MsgDnlcstatsReq   *req = (fs_MsgDnlcstatsReq *) context->reqHdr;
    fs_FileSystem        *fileSystem;

    resp->volId = req->volId;

    if (req->volId.whole_thing != 0)
    {
        fileSystem = fs_fileSystemForVolume(req->volId);
        if (!fileSystem)
        {
            printf("fileSystem not found for vol %llx\n",
                req->volId.whole_thing);
            return NFX_ERR;
        }

        resp->stats = fileSystem->dnlcStats;
        if (req->isClear)
        {
            fileSystem->dnlcStats.hits = 0;
            fileSystem->dnlcStats.hitsNegative = 0;
            fileSystem->dnlcStats.missNotFound = 0;
            fileSystem->dnlcStats.missTooBig = 0;
            fileSystem->dnlcStats.missInode = 0;
            fileSystem->dnlcStats.inserts = 0;
            fileSystem->dnlcStats.removes = 0;
            fileSystem->dnlcStats.setNegatives = 0;
            fileSystem->dnlcStats.setPositives = 0;
        }
        if (req->isFlush)
        {
            fs_dnlcDestroyVolume(fileSystem);
        }
    }
    else
    {
        int i;

        resp->stats = fs_dnlc.stats;
        resp->usedBuckets = 0;
        resp->maxDepth = 0;
        for (i = 0 ; i < FS_DNLC_HASH_MAX; i++)
        {
            if (fs_dnlc.depth[i])
            {
                resp->usedBuckets += 1;
                if (resp->maxDepth < fs_dnlc.depth[i])
                {
                    resp->maxDepth = fs_dnlc.depth[i];
                }
            }
        }
        if (req->isClear)
        {
            fs_dnlc.stats.hits = 0;
            fs_dnlc.stats.hitsNegative = 0;
            fs_dnlc.stats.missNotFound = 0;
            fs_dnlc.stats.missTooBig = 0;
            fs_dnlc.stats.missInode = 0;
            fs_dnlc.stats.inserts = 0;
            fs_dnlc.stats.removes = 0;
            fs_dnlc.stats.setNegatives = 0;
            fs_dnlc.stats.setPositives = 0;
        }
    }

    context->respSize = sizeof(fs_MsgDnlcstatsResp);

    return NFX_OK;

} /* fs_dnlcstatsThread */


/*-----------------------------------------------------------------------------
 * fs_snapdiagThread
 */
int32
fs_snapdiagThread(
    fs_Context      *context)
{
    fs_FileSystem       *fileSystem;
    evm_volId_t         volId;
    fs_MsgSnapdiagReq   *req = (fs_MsgSnapdiagReq *) context->reqHdr;
    fs_MsgSnapdiagResp  *resp = (fs_MsgSnapdiagResp *) context->respHdr;

    volId = req->volId;
    if (volId.whole_thing) {
        fileSystem = fs_fileSystemForVolume(volId);
        context->volId = volId;
        context->fileSystem = fileSystem;
    }

    // dump out the allocated ref bytes
    if (req->cmd == FS_SNAP_DIAG_GET_REFBYTES)
        fs_refBytePrintAllocated(context);

    // dump out the snapbits file
    if (req->cmd == FS_SNAP_DIAG_GET_SNAPBITS) {
        if (fs_getSnapNum(context, req->snapDirName) == -1) {
            uint32 snap = atoi(req->snapDirName) + FS_MAX_USER_SNAPSHOTS;
            // try by snapnum
            if ((snap >= 1) && (snap < FS_MAX_SNAPSHOTS)) {
                fs_snapDiagBitsFile(context, NULL, snap);
            } else {
                printf("snapdiag thread: invalid snap %d\n", snap);
            }
        } else {
            fs_snapDiagBitsFile(context, req->snapDirName, 0);
        }
    }

    // get snap stats
    if (req->cmd == FS_SNAP_DIAG_GET_STATS)
        fs_snapDiagGetStats(context, &resp->stats);

#ifdef FS_DEBUG_SNAP_CRASH
    // force a crash at phase during the next snap operation
    if (req->cmd == FS_SNAP_DIAG_FORCE_CRASH)
        fs_snapPhaseToCrash = req->phase;
#endif

    context->respSize = sizeof(fs_MsgSnapdiagResp);
    resp->cmd = req->cmd;
    return NFX_OK;

} /* fs_snapdiagThread */

#ifndef FS_STANDALONE
void fs_memCheckLeaks(void)
{
}
#endif

/*-----------------------------------------------------------------------------
 * fs_memstatsThread
 */

#ifndef FS_STANDALONE
#include "../sm-eee/eee.h"
#endif // FS_STANDALONE



int32
fs_memstatsThread(
    fs_Context      *context)
{
    fs_MsgMemstatsResp  *resp = (fs_MsgMemstatsResp *) context->respHdr;
    int32               how, type;
	uint32	            num;

    fs_memCheckLeaks();

    NFX_SPIN_LOCK(&fs_ramStatsSpinId);
    for (how = 0; how < FS_RAM_STATS_HOW_MAX; how++)
    {
        for (type = 0; type < FS_RAM_TYPE_MAX; type++)
        {
            resp->ramStats[how][type] = fs_ramStats[how][type];
        }
    }
    NFX_SPIN_UNLOCK(&fs_ramStatsSpinId);

#ifndef FS_STANDALONE

    resp->total_flex_size =  eee_ramTotalFlexSize();
    resp->total_flex_used =  eee_ramTotalFlexUsed();
    resp->total_stack_size = 0;
    resp->total_stack_used = 0;
    resp->total_cont_size = eee_ramTotalHeapSize() - eee_ramTotalFlexSize();
    resp->total_cont_used = resp->total_cont_size - (eee_ramTotalHeapAvail() - eee_ramTotalFlexAvail());

    resp->eeeBufs[EEE_BUF_SMALL] = eee_getTotalNbrOfBufs(EEE_BUF_SMALL);
    resp->eeeAvailableBufs[EEE_BUF_SMALL] = eee_getAvailableNbrOfBufs(EEE_BUF_SMALL);
    resp->eeeLWMBufs[EEE_BUF_SMALL] = eee_getLWMBufs(EEE_BUF_SMALL);

    resp->eeeBufs[EEE_BUF_LARGE] = eee_getTotalNbrOfBufs(EEE_BUF_LARGE);
    resp->eeeAvailableBufs[EEE_BUF_LARGE] = eee_getAvailableNbrOfBufs(EEE_BUF_LARGE);
    resp->eeeLWMBufs[EEE_BUF_LARGE] = eee_getLWMBufs(EEE_BUF_LARGE);

    resp->eeeBufs[EEE_BUF_ULTRA] = eee_getTotalNbrOfBufs(EEE_BUF_ULTRA);
    resp->eeeAvailableBufs[EEE_BUF_ULTRA] = eee_getAvailableNbrOfBufs(EEE_BUF_ULTRA);
    resp->eeeLWMBufs[EEE_BUF_ULTRA] = eee_getLWMBufs(EEE_BUF_ULTRA);
#ifdef LARGE_MEMORY
    resp->eeeBufs[EEE_BUF_MGMT] = eee_getTotalNbrOfBufs(EEE_BUF_MGMT);
    resp->eeeAvailableBufs[EEE_BUF_MGMT] = eee_getAvailableNbrOfBufs(EEE_BUF_MGMT);
    resp->eeeLWMBufs[EEE_BUF_MGMT] = eee_getLWMBufs(EEE_BUF_MGMT);
#else
    resp->eeeBufs[EEE_BUF_MGMT] = 0;
#endif


	num = (address_t)eee.buf_end[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_SMALL] -
		((address_t)eee.buf_base[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_SMALL]);
    num = num / EEE_BUFSIZE_SMALL;
    resp->eeeTxrxBufs[EEE_BUF_SMALL] = num;
    resp->eeeTxrxAvailableBufs[EEE_BUF_SMALL] = sbm_sac_info->num_buf_small_avail;
    resp->eeeTxrxLWMBufs[EEE_BUF_SMALL] = sbm_sac_info->num_buf_LWM_small;

	num = (address_t)eee.buf_end[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_LARGE] -
		((address_t)eee.buf_base[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_LARGE]);
    num = num / EEE_BUFSIZE_LARGE;
    resp->eeeTxrxBufs[EEE_BUF_LARGE] = num;
    resp->eeeTxrxAvailableBufs[EEE_BUF_LARGE] = sbm_sac_info->num_buf_large_avail;
    resp->eeeTxrxLWMBufs[EEE_BUF_LARGE] = sbm_sac_info->num_buf_LWM_large;

	num = (address_t)eee.buf_end[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_ULTRA] -
		((address_t)eee.buf_base[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_ULTRA]);
    num = num / EEE_BUFSIZE_ULTRA;
    resp->eeeTxrxBufs[EEE_BUF_ULTRA] = num;
    resp->eeeTxrxAvailableBufs[EEE_BUF_ULTRA] = sbm_sac_info->num_buf_ultra_avail;
    resp->eeeTxrxLWMBufs[EEE_BUF_ULTRA] = sbm_sac_info->num_buf_LWM_ultra;

#ifdef LARGE_MEMORY
    resp->eeeTxrxBufs[EEE_BUF_MGMT] = 0;
    resp->eeeTxrxAvailableBufs[EEE_BUF_MGMT] = 0;
    resp->eeeTxrxLWMBufs[EEE_BUF_MGMT] = 0; 
#else
    resp->eeeTxrxBufs[EEE_BUF_MGMT] = 0;
#endif

#endif  // FS_STANDALONE
    
    context->respSize = sizeof(fs_MsgMemstatsResp);

    return NFX_OK;

} /* fs_memstatsThread */


/** 
 * fs_reqstatsThread
 */
int32
fs_reqstatsThread(
    fs_Context      *context)
{
    fs_MsgReqstatsResp  *resp = (fs_MsgReqstatsResp *) context->respHdr;
    fs_MsgReqstatsReq   *req = (fs_MsgReqstatsReq *) context->reqHdr;

    if (context->fileSystem)
    {
        NFX_SPIN_LOCK(&context->fileSystem->fs_statsReqRcvdSpinId);
        resp->rcvd = context->fileSystem->recvReqStats;
        resp->reply = context->fileSystem->replyStats;
        if (req->isClear)
        {
            bzero(&context->fileSystem->recvReqStats, sizeof(context->fileSystem->recvReqStats));
            bzero(&context->fileSystem->replyStats, sizeof(context->fileSystem->replyStats));
        }
        NFX_SPIN_UNLOCK(&context->fileSystem->fs_statsReqRcvdSpinId);
    }
    else
    {
        NFX_SPIN_LOCK(&fs_statsReqRcvdSpinId);
        resp->rcvd = fs_recvReqStats;
        resp->reply = fs_replyStats;
        if (req->isClear)
        {
            bzero(&fs_recvReqStats, sizeof(fs_recvReqStats));
            bzero(&fs_replyStats, sizeof(fs_replyStats));
        }
        NFX_SPIN_UNLOCK(&fs_statsReqRcvdSpinId);
    }
    
    resp->volId = req->volId;
    resp->isVerbose = req->isVerbose;

    context->respSize = sizeof(fs_MsgReqstatsResp);

    return NFX_OK;

} /* fs_reqstatsThread */


/** 
 * fs_logstatsThread
 */
int32
fs_logstatsThread(
    fs_Context      *context)
{
    fs_MsgLogstatsResp  *resp = (fs_MsgLogstatsResp *) context->respHdr;
    fs_MsgLogstatsReq   *req = (fs_MsgLogstatsReq *) context->reqHdr;
    fs_LogFsRoot        *logRoot = context->fileSystem->logRoot;

    resp->logDevType = logRoot->logDevType;
    resp->logSize = context->fileSystem->logSize;
    resp->head = logRoot->head;
    resp->currLen = logRoot->currLen;
    resp->tail = logRoot->tail;
    resp->logStats = logRoot->logStats;
    resp->isVerbose = req->isVerbose;

    if (req->isClear)
    {
        bzero(&logRoot->logStats, sizeof(fs_LogStats));
    }
    
    switch (logRoot->logDevType) 
    {
    case FS_LOG_DEVTYPE_SELF:
    case FS_LOG_DEVTYPE_VOLID:
        fs_vl_getStats (context, req->isClear, &resp->u.vollog);
        break;
    default:
        break;
    }
    
    context->respSize = sizeof(fs_MsgLogstatsResp);

    return NFX_OK;

} /* fs_logstatsThread */


/**
 * fs_nfsstatsThread
 */
int32
fs_nfsstatsThread(
    fs_Context      *context)
{   
    fs_MsgNfsstatsResp  *resp = (fs_MsgNfsstatsResp *) context->respHdr;
    fs_MsgNfsstatsReq   *req = (fs_MsgNfsstatsReq *) context->reqHdr;
    fs_NfsStats         *nfsStats;
    uint32              *currentActiveReqs;
    fs_YieldStats       *yieldStats;
    fs_RecvNfsReqStats  *rcvdStats;
    nfx_spinlock_t      *spinp;
    
    if (context->fileSystem)
    {
        nfsStats = &context->fileSystem->nfsStats;
        currentActiveReqs = &context->fileSystem->currentActiveReqs;
        yieldStats = &context->fileSystem->yieldStats;
        rcvdStats = (fs_RecvNfsReqStats*)&context->fileSystem->recvReqStats;
        spinp = &context->fileSystem->fs_statsReqRcvdSpinId;
    }
    else
    {
        nfsStats = &fs_nfsStats;
        currentActiveReqs = &fs_currentActiveReqs;
        yieldStats = &fs_yieldStats;
        rcvdStats = (fs_RecvNfsReqStats*)&fs_recvReqStats;
        spinp = &fs_statsReqRcvdSpinId;
    }

    NFX_SPIN_LOCK(spinp);
    resp->nfsStats = *nfsStats;
    resp->currentActiveReqs = *currentActiveReqs;
    resp->yieldStats = *yieldStats;
    memcpy(&resp->recvNfsReqStats, rcvdStats, sizeof(fs_RecvNfsReqStats));

    if (req->isClear)
    {
        bzero(nfsStats, sizeof(fs_NfsStats));
        bzero(yieldStats, sizeof(fs_YieldStats));
        // watchout: req rcvd also needs to be cleared for yield averages
        bzero(rcvdStats, sizeof(fs_RecvReqStats));
    }
    NFX_SPIN_UNLOCK(spinp);

    resp->volId = req->volId;
    resp->isVerbose = req->isVerbose;
	resp->count_per_usec = count_per_usec;

    context->respSize = sizeof(fs_MsgNfsstatsResp);

    return NFX_OK;

} /* fs_nfsstatsThread */

/**
 * fs_errstatsThread
 */
int32
fs_errstatsThread(
    fs_Context      *context)
{   
    fs_MsgErrstatsResp  *resp = (fs_MsgErrstatsResp *) context->respHdr;
    fs_MsgErrstatsReq   *req = (fs_MsgErrstatsReq *) context->reqHdr;
    fs_ErrStats         *errs;
    nfx_spinlock_t      *spinp;
    
    if (context->fileSystem)
    {
        errs = &context->fileSystem->errStats;
        spinp = &context->fileSystem->fs_statsReqRcvdSpinId;
    }
    else
    {
        errs = &fs_errStats;
        spinp = &fs_errStatsSpinId;
    }
    NFX_SPIN_LOCK(spinp);
    resp->errStats = *errs;
    if (req->isClear)
    {
        bzero(errs, sizeof(fs_ErrStats));
    }
    NFX_SPIN_UNLOCK(spinp);

    context->respSize = sizeof(fs_MsgErrstatsResp);

    return NFX_OK;

} /* fs_errstatsThread */


/**
 * fs_yieldstatsThread
 */
int32
fs_yieldstatsThread(
    fs_Context      *context)
{
    fs_MsgYieldstatsResp    *resp = (fs_MsgYieldstatsResp *) context->respHdr;
    fs_MsgYieldstatsReq     *req = (fs_MsgYieldstatsReq *) context->reqHdr;
    fs_YieldStats           *yieldStats;
    fs_RecvNfsReqStats      *rcvdStats;
    uint32                  *currentActiveReqs;
    uint32                  *maxActiveReqs;
    uint32                  *maxNbrContexts;
    nfx_spinlock_t          *spinp;
	extern uint64	        ht_bytes_tx;
	extern uint64			ht_bytes_rx;

    // if (req->volId.whole_thing)
    if (context->fileSystem)
    {
        rcvdStats = (fs_RecvNfsReqStats*)&context->fileSystem->recvReqStats;
        currentActiveReqs = &context->fileSystem->currentActiveReqs;
        maxActiveReqs = &context->fileSystem->maxActiveReqs;
        maxNbrContexts = &fs_maxNbrOfContexts;
        yieldStats = &context->fileSystem->yieldStats;
        spinp = &context->fileSystem->fs_statsReqRcvdSpinId;
    }
    else
    {
        rcvdStats = (fs_RecvNfsReqStats*)&fs_recvReqStats;
        currentActiveReqs = &fs_currentActiveReqs;
        maxActiveReqs = &fs_maxActiveReqs;
        maxNbrContexts = &fs_maxNbrOfContexts;
        yieldStats = &fs_yieldStats;
        spinp = &fs_statsReqRcvdSpinId;
    }
    

    NFX_SPIN_LOCK(spinp);

    resp->currentActiveReqs = *currentActiveReqs;
    resp->maxActiveReqs = *maxActiveReqs;
    resp->maxNbrContexts = *maxNbrContexts;
    resp->yieldStats = *yieldStats;
    memcpy(&resp->recvNfsReqStats, rcvdStats, sizeof(fs_RecvNfsReqStats));
    resp->isAverage  = req->isAverage;
    resp->showIO  = req->showIO;
    resp->showRestore  = req->showRestore;
    resp->showDaemon  = req->showDaemon;

#ifdef FS_STANDALONE
    resp->ht_bytes_tx = 0;
	resp->ht_bytes_rx = 0;
#else // FS_STANDALONE
    resp->ht_bytes_tx = ht_bytes_tx;
	resp->ht_bytes_rx = ht_bytes_rx;
    ht_bytes_tx = 0;
#endif // FS_STANDALONE
    
#ifdef FS_VOLLOG_WAIT_FOR_ONGOING_TRANS
    if (context->fileSystem)
    {
        resp->nbrOngoingTrans = context->fileSystem->nbrOngoingTrans;
        resp->maxOngoingTrans = context->fileSystem->maxOngoingTrans;
    }
    else
#endif
    {
        resp->nbrOngoingTrans = 0;
        resp->maxOngoingTrans = 0;
    }

    if (req->isClear)
    {
        bzero(yieldStats, sizeof(fs_YieldStats));
        // watchout: req rcvd also needs to be cleared for yield averages
        bzero(rcvdStats, sizeof(fs_RecvReqStats));
        *maxActiveReqs = 0;
        *maxNbrContexts = 0;
#ifdef FS_VOLLOG_WAIT_FOR_ONGOING_TRANS
        if (context->fileSystem)
        {
            context->fileSystem->maxOngoingTrans = 0;
        }
#endif        
    }
    NFX_SPIN_UNLOCK(spinp);

    resp->volId = req->volId;
	resp->count_per_usec = count_per_usec;

    context->respSize = sizeof(fs_MsgYieldstatsResp);

    return NFX_OK;

} /* fs_yieldstatsThread */


/**
 * fs_lockstatsThread
 */
int32
fs_lockstatsThread(
    fs_Context      *context)
{
    fs_MsgLockstatsResp     *resp = (void *) context->respHdr;
    fs_MsgLockstatsReq      *req = (void *) context->reqHdr;
    fs_FileSystem           *fs = context->fileSystem;
    fs_Lock                 *lockp[FS_LOCKSTATS_MAX];
    int32                   i;
    
#ifdef FS_DEBUG_CHECK_SHAREDLOCK
    resp->maxSharedLockListSize = fs_lockMaxSharedLockListSize;
    if (req->isClear)
    {
        fs_lockMaxSharedLockListSize = 0;
    }
#else
    resp->maxSharedLockListSize = 0;
#endif
        
    if (context->fileSystem)
    {
        // Return info about some specific locks
        // for the specified filesystem.

        bzero(lockp, sizeof lockp);
        lockp[0] = &fs->modifyLock;
        lockp[1] = &fs->accessLock;
        lockp[2] = &fs->freezeLock;
        lockp[3] = &fs->dumpContextsLock;
        lockp[4] = &fs->restoreContextsLock;
        FS_ASSERT(5 < FS_LOCKSTATS_MAX)

        // TODO - add fileset locks
    
        if (req->isClear)
        {
            for (i = 0; i < FS_LOCKSTATS_MAX; i++)
            {
                if (lockp[i] != NULL)
                {
                    NFX_SPIN_LOCK(&lockp[i]->spin);
                    lockp[i]->waitTime = *(eee_rtcZero());
                    lockp[i]->totalWaits = 0;
                    NFX_SPIN_UNLOCK(&lockp[i]->spin);
                }
            }
        }
        else
        {
            for (i = 0; i < FS_LOCKSTATS_MAX; i++)
            {
                if (lockp[i] != NULL)
                {
                    NFX_SPIN_LOCK(&lockp[i]->spin);
                    resp->u.waitTime[i] = lockp[i]->waitTime;
                    if (req->isAverage && lockp[i]->totalWaits)
                    {
						eee_rtcDiv(&(resp->u.waitTime[i]), lockp[i]->totalWaits);
                    }
                    NFX_SPIN_UNLOCK(&lockp[i]->spin);
                }
            }
        }
    
    }
    else
    {
        // Return info about lock contention between
        // different requests/daemons and lock types.

        NFX_SPIN_LOCK(&fs_fileSystemTableLock);

        if (req->lockReqType < FS_LOCK_OWNER_MAX)
        {
            int32   lockType, owner;

            for (lockType = 0; lockType < FS_LOCK_TYPE_MAX; lockType++)
            {
                for (owner = 0; owner < FS_LOCK_OWNER_MAX; owner++)
                {
                    resp->u.waitStats[lockType][owner] =
                        fs_lockWaitStats[req->lockReqType][lockType][owner];

                    if (req->isClear)
                    {
                      fs_lockWaitStats[req->lockReqType][lockType][owner] = 0;
                    }
                }
            }
            resp->lockReqType = req->lockReqType;
        }
        else
        {
            resp->u.lockStatBuf = fs_lockStatBuf;
            resp->lockReqType = req->lockReqType;
            if (req->isClear)
            {
                bzero(&fs_lockStatBuf, sizeof(fs_lockStatBuf));
            }
        }
        
        NFX_SPIN_UNLOCK(&fs_fileSystemTableLock);
    }

    resp->volId = req->volId;
	resp->count_per_usec = count_per_usec;
    context->respSize = sizeof(fs_MsgLockstatsResp);

    return NFX_OK;

} /* fs_lockstatsThread */


#ifdef NOTDEF
    // play with file i/o lib
    {
        int32   rc, fromFd, toFd;
        char    *buf;
        char    toName[FS_NAME_MAX + 32];
        char    fromName[FS_NAME_MAX + 32];

        buf = eee_ramAlloc(64*1024);
        if (!buf)
        {
            printf("failed to get mem for buf\n");
            return NFX_OK;
        }

        strcpy(fromName, context->fileSystem->super->mountName);
        strcat(fromName, ":from");
        strcpy(toName, context->fileSystem->super->mountName);
        strcat(toName, ":to");

        fromFd = file_open(fromName);
        if (fromFd < 0)
        {
            printf("error opening %s\n", fromName);
            eee_ramDealloc(buf);
            return NFX_OK;
        }

        toFd = file_create(toName, 0666);
        if (toFd < 0)
        {
            printf("error creating %s\n", toName);
            eee_ramDealloc(buf);
            return NFX_OK;
        }

        rc = file_read(fromFd, buf, 64*1024);
        //printf("read %d bytes from %s\n", rc, fromName);
        rc = file_write(toFd, buf, rc);
        //printf("wrote %d bytes to %s\n", rc, toName);

        file_close(fromFd);
        file_close(toFd);

        eee_ramDealloc(buf);
    }
#endif


/*-----------------------------------------------------------------------------
 * fs_incoreTuneThread
 */
int32
fs_incoreTuneThread(
    fs_Context      *context)
{
    fs_MsgIncoreTuneResp     *resp = (void *) context->respHdr;
    fs_MsgIncoreTuneReq      *req = (void *) context->reqHdr;

    if (req->fieldId >= 0)
    {
        switch (req->fieldId)
        {
        case FS_TUNE_inodePctAvailLoWtrMk:
            fs_incoreTune->inodePctAvailLoWtrMk = (uint32)req->value;
            break;
        case FS_TUNE_inodePctAvailHiWtrMk:
            fs_incoreTune->inodePctAvailHiWtrMk = (uint32)req->value;
            break;
        case FS_TUNE_logPctUseLoWtrMk:
            fs_incoreTune->logPctUseLoWtrMk = (uint32)req->value;
            break;
        case FS_TUNE_logPctUseHiWtrMk:
            fs_incoreTune->logPctUseHiWtrMk = (uint32)req->value;
            break;
        case FS_TUNE_bufPctAvailLoWtrMk:
            fs_incoreTune->bufPctAvailLoWtrMk = (uint32)req->value;
            break;
        case FS_TUNE_bufPctAvailHiWtrMk:
            fs_incoreTune->bufPctAvailHiWtrMk = (uint32)req->value;
            break;
        case FS_TUNE_memPctAvailMin:
            fs_incoreTune->memPctAvailMin = (uint32)req->value;
            break;
        case FS_TUNE_bufWritesPerEvent:
            fs_incoreTune->bufWritesPerEvent = (uint32)req->value;
            break;
        case FS_TUNE_writesPendingMax:
            fs_incoreTune->writesPendingMax = (uint32)req->value;
            break;
        case FS_TUNE_perFsIoPendingMax:
            fs_incoreTune->perFsIoPendingMax = (uint32)req->value;
            break;
        case FS_TUNE_inodeCleanPerPass:
            fs_incoreTune->inodeCleanPerPass = (uint32)req->value; 
            break;
        case FS_TUNE_activeTransToWait:
            fs_incoreTune->activeTransToWait = (uint32)req->value; 
            break;
        case FS_TUNE_maxCpuUsageUsec:
            eee_usecToRTC((uint64)req->value,
						  &fs_incoreTune->maxCpuUsageRTC);
            break;
        case FS_TUNE_inodePctMem:
            fs_incoreTune->inodePctMem = (uint32)req->value;
            fs_inodeSetCacheLimit();
            break;
        case FS_TUNE_wbSleepMsecs:
            fs_incoreTune->wbSleepMsecs = (uint32)req->value;
            break;
        case FS_TUNE_readdirMaxSpace:
            fs_incoreTune->readdirMaxSpace = (uint32)req->value;
            break;
        case FS_TUNE_userDirtyThreshold:
            fs_incoreTune->userDirtyThreshold = (uint32)req->value;
            break;
        case FS_TUNE_metaDirtyThreshold:
            fs_incoreTune->metaDirtyThreshold = (uint32)req->value;
            break;
        case FS_TUNE_enableReadAhead:
            fs_incoreTune->enableReadAhead = (uint32)req->value;
            break;
        case FS_TUNE_readAheadMaxSize:
            fs_incoreTune->readAheadMaxSize = (uint32)req->value;
            break;
        case FS_TUNE_readAheadDefaultSize:
            fs_incoreTune->readAheadDefaultSize = (uint32)req->value;
            break;
        case FS_TUNE_enableAtimeUpdate:
            fs_incoreTune->enableAtimeUpdate = (uint32)req->value;
            break;
        case FS_TUNE_vollogIoMax:
            fs_incoreTune->vollogIoMax = (uint32)req->value;
            break;
        case FS_TUNE_vollogLoopBack:
            fs_incoreTune->vollogLoopBack = (uint32)req->value;
            break;
        case FS_TUNE_sacMinAvailable:
            fs_incoreTune->sacMinAvailable = (uint32)req->value;
            break;
        case FS_TUNE_sacFreeCount:
            fs_incoreTune->sacFreeCount = (uint32)req->value;
            break;
        case FS_TUNE_ultraMinAvailable:
            fs_incoreTune->ultraMinAvailable = (uint32)req->value;
            break;
        case FS_TUNE_memMinAvailable:
            fs_incoreTune->memMinAvailable = (uint32)req->value;
            break;
        case FS_TUNE_bufFreeCount:
            fs_incoreTune->bufFreeCount = (uint32)req->value;
            break;
        case FS_TUNE_inodeFreeCount:
            fs_incoreTune->inodeFreeCount = (uint32)req->value;
            break;
        case FS_TUNE_userBufMinInUse:
            fs_incoreTune->userBufMinInUse = (uint32)req->value;
            break;
        case FS_TUNE_logCommitHiWtrMk:
            fs_incoreTune->logCommitHiWtrMk = (uint32)req->value; 
            break;
        case FS_TUNE_bufUserPctMem:
            fs_incoreTune->bufUserPctMem = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_bufMetaNonDirPctMem:
            fs_incoreTune->bufMetaNonDirPctMem = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_bufMetaDirPctMem:
            fs_incoreTune->bufMetaDirPctMem = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_bufMetaCompNonDirPctMem:
            fs_incoreTune->bufMetaCompNonDirPctMem = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_bufMetaCompDirPctMem:
            fs_incoreTune->bufMetaCompDirPctMem = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_metaFreeCount:
            fs_incoreTune->metaFreeCount = (uint32)req->value;
            break;
        case FS_TUNE_metaDirMinInUse:
            fs_incoreTune->metaDirMinInUse = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_metaNonDirMinInUse:
            fs_incoreTune->metaNonDirMinInUse = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_metaCompDirMinInUse:
            fs_incoreTune->metaCompDirMinInUse = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;
        case FS_TUNE_metaCompNonDirMinInUse:
            fs_incoreTune->metaCompNonDirMinInUse = (uint32)req->value;
            fs_bufSetCacheLimits();
            break;

        case FS_TUNE_ABS_LOG:
            fs_incoreTune->resource[FS_RESOURCE_LOG].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_LOG:
            fs_incoreTune->resource[FS_RESOURCE_LOG].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_LOG:
            fs_incoreTune->resource[FS_RESOURCE_LOG].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_UBC:
            fs_incoreTune->resource[FS_RESOURCE_USERBUFCACHE].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_UBC:
            fs_incoreTune->resource[FS_RESOURCE_USERBUFCACHE].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_UBC:
            fs_incoreTune->resource[FS_RESOURCE_USERBUFCACHE].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_MBC_ND:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_NONDIR].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_MBC_ND:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_NONDIR].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_MBC_ND:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_NONDIR].loadFactor = (uint32)req->value;
            break;

        case FS_TUNE_ABS_MBC_CND:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_MBC_CND:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_MBC_CND:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_MBC_D:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_DIR].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_MBC_D:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_DIR].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_MBC_D:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_DIR].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_MBC_CD:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_DIR].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_MBC_CD:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_DIR].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_MBC_CD:
            fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_DIR].loadFactor = (uint32)req->value;
            break;

        case FS_TUNE_ABS_INC:
            fs_incoreTune->resource[FS_RESOURCE_INODECACHE].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_INC:
            fs_incoreTune->resource[FS_RESOURCE_INODECACHE].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_INC:
            fs_incoreTune->resource[FS_RESOURCE_INODECACHE].loadFactor = (uint32)req->value;
            break;
            
#ifndef FS_STANDALONE
       case FS_TUNE_ABS_MEM:
            fs_incoreTune->resource[FS_RESOURCE_MEM].absMin = (uint32)req->value;
			fs_incoreTune->resource[FS_RESOURCE_MEM].totalMinusRelAbs =
				(uint32)(eee_ramTotalFlexSize() / 1024) -
				(uint32)(eee_ramTotalFlexSize() / 1024) / 128 *
				fs_incoreTune->resource[FS_RESOURCE_MEM].relMin -
				fs_incoreTune->resource[FS_RESOURCE_MEM].absMin;
            break;
        case FS_TUNE_REL_MEM:
            fs_incoreTune->resource[FS_RESOURCE_MEM].relMin = (uint32)req->value;
			fs_incoreTune->resource[FS_RESOURCE_MEM].totalMinusRelAbs =
				(uint32)(eee_ramTotalFlexSize() / 1024) -
				(uint32)(eee_ramTotalFlexSize() / 1024) / 128 *
				fs_incoreTune->resource[FS_RESOURCE_MEM].relMin -
				fs_incoreTune->resource[FS_RESOURCE_MEM].absMin;
            break;
#endif // FS_STANDALONE

        case FS_TUNE_LOAD_MEM:
            fs_incoreTune->resource[FS_RESOURCE_MEM].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_SAC:
            fs_incoreTune->resource[FS_RESOURCE_SAC].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_SAC:
            fs_incoreTune->resource[FS_RESOURCE_SAC].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_SAC:
            fs_incoreTune->resource[FS_RESOURCE_SAC].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_ULT:
            fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].absMin = (uint32)req->value;
 			fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].totalMinusRelAbs =
				eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) -
				eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) / 128 *
				fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].relMin -
				fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].absMin;
           break;
        case FS_TUNE_REL_ULT:
            fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].relMin = (uint32)req->value;
			fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].totalMinusRelAbs =
				eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) -
				eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) / 128 *
				fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].relMin -
				fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].absMin;
            break;
        case FS_TUNE_LOAD_ULT:
            fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].loadFactor = (uint32)req->value;
            break;
            
        case FS_TUNE_ABS_VOL:
            fs_incoreTune->resource[FS_RESOURCE_VOL_LOG].absMin = (uint32)req->value;
            break;
        case FS_TUNE_REL_VOL:
            fs_incoreTune->resource[FS_RESOURCE_VOL_LOG].relMin = (uint32)req->value;
            break;
        case FS_TUNE_LOAD_VOL:
            fs_incoreTune->resource[FS_RESOURCE_VOL_LOG].loadFactor = (uint32)req->value;
            break;
        }
    }

    resp->incore = *fs_incoreTune;
	resp->count_per_usec = count_per_usec;

    context->respSize = sizeof(fs_MsgIncoreTuneResp);

    return NFX_OK;
}


/*-----------------------------------------------------------------------------
 * fs_contextThread
 */

int32
fs_contextThread(
    fs_Context      *context)
{
    //fs_FileSystem       *fileSystem;
    //fs_MsgContextResp   *resp = (fs_MsgContextResp *) context->respHdr;

    // todo - find the context for the specified handle,
    // if found copy and return ok, else return err.

    context->respSize = sizeof(fs_MsgContextResp);

    return NFX_OK;

} /* fs_contextThread */

void
fs_dumpBuf(
    fs_Buf  *buf)
{
    printf("fs_Buf at %p:\n", buf);

    printf("volId %lld\n", buf->fileSystem->volId.whole_thing);
    printf("block %lld\n", fs_blockNumber(buf->block));
    printf("inum %lld\n", buf->inum);
    printf("page %lld\n", buf->page);
    printf("lock own %llx mode %d cnt %d flags %x wtcnt %d\n",
            (uint64)((uintptr_t)buf->lock.owner), buf->lock.mode,
			buf->lock.count, buf->lock.flags, buf->lock.waitCount);
    printf("flags %x\n", buf->flags);
    printf("numDeps %d\n", buf->logSegDep.count);
    if (buf->flags & FS_BUF_TYPE_USER)
    {
        printf("cod %llx offset %d len %d\n", buf->u.user.codHandle,
               buf->u.user.codOffset, buf->u.user.codLength);
    }
    else
    {
        printf("u: %p\n", buf->u.data);
    }

} /* fs_dumpBuf */

/*-----------------------------------------------------------------------------
 * fs_logdataThread
 */
int32
fs_logdataThread(
    fs_Context      *context)
{
    fs_MsgLogdataResp  *resp = (fs_MsgLogdataResp *) context->respHdr;
    uint32 size;

    size = fs_logMakeDumpLogRecord(context, resp->data, FS_LOGDATA_MAX_SIZE - 1);
    resp->data[size] = 0 ;
    
    context->respSize = sizeof(fs_MsgRespHdr) + size + 1;

    return NFX_OK;

} /* fs_logdataThread */


/*-----------------------------------------------------------------------------
 * fs_logtestThread
 */
int32
fs_logtestThread(
    fs_Context      *context)
{
    fs_MsgLogtestReq *req = (fs_MsgLogtestReq *) context->reqHdr;

    switch (req->type)
    {
        case FS_LOG_TEST_FORCE_CRASH:
            break;
        case FS_LOG_TEST_CRASH_NOW:
#ifdef FS_DEBUG_CHECK_CRASH
            fs_logPrintStats(&context->fileSystem->logRoot->logStats,
                             context->fileSystem->logRoot->currLen);
            fs_debugCrash();
#endif
        default:
            return NFX_ERR;
    }
    return NFX_OK;

} /* fs_logtestThread */


/*-----------------------------------------------------------------------------
 * fs_crashThread
 */

#if defined(FS_DEBUG_CHECK_CRASH)

int32 fs_debugCrashEnabled = 0 ;
int32 fs_debugCrashType = 0 ;
int32 fs_debugCrashCount = 0 ;
boolean fs_debugWriteYield = 0 ;
boolean fs_debugZeroFillYield = 0 ;

#define FS_DEBUG_CRASH_IMMEDIATE           1
#define FS_DEBUG_CRASH_AFTER_COMMIT        2
#define FS_DEBUG_CRASH_AFTER_METAONE       3
#define FS_DEBUG_CRASH_AFTER_METATWO       4
#define FS_DEBUG_CRASH_AFTER_USER          5
#define FS_DEBUG_CRASH_AFTER_ALLOC         6
#define FS_DEBUG_CRASH_AFTER_USERRM        7

#define FS_DEBUG_CRASH_AFTER_SETATTR       8 
#define FS_DEBUG_CRASH_AFTER_CREATE        9
#define FS_DEBUG_CRASH_WRITE_YIELD	       10
#define FS_DEBUG_CRASH_ZERO_FILL_YIELD	   11

int32
mcpu_sim_write_to_file();
int32
nvram_write_to_file();
extern int nvram_ever_used;
#ifndef FS_SIM_VOL_MAX
#define FS_SIM_VOL_MAX 3
#endif
extern int volFd[FS_SIM_VOL_MAX + 1];

void fs_debugCrash ()
{
#if defined(FS_STANDALONE)
    uint32 loop;
	extern int volFd[FS_SIM_VOL_MAX + 1];
    
    if (nvram_ever_used)
    {
        mcpu_sim_write_to_file();
        nvram_write_to_file();
    }

    for (loop = 0 ; loop < (FS_SIM_VOL_MAX + 1) ; loop++)
    {
        if (volFd[loop])
            close (volFd[loop]);
    }

    printf ("FS Crash on request\n");

    exit(1);
#else
    panic("fs_debugCrash");
#endif
}


void fs_debugCheckCrash (fs_Context *context)
{
    // Stats are locked by caller (see fs_statsReqRcvdSpinId)

    switch (fs_debugCrashType)
    {
    case FS_DEBUG_CRASH_AFTER_COMMIT:
        if (context && 
            (context->fileSystem->logRoot->logStats.nbrCommit > fs_debugCrashCount))
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_METAONE:
        if (context && 
            (context->fileSystem->logRoot->logStats.nbrMetaOne > fs_debugCrashCount))
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_METATWO:
        if (context && 
            (context->fileSystem->logRoot->logStats.nbrMetaTwo > fs_debugCrashCount))
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_USER:
        if (context && 
            (context->fileSystem->logRoot->logStats.nbrUser > fs_debugCrashCount))
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_ALLOC:
        if (context && 
            (context->fileSystem->logRoot->logStats.nbrAlloc > fs_debugCrashCount))
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_USERRM:
        if (context && 
            (context->fileSystem->logRoot->logStats.nbrUserRm > fs_debugCrashCount))
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_SETATTR:
        if (context->fileSystem->recvReqStats.num[FS_REQ_SET_ATTR] > fs_debugCrashCount)
            fs_debugCrash();
        break;
    case FS_DEBUG_CRASH_AFTER_CREATE:
        if (context->fileSystem->recvReqStats.num[FS_REQ_CREATE] > fs_debugCrashCount)
            fs_debugCrash();
        break;
    }
}
#endif

int32
fs_crashThread(
    fs_Context      *context)
{
#if defined(FS_DEBUG_CHECK_CRASH)
    uint32 count = 0;
    
    context->respSize = sizeof(fs_MsgCrashResp);

    if (((fs_MsgCrashReq*)context->reqHdr)->type == FS_DEBUG_CRASH_IMMEDIATE)
    {
        fs_debugCrash();
    }
    else
    {
        fs_debugCrashType = ((fs_MsgCrashReq*)context->reqHdr)->type ;
        switch (fs_debugCrashType)
        {
        case FS_DEBUG_CRASH_AFTER_COMMIT:
            count = context->fileSystem->logRoot->logStats.nbrCommit;
            break;
        case FS_DEBUG_CRASH_AFTER_METAONE:
            count = context->fileSystem->logRoot->logStats.nbrMetaOne;
            break;
        case FS_DEBUG_CRASH_AFTER_METATWO:
            count = context->fileSystem->logRoot->logStats.nbrMetaTwo;
            break;
        case FS_DEBUG_CRASH_AFTER_USER:
            count = context->fileSystem->logRoot->logStats.nbrUser;
            break;
        case FS_DEBUG_CRASH_AFTER_ALLOC:
            count = context->fileSystem->logRoot->logStats.nbrAlloc;
            break;
        case FS_DEBUG_CRASH_AFTER_USERRM:
            count = context->fileSystem->logRoot->logStats.nbrUserRm;
            break;
        case FS_DEBUG_CRASH_AFTER_SETATTR:
            NFX_SPIN_LOCK(&context->fileSystem->fs_statsReqRcvdSpinId);
            count = context->fileSystem->recvReqStats.num[FS_REQ_SET_ATTR];
            NFX_SPIN_UNLOCK(&context->fileSystem->fs_statsReqRcvdSpinId);
            break;
        case FS_DEBUG_CRASH_AFTER_CREATE:
            NFX_SPIN_LOCK(&context->fileSystem->fs_statsReqRcvdSpinId);
            count = context->fileSystem->recvReqStats.num[FS_REQ_CREATE];
            NFX_SPIN_UNLOCK(&context->fileSystem->fs_statsReqRcvdSpinId);
            break;
		case FS_DEBUG_CRASH_WRITE_YIELD:
        	fs_debugWriteYield = TRUE;
			return NFX_OK;
		case FS_DEBUG_CRASH_ZERO_FILL_YIELD:
        	fs_debugZeroFillYield = TRUE;
			return NFX_OK;
        }
        fs_debugCrashCount = count + ((fs_MsgCrashReq*)context->reqHdr)->count ;
        fs_debugCrashEnabled = 1 ;
    }
    
#endif

    return NFX_OK;

} /* fs_crashThread */


int32
fs_traceThread(
    fs_Context  *context)
{
    // Obseleted by elog
    return NFX_OK;
}

static void
zap_dirent(fs_Context *context, uint64 inum)
{
	fs_Inode *inode = I_GET_EXCL(0, inum);
	char name[32];
	int namelen = 16;
	uint64 random_inum;
	int i;

    if (inode == NULL) {
        printf("zap dirent: ino %lld not found\n", inum);
        return;
    }

    if (!FS_I_IS_DIR(inode->dinode.attrs)) {
        printf("zap dirent: ino %lld not a directory\n", inum);
        fs_inodeReleaseExclusive(context, inode);
        return;
    }

	/*
	 * Introduce a random dirent.
	 */
	for (i = 0; i < namelen; i++) {
		name[i] = fs_random64(context, 26) + 'a';
	}
	name[namelen] = 0;

	random_inum = fs_random64(context, ~FS_VALID_INODE_LIMIT);

	fs_dir_addent(context, inode, name, namelen, random_inum);

    inode->dinode.ctime = fs_getTimeOfDay();
    inode->dinode.mtime = inode->dinode.ctime;
    fs_inodeWrite(context, inode, INO_WRITE_LOG);

    printf("zap dirent: added random ino entry %lld to ino %lld\n",
            random_inum, inode->inum);
}

static void
zap_inode(fs_Context *context, uint64 inum)
{
    fs_Inode *zapInode;
	fs_Inode *inode = I_GET_SHARED(0, inum);

    if (inode == NULL) {
        printf("zap inode: ino %lld not found\n", inum);
        return;
    }
	zapInode = fs_allocateFileInode(context, inode, 0400, 0, 0, 0);
	fs_inodeRelease(context, inode);
    printf("zap inode: allocated unreferenced ino %lld\n", zapInode->inum);
}

static void
zap_link(fs_Context *context, uint64 inum, uint64 n)
{
	fs_Inode *inode = I_GET_EXCL(0, inum);

    if (inode == NULL) {
        printf("zap link: ino %lld not found\n", inum);
        return;
    }
    inode->dinode.linkCnt = n;
    fs_inodeWrite(context, inode, INO_WRITE_LOG);
    printf("zap link: ino %lld linkCnt -> %d\n",
           inum, inode->dinode.linkCnt);
}

struct zap_block {
	int count;
	fs_Block block;
};

static boolean
zap_worker(
	fs_Context *context,
	fs_Inode *inode,
    fs_WalkResult *result,
	void *arg0)
{
	struct zap_block *zb = arg0;

	if (zb->count == 0) {
		zb->block = result->block;
        printf("zap walk: ino %lld\n", inode->inum);
	}
    switch (result->type) {
        case FS_WALK_INNER_NODE_TYPE:
            printf("  inner node %lld\n", fs_blockNumber(result->block));
            break;
        case FS_WALK_LEAF_NODE_TYPE:
            printf("  leaf node %lld\n", fs_blockNumber(result->block));
            break;
        case FS_WALK_EXTENT_TYPE:
            break;
    }
    zb->count++;
    return TRUE;
}

/*
 * Zap the block's btree root header
 */
static void
zap_btree(fs_Context *context, uint64 inum)
{
	fs_Inode *inode = I_GET_EXCL(0, inum);
    fs_Emapflatroot *r;

    if (inode == NULL) {
        printf("zap btree: no inode found for %lld\n", inum);
        return;
    }

    /* zap the keys in the emaproot header */
    r = &inode->dinode.u.emapflatroot;

    if (r->hdr.level == 0) {
        r->hdr.nkeys = FS_INOFLATROOT_MAXKEYS + 1;
        printf("zap btree: emaproot zap'd for ino %lld\n", inum);
    } else {
        printf("zap btree: not emaproot for ino %lld\n", inum);
        fs_inodeReleaseExclusive(context, inode);
        return;
    }

    fs_inodeWrite(context, inode, INO_WRITE_LOG);
}

/*
 * zap a block directly on disk (without using buffer cache)
 * can use zap_show to see all blocks for an inode
 */
static void
zap_raw(fs_Context *context, uint64 block, uint8 value)
{
    int i;
    fs_RdwrReq *rdwr;
    uint8 *rawData;

    /*
     * sync and flush so this data can be written out to disk
     * for example, if this block is in the buffer cache
     * when the writeback daemon flushes this buffer out to
     * disk, it will overwrite our raw data
     */
    fs_syncThread(context);
    fs_bufCacheFlush(context, NULL);

    rdwr = fs_rdwrAlloc();
    printf("zap raw: reading block %lld from disk\n", block);
    fs_volReadBlock(context, rdwr, fs_makeBlock(block));
    rawData = rdwr->evmStat.rDataHandle.dataPtr;
	for (i = 0; i < FS_BLOCK_SIZE; i++) {
		rawData[i] = value;
    }
    printf("zap raw: writing back block %lld with data pattern %x\n", block, value);
    fs_volWriteBlock(context, rdwr, fs_makeBlock(block), rawData);
    fs_rdwrDealloc(rdwr);
}

/*
 * Show blocks for inode so it can be zapped
 */
static void
zap_show(fs_Context *context, uint64 inum)
{
	fs_Inode *inode = I_GET_SHARED(0, inum);
	fs_Block inval = FS_INVALID_BLOCK;
	struct zap_block zb;
	int i;
    fs_Emaproot_hdr *h;

    if (inode == NULL) {
        printf("zap show: no inode found for %lld\n", inum);
        return;
    }

	zb.count = 0;
	zb.block = inval;

    h = fs_getBtreeShared(context, inode);
    if (h->level == 0) {
        fs_Emapflatroot *r = (fs_Emapflatroot *) h;
        printf("zap show: ino %lld flat emaproot nkeys %d\n",
                inum, r->hdr.nkeys);
        printf("{ ");
        for (i = 0; i < r->hdr.nkeys; i++) {
            printf("%lld ", r->offsets[i]);
        }
        printf("}\n{");
        for (i = 0; i < r->hdr.nkeys; i++) {
            printf("%llx ", r->extents[i].lengthAndStartBlock);
        }
        printf("}\n\n");
    } else {
        fs_Emaproot *r = (fs_Emaproot *) h;
        printf("zap show: ino %lld emaproot nkeys %d level %d right %lld\n",
                inum, r->hdr.nkeys, r->hdr.level, r->right);
        if (r->hdr.nkeys) {
            printf("          { ");
            for (i = 0; i < r->hdr.nkeys; i++) {
                printf("%lld ", r->offsets[i]);
            }
            printf("}\n");
            printf("          { ");
            for (i = 0; i < r->hdr.nkeys; i++) {
                printf("%lld ", r->left[i]);
            }
        }
        printf("}\n\n");
    }
    fs_releaseBtreeShared(context, inode);
    fs_emap_walk(context, inode, zap_worker, &zb);

	fs_inodeRelease(context, inode);
}

/**
 * Zap a given object in the filesystem for eek testing.
 */
static void
fs_zap(fs_Context *context, char action, uint64 arg1, uint64 arg2)
{
    uint64 inum = arg1;

	printf("zap '%c' inum=%lld\n", action, inum);
	switch (action) {
	case 'b': zap_btree(context, inum); break;
	case 'd': zap_dirent(context, inum); break;
	case 'i': zap_inode(context, inum); break;
    case 'l': zap_link(context, inum, arg2); break;
    case 'r': zap_raw(context, arg1, arg2); break;
    case 's': zap_show(context, inum); break;
	}
}

int32
fs_zapThread(fs_Context *context)
{
    fs_MsgZapReq *req = (fs_MsgZapReq *) context->reqHdr;

    FS_TRACE(context, FS_TRACE_REQ, ("zap"));
    context->respSize = sizeof (fs_MsgZapResp);
    fs_logBeginTransaction(context);
    fs_zap(context, req->action, req->inum, req->data);
    fs_logCommitEntry(context);
    return NFX_OK;
}


/**
 * Abort the specified volume.
 */
int32
fs_abortThread(fs_Context *context)
{
    FS_TRACE(context, FS_TRACE_REQ, ("abort"));

    fs_volumeException(context, context->fileSystem, FS_VOLUME_EXCEPTION);

    context->respSize = sizeof (fs_MsgAbortResp);
    return NFX_OK;
}


#endif // SSC
