/*
 * Name:  fs-dir.h
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * RCS:  $Id: fs-dir.h,v 1.1 2002/11/26 23:54:42 bob Exp $
 *
 * Description:  API for directory code.
 *
 */

#if !defined(FS_DIR_H)
#define FS_DIR_H


/*
 * Dirent name hashes depend on the type of name
 * being hashed.
 */
enum {
    DIR_NAME_TYPE_LONG,     // the "normal" long name of the dirent
    DIR_NAME_TYPE_SEQ,      // an 8.3 alias in the xxxxxx~1.xxx form
    DIR_NAME_TYPE_RAND,     // an 8.3 alias in our random number form (the
                            // hash is the random number itself)
    DIR_NAME_TYPE_MAX
};
struct fs_DirSearchCb
{
    uint64  inum;       // returned inum if found
    uint64  page;       // returned  page if found
    int32   index;      // returned index on page if found
    char    *name;
    int     len;
    int     nameType;   // the type currently being searched for
    uint32  hash;       // the type specific hash for the name
    boolean hashMatch;  // for detecting hash collisions
    int32   seq;        // the ~sequence currently being searched for
    uint32  aliasFlags; 
    union {
#define DIR_SEQ_NUM_DWORDS (FS_DIR_ALIAS_SEQ_MAX / 64) + 1
#define DIR_SEQ_NUM_BYTES  (FS_DIR_ALIAS_SEQ_MAX / 8) + 1
        uint64           dwords[DIR_SEQ_NUM_DWORDS];
        unsigned char    bytes[DIR_SEQ_NUM_BYTES];
    } seqBits;
};

extern uint64
fs_dir_lookupent(
    fs_Context  *context,
    fs_Inode    *inode,
    char        *name,
    int         len);

extern void
fs_dirroot_init(
    fs_Context  *context, 
    fs_Inode    *inode, 
    uint64      parent_inum);

extern int
fs_dir_addent(
    fs_Context  *context, 
    fs_Inode    *inode, 
    char        *name, 
    int         len, 
    uint64      inum);

extern int
fs_dir_removeent(
    fs_Context  *context, 
    fs_Inode    *inode, 
    char        *name, 
    int         len);

extern void
fs_dir_readdir(
    fs_Context  *context, 
    fs_Inode    *inode,
	fs_DirEntry *entrytab,
	int32       max_size,
	uint64      cookie,
    boolean     show_dot_snapshot,
	int32       *num_entries,
    int32       *eof);

extern void
fs_dirroot_set_parent(
    fs_Context  *context,
    fs_Inode    *inode,
    uint64      parent_inum);

extern uint64
fs_dirroot_get_parent(
    fs_Context  *context,
    fs_Inode    *inode);

extern boolean
fs_dir_mark_used_inodes(
    fs_Context  *context, 
    uint8       *bitmap, 
    fs_Inode    *dir, 
    boolean     query);

extern int
fs_dir_namecmp(
    fs_Context  *context,
    fs_Inode    *inode,
    fs_Dirleaf  *d, 
    fs_Dirent   *dirent, 
    fs_DirSearchCb  *cb);

extern int
fs_dir_hashcmp(
    fs_Dirent   *dirent,
    uint32      hash,
    int         nameType);

extern boolean
fs_dir_is_empty(
    fs_Context  *context, 
    fs_Inode    *inode);


#endif // FS_DIR_H
