/*
 * Name		: fs-filehistory.h
 * RCS		: $Id: fs-filehistory.h,v 1.2 2002/10/26 03:00:24 laurentb Exp $
 * Description	: File History implementation for Incremental backup
 * Created by	: Rahul Khona
 * Date Created	: 9/19/2001
 */

#ifndef _FS_FILEHISTORY_H_
#define _FS_FILEHISTORY_H_
#define FS_NULL_DISKPTR_BLOCK   (int64)-1

typedef struct fs_NdmpHistoryNode {
        uint64  size;               // 8 bytes : 0  : 8
        fs_Time atime;              // 8 bytes : 8  : 16
        fs_Time mtime;              // 8 bytes : 16 : 24
        fs_Time ctime;              // 8 bytes : 24 : 32
        uint64  inum;               // 8 bytes : 32 : 40
        uint64  parent;             // 8 bytes : 40 : 48
        uint32  uid;                // 4 bytes : 48 : 52
        uint32  gid;                // 4 bytes : 52 : 56
        uint32  mode;               // 4 bytes : 56 : 60
        uint32  linkCnt;             // 4 bytes : 60 : 64
        int64   firstChildBlock;    // 8 bytes : 64 : 72
        int64   lastChildBlock;     // 8 bytes : 72 : 80
        int64   sibling;            // 8 bytes : 80 : 88
        uint64  nameBlock;          // 8 bytes : 88 : 96
        uint16  firstChildBoff;     // 2 bytes : 96 : 98
        uint16  lastChildBoff;       // 2 bytes : 98 : 100
        uint16  siblingBoff;        // 2 bytes : 100 : 102
        uint16  nameBoff;           // 2 bytes : 102 : 104
        uint16  nameLen;            // 2 bytes : 104 : 106
        char    pad1[2];            // 2 bytes : 106 : 108
        uint64  tapeOff;            // 8 bytes : 108 : 116
        uint64  tapelen;            // 8 bytes : 116 : 124 
        char    pad2[4];            // 8 bytes : 124 : 128
} fs_NdmpHistoryNode_t;  // 128 bytes


#define FS_MAX_NDMP_NODES_PER_BLOCK                \
    (FS_BLOCK_SIZE / sizeof (fs_NdmpHistoryNode_t))
#define FS_MAX_NDMP_BOFF_PER_BLOCK                 \
        (FS_MAX_NDMP_NODES_PER_BLOCK * sizeof (fs_NdmpHistoryNode_t))

typedef struct fs_NdmpHistoryNodeQueue {
        DLL_ENTRY(struct fs_NdmpHistoryNodeQueue) link;
        uint64          block;
        uint16          boff;
} fs_NdmpHistoryNodeQueue_t;

typedef struct fs_NdmpHistoryNodeQueueHead {
    fs_NdmpHistoryNodeQueue_t   *head;
    fs_NdmpHistoryNodeQueue_t   *tail;
} fs_NdmpHistoryNodeQueueHead_t;

extern void
fs_ndmpAddNode(
        fs_Context              *context,
        fs_DumpContext_t        *dumpContext,
        int64                   parentBlock,
        int64                   parentBoff,
        fs_Inode                *inode,
        fs_Inumber              parent,
        char                    *newName,
        int                     newNameLen,
        int64                   *newBlock,
        uint16                  *newBoff
);

extern void
fs_walkHistoryNodes(
        fs_Context          *context,
        fs_DumpContext_t    *dumpContext,
        void                (*func)(fs_Context *,
                                    fs_DumpContext_t *,
                                    fs_Inode *, int64, uint16)
);
extern void
fs_fileHistoryProcessNodes(
                fs_Context          *context,
                fs_DumpContext_t    *dumpContext,
                fs_Inode            *nodeFile,
                int64               block,
                uint16              boff
);
extern void
fs_fileHistoryProcessDirs(
                fs_Context          *context,
                fs_DumpContext_t    *dumpContext,
                fs_Inode            *nodeFile,
                int64               block,
                uint16              boff
);
extern void
fs_createFileHistories(
        fs_Context          *context,
        fs_DumpContext_t    *dumpContext
);

void
fs_cleanFileHistories(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext
);
#endif  // _FS_FILEHISTORY_H_
