/*
 * Name:  fs-global.c
 *
 * RCS:  $Id: fs-global.c,v 1.112 2002/12/04 02:47:50 laurentb Exp $
 *
 * Description:  Filesystem globals.
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

#if !defined(SSC)

#include "fs-includes.h"
#include <exports/elog-api.h>

#define FS_ULTRA_BUF_OFFSET 128

/*
 * Number of bits in a byte.
 * Example: onesPerByte[2]=1, onesPerByte[127]=7
 */
uint8 fs_onesPerByte[256] = {
    0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 
    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 
    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 
    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 
    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 
    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 
    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 
    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 
    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 
    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 
    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 
    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 
    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 
    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 
    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 
    4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8
};


/**
 * Pointer to the inode cache.
 */
fs_InodeCache *fs_inodeCache;


/**
 * The security descriptor cache.
 */
fs_SecurityDescCache fs_securityDesc;


/**
 * The Quota cache.
 */  
fs_QuotaCache fs_quotaCache;


/**
 * Initialize in-core tuneables.
 */
fs_IncoreTune *fs_incoreTune;
#ifdef FS_INCORE_STATS
fs_IncoreStats *fs_incoreStats;
#endif


void
fs_incorePrintResSettings(
    fs_Context      *context)
{
    uint32          loop ;
    
    printf("Settings for the resource gate keeper\n");
    for (loop = 0 ; loop < FS_RESOURCE_TYPE_MAX; loop++)
    {
        printf(" %s   Abs: %10d - Rel: %3d - Load: %10d\n",
               fs_resourceNames[loop],
               fs_incoreTune->resource[loop].absMin,
               fs_incoreTune->resource[loop].relMin,
               fs_incoreTune->resource[loop].loadFactor);
    }
}

void
fs_incorePrintCurrResources(
    fs_Context      *context)
{
    int32 itype;

    printf("Current used/available resources\n");

    if (context && context->fileSystem)
    {
        printf("  Log             Total: %10d - Use: %10d - Available: %10d\n",
               context->fileSystem->logSize,
               context->fileSystem->logRoot->currLen,
               context->fileSystem->logSize - context->fileSystem->logRoot->currLen);
    }
    
    itype = FS_BUF_CACHE_ITYPE_NONDIR;
    printf("  User Buf Cache  Total: %10d - Use: %10d - Available: %10d\n",
           fs_bufListGlobal[FS_BUFLIST_USER].count.usedBufLimit[itype],
           fs_bufListGlobal[FS_BUFLIST_USER].count.dirtyBufs[itype],
           fs_bufListGlobal[FS_BUFLIST_USER].count.usedBufLimit[itype] - fs_bufListGlobal[FS_BUFLIST_USER].count.dirtyBufs[itype]);

    itype = FS_BUF_CACHE_ITYPE_NONDIR;
    printf("  Meta 8k Nondir Buf Cache  Total: %10d - Use: %10d - Available: %10d\n",
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype] - fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype]);

    itype = FS_BUF_CACHE_ITYPE_DIR;
    printf("  Meta 8k Dir Buf Cache  Total: %10d - Use: %10d - Available: %10d\n",
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype] - fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype]);

    itype = FS_BUF_CACHE_ITYPE_COMP_NONDIR;
    printf("  Meta 1k Nondir Buf Cache  Total: %10d - Use: %10d - Available: %10d\n",
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype] - fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype]);

    itype = FS_BUF_CACHE_ITYPE_COMP_DIR;
    printf("  Meta 1k Dir Buf Cache  Total: %10d - Use: %10d - Available: %10d\n",
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype],
           fs_bufListGlobal[FS_BUFLIST_META].count.usedBufLimit[itype] - fs_bufListGlobal[FS_BUFLIST_META].count.dirtyBufs[itype]);

    printf("  Inode Cache     Total: %10d - Use: %10d - Available: %10d\n",
           fs_inodeCache->count.numCachedLimit,
           fs_inodeCache->count.numDirty,
           fs_inodeCache->count.numCachedLimit - fs_inodeCache->count.numDirty);
    printf("  Memory          Total: %10lld - Use: %10lld - Available: %10lld\n",
           eee_ramTotalHeapSize(),
           eee_ramTotalHeapSize() - eee_ramTotalHeapAvail(),
           eee_ramTotalHeapAvail());
#if ! defined(FS_STANDALONE)
#ifndef SBM_REV_2
    printf("  SAC buffers     Total: %10d - Use: %10d - Available: %10d\n",
           sbm_sac_info->total_sac_blocks,
           sbm_sac_info->total_sac_blocks - sbm_sac_info->num_sac_blocks_avail,
           sbm_sac_info->num_sac_blocks_avail);
#endif
    printf("  Ultra buffers   Total: %10d - Use: %10d - Available: %10d\n",
           eee_getTotalNbrOfBufs(EEE_BUF_ULTRA),
           eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) - eee_getAvailableNbrOfBufs(EEE_BUF_ULTRA),
           eee_getAvailableNbrOfBufs(EEE_BUF_ULTRA));
#endif
    if (context && 
        context->fileSystem && 
        context->fileSystem->logRoot && 
        context->fileSystem->logRoot->volBufRoot)
    {
        vl_VolBufRoot   *volBufRoot;

        volBufRoot = context->fileSystem->logRoot->volBufRoot;

        NFX_SPIN_LOCK(&volBufRoot->idSpin);

        printf("  Vol log buffers Total: %10d - Use: %10d - maxNbrIO: %10d\n",
               VL_BUFFER_LOG_HWM,
               volBufRoot->nbr.currNbrBuf,
               volBufRoot->nbr.maxNbrIO);
        NFX_SPIN_UNLOCK(&volBufRoot->idSpin);
    }
    
}

void
fs_incorePrintStats(
    fs_Context      *context)
{
#ifdef FS_INCORE_STATS
    uint32          loop ;
    
    printf("Statistics for resource gate keeper\n");
    printf(" Dropped no resource       %10d\n", fs_incoreStats->nbrDropNoResource);
    
    for (loop = 0 ; loop < FS_RESOURCE_TYPE_MAX; loop++)
    {
        printf(" %s            Nbr dropped: %10d  Max Used: %10d\n",
               fs_resourceNames[loop],
               fs_incoreStats->resource[loop].nbrDrop,
               fs_incoreStats->resource[loop].maxUsed);
    }
#endif
}


void
fs_initIncoreTune()
{
	uint64	            available;

    fs_incoreTune = fs_ramAlloc(FS_RAM_CORE_MISC, sizeof *fs_incoreTune);
#ifdef FS_INCORE_STATS
    fs_incoreStats = fs_ramAlloc(FS_RAM_CORE_MISC, sizeof *fs_incoreStats);

    bzero(fs_incoreStats, sizeof *fs_incoreStats);
#endif

    fs_incoreTune->inodePctAvailHiWtrMk = FS_INODE_PCT_AVAIL_HI;
    fs_incoreTune->inodePctAvailLoWtrMk = FS_INODE_PCT_AVAIL_LO;
    fs_incoreTune->logPctUseHiWtrMk = FS_LOG_PCT_USE_HI;
    fs_incoreTune->logPctUseLoWtrMk = FS_LOG_PCT_USE_LO;
    fs_incoreTune->bufPctAvailLoWtrMk = FS_BUF_PCT_AVAIL_LO;
    fs_incoreTune->bufPctAvailHiWtrMk = FS_BUF_PCT_AVAIL_HI;
    fs_incoreTune->memPctAvailMin = FS_MEM_PCT_AVAIL_MIN;
    fs_incoreTune->bufWritesPerEvent = FS_BUF_WB_PER_EVENT;
    fs_incoreTune->writesPendingMax = FS_IO_WRITE_PENDING_MAX;
    fs_incoreTune->perFsIoPendingMax = FS_PER_FS_IO_PENDING_MAX;

    fs_incoreTune->bufMetaNonDirPctMem = FS_BUF_META_NONDIR_PCT_MEM;
    fs_incoreTune->bufMetaDirPctMem = FS_BUF_META_DIR_PCT_MEM;
    fs_incoreTune->bufMetaCompNonDirPctMem = FS_BUF_META_COMP_NONDIR_PCT_MEM;
    fs_incoreTune->bufMetaCompDirPctMem = FS_BUF_META_COMP_DIR_PCT_MEM;

    fs_incoreTune->bufUserPctMem = FS_BUF_USER_PCT_MEM;
    fs_incoreTune->inodePctMem = FS_INODE_PCT_MEM;

	eee_usecToRTC((uint64)FS_MAX_CPU_USAGE,
				  &fs_incoreTune->maxCpuUsageRTC);

    fs_incoreTune->readdirMaxSpace = FS_READDIR_SPACE_MAX;
    // fs_incoreTune->readdirMaxSpace = 512;

    fs_incoreTune->userDirtyThreshold = FS_DIRTY_THRESHOLD;
    fs_incoreTune->metaDirtyThreshold = FS_DIRTY_THRESHOLD;

    fs_incoreTune->wbSleepMsecs = FS_WB_SLEEP_MSECS;

    fs_incoreTune->enableReadAhead = TRUE;
    // fs_incoreTune->enableReadAhead = FALSE;
    fs_incoreTune->readAheadMaxSize = FS_MAX_READ_AHEAD_SIZE;
    fs_incoreTune->readAheadDefaultSize = FS_DEFAULT_READ_AHEAD_SIZE;

    fs_incoreTune->enableAtimeUpdate = TRUE;
    // fs_incoreTune->enableAtimeUpdate = FALSE;

    fs_incoreTune->vollogIoMax = VL_BUFFER_IO_MAX;

    fs_incoreTune->logCommitHiWtrMk = FS_LOG_COMMIT_THRESHOLD;

    fs_incoreTune->inodeCleanPerPass = FS_INO_CLEAN_PER_PASS;

    fs_incoreTune->activeTransToWait = 4;

    fs_incoreTune->vollogLoopBack = FALSE;

    // available sac size in bytes
#ifdef FS_STANDALONE
    available = (200 * 1024 * 1024);
#else
    available = sbm_get_sac_size(); 
#endif
    // 10 % must remain available
    // Calculations are done in nbr of EEE_BUFSIZE_LARGE buffers
    fs_incoreTune->sacMinAvailable = (available / EEE_BUFSIZE_LARGE) / 100 * (100 - FS_BUF_USER_PCT_MEM_MAX);  
    // The minumum buffers we keep around
    fs_incoreTune->userBufMinInUse = ((available / FS_BLOCK_SIZE) / 100 * fs_incoreTune->bufUserPctMem);
    // the number of buffers we free at this moment
    // an additional buf will be reused 
    fs_incoreTune->sacFreeCount = 16;
    fs_incoreTune->metaFreeCount = 8;

#ifdef FS_STANDALONE
    available = (1 * 1024 * 1024);
#else
    available = eee_getTotalNbrOfBufs(EEE_BUF_ULTRA); 
#endif
    fs_incoreTune->ultraMinAvailable = available / 100 * (100 - FS_BUF_META_PCT_MEM_MAX) + FS_BUF_META_RESERVED ;

#ifdef FS_STANDALONE
    available = (20 * 1024); // 20 M --> in K bytes
#else
    available = eee_ramTotalFlexSize() / 1024;
#endif
    fs_incoreTune->memMinAvailable = available / 100 * (100 - FS_PCT_MEM_MAX_USAGE);
    fs_incoreTune->bufFreeCount = 32;
    fs_incoreTune->inodeFreeCount = 32;

    // The minumum full size non-directory meta buffers we keep around
    fs_incoreTune->metaNonDirMinInUse = ((available / FS_BLOCK_SIZE) / 100 *
                                                    fs_incoreTune->bufMetaNonDirPctMem);
    // The minumum full size directory meta buffers we keep around
    fs_incoreTune->metaDirMinInUse = ((available / FS_BLOCK_SIZE) / 100 *
                                                    fs_incoreTune->bufMetaDirPctMem);
    // The minumum compact non-directory meta buffers we keep around
    fs_incoreTune->metaCompNonDirMinInUse = ((available / FS_BLOCK_SIZE) / 100 *
                                              fs_incoreTune->bufMetaCompNonDirPctMem);
    // The minumum compact directory meta buffers we keep around
    fs_incoreTune->metaCompDirMinInUse = ((available / FS_BLOCK_SIZE) / 100 *
                                              fs_incoreTune->bufMetaCompDirPctMem);

    // The following parameters are tunable for each of the resource gate keeper
    // In general we can define an absolute value, a relative percentage, and a 
    // use factor related to the load (defined as number of active contexts). 
    // We will drop the request if:
    // The available_resource < absolute_minimum_required +
    //                          relative_minimum_required +
    //                          nbr_of_active_contexts * required_minimum_per_context;
    // e.g: user bufs: available_resource = usedBlockLimit - dirtyBlocks
    //                 absolute_minimum_required, e.g.:  20 (at least 20 user bufs must be available)
    //                 relative_minimum_required, e.g.:  10 (at least 10 % must be available)
    //                 required_minimum_per_context, e.g.: 8 (per context, a assume up to 8 user bufs
    //                                                        could be used)
    // If some parameter is not used: just set to 0.

    //  log space
    fs_incoreTune->resource[FS_RESOURCE_LOG].absMin = 0;                // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_LOG].relMin = 38;               // 30 % - 38 FOR 128
    fs_incoreTune->resource[FS_RESOURCE_LOG].loadFactor = 0;            // required_minimum_per_context in Bytes
    strcpy(fs_resourceNames[FS_RESOURCE_LOG], "Log            ");
    //  User buffer cache
    fs_incoreTune->resource[FS_RESOURCE_USERBUFCACHE].absMin = 20;      // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_USERBUFCACHE].relMin = 0;       // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_USERBUFCACHE].loadFactor = 8;   // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_USERBUFCACHE], "User Buf Cache ");
    //  Meta buffer cache full size nondir
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_NONDIR].absMin = 100;      // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_NONDIR].relMin = 0;       // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_NONDIR].loadFactor = 8;   // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_METABUFCACHE_NONDIR], "Meta Cache Nondir");
    //  Meta buffer cache compact nondir
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR].absMin = 0;      // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR].relMin = 0;       // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR].loadFactor = 0;   // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_METABUFCACHE_COMP_NONDIR], "Meta Cache Compact Nondir");
    //  Meta buffer cache directories
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_DIR].absMin = 100;      // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_DIR].relMin = 0;       // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_DIR].loadFactor = 1;   // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_METABUFCACHE_DIR], "Meta Cache Dir");
    //  Meta buffer cache compact directories
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_DIR].absMin = 100;      // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_DIR].relMin = 0;       // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_METABUFCACHE_COMP_DIR].loadFactor = 8;   // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_METABUFCACHE_COMP_DIR], "Meta Cache Compact Dir");
    //  Inode cache
    fs_incoreTune->resource[FS_RESOURCE_INODECACHE].absMin = 100;       // absolute_minimum_required 
    fs_incoreTune->resource[FS_RESOURCE_INODECACHE].relMin = 0;         // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_INODECACHE].loadFactor = 2;     // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_INODECACHE], "Inode Cache    ");
    //  memory, in K Bytes
    fs_incoreTune->resource[FS_RESOURCE_MEM].absMin = 0;                // absolute_minimum_required in K Bytes
    fs_incoreTune->resource[FS_RESOURCE_MEM].relMin = 4;                // 4 % --> 5 FOR 128, see FS_PCT_MEM_MAX_USAGE
    fs_incoreTune->resource[FS_RESOURCE_MEM].loadFactor = 0;           // required_minimum_per_context in K Bytes

#ifdef FS_STANDALONE
	fs_incoreTune->resource[FS_RESOURCE_MEM].totalMinusRelAbs =
		(uint32)((50 * 1024 * 1024) / 1024) -
		(uint32)((50 * 1024 * 1024) / 1024) / 128 *
		fs_incoreTune->resource[FS_RESOURCE_MEM].relMin -
		fs_incoreTune->resource[FS_RESOURCE_MEM].absMin;
#else  // STANDALONE

#ifdef FS_USE_MALLOC_SLAB
	fs_incoreTune->resource[FS_RESOURCE_MEM].totalMinusRelAbs =
		(uint32)((mm_pool_end - mm_pool_start) / 1024) -
		(uint32)((mm_pool_end - mm_pool_start) / 1024) / 128 *
		fs_incoreTune->resource[FS_RESOURCE_MEM].relMin -
		fs_incoreTune->resource[FS_RESOURCE_MEM].absMin;
#else // FS_USE_MALLOC_SLAB
	fs_incoreTune->resource[FS_RESOURCE_MEM].totalMinusRelAbs =
		(uint32)(eee_ramTotalFlexSize() / 1024) -
		(uint32)(eee_ramTotalFlexSize() / 1024) / 128 *
		fs_incoreTune->resource[FS_RESOURCE_MEM].relMin -
		fs_incoreTune->resource[FS_RESOURCE_MEM].absMin;
#endif // FS_USE_MALLOC_SLAB

#endif // STANDALONE
		
    strcpy(fs_resourceNames[FS_RESOURCE_MEM], "Memory         ");
	
    //  SAC
    fs_incoreTune->resource[FS_RESOURCE_SAC].absMin = 32;               // absolute_minimum_required
    fs_incoreTune->resource[FS_RESOURCE_SAC].relMin = 13;               // 10 % --> 13 FOR 128
    fs_incoreTune->resource[FS_RESOURCE_SAC].loadFactor = 8;            // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_SAC], "SAC            ");
    //  Ultra Buf on the FS CPU
    fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].absMin = 32;          // absolute_minimum_required
    fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].relMin = 0;           // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].loadFactor = 8;       // required_minimum_per_context
	fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].totalMinusRelAbs =
		eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) -
		eee_getTotalNbrOfBufs(EEE_BUF_ULTRA) / 128 *
		fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].relMin -
		fs_incoreTune->resource[FS_RESOURCE_ULTRABUF].absMin;
    strcpy(fs_resourceNames[FS_RESOURCE_ULTRABUF], "Ultra Buf      ");

    //  Nbr of Vol log buffers in use per FS
    fs_incoreTune->resource[FS_RESOURCE_VOL_LOG].absMin = VL_BUFFER_LOG_MAX - VL_BUFFER_LOG_HWM; 
                                                                       // absolute_minimum_required
    fs_incoreTune->resource[FS_RESOURCE_VOL_LOG].relMin = 0;           // relative_minimum_required in %
    fs_incoreTune->resource[FS_RESOURCE_VOL_LOG].loadFactor = 1;       // required_minimum_per_context
    strcpy(fs_resourceNames[FS_RESOURCE_VOL_LOG],  "Vol log Buf    ");
}

/*
 * CRC-32
 */
uint32 crc32[] = {
    0x00000000, 0x77073096, 0xee0e612c, 0x990951ba, 0x076dc419, 0x706af48f,
    0xe963a535, 0x9e6495a3, 0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
    0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91, 0x1db71064, 0x6ab020f2,
    0xf3b97148, 0x84be41de, 0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
    0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec, 0x14015c4f, 0x63066cd9,
    0xfa0f3d63, 0x8d080df5, 0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
    0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b, 0x35b5a8fa, 0x42b2986c,
    0xdbbbc9d6, 0xacbcf940, 0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
    0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116, 0x21b4f4b5, 0x56b3c423,
    0xcfba9599, 0xb8bda50f, 0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
    0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d, 0x76dc4190, 0x01db7106,
    0x98d220bc, 0xefd5102a, 0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
    0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818, 0x7f6a0dbb, 0x086d3d2d,
    0x91646c97, 0xe6635c01, 0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
    0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457, 0x65b0d9c6, 0x12b7e950,
    0x8bbeb8ea, 0xfcb9887c, 0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
    0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2, 0x4adfa541, 0x3dd895d7,
    0xa4d1c46d, 0xd3d6f4fb, 0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
    0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9, 0x5005713c, 0x270241aa,
    0xbe0b1010, 0xc90c2086, 0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
    0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4, 0x59b33d17, 0x2eb40d81,
    0xb7bd5c3b, 0xc0ba6cad, 0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
    0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683, 0xe3630b12, 0x94643b84,
    0x0d6d6a3e, 0x7a6a5aa8, 0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
    0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe, 0xf762575d, 0x806567cb,
    0x196c3671, 0x6e6b06e7, 0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
    0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5, 0xd6d6a3e8, 0xa1d1937e,
    0x38d8c2c4, 0x4fdff252, 0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
    0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60, 0xdf60efc3, 0xa867df55,
    0x316e8eef, 0x4669be79, 0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
    0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f, 0xc5ba3bbe, 0xb2bd0b28,
    0x2bb45a92, 0x5cb36a04, 0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
    0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a, 0x9c0906a9, 0xeb0e363f,
    0x72076785, 0x05005713, 0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
    0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21, 0x86d3d2d4, 0xf1d4e242,
    0x68ddb3f8, 0x1fda836e, 0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
    0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c, 0x8f659eff, 0xf862ae69,
    0x616bffd3, 0x166ccf45, 0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
    0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db, 0xaed16a4a, 0xd9d65adc,
    0x40df0b66, 0x37d83bf0, 0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
    0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6, 0xbad03605, 0xcdd70693,
    0x54de5729, 0x23d967bf, 0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
    0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
};


/**
 * CRC-32 a single byte.
 */
inline uint32
fs_checksum_byte(
    uint32  crc,
    char    ch)
{
    return crc32[(crc ^ ch) & 0xff] ^ (crc >> 8);
}


/**
 * CRC-32 the specified buffer.
 */
uint32
fs_checksum(
    void    *ptr,
    int32   len,
	uint32  crc)
{
    char    *charPtr = ptr;
    uint32  i;

    for (i = 0; i < len; i++)
    {
        crc = fs_checksum_byte(crc, *(charPtr + i));
    }

    return crc;
}


/**
 * Mix function for Bob Jenkin's hash function.
 *
 *  His comments:
 *  mix 3 64-bit values reversibly.
 *  All 1-bit deltas, all 2-bit deltas, all deltas composed of top bits of
 *  (a,b,c), and all deltas of bottom bits were tested.  All deltas were
 *  tested both on random keys and on keys that were nearly all zero.
 *  These deltas all cause every bit of c to change between 1/3 and 2/3
 *  of the time (well, only 113/400 to 287/400 of the time for some
 *  2-bit delta).  These deltas all cause at least 80 bits to change
 *  among (a,b,c) when the mix is run either forward or backward (yes it
 *  is reversible).
 *  This implies that a hash using mix64 has no funnels.  There may be
 *  characteristics with 3-bit deltas or bigger, I didn't test for
 *  those. 
 */
#define FS_CORE_HASH_MIX_64(a,b,c)      \
{                                       \
  a -= b; a -= c; a ^= (c>>43);         \
  b -= c; b -= a; b ^= (a<<9);          \
  c -= a; c -= b; c ^= (b>>8);          \
  a -= b; a -= c; a ^= (c>>38);         \
  b -= c; b -= a; b ^= (a<<23);         \
  c -= a; c -= b; c ^= (b>>5);          \
  a -= b; a -= c; a ^= (c>>35);         \
  b -= c; b -= a; b ^= (a<<49);         \
  c -= a; c -= b; c ^= (b>>11);         \
  a -= b; a -= c; a ^= (c>>12);         \
  b -= c; b -= a; b ^= (a<<18);         \
  c -= a; c -= b; c ^= (b>>22);         \
}   
  

/**
 * Hash courtesy of Bob Jenkins, find it on the web.
 *
 * Input must be pointer to array of uint64's, length
 * indicates number of uint64's in the array.
 */
uint64
fs_coreHash(
    void    *input, // key to be hashed
    uint64  length, // length of input in uint64's
    uint64  level)  // previous hash or arbitrary value
{
    uint64  a, b, c, len;
    uint64  *k = input;
  
    // Set up the internal state.
    len = length;
    a = b = level;              // the previous hash value
    c = 0x9e3779b97f4a7c13LL;   // the golden ratio; an arbitrary value
  
    // Handle most of the key.
    while (len >= 3)
    {
        a += k[0];
        b += k[1];
        c += k[2];
        FS_CORE_HASH_MIX_64(a, b, c);
        k += 3; len -= 3;
    } 
  
    // Handle the last 2 uint64's
    c += length;
    switch(len)              // all the case statements fall through
    {
        // c is reserved for the length
        case  2: b+=k[1];
        case  1: a+=k[0];
        // case 0: nothing left to add
    }

    FS_CORE_HASH_MIX_64(a, b, c);

    return c;
}   


/*-----------------------------------------------------------------------------
 * Tracing.  Each FP has a separate trace level variable.
 */

char *fs_traceLvlName[FS_TRACE_LVL_MAX] =
{
    "req",
    "buf",
    "ino",
    "lck",
    "log",
    "ior",
    "spa",
    "snp",
    "cod",
    "ver",
    "vol",
    "dir",
    "dump_restore",
};


/**
 * Return the FP CPU ID.
 */
static inline int32 fs_getCpuId(void)
{
#if defined(FS_STANDALONE)
    return (int32) pthread_getspecific(fs_simKeyCpuId);
#else
    return nfxMyCPUId;
#endif    
}


/**
 * Use elog to print/send a log message, if the right flags are set.
 */
#define FS_LOG_STRING_MAX 1024

extern void
fs_contextLog (
    fs_Context      *context,
    int              severity, 
    int              app_type, 
    int              line, 
    char            *file,
    char            *msg)
{
    evm_volId_t volId;
    char        *the_msg;
    char        *s;
    uint32      this, remain = FS_LOG_STRING_MAX - 1;
    int32       flags = elog_appl_data[fs_applicationId].bitmap;
//    int32       fpNum = fs_getCpuId() % 2;

    if ((severity >= LOG_DEBUG) && ((flags & (1 << app_type)) == 0))
    {
        return ;
    }

    if (context == NULL)
    {
        e_log (class_1, severity, fs_applicationId, app_type, 0, 0, 
               line, file, msg);
        return ;
    }

    volId = context->volId;

    the_msg =  (char *)eee_ramAlloc(FS_LOG_STRING_MAX);
    s = the_msg;
    
    if (context->fileSystem &&
        context->fileSystem->super &&
        context->fileSystem->super->mountName)
    {
        this = snprintf(s, remain, "FS: %-12s ", context->fileSystem->super->mountName);
    }
    else
    {
        this = snprintf(s, remain, "FS: %-12s ", " ");
    }
    s += this;
    remain -= this;

    this = snprintf(s, remain, "0x%llx - ", context->volId.whole_thing);
    s += this;
    remain -= this;

    //printf("Context at %p:\n", context);
    if (context->reqHdr != NULL)
    {
        this = snprintf(s, remain, "%s - ", fs_reqTypeTbl[context->reqHdr->requestType].name);
        s += this;
        remain -= this;
    }

    if ((app_type >= 0) && (app_type < FS_TRACE_LVL_MAX))
    {
        this = snprintf(s, remain, "%s - ", fs_traceLvlName[app_type]);
        s += this;
        remain -= this;
    }
    
    if ((flags & (1 << FS_TRACE_VERBOSE)) != 0)
    {
        this = snprintf(s, remain, "esm 0x%07llx - ", (uint64)context->smHandle);
        s += this;
        remain -= this;
    }
    
    // this = snprintf(s, remain, "fp%d ", fpNum);
    //printf("srcAddr %x\n", context->srcAddr);
    //printf("respHdr @ %p, size %d\n", context->respHdr, context->respSize);
    //printf("initRespSize %d\n", context->initRespSize);
    //printf("reqInfo @ %p\n", context->reqInfo);
    //printf("logSummary @ %p\n", context->logSummary);
    //printf("arena @ %p\n", context->arena);
    //printf("randomSeed %lld\n", context->randomSeed);
    //printf("flags %x\n", context->flags);

    this = snprintf(s, remain, "%s", msg);
    s += this;
    remain -= this;
    
    e_log (class_1, severity, fs_applicationId, app_type, 0, 0, 
           line, file, the_msg);

    eee_ramDealloc (the_msg);                                              \

	return ;
}

/**
 * Id's for applications we communicate with.
 */
int32 fs_applicationId;
int32 fs_srcPortId;
int32 fs_efsSscAppId;
int32 fs_nfsAppId;
int32 fs_mcpuDestPortId;
int32 fs_mcpuAppId;


/*-----------------------------------------------------------------------------
 * Help for tracking mem allocs/deallocs.
 */
uint32 fs_ramStats[FS_RAM_STATS_HOW_MAX][FS_RAM_TYPE_MAX];
nfx_spinlock_t fs_ramStatsSpinId;

#ifdef FS_USE_MALLOC_SLAB
struct mm_cache *fs_compact_leaf_cache;
struct mm_cache *fs_bufptr_cache;
struct mm_cache *fs_bufpagehdr_cache;
struct mm_cache *fs_context_cache;
struct mm_cache *fs_inode_cache;
struct mm_cache *fs_logsummary_cache;
struct mm_cache *fs_vl_unstableFH_cache;
struct mm_cache *fs_log_buf_cache;
struct mm_cache *fs_log_undo_summ_cache;
struct mm_cache *fs_waiter_cache;
#endif

extern void
fs_initAlloc(void)
{
#ifdef FS_USE_MALLOC_SLAB
    fs_compact_leaf_cache =
        cache_create(0,
                     FS_COMPACT_PAGE_SIZE,
                     64,
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_bufptr_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_Buf),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_bufpagehdr_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_BufPageHdr),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_vl_unstableFH_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(vl_unstableFileHandle),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_log_buf_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(vl_VolBuf),
                     sizeof(void *),
                     FS_VOL_LOG_SLAB_SIZE,
                     NULL, NULL);

    fs_log_undo_summ_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_LogUndoSummary),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_waiter_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_Waiter),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_logsummary_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_LogSummary),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_inode_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_Inode),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     NULL, NULL);

    fs_context_cache =
        cache_create(CACHE_INMEM_SLAB,
                     sizeof(fs_Context),
                     sizeof(void *),
                     FS_DEFAULT_SLAB_SIZE,
                     fs_threadContextCtor,
                     fs_threadContextDtor);
#endif
}

/*
 * dump mem usage
 */
extern void
fs_dumpAlloc(int32 what, int32 numBytes)
{
    int i;
   
    printf("\nType                       Alloc       Free       Diff\n");
    printf("------------------------------------------------------\n");
    for (i = 0; i < FS_RAM_TYPE_MAX; i++)
    {
        printf("%d %10d %10d %10d\n", i,
               fs_ramStats[FS_RAM_STATS_ALLOC][i],
               fs_ramStats[FS_RAM_STATS_DEALLOC][i],
               fs_ramStats[FS_RAM_STATS_ALLOC][i] -
               fs_ramStats[FS_RAM_STATS_DEALLOC][i]);
    }
    printf("\n");

    fs_error(NULL, FS_RAM_ALLOC_ERR, "fs_ramAlloc(%d, %d)",
             what, numBytes);
}



void
fs_threadContextCtor(void *o, uint32 size)
{
	fs_Context *context = o;

	context->randomSeed = 0;
	context->pendingFreeExtents = NULL;
	context->eek.inUse = FALSE;
	context->arena = NULL;
	context->logSummary = NULL;
	context->reqInfo = NULL;
    context->daemonType = FS_DAEMON_TYPE_INVALID;
    context->use83Names = FALSE;
    context->caseInsensitive = FALSE;
    context->hashCollides = FALSE;
	DLL_ISOLATE_ELEM(context, fsEntry);
    context->sharedLockSize = 0;
    context->sharedLockCount = 0;
    context->sharedLocksPtr = NULL;
}


void
fs_threadContextDtor(void *o)
{
    fs_Context *context = o;
    assert (context->sharedLocksPtr == NULL);
}

void
fs_threadContextReinit(fs_Context *context)
{
	fs_threadContextDtor(context);
	bzero(context, sizeof(*context));
	fs_threadContextCtor(context, sizeof(fs_Context));
}

#if !defined(FS_STANDALONE)
/*------------------------------------------------------------------------------
 * Name         : fs_postEvt
 *
 * Description  : Send an event.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 30-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

void
fs_postEvt(fs_Context * context,
           uint32       event,
           int			state)
    
{
    nfx_eventMsg_t      evtMsg;
    
    /* Fill event.
     */
    bzero((void *) &evtMsg, sizeof(nfx_eventMsg_t));
    evtMsg.msgType = NFX_EVENT_POST;
    //evtMsg.nodeName;
    evtMsg.needAck = FALSE;
    //evtMsg.status;
    evtMsg.eventScope = NFX_EVENT_LOCAL;
    evtMsg.eventType = NFX_EVENT_VOLUME;
    //eventType.reason;
    evtMsg.param.volEvent.event = event;
    evtMsg.param.volEvent.volId = context->volId;
    evtMsg.param.volEvent.state = state;
    if ((context->fileSystem != NULL) && (context->fileSystem->super != NULL))
      {
	strcpy(evtMsg.param.volEvent.volName,
	       context->fileSystem->super->mountName);
      }
    
    /* Post event.
     */
    event_postEvent(&evtMsg);
}
#endif

#endif // SSC

