/*
 * Name:  fs-global.h
 *
 * RCS:  $Id: fs-global.h,v 1.155 2002/12/10 18:42:46 bob Exp $
 *
 * Description:  Filesystem global definitions.
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

#if !defined(FS_GLOBAL_H)
#define FS_GLOBAL_H

/**
 * Cache sizes.

 */
enum {
    FS_ICACHE_HASH_MAX          = (4 * 1024 * 1024),  // if 4G 
    FS_ICACHE_HASH_SHIFT        = 22,
    // FS_ICACHE_HASH_MAX          = (128 * 1024),
    // FS_ICACHE_HASH_SHIFT        = 17,

    FS_BUFLIST_HASH_MAX         = (1024 * 1024), // if 4G
    FS_BUFLIST_HASH_SHIFT       = 20,
    // FS_BUFLIST_HASH_MAX         = (64 * 1024),
    // FS_BUFLIST_HASH_SHIFT       = 16,

    FS_DEFER_ALLOC_HASH_MAX     = 64,
    FS_QCACHE_HASH_MAX	        = 256,
    FS_FILESET_CACHE_HASH_MAX	= 16,
    FS_ICACHE_COW_HASH_MAX      = 256,

    FS_FILE_SYSTEM_HASH_MAX     = 256,

    FS_SDESC_HASH_MAX           = 1024,

    FS_DNLC_USED_MAX            = (1024 * 1024),
    FS_DNLC_HASH_MAX            = (1024 * 1024),
    FS_DNLC_HASH_SHIFT          = 20,
};


/**
 * Error statistics
 */
typedef enum {
    FS_RESOURCE_LOG = 0,
    FS_RESOURCE_USERBUFCACHE,
    FS_RESOURCE_METABUFCACHE_NONDIR,
    FS_RESOURCE_METABUFCACHE_COMP_NONDIR,
    FS_RESOURCE_METABUFCACHE_DIR,
    FS_RESOURCE_METABUFCACHE_COMP_DIR,
    FS_RESOURCE_INODECACHE,
    FS_RESOURCE_MEM,
    FS_RESOURCE_SAC,
    FS_RESOURCE_ULTRABUF,
    FS_RESOURCE_VOL_LOG,
    FS_RESOURCE_NO_ESM,
    FS_RESOURCE_IO,
    FS_RESOURCE_TYPE_MAX
} fs_IncoreResourceType;
typedef struct fs_ErrStats
{
    uint32 resource[FS_RESOURCE_TYPE_MAX];    // resource errors
} fs_ErrStats;

extern fs_ErrStats     fs_errStats;
extern nfx_spinlock_t  fs_errStatsSpinId;


/**
 * Parameters for tuning various in-core thresholds.
 */
#define FS_RESOURCE_NAME_MAX_SIZE   20
typedef struct fs_IncoreTuneResource
{
    uint32  absMin;             // the absolute min value needed to accept req
    uint32  relMin;             // the relative (%) min value needed to accept req
    uint32  loadFactor;         // the min value related to the load factor (nbr of contexts)
	uint32  totalMinusRelAbs; // cache total - total / 128 * relMin - absMin
} fs_IncoreTuneResource;

char    fs_resourceNames[FS_RESOURCE_TYPE_MAX][FS_RESOURCE_NAME_MAX_SIZE];


typedef struct fs_IncoreTune
{
#define FS_TUNE_inodePctAvailLoWtrMk    0
    int32   inodePctAvailLoWtrMk; // Start the inode cleaner thread when
                                  // available inodes crosses this threshold.
#define FS_TUNE_inodePctAvailHiWtrMk    1
    int32   inodePctAvailHiWtrMk; // Quiesce the inode cleaner thread when
                                  // available inodes crosses this threshold.
#define FS_TUNE_logPctUseLoWtrMk        2
    int32   logPctUseLoWtrMk;   // Start writeback/cleaner when in use
                                // log space crosses this threshold.
#define FS_TUNE_logPctUseHiWtrMk        3
    int32   logPctUseHiWtrMk;   // Quiesce writeback/cleaner when in use
                                // log space crosses this threshold.
#define FS_TUNE_bufPctAvailLoWtrMk      4
    int32   bufPctAvailLoWtrMk; // Start writeback when available bufs
                                // crosses this threshold.
#define FS_TUNE_bufPctAvailHiWtrMk      5
    int32   bufPctAvailHiWtrMk; // Quiesce writeback when available bufs
                                // crosses this threshold.
#define FS_TUNE_memPctAvailMin          6
    int32   memPctAvailMin;     // Block new requests when available heap
                                // space gets below this threshold.
#define FS_TUNE_bufWritesPerEvent       7
    int32   bufWritesPerEvent;  // Yield back the CPU when this many buf
                                // writes have been scheduled.
#define FS_TUNE_writesPendingMax        8
    int32   writesPendingMax;   // Stop scheduling new writeback requests
                                // when this many are outstanding.
#define FS_TUNE_perFsIoPendingMax       9
    int32   perFsIoPendingMax;  // Stop scheduling new writeback requests
                                // when this many IO are outstanding per fs.
#define FS_TUNE_inodeCleanPerPass       10
    int32   inodeCleanPerPass;  // The max nbr of inodes cleaned per pass

#define FS_TUNE_activeTransToWait       11
    int32   activeTransToWait;  // The nbr of active transactions ongoing
                                // to avoid IO the log buffer.

#define FS_TUNE_maxCpuUsageUsec         12
    eeeRTC_t maxCpuUsageRTC;    // max nbr of time before yielding in rtc form

    // skip 13, to indicate the uint32 (used for alignment)

#define FS_TUNE_inodePctMem             14
    int32   inodePctMem;        // Percent of avail mem used for inodes.

#define FS_TUNE_wbSleepMsecs            15
    int32   wbSleepMsecs;       // Time to sleep writeback daemon if
                                // need to reschedule due to max io pending.

#define FS_TUNE_readdirMaxSpace         16
    int32   readdirMaxSpace;    // use this space for readdir to reduce
                                // cpu overhead of individual readdir requests.
                                // Obviously, this can cause additional delays
                                // on the client. Its intended for special case only. 

#define FS_TUNE_userDirtyThreshold      17
    int32   userDirtyThreshold; // Start IO writeback on the specified inode's 
#define FS_TUNE_metaDirtyThreshold      18
    int32   metaDirtyThreshold; // dirty page threshold
    
#define FS_TUNE_enableReadAhead         19
    int32   enableReadAhead;
#define FS_TUNE_readAheadMaxSize        20
    int32   readAheadMaxSize;
#define FS_TUNE_readAheadDefaultSize    21
    int32   readAheadDefaultSize;

#define FS_TUNE_enableAtimeUpdate       22
    int32   enableAtimeUpdate;

#define FS_TUNE_vollogIoMax             23
    int32   vollogIoMax;        // Maximum number of buffers which can be simultaneous 
                                // in the IO state

#define FS_TUNE_vollogLoopBack          24
    int32   vollogLoopBack;     // Do loopback on the FC NIM for vol log writes
                                // Can only be enabled for special test purposes

#define FS_TUNE_sacMinAvailable         25
    int32   sacMinAvailable;    // the min nbr of free sac buffers, 
                                // to allow the grow of the usedBlockLimit
#define FS_TUNE_sacFreeCount            26
    int32   sacFreeCount;       // How many cod's are freed when usedblocklimit is reached

#define FS_TUNE_ultraMinAvailable       27
    int32   ultraMinAvailable;  // the min nbr of ultra buffers left available 

#define FS_TUNE_memMinAvailable         28
    int32   memMinAvailable;    // the min K bytes left available

#define FS_TUNE_bufFreeCount            29
    int32   bufFreeCount;       // How many bufs are freed when limit is reached
#define FS_TUNE_inodeFreeCount            30
    int32   inodeFreeCount;       // How many inodes are freed when limit is reached

#define FS_TUNE_userBufMinInUse         31
    int32   userBufMinInUse;    // Min nbr of user buf which will stay in use 

#define FS_TUNE_logCommitHiWtrMk        32
    int32   logCommitHiWtrMk;   //

#define FS_TUNE_bufUserPctMem           33
    int32   bufUserPctMem;      // Percent of the SAC used for user bufs.

#define FS_TUNE_bufMetaNonDirPctMem     34
    int32   bufMetaNonDirPctMem;// Percent of avail mem used for full sized
                                // non-directory meta bufs.
#define FS_TUNE_bufMetaDirPctMem        35
    int32   bufMetaDirPctMem;   // Percent of avail mem used for full
                                // sized directory meta bufs.
#define FS_TUNE_bufMetaCompNonDirPctMem 36
    int32   bufMetaCompNonDirPctMem;// Percent of avail mem used for compact
                                // non-directory meta bufs.
#define FS_TUNE_bufMetaCompDirPctMem    37
    int32   bufMetaCompDirPctMem;   // Percent of avail mem used for compact
                                // directory meta bufs.
#define FS_TUNE_metaFreeCount           38
    int32   metaFreeCount;      // How many meta bufs are freed when usedblocklimit
                                // is reached
#define FS_TUNE_metaDirMinInUse         39
    int32   metaDirMinInUse;    // Min nbr of full sized directory bufs which will 
                                // stay in use
#define FS_TUNE_metaNonDirMinInUse      40
    int32   metaNonDirMinInUse; // Min nbr of full sized non-directory bufs which will
                                // stay in use.
#define FS_TUNE_metaCompDirMinInUse     41
    int32   metaCompDirMinInUse;    // Min nbr of compact directory bufs which will 
                                // stay in use
#define FS_TUNE_metaCompNonDirMinInUse  42
    int32   metaCompNonDirMinInUse; // Min nbr of compact non-directory bufs which will
                                // stay in use.

    // int32   pad[1];             // padding, not needed

#define FS_TUNE_RESOURCE                43

    // see fs-global.c for more comments 
    fs_IncoreTuneResource   resource [FS_RESOURCE_TYPE_MAX];
} fs_IncoreTune;
extern fs_IncoreTune *fs_incoreTune;

enum {
    FS_TUNE_ABS_LOG = FS_TUNE_RESOURCE,
    FS_TUNE_REL_LOG,
    FS_TUNE_LOAD_LOG,

    FS_TUNE_ABS_UBC,
    FS_TUNE_REL_UBC,
    FS_TUNE_LOAD_UBC,

    FS_TUNE_ABS_MBC_ND,
    FS_TUNE_REL_MBC_ND,
    FS_TUNE_LOAD_MBC_ND,

    FS_TUNE_ABS_MBC_CND,
    FS_TUNE_REL_MBC_CND,
    FS_TUNE_LOAD_MBC_CND,

    FS_TUNE_ABS_MBC_D,
    FS_TUNE_REL_MBC_D,
    FS_TUNE_LOAD_MBC_D,

    FS_TUNE_ABS_MBC_CD,
    FS_TUNE_REL_MBC_CD,
    FS_TUNE_LOAD_MBC_CD,

    FS_TUNE_ABS_INC,
    FS_TUNE_REL_INC,
    FS_TUNE_LOAD_INC,

    FS_TUNE_ABS_MEM,
    FS_TUNE_REL_MEM,
    FS_TUNE_LOAD_MEM,

    FS_TUNE_ABS_SAC,
    FS_TUNE_REL_SAC,
    FS_TUNE_LOAD_SAC,

    FS_TUNE_ABS_ULT,
    FS_TUNE_REL_ULT,
    FS_TUNE_LOAD_ULT,

    FS_TUNE_ABS_VOL,
    FS_TUNE_REL_VOL,
    FS_TUNE_LOAD_VOL,
};


/**
 * Parameters for various in-core statistics.
 */
#define FS_INCORE_STATS  1

#ifdef FS_INCORE_STATS

typedef struct fs_IncoreStatsRecord
{
    uint32               nbrDrop;       // number of times, a req was dropped due to this resource    
    uint32               maxUsed;
} fs_IncoreStatsRecord;


typedef struct fs_IncoreStats
{
    uint32                nbrDropNoResource;
    fs_IncoreStatsRecord  resource[FS_RESOURCE_TYPE_MAX];
} fs_IncoreStats;

extern fs_IncoreStats *fs_incoreStats;

#endif

/**
 * Defaults for fs_IncoreTune.
 */
#define FS_INODE_PCT_AVAIL_LO       25  // inodePctAvailLoWtrMk
#define FS_INODE_PCT_AVAIL_HI       35  // inodePctAvailHiWtrMk

#define FS_LOG_PCT_USE_HI           50  // logPctUseLoWtrMk
#define FS_LOG_PCT_USE_LO           40  // logPctUseHiWtrMk

#define FS_BUF_PCT_AVAIL_LO         25  // bufPctAvailLoWtrMk
#define FS_BUF_PCT_AVAIL_HI         35  // bufPctAvailHiWtrMk

#define FS_MEM_PCT_AVAIL_MIN        10  // memPctAvailMin
#define FS_BUF_WB_PER_EVENT         16  // bufWritesPerEvent
#define FS_INO_CLEAN_PER_PASS       16  // fs_inodeCleanBatch
#define FS_IO_WRITE_PENDING_MAX   2048  // writesPendingMax. Must be less
                                        // than MAX_QUEUE_ENTRIES in
                                        // fs-wb.sm. (this counts writes
                                        // only, whereas MAX_QUEUE_ENTRIES
                                        // counts reads too)
#define FS_PER_FS_IO_PENDING_MAX   256  // perFsIoPendingMax

#define FS_BUF_META_RESERVED       (9*1024) // reserved number of ultra buffers for polling rings
                                         // see BC1250_MAC_RX_Q_CNT * BC1250_MAC_RX_Q_LEN * 2 + CORE_RESERVE_DUMP

#ifdef FS_USE_MALLOC_SLAB
#define FS_DEFAULT_SLAB_SIZE       (8 * 1024)
#define FS_VOL_LOG_SLAB_SIZE       (64 * 1024)
#endif

#ifdef FS_DYN_CACHE_SIZE

#define FS_BUF_META_NONDIR_PCT_MEM      50  // bufMetaNonDirPctMem (ultra buf)
#define FS_BUF_META_COMP_NONDIR_PCT_MEM 10  // bufMetaCompNonDirPctMem (flex mem)
#define FS_BUF_META_DIR_PCT_MEM          2  // bufMetaDirPctMem (ultra buf)
#define FS_BUF_META_COMP_DIR_PCT_MEM    10  // bufMetaCompDirPctMem (flex mem)
#define FS_BUF_USER_PCT_MEM             30  // bufUserPctSac (see the available sac buffers at runtime)
#define FS_BUF_USER_PCT_MEM_MAX         90  // bufUserPctSac (see the available sac buffers at runtime)
#define FS_BUF_META_PCT_MEM_MAX         95  // max usage (%) of ultra buffers on the FP
#define FS_PCT_MEM_MAX_USAGE            85  // max usage (%) of malloc mem on the FP (see also [FS_RESOURCE_MEM].relMin) 
#define FS_INODE_PCT_MEM                10  // inodePctMem

#else  // FS_DYN_CACHE_SIZE

#define FS_BUF_META_NONDIR_PCT_MEM      80  // bufMetaNonDirPctMem (ultra buf)
#define FS_BUF_META_COMP_NONDIR_PCT_MEM 12  // bufMetaCompNonDirPctMem (flex mem)
#define FS_BUF_META_DIR_PCT_MEM          3  // bufMetaDirPctMem (ultra buf)
#define FS_BUF_META_COMP_DIR_PCT_MEM    16  // bufMetaCompDirPctMem (flex mem)
#define FS_BUF_USER_PCT_MEM             50  // bufUserPctSac (see the available sac buffers at runtime)
#define FS_BUF_USER_PCT_MEM_MAX         90  // bufUserPctSac (see the available sac buffers at runtime)
#define FS_BUF_META_PCT_MEM_MAX         90  // max usage (%) of ultra buffers on the FP
#define FS_PCT_MEM_MAX_USAGE            90  // max usage (%) of malloc mem on the FP (see also [FS_RESOURCE_MEM].relMin)
#define FS_INODE_PCT_MEM                20  // inodePctMem

#endif // FS_DYN_CACHE_SIZE

#define FS_SYNC_TRUNCATE_MAX_BLOCKS  8  // use trashcan if this many blocks

#define FS_DIRTY_THRESHOLD           8

#define FS_MAX_CPU_USAGE         20000  // max nbr of usec, before yielding the CPU
#define FS_WB_SLEEP_MSECS           20  // wbSleepMsecs

#define FS_MAX_READ_AHEAD_SIZE      64
#define FS_DEFAULT_READ_AHEAD_SIZE  16

#define VL_BUFFER_IO_MAX           200  // Maximum number of buffers which can be simultaneous 
                                        // in the IO state

#define FS_LOG_COMMIT_THRESHOLD     75  // percent of log which requires a mandatory flush of
                                        // dirty metadata and inode blocks to disk to remove dependencies.


/**
 * Buf inode types to account for space in the
 * buffer cache.
 */
enum {
    FS_BUF_CACHE_ITYPE_NONDIR,      // full size non-directory bufs
    FS_BUF_CACHE_ITYPE_DIR,         // full size directory bufs
    FS_BUF_CACHE_ITYPE_COMP_NONDIR, // compact non-directory bufs
    FS_BUF_CACHE_ITYPE_COMP_DIR,    // compact directory bufs
    FS_BUF_CACHE_ITYPE_TOTAL,       // totals
    FS_BUF_CACHE_ITYPE_MAX,
};


/**
 * Stats/counters for buffer cache.
 */
typedef struct fs_BufListCount
{
    uint32      usedBufs[FS_BUF_CACHE_ITYPE_MAX];           // Number of currently used bufs.
    uint32      freeBufs[FS_BUF_CACHE_ITYPE_MAX];           // Number of currently free/available bufs (mrulist).
    uint32      usedBufLimit[FS_BUF_CACHE_ITYPE_MAX];       // Max allowed buffers is system
#ifdef FS_DYN_CACHE_SIZE
    uint32      usedBufLimitLWM[FS_BUF_CACHE_ITYPE_MAX];    // usedBufLimit won't drop below this LWM
    uint32      usedBufLimitHWM[FS_BUF_CACHE_ITYPE_MAX];    // usedBufLimit won't increase above this HWM
#endif
    uint32      dirtyBufs[FS_BUF_CACHE_ITYPE_MAX];
    uint32      dirtyBufLimit[FS_BUF_CACHE_ITYPE_MAX];
    uint32      dirtyBufLowThresh[FS_BUF_CACHE_ITYPE_MAX];
    uint32      minInUse[FS_BUF_CACHE_ITYPE_MAX];
    uint32      missCount[FS_BUF_CACHE_ITYPE_MAX];
    uint32      hitCount[FS_BUF_CACHE_ITYPE_MAX];
    uint32      cowCount[FS_BUF_CACHE_ITYPE_MAX];

#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
    uint32      missCntType[FS_MAX_META_INODE];
    uint32      useCntType[FS_MAX_META_INODE]; 
#endif

} fs_BufListCount;


/**
 * Global buffer cache structure.
 */
typedef DLL_HDR(fs_BufPageHdr) fs_BufPageDLL;
typedef DLL_HDR(fs_Buf)        fs_BufDllHdr;
struct fs_BufListGlobal
{
    nfx_spinlock_t      spin;
    short16             type;       // META or USER buflist.
    fs_BufDllHdr        mruHdr[FS_BUF_CACHE_ITYPE_MAX];
                                    // Header for MRU buffer lists.
    fs_BufDllHdr        bufsBlk[FS_BUFLIST_HASH_MAX];
                                    // Ptr to hash buckets for bufs by
                                    // block number.
    fs_BufDllHdr        bufsPage[FS_BUFLIST_HASH_MAX];
                                    // Ptr to hash buckets for bufs by
                                    // inum and page.
    fs_BufPageDLL       bufsInum[FS_BUFLIST_HASH_MAX];
                                    // Ptr to hash buckets for bufs by
                                    // inode number.
#ifdef FS_BUF_BLK_CHECK_HASH_DEPTH
    ushort16            blkDepth[FS_BUFLIST_HASH_MAX];
#endif
#ifdef FS_BUF_PAGE_CHECK_HASH_DEPTH
    ushort16            pageDepth[FS_BUFLIST_HASH_MAX];
#endif
#ifdef FS_BUF_INUM_CHECK_HASH_DEPTH
    ushort16            inumDepth[FS_BUFLIST_HASH_MAX];
#endif

    fs_BufListCount     count;      // Counters for the cache.
};
extern fs_BufListGlobal fs_bufListGlobal[FS_BUFLIST_MAX];


/**
 * Per filesystem buffer cache structure.
 */
struct fs_BufList
{
    short16             type;       // META or USER buflist.

    fs_BufDllHdr        mrdHdr;     // Header for MRD buffer list.
    fs_BufDllHdr        mrdWaitingHdr;  // Header for dirty buffers that
				                    // a context is waiting for IO to occur.
    fs_BufDllHdr        logSegmentHdr[FS_LOG_MAX_SEGMENTS];  // Header for
                                    // dirty buffers by oldest log segment 
                                    // they dirty.
    DLL_HDR(fs_Waiter)  waiterHdr;  // contexts waiting for I/O to complete
    int32               numWaiters;
#define FS_WB_TIMEOUT   5000        // Every 5 seconds
    int32               wbTimerHandle;    // Writeback timer handle
    int32               tmpTimerHandle;   // Temporary writeback timer handle.
    boolean             wbTimerExists;
    boolean             tmpTimerExists;
    esm_handle_t        wbHandle;   // Writeback state machine handle.
    fs_Context          *wbContext; // Writeback state machine context ptr.
    int32               wbSignal;   // Indicator that wb daemon is waking up.

    fs_BufListCount     count;      // Counters for the cache.
};


/** 
 * Counters for inode cache.
 */
typedef struct fs_InodeCacheCount
{
    uint32  numCachedLimit;       // Max inodes allowed in cache.
#ifdef FS_DYN_CACHE_SIZE
    uint32  numCachedLimitLWM;    // usedBufLimit won't drop below this LWM
    uint32  numCachedLimitHWM;    // usedBufLimit won't increase above this HWM
#endif
    uint32  numCached;            // Number of inodes currently cached.
    uint32  numDirty;             // Number of inodes on dirty list.
    uint32  hitCount;
    uint64  numCleaned;
    uint32  missCount;
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
    uint32  missCntType[FS_MAX_META_INODE];
#endif
} fs_InodeCacheCount;

typedef DLL_HDR(fs_Inode)        fs_InodeDllHdr;

typedef struct fs_InodeCache
{
    nfx_spinlock_t      spin;           // Spinlock to protect inode cache.
                                        // TODO - one per hash chain?
    fs_InodeDllHdr      cache[FS_ICACHE_HASH_MAX]; // Hash chains of
                                        // in-core inodes.
#ifdef FS_INODE_CHECK_HASH_DEPTH
    ushort16            depth[FS_ICACHE_HASH_MAX];

    uint32              nbrLookups;
    uint32              nbrScanned;
#endif
    fs_InodeDllHdr      mruHdr;         // Header for MRU inode list.
                                        // allocation trimmed.
    fs_InodeDllHdr      cowHdr[FS_ICACHE_COW_HASH_MAX];
                                        // Headers for inodes that might need COW.
    fs_InodeCacheCount  count;          // Counters for the entire cache.
    int32               cleanerSignal;  // Wakeup indicator for cleaner thread.
    esm_handle_t        cleanerHandle;  // Handle for inode cleaner thread.
    int32               trimmerSignal;  // Wakeup indicator for trimmer thread.
    esm_handle_t        trimmerHandle;  // Handle for inode alloc trimmer
                                        // thread.
#define FS_INODE_SYNC_TIMEOUT   30000   // Every 30 seconds
    int32               syncTimer;      // Writeback timer

} fs_InodeCache;
extern fs_InodeCache *fs_inodeCache;


/*
 */
typedef struct
{
    nfx_spinlock_t      spin;
#define FS_BUDDY_FULL       0x0001
    byte                flags;
    byte                pad[3];
    fs_RegionSummary *  summary;
} fs_buddy_t;


/**
 * A security id is essentially a 32-byte
 * offset into the security file.
 */
#define FS_SDESC_ID_PER_PAGE    (FS_BLOCK_SIZE / sizeof(fs_SecurityDescNfs))
NFX_ASSERT_STATIC(FS_SDESC_ID_PER_PAGE == 256);

/** 
 * The security file is divided into
 * 4 equal parts, one for each of up to
 * 4 possible security types.
 */
enum {
    FS_SDESC_ID_START_NFS   = 0x00000000,
    FS_SDESC_ID_START_CIFS  = 0x40000000,
    FS_SDESC_ID_START_F1    = 0x80000000,    // Future use
    FS_SDESC_ID_START_F2    = 0xc0000000,    // Future use
};

enum {
    FS_SDESC_TYPE_INVALID   = 0,
    FS_SDESC_TYPE_NFS       = 1,
    FS_SDESC_TYPE_CIFS      = 2,
    FS_SDESC_TYPE_MAX       = 3,
};


/**
 * TODO: Move these to in-core tune.
 */
#define FS_SDESC_NUM_USED_LIMIT     (64 * 1024)
#define FS_SDESC_NUM_FREE_HWM       1024
#define FS_SDESC_NUM_FREE_LWM       512

/**
 * In-core cached security descriptor.
 */
struct fs_SecurityDescCore
{
    evm_volId_t     volId;
    uint16          attr;
    uint8           snapNum;
    uint8           spare1;
    uint32          secId;
    uint32          refCount;
    DLL_ENTRY(fs_SecurityDescCore)  secIdEntry;
    DLL_ENTRY(fs_SecurityDescCore)  uidEntry;
    DLL_ENTRY(fs_SecurityDescCore)  freeEntry;
    union {
        fs_SecurityDescHdr  hdr;
        fs_SecurityDescNfs  nfs;
        fs_SecurityDescCifs cifs;
    } u;
};


/**
 * Security descriptor cache.
 */
struct fs_SecurityDescCache
{
    nfx_spinlock_t      spin;
    DLL_HDR(fs_SecurityDescCore)    secIdHdr[FS_SDESC_HASH_MAX];
    DLL_HDR(fs_SecurityDescCore)    uidHdr[FS_SDESC_HASH_MAX];
    DLL_HDR(fs_SecurityDescCore)    freeHdr[FS_SDESC_TYPE_MAX];
    int32                           numUsed[FS_SDESC_TYPE_MAX];
    int32                           numUsedLimit[FS_SDESC_TYPE_MAX];
    int32                           numFree[FS_SDESC_TYPE_MAX];
    int32                           numReclaims[FS_SDESC_TYPE_MAX];
    uint32                          startId[FS_SDESC_TYPE_MAX];
};
extern fs_SecurityDescCache fs_securityDesc;


/**
 * A quota Id is essentially a record offset into the quota file.
 */


/**
 * Quota cache.
 */ 
typedef struct fs_QuotaCache
{
	nfx_spinlock_t		spin;

	uint32				quotaCacheLimit;	// max number of quotas to cache
	uint32				quotasCached;		// number of quotas cached
    uint32              hitCount;
    uint32              missCount;
	DLL_HDR(fs_QuotaIncoreRecord_t)	quotaFreeList;
    //user quota hash;
	DLL_HDR(fs_QuotaIncoreRecord_t) userQuotaHashList[FS_QCACHE_HASH_MAX];
    //group quota hash;
    DLL_HDR(fs_QuotaIncoreRecord_t) groupQuotaHashList[FS_QCACHE_HASH_MAX];
}fs_QuotaCache;
extern fs_QuotaCache fs_quotaCache;

    
/**
 * Statistics for dnlc.
 */
typedef struct fs_DnlcStats fs_DnlcStats;
struct fs_DnlcStats
{
    int32   usedLimit;
    int32   used;
    int32   hits;
    int32   hitsNegative;
    int32   missNotFound;
    int32   missTooBig;
    int32   missInode;
    int32   inserts;
    int32   removes;
    int32   setNegatives;
    int32   setPositives;
    int32   pad;
};

/**
 * The dnlc entry structure.
 */
typedef struct fs_DnlcEntry fs_DnlcEntry;
struct fs_DnlcEntry
{
#define FS_DNLC_NAME_MAX        14
    char                    name[FS_DNLC_NAME_MAX];
    char                    len;
    char                    negative;
    evm_volId_t             volId;
    uint64                  parentInumSnap;
    uint64                  inum;
    DLL_ENTRY(fs_DnlcEntry) nameEntry;
    DLL_ENTRY(fs_DnlcEntry) mruEntry;
};


/**
 * The dnlc structure.
 */
typedef struct fs_Dnlc fs_Dnlc;
struct fs_Dnlc
{
    nfx_spinlock_t          spin;
    fs_DnlcStats            stats;
    DLL_HDR(fs_DnlcEntry)   mruHdr;
    DLL_HDR(fs_DnlcEntry)   nameHdr[FS_DNLC_HASH_MAX];
    short16                 depth[FS_DNLC_HASH_MAX];
};
extern fs_Dnlc fs_dnlc;

/**
 * In-core global data for a filesystem.
 */
struct fs_FileSystem
{
    nfx_spinlock_t      spin;       // Lock for state, etc.
    fs_FileSystemState  state;      // State of mount for this volume.
    boolean             lightsOut;  // TRUE if in final phase of volException.
    evm_volId_t         volId;      // Volume this fs mounted on.
    int32               ioPendingCount; // Number of outstanding IO reqs.
	int32				logSize;	// Size in bytes of the log device.
    fs_LogFsRoot        *logRoot;   // Log associated with this fs.
    fs_LogDevTypeTable  *logDevTypeTable; // Pointer to table describing log
                                    // device type specific attributes.
    boolean             persistentUserBufCache;
                                    // We're using a persistent (eg. battery
                                    // backed) cache for user bufs.
    uint64              commitedBlksUsed;   // blocks in use
    uint64              commitedInodesUsed; // inodes in use
    uint64              minFreeBlks; // number of blocks reserved for internals
    fs_Parameters       params;     // File system size parameters.
    fs_SuperBlock       *super;     // Superblock.
    DLL_HDR(fs_Context) activeThreadHdr; // Threads currently active with
                                    // this filesystem.
    fs_InodeCacheCount  inodeCacheCount;// Counters for the inode cache.
    fs_InodeDllHdr      inodeMrdHdr;// Header for inodes that are dirty but
                                    // not (yet) on the logSegmentHdr.
    fs_InodeDllHdr      inodeLogSegmentHdr[FS_LOG_MAX_SEGMENTS];// Headers
                                    // for dirty inodes based on the
                                    // first log segment they're dirty in.
    fs_DnlcStats        dnlcStats;

#ifdef FS_VOLLOG_WAIT_FOR_ONGOING_TRANS
    int32   nbrOngoingTrans;      // The curr nbr of ongoing active transactions 
                                    // garanteed to write a commit record.
                                    // used to avoid IO the curr partial log buffer.
    int32   maxOngoingTrans;         // The max nbr of ongoing active transactions 
#endif
    DLL_HDR(fs_Waiter)  inodeWaiterHdr; // Header for contexts waiting on
                                        // inode cache sync.

    fs_BufList          bufList[FS_BUFLIST_MAX];
    esm_handle_t        trashcanHandle; // Handle for trashcan thread.
    int32               trashcanSignal; // Indicates trashcan thread has been
                                    // awakened.
    esm_handle_t        inodeCleanerHandle; // Handle for inode cleaner thread.
    int32               inodeCleanerSignal; // Indicates inode cleaner thread
                                    // has been awakened.
    
    fs_FileSystem       *next;      // Next FileSystem struct in hash bucket.

    /**
     * Allocator structures for mkfs time
     */
    fs_BootAllocator    *bootstrap;


    /**
     * Lock on modification operations.  Must be acquired shared by any
     * operation which will modify the file system, and exclusive by
     * any operation which needs to quiesce the system.
     */
    fs_Lock modifyLock;

    /**
     * Lock on all operations.  Must be acquired shared by any
     * operation which will access the file system, and exclusive by
     * any operation which needs to quiesce the system.
     */
    fs_Lock accessLock;

    /**
     * Lock around superblock count modification (blocks in use and
     * inodes in use).
     */
    nfx_spinlock_t superLock;

    /**
     * The state machine handle for the freezer thread, or zero if the
     * file system is not frozen.  May only be accessed while the
     * freezeLock is held.
     */
    esm_handle_t freezeHandle;

    /**
     * A lock to use solely while starting and stopping the freezer thread.
     */
    fs_Lock freezeLock;

    /**
     * Indicates if the thread to grow the inode file is running.
     */
    boolean igrowThreadStarted;

/* TODO REMOVE THIS. NOW QUOTA CACHE IS GLOBAL    
	nfx_spinlock_t		quotaListLock;		// protects quota lists

	uint32				quotaCacheLimit;	// max number of quotas to cache
	uint32				quotasCached;		// number of quotas to cached
	DLL_HDR(fs_QuotaIncoreRecord_t)	quotaFreeList;
	DLL_HDR(fs_QuotaIncoreRecord_t) userQuotaHashList[FS_QCACHE_HASH_MAX]; //user quota hash;
    DLL_HDR(fs_QuotaIncoreRecord_t) groupQuotaHashList[FS_QCACHE_HASH_MAX]; //user quota hash;
*/
	nfx_spinlock_t	filesetListLock;		// protects fileset lists
	uint32			filesetsCount;			// number of filesets
	uint32			filesetCacheLimit;		// max num of fileset to cache
	uint32			filesetsCached;			// number of filesets to cached
	DLL_HDR(fs_FileSetIncoreRecord_t)	filesetList;
	DLL_HDR(fs_FileSetIncoreRecord_t) fileSetHash[FS_FILESET_CACHE_HASH_MAX]; // fileset hash
    fs_Lock         dumpContextsLock;   // protects dumpContexts
    DLL_HDR(fs_DumpContext_t)   dumpContexts;   // list of active dump contexts
    fs_Lock         restoreContextsLock;   // protects dumpContexts
    DLL_HDR(fs_RestoreContext_t)   restoreContexts;   // list of active restore

    fs_NfsStats         nfsStats;           // nfs statistics for this fs.
    uint32              currentActiveReqs;  // number of active reqs
    uint32              maxActiveReqs;      // max number of active reqs
    fs_YieldStats       yieldStats;         // CPU yield stats for this fs.
    fs_RecvReqStats     recvReqStats;       // rec request stats for this fs
    fs_ReplyStats       replyStats;         // reply stats for this fs
    fs_ErrStats         errStats;           // error stats for this fs.

    nfx_spinlock_t fs_statsReqRcvdSpinId;

#if defined(FS_DEBUG_BUF_CKSUM)
    DLL_HDR(fs_BufCksum) bufCksumHdr[FS_BUFLIST_HASH_MAX];
    uint32          bufCksumNbrEntries[FS_BUFLIST_HASH_MAX];
#endif  // FS_DEBUG_BUF_CKSUM

    uchar8          *blockAllocShadow;
    uint32          blockAllocShadowSize;
    uchar8          *inodeAllocShadow;
    uint32          inodeAllocShadowSize;

    fs_AmSourceSession_t  *amSourceSession;
    fs_AmTargetSession_t  *amTargetSession;

    fs_SnapInfo    snapInfo;

    /* Pointer to all the buddy page of the filesystem.
     */
    fs_buddy_t *    buddies;

    nfx_spinlock_t  buddyBitmapSpin;        // protect buddyBitmap
    uint8           *buddyBitmap;           // buddyBitmap of valid regions
    uint32          buddyRegions;           // number of buddy regions
    esm_handle_t    buddyThreadHandle;      // background buddy recal thread

    fs_Lock         inodeLock;      // Lock for creating new page in inode file.

    uint32          ioTotalMetaWrites;
    uint32          ioTotalMetaReads;
    uint32          ioTotalMetaReadAheads;
    uint32          ioTotalUserWrites;
    uint32          ioTotalUserReads;
    uint32          ioTotalUserReadAheads;
    uint32          ioTotalLogWrites;
    uint32          ioTotalLogReads;
    uint32          ioWritesPending;
    uint32          ioMaxWritesPending;
    uint32          ioReadsPending;
    uint32          ioMaxReadsPending;

    /*
     * Additional flags that influence behaviour of the filesystem
     */
    uint16          flags;
#define FS_IGNORE_ATIME_UPDATE  0x0001

    uint8           percentMinFree; // percent free of filesystem before
                                    // grow is triggered
    eeeRTC_t        whenGrowSent;   // when grow was sent

#ifdef NEW_ZF
    // List of user data to zero.
    DLL_HDR(fs_zfInode)     zfInodeHdr;
    esm_handle_t            zfThreadHandle;
    nfx_spinlock_t          zfSpin;
#endif // NEW_ZF
};

extern uint32 fs_zfInodeHashTab;
extern uint32 fs_zfPageHashTab;

/*
 * File sets are looked up for their root inodes, to optimize lookup.
 * Searching for individual fsets to see if they are incore would have
 * to go through a very slow process of searching through all the hash lists.
 * The idea is that all the fileset structures would always be incore at
 * mount time. If a fileset is removed, the entry would be removed as well,
 * so searching for fileset on the basis of their fileset is going to be
 * a less often performed. We would sequentially scan the filesetList for
 * this purpose.
 */
 
#define	FS_FILESET_HASH(_inum_)		((_inum_) % FS_FILESET_CACHE_HASH_MAX)

#define FS_QCACHE_HASH(_id_)		((_id_) % FS_QCACHE_HASH_MAX)

/*-----------------------------------------------------------------------------
 * Accessors for FileSystem structures.
 */

/**
 * An array of the in-core root structures for filesystems.
 */
extern fs_FileSystem *fs_fileSystems[FS_FILE_SYSTEM_HASH_MAX];

/**
 * Initialize in-core tuneables.
 */
extern void fs_initIncoreTune();

/**
 * Initialize the file system table.
 */
extern void fs_initFileSystemTable();

/**
 * Return the FileSystem structure for a specified volume id, or NULL if the
 * volume is not known.  Holds the fs_fileSystem lock while searching.
 */
extern fs_FileSystem *fs_fileSystemForVolume(
    evm_volId_t volId);

/**
 * Return the FileSystem structure for a specified volume id, or NULL if the
 * volume is not known.  Caller must hold the fs_fileSystem lock.
 */
extern fs_FileSystem *fs_fileSystemForVolumeLocked(
    evm_volId_t volId);

/**
 * A spinlock for the file system table.
 */
extern nfx_spinlock_t fs_fileSystemTableLock;

/**
 * Return list of volume ids
 */
extern int32 fs_fileSystemGetVols(
    fs_Context *context, evm_volId_t *vols, int32 nVols);


/**
 * Limit response size 
 */
#define FS_SHELL_RESP_MAX   6000  // Allow room for protocol headers

/**
 * Default for minimum space free on fs
 */

#define FS_DEFAULT_FS_MIN_FREE 5


extern uint8 fs_onesPerByte[256];


//   M I S C E L L A N E O U S   U T I L I T I E S


/**
 * A cheap, fast, really bad random bit generator.
 */
boolean fs_cheapRandomBoolean(void);

/* 
 * input  UTC sec since 00:00:00 on January 1, 1970, 
 *                      Coordinated Universal Time (UTC)
 * output "Mon Oct  8 11:21:00 2001"
 *
 * Todo: set the timezone: today hardcoded to -7 hours = PDT
 */
char *
fs_humandate(uint32 time);


/**
 * Do the work of mounting the filesystem.
 */
int32
fs_doMount(
	fs_Context		*context,
	fs_MsgMountReq	*req);


/**
 * Do the real work of unmounting a volume.
 */
void
fs_doUnmount(
    fs_Context      *context,
    fs_FileSystem   *fs,
    boolean         isForce);

/**
 * CRC-32 the specified buffer.
 */
extern uint32
fs_checksum(
    void    *ptr,
    int32   len,
	uint32	crc);
extern uint32 crc32[];

/**
 * Hash courtesy of Bob Jenkins, find it on the web.
 *
 * Input must be pointer to array of uint64's, length
 * indicates number of uint64's in the array.
 */
extern uint64
fs_coreHash(
    void    *input, // key to be hashed
    uint64  length, // length of input in uint64's
    uint64  level); // previous hash or arbitrary value

extern void
fs_incorePrintResSettings(
    fs_Context      *context);

extern void
fs_incorePrintCurrResources(
    fs_Context      *context);

extern void
fs_incorePrintStats(
    fs_Context      *context);

extern void
fs_contextLog (
    fs_Context      *context,
    int              severity, 
    int              app_type, 
    int              line, 
    char            *file,
    char            *msg);

extern void
fs_postEvt(fs_Context * context,
           uint32       event,
           int          state);

extern void 
fs_initAlloc(void);


#endif /* FS_GLOBAL_H */
