/*
 * Name:  fs-includes.h
 *
 * RCS:  $Id: fs-includes.h,v 1.86 2002/12/04 22:51:43 isabellec Exp $
 *
 * Description:  Include files for filesystem
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

#if !defined(FS_INCLUDES_H)
#define FS_INCLUDES_H

// compiles the inlines as inline 
#define FS_USE_INLINES      1

// New User Zero fill
#define NEW_ZF              1

// pack everthing at the beginning of the disk
// #define NEW_ALLOC_POLICY   1

// Don't write the buddy pages to disk,
// recalc always from ref page
#define BUDDY_INCORE       1

// dynamically update the used buf/inode limits
// depending of the available resources (mem/ultrabuf)
#ifdef FS_STANDALONE
#undef FS_USE_MALLOC_SLAB
#else
#define FS_DYN_CACHE_SIZE  1
#endif

// Count the number of active transactions which are garanteed to
// write a commit record to the log.
// if this number is an incoretune, postpone the writting of a
// partially full log record.
#define FS_VOLLOG_WAIT_FOR_ONGOING_TRANS      1


// check the inode hash algoritm, keep track of the nbr of entries 
// for each hash value
// #define FS_INODE_CHECK_HASH_DEPTH             1
// #define FS_BUF_BLK_CHECK_HASH_DEPTH           1
// #define FS_BUF_PAGE_CHECK_HASH_DEPTH          1
// #define FS_BUF_INUM_CHECK_HASH_DEPTH          1

// Create/compare the checksum for each buf when write/read to/from disk
// #define FS_DEBUG_BUF_CKSUM                    1

// Kickoff a read to verify just written data
// #define FS_DEBUG_BUF_READ_VERIFY              1

// shadow allocator
// #define FS_DEBUG_ALLOC                        1

// Check the summary on a buddy page.
// #define FS_DEBUG_BUDDY_SUMMARY                1

// fs_buddyFindExtent check for ref byte == 0
// #define FS_DEBUG_BUDDY_CHECK_REFBYTE          1

// check for duplicate cache insert in fs_bufBlknumCacheInsert
// #define FS_DEBUG_DUP_BUF_BLKNUM_CACHE_INSERT  1

// check for duplicate cache insert in fs_bufPageCacheInsert
// #define FS_DEBUG_DUP_BUF_PAGE_CACHE_INSERT    1

// check for duplicate cache insert in inodeCacheInsert
// #define FS_DEBUG_DUP_INODE_CACHE_INSERT       1

// check if inode already exist in fs_inodeInstantiateAllocator
// #define FS_DEBUG_NEW_INODE_INSERT             1

// Use shadow buffer
// #define FS_DEBUG_SHADOW_BUF                1

// get writeback timing performance 
// #define FS_DEBUG_WRITEBACK_TIMING          1

// get inode cleaner timing performance 
// #define FS_DEBUG_INODE_CLEANER_TIMING      1

// print lock owner list in case of fs_volumeException
#define FS_DEBUG_DUMP_LOCK_OWNERS          1

// collect/print stats for cando lists
// #define FS_DEBUG_CANDO_STATS               1

// Check that the inode counts are correct.
// #define FS_DEBUG_INODE_COUNTS              1

// Dump log records if log is full
#define FS_DEBUG_DUMP_FULL_LOG             1


// enable the possibility to crash on command
// #define FS_DEBUG_CHECK_CRASH               1

// snap debugging ref bytes
// #define FS_DEBUG_SNAP_REMOVE               
// #define FS_DEBUG_SNAP_REVERT

// snap crash testing
// Remark: !!!  FS_DEBUG_CHECK_CRASH must be on
// #define FS_DEBUG_SNAP_CRASH                1

// check offsets during volume logging
// #define FS_DEBUG_OFFSETS_VOL_LOG          1

// Perform consistency checks on directory leaf.
// #define FS_DEBUG_DIRLEAF_CHECK            1

// Perform consistency checks on emap leaf.
// #define FS_DEBUG_EMAP_LEAF_CHECK          1

// print timing info for buf free inode
// #define FS_DEBUG_PRINT_BUF_FREE_INODE     1

// print timing info for fs_trashDirent
// #define FS_DEBUG_PRINT_TRASHCAN_PERF      1

// check for unexpected valid blocks during truncate and trashcan operations
// #define FS_DEBUG_TRASH_EMAP_LOOKUP        1

// lock debugging
// Remark: if FS_DEBUG_LOCK_ORDER set, also set FS_DEBUG_LOCK
// #define FS_DEBUG_LOCK_ORDER               1
#define FS_DEBUG_LOCK                     1

// vol log statistics
// #define FS_DEBUG_VOL_LOG_BUCKETS          1

// cpu - yield - io performance statistics
// #define FS_DEBUG_PRINT_DELTA_THRESHOLD    1
// #define FS_DEBUG_NFS_DELTA_BUCKET         1

// measure the rdwr time delays
// #define FS_RDWR_TIMESTAMP               1

// measure the lock waiter delay
// #define FS_LOCK_WAITER_TIME             1

// count which specific inodes are missed in the inode/buf cache
// #define FS_DEBUG_INODE_CNT_MISS_TYPE    1
// #define FS_DEBUG_BUF_CNT_MISS_TYPE      1

// quota debugging
#define FS_DEBUG_QUOTA                  1

#if defined(FS_STANDALONE)
#include <stdarg.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <rpc/rpc.h>
#include <sys/resource.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <pthread.h>
#include <memory.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
#include <nfx-incl.h>
#else
#ifdef SSC
#include <nfx-target.h>
#include <nfx-types.h>
#include <nfx-defs.h>
#include <nfx-error.h>
#include <stdlib.h>
#include <stdio.h>
#else
#include <nfx-incl.h>
#endif
#endif

#ifdef FS_STANDALONE
extern address_t get_ra();
#endif

#include <exports/cm-api.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>

#ifdef FS_STANDALONE
#undef FS_USE_MALLOC_SLAB
#endif

#include <exports/queue-api.h>
#include <exports/sbm-api.h>
#include <exports/cod-api.h>
#include <exports/nvram-api.h>
#include <exports/esm-api.h>
#include <exports/thread-api.h>
#include <exports/elog-api.h>
#include <exports/sec-cifstruct.h>
#ifndef SSC
#include <exports/mem-api.h>
#endif

#if !defined(FS_STANDALONE)
#include <exports/event-api.h>
#include <exports/evm-api.h>
#else
#include "fs-standalone-evm.h"
#endif

#include "fs-primitive.h"
#include "exports/esm-api.h"
#include "fs-api.h"
#include "chash-api.h"
#include "fs-limits.h"

#if defined(FS_STANDALONE)
#include <limits.h>
#include "fs-sim.h"
#include "btrace.h"
#endif

#if !defined(FS_STANDALONE)
#include <exports/timer-api.h>
#endif


#include "fs-lock.h"
#include "fs-fileset.h"
#include "fs-quota.h"
#include "fs-inode.h"
#include "fs-layout.h"
#include "fs-snap.h"
#include "fs-eek.h"
#include "fs-super.h"
#include "fs-context.h"
#include "fs-requests.h"
#include "fs-rdwr.h"
#include "fs-buf.h"
#include "fs-log.h"
#include "fs-path.h"
#include "fs-link.h"
#include "fs-dump.h"
#include "fs-restore.h"
#include "fs-ndmp-api.h"
#include "fs-am.h"
#include "fs-global.h"
#include "fs-ref.h"
#include "fs-block-set.h"
#include "fs-skip-heap.h"
#include "fs-alloc.h"
#include "fs-arena.h"
#include "fs-filehistory.h"
#include "fs-proto.h"
#ifdef SBM_REV_2
#include "fs-cod-rev2.h"
#else
#include "fs-cod.h"
#endif
#include "fs-bitmap.h"
#include "fs-trashcan.h"
#include "fs-mkfs.h"
#include "btree/api.h"
#include "fs-dir.h"
#include "fs-dirhash.h"
#include "fs-debug.h"
#include "fs-wb.h"
#include "fs-nvram.h"
#include "fs-vl.h"
#include "fs-vol-log.h"
#include "fs-am-api.h"
#include "fs-buddy.h"
#include "fs-vol-exer.h"
#include "fs-benchmark.h"

#ifdef FS_STANDALONE
#ifndef FS_FSDB
// Implement an equivalent function for standalone whenever convenient
#define uint32_checksum(data, size, len)  0xFEADFACE
#endif
#endif


//   I N L I N E   F U N C T O N S

/*
 * This section is ignored if we're inside the file which defines the
 * inline functions.
 */
#ifndef FS_INLINES_C

#ifdef FS_USE_INLINES

/**
 * For production uses, we will actually define the inline functions
 * using "static inline," counting on the bodies not showing up
 * redundantly.
 */
#define INLINE_FUNCTION(proto, body) static inline proto body

#else /* ! FS_USE_INLINES */

/**
 * For development uses, we simple declare prototypes and let the
 * functions all live as externals in fs-inlines.c
 */
#define INLINE_FUNCTION(proto, body) extern proto;

#endif /* FS_USE_INLINES */

#include "fs-inlines.c"

#undef INLINE_FUNCTION

#endif /* FS_INLINES_C */



//   L E A K   D E T E C T O R


#ifdef LEAK_DETECTOR

#ifdef EFENCE
#error define exactly one of LEAK_DETECTOR or EFENCE
#endif

#define GC_DEBUG
#define LINUX_THREADS
#include "gc.h"

#endif

#endif /* FS_INCLUDES_H */
