/*
 * Name		: fs-ndmp-api.h
 * RCS		: $ID$
 * Description	: FS-NDMP interface
 * Created by	: Rahul Khona
 * Date Created	: 7/11/2001
 */

#include "fs-includes.h"
#include "fs-ndmp-api.h"
#include <exports/ndmp-fs-api.h>

#if !defined (SSC)

#define NFX_NFP_CPU 0
/*
 * Setup a dump state machine. We will recieve names setup and start
 * messages in addition to iocompletion and abort messages.
 * All the dumps for filesystem that are in progress will hand off the
 * filesystem structure. 
 */

#ifdef FS_DEBUG
#define D   printf
#else
#define D   
#endif
#ifdef FS_HIST_WAIT
extern fs_DumpSessionsHdr_t fs_dumpSessionsHdr;
extern nfx_spinlock_t       fs_dumpSessionsSpinLock;
#endif

int32
fs_dumpSetupThread(
    fs_Context      *context
)
{

    fs_FileSystem           *fs = context->fileSystem;
    fs_DumpContext_t        *dumpContext;
    fs_MsgSetupDumpReq_t    *req;
    fs_MsgNdmpResp_t        *resp = (fs_MsgNdmpResp_t *)context->respHdr;
    uint32                  labelLen;
    uint32                  prevSnapLen;
    uint32                  cwdLen;
    char                    *blob;
    int                     i;
#ifdef FS_HIST_WAIT
    fs_DumpSession_t        *session;
#endif

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In Dump Setup"));

    req = (fs_MsgSetupDumpReq_t *)context->reqHdr;
    
    /*
     * Validates the arguments and if invalid sends an abort message
     */


    /*
     * Allocate a dumpContext structre and hang it off the fs. Pointer
     * to dumpContext will act as our cookie
     */

    dumpContext = fs_ramAlloc(FS_RAM_DUMP_CONTEXT, sizeof (fs_DumpContext_t));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In Dump Setup - allocated context"));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("dumpcontext is %p", (char *)dumpContext));
    bzero(dumpContext, sizeof (fs_DumpContext_t));
#ifdef FS_HIST_WAIT
    FS_ASSERT(dumpContext != NULL);
    session = fs_ramAlloc(FS_RAM_DUMP_SESSION, sizeof (fs_DumpSession_t));
    bzero(session, sizeof (fs_DumpSession_t));
    FS_ASSERT(session != NULL);
    session->dumpContext = dumpContext;
    NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
    DLL_INSERT_TAIL(&fs_dumpSessionsHdr, session, link);
    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
    dumpContext->session = session;
    session->charSpace = FS_HISTORY_BUF_SIZE * 4;
    session->mbufSpace = FS_HISTORY_BUF_SIZE * 4;
#endif
    dumpContext->cookie = req->dump.ndmpSessId;
    dumpContext->ndmp_ssc_src_port = req->dump.ndmp_ssc_src_port;
    dumpContext->ndmpSessId = req->dump.ndmpSessId;
    resp->clientCookie = dumpContext->cookie;
    dumpContext->files = 0;
    dumpContext->currentId = 0;
    dumpContext->label = NULL;
    dumpContext->doFileHistory = req->dump.fileHistory;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("level is %d", req->dump.level));
    dumpContext->level = req->dump.level;
    dumpContext->recursiveDescent = req->dump.recursiveDescent;
    dumpContext->prevDumpTime.secs = req->dump.prevSnapTime;
    dumpContext->historyBuf = fs_ramAlloc(FS_RAM_DUMP_CONTEXT,
                                            FS_HISTORY_BUF_SIZE);
    FS_ASSERT(dumpContext->historyBuf != NULL);
    dumpContext->curDirHist = (fs_DirHistory_t *)dumpContext->historyBuf;
    dumpContext->curNodeHist = (fs_NodeHistory_t *)dumpContext->historyBuf;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In Dump Setup - initialized scalars"));

    dumpContext->stackIndex = -1;
	dumpContext->ioSequence = 1;	/* NITIN NEEDS IT TO START WITH 1 */
	dumpContext->neSequence = 1;	/* NITIN NEEDS IT TO START WITH 1 */

    /*
     * Initialize the tape lock
     */

    fs_lockInit(&dumpContext->tapeLock, FS_LOCK_TYPE_TAPE);
#ifdef FS_HIST_WAIT
    fs_lockInit(&dumpContext->histLock, FS_LOCK_TYPE_TAPE);
#endif
    NFX_SPIN_INIT(&dumpContext->spinLock,SPINLOCK_LEVEL_UNKNOWN);
    /*
     * Now extract various names
     */

    labelLen = req->dump.labelLen;
    prevSnapLen = req->dump.prevSnapNameLen;
    cwdLen = req->dump.cwdLen;
    
    blob = req->dump.blob;

    if (cwdLen) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("In Dump Setup - initialing cwdlen"));
        dumpContext->cwd = fs_ramAlloc(FS_RAM_DUMP_NAME, cwdLen + 1);
        FS_ASSERT(dumpContext->cwd != NULL);
        memcpy(dumpContext->cwd, blob, cwdLen);
        dumpContext->cwd[cwdLen] = '\0';
        blob += cwdLen;
    }
    if (labelLen) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("In Dump Setup - initialing labellen"));
        dumpContext->label = fs_ramAlloc(FS_RAM_DUMP_NAME, labelLen + 1);
        FS_ASSERT(dumpContext->label != NULL);
        memcpy(dumpContext->label, blob, labelLen);
        dumpContext->label[labelLen] = '\0';
        blob += labelLen;
    }
    if (prevSnapLen) {
        FS_ASSERT(prevSnapLen < FS_DUMP_MAX_SNAP_LEN);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("In Dump Setup - initialing snaplen"));
        dumpContext->prevSnapName = fs_ramAlloc(FS_RAM_DUMP_NAME,
                                                prevSnapLen + 1);
        FS_ASSERT(prevSnapLen != 0);
        memcpy(dumpContext->prevSnapName, blob, prevSnapLen);
        dumpContext->prevSnapName[prevSnapLen] = '\0';
        blob += prevSnapLen;
    }

    DLL_INIT(&dumpContext->nameList);
    for (i = 0; i < 1024; i++) {
        DLL_INIT(&dumpContext->renames[i]);
    }
    for (i = 0; i < 127; i++) {
        DLL_INIT(&dumpContext->links[i]);
    }
	NFX_SPIN_INIT(&dumpContext->spinLock,SPINLOCK_LEVEL_UNKNOWN);

    /*
     * Now insert the dumpContext into fs_FileSystem lists
     */

    //DLL_INIT(&fs->dumpContexts);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("trying to lock dumpContextsLock"));
    fs_lockExclusive(context, &fs->dumpContextsLock, NULL);
    DLL_INSERT_TAIL(&fs->dumpContexts, dumpContext, dumpList);
    fs_unlockExclusive(context, &fs->dumpContextsLock);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("returning from dump setup"));

    /*
     * Initialize I/O Queue
     */

    dumpContext->ioSize = req->dump.recSize;
    DLL_INIT(&dumpContext->freeList);
    DLL_INIT(&dumpContext->ioList);
    DLL_INIT(&dumpContext->pendingIoList);
    dumpContext->currOffset = 0;
    dumpContext->currentBuffer = NULL; //DLL_NIL;
    for (i = 0; i < FS_DUMP_TAPE_IO_BUFFERS; i++) {
            DLL_INSERT_TAIL(&dumpContext->freeList,
                            &dumpContext->queueHdrs[i], link);
            dumpContext->queueHdrs[i].ioBuf =
                    fs_ramAlloc(FS_RAM_DUMP_IOBUF, dumpContext->ioSize);
            FS_ASSERT(dumpContext->queueHdrs[i].ioBuf != NULL);
            if (dumpContext->queueHdrs[i].ioBuf == NULL) {
                    /*
                     * There aren't enough resources
                     */
                    if (i != 0) {
                            int j;
                            for (j = i; j < FS_DUMP_TAPE_IO_BUFFERS; j++) {
                                    dumpContext->queueHdrs[j].ioBuf = NULL;
                                    dumpContext->queueHdrs[j].ioSequence =
                                            0xFFFFFFFFFFFFFFFFL;
                            }
                            break;
                    } else {
                            fs_cleanDumpContext(context, dumpContext, TRUE);
                            fs_ramDealloc(FS_RAM_DUMP_CONTEXT, dumpContext);
                            FS_ELOG(context, LOG_ERR, FS_TRACE_DUMP_RESTORE,
                                    ("Cannot dump for lack of resources"));
                            return NFX_ERR;
                    }
            }
    }


#ifdef FS_STANDALONE
    dumpContext->fd = creat("/tmp/dumpfile1", 0777);
    if (dumpContext->fd == -1) {
        FS_ASSERT(0);
        dumpContext->fd = 1;
    }
#endif
    return NFX_OK;
}

/*
 * Add names
 */

int32
fs_dumpNamesThread(
    fs_Context      *context
)
{
    //Should never be called TO BE REMOVED
//    FS_ASSERT (0);

#if 0
    fs_FileSystem   *fs = context->fileSystem;
    fs_MsgDumpNamesReq_t    *req = (fs_MsgDumpNamesReq_t *)context->reqHdr;
    fs_MsgNdmpResp_t        *resp = (fs_MsgNdmpResp_t *)context->respHdr;
    uint32                  len;
    char                    *blob;
    fs_DumpNameList_t       *nameEntry;
    fs_DumpContext_t        *dumpContext = NULL; //DLL_NIL;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In dumpNames"));
    len = req->names.len;
    blob = req->names.msgBlock;

    /*
     * First find the dumContext
     */

    fs_lockShared(context, &fs->dumpContextsLock, NULL);
    DLL_ITERATE(dumpContext, &fs->dumpContexts, dumpList) {
        //if (dumpContext != DLL_NIL) {
        if (dumpContext != NULL && dumpContext != DLL_NIL) {
            if (dumpContext->cookie == req->names.ndmpSessId) {
                break;
            }
        }
    }
    fs_unlockShared(context, &fs->dumpContextsLock);

    //if (dumpContext == DLL_NIL) {
    if (dumpContext == NULL || dumpContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        resp->clientCookie = req->names.ndmpSessId;
        return NFX_ERR;
    }
    resp->clientCookie = dumpContext->cookie;
    
    if (blob && *blob) {

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("dump names entering entries"));
        
        nameEntry = fs_ramAlloc(FS_RAM_DUMP_NAMEENTRY,
                                sizeof (fs_DumpNameList_t));
		bzero(nameEntry, sizeof (fs_DumpNameList_t));
        FS_ASSERT(nameEntry != NULL);
        nameEntry->name = fs_ramAlloc(FS_RAM_DUMP_NAME, len + 1);
        strncpy(nameEntry->name, blob, len);
        nameEntry->name[len] = '\0';
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("DumpNames:Inserted %s in namelist",
                                        blob));
		DLL_ISOLATE_ELEM(nameEntry, link);
        DLL_INSERT_TAIL(&dumpContext->nameList, nameEntry, link);
        dumpContext->files = 1;
    }
#endif    
    return NFX_OK;
}

/*
 * Start dump
 */

int32
fs_dumpStartThread(
    fs_Context          *context
)
{
    fs_FileSystem       *fs = context->fileSystem;
    fs_DumpContext_t    *dumpContext = NULL; //DLL_NIL;
    fs_MsgStartDumpReq_t  *req = (fs_MsgStartDumpReq_t *)context->reqHdr;
    fs_Status           status;
#ifdef FS_DEBUG
    eeeGMT_t            tmpTime1;
    eeeGMT_t            tmpTime2;
#endif

    fs_lockExclusive(context, &fs->dumpContextsLock, NULL);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In Dump Start"));
    DLL_ITERATE(dumpContext, &fs->dumpContexts, dumpList) {
        //if (dumpContext != DLL_NIL) {
        if (dumpContext != NULL && dumpContext != DLL_NIL) {
            if (dumpContext->cookie == req->start.ndmpSessId) {
                break;
            }
        }
    }

    fs_unlockExclusive(context, &fs->dumpContextsLock);
    //if (dumpContext == DLL_NIL) {
    if (dumpContext == NULL || dumpContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        (void)fs_sendNdmpResponse(context, FS_REQ_DUMP_START, req->start.ndmpSessId,
                            req->hdr.clientPrivate,
                            FS_ERR_INVAL);

        return NFX_ERR;
    }

    /*
     * Setup the dumpContext to contain this thread's handle for
     * iocompletion wakeups
     */

    dumpContext->smHandle = context->smHandle;

    (void)fs_sendNdmpResponse(context, FS_REQ_DUMP_START, req->start.ndmpSessId,
                              req->hdr.clientPrivate,
                              FS_ERR_NONE);
    /*
     * Do the actual dump. On return dumpContext is full cleaned
     * and also done/abort message is already sent to ndmp
     */

    status = fs_dump(context, dumpContext);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Checking dump status"));
    if (status != FS_ERR_NONE && !dumpContext->abort) {
        char errstr[128];
        /*
         * Log an abort log and send an abort message
         */

        sprintf(errstr, "%s - Aborting dump\n", fs_errCodeToString(status));
        fs_ndmpSendDumpLog(context, dumpContext, status, NDMP_LOG_MSG_TYP_ERR,
                            0, NULL, errstr);
        fs_ndmpSendDoneOrAbort(context, dumpContext, FS_ERR_IO);
    } else {

        /*
         * Send the completion message
         */

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Completed dump"));
        fs_ndmpSendDoneOrAbort(context, dumpContext, FS_ERR_NONE);
    }
#ifdef FS_DEBUG
	eee_rtcDurationTV(&dumpContext->startTime, &tmpTime1);
	eee_rtcGMT(&tmpTime2);
    printf("Elapsed dump time %u msecs\n",
		   (tmpTime1.sec * 1000) + (tmpTime1.usec / 1000));
	eee_rtcToGMT(&dumpContext->startTime, &tmpTime1);
    printf("start time secs %d end time secs %d\n",
		   tmpTime1.sec, tmpTime2.sec);
    printf("total tape ios %lld total tape waits %lld"
           "max ios in transit %d avg tape queue len %lld \n",
           dumpContext->totTapeIo, dumpContext->totTapeWait,
           dumpContext->maxTapeIosInTransit,
           (dumpContext->totTapeIo == 0) ? -1 :
           dumpContext->totTapeQueueLen/dumpContext->totTapeIo);
    printf("Total time waiting for tape i/o in msecs %lld\n",
                    dumpContext->totTapeWaitTime / 1000);
#endif
#ifdef FS_HIST_WAIT
    NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
    DLL_REMOVE(&fs_dumpSessionsHdr, dumpContext->session, link);
    fs_ramDealloc(FS_RAM_DUMP_SESSION, dumpContext->session);
    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
#endif
    fs_cleanDumpContext(context, dumpContext, FALSE);
    fs_ramDealloc(FS_RAM_DUMP_CONTEXT, dumpContext);
    if (status != FS_ERR_NONE) {
        return NFX_ERR;
    }
    return NFX_OK;
}

/*
 * Handle dump io completion
 */

int32
fs_dumpIODoneThread(
    fs_Context  *context
)
{
    fs_FileSystem       *fs = context->fileSystem;
    fs_DumpContext_t    *dumpContext = NULL; //DLL_NIL;
    fs_MsgDumpIODoneReq_t  *req = (fs_MsgDumpIODoneReq_t *)context->reqHdr;
    esm_event_t         event;
    int                 rc = NFX_OK;
    fs_DumpTapeIoQueue_t    *q = NULL, *t;
    int                     i;
    int                     signal = 0, inserted = 0;

    fs_lockShared(context, &fs->dumpContextsLock, NULL);
    DLL_ITERATE(dumpContext, &fs->dumpContexts, dumpList) {
        //if (dumpContext != DLL_NIL) {
        if (dumpContext != NULL && dumpContext != DLL_NIL) {
            if (dumpContext->cookie == req->iodone.ndmpSessId) {
                break;
            }
        }
    }
    fs_unlockShared(context, &fs->dumpContextsLock);
    //if (dumpContext == DLL_NIL || dumpContext != DLL_NIL) {
    if (dumpContext == NULL) {
        /*
         * XXX - Send an abort message
         */

        return NFX_ERR;
    }

    //printf("dump iodone\n");

    FS_ASSERT(req->iodone.dataLen == 0);

    for (i = 0; i < FS_DUMP_TAPE_IO_BUFFERS; i++) {
            if (dumpContext->queueHdrs[i].ioSequence == req->iodone.sequence) {
                    q = &dumpContext->queueHdrs[i];
                    DLL_ISOLATE_ELEM(q, link);
                    break;
            }
    }
    FS_ASSERT(q != NULL);

    NFX_SPIN_LOCK(&dumpContext->spinLock);
    DLL_REMOVE(&dumpContext->pendingIoList, q, pendingIoLink);
    if (req->iodone.sequence == dumpContext->neSequence) {
            dumpContext->neSequence++;
            //printf("placing buf on freelist for %d\n", req->iodone.sequence);
            DLL_INSERT_TAIL(&dumpContext->freeList, q, link);
            if (dumpContext->needBuffer) {
                    signal = 1;
            }
more:
            i = 0;
            DLL_ITERATE(q, &dumpContext->ioList, link) {
                    if (q->ioSequence == dumpContext->neSequence) {
                            dumpContext->neSequence++;
                            DLL_REMOVE(&dumpContext->ioList, q, link);
                            DLL_INSERT_TAIL(&dumpContext->freeList, q, link);
                            goto more;
                    }
                    i++;
                    FS_ASSERT(i <= FS_DUMP_TAPE_IO_BUFFERS);
            }
    } else {
            //printf("placing buf on iolist for %d\n", req->iodone.sequence);
            if (DLL_IS_EMPTY(&dumpContext->ioList)) {
                    DLL_INSERT_TAIL(&dumpContext->ioList, q, link);
            } else {
                    i = 0;
                    DLL_ITERATE(t, &dumpContext->ioList, link) {
                            if (t->ioSequence > q->ioSequence) {
                                    DLL_INSERT_BEFORE(&dumpContext->ioList,
                                                    t, q, link);
                                    inserted = 1;
                                    break;

                            }
                            i++;
                            FS_ASSERT(i <= FS_DUMP_TAPE_IO_BUFFERS);
                    }
                    if (!inserted) {
                            DLL_INSERT_TAIL(&dumpContext->ioList, q, link);
                    }
            }
    }

    if (signal) {
            //printf("waking up waiter\n");
            event.event = FS_THREAD_EVENT_DUMP_REST_IOCOMPLETE;
            event.p1.u.w = req->iodone.status;
            event.p2.u.p = NULL;
            rc = thread_event(dumpContext->tapeWaiter, &event);
            dumpContext->needBuffer = FALSE;
            dumpContext->tapeWaiter = 0;
    }
    NFX_SPIN_UNLOCK(&dumpContext->spinLock);

    if (req->iodone.status != FS_ERR_NONE) {
            dumpContext->ioStatus = req->iodone.status;
    }
    FS_ASSERT(rc == NFX_OK);
    return rc;
}

int32
fs_dumpAbortThread(
    fs_Context          *context
)
{
    fs_FileSystem       *fs = context->fileSystem;
    fs_DumpContext_t    *dumpContext = NULL; //DLL_NIL;
    fs_MsgNdmpResp_t        *resp = (fs_MsgNdmpResp_t *)context->respHdr;
    fs_MsgAbortDumpReq_t  *req = (fs_MsgAbortDumpReq_t *)context->reqHdr;
    int                 rc = NFX_OK;
    fs_DumpTapeIoQueue_t    *q = NULL;

	if ( fs->dumpContexts.tail == NULL ||
	    fs->dumpContexts.head == NULL) {
		goto no_context;
	}
    fs_lockShared(context, &fs->dumpContextsLock, NULL);
    if (fs->dumpContexts.head == NULL ||
        fs->dumpContexts.tail == NULL) {
        goto done_search;
    }
    DLL_ITERATE(dumpContext, &fs->dumpContexts, dumpList) {
        //if (dumpContext != DLL_NIL) {
        if (dumpContext != NULL && dumpContext != DLL_NIL) {
            if (dumpContext->cookie == req->abort.ndmpSessId) {
                break;
            }
        }
    }
done_search:
    fs_unlockShared(context, &fs->dumpContextsLock);
no_context:
    //if (dumpContext == DLL_NIL) {
    if (dumpContext == NULL || dumpContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        resp->clientCookie = req->abort.ndmpSessId;
        return NFX_ERR;
    }
    resp->clientCookie = dumpContext->cookie;

    dumpContext->abort = 1;

    /*
     * Handle completion of i/o buffers as NDMP does not handle it
     */
    NFX_SPIN_LOCK(&dumpContext->spinLock);
    if (dumpContext->needBuffer) {
            //printf("waking up waiter\n");
            esm_event_t event;
            event.event = FS_THREAD_EVENT_DUMP_REST_IOCOMPLETE;
            event.p1.u.w = FS_ERR_IO;
            event.p2.u.p = NULL;
            rc = thread_event(dumpContext->tapeWaiter, &event);
            dumpContext->needBuffer = FALSE;
            dumpContext->tapeWaiter = 0;
    }
    DLL_ITERATE(q, &dumpContext->pendingIoList, link) {
            DLL_REMOVE(&dumpContext->pendingIoList, q, pendingIoLink);
            DLL_INSERT_TAIL(&dumpContext->freeList, q, link);
    }
    DLL_ITERATE(q, &dumpContext->ioList, link) {
            DLL_REMOVE(&dumpContext->ioList, q, link);
            DLL_INSERT_TAIL(&dumpContext->freeList, q, link);
    }
    if (dumpContext->currentBuffer) {
            DLL_INSERT_TAIL(&dumpContext->freeList,
                            dumpContext->currentBuffer, link);
    }
    dumpContext->currentBuffer = NULL;
    dumpContext->ioStatus = FS_ERR_IO;
    NFX_SPIN_UNLOCK(&dumpContext->spinLock);
    FS_ASSERT(rc == NFX_OK);
    return rc;

}

/*
 * Handle messages to be send to ndmp on write's behalf
 */


PRIVATE fs_Status
fs_ndmpStartTapeWrite(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext
)
{
    ndmp_write_data_req_msg_t   msg;
    int     rc;

    if (dumpContext->abort) {
        return NFX_ERR;
    }
    /*
     * Construct the message
     */
    //printf("Writin to tape\n");

    dumpContext->currentBuffer->ioSequence = dumpContext->ioSequence;
    msg.cl_cookie = (void *)dumpContext;
    msg.ioCookie = dumpContext->ioSequence;
    msg.hdr.mid = NDMP_MID_WRITE_DATA_REQ;
    msg.hdr.clientCookie = dumpContext->cookie;
    msg.buf = dumpContext->currentBuffer->ioBuf;
    dumpContext->currOffset = 0;
    dumpContext->ioSequence++;
    //dumpContext->neSequence++;
    DLL_INSERT_TAIL(&dumpContext->pendingIoList, dumpContext->currentBuffer,
                    pendingIoLink);
    dumpContext->currentBuffer = NULL;
#ifdef FS_DEBUG
    dumpContext->totTapeIo++;
    if ((dumpContext->ioSequence - dumpContext->neSequence) >
                    dumpContext->maxTapeIosInTransit) {
        dumpContext->maxTapeIosInTransit =
                (dumpContext->ioSequence - dumpContext->neSequence);
    }

    dumpContext->totTapeQueueLen +=
            (dumpContext->ioSequence - dumpContext->neSequence);
#endif
    
#ifndef FS_STANDALONE
    rc = eee_sendMessage((void *)&msg, sizeof (msg),
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                  NFX_NFP_FP_ANY,
                                                  eee_getApplicationId("ndmp"),
                                                  EEE_CONTROL,
                                                  NFX_COS_HIGH,
											      0),
                        fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
    if (rc != NFX_OK) {
            FS_ASSERT(0);
            dumpContext->ioStatus = FS_ERR_IO;
    }
#else
    rc = NFX_OK;
#endif
    return dumpContext->ioStatus;
}

fs_Status
fs_ndmpWriteTape(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    char                *buf
)
{
        fs_Status   status = FS_ERR_NONE;
        esm_event_t event;

        /*
         * If there is room in current buffer write to it. If not
         * start i/o on the current buffer and fetch the next one
         */

        if (dumpContext->abort) {
                //printf("Returning from WriteTape because of abort\n");
                return NFX_ERR;
        }
fetchBuffer:
        NFX_SPIN_LOCK(&dumpContext->spinLock);
        //if (dumpContext->currentBuffer == DLL_NIL) {
        if (dumpContext->currentBuffer == NULL ||
            dumpContext->currentBuffer == DLL_NIL) {
                DLL_REMOVE_HEAD(&dumpContext->freeList, 
                                dumpContext->currentBuffer, link);
                //if (dumpContext->currentBuffer != DLL_NIL) {
                if (dumpContext->currentBuffer != NULL &&
                                dumpContext->currentBuffer != DLL_NIL) {
                        dumpContext->currOffset = 0;
                }
        }

        /*
         * No current buffers are available, wait for tape i/o
         * to complete
         */
        //if (dumpContext->currentBuffer == DLL_NIL) {
        if (dumpContext->currentBuffer == NULL ||
                        dumpContext->currentBuffer == DLL_NIL) {
#ifdef FS_DEBUG
                eeeRTC_t  now, delta;
#endif
                dumpContext->needBuffer = TRUE;
                dumpContext->tapeWaiter = context->smHandle;
#ifdef FS_DEBUG
                dumpContext->totTapeWait++;
#endif
                NFX_SPIN_UNLOCK(&dumpContext->spinLock);
#ifdef FS_DEBUG
                eee_rtc(&now);
#endif
                fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                         FS_YIELD_TYPE_IO_TW);
#ifdef FS_DEBUG
				eee_rtcDuration(&now, &delta);
                dumpContext->totTapeWaitTime += eee_rtcToUsec(&delta);
#endif
                //printf("Going to fetchBuffer again\n");
                goto fetchBuffer;
        }

        memcpy(dumpContext->currentBuffer->ioBuf + dumpContext->currOffset,
              buf, FS_DR_TBSIZE);
        dumpContext->currOffset += FS_DR_TBSIZE;
        if (dumpContext->currOffset == dumpContext->ioSize) {
                status = fs_ndmpStartTapeWrite(context, dumpContext);
                NFX_SPIN_UNLOCK(&dumpContext->spinLock);
                fs_sleep(context, 0);    // Give ndmp a chance
        } else {
                status = dumpContext->ioStatus;
                NFX_SPIN_UNLOCK(&dumpContext->spinLock);
        }
        return status;

}

int
fs_ndmpFlushLastBuffer(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext
)
{
        int rc = NFX_OK;
        if (dumpContext->abort) {
                return NFX_ERR;
        }
        //if (dumpContext->currentBuffer == DLL_NIL) {
        if (dumpContext->currentBuffer == NULL ||
                        dumpContext->currentBuffer == DLL_NIL) {
                return rc;
        }
        //printf("Flushing last buffer\n");
        bzero(dumpContext->currentBuffer->ioBuf + dumpContext->currOffset,
                        dumpContext->ioSize - dumpContext->currOffset);
        dumpContext->currOffset = dumpContext->ioSize;
        rc = fs_ndmpStartTapeWrite(context, dumpContext);
        return rc;

}

void
fs_ndmpWaitForDumpBuffers(
    fs_Context  *context,
    fs_DumpContext_t    *dumpContext
)
{
        esm_event_t event;
        //printf("Waiting for  buffers\n");
        NFX_SPIN_LOCK(&dumpContext->spinLock);
        while (dumpContext->neSequence < dumpContext->ioSequence &&
						!dumpContext->abort) {
#ifdef FS_DEBUG
			eeeRTC_t  now, delta;
#endif
                dumpContext->needBuffer = TRUE;
                dumpContext->tapeWaiter = context->smHandle;
                NFX_SPIN_UNLOCK(&dumpContext->spinLock);
#ifdef FS_DEBUG
				eee_rtc(&now);
#endif
                fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                         FS_YIELD_TYPE_IO_MW);
        		NFX_SPIN_LOCK(&dumpContext->spinLock); //Anh
#ifdef FS_DEBUG
				eee_rtcDuration(&now, &delta);
                dumpContext->totTapeWaitTime += eee_rtcToUsec(&delta);
#endif
        }
        NFX_SPIN_UNLOCK(&dumpContext->spinLock);
}

/*
* Send a log message to ndmp
*/

int32
fs_ndmpSendDumpLog(
        fs_Context              *context,
        fs_DumpContext_t        *dumpContext,
        fs_Status               status,
        int32                   type,
        fs_Status               fileStatus,
        char                    *fileName,
        char                    *message
)
{
    int     rc;
    char    *c;
    ndmp_send_log_req_msg_t   *msg;
	int		msgSize = sizeof (ndmp_send_log_req_msg_t); 

	if (fileName) {
		msgSize += strlen(fileName);
	}

	if (message) {
		msgSize += strlen(message);
	}
    /*
     * Construct the message
     */

	msg = fs_ramAlloc(FS_RAM_DUMP_NAME, msgSize);
	bzero(msg, msgSize);
    msg->cl_cookie = (void *)dumpContext;
    msg->hdr.mid = NDMP_MID_SEND_LOG_REQ;
    msg->hdr.clientCookie = dumpContext->cookie;
    msg->status = status;
    msg->log_msgTyp = type;
    c = msg->blob;
    if (type == NDMP_LOG_MSG_TYP_FILE || fileName != NULL) {
        msg->file_recov_stat = fileStatus;
        msg->file_name_len = strlen(fileName) + 1;
        strcpy(c, fileName);
        c += msg->file_name_len;
    }

    msg->log_txt_len = 0;
    if (message) {
        strcpy(c, message);
        msg->log_txt_len = strlen(message) + 1;
    }
    
#ifndef FS_STANDALONE
    rc = eee_sendMessage((void *)msg, msgSize,
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                  NFX_NFP_FP_ANY,
                                                  eee_getApplicationId("ndmp"),
                                                  EEE_CONTROL,
                                                  NFX_COS_HIGH,
											      0),
                        fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
    rc = NFX_OK;
#endif
	fs_ramDealloc(FS_RAM_DUMP_NAME, msg);
    return rc;
}

int32
fs_ndmpSetDumpDate(
    fs_Context              *context,
    fs_DumpContext_t        *dumpContext,
    char                    *snapname,
    uint32                  date
)
{
    int     rc;
    ndmp_set_dumpdate_req_msg_t   msg;

    if (dumpContext->abort) {
        return NFX_OK;
    }
    /*
     * Construct the message
     */
    msg.cl_cookie = (void *)dumpContext;
    msg.hdr.mid = NDMP_MID_SET_DUMPDATE_REQ;
    msg.hdr.clientCookie = dumpContext->cookie;
    msg.snapshot_len = strlen(snapname) + 1;
    strcpy(msg.snapshot, snapname);
    msg.file_sys_name = context->fileSystem->volId;
    msg.date = date;
    msg.dump_level = dumpContext->level;
#ifndef FS_STANDALONE
    rc = eee_sendMessage((void *)&msg, sizeof (msg),
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                  NFX_NFP_FP_ANY,
                                                  eee_getApplicationId("ndmp"),
                                                  EEE_CONTROL,
                                                  NFX_COS_HIGH,
											      0),
                        fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
    rc = NFX_OK;
#endif
    return rc;
}

int32
fs_ndmpSendDoneOrAbort(
    fs_Context              *context,
    fs_DumpContext_t        *dumpContext,
    fs_Status               status
)
{
    int rc;
    ndmp_ops_cmplt_req_msg_t    msg;
    FS_ASSERT(status == FS_ERR_NONE || FS_ERR_IO);

    /*
     * Flush all the out standing buffers and wait for their
     * completion
     */
    if (status == FS_ERR_NONE) {
            fs_ndmpFlushLastBuffer(context, dumpContext);
    }
    fs_ndmpWaitForDumpBuffers(context, dumpContext);

    if (dumpContext->ioStatus != FS_ERR_NONE) {
            /*
             * Overload passed in argument
             */
            status = dumpContext->ioStatus;
    }

    msg.cl_cookie = (void *)dumpContext;
    msg.hdr.mid = NDMP_MID_OPS_COMPLETE_REQ;
    msg.hdr.clientCookie = dumpContext->cookie;
    msg.status = status;
#ifndef FS_STANDALONE
    rc = eee_sendMessage((void *)&msg, sizeof (msg),
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                  NFX_NFP_FP_ANY,
                                                  eee_getApplicationId("ndmp"),
                                                  EEE_CONTROL,
                                                  NFX_COS_HIGH,
											      0),
                        fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
    rc = NFX_OK;
#endif
    return rc;
}


/*
 * Setup a restore state machine. We will receive names setup and start
 * messages in addition to iocompletion and abort messages.
 * All the restores for filesystem that are in progress will hand off the
 * filesystem structure. 
 */

int32
fs_restoreSetupThread(
    fs_Context      *context
)
{

    fs_FileSystem           *fs = context->fileSystem;
    fs_RestoreContext_t     *restoreContext;
    fs_MsgNdmpResp_t        *resp = (fs_MsgNdmpResp_t *)context->respHdr;
    fs_MsgSetupRestoreReq_t *req;
    uint32                  cwdLen;

    req = (fs_MsgSetupRestoreReq_t *)context->reqHdr;
/*   
elog_enabled = TRUE;
elog_severity = LOG_DEBUG;
elog_appl_data[fs_applicationId].severity = LOG_DEBUG;
elog_appl_data[fs_applicationId].bitmap |= (1 << FS_TRACE_DUMP_RESTORE);
*/    
    /*
     * Validates the arguments and if invalid sends an abort message
     */

    /*
     * Allocate a restoreContext structre and hang it off the fs. Pointer
     * to restoreContext will act as our cookie
     */

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("ioSize %d", req->restore.ioSize));

    restoreContext = fs_ramAlloc(FS_RAM_RESTORE_CONTEXT,
                                 sizeof (fs_RestoreContext_t) +
                                 req->restore.ioSize - 1);
    bzero(restoreContext, sizeof (fs_RestoreContext_t));
    restoreContext->cookie = req->restore.ndmpSessId;
    restoreContext->currentId = 0;
    restoreContext->recursiveDescent = req->restore.recursiveDescent;
    restoreContext->iobufLen = req->restore.ioSize;
    restoreContext->curOffset = req->restore.ioSize;
    restoreContext->numBytes = req->restore.numBytes;
	restoreContext->ioSequence = 1;	/* NITIN NEEDS IT TO START WITH 1 */
	restoreContext->neSequence = 1;	/* NITIN NEEDS IT TO START WITH 1 */

    /*
     * Setup so that we can handle both the cases of ndmp knowing
     * backup size and not knowing. In case of not knowing, we
     * don't do any read aheads on tape side
     */
    restoreContext->maxPendingIo = FS_MAX_TAPE_READ_PENDING;
    if (restoreContext->numBytes == 0) {
            restoreContext->maxPendingIo = 0;
    }

    DLL_INIT(&restoreContext->nameList);
	DLL_INIT(&restoreContext->ioList);
	NFX_SPIN_INIT(&restoreContext->spinLock,SPINLOCK_LEVEL_UNKNOWN);

    //Extract cwd    
    cwdLen = req->restore.cwdLen;
    if (cwdLen) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("In Restore Setup - initialing cwdlen"));
        restoreContext->cwd = fs_ramAlloc(FS_RAM_DUMP_NAME, cwdLen + 1);
        FS_ASSERT(restoreContext->cwd != NULL);
        memcpy(restoreContext->cwd, req->restore.cwd, cwdLen);
        restoreContext->cwd[cwdLen] = '\0';
    }

    /*
     * Now insert the restoreContext into fs_FileSystem lists
     */

    //DLL_INIT(&fs->restoreContexts);
    fs_lockExclusive(context, &fs->restoreContextsLock, NULL);
    DLL_INSERT_TAIL(&fs->restoreContexts, restoreContext, restoreList);
    fs_unlockExclusive(context, &fs->restoreContextsLock);
#ifdef FS_STANDALONE
    restoreContext->fd = open("/tmp/dumpfile1", 0);
    if (restoreContext->fd == -1) {
        perror("Cannot open dumpfile\n");
        FS_ASSERT(0);
        restoreContext->fd = 0;
    }
#endif  // FS_STANDALONE
    resp->clientCookie = restoreContext->cookie;
    return NFX_OK;
}

/*
 * Add names
 */

int32
fs_restoreNamesThread(
    fs_Context      *context
)
{
    fs_FileSystem   *fs = context->fileSystem;
    fs_MsgRestoreNamesReq_t    *req =
                (fs_MsgRestoreNamesReq_t *)context->reqHdr;
    fs_MsgNdmpResp_t        *resp = (fs_MsgNdmpResp_t *)context->respHdr;
    char                    *blob;
    int                     found = 0;
    fs_RestoreNameList_t       *nameEntry, *entry;
    fs_RestoreContext_t        *restoreContext = NULL; //DLL_NIL;

    blob = req->names.msgBlock;

    /*
     * First find the restoreContext
     */

    fs_lockShared(context, &fs->restoreContextsLock, NULL);
    DLL_ITERATE(restoreContext,&fs->restoreContexts, restoreList) {
        //if (restoreContext != DLL_NIL) {
        if (restoreContext != NULL && restoreContext != DLL_NIL) {
            if (restoreContext->cookie == req->names.ndmpSessId) {
                break;
            }
        }
    }
    fs_unlockShared(context, &fs->restoreContextsLock);

    //if (restoreContext == DLL_NIL) {
    if (restoreContext == NULL || restoreContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        resp->clientCookie = req->names.ndmpSessId;
        return NFX_ERR;
    }
    resp->clientCookie = restoreContext->cookie;
    if (blob && *blob) {
        
        nameEntry = fs_ramAlloc(FS_RAM_RESTORE_NAME,
                                sizeof (fs_RestoreNameList_t));
        FS_ASSERT(nameEntry != NULL);
		bzero(nameEntry, sizeof (fs_RestoreNameList_t));
        nameEntry->name = fs_ramAlloc(FS_RAM_RESTORE_NAME, req->names.oldLen +
                                      1);
        memcpy(nameEntry->name, blob, req->names.oldLen);
        nameEntry->name[req->names.oldLen] = '\0';
        nameEntry->nameLen = req->names.oldLen;
        nameEntry->newPath = fs_ramAlloc(FS_RAM_RESTORE_NAME,
                                         req->names.newLen + 1);
        memcpy(nameEntry->newPath, &blob[req->names.oldLen],
               req->names.newLen);
        nameEntry->newPath[req->names.newLen] = '\0';
        nameEntry->newPathLen = req->names.newLen;
		nameEntry->restored = 0;
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Restore Names: name is %s new %s",
                 nameEntry->name, nameEntry->newPath));
        found = 0;
        DLL_ITERATE(entry, &restoreContext->nameList, link) {
            if (strcmp(entry->name, nameEntry->name) < 0) {
                DLL_INSERT_BEFORE(&restoreContext->nameList,
                                  entry, nameEntry, link);
                found = 1;
                break;
            }
        }
        if (!found) {
            DLL_INSERT_TAIL(&restoreContext->nameList, nameEntry, link);
			if (strcmp(nameEntry->name, "/") != 0) {
				restoreContext->extract = 1;
			}
        }
    }
    
    return NFX_OK;
}

    /*
     * Start restore
     */

extern int64    lockStats[FS_LOCK_TYPE_MAX];
int32
fs_restoreStartThread(
    fs_Context          *context
)
{
    fs_FileSystem           *fs = context->fileSystem;
    fs_RestoreContext_t     *restoreContext = NULL; //DLL_NIL;
    fs_MsgStartRestoreReq_t *req = (fs_MsgStartRestoreReq_t *)context->reqHdr;
    fs_Status               status;
    fs_RestoreNameList_t    *entry;
	fs_RestoreIoQueue_t		*q;
    eeeRTC_t				sTime, delta;
    eee_desc_t              *edesc, *tedesc;

    fs_lockShared(context, &fs->restoreContextsLock, NULL);
    DLL_ITERATE(restoreContext, &fs->restoreContexts, restoreList) {
        //if (restoreContext != DLL_NIL) {
        if (restoreContext != NULL && restoreContext != DLL_NIL) {
            if (restoreContext->cookie == req->start.ndmpSessId) {
                break;
            }
        }
    }
    fs_unlockShared(context, &fs->restoreContextsLock);
    //if (restoreContext == DLL_NIL) {
    if (restoreContext == NULL || restoreContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        DLL_ITERATE(restoreContext, &fs->restoreContexts, restoreList) {
            if (restoreContext != DLL_NIL && restoreContext != NULL) {
                if (restoreContext->cookie == req->start.ndmpSessId) {
                    DLL_REMOVE(&context->fileSystem->restoreContexts,
                                restoreContext, restoreList);
                    break;
                }
            }
        }
        (void)fs_sendNdmpResponse(context, FS_REQ_REST_START, req->start.ndmpSessId,
                                  req->hdr.clientPrivate,
                                  FS_ERR_INVAL);
        return NFX_ERR;
    }

    /*
     * Setup the restoreContext to contain this thread's handle for
     * iocompletion wakeups
     */

    restoreContext->smHandle = context->smHandle;
    (void)fs_sendNdmpResponse(context, FS_REQ_REST_START, req->start.ndmpSessId,
                                  req->hdr.clientPrivate,
                                  FS_ERR_NONE);

	eee_rtc(&sTime);

	/*
	 *  Initiate the tape i/o first
	 */

    restoreContext->curOffset = restoreContext->iobufLen;
	(void)fs_ndmpReadTape(context, restoreContext);
    /*
     * Do the actual restore. On return restoreContext is full cleaned
     * and also done/abort message is already sent to ndmp
     */

    status = fs_restore(context, restoreContext);
    DLL_ITERATE(entry, &restoreContext->nameList, link) {

#if NOTDEF
		if (status == FS_ERR_NONE || entry->restored ||
			 strcmp(entry->name, "/") == 0) {
#endif
		if (status == FS_ERR_NONE) {
				fs_ndmpSendRestoreLog(context, restoreContext, FS_ERR_NONE,
									  NDMP_LOG_MSG_TYP_FILE,
									  0, entry->name, NULL);
		} else {
				fs_ndmpSendRestoreLog(context, restoreContext, status,
									  NDMP_LOG_MSG_TYP_ERR,
									  0, entry->name, 
                                      fs_errCodeToString(status));
		}

        /*
         * Give NDMP a chance to process the message
         */

        fs_yieldTemp(context);
    }

    eee_rtcDuration(&sTime, &delta);
    if (status != FS_ERR_NONE && !restoreContext->abort) {

        /*
         * Log an abort log and send an abort message
         */
        fs_ndmpSendRestoreLog(context, restoreContext, status,
                              NDMP_LOG_MSG_TYP_ERR,
                              0, NULL, "Aborting restore");
        fs_ndmpSendRestoreDoneOrAbort(context, restoreContext, FS_ERR_IO);
    } else {

        /*
         * Send the completion message
         */

        fs_ndmpSendRestoreDoneOrAbort(context, restoreContext, FS_ERR_NONE);
    }
#ifdef FS_DEBUG
    D("Total tape time %lld ms total tape io %u\n",
            eee_rtcToMS(&restoreContext->totTapeTime), restoreContext->totTapeIo);
    D("Total iowaits %u\n", restoreContext->totIoWaits);
    D("Total restore time %lld ms\n", eee_rtcToMS(&delta));
#endif
    fs_cleanRestoreContext(context, restoreContext, FALSE);
#if 0
    DLL_ITERATE(restoreContext, &fs->restoreContexts, restoreList) {
        if (restoreContext != DLL_NIL && restoreContext != NULL) {
            if (restoreContext->cookie == req->start.ndmpSessId) {
                DLL_REMOVE(&context->fileSystem->restoreContexts,
                            restoreContext, restoreList);
                break;
            }
        }
    }
#else
    fs_lockExclusive(context, &fs->restoreContextsLock, NULL);
    DLL_REMOVE(&context->fileSystem->restoreContexts, restoreContext,
                restoreList);
    fs_unlockExclusive(context, &fs->restoreContextsLock);
#endif
more_iolist:
	DLL_ITERATE(q, &restoreContext->ioList, list) {
#if 0
			if (q->sequence == restoreContext->neSequence) {
					restoreContext->neSequence++;
					DLL_REMOVE(&restoreContext->ioList, q, list);
					NFX_SPIN_UNLOCK(&restoreContext->spinLock);
					status = q->fsReq->iodone.status;
					if (status == FS_ERR_NONE) {
        				memcpy(restoreContext->iobuf,
							   q->fsReq->iodone.data +
                               q->fsReq->iodone.data_offset,
                               q->fsReq->iodone.dataLen);
                        bzero(&restoreContext->iobuf[q->fsReq->iodone.dataLen],
                              restoreContext->iobufLen -
                              q->fsReq->iodone.dataLen);

                        eee_ramDealloc((void *)q->fsReq->iodone.data);
					}
                    fs_ramDealloc(FS_RAM_RESTORE_IODONE, q->fsReq);
                    fs_ramDealloc(FS_RAM_RESTORE_IODONE_Q, q);
                    return status;
			}
#else
            /*
             * At this point all the pending io completetions should
             * be destoryed
             */

			DLL_REMOVE(&restoreContext->ioList, q, list);
            //FS_ASSERT(status != FS_ERR_NONE);
#ifndef FS_STANDALONE
            edesc = (eee_desc_t *)q->fsReq->iodone.data;
            if (edesc)
                eee_freePacket(edesc);
#endif
            fs_ramDealloc(FS_RAM_RESTORE_IODONE, q->fsReq);
            fs_ramDealloc(FS_RAM_RESTORE_IODONE_Q, q);
            goto more_iolist;
#endif
	}
    fs_ramDealloc(FS_RAM_RESTORE_CONTEXT, restoreContext);
    if (status != FS_ERR_NONE) {
        return NFX_ERR;
    }
    fs_syncThread(context);
    //fs_bufSyncWritebackLists(context);
    return NFX_OK;
}

/*
 * Handle restore io completion
 */

int32
fs_restoreIODoneThread(
    fs_Context          *context
)
{
    fs_FileSystem       *fs = context->fileSystem;
    fs_RestoreContext_t    *restoreContext = NULL; //DLL_NIL;
    fs_MsgRestoreIODoneReq_t  *req =
                (fs_MsgRestoreIODoneReq_t *)context->reqHdr, *fsReq;
    esm_event_t         event;
    int                 rc = NFX_OK; 
	fs_RestoreIoQueue_t	*q, *tempQ;
	int					put = 0;

    //printf("Restore IO Done\n");
    fs_lockShared(context, &fs->restoreContextsLock, NULL);
    DLL_ITERATE(restoreContext, &fs->restoreContexts, restoreList) {
        //if (restoreContext != DLL_NIL) {
        if (restoreContext != NULL && restoreContext != DLL_NIL) {
            if (restoreContext->cookie == req->iodone.ndmpSessId) {
                break;
            }
        }
    }
    fs_unlockShared(context, &fs->restoreContextsLock);
    //if (restoreContext == DLL_NIL) {
    if (restoreContext == NULL || restoreContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        return NFX_ERR;
    }

    /*
     * Wakeup the thread sleeping for this i/o
     */
    event.event = FS_THREAD_EVENT_DUMP_REST_IOCOMPLETE;
	fsReq = fs_ramAlloc(FS_RAM_RESTORE_IODONE,
                        sizeof(fs_MsgRestoreIODoneReq_t));
    memcpy(fsReq, req, sizeof(fs_MsgRestoreIODoneReq_t));

	/*
	 *  Queue the response in i/o list
	 */

	q = fs_ramAlloc(FS_RAM_RESTORE_IODONE_Q, sizeof (fs_RestoreIoQueue_t));
    bzero(q, sizeof (fs_RestoreIoQueue_t));
	FS_ASSERT(q != NULL);
	q->fsReq = fsReq;
    q->sequence = fsReq->iodone.sequence;
    //printf("Done i/o for sequence %d\n", q->sequence);

	NFX_SPIN_LOCK(&restoreContext->spinLock);
	restoreContext->pendingIo--;
	FS_ASSERT(restoreContext->pendingIo >= 0 && restoreContext->pendingIo <= 64);
	if (DLL_IS_EMPTY(&restoreContext->ioList)) {
			DLL_INSERT_HEAD(&restoreContext->ioList, q, list);
	} else {
            DLL_ITERATE_BACK(tempQ, &restoreContext->ioList, list) {
                    if (tempQ->sequence > q->sequence) {
                            continue;
                    }
                    DLL_INSERT_AFTER(&restoreContext->ioList, tempQ, q, list);
					put++;
                    break;
            }
			if (!put) {
				DLL_INSERT_HEAD(&restoreContext->ioList, q, list);
			}
    }
	
	if (restoreContext->ioWait) {
			event.p1.u.p = NULL;
			event.p2.u.w = fsReq->iodone.status;
			rc = thread_event(restoreContext->smHandle, &event);
            //printf("Waking up waiter\n");
			restoreContext->ioWait = 0;
	}
	NFX_SPIN_UNLOCK(&restoreContext->spinLock);
    FS_ASSERT(rc == NFX_OK);
    return rc;

}

int32
fs_restoreAbortThread(
    fs_Context          *context
)
{
    fs_FileSystem       *fs = context->fileSystem;
    fs_RestoreContext_t    *restoreContext = NULL; //DLL_NIL;
    fs_MsgAbortRestoreReq_t  *req = (fs_MsgAbortRestoreReq_t *)context->reqHdr;
    fs_MsgNdmpResp_t        *resp = (fs_MsgNdmpResp_t *)context->respHdr;
    int                 rc = NFX_OK;
	esm_event_t			event;
	if ( fs->restoreContexts.tail == NULL ||
	    fs->restoreContexts.head == NULL) {
		goto no_context;
	}
    fs_lockShared(context, &fs->restoreContextsLock, NULL);
    if (fs->dumpContexts.head == NULL ||
        fs->dumpContexts.tail == NULL) {
        goto done_search;
    }
    DLL_ITERATE(restoreContext, &fs->restoreContexts, restoreList) {
        //if (restoreContext != DLL_NIL) {
        if (restoreContext != NULL && restoreContext != DLL_NIL) {
            if (restoreContext->cookie == req->abort.ndmpSessId) {
                break;
            }
        }
    }
done_search:
    fs_unlockShared(context, &fs->restoreContextsLock);
no_context:
    //if (restoreContext == DLL_NIL) {
    if (restoreContext == NULL || restoreContext == DLL_NIL) {
        /*
         * XXX - Send an abort message
         */

        resp->clientCookie = req->abort.ndmpSessId;
        return NFX_ERR;
    }
    resp->clientCookie = restoreContext->cookie;

    restoreContext->abort = 1;
	NFX_SPIN_LOCK(&restoreContext->spinLock);
	if (restoreContext->ioWait) {
            event.event = FS_THREAD_EVENT_DUMP_REST_IOCOMPLETE;
			event.p1.u.p = NULL;
			event.p2.u.w = FS_ERR_IO;
			restoreContext->ioWait = 0;
			rc = thread_event(restoreContext->smHandle, &event);
	}
	NFX_SPIN_UNLOCK(&restoreContext->spinLock);
    FS_ASSERT(rc == NFX_OK);
    return rc;

}

/*
 * Handle messages to be send to ndmp on write's behalf
 */

int32
fs_ndmpReadTape(
    fs_Context              *context,
    fs_RestoreContext_t     *restoreContext
)
{
    int     rc;
    int32   numIOs = 2; // Rahul 12 to 4
    fs_Status                  status = FS_ERR_NONE;
	ndmp_read_data_req_msg_t   msg;

    if (restoreContext->numBytes == 0) {
        return status;
    }


	/*
	 * Rahul - Throttle i/o on i/o bandwidth as well as memory availability
	 */
	NFX_SPIN_LOCK(&restoreContext->spinLock);
#if 0
	if (restoreContext->pendingIo > restoreContext->maxPendingIo) {
            NFX_SPIN_UNLOCK(&restoreContext->spinLock);
			return status;
	}
#else
	if (restoreContext->pendingIo > FS_MAX_TAPE_READ_PENDING) {
            NFX_SPIN_UNLOCK(&restoreContext->spinLock);
			return status;
	}
#endif

#if 0
	if (((int64)(restoreContext->ioSequence - restoreContext->neSequence)) >
					(restoreContext->maxPendingIo * 2)) {
            NFX_SPIN_UNLOCK(&restoreContext->spinLock);
			return status;
	}
#else
	if (((int64)(restoreContext->ioSequence - restoreContext->neSequence)) >
					(FS_MAX_TAPE_READ_PENDING * 2)) {
            NFX_SPIN_UNLOCK(&restoreContext->spinLock);
			return status;
	}
#endif
    NFX_SPIN_UNLOCK(&restoreContext->spinLock);

    numIOs = MIN(numIOs,
                (int32)(restoreContext->numBytes / restoreContext->iobufLen));
    
    restoreContext->numBytes -= (numIOs * restoreContext->iobufLen);
    /*
     * Construct the message
     */

    //printf("Doing i/o for sequence %d\n", restoreContext->ioSequence);
	msg.cl_cookie = (void *)restoreContext;
	msg.hdr.mid = NDMP_MID_READ_DATA_REQ;
	msg.ioCookie = restoreContext->ioSequence;
    msg.numIOs = numIOs;
	msg.hdr.clientCookie = restoreContext->cookie;

	FS_ASSERT(restoreContext->pendingIo >= 0 &&
			  restoreContext->pendingIo <= 64);
    restoreContext->pendingIo += numIOs;
    restoreContext->totTapeIo += numIOs;
    restoreContext->ioSequence += numIOs;
    //printf("Started %d target i/o ios new sequence %llu\n", numIOs,
    //               restoreContext->ioSequence);
	
#ifndef FS_STANDALONE
	rc = eee_sendMessage((void *)&msg, sizeof (msg),
						 PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
												  NFX_NFP_FP_ANY,
												  eee_getApplicationId("ndmp"),
												  EEE_CONTROL,
												  NFX_COS_HIGH,
												  0),
						fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
	rc = NFX_OK;
#endif
	FS_ASSERT(rc == NFX_OK);
    //printf("Number of pending tape reads %d\n", restoreContext->pendingIo);
    return status;
}

int
fs_ndmpFetchTapeData(
		fs_Context	*context,
		fs_RestoreContext_t	*restoreContext)
{
	fs_RestoreIoQueue_t	*q;
	fs_Status			status;
    esm_event_t         event;
    eeeRTC_t			sTime, delta;
    eee_desc_t          *edesc, *tedesc;
    int                 nBufs, i;
    int                 ebufSize = 8192;
    uint32  copySize;
    uint32  iolen = 0;
    char    *cp = NULL;
    uint32  offset;
#ifdef FS_DO_SLOW_PATH    
	ndmp_read_data_req_msg_t   msg;

    /*
     * Handle the case of ndmp not knowing backup size. Slow read path
     */

    // When we are ready to handle unavailability of bkup
    // size
    if (restoreContext->numBytes == 0) {
            int rc = NFX_OK;
            if (restoreContext->abort) {
                    NFX_SPIN_UNLOCK(&restoreContext->spinLock);
                    return FS_ERR_IO;
            }
            /*
             * Construct the message
             */

            //printf("Doing i/o for sequence %d\n", restoreContext->ioSequence);
            msg.cl_cookie = (void *)restoreContext;
            msg.hdr.mid = NDMP_MID_READ_DATA_REQ;
            msg.ioCookie = restoreContext->ioSequence;
            msg.numIOs = 1;
            msg.hdr.clientCookie = restoreContext->cookie;
            restoreContext->pendingIo += 1;
            restoreContext->totTapeIo += 1;
            restoreContext->ioSequence += 1;
            //printf("Started %d target i/o ios new sequence %llu\n", numIOs,
             //               restoreContext->ioSequence);
            
#ifndef FS_STANDALONE
            rc = eee_sendMessage((void *)&msg, sizeof (msg),
                                 PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                          NFX_NFP_FP_ANY,
                                                          eee_getApplicationId("ndmp"),
                                                          EEE_CONTROL,
                                                          NFX_COS_HIGH,
                                                          0),
                                fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
            rc = NFX_OK;
#endif
            restoreContext->ioWait = 1;
            restoreContext->totIoWaits++;
            NFX_SPIN_UNLOCK(&restoreContext->spinLock);
            //printf("waiting for tape reads to complete\n");
            //printf("Waiting for %llu\n", restoreContext->neSequence);
            eee_rtc(&sTime);
#ifdef FS_NO_LOCK_TIMEOUT
            fs_unlockShared(context, &context->fileSystem->accessLock);         
            context->flags &= ~FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
            fs_unlockShared(context, &context->fileSystem->modifyLock);
            context->flags &= ~FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
#endif
            fs_yield(context, &event, FS_YIELD_TIME_FOREVER, FS_YIELD_TYPE_IO_TR);
#ifdef FS_NO_LOCK_TIMEOUT
            fs_lockShared(context, &context->fileSystem->modifyLock, NULL);
            context->flags |= FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
            fs_lockShared(context, &context->fileSystem->accessLock, NULL);
            context->flags |= FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
#endif
            if (event.event != FS_THREAD_EVENT_DUMP_REST_IOCOMPLETE) {
                FS_ASSERT(0);
            }
            if (restoreContext->abort) {
                    return FS_ERR_IO;
            }
            eee_rtcDuration(&sTime, &delta);
            (void)eee_rtcIncr(&restoreContext->totTapeTime, &delta);
            return NFX_OK;
    }
#endif

try_again:
    //printf("Fetching for %llu\n", restoreContext->neSequence);
	NFX_SPIN_LOCK(&restoreContext->spinLock);
	DLL_ITERATE(q, &restoreContext->ioList, list) {
            //printf("q sequence %d exp seq %d\n", q->sequence,
             //               restoreContext->neSequence);
			if (q->sequence == restoreContext->neSequence) {
					restoreContext->neSequence++;
					DLL_REMOVE(&restoreContext->ioList, q, list);
					NFX_SPIN_UNLOCK(&restoreContext->spinLock);
					status = q->fsReq->iodone.status;
					if (status == FS_ERR_NONE) {
#if 0
        				memcpy(restoreContext->iobuf,
							   q->fsReq->iodone.data +
                               q->fsReq->iodone.data_offset,
                               q->fsReq->iodone.dataLen);
                        bzero(&restoreContext->iobuf[q->fsReq->iodone.dataLen],
                              restoreContext->iobufLen -
                              q->fsReq->iodone.dataLen);

                        eee_ramDealloc((void *)q->fsReq->iodone.data);
#endif
                        nBufs = (q->fsReq->iodone.dataLen + ebufSize - 1) /
                                ebufSize;
                        edesc = (eee_desc_t *)q->fsReq->iodone.data;
                        iolen = q->fsReq->iodone.dataLen;
                        cp = restoreContext->iobuf;
                        while (edesc && iolen > 0) {
                                FS_ASSERT(edesc->bd[1].len <= ebufSize);
                                //offset = 0;
                                //copySize = MIN(iolen, ebufSize - offset);
                                copySize = MIN(iolen, edesc->bd[1].len);
                                memcpy(cp, edesc->bd[1].buf, copySize);
                                cp += copySize;
                                iolen -= copySize;
                                tedesc = edesc->hdr.next;
                                eee_deallocateBuffer(edesc->bd[1].buf);
                                eee_deallocateDesc(edesc);
                                edesc = tedesc;
					    }
                        if (iolen != 0) {
                            printf("iolen remaining %d\n", iolen);
                            bzero(cp, iolen);
                        }
					}
                    fs_ramDealloc(FS_RAM_RESTORE_IODONE, q->fsReq);
                    fs_ramDealloc(FS_RAM_RESTORE_IODONE_Q, q);
                    return status;
			}
	}

    /*
     * If we have been aborted, don't wait for i/o completion,
     * NDMP does not send any more responses
     */
    if (restoreContext->abort) {
            NFX_SPIN_UNLOCK(&restoreContext->spinLock);
            return FS_ERR_IO;
    }

    restoreContext->ioWait = 1;
    restoreContext->totIoWaits++;
	NFX_SPIN_UNLOCK(&restoreContext->spinLock); //Anh
	fs_ndmpReadTape(context, restoreContext);
    //printf("waiting for tape reads to complete\n");
    //printf("Waiting for %llu\n", restoreContext->neSequence);
	eee_rtc(&sTime);
#ifdef FS_NO_LOCK_TIMEOUT
            fs_unlockShared(context, &context->fileSystem->accessLock);         
            context->flags &= ~FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
            fs_unlockShared(context, &context->fileSystem->modifyLock);
            context->flags &= ~FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
#endif
    fs_yield(context, &event, FS_YIELD_TIME_FOREVER, FS_YIELD_TYPE_IO_TR);
#ifdef FS_NO_LOCK_TIMEOUT
            fs_lockShared(context, &context->fileSystem->modifyLock, NULL);
            context->flags |= FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
            fs_lockShared(context, &context->fileSystem->accessLock, NULL);
            context->flags |= FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
#endif
    if (event.event != FS_THREAD_EVENT_DUMP_REST_IOCOMPLETE) {
        FS_ASSERT(0);
    }
	if (restoreContext->abort) {
			return FS_ERR_IO;
	}
    eee_rtcDuration(&sTime, &delta);
    (void)eee_rtcIncr(&restoreContext->totTapeTime, &delta);
    //printf("Woken up trying again - pending reads %d\n",
     //       restoreContext->pendingIo);
    goto try_again;
}

/*
 * Send a log message to ndmp
 */

int32
fs_ndmpSendRestoreLog(
    fs_Context              *context,
    fs_RestoreContext_t        *restoreContext,
    fs_Status               status,
    int32                   type,
    fs_Status               fileStatus,
    char                    *fileName,
    char                    *message
)
{
    int     rc;
    char    *c;
    ndmp_send_log_req_msg_t   *msg;
	int		msgSize = sizeof (ndmp_send_log_req_msg_t);

	if (fileName) {
		msgSize += strlen(fileName);
	}
	if (message) {
		msgSize += strlen(message);
	}
    /*
     * Construct the message
     */

	msg = fs_ramAlloc(FS_RAM_RESTORE_NAME, msgSize);
	bzero(msg, msgSize);
    msg->cl_cookie = (void *)restoreContext;
    msg->hdr.mid = NDMP_MID_SEND_LOG_REQ;
    msg->hdr.clientCookie = restoreContext->cookie;
    msg->status = status;
    msg->log_msgTyp = type;
    c = msg->blob;
    if (type == NDMP_LOG_MSG_TYP_FILE || fileName != NULL) {
        msg->file_recov_stat = fileStatus;
        msg->file_name_len = strlen(fileName) + 1;
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("copying filename"));
        strcpy(c, fileName);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("copied filename"));
        c += msg->file_name_len;
    }

    msg->log_txt_len = 0;
    if (message) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("copying message %s", message));
        strcpy(c, message);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("copied message"));
        msg->log_txt_len = strlen(message) + 1;
    }
    
#ifndef FS_STANDALONE
    rc = eee_sendMessage((void *)msg, msgSize,
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                  NFX_NFP_FP_ANY,
                                                  eee_getApplicationId("ndmp"),
                                                  EEE_CONTROL,
                                                  NFX_COS_HIGH,
											      0),
                        fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
    rc = NFX_OK;
#endif
	fs_ramDealloc(FS_RAM_RESTORE_NAME, msg);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Returning from send log"));
    return rc;
}


int32
fs_ndmpSendRestoreDoneOrAbort(
    fs_Context              *context,
    fs_RestoreContext_t        *restoreContext,
    fs_Status               status
)
{
    int rc;
    ndmp_ops_cmplt_req_msg_t    msg;
    FS_ASSERT(status == FS_ERR_NONE || FS_ERR_IO);
    msg.cl_cookie = (void *)restoreContext;
    msg.hdr.mid = NDMP_MID_OPS_COMPLETE_REQ;
    msg.hdr.clientCookie = restoreContext->cookie;
    msg.status = status;
#ifndef FS_STANDALONE
    rc = eee_sendMessage((void *)&msg, sizeof (msg),
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                  NFX_NFP_FP_ANY,
                                                  eee_getApplicationId("ndmp"),
                                                  EEE_CONTROL,
                                                  NFX_COS_HIGH,
											      0),
                        fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#else
    rc = NFX_OK;
#endif
    return rc;
}

/*
 * Cleanup dump/restore states on unmount. This should have to do work
 * only if we are force unmounting on a crash
 */
EXPORTED void
fs_destroyDumpContextOnUmount(fs_Context  *context)
{
    fs_FileSystem       *fs = context->fileSystem;
    fs_DumpContext_t    *dumpContext, *nextDumpContext;

    DLL_ITERATE_NEXT(dumpContext, nextDumpContext, &fs->dumpContexts, dumpList)
    {
		fs_cleanDumpContext(context, dumpContext, TRUE);

		// fs_cleanDumpContext removed dumpContext from
		// the fs list, so we can simply dealloc it now.
		fs_ramDealloc(FS_RAM_DUMP_CONTEXT, dumpContext);
	}
}


/**
 * Cleanup the restore context.  This should only have to be
 * used during a forced unmount.
 */
EXPORTED void
fs_destroyRestoreContextOnUmount(fs_Context  *context)
{
    fs_FileSystem           *fs = context->fileSystem;
    fs_RestoreContext_t     *restoreContext, *nextRestoreContext;

    DLL_ITERATE_NEXT(restoreContext, nextRestoreContext, &fs->restoreContexts,
                     restoreList)
    {
		fs_cleanRestoreContext(context, restoreContext, TRUE);

		// fs_cleanRestoreContext didn't remove it from the
		// fs list, so we must do that here before deallocing it.
		DLL_REMOVE(&fs->restoreContexts, restoreContext, restoreList);

		fs_ramDealloc(FS_RAM_DUMP_CONTEXT, restoreContext);
	}
}

void
fs_ndmpDumpNodeHistory(fs_Context *context,
                       fs_DumpContext_t *dumpContext,
                       fs_Inode   *inode)
{
    ndmp_NFX_FHReq_t   *req;
    int rc;
    int  maxNodeCount = FS_HISTORY_BUF_SIZE / sizeof (fs_NodeHistory_t);

    fs_yieldPeriodic(context);
#ifdef FS_HIST_WAIT
    fs_lockExclusive(context, &dumpContext->histLock, NULL);
#else
    NFX_SPIN_LOCK(&dumpContext->spinLock);
#endif
    if (FS_I_IS_DIR(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_DIR;
    } else if (FS_I_IS_REG(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_REG;
    } else if (FS_I_IS_FIFO(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_FIFO;
    } else if (FS_I_IS_SOCK(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_SOCK;
    } else if (FS_I_IS_CHR(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_CSPEC;
    } else if (FS_I_IS_BLK(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_BSPEC;
    } else if (FS_I_IS_SYML(inode->dinode.attrs)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_SLINK;
    } else {
        dumpContext->curNodeHist->file_Type = NDMP_F_OTHER;
    }
    dumpContext->curNodeHist->mtime = inode->dinode.mtime;
    dumpContext->curNodeHist->ctime = inode->dinode.ctime;
    dumpContext->curNodeHist->atime = inode->dinode.atime;
    dumpContext->curNodeHist->group = inode->sd->u.nfs.gid;
    dumpContext->curNodeHist->owner = inode->sd->u.nfs.uid;
    dumpContext->curNodeHist->size =  inode->dinode.size;
    dumpContext->curNodeHist->links = inode->dinode.linkCnt;
    //msg.fattr = inode->dinode.attrs & FS_IFMT_MASK;
    dumpContext->curNodeHist->fattr = inode->sd->u.nfs.mode & FS_IMODE_MASK;
    
    //For partial dump, the root of the partial tree has to be 2 for Veritas
    if (inode->inum != dumpContext->partialTreeRootInum) {
        dumpContext->curNodeHist->inum = inode->inum;
    } else {
        dumpContext->curNodeHist->inum = FS_ROOT_INODE;
    }
    dumpContext->curNodeHist++;
    dumpContext->nodeCount++;
    //if (((char *)dumpContext->curNodeHist - dumpContext->historyBuf) >=
            //FS_HISTORY_BUF_SIZE) {
    if (dumpContext->nodeCount >= maxNodeCount) {
                req = fs_ramAlloc(FS_RAM_DUMP_CONTEXT,
                                  sizeof (ndmp_NFX_FHReq_t) +
                                (uint32)((char *)dumpContext->curNodeHist -
                                    dumpContext->historyBuf) - 1);
                req->sessId = dumpContext->ndmpSessId;
                req->mid = FH_NODE_REQ;
                req->num_files = dumpContext->nodeCount;
                req->tot_len = ((char *)dumpContext->curNodeHist -
                                dumpContext->historyBuf);
                //bcopy(req->content, dumpContext->historyBuf, req->tot_len);
                bcopy(dumpContext->historyBuf, req->content, req->tot_len);
#ifndef FS_STANDALONE
#ifdef FS_HIST_WAIT
check_again:
                NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
                if (!FS_WILL_HIST_MESSAGE_FIT(dumpContext, (req->tot_len - 1 +
                                               sizeof (ndmp_NFX_FHReq_t)))) {
                    esm_event_t    event;
                    dumpContext->histWait = TRUE;
                    dumpContext->histWaiter = context->smHandle;
                    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
                    fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                            FS_YIELD_TYPE_ADMIN);
                    FS_ASSERT(event.event == FS_THREAD_EVENT_SEND_HIST);
                    goto check_again;
                }
                FS_USE_HIST_MESSAGE(dumpContext, req->tot_len - 1 + 
                                    sizeof (ndmp_NFX_FHReq_t));
                NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
#endif
                rc = eee_sendMessage((void *)req,
                                sizeof (ndmp_NFX_FHReq_t) +
                                req->tot_len - 1,
                                dumpContext->ndmp_ssc_src_port,
                                fs_packedPortAddr(nfxMyCPUId,
                                                  EEE_CONTROL));
                dumpContext->nodeCount = 0;
                dumpContext->curNodeHist = (void *) dumpContext->historyBuf;
                fs_ramDealloc(FS_RAM_DUMP_CONTEXT, req);
#else
                rc = NFX_OK;
#endif
    }
#ifdef FS_HIST_WAIT
    fs_unlockExclusive(context, &dumpContext->histLock);
#else
    NFX_SPIN_UNLOCK(&dumpContext->spinLock);
#endif
    //fs_sleep(context, 0);
    //fs_yieldPeriodic(context);
    return;
}

void
fs_ndmpDumpNodeHistoryNodeBuf(fs_Context *context,
                       fs_DumpContext_t *dumpContext,
                       fs_NdmpHistoryNode_t   *node)
{
    ndmp_NFX_FHReq_t   *req;
    int  maxNodeCount = FS_HISTORY_BUF_SIZE / sizeof (fs_NodeHistory_t);
    int rc;

    fs_yieldPeriodic(context);
#ifdef FS_HIST_WAIT
    fs_lockExclusive(context, &dumpContext->histLock, NULL);
#else
    NFX_SPIN_LOCK(&dumpContext->spinLock);
#endif
    if (FS_I_IS_DIR(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_DIR;
    } else if (FS_I_IS_REG(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_REG;
    } else if (FS_I_IS_FIFO(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_FIFO;
    } else if (FS_I_IS_SOCK(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_SOCK;
    } else if (FS_I_IS_CHR(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_CSPEC;
    } else if (FS_I_IS_BLK(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_BSPEC;
    } else if (FS_I_IS_SYML(node->mode)) {
        dumpContext->curNodeHist->file_Type = NDMP_F_SLINK;
    } else {
        dumpContext->curNodeHist->file_Type = NDMP_F_OTHER;
    }
    dumpContext->curNodeHist->mtime = node->mtime;
    dumpContext->curNodeHist->ctime = node->ctime;
    dumpContext->curNodeHist->atime = node->atime;
    dumpContext->curNodeHist->group = node->gid;
    dumpContext->curNodeHist->owner = node->uid;
    dumpContext->curNodeHist->size =  node->size;
    dumpContext->curNodeHist->links = node->linkCnt;
    //msg.fattr = inode->dinode.attrs & FS_IFMT_MASK;
    dumpContext->curNodeHist->fattr = node->mode & FS_IMODE_MASK;
    
    dumpContext->curNodeHist->inum = node->inum;
    dumpContext->curNodeHist++;
    dumpContext->nodeCount++;
    if (dumpContext->nodeCount >= maxNodeCount) {
                req = fs_ramAlloc(FS_RAM_DUMP_CONTEXT,
                                  sizeof (ndmp_NFX_FHReq_t) +
                                (uint32)((char *)dumpContext->curNodeHist -
                                    dumpContext->historyBuf) - 1);
                req->sessId = dumpContext->ndmpSessId;
                req->mid = FH_NODE_REQ;
                req->num_files = dumpContext->nodeCount;
                req->tot_len = ((char *)dumpContext->curNodeHist -
                                dumpContext->historyBuf);
                bcopy(dumpContext->historyBuf, req->content, req->tot_len);
#ifndef FS_STANDALONE
#ifdef FS_HIST_WAIT
check_again:
                NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
                if (!FS_WILL_HIST_MESSAGE_FIT(dumpContext, (req->tot_len - 1 +
                                               sizeof (ndmp_NFX_FHReq_t)))) {
                    esm_event_t    event;
                    dumpContext->histWait = TRUE;
                    dumpContext->histWaiter = context->smHandle;
                    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
                    fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                            FS_YIELD_TYPE_ADMIN);
                    FS_ASSERT(event.event == FS_THREAD_EVENT_SEND_HIST);
                    goto check_again;
                }
                FS_USE_HIST_MESSAGE(dumpContext, req->tot_len - 1 + 
                                    sizeof (ndmp_NFX_FHReq_t));
                NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
#endif
                rc = eee_sendMessage((void *)req,
                                sizeof (ndmp_NFX_FHReq_t) +
                                req->tot_len - 1,
                                dumpContext->ndmp_ssc_src_port,
                                fs_packedPortAddr(nfxMyCPUId,
                                                  EEE_CONTROL));
                dumpContext->nodeCount = 0;
                dumpContext->curNodeHist = (void *) dumpContext->historyBuf;
                fs_ramDealloc(FS_RAM_DUMP_CONTEXT, req);
#else
                rc = NFX_OK;
#endif
    }
#ifdef FS_HIST_WAIT
    fs_unlockExclusive(context, &dumpContext->histLock);
#else
    NFX_SPIN_UNLOCK(&dumpContext->spinLock);
#endif
    //fs_sleep(context, 0);
    fs_yieldPeriodic(context);
    return;
}

void
fs_ndmpFlushNodeHistory(fs_Context *context, fs_DumpContext_t *dumpContext)
{
    ndmp_NFX_FHReq_t   *req;
    int rc;
    fs_yieldPeriodic(context);
#ifdef FS_HIST_WAIT
    fs_lockExclusive(context, &dumpContext->histLock, NULL);
#else
    NFX_SPIN_LOCK(&dumpContext->spinLock);
#endif
    if (dumpContext->nodeCount != 0) {
                req = fs_ramAlloc(FS_RAM_DUMP_CONTEXT,
                                  sizeof (ndmp_NFX_FHReq_t) +
                                (uint32)((char *)dumpContext->curNodeHist -
                                    dumpContext->historyBuf) - 1);
                req->sessId = dumpContext->ndmpSessId;
                req->mid = FH_NODE_REQ;
                req->num_files = dumpContext->nodeCount;
                req->tot_len = ((char *)dumpContext->curNodeHist -
                                dumpContext->historyBuf);
                bcopy(dumpContext->historyBuf, req->content, req->tot_len);
#ifndef FS_STANDALONE
#ifdef FS_HIST_WAIT
check_again:
                NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
                if (!FS_WILL_HIST_MESSAGE_FIT(dumpContext, (req->tot_len - 1 +
                                               sizeof (ndmp_NFX_FHReq_t)))) {
                    esm_event_t    event;
                    dumpContext->histWait = TRUE;
                    dumpContext->histWaiter = context->smHandle;
                    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
                    fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                            FS_YIELD_TYPE_ADMIN);
                    FS_ASSERT(event.event == FS_THREAD_EVENT_SEND_HIST);
                    goto check_again;
                }
                FS_USE_HIST_MESSAGE(dumpContext, req->tot_len - 1 + 
                                    sizeof (ndmp_NFX_FHReq_t));
                NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
#endif
                rc = eee_sendMessage((void *)req,
                                sizeof (ndmp_NFX_FHReq_t) +
                                req->tot_len - 1,
                                dumpContext->ndmp_ssc_src_port,
                                fs_packedPortAddr(nfxMyCPUId,
                                                  EEE_CONTROL));
                dumpContext->nodeCount = 0;
                dumpContext->curNodeHist = (void *) dumpContext->historyBuf;
                fs_ramDealloc(FS_RAM_DUMP_CONTEXT, req);
#else
                rc = NFX_OK;
#endif

            
    }
#ifdef FS_HIST_WAIT
    fs_unlockExclusive(context, &dumpContext->histLock);
#else
    NFX_SPIN_UNLOCK(&dumpContext->spinLock);
#endif
    fs_yieldPeriodic(context);
}

void
fs_ndmpDumpDirHistory(fs_Context *context,
                       fs_DumpContext_t *dumpContext,
                       fs_Inumber   parent,
                       fs_Inumber   inum,
                       char         *name,
                       int          nameLen)
{
        ndmp_NFX_FHReq_t   *req;
        uint16          availSpace;
        uint16          reqSpace;
        int             rc;

        fs_yieldPeriodic(context);
        availSpace = FS_HISTORY_BUF_SIZE -
                        ((char *)dumpContext->curDirHist -
                            dumpContext->historyBuf);
        reqSpace = sizeof (fs_DirHistory_t) + nameLen;
        if (reqSpace > availSpace) {
                req = fs_ramAlloc(FS_RAM_DUMP_CONTEXT,
                                  sizeof (ndmp_NFX_FHReq_t) +
                                (uint32)((char *)dumpContext->curDirHist -
                                    dumpContext->historyBuf) - 1);
                FS_ASSERT(req != NULL);
                /*
                 * Send the dir history
                 */
                req->sessId = dumpContext->ndmpSessId;
                req->mid = FH_DIR_REQ;
                req->num_files = dumpContext->dirCount;
                req->tot_len = ((char *)dumpContext->curDirHist -
                                dumpContext->historyBuf);
                bcopy(dumpContext->historyBuf, req->content, req->tot_len);
#ifndef FS_STANDALONE
#ifdef FS_HIST_WAIT
check_again:
                NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
                if (!FS_WILL_HIST_MESSAGE_FIT(dumpContext, (req->tot_len - 1 +
                                               sizeof (ndmp_NFX_FHReq_t)))) {
                    esm_event_t    event;
                    dumpContext->histWait = TRUE;
                    dumpContext->histWaiter = context->smHandle;
                    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
                    fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                            FS_YIELD_TYPE_ADMIN);
                    FS_ASSERT(event.event == FS_THREAD_EVENT_SEND_HIST);
                    goto check_again;
                }
                FS_USE_HIST_MESSAGE(dumpContext, req->tot_len - 1 + 
                                    sizeof (ndmp_NFX_FHReq_t));
                NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
#endif
                rc = eee_sendMessage((void *)req,
                                sizeof (ndmp_NFX_FHReq_t) +
                                req->tot_len - 1,
                                dumpContext->ndmp_ssc_src_port,
                                fs_packedPortAddr(nfxMyCPUId,
                                                  EEE_CONTROL));
                dumpContext->dirCount = 0;
                dumpContext->curDirHist = (void *) dumpContext->historyBuf;
                fs_ramDealloc(FS_RAM_DUMP_CONTEXT, req);
#else
                rc = NFX_OK;
#endif

        }
        if (inum != dumpContext->partialTreeRootInum)   {
                dumpContext->curDirHist->inum = inum;
        } else {
                dumpContext->curDirHist->inum = FS_ROOT_INODE;
        }
        if (parent != dumpContext->partialTreeRootInum)   {
                dumpContext->curDirHist->parent = parent;
        } else {
                dumpContext->curDirHist->parent = FS_ROOT_INODE;
        }
        dumpContext->curDirHist->len = nameLen;
        strcpy(dumpContext->curDirHist->name, name);
        dumpContext->dirCount++;
        FS_DIR_HISTORY_ADV(dumpContext->curDirHist);
        fs_yieldPeriodic(context);
}

void
fs_ndmpFlushDirHistory(fs_Context *context, fs_DumpContext_t *dumpContext)
{
        ndmp_NFX_FHReq_t   *req;
        int             rc;
        fs_yieldPeriodic(context);
        if ((char *)dumpContext->curDirHist != dumpContext->historyBuf) {
                req = fs_ramAlloc(FS_RAM_DUMP_CONTEXT,
                                  sizeof (ndmp_NFX_FHReq_t) +
                                  FS_HISTORY_BUF_SIZE);
                FS_ASSERT(req != NULL);
                /*
                 * Send the dir history
                 */
                req->sessId = dumpContext->ndmpSessId;
                req->mid = FH_DIR_REQ;
                req->num_files = dumpContext->dirCount;
                req->tot_len = ((char *)dumpContext->curDirHist -
                                dumpContext->historyBuf);
                bcopy(dumpContext->historyBuf, &req->content[0], req->tot_len);
#ifndef FS_STANDALONE
#ifdef FS_HIST_WAIT
check_again:
                NFX_SPIN_LOCK(&fs_dumpSessionsSpinLock);
                if (!FS_WILL_HIST_MESSAGE_FIT(dumpContext, (req->tot_len - 1 +
                                               sizeof (ndmp_NFX_FHReq_t)))) {
                    esm_event_t    event;
                    dumpContext->histWait = TRUE;
                    dumpContext->histWaiter = context->smHandle;
                    NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
                    fs_yield(context, &event, FS_YIELD_TIME_FOREVER,
                            FS_YIELD_TYPE_ADMIN);
                    FS_ASSERT(event.event == FS_THREAD_EVENT_SEND_HIST);
                    goto check_again;
                }
                FS_USE_HIST_MESSAGE(dumpContext, req->tot_len - 1 + 
                                    sizeof (ndmp_NFX_FHReq_t));
                NFX_SPIN_UNLOCK(&fs_dumpSessionsSpinLock);
#endif
                rc = eee_sendMessage((void *)req,
                                sizeof (ndmp_NFX_FHReq_t) +
                                req->tot_len - 1,
                                dumpContext->ndmp_ssc_src_port,
                                fs_packedPortAddr(nfxMyCPUId,
                                                  EEE_CONTROL));
                dumpContext->dirCount = 0;
                dumpContext->curDirHist = (void *) dumpContext->historyBuf;
                fs_ramDealloc(FS_RAM_DUMP_CONTEXT, req);
#else
                rc = NFX_OK;
#endif
        }
        fs_yieldPeriodic(context);
        return;

}
#endif
