/*
 * Name:  fs-standalone-evm.h
 *
 * RCS:  $Id: fs-standalone-evm.h,v 1.7 2002/12/10 18:44:59 bobonfeline Exp $
 *
 * Description:  EVM structures and definitions for
 *               the standalone filesystem.
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */
#if !defined(FS_STANDALONE_EVM_H)
#define FS_STANDALONE_EVM_H

#if defined(FS_STANDALONE)

/*
 * EVM STUFF - don't include its api.h, since it includes bunches of
 * unwanted stuff for our standalone build.
 */
typedef	uint64	evm_volOffset_t;
typedef	uint64	evm_volSize_t;

typedef union
{
    int64   whole_thing;
    struct {
        union {
            int32   id;
            struct {
                unsigned    clusterUnique:26;   // unique number in cluster
                unsigned    reserved:6;         // for filesystem use
            } parts;
        } u;
        int32   worldUnique;                    // unique number in the world
    } details;
} evm_volId_t;

typedef	union {
    void		*dataPtr;	// Pointer to data to read/write
    nfx_cod_t		codHandle;	// Where in the SAC it is
} handle_t;

typedef	enum {
    EVM_READ_REQ,
    EVM_WRITE_REQ
} evm_IO_t;

typedef	enum {
    EVM_META_DATA_REQ,
    EVM_USER_DATA_REQ,
    EVM_DATA_TYPE_MAX
} evm_req_t;

typedef	struct {
    handle_t		rDataHandle;	// The the read data is here
    uint32			status;			// status of this request
#define	EVM_STAT_NO_ERROR	 0      // Success
#define	EVM_STAT_INVALID	-1      // Invalid parameter(s) in request
#define EVM_STAT_REJECT	    -2      // Request rejected
#define	EVM_STAT_TIMEOUT	-3      // No response from SCSI layers
#define	EVM_STAT_DEV_ERROR	-4      // SCSI error
#define EVM_STAT_SYS_ERR	-5      // System error (memory...)
#define EVM_STAT_MSG_ERR	-6      // Error detected on a received message
    uint32			devErrorCode;	// As returned by SCSI
    uint64			clientPrivate;	// As client passed to us in req
} evm_IOStat_t;

#define SCSI_MSG_ATTR_PRI_MASK					0x000000000000000FLL
#define SCSI_MSG_ATTR_NO_REORDER				0x0000000000000010LL

typedef	struct evm_IOReq evm_IOReq_t;
struct evm_IOReq {
    evm_IO_t		reqType;		// Type of request
    evm_req_t		dataType;		// Type of data
    evm_volId_t		id;				// to this logical volume
    evm_volOffset_t	offset;			// at this block position
    evm_volSize_t	reqSize;		// how many blocks
	uint64          attributes;     // see SCSI_MSG_ATTR_* in scsi-msg.h
    handle_t		wDataHandle;	// if writing, here is the data
    uint64			clientPrivate;	// returned with call back
    int32			codOffset;		// offset into cod for writes
    void            (*callback)(evm_IOStat_t *);
    DLL_ENTRY(evm_IOReq_t)
                    evmEntry;       // Entry for EVM ioreq queue.
};

#define EVM_LBLK_SIZE  512

// This is what EVM clients call to get things rolling.
extern int
evm_IOReq(
    evm_IOReq_t     *req, 
    void            (*completion_callback)(evm_IOStat_t *stat)
);

//
// prototypes to get timer tics and the GMT.  We need these to get our own timing stats
// on the vol exerciser in standalone mode. 
//
extern uint32  eee_timerGetCurrTic(void);

#endif // FS_STANDALONE

#endif // FS_STANDALONE_EVM_H
