/*------------------------------------------------------------------------------
 * RCS:  $Id: fssf-api.h,v 1.15 2002/03/27 00:09:48 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : fssf-api.h
 *
 * Description  : File System State-Full (FSSF) client interface.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 04-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

#ifndef _FSSF_API_H_
#define _FSSF_API_H_

/*------------------------------------------------------------------------------
 * Include files
 *------------------------------------------------------------------------------
 */

#ifndef FS_STANDALONE
#include "nfx-incl.h"
#include "exports/evm-api.h"
#include "exports/fs-api.h"
#endif

/*------------------------------------------------------------------------------
 * Constants definition.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Common data structures.
 *------------------------------------------------------------------------------
 */


/* State identifier.
 */
typedef union
{
    uint64      id;
    struct
    {
        uint32  cid:32;     // chassis id
        uint32  seq:8;      // sequence number
        uint32  idx:24;     // index in the table
    } parts;
} fssf_stateid_t;

/* Client identifier. Opaque to FSSF.
 */
typedef uint64  fssf_clientid_t;

/* Share access reservation.
 */
typedef enum
{
    FSSF_SHARE_ACCESS_READ   = 0x00000001,
    FSSF_SHARE_ACCESS_WRITE  = 0x00000002,
    FSSF_SHARE_ACCESS_BOTH   = 0x00000003
} fssf_shareAccess_t;

/* Share deny reservation.
 */
typedef enum
{
    FSSF_SHARE_DENY_NONE     = 0x00000000,
    FSSF_SHARE_DENY_READ     = 0x00000001,
    FSSF_SHARE_DENY_WRITE    = 0x00000002,
    FSSF_SHARE_DENY_BOTH     = 0x00000003
} fssf_shareDeny_t;

/* Return status codes.
 */
typedef enum
{
#ifdef ALREADY_DEFINED_IN_FS_API_H
    FS_ERR_GENERIC          = -1,   // NFX_ERR
    FS_ERR_NONE             = 0,    // NFX_OK, NFS_OK, NO_ERROR
    FS_ERR_PERM             = 1,
    FS_ERR_NOENT            = 2,
    FS_ERR_IO               = 5,
    FS_ERR_NXIO             = 6,
    FS_ERR_ACCESS           = 13,
    FS_ERR_EXIST            = 17,
    FS_ERR_XDEV             = 18,
    FS_ERR_NODEV            = 19,
    FS_ERR_NOTDIR           = 20,
    FS_ERR_ISDIR            = 21,
    FS_ERR_INVAL            = 22,
    FS_ERR_FBIG             = 27,
    FS_ERR_NOSPC            = 28,
    FS_ERR_ROFS             = 30,
    FS_ERR_MLINK            = 31,
    FS_ERR_NAMETOOLONG      = 63,
    FS_ERR_NOTEMPTY         = 66,
    FS_ERR_DQUOT            = 69,
    FS_ERR_STALE            = 70,
    FS_ERR_REMOTE           = 71,
    FS_ERR_WFLUSH           = 99,

    FS_ERR_BADHANDLE        = 10001,
    FS_ERR_NOT_SYNC         = 10002,
    FS_ERR_BAD_COOKIE       = 10003,
    FS_ERR_NOTSUPP          = 10004,
    FS_ERR_TOOSMALL         = 10005,
    FS_ERR_SERVERFAULT      = 10006,
    FS_ERR_BADTYPE          = 10007,
    FS_ERR_JUKEBOX          = 10008,
	FS_ERR_BUSY				= 10009,

    FS_ERR_DEFERRED         = 20000,
    FS_ERR_WRONG_EVENT      = 20001,
    FS_ERR_LOCK_DESTROYED   = 20002
#endif
    // all code defined in fs_Status + the following codes:
    FSSF_ERR_DELAY                  = 10008, // ???
    FSSF_ERR_SAME                   = 10009,/* nverify says attrs same */ // ???
    FSSF_ERR_DENIED                 = 10010,/* lock unavailable        */
    FSSF_ERR_EXPIRED                = 10011,/* lock lease expired      */
    FSSF_ERR_LOCKED                 = 10012,/* I/O failed due to lock  */
    FSSF_ERR_GRACE                  = 10013,/* in grace period         */
    FSSF_ERR_FHEXPIRED              = 10014,/* file handle expired     */
    FSSF_ERR_SHARE_DENIED           = 10015,/* share reserve denied    */
    FSSF_ERR_WRONGSEC               = 10016,/* wrong security flavor   */
    FSSF_ERR_CLID_INUSE             = 10017,/* clientid in use         */
    FSSF_ERR_RESOURCE               = 10018,/* resource exhaustion     */
    FSSF_ERR_MOVED                  = 10019,/* filesystem relocated    */
    FSSF_ERR_NOFILEHANDLE           = 10020,/* current FH is not set   */
    FSSF_ERR_MINOR_VERS_MISMATCH    = 10021,/* minor vers not supp */
    FSSF_ERR_STALE_CLIENTID         = 10022,
    FSSF_ERR_STALE_STATEID          = 10023,
    FSSF_ERR_OLD_STATEID            = 10024,
    FSSF_ERR_BAD_STATEID            = 10025,
    FSSF_ERR_BAD_SEQID              = 10026,
    FSSF_ERR_NOT_SAME               = 10027,/* verify - attrs not same */
    FSSF_ERR_LOCK_RANGE             = 10028,
    FSSF_ERR_SYMLINK                = 10029,
    FSSF_ERR_READDIR_NOSPC          = 10030,
    FSSF_ERR_LEASE_MOVED            = 10031
} fssf_status_t;

/* Lock types.
 */
typedef enum
{
    FSSF_LOCK_TYPE_READ      = 1,
    FSSF_LOCK_TYPE_WRITE     = 2,
    FSSF_LOCK_TYPE_READW     = 3,    // blocking read
    FSSF_LOCK_TYPE_WRITEW    = 4     // blocking write
} fssf_lockType_t;

/* File position indicator.
 */
typedef enum
{
    FSSF_SEEK_SET   = 0,    // set file offset to offset
    FSSF_SEEK_CURR  = 1,    // set file offset to current plus offset
    FSSF_SEEK_END   = 2 	// set file offset to EOF plus offset
} fssf_pos_t;

/*------------------------------------------------------------------------------
 * Header of the messages.
 *------------------------------------------------------------------------------
 */

/* Message type
 */
typedef enum
{
    // No operation
    FSSF_NULL_REQ = FSSF_MSG_BASE,
    // Close file
    FSSF_CLOSE_REQ,                     // fssf_closeReq_t
    FSSF_CLOSE_RSP,                     // fssf_closeRsp_t
    // Create a non-regular file object
    FSSF_CREATE_REQ,                    // fssf_createReq_t
    FSSF_CREATE_RSP,                    // fssf_createRsp_t
    // Get attributes
    FSSF_GETATTR_REQ,                   // fssf_getattrReq_t
    FSSF_GETATTR_RSP,                   // fssf_getattrRsp_t
    // Create lock
    FSSF_LOCK_REQ,                      // fssf_lockReq_t
    FSSF_LOCK_RSP,                      // fssf_lockRsp_t
    // Open a regular file
    FSSF_OPEN_REQ,                      // fssf_openReq_t
    FSSF_OPEN_RSP,                      // fssf_openRsp_t
    // Read from file
    FSSF_READ_REQ,                      // fssf_readReq_t
    FSSF_READ_RSP,                      // fssf_readRsp_t
    // Remove filesystem object
    FSSF_REMOVE_REQ,                    // fssf_removeReq_t
    FSSF_REMOVE_RSP,                    // fssf_removeRsp_t
    // Get attributes
    FSSF_SETATTR_REQ,                   // fssf_setattrReq_t
    FSSF_SETATTR_RSP,                   // fssf_setattrRsp_t
    // Unlock file
    FSSF_UNLOCK_REQ,                    // fssf_unlockReq_t
    FSSF_UNLOCK_RSP,                    // fssf_unlockRsp_t
    // Write to file
    FSSF_WRITE_REQ,                     // fssf_writeReq_t
    FSSF_WRITE_RSP,                     // fssf_writeRsp_t
    
    FSSF_DUMMY
} fssf_msgType_t;

/* Header of the message.
 */
typedef struct
{
    // BEGIN common header with FS
    fssf_msgType_t  msgType;        // the message type SHALL always be first
    int32           len;
#ifdef FS_STANDALONE
    int32           src;
    int32           dest;
    int32           nfsReqNum;
    int32           sockIndex;
#endif // FS_STANDALONE
    // END common header with FS
    uint64          clientPrivate;
    fs_Proto        protocol;
} fssf_msgHdr_t;

/*------------------------------------------------------------------------------
 * Messages.
 *------------------------------------------------------------------------------
 */

/* CLOSE operation
 * ---------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_stateid_t      stateid;                // state id
} fssf_closeReq_t;                              // FSSF_CLOSE_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
    //fssf_stateid_t      stateid;                // state id
} fssf_closeRsp_t;                              // FSSF_CLOSE_RSP

/* CREATE operation
 * ----------------
 * Note: only objectType = FS_FTYPE_DIR is supported.
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fs_Handle           parentHandle;           // parent handle
    char                pathName[FS_PATH_MAX];  // path name
    char                objName[FS_NAME_MAX];   // object name
    fs_FileType         objType;                // object type (all but file!)
} fssf_createReq_t;                             // FSSF_CREATE_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
} fssf_createRsp_t;                             // FSSF_CREATE_RSP

/* GETATTR operation
 * -----------------
 * Note: return only extra file attributes kept in FSSF.
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_stateid_t      stateid;                // state id
} fssf_getattrReq_t;                            // FSSF_GETATTR_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
    fs_Offset           currPos;                // current position
    fssf_shareAccess_t  shareAccess;            // share reservation access
    fssf_shareDeny_t    shareDeny;              // share reservation deny
    fs_Handle           filehandle;             // handle to the open file
} fssf_getattrRsp_t;                            // FSSF_GETATTR_RSP

/* LOCK operation
 * --------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_stateid_t      stateid;                // state id
    fssf_lockType_t     lockType;               // lock type
    fs_Offset           offset;                 // record offset
    uint32              len;                    // record length
} fssf_lockReq_t;                               // FSSF_LOCK_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
    fssf_stateid_t      stateid;                // lock state id
} fssf_lockRsp_t;                               // FSSF_LOCK_RSP

/* OPEN operation
 * --------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_clientid_t     owner;                  // client id
    fs_Handle           parentHandle;           // parent handle
    char                pathName[FS_PATH_MAX];  // path name
    char                fileName[FS_NAME_MAX];  // file name
    uint32              createFlag;             // TRUE: create, FALSE: open
    ushort16            createHow;              // UNCHECKED/GUARDED/EXCLUSIVE
    union
    {
        fs_Attr         attr;                   // Attributes of file. 
        uint64          verifier;               // Exclusive verification cookie
    } u;
    fssf_shareAccess_t  shareAccess;            // share reservation access
    fssf_shareDeny_t    shareDeny;              // share reservation deny
} fssf_openReq_t;                               // FSSF_OPEN_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
    fssf_stateid_t      stateid;                // state id
    fs_Attr             attr;                   // file attributes
} fssf_openRsp_t;                               // FSSF_OPEN_RSP

/* READ operation
 * --------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_stateid_t      stateid;                // state id.
    fssf_pos_t          pos;                    // from file position
    fs_Offset           offset;                 // offset
    uint32              len;                    // read length    
} fssf_readReq_t;                               // FSSF_READ_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
    uint32              nbrUsedCod;             // number of used COD's 
    nfx_cod_entry_t     cod[NFX_MAX_COD_MSG];  // data read
	uint64				fsPrivate;              // must be returned to FS
                                                // in TX_COMPL message.
    uint32              bytesRead;              // number of bytes actually read
    uint32              eof;                    // TRUE if end of file reached
} fssf_readRsp_t;                               // FSSF_READ_RSP

/* REMOVE operation
 * ----------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fs_Handle           parentHandle;           // parent handle
    char                pathName[FS_PATH_MAX];  // path name
    char                objName[FS_NAME_MAX];   // object name
} fssf_removeReq_t;                             // FSSF_REMOVE_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
} fssf_removeRsp_t;                             // FSSF_REMOVE_RSP

/* SETATTR operation
 * -----------------
 * Any settable fields in attr that are -1 are ignored and will not be updated.
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fs_Handle           parentHandle;           // parent handle
    char                pathName[FS_PATH_MAX];  // path name
    char                fileName[FS_NAME_MAX];  // file name
    fs_Attr             attr;                   // file attributes
} fssf_setattrReq_t;                            // FSSF_SETATTR_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
} fssf_setattrRsp_t;                            // FSSF_SETATTR_RSP

/* UNLOCK operation
 * ----------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_stateid_t      stateid;                // state id
    fssf_lockType_t     lockType;               // lock type
    fs_Offset           offset;                 // record offset
    uint32              len;                    // record length
} fssf_unlockReq_t;                             // FSSF_UNLOCK_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
} fssf_unlockRsp_t;                             // FSSF_UNLOCK_RSP

/* WRITE operation
 * ---------------
 */
typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_stateid_t      stateid;                // state id
    fssf_pos_t          pos;                    // from file position
    fs_Offset           offset;                 // offset
    uint32              len;                    // write length
    uint32              nbrUsedCod;             // number of used COD's     
    nfx_cod_entry_t     cod[NFX_MAX_COD_MSG];  // data to write
    fs_WriteStableHow   stableHow;              // data stability required
} fssf_writeReq_t;                              // FSSF_WRITE_REQ

typedef struct
{
    fssf_msgHdr_t       hdr;
    fssf_status_t       status;                 // completion status of request
    fs_WriteStableHow   stableHow;              // data stability provided
    uint32              verifier;               // write verifier cookie.
    uint32              bytesWritten;           // number of bytes written
} fssf_writeRsp_t;                              // FSSF_WRITE_RSP

/*------------------------------------------------------------------------------
 * External function prototype.
 *------------------------------------------------------------------------------
 */

int32
fssf_init(void);

void
fssf_rcvMsg(eee_descPtr_t desc);

void
fssf_rcvMcastMsg(eee_descPtr_t desc);

int32
fssf_admin(int32    argc,
           char8 *  argv[]);

void
fssf_processMsg(char *  msgBuf,
                uint32  srcPort);

/*----------------------------------------------------------------------------*/

#endif // _FSSF_API_H_
