/*
 *------------------------------------------------------------------------
 * File:        fssf_chash-api.h
 *
 * RCS: $Id: fssf-hash.h,v 1.2 2002/03/20 04:35:10 isabellec Exp $
 * 
 * Purpose:
 *      API for hash tables for simple chained hash tables
 *
 * File History:
 *      
 *
 *------------------------------------------------------------------------
 */

#if !defined(FSSF_CHASH_API_H)
#define FSSF_CHASH_API_H

//#include "fs-primitive.h"
#include "fssf.h"

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_create
 * 
 * Description:
 *      Create a chained hash. passed hash, compare and destroy functions
 *      returns hid >=0 if okay, -1 in case of failure
 *
 * Possible extensions:
 *      Moduke owner, id, name Used to describe/debug the hash table.
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
uint32 fssf_chash_create (uint32 array_elements, 
                     uint32 (*hash_func) (const void *key),
                     int32 (*keycmp_func) (const void *key1, const void *key2),
                     void (*destruct_func) (void *key, void *obj));

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_destroy
 * 
 * Description:
 *      Destroy a chained hash.
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
void fssf_chash_destroy (uint32 hid);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_insert
 * 
 * Description:
 *      Insert a node into the chained hash.
 *      return 0 if OK, -1 in case of failure
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
int32 fssf_chash_insert (uint32 hid, void *key, void *obj);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_remove
 * 
 * Description:
 *      Remove a node from the chained hash.
 *      return 0 if OK, -1 in case of failure
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
int32 fssf_chash_remove (uint32 hid, void *key);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_find
 * 
 * Description:
 *      Find a node in the chained hash.
 *      return the obj, or NULL if not found
 * 
 * Function History:
 *      
 * It is up to the caller the lock the object !!!!
 *
 *------------------------------------------------------------------------
 */
void *fssf_chash_find (uint32 hid, void *key);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_reset
 * 
 * Description:
 *      Reset the iterator for this chained hash.
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
int32 fssf_chash_reset (uint32 hid);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_next
 * 
 * Description:
 *      Get the next value, move the iterator
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
int32 fssf_chash_next (uint32 hid, void **keyp, void **objp);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_stat
 * 
 * Description:
 *      View the statistics for given hash table
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
int32 fssf_chash_stat (uint32 hid);

/*
 *------------------------------------------------------------------------
 * Function:    fssf_chash_stat_nbr
 * 
 * Description:
 *      returns the statistics for given hash table
 *      array_sz: the size of the array
 *      elements: the current total number of elements/nodes in the table 
 *      curr_max_size: the current max on one row
 *      max_size: the overall max once on one row
 * 
 * Function History:
 *      
 *
 *------------------------------------------------------------------------
 */
int32
fssf_chash_stat_nbr (
    uint32 hid, 
    uint32 *array_sz, 
    uint32 *elements, 
    uint32 *curr_max_size,
    uint32 *max_size);


#endif // FSSF_CHASH_API_H
