/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc-debug.c
 *
 * RCS:  $Id: gmc-debug.c,v 1.3 2002/02/04 18:26:43 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Debug display routines for the GMC emulation
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */

#ifdef NFP_TXRX

#include <nfx-incl.h>
#ifndef SBM_REMOVE_GMC
#include "../sm-sbm/sbm-gmc.h"
#include "gmc.h"

/*-----------------------------------------------------------------
 * Name:	gmc_debugIFQDisplay()
 *
 * Description:	Display the internal GMC IFQ
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_debugIFQDisplay()
{
	sbm_ifqEntry_t	* ptr;

	printf("Internal GMC IFQ entries\n");
	printf("start = 0x%x, end = 0x%x, head = 0x%x, tail = 0x%x\n",
									gmc->ifq_cntl.start, gmc->ifq_cntl.end,
									gmc->ifq_cntl.head, gmc->ifq_cntl.tail);
	for (ptr = gmc->ifq_cntl.start; ptr < gmc->ifq_cntl.end; ptr++) {
		if ((uint32)ptr & 0x1F)
			printf("             0x%08x\n", *ptr);
		else
			printf("0x%08x : 0x%08x\n", ptr, *ptr);
	}

}	/* gmc_debugIFQDisplay() */

/*-----------------------------------------------------------------
 * Name:	gmc_debugFBQDisplay()
 *
 * Description:	Display the internal GMC FBQ
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_debugFBQDisplay()
{
	sbm_fbqEntry_t	* ptr;

	printf("Internal GMC FBQ entries\n");
	printf("start = 0x%x, end = 0x%x, head = 0x%x, tail = 0x%x\n",
									gmc->fbq_cntl.start, gmc->fbq_cntl.end,
									gmc->fbq_cntl.head, gmc->fbq_cntl.tail);
	for (ptr = gmc->fbq_cntl.start; ptr < gmc->fbq_cntl.end; ptr++) {
		if ((uint32)ptr & 0x1F)
			printf("             0x%08x\n", *ptr);
		else
			printf("0x%08x : 0x%08x\n", ptr, *ptr);
	}

}	/* gmc_debugFBQDisplay() */

/*-----------------------------------------------------------------
 * Name:	gmc_debugLUCTxQDisplay()
 *
 * Description:	Display the GMC's internal LUC TxQ
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_debugLUCTxQDisplay()
{
	sbm_txqSendDataToLUC_t	* ptr;
	uint32					* iptr;
	uint32					* cl_ptr;

	printf("Internal LUC TxQ\n");
	printf("start = 0x%x, end = 0x%x, head = 0x%x, tail = 0x%x\n",
							gmc->txq_luc_cntl.start, gmc->txq_luc_cntl.end,
							gmc->txq_luc_cntl.head, gmc->txq_luc_cntl.tail);

	for (iptr = (uint32 *)gmc->txq_luc_cntl.start;
						iptr != (uint32 *)gmc->txq_luc_cntl.end; iptr++) {
		if ((uint32)iptr & 0x1F)
			printf("             0x%08x\n", *iptr);
		else
			printf("0x%08x : 0x%08x\n", iptr, *iptr);
	}

}	/* gmc_debugLUCTxQDisplay() */

/*-----------------------------------------------------------------
 * Name:	gmc_debugSNIMTxQDisplay()
 *
 * Description:	Display the GMC's internal SNIM TxQ
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_debugSNIMTxQDisplay()
{
	sbm_txqSendDataToSNIM_t	* dptr;
	sbm_txqSendReadReq_t	* rptr;
	uint32					* iptr;

	printf("Internal SNIM TxQ\n");
	printf("start = 0x%x, end = 0x%x, head = 0x%x, tail = 0x%x\n",
							gmc->txq_snim_cntl.start, gmc->txq_snim_cntl.end,
							gmc->txq_snim_cntl.head, gmc->txq_snim_cntl.tail);

	for (iptr = (uint32 *)gmc->txq_snim_cntl.start;
						iptr != (uint32 *)gmc->txq_snim_cntl.end; iptr++) {
		if ((uint32)iptr & 0x1F)
			printf("             0x%08x\n", *iptr);
		else
			printf("0x%08x : 0x%08x\n", iptr, *iptr);
	}

}	/* gmc_debugSNIMTxQDisplay() */

/*-----------------------------------------------------------------
 * Name:	gmc_debugMetaQDisplay()
 *
 * Description:	Display the GMC's internal Metadata retrieval queue
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_debugMetaQDisplay()
{
	sbm_txqRetrieveSACData_t	* dptr;
	uint32						* iptr;

	printf("Internal MetaQ\n");
	printf("start = 0x%x, end = 0x%x, head = 0x%x, tail = 0x%x\n",
							gmc->metaq_cntl.start, gmc->metaq_cntl.end,
							gmc->metaq_cntl.head, gmc->metaq_cntl.tail);

	for (iptr = (uint32 *)gmc->metaq_cntl.start;
						iptr != (uint32 *)gmc->metaq_cntl.end; iptr++) {
		if ((uint32)iptr & 0x1F)
			printf("             0x%08x\n", *iptr);
		else
			printf("0x%08x : 0x%08x\n", iptr, *iptr);
	}

}	/* gmc_debugMetaQDisplay() */

/*-----------------------------------------------------------------
 * Name:	gmc_debugCopyQDisplay()
 *
 * Description:	Display the GMC's internal CopyQ
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_debugCopyQDisplay()
{
	sbm_txqCopySACData_t	* dptr;
	uint32					* iptr;

	printf("Internal CopyQ\n");
	printf("start = 0x%x, end = 0x%x, head = 0x%x, tail = 0x%x\n",
							gmc->copyq_cntl.start, gmc->copyq_cntl.end,
							gmc->copyq_cntl.head, gmc->copyq_cntl.tail);

	for (iptr = (uint32 *)gmc->copyq_cntl.start;
						iptr != (uint32 *)gmc->copyq_cntl.end; iptr++) {
		if ((uint32)iptr & 0x1F)
			printf("             0x%08x\n", *iptr);
		else
			printf("0x%08x : 0x%08x\n", iptr, *iptr);
	}

}	/* gmc_debugCopyQDisplay() */

#endif	/* !SBM_REMOVE_GMC */
#endif	/* NFP_TXRX */

