/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc-init.c
 *
 * RCS:  $Id: gmc-init.c,v 1.4 2002/02/04 18:26:42 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	emulation init for the GMC FPGA.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#ifndef SBM_REMOVE_GMC
#include "../sm-sbm/sbm-gmc.h"
#include "../sm-sbm/sbm.h"
#include "gmc.h"

gmc_t		* gmc;

gmc_reg_t	* gmc_regs;

extern void gmc_sacify_rcv();
extern void gmc_process_internal_queues();

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void * allocate_aligned_block(uint32 num_bytes, uint32 alignment)
{
	uint32	cl_aligned_space;

	cl_aligned_space = eee_ramAllocZero(num_bytes + alignment);
	cl_aligned_space = (cl_aligned_space+(alignment - 1));
	cl_aligned_space &= ~(alignment-1);

	return (void *)cl_aligned_space;
}

/*-----------------------------------------------------------------
 * Name:	gmc_init(pass)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_init(
	uint32	pass
)
{
	uint32	sac_app_id;

	if (pass == 1) {
		printf("[gmc_init] pass 1\n");
		gmc = (gmc_t *)eee_ramAlloc(sizeof(gmc_t));
		gmc_regs = &(gmc->regs);

		bzero(gmc, sizeof(gmc_t));

		gmc->ifq = allocate_aligned_block(GMC_IFQ_SIZE, 0x20);
		gmc->ifq_cntl.start = (sbm_ifqEntry_t *)gmc->ifq;
		gmc->ifq_cntl.head = (sbm_ifqEntry_t *)gmc->ifq;
		gmc->ifq_cntl.end = (sbm_ifqEntry_t *)(gmc->ifq + GMC_IFQ_SIZE);

		gmc->fbq = allocate_aligned_block(GMC_FBQ_SIZE, 0x20);
		gmc->fbq_cntl.start = (sbm_fbqEntry_t *)gmc->fbq;
		gmc->fbq_cntl.head = (sbm_fbqEntry_t *)gmc->fbq;
		gmc->fbq_cntl.end = (sbm_fbqEntry_t *)(gmc->fbq + GMC_FBQ_SIZE);

		gmc->txq_luc = allocate_aligned_block(GMC_TXQ_LUC_SIZE, 0x20);
		gmc->txq_luc_cntl.start = (sbm_txqQueueEntry_t *)gmc->txq_luc;
		gmc->txq_luc_cntl.head = (sbm_txqQueueEntry_t *)gmc->txq_luc;
		gmc->txq_luc_cntl.end = (sbm_txqQueueEntry_t *)
								((uint32)(gmc->txq_luc) + GMC_TXQ_LUC_SIZE);

		gmc->txq_snim = allocate_aligned_block(GMC_TXQ_LUC_SIZE, 0x20);
		gmc->txq_snim_cntl.start = (sbm_txqQueueEntry_t *)gmc->txq_snim;
		gmc->txq_snim_cntl.head = (sbm_txqQueueEntry_t *)gmc->txq_snim;
		gmc->txq_snim_cntl.end = (sbm_txqQueueEntry_t *)
								((uint32)(gmc->txq_snim) + GMC_TXQ_SNIM_SIZE);

		gmc->copyq = allocate_aligned_block (GMC_COPYQ_SIZE, 0x20);
		gmc->copyq_cntl.start = (sbm_copyqQueueEntry_t *)gmc->copyq;
		gmc->copyq_cntl.head = (sbm_copyqQueueEntry_t *)gmc->copyq;
		gmc->copyq_cntl.end = (sbm_copyqQueueEntry_t *)
									((uint32)gmc->copyq + GMC_COPYQ_SIZE);

		gmc->metaq = allocate_aligned_block(GMC_METAQ_SIZE, 0x20);
		gmc->metaq_cntl.start = (sbm_txqQueueEntry_t *)gmc->metaq;
		gmc->metaq_cntl.head = (sbm_txqQueueEntry_t *)gmc->metaq;
		gmc->metaq_cntl.end = (sbm_txqQueueEntry_t *)
									((uint32)gmc->metaq + GMC_METAQ_SIZE);

		gmc->sac = allocate_aligned_block(GMC_SAC_SIZE, GMC_SAC_BLOCK_SIZE);
		if (!gmc->sac)
			panic("allocate_aligned_block failed!\n");
		gmc_regs->gmc_sac_size			= GMC_SAC_SIZE;
		gmc_regs->gmc_sac_block_size	= GMC_SAC_BLOCK_SIZE;

		gmc_regs->hmt_base_addr			= 0;

		gmc_regs->gmc_ifq_base_addr		= (uint32)gmc->ifq;
		gmc_regs->gmc_ifq_size			= GMC_IFQ_SIZE;
		gmc_regs->ifq_credit_addr		= 0;

		gmc_regs->bdl_base_addr			= 0;

		gmc_regs->gmc_fbq_base_addr		= (uint32)gmc->fbq;
		gmc_regs->gmc_fbq_size			= GMC_FBQ_SIZE;
		gmc_regs->fbq_credit_addr		= 0;

		gmc_regs->dsokq_base_addr		= 0;
		gmc_regs->dsokq_size			= 0;

		gmc_regs->gmc_txq_luc_base_addr	= (uint32)gmc->txq_luc;
		gmc_regs->gmc_txq_luc_size		= GMC_TXQ_LUC_SIZE;
		gmc_regs->txq_luc_credit_addr	= 0;

		gmc_regs->gmc_txq_snim_base_addr	= (uint32)gmc->txq_snim;
		gmc_regs->gmc_txq_snim_size			= GMC_TXQ_SNIM_SIZE;
		gmc_regs->txq_snim_credit_addr		= 0;

		gmc_regs->gmc_copyq_base_addr	= (uint32)gmc->copyq;
		gmc_regs->gmc_copyq_size		= GMC_COPYQ_SIZE;
		gmc_regs->copyq_credit_addr		= 0;

		gmc_regs->gmc_metaq_base_addr	= (uint32)gmc->metaq;
		gmc_regs->gmc_metaq_size		= GMC_METAQ_SIZE;
		gmc_regs->metaq_credit_addr		= 0;

		gmc_regs->own_mac_addr[0] =
		gmc_regs->own_mac_addr[1] =
		gmc_regs->own_mac_addr[2] =
		gmc_regs->own_mac_addr[3] =
		gmc_regs->own_mac_addr[4] =
		gmc_regs->own_mac_addr[5] = 0;

		gmc_regs->cluster_mac_addr[0] =
		gmc_regs->cluster_mac_addr[1] =
		gmc_regs->cluster_mac_addr[2] =
		gmc_regs->cluster_mac_addr[3] =
		gmc_regs->cluster_mac_addr[4] =
		gmc_regs->cluster_mac_addr[5] = 0;
	}
	else if (pass == 2) {
		printf("[gmc_init] pass 2\n");
		gmc->dsokq.start = (sbm_dsokqEntry_t *)gmc_regs->dsokq_base_addr;
		gmc->dsokq.end = gmc->dsokq.start + gmc_regs->dsokq_size;
		gmc->dsokq.curr = gmc->dsokq.start;

		*(gmc_regs->ifq_credit_addr)	= (uint32)(gmc->ifq_cntl.end - 8);
		*(gmc_regs->fbq_credit_addr)	= (uint32)(gmc->fbq_cntl.end - 8);
		*(gmc_regs->txq_luc_credit_addr)= (uint32)(gmc->txq_luc_cntl.end - 1);
		*(gmc_regs->txq_snim_credit_addr)= (uint32)(gmc->txq_snim_cntl.end - 1);
		*(gmc_regs->metaq_credit_addr)	= (uint32)(gmc->metaq_cntl.end - 1);
		*(gmc_regs->copyq_credit_addr)	= (uint32)(gmc->copyq_cntl.end - 1);

		sac_app_id = eee_allocateApplicationId("sac");
		eee_registerReceiveFunc(sac_app_id, gmc_sacify_rcv, gmc_sacify_rcv);

		eee_registerPollingFunc(EEE_HIGH_PRIORITY,
											gmc_process_internal_queues, NULL);
	}

}	/* gmc_init() */

#endif	/* !SBM_REMOVE_GMC */
