/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc.h
 *
 * RCS:  $Id: gmc.h,v 1.7 2002/01/30 22:50:32 bobonfrogger Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	GMC and SAC for emulator
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _GMC_H
#define _GMC_H

#include "exports/elog-api.h"
 
// hardcoded applid for gmc 
// Todo: clean this up
#define GMC_TRACE(_par_) E_LOG(class_1, LOG_DEBUG, 14, 0, 0, 0, _par_)

/*
 * All sizes are in bytes and in multiples of 0x20
 */
#define GMC_IFQ_SIZE		0xC0
#define GMC_FBQ_SIZE		0xC0
#define GMC_TXQ_LUC_SIZE	0x200
#define GMC_TXQ_SNIM_SIZE	0x200
#define GMC_COPYQ_SIZE		0x100
#define GMC_METAQ_SIZE		0x60
#define GMC_XORQ_SIZE		0x60

#define GMC_SAC_SIZE		(7 * 1024 * 1024)
#define GMC_SAC_BLOCK_SIZE	0x1000

typedef struct gmc {
	/*
	 * GMC internal registers
	 */
	gmc_reg_t				regs;

	/*
	 * GMC internal queues
	 */
	uchar8					* ifq;			/* pointer instead of array,
											 * so that I can allocate on
											 * cacheline boundary
											 */
	sbm_ifq_t				ifq_cntl;

	uchar8					* fbq;			/* ditto */
	sbm_fbq_t				fbq_cntl;

	uchar8					* txq_luc;		/* ditto */
	sbm_gmcTxq_t			txq_luc_cntl;

	uchar8					* txq_snim;		/* ditto */
	sbm_gmcTxq_t			txq_snim_cntl;

	uchar8					* metaq;		/* ditto */
	sbm_gmcTxq_t			metaq_cntl;
	
	uchar8					* copyq;		/* ditto */
	sbm_gmcCopyq_t			copyq_cntl;
	
	/*
	 * Structures in host memory
	 */
	sbm_dsokqQueue_t		dsokq;

	/*
	 * SAC
	 */
	uchar8					* sac;
} gmc_t;

extern gmc_t		* gmc;
extern gmc_reg_t	* gmc_regs;

#endif	/* _GMC_H */

