/* =============================================================================
 * File Name  :	gt64240mii-ui.c
 *
 * Description:    
 *
 * Created    :	08-07-2001
 *
 * Author     :	Tony Wong
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 * =============================================================================
 */
#include "gt64240mii.h"

extern int gt_mii_diagWriteSMI(uint32);
extern int gt_mii_diagReadSMI(uint32);
extern int gt_mii_diagHashScan(uint32);
extern int gt_mii_diagDeleteHashEntry(uint32);
extern int gt_mii_diagViewTestStats(void);
extern int gt_mii_diagLBTest(uint32);
extern int gt_mii_diagReadReg(void);
extern int gt_mii_diagPortEna(uint32);
extern int gt_mii_diagTxPkt(uint32);
extern int gt_mii_diagAddHashEntry(uint32);
extern int gt_mii_diagWriteReg(void);
extern int gt_mii_uiGetInput(char *);
extern int get_cmd(char8 *);

int gt_mii_diagSetLbMode(uint32, uint32);
int gt_mii_uiShowTxq(uint32);
int gt_mii_uiShowRxq(uint32);
int gt_mii_uiConfigLBMode(uint32);
int gt_mii_uiShowPortRegs(uint32);
int gt_mii_uiViewStats(uint32);
int gt_mii_uiViewCfg(uint32);
int gt_mii_uiDiagMenu(uint32);
int gt_mii_bpStats(uint32);
void gt64240mii_clearStats(uint32);
void gt64240mii_get_stats(uint32 *);

extern uint32 gt_mii_test_port;
uint32 gt_mii_test = 0;

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiDiag
 *
 * Description: 	GT64240-MII UI entry function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiDiag()
{
   uint32 port;

   port = gt_mii_uiGetInput("Enter GT64240 MII port number (1-3): \n");
   if (port < 1 || port > 3) {
      printf("Invalid port number!\n");
      return(NFX_OK);
   }
   gt_mii_test_port = port - 1;
   gt_mii_uiDiagMenu( gt_mii_test_port );
   return(NFX_OK);

} /* gt_mii_uiDiag */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowDiagMenu
 *
 * Description: 	Prints GT64240-MII diagnostic menu items.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowDiagMenu()
{
   int sel;
   char cmd[80];

   sel = 0;
   printf("GT64240 MII Menu - Port %d\n", gt_mii_test_port+1);
   printf("=========================\n");
#ifndef SSC_MGMT
   printf("1  : Show Driver Config\n");
   printf("2  : Show Driver Stats\n");
   printf("3  : Show Port Regs\n");
   printf("4  : Set Loopback Mode\n");
   printf("5  : Show Rx Desc Queue\n");
   printf("6  : Show Tx Desc Queue\n");
   printf("7  : Transmit Packet\n");
   printf("8  : Port Enable\n");
   printf("9  : Write register\n");
   printf("10 : Read register\n");
   printf("11 : Add hash table entry\n");
   printf("12 : Delete hash table entry\n");
   printf("13 : Scan hash table\n");
   printf("14 : Read SMI Reg\n");
   printf("15 : Write SMI Reg\n");
   printf("16 : Enable TX Packet Trace\n");
   printf("17 : Disable TX Packet Trace\n");
   printf("18 : Enable RX Packet Trace\n");
   printf("19 : Disable RX Packet Trace\n");
#else
	printf("1: Read SMI Reg\n");
	printf("2: Write SMI Reg\n");
#endif
   printf("999: Quit\n");
   printf("Enter Selection: ");

   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   printf("\n");
   return sel;

} /* gt_mii_uiShowDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiGetInput	
 *
 * Description: 	Gets console input.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiGetInput( char *msg )
{
   int sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   return sel; 
} /* gt_mii_uiGetInput */

int
gt_mii_uiGetHex( char *msg )
{
   uint32 sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%x", &sel);
   return sel; 
} /* gt_mii_uiGetHex */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiDiagMenu	
 *
 * Description: 	Runs the GT64240-MII diagnostic menu.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiDiagMenu(uint32 port)
{
   int sel ;
   for(;;) {
      sel = gt_mii_uiShowDiagMenu(port);
      switch(sel)
      {
#ifndef SSC_MGMT
	case 1:
		gt_mii_uiViewCfg(port);
		break;
	case 2:
		gt_mii_uiViewStats(port);
		break;
	case 3:
		gt_mii_uiShowPortRegs(port);
		break;
	case 4:
		gt_mii_uiConfigLBMode(port);
		break;
	case 5:
		gt_mii_uiShowRxq(port);
		break;
	case 6:
		gt_mii_uiShowTxq(port);
		break;
	case 7:
		gt_mii_test = 1;
		gt_mii_diagTxPkt(port);
		break;
	case 8:
		gt_mii_test = 0;
		gt_mii_diagPortEna(port);
		break;
	case 9:
		gt_mii_diagWriteReg();
		break;
	case 10:
		gt_mii_diagReadReg();
		break;
	case 11:
		gt_mii_diagAddHashEntry(port);
		break;
	case 12:
		gt_mii_diagDeleteHashEntry(port);
		break;
	case 13:
		gt_mii_diagHashScan(port);
		break;
	case 14:
		gt_mii_diagReadSMI(port);
		break;
	case 15:
		gt_mii_diagWriteSMI(port);
		break;
	case 16:
		gt64240mii->port[port].tx_trace = 1;
		break;
	case 17:
		gt64240mii->port[port].tx_trace = 0;
		break;
	case 18:
		gt64240mii->port[port].rx_trace = 1;
		break;
	case 19:
		gt64240mii->port[port].rx_trace = 0;
		break;
#else
	case 1:
		gt_mii_diagReadSMI(port);
		break;
	case 2:
		gt_mii_diagWriteSMI(port);
		break;
#endif
	case 999:
		return 0;
	default:
		break;
      }
   } 
   return(NFX_OK);
} /* gt_mii_uiDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowPortRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowPortRegs(uint32 port)
{
   printf("GT64240-MII PORT REGISTER DUMP: [Slot:%02d -- CPU:%02d] Port %d\n", 
		   nfxMySlotId, nfxMyCPUId, port);
   printf("===============================================================\n\n");
   printf("PCR        :%08x\n", gt_mii_readPortReg(port, GT_MII_PCR));
   printf("PCXR       :%08x\n", gt_mii_readPortReg(port, GT_MII_PCXR));
   printf("CMR        :%08x\n", gt_mii_readPortReg(port, GT_MII_CMR));
   printf("PSR        :%08x\n", gt_mii_readPortReg(port, GT_MII_PSR));
   printf("SPR        :%08x\n", gt_mii_readPortReg(port, GT_MII_SPR));
   printf("HTPR       :%08x\n", gt_mii_readPortReg(port, GT_MII_HTPR));
   printf("FCSAL      :%08x\n", gt_mii_readPortReg(port, GT_MII_FCSAL));
   printf("FCSAH      :%08x\n", gt_mii_readPortReg(port, GT_MII_FCSAH));
   printf("SDCR       :%08x\n", gt_mii_readPortReg(port, GT_MII_SDCR));
   printf("SDCMR      :%08x\n", gt_mii_readPortReg(port, GT_MII_SDCMR));
   printf("ICR        :%08x\n", gt_mii_readPortReg(port, GT_MII_ICR));
   printf("IMR        :%08x\n", gt_mii_readPortReg(port, GT_MII_IMR));
   printf("FRDP3      :%08x\n", gt_mii_readPortReg(port, GT_MII_FRDP3));
   printf("CRDP3      :%08x\n", gt_mii_readPortReg(port, GT_MII_CRDP3));
   printf("CTDP1      :%08x\n", gt_mii_readPortReg(port, GT_MII_CTDP1));

   return(NFX_OK);
} /* gt_mii_uiShowPortRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiViewStats	
 *
 * Description: 	Display device/driver statistics.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiViewStats(uint32 port)
{
   uint32 txframes, mcast, bcast; 
   uint32 i;

   printf("GT64240-MII STATISTICS: [Slot:%02d -- CPU:%02d] Port %d\n", 
		   nfxMySlotId, nfxMyCPUId, port);
   printf("======================================================= \n\n");
   printf("Rx good bytes  : %d\n",gt_mii_readPortReg(port, GT_MII_OK_RXBYTES));
   printf("Tx bytes       : %d\n",gt_mii_readPortReg(port, GT_MII_TXBYTES));
   printf("Rx good frames : %d\n",gt_mii_readPortReg(port, GT_MII_OK_RXFRAMES));
   txframes = gt_mii_readPortReg(port, GT_MII_TXFRAMES);
   printf("Tx frames      : %d\n", txframes);
   printf("Rx total bytes : %d\n",gt_mii_readPortReg(port, GT_MII_ALL_RXBYTES));
   printf("Rx total frames: %d\n",gt_mii_readPortReg(port, GT_MII_ALL_RXFRAMES));
   printf("Rx bcast frames: %d\n",gt_mii_readPortReg(port, GT_MII_OK_RXBCAST));
   printf("Rx mcast frames: %d\n",gt_mii_readPortReg(port, GT_MII_OK_RXMCAST));
   printf("CRC errors     : %d\n",gt_mii_readPortReg(port, GT_MII_CRC_ERRS));
   printf("Oversize frames: %d\n",gt_mii_readPortReg(port, GT_MII_TOO_LARGE));
   printf("Fragments      : %d\n",gt_mii_readPortReg(port, GT_MII_FRAGMENTS));
   printf("Jabber         : %d\n",gt_mii_readPortReg(port, GT_MII_JABBERS));
   printf("Collisions     : %d\n",gt_mii_readPortReg(port, GT_MII_COLLISIONS));
   printf("Late collisions: %d\n",gt_mii_readPortReg(port, GT_MII_LATE_COLLS));
   printf("64-byte frames : %d\n",gt_mii_readPortReg(port, GT_MII_64_BYTES));
   printf("65-127 frames  : %d\n",gt_mii_readPortReg(port, GT_MII_65_127));
   printf("128-255 frames : %d\n",gt_mii_readPortReg(port, GT_MII_128_255));
   printf("256-511 frames : %d\n",gt_mii_readPortReg(port, GT_MII_256_511));
   printf("512-1023 frames: %d\n",gt_mii_readPortReg(port, GT_MII_512_1023));
   printf("1024-max frames: %d\n",gt_mii_readPortReg(port, GT_MII_1024_MAX));
   printf("Rx errors      : %d\n",gt_mii_readPortReg(port, GT_MII_RX_ERRS));
   mcast = gt_mii_readPortReg(port, GT_MII_TX_MCAST);
   printf("Tx mcast frames: %d\n", mcast);
   bcast = gt_mii_readPortReg(port, GT_MII_TX_BCAST);
   printf("Tx bcast frames: %d\n", bcast);
   printf("Tx ucast frames: %d\n", txframes - mcast - bcast);
   printf("Undersize frames:%d\n", gt_mii_readPortReg(port, GT_MII_TOO_SMALL));
   printf("Rx No Buff Errs :%08u\n", gt64240mii->port[port].stats.rxNoBuf);
   printf("\n");
   printf("================= Receive Q Statistics ================ \n\n");
   for(i=0; i < GT_MII_RX_Q_CNT; i++) {
      printf("RxQ %d avail            : %08u\n", i, 
					gt64240mii->port[port].rxc[i].avail);
      printf("RxQ %d pkts             : %08u\n", i, 
					gt64240mii->port[port].rxc[i].pkts);
      printf("RxQ %d octets           : %08u\n", i, 
					gt64240mii->port[port].rxc[i].octets);
      printf("RxQ %d too short        : %08u\n", i, 
					gt64240mii->port[port].rxc[i].shortErr);
      printf("RxQ %d too long         : %08u\n", i, 
					gt64240mii->port[port].rxc[i].longErr);
      printf("RxQ %d overrun          : %08u\n", i, 
					gt64240mii->port[port].rxc[i].ovrErr);
      printf("RxQ %d collision        : %08u\n", i, 
					gt64240mii->port[port].rxc[i].collErr);
      printf("RxQ %d crc err          : %08u\n", i, 
					gt64240mii->port[port].rxc[i].crcErr);
      printf("RxQ %d no buffer        : %08u\n", i, 
					gt64240mii->port[port].rxc[i].noBufErr);
      printf("RxQ %d no desc          : %08u\n", i, gt64240mii->port[port].rxc[i].noDescErr);
   }
   printf("\n");
   printf("================= Transmit Q Statistics =============== \n\n");
   for(i=0; i < GT_MII_TX_Q_CNT; i++) {
      printf("TxQ %d avail            : %08u\n", i, 
					gt64240mii->port[port].txc[i].avail);
      printf("TxQ %d pkts             : %08u\n", i, 
					gt64240mii->port[port].txc[i].pkts);
      printf("TxQ %d octets           : %08u\n", i, 
					gt64240mii->port[port].txc[i].octets);
      printf("TxQ %d qFull err        : %08u\n", i, 
					gt64240mii->port[port].txc[i].qFull);
      printf("TxQ %d collision        : %08u\n", i, 
					gt64240mii->port[port].txc[i].collErr);
      printf("TxQ %d late collision   : %08u\n", i, 
					gt64240mii->port[port].txc[i].lcolErr);
      printf("TxQ %d underrun         : %08u\n", i, 
					gt64240mii->port[port].txc[i].udrErr);
      printf("TxQ %d retransmit limit : %08u\n", i, 
					gt64240mii->port[port].txc[i].relimErr);
      printf("TxQ %d bufs queued      : %08u\n", i, 
					gt64240mii->port[port].txc[i].qd);
      printf("TxQ %d bufs unqueued    : %08u\n", i, 
					gt64240mii->port[port].txc[i].unqd);
   }
   printf("\n");
   return(NFX_OK);
} /* gt64240mii_uiViewStats */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiViewCfg
 *
 * Description: 	Display device/driver configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 20, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiViewCfg(uint32 port)
{
   uint32 i;

   printf("GT64240-MII CONFIGURATION: [Slot:%02d -- CPU:%02d] Port %d\n", 
		   nfxMySlotId, nfxMyCPUId, port);
   printf("===================== General Config =====================\n\n");
   printf("Application Id             : %08d\n", gt64240mii->appId);
   printf("Base Addr                  : %08x\n", gt64240mii->baseAddr);
   printf("Phy Addr                   : %08x\n", gt64240mii->phyAddr);
   printf("Hash Table Addr            : %08x\n", (uint32)gt64240mii->port[port].hash_tbl);
   for(i=0; i < GT_MII_RX_Q_CNT; i++)
      printf("Rx Desc Pool %d Base Addr   : %08x\n", i, 
					(uint32)gt64240mii->port[port].rxQ[i]);
   printf("\n");
   printf("===================== Transmit Control =================== \n\n");
   for(i=0; i < GT_MII_TX_Q_CNT; i++) {
      printf("TxQ %d Start                : %08x\n", i, 
					(uint32)gt64240mii->port[port].txc[i].txqStart);
      printf("TxQ %d End                  : %08x\n", i, 
 					(uint32)gt64240mii->port[port].txc[i].txqEnd);
      printf("TxQ %d Current Addr         : %08x\n", i, 
					(uint32)gt64240mii->port[port].txc[i].txCur);
      printf("TxQ %d Cleanup Addr         : %08x\n", i, 
					(uint32)gt64240mii->port[port].txc[i].txCln);
   }
   printf("\n");
   printf("===================== Receive Control ==================== \n\n");
   for(i=0; i < GT_MII_RX_Q_CNT; i++) {
      printf("RxQ %d Start                : %08x\n", i, 
					(uint32)gt64240mii->port[port].rxc[i].rxqStart);
      printf("RxQ %d End                  : %08x\n", i, 
					(uint32)gt64240mii->port[port].rxc[i].rxqEnd);
      printf("RxQ %d Current Addr         : %08x\n", i, 
					(uint32)gt64240mii->port[port].rxc[i].rxCur);
      printf("RxQ %d Buffer Count         : %08d\n", i, 
					(uint32)gt64240mii->port[port].rxc[i].bufCnt);
   }
   printf("\n");
   return(NFX_OK);
} /* gt_mii_uiViewCfg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiConfigLBMode
 *
 * Description: 	Allows user to enable/disable device loopback.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiConfigLBMode(uint32 port)
{
   int32 mode;

   mode = gt_mii_uiGetInput("Set Loopback Mode [NONE:0 | INT:1 | EXT:2]: \n");
   if((mode < 0) ||  (mode > 2)) {
      printf("Invalid Mode: %d. Exiting....\n", mode);
      return(NFX_OK);
   }    
   gt_mii_diagSetLbMode(port, mode); 

   return(NFX_OK);
} /* gt_mii_uiConfigLBMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowRxq
 *
 * Description: 	Dump the chip receive descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowRxq(uint32 port)
{
   int i, quit;
   volatile gt64240mii_rxdesc_t *descp;

   descp = gt64240mii->port[port].rxc[0].rxqStart;

   printf("GT64240-MII Receive Descriptor Queue Dump - Port %d\n", port);
   printf("===================================================\n");
   for(i=0; i < GT_MII_RX_Q_LEN; i++) {
      printf("%03d   Link: %08x   Buf: %08x   Cmdsts: %08x\n", i, 
                     gt64240mii_noswap((uint32)descp->link), 
                     gt64240mii_noswap((uint32)descp->bufptr), 
                     gt64240mii_noswap(descp->cmdsts));
      descp++;
      if( !(i % 25) && i != 0 ) {
	 quit = gt_mii_uiGetInput("Quit [99]: ");
      if( quit == 99 )
	 return 0;
      }
   }
   printf("\n");
   return 0;
} /* gt_mii_uiShowRxq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowTxq
 *
 * Description: 	Dump the chip transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowTxq(uint32 port)
{
   int i, quit;
   volatile gt64240mii_txdesc_t *descp;

   descp = gt64240mii->port[port].txc[0].txqStart;

   printf("GT64240-MII Transmit Descriptor Queue Dump - Port %d\n", port);
   printf("====================================================\n");
   for(i=0; i < GT_MII_TX_Q_LEN; i++) {
      printf("%03d   Link: %08x   Buf: %08x  Count: %d  Cmdsts: %08x\n", i, 
                     gt64240mii_noswap((uint32 )descp->link), 
                     gt64240mii_noswap((uint32)descp->bufptr), 
                     gt64240mii_noswap(descp->bytecnt_rsvd) >> 16,
                     gt64240mii_noswap(descp->cmdsts));
      descp++;
      if( !(i % 25) && i != 0 ) {
         quit = gt_mii_uiGetInput("Quit [99]: ");
         if( quit == 99 )
	    return 0;
      }
   }
   printf("\n");
   return 0;
} /* gt_mii_uiShowTxq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_bpStats
 *
 * Description: 	Display stats
 *
 * Created by:		Tony Wong
 *
 * Data Created:	01/25/02
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int gt_mii_bpStats(uint32 port)
{
    uint32 errors = 0;

    errors = gt_mii_readPortReg(port, GT_MII_CRC_ERRS);
    errors += gt_mii_readPortReg(port, GT_MII_TOO_LARGE);
    errors += gt_mii_readPortReg(port, GT_MII_JABBERS);
    errors += gt_mii_readPortReg(port, GT_MII_COLLISIONS);
    errors += gt_mii_readPortReg(port, GT_MII_LATE_COLLS);
    errors += gt_mii_readPortReg(port, GT_MII_RX_ERRS);
    errors += gt_mii_readPortReg(port, GT_MII_TOO_SMALL);
    printf("Rx bytes  : %08u\n",gt_mii_readPortReg(port, GT_MII_OK_RXBYTES));
    printf("Tx bytes  : %08u\n",gt_mii_readPortReg(port, GT_MII_TXBYTES));
    printf("Rx packets: %08u\n",gt_mii_readPortReg(port, GT_MII_OK_RXFRAMES));
    printf("Tx packets: %08u\n", gt_mii_readPortReg(port, GT_MII_TXFRAMES));
    printf("Errors    : %08u\n", errors);
    printf("\n");
	return 0;
}

/*
 *  This routine can be polled to gather stats for SNMP. All counters are
 *  cleared after each call. Counters returned are sequential as listed
 *  below. Caller must pass pointer to buffer with space for 15 32-bit values
 */
void gt64240mii_get_stats(uint32 *stat_buf)
{
    int port = 1; // only mgmt port is used

    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_OK_RXBYTES); // rx bytes
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_TXBYTES); // tx bytes
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_OK_RXFRAMES); // rx pkts
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_TXFRAMES); // tx pkts
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_CRC_ERRS); // rx crc
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_FRAGMENTS); // rx fragments
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_JABBERS); // rx jabbers
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_COLLISIONS); // collisions
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_LATE_COLLS); // late colls
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_RX_ERRS); // rx errors
    *stat_buf++ = gt_mii_readPortReg(port, GT_MII_TOO_SMALL); // rx undersized

    *stat_buf++ = gt64240mii->port[port].rxc[0].noDescErr; // no EEE desc
    *stat_buf++ = gt64240mii->port[port].stats.rxNoBuf; // no buffer
    *stat_buf++ = gt64240mii->port[port].txc[0].udrErr; // tx underruns
    *stat_buf++ = gt64240mii->port[port].txc[0].qFull; // queue full drops

    gt64240mii->port[port].rxc[0].noDescErr = 0;
    gt64240mii->port[port].stats.rxNoBuf = 0;
    gt64240mii->port[port].txc[0].udrErr = 0;
    gt64240mii->port[port].txc[0].qFull = 0;
}

void gt64240mii_clearStats(uint32 port)
{
	uint32 dummy;

	dummy = gt_mii_readPortReg(port, GT_MII_OK_RXBYTES);
	dummy = gt_mii_readPortReg(port, GT_MII_TXBYTES);
	dummy = gt_mii_readPortReg(port, GT_MII_OK_RXFRAMES);   
	dummy = gt_mii_readPortReg(port, GT_MII_TXFRAMES);
	dummy = gt_mii_readPortReg(port, GT_MII_ALL_RXBYTES);   
	dummy = gt_mii_readPortReg(port, GT_MII_ALL_RXFRAMES);
	dummy = gt_mii_readPortReg(port, GT_MII_OK_RXBCAST);
	dummy = gt_mii_readPortReg(port, GT_MII_OK_RXMCAST);
	dummy = gt_mii_readPortReg(port, GT_MII_CRC_ERRS);
	dummy = gt_mii_readPortReg(port, GT_MII_TOO_LARGE);
	dummy = gt_mii_readPortReg(port, GT_MII_FRAGMENTS);
	dummy = gt_mii_readPortReg(port, GT_MII_JABBERS);
	dummy = gt_mii_readPortReg(port, GT_MII_COLLISIONS);
	dummy = gt_mii_readPortReg(port, GT_MII_LATE_COLLS);
	dummy = gt_mii_readPortReg(port, GT_MII_64_BYTES);
	dummy = gt_mii_readPortReg(port, GT_MII_65_127);
	dummy = gt_mii_readPortReg(port, GT_MII_128_255);
	dummy = gt_mii_readPortReg(port, GT_MII_256_511);
	dummy = gt_mii_readPortReg(port, GT_MII_512_1023);
	dummy = gt_mii_readPortReg(port, GT_MII_1024_MAX);
	dummy = gt_mii_readPortReg(port, GT_MII_RX_ERRS);
	dummy = gt_mii_readPortReg(port, GT_MII_TX_MCAST);
	dummy = gt_mii_readPortReg(port, GT_MII_TX_BCAST);
	dummy = gt_mii_readPortReg(port, GT_MII_TOO_SMALL);
}
