/*
 *-----------------------------------------------------------------
 *
 * Name:	tlb.c
 *
 * RCS:  $Id: tlb.c,v 1.5 2002/10/14 18:09:53 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	TLB read/write commands
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/mem-api.h>
#include <exports/hwMap.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
void
tlb_display()
{
	uint32	pgmask, mask, pgsize;
	int32	idx, valid;
	uint32	valuehi, valuelo0, valuelo1, pa0, pa1;
	int32	low = 0, high = 64;

	for (idx = low ; idx < high; idx++) {

		__asm__ __volatile__ (
		"     .set noreorder \n"
		"     mtc0 %4, $0 \n"
		"     tlbr \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     mfc0 %0,$10 \n"
		"     mfc0 %1,$2  \n"
		"     mfc0 %2,$3  \n"
		"     mfc0 %3,$5  \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     .set reorder \n"
		: "=r" (valuehi) , "=r" (valuelo0) , "=r" (valuelo1), "=r" (pgmask)
		: "r" (idx)
		);

		mask = pgmask | 0x1fff;
		pa0 = (valuelo0 & 0xFFFFF000) << (12-6);
		pa1 = (valuelo1 & 0xFFFFF000) << (12-6);
		valid = (valuelo0 & TLBLO_V) || (valuelo1 & TLBLO_V);

		printf("%c%2d: hi=0x%08x", valid ? '*' : ' ', idx, valuehi);

		pgsize = (mask + 1)/2;
		if (pgsize >= 1024*1024)
			printf(" sz=%dM", pgsize/(1024*1024));
		else
			printf(" sz=%dK", pgsize/1024);

		printf(" 0x%08x %c%c%c%c", pa0,
				valuelo0 & TLBLO_V ? 'V':'v',
				valuelo0 & TLBLO_D ? 'D':'d',
				valuelo0 & TLBLO_G ? 'G':'g',
				((valuelo0 & TLBLO_CALGMASK) >> 3) == CFG_C_UNCACHED ? 'U':'c');
		printf(" 0x%08x %c%c%c%c", pa1,
				valuelo1 & TLBLO_V ? 'V':'v',
				valuelo1 & TLBLO_D ? 'D':'d',
				valuelo1 & TLBLO_G ? 'G':'g',
				((valuelo1 & TLBLO_CALGMASK) >> 3) == CFG_C_UNCACHED ? 'U':'c');
		printf("\n");
	}
}	/* tlb_display() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
int32
tlb_set(
	int32	index,
	uint32	va,
	uint32	pa,
	uint32	size,
	uint32	calg
)
{
	uint32	asm_index, mask;
	uint32	asm_src, asm_dest0, asm_dest1, asm_size;

	if (size == 0x100000)
		asm_size = TLBMASK_MASK1Mb;
	else if (size == 0x400000)
		asm_size = TLBMASK_MASK4Mb;
	else if (size == 0x1000000)
		asm_size = TLBMASK_MASK16Mb;
	else {
		printf("Invalid TLB size, 0x%x\n", size);
		return 0;
	}
	mask = size-1;
	if (va & mask) {
		printf("Invalid Virtual address for requested entry size, 0x%x\n", va);
		return -1;
	}
	if (pa & mask) {
		printf("Invalid Physical address for requested entry size, 0x%x\n", pa);
		return -1;
	}

	asm_index = index;
	asm_src = (va & TLBHI_ADDR_MASK);
	asm_dest0 = ((pa & TLBLO_ADDR_MASK) >> TLBLO_ADDR_SHIFT) |
							(TLBLO_G | TLBLO_V | TLBLO_D) | (calg << 3);
	asm_dest1 = (((pa+size) & TLBLO_ADDR_MASK) >> TLBLO_ADDR_SHIFT)|
							(TLBLO_G | TLBLO_V | TLBLO_D) | (calg << 3);

	__asm__ __volatile__ (
	"     mtc0 %0,$0 \n"
	"     mtc0 %4,$5 \n"
	"     mtc0 %1,$10 \n"
	"     mtc0 %2,$2  \n"
	"     mtc0 %3,$3  \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     tlbwi \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	:
	: "r" (asm_index), "r" (asm_src), "r" (asm_dest0), "r" (asm_dest1), "r" (asm_size)
	);

	return 0;

}	/* tlb_set() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
void
mem_map_tlb_init()
{
#ifdef FC_USE_NEW_MEM_MAP
	int32	index = 0;
	uint32	tlb_size, virt_addr, phys_addr;
	extern uint64 get_memorysize();

	/*
	 * Map DRAM at virtual 0x40000000 (cached) to physical 0, 256 MB
	 * Map DRAM at virtual 0x60000000 (uncached) to physical 0, 256 MB
	 */
	tlb_size = 0x1000000;
	if (get_memorysize() >= 0x10000000) {
		for (virt_addr = SW_RAM_BASE_CACHED, phys_addr = 0;
				virt_addr < SW_RAM_BASE_CACHED+0x10000000;
						virt_addr += (tlb_size*2), phys_addr += (tlb_size*2)) {
			tlb_set(index++, virt_addr, phys_addr, tlb_size, CFG_C_WBACK);
		}
		for (virt_addr = SW_RAM_BASE_UNCACHED, phys_addr = HW_RAM_BASE;
				virt_addr < SW_RAM_BASE_UNCACHED+0x10000000;
						virt_addr += (tlb_size*2), phys_addr += (tlb_size*2)) {
			tlb_set(index++, virt_addr, phys_addr, tlb_size, CFG_C_UNCACHED);
		}
	}

	/*
	 * Map DRAM at virtual 0x50000000 (cached) to physical 0x20000000, 256 MB
	 * Map DRAM at virtual 0x70000000 (uncached) to physical 0x20000000, 256 MB
	 */
	if (get_memorysize() >= 0x20000000) {
		for (virt_addr = SW_RAM_BASE_CACHED+0x10000000, phys_addr = 0x20000000;
				virt_addr < SW_RAM_BASE_CACHED+0x20000000;
						virt_addr += (tlb_size*2), phys_addr += (tlb_size*2)) {
			tlb_set(index++, virt_addr, phys_addr, tlb_size, CFG_C_WBACK);
		}
		for (virt_addr = SW_RAM_BASE_UNCACHED+0x10000000, phys_addr = 0x20000000;
				virt_addr < (uint32)SW_RAM_BASE_UNCACHED+0x20000000;
						virt_addr += (tlb_size*2), phys_addr += (tlb_size*2)) {
			tlb_set(index++, virt_addr, phys_addr, tlb_size, CFG_C_UNCACHED);
		}
	}

	pcisize = 0x10000000;
	/*
	 * PCI spaces (0x30000000 - 0x40000000)
	 */
	for (virt_addr = SW_PCI_BASE, phys_addr = SW_PCI_BASE;
				virt_addr < SW_PCI_BASE + pcisize;
						virt_addr += (tlb_size*2), phys_addr += (tlb_size*2)) {
			tlb_set(index++, virt_addr, phys_addr, tlb_size, CFG_C_UNCACHED);
	}

#endif
}
