/*
 *----------------------------------------------------------------------------
 *
 * Name:         gt64440mii-diag.c
 *
 * Copyright (C) 2004, OnStor, Inc.
 *
 * Description : Contains Disco3 Ethernet Driver diagnostic routines.
 *               Ported from sm-gt64240-mii module.
 *
 * Created by  : Shaun Hui
 *
 * Date Created: 04/13/04
 *
 *-----------------------------------------------------------------
 */
#ifndef SIBYTE_BC

#include <exports/hwMap.h>
#include "gt64440mii.h"

typedef struct gt64440mii_test {
	uint32 testState;				
#define GT_LB_TEST_DONE		0
#define GT_LB_TEST_INPROGRESS	1
#define GT_LB_TEST_VERIFY	2
#define GT_LB_TEST_CONTINUE	3
#define GT_LB_TEST_END		4
	uint32 testMode;		
#define GT_LB_TEST_INT		1
#define GT_LB_TEST_EXT		2
	uint32 testPktCnt;
	uint32 testPktSize;
	uint32 testDestAddr;
	uchar8 *testBuffer;
	uint32 fVerify;
	uint32 rxPktCnt;
	uint32 rxErrPktCnt;
	uint32 rxByteCnt;
	uint32 txPktCnt;
	uint32 txByteCnt;
} gt64440mii_test_t;

extern int32 gt64440_scanHashTable(int32);
extern int32 gt64440_addHashTableEntry(uint32,uint32, uint32, uint32, uint32); 
extern int gt_mii_uiGetHex(char *);
extern int gt_mii_uiGetInput(char *);
extern int32 gt64440mii_txUnicast(uint32, eee_desc_t *);

int gt_mii_diagShowLbTestResults(gt64440mii_test_t *);
int gt_mii_diagLbVerifyData(uchar8 *, int32, uchar8 *);

static gt64440mii_test_t gt64440mii_testControl;
uint32 gt_mii_test_port = 0;

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagSetLbMode
 *
 * Description: 	Set loopback mode. Loop back at PHY since Disco 3 
 *                  has no MAC level loopback,
 *
 * Returns: 		NFX_OK 
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_diagSetLbMode( uint32 port, uint32 mode )
{
   /*
    * set/reset LB Mode at PHY level
    */
   if (mode != GT_MII_LB_NONE)
   {
      gt64440mii->port[port].rxc[0].loopback = 1;
	  gt_mii_writeSMI( GT_MII_GET_PHYID(port), MII_BMCR, BMCR_LOOP, 0xffff);
   }
   else
   {
      gt64440mii->port[port].rxc[0].loopback = 0;
	  gt_mii_writeSMI( GT_MII_GET_PHYID(port), MII_BMCR, BMCR_LOOP, 0xffff);
   }
	
   return(NFX_OK);

} /* gt_mii_diagSetLbMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagLBTest
 *
 * Description: 	Runs loopback test 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_diagLBTest(uint32 port)
{
   int32  i, rc;
   int32  pktCnt;
   int32  pktSize;
   int32  testMode;
   uint32 destAddr = 0;
   uchar8 *bufptr;
   uchar8 *tmp;

   pktCnt = gt_mii_uiGetInput("Enter Pkt Count: ");
   if( pktCnt <= 0 ) {
      printf("Zero Pkt Count. Terminating Loopback Test\n"); 
      return(NFX_OK);
   }

   pktSize = gt_mii_uiGetInput("Enter Pkt Size (64-9000): ");
   if((pktSize < 64) || (pktSize > 9000)) {
      printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize); 
      return(NFX_OK);
   }

   testMode = gt_mii_uiGetInput("Enter Test Mode [INT:1 | EXT:2]: ");
   if((testMode < 1) || (testMode > 2)) {
      printf("Invalid entry for Test Mode: %d\n", testMode); 
      return(NFX_OK);
   }

   memset(&gt64440mii_testControl,0,sizeof(gt64440mii_testControl));

   gt64440mii_testControl.testState   = GT_LB_TEST_INPROGRESS; 
   gt64440mii_testControl.testMode    = testMode;
   gt64440mii_testControl.testPktCnt  = pktCnt;
   gt64440mii_testControl.testPktSize = pktSize;

   /*
    * Allocate and initialize test data.
    */
   bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
   if( !bufptr ) {
      printf("Test data buffer allocation failed.\n");
      return(NFX_OK);
   }
   gt64440mii_testControl.testBuffer = bufptr;
   gt64440mii_testControl.fVerify = 1;

   if(testMode == GT_LB_TEST_INT)
      gt_mii_diagSetLbMode(port, GT_MII_LB_INTERNAL);

   printf("bufptr: %x, pktSize: %d\n", (uint32)bufptr, pktSize);

   tmp = bufptr;
   for(i=0; i < pktSize; i++ )
      *tmp++ = i & 0xFF;

   gt_mii_portEna(port); 

#ifdef NFX_MOD_SSC
   /*
    * Send first set of loopback packets.
    */
   destAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
				   						 nfxMyCPUId,
				 						 gt64440mii->appId,
				 						 EEE_USER_DATA,
				 						 NFX_COS_HIGH, 
										 SSC_PORT_BP_MGMT);
#endif // NFX_MOD_SSC

   gt64440mii_testControl.testDestAddr = destAddr;

   for(i=0; i < 1; i++)
   {
      rc = eee_sendMessage(bufptr, pktSize, destAddr, destAddr);
      if (rc != EEE_FORWARDED_PACKET)
         printf("gt_mii_diagLBTest: error: eee_sendMessage, rc=%0x\n", rc);
   }
   gt64440mii_testControl.txPktCnt += 1;
   gt64440mii_testControl.txByteCnt += (1 * pktSize);
   return(NFX_OK);

} /* gt_mii_diagLBTest */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagLbPktReceive
 *
 * Description: 	Receives Loop back packet 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_diagLbPktReceive( eee_desc_t *edesc )
{
   int32 rc;
   int32 pktLen;
   int32 bufCnt;
   gt64440mii_test_t *dptc;

   dptc = &gt64440mii_testControl;
   pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
   bufCnt = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);

   while(1) {
      switch( dptc->testState )
      {
	case GT_LB_TEST_INPROGRESS:
		dptc->rxPktCnt++; 
		dptc->rxByteCnt += pktLen;

		if( dptc->fVerify )
		   dptc->testState = GT_LB_TEST_VERIFY;
		else 
		{
		   if( dptc->rxPktCnt >= dptc->testPktCnt)
		      dptc->testState = GT_LB_TEST_END;
		   else
		      dptc->testState = GT_LB_TEST_CONTINUE;
		}
	        break;

	case GT_LB_TEST_VERIFY:
		rc = gt_mii_diagLbVerifyData(edesc->bd[1].buf, 
				             edesc->bd[1].len, 
					     dptc->testBuffer);

		if( dptc->rxPktCnt >= dptc->testPktCnt)
		   dptc->testState = GT_LB_TEST_END;
		else
		   dptc->testState = rc;
		break;

	case GT_LB_TEST_CONTINUE:
		dptc->txPktCnt++; 
		dptc->txByteCnt += pktLen;
		edesc->hdr.dest_port = dptc->testDestAddr;
		eee_forwardPacket(edesc);
		gt_mii_portEna(gt_mii_test_port); 
		dptc->testState = GT_LB_TEST_INPROGRESS;
		return(NFX_OK);
				
	case GT_LB_TEST_END:
		if(dptc->testBuffer)
		   eee_deallocateBuffer(dptc->testBuffer);
		if(dptc->testMode == GT_LB_TEST_INT)
		   gt_mii_diagSetLbMode(gt_mii_test_port, GT_MII_LB_NONE);

		gt_mii_diagShowLbTestResults(dptc);
		dptc->testState = GT_LB_TEST_DONE;
		break;

	case GT_LB_TEST_DONE:
		eee_freePacket(edesc);
		return(NFX_OK);
      }		
   }
   return(NFX_OK);

} /* gt_mii_diagLbPktReceive */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagLbVerifyData
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_diagLbVerifyData( uchar8 *bufptr, int32 len, uchar8 *tbufptr )
{
	int32  i;
#if 0
	int32 j;
	uchar8 *tmp;
#endif
   if( !bufptr || !tbufptr )
      return(NFX_ERR);

#if 0
   tmp = (uchar8 *)bufptr;
   printf("Rx buffer at %0x: ",tmp);
   for (j=0; j < len; j++)
      printf("%0x ", *tmp++);
   printf("\n");
#endif

   for(i=0; i < len; i++) {
      if( *bufptr != *tbufptr ) {
	 printf("LB Data Error: TestVal=%02x, ActualVal=%02x, Addr=%x\n",
				   *tbufptr, *bufptr, (uint32)bufptr);
	 return(GT_LB_TEST_END);
      }
      bufptr++;
      tbufptr++;
   }
   return(GT_LB_TEST_CONTINUE);

} /* gt_mii_diagLbVerifyData */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagShowLbTestResults
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_diagShowLbTestResults(gt64440mii_test_t *dptc)
{
	printf("\nGT64440-MII LOOPBACK TEST RESULTS\n");
	printf("=================================\n\n");
	printf("Test Mode....................%s\n", 
	      (dptc->testMode == 1) ? "INT": "EXT");
	printf("Test Packet Size: ...........%d\n", dptc->testPktSize);
	printf("Test Packet Count: ..........%d\n", dptc->testPktCnt);
	printf("Packets Received: ...........%d\n", dptc->rxPktCnt);
	printf("Errored Packets Received: ...%d\n", dptc->rxErrPktCnt);
	printf("Bytes Received: .............%d\n", dptc->rxByteCnt);
	printf("Packets Transmitted: ........%d\n", dptc->txPktCnt);
	printf("Bytes Transmitted: ..........%d\n", dptc->txByteCnt);
	return(NFX_OK);
} /* gt_mii_diagShowLbTestResults */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagTxPkt
 *
 * Description: 	Transmits a test packet.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 25, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_diagTxPkt(uint32 port)
{
   int32  i,j,k;
   int32  bufCnt;
   int32  bufLen;
   int32  lastBufLen;
   int32  pktCnt;
   int32  pktSize;
   uchar8 *bufptr = NULL;
   uchar8 *tmp;
   eee_desc_t *edesc;
   eee_descHdr_t *hdr;
   int32 dtype_port;

   if (isFC()) {
      switch (port) {
        case 0: dtype_port = EEE_USER_DATA;
                break;
        case 1: dtype_port = EEE_MGMT;
                break;
        case 2: dtype_port = EEE_CONTROL;
                break;
        default:return(NFX_OK);
      }
   }
   else {
      switch (port) {
        case 0: dtype_port = EEE_MGMT;
                break;
        case 1: dtype_port = EEE_USER_DATA;
                break;
        case 2: dtype_port = EEE_CONTROL;
                break;
        default:return(NFX_OK);
      }
   }

   pktCnt = gt_mii_uiGetInput("Enter Pkt Count: ");
   if( pktCnt <= 0 ) {
      printf("Zero Pkt Count. Terminating Loopback Test\n"); 
      return(NFX_OK);
   }

   pktSize = gt_mii_uiGetInput("Enter Pkt Size: ");
   if((pktSize < 64) || (pktSize > 9000)) {
      printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize); 
      return(NFX_OK);
   }

   if( (pktSize % 2048) == 0) {
      bufCnt = (pktSize / 2048);
      lastBufLen = 2048;
   }
   else 
   {
      bufCnt = (pktSize / 2048) + 1;
      lastBufLen = pktSize % 2048;
   }

   printf("BufCnt: %d, LastBufLen: %d\n", bufCnt, lastBufLen);

   for(i=0; i < pktCnt; i++) {

	edesc = (eee_desc_t *)eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	memset(edesc,0, sizeof(eee_desc_t));
	hdr = (eee_descHdr_t *)edesc;
	hdr->control = (bufCnt << EDESC_BUFCNT_SHIFT) | pktSize;
    hdr->dest_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
				gt64440mii->appId, dtype_port, NFX_COS_HIGH, PORT_BEST_FIT);
    hdr->src_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId, 
				gt64440mii->appId, dtype_port, NFX_COS_HIGH, PORT_BEST_FIT);

	for(k=1; k <= bufCnt; k++) {
	   if(k == bufCnt)
	      bufLen = lastBufLen;
	   else
	      bufLen = 2048;

	   bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
	   if( !bufptr ) {
	      printf("Test data buffer allocation failed.\n");
	      return(NFX_OK);
	   }
	   edesc->bd[k].buf = bufptr;
	   edesc->bd[k].len = bufLen;
	   edesc->bd[k].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
    }
	tmp = (uchar8 *)bufptr;
	for(j=pktSize; j > 0; j-- )
	   *tmp++ = j & 0xFF;

        gt_mii_portEna(port); 

	gt64440mii_txUnicast( port, edesc );
   }
   return(NFX_OK);
} /* gt_mii_diagTxPkt */


char *gt_miiTestStates[] = {"TEST_DONE",
			    "TEST_INPROGRESS",
			    "TEST_VERIFY",
			    "TEST_CONTINUE",
			    "TEST_END"
};

char *gt_miiTestMode[] = {"INTERNAL LOOPBACK","EXTERNAL LOOPBACK"};

int
gt_mii_diagViewTestStats()
{
   gt64440mii_test_t *t;

   t = &gt64440mii_testControl;

   printf("GT64440-MII LOOPBACK TEST RESULTS\n");
   printf("=================================\n\n");
   printf("Test State        : %s\n", gt_miiTestStates[t->testState]);
   printf("Test Mode         : %s\n", gt_miiTestMode[t->testMode - 1]);
   printf("Test Packet Cnt   : %d\n", t->testPktCnt);
   printf("Test Packet Size  : %d\n", t->testPktSize);
   printf("Test Verify Flag  : %d\n", t->fVerify);
   printf("Actual Rx Pkt Cnt : %d\n", t->rxPktCnt);
   printf("Actual Rx Byte Cnt: %d\n", t->rxByteCnt);
   printf("Actual Rx Err Cnt : %d\n", t->rxErrPktCnt);
   printf("Actual Tx Pkt Cnt : %d\n", t->txPktCnt);
   printf("Actual Tx Byte Cnt: %d\n", t->txByteCnt);
   printf("\n");
   return 0;

} /* gt64440mii_diagViewTestStats */


int
gt_mii_diagReadSMI(uint32 port)
{
	uint32 reg, dev, data;

	dev = gt_mii_uiGetHex("Enter dev addr: ");
    reg = gt_mii_uiGetHex("Enter reg addr: ");

	gt_mii_readSMI(dev, reg, &data );

	printf("Value at dev=%0x reg=%0x data=%0x\n", dev, reg, data);

	return 0;
}

int
gt_mii_diagWriteSMI(uint32 port)
{
	uint32 reg, dev, data;

	dev = gt_mii_uiGetHex("Enter dev addr: ");
    reg = gt_mii_uiGetHex("Enter reg addr: ");
	data = gt_mii_uiGetHex("Enter hex data: ");

	gt_mii_writeSMI(dev, reg, data, 0xffff);

	return 0;
}

int
gt_mii_diagWriteReg()
{
	uint32 reg_addr;
	uint32 reg_val;

   reg_addr = gt_mii_uiGetHex("Enter reg addr: ");
   reg_val  = gt_mii_uiGetHex("Enter long hex value: ");

   GT_WRITE(reg_addr, reg_val);

	return 0;
}

int
gt_mii_diagReadReg()
{
   uint32 reg_addr;
   uint32 reg_val;

   reg_addr = gt_mii_uiGetHex("Enter reg addr: ");
   reg_val  = GT_READ(reg_addr);

   printf("Value at %0x: %0x\n", reg_addr, reg_val);
	return 0;
}
#endif
