/*
 *----------------------------------------------------------------------------
 *
 * Name:         gt64440mii-ui.c
 *
 * Copyright (C) 2004, OnStor, Inc.
 *
 * Description : Contains Disco3 Ethernet Driver UI/display routines.
 *               Ported from sm-gt64240-mii module.
 *
 * Created by  : Shaun Hui
 *
 * Date Created: 04/13/04
 *
 *-----------------------------------------------------------------
 */

#ifndef SIBYTE_BC

#include "gt64440mii.h"

extern int gt_mii_diagWriteSMI(uint32);
extern int gt_mii_diagReadSMI(uint32);
extern int gt_mii_diagHashScan(uint32);
extern int gt_mii_diagDeleteHashEntry(uint32);
extern int gt_mii_diagViewTestStats(void);
extern int gt_mii_diagLBTest(uint32);
extern int gt_mii_diagReadReg(void);
extern int gt_mii_diagTxPkt(uint32);
extern int gt_mii_diagAddHashEntry(uint32);
extern int gt_mii_diagWriteReg(void);
extern int gt_mii_uiGetInput(char *);
extern int get_cmd(char8 *);

int gt_mii_diagSetLbMode(uint32, uint32);
int gt_mii_uiShowTxq(uint32);
int gt_mii_uiShowRxq(uint32);
int gt_mii_uiConfigLBMode(uint32);
int gt_mii_uiShowPortRegs(uint32);
int gt_mii_uiViewStats(uint32);
int gt_mii_uiViewCfg(uint32);
int gt_mii_uiDiagMenu(uint32);
int gt_mii_bpStats(uint32);
void gt64440mii_clearStats(uint32);
void gt64440mii_get_stats(uint32 *);

extern uint32 gt_mii_test_port;
uint32 gt_mii_test = 0;
                                                                                
const char *mib_regName[ETH_MIB_MAX_IDX] =
{
    "Rx Good Octets Lo : ",
    "Rx Good Octets Hi : ",
    "Rx Bad Octets     : ",
    "Mac Trans Error   : ",
    "Rx Good Frames    : ",
    "Rx Bad Frames     : ",
    "Rx BCast Frames   : ",
    "Rx MCast Frames   : ",
    "Frames 64 Octets  : ",
    "Frames 65-127     : ",
    "Frames 128-255    : ",
    "Frames 256-511    : ",
    "Frames 512-1023   : ",
    "Frames 1023-Max   : ",
    "Tx Good Octets Lo : ",
    "Tx Good Octets Hi : ",
    "Tx Good Frames    : ",
    "Excess Collisions : ",
    "Tx MCast Frames   : ",
    "Tx BCast Frames   : ",
    "Excess Collisions : ",
    "Rx Unknown Ctrl   : ",
    "Rx Good FC        : ",
    "Rx Bad FC         : ",
    "Rx Undersize      : ",
    "Rx Fragments      : ",
    "Rx Oversize       : ",
    "Rx Jabber         : ",
    "Rx MAC Err Events : ",
    "Bad CRC           : ",
    "Collisions        : ",
    "Late Collisions   : "
};

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiDiag
 *
 * Description: 	GT64440-MII UI entry function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiDiag()
{
   uint32 port;

   port = gt_mii_uiGetInput("Enter GT64440 MII port number (1-3): \n");
   if (port < 1 || port > 3) {
      printf("Invalid port number!\n");
      return(NFX_OK);
   }
   gt_mii_test_port = port - 1;
   gt_mii_uiDiagMenu( gt_mii_test_port );
   return(NFX_OK);

} /* gt_mii_uiDiag */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowDiagMenu
 *
 * Description: 	Prints GT64440-MII diagnostic menu items.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowDiagMenu()
{
   int sel;
   char cmd[80];

   sel = 0;
   printf("GT64440 MII Menu - Port %d\n", gt_mii_test_port+1);
   printf("=========================\n");
#ifndef SSC_MGMT
   printf("1  : Show Driver Config\n");
   printf("2  : Show Driver Stats\n");
   printf("3  : Show Port Regs\n");
   printf("4  : Set Loopback Mode\n");
   printf("5  : Show Rx Desc Queue\n");
   printf("6  : Show Tx Desc Queue\n");
   printf("7  : Transmit Packet\n");
   printf("8  : Port Enable\n");
   printf("9  : Write register\n");
   printf("10 : Read register\n");
#ifdef NOTUSED
   printf("11 : Add hash table entry\n");
   printf("12 : Delete hash table entry\n");
   printf("13 : Scan hash table\n");
#endif
   printf("14 : Read SMI Reg\n");
   printf("15 : Write SMI Reg\n");
   printf("16 : Enable TX Packet Trace\n");
   printf("17 : Disable TX Packet Trace\n");
   printf("18 : Enable RX Packet Trace\n");
   printf("19 : Disable RX Packet Trace\n");
#else
	printf("1: Read SMI Reg\n");
	printf("2: Write SMI Reg\n");
#endif
   printf("999: Quit\n");
   printf("Enter Selection: ");

   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   printf("\n");
   return sel;

} /* gt_mii_uiShowDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiGetInput	
 *
 * Description: 	Gets console input.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiGetInput( char *msg )
{
   int sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   return sel; 
} /* gt_mii_uiGetInput */

int
gt_mii_uiGetHex( char *msg )
{
   uint32 sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%x", &sel);
   return sel; 
} /* gt_mii_uiGetHex */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiDiagMenu	
 *
 * Description: 	Runs the GT64440-MII diagnostic menu.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiDiagMenu(uint32 port)
{
   int sel ;
   for(;;) {
      sel = gt_mii_uiShowDiagMenu(port);
      switch(sel)
      {
#ifndef SSC_MGMT
	case 1:
		gt_mii_uiViewCfg(port);
		break;
	case 2:
		gt_mii_uiViewStats(port);
		break;
	case 3:
		gt_mii_uiShowPortRegs(port);
		break;
	case 4:
		gt_mii_uiConfigLBMode(port);
		break;
	case 5:
		gt_mii_uiShowRxq(port);
		break;
	case 6:
		gt_mii_uiShowTxq(port);
		break;
	case 7:
		gt_mii_test = 1;
		gt_mii_diagTxPkt(port);
		break;
	case 8:
		gt_mii_test = 0;
		gt_mii_portEna(port);
		break;
	case 9:
		gt_mii_diagWriteReg();
		break;
	case 10:
		gt_mii_diagReadReg();
		break;
#ifdef NOTUSED
	case 11:
		gt_mii_diagAddHashEntry(port);
		break;
	case 12:
		gt_mii_diagDeleteHashEntry(port);
		break;
	case 13:
		gt_mii_diagHashScan(port);
		break;
#endif
	case 14:
		gt_mii_diagReadSMI(port);
		break;
	case 15:
		gt_mii_diagWriteSMI(port);
		break;
	case 16:
		gt64440mii->port[port].tx_trace = 1;
		break;
	case 17:
		gt64440mii->port[port].tx_trace = 0;
		break;
	case 18:
		gt64440mii->port[port].rx_trace = 1;
		break;
	case 19:
		gt64440mii->port[port].rx_trace = 0;
		break;
#else
	case 1:
		gt_mii_diagReadSMI(port);
		break;
	case 2:
		gt_mii_diagWriteSMI(port);
		break;
#endif
	case 999:
		return 0;
	default:
		break;
      }
   } 
   return(NFX_OK);
} /* gt_mii_uiDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowPortRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowPortRegs(uint32 port)
{
   printf("GT64440-MII PORT REGISTER DUMP: [Slot:%02d -- CPU:%02d] Port %d\n", 
		   nfxMySlotId, nfxMyCPUId, port);
   printf("===============================================================\n\n");
   printf("MAC HI     :%08x\n", GT_READ( ETH_MAC_ADDR_HIGH(port) ));
   printf("MAC LO     :%08x\n", GT_READ( ETH_MAC_ADDR_LOW(port) ));
   printf("PCR        :%08x\n", GT_READ( ETH_PORT_CONFIG_REG(port) ));
   printf("PCXR       :%08x\n", GT_READ( ETH_PORT_CONFIG_EXTEND_REG(port) ));
   printf("PSCR       :%08x\n", GT_READ( ETH_PORT_SERIAL_CONTROL_REG(port) ));
   printf("SDCR       :%08x\n", GT_READ( ETH_SDMA_CONFIG_REG(port) ));
   printf("PSR        :%08x\n", GT_READ( ETH_PORT_STATUS_REG(port) ));
   printf("ICR        :%08x\n", GT_READ( ETH_INTERRUPT_CAUSE_REG(port) ));
   printf("IMR        :%08x\n", GT_READ( ETH_INTERRUPT_MASK_REG(port) ));
   printf("CRDP0      :%08x\n", GT_READ(ETH_RX_CURRENT_QUEUE_DESC_PTR_0(port)));
   printf("CTDP0      :%08x\n", GT_READ(ETH_TX_CURRENT_QUEUE_DESC_PTR_0(port)));

   return(NFX_OK);
} /* gt_mii_uiShowPortRegs */


/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiViewStats	
 *
 * Description: 	Display device/driver statistics.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiViewStats(uint32 port)
{
	uint32 i, offset;

	printf("GT64440-MII STATISTICS: [Slot:%02d -- CPU:%02d] Port %d\n", 
		   nfxMySlotId, nfxMyCPUId, port);
	printf("======================================================= \n\n");

    for (offset  = ETH_MIB_GOOD_OCTETS_RCVD_LO; 
		 offset <= ETH_MIB_LATE_COLLISIONS; 
		 offset += sizeof(uint32))
	{
		printf("%s  %10d\n", mib_regName[offset>>2], GT_READ_MIB(port,offset));
	}

   printf("Rx No Buff Errs :%08u\n", gt64440mii->port[port].stats.rxNoBuf);
   printf("\n");
   printf("================= Receive Q Statistics ================ \n\n");
   for(i=0; i < GT_MII_RX_Q_CNT; i++) {
      printf("RxQ %d avail            : %08u\n", i, 
					gt64440mii->port[port].rxc[i].avail);
      printf("RxQ %d pkts             : %08u\n", i, 
					gt64440mii->port[port].rxc[i].pkts);
      printf("RxQ %d octets           : %08u\n", i, 
					gt64440mii->port[port].rxc[i].octets);
      printf("RxQ %d too short        : %08u\n", i, 
					gt64440mii->port[port].rxc[i].shortErr);
      printf("RxQ %d too long         : %08u\n", i, 
					gt64440mii->port[port].rxc[i].longErr);
      printf("RxQ %d overrun          : %08u\n", i, 
					gt64440mii->port[port].rxc[i].ovrErr);
      printf("RxQ %d collision        : %08u\n", i, 
					gt64440mii->port[port].rxc[i].collErr);
      printf("RxQ %d crc err          : %08u\n", i, 
					gt64440mii->port[port].rxc[i].crcErr);
      printf("RxQ %d no buffer        : %08u\n", i, 
					gt64440mii->port[port].rxc[i].noBufErr);
      printf("RxQ %d no desc          : %08u\n", i, gt64440mii->port[port].rxc[i].noDescErr);
   }
   printf("\n");
   printf("================= Transmit Q Statistics =============== \n\n");
   for(i=0; i < GT_MII_TX_Q_CNT; i++) {
      printf("TxQ %d avail            : %08u\n", i, 
					gt64440mii->port[port].txc[i].avail);
      printf("TxQ %d pkts             : %08u\n", i, 
					gt64440mii->port[port].txc[i].pkts);
      printf("TxQ %d octets           : %08u\n", i, 
					gt64440mii->port[port].txc[i].octets);
      printf("TxQ %d qFull err        : %08u\n", i, 
					gt64440mii->port[port].txc[i].qFull);
      printf("TxQ %d collision        : %08u\n", i, 
					gt64440mii->port[port].txc[i].collErr);
      printf("TxQ %d late collision   : %08u\n", i, 
					gt64440mii->port[port].txc[i].lcolErr);
      printf("TxQ %d underrun         : %08u\n", i, 
					gt64440mii->port[port].txc[i].udrErr);
      printf("TxQ %d retransmit limit : %08u\n", i, 
					gt64440mii->port[port].txc[i].relimErr);
      printf("TxQ %d bufs queued      : %08u\n", i, 
					gt64440mii->port[port].txc[i].qd);
      printf("TxQ %d bufs unqueued    : %08u\n", i, 
					gt64440mii->port[port].txc[i].unqd);
   }
   printf("\n");
   return(NFX_OK);
} /* gt64440mii_uiViewStats */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiViewCfg
 *
 * Description: 	Display device/driver configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiViewCfg(uint32 port)
{
   uint32 i;

   printf("GT64440-MII CONFIGURATION: [Slot:%02d -- CPU:%02d] Port %d\n", 
		   nfxMySlotId, nfxMyCPUId, port);
   printf("===================== General Config =====================\n\n");
   printf("Application Id             : %08d\n", gt64440mii->appId);
   printf("Base Addr                  : %08x\n", gt64440mii->baseAddr);
   printf("Phy Addr                   : %08x\n", gt64440mii->phyAddr);
   printf("Hash Table Addr            : %08x\n", (uint32)gt64440mii->port[port].hash_tbl);
   for(i=0; i < GT_MII_RX_Q_CNT; i++)
      printf("Rx Desc Pool %d Base Addr   : %08x\n", i, 
					(uint32)gt64440mii->port[port].rxQ[i]);
   printf("\n");
   printf("===================== Transmit Control =================== \n\n");
   for(i=0; i < GT_MII_TX_Q_CNT; i++) {
      printf("TxQ %d Start                : %08x\n", i, 
					(uint32)gt64440mii->port[port].txc[i].txqStart);
      printf("TxQ %d End                  : %08x\n", i, 
 					(uint32)gt64440mii->port[port].txc[i].txqEnd);
      printf("TxQ %d Current Addr         : %08x\n", i, 
					(uint32)gt64440mii->port[port].txc[i].txCur);
      printf("TxQ %d Cleanup Addr         : %08x\n", i, 
					(uint32)gt64440mii->port[port].txc[i].txCln);
   }
   printf("\n");
   printf("===================== Receive Control ==================== \n\n");
   for(i=0; i < GT_MII_RX_Q_CNT; i++) {
      printf("RxQ %d Start                : %08x\n", i, 
					(uint32)gt64440mii->port[port].rxc[i].rxqStart);
      printf("RxQ %d End                  : %08x\n", i, 
					(uint32)gt64440mii->port[port].rxc[i].rxqEnd);
      printf("RxQ %d Current Addr         : %08x\n", i, 
					(uint32)gt64440mii->port[port].rxc[i].rxCur);
      printf("RxQ %d Buffer Count         : %08d\n", i, 
					(uint32)gt64440mii->port[port].rxc[i].bufCnt);
   }
   printf("\n");
   return(NFX_OK);
} /* gt_mii_uiViewCfg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiConfigLBMode
 *
 * Description: 	Allows user to enable/disable device loopback.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiConfigLBMode(uint32 port)
{
   int32 mode;

   mode = gt_mii_uiGetInput("Set Loopback Mode [NONE:0 | INT:1 | EXT:2]: \n");
   if((mode < 0) ||  (mode > 2)) {
      printf("Invalid Mode: %d. Exiting....\n", mode);
      return(NFX_OK);
   }    
   gt_mii_diagSetLbMode(port, mode); 

   return(NFX_OK);
} /* gt_mii_uiConfigLBMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowRxq
 *
 * Description: 	Dump the chip receive descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowRxq(uint32 port)
{
   int i, quit;
   volatile gt64440mii_rxdesc_t *descp;

   descp = gt64440mii->port[port].rxc[0].rxqStart;

   printf("GT64440-MII Receive Descriptor Queue Dump - Port %d\n", port);
   printf("===================================================\n");
   for(i=0; i < GT_MII_RX_Q_LEN; i++) {
      printf("%03d   Link: %08x   Buf: %08x   Cmdsts: %08x\n", i, 
                     ((uint32)descp->link), 
                     ((uint32)descp->bufptr), 
                     (descp->cmdsts));
      descp++;
      if( !(i % 25) && i != 0 ) {
	 quit = gt_mii_uiGetInput("Quit [99]: ");
      if( quit == 99 )
	 return 0;
      }
   }
   printf("\n");
   return 0;
} /* gt_mii_uiShowRxq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_uiShowTxq
 *
 * Description: 	Dump the chip transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
gt_mii_uiShowTxq(uint32 port)
{
   int i, quit;
   volatile gt64440mii_txdesc_t *descp;

   descp = gt64440mii->port[port].txc[0].txqStart;

   printf("GT64440-MII Transmit Descriptor Queue Dump - Port %d\n", port);
   printf("====================================================\n");
   for(i=0; i < GT_MII_TX_Q_LEN; i++) {
      printf("%03d   Link: %08x   Buf: %08x  Count: %d  Cmdsts: %08x\n", i, 
                     ((uint32 )descp->link), 
                     ((uint32)descp->bufptr), 
                     (descp->bytecnt_rsvd) >> 16,
                     (descp->cmdsts));
      descp++;
      if( !(i % 25) && i != 0 ) {
         quit = gt_mii_uiGetInput("Quit [99]: ");
         if( quit == 99 )
	    return 0;
      }
   }
   printf("\n");
   return 0;
} /* gt_mii_uiShowTxq */


/*
 *  This routine can be polled to gather stats for SNMP. All counters are
 *  cleared after each call. Counters returned are sequential as listed
 *  below. Caller must pass pointer to buffer with space for 15 32-bit values
 */
void gt64440mii_get_stats(uint32 *stat_buf)
{
    int 	port = 1; // only mgmt port is used
                                                                                
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_GOOD_OCTETS_RCVD_LO); // rx byts
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_GOOD_OCTETS_SENT_LO); // tx bytes
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_GOOD_FRAMES_RCVD);// rx pkts
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_GOOD_FRAMES_SENT);// tx pkts
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_BAD_CRC); 		  // rx crc
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_FRAGMENTS_RCVD);  // rx fragments
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_JABBER_RCVD);     // rx jabbers
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_COLLISIONS);      // collisions
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_LATE_COLLISIONS); // late colls
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_MAC_RCV_ERR);     // rx errors
    *stat_buf++ = GT_READ_MIB(port, ETH_MIB_UNDERSIZE_RCVD);  // rx undersized

    *stat_buf++ = gt64440mii->port[port].rxc[0].noDescErr; // no EEE desc
    *stat_buf++ = gt64440mii->port[port].stats.rxNoBuf; // no buffer
    *stat_buf++ = gt64440mii->port[port].txc[0].udrErr; // tx underruns
    *stat_buf++ = gt64440mii->port[port].txc[0].qFull; // queue full drops

    gt64440mii->port[port].rxc[0].noDescErr = 0;
    gt64440mii->port[port].stats.rxNoBuf = 0;
    gt64440mii->port[port].txc[0].udrErr = 0;
    gt64440mii->port[port].txc[0].qFull = 0;
}

#endif
