/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: gtinit.c,v 1.3 2002/10/14 18:38:55 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/22/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <sys/mips.h>
#include <sys/bmfpga.h>
#include <sys/gt64440r.h>
#include <exports/pci-api.h>
#include <exports/pci-reg.h>
#include <exports/cm-api.h>
                                                                                
void	pci_init_bobcat(void );
void	pcix_remap_bus_dev(uint32, uint32);
#ifdef R9000
extern  int is_BC2220;
#endif

void
mem_map_gt_init()
{
	int32   rc;
	uint32  bus, dev, data;
                                                                                
	/*
	 * If PCI_RST is asserted (low),
	 * write BMFPGA to take it out of reset.
	 */
	if ((GT_READ(PCI_0MODE)  & 0x80000000) == 0)
	{
		printf("Taking PCI out of reset.\n");

		/* 
		 * Enable FP GbE ports. Warm reset FP & TXRX
		 */
		data = (BMFPGA_GPR_FP_PORT0_ENA | BMFPGA_GPR_FP_PORT1_ENA |
			    BMFPGA_GPR_FP_PORT2_ENA | BMFPGA_GPR_FP_PORT3_ENA |
			    BMFPGA_GPR_FP_WARM_RESET | BMFPGA_GPR_TXRX_WARM_RESET);
		BMFPGA_WRITE( BMFPGA_REG_GPR, data );

		delay_us(1000000);

		/* 
		 * Take FP & TXRX out of reset.
		 */
		data = (BMFPGA_GPR_FP_PORT0_ENA | BMFPGA_GPR_FP_PORT1_ENA |
			    BMFPGA_GPR_FP_PORT2_ENA | BMFPGA_GPR_FP_PORT3_ENA);
#ifdef R9000
        /* Configures LED for 2-port unit if Bobcat model is 2220,
         * which only has 2 FP ports.
         */
        if (is_BC2220) {
            data |= (1 << 11);
        }
#endif
		BMFPGA_WRITE( BMFPGA_REG_GPR, data );
	}

	printf("Waiting for FP/TXRX out of reset.");
	do
	{
		delay_us(1000000);
		/*
		 * TXRX/FP will update FP STATUS in BMFPGA to
		 * PROM_INIT state when it is out of reset.
		 * Wait util status is updated before configuring PCI.
		 */
		data = BMFPGA_READ(BMFPGA_REG_FP_STATUS);
		printf(".");

	} while ( (data & BMFPGA_SLOT_STATUS_SLICE_STATE_MASK ) != 
				BMFPGA_SLOT_STATUS_SLICE_STATE_PROM_INIT );

	printf("\n");

	/*
	 * Program bus numbers
	 */
	GT_WRITE(PCI_0P2P_CONFIGURATION, 0x010000ff);
	GT_WRITE(PCI_1P2P_CONFIGURATION, 0x030000ff);

	/*
	 * Remap PCI-X Bus & Dev IDs for GT64440 from
	 * bus=0xFF, dev=0x1F to bus=0x00, dev=0x01
	 * 
	 * This MUST be done BEFORE other PCI config.
	 */
	pcix_remap_bus_dev(PCI0_PRI_BUS, PCI0_GT_DEV);

	/*
	 *  Enable PCI 0 as master to do config cycles.
	 */
	bus = PCI0_PRI_BUS;
	dev = PCI0_GT_DEV;
	rc = pci_cfg_read(bus, dev, 0, PCI0_COMMAND_STATUS_REG, &data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_read failed\n", __FUNCTION__, __LINE__);
	}
                                                                                
	data |= PCI_COMMAND_MASTER_ENABLE | PCI_COMMAND_MEM_ENABLE;
	rc = pci_cfg_write(bus, dev, 0, PCI0_COMMAND_STATUS_REG, data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_write failed\n", __FUNCTION__, __LINE__);
	}
                                                                                
	/*
	 *  Enable PCI 1 as master to do config cycles.
	 */
	bus = PCI1_PRI_BUS;
	dev = PCI1_GT_DEV;
	rc = pci_cfg_read(bus, dev, 0, PCI1_COMMAND_STATUS_REG, &data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_read failed\n", __FUNCTION__, __LINE__);
	}
	data |= PCI_COMMAND_MASTER_ENABLE | PCI_COMMAND_MEM_ENABLE;
	rc = pci_cfg_write(bus, dev, 0, PCI1_COMMAND_STATUS_REG, data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_write failed\n", __FUNCTION__, __LINE__);
	}
                                                                                

	GT_WRITE(SCS_0_BASE_ADDRESS, (HW_GT_MEM_0_BASE >> 16));
	GT_WRITE(SCS_0_SIZE,         ((GT_MEM_0_SIZE-1) >> 16));

	GT_WRITE(SCS_1_BASE_ADDRESS, (HW_GT_MEM_1_BASE >> 16));
	GT_WRITE(SCS_1_SIZE,         ((GT_MEM_1_SIZE-1) >> 16));

	GT_WRITE(SCS_2_BASE_ADDRESS, 0);
	GT_WRITE(SCS_2_SIZE,         0);

	GT_WRITE(SCS_3_BASE_ADDRESS, 0);
	GT_WRITE(SCS_3_SIZE,         0);

	GT_WRITE(PCI_0I_O_BASE_ADDRESS, 0);
	GT_WRITE(PCI_0I_O_SIZE, 0);

	GT_WRITE(PCI_0MEMORY0_BASE_ADDRESS, (HW_QLOGIC_0_BASE >> 16));
	GT_WRITE(PCI_0MEMORY0_SIZE, ((QLOGIC_MEM_SIZE - 1) >> 16));
                                                                                
	GT_WRITE(PCI_0MEMORY1_BASE_ADDRESS, 0);
	GT_WRITE(PCI_0MEMORY1_SIZE, 		0);
                                                                                
	GT_WRITE(PCI_0MEMORY2_BASE_ADDRESS, 0);
	GT_WRITE(PCI_0MEMORY2_SIZE, 		0);
                                                                                
	GT_WRITE(PCI_0MEMORY3_BASE_ADDRESS, 0);
	GT_WRITE(PCI_0MEMORY3_SIZE, 		0);
                                                                                
	GT_WRITE(PCI_1I_O_BASE_ADDRESS, (HW_CF_CTRL_IO_BASE >> 16));
	GT_WRITE(PCI_1I_O_SIZE, ((CF_CTRL_IO_SIZE - 1)>> 16));

	GT_WRITE(PCI_1MEMORY0_BASE_ADDRESS, (HW_TXRX_SHMEM_BASE >> 16));
	GT_WRITE(PCI_1MEMORY0_SIZE, ((TXRX_SHMEM_SIZE - 1) >> 16));
                                                                                
	GT_WRITE(PCI_1MEMORY1_BASE_ADDRESS, (HW_DP83816_0_BASE >> 16));
	GT_WRITE(PCI_1MEMORY1_SIZE, ((DP83816_SIZE - 1) >> 16));
                                                                                
	GT_WRITE(PCI_1MEMORY2_BASE_ADDRESS, (HW_CF_CTRL_MEM_BASE >> 16));
	GT_WRITE(PCI_1MEMORY2_SIZE, ((CF_CTRL_MEM_SIZE - 1) >> 16));
                                                                                
	GT_WRITE(PCI_1MEMORY3_BASE_ADDRESS, (HW_TXRX_MBOX0_BASE >> 16));
	GT_WRITE(PCI_1MEMORY3_SIZE,((MBOX_SIZE - 1) >> 16));
                                                                                
	GT_WRITE(PCI_0SCS_0_BANK_SIZE, (GT_MEM_0_SIZE-1) & 0xFFFFF000);
	GT_WRITE(PCI_1SCS_0_BANK_SIZE, (GT_MEM_0_SIZE-1) & 0xFFFFF000);

	GT_WRITE(PCI_0SCS_1_BANK_SIZE, (GT_MEM_1_SIZE-1) & 0xFFFFF000);
	GT_WRITE(PCI_1SCS_1_BANK_SIZE, (GT_MEM_1_SIZE-1) & 0xFFFFF000);

	GT_WRITE(PCI_0SCS_2_BANK_SIZE, 0);
	GT_WRITE(PCI_1SCS_2_BANK_SIZE, 0);

	GT_WRITE(PCI_0SCS_3_BANK_SIZE, 0);
	GT_WRITE(PCI_1SCS_3_BANK_SIZE, 0);

	GT_WRITE(PCI_0CS_0_BANK_SIZE, 0);
	GT_WRITE(PCI_1CS_0_BANK_SIZE, 0);

	GT_WRITE(PCI_0CS_1_BANK_SIZE, 0);
	GT_WRITE(PCI_1CS_1_BANK_SIZE, 0);

	GT_WRITE(PCI_0CS_2_BANK_SIZE, 0);
	GT_WRITE(PCI_1CS_2_BANK_SIZE, (BMFPGA_SIZE-1) & 0xFFFFF000);

	GT_WRITE(PCI_0CS_3_BANK_SIZE, 0);
	GT_WRITE(PCI_1CS_3_BANK_SIZE, 0);

	GT_WRITE(PCI_0CS_BOOT_BANK_SIZE, 0);
	GT_WRITE(PCI_1CS_BOOT_BANK_SIZE, 0);

	GT_WRITE(PCI_0P2P_MEM0_BAR_SIZE, 0);
	GT_WRITE(PCI_1P2P_MEM0_BAR_SIZE, 0);

	GT_WRITE(PCI_0P2P_MEM1_BAR_SIZE, 0);
	GT_WRITE(PCI_1P2P_MEM1_BAR_SIZE, 0);

	GT_WRITE(PCI_0P2P_I_O_BAR_SIZE, 0);
	GT_WRITE(PCI_1P2P_I_O_BAR_SIZE, 0);

	GT_WRITE(PCI_0INTERNAL_SRAM_BAR_SIZE, 0);
	GT_WRITE(PCI_1INTERNAL_SRAM_BAR_SIZE, 0);

	GT_WRITE(PCI_0EXPANSION_ROM_BAR_SIZE, 0);
	GT_WRITE(PCI_1EXPANSION_ROM_BAR_SIZE, 0);

	GT_WRITE(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0xffff7dfc);
	GT_WRITE(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0xffff7dbc);

	GT_WRITE(PCI_0SCS_0_BASE_ADDRESS_REMAP, HW_GT_MEM_0_BASE);
	GT_WRITE(PCI_1SCS_0_BASE_ADDRESS_REMAP, HW_GT_MEM_0_BASE);

	GT_WRITE(PCI_0SCS_1_BASE_ADDRESS_REMAP, HW_GT_MEM_1_BASE);
	GT_WRITE(PCI_1SCS_1_BASE_ADDRESS_REMAP, HW_GT_MEM_1_BASE);

	GT_WRITE(PCI_0SCS_2_BASE_ADDRESS_REMAP, 0);
	GT_WRITE(PCI_1SCS_2_BASE_ADDRESS_REMAP, 0);

	GT_WRITE(PCI_0SCS_3_BASE_ADDRESS_REMAP, 0);
	GT_WRITE(PCI_1SCS_3_BASE_ADDRESS_REMAP, 0);

	GT_WRITE(PCI_1I_O_ADDRESS_REMAP, 0);
	GT_WRITE(PCI_1MEMORY2_ADDRESS_REMAP, 0x00000080);

        // add PCI0_PADS_CALIBRATION register config  Brian's e-mail 04/13/2006
        GT_WRITE(PCI0_PADS_CALIBRATION, 0x83f3ffff);    // Enable writes to register
        GT_WRITE(PCI0_PADS_CALIBRATION, 0x83f2ffff);    // Turn off auto-calibration
        GT_WRITE(PCI0_PADS_CALIBRATION, 0x83f2fff4);    // Set drive strength
        GT_WRITE(PCI0_PADS_CALIBRATION, 0x03f2fff4);    // Disable writes

        // add PCI1_PADS_CALIBRATION register config  Brian's e-mail 04/13/2006
        GT_WRITE(PCI1_PADS_CALIBRATION, 0x83f3ffff);    // Enable writes to register
        GT_WRITE(PCI1_PADS_CALIBRATION, 0x83f2ffff);    // Turn off auto-calibration
        GT_WRITE(PCI1_PADS_CALIBRATION, 0x83f2fff4);    // Set drive strength
        GT_WRITE(PCI1_PADS_CALIBRATION, 0x03f2fff4);    // Disable writes

	GT_WRITE(PCI_0COMMAND, 0x0107e371);
	GT_WRITE(PCI_1COMMAND, 0x0107e371);

	GT_WRITE(PCI_0TIMEOUT_RETRY, 0x00400000);
	GT_WRITE(PCI_1TIMEOUT_RETRY, 0x00400000);

	GT_WRITE(PCI_0READ_BUFFER_DISCARD_TIMER, 0x0000ffff);
	GT_WRITE(PCI_1READ_BUFFER_DISCARD_TIMER, 0x0000ffff);

	GT_WRITE(MSI_0TRIGGER_TIMER, 0x0000ffff);
	GT_WRITE(MSI_1TRIGGER_TIMER, 0x0000ffff);

	GT_WRITE(PCI_0ARBITER_CONTROL, 0x80000030);
	GT_WRITE(PCI_1ARBITER_CONTROL, 0x80000030);

	GT_WRITE(CPU_BASE_ADDRESS_ENABLE, 0x00003a0c);

	pci_init_bobcat();
}

