/*
 *-----------------------------------------------------------------
 *
 * Name:	wd.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc
 *
 * Description: Watchdog implementation using GT64440
 *              Ported from sm-gt64240.
 *
 * Created by:	Shaun Hui
 *
 * Date Created:4/18/04
 *
 *-----------------------------------------------------------------
 */

#ifndef	PROM_R9000

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/gt64440r.h>
#include <exports/hwMap.h>

uint32	wd_cnt;

extern uint32	watchdog_state;

#define		WD_CTL1_01		(1 << 24)
#define		WD_CTL1_02		(1 << 25)
#define		WD_CTL2_01		(1 << 26)
#define		WD_CTL2_02		(1 << 27)

/*-----------------------------------------------------------------
 * Name:		wd_enable
 *
 * Description: Enables watchdog timer by toggling WD CFG reg.
 *
 *-----------------------------------------------------------------
 */
void
wd_enable( uint32	nsec )
{
	extern uint32 get_busclock(void);

	wd_cnt = (nsec * get_busclock() * 1000000) >> 8;

	if (!watchdog_state) {
		GT_WRITE( WATCHDOG_CONFIGURATION_REGISTER, wd_cnt | WD_CTL1_01 );
		GT_WRITE( WATCHDOG_CONFIGURATION_REGISTER, wd_cnt | WD_CTL1_02 );
	}
}

/*-----------------------------------------------------------------
 * Name:		wd_disable
 *
 * Description: Disables watchdog timer by toggling WD CFG reg.
 *
 *-----------------------------------------------------------------
 */
void
wd_disable( void )
{

	if (watchdog_state) {
		GT_WRITE( WATCHDOG_CONFIGURATION_REGISTER, wd_cnt | WD_CTL1_01 );
		GT_WRITE( WATCHDOG_CONFIGURATION_REGISTER, wd_cnt | WD_CTL1_02 );
	}
}

/*-----------------------------------------------------------------
 * Name:		wd_kick
 *
 * Description: Service watchdog timer.
 *
 *-----------------------------------------------------------------
 */
void
wd_kick( void )
{
	extern uint32	watchdog_state;

	if (watchdog_state) {
		GT_WRITE( WATCHDOG_CONFIGURATION_REGISTER, wd_cnt | WD_CTL2_01 );
		GT_WRITE( WATCHDOG_CONFIGURATION_REGISTER, wd_cnt | WD_CTL2_02 );
	}
}

/*-----------------------------------------------------------------
 * Name:		wd_init
 *
 * Description: Initialize gt64440 watchdog timer.
 *
 *-----------------------------------------------------------------
 */
void
wd_init( void )
{
	uint32	reg;


	if (watchdog_state) {
		watchdog_disable();
	}
	GT_WRITE( WATCHDOG_VALUE_REGISTER, 0);

	/*
	 * GT WD NMI is not connected to R9K.
	 * Generate a CPU 0 INT instead.
	 * Enable CPU interrupt 0 for WDE | WDNMI
	 */
	reg = GT_READ(CPU_INT_0_MASK_LOW) | 
		(INT_MASK_LOW_WDNMI | INT_MASK_LOW_WDE) ;
	GT_WRITE(CPU_INT_0_MASK_LOW,  reg);

	/*
	 * Enable R9K external Int. & HW INT 1
	 * GT CPU INT 0 -> HWINT 1.
	 */
	reg  = _mips_mfc0(C0_SR);
	reg |= SR_HINT1 | SR_IE;
	_mips_mtc0(C0_SR, reg);

}
#endif // PROM_R9000

