/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: hw-api.c,v 1.16 2002/10/31 17:54:42 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/30/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/gt64240r.h>
#include <exports/hwMap.h>
#ifdef R7000
#include "../sm-gt64240/gpp-defs.h"
#endif
#include <exports/prom-part-api.h>
#include <sys/led-api.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
int
isFC()
{
#ifdef LINUX_TEST
#if defined (NFX_MOD_FCNIM)
	return 1;
#else
	return 0;
#endif
#else
#if defined(R9000)
	uint32	cpu_id;

	cpu_id   = _mips_mfc0(C0_PRID);
	cpu_id  &= PRID_R9K_CPU_MASK;
	cpu_id >>= PRID_R9K_CPU_SHIFT;

	return (cpu_id == 1);

#elif !(defined (R7000))
	return 0;
#else
	int32	cfg;
	cfg = _mips_mfc0(16);
	cfg &= 0x00300000;
	cfg >>= 20;
	return ((cfg & 1) == 1);
#endif
#endif
}

#ifdef R7000
int
isOldFC()
{
#ifdef R9000

	return 0;
#else
	/* bit 23 indicates FC rev (1 = old board, 0 = Rev A board) */
	return ((*(volatile uint32 *)PHYS_TO_K1(HW_GALILEO_BASE + GPP_VALUE)) &
															GPP_FC_REV_MASK);
#endif
}
#endif

int
isSSC()
{
#ifdef LINUX_TEST
#if defined (NFX_MOD_SSC)
	return 1;
#else
	return 0;
#endif
#else
#if defined(R9000)
	uint32	cpu_id;

	cpu_id   = _mips_mfc0(C0_PRID);
	cpu_id  &= PRID_R9K_CPU_MASK;
	cpu_id >>= PRID_R9K_CPU_SHIFT;

	return (cpu_id == 0);

#elif !(defined (R7000) || defined(SSC_MGMT))
	return 0;
#else
	uint32	cfg;
	cfg = _mips_mfc0(16);
	cfg &= 0x00300000;
	cfg >>= 20;
	return ((cfg & 1) == 0);
#endif
#endif
}

int
isOldSSC()
{
#ifdef LINUX_TEST
#if defined(NFX_MOD_SSC)
    return 1;
#else
    return 0;
#endif
#else
#if !defined(R7000) || defined(R9000)
	return 0;
#else
    uint32  cfg;
    cfg = _mips_mfc0(16);
    cfg &= 0x00300000;
    cfg >>= 20;
    return ((cfg & 2) == 2);
#endif
#endif
}

#if !defined(NFP_TXRX) && !defined(NFP_FP)
int
isTXRX()
{
#ifdef NFP_TXRX
	return 1;
#endif

#ifdef SIBYTE
	uint64	config;
	config = ((*(volatile uint64 *)PHYS_TO_K1(0x10020008)) & 0x00000000fc000000ll) >> 28;
	if ((config & 0xF) == 1)
		return 1;
	else
		return 0;
#endif

	return 0;
}

int
isFP()
{
#ifdef NFP_FP
	return 1;
#endif

#ifdef SIBYTE
	uint64	config;
	config = ((*(volatile uint64 *)PHYS_TO_K1(0x10020008)) & 0x00000000fc000000ll) >> 28;
	if ((config & 0xF) == 3)
		return 1;
	else
		return 0;
#endif

	return 0;
}
#endif

#ifdef SIBYTE
static int hw_is_cheetah = -1 ;
#endif

int
isCheetah()
{
#ifdef SIBYTE
	uint64	config;

    if (hw_is_cheetah == -1)
    {
        config = ((*(volatile uint64 *)PHYS_TO_K1(0x10020008)) & 0x00000000fc000000ll) >> 27;
        if (config & 0x1)
            hw_is_cheetah = 1;
        else
            hw_is_cheetah = 0;
    }
    return (hw_is_cheetah);
    
#else
	return 0;
#endif
}

int
isSSCmgmt()
{
#ifdef SSC_MGMT
	return 1;
#else
	return 0;
#endif
}

/*
 * find out if the RecvBoot jumper is on
 * txrx, fc-control - bmflapga->status
 * fc cpuid=0,1,2 gpp pins
 * fp ???
 */
int
isRecovBoot(void)
{
#if defined(SIBYTE)
    return 0;
#elif defined (R7000)
    return 0;
#else
    return 0;
#endif
}

/*
 * find out if the Diag jumper is on
 * txrx, fc-control - bmflapga->status
 * fc cpuid=0,1,2 gpp pins
 * fp ???
 */
int
isDiagJumpered(void)
{
    // for boards with Diag Proms check for diag jumper
#if defined(SIBYTE)
    return 0;
#elif defined (R7000)
    return 0;
#else
    // no diag proms
    return 0;
#endif
}

#ifdef PROM

int
uncompressProms(dest, destend)
    uchar8 * dest;
    uchar8 * destend;
{
    prom_part_cb_p  cb_p;
    comp_rec_t *    cr_p;
    int             rv;
    int             max;
    unsigned long   destLen;
    unsigned long   res;
    uchar8 *        bp;
    uchar8 *        ep;
    uint64 *        dp;
    unsigned long   decomp_size;
    uchar8 *        base_p = (uchar8 *) PA_TO_KVA1(HW_PROM_BASE);


#ifdef PROM_R9000
    base_p += IO_OFFSET;
#endif

    rv = NFX_ERR;


    destLen = (unsigned long)(destend - dest);
    cr_p = (comp_rec_t *) (base_p + OLD_PROM_COMP_REC_OFFSET);
    if (cr_p->signature == COMPRESS_SIGNATURE &&
        cr_p->compressed_start > 0x1000 &&
        cr_p->compressed_start < OLD_PROM_COMP_REC_OFFSET &&
        cr_p->compressed_size  > 0x1000 &&
        cr_p->compressed_size  < OLD_PROM_COMP_REC_OFFSET) {

        decomp_size = destLen;
        rv = uncompress(
                dest, 
                &decomp_size, 
                base_p + cr_p->compressed_start, 
                cr_p->compressed_size);
        if (rv == 0)
            goto uncompress_memz;
    }

    cb_p = (prom_part_cb_p)
        (base_p + PROM_RPRM_OFFS + PROM_RPRM_SIZE - PROM_PART_CB_SIZE);
    max = PROM_RPRM_SIZE - PROM_PART_CB_SIZE;

    decomp_size = destLen;
    if (cb_p->signature == PROM_PART_SIGNATURE &&
        cb_p->compressed_start > 0x1000 &&
        cb_p->compressed_start < max &&
        cb_p->compressed_size  > 0x1000 &&
        cb_p->compressed_size  < max) {
        rv = uncompress(dest,
                        &decomp_size,
                        base_p + cb_p->compressed_start,
                        cb_p->compressed_size);
        if (rv == 0)
            goto uncompress_memz;
    }
    /*
     * rprom boot assumption failed must be boot prom, our last chance
     */
    cb_p = (prom_part_cb_p)
        (base_p + PROM_BOOT_OFFS + PROM_BOOT_SIZE - PROM_PART_CB_SIZE);
    decomp_size = destLen;
    if (cb_p->signature == PROM_PART_SIGNATURE) {
        rv = uncompress(dest,
                        &decomp_size,
                        base_p + cb_p->compressed_start,
                        cb_p->compressed_size);
        goto uncompress_memz;
    }
    return rv;

uncompress_memz:

    /*
     * zero the unused decomp area
     */
    /* take care of non-cache aligned start */
    bp = &dest[decomp_size];
    res = (destLen - decomp_size) & CACHELINE_MASK;
    if (res) {
        while (res < CACHELINE_SIZE) {
            *bp++ = 0;
            res++;
        }
    }
    /* take care of non-cache aligned end */
    ep = destend;
    res = (unsigned long)((address_t)destend & CACHELINE_MASK);
    while (res) {
        --ep;
        *ep = 0;
        res--;
    }
    dp = (uint64 *) bp;
    do {
        dp[0] = 0;
        dp[1] = 0;
        dp[2] = 0;
        dp[3] = 0;
        dp += 4;
    } while ((address_t)dp <= (address_t)ep);

    DIAG_LED(LED_DIAG_12);
    return rv;
}

#ifdef  SIBYTE

extern void memz(uint64, uint64);

void
zero_lomem(void)
{
    extern char end[];
    uint64 pa;
    volatile uint32 * wd_p;
    int len;

    memz(PHYS_TO_K0(0x00000000),   /* 0 */ 
         PHYS_TO_K0(0x00800000));  /* 8MB */ 

    pa = (uint64)(end); 
    len = (pa & 0x1C);            /* truncat down to 32 align */ 
    wd_p = (uint32 *)pa;
    while (len < 0x20) {
        wd_p = 0;
        wd_p++;
        len += 4;
    }
    pa = ((pa + 0x1f) & ~0x1f);
    memz(pa, PHYS_TO_K0(0x04800000));  /* 48 MB  + 24 MB*/ 
}
#else
void zero_lomem(void)
{}
#endif  /* SIBYTE */
#endif
