/*
 *-----------------------------------------------------------------
 *
 * Name:	i2c.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.,
 *
 * Description: I2C related routines. Ported from sm-gt64240/i2c.c
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/05/04
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/gt64440r.h>
#include <exports/hw-api.h>
#include "i2c-api.h"
#ifdef SIBYTE_BC
#include <sys/sb1250/sb1250_defs.h>
#endif

#if defined(__OpenBSD__) && defined(_KERNEL)
#include <sys/types.h>
#include <sys/systm.h>
#include <machine/param.h>

#define delay_us(x) delay(x)

#endif

uchar8	* i2c_reg_string[] = {
	"SLAVE ADDR", 
	"DATA",
	"CONTROL",
	"STATUS",
	"EXT SLAVE ADDR",
	"unused",
	"unused",
	"SOFT RESET"
};

/*-----------------------------------------------------------------
 * Name:		i2c_reg_write
 *
 * Description: write to i2c register.
 *
 *-----------------------------------------------------------------
 */
static void
i2c_reg_write(
	uint32	reg,
	uchar8	data
)
{

/*
	printf("%s <== 0x%x\n", i2c_reg_string[(reg - 0xc000)/4], data);
*/

	GT_WRITE( reg, data );
}

/*-----------------------------------------------------------------
 * Name:		i2c_reg_read
 *
 * Description: Read from i2c register.
 *
 *-----------------------------------------------------------------
 */
static uchar8
i2c_reg_read(
	uint32	reg
)
{
	uchar8	data;

	data = GT_READ( reg );

/*
	printf("%s ==> 0x%x\n", i2c_reg_string[(reg - 0xC000)/4], data);
*/

	return data;
}

/*
 * fake_i2c() moved to sm-seep/seep-api.c
 */

/*-----------------------------------------------------------------
 * Name:	i2c()
 *
 * Description: Do i2c read/write commands.
 *
 *-----------------------------------------------------------------
 */
int32
gt64440_i2c(
	uint32	op,
	uchar8	i2c_addr,
	uchar8	byte_addr,
	uint32	len,
	uchar8	* buf
)
{
	uchar8	data;
	uint32	i;

	switch (op) {
		case I2C_READ:
			for (i=0; i<len; i++) {
				/*
				 * transmit start + device address
				 */
				i2c_reg_write(I2C_CONTROL, 0x60);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				i2c_reg_write(I2C_DATA, i2c_addr | 0);
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if (data != 0x18)
					return -1;

				/*
				 * transmit word address
				 */
				i2c_reg_write(I2C_DATA, (byte_addr+i) | 0);
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if (data != 0x28)
					return -1;

				/*
				 * transmit start + device address
				 */
				i2c_reg_write(I2C_CONTROL, 0x60);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				i2c_reg_write(I2C_DATA, i2c_addr | 1);
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if (data != 0x40)
					return -1;

				/*
				 * read data byte
				 */
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));
				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if ((data != 0x50) && (data != 0x58))
					return -1;

				buf[i] = i2c_reg_read(I2C_DATA);

				/*
				 * transmit stop
				 */
				i2c_reg_write(I2C_CONTROL, 0x50);

				while ((data = i2c_reg_read(I2C_CONTROL)) & 0x10);
//printf("I2C_READ, offset = %d, byte = 0x%x\n", byte_addr+i, buf[i]);
			}

			break;

		case I2C_WRITE:
			for (i=0; i<len; i++) {
//printf("I2C_WRITE, offset = %d, byte = 0x%x\n", byte_addr+i, buf[i]);
				/*
				 * transmit start + device address
				 */
				i2c_reg_write(I2C_CONTROL, 0x60);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				i2c_reg_write(I2C_DATA, i2c_addr | 0);
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if (data != 0x18)
					return -1;

				/*
				 * transmit word address
				 */
				i2c_reg_write(I2C_DATA, (byte_addr+i) | 0);
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if (data != 0x28)
				return -1;

				/*
				 * write data byte
				 */
				i2c_reg_write(I2C_DATA, buf[i]);
				i2c_reg_write(I2C_CONTROL, 0x40);

				while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

				while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
				if (data != 0x28)
					return -1;

				/*
				 * transmit stop
				 */
				i2c_reg_write(I2C_CONTROL, 0x50);

				while ((data = i2c_reg_read(I2C_CONTROL)) & 0x10);

				delay_us(11 * 1000);
			}

			break;

		default:
			printf("bad i2c op, %d\n", op);
	}

	return 0;

}	/* i2c() */

