/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipmd-lib.c
 *
 * RCS:  $Id: ipmd-lib.c,v 1.28 2002/11/21 21:19:40 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: ipm libary message functions.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *---------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "nfx-error.h"
#include "nfx-types.h"
#include "nfx-defs.h"
#include "exports/eee-api.h"
#include "exports/anpssc-api.h"
#include "exports/ncm-api.h"
#include "exports/ipm-api.h"
#include "../sm-ipm/ipm.h"
#include "../sm-ipmd/ipmd-stats.h"

#define IPM_PRINTF(a)	

#define IPM_FLAG_TO_RC(_flag, _rc)	\
{									\
	if (_flag & IPM_MSG_CMD_NOBUF)	\
		_rc = IPM_CMDERR_SYSNOBUF;		\
	else if (_flag & IPM_MSG_CMD_NOINTF)\
		_rc = IPM_CMDERR_NOINTF;		\
	else if (_flag & IPM_MSG_CMD_NORT)	\
		_rc = IPM_CMDERR_NORT;			\
	else if (_flag & IPM_MSG_CMD_DUPRT)	\
		_rc = IPM_CMDERR_DUPRT;			\
	else if (_flag & IPM_MSG_CMD_TIMEOUT)	\
		_rc = IPM_CMDERR_TIMEOUT;		\
	else if (_flag & IPM_MSG_CMD_DUPIP)	\
		_rc = IPM_CMDERR_DUPIP;			\
	else if (_flag & IPM_MSG_CMD_NOIP)	\
		_rc = IPM_CMDERR_NOIP;			\
	else if (_flag & IPM_MSG_CMD_NOGW)	\
		_rc = IPM_CMDERR_NOGW;			\
	else if (_flag & IPM_MSG_CMD_NOVIF)	\
		_rc = IPM_CMDERR_NOVIF;			\
	else if (_flag & IPM_MSG_CMD_DUPVIF)	\
		_rc = IPM_CMDERR_DUPVIF;		\
	else if (_flag & IPM_MSG_CMD_HASIP_VIF)	\
		_rc = IPM_CMDERR_HASIP_VIF;		\
	else if (_flag & IPM_MSG_CMD_ACTLINK_VIF)	\
		_rc = IPM_CMDERR_ACTLINK_VIF;			\
	else if (_flag & IPM_MSG_CMD_BADMODE_VIF)	\
		_rc = IPM_CMDERR_BADMODE_VIF;			\
	else if (_flag & IPM_MSG_CMD_NOLINK_VIF)	\
		_rc = IPM_CMDERR_NOLINK_VIF;			\
	else if (_flag & 	IPM_MSG_CMD_MANYVIF)	\
		_rc = IPM_CMDERR_MANYVIF;				\
	else if (_flag & IPM_MSG_CMD_OK)	\
		_rc = NFX_OK;					\
	else								\
		_rc = NFX_ERR;					\
	IPM_PRINTF(("lib rc is %d flag is %x\n", _rc, _flag));	\
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_PrntErrMsg	
 *
 * Description: print err messages
 *
 *-----------------------------------------------------------------
 */
void
ipmd_PrntErrMsg(int			errType, 
				char		*ifname,
				char		*vifname,
				uint32		ipaddr1,
				uint32		ipaddr2) 
{
	
	switch (errType) {
	case IPM_CMDERR_SYSNOBUF:
		printf("IPMD did not has enough buffer to execute this cmd\n");
		break;
		
	case IPM_CMDERR_TIMEOUT:
		printf("TXRX did not respond for this cmd\n");
		break;
		
	case IPM_CMDERR_NOINTF:
		if (vifname)
			printf("some interfaces %s are not found, or illegal for this command\n",
				   ifname);
		else
			printf("Interface %s was not found\n", ifname);
		break;
		
	case IPM_CMDERR_DUPRT:
		printf("Route %s already exist\n", 
			   inet_ntoa(*((struct in_addr*)&ipaddr1)));
		break;
		
	case IPM_CMDERR_NORT:
		printf("Route %s was not found, or is directed connected\n", 
			   inet_ntoa(*((struct in_addr*)&ipaddr1)));
		break;

	case IPM_CMDERR_APPNOBUF:
		printf("App. did not provide enough buffer for this cmd\n");
		break;
		
	case IPM_CMDERR_IPMTIMEOUT:
		printf("IPMD did not repond to the cmd\n");
		break;
		
	case IPM_CMDERR_DUPIP:
		printf("IP %s already in use\n", 
			   inet_ntoa(*((struct in_addr*)&ipaddr1)));
		break;
		
	case IPM_CMDERR_NOIP:
		printf("IP %s did not exist\n", 
			   inet_ntoa(*((struct in_addr*)&ipaddr1)));
		break;

	case IPM_CMDERR_NOGW:
		printf("Gateway %s was not directly connected\n", 
			   inet_ntoa(*((struct in_addr*)&ipaddr2)));
		break;

#if defined(LINK_AGG)
	case  IPM_CMDERR_NOVIF:
		printf("Virtual interface %s not found or has bad name\n",
			   vifname?vifname:ifname); 
		break;
		
	case IPM_CMDERR_DUPVIF:
		printf("Virtual interface %s already exist\n", 
			   vifname);
		break;

	case IPM_CMDERR_HASIP_VIF:
		printf("Virtual interface %s has IP(s) assigned to it\n",
			   vifname);
		break;

	case IPM_CMDERR_ACTLINK_VIF:
		printf("one of link %s is active link \n", ifname);
		break;
		
	case IPM_CMDERR_BADMODE_VIF:
		printf("Virtual interface %s is not configued for this action\n",
			   vifname);
		break;
		
	case IPM_CMDERR_NOLINK_VIF:
		printf("Virtual interface %s contains no link\n",
			   vifname?vifname:ifname);
		break;
		
	case IPM_CMDERR_MANYLINK_VIF:
		printf("Virtual interface %s cannot be given more than %d links\n",
			   vifname, IPM_MAXLINK_VIF);
		break;

	case IPM_CMDERR_MANYVIF:
		printf("Max number of allowd virtual interface %d reached\n",
			   IPM_MAX_VIF);
		break;
#endif /* LINK_AGG */
	}

	return;
}

#if defined(LINK_AGG)
void ipmd_PrintVif(ipm_vif_t *ipm_vifp, uint32 mtype)
{
	
	printf("cmd %d for vif %s\n", mtype, ipm_vifp->vif_name);
	printf("links are(count = %d) 1=%s, 2=%s, 3=%s, 4=%s\n",
		   ipm_vifp->vif_linkCnt, &ipm_vifp->vif_links[0][0],
		   &ipm_vifp->vif_links[1][0],
		   &ipm_vifp->vif_links[2][0],
		   &ipm_vifp->vif_links[3][0]);
	printf("pref link %s, ack link %s\n", 
		   ipm_vifp->vif_prefLink, ipm_vifp->vif_actLink);
	printf("slot is %d, ip count is %d\n", 
		   ipm_vifp->vif_slot, ipm_vifp->vif_ipCnt);
	printf("ip = %s, ", inet_ntoa(*((struct in_addr*)&(ipm_vifp->vif_ip))));
	printf("mask = %s \n", inet_ntoa(*((struct in_addr*)&(ipm_vifp->vif_mask))));
	printf("flags = %x\n", ipm_vifp->vif_flags);
	printf("\n");
  return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ModifyVif	
 *
 * Description: core function to manipulate virtual interfaces.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
static int32					/* NFX_OK, NFX_ERR */
ipmd_ModifyVif  (int		mtype,	
				 char		*vifName,
				 char		*links,
				 int		count,
				 uint32		ip,
				 uint32		mask,
				 uint32		flags,
				 char		*app,	
				 int		sock,
				 char		*node)
{
  int			bufsize;
  ipm_mhdr_t	*ipm_hdrp = 0;
  ipm_mvif_t	*ipm_vifp = 0;
  int32			rc = NFX_OK;
  int			nameLen, i;
  int			myCount=0;

  /* calculate msg size */
  bufsize = sizeof(ipm_mhdr_t) + sizeof(ipm_mvif_t);
  
  /* allocate msg buffer */
  IPM_ALLOC_NCM(ipm_hdrp, bufsize);
  if (ipm_hdrp == NULL) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* fill in ipm hdr */
  if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, bufsize)
	  != NFX_OK) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* fill in vif info */
  ipm_vifp = (ipm_mvif_t*)(ipm_hdrp + 1);
  bzero(ipm_vifp, sizeof(ipm_mvif_t));
  if ((nameLen = strlen(vifName)) > IPM_STR_SIZE) {
	  rc = NFX_ERR;
	  goto out;
  }
  
  strcpy(ipm_vifp->vif_name, vifName);


  /* set prefered link */
  if (mtype == IPM_MSG_VIF_PREF) {
	  memcpy(ipm_vifp->vif_prefLink, links, IPM_STR_SIZE);
	  goto skip_link;
  }
  

  /* copy links, if any */
  if (count > IPM_MAXLINK_VIF) {
	  rc = IPM_CMDERR_MANYLINK_VIF;
	  goto out;
  }
  for (i=0; i < count; i++) {
	  int j, dup;
	  
	  /* make sure there is no duplicate */
	  dup = 0;
	  for (j=0; j<i; j++) {
		  if (strcmp(links, &ipm_vifp->vif_links[i][0]) == 0) {
			  dup =1;
			  break;
		  }
	  }
	  
	  if (!dup) {
		  strncpy(ipm_vifp->vif_links[i], links, IPM_STR_SIZE);
		  myCount ++;
	  }
	  
	  links += IPM_STR_SIZE;
  }
  ipm_vifp->vif_linkCnt = myCount;


skip_link:	  
  /* copy ip */
  ipm_vifp->vif_ip = ip;
  ipm_vifp->vif_mask = mask;

  /* copy flags */
  ipm_vifp->vif_flags = flags;

  /* hard copy slot to be 1 for now */
  ipm_vifp->vif_slot = 1;

  /* print cmd */
  //ipmd_PrintVif(ipm_vifp, mtype);

  /* 
   *send msg to ipmd 
   */
  sock = sock ? sock : -1;

  /* use the same same buffer for reply */
  rc = ipmd_SendMsgByNcm(sock, ipm_hdrp, bufsize, "ipm", 
						 node, ipm_hdrp, bufsize);
  
  /* send error or rely timeout */
  if (rc <= 0) {
	  rc = IPM_CMDERR_IPMTIMEOUT;
  }
  else {
	  IPM_PRINTF(("relay received\n"));
	  IPM_FLAG_TO_RC(ipm_hdrp->ipmh_flags, rc);
  }

out:
  if (ipm_hdrp)
	IPM_FREE_NCM(ipm_hdrp);

  return rc;
}
#endif /* LINK_AGG */

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ModifyIntf	
 *
 * Description: core function to enable/disable and add/delete
 *              a given interface across system.
 *
 * Created by:	Su Wang
 *
 * parameters:
 * int		mtype,		// cmd 
 * char		*ifname,	// interface name without node
 * uint32	ip,			// ip addr if applicable, may also as extra 
 *						   params for some messages
 * uint32	mask,		// ip mask if applicable
 * char		*app,		// appl. name, option
 * int		sock,		// send sock optional
 * char		*node		// destination node
 *
 * Date Created: 12/06/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* NFX_OK, NFX_ERR */
ipmd_ModifyIntf  (int		mtype,	
				  char		*ifname,
				  uint32	ip,
				  uint32	mask,
				  char		*app,	
				  int		sock,
				  char		*node)
{
  int			bufsize;
  ipm_mhdr_t	*ipm_hdrp = 0;
  ipm_mintf_t	*ipm_intfp = 0;
  ipm_mip_t		*ipm_ipp = 0;
  int32			rc = NFX_OK;

  /* calculate msg size */
  if (mtype == IPM_MSG_INTF_ENA || 
	  mtype == IPM_MSG_INTF_DIS ||
	  mtype == IPM_MSG_INTF_SETMTU || 
	  mtype == IPM_MSG_INTF_AUTOEN ||
	  mtype == IPM_MSG_INTF_AUTODIS)
	  /* msg body contains only ipm_mintf_t */
	  bufsize = sizeof(ipm_mhdr_t) + sizeof(ipm_mintf_t);
  else
	  /* msg body contains ipm_mip_t, ipm_mintf_t */
	  bufsize = sizeof(ipm_mhdr_t) + sizeof(ipm_mintf_t) + sizeof(ipm_mip_t);

  
  /* allocate msg buffer */
  IPM_ALLOC_NCM(ipm_hdrp, bufsize);
  if (ipm_hdrp == NULL) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* fill in ipm hdr */
  if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, bufsize)
	  != NFX_OK) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* fill in interface info */
  ipm_intfp = (ipm_mintf_t*)(ipm_hdrp + 1);
  if (ipmd_FillIpmMsgIntfId(ipm_intfp, ifname) != NFX_OK) {
	  rc = NFX_ERR;
	  goto out;
  }
  
  
  /*
   * msg dependent content 
   */
  /* fill in ip info */
  if (mtype == IPM_MSG_IP_ADD || mtype == IPM_MSG_IP_DEL) {
	  ipm_ipp = (ipm_mip_t*)(ipm_intfp + 1);
	  if (ipm_FillIpmMsgIp(ipm_ipp, ip, mask) != NFX_OK) {
		  rc = NFX_ERR;
		  goto out;
	  }
  }
  else if (mtype == IPM_MSG_INTF_SETMTU)
	  ipm_intfp->if_mtusize = (ushort)ip;

  /* 
   *send msg to ipmd 
   */
  sock = sock ? sock : -1;
  
  /* use the same same buffer for reply */
  rc = ipmd_SendMsgByNcm(sock, ipm_hdrp, bufsize, "ipm", 
						 node, ipm_hdrp, bufsize);
  
  /* send error or rely timeout */
  if (rc <= 0) {
	  rc = IPM_CMDERR_IPMTIMEOUT;
  }
  else {
	  IPM_PRINTF(("relay received\n"));
	  IPM_FLAG_TO_RC(ipm_hdrp->ipmh_flags, rc);
  }

out:
  if (ipm_hdrp)
	IPM_FREE_NCM(ipm_hdrp);

  return rc;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_EnableIntfCluster	
 *
 * Description: enable a given interface across system.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_EnableIntfCluster(char	*intf,	
					   char	*app,	
					   int	sock,
					   char	*node)
{
  return ipmd_ModifyIntf(IPM_MSG_INTF_ENA, intf, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DisableIntfCluster	
 *
 * Description: disable a given interface across system.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
 int32							/* NFX_OK, NFX_ERR */
ipmd_DisableIntfCluster(char	*intf,
						char	*app,	
						int		sock,
						char	*node)
{
  return ipmd_ModifyIntf(IPM_MSG_INTF_DIS, intf, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_EnableIntfAutoCluster	
 *
 * Description: enable gigE auto negociation.
 *
 * Created by:	Su Wang
 *
 * Date Created: 05/08/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_EnableIntfAutoCluster(char	*intf,
						   char	*app, 
						   int	sock, 
						   char	*node)
{
	return ipmd_ModifyIntf(IPM_MSG_INTF_AUTOEN, intf, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DisableIntfAutoCluster	
 *
 * Description: disable gigE auto negociation.
 *
 * Created by:	Su Wang
 *
 * Date Created: 05/08/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DisableIntfAutoCluster(char	*intf,
							char	*app, 
							int		sock, 
							char	*node)
{
	return ipmd_ModifyIntf(IPM_MSG_INTF_AUTODIS, intf, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_SetIntfMtuCluster	
 *
 * Description: set intf Mtu size
 *
 * Created by:	Su Wang
 *
 * Date Created: 2/28/2002
 *
 *-----------------------------------------------------------------
 */
 int32							/* NFX_OK, NFX_ERR */
ipmd_SetIntfMtuCluster (char	*intf,
						int		mtu,
						char	*app,	
						int		sock,
						char	*node)
{
  return ipmd_ModifyIntf(IPM_MSG_INTF_SETMTU, intf, mtu, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddIpCluster
 *
 * Description: add an ip to interface across the system.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_AddIpCluster(char*		intf,	
				  uint32	ip,
				  uint32	mask,	
				  char		*app,
				  int		sock,
				  char		*node)
{
#if defined(LINK_AGG)
	if (strncmp(intf, IPM_VIF_STR, 3) == 0) /* ip added to virtual interface */
		return ipmd_ModifyVif(IPM_MSG_VIP_ADD, intf, NULL, 0, 
							  ip, mask, 0, app, sock, node);
	else
#endif /* LINK_AGG */
		return ipmd_ModifyIntf(IPM_MSG_IP_ADD, intf, ip, mask, app, sock, node);
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DelIpCluster
 *
 * Description: delete an ip to interface across the system.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DelIpCluster( char*	intf,	
				   uint32	ip,
				   char		*app,
				   int		sock,
				   char		*node)
{
#if defined(LINK_AGG)
	if (strncmp(intf, IPM_VIF_STR, 3) == 0) /* ip added to virtual interface */
		return ipmd_ModifyVif(IPM_MSG_VIP_DEL, intf, NULL, 0, 
							  ip, 0, 0, app, sock, node);
	else
#endif /* LINK_AGG */
		return ipmd_ModifyIntf(IPM_MSG_IP_DEL, intf, ip, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ModifyRtTable	
 *
 * Description: core function to add/delete static rt and arp entries
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/06/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* NFX_OK, NFX_ERR */
ipmd_ModifyRtTable(int			mtype,
				   uint32		ip,
				   uint32		mask,
				   char			*gw,
				   char			*app,
				   int			sock,
				   char			*node )
{
  int			bufsize;
  ipm_mhdr_t	*ipm_hdrp = 0;
  ipm_mrt_t		*ipm_rtp = 0;
  ipm_marp_t	*ipm_arpp = 0;
  int			gwsize = 0;
  int32			rc = NFX_OK;

  /* calcute msg size */
  if (mtype == IPM_MSG_RT_ADD || mtype == IPM_MSG_RT_DEL)
	{
	  /* msg body contains ipm_mip_t */
		bufsize = sizeof(ipm_mhdr_t) + sizeof(ipm_mrt_t);
		gwsize = 4;				/* ip nexthop */
	}
  else
	{
	  /* msg body contains ipm_marp_t */
		bufsize = sizeof(ipm_mhdr_t) + sizeof(ipm_marp_t);
		gwsize = 6;				/* mac address */
	}
  
  /* allocate msg buffer */
  IPM_ALLOC_NCM(ipm_hdrp, bufsize);
  if (ipm_hdrp == NULL) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* fill in ipm hdr */
  if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, bufsize)
	  != NFX_OK) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* fill in rt or arp */
  if (mtype == IPM_MSG_RT_ADD || mtype == IPM_MSG_RT_DEL)	{
	  ipm_rtp = (ipm_mrt_t*)(ipm_hdrp + 1);
	  if (ipm_FillIpmMsgRt(ipm_rtp, ip, mask, *(uint32*)gw)
		  != NFX_OK) {
		  rc = NFX_ERR;
		  goto out;
	  }
  }
  else {						/* arp */
	  ipm_arpp = (ipm_marp_t*)(ipm_hdrp + 1);
	  if (ipm_FillIpmMsgArp(ipm_arpp, ip, gw, gwsize)
		  != NFX_OK) {
		  rc = NFX_ERR;
		  goto out;
	  }
  }

  /* 
   *send msg to ipmd 
   */
  sock = sock ? sock : -1;
  
  /* use the same same buffer for reply */

  IPM_PRINTF(("msg send\n"));
  //bsd_ShowPkt((char*)ipm_hdrp, ipm_hdrp->ipmh_size);  
  rc = ipmd_SendMsgByNcm(sock, ipm_hdrp, bufsize, "ipm", 
						 node, ipm_hdrp, bufsize);

  /* send error or rely timeout */
  if (rc <= 0)
	  rc = IPM_CMDERR_IPMTIMEOUT;
  else {
	  IPM_FLAG_TO_RC(ipm_hdrp->ipmh_flags, rc);
  }

out:
  if (ipm_hdrp)
	IPM_FREE_NCM(ipm_hdrp);

  return rc;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddRtCluster
 *
 * Description: add a static rt across system

 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_AddRtCluster( uint32		ip,	
				   uint32		mask,	
				   uint32	    nexthop,
				   char			*app,
				   int			sock,
				   char			*node)
{
	return ipmd_ModifyRtTable(IPM_MSG_RT_ADD, ip, mask, 
							  (char*)&nexthop, app, sock, node); 
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DelRtCluster
 *
 * Description: delete a static rt across system

 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DelRtCluster( uint32		ip,	
				   uint32		mask,	
				   uint32	    nexthop,
				   char			*app,
				   int			sock,
				   char			*node)
{
	return ipmd_ModifyRtTable(IPM_MSG_RT_DEL, ip, mask, 
							  (char*)&nexthop, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddArpCluster
 *
 * Description: add a static arp across system

 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_AddArpCluster(uint32		ip,	
				   char	    *mac,
				   char		*app,
				   int		sock,
				   char		*node)
{
	return ipmd_ModifyRtTable(IPM_MSG_ARP_ADD, ip, 0, 
							  mac, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DelArpCluster
 *
 * Description: delete a static arp across system

 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DelArpCluster(uint32	ip,	
				   char		*app,
				   int		sock,
				   char		*node)
{
	return ipmd_ModifyRtTable(IPM_MSG_ARP_DEL, ip, 0, 
							  0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_SendHdr()
 *
 * Description: core function to send IPM hdr to ipmd
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
static int32					
ipmd_SendHdr(int mtype, char *app, char *node)
{
  ipm_mhdr_t	*ipm_hdrp = 0;
  int32			rc = NFX_OK;
  int			bufsize;


  bufsize = sizeof(ipm_mhdr_t);
  
  /* allocate msg buffer */
  IPM_ALLOC_NCM(ipm_hdrp, bufsize);
  if (ipm_hdrp == NULL) {
	  rc = NFX_ERR;
	  goto out;
  }
  
  /* fill in ipm header */
  if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, sizeof(ipm_mhdr_t))
	  != NFX_OK) {
	  rc = NFX_ERR;
	  goto out;
  }

  /* use the same same buffer for reply */
  rc = ipmd_SendMsgByNcm(-1, ipm_hdrp, bufsize, "ipm", 
						 node, ipm_hdrp, bufsize);

  /* send error or rely timeout */
  if (rc <= 0)
	  rc = IPM_CMDERR_IPMTIMEOUT;
  else {
	  IPM_FLAG_TO_RC(ipm_hdrp->ipmh_flags, rc);
  }

out:
  if (ipm_hdrp)
	IPM_FREE_NCM(ipm_hdrp);

  return rc; 
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ShutDownCluster()
 *
 * Description: This function can be used by application to shutdown
 *				IPMD
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32					
ipmd_ShutDownCluster(char *app, char* node)
{
  return ipmd_SendHdr(IPM_MSG_SHUTDOWN, app, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_RestartCluster()
 *
 * Description: This function can be used by application to restart
 *				IPMD
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
int32					
ipmd_RestartCluster(char *app, char* node)
{
  return ipmd_SendHdr(IPM_MSG_RESTART, app, node);
}


/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetStatsCore
 *
 * Description: core function to retrive stats from ipmd
 *
 * parameters:
 * int				mtype,	// cmd
 * ipm_mstat_req_t	*reqp,	// requst structure
 * int				recsize,// struct size of stats
 * void				*buf,	// copy buff
 * size				*bufsize, // copy buff size
 * char				*node,	// which node to send request
 * int				*count, // num. of record copyed
 * int				*need	// bytes needed
 * 
 * Created by:	Su Wang
 *
 * Date Created: 02/14/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetStatsCore(int				mtype,
				  ipm_mstat_req_t	*reqp,
				  int				recsize,
				  void				*buf,
				  int				bufsize,
				  char				*node,
				  int				*count,
				  int				*need)
{
	static uint64	resp_buf[IPM_SSC_BUFSIZE];
	ipm_mhdr_t		*ipm_hdrp, *ripm_hdrp;
	int				msgsize, resp_bufsize;
	ipm_mstat_req_t *resp_stat_reqp;
	int32				rc, ret;
	
	/* calculate msg size */
	msgsize = sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);

	/* allocate msg buffer */
	IPM_ALLOC_NCM(ipm_hdrp, msgsize);
	if (ipm_hdrp == NULL) {
		rc = IPM_GET_ERR;
		goto out;
	}
	
	/* fill in ipm hdr */
	if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, msgsize)
		!= NFX_OK) {
		rc = IPM_GET_ERR;
		goto out;
	}

	/* fill in request */
	memcpy((ipm_hdrp+1), reqp, sizeof(ipm_mstat_req_t));
	
	
	/* send request */
	ripm_hdrp = (ipm_mhdr_t*)(((char*)resp_buf) + IPM_NCM_HDR);
	resp_bufsize = IPM_SSC_BUFSIZE*sizeof(uint64) - IPM_NCM_HDR;
	rc = ipmd_SendMsgByNcm(-1, ipm_hdrp, msgsize, "ipm", 
						   node, ripm_hdrp, resp_bufsize);

	/* 
	 * process reply 
	 */
	if (rc <= 0)	{			/* no response */
		rc = IPM_CMDERR_IPMTIMEOUT;
		goto out;
	}
	
	/* size matchs */
	if (rc != ripm_hdrp->ipmh_size) {
		IPM_PRINTF(("incorrect msg size %d, rc =%d\n", 
					ripm_hdrp->ipmh_size, rc));
		rc = IPM_GET_ERR;
		goto out;
	}
	

	/* calculate buffer size without headers */
	resp_stat_reqp = (ipm_mstat_req_t*)(ripm_hdrp+1);
	rc -= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);

	/* stats successfully returned */
	IPM_FLAG_TO_RC(ripm_hdrp->ipmh_flags, ret);
	if (ret != IPM_GET_OK) {
		rc = ret;
		IPM_PRINTF(("bad rc %d\n", rc));
		if (rc == IPM_GET_NO_BUF) {
			IPM_PRINTF(("count is %d, resize is %d\n", resp_stat_reqp->count, recsize));
			*need = resp_stat_reqp->count*recsize;
		}
		goto out;
	}
		
	
	
	/* num. of record matches buf size */
	/* IPM_MSG_VIFTBL_REQ can have different record
	 * ipm_vif_t and ipm_if_t
	 */
	if ((mtype != IPM_MSG_VIFTBL_REQ) &&
		(rc/recsize) != resp_stat_reqp->count) {
		IPM_PRINTF(("mtype = %d, base= %d, rc = %d, recsize=%d, count %d\n", 
				   mtype, IPMD_MSG_BASE, rc, recsize, resp_stat_reqp->count));
		rc = IPM_GET_ERR;
		goto out;
	}	
		
	/* input buffer large enough */
	if (rc > bufsize) {
		*need = rc;
		rc = IPM_CMDERR_APPNOBUF;
		goto out;
	}	

	
	
	/* now we can copy to input buffer */
	memcpy(buf, (resp_stat_reqp+1), rc);
	*count = resp_stat_reqp->count;
	IPM_PRINTF(("count is %d, resize is %d\n", resp_stat_reqp->count, recsize));
	rc = IPM_GET_OK;
	
out:
	if (ipm_hdrp)
		IPM_FREE_NCM(ipm_hdrp);

	return rc;	
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ValidateIpmMem	
 *
 * Description: validate ipm memory, debug function.
 *
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/17/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_ValidateIpmMemCluster(char	*node)
{
	static char		buf[2048];
	ipm_mstat_req_t	req;
	int				size, count, needed;
	int				rc;
	ipm_root_t		*root;
	ipm_card_t		*card;
	ipm_stats_t		*stats;
	int				i;
	
	size = 256 + sizeof(ipm_root_t)+
		NFX_NUM_SLOTS_PER_CHASSIS*sizeof(ipm_card_t) + 
		sizeof(ipm_stats_t);
	
	
	rc = ipmd_GetStatsCore(IPM_MSG_ROOT_REQ, &req,
						   size, buf, 2048, node, &count, &needed);

	if (rc != IPM_GET_OK)
		return NFX_ERR;
	
	printf("%s\n", buf);
	root = (ipm_root_t*)(buf + 256);
	card = (ipm_card_t*)(root + 1);
	stats = (ipm_stats_t*)(card + NFX_NUM_SLOTS_PER_CHASSIS);
 
	printf("size of structures\n");
	printf("root:			%d\n", sizeof(ipm_root_t));
	printf("card:			%d\n", sizeof(ipm_card_t));
	printf("ifstat  :		%d\n", sizeof(ipm_ifstat_t));
	printf("ipstat  :		%d\n", sizeof(ipm_ipstat_t));		
	printf("icmpstat:		%d\n", sizeof(ipm_icpstat_t));
	printf("tcpstat :		%d\n", sizeof(ipm_tcpstat_t));
	printf("udpstat :		%d\n", sizeof(ipm_udpstat_t));
	printf("conn    :		%d\n", sizeof(ipm_inconn_t));
	printf("\n");

	printf("card array:%x\n", (uint32)root->cardp);
	printf("fconn     :%x\n", (uint32)root->freeconnp);
	printf("tcpconn   :%x\n", (uint32)root->tcpconnp);
	printf("udpconn   :%x\n", (uint32)root->udpconnp);
	printf("tcp conn #:%d\n", root->tcpcnt);
	printf("udp conn #:%d\n", root->udpcnt);
	printf("free conn#:%d\n", root->freeconncnt);
	printf("\n\n");

	printf("stats ptr per card \n");
	printf("--------------------------------------------------------\n");
	printf("card	ifcnt	ifstatp	ipstatp	icmpsp	tcpstp	udpstp	\n");
	printf("--------------------------------------------------------\n");
	for (i=0; i<NFX_NUM_SLOTS_PER_CHASSIS; i++) {
			printf("%-8x%-8x%-8x%-8x%-8x%-8x%-8x\n",
				   card->card,
				   card->ifcnt,
				   (uint32)card->ifstatp[0],
				   (uint32)card->ipstatp,
				   (uint32)card->icmpstatp,
				   (uint32)card->tcpstatp,
				   (uint32)card->udpstatp);
			card++;
	}	

	ipm_PrintStats(stats);
	return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetIpByNameCluster
 *
 * Description: get IP interface associated with a device. 
 *				if device is NULL, cpy all ip interface into
 *				buff in consecutive ipm_if_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR,IPM_GET_NO_BUF, IPM_GET_OK */
ipmd_GetIpByNameCluster(char	*ifname,
						ipm_ip_t *buf, 
						int		bufsize,
						int		*count,
						int		*needed,	
						char	*node)
{
	ipm_mstat_req_t	req;
	
	if (ifname)
		strcpy(req.req.ifname, ifname);
	else
		bzero(req.req.ifname, IPM_STR_SIZE);
	
	return ipmd_GetStatsCore(IPM_MSG_IPTBL_REQ, &req, 
							 sizeof(ipm_ip_t), (void*)buf, 
							 bufsize, node, count, needed);	
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetIfByNameCluster
 *
 * Description: get device if configuration, if device = NULL,
 *				all device interface are copied
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR, IPM_GET_OK, IPM_GET_NO_BUF */
ipmd_GetIfByNameCluster(char	*ifname,
						ipm_if_t *buf,
						int		bufsize,
						int		*count,
						int		*needed,
						char	*node)
{
	ipm_mstat_req_t	req;
	
	if (ifname)
		strcpy(req.req.ifname, ifname);
	else
		bzero(req.req.ifname, IPM_STR_SIZE);
	
	return ipmd_GetStatsCore(IPM_MSG_IFTBL_REQ, &req, 
							 sizeof(ipm_if_t), (void*)buf, 
							 bufsize, node, count, needed);		
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetL2ConnEntCluster
 *
 * Description: Get entry of given IP from Lay2 connection table. 
 *				count indicates number of successful retrivals.
 *
 * Created by:	Su Wang
 *
 * Date Created: 5/28/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR, IPM_GET_OK, IPM_GET_NO_BUF */
ipmd_GetL2ConnEntCluster(uint32		ip,
						 ipm_arp_t	*buf,
						 int		bufsize,
						 int		*count,
						 int		*needed,
						 char		*node)
{
	ipm_mstat_req_t	req;
	

	req.req.ip = ip;
	
	return ipmd_GetStatsCore(IPM_MSG_L2CONN_ENT_REQ, &req, 
							 sizeof(ipm_arp_t), (void*)buf, 
							 bufsize, node, count, needed);		
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetRtTblCluster
 *
 * Description: get entire route table in buf. 
 *
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetRtTblCluster(ipm_rt_t	*buf,	/* buff to cpy to, */
					 int		bufsize,/* size of buff */
					 int		*count,	/* how many being cpyed*/
					 int		*needed,
					 char		*node)
{
	ipm_mstat_req_t	req;
	
	return ipmd_GetStatsCore(IPM_MSG_RTTBL_REQ, &req, 
							 sizeof(ipm_rt_t), (void*)buf, 
							 bufsize, node, count, needed);		
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetArpTblCluster
 *
 * Description: get entire ARP table in buf. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_NO_BUF */
ipmd_GetArpTblCluster(ipm_arp_t *buf,
					  int		bufsize,
					  int		*count,
					  int		*needed,
					  char		*node)
{
	ipm_mstat_req_t	req;
	
	return ipmd_GetStatsCore(IPM_MSG_ARPTBL_REQ, &req, 
							 sizeof(ipm_arp_t), (void*)buf, 
							 bufsize, node, count, needed);		
}


/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetIfStatsCluster
 *
 * Description: get stats associated with a device. 
 *				if device is NULL, cpy all ifstats in
 *				buff in consecutive ipm_ifstat_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/12/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetIfStatsCluster(char*		ifname,	
					   ipm_ifstat_t	*ifstat,
					   int			bufsize, 
					   int			*count,	
					   int			*needed,
					   char			*node)	
{
	ipm_mstat_req_t	req;
	
	if (ifname)
		strcpy(req.req.ifname, ifname);
	else
		bzero(req.req.ifname, IPM_STR_SIZE);
	
	return ipmd_GetStatsCore(IPM_MSG_INTF_STATS_REQ, &req, 
							 sizeof(ipm_ifstat_t), (void*)ifstat, 
							 bufsize, node, count, needed);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetIpStatsCluster
 *
 * Description: get ip stats  associated with a card. 
 *				if card is -1, cpy ip stats of all cards in
 *				buff in consecutive ipm_ipstat_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetIpStatsCluster(int32			card,	
					   ipm_ipstat_t		*ipstat, 
					   int				bufsize,
					   int				*count,	
					   int				*needed,
					   char				*node)
{
	ipm_mstat_req_t	req;
	
	req.req.slot = card;
	
	return ipmd_GetStatsCore(IPM_MSG_IP_STATS_REQ, &req, 
							 sizeof(ipm_ipstat_t),
							 (void*)ipstat, bufsize,
							 node, count, needed);	
}


/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetIcmpStatsCluster
 *
 * Description: get icmp stats  associated with a card. 
 *				if card is -1, cpy ip stats of all cards in
 *				buff in consecutive ipm_icpstat_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetIcmpStatsCluster(int32				card,	
						 ipm_icpstat_t		*icmpstat, 
						 int				bufsize,
						 int				*count,	
						 int			    *needed,
						 char				*node)
{
	ipm_mstat_req_t	req;
	
	req.req.slot = card;

	return ipmd_GetStatsCore(IPM_MSG_ICMP_STATS_REQ, &req, 
							 sizeof(ipm_icpstat_t),
							 (void*)icmpstat, bufsize,
							 node, count, needed);		
}


/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetTcpStatsCluster
 *
 * Description: get Tcp stats  associated with a card. 
 *				if card is -1, cpy ip stats of all cards in
 *				buff in consecutive ipm_tcpstat_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetTcpStatsCluster(int32			card,	
						ipm_tcpstat_t	*tcpstat, 
						int				bufsize,
						int				*count,
						int				*needed,
						char			*node)
{
	ipm_mstat_req_t	req;
	
	req.req.slot = card;

	return ipmd_GetStatsCore(IPM_MSG_TCP_STATS_REQ, &req, 
							 sizeof(ipm_tcpstat_t),
							 (void*)tcpstat, bufsize,
							 node, count, needed);		
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetUdpStatsCluster
 *
 * Description: get Udp stats  associated with a card. 
 *			    if card is -1, cpy ip stats of all cards in
 *				buff in consecutive ipm_udpstat_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetUdpStatsCluster(int32			card,	
						ipm_udpstat_t	*udpstat, 
						int				bufsize,
						int				*count,
						int				*needed,
						char			*node)
{
	ipm_mstat_req_t	req;
	
	req.req.slot = card;

	return ipmd_GetStatsCore(IPM_MSG_UDP_STATS_REQ, &req, 
							 sizeof(ipm_udpstat_t),
							 (void*)udpstat, bufsize,
							 node, count, needed);	
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetTcpConnTableCluster
 *
 * Description: get system wide TCP connection table.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetTcpConnTableCluster(ipm_conn_t		*tcpconn,
							int				bufsize,
							int				index, 
							int				incount,
							int				*count,
							int				*needed,
							char			*node)
{
	ipm_mstat_req_t	req;
	
	req.req.conn.conn_lAddr = (uint32)index;
	req.req.conn.conn_rAddr = (uint32)incount;
	
	return ipmd_GetStatsCore(IPM_MSG_TCP_CTBL_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)tcpconn, bufsize,
							 node, count, needed);	
	
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetUdpConnTableCluster
 *
 * Description: get system wide UDP connection table.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetUdpConnTableCluster(ipm_conn_t	*udpconn, 
							int			bufsize,
							int			index,
							int			incount,
							int			*count,
							int			*needed,
							char		*node)
{
	ipm_mstat_req_t	req;
	
	req.req.conn.conn_lAddr = (uint32)index;
	req.req.conn.conn_rAddr = (uint32)incount;
	
	return ipmd_GetStatsCore(IPM_MSG_UDP_CTBL_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)udpconn, bufsize,
							 node, count, needed);	
	
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetTcpConnCluster
 *
 * Description: find a TCP connection in record.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR, IPM_GET_OK */
ipmd_GetTcpConnCluster(ipm_conn_t	*connp,
					   ipm_conn_t	*result,
					   char			*node)
{
	int				needed =0, count=0;
	ipm_mstat_req_t	req;
	
	memcpy(&(req.req.conn), connp, sizeof(ipm_conn_t));
	
	return ipmd_GetStatsCore(IPM_MSG_TCP_CONN_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)result, sizeof(ipm_conn_t),
							 node, &count, &needed);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetUdpConnCluster
 *
 * Description: find a UDP connection in record.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR*/
ipmd_GetUdpConnCluster(ipm_conn_t	*connp,
					   ipm_conn_t	*result,
					   char			*node)
{
	ipm_mstat_req_t	req;
	int				needed =0, count=0;

	memcpy(&(req.req.conn), connp, sizeof(ipm_conn_t));
	
	return ipmd_GetStatsCore(IPM_MSG_UDP_CONN_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)result, sizeof(ipm_conn_t),
							 node, &count, &needed);	
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetTcpNxtConnCluster
 *
 * Description: find next TCP connection in record.
 *
 * parameters:
 * ipm_conn_t	*connp,	 // used to search, NULL returns first record
 * ipm_conn_t	*nextp, // record copied here if found
 * char			*node	// node if interest
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR */
ipmd_GetTcpNxtConnCluster(ipm_conn_t	*connp, 
						  ipm_conn_t	*nextp,
						  char			*node)
{
	int				needed =0, count=0;
	ipm_mstat_req_t	req;
	
	if (connp) {
		req.count = 0;
		memcpy(&(req.req.conn), connp, sizeof(ipm_conn_t));
	}
	else
		req.count = -1;			/* indicate no ptr given */
		
	return ipmd_GetStatsCore(IPM_MSG_TCP_NXTC_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)nextp, sizeof(ipm_conn_t),
							 node, &count, &needed);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetUdpNxtConnCluster
 *
 * Description: find next UDP connection in record.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR */
ipmd_GetUdpNxtConnCluster(ipm_conn_t	*connp,
						  ipm_conn_t	*nextp,
						  char			*node)
{
	ipm_mstat_req_t	req;
	int				needed =0, count=0;

	if (connp) {
		req.count = 0;
		memcpy(&(req.req.conn), connp, sizeof(ipm_conn_t));
	}
	else
		req.count = -1;			/* indicate no ptr given */
	
	return ipmd_GetStatsCore(IPM_MSG_UDP_NXTC_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)nextp, sizeof(ipm_conn_t),
							 node, &count, &needed);	
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetTcpConnSSCCluster
 *
 * Description: get tcp conn table on ssc.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/11/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetTcpConnSSCCluster(ipm_conn_t	*tcpconn,
						  int			bufsize,
						  int			*count,
						  int			*needed,
						  char			*node)
{
	ipm_mstat_req_t	req;

	
	return ipmd_GetStatsCore(IPM_MSG_TCP_KCTBL_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)tcpconn, bufsize,
							 node, count, needed);		
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetUdpConnSSCCluster
 *
 * Description: get udp conn table on ssc.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/11/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetUdpConnSSCCluster(ipm_conn_t	*udpconn,
						  int			bufsize,
						  int			*count,	
						  int			*needed,
						  char			*node)
{
	ipm_mstat_req_t	req;

	
	return ipmd_GetStatsCore(IPM_MSG_UDP_KCTBL_REQ, &req, 
							 sizeof(ipm_conn_t),
							 (void*)udpconn, bufsize,
							 node, count, needed);	
}



/*-----------------------------------------------------------------
 *
 * Name: ipmd_disableIntfOnCardCluster
 *
 * Description: disable all interface on a card
 *
 * Created by:	Su Wang
 *
 * Date Created: 03/20/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DisableIntfOnCardCluster(int	card,
							  char	*node)
{
	ipm_if_t	*ipm_ifp = 0;
	int			count, needed, size;
	int			rc, i, ret=NFX_OK;
	

	size = sizeof(ipm_if_t)*24;
	
	ipm_ifp = malloc(size);
	
	rc = ipmd_GetIfByNameCluster(NULL, ipm_ifp, size, 
								 &count, &needed, node);

	if (rc != IPM_GET_OK) {
		ret = NFX_ERR;
		goto out;
	}
	
	for (i=0; i< count; i++, ipm_ifp++) {
		if (ipm_ifp->if_slot != card)
			continue;

		rc = ipmd_DisableIntfCluster(ipm_ifp->if_name,
									 NULL, 0, node);
		if (rc != NFX_OK) {
			ret = NFX_ERR;
			break;
		}
	}

out:
	if (ipm_ifp)
		free(ipm_ifp);
	
	return ret;
}


/*-----------------------------------------------------------------
 *
 * Name: ipmd_EnableIntfOnCardCluster
 *
 * Description: enable all interface on a card
 *
 * Created by:	Su Wang
 *
 * Date Created: 03/20/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_EnableIntfOnCardCluster(int	card,
							 char	*node)
{
	ipm_if_t	*ipm_ifp = 0;
	int			count, needed, size;
	int			rc, i, ret=NFX_OK;
	

	size = sizeof(ipm_if_t)*24;
	
	ipm_ifp = malloc(size);
	
	rc = ipmd_GetIfByNameCluster(NULL, ipm_ifp, size, 
								 &count, &needed, node);

	if (rc != IPM_GET_OK) {
		ret = NFX_ERR;
		goto out;
	}
	
	for (i=0; i< count; i++, ipm_ifp++) {
		if (ipm_ifp->if_slot != card)
			continue;

		rc = ipmd_EnableIntfCluster(ipm_ifp->if_name,
									 NULL, 0, node);
		if (rc != NFX_OK)
			return NFX_ERR;
		
	}

out:
	if (ipm_ifp)
		free(ipm_ifp);
	
	return ret;
}

#if defined(LINK_AGG)
/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_CreateVifCluster
 *
 * Description: create virtual interface
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, or ERRs */
ipmd_CreateVifCluster (char		*vif, /* interface to be created */
					   char		*link, /* link contained in vif */
					   int		count,	/* num. of links */
					   uint32	flags, /* what mode */
					   char		*app,	
					   int		sock,
					   char		*node)
{
	int rc;

	/* we require link to be specified */
	if (!count)
		return IPM_CMDERR_NOLINK_VIF;
	
	rc =  ipmd_ModifyVif(IPM_MSG_VIF_CREAT, vif, link,
						  count, 0, 0, flags, app, sock, node);
	if (rc != NFX_OK)
		return rc;
	
	if ((flags & IPM_VIF_MODE_FO))
		rc = ipmd_ModifyVif(IPM_MSG_VIF_PREF, vif, link,
						  1, 0, 0, 0, app, sock, node);
	return rc;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DeleteVifCluster
 *
 * Description: delete virtual interface
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, or ERRs */
ipmd_DeleteVifCluster (char		*vif, /* interface to be created */
					   char		*app,	
					   int		sock,
					   char		*node)
{
	return ipmd_ModifyVif(IPM_MSG_VIF_DEL, vif, NULL,
						  0, 0, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_PrefVifCluster
 *
 * Description: setup prefered link within a application
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, or ERRs */
ipmd_PrefVifCluster (char		*vif, /* interface to be created */
					char		*link,
					char		*app,	
					int			sock,
					char		*node)
{
	return ipmd_ModifyVif(IPM_MSG_VIF_PREF, vif, link,
						  1, 0, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddLinkVifCluster
 *
 * Description: add link(s) to existing virtual interface
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, or ERRs */
ipmd_AddLinkVifCluster(char		*vif, /* interface to be created */
					   char		*links, /* link contained in vif */
					   int		count,	/* num. of links */
					   char		*app,	
					   int		sock,
					   char		*node)
{
	return ipmd_ModifyVif(IPM_MSG_VIF_ADDLNK, vif, links,
						  count, 0, 0, 0, app, sock, node);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_RmLinkVifCluster
 *
 * Description: remove link(s) to existing virtual interface
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, or ERRs */
ipmd_RmLinkVifCluster(char		*vif, /* interface to be created */
					  char		*links, /* link contained in vif */
					  int		count,	/* num. of links */
					  char		*app,	
					  int		sock,
					  char		*node)
{
	return ipmd_ModifyVif(IPM_MSG_VIF_RMLNK, vif, links,
						  count, 0, 0, 0, app, sock, node);	
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetVifByNameCluster
 *
 * Description: get virtual interface info
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/07/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, or ERRs */
ipmd_GetVifByNameCluster(char		*vifName, /* NULL to get all */
						 ipm_vif_t	*buf,
						 int		bufsize,
						 int		*count,
						 int		*need,
						 char		*node)
{
	
	ipm_mstat_req_t	req;
	int				nameLen;
	
	if (vifName && ((nameLen = strlen(vifName)) > IPM_STR_SIZE))
		return NFX_ERR;
	
	if (vifName)
		strcpy(req.req.ifname, vifName);
	else
		bzero(req.req.ifname, IPM_STR_SIZE);
	
	return ipmd_GetStatsCore(IPM_MSG_VIFTBL_REQ, &req, 
							 sizeof(ipm_vif_t), (void*)buf, 
							 bufsize, node, count, need);	
	
}

#endif /* LINK_AGG */
