/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_fcadmin.c
 *
 * RCS:		$Id: ispfc_fcadmin.c,v 1.19 2002/10/10 22:36:32 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver diagnostic and debug interface code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/19/01
 *
 *------------------------------------------------------------------------------
 */

#include "ispfc_private.h"
#include "ispfc_mbox.h"
#include "ispfc_ct.h"
#include "ispfc_device.h"
#include "ispfc_misc.h"

#ifdef ISPFC_SIM
int    ispfc_fcadmin_disk_count = 2;
#if (ISPFC_SIM == ISPFC_SIM_LINUX_FILE)
int    ispfc_fcadmin_tape_count = 2;
#else
int    ispfc_fcadmin_tape_count = 0;
#endif
#if 0
wwn_t  ispfc_write_protected_device = { 0x20000000000202efLL };
#else
wwn_t  ispfc_write_protected_device = { 0x0000000000000000LL };
#endif
#endif

int32
ispfc_fcadmin(int32 argc, char **argv)
{
	hal_dispatch_entry_t *	hal_dt = hal_dt_entry_get();
	ispfc_cb_t *		cb = hal_dt->hal_handle;

	if (argc == 1) {
		P("Usage : \n");
		P(" %s                    - dump this usage info\n", argv[0]);
		P(" %s mode_normal        - return to normal mode\n", argv[0]);
		P(" %s mode_diag_sram     - run SRAM diag\n", argv[0]);
		P(" %s mode_diag_elback   - run external loopback diag\n", argv[0]);
		P(" %s mode_diag_ilback   - run internal loopback diag\n", argv[0]);
		P(" %s adapter_info       - show adapter info\n", argv[0]);
		P(" %s dump_regs          - dump ISP2300 registers\n", argv[0]);
		P(" %s dump_pio_trace <n> - dump ISP2300 register access trace\n", argv[0]);
#ifdef ISPFC_SIM
		P(" %s discover <n> [<m>] - discover <n> disk devices and <m> tape devices.\n", argv[0]);
		P(" %s discover_all       - discover all available devices\n", argv[0]);
#endif
		P(" %s dump_devs          - dump device state\n", argv[0]);
		P(" %s dump_channels      - dump channel info\n", argv[0]);
		P(" %s reset_link         - reset link and adapter\n", argv[0]);
		P(" %s dbg_level <n>      - set driver verbosity/debug level\n", argv[0]);
		return(NFX_OK);
	}

	if (strcmp(argv[1], "mode_normal") == 0) {
		cb->link_state &= ~ISPFC_CLS_MODE_MASK;
		cb->link_state |= ISPFC_CLS_MODE_RUNTIME;
	}
	else
	if (strcmp(argv[1], "mode_diag_sram") == 0) {
		if ((cb->link_state & ISPFC_CLS_MODE_MASK) != ISPFC_CLS_MODE_RUNTIME) {
			printf("************************************************************\n");
			printf("WARNING: Another diag already running.  Return to normal    \n");
			printf("  first by using the command 'fcadmin mode_normal'.         \n");
			printf("************************************************************\n");
		}
		else 
		if (cb->link_state & ISPFC_CLF_FC_OFFLINE_LATCH) {
			printf("************************************************************\n");
			printf("WARNING: Link disabled; reenabled before running diag.      \n");
			printf("************************************************************\n");
		}
		else {
			cb->link_state &= ~ISPFC_CLS_MODE_MASK;
			cb->link_state |= ISPFC_CLS_MODE_DIAG_SRAM;
			SET_LINK_STATE(cb, ISPFC_CLS_STATE_RESET_REQUESTED);
		}
	}
	else
	if (strcmp(argv[1], "mode_diag_elback") == 0) {
		if ((cb->link_state & ISPFC_CLS_MODE_MASK) != ISPFC_CLS_MODE_RUNTIME) {
			printf("************************************************************\n");
			printf("WARNING: Another diag already running.  Return to normal    \n");
			printf("  first by using the command 'fcadmin mode_normal'.         \n");
			printf("************************************************************\n");
		}
		else 
		if (cb->link_state & ISPFC_CLF_FC_OFFLINE_LATCH) {
			printf("************************************************************\n");
			printf("WARNING: Link disabled; reenabled before running diag.      \n");
			printf("************************************************************\n");
		}
		else {
			cb->link_state &= ~ISPFC_CLS_MODE_MASK;
			cb->link_state |= ISPFC_CLS_MODE_DIAG_LOOPBACK_EXTERNAL;
			SET_LINK_STATE(cb, ISPFC_CLS_STATE_RESET_REQUESTED);
		}
	}
	else
	if (strcmp(argv[1], "mode_diag_ilback") == 0) {
		if ((cb->link_state & ISPFC_CLS_MODE_MASK) != ISPFC_CLS_MODE_RUNTIME) {
			printf("************************************************************\n");
			printf("WARNING: Another diag already running.  Return to normal    \n");
			printf("  first by using the command 'fcadmin mode_normal'.         \n");
			printf("************************************************************\n");
		}
		else 
		if (cb->link_state & ISPFC_CLF_FC_OFFLINE_LATCH) {
			printf("************************************************************\n");
			printf("WARNING: Link disabled; reenabled before running diag.      \n");
			printf("************************************************************\n");
		}
		else {
			cb->link_state &= ~ISPFC_CLS_MODE_MASK;
			cb->link_state |= ISPFC_CLS_MODE_DIAG_LOOPBACK_INTERNAL;
			SET_LINK_STATE(cb, ISPFC_CLS_STATE_RESET_REQUESTED);
		}
	}
	else
	if (strcmp(argv[1], "adapter_info") == 0) {
		hal_adapter_info_t	    info;
		int			    rc;

		rc = (*hal_dt->hal_ioctl)(HAL_IOCTL_COMMON_GET_ADAPTER_INFO, &info);
		if (rc == NFX_OK) {
			P("adapter name                 : %s\n", info.name);
			P("firmware revision            : %s\n", info.firmware_rev);
			P("hardware revision            : %s\n", info.hardware_rev);
			P("interface type               : ");
			switch(info.interface_type) {
			case HAL_IF_FC_L_PORT:	    P("L_PORT  "); break;
			case HAL_IF_FC_NL_PORT:	    P("NL_PORT "); break;
			case HAL_IF_FC_N_PORT:	    P("N_PORT  "); break;
			case HAL_IF_FC_F_PORT:	    P("F_PORT  "); break;
			case HAL_IF_FC_FL_PORT:	    P("FL_PORT "); break;
			default:		    P("???");
			}
			P("\n");
			P("link speed                   : %d Gbit\n", info.u.fc_data.link_speed);
			P("Max. SCSI tags per device    : %d\n", info.max_tags_per_device);
			P("Fibrechannel nodename        : 0x%llx\n", (info.u.fc_data.fc_node_name).u.wwn);
			P("Max. Fibrechannel frame size : %d\n", info.u.fc_data.fc_packet_size);
			P("SRAM parity available        : %s\n", info.u.fc_data.sram_parity ? "YES" : "NO");
			P("Link state                   : 0x%x\n", cb->link_state);
		}
	}
	else
	if (strcmp(argv[1], "dump_regs") == 0) {
		ispfc_dump_registers(cb);
	}
	else
	if (strcmp(argv[1], "dump_pio_trace") == 0) {
		ispfc_dump_pio_trace(cb, atoi(argv[2]));
	}
#ifdef ISPFC_SIM
	else
	if (strcmp(argv[1], "discover") == 0) {
		ispfc_fcadmin_disk_count = atoi(argv[2]);
		if (argc > 3)
			ispfc_fcadmin_tape_count = atoi(argv[3]);
		if (LINK_STATE(cb) == ISPFC_CLS_STATE_UP)
			SET_LINK_STATE(cb, ISPFC_CLS_STATE_LIP_OCCURRED);
	}
	else
	if (strcmp(argv[1], "discover_all") == 0) {
		ispfc_fcadmin_disk_count = ISPFC_SIM_MAX_DISKS;
		ispfc_fcadmin_tape_count = ISPFC_SIM_MAX_TAPES;
		if (LINK_STATE(cb) == ISPFC_CLS_STATE_UP)
			SET_LINK_STATE(cb, ISPFC_CLS_STATE_LIP_OCCURRED);
	}
#endif
	else
	if (strcmp(argv[1], "dump_devs") == 0) {
		ispfc_dump_devs(cb);
	}
	else
	if (strcmp(argv[1], "dump_channels") == 0) {
		ispfc_channels_dump();
	}
	else
	if (strcmp(argv[1], "reset_link") == 0) {
		if (LINK_STATE(cb) == ISPFC_CLS_STATE_UP) 
			SET_LINK_STATE(cb, ISPFC_CLS_STATE_RESET_REQUESTED);
	}
	else
	if (strcmp(argv[1], "dbg_level") == 0) {
		ispfc_dbg_level = atoi(argv[2]);
	}

	return(NFX_OK);
}
