/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_iocb.h
 *
 * RCS:		$Id: ispfc_iocb.h,v 1.3 2002/08/08 00:43:48 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver IOCB definitions
 *
 * Created by:	Radek Aster
 *
 * Date Created: 5/31/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_IOCB_H__
#define __ISPFC_IOCB_H__

/* IOCB Header */
typedef struct {
	byte entry_type;	/* IOCB type */
	byte entry_cnt;		/* Number of entries contained */
	byte host_private;	/* Private use of host driver */
	byte entry_status;	/* entry status flags */
} ispfc_iocb_header_t;

/* ispfc_iocb_header_t.entry_type */
#define ISPFC_ENTRY_TYPE_MARKER		0x04    /* marker entry */
#define ISPFC_ENTRY_TYPE_COMMAND2	0x11    /* command entry type 2 */
#define ISPFC_ENTRY_TYPE_CONTIN32	0x02    /* continuation entry 32 bit */
#define ISPFC_ENTRY_TYPE_STATUS		0x03    /* completion entry status */
#define ISPFC_ENTRY_TYPE_STATUS_CONT	0x10	/* continuation entry status */
#define ISPFC_ENTRY_TYPE_CTIOCB		0x29    /* Name/Management Server */

/* ispfc_iocb_header_t.entry_status */
#define ISPFC_ENTRY_STATUS_DMA_ERROR	0x40
#define ISPFC_ENTRY_STATUS_BAD_ORDER	0x20
#define ISPFC_ENTRY_STATUS_BAD_COUNT	0x10
#define ISPFC_ENTRY_STATUS_BAD_PARAM	0x08
#define ISPFC_ENTRY_STATUS_BAD_TYPE	0x04
#define ISPFC_ENTRY_STATUS_BUSY		0x02

/* Marker IOCB - type = 4 */
typedef struct {
        ispfc_iocb_header_t header;	
        byte		    _resvd1[4];
	byte		    _resvd2;
	byte		    target;
	byte		    modifier;
	byte		    _resvd3;
	ushort16	    flags;
	ushort16	    lun;
	byte		    _resvd4[48];
} ispfc_iocb_marker_t;

/* ispfc_iocb_marker_t.modifier definitions */
#define MARKER_SYNC_LUN	      0
#define MARKER_SYNC_TARGET    1
#define MARKER_SYNC_ALL	      2

/* Data Segment Descriptor List Type 0 */
typedef struct {
	uint32 buf;
	uint32 len;
} ispfc_dsd0_t;

/* Data Segment Descriptor List Type 1 */
typedef struct {
	uint32 buflo;
	uint32 bufhi;
	uint32 len;
} ispfc_dsd1_t;

#define CMD_TYPE2_SG_CNT	3
#define CONT_TYPE0_SG_CNT	7

#define CMD_MAX_CDB		16

/* Command Type 2 IOCB - type = 0x11 */
typedef struct {
	ispfc_iocb_header_t header;
	uint32		    handle;
	byte		    _resvd1;
	byte		    target;
	ushort16	    lun;
	ushort16	    control_flags;
	byte		    ref_number;
	byte		    _resvd2;
	ushort16	    timeout;
	ushort16	    seg_count;
	byte		    cdb[CMD_MAX_CDB];
	uint32		    byte_count;
	ispfc_dsd0_t	    segments[CMD_TYPE2_SG_CNT];
} ispfc_iocb_command2_t;

/* ispfc_iocb_command2_t.control_flags definitions */
#define ISPFC_CONTROL_NO_FAST	       0x0080
#define ISPFC_CONTROL_NO_DATA	       0x0000
#define ISPFC_CONTROL_WRITE	       0x0040
#define ISPFC_CONTROL_READ	       0x0020
#define ISPFC_CONTROL_SIMPLE_Q	       0x0008
#define ISPFC_CONTROL_ORDER_Q	       0x0004
#define ISPFC_CONTROL_HEAD_Q	       0x0002
#define ISPFC_CONTROL_ACA_Q	       0x0001

/* Continuation Type 0 IOCB - type = 2 */
typedef struct {
	ispfc_iocb_header_t header;
	ushort16	    _resvd1;
	ushort16	    _resvd2;
	ispfc_dsd0_t	    segments[CONT_TYPE0_SG_CNT];
} ispfc_iocb_cont0_t;

#define IOCB_SENSE_LEN	32

/* Status IOCB - type = 3 */
typedef struct {
	ispfc_iocb_header_t header;
	uint32		    handle;
	ushort16	    scsi_status;
	ushort16	    completion_status;
	ushort16	    state_flags;
	ushort16	    status_flags;
	ushort16	    resp_info_len;
	ushort16	    sense_data_len;
	uint32		    residual;
	byte		    resp_info[8];
	byte		    scsi_sense[IOCB_SENSE_LEN];
} ispfc_iocb_status_t;

/* ispfc_iocb_status_t.scsi_status definitions */
#define ISPFC_SS_RESIDUAL_UNDER		0x0800
#define ISPFC_SS_RESIDUAL_OVER		0x0400
#define ISPFC_SS_SENSE_LEN_VALID	0x0200
#define ISPFC_SS_RESP_INFO_LEN_VALID	0x0100
#define ISPFC_SS_SCSI_STATUS_MASK	0x00FF

/* ispfc_iocb_status_t.completion_status definitions */
#define ISPFC_CS_COMPLETE		0x0000
#define ISPFC_CS_DMA_ERROR		0x0002
#define ISPFC_CS_TRANSPORT_ERROR	0x0003
#define ISPFC_CS_RESET_OCCURRED		0x0004
#define ISPFC_CS_ABORTED		0x0005
#define ISPFC_CS_TIMEOUT		0x0006
#define ISPFC_CS_DATA_OVERRUN		0x0007
#define ISPFC_CS_DATA_UNDERRUN		0x0015
#define ISPFC_CS_QUEUE_FULL		0x001c
#define ISPFC_CS_PORT_UNAVAIL		0x0028
#define ISPFC_CS_LOGGED_OUT		0x0029
#define ISPFC_CS_CONFIG_CHANGED		0x002a

/* ispfc_iocb_status_t.state_flags */
#define ISPFC_SF_GOT_STATUS		0x1000
#define ISPFC_SF_XFER_DATA		0x0800
#define ISPFC_SF_SENT_CDB		0x0400
#define ISPFC_SF_CANNOT_OPEN_DEVICE	0x0100
#define ISPFC_SF_NO_FAST_POST		0x0080
#define ISPFC_SF_WRITE			0x0040
#define ISPFC_SF_READ			0x0020
#define ISPFC_SF_SIMPLE_Q		0x0008
#define ISPFC_SF_ORDERED_Q		0x0004
#define ISPFC_SF_HEAD_Q			0x0002
#define ISPFC_SF_ACA_Q			0x0001

/* ispfc_iocb_status_t.status_flags */
#define ISP_STAT_DMA_ERROR		0x0080
#define ISP_STAT_TIMEOUT		0x0040
#define ISP_STAT_ABORTED		0x0020

/* NS/MS CT IOCB - type = 0x29 */
typedef struct {
	ispfc_iocb_header_t header;
        uint32		    handle;
	byte		    _resvd1;
	byte		    target;
        ushort16	    status;
	ushort16	    control_flags;
	ushort16	    _resvd2;
	ushort16	    timeout;
	ushort16	    req_dsd_count;	/* == 1 */
	ushort16	    total_dsd_count;	/* == 2 */
	byte		    _resvd[10];
	uint32		    resp_byte_count;
	uint32		    req_byte_count;
	ispfc_dsd1_t	    req_dsd;
	ispfc_dsd1_t	    resp_dsd;
} ispfc_iocb_ct_t;

int ispfc_send_marker(ispfc_cb_t *, ispfc_dev_t *, int, int);
int ispfc_send_scsi_desc(ispfc_cb_t *, ispfc_dev_t *, scsi_desc_t *);
int ispfc_send_ct_desc(ispfc_cb_t *, ct_desc_t *);

void  ispfc_process_iocb_completions(ispfc_cb_t *, int);
void  ispfc_iocb_handle_init(ispfc_cb_t *);

#endif /* __ISPFC_IOCB_H__ */
