/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_misc.h
 *
 * RCS:		$Id: ispfc_misc.c,v 1.34 2002/10/22 01:30:38 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver miscellaneous code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/6/01
 *
 *------------------------------------------------------------------------------
 */

/* #define ISPFC_DEBUG_DUP_BUFS */

#include "ispfc_private.h"
#include "ispfc_mbox.h"
#include "ispfc_link.h"
#ifdef ISPFC_DEBUG_DUP_BUFS
#include "ispfc_ct.h"
#include "ispfc_device.h"
#include <exports/hal.h>
#endif

void ispfc_qlogic_dump_1(ispfc_cb_t *, void * cookie, int32);

#ifdef DEBUG
ushort16 
_ispfc_reg_read(ispfc_cb_t *cb, ushort16 offset)
{
        ushort16   val;
        uint32	   trace;
        uint32	   prev_i;

#if !defined(ISPFC_SIM)
	val = *(volatile ushort16 *)(cb->reg_mem_base + offset); 
#else
	val = 0;
#endif

#if defined(BIG_ENDIAN_EVAL_BOARD)
	SWAP2(&val);
#endif

	if (offset != ISPFC_REG_GPIOD) {
        VERIFY(cb->pio_trace_i < PIO_TRACE_MAX);
        trace = ((offset & 0x7FFF) << 16) | val;
        if (cb->pio_trace_i == 0)
                prev_i = PIO_TRACE_MAX - 1;
        else
                prev_i = cb->pio_trace_i - 1;
        if (trace == cb->pio_trace[prev_i].trace_val)
                ++cb->pio_trace[prev_i].trace_count;
        else {
                cb->pio_trace[cb->pio_trace_i].trace_val = trace;
                cb->pio_trace[cb->pio_trace_i].trace_count = 1;
                eee_rtc(&(cb->pio_trace[cb->pio_trace_i].trace_rtc));
                if (++cb->pio_trace_i == PIO_TRACE_MAX)
			cb->pio_trace_i = 0;
        }
	}
        return(val);
}

void	 
_ispfc_reg_write(ispfc_cb_t *cb, ushort16 offset, ushort16 val)
{
	uint32    trace;
        uint32    prev_i;

	if (offset != ISPFC_REG_GPIOD) {
        VERIFY(cb->pio_trace_i < PIO_TRACE_MAX);
        trace = (1 << 31) | ((offset & 0x7FFF) << 16) | val;
        if (cb->pio_trace_i == 0)
                prev_i = PIO_TRACE_MAX - 1;
        else
                prev_i = cb->pio_trace_i - 1;
        if (trace == cb->pio_trace[prev_i].trace_val)
                ++cb->pio_trace[prev_i].trace_count;
        else {
                cb->pio_trace[cb->pio_trace_i].trace_val = trace;
                cb->pio_trace[cb->pio_trace_i].trace_count = 1;
                eee_rtc(&(cb->pio_trace[cb->pio_trace_i].trace_rtc));
                if (++cb->pio_trace_i == PIO_TRACE_MAX)
			cb->pio_trace_i = 0;
        }
	}
#if defined(BIG_ENDIAN_EVAL_BOARD)
	SWAP2(&val);
#endif

#if !defined(ISPFC_SIM)
	*(volatile ushort16 *)(cb->reg_mem_base + offset) = val; 
#endif
}

void	 
ispfc_dump_pio_trace(ispfc_cb_t *cb, int count)
{
	uint32		last_index = cb->pio_trace_i;
	uint32		i, j = 0;
	ushort16	dir, reg, val;
	uint32		t_count;
	uint64		t_msecs;
	char		regname[10];
		
	if ((count > PIO_TRACE_MAX) || (count == 0))
		count = PIO_TRACE_MAX;
	i = (last_index -  count + PIO_TRACE_MAX) % PIO_TRACE_MAX;
	for (j = 0; j < count; ++j) {
		dir = (cb->pio_trace[i].trace_val & 0x80000000) >> 31;
		reg = (cb->pio_trace[i].trace_val & 0x7FFF0000) >> 16;
		val = cb->pio_trace[i].trace_val & 0x0000FFFF;
		t_count = cb->pio_trace[i].trace_count;
		t_msecs = eee_rtcToMS(&cb->pio_trace[i].trace_rtc);
		switch(reg) {
		case 0x0010: 
			sprintf(regname, "REQ_IN"); 
			break;
		case 0x0012: 
			sprintf(regname, "REQ_OUT"); 
			break;
		case 0x0014: 
			sprintf(regname, "RESP_IN"); 
			break;
		case 0x0016: 
			sprintf(regname, "RESP_OUT"); 
			break;
		case 0x0018: 
			sprintf(regname, "RTOH_LO"); 
			break;
		case 0x001A: 
			sprintf(regname, "RTOH_HI"); 
			break;
		case 0x001C: sprintf(regname, "MAILBOX6"); break;
		case 0x001E: sprintf(regname, "MAILBOX7"); break;
		case 0x0008: sprintf(regname, "ICR"); break;
		case 0x000A: sprintf(regname, "ISR"); break;
		case 0x000C: sprintf(regname, "SEMA"); break;
		case 0x0040: sprintf(regname, "MAILBOX0"); break;
		case 0x0042: sprintf(regname, "MAILBOX1"); break;
		case 0x0044: sprintf(regname, "MAILBOX2"); break;
		case 0x0046: sprintf(regname, "MAILBOX3"); break;
		case 0x0048: sprintf(regname, "MAILBOX4"); break;
		case 0x004A: sprintf(regname, "MAILBOX5"); break;
		case 0x004C: sprintf(regname, "MAILBOX6"); break;
		case 0x004E: sprintf(regname, "MAILBOX7"); break;
		case 0x00C0: sprintf(regname, "HCCR"); break;
		default: sprintf(regname, "0x%x", reg);
		}
		P("[%4d] %s 0x%4x (%9s): 0x%4x [msecs %lld]", 
		  j - count, (dir ? "W":"R"), 
		  reg, regname, val, t_msecs);
		if (t_count > 1)
			P(" (count = %d)", t_count);
		P("\n");
		if (i++ == PIO_TRACE_MAX)
			i = 0;
	}
}

void
ispfc_dump_registers(ispfc_cb_t *cb)
{
	struct dump_regs_struct {
			char	*reg_name;
			int	reg_offset;
	};

	struct dump_regs_struct nonpause_dump_regs[] = {
		{ "ICR      ", 0x0008 },
		{ "ISR      ", 0x000A },
		{ "SEMA     ", 0x000C },
		{ "HCCR     ", 0x00C0 },
		{ "REQ_IN   ", 0x0010 },
		{ "REQ_OUT  ", 0x0012 },
		{ "RESP_IN  ", 0x0014 },
		{ "RESP_OUT ", 0x0016 },
		{ "RTOH_LO  ", 0x0018 },
		{ "RTOH_HI  ", 0x001a },
		{ "GPIOD    ", 0x00cc },
		{ "GPIOE    ", 0x00ce },
		{ NULL,       0 }};

	struct dump_regs_struct pause_dump_regs[] = {
		{ "COMMAND_DMA_CONTROL        ", 0x0094 },
		{ "COMMAND_DMA_STATUS         ", 0x0096 },
		{ "COMMAND_FIFO_PORT          ", 0x0090 },
		{ "COMMAND_DMA_XFER_BYTE_COUNT", 0x0088 },
		{ "COMMAND_DMA_ADDR_0         ", 0x0080 },
		{ "COMMAND_DMA_ADDR_1         ", 0x0082 },
		{ "COMMAND_DMA_ADDR_2         ", 0x0084 },
		{ "COMMAND_DMA_ADDR_3         ", 0x0086 },
		{ NULL,       0 }};
	/*
		{ "XMIT_DMA_CONTROL           ", 0x0040 },
		{ "XMIT_DMA_STATUS            ", 0x0042 },
		{ "XMIT_DMA_FRAME_BYTE_SIZE   ", 0x0048 },
		{ "XMIT_DMA_SEQ_BYTE_COUNT_0  ", 0x004C },
		{ "XMIT_DMA_SEQ_BYTE_COUNT_1  ", 0x004E },
		{ "XMIT_DMA_XFER_BYTE_COUNT_0 ", 0x0050 },
		{ "XMIT_DMA_XFER_BYTE_COUNT_1 ", 0x0052 },
		{ "XMIT_DMA_ADDR_0            ", 0x0058 },
		{ "XMIT_DMA_ADDR_1            ", 0x005A },
		{ "XMIT_DMA_ADDR_2            ", 0x005C },
		{ "XMIT_DMA_ADDR_3            ", 0x005E },

		{ "RECV_DMA_CONTROL           ", 0x0060 },
		{ "RECV_DMA_STATUS            ", 0x0062 },
		{ "RECV_DMA_PAYLOAD_BYTE_COUNT", 0x006A },
		{ "RECV_DMA_XFER_BYTE_COUNT_0 ", 0x0070 },
		{ "RECV_DMA_XFER_BYTE_COUNT_1 ", 0x0072 },
		{ "RECV_DMA_ADDR_0            ", 0x0078 },
		{ "RECV_DMA_ADDR_1            ", 0x007A },
		{ "RECV_DMA_ADDR_2            ", 0x007C },
		{ "RECV_DMA_ADDR_3            ", 0x007E },
	*/

	int	i;
	ushort16 val;
	ushort16 control;

	/* Dump mailbox registers */
	for (i = 0; i < 16; ++i) {
		val = ISPFC_REG_READ(cb, ISPFC_REG_MBOX(i));
		P("[0x%04x]: MAILBOX%02d: 0x%x\n", i * 2 + ISPFC_REG_MBOX_BASE, i, val);
	}
	/* Dump non-pause registers */
	for (i = 0; nonpause_dump_regs[i].reg_name; ++i) {
		val = ISPFC_REG_READ(cb, nonpause_dump_regs[i].reg_offset);
		P("[0x%04x]: %s: 0x%x\n", 
		  nonpause_dump_regs[i].reg_offset, 
		  nonpause_dump_regs[i].reg_name, 
		  val);
	}

	/*
	 * Pause RISC, dump the pause registers and then restart the RISC
	 */
	ISPFC_REG_WRITE(cb, ISPFC_REG_HCCR, HCCR_CMD_PAUSE);
	for (i = 10000; ; ) {
		eee_delay(100);
		if (ISPFC_REG_READ(cb, ISPFC_REG_HCCR) & HCCR_PAUSE)
			break;
		if (i-- == 0) {
			P("ispfc_dump_registers: RISC pause failed\n");
			return;
		}
	}

	/* Switch banks */
	control = ISPFC_REG_READ(cb, ISPFC_REG_CONSTAT);
	ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, (control | (5 << 4)));
	
	for (i = 0; pause_dump_regs[i].reg_name; ++i) {
		val = ISPFC_REG_READ(cb, pause_dump_regs[i].reg_offset);
		P("[0x%04x]: %s: 0x%x\n", pause_dump_regs[i].reg_offset, pause_dump_regs[i].reg_name, val);
	}

	/* Switch banks */
	ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, control & 0xFF8F);

	ISPFC_REG_WRITE(cb, ISPFC_REG_HCCR, HCCR_CMD_RELEASE);
	for (i = 10000; ; ) {
		eee_delay(100);
		if (!(ISPFC_REG_READ(cb, ISPFC_REG_HCCR) & HCCR_PAUSE))
			break;
		if (i-- == 0) {
			P("ispfc_dump_registers: RISC unpause failed\n");
			return;
		}
	}
}

#endif /* DEBUG */

#define II      0xFF
const
byte alpa_to_id[] = {0x7e, 0x7d, 0x7c,   II, 0x7b,   II,   II,   II, /* 0-7 */
		     0x7a,   II,   II,   II,   II,   II,   II, 0x79, /* 8-15 */
		     0x78,   II,   II,   II,   II,   II,   II, 0x77, /* 16-23 */
		     0x76,   II,   II, 0x75,   II, 0x74, 0x73, 0x72, /* 24-31 */
		       II,   II,   II, 0x71,   II, 0x70, 0x6f, 0x6e, /* 32-39 */
		       II, 0x6d, 0x6c, 0x6b, 0x6a, 0x69, 0x68,   II, /* 40-47 */
		       II, 0x67, 0x66, 0x65, 0x64, 0x63, 0x62,   II, /* 48-55 */
		       II, 0x61, 0x60,   II, 0x5f,   II,   II,   II, /* 56-63 */
		       II,   II,   II, 0x5e,   II, 0x5d, 0x5c, 0x5b, /* 64-71 */
		       II, 0x5a, 0x59, 0x58, 0x57, 0x56, 0x55,   II, /* 72-79 */
		       II, 0x54, 0x53, 0x52, 0x51, 0x50, 0x4f,   II, /* 80-87 */
		       II, 0x4e, 0x4d,   II, 0x4c,   II,   II,   II, /* 88-95 */
		       II,   II,   II, 0x4b,   II, 0x4a, 0x49, 0x48, /* 96-103 */
		       II, 0x47, 0x46, 0x45, 0x44, 0x43, 0x42,   II, /* 104-111 */
		       II, 0x41, 0x40, 0x3f, 0x3e, 0x3d, 0x3c,   II, /* 112-119 */
		       II, 0x3b, 0x3a,   II, 0x39,   II,   II,   II, /* 120-127 */
		     0x38, 0x37, 0x36,   II, 0x35,   II,   II,   II, /* 128-135 */
		     0x34,   II,   II,   II,   II,   II,   II, 0x33, /* 136-143 */
		     0x32,   II,   II,   II,   II,   II,   II, 0x31, /* 144-151 */
		     0x30,   II,   II, 0x2f,   II, 0x2e, 0x2d, 0x2c, /* 152-159 */
		       II,   II,   II, 0x2b,   II, 0x2a, 0x29, 0x28, /* 160-167 */
		       II, 0x27, 0x26, 0x25, 0x24, 0x23, 0x22,   II, /* 168-175 */
		       II, 0x21, 0x20, 0x1f, 0x1e, 0x1d, 0x1c,   II, /* 176-183 */
		       II, 0x1b, 0x1a,   II, 0x19,   II,   II,   II, /* 184-191 */
		       II,   II,   II, 0x18,   II, 0x17, 0x16, 0x15, /* 192-199 */
		       II, 0x14, 0x13, 0x12, 0x11, 0x10, 0x0f,   II, /* 200-207 */
		       II, 0x0e, 0x0d, 0x0c, 0x0b, 0x0a, 0x09,   II, /* 208-215 */
		       II, 0x08, 0x07,   II, 0x06,   II,   II,   II, /* 216-223 */
		     0x05, 0x04, 0x03,   II, 0x02,   II,   II,   II, /* 224-231 */
		     0x01,   II,   II,   II,   II,   II,   II, 0x00  /* 232-239 */
};


byte
ispfc_alpa_to_loopid(byte alpa)
{
        return(alpa_to_id[alpa]);
}

byte
ispfc_loopid_to_alpa(byte loopid)
{
        int i;

        for (i = 0; i < sizeof(alpa_to_id); ++i) {
                if (alpa_to_id[i] == loopid)
                        return(i);
        }
        return(II);
}

char *
ispfc_scsi_cdb_to_string(byte *cdb)
{
	static char  outstring[64];

	if (cdb[0] == 0x28 || cdb[0] == 0x2a) {
		sprintf(outstring, "0x%02x:%02x%02x%02x%02x:%02x%02x", 
			cdb[0],					/* Opcode */
			cdb[2], cdb[3], cdb[4], cdb[5],		/* LBA */
			cdb[7], cdb[8]);			/* Transfer Length */
	}
	else
		sprintf(outstring, "0x%02x", cdb[0]);

	return(outstring);
}


char *
ispfc_scsi_sense_to_string(byte *sense_data)
{
	static char outstring[128];
	static struct { 
		int code;
		int qualifier;
		char    *fcal_string;
	} scode_strings[] = {
		{ 4,    1,  "Drive spinning up"			},
		{ 0x10, 0,  "ID CRC or ECC error"		}, 
		{ 0x11, -1, "Unrecovered read error"		}, 
		{ 0x29, -1, "Unit Attention"			},
		{ 0x31, 0,  "Medium format corrupted"		}, 
		{ 0x47, 0,  "Fibre Channel frame CRC error"	},
		{ 0,    0,  NULL				},
	};
	int	i;
	char *	p;
	byte	key = sense_data[2];
	byte	code = sense_data[12];
	byte	qualifier = sense_data[13];

	p = outstring;
	sprintf(p, "%d/0x%x/0x%x", key, code, qualifier);
	p += strlen(p);

	for (i = 0; scode_strings[i].code; ++i) {
		if (scode_strings[i].code == code &&
		    (scode_strings[i].qualifier == qualifier ||
		     scode_strings[i].qualifier == -1))
			sprintf(p, " - %s", scode_strings[i].fcal_string);
	}
	return(outstring);
}

eee_desc_t *  
ispfc_edesc_allocate(scsi_desc_t *sd)
{
#define FIRST_BUFFER_OFFSET	128

	eee_desc_t *	ed;
	eee_desc_t *	first_ed = NULL;
	eee_descBuf_t * bd;
	int32		byte_count = sd->expectDataLen;
	int		dest = sd->dataDest ? sd->dataDest : sd->rspDest;
	int		requested_count = byte_count;
	int		alloced_count = 0;
	int		buf_count;
	int		buf_byte_count;
	int		packet_len;

	VERIFY(sd->dataDest ? 
	       ((EEE_GET_DTYPE(sd->dataDest) == EEE_USER_DATA) | 
		(EEE_GET_DTYPE(sd->dataDest) == EEE_SAC) | 
		(EEE_GET_DTYPE(sd->dataDest) == EEE_METADATA)) :
	       1);
	
	switch(EEE_GET_DTYPE(dest)) {
	    case EEE_USER_DATA:
	    case EEE_SAC:					
	    case EEE_METADATA:
		    packet_len = (sd->control & SD_FLAG_EMBEDDED_CS) ? (520 * 16) : (512 * 16);
		    break;

	    case EEE_CONTROL:
	    case EEE_MGMT:
		    if (EEE_GET_SLOT_NUM(dest) == 1) {
			    printf("***************************************************************************\n");
			    printf("WARNING: Data Destination is the FP-NIM, yet selected plane is CONTROL/MGMT\n");
			    printf("***************************************************************************\n");
		    }
		    packet_len = (sd->control & SD_FLAG_EMBEDDED_CS) ? (520 * 2) : (512 * 2);
		    break;

	    default:
		    VERIFY(0);
	}

	while (alloced_count < byte_count) {
		if (first_ed == NULL) {
			first_ed = ed = eee_allocateDesc(EEE_SHARED_POOL, 0);
		}
		else {
			VERIFY(!(ed->bd[EEE_MAX_BUFS-1].control & EDESC_BD_CMD_IN_USE));
			ed->bd[EEE_MAX_BUFS-1].buf = (uchar8 *)eee_allocateDesc(EEE_SHARED_POOL, 0);
			ed->hdr.control |= EDESC_COMMAND_CHAIN_LAST_BUFFER;
			ed = (eee_desc_t *)ed->bd[EEE_MAX_BUFS-1].buf;
		}
		VERIFY(ed);
		bzero(ed, sizeof(eee_desc_t));

		requested_count = MIN(byte_count, packet_len);

		requested_count += FIRST_BUFFER_OFFSET;

		buf_byte_count = 0;
		buf_count = 0;
		bd = &ed->bd[1];
		while (requested_count > 0) {
			bd->control = EDESC_BD_CMD_IN_USE;
			bd->len = MIN(requested_count, packet_len + FIRST_BUFFER_OFFSET);
			bd->len = MIN(bd->len, EEE_BUFSIZE_ULTRA);
			bd->control |= EDESC_BD_ULTRA;
			bd->buf = eee_allocateBuffer(EEE_SHARED_POOL, bd->control & EDESC_BD_BUFSIZE_MASK, 0);
#ifdef ISPFC_DEBUG_DUP_BUFS
			{
				/* 
				 * Walk thru all devices and their
				 * pending/waiting queues. Check that
				 * the just allocated buffer isn't
				 * being used anywhere else. 
				 */
				ispfc_cb_t *	__cb = hal_dt_entry_get()->hal_handle;
				ispfc_dev_t *	__dev;
				scsi_desc_t *	__sd;
				eee_desc_t *	__ed;
				uint32		__bufcnt, __i;
				eee_descBuf_t * __bd;

				if ((__dev = __cb->dev_list)) { 
					do {
						/* Check busy queue */
						if ((__sd = __dev->scsi_desc_busy)) {
							do {
								__ed = __sd->eDesc;
								while (__ed) {
									__bufcnt = (__ed->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
									__bd = &__ed->bd[0];
									if (!(__bd->control & EDESC_BD_CMD_IN_USE))
										++__bd;
									for (__i = 0; __i < __bufcnt; ++__i) {
										VERIFY((address_t)(bd->buf) != ((address_t)(__bd->buf) & 0xFFFFFF00));
										++__bd;
									}
								
									if (__ed->hdr.control & EDESC_COMMAND_CHAIN_LAST_BUFFER) {
										__ed = (eee_desc_t *)__ed->bd[EEE_MAX_BUFS-1].buf;
										VERIFY(__ed);
									}
									else
										__ed = NULL;
								}

								__sd = __sd->next;
							} while (__sd != __dev->scsi_desc_busy);
						}
					
						/* Check waiting queue */
						if ((__sd = __dev->scsi_desc_waiting)) {
							do {
								__ed = __sd->eDesc;
								while (__ed) {
									__bufcnt = (__ed->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
									__bd = &__ed->bd[0];
									if (!(__bd->control & EDESC_BD_CMD_IN_USE))
										++__bd;
									for (__i = 0; __i < __bufcnt; ++__i) {
										VERIFY((address_t)(bd->buf) != ((address_t)(__bd->buf) & 0xFFFFFF00));
										++__bd;
									}
								
									if (__ed->hdr.control & EDESC_COMMAND_CHAIN_LAST_BUFFER) {
										__ed = (eee_desc_t *)__ed->bd[EEE_MAX_BUFS-1].buf;
										VERIFY(__ed);
									}
									else
										__ed = NULL;
								}

								__sd = __sd->next;
							} while (__sd != __dev->scsi_desc_waiting);
						}
						__dev = __dev->next;
					} while (__dev != __cb->dev_list);
				}
			}
#endif
			VERIFY(bd->buf);
			requested_count -= bd->len;
			alloced_count += bd->len;
			buf_byte_count += bd->len;
			++bd;
			++buf_count;
		}
		ed->hdr.control = ((buf_count << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
		ed->hdr.control |= (buf_byte_count & EDESC_TOTAL_LEN);

		ispfc_edesc_offset_buf(ed, FIRST_BUFFER_OFFSET, 1);
		alloced_count -= FIRST_BUFFER_OFFSET;
	}
	VERIFY(alloced_count >= byte_count);
	return(first_ed);
}

void
ispfc_edesc_set_len(eee_desc_t *ed, int32 byte_count)
{
	int		i;
	eee_desc_t *	next_ed;
	eee_descBuf_t *	bd;
	int32		buf_count;
	int32		tmp_buf_count;
	int32		buf_byte_count;

	do {
		VERIFY(ed);
		bd = &ed->bd[0];
		if (!(bd->control & EDESC_BD_CMD_IN_USE))
			++bd;
		buf_byte_count = ed->hdr.control & EDESC_TOTAL_LEN;
		buf_count = (ed->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
		for (i = 0; i < buf_count; ++i, ++bd) {
			VERIFY(bd->control & EDESC_BD_CMD_IN_USE);
			byte_count -= bd->len;
			if (byte_count < 0) {
				buf_byte_count += byte_count;
				bd->len += byte_count;
				byte_count = 0;
				if (bd->len == 0) {
				    eee_deallocateBuffer(bd->buf);	
				    bd->buf = NULL;
				    bd->control &= ~EDESC_BD_CMD_IN_USE;
				    tmp_buf_count = (ed->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
				    tmp_buf_count--;
				    ed->hdr.control &= ~EDESC_BUFCNT_MASK;
				    ed->hdr.control |= (tmp_buf_count << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK;
				}
			}
		}
		ed->hdr.control = (ed->hdr.control & ~EDESC_TOTAL_LEN) | (buf_byte_count & EDESC_TOTAL_LEN);
		if (ed->hdr.control & EDESC_COMMAND_CHAIN_LAST_BUFFER)
			next_ed = (eee_desc_t *)ed->bd[EEE_MAX_BUFS-1].buf;
		else
			next_ed = NULL;
		if (byte_count == 0)
			ed->hdr.control &= ~EDESC_COMMAND_CHAIN_LAST_BUFFER;
		ed = next_ed;
	} while (byte_count);

	/* Free "excess" descriptors */
	while (ed) {
		if (ed->hdr.control & EDESC_COMMAND_CHAIN_LAST_BUFFER)
			next_ed = (eee_desc_t *)ed->bd[EEE_MAX_BUFS-1].buf;
		else
			next_ed = NULL;
		eee_freePacket(ed);
		ed = next_ed;
	}
}

void
ispfc_edesc_offset_buf(eee_desc_t *ed, int32 byte_count, int buf_index)
{
	int32		total_byte_count = ed->hdr.control & EDESC_TOTAL_LEN;

	VERIFY(total_byte_count >= byte_count);
	VERIFY(ed->bd[buf_index].control & EDESC_BD_CMD_IN_USE);
	VERIFY(ed->bd[buf_index].buf);
	VERIFY(ed->bd[buf_index].len >= byte_count);

	ed->bd[buf_index].buf += byte_count;
	ed->bd[buf_index].len -= byte_count;
	total_byte_count -= byte_count;
	
	ed->hdr.control = (ed->hdr.control & ~EDESC_TOTAL_LEN) | (total_byte_count & EDESC_TOTAL_LEN);
}

void 
ispfc_trigger_pci_analyzer(void)
{
	hal_dispatch_entry_t *	hal_dte = hal_dt_entry_get();

	pci_configRead32(hal_dte->hal_bus_id, hal_dte->hal_dev_id, 0x20);
}



typedef struct {
	uint32	dump_state;
	void (*dump_callback)(ispfc_cb_t *, void *, int32);
	void *	dump_cookie;
	char *	dump_buffer;
	uint32	dump_generic_counter;
	uint32  dump_transfer_length;
} ispfc_dump_state_t;

void
ispfc_qlogic_dump(ispfc_cb_t *cb, void (*callback)(ispfc_cb_t *, void *, int32), void *cookie)
{
	ispfc_dump_state_t * st;

	st = eee_ramAlloc(sizeof(ispfc_dump_state_t));
	VERIFY(st);
	bzero(st, sizeof(ispfc_dump_state_t));

	st->dump_state = 0;
	st->dump_callback = callback;
	st->dump_cookie = cookie;

	ispfc_qlogic_dump_1(cb, st, MBOX_CMD_STATUS_SUCCESS);
}

void
ispfc_qlogic_dump_1(ispfc_cb_t *cb, void *cookie, int32 status)
{
	ispfc_dump_state_t *	st = cookie;
	int			i;

 next_state:

	FC_DBG(1, ("%s: ispfc_qlogic_dump_1: state = %d, status = 0x%x\n", 
		   cb->debug_name, st->dump_state, status));

	switch(st->dump_state) {
	case 0:
		/* Dump registers */

		/* 
		 * Step 1: 
		 * Pause RISC 
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_HCCR, HCCR_CMD_PAUSE);
		for (i = 10000; ; ) {
			eee_delay(100);
			if (ISPFC_REG_READ(cb, ISPFC_REG_HCCR) & HCCR_PAUSE)
				break;
			if (i-- == 0) {
				P("ispfc_qlogic_dump_1: RISC pause failed\n");
				return;
			}
		}

		/* 
		 * Step 3:
		 * PBIU Registers:
		 * 0000  ffff  0000  0004  8008  0000  0002  0000
		 */
		P("PBIU Registers:\n");
		for (i = 0; i < 8; ++i) {
			P("%04x  ", ISPFC_REG_READ(cb, 0x00 + i*2));
		}
		P("\n\n");

		/*
		 * Step 4:
		 * ReqQ-RspQ-Risc2Host Status registers:
		 * 00ac  00ac  0029  0029  0111  4005  0000  0000
		 */
		P("ReqQ-RspQ-Risc2Host Status registers:\n");
		for (i = 0; i < 8; ++i) {
			P("%04x  ", ISPFC_REG_READ(cb, 0x10 + i*2));
		}
		P("\n\n");

		/*
		 * Step 5:
		 * Mailbox Registers:
		 * 4005  0004  0007  0001 55aa  a5a5  00dd  7372
		 */
		P("Mailbox Registers:\n");
		for (i = 0; i < 32; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x40 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 6:
		 * Auto Request Response DMA Registers:
		 * 0080  c001  0000  0000	2b80  c001  0100  4040
		 * 05fa  0565  0001  801f	1f10  0019  00ac  00ac
		 * 0100  c002  0000  0000	0b40  c002  0040  4040
		 * 05b4  05b4  0001  8017	1f00  0019  0029  0029
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, 0x40);
		P("Auto Request Response DMA Registers:\n");
		for (i = 0; i < 32; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");

		/*
		 * Step 7:
		 * DMA Registers:
		 * 0140  c003  0000  0000	0000  0000  0000  0000
		 * 0000  0086  0000  0004	0000  0000  0000  0000
		 * 3388  c0e1  0000  0000	0000  0000  0000  0000
		 * 0000  0000  0000  000c	0000  0800  0000  0000
		 * 0000  0000  0000  0003	0003  3000  0000  0002
		 * 0000  0000  0003  3000	0000  0002  0000  0000
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, 0x50);
		P("DMA Registers:\n");
		for (i = 0; i < 48; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 8 & 9:
		 * RISC Hardware Registers:
		 * a100  228a  103c  119f	5d39  0001  5dd4  0000
		 * 0004  14fb  007f  00c0	0001  0001  19f8  0004
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, 0x00);
		P("RISC Hardware Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0xA0 + i*2));
		}
		P("\n\n");

		/*
		 * Step 10:
		 * RISC GP0 Registers:
		 * 0000  0000  0000  0001	4005  1a40  0000  00ff
		 * 0101  0d00  1100  0000	45a8  0000  1240  0050
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2000);
		P("RISC GP0 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");

		/*
		 * Step 11:
		 * RISC GP1 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2200);
		P("RISC GP1 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 12:
		 * RISC GP2 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2400);
		P("RISC GP2 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 13:
		 * RISC GP3 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2600);
		P("RISC GP3 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 14:
		 * RISC GP4 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2800);
		P("RISC GP4 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 15:
		 * RISC GP5 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2a00);
		P("RISC GP5 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 16:
		 * RISC GP6 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2c00);
		P("RISC GP6 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 17:
		 * RISC GP7 Registers:
		 * 0003  0040  0001  0031	c003  0140  0190  0000
		 * 17c7  0009  0000  5fc0	19f8  5d00  1343  0300
		 */
		ISPFC_REG_WRITE(cb, 0xa4, 0x2e00);
		P("RISC GP7 Registers:\n");
		for (i = 0; i < 16; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");
			
		/*
		 * Step 18:
		 * Frame Buffer Hardware Registers:
		 * 0600  0004  0000  0000	0009  0000  0002  0008
		 * 0000  0000  0000  0895	0000  0020  001e  0005
		 * 100c  0101  1001  0000	42f0  ffff  0000  0000
		 * 2004  0081  0000  0000	0000  0000  0000  0000
		 * 0000  0509  0d11  1519	1d21  000d  041e  0201
		 * 0200  0001  0167  10db	1ca5  02de  05f0  013a
		 * 00d7  1ca5  00cf  05f0	0000  0000  0000  0000
		 * 0000  0000  0000  0000	0000  0000  0000  0000
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, 0x10);
		P("Frame Buffer Hardware Registers:\n");
		for (i = 0; i < 64; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");

		/*
		 * Step 19:
		 * FPM B0 Registers:
		 * 0007  049f  049f  006b	006b  0004  001e  001e
		 * 0634  0100  1e2f  0000	0000  0000  0004  0004
		 * 0000  0000  f8f7  f7f7	1404  006b  0233  0233
		 * 0101  07e8  0001  0a00	0809  0008  3500  0004
		 * 3ae0  ffff  0000  2000	2000  2000  0000  00e8
		 * 0056  9575  0036  95d5	0000  0000  0004  0000
		 * 0000  0000  0000  0800	0000  0000  0000  0000
		 * 0003  0001  0a00  0800	0000  0000  0000  0000
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, 0x20);
		P("FPM B0 Registers:\n");
		for (i = 0; i < 64; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");

		/*
		 * Step 20:
		 * FPM B1 Registers:
		 * 0007  049f  049f  006b	006b  0004  001e  001e
		 * 0634  0100  1e2f  0000	0000  0000  0004  0004
		 * 0000  0000  f8f7  f7f7	1404  006b  0233  0233
		 * 0101  07e8  0001  0a00	0809  0008  3500  0004
		 * 3ae0  ffff  0000  2000	2000  2000  0000  00e8
		 * 0056  9575  0036  95d5	0000  0000  0004  0000
		 * 0000  0000  0000  0800	0000  0000  0000  0000
		 * 0003  0001  0a00  0800	0000  0000  0000  0000
		 */
		ISPFC_REG_WRITE(cb, ISPFC_REG_CONSTAT, 0x30);
		P("FPM B1 Registers:\n");
		for (i = 0; i < 64; ++i) {
			if (i > 0 && (i % 8) == 0)
				P("\n");
			P("%04x  ", ISPFC_REG_READ(cb, 0x80 + i*2));
		}
		P("\n\n");

		/*
		 * Step 21:
		 */
		ispfc_soft_reset_adapter(cb);
		st->dump_state = 1;
		goto next_state;
		
	case 1:
		st->dump_generic_counter = 0x800;
		st->dump_transfer_length = 0x800 * 2;
		st->dump_buffer = HAL_ALIGN_TO(eee_ramAlloc(st->dump_transfer_length + 64), 64);

		CACHE_INVALIDATE_STRUCT((address_t)st->dump_buffer, st->dump_transfer_length);

		/* Fire off  ISPFC_DUMP_SRAM_EXT mailbox commands */
		cb->mbox_in_valid_count = 9;
		cb->mbox_in[0] = ISPFC_DUMP_SRAM_EXT;
		cb->mbox_in[1] = (st->dump_generic_counter & 0xFFFF);
		cb->mbox_in[2] = (VA_TO_IO32(st->dump_buffer) >> 16) & 0xFFFF;
		cb->mbox_in[3] = VA_TO_IO32(st->dump_buffer) & 0xFFFF;
		cb->mbox_in[4] = st->dump_transfer_length / 2;
		cb->mbox_in[5] = 0;
		cb->mbox_in[6] = 0;
		cb->mbox_in[7] = 0;
		cb->mbox_in[8] = st->dump_generic_counter >> 16;
		st->dump_state = 2;
		ispfc_mbox_start(cb, ispfc_qlogic_dump_1, st);
		return;

	case 2:
		if (status != MBOX_CMD_STATUS_SUCCESS) {
			ispfc_dump_registers(cb);
			st->dump_state = 0;
			goto next_state;
		}

		if (st->dump_generic_counter == 0x800)
			P("Code RAM Dump:\n");
		else
		if (st->dump_generic_counter == 0x10000)
			P("Stack RAM Dump:\n");
		else
		if (st->dump_generic_counter == 0x11000)
			P("Data RAM Dump:\n");

		/* Display SRAM contents */
		{
			ushort16 *  p = (ushort16 *)st->dump_buffer;
			int32	    i;

			P("%05x: ", st->dump_generic_counter);
			for (i = 0; i < st->dump_transfer_length / 2; ++i) { 
				if (i > 0 && (i % 8) == 0) {
					P("\n%05x: ", i + st->dump_generic_counter);
				}
				P("%04x ", *p++);
			}
			P("\n");
		}

		st->dump_generic_counter += 0x800;
		CACHE_INVALIDATE_STRUCT((address_t)st->dump_buffer, st->dump_transfer_length);

		if (st->dump_generic_counter < 0x80000) { /* 512K 16-bit words */
			/* Fire off  ISPFC_DUMP_SRAM_EXT mailbox commands */
			cb->mbox_in_valid_count = 9;
			cb->mbox_in[0] = ISPFC_DUMP_SRAM_EXT;
			cb->mbox_in[1] = (st->dump_generic_counter & 0xFFFF);
			cb->mbox_in[2] = (VA_TO_IO32(st->dump_buffer) >> 16) & 0xFFFF;
			cb->mbox_in[3] = VA_TO_IO32(st->dump_buffer) & 0xFFFF;
			cb->mbox_in[4] = st->dump_transfer_length / 2;
			cb->mbox_in[5] = 0;
			cb->mbox_in[6] = 0;
			cb->mbox_in[7] = 0;
			cb->mbox_in[8] = st->dump_generic_counter >> 16;
			st->dump_state = 2;
			ispfc_mbox_start(cb, ispfc_qlogic_dump_1, st);
			return;
		}
		else {
			P("[<==END] ISP Debug Dump\n");
			if (st->dump_callback) {
				(*st->dump_callback)(cb, st->dump_cookie, NFX_OK);
				eee_ramDealloc(st);
				return;
			}
			else 
				while (1) ;
		}
	}
}
