/*
 *=============================================================================
 * File Name   :    ispfc_td.h
 *
 * Description :
 *
 * Created     :    08-28-2001
 *
 * Author      :    Tony Wong
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/esm-api.h>

#ifndef FCTD_H_INCLUDED
#define FCTD_H_INCLUDED

#define FCTD_MAX_IES			16
#define FCTD_MAX_IE_PORTS		32
#define FCTD_TIMER_INTERVAL		10000	/* 10 secs */
#define FCTD_MAX_MSG_LEN		1000	/* calculate later */
#define FCTD_MAX_ATTACHED_PORTS		120	/* verify later */
#define FCTD_MAX_SYMBOLIC_NAME_LEN	32

/* Message IDs for SSC */

#define FCTD_SSC_TOPOLOGY_REQ		1
#define FCTD_SSC_TOPOLOGY_RSP		2
#define FCTD_SSC_FAIL_RSP		3

/* FCTD internal message codes */

#define FCTD_SEND_GIEL_REPLY		1
#define FCTD_SEND_GIELN_REPLY		2
#define FCTD_SEND_GPL_REPLY		3
#define FCTD_SEND_GPPN_REPLY		4
#define FCTD_SEND_GAPNL_REPLY		5
#define FCTD_SEND_GET_LOOP_MAP_REPLY	6
#define FCTD_SEND_CHANGE_NOTICE		7
#define FCTD_SEND_FAIL_REPLY		8

/* Discovery states */

#define FCTD_IDLE_STATE			1
#define FCTD_SEND_GIEL_REQ_STATE	2
#define FCTD_SEND_GIELN_REQ_STATE	3
#define FCTD_SEND_GPL_REQ_STATE		4
#define FCTD_SEND_GPPN_REQ_STATE	5
#define FCTD_SEND_GAPNL_REQ_STATE	6
#define FCTD_SEND_GET_LOOP_MAP		7
#define FCTD_DISCOVERY_DONE_STATE	8

/* Command reject reasons */

#define FC_REASON_BAD_CMD  	 	0x01
#define FC_REASON_BAD_VER		0x02
#define FC_REASON_LOGIC_ERR		0x03
#define FC_REASON_BAD_SIZE 		0x04
#define FC_REASON_BUSY     		0x05
#define FC_REASON_PROTOCOL 		0x07
#define FC_REASON_UNABLE   		0x09
#define FC_REASON_NO_SUPPORT		0x0b
#define FC_REASON_VENDOR  	 	0xff

/* Interconnect element (node) types */

#define FC_IE_UNKNOWN			0x00
#define FC_IE_SWITCH			0x01
#define FC_IE_HUB			0x02
#define FC_IE_BRIDGE			0x03

typedef struct _fctd_msg_hdr {
	uchar8	msg_id;
	uint32	msg_len;
	uchar8	msg_data[1];
} fctd_msg_hdr_t;

typedef struct _atportList { /* attached port */
	wwn_t		     port_name;
	uint32		     port_type;
	struct _atPortList * next;
} atPortList_t;

typedef struct _portList { /* list of ports including info */
	wwn_t		     port_name;
	uint32		     port_type;
	uint32		     physical_port_number;
	uint32		     num_atports;
	atPortList_t *	     atop_list;
	atPortList_t *	     atport_list;
	struct _portList *   next;
} portList_t;

typedef struct _fcNode { /* FC node with port list */
	wwn_t		     node_name;
	char8		     node_type;
	char8		     symbolic_name[FCTD_MAX_SYMBOLIC_NAME_LEN+1];
	uint32		     num_ports;
	portList_t *	     top_list;
	portList_t *	     port_list;
} fcNode_t;

typedef struct _fctd {  /* FCTD control block */
	char		     debug_name[32];
	uint32		     poll_timer_handle;
	wwn_t		     port_name;
	uchar8		     rsp_id;
	uchar8		     state;
	uchar8		     slot_num;
	uchar8		     port_num;
	uint32		     hal_state;
	uint32		     hal_interface_type;
	hal_device_map_t     loop_map;
	fcNode_t	     ie_list[FCTD_MAX_IES];
	uint32		     ie_count;
	uint32		     ie_index;
	ct_desc_t	     ctd;
	uint32		     appId;
	int32		     tmr_handle;
	esm_handle_t	     sm_handle;
} fctd_t;

extern fctd_t fctd_cb;

/* Function prototypes */

void  fctd_init(uint32);
int32 fctd_format_topology(char *, uint32 *);

#endif /* FCTD_H_INCLUDED */
