/*
 *-----------------------------------------------------------------
 *
 * Name: getopt.c
 *
 * RCS:  $Id: qsort.c,v 1.1 2001/11/22 00:04:14 haahr Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:  Getopt from net2 bsd, modified to allow resetting
 *
 * Created by:   Byron Rakitzis
 *
 * Date Created: 12/12/2002
 *
 *-----------------------------------------------------------------
 */


/*
 * Copyright (c) 1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <nfx-incl.h>

#ifndef PROM
/*
 * get option letter from argument vector
 */
int	optind = 1;		/* index into parent argv vector */
char	*optarg;		/* argument associated with option */

#define	BADCH	(int)'?'
#define	EMSG	""

#ifndef EOF
#define EOF (-1)
#endif

#if defined(FS_STANDALONE)
#include <string.h>
#endif

char *optplace = EMSG;		/* option letter processing */

int
getopt(nargc, nargv, ostr)
	int nargc;
	char * const *nargv;
	const char *ostr;
{
	int optopt;
	char *oli;				/* option letter list index */
	char *p;

	if (!*optplace) {				/* update scanning pointer */
		if (optind >= nargc || *(optplace = nargv[optind]) != '-') {
			optplace = EMSG;
			return(EOF);
		}
		if (optplace[1] && *++optplace == '-') {	/* found "--" */
			++optind;
			optplace = EMSG;
			return(EOF);
		}
	}					/* option letter okay? */
	if ((optopt = (int)*optplace++) == (int)':' ||
	    !(oli = strchr(ostr, optopt))) {
		/*
		 * if the user didn't specify '-' as an option,
		 * assume it means EOF.
		 */
		if (optopt == (int)'-')
			return(EOF);
		if (!*optplace)
			++optind;
		return(BADCH);
	}
	if (*++oli != ':') {			/* don't need argument */
		optarg = NULL;
		if (!*optplace)
			++optind;
	}
	else {					/* need an argument */
		if (*optplace)			/* no white space */
			optarg = optplace;
		else if (nargc <= ++optind) {	/* no arg */
			optplace = EMSG;
			if (!(p = strrchr(*nargv, '/')))
				p = *nargv;
			else
				++p;
			return(BADCH);
		}
	 	else				/* white space */
			optarg = nargv[optind];
		optplace = EMSG;
		++optind;
	}
	return(optopt);				/* dump back option letter */
}
#endif
