/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: newmemcpy.S,v 1.5 2002/10/21 19:59:24 sandrineb Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/4/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>

#if __mips >= 3
#define ADDIU	daddiu
#else
#define ADDIU	addiu
#endif

/*-----------------------------------------------------------------
 * Name:	void * memcpy(dst, src, nbytes)
 *
 * Description:  returns pointer to dest
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/4/02
 *
 *-----------------------------------------------------------------
 */
	.set	noreorder

	.globl	fastmemcpy
	.ent	fastmemcpy
fastmemcpy:
	and		v0,a0,1
	bne		v0,zero,memcpy_byte_aligned
	and		v1,a1,1
	bne		v1,zero,memcpy_byte_aligned
	and		v0,a0,2
	bne		v0,zero,memcpy_short_aligned
	and		v1,a1,2
	bne		v1,zero,memcpy_short_aligned
#if __mips >= 3
	and		v0,a0,4
	bne		v0,zero,memcpy_word_aligned
	and		v1,a1,4
	bne		v1,zero,memcpy_word_aligned
#else
	nop
	b		memcpy_word_aligned
#endif

#if __mips >= 3
memcpy_doubleword_aligned:
	move	v1,a0
	srl		v0,a2,6		 /*  (nbytes / 64) */
	beq		v0,zero,memcpy_remain
	nop
memcpy_doubleword_loop:
	ld		t0,0x00(a1)
	ld		t1,0x08(a1)
	ld		t2,0x10(a1)
	ld		t3,0x18(a1)
	ld		t4,0x20(a1)
	ld		t5,0x28(a1)
	ld		t6,0x30(a1)
	ld		t7,0x38(a1)
	ADDIU	a2,a2,-64
	ADDIU	a1,a1,64
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,64
	sd		t0,-64(a0)
	sd		t1,-56(a0)
	sd		t2,-48(a0)
	sd		t3,-40(a0)
	sd		t4,-32(a0)
	sd		t5,-24(a0)
	sd		t6,-16(a0)
	bne		v0,zero,memcpy_doubleword_loop
	sd		t7,-8(a0)
#endif

memcpy_remain:
	beqz	a2,2f
	nop
1:	lbu		t0,0(a1)
	ADDIU	a2,a2,-1
	ADDIU	a1,a1,1
	ADDIU	a0,a0,1
	bne		a2,zero,1b
	sb		t0,-1(a0)
2:	j		ra
	move	v0,v1

memcpy_word_aligned:
	move	v1,a0
	srl		v0,a2,5		 /* (nbytes / 32) */
	beq		v0,zero,memcpy_remain
	nop
memcpy_word_loop:
	lw		t0,0x00(a1)
	lw		t1,0x04(a1)
	lw		t2,0x08(a1)
	lw		t3,0x0C(a1)
	lw		t4,0x10(a1)
	lw		t5,0x14(a1)
	lw		t6,0x18(a1)
	lw		t7,0x1C(a1)
	ADDIU	a2,a2,-32
	ADDIU	a1,a1,32
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,32
	sw		t0,-32(a0)
	sw		t1,-28(a0)
	sw		t2,-24(a0)
	sw		t3,-20(a0)
	sw		t4,-16(a0)
	sw		t5,-12(a0)
	sw		t6,-8(a0)
	bne		v0,zero,memcpy_word_loop
	sw		t7,-4(a0)
	b		memcpy_remain
	nop

memcpy_short_aligned:
	move	v1,a0
	srl		v0,a2,4		 /* (nbytes / 16) */
	beq		v0,zero,memcpy_remain
	nop
memcpy_short_loop:
	lhu		t0,0x00(a1)
	lhu		t1,0x02(a1)
	lhu		t2,0x04(a1)
	lhu		t3,0x06(a1)
	lhu		t4,0x08(a1)
	lhu		t5,0x0A(a1)
	lhu		t6,0x0C(a1)
	lhu		t7,0x0E(a1)
	ADDIU	a2,a2,-16
	ADDIU	a1,a1,16
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,16
	sh		t0,-16(a0)
	sh		t1,-14(a0)
	sh		t2,-12(a0)
	sh		t3,-10(a0)
	sh		t4,-8(a0)
	sh		t5,-6(a0)
	sh		t6,-4(a0)
	bne		v0,zero,memcpy_short_loop
	sh		t7,-2(a0)
	b		memcpy_remain

memcpy_byte_aligned:
	move	v1,a0
	srl		v0,a2,3		 /*  (nbytes / 8) */
	beq		v0,zero,memcpy_remain
	nop
memcpy_byte_loop:
	lbu		t0,0x00(a1)
	lbu		t1,0x01(a1)
	lbu		t2,0x02(a1)
	lbu		t3,0x03(a1)
	lbu		t4,0x04(a1)
	lbu		t5,0x05(a1)
	lbu		t6,0x06(a1)
	lbu		t7,0x07(a1)
	ADDIU	a2,a2,-8
	ADDIU	a1,a1,8
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,8
	sb		t0,-8(a0)
	sb		t1,-7(a0)
	sb		t2,-6(a0)
	sb		t3,-5(a0)
	sb		t4,-4(a0)
	sb		t5,-3(a0)
	sb		t6,-2(a0)
	bne		v0,zero,memcpy_byte_loop
	sb		t7,-1(a0)
	b		memcpy_remain
	nop

	.end	fastmemcpy

/*-----------------------------------------------------------------
 * Name:	void * memcpy(dst, src, nbytes)
 *
 * Description:  returns pointer to dest
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/4/02
 *
 *-----------------------------------------------------------------
 */
	.set	noreorder

	.globl	ultranew_memcpy
	.ent	ultranew_memcpy
ultranew_memcpy:
	and		v0,a0,1
	bne		v0,zero,ultranew_memcpy_byte_aligned
	move	v1,a0

	and		v0,a1,1
	bne		v0,zero,ultranew_memcpy_byte_aligned
	move	v1,a0

	and		v0,a0,2
	bne		v0,zero,ultranew_memcpy_short_aligned
	move	v1,a0

	and		v0,a1,2
	bne		v0,zero,ultranew_memcpy_short_aligned
	move	v1,a0

#if __mips >= 3
	and		v0,a0,4
	bne		v0,zero,ultranew_memcpy_word_aligned
	move	v1,a0

	and		v0,a1,4
	bne		v0,zero,ultranew_memcpy_word_aligned
	move	v1,a0
#else
	b		ultranew_memcpy_word_aligned
	move	v1,a0
#endif

#if __mips >= 3
ultranew_memcpy_doubleword_aligned:
	srl		v0,a2,6		 /*  (nbytes / 64) */
	beq		v0,zero,ultranew_memcpy_remain
	nop
ultranew_memcpy_doubleword_loop:
	ld		t0,0x00(a1)
	ld		t1,0x08(a1)
	ld		t2,0x10(a1)
	ld		t3,0x18(a1)
	ld		t4,0x20(a1)
	ld		t5,0x28(a1)
	ld		t6,0x30(a1)
	ld		t7,0x38(a1)
	ADDIU	a2,a2,-64
	ADDIU	a1,a1,64
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,64
	sd		t0,-64(a0)
	sd		t1,-56(a0)
	sd		t2,-48(a0)
	sd		t3,-40(a0)
	sd		t4,-32(a0)
	sd		t5,-24(a0)
	sd		t6,-16(a0)
	bne		v0,zero,ultranew_memcpy_doubleword_loop
	sd		t7,-8(a0)
	b		ultranew_memcpy_byte_aligned
	nop
#endif

ultranew_memcpy_word_aligned:
	srl		v0,a2,5		 /* (nbytes / 32) */
	beq		v0,zero,ultranew_memcpy_remain
	nop
ultranew_memcpy_word_loop:
	lw		t0,0x00(a1)
	lw		t1,0x04(a1)
	lw		t2,0x08(a1)
	lw		t3,0x0C(a1)
	lw		t4,0x10(a1)
	lw		t5,0x14(a1)
	lw		t6,0x18(a1)
	lw		t7,0x1C(a1)
	ADDIU	a2,a2,-32
	ADDIU	a1,a1,32
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,32
	sw		t0,-32(a0)
	sw		t1,-28(a0)
	sw		t2,-24(a0)
	sw		t3,-20(a0)
	sw		t4,-16(a0)
	sw		t5,-12(a0)
	sw		t6,-8(a0)
	bne		v0,zero,ultranew_memcpy_word_loop
	sw		t7,-4(a0)
	b		ultranew_memcpy_byte_aligned
	nop

ultranew_memcpy_short_aligned:
	srl		v0,a2,4		 /* (nbytes / 16) */
	beq		v0,zero,ultranew_memcpy_remain
	nop
ultranew_memcpy_short_loop:
	lhu		t0,0x00(a1)
	lhu		t1,0x02(a1)
	lhu		t2,0x04(a1)
	lhu		t3,0x06(a1)
	lhu		t4,0x08(a1)
	lhu		t5,0x0A(a1)
	lhu		t6,0x0C(a1)
	lhu		t7,0x0E(a1)
	ADDIU	a2,a2,-16
	ADDIU	a1,a1,16
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,16
	sh		t0,-16(a0)
	sh		t1,-14(a0)
	sh		t2,-12(a0)
	sh		t3,-10(a0)
	sh		t4,-8(a0)
	sh		t5,-6(a0)
	sh		t6,-4(a0)
	bne		v0,zero,ultranew_memcpy_short_loop
	sh		t7,-2(a0)

ultranew_memcpy_byte_aligned:
	srl		v0,a2,3		 /*  (nbytes / 8) */
	beq		v0,zero,ultranew_memcpy_remain
	nop
ultranew_memcpy_byte_loop:
	lbu		t0,0x00(a1)
	lbu		t1,0x01(a1)
	lbu		t2,0x02(a1)
	lbu		t3,0x03(a1)
	lbu		t4,0x04(a1)
	lbu		t5,0x05(a1)
	lbu		t6,0x06(a1)
	lbu		t7,0x07(a1)
	ADDIU	a2,a2,-8
	ADDIU	a1,a1,8
	ADDIU	v0,v0,-1
	ADDIU	a0,a0,8
	sb		t0,-8(a0)
	sb		t1,-7(a0)
	sb		t2,-6(a0)
	sb		t3,-5(a0)
	sb		t4,-4(a0)
	sb		t5,-3(a0)
	sb		t6,-2(a0)
	bne		v0,zero,ultranew_memcpy_byte_loop
	sb		t7,-1(a0)

ultranew_memcpy_remain:
	beqz	a2,2f
	nop
1:	lbu		t0,0(a1)
	ADDIU	a2,a2,-1
	ADDIU	a1,a1,1
	ADDIU	a0,a0,1
	bne		a2,zero,1b
	sb		t0,-1(a0)
2:	j		ra
	move	v0,v1

	.end	ultranew_memcpy

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/02
 *
 *-----------------------------------------------------------------
 */
	.globl	memcpy4
	.ent	memcpy4
memcpy4:
	li		v0,4
	bne		a2,v0,ultranew_memcpy
	nop
	lbu		t0,0(a1)
	lbu		t1,1(a1)
	lbu		t2,2(a1)
	lbu		t3,3(a1)
	sb		t0,0(a0)
	sb		t1,1(a0)
	sb		t2,2(a0)
	sb		t3,3(a0)
	j		ra
	move	v0,a0

	.end	memcpy4

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/02
 *
 *-----------------------------------------------------------------
 */
	.globl	memcpy6
	.ent	memcpy6
memcpy6:
	li		v0,6
	bne		a2,v0,ultranew_memcpy
	nop
	lbu		t0,0(a1)
	lbu		t1,1(a1)
	lbu		t2,2(a1)
	lbu		t3,3(a1)
	lbu		t4,4(a1)
	lbu		t5,5(a1)
	sb		t0,0(a0)
	sb		t1,1(a0)
	sb		t2,2(a0)
	sb		t3,3(a0)
	sb		t4,4(a0)
	sb		t5,5(a0)
	j		ra
	move	v0,a0

	.end	memcpy6

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/02
 *
 *-----------------------------------------------------------------
 */
	.globl	memcpy40
	.ent	memcpy40
memcpy40:
	li		v0,40
	bne		a2,v0,ultranew_memcpy
	nop

	ld		t0,0x00(a1)
	ld		t1,0x08(a1)
	ld		t2,0x10(a1)
	ld		t3,0x18(a1)
	ld		t4,0x20(a1)
	sd		t0,0x00(a0)
	sd		t1,0x08(a0)
	sd		t2,0x10(a0)
	sd		t3,0x18(a0)
	sd		t4,0x20(a0)
	j		ra
	move	v0,a0

	.end	memcpy40

