/*
 *-----------------------------------------------------------------
 *
 * Name:	vsprintf.c
 *
 * RCS:  $Id: vsprintf.c,v 1.9 2002/04/06 02:49:20 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/7/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifndef LINUX_TEST

#include "libc.h"

#undef FLOATINGPT

int32
vsprintf(
	char8		* d,
	const char8	* s,
	va_list		ap
)
{
	const char8	* t;
	char8		* p, * dst, tmp[40];
	uint32		n;
	int32		fmt, trunc, haddot, width, base = 0, longlong;
#ifdef FLOATINGPT
	double		dbl;
#endif

	dst = d;
	for (; *s;) {
		if (*s == '%') {
			s++;
			fmt = FMT_RJUST;
			width = trunc = haddot = longlong = 0;
			for (; *s; s++) {
				if (strchr ("dobpPxXulscefg%", *s))
					break;
				else if (*s == '-')
					fmt = FMT_LJUST;
				else if (*s == '0')
					fmt = FMT_RJUST0;
				else if (*s == '~')
					fmt = FMT_CENTER;
				else if (*s == '*') {
					if (haddot)
						trunc = va_arg (ap, int);
					else
						width = va_arg (ap, int);
				} else if (*s >= '1' && *s <= '9') {
					for (t = s; isdigit (*s); s++);
						strncpy (tmp, t, s - t);
					tmp[s - t] = '\0';
					atob (&n, tmp, 10);
					if (haddot)
						trunc = n;
					else
						width = n;
					s--;
				} else if (*s == '.')
					haddot = 1;
			}
			if (*s == '%') {
				*d++ = '%';
				*d = 0;
			} else if (*s == 's') {
				p = va_arg (ap, char *);

				if (p)
					strcpy (d, p);
				else
					strcpy (d, "(null)");
			} else if (*s == 'c') {
				n = va_arg (ap, int);

				*d = n;
				d[1] = 0;
			} else {
				if (*s == 'l') {
#if __long64
					longlong = 1;
#endif
					if (*++s == 'l') {
						longlong = 1;
						++s;
					}
				}
				if (*s == 'p') {
#if __long64
					longlong = 1;
#endif
					*d++ = '0';
					*d++ = 'x';
				}
				if (strchr ("dobpPxXu", *s)) {
					if (*s == 'd')
						base = -10;
					else if (*s == 'u')
						base = 10;
					else if (*s == 'x' || *s == 'X' || *s == 'p' || *s == 'P')
						base = 16;
					else if (*s == 'o')
						base = 8;
					else if (*s == 'b')
						base = 2;
#if  __mips >= 3
					if (longlong)
						llbtoa (d, va_arg (ap, long long), base);
					else
#endif
						btoa (d, va_arg (ap, int), base);

					if (*s == 'X' || *s == 'P')
						strtoupper (d);
				}
#ifdef FLOATINGPT
				else if (strchr ("eEfgG", *s)) {
					/* convert floats */
					static void dtoa (char *, double, int, int, int);
					dbl = va_arg (ap, double);
					dtoa (d, dbl, *s, width, trunc);
					trunc = 0;
				}
#endif
			}
			if (trunc)
				d[trunc] = 0;
			if (width)
				str_fmt (d, width, fmt);
			for (; *d; d++);
			s++;
		} else
			*d++ = *s++;
	}

	*d = 0;

	return (d - dst);

}	/* vsprintf () */

#endif	/* LINUX_TEST */
