/*
 *-----------------------------------------------------------------
 *
 * Name:	load-auto.c
 *
 * RCS:  $Id: load-auto.c,v 1.23 2002/10/31 01:02:37 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/11/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/cm-api.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>
#include <exports/eee-api.h>
#include <exports/mem-api.h>
#include "load.h"

uint32 load_ssc_ip_addr = 0xc0a8c001;		/* 192.168.192.1 */

#ifdef NEW_LOADER

#define CNTRL(x)	((x) & 0x1f)

extern unsigned long	pipefreq;
extern int warm_start;

void load_launch_embedded();
void load_launch_bsd(address_t addr, int32 argc, uchar8 * argv[]);

#if defined(PROM_R9000)
#define LOAD_BSD_ADDRESS	(0x83000080)
#else
#define LOAD_BSD_ADDRESS	(0x80100080)
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	Loads runtime images from SSC.
 *
 *				Returns NFX_OK if successfully loaded, -1 for error.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
int32
load_module_fc()
{
	uint32	ipaddr;
	uchar8	fname[80];
	int32	rc;
	
	load_get_fname(LOADER_FNAME_SP, nfxMySlotId, nfxMyCPUId, fname);

	get_external_addr();
		
	ipaddr = load_ssc_ip_addr;

	rc = load_net(fname, ipaddr, (int)0x83000000, NULL);
/*
	rc = load_net(fname, ipaddr, EMBEDDED_ADDR, NULL);
*/

	if (rc != NFX_OK)
		return LOADER_ERR;

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
void
load_set_fp_boot_block(uint32 nbytes)
{
	volatile load_fp_boot_block_t	* bb = (load_fp_boot_block_t *)0x73100000;

	bb->src = (uchar8 *)0x40000000;
	bb->dest = (uchar8 *)(int)0x83000000;
	bb->len = nbytes;

	delay_us(1000);

	bb->signature = FP_BOOT_BLOCK_SIGNATURE;
}

void
load_set_fp_jump_block()
{
	volatile load_fp_jump_block_t	* jb = (load_fp_jump_block_t *)0x73100020;

	jb->entry_addr = (address_t)(int)0x83000000;

	delay_us(1000);

	jb->signature = FP_JUMP_BLOCK_SIGNATURE;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
load_fp_boot_block_t *
load_get_fp_boot_block()
{
	volatile load_fp_boot_block_t	* bb = (load_fp_boot_block_t *)0x73100000;

	bb->signature = 0;

	while (bb->signature != FP_BOOT_BLOCK_SIGNATURE) {
		if (char_avail()) {
			if (getchar() == CNTRL('e'))
				return 0;
		}
#ifdef USE_WATCHDOG
		watchdog_kick();
#endif
		delay_us(100000);
	}

	return (load_fp_boot_block_t *)bb;
}

load_fp_jump_block_t *
load_get_fp_jump_block()
{
	volatile load_fp_jump_block_t	* jb = (load_fp_jump_block_t *)0x73100020;

	jb->signature = 0;

	while (jb->signature != FP_JUMP_BLOCK_SIGNATURE) {
		if (char_avail()) {
			if (getchar() == CNTRL('e'))
				return 0;
		}
#ifdef USE_WATCHDOG
		watchdog_kick();
#endif
		delay_us(100000);
	}

	return (load_fp_jump_block_t *)jb;
}


uint32 hostid;

void
recv_hostid(eee_descPtr_t desc)
{
	hostid = *((uint32 *)edesc_firstBuf(desc)->buf);
	eee_freePacket(desc);
}

void
get_external_addr()
{
	uint32 appid = eee_getApplicationId("hostid");
	uint32 dummy;
	uint32 dp, sp;
	if (eee_registerReceiveFunc(appid, recv_hostid, recv_hostid) == NFX_ERR) {
		panic("get_external_addr: can not register receive func\n");
	}

	while (!hostid) {
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, appid,
									   EEE_MGMT, NFX_COS_HIGH, 0);
		sp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId, appid,
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&dummy, sizeof(dummy), dp, sp) != NFX_OK) {
			panic("get_external_addr: can not send message\n");
		}
		delay_us(500000);
		eee_poll(1);
		if (char_avail()) {
			if (getchar() == CNTRL('e'))
				return;
		}
	}
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description: Loads runtime images for FPNIM.
 *
 *				Returns NFX_OK if successfully loaded, -1 for error.
 *
 *				Loads FP image, sends boot block to FP, then loads TXRX image.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
int32
load_module_nfp()
{
	uint32	ipaddr;
	uchar8	fname[80];
	int32	rc;
	uint32	fp_len;

	if (isTXRX()) {
		load_get_fname(LOADER_FNAME_FP, nfxMySlotId, nfxMyCPUId, fname);

		ipaddr = load_ssc_ip_addr;

		get_external_addr();

		rc = load_net(fname, ipaddr, 0x40000000, &fp_len);
		if (rc != NFX_OK)
			return LOADER_ERR;

printf("setting boot_block\n");
		load_set_fp_boot_block(fp_len);

		load_get_fname(LOADER_FNAME_TXRX, nfxMySlotId, nfxMyCPUId, fname);

/*
		rc = load_net(fname, ipaddr, EMBEDDED_ADDR, NULL);
*/
		rc = load_net(fname, ipaddr, (int)0x83000000, NULL);

		if (rc == NFX_OK) {
			load_set_fp_jump_block();
			return NFX_OK;
		}

		return LOADER_ERR;
	}
	else if (isFP()) {
		load_fp_boot_block_t	* bb;
		load_fp_jump_block_t	* jb;

printf("getting boot_block\n");
		bb = load_get_fp_boot_block();
		if (bb == 0)
			return LOADER_BREAK;

printf("copying 0x%x bytes from 0x%x to 0x%x\n", bb->len, bb->src, bb->dest);
		memcpy(bb->dest, bb->src, bb->len);

		jb = load_get_fp_jump_block();
		if (jb == 0)
			return LOADER_BREAK;

printf("set_entry_addr to 0x%x\n", jb->entry_addr);
		set_entry_addr(jb->entry_addr);

		return NFX_OK;
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
int32
load_module_ssc()
{
	uchar8	envchar;
	uchar8	mgmt_file[80];
	uchar8	bsd_file[80];
	uchar8	str[ENV_MAX_STR_LEN];


	uchar8	*av[1];

#if defined(PROM_R7000) || defined(PROM_R9000)


#ifdef PROM_R7000
	if (!warm_start)
		load_set_toshiba_boot_block();
#endif

	if (seepenv_get(ENV_KEY_LOAD_SRC, str) != NFX_OK)
		return LOADER_BREAK;

	if (!strcmp(str, "network"))
		return LOADER_BREAK;

	if (seepenv_get(ENV_KEY_BOOT_DEV, str) != NFX_OK)
		return LOADER_BREAK;

	sprintf(mgmt_file, "/dev/%s/ssc-mgmt.bin", str);
	sprintf(bsd_file, "/dev/%s/bsd", str);

#ifdef PROM_R7000
	if (!warm_start) {
		if (disk_load_file(mgmt_file, 0xb0008000)) {
			printf("Error loading %s\n", mgmt_file);
			return LOADER_BREAK;
		}
	}
#endif

	if (disk_load_file(bsd_file, LOAD_BSD_ADDRESS)) {
		printf("Error loading %s\n", bsd_file);
		return LOADER_BREAK;
	}

#ifdef PROM_R7000
	if (!warm_start)
		load_set_toshiba_jump_block();
#endif

//	load_launch_bsd((address_t)LOAD_BSD_ADDRESS);			/* will not return */
        av[0] = (uchar8 *) malloc(80 * sizeof(uchar8));
        strcpy(av[0], "bsd");

        load_launch_bsd((address_t)LOAD_BSD_ADDRESS, 1, av);

#endif

	return LOADER_BREAK;
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
#if defined(PROM_R9000) || defined(PROM_SIBYTE_BC)
int32
autoload(void)
{
	return autoload_bc();
}
#else
int32
autoload()
{
	int32	rc = LOADER_ERR;
	int32	cnt = 0;

#ifdef NEW_LOADER
	cmSetCPUState(CM_SLICE_STATE_PROM_LOADING);

	while (rc != NFX_OK) {

		if (isTXRX() || isFP()) {
			rc = load_module_nfp();
		}	
		else if (isFC())
			rc = load_module_fc();
		else if (isSSC())
			rc = load_module_ssc();

		if (rc == LOADER_BREAK)
			break;
		if (rc != NFX_OK) {
			printf("Error auto-loading images, retry cnt = %d\n", ++cnt);
			delay_us(1000000);
		}
		if (char_avail()) {
			if (getchar() == CNTRL('e'))
				break;
		}
	}
#endif

	if (rc == NFX_OK)
		load_launch_embedded();

	return rc;
}
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
int32
do_autoload(
	int32	argc,
	uchar8	* argv[]
)
{
	autoload();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/17/02
 *
 *-----------------------------------------------------------------
 */
void
load_launch_embedded()
{
	cmSetCPUState(CM_SLICE_STATE_SWITCH_TO_RUNTIME);
	
	do_cmd("g");
}

#if defined (PROM_R7000) || defined (PROM_R9000)

extern uint32 util_routines;


typedef int	(*PFI)();

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/17/02
 *
 *-----------------------------------------------------------------
 */
void
load_launch_bsd(
        address_t  addr,
        int32      argc,
        uchar8     * argv[]
)
{
	uchar8	* env_strings[16];
	uchar8	bsd_env_strings[16][80];
	uchar8	str[ENV_MAX_STR_LEN];
	uchar8	src_str[ENV_MAX_STR_LEN];
	uchar8  bootdev_str[ENV_MAX_STR_LEN];
	int32	i = 0, j;
	uchar8	ipaddr_str[20];
	uchar8	ipmask_str[20];
	uchar8	macaddr0_str[6];
	uchar8	macaddr1_str[6];
 
        address_t * ptr_prom_util_routines = (address_t *)&util_routines;

	seepenv_get(ENV_KEY_LOAD_SRC, src_str);
	seepenv_get(ENV_KEY_LOAD_OPTS, str);
	seepenv_get(ENV_KEY_BOOT_DEV, bootdev_str);

	sprintf(bsd_env_strings[i++], "cpuclock=%u", pipefreq);                 // [0]
#ifdef PROM_R9000

	// give only half the memory to SSC, the other half to FC
	sprintf(bsd_env_strings[i++], "memsize=%d", memorysize/(2*1024*1024));    // [1]
#else
	sprintf(bsd_env_strings[i++], "memsize=%d", memorysize/(1024*1024));    // [1]
#endif
	sprintf(bsd_env_strings[i++], "osloadoptions=%s", str);                 // [2]
	sprintf(bsd_env_strings[i++], "boot=%s", warm_start ? "warm" : "cold"); // [3]
	sprintf(bsd_env_strings[i++], "busclock=%d", get_busclock());           // [4]
	bzero(ipaddr_str, 20);
	bzero(ipmask_str, 20);
	seepGetIpAddr(ipaddr_str);
	seepGetIpMask(ipmask_str);
	sprintf(bsd_env_strings[i++], "ipaddr=%s", ipaddr_str);                 // [5]
	sprintf(bsd_env_strings[i++], "netmask=%s", ipmask_str);                // [6]

	memcpy(macaddr0_str, seepGetBSDMacAddr(0), 6);
	memcpy(macaddr1_str, seepGetBSDMacAddr(1), 6);
	sprintf(bsd_env_strings[i], "macaddr0=");
	memcpy(&(bsd_env_strings[i][strlen(bsd_env_strings[i])]), macaddr0_str, 6); // [7]
	i++;
	sprintf(bsd_env_strings[i], "macaddr1=");
	memcpy(&(bsd_env_strings[i][strlen(bsd_env_strings[i])]), macaddr1_str, 6); // [8]
	i++;

	if (!strcmp(src_str, "disk"))
		sprintf(bsd_env_strings[i++], "bootdev=/dev/%s", bootdev_str); // [9]

	for (j=0; j<i; j++)
		env_strings[j] = &(bsd_env_strings[j][0]);
	env_strings[j] = NULL;

	for (j=0; j<i; j++) {
        if (j == 7 || j == 8) { 
            // for items 7 & 8 the mac addr is in binary format
            printf("env[%d] = %x:.%s.%02x:%02x:%02x:%02x:%02x:%02x.\n",
                    j, env_strings[j],env_strings[j],
                    env_strings[j][9],
                    env_strings[j][10],
                    env_strings[j][11],
                    env_strings[j][12],
                    env_strings[j][13],
                    env_strings[j][14]);
        }
        else {
            printf("env[%d] = %x:.%s.\n", j, env_strings[j], env_strings[j]);
        }
    }

//	av[0] = (uchar8 *) malloc(80 * sizeof(uchar8));
//	strcpy(av[0], "bsd");


if( argc > 1) {
 printf(" Load options and params for [%s] \n",argv[0]);
 printf("  Address %X argc = %d \n",addr,argc);
 for( i = 0; i < argc; i++) {
    printf("   argv [%d] = %s \n",i,argv[i]);
 }
}
printf(" pointer to Prom Util routines = %X \n",ptr_prom_util_routines);
printf(" Command should be  (addr)(argc, argv, env_strings, ptr_prom_util_routines) \n\n\n");
// return;

//	(*(PFI)addr)(1, av, env_strings);

        (*(PFI)addr)(argc, argv, env_strings, ptr_prom_util_routines);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
int32
do_bsd_launch(
	int32	argc,
	uchar8	* argv[]
)
{
	//load_launch_bsd((address_t)LOAD_BSD_ADDRESS);
        if( argc > 1)
            printf(" do_bsd_launch argc = %d argv[%d] = %s \n\n",argc,argc,argv[(argc-1)]);

        if( argc == 0 ) {       
            argv[0] = (uchar8 *) malloc(80 * sizeof(uchar8));
            strcpy(argv[0], "bsd");
            argc=1;
            printf(" -- Do BSD launch (argc = 0)\n\n");
        }
        else if (( argc == 3) && (!strcmp(argv[2],"bsd"))) {
            argv[0] = (uchar8 *) malloc(80 * sizeof(uchar8));
            strcpy(argv[0], "bsd");
            argc=1;
            printf(" -- Do BSD launch (argc = 3 && bsd)\n\n");
        }

        load_launch_bsd((address_t)LOAD_BSD_ADDRESS, argc, argv);
	return 0;
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
int32
env_autoload()
{
	uchar8	auto_str[ENV_MAX_STR_LEN];

	 seepenv_get(ENV_KEY_AUTOLOAD, auto_str);
	if (!strcmp(auto_str, "on"))
		return 1;
	else
		return 0;
}

