/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat-diag-err.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2005, OnStor Inc.
 *
 * Description: Bobcat platform related auto diag Error code.
 *              Gets Status from Auto Diag tests and prints
 *              error info after tests are done. This is helpful
 *              when TXRX or FP return failure status. 
 *              Please see "load-bobcat-diag.h" for definitions
 *              of the returned status.
 *
 * Created by:  Warren Gale
 *
 * Date Created: 4/21/05
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "load-bobcat.h"
#include "load-bobcat-diag.h"


/***************************************************
* This takes the returned status and cuts it up 
* into all it's little pieces...
*  Group Number
*   Test Number
*    Port Number
*
*  Then it's a big switch to print the correct error
*****************************************************/

void 
autoDiagErrorPrint( uint32 autoTestError )
{
 uint32 adgroup = 0;
 uint32 adtest = 0;
 uint32 adport = 0;

    adgroup = autoTestError & 0xFF000000; // get group number
    adtest = autoTestError & 0x00FF0000; // get test number
    adport = autoTestError & 0x0000F000; // get port number

    switch( adgroup) {

        case DIAG_TEST_GROUP_INFO: {
            printf("  -- Test Info  Can NOT determin Module [SSC, TXRX, or FP] \n ");
            break;
        }
        
        case DIAG_TEST_GROUP_PWRFAN: {
            printf("  -- Test PWR/FAN ");
            switch (adtest) {
                case DIAG_TEST_PWR0: {
                    printf(" PWR 0 ");
                    break;
                }

                case DIAG_TEST_PWR1: {
                    printf(" PWR 1 ");
                    break;
                }

                case DIAG_TEST_FAN0: {
                    printf(" Fan control 0 ");
                    break;
                }

                case DIAG_TEST_FAN1: {
                    printf(" Fan control 1 ");
                    break;
                }

                case DIAG_TEST_FAN_SPEED: {
                    printf(" Fan at High Speed ");
                    break;
                }

                default: {
                    printf(" - Diag Software Error No Test number.");
                    break;
                }

            } // end switch test number

            break;
        }
        
        case DIAG_TEST_GROUP_PCI: {
            printf("  -- Test PCI  ");
            switch (adtest) {
                case DIAG_TEST_GT_PCI: {
                    printf(" GT PCI ");
                    break;
                }

                case DIAG_TEST_TXRX_PCI: {
                    printf(" TXRX PCI ");
                    break;
                }

                case DIAG_TEST_FP_PCI: {
                    printf(" FP PCI ");
                    break;
                }

                case DIAG_TEST_QLOGIC_PCI0: {
                    printf(" QLogic PCI 0 ");
                    break;
                }

                case DIAG_TEST_QLOGIC_PCI1: {
                    printf(" QLogic PCI 1 ");
                    break;
                }

                case DIAG_TEST_BRIDGE_PCI: {
                    printf(" Bridge PCI ");
                    break;
                }

                case DIAG_TEST_MAC0_PCI: {
                    printf(" MAC 0 PCI ");
                    break;
                }

                case DIAG_TEST_MAC1_PCI: {
                    printf(" MAC 1 PCI ");
                    break;
                }

                case DIAG_TEST_CF_PCI: {
                    printf(" CF PCI ");
                    break;
                }

                default: {
                    printf(" - Diag Software Error No Test number.");
                    break;
                }

            } // end switch test number
            break;
        }
        
        case DIAG_TEST_GROUP_I2C: {
            printf("  -- Test I2C  ");
            switch (adtest) {
                case DIAG_TEST_BOARD_SEEP: {
                    printf(" Board SEEP ");
                    break;
                }

                case DIAG_TEST_R9K_DRAM: {
                    printf(" R9K Dram SEEP ");
                    break;
                }

                case DIAG_TEST_GT_DRAM: {
                    printf(" GT Dram SEEP ");
                    break;
                }

                case DIAG_TEST_DRAM0_SEEP: {
                    printf(" Dram 0 SEEP ");
                    break;
                }

                case DIAG_TEST_DRAM1_SEEP: {
                    printf(" Dram 1 SEEP ");
                    break;
                }

                case DIAG_TEST_DRAM1_EXTRA: {
                    printf(" Extra Dram 1  ");
                    break;
                }

                case DIAG_TEST_DRAM1_MISS: {
                    printf(" Dram 1  ");
                    break;
                }

                case DIAG_TEST_DRAM2_SEEP: {
                    printf(" Dram 2 SEEP ");
                    break;
                }

                case DIAG_TEST_DRAM2_EXTRA: {
                    printf(" Extra Dram 2  ");
                    break;
                }

                case DIAG_TEST_DRAM2_MISS: {
                    printf(" Dram 2  ");
                    break;
                }

                case DIAG_TEST_DRAM3_SEEP: {
                    printf(" Dram 3 SEEP ");
                    break;
                }

                case DIAG_TEST_DRAM3_EXTRA: {
                    printf(" Extra Dram 3  ");
                    break;
                }

                case DIAG_TEST_DRAM3_MISS: {
                    printf(" Dram 3  ");
                    break;
                }

                case DIAG_TEST_MUX_READ: {
                    printf(" MUX  ");
                    break;
                }

                case DIAG_TEST_FAN0_READ: {
                    printf(" Fan Control 0  ");
                    break;
                }

                case DIAG_TEST_FAN1_READ: {
                    printf(" Fan Control 1  ");
                    break;
                }

                case DIAG_TEST_TEMPSEN_READ: {
                    printf(" Temp Sense  ");
                    break;
                }

                case DIAG_TEST_MODEL_JUMPER: {
                    printf(" FMUX Mode Jumper Mismatch  ");
                    break;
                }

                default: {
                    printf(" - Diag Software Error No Test number.");
                    break;
                }

            } // end switch test number
            break;
        }
        
        case DIAG_TEST_GROUP_DEV: {
            printf("  -- Test Device  ");
            switch (adtest) {
                case DIAG_TEST_BMFPGA_REV: {
                    printf(" BMFPGA version ");
                    break;
                }

                case DIAG_TEST_BMFPGA_REG: {
                    printf(" BMFPGA register ");
                    break;
                }

                case DIAG_TEST_BMFPGA_STAT: {
                    printf(" BMFPGA Status ");
                    break;
                }

                case DIAG_TEST_CF_INIT: {
                    printf(" CF Init ");
                    break;
                }

                case DIAG_TEST_CF_DEV_OK: {
                    printf(" CF not in Socket ");
                    break;
                }

                case DIAG_TEST_ETH_LINK: {
                    printf(" 10/100 Eth Link ");
                    break;
                }

                case DIAG_TEST_LUC_REV: {
                    printf(" LUC Revision ");
                    break;
                }

                case DIAG_TEST_LUC_PHY_ID: {
                    printf(" LUC PHY ID ");
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf("  Device 0 ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf("  Device 1 ");
                    if( (adport & DIAG_PORT_2_ERROR))
                        printf("  Device 2 ");
                    if( (adport & DIAG_PORT_3_ERROR))
                        printf("  Device 3 ");
                    break;
                }

                case DIAG_TEST_LSI_REG: {
                    printf(" LSI register ");
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf("  Device 0 ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf("  Device 1 ");
                    if( (adport & DIAG_PORT_2_ERROR))
                        printf("  Device 2 ");
                    if( (adport & DIAG_PORT_3_ERROR))
                        printf("  Device 3 ");
                    break;
                }

                case DIAG_TEST_GIGE_AD_SFP: {
                    printf(" GigE auto detect SFP Error  ");
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf("  Port 0 ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf("  Port 1 ");
                    if( (adport & DIAG_PORT_2_ERROR))
                        printf("  Port 2 ");
                    if( (adport & DIAG_PORT_3_ERROR))
                        printf("  Port 3 ");
                    break;
                }

                case DIAG_TEST_GIGE_LUCSETUP: {
                    printf(" GigE LUC init ");
                    // port bits tell me if its MAC 0 or MAC 2
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf("  MAC 0 Link 0 ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf("  MAC 0 Link 1 ");
                    if( (adport & DIAG_PORT_2_ERROR))
                        printf("  MAC 2 Link 2 ");
                    if( (adport & DIAG_PORT_3_ERROR))
                        printf("  MAC 2 Link 3 ");
                    break;
                }

                case DIAG_TEST_GIGE_PKT: {
                    printf(" GigE packet count  ");
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf("  Port 0 ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf("  Port 1 ");
                    if( (adport & DIAG_PORT_2_ERROR))
                        printf("  Port 2 ");
                    if( (adport & DIAG_PORT_3_ERROR))
                        printf("  Port 3 ");
                    break;
                }

                case DIAG_TEST_GIGE_PKT_ECNT: {
                    printf(" GigE error counts  \n");
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf("  Port 0 ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf("  Port 1 ");
                    if( (adport & DIAG_PORT_2_ERROR))
                        printf("  Port 2 ");
                    if( (adport & DIAG_PORT_3_ERROR))
                        printf("  Port 3 ");
                    break;
                }

                case DIAG_TEST_SERDES_LB: {
                    printf(" SERDES internal loop-back  ");
                    break;
                }

                case DIAG_TEST_RTC_REG: {
                    printf(" RTC register error  ");
                    break;
                }

                default: {
                    printf(" - Diag Software Error No Test number.");
                    break;
                }

            } // end switch test number
            break;
        }
        
        case DIAG_TEST_GROUP_MEM: {
            printf("  -- Test Memory  ");
            switch (adtest) {
                case DIAG_TEST_MEM_HT: {
                    printf(" Hyper Transport ");
                    break;
                }

                case DIAG_TEST_MEM_TXRX_SSC: {
                    printf(" TXRX to SSC Shared Mem");
                    break;
                }

                case DIAG_TEST_MEM_SSC_TXRX: {
                    printf(" SSC to TXRX Shared Mem");
                    break;
                }

                case DIAG_TEST_MEM_FP_SSC: {
                    printf(" FP to SSC Shared Mem");
                    break;
                }

                case DIAG_TEST_MEM_SSC_FP: {
                    printf(" SSC to FP Shared Mem");
                    break;
                }

                case DIAG_TEST_MEM_FP_DRAM0: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 0  (J0901)");
                    break;
                }

                case DIAG_TEST_MEM_FP_DRAM1: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 1  (J0902)");
                    break;
                }

                case DIAG_TEST_MEM_FP_DRAM2: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 2  (J1001)");
                    break;
                }

                case DIAG_TEST_MEM_FP_DRAM3: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 3  (J1002)");
                    break;
                }

                case DIAG_TEST_MEM_TXRX_DRAM0: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 0  (J2301)");
                    break;
                }

                case DIAG_TEST_MEM_TXRX_DRAM1: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 1  (J2302)");
                    break;
                }

                case DIAG_TEST_MEM_TXRX_DRAM2: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 2  (J2401)");
                    break;
                }

                case DIAG_TEST_MEM_TXRX_DRAM3: {
                    if( (adport & DIAG_PORT_0_ERROR))
                        printf(" L2 ECC Error  ");
                    if( (adport & DIAG_PORT_1_ERROR))
                        printf(" Mem ECC Error  ");
                    printf(" DRAM 3  (J2402)");
                    break;
                }
                // these 2 cases are when we see single bit errors and No address
                // to check for a DIMM
                case DIAG_TEST_MEM_L2_ECC_ERROR: {
                    printf(" DRAM L2 ECC Error");
                    break;
                }

                case DIAG_TEST_MEM_ECC_ERROR: {
                    printf(" DRAM Mem ECC Error");
                    break;
                }

                case DIAG_TEST_MEM_GT_MAIN: {
                    printf(" GT Main ");
                    break;
                }

                case DIAG_TEST_MEM_R9K_MAIN: {
                    printf(" R9K Main ");
                    break;
                }

                case DIAG_TEST_MEM_R9K_SCRATCH: {
                    printf(" R9K Scratch ");
                    break;
                }

                case DIAG_TEST_MEM_GT_SCRATCH: {
                    printf(" GT Scratch ");
                    break;
                }

                case DIAG_TEST_MEM_DRAM_SIZE: {
                    printf(" DRAM Size");
                    break;
                }

                case DIAG_TEST_MEM_DRAM_ERROR: {
                    printf(" DRAM memtest ECC Error");
                    break;
                }

                default: {
                    printf(" - Diag Software Error No Test number.");
                    break;
                }

            } // end switch test number
            break;
        }
        
        default: {
            printf("\n Diag Software Error.  No Group Number\n");
            break;
        }

    } // end switch group

}
