/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat-diag-tests.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Bobcat platform related auto diag code.
 *
 * Created by:  Warren Gale
 *
 * Date Created: 6/02/05
 *
 *-----------------------------------------------------------------
 */
#if defined(PROM_R9000) || defined(PROM_SIBYTE_BC)
                                                                                                                                    
#include <nfx-incl.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>

#include <exports/i2c-api.h>
#include <exports/hwMap.h>
#include <exports/pci-api.h>
#include <exports/seep.h>
#include <exports/gt64440mii.h>
#include <sys/bmfpga.h>

#if defined(PROM_SIBYTE_BC)
#include <exports/bmc12500-eth.h>
#include <exports/lsi8104.h>
#include <sys/sb1250/sb1250_defs.h>
#include <sys/sb1250/sb1250_regs.h>
#endif
#include "load-bobcat.h"
#include "load-bobcat-diag.h"


// for memtest termination after 1 pass
unsigned long admenu_running_mtest = 0;

#ifdef PROM_SIBYTE_BC

// these are here cuz exports didn't have then and in the h file it doesn't know about
//  the uint32 ... types without lots of other stuff....
extern int lsi_gmac_write(int, uint32, uint16);
extern int lsi_gmac_read(int, uint32);

extern unsigned long long  memorysize;
// For DIMM callout in diags
volatile unsigned long long diagDramErrorAddr = 0;
unsigned long dramEccErr = 0;
// for memtest termination after 1 pass
//unsigned long admenu_running_mtest = 0;

//-----------------------------------------------
// I2C read tests
//-----------------------------------------------
int
ad_i2c_tst()
{                                                                                                                              
 int testGroup = 0x00000000;
 int rc, diagModelNum=0;
 int status = 0;
 unsigned char   buf[20];
 volatile uint64 * fp_txrx_gpio;
 uint32 rtxrxgpio = 0;



    // This requires that the board has had it's model number already programmed
    // Test the number of DIMMS as per Model number 
    diagModelNum = diag_get_model_info();

    printf(" -I2C read tests\n");
    testGroup = DIAG_TEST_GROUP_I2C;  // Set group Number 
    // SEEP 
    rc = i2c(I2C_READ, 0, 0x50, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ, rc = 0x%x\n", rc);
        return (status = (testGroup | AUTO_DIAG_TEST_ERROR | DIAG_TEST_BOARD_SEEP));
    }

    // ------------------------------------------------------------------------------------
    // New test for 2280 jumper in correct position.  Model 2280 should be in Fifo mode 
    //  models 22[2,4,6]0 should be in GMII mode
    //  read the TXRX GPIO reg from FP
    if( isFP())
        fp_txrx_gpio = (uint64 *)0x900000e010061aa0;
    else  // or txrx read my own GPIO
        fp_txrx_gpio = (uint64 *)0xffffffffb0061aa0;
    rtxrxgpio = *fp_txrx_gpio;
    if( diagModelNum == 2280) {
        // bit 9 determines what mode the FMUX is in  0 = GMII  (Ethernet)   1 = FIFO
        if( !(rtxrxgpio & 0x00000200) ) {
            return (status = (testGroup | AUTO_DIAG_TEST_ERROR | DIAG_TEST_MODEL_JUMPER));
        }
    }
    // test for 2220 2240 2260 make sure jumper is in GMII mode
    else {
        if( (rtxrxgpio & 0x00000200) ) {
            return (status = (testGroup | AUTO_DIAG_TEST_ERROR | DIAG_TEST_MODEL_JUMPER));
        }
    }

    // DRAM 0      All models  (this used to be 2220 Only , but not any more) 
    rc = i2c(I2C_READ, 0, 0x54, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ DRAM 0, rc = 0x%x\n", rc);
        return (status = (testGroup | AUTO_DIAG_TEST_ERROR | DIAG_TEST_DRAM0_SEEP));
    }

    // DRAM 2    2240 will have Dimm 0 & 2 (2220 can have 2 512 Meg dimms now)
    rc = i2c(I2C_READ, 0, 0x56, 0, 1, buf);
       // special case to allow 2220 with 1 dimm, but error if 2240 and above have missing dimm
    if ((rc < 0) && ( diagModelNum >= 2240)) {   // special case to allow 2220 with 2 dimms but error if 2240
        printf("Error in I2C_READ DRAM 2, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup |DIAG_TEST_DRAM2_MISS));
    }
    // case where ic2 works but model number says we shouldn't get a valid read
    // 2220 new Dram config allow 2 Dimm's 
    //else if((rc == 0) && ( diagModelNum < 2240)) { 
    //    printf("Error extra DRAM 2 found and Model Number Mis-match\n");
    //    return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_DRAM2_EXTRA));
    //}
    else { // continue  do nothing 
        // other 2 cases  i2c read works and model match     (Good)
        //  or            12c read fails and model mis match (Dont care)
    }

    // DRAM 1   2260 will have dimm 0 , 2, 1, & 3
    rc = i2c(I2C_READ, 0, 0x55, 0, 1, buf);
    if ((rc < 0) && ( diagModelNum > 2240)) {
        printf("Error in I2C_READ DRAM 1, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup |DIAG_TEST_DRAM1_MISS));
    }
     // case where ic2 works but model number says we shouldn't get a valid read
    else if((rc == 0) && ( diagModelNum < 2260)) {
        printf("Error extra DRAM 1 found and Model Number Mis-match\n");
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_DRAM1_EXTRA));
    }
    else { // continue  do nothing 
        // other 2 cases  i2c read works and model match     (Good)
        //  or            12c read fails and model mis match (Dont care)
    }

    // DRAM 3
    rc = i2c(I2C_READ, 0, 0x57, 0, 1, buf);
    if ((rc < 0) && (diagModelNum > 2240)) {
        printf("Error in I2C_READ DRAM 3, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_DRAM3_MISS));
    }
     // case where ic2 works but model number says we shouldn't get a valid read
    else if((rc == 0) && ( diagModelNum < 2260)) {
        printf("Error extra DRAM 3 found and Model Number Mis-match\n");
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_DRAM3_EXTRA));
    }
    else { // continue  do nothing 
        // other 2 cases  i2c read works and model match     (Good)
        //  or            12c read fails and model mis match (Dont care)
    }

    // MUX
    if( isTXRX()) {
        rc = i2c(I2C_READ, 1, 0x70, 0, 1, buf);
        if (rc < 0) {
            printf("Error in I2C_READ MUX 70, rc = 0x%x\n", rc);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MUX_READ));
        }
    }

    return(status);
}

//-----------------------------------------------
//  TXRX has more tests to do 
//  LUC  from "./sm-bmc12500-eth/bmc12500-eth-ui.c"
//-----------------------------------------------
int 
ad_luc_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int diagModelNum=0;
 uint32 *luc_sw_rev;
 uint32 *luc_hw_rev;
 uint32 reg_val, high, low;
 uint32 i;
 
    if(isTXRX()) {
        printf(" -LUC tests\n");
        testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 

        // get Model number
        diagModelNum = diag_get_model_info();

        // LUC   Version check  Check against a base value.. may need to be revisited if 
        //  Software version rolls over.. 
        luc_sw_rev = (uint32 *)LUC_SW_REV_REG;
        luc_hw_rev = (uint32 *)LUC_HW_REV_REG;
        reg_val = *luc_hw_rev;
        if( reg_val < 10) {
            printf(" -- Error LUC Hardware revision: Exp [10] Got [%d]\n",reg_val);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_LUC_REV));
        }
        // now software version   
        reg_val = *luc_sw_rev;
        high = reg_val >> 24;
        low = reg_val & 0x00ffffff;
        if( high < 3) {
            printf(" -- Error LUC Software revision: Exp [3] Got [%d]\n",high);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_LUC_REV));
        }
        // *** If software rolls to say 4.0 this will fail.....NOTE VERSION revisit
        if( low < 2) {
            printf(" -- Error LUC Software revision: Exp [2] Got [%d]\n",low);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_LUC_REV));
        }
        // LUC  check Phys ID's  (8354)
        if (diagModelNum > 2220) {  // This tells me the Model is 2240 or above 
            for (i = HW_PMC8354_ADDR; i <= 0x13; i++) {
                (uint32)gt_mii_readSMI(i, 0, &reg_val);
                if( reg_val != 0x1140) {
                    printf(" -- Error 8354 Phy Id 0x%x  Exp: [0x1140] Got: 0x%x\n", i, reg_val);
                    // set port bit to indicate which device failed 
                    testGroup |= ( ( 1 << ( i & 0x0000000F)) << 12); // set a port bit to return with status
                    return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_LUC_PHY_ID));
                }
            }
        }
        else {  //  model is a 2220  only has 2 Phy's 
            for (i = HW_PMC8354_ADDR; i <= 0x11; i++) {
                (uint32)gt_mii_readSMI(i, 0, &reg_val);
                if( reg_val != 0x1140) {
                    printf(" -- Error 8354 Phy Id 0x%x  Exp: [0x1140] Got: 0x%x\n", i, reg_val);
                    testGroup |= ( ( 1 << ( i & 0x0000000F)) << 12); // set a port bit to return with status
                    return (status = (AUTO_DIAG_TEST_ERROR | testGroup) | DIAG_TEST_LUC_PHY_ID);
                }
            }
        }
    }// end if(isTXRX())
    return(status);
}


//-----------------------------------------------
// LSI  from "./sm-lsi8104/lsi8104.c lsi_gmac_uiDiag()"
//-----------------------------------------------
int
ad_lsi_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int diagModelNum=0;
 uint32 reg_val, num_devices;
 uint32 i;

    if(isTXRX()) {
        printf(" -LSI tests\n");
        testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 

        // get Model number
        diagModelNum = diag_get_model_info();

        if( diagModelNum == 2220) 
            num_devices = 2;  // default setting for Model 2220
        else
            num_devices = 4;  // default setting for Model 2240 & 2260
        // Write all 
        for (i = 0; i < num_devices; i++)
            lsi_gmac_write(i, 0, i+1);
        for (i = 0; i < num_devices; i++) {
            reg_val = lsi_gmac_read(i, 0);
            if (reg_val != i+1) {
                printf(" -- Error lsi8104 Register Test failed on device #%d\n", i);
                // set port bit to indicate which device failed 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_LSI_REG ));
            }
        }
        // Write all 0's
        for (i = 0; i < num_devices; i++)
            lsi_gmac_write(i, 0, 0);
        for (i = 0; i < num_devices; i++) {
            reg_val = lsi_gmac_read(i, 0);
            if (reg_val != 0) {
                printf(" -- Error lsi8104 Register Test failed on device #%d\n", i);
                // set port bit to indicate which device failed 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_LSI_REG ));
            }
        }
    }// end if(isTXRX())
    return(status);
}

//-----------------------------------------------
// GIGE
//-----------------------------------------------
int
ad_gige_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int diagModelNum=0;
 int mac, linkId;
 uint32 *luc_reg0;
 uint32 *luc_reg1;
 uint32 high, low;
 uint32 i, data0, data1;
 extern uint32 luc_loopback_test;

//extern uint32 bmc12500Eth_tx_pkt_trace; // debug

    if(isTXRX()) {
        printf(" -GigE tests\n");
        testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 

        // get Model number
        diagModelNum = diag_get_model_info();

        // Clear Error Counters left over from SERDES internal loop-back test
        for ( i = 0; i < 4; i++) {
            diag_lsi_clearStats( i);   // Clear Error Counters 
        }

        // Clear LUC Stats  (mainly for looping test, power up is ok since LUC was reset)
        luc_clear_stats();

        // Go Auto detect SFP's and if it's copper set it up for loop back test
        status = ad_detect_sfp_all( diagModelNum); // Model number
        if( status != NFX_OK) {
            printf(" -- Error Auto Detect SFP / SFP Loopback setup error. \n");
            // port bit for the error was set in the status returned from above 
            return (status |= (testGroup | DIAG_TEST_GIGE_AD_SFP ));
        }
        
        delay_us(500000); // wait for copper setup yes.. without delay get pkt count errs
        // I added a sync  but that didn't help.. doubling the delay works (test looping)
        delay_us(500000); // wait for copper setup yes.. without delay get pkt count errs

        // In "./sm-bmc12500-eth/bmc12500-eth-diag.c"  too much stuff
        //  mac = 0 linkId = 0 
	 // set up default pkt count, pkt size , link id, buffer...
        mac = 0;
        linkId = 0;
	status = auto_bmc12500Eth_diagTxLuc( mac, linkId); 
        if( status != NFX_OK) {
            printf(" -- Error LUC Loopback setup error.  MAC[%d] LinkID[%d]\n",mac,linkId);
            // also set a port bit for the error print to know which MAC failed 
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_LUCSETUP | DIAG_PORT_0_ERROR));
        }
        //  mac = 0 linkId = 1 
        mac = 0;
        linkId = 1;
	status = auto_bmc12500Eth_diagTxLuc( mac, linkId); 
        if( status != NFX_OK) {
            printf(" -- Error LUC Loopback setup error.  MAC[%d] LinkID[%d]\n",mac,linkId);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_LUCSETUP | DIAG_PORT_1_ERROR));
        }

        if( diagModelNum > 2220) { // set up other 2 ports?  Model is 2240 or 2260
            //  mac = 2 linkId = 2 
	     // set up default pkt count, size , link id, buffer...
            mac = 2;
            linkId = 2;
	    status = auto_bmc12500Eth_diagTxLuc( mac, linkId);
            if( status != NFX_OK) {
                printf(" -- Error LUC Loopback setup error.  MAC[%d] LinkID[%d]\n",mac,linkId);
            // also set a port bit for the error print to know which MAC failed 
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_LUCSETUP | DIAG_PORT_2_ERROR));
            }
            //  mac = 2 linkId = 3 
            mac = 2;
            linkId = 3;
	    status = auto_bmc12500Eth_diagTxLuc( mac, linkId); 
            if( status != NFX_OK) {
                printf(" -- Error LUC Loopback setup error.  MAC[%d] LinkID[%d]\n",mac,linkId);
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_LUCSETUP | DIAG_PORT_3_ERROR));
            }
        }

        delay_us(500000); // lets give it a little time to do it
	
        // Now check if all Ok  in LUC Stats..   looking at packet counts  then LSI Error Counts
        luc_reg0 = (uint32 *)LUC_FROM_SIBYTE0_REG;
        luc_reg1 = (uint32 *)LUC_TO_SIBYTE0_REG;
        high = *luc_reg0;  //from 1st SIBYTE:
	low = *luc_reg1;   //to 1st SIBYTE:
        if( high != low) {
            printf(" -- Error Packet Count Mismatch [1st SIBYTE]\n");
                // Look for which port  0 or 1
                luc_reg0 = (uint32 *)LUC_INGRESS0_REG;
                luc_reg1 = (uint32 *)LUC_EGRESS0_REG;
                high = *luc_reg0;  //from 1st SIBYTE:
	        low = *luc_reg1;   //to 1st SIBYTE:
                if( (high == 0) || (high < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts from LSI 0 = %d  [INGRESS]\n", high);
                    testGroup |= DIAG_PORT_0_ERROR; // need to set a bit to return with status
                }
                if( (low == 0) || (low < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts to LSI 0 = %d  [EGRESS]\n", low);
                    testGroup |= DIAG_PORT_0_ERROR; // need to set a bit to return with status
                }
                // check port 1
                luc_reg0 = (uint32 *)LUC_INGRESS1_REG;
                luc_reg1 = (uint32 *)LUC_EGRESS1_REG;
                high = *luc_reg0;  //from 1st SIBYTE:
	        low = *luc_reg1;   //to 1st SIBYTE:
                if( (high == 0) || (high < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts from LSI 1 = %d  [INGRESS]\n", high);
                    testGroup |= DIAG_PORT_1_ERROR; // need to set a bit to return with status
                }
                if( (low == 0) || (low < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts to LSI 1 = %d  [EGRESS]\n", low);
                    testGroup |= DIAG_PORT_1_ERROR; // need to set a bit to return with status
                }
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_PKT ));
        }

        if( diagModelNum > 2220) { // check other ports ?  if Model 2240 or 2260
            luc_reg0 = (uint32 *)LUC_FROM_SIBYTE1_REG;
            luc_reg1 = (uint32 *)LUC_TO_SIBYTE1_REG;
            high = *luc_reg0;  //from 2nd SIBYTE:
            low = *luc_reg1;   //to 2nd SIBYTE:
            if( high != low) {
                printf(" -- Error Packet Count Mismatch [2nd SIBYTE] \n");
                // Look for which port  2 or 3
                luc_reg0 = (uint32 *)LUC_INGRESS2_REG;
                luc_reg1 = (uint32 *)LUC_EGRESS2_REG;
                high = *luc_reg0;  //from 1st SIBYTE:
	        low = *luc_reg1;   //to 1st SIBYTE:
                if( (high == 0) || (high < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts from LSI 2 = %d  [INGRESS]\n", high);
                    testGroup |= DIAG_PORT_2_ERROR; // need to set a bit to return with status
                }
                if( (low == 0) || (low < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts to LSI 2 = %d  [EGRESS]\n", low);
                    testGroup |= DIAG_PORT_2_ERROR; // need to set a bit to return with status
                }
                // check port 3
                luc_reg0 = (uint32 *)LUC_INGRESS3_REG;
                luc_reg1 = (uint32 *)LUC_EGRESS3_REG;
                high = *luc_reg0;  //from 1st SIBYTE:
	        low = *luc_reg1;   //to 1st SIBYTE:
                if( (high == 0) || (high < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts from LSI 3 = %d  [INGRESS]\n", high);
                    testGroup |= DIAG_PORT_3_ERROR; // need to set a bit to return with status
                }
                if( (low == 0) || (low < 16)) { // with set params .. all counts were 16
                    printf("   - Pkts to LSI 3 = %d  [EGRESS]\n", low);
                    testGroup |= DIAG_PORT_3_ERROR; // need to set a bit to return with status
                }
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_PKT ));
            }
        }
        // Now that GigE loop back packet counts seems Ok check for Errors in LSI regs
        //  All error counters should be 0
        high = 2;
        if( diagModelNum > 2220) // means that it's a 2240 or 2260
            high =4; // number of LSI 
        for ( i = 0; i < high; i++ ) { // check all units per model

            data0 = lsi_gmac_read(i, LSI_GMAC_CRCALIGN0);
            data1 = lsi_gmac_read(i, LSI_GMAC_CRCALIGN1);
            if( (data0 | (data1 << 16)) != 0) {
                printf("   - LSI %d CRC align errors: %d\n",i, data0 | (data1 << 16));
                //  this sets the port error bit in the return status 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
            }
                                                                                                                              
            data0 = lsi_gmac_read(i, LSI_GMAC_UNDERSIZE0);
            data1 = lsi_gmac_read(i, LSI_GMAC_UNDERSIZE1);
            if( (data0 | (data1 << 16)) != 0) {
                printf("   - LSI %d Undersized errors: %d\n",i, data0 | (data1 << 16));
                //  this sets the port error bit in the return status 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
            }
                                                                                                                              
            data0 = lsi_gmac_read(i, LSI_GMAC_TX_DISC0);
            data1 = lsi_gmac_read(i, LSI_GMAC_TX_DISC1);
            if( (data0 | (data1 << 16)) != 0) {
                printf("   - LSI %d Tx underflows: %d\n",i, data0 | (data1 << 16));
                //  this sets the port error bit in the return status 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
            }
                                                                                                                              
            data0 = lsi_gmac_read(i, LSI_GMAC_TX_BAD0);
            data1 = lsi_gmac_read(i, LSI_GMAC_TX_BAD1);
            if( (data0 | (data1 << 16)) != 0) {
                printf("   - LSI %d Tx errors: %d\n",i, data0 | (data1 << 16));
                //  this sets the port error bit in the return status 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
            }
                                                                                                                              
            data0 = lsi_gmac_read(i, LSI_GMAC_RX_SYM0);
            data1 = lsi_gmac_read(i, LSI_GMAC_RX_SYM1);
            if( (data0 | (data1 << 16)) != 0) {
                printf("   - LSI %d Rx symbol errors: %d\n",i, data0 | (data1 << 16));
                //  this sets the port error bit in the return status 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
            }
                                                                                                                              
            data0 = lsi_gmac_read(i, LSI_GMAC_RX_DROPS0);
            data1 = lsi_gmac_read(i, LSI_GMAC_RX_DROPS1);
            if( (data0 | (data1 << 16)) != 0) {
                printf("   - LSI %d Rx overflows: %d\n",i, data0 | (data1 << 16));
                //  this sets the port error bit in the return status 
                testGroup |= (( 1 << i)<< 12); // need to set a port bit to return with status
            }
       
        }
        // Wanted to print All Errors before Exit .. So check to see if any Port Error bits set
        if( (testGroup & 0x0000F000) != 0) {
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GIGE_PKT_ECNT ));
        }
        // Done with GigE loop back
   
        luc_loopback_test = 0;   // as seen on bmc12500 ui menu...

    }// end if(isTXRX())

    return(status);
}

//-----------------------------------------------
// Hyper Transport Memory Tests
//-----------------------------------------------
int 
ad_hypermem_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -Hyper Transport DRAM test\n");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
    // These mem tests run over Hyper Transport at different address so they can run at the same time
    if( isTXRX()) { // got these address from "mem_tst_addr_t txrx_overldt_mt[] ="  sm-mdk/
        rc = domt( 0x40000000, 1*MEG);  // TXRX testing FP's SHMEM
        if (rc) {
            printf("Error in HT Mem Test, rc = 0x%x\n", rc); // if -1 there was a param error ??
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_HT));
        }
    }
    else if( isFP()) { // got these address from "mem_tst_addr_t fp_overldt_mt[] ="  sm-mdk/
        //rc = memtst( 0x20000010, 1*MEG);  // FP testing TXRX's SHMEM
        rc = domt( 0x20000010, 1*MEG);  // FP testing TXRX's SHMEM
        if (rc) {
            printf("Error in HT Mem Test, rc = 0x%x\n", rc); // if -1 there was a param error ??
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_HT));
        }
    }
    else {
        printf("Error in HT Mem Test -- UnKnown Module \n"); // Don't know who we are?
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_UNKNOWN_MOD));
    }
    return(status);
}

//-----------------------------------------------
// Shared Memory Tests
//-----------------------------------------------
int
ad_shmem_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
    if( isTXRX()) { // got these address from "mem_tst_addr_t txrx_overldt_mt[] ="  sm-mdk/
        printf(" -TXRX to SSC Shared Mem test\n");
        rc = domt( 0x800000f850000000, 1*MEG);  // TXRX testing SSC's SHMEM
        if (rc) {
            printf("Error in Shared SSC  Mem Test, rc = 0x%x\n", rc); // if -1 there was a param error ??
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_TXRX_SSC));
        }
    }

    else if( isFP()) { // got these address from "mem_tst_addr_t fp_overldt_mt[] ="  sm-mdk/
        printf(" -FP to SSC Shared Mem test\n");
        rc = domt( 0x800000f851000000, 1*MEG);  // FP testing SSC's SHMEM
        if (rc) {
            printf("Error in Shared SSC  Mem Test, rc = 0x%x\n", rc); // if -1 there was a param error ??
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_FP_SSC));
        }
    }
    else {
        printf("Error in HT Mem Test -- UnKnown Module \n"); // Don't know who we are?
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_UNKNOWN_MOD));
    }

    return(status);
}

typedef struct {
        char  *str0;
        char  *str1;
        char  *str2;
} cmtest_set_t;

//-----------------------------------------------
// Dram Tests
//-----------------------------------------------
int
ad_dram_tst( int shortMem, int cacheMiss)
{
 int testGroup = 0x00000000;
 int status = 0;
 int diagModelNum=0;
 unsigned long long  memsize;
 unsigned char   buf[20];
 int ac = 2;

    if( cacheMiss)
        printf(" -DRAM cache miss test\n");
    else
        printf(" -DRAM tests\n");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
    // This varailble holds the 1st error. Must be cleared before test is run
    diagDramErrorAddr = 0;
    dramEccErr = 0; 

    diagModelNum = diag_get_model_info();
    // need to detect size .. make sure it's one of 1, 2 or 4 Meg ...
    //  dram_memtst()  does what ever size it finds..  
    //  Problem is that runtime doesn't support other that above sizes..
    //  Error seen with a 512 Meg dimm in place where a 1024 Meg dimm should be
    //  runtime will panic with TLBs -- 

    memsize = memorysize;    // this is set early in init sbdreset() ?? 
    if (memsize % MEG != 0) {
        printf ("   -- Error Memory size %3lld KB.\n", memsize / 1024);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_DRAM_SIZE));
    }
    else {
        // now check dependant on model.. 2220 = 1Gig, 2240 = 2Gig, & 2260 = 4Gig
        if( diagModelNum == 2220) {
             if( memsize  != ((unsigned long long)1024 * MEG) )  {
                printf("  -- Error Memory size for model. SB 1024 MB got - %3lld MB.\n", memsize / MEG);
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_DRAM_SIZE));
            } 
        }
        if( diagModelNum == 2240) {
             if( memsize  != ((unsigned long long)2048 * MEG) )  {
                printf("  -- Error Memory size for model. SB 2048 MB got - %3lld MB.\n", memsize / MEG);
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_DRAM_SIZE));
            } 
        }
        if( (diagModelNum == 2260) || (diagModelNum == 2280)) {
            if( memsize != ((unsigned long long)4096 * MEG) ) {
                printf("  -- Error Memory size for model. SB 4096 MB got - %3lld MB.\n", memsize / MEG);
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_DRAM_SIZE));
            } 
        }
    }
    printf ("  - Memory size %3lld MB.\n", memsize / MEG);
    
if(shortMem)
    status = domt( 0x80000000, 0x00040000); // for testing diag code only
else
    if( cacheMiss ) {
        cmtest_set_t mstr;
        mstr.str0 = "";
        mstr.str1 = "-C";
        mstr.str2 = "\n";

        status = dram_memtst(ac, (char **)&mstr); 
    }
    else
        status = dram_memtst(0, NULL); 

    if (status) {

        // First check to see if we got ECC errors.
        if( dramEccErr == DRAM_ECC_L2_ERR) {
            status = testGroup | DIAG_TEST_MEM_L2_ECC_ERROR;  // Sets error report info 
            // get the address from the Bus Watcher reg if NOT an address in the 1st saved error
            //if( diagDramErrorAddr == 0) {
            //    testGroup |= DIAG_PORT_0_ERROR;  // Sets error report info (L2 ECC) 
            //    diagDramErrorAddr = *(volatile uint64 *)PHYS_TO_K1(A_BUS_ERR_DATA_3);
            // }
        }
        if( dramEccErr == DRAM_ECC_MEM_ERR) {
            status = testGroup | DIAG_TEST_MEM_ECC_ERROR;  // Sets error report info 
            // get the address from the Bus Watcher reg if NOT an address in the 1st saved error
            //if( diagDramErrorAddr == 0) {
            //    testGroup |= DIAG_PORT_1_ERROR;  // Sets error report info (Mem ECC)
            //    diagDramErrorAddr = *(volatile uint64 *)PHYS_TO_K1(A_BUS_ERR_DATA_3);
            // }
        }
            
        // need to determine which DIMM is bad and call out location
        // diagDramErrorAddr has the 1st virtual address that failed.
        // knowing that the memory controllers 0 & 1 are interleaved and 
        // set to use address bit 5 to switch between banks, check bit 5, then
        // look at the higher address bits.. 0x10-M000-0000

        if( diagDramErrorAddr != 0 ) {  //must have an address
 
            if( (diagDramErrorAddr & 0xF0000000) < 0x80000000) {
                if (diagModelNum == 2220) {
                    // check to see if we have 2 512 DIMMS
                    if( (i2c(I2C_READ, 0, 0x56, 0, 1, buf)) == 0) {
printf(" - Yes have 2 DIMMS and 2220 \n");
                        if(diagDramErrorAddr & 0x20) {  //  bit 5 on
printf("  -- Yes Bit 5 on  Addr = 0x%16llX \n",diagDramErrorAddr);
                            if( isTXRX()) 
                                // DRAM 2 J2401   // TXRX DIMM 2
                                status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM2);
                            else 
                                // DRAM 2 J1001   // FP DIMM 2
                                status |= (testGroup | DIAG_TEST_MEM_FP_DRAM2);
                        }
                        else {  // Bit 5  off
printf("  -- No Bit 5 off  Addr = 0x%16llX \n",diagDramErrorAddr);
                            if( isTXRX()) 
                                // DRAM 0 J2301   // TXRX DIMM 0
                                status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM0);
                            else 
                                // DRAM 0 J0901   // FP DIMM 0
                                status |= (testGroup | DIAG_TEST_MEM_FP_DRAM0);
                        }
                    }
                    else {
printf(" - have 1 DIMM and 2220 \n");
                        // Ok a 2220 with 1 1GB DIMM  do normal decode
                        if( isTXRX())
                            // DRAM 0 J2301   // TXRX DIMM 0
                            status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM0);
                        else
                            // DRAM 0 J0901   // FP DIMM 0
                            status |= (testGroup | DIAG_TEST_MEM_FP_DRAM0);
                    }
                } 
                else {
printf(" - NOT 2220 \n");
                    // Not a 2220 special case 
                    if(diagDramErrorAddr & 0x20) {  //  bit 5 on
                        if( isTXRX()) 
                            // DRAM 2 J2401   // TXRX DIMM 2
                            status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM2);
                        else 
                            // DRAM 2 J1001   // FP DIMM 2
                            status |= (testGroup | DIAG_TEST_MEM_FP_DRAM2);
                    }
                    else {  // Bit 5  off
                        if( isTXRX()) 
                            // DRAM 0 J2301   // TXRX DIMM 0
                            status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM0);
                        else 
                            // DRAM 0 J0901   // FP DIMM 0
                            status |= (testGroup | DIAG_TEST_MEM_FP_DRAM0);
                    }
                }
            }    
            // else if ((diagDramErrorAddr & 0xF0000000) < 0x80000000) 
            // else if ((diagDramErrorAddr & 0xF0000000) < 0xC0000000) 
            else {
                if(diagDramErrorAddr & 0x20) {  //  bit 5 on
printf("  -- Yes Bit 5 on  Addr = 0x%16llX \n",diagDramErrorAddr);
                    if( isTXRX()) 
                        // DRAM 3 J2402   // TXRX DIMM 3
                        status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM3);
                    else 
                        // DRAM 3 J1002   // FP DIMM 3
                        status |= (testGroup | DIAG_TEST_MEM_FP_DRAM3);
                }
                else {  // Bit 5  off
printf("  -- No Bit 5 off  Addr = 0x%16llX \n",diagDramErrorAddr);
                    if( isTXRX()) 
                        // DRAM 1 J2302   // TXRX DIMM 1
                        status |= (testGroup | DIAG_TEST_MEM_TXRX_DRAM1);
                    else 
                        // DRAM 1 J0902   // FP DIMM 1
                        status |= (testGroup | DIAG_TEST_MEM_FP_DRAM1);
                }
            }
        }
    }

    return(status);
}
#endif


// New Prom memtest 
int
ad_dram_tst2( int local, int cached, int gtmem )
{
 int testGroup = 0x00000000;
 int status = 0;
    printf(" -DRAM2 tests\n");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
    admenu_running_mtest = 1;
    status = do_bmt_rw_local_cached(local, cached, gtmem); 
    admenu_running_mtest = 0;
    if( status) 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_DRAM_ERROR));
    else 
        return(status);
}


#ifdef PROM_R9000
// Called by bc_run_ssc_diags() in load-bobcat.c
// This will be the list of tests run fo Go No Go testing on Bobcat
// SSC ONLY !!!  The SSC will have some pre tests that will run before lettiing the TXRX and FP
//  processors run their diags.  SSC must 1st check out the Marval Internal Local SRAM.
//  This is where all communications (States) are sent and received.
//  runSSC_preDiags( int shortMem)

//---------------------------------------------------------
// PCI tests
//---------------------------------------------------------
int
ad_pci_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;
 unsigned int data;

    printf(" -PCI read tests\n");
    testGroup = DIAG_TEST_GROUP_PCI;  // Set group Number 
    //Gt_PCI0="pciread 0 1 0 0
    //if (rc = pci_cfg_read(bus, dev, func, off, &data))
    if ((rc = pci_cfg_read(0, 1, 0, 0, &data))) {
        printf("pci_cfg_read returned error -Gt_PCI 0-, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GT_PCI));
    }
    else { // test for correct ID
        printf("    - Gt PCI 0  reg = 0x%x\n", data);
        //GT_PCI_ID="648011ab -A1 part" 0x648511ab -B1 part
        if((data != 0x648011ab) && (data != 0x648511ab)) { 
            printf("  Error Gt_PCI 0 ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GT_PCI));
        }
    }

    // TXRX_PCI="pciread 8 0 0 0"
    if ((rc = pci_cfg_read(8, 0, 0, 0, &data))) {
        printf("pci_cfg_read returned error -TXRX_PCI -, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_TXRX_PCI));
    }
    else { // test for correct ID
        printf("    - TXRX PCI  reg = 0x%x\n", data);
        if(data != 0x1166d) { //TXRX_PCI_ID="1166d"
            printf("  Error TXRX_PCI ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_TXRX_PCI));
        }
    }

    // FP_PCI="pciread 8 1 0 0"
    if ((rc = pci_cfg_read(8, 1, 0, 0, &data))) {
        printf("pci_cfg_read returned error -FP_PCI -, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FP_PCI));
    }
    else { // test for correct ID
        printf("    - FP PCI  reg = 0x%x\n", data);
        if(data != 0x1166d) { //FP_PCI_ID="1166d"
            printf("  Error FP_PCI ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FP_PCI));
        }
    }
    return(status);
}

//---------------------------------------------------------
// Power Supply / Fans / Temperature 
//---------------------------------------------------------
int
ad_pwr_fan_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;
 unsigned int data, loopcnt;
 volatile unsigned int *adrPtr;

    printf(" -Power, Fan, and Temperature Status check\n");
    testGroup = DIAG_TEST_GROUP_PWRFAN;  // Set group Number 
    adrPtr = (unsigned int *)0xbf400004;   // BMFPGA status register 
    for( loopcnt = 0; loopcnt < 20; loopcnt++) {  // We do the checking at least 20 times

        if( (*adrPtr & 0xFFFFFFF0) != 0x000003F0) { // expected good status
            data = *adrPtr;  // read it to test all bits
            if( (data & BMFPGA_STATUS_CTRL_PS0_OK) != BMFPGA_STATUS_CTRL_PS0_OK){ // PS 0
                printf("   - PS 0 Error  Not OK  - BMFPGA status reg\n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_PWR0));
            }
            if( (data & BMFPGA_STATUS_CTRL_PS0_GOOD) != BMFPGA_STATUS_CTRL_PS0_GOOD){ // PS 0 Good
                printf("   - PS 0 Error  Not Good - BMFPGA status reg\n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_PWR0));
            }
            if( (data & BMFPGA_STATUS_CTRL_PS1_OK) != BMFPGA_STATUS_CTRL_PS1_OK){ // PS 1 
                printf("   - PS 1 Error  Not OK - BMFPGA status reg\n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_PWR1));
            }
            if( (data & BMFPGA_STATUS_CTRL_PS1_GOOD) != BMFPGA_STATUS_CTRL_PS1_GOOD){ // PS 1 Good
                printf("   - PS 1 Error  Not Good - BMFPGA status reg \n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_PWR1));
            }
            //  Lets see if it's the FAN controllers
            if( (data & BMFPGA_STATUS_CTRL_FAN0_STATUS) != BMFPGA_STATUS_CTRL_FAN0_STATUS){ // Fan 0 Good
                printf("   - Fan 0 Controller Error - BMFPGA status reg \n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FAN0));
            }
            if( (data & BMFPGA_STATUS_CTRL_FAN1_STATUS) != BMFPGA_STATUS_CTRL_FAN1_STATUS){ // Fan 1 Good
                printf("   - Fan 1 Controller Error - BMFPGA status reg \n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FAN1));
            }
            if( (data & BMFPGA_STATUS_CTRL_FAN_CTRL_ON) != BMFPGA_STATUS_CTRL_FAN_CTRL_ON){ // Fan Full Speed
                printf("   - Fan Speed Controller Error - BMFPGA status reg \n");
                return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FAN_SPEED));
            }
        } // end of if not good status

        // We need to check all the fan tach speedes also ... added temperature check too.
        rc = ad_fan_ctrl_tach_check();  // go check all fan tachs, and temperature are wintin range.
        if( rc != NFX_OK) {
           printf("  Error Fan Tach Check \n");
           return( status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FAN_SPEED));
        }

        delay_us(200000);  // wait a bit berfore re-checking status

    } // end of for loop 20 times

    return(status);
}

//---------------------------------------------------------
// I2C tests
//---------------------------------------------------------
int
ad_ssc_i2c_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;
 int rmsize = 0, gmsize = 0;
 unsigned char   buf[0x20];

    printf(" -I2C read tests\n");
    testGroup = DIAG_TEST_GROUP_I2C;  // Set group Number 
    // SEEP 
    rc = i2c(I2C_READ, 0, 0xa0, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ SEEP, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup |DIAG_TEST_BOARD_SEEP));
    }

    rc = i2c(I2C_READ, 0, 0xa8, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ RM9K_MEM, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_R9K_DRAM));
    }

    rc = i2c(I2C_READ, 0, 0xaa, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ GT_MEM, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GT_DRAM));
    }

    // Ok Now make sure the SIZE of Memory for PMC and GT (512Meg and 1024Meg)
    // PMC Mem  should be 512 Meg
    rc = i2c(I2C_READ, 0, 0xa8, 0, sizeof(buf), buf);
    if (rc < 0) {
        printf("Error in I2C_READ RM9K_MEM, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_R9K_DRAM));
    }
    //    mem_size (MB) = #of banks * Module bank density * banks on device
    rmsize = buf[5] * buf[31] * buf[17];
    if( rmsize != 0x200) {
       printf("Error PMC Memory Size   - got buf[5]=%02X buf[31]=%02X buf[17]=%02X\n",buf[5], buf[31], buf[17]);
       printf("Error PMC Memory Size   - got %04X  Expected 0x200 \n",rmsize);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_R9K_DRAM));
    }
    // GT Mem shopuld be 1 Gig (1024 Meg) 
    rc = i2c(I2C_READ, 0, 0xaa, 0, sizeof(buf), buf);
    if (rc < 0) {
        printf("Error in I2C_READ GT_MEM, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GT_DRAM));
    }
    //    mem_size (MB) = #of banks * Module bank density * banks on device
    gmsize = buf[5] * buf[31] * buf[17];
    if( gmsize != 0x400) {
       printf("Error GT Memory Size   - got buf[5]=%02X buf[31]=%02X buf[17]=%02X\n",buf[5], buf[31], buf[17]);
       printf("Error GT Memory Size   - got %04X  Expected 0x400 \n",gmsize);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_GT_DRAM));
    }
    
    return(status);
}

// Device Tests
//---------------------------------------------------------
// Serdes
//---------------------------------------------------------
int
ad_serdes_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 unsigned int data;

  // Got these from "./sm-gt64440-mii/"
    
  // Note: this test causes the MAC's lsi8104 to catch an error (sym Error counter = 1)
  //  So the GigE Loop-Back test must clear statistics (err counters) 1st before running test                                                                                                                                        
/*
   gt_mii_readSMI(uint32 phyAddr, uint32 regAddr, uint32 *value)
   gt_mii_writeSMI(uint32 phyAddr, uint32 regAddr, uint32 value, uint32 mask)
   Both return NFX_OK  or NFX_ERR
   The write here does a read verify after the write..It uses the mask for
    the read and compare to determine if there was an error.....
    so Write [ffe4] get back [aae4] if don't want an error mask sb [aae4] to test all bits
    or mask of [00e4] to test lower byte only....
*/
#if 0
// #ifdef 0
    printf(" -SERDES Test \n");
    testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 
    status = 0;  // now that we are here make sure status is clear...
    // Read SMI reg verify that is has "8051" in it
    if( gt_mii_readSMI( 0x10, 3, &data) != NFX_OK) {
        printf("  -- Error reading SMI dev 0x10 reg 3 \n");
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    if ((data & 0x0000FFFF) != 0x8051) {
        printf("  -- Error reading SMI dev 0x10 reg 3  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    // Now do writes..
    delay_us(10000);
    data = 0x63C0;
    if( gt_mii_writeSMI( 0x10, 0x11, data, 0xFFFF) != NFX_OK) {
       printf("  -- Error writing SMI dev 0x10 reg 0x11  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0xFFE4;
    if( gt_mii_writeSMI( 0x10, 0x10, data, 0xAAFF) != NFX_OK) {  // this one I got an error on my setup.. AAE4
       printf("  -- Error writing SMI dev 0x10 reg 0x10  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0x63C8;
    if( gt_mii_writeSMI( 0x10, 0x11, data, 0xFFFF) != NFX_OK) {
       printf("  -- Error writing SMI dev 0x10 reg 0x11  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0x63C0;
    if( gt_mii_writeSMI( 0x10, 0x11, data, 0xFFFF) != NFX_OK) {
       printf("  -- Error writing SMI dev 0x10 reg 0x11  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0xFF00;
    if( gt_mii_writeSMI( 0x10, 0x16, data, 0xFFFF) != NFX_OK) {
       printf("  -- Error writing SMI dev 0x10 reg 0x16  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0x8000;
    if( gt_mii_writeSMI( 0x10, 0x1C, data, 0xFFFF) != NFX_OK) {
       printf("  -- Error writing SMI dev 0x10 reg 0x1C  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0xC000;
    if( gt_mii_writeSMI( 0x10, 0x1B, data, 0xFDFF) != NFX_OK) {  // got back c200 so mask fdff
       printf("  -- Error writing SMI dev 0x10 reg 0x1B  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    data = 0xD000;
    if( gt_mii_writeSMI( 0x10, 0x1B, data, 0xEFFE) != NFX_OK) {  // got back c001 so mask EFFE
       printf("  -- Error writing SMI dev 0x10 reg 0x1B  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    delay_us(10000);
    // Read SMI reg verify that is has "C001" in it   SERDES Internal Loop back
    if( gt_mii_readSMI( 0x10, 0x1B, &data) != NFX_OK) {
        printf("  -- Error reading SMI dev 0x10 reg 0x1B \n");
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    if ((data & 0x0000FFFF) != 0xC001) {
        printf("  -- Error reading SMI dev 0x10 reg 0x1B  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }

#endif

    printf(" -SERDES Reset Test \n");
    // Now reset the Serdes so the GigE Loop-Back test can run from TXRX
    delay_us(10000);
    gt_mii_writeSMI( 0x10, 0, 0x8000, 0); // Reset  
    delay_us(10000);
    if( gt_mii_readSMI( 0x10, 0x03, &data) != NFX_OK) {
        printf("  -- Error reading SMI dev 0x10 reg 3 \n");
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    if ((data & 0x0000FFFF) != 0x8051) {
        printf("  -- Error reading SMI dev 0x10 reg 3  Data = %x \n",data);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_SERDES_LB ));
    }
    // set serdes registers back to Init values
    delay_us(10000);
    gt_mii_writeSMI( 0x10, 0x10, 0xaae8, 0xffff);
    delay_us(10000);
    gt_mii_writeSMI( 0x10, 0x11, 0xe300, 0xffff);
    delay_us(10000);
    gt_mii_writeSMI( 0x10, 0x16, 0, 0);
    delay_us(10000);
    gt_mii_writeSMI( 0x10, 0x1c, 0, 0);
    delay_us(10000);
    gt_mii_writeSMI( 0x10, 0x1b, 0, 0);

    return(status);
}

//---------------------------------------------------------
// Mem Tests
//---------------------------------------------------------
int
ad_r9k_scratch_mem_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -R9K Scratch Mem Test ");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
    rc = domt( 0x90004000, 0x2000 );  // RM9KScratch="mt 90004000 2000"
    if (rc) {
        printf("Error in R9K Scratch Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_R9K_SCRATCH ));
    }

    return(status);
}

//---------------------------------------------------------
// Gt Scratch Mem Test
//---------------------------------------------------------
int
ad_gt_scratch_mem_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -Gt Scratch Mem Test ");
    rc = domt( 0xbc040000, 0x40000 );  // GtScratch="mt bc040000 40000"
    if (rc) {
        printf("Error in Gt Scratch Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_GT_SCRATCH));
    }

    return(status);
}

//---------------------------------------------------------
// SSC to TXRX image Mem Test
//---------------------------------------------------------
int
ad_ssc_txrx_image_mem_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -SSC to TXRX image Mem Test ");
    rc = domt( 0x9a000000, 0x40000 );  // M9KTXRX="mt 9a000000 1000000"
    if (rc) {
        printf("Error in SSC to TXRX image Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_SSC_TXRX));
    }

    return(status);
}

//---------------------------------------------------------
// SSC to FP image Mem Test
//---------------------------------------------------------
int
ad_ssc_fp_image_mem_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -SSC to FP image Mem Test ");
    rc = domt( 0x9b000000, 0x40000 );  // RM9KFP="mt 9b000000 1000000"
    if (rc) {
        printf("Error in SSC to FP image Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_SSC_FP));
    }

    return(status);
}

//---------------------------------------------------------
// Gt Main Mem Test
//---------------------------------------------------------
int
ad_gt_mem_tst( int shortMem)
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    // Gt Memory needs to be tested before letting the TXRX and FP do it's shared mem tests
    printf(" -Gt Main Mem Test [cached] ");
    // Seems that if I do 4000.0000 to 4100.0000 it causes the PROM display to go nuts and
    //  eventually get an NMI and reboots.
    // Do the rest later 6000.0000 to 8000.0000 -- do unchached later with some overlap
if(shortMem)
    rc = domt( 0x41000000, 0x00040000 );  // GtMem="mt 41000000 - 6000.0000"
else
    rc = domt( 0x41000000, 0x1f000000 );  // GtMem="mt 41000000 - 6000.0000"
    if (rc) {
        printf("Error in Gt Main Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_GT_MAIN ));
    }

    // After running Memory tests Seep Info was destroyed.. so re init it
    // or else autoload will pannic on invalid mac address
    seepInit();

    return(status);
}

/***************************************************************
** runSSC_AutoDiags runs after the Pre tests so
**  all modules can run at the same time *
**   Returns Status -- If status = 0 (Ok)  *
**    else return test number ored with test error
**      [xxxx.0001]  xxxx = test number  0001 = test error
***************************************************************/

//---------------------------------------------------------
// PCI tests
//---------------------------------------------------------
int
ad_pci2_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;
 unsigned int data;

    printf(" -PCI read tests\n");
    // QLogic_PCI0="pciread 0 0 0 0"
    testGroup = DIAG_TEST_GROUP_PCI;  // Set group Number 
    if ((rc = pci_cfg_read(0, 0, 0, 0, &data))) {
        printf("pci_cfg_read returned error -QLOGIC_PCI 0 -, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_QLOGIC_PCI0));
    }
    else { // test for correct ID
        printf("    - QLOGIC PCI 0 reg = 0x%x\n", data);
        if(data != 0x23121077) { //QLOGIC_PCI_ID="23121077"
            printf("  Error QLOGIC_PCI 0 ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_QLOGIC_PCI0));
        }
    }

    // QLogic_PCI1="pciread 0 0 1 0"
    if ((rc = pci_cfg_read(0, 0, 1, 0, &data))) {
        printf("pci_cfg_read returned error -QLOGIC_PCI 1 -, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_QLOGIC_PCI1));
    }
    else { // test for correct ID
        printf("    - QLOGIC PCI 1 reg = 0x%x\n", data);
        if(data != 0x23121077) { //QLOGIC_PCI_ID="23121077"
            printf("  Error QLOGIC_PCI 1 ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_QLOGIC_PCI1));
        }
    }

    // Bridge_PCI="pciread 8 2 0 0"
    if ((rc = pci_cfg_read(8, 2, 0, 0, &data))) {
        printf("pci_cfg_read returned error -Bridge_PCI-, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_BRIDGE_PCI));
    }
    else { // test for correct ID
        printf("    - Bridge PCI reg = 0x%x\n", data);
        if(data != 0xb1548086) { //Bridge_PCI_ID="b1548086"
            printf("  Error Bridge_PCI ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_BRIDGE_PCI));
        }
    }

    // MAC0_PCI="pciread 9 0 0 0"
    if ((rc = pci_cfg_read(9, 0, 0, 0, &data))) {
        printf("pci_cfg_read returned error -MAC0_PCI 0 -, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MAC0_PCI));
    }
    else { // test for correct ID
        printf("    - MAC PCI 0 reg = 0x%x\n", data);
        if(data != 0x20100b) { // MAC_PCI_ID="20100b"
            printf("  Error MAC_PCI 0 ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MAC0_PCI));
        }
    }

    // MAC1_PCI="pciread 9 1 0 0"
    if ((rc = pci_cfg_read(9, 1, 0, 0, &data))) {
        printf("pci_cfg_read returned error -MAC1_PCI 1 -, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MAC1_PCI));
    }
    else { // test for correct ID
        printf("    - MAC PCI 1 reg = 0x%x\n", data);
        if(data != 0x20100b) { // MAC_PCI_ID="20100b"
            printf("  Error MAC_PCI 1 ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MAC1_PCI));
        }
    }

    // CF_PCI="pciread 9 2 0 0"
    if ((rc = pci_cfg_read(9, 2, 0, 0, &data))) {
        printf("pci_cfg_read returned error -CF_PCI-, 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_CF_PCI));
    }
    else { // test for correct ID
        //  New part RoHS compliant
        // Old part   INTEL PD6729   0x11001013
        // New Part   TI PCI1510     0xAC55104C
        printf("    - CF PCI reg = 0x%x\n", data);
        if((data != 0x11001013) && (data != 0xAC55104C)) { // CF_PCI_ID="11001013"
            printf("  Error CF_PCI ID mis-match  Got 0x%08x\n",data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_CF_PCI));
        }
    }

    return(status);
}


//---------------------------------------------------------
// I2C tests
//---------------------------------------------------------
int
ad_ssc_i2c2_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;
 unsigned char   buf[20];

    printf(" -I2C read tests\n");
    testGroup = DIAG_TEST_GROUP_I2C;  // Set group Number 
    // FAN 1 
    rc = i2c(I2C_READ, 0, 0x90, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ -Fan 1-, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FAN1_READ));
    }

    // FAN 0 
    rc = i2c(I2C_READ, 0, 0x96, 0, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ -Fan 0-, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_FAN0_READ));
    }

    // Temp 
    rc = i2c(I2C_READ, 0, 0x9a, 0xaa, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ -Temp Sensor-, rc = 0x%x\n", rc);
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_TEMPSEN_READ));
    }

    return(status);
}

//---------------------------------------------------------
// Mem Tests R9K Mem Test [cached]
//---------------------------------------------------------
int
ad_r9k_cache_mem_tst( int shortMem)
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -R9K Mem Test [cached] ");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
if(shortMem)
    rc = domt( 0x10000000, 0x00040000 );  // RM9KMem="mt 1000.0000 - 2000.0000"
else
    rc = domt( 0x10000000, 0x10000000 );  // RM9KMem="mt 1000.0000 - 2000.0000"
    if (rc) {
        printf("  -- Error in R9K Main Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_R9K_MAIN));
    }

    return(status);
}

//---------------------------------------------------------
// R9K Mem Test [kseg0]
//---------------------------------------------------------
int
ad_r9k_kseg0_mem_tst( int shortMem)
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -R9K Mem Test [kseg0] ");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
if(shortMem)
    rc = domt( 0x83000000, 0x00040000 );  // RM9KMem="mt 8300.0000 - 9000.0000"
else
    rc = domt( 0x83000000, 0x0b000000 );  // RM9KMem="mt 8300.0000 - 9000.0000"
    if (rc) {
        printf("  -- Error in R9K Main Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_R9K_MAIN));
    }

    return(status);
}

//---------------------------------------------------------
// R9K Mem Test [kseg1]
//---------------------------------------------------------
int
ad_r9k_kseg1_mem_tst( int shortMem)
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

    printf(" -R9K Mem Test [kseg1] ");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
if(shortMem)
    rc = domt( 0xa3000000, 0x00040000 );  // RM9KMem="mt a300.0000 - 9000.0000"
else
    rc = domt( 0xa3000000, 0x0b000000 );  // RM9KMem="mt a300.0000 - 9000.0000"
    if (rc) {
        printf("  -- Error in R9K Main Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_R9K_MAIN));
    }

    return(status);
}

//---------------------------------------------------------
// Un-Cached GT Mem
//---------------------------------------------------------
int
ad_gt_uncached_mem_tst( int shortMem)
{
 int testGroup = 0x00000000;
 int status = 0;
 int rc;

  // GT Mem is done in Pre tests so TXRX and FP can run Shared mem test to SSC
    printf(" -Gt Main Mem Test [uncached] ");
    testGroup = DIAG_TEST_GROUP_MEM;  // Set group Number 
if(shortMem)
    rc = domt( 0xd0000000, 0x00040000 );  // GtMem="mt D000.0000 - 0000.0000"
else
    rc = domt( 0xd0000000, 0x30000000 );  // GtMem="mt D000.0000 - 0000.0000"
    if (rc) {
        printf("Error in Gt Main Mem Test, rc = 0x%x\n", rc); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_MEM_GT_MAIN ));
    }

    return(status);
}

//---------------------------------------------------------
// BM-FPGA
//---------------------------------------------------------
int
ad_bmfpga_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 unsigned int data;
 volatile unsigned int *adrPtr;

    printf(" -BMFPGA Test \n");
    testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 
    adrPtr = (unsigned int *)0xbf400000;  // BMFPGA Base 
    data = *adrPtr;  // Read Status reg look for version  sb 2
    if( (data & 0xFFFF0000)!= 0x00020000) { // If its not at least the base  version 2 
        if( (((data & 0xFFFF0000) >> 15) < 2)) { // see if it's a newer version 
            printf("  -- Error Read BMFPGA Version  Got 0x%x\n",data); 
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_BMFPGA_REV));
        }
    }

    // Wrtie and verify write data in test register
    adrPtr = (unsigned int *)0xbf40000c;  // BMFPGA test reg  
    data = 0xdeadbeef; // test data
    *adrPtr = data;  // write it
    delay_us(200000); // delay after write sb 200 ms ??
    status = *adrPtr; // Read it
    if( status != 0xdeadbeef ) {
        printf("  -- Error BMFPGA Test reg Expected 0xdeadbeef  Got 0x%08x\n",status); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_BMFPGA_REG));
    }
    status = 0; // reset status default value after using it as a scratch reg
    
    // Write and verify data FC Mailbox msg out core 1 (68)
    adrPtr = (unsigned int *)0xbf400068;  // BMFPGA FC Mail Box Msg Out core 1 reg  
    data = 0x55667788; // test data
    *adrPtr = data;  // write it
    delay_us(200000); // delay after write sb 200 ms ??
    status = *adrPtr; // Read it
    if( status != 0x55667788 ) {
        printf("  -- Error BMFPGA FC Mail Box Msg Out core 1  reg  Exp 0x55667788 Got 0x%x\n",status); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_BMFPGA_REG));
    }
    status = 0; // reset status default value after using it as a scratch reg

    return(status);
}

//---------------------------------------------------------
// RTC tests
//---------------------------------------------------------
int
ad_rtc_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 unsigned int data;
 volatile unsigned char *adrBytePtr;
 uchar8		i;

    printf(" -RTC Test \n");
    testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 
    printf("    - battery check \n");
    // Init the control regs
    adrBytePtr = (unsigned char *)0xbf000078;  // RTC Control B reg 0F
    *adrBytePtr = 0x80;  // clear IRQ enables and enable timer
    delay_us(500); 
    // check battery status 5 times
    for (i=0; i<5; i++) { // do it 5 times
        adrBytePtr = (unsigned char *)0xbf000070;  // RTC Control A reg 0E
        data = *adrBytePtr; // go read it
        if((data & 0x00000080) == 0x80) { // Battery Check bit  if 1 Battery bad
            printf("  -- Error RTC Battery Check Failed  Exp 0 in bit8[0xxx.xxxx] Got 0x%x\n",data); 
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_RTC_REG));
        }
        delay_us(500); 
    }
    // Check to see if timer is running
    printf("    - timer check \n");
    adrBytePtr = (unsigned char *)0xbf000078;  // RTC Control B reg 0F
    data = *adrBytePtr; // go read it
    if( (data & 0x00000080) != 0x80) { // Not enabled 
        printf("  -- Error RTC Timer Enable Check Failed  Exp 1 in bit8[1xxx.xxxx] Got 0x%x\n",data); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_RTC_REG));
    }
    adrBytePtr = (unsigned char *)0xbf000000;  // RTC seconds  regs 00
    // Only do it if the register isn't about to roll over (BCD 0 -9)
    //  may have to wait for a second...
    while ( (*adrBytePtr & 0x0F) < 8) {
        data = *adrBytePtr; // go read it
        delay_us(1000500);  // 1 second  500 usec wait
        status = *adrBytePtr; // go read it
        if(!(status > data)) { // 2nd read should be at leaset 1 more that 1st read
            printf("  -- Error RTC Second Timer Check Failed  1st read = 0x%x 2nd read = 0x%x\n",data,status); 
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_RTC_REG));
        }
        status = 0; // reset status default value after using it as a scratch reg
        break;  // Only want to do it once
    }
    
    // do very short memory test   i = uchar8 value
    printf("    - SRAM check \n");
    for (i = 0; i < 0x10; i++) {
        adrBytePtr = (unsigned char *)0xbf000080;  // RTC SRAM add reg 10
        *adrBytePtr = i;
        adrBytePtr = (unsigned char *)0xbf000098;  // RTC SRAM data reg 13
        *adrBytePtr = i;
    }
    // verify write
    for (i = 0; i < 0x10; i++) {
        adrBytePtr = (unsigned char *)0xbf000080;  // RTC SRAM add reg 10
        *adrBytePtr = i;
        adrBytePtr = (unsigned char *)0xbf000098;  // RTC SRAM data reg 13
        data = *adrBytePtr;
        if( data != i){
            printf("  -- Error RTC SRAM Failed Addr 0x%02X Exp 0x%02X Got 0x%02X\n", i, i, data);
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_RTC_REG));
        }
        // printf("      -- debug RTC SRAM Addr 0x%02X Exp 0x%02X Got 0x%02X\n", i, i, data);
    }

    return(status);
}

//---------------------------------------------------------
// CF Controller
//---------------------------------------------------------
int
ad_cf_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 unsigned int data;
 
    // This checks to see if both CF's are installed.
    printf(" -CF Test \n");
    testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 
    data = diag_pcmcia_socket_check();
    // Test for devPresent bits ... 
    //   bit 0 = 1 == SocketA dev found 
    //   bit 1 = 1 == SocketB dev found 
    //   bit 3 = 1 == Bridge Init Error
    if((data & 0x00000008) == 0x08){
        printf("  -- Error Compact Flash Bridge Init Failed\n"); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup| DIAG_TEST_CF_INIT ));
    }
    if((data & 0x00000001) != 0x01){
        printf("  -- Error Compact Flash Device in Socket A, Not found \n"); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_CF_DEV_OK ));
    }
    if((data & 0x00000002) != 0x02){
        printf("  -- Error Compact Flash Device in Socket B, Not found \n"); 
        return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_CF_DEV_OK ));
    }

    return(status);
}

//---------------------------------------------------------
// Ethernet
//---------------------------------------------------------
int
ad_eth_tst()
{
 int testGroup = 0x00000000;
 int status = 0;
 unsigned int data;
 volatile unsigned int *adrPt0;
 volatile unsigned int *adrPt1;

    // requires Xover cable  looking for link
    printf(" -ETH Test \n");
    testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number 
    adrPt0 = (unsigned int *)0xb9000004;  // port 0 addr
    adrPt1 = (unsigned int *)0xb9800004;  // port 1 addr
    data = *adrPt0;   
    if( (data & 0x80000000) != 0x80000000) {  // just look for link (old was E800.0000) 
        // Try auto-neg off  --  Must set up both ports.....
        *adrPt0 = 0x0007C00C; 
        delay_us(500);  //  500 usec wait
        *adrPt1 = 0x0007C00C; 
        delay_us(500);  //  500 usec wait
        adrPt0 = (unsigned int *)0xb9000080;
        adrPt1 = (unsigned int *)0xb9800080;
        *adrPt0 = 0x0000A100;  // do reset- sel speed 100-  auto nego off-  pwr down
        delay_us(500);  //  500 usec wait
        *adrPt1 = 0x0000A100;  // do reset- sel speed 100-  auto nego off-  pwr down
        delay_us(1000500);  // 1 second  500 usec wait
        adrPt0 = (unsigned int *)0xb9000004;
        data = *adrPt0;   
        if ( (data & 0x80000000) != 0x80000000) {  // old was looking for E000.0000
            printf("  --  Error Eth test Auto Neg and Non Auto mode Failed [port 0]\n"); 
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_ETH_LINK));
        }
    }
    printf("  -Port 0  Link %s - Speed %s - %s Duplex - Auto Neg %s \n",
             (data & 0x80000000)?"Good":"Bad", (data & 0x40000000)?"100Mbs":"10Mbs",
             (data & 0x20000000)?"Full":"Half", (data & 0x08000000)?"Done":"Not Active");

    // port 1 
    adrPt1 = (unsigned int *)0xb9800004;
    adrPt0 = (unsigned int *)0xb9000004;  // port 0 addr
    data = *adrPt1;   
    if( (data & 0x80000000) != 0x80000000) {  // just look for link
        // Try auto-neg off
        adrPt1 = (unsigned int *)0xb9800004;
        *adrPt1 = 0x0007C00C; 
        delay_us(500);  //  500 usec wait
        *adrPt0 = 0x0007C00C; 
        delay_us(500);  //  500 usec wait
        adrPt1 = (unsigned int *)0xb9800080;
        adrPt0 = (unsigned int *)0xb9000080;
        *adrPt1 = 0x0000A100;
        delay_us(500);  //  500 usec wait
        *adrPt0 = 0x0000A100;  // do reset- sel speed 100-  auto nego off-  pwr down
        delay_us(1000500);  // 1 second  500 usec wait
        adrPt1 = (unsigned int *)0xb9800004;
        data = *adrPt1;   
        if ( (data & 0x80000000) != 0x80000000) {
            printf("   -- Error Eth test Auto Neg and Non Auto mode Failed [port 1]\n"); 
            return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_ETH_LINK));
        }
    }
    printf("  -Port 1  Link %s - Speed %s - %s Duplex - Auto Neg %s \n",
             (data & 0x80000000)?"Good":"Bad", (data & 0x40000000)?"100Mbs":"10Mbs",
             (data & 0x20000000)?"Full":"Half", (data & 0x08000000)?"Done":"Not Active");

    return(status);
}

//  Add a test to read files from Compact Flash cards.
int 
ad_read_cf_files()
{
    int testGroup = 0x00000000;
    int status = 0;

    int32   mod_id;
    uchar8  dev_str[ENV_MAX_STR_LEN];

    printf(" Read files from both CF's (compact flash cards)\n");
    testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number
   
        sprintf( dev_str, "wd0a");
        printf("\n Diag Read files from dev %s \n",dev_str);
        for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
        {
            // Stop if load err (disk/file not found)
            if (load_bc_load_mod_image(mod_id, dev_str) != NFX_OK)
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_CF_DEV_OK));
        }
        sprintf( dev_str, "wd1a");
        printf("\n Diag Read files from dev wd1a \n");
        for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
        {
            // Stop if load err (disk/file not found)
            if (load_bc_load_mod_image(mod_id, dev_str) != NFX_OK)
                return (status = (AUTO_DIAG_TEST_ERROR | testGroup | DIAG_TEST_CF_DEV_OK));
        }
    return(status);
}


#endif

#endif  // PROM_R9000 || PROM_SIBYTE_BC
