/*
 *-----------------------------------------------------------------
 *
 * Name:	load.c
 *
 * RCS:  $Id: load.c,v 1.10 2002/09/28 00:07:55 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-incl.h>
#include <exports/tftp-api.h>
#include "loadfn.h"
#include "load.h"

void *	binary_addr;

extern uint32	save_len;
extern uint32	load_tot_len;
extern int32	load_dltype;
extern uint32	load_recno;

#ifdef NEW_LOADER
extern int32	load_busy;
extern int32	load_ret;
#endif

uint32	xlnx_load_size = 0;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32	load_cmdDataInd(
	char8	* pData,
	int32	size
)
{
	uchar8	* line_ptr;
/*
	int32	i;
	printf("load_cmdData: pData at %p, size = %d: ", pData, size);
	for (i=0; i<16; i++)
		if (isprint(pData[i]))
			printf("%c", pData[i]);
		else
			printf(".");
	printf("\n");
*/

	/*
	 * The tftp session should return buffers of 512 bytes, with the
	 * exception of the last transfer.
	 *
	 * For loading binary files, this is fine, we'll just copy the data
	 * to the desired address.
	 *
	 * For non-binary files, we need to load by line, so we must split up
	 * the received buffer into ascii lines, and then send those lines to
	 * the loading functions.
	 */

	if (binary_addr) {
		memcpy(binary_addr, pData, size);
		binary_addr += size;
		xlnx_load_size += size;
	}
	else {
		load_add_line(pData, size);
		while (line_ptr = load_get_line()) {
			if (load_load_line(line_ptr, load_dltype) != NFX_OK) {
				printf("Error loading line\n");
				return -1;
			}
		}
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int
load_cmdEventInd(
	void	* tftpCb,
	int32	event
)
{
#ifdef NEW_LOADER
	load_ret = -1;
#endif
	switch (event) {
		case TFTP_EVT_TIMEOUT:
				printf("TFTP Timeout.\n");
				break;

		case TFTP_EVT_ERROR:
				printf("TFTP Error.\n");
				break;

		case TFTP_EVT_EOF:
				printf("TFTP transfer completed.\n");
#ifdef NEW_LOADER
				load_ret = NFX_OK;
#endif
				break;

		default:
				printf("Unknown TFTP Event.\n");
				break;
	}

#ifdef NEW_LOADER
	load_busy = 0;
#endif

	tftp_close(tftpCb);
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32	isIpAddr(
	uchar8	* str
)
{
	uchar8	* p;

	p = str;

	if (p = (uchar8 *)strchr(str, '.'))
		if (p = (uchar8 *)strchr(p+1, '.'))
			if (p = (uchar8 *)strchr(p+1, '.'))
				return 1;

	return 0;
}

/*-----------------------------------------------------------------
 * Name:	load
 *
 * Description: Load file from network or from flash
 *				load [-a addr] [ip_addr] <fname>
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32
load(
	int32	ac,
	char	* av[]
)
{
	extern int32	optind;
	extern char8	* optarg;
	int32			c;
	uint32			IpAddr;
	int32			netload;
	uchar8			* fname;
#ifdef NEW_LOADER
	address_t		addr;
	uint32			nbytes;
#else
	void			* fd;
#endif

	xlnx_load_size = 0;
	optind		= 0;
	IpAddr		= 0;
#ifdef NEW_LOADER
	addr		=  (address_t)(0x83000000);
#else
	binary_addr	= 0;
#endif
	netload		= 0;
	save_len	= 0;
	load_recno	= 0;
	load_dltype	= -1;

    while ((c = getopt (ac, av, "a:")) != -1) {
	    switch (c) {
		    case 'a':
#ifdef NEW_LOADER
				if (!get_rsa (&addr, optarg))
#else
				if (!get_rsa (&binary_addr, optarg))
#endif
					return (-1);
				break;
			default:
				return -1;
	    }
	}

    if (optind >= ac)
      return (-1);

	if (isIpAddr(av[optind])) {
		if (optind+2 == ac) {
			if (getIpAddr( av[optind], &IpAddr ) < 0)
				return -1;
			optind++;
			fname = av[optind];
			netload = 1;
		}
		else
			return -1;
	}
	else {
		if (optind+1 == ac)
			fname = av[optind];
		else
			return -1;
	}

	if (netload)
		printf("load, ip addr = 0x%x, fname = %s\n", IpAddr, fname);
	else
		printf("load, fname = %s\n", fname);

#ifdef NEW_LOADER
#if __long64
    signextend(&addr);
#endif
    if (netload) { 
        if (load_net(fname, IpAddr, addr, &nbytes)) {
            printf("Error loading from network\n");
            return -1;
        }
#ifdef	PROM_R9000
	// flush the downloaded data from cache.
	mips_flush_dcache();
#endif
    }
    else {
        if (load_flash(fname, addr, &nbytes) != NFX_OK) {
            printf("Error loading from flash\n");
            return -1;
        }
    }
    return 0;
#else
	if (binary_addr) {
		printf(" at %p\n", binary_addr);
		set_entry_addr(binary_addr);
	}
	else
		printf("\n");

	fd = tftp_open(fname, IpAddr, load_cmdDataInd, load_cmdEventInd);
	if (!fd) {
		printf("Error opening remote file, %s\n", fname);
		return 0;
	}
#endif
/*
	{
	uchar8	* ptr;
	ptr = tbuf;
	while (1) {
		load_cmdDataInd(ptr, len);
		ptr+=512;
		printf("Hit a character to read next 512 bytes: ");
		getchar();
	}
	}
*/
}

#endif	/* PROM */

