/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:			lock-btree.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2002, ClariStor Inc.
 *
 * Description: Binary tree definitions.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	10/29/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _LOCK_BTREE_H
#define _LOCK_BTREE_H

#define LOCK_BT_LEFT		1       /*Left child*/
#define LOCK_BT_RIGHT		0       /*Right child*/

/*---------------
 * Data structure
 *---------------
 */

/*
 * btree node
 */
typedef struct lock_btNode
{
	vs_mem_index_t  parent;    /*Parent*/
	vs_mem_index_t  child[2];  /*Left and right children*/
} lock_btNode_t;


/*--------------------
 * Function prototypes
 *--------------------
 */
int32
lock_btree_insert(vs_mem_index_t	*rootIdxp,
				  vs_mem_index_t	inNodeIdx,
				  lock_btNode_t		*inNodep,
				  uint16			searchMask,
				  uint8				treeType);

lock_btNode_t *
lock_btree_find(vs_mem_index_t	*rootIdxp,
				lock_btNode_t   *inNodep,
				void			*inLockInfop,
				uint8			inLockOwnerType,
				boolean         useNode,
				uint16			searchMask,
				uint8			treeType);

lock_btNode_t *
lock_btree_delete(vs_mem_index_t	*rootIdxp,
				  lock_btNode_t		*inNodep,
				  uint16			searchMask,
				  uint8				treeType,
				  vs_mem_index_t	*delNodeIdxp);

int32
lock_btree_deleteTree(lock_fileLock_t	*fileLockp,
				      uint8				treeType);

void
lock_btree_traverse_preorder(
				vs_mem_index_t	rootIdx,
				uint8			treeType,
				void			(*doFunc)(vs_mem_index_t	nodeIdx,
										  int32				level,
										  int32				dir,
										  uint32			count,
										  uint8				treeType)
				   );

void
lock_btree_traverse_inorder(
				vs_mem_index_t	rootIdx,
				uint8			treeType,
				void			(*doFunc)(vs_mem_index_t	nodeIdx,
										  int32				level,
										  int32				dir,
										  uint32			count,
										  uint8				treeType)
				   );

lock_btNode_t *
lock_btree_getNextNode(vs_mem_index_t	currNodeIdx,
					   boolean			firstTime,
					   vs_mem_index_t	*nextNodeIdxp);

#endif /*_LOCK_BTREE_H*/
