/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        lock-utils.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2002, ClariStor Inc.
 *
 * Description: Byte range lock utility definitions and function
 *              prototypes.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:    11/25/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _LOCK_UTILS_H
#define _LOCK_UTILS_H

#include "lock-api.h"

#define LOCK_SPLIT_LOCK_FLAG_KEEP			0
#define LOCK_SPLIT_LOCK_FLAG_DELETE			1
#define LOCK_SPLIT_LOCK_FLAG_DELETE_ADD		2

#define LOCK_DELPENDLOCKS_TYPE_UNLOCKALL		1
#define LOCK_DELPENDLOCKS_TYPE_UNLOCKSINGLE		2
#define LOCK_DELPENDLOCKS_TYPE_GRANTPEND		3
#define LOCK_DELPENDLOCKS_TYPE_TIMEOUT			4

typedef struct lock_delPendLocksAll {
	uint32		key;
	ip_index_t	clientIpAddr;
	uint16		searchMask;
} lock_delPendLocksAll_t;

typedef struct lock_delPendLocksSingle_t {
	void	*lockInfop;
	uint8	lockOwnerType;
	uint16	searchMask;
} lock_delPendLocksSingle_t;

typedef struct lock_delPendLocksParam {
	uint8	type;
	union {
		lock_delPendLocksAll_t		delAll;
		lock_delPendLocksSingle_t	delSingle;
		vs_mem_index_t				pendLockIdx;
	} u;
} lock_delPendLocksParam_t;

/*--------------------
 * Function prototypes
 *--------------------
 */
lock_cifsLockOwner_t *
lock_getCifsLockOwner(lock_requestCifsOwnerInfo_t	*cifsOwnerInfop,
					  vs_mem_index_t				*cifsLockOwnerIdxp);

lock_nlmLockClient_t *
lock_getNlmLockClient(lock_requestNlmOwnerInfo_t	*nlmOwnerInfop,
					  vs_mem_index_t				*nlmLockClientIdxp);

lock_nlmLockOwner_t *
lock_getNlmLockOwner(lock_requestNlmOwnerInfo_t		*nlmOwnerInfop,
					 vs_mem_index_t					fcb,
					 vs_mem_index_t					*nlmLockOwnerIdxp);

uint8
lock_getLockOwnerTypeByIdx(vs_mem_index_t   lockOwnerIdx);

void *
lock_getLockOwnerByIdx(vs_mem_index_t   lockOwnerIdx,
                       uint8            *lockOwnerTypep);

uint8
lock_getOwnerTypeByLockInfoIdx(vs_mem_index_t lockInfoIdx);

vs_mem_index_t
lock_getLockOwnerIdxByLockInfo(vs_mem_index_t   lockInfoIdx,
                               void             *lockInfop,
                               uint8            lockOwnerType);

int32
lock_initCifsLockInfo(lock_request_t		*lockReqp,
					  lock_cifsLockInfo_t	*cifsLockInfop);

int32
lock_initNlmLockInfo(lock_request_t		*lockReqp,
					 lock_nlmLockInfo_t	*nlmLockInfop);

void *
lock_createLockInfo(void			*lockInfop,
					uint8			lockOwnerType,
					vs_mem_index_t	*lockInfoIdxp);

vs_mem_index_t
lock_getLockInfoIdxByLockIdx(vs_mem_index_t lockIdx);

void *
lock_getLockInfo(vs_mem_index_t	lockInfoIdx,
				 uint8			*lockOwnerTypep);

uint8
lock_getLockTypeByLockIdx(vs_mem_index_t lockIdx);

void *
lock_getGrantedLock(vs_mem_index_t	lockIdx,
					uint8			*lockTypep);

int32
lock_getLockInformationFromLockInfo(vs_mem_index_t	lockInfoIdx,
									void			*lockInfop,
									uint8			*lockOwnerTypep,
									uint64			*lockOffsetp,
									uint64			*lockLenp);

int32
lock_getLockInformationFromShNode(lock_shNode_t	*shNodep,
								  uint64		*lockOffsetp,
								  uint64		*lockLenp);

boolean
lock_matchKey(vs_mem_index_t	lockOwnerIdx,
			  uint32			inKey,
			  boolean			fidOnly);

boolean
lock_matchClientIpAddr(vs_mem_index_t	lockOwnerIdx,
					   uint32			inClientIpAddr);

boolean
lock_match(vs_mem_index_t	lockInfoIdx,
		   vs_mem_index_t	lockOwnerIdx,
		   boolean			useLockInfo,
		   uint32			inKey,
		   uint32			inClientIpAddr,
		   uint16			searchMask);

boolean
lock_sameOwner(vs_mem_index_t   lockOwnerIdx1,
               vs_mem_index_t   lockOwnerIdx2);

boolean
lock_haveSameOwner(vs_mem_index_t	lockInfoIdx1,
				   void				*lockInfop1,
				   vs_mem_index_t	lockInfoIdx2,
				   uint8			lockOwnerType1,
				   uint8			lockOwnerType2);

lock_exLock_t *
lock_findExclLockByLockInfo(
				vs_mem_index_t	*rootIdxp,
				void			*inLockInfop,
				uint8			inLockOwnerType,
				uint16			searchMask);

lock_exLock_t *
lock_findExclLockByLock(vs_mem_index_t	*rootIdxp,
						lock_exLock_t	*inLockp,
						uint16			searchMask);

lock_shNode_t *
lock_findSharedNodeByLockInfo(vs_mem_index_t	*rootIdxp,
							  void				*inLockInfop,
							  uint8				inLockOwnerType,
							  uint16			searchMask);

lock_shNode_t *
lock_findSharedNodeByNode(vs_mem_index_t	*rootIdxp,
						  lock_shNode_t		*inShNodep,
						  uint16			searchMask);

lock_shLock_t *
lock_findSharedLockByLockInfo(vs_mem_index_t	*rootIdxp,
							  void				*inLockInfop,
							  uint8				inLockOwnerType,
							  uint16			searchMask,
							  lock_shNode_t		**shNodepp,
							  vs_mem_index_t	*prevShLockIdxp);

int32
lock_mergeLockRanges(vs_mem_index_t *lockInfoIdxp1,
					 vs_mem_index_t *lockInfoIdxp2,
					 void			 *lockInfop2,
					 uint8			 lockOwnerType2);

int32
lock_mergeExLocks(lock_fileLock_t	*fileLockp,
				  void				*lockInfop,
				  uint8				lockOwnerType,
				  vs_mem_index_t	*exLockIdxp,
				  lock_exLock_t     **exLockpp);

int32
lock_splitNlmLock(lock_fileLock_t		*fileLockp,
                  vs_mem_index_t        grantedLockInfoIdx,
                  lock_nlmLockInfo_t    *delLockInfop,
                  uint8                 lockType,
                  uint8                 *deleteFlag);

lock_shNode_t *
lock_mergeShNodes(lock_fileLock_t	*fileLockp,
				  lock_shNode_t		*shNodep);

int32
lock_splitSharedNode(lock_fileLock_t	*fileLockp,
					 vs_mem_index_t		shNodeIdx,
					 lock_shNode_t		*shNodep);

int32
lock_addSharedLockToNode(lock_shNode_t	*shNodep,
						 vs_mem_index_t	newShLockIdx,
						 lock_shLock_t	*newShLockp,
						 vs_mem_index_t	newLockInfoIdx,
						 void			*newLockInfop,
						 uint8			newLockOwnerType);

int32
lock_delSharedLockFromNode(lock_fileLock_t	*fileLockp,
						   lock_shNode_t	**shNodepp,
						   vs_mem_index_t	prevShLockIdx,
						   lock_shLock_t	*shLockp,
						   uint8			lockOwnerType,
						   boolean			deleteLock);

int32
lock_delBtNode(lock_fileLock_t	*fileLockp,
			   lock_btNode_t	*delNodep,
			   uint8			treeType,
			   boolean			freeNode,
			   boolean			freeShLock);

int32
lock_insertPendLockToLocalList(vs_mem_index_t		*pendingListHeadp,
							   vs_mem_index_t		newPendLockIdx,
							   lock_pendingLock_t	*newPendLockp);

int32
lock_deletePendingLocksFromLocalList(
				lock_fileLock_t				*fileLockp,
				lock_delPendLocksParam_t	param);

int32
lock_insertPendLockToGlobalList(vs_mem_index_t		newPendLockIdx,
								lock_pendingLock_t	*newPendLockp);

int32
lock_removePendLockFromGlobalList(vs_mem_index_t	delPendLockIdx);

int32
lock_deletePendingList(vs_mem_index_t *pendingListp);

int32
lock_updateShNodeEndAndHoley(lock_shNode_t	*shNodep);

int32
lock_grantPendingLocks(lock_fileLock_t *fileLockp);

int32
lock_fillNlmTestResultOwnerInfo(vs_mem_index_t      confOwnerIdx,
								lock_testResult_t   *testResultp);

int32
lock_fillNlmTestResult(uint8                confResult,
					   vs_mem_index_t       confLockIdx,
					   lock_testResult_t    *testResultp);

void
lock_incNlmLockCntOfFile(lock_fileLock_t *fileLockp);

void
lock_decNlmLockCntOfFile(lock_fileLock_t *fileLockp);

#endif /* _LOCK_UTILS_H */
