/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc.h
 *
 * RCS:  $Id: luc-cam.h,v 1.5 2002/04/16 21:52:46 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains Look Up Classifier CAM (Lara LNI7020) related 
 *				data structure definitions.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 7/01/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _LUC_CAM_H
#define _LUC_CAM_H


/*--------------------------------------
 * CAM Driver error code.
 *--------------------------------------
 */
#define LUC_CAM_ERR_BASE		(-800000)				/* Err. No. base */
#define LUC_CAM_ERR_NO_RSRC		(LUC_CAM_ERR_BASE + -1)	/* No resource */
#define LUC_CAM_ERR_BAD_IDX		(LUC_CAM_ERR_BASE + -2)	/* Invalid index */
#define LUC_CAM_ERR_BAD_BLK		(LUC_CAM_ERR_BASE + -3)	/* Invalid block */
#define LUC_CAM_ERR_BAD_TBL 	(LUC_CAM_ERR_BASE + -4)	/* Invalid table */
#define LUC_CAM_ERR_DUP_FREE	(LUC_CAM_ERR_BASE + -4)	/* Duplicate free */


/*--------------------------------------
 * Supports 4K flow entries
 *--------------------------------------
 */
#define CAM_MAX_FLOW_ENTRY	NFX_MAX_CONN
#define	CAM_MAX_FLOW_BITMAP (CAM_MAX_FLOW_ENTRY/sizeof(uint32) )

/*--------------------------------------
 * Supports 4K IP I/F & VLAN combined entries
 *--------------------------------------
 */
#define CAM_MAX_IPIF_ENTRY	4096
#define	CAM_MAX_IPIF_BITMAP (CAM_MAX_IPIF_ENTRY/sizeof(uint32) )

#define CAM_MAX_VLAN_ENTRY	4096
#define	CAM_MAX_VLAN_BITMAP (CAM_MAX_VLAN_ENTRY/sizeof(uint32) )

/*--------------------------------------
 * CAM Table IDs
 *--------------------------------------
 */
#define CAM_FLOW_TBL_ID		0x00000001		/* flow table id */
#define CAM_IPIF_TBL_ID		0x00000002		/* ip if table id */
#define CAM_VLAN_TBL_ID		0x00000003		/* vlan table id */

#define CAM_FLOW_BASE_IDX	0
#define CAM_IPIF_BASE_IDX	0x8000			/* interface table base index */

#define CAM_VLAN_FLAG		0x00004000		/* vlan flag */

#define WORD_MASK		(sizeof(uint32) * 8 - 1)
#define BITS_PER_WORD	(sizeof(uint32) * 8 )


#define CAM_MAX_ARRAY_IDX	0x7FFF	/* max. data/mask array idx. (0..32K-1) */
#define CAM_MAX_REG_IDX		0x003F	/* max. register idx. (0..63) */
#define CAM_MAX_SRAM_IDX	0x0FFF	/* max. sram idx. (0..4K-1 ?)*/

/*--------------------------------------
 * LNI7020 CAM Register Index
 *--------------------------------------
 */
#define CAM_COMP_0_IDX	0
#define CAM_COMP_1_IDX	(CAM_COMP_0_IDX + 1)
#define CAM_COMP_2_IDX	(CAM_COMP_1_IDX + 1)
#define CAM_COMP_3_IDX	(CAM_COMP_2_IDX + 1)
#define CAM_COMP_4_IDX	(CAM_COMP_3_IDX + 1)
#define CAM_COMP_5_IDX	(CAM_COMP_4_IDX + 1)
#define CAM_COMP_6_IDX	(CAM_COMP_5_IDX + 1)
#define CAM_COMP_7_IDX	(CAM_COMP_6_IDX + 1)
#define CAM_COMP_8_IDX	(CAM_COMP_7_IDX + 1)
#define CAM_COMP_9_IDX	(CAM_COMP_8_IDX + 1)
#define CAM_COMP_10_IDX	(CAM_COMP_9_IDX + 1)
#define CAM_COMP_11_IDX	(CAM_COMP_10_IDX + 1)
#define CAM_COMP_12_IDX	(CAM_COMP_11_IDX + 1)
#define CAM_COMP_13_IDX	(CAM_COMP_12_IDX + 1)
#define CAM_COMP_14_IDX	(CAM_COMP_13_IDX + 1)
#define CAM_COMP_15_IDX	(CAM_COMP_14_IDX + 1)
#define CAM_COMP_16_IDX	(CAM_COMP_15_IDX + 1)
#define CAM_COMP_17_IDX	(CAM_COMP_16_IDX + 1)
#define CAM_COMP_18_IDX	(CAM_COMP_17_IDX + 1)
#define CAM_COMP_19_IDX	(CAM_COMP_18_IDX + 1)
#define CAM_COMP_20_IDX	(CAM_COMP_19_IDX + 1)
#define CAM_COMP_21_IDX	(CAM_COMP_20_IDX + 1)
#define CAM_COMP_22_IDX	(CAM_COMP_21_IDX + 1)
#define CAM_COMP_23_IDX	(CAM_COMP_22_IDX + 1)
#define CAM_COMP_24_IDX	(CAM_COMP_23_IDX + 1)
#define CAM_COMP_25_IDX	(CAM_COMP_24_IDX + 1)
#define CAM_COMP_26_IDX	(CAM_COMP_25_IDX + 1)
#define CAM_COMP_27_IDX	(CAM_COMP_26_IDX + 1)
#define CAM_COMP_28_IDX	(CAM_COMP_27_IDX + 1)
#define CAM_COMP_29_IDX	(CAM_COMP_28_IDX + 1)
#define CAM_COMP_30_IDX	(CAM_COMP_29_IDX + 1)
#define CAM_COMP_31_IDX	(CAM_COMP_30_IDX + 1)

#define CAM_GMR_0_IDX	(CAM_COMP_31_IDX + 1)
#define CAM_GMR_1_IDX	(CAM_GMR_0_IDX + 1)
#define CAM_GMR_2_IDX	(CAM_GMR_1_IDX + 1)
#define CAM_GMR_3_IDX	(CAM_GMR_2_IDX + 1)
#define CAM_GMR_4_IDX	(CAM_GMR_3_IDX + 1)
#define CAM_GMR_5_IDX	(CAM_GMR_4_IDX + 1)
#define CAM_GMR_6_IDX	(CAM_GMR_5_IDX + 1)
#define CAM_GMR_7_IDX	(CAM_GMR_6_IDX + 1)
#define CAM_GMR_8_IDX	(CAM_GMR_7_IDX + 1)
#define CAM_GMR_9_IDX	(CAM_GMR_8_IDX + 1)
#define CAM_GMR_10_IDX	(CAM_GMR_9_IDX + 1)
#define CAM_GMR_11_IDX	(CAM_GMR_10_IDX + 1)
#define CAM_GMR_12_IDX	(CAM_GMR_11_IDX + 1)
#define CAM_GMR_13_IDX	(CAM_GMR_12_IDX + 1)
#define CAM_GMR_14_IDX	(CAM_GMR_13_IDX + 1)
#define CAM_GMR_15_IDX	(CAM_GMR_14_IDX + 1)

#define CAM_SSR_0_IDX	(CAM_GMR_15_IDX + 1)
#define CAM_SSR_1_IDX	(CAM_SSR_0_IDX + 1)
#define CAM_SSR_2_IDX	(CAM_SSR_1_IDX + 1)
#define CAM_SSR_3_IDX	(CAM_SSR_2_IDX + 1)
#define CAM_SSR_4_IDX	(CAM_SSR_3_IDX + 1)
#define CAM_SSR_5_IDX	(CAM_SSR_4_IDX + 1)
#define CAM_SSR_6_IDX	(CAM_SSR_5_IDX + 1)
#define CAM_SSR_7_IDX	(CAM_SSR_6_IDX + 1)

#define CAM_CMD_IDX		(CAM_SSR_7_IDX + 1)
#define CAM_INFO_IDX	(CAM_CMD_IDX + 1)

#define CAM_RBUR_IDX	(CAM_INFO_IDX + 1)
#define CAM_WBUR_IDX	(CAM_RBUR_IDX + 1)

#define CAM_NFA_IDX		(CAM_WBUR_IDX + 1)

#define CAM_IF_REG_BASE	0x200000;				/* CAM Interface Reg. base */

/*--------------------------------------
 * CAM Command Register Bit Definitiions
 *--------------------------------------
 */
#define CAM_CMD_SRST	0x00000001		/* [0]   Software Reset */
#define CAM_CMD_DEVE	0x00000002		/* [1]   Device enable */
#define CAM_CMD_TLSZ_1	0x00000000		/* [3:2] Table Size: 1 device */
#define CAM_CMD_HLAT	0x00000000		/* [6:4] Hit Signal Latency: 0 */
#define CAM_CMD_LDEV	0x00000080		/* [7]   Last Device: Set */
#define CAM_CMD_LRAM	0x00000100		/* [8]   Last Device on SRAM: Set */
#define CAM_CMD_CFG_136	0x0000AA00		/* [16:9] DB Cfg: 4 x 4K x 136 */

/*--------------------------------------
 * LUC STATUS register Bit Definition
 *--------------------------------------
 */
#define CAM_STATUS_CYC	 0x00000001	/* read in the same cycle as was received */
#define CAM_STATUS_NCLR  0x00000002 /* the previous commands has not be cleared*/
#define CAM_STATUS_OK	 0x00000004	/* cmd ok */
#define CAM_STATUS_INIT	 0x00000008 /* cam initiation done */


/*--------------------------------------
 * CAM INDEX SEARCH register Bit Definition
 *--------------------------------------
 */
#define CAM_IDXS_HIT	0x00001000 /* search hit */

/*--------------------------------------
 * CAM Info Register Bit Definitiions
 *--------------------------------------
 */
#define CAM_INFO_REV_MASK	0x0000000F	/* [3:0] Revision # mask */
#define CAM_INFO_IMP_MASK	0x00000070	/* [6:4] Implementation mask */
#define CAM_INFO_LDEV_MASK	0x00000F00	/* [11:8] Device ID LSB mask */
#define CAM_INFO_HDEV_MASK	0x0000E000	/* [15:13] Device ID MSB mask */
#define CAM_INFO_MFG_MASK	0xFFFF0000	/* [31:16] Mfg ID mask */

#define CAM_INFO_REV_ID		0x00000001	/* Rev = 0001 */
#define CAM_INFO_IMP_ID0	0x00000000	/* LNI7020 Impl.# = 000 */
#define CAM_INFO_IMP_ID1	0x00000010	/* LNI7020 Impl.# = 001 */
#define CAM_INFO_LDEV_ID1	0x00000100	/* LSB Dev ID = 0001 */
#define CAM_INFO_LDEV_ID2	0x00000200	/* LSB Dev ID = 0010 */
#define CAM_INFO_HDEV_ID	0x00000000	/* MSB Dev ID = 000 */
#define CAM_INFO_MFG_ID		0xDC7F0000	/* MFG ID = 1101_1100_0111_1111 */

/*--------------------------------------
 * CAM Interface Command Reg Bit Definitiions
 *--------------------------------------
 */
#define CAM_IFCMD_BLK_DATA	0x00000000	/* [1:0] data array block: 00 */
#define CAM_IFCMD_BLK_MASK	0x00000001	/* [1:0] mask array block: 01 */
#define CAM_IFCMD_BLK_REG	0x00000003	/* [1:0] register block:   11 */

#define CAM_IFCMD_CMD_RD	0x00000000	/* [3:2] read cmd code : 00 */
#define CAM_IFCMD_CMD_WR	0x00000004	/* [3:2] write cmd code: 01 */
#define CAM_IFCMD_CMD_SR	0x00000008	/* [3:2] search cmd code:10 */

/*--------------------------------------
 * LUC-CAM IF Reg per LUC HW Arch. Doc.
 *--------------------------------------
 */

typedef struct
{
	uint32	lucCmd;				/* LUC_CMD */
	uint32	lucStatus;			/* LUC_STATUS */
	uint32	lucStatusClr;		/* LUC_STATUS_CLEAR */
	uint32	reserv1[29];		/* 29 reserved space, 0xC-0x7F */
	uint32	ndmpDataPort;		/* NDMP_DATA_PORT */
	uint32	reserv2[29];		/* 29 reserved space 0x84-0xF7 */
	uint32	camAssDataW[2];		/* CAM_ASC_DATA_W_0 and 1 */
	uint32	camDataW[6];		/* CAM_DATA_W[0-4] and one reserve */
	uint32	camCmd;				/* CAM_CMD */
	uint32	camIdx;				/* CAM_INDEX */
	uint32	camAssDataR[2];		/* CAM_ASC_DATA_R_0 and 1 */
	uint32	camDataR[7];		/* CAM_DATA_R[0-4], and two reserve */
	uint32	camIdxS;			/* CAM_INDX_S */
	uint32	reserv3[2];			/* 2 reserved space 0x148-0x14F */
	uint32	gioPar;				/* GIO_PAR */
	uint32	gioCfg;				/* GIO_CFG */
	uint32  reserv4[2];			/* 2 reserved space 0x158-0x15F */
	uint32	txIpgCfg;			/* TX_IPG_CFG */
	uint32	txFifoWm;			/* TX_FIFO_WM */
} luc_camIfReg_t;


/*--------------------------------------
 * LUC-CAM config structure.
 *--------------------------------------
 */
typedef struct
{
	luc_camIfReg_t		*camBase;
	uint32				*sramBase;
	uint32				flowBitMap[CAM_MAX_FLOW_BITMAP];
	uint32				ipIfBitMap[CAM_MAX_IPIF_BITMAP];
	void				*lkupTbl[CAM_MAX_FLOW_ENTRY]; /* flow table, ptr to pConn */
	void				*lkupIpTbl[CAM_MAX_IPIF_ENTRY]; /* interface table, TBD */
} luc_camCfg_t;

/*--------------------------------------
 * Function Prototypes.
 *--------------------------------------
 */
void luc_camInit( void );
void luc_camDelay( uint32 delay );
void *luc_camGetUsrInfo( int32 idx );
void luc_camSetUsrInfo( int32 idx, void *usrInfo );
int32 luc_camCheck(uint32 blk, uint32 idx);

int32 luc_camWrite(int32	blk, 	/* register, data or mask array block */
				   int32	idx, 	/* register, data or mask array index */
				   int		host,   /* is data in host order? */
				   uint32	data_w0,
				   uint32	data_w1,
				   uint32	data_w2,
				   uint32	data_w3,
				   uint32	data_w4);

int32 luc_camRead(int32		blk, 		/* register, data or mask array */
				  int32		idx, 		/* register, data or mask array index */
				  int		host,		/* data in host order or network order */
				  uint32	*data_r0,	
				  uint32	*data_r1,
				  uint32	*data_r2,
				  uint32	*data_r3,
				  uint32	*data_r4);

int32 luc_camFind(int32		blk, 		/* register, data or mask array */
				  int		host,		/* data in host order or network order */
				  uint32	data_r0,	
				  uint32	data_r1,
				  uint32	data_r2,
				  uint32	data_r3,
				  uint32	data_r4);

int32 luc_camAlloc(int32 type);

static int32 luc_camFree( int32 type, int32 idx );

int32 luc_camAddFlow(int32 	prot, 		/* protocol: UDP/TCP */
					 uint32 sip, 		/* source IP */
					 uint32 dip, 		/* destination IP */
					 uint32 sport, 		/* source port */
					 uint32 dport);		/* destination port */

int32 luc_camDelFlow(int32 idx );

int32 luc_camAddIpIf( int32 link, int32 vlan, uint32 ip );

int32 luc_camDelIpIf( int32 idx );
	
#endif /* _LUC_CAM_H */
