/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-pkthdr.h
 *
 * RCS:  $Id: luc-pkthdr.h,v 1.3 2002/08/13 16:51:24 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: contains luc pkt hdr definition
 *
 * Created by:  Su Wang
 *
 * Date Created: 03/18/2002
 *
 *-----------------------------------------------------------------
 */

#ifndef _LUC_PKTHDR_H
#define _LUC_PKTHDR_H


#define LUC_RXPKT_MAXCLASS	64
#define LUC_ETH_MIN			60
#define LUC_ETH_MAX			1514

typedef struct 
{
	uint32	pktCnt;
	uint32	byteCnt;
} luc_class_stats_t;

extern luc_class_stats_t	lucClassStats[];

/*----------------------------------------------- 
 * rx luc pkt hdr according to "luc specification" 
 *section 3.1 
 *-----------------------------------------------
 */

/* rx cam index flags */
#define	LUC_RXPKT_CAM_MISS		0x8000

/* rx classifier code */
#define LUC_RXPKT_CLASS_FULL	0 /* sec. 3.1.2 */
#define LUC_RXPKT_CLASS_MAC		1
#define LUC_RXPKT_CLASS_NOPROT	2	
#define LUC_RXPKT_CLASS_ARP		11
#define LUC_RXPKT_CLASS_ICMP	12
#define LUC_RXPKT_CLASS_FRAG	14
#define LUC_RXPKT_CLASS_IPNOPROT  16

/* rx error code */
#define LUC_RXPKT_ERR_IPCSUM	0x1 /* sec. 3.1.1, ip cksum err */
#define LUC_RXPKT_ERR_TPLCSUM	0x2	/* tcp/udp cksum err */
#define	LUC_RXPKT_ERR_IPHLEN	0x4	/* ip hdr err */
#define LUC_RXPKT_ERR_TPLHLEN	0x8	/* tcp/udp cksum err */


typedef union 
{
	ushort16		data;
	struct 
	{
		/* little endian packing */
		ushort16		f_lucrx_classifier	:6;
		ushort16		f_lucrx_l3OffHW		:4;	
		ushort16		f_lucrx_linkId		:2;
		ushort16		f_lucrx_err			:4;
	} bits;
} luc_rxBit_t;

typedef struct
{
	ushort16	lucrx_txSeq;
	ushort16	lucrx_txErrSeq;
	luc_rxBit_t	lucrx_field;
	ushort16	lucrx_camIdx;
	uchar8		lucrx_l4Offset;
	uchar8		lucrx_l5Offset;
	ushort16	lucrx_chkSum;
	ushort16	lucrx_pktSize;
} luc_rxPktHdr_t;

/* bits field definition for easy reference */ 
#define lucrx_data		lucrx_field.data
#define lucrx_classifier lucrx_field.bits.f_lucrx_classifier
#define lucrx_l3OffHW	lucrx_field.bits.f_lucrx_l3OffHW
#define lucrx_linkId	lucrx_field.bits.f_lucrx_linkId
#define lucrx_err		lucrx_field.bits.f_lucrx_err


/*---------------------------------------------------
 * tx luc pkt hdr according to "luc specification" 
 * section 3.2 
 *---------------------------------------------------
 */

/* tx control bits */
#define LUC_TXPKT_CTRL_NOCSUM	0
#define LUC_TXPKT_CTRL_IPCSUM	1
#define LUC_TXPKT_CTRL_TPLCSUM	0x2

typedef union 
{
	ushort16	data;
	struct 
	{
		ushort16	f_luctx_proto	:8;
		ushort16	f_luctx_ctrl	:4;
		ushort16	f_luctx_linkId	:2;
		ushort16	f_luctx_lFrag	:1;
		ushort16	f_luctx_frag	:1;
	} bits;
} luc_txBit_t;

typedef struct 
{
	ushort16	luctx_seqNum;
	uchar8		luctx_pad;
	uchar8		luctx_hdrOffset; /* offset to start for mac frame */
	luc_txBit_t	luctx_field;
	ushort16    luctx_tplLen;	/* TCP, UDP length of the pkts */	
	uint32		luctx_srcAddr;
	uint32		luctx_dstAddr;
	uchar8		luctx_l3Offset;
	uchar8		luctx_l4Offset;
	uchar8		luctx_tplCsumOffset; /* chksum offset for tcp/udp */
	uchar8		luctx_ipCsumOffset;	/* chksum for ip */
} luc_txPktHdr_t;

/* bits field definition for easy reference */
#define luctx_data		luctx_field.data
#define luctx_proto		luctx_field.bits.f_luctx_proto
#define luctx_ctrl		luctx_field.bits.f_luctx_ctrl
#define luctx_linkId	luctx_field.bits.f_luctx_linkId
#define luctx_lFrag		luctx_field.bits.f_luctx_lFrag
#define luctx_frag		luctx_field.bits.f_luctx_frag

int32 luc_AddTxHdr(eee_desc_t   *eDescp, pkt_desc_t *pDesc);
#endif /* _LUC_PKTHDR_H */



