/* vi:ts=4 
 *-----------------------------------------------------------------
 *
 * Name: luc-reassem.h
 *
 * RCS:  $Id: luc-reas.h,v 1.4 2002/09/10 04:47:17 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains IP re-assembly code. This in effect takes place
 *				of bsd reassembly. We need to perform IP reassembly before
 *				fastpath or slow path is determined.
 *
 * Created by:	Su Wang
 *
 * Date Created: 03/06/2002
 *
 *-----------------------------------------------------------------
 */

#define		LUC_REASHASH_SIZE		256

/* num. of reassemble queues that the same src ip can setup */
#define		LUC_MAX_REAS_PER_SRC	3
#define		LUC_REAS_TIME_OUT	    6	/* 3 second */

#define		LUC_HASH(_src)				\
(((_src & 0xff) + ((_src >> 8) & 0xff) + ((_src>>16)&0xff) +  \
 ((_src>>24)&0xff)) % LUC_REASHASH_SIZE)

/* reassembly pkt structure */
typedef struct luc_frag
{
	struct luc_frag	*frag_next;
	uint32			frag_len;	/* adjusted ip pkt length */
	uint32			frag_off;	/* adjusted ip offset */
	pkt_desc_t		*frag_pdesc;	/* pkt descriptor */
	char			*frag_pHdr;	/* old pHdr from pkt desc */
} luc_frag_t;

/* the children of each hashing element, 
 * start of queue for single reassembling pkt 
 */
struct luc_srcq;
typedef struct luc_reasq 
{
	
	struct luc_reasq  *reas_next; /* next in queue that has the same src address */
	struct luc_srcq *reas_bck;	/* bck ptr to srcq */
	uint32			reas_srcip;
	uint32			reas_destip;
	ushort16		reas_id;
	luc_frag_t		*reas_pkt;	/* chained pkt */
	uint32			reas_count;	/* num. of pkt in queue */
	uint32			reas_ttl;
} luc_reasq_t;

/* hashing element based on src IP address only */
typedef struct luc_srcq
{
	struct luc_srcq	*src_next;
	uint32			src_srcip;
	uint32			src_count;
	luc_reasq_t		*src_first;
	luc_reasq_t		*src_last;
} luc_srcq_t;

typedef struct 
{
	uint32		frag_alloc;
	uint32		frag_free;
	uint32		reas_alloc;
	uint32		reas_free;
	uint32		src_alloc;
	uint32		src_free;
	uint32		force_free;
	uint32		timeout_free;
	uint32		fatal;			/* fatal errs */
	uint32		alloc_err;		/* memory allocation errors */
} luc_memstat_t;


extern luc_memstat_t	luc_memstat;

pkt_desc_t*	luc_Reassem(pkt_desc_t	*pdesc, int link, void** pConn);


