/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-tx.c
 *
 * RCS:  $Id: luc-tx.c,v 1.56 2002/11/20 18:03:55 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: LUC transmit related functions.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/10/2001
 *				 10/01/2001 LUC function is moved from ASIC to SW.
 *							Remove TCR & TRR. Use EEE forward
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>

#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cod-api.h>
#ifndef PROM
#include <exports/sbm-api.h>
#endif
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#include <exports/tpl-api.h>
#include "../sm-tpl/tpl.h"

#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/ip_stats.h"
#include "../sm-bsd-netinet/udp.h"

#include "luc-cam.h"
#include "luc-pkthdr.h"

extern int32 luc_idx;

int
bmc12500Eth_txUnicast( uint32 dest, eee_desc_t *pktDesc );

int32 luc_xmtGmcDataReq(pkt_desc_t *pDesc);

int32 luc_ifGetMtuSize( int32 link )
{
	return 1514;
}

#ifndef PROM
void luc_xmtCmp( pkt_desc_t *pDesc)
{
	luc_stats_t			*pStats;
	int32				ret;
	uint32				status;


#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp(pDesc->pTime, PKT_TIME_IDX_LUC_TX_CMP);
#endif

	status  = 0;

#ifdef PKT_VALIDATE_ENABLE
		if ((ret = pkt_validateDesc(pDesc, PKT_CHK_DUP)) != NFX_OK)
		{
			PKT_LOGERR( ret, pDesc );
		}
#endif

		pStats = &pLucCfg->linkCfg[pDesc->link].txStats;

#if 0
		/* Su, 09/03/02 *
		 * if pkt not owned by appliction free them now
		 */
		if (!(pDesc->flags & PKT_USR_APPL_OWN)) {
			/* Three cases:
			 * 1. TCP send queue owns pkt, do nothing
			 * 2. Conn owns pkt, free back to conn.
			 * 3. none of above, free back to common pool
			 */ 
			if (pDesc->flags & PKT_USR_TCP_TX_Q)
				goto fastfree_end;
			else if (pDesc->flags & PKT_USR_TPL_CONN) {
				pDesc->pTplNext = 0;
				TPL_FREE_PACKET(pDesc->pTplConn, pDesc);
			}
			else {
				pDesc->pNext = 0;
				PKT_FREE_PACKET(pDesc);
			}

fastfree_end:				
			if (pDesc->flags & PKT_FAST_PATH)
				pStats->fastPktCnt++;
			else
				pStats->slowPktCnt++;

			return;
		}
#endif 
		
		/*-------------------------------
		 * Clear driver tx owner flag.
		 *-------------------------------
		 */
		pDesc->flags &= ~PKT_USR_LUC_TX;

		/*-------------------------------
		 * Free EEE desc. from GMC.
		 *-------------------------------
		 */
		if (pDesc->gmcDescp)
		{
			eee_freePacket(pDesc->gmcDescp);
			pDesc->gmcDescp = 0;
			pStats->gmcPktFree++;
		}

		/*-------------------------------
		 * Upcall fast or slow path xmt
		 * complete based on PKT_FAST_PATH. 
		 *-------------------------------
		 */
		if (pDesc->flags & PKT_FAST_PATH)
		{
			/*-------------------------------
			 * Upcall upper layer with Tx Status. 
		 	 * If upper layer done with pkt, 
			 * free pkt desc & buffers.
			 *-------------------------------
			 */
			ret = tpl_xmtCmp( pDesc->pTplConn, pDesc);
			if (ret == PKT_FREE_PKT)
				PKT_FREE_PACKET( pDesc );
			pStats->fastPktCnt++;
		}
		else
		{
			/*-------------------------------
			 * Slow path xmt complete, 
			 * free pkt desc & buffers.
			 *-------------------------------
			 */
			PKT_FREE_PACKET( pDesc );
			pStats->slowPktCnt++;
		}
		

}
#else
void luc_xmtCmp( eee_desc_t *eDescp )
{
	pkt_desc_t		*pDesc;
	luc_stats_t		*pStats;

	// printf("luc_xmtCmp : edesc=%x\n", eDescp );

	pDesc  = (pkt_desc_t *)eDescp->hdr.attr;
	pStats = &pLucCfg->linkCfg[pDesc->link].txStats;

	PKT_FREE_PACKET( pDesc );

	pStats->slowPktCnt++;
}
#endif /* #ifndef PROM */

#ifdef LINUX_TEST
/*-----------------------------------------------------------------
 * Name: 		luc_pollTCR
 *
 * Description:	Polls LUC Transmit Complete Ring and up-calls upper
 *				layer transmit complete routine.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/10/2001
 *-----------------------------------------------------------------
 */
void luc_pollTCR( void *ptr, uint32 tmr_ref)
{
	luc_ring_t 			*ringPtr;
	luc_txCmplDesc_t	*startPtr;
	luc_txCmplDesc_t	*endPtr;
	luc_txCmplDesc_t	*rdPtr;
	luc_txCmplDesc_t	*wrPtr;
	luc_stats_t			*pStats;
	eee_desc_t			*eDescp;
	pkt_desc_t			*pDesc;
	int32				rc;
	int32				ret;
	uint32				pktCnt;
	uint32				status;

	ringPtr  = ptr;
	startPtr = ringPtr->startAddr;
	endPtr   = ringPtr->endAddr;
	rdPtr    = ringPtr->rdAddr;
	wrPtr    = ringPtr->wrAddr;
	
	pDesc  = NULL;
	pktCnt = 0;

	/*-------------------------------
 	 * Test for ring empty conditions.
	 *-------------------------------
	 */
	while (rdPtr != wrPtr)
	{
		/*-------------------------------
	 	 * Get tx status. 
		 *-------------------------------
		 */
		status = rdPtr->status;
		pDesc  = rdPtr->pDesc;

#ifdef PKT_VALIDATE_ENABLE
		if ((rc = pkt_validateDesc(pDesc, PKT_CHK_DUP)) != NFX_OK)
		{
			PKT_LOGERR( rc, pDesc );
		}
#endif

		pStats = &pLucCfg->linkCfg[pDesc->link].txStats;

		/*-------------------------------
		 * Free EEE desc. from GMC.
		 *-------------------------------
		 */
		if (pDesc->gmcDescp)
		{
			eee_freePacket(pDesc->gmcDescp);
			pDesc->gmcDescp = 0;
			pStats->gmcPktFree++;
		}

		/*-------------------------------
		 * Upcall fast or slow path xmt
		 * complete based on PKT_FAST_PATH. 
		 *-------------------------------
		 */
		if (pDesc->flags & PKT_FAST_PATH)
		{
			/*-------------------------------
			 * Upcall upper layer with Tx Status. 
		 	 * If upper layer done with pkt, 
			 * free pkt desc & buffers.
			 *-------------------------------
			 */
			ret = tpl_xmtCmp( pDesc->pTplConn, pDesc);
			if (ret == PKT_FREE_PKT)
				PKT_FREE_PACKET( pDesc );
			pStats->fastPktCnt++;
		}
		else
		{
			/*-------------------------------
			 * Slow path xmt complete, 
			 * free pkt desc & buffers.
			 *-------------------------------
			 */
			ret = pkt_xmtCmp( pDesc, status );
			PKT_FREE_PACKET( pDesc );
			pStats->slowPktCnt++;
		}
		
		pktCnt++;

		/*-------------------------------
	 	 * Update rd ptr & handle wrap.
		 *-------------------------------
		 */
		if (rdPtr != endPtr)
			rdPtr++;
		else
			rdPtr = startPtr;

		if (pktCnt > LUC_TCR_LOOPCOUNT )
			break;
	}

	ringPtr->rdAddr = rdPtr;
}
#endif


#ifndef PROM
/*-----------------------------------------------------------------
 * Name: 	luc_fragDataReq
 *
 * Description:	Performs IP fragmentation for Data Request to GMC.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 01/30/2002
 *-----------------------------------------------------------------
 */
int32 
luc_fragDataReq(pkt_desc_t *pDesc, int32 *fcnt, luc_fragCb_t *fcb)
{
	int32			ip_offset;
	int32			mtuSize;
	int32			hdrSize;
	int32			dataSize;
	int32			xferSize;
	int32			codSize;
	int32			codOffset;
	int32			codNum;
	int32			cnt;
	nfx_cod_entry_t	*pCod;


	pCod      = &pDesc->codEntry[0];
	dataSize  = pDesc->dataSize;
	codOffset = pCod->offset;
	mtuSize	  = luc_ifGetMtuSize( pDesc->link );

	cnt = 0;
	codNum = 0;
	ip_offset = 0;


    /*-------------------------------
     * Size of Pkt Hdrs to be copied:
	 * 1st frag- MAC + IP + UDP + APP.
     *-------------------------------
     */
	if (pDesc->tplProt == TPL_PROT_UDP)
	{
		hdrSize = PKT_ETHER_HDR_SIZE + PKT_IP_HDR_SIZE +
				  PKT_UDP_HDR_SIZE   + pDesc->hdrSize;
		pDesc->refCnt = 0;
	}
	else
		hdrSize = PKT_ETHER_HDR_SIZE + PKT_IP_HDR_SIZE +
				  PKT_TCP_HDR_SIZE   + pDesc->hdrSize;

    /*-------------------------------
     * Go thru the CODs.
     *-------------------------------
	 */
	for (codNum=0; codNum < pDesc->numOfCod; codNum++)
	{
		codSize   = pCod->length;
		codOffset = pCod->offset;

	    /*-------------------------------
	     * Due to upper layer padding,
		 * use dataSize instead of codSize
		 * for last COD.
	     *-------------------------------
		 */
		if (codNum == (pDesc->numOfCod - 1))
		{
			codSize = dataSize;
		}

	    /*-------------------------------
	     * Fragment each COD into MTU.
	     *-------------------------------
		 */
		//printf(" cod length %d, cod size %d, dataSize %d\n", 
		//	   pCod->length, codSize, dataSize);
		
		while (codSize  > 0)
		{
		    /*-------------------------------
		     * Size of avail. xfer buffer:
			 * MTU size excluding headers.
		     *-------------------------------
			 */
			xferSize  = mtuSize - hdrSize;	

		    /*-------------------------------
		     * Transfers upto amount of data.
		     *-------------------------------
			 */
			if (codSize  < xferSize)
				xferSize = codSize ;

		    /*-------------------------------
		     * See if data fits in xfer buffer.
		     *-------------------------------
			 */
			fcb->fcbCodEntry[0].cod       = pCod->cod;
			fcb->fcbCodEntry[0].codOffset = codOffset;
			fcb->fcbCodEntry[0].codLength = xferSize;
//			fcb->hdrSize   = hdrSize;
			fcb->fcbCodNum = 1;	/* by default, each pdesc has only one cod */
//			fcb->padSize   = 0;


		if (pDesc->tplProt == TPL_PROT_UDP)
		{

			int	udpLen = xferSize;

			/* first fragment, xfersize depends on pDesc->hdrSize,
			 * udpLen should be multiple of 8
			 */
			if (cnt == 0)
				udpLen += pDesc->hdrSize;

			/*
			 *-------------------------------
			 * Su, 11/18/02
			 * if 1) xferSize is not multiple of 8 
			 *    2) connection is udp
			 *	  3) there is next cod
			 * we need to include the bytes of
			 * next cod, such that ip fragment size is multiple 
			 * of 8 bytes or makeup whole pkts
			 *
			 * note: if the xfersize is not multiple of 8,
			 *		 it must be the first or last segment of that cod
			 *-------------------------------
			 */
			
			if (((udpLen & 0x7) || (pDesc->pktSize <= mtuSize)) && 
				(codNum < (pDesc->numOfCod - 1))) {

				int					moveBytes;
				nfx_cod_entry_t		*nxtCodPtr;
				
				nxtCodPtr = pCod + 1;
				
				
				/* determine how many bytes we want to move 
				 * from next cod,
				 */
				if (dataSize <= (mtuSize - hdrSize)) 
					/* we can move the entire nxtCod up.
					 * taking padSize into account. nxtCod
					 * must be the last Cod
					 */
					moveBytes = dataSize - xferSize;
				else 
					/* move few bytes up to makeup for multiple of 8 */
					moveBytes = 8 - (xferSize & 0x7);
									
				/* assign second cod in pdesc*/
				fcb->fcbCodEntry[1].cod = nxtCodPtr->cod;
				fcb->fcbCodEntry[1].codOffset = nxtCodPtr->offset;
				fcb->fcbCodEntry[1].codLength = moveBytes;
				
				/* adjust the offset and bytes for next cod */
				nxtCodPtr->length -= moveBytes;
				nxtCodPtr->offset += moveBytes;

				/* adjust xferSize */
				xferSize += moveBytes;
				
				/* two cod with pdesc */
				fcb->fcbCodNum = 2;
			}
			
				

			/*-------------------------------
			 * Set fragment offset, MF bit.
			 *-------------------------------
			 */
			if (cnt==0)
			{
				/*-------------------------------
				 * If pkt fits in MTU, no fragment.
				 * Else set MF bit (offset 0). 
				 *-------------------------------
				 */
				fcb->ipOffset = (pDesc->pktSize > mtuSize) ? (IP_MF) : 0;
				ip_offset = (PKT_UDP_HDR_SIZE + pDesc->hdrSize); 

			}
			else
			{
				/*-------------------------------
				 * IP fragments, set offset. If more
				 * data than xfer buf, set MF bit.
				 *-------------------------------
				 */
				fcb->ipOffset   = (ip_offset >> 3);
				fcb->ipOffset  |= (dataSize > xferSize) ? (IP_MF) : 0;
			}

			pDesc->refCnt++;

		    /*-------------------------------
		     * Size of Pkt Hdrs to be copied:
			 * frag - MAC + IP. 
		     *-------------------------------
		     */
			hdrSize = PKT_ETHER_HDR_SIZE + PKT_IP_HDR_SIZE;
		}
		else
		{
				fcb->ipOffset   = 0;
				hdrSize = PKT_ETHER_HDR_SIZE + PKT_IP_HDR_SIZE +
						  PKT_TCP_HDR_SIZE   ;

		}

			/*-------------------------------
			 * Adjust sizes & offsets by xferSize.
			 *-------------------------------
			 */
			dataSize  -= xferSize;
			codSize   -= xferSize;
			codOffset += xferSize;
			ip_offset += xferSize;

			cnt++;
			fcb++;

		} 	/* while */

		/*-------------------------------
		 * Handle next COD.
		 *-------------------------------
		 */
		pCod++;

	} 	/* for */

	/*-------------------------------
	 * Shaun 09/11/02 bug #2075.
	 *
	 * Remember pad size for last fragment.
	 * To be passed into sbm_sendDataToLuc()
	 * & be excluded from cod length check.
	 *
	 * #2458: use padSize from pRefDesc.
	 *-------------------------------
	 */
	if ((pDesc->pRefDesc) &&
		(pDesc->pRefDesc->padSize))
	{
		fcb--;
//		fcb->padSize = pDesc->pRefDesc->padSize;

//		printf("%s: cod=%llx len=%d offset=%d padSize=%d\n",
//		__FUNCTION__, fcb->cod, fcb->codLength, fcb->codOffset, fcb->padSize);
	}

	*fcnt = cnt;

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		luc_xmtGmcDataReq
 *
 * Description:	Generates a "Send data to LUC" request to MCPU
 *				which in turn sends a message to GMC.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 08/27/2001
 * 				 01/30/2002 Added multi-cod & ip fragmentation support.
 * 				 02/25/2002 Moved call to luc_fragDataReq to TPL. 
 * 				 02/25/2002 Removed luc_xmtDataReq().
 * 				 08/07/2002 Performance tuning: 
 *								replaced eee_sendMsg with direct function 
 *								call to SBM.
 *								combined luc_rcvGmcData in this function.
 *-----------------------------------------------------------------
 */
int32 
luc_xmtGmcDataReq(pkt_desc_t *pDesc)
{
	int32		hdrSize;
	eee_desc_t	*eDescp;
	int			padSize = 0;
	int32		numCod;
	int			i;
	char		*codPtr;
	int			codCpyLen;

#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp(pDesc->pTime, PKT_TIME_IDX_LUC_TX);
#endif

    /*-------------------------------
     * Make sure only 1 COD  to send.
	 * COD, length & offset are setup
	 * by TPL. 
     *-------------------------------
     */
	if (pDesc->numOfCod > 2)
	{
		panic("%s BAD COD Count=%d\n", __FUNCTION__, pDesc->numOfCod);
	}

    /*-------------------------------
     * IP fragment has MAC & IP hdrs 
	 * Non fragment or 1st fragment
	 * has MAC, IP, TCP/UDP & APP hdrs.
	 * 
	 * IP offset & UDP/TCP hdr are
	 * setup by TPL.
	 *
	 * su, 06/11/02, non 1st fragment 
     *-------------------------------
     */
	if ((pDesc->flags & PKT_IP_FRAGMENT) &&
		!(pDesc->flags & PKT_IP_FIRST_FRAG)) 
		hdrSize = (pDesc->pTplHdr - pDesc->pMac); 
	else
		hdrSize = pDesc->hdrSize + (pDesc->pHdr - pDesc->pMac); 

	/*-------------------------------
	 * Su, 09/06/02
	 * store send seq.
	 *-------------------------------
	 */
	pDesc->sbmSeq = pLucCfg->sbmSndSeq++;

#ifdef TPL_DBG
	printf("%s: pDesc=%x pktSize=%d dataSize=%d hdrSize=%d\n",
		__FUNCTION__, pDesc, pDesc->pktSize, pDesc->dataSize, hdrSize);
#endif


    /*-------------------------------
     * Setup EEE desc for tx.
	 * MAC, IP, TCP/UDP & App headers
	 * are already setup.
     *-------------------------------
     */
	eDescp = pDesc->eDescp;
    eDescp->bd[0].control = 0;
    eDescp->bd[1].buf     = pDesc->pMac;
    eDescp->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
    eDescp->bd[1].len     = pDesc->pktSize;

	eDescp->hdr.attr      = (void *) pDesc;

    eDescp->hdr.next      = NULL;

    EEE_SET_NUM_BUFS( eDescp, 1 );
    EEE_SET_PKT_LEN(  eDescp, pDesc->pktSize );

	if ((pDesc != pDesc->pRefDesc) ||
		(pDesc->refCnt == 1)) 
		padSize = pDesc->padSize;
	
    /*-------------------------------
     * Call SBM to fill in user data
	 * after app hdr.
     *-------------------------------
     */
//printf("sendDataToLuc, pDesc = %p\n", pDesc);
	numCod = pDesc->numOfCod;
	codPtr = pDesc->pMac + hdrSize;
	for (i = 0; i < numCod; i++) {

		if (numCod == i+1)
			codCpyLen = pDesc->codEntry[i].length - padSize;
		else
			codCpyLen = pDesc->codEntry[i].length;
		
		sbm_sendDataToLuc(pDesc->codEntry[i].cod, 
						  codCpyLen,
						  pDesc->codEntry[i].offset, 
						  0,
						  codPtr,
						  pDesc);
		codPtr += codCpyLen;
	}

	return NFX_OK;
}

int32
luc_xmtGmcDataReqCB( uchar8 * buf, uint32 len, pkt_desc_t *pDesc)
{
	int32		hdrSize;
	eee_desc_t	*eDescp;
	luc_stats_t	*pStats;
	int			more = 0;
	pkt_desc_t *lastFrag = NULL;

    if (!pDesc)
        return NFX_OK;

	/*----------------------------- 
	 * Su, 09/06/02
	 * Check pkt back from sbm is in order
	 * of request sent
	 *-----------------------------
	 */
	if (pDesc->sbmSeq != pLucCfg->sbmAckSeq) 
	{
		panic("%s: sbm pdesc %p out of order, expect %u, get %u\n",
				__FUNCTION__, pDesc, pLucCfg->sbmAckSeq, pDesc->sbmSeq);
	}

	/* su, 11/18/02
	 * make sure all cod requests for pdesc has been answered
	 */
	pDesc->numOfCod--;
	if (pDesc->numOfCod) {
		pStats = &pLucCfg->linkCfg[pDesc->link].txStats;
		pStats->cntCodBound++;
		return NFX_OK;
	}
	
	pLucCfg->sbmAckSeq++;


	/*----------------------------- 
	 * Su, 09/06/02
	 * for fragmented udp pkts, we must 
	 * wait until all fragments are 
	 * ready before send them.
	 *-----------------------------
	 */
	if ((pDesc->tplProt == TPL_PROT_UDP) && (pDesc->pRefDesc)) 
	{
		if (pDesc->pRefDesc != pDesc)	/* this is not the last fragment, */
			return NFX_OK;
		else	
		{						
			/* ready to send, save last fragment last, where
			 * chksum is calculated
			 */
			lastFrag= pDesc;
			more	= (pDesc->refCnt - 1);
			if (more)
				pDesc	= pDesc->pTplNext;
		}
	}

send_again:
//printf("luc_xmtGmcDataReqCB, pDesc = %p\n", pDesc);
	eDescp = pDesc->eDescp;

    pStats = &pLucCfg->linkCfg[pDesc->link].txStats;
	pStats->byteCnt += pDesc->pktSize;
	pStats->gmcPktCnt++;

    /*-------------------------------
     * su, 05/06/02.
     * incr. ip stats, tcp stats already done
     *-------------------------------
     */
    ipstat.ips_localout++;

    /*-------------------------------
     * Set hdrSize to size of usr data
	 * for IP & UDP/TCP checksum.
     *-------------------------------
     */
    hdrSize = pDesc->hdrSize ;
    pDesc->hdrSize = pDesc->pktSize - (pDesc->pHdr  - pDesc->pMac);

    /*-------------------------------
     * Mark pkt as being sent. 
     *-------------------------------
     */
    pDesc->flags |= PKT_USR_LUC_TX | PKT_LUC_THRU ;

    /*-------------------------------
     * Set IP len & checksum.
     *-------------------------------
     */
	tpl_setIpHdr(pDesc->pTplConn, pDesc);

    /*-------------------------------
     * Set TCP/UDP len & checksum.
     *-------------------------------
     */
    if (pDesc->tplProt == TPL_PROT_TCP)
        tpl_setTcpHdr(pDesc->pTplConn, pDesc);
    else
        tpl_setUdpHdr(pDesc->pTplConn, pDesc);

    /*-------------------------------
     * restore hdrsize.
     *-------------------------------
     */
    pDesc->hdrSize = hdrSize;
	
    if (isCheetah()) 
	{               
    	/*-------------------------------
		 * Add luc headers.
	     *-------------------------------
		 */
        luc_AddTxHdr(eDescp, pDesc);

    	/*-------------------------------
		 * load balancing for cheetah
	     *-------------------------------
		 */
#if 0
        if (bmc12500Eth_txLink_edesc(pLucCfg->drvPort[pDesc->link], eDescp )
            != EEE_FORWARDED_PACKET) 
		{
            pStats->discards++;
            luc_simQueueXmtCmpl( eDescp , 0);
        }
#else
        bmc12500Eth_txUnicast(pLucCfg->drvPort[pDesc->link], eDescp );
#endif /* 0 */
    }
    else
        bmc12500Eth_txUnicast(pLucCfg->drvPort[pDesc->link], eDescp );


	if (more) {
		if (more == 1) {			/* send the last udp fragment */
			/* refcnt smaller than actually chained */
			if (pDesc->pTplNext)
				panic("%s: pkt refCnt too small %d\n", more);
			pDesc = lastFrag;
		}
		else if (!(pDesc = pDesc->pTplNext)) {
			/* refCnt bigger than actually chained */
			panic("%s: pkt refCnt too big %d\n", __FUNCTION__, more);
		}
		more--;
		goto send_again;
	}
	
#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp(pDesc->pTime, PKT_TIME_IDX_LUC_TX_CMP);
#endif
	
	return NFX_OK;
}


#endif	/* #ifndef PROM */


/*-----------------------------------------------------------------
 * Name: 	luc_xmtPktReq
 *
 * Description:	Converts a PKT Descriptor to an EEE descriptor &
 *				forwards it to data link driver for transmission.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/10/2001
 *-----------------------------------------------------------------
 */
int32 luc_xmtPktReq( int32 link, pkt_desc_t	 *pDesc )
{
	eee_desc_t		*eDescp;
	luc_stats_t		*pStats;
	uint32			byteCnt;
	int32			rc;

#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp(pDesc->pTime, PKT_TIME_IDX_LUC_TX);
#endif

	byteCnt = (pDesc->hdrSize + pDesc->dataSize);
	pStats  = &pLucCfg->linkCfg[link].txStats;

	/*-------------------------------
 	 * Make sure pkt desc is valid.
	 *-------------------------------
	 */
#ifdef PKT_VALIDATE_ENABLE
	if ((rc = pkt_validateDesc( pDesc , 1)) != NFX_OK)
	{
		PKT_LOGERR( rc, pDesc );
		pStats->badPktDesc++;
	}
#endif

	/*-------------------------------
	 * Mark pkt as being sent.
	 *-------------------------------
	 */
	pDesc->flags |= PKT_USR_LUC_TX | PKT_LUC_THRU;

	/*-------------------------------
 	 * Setup buf desc. len & buff ptr.
	 *-------------------------------
	 */
	eDescp= pDesc->eDescp;
	eDescp->bd[1].len = pDesc->pktSize;
	eDescp->bd[1].buf = pDesc->pMac;

	/*-------------------------------
 	 * Save pkt desc in attr.
	 *-------------------------------
	 */
	eDescp->hdr.attr = (void *) pDesc;

	/*-------------------------------
 	 * Setup dest port to Data link driver.
	 *-------------------------------
	 */
	eDescp->hdr.dest_port = pLucCfg->drvPort[link];
	eDescp->hdr.src_port  = pLucCfg->rcvPort;
    eDescp->hdr.control   = (pDesc->pktSize & EDESC_TOTAL_LEN) |
                            (( 1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

#ifdef	PROM
	/*-------------------------------
 	 * Setup tx cmp func idx.
	 *-------------------------------
	 */
    eDescp->hdr.control  |= (pLucCfg->xmtCmpId << EDESC_FREE_ROUTINE_SHIFT);
#endif

	luc_trace("luc_xmtPkt",LUC_TRACE_PKT, pDesc, pDesc->pMac, pDesc->pktSize);

	if (isCheetah()) {
#if !defined(PROM)  
		/* add luc headers */
		if (luc_AddTxHdr(eDescp, pDesc) != NFX_OK) {
			pStats->verifyErr++;
			luc_xmtCmp(pDesc);
			return NFX_ERR;
		}
#else
#if defined(SIBYTE)
		if (link != LUC_BP_LINK_ID) {
			if (luc_AddTxHdr(eDescp, pDesc) != NFX_OK) {
				pStats->verifyErr++;
				luc_xmtCmp(pDesc);
				return NFX_ERR;
			}
		}
#endif /* SIBYTE */	
#endif /* PROM */
	}
	

	/* su, 05/06/02.
	 * incr. ip stats, tcp stats already
	 * done
	 */
	ipstat.ips_localout++;
	

	/*-------------------------------
 	 * Check if user data exists. 
	 *-------------------------------
	 */

#ifdef PROM
#ifndef SIBYTE
#ifdef R9000
#ifdef AGILE_HARDWARE
		dp83816_txUnicast( eDescp->hdr.dest_port, eDescp );
#else
		gt64440mii_txUnicast( eDescp->hdr.dest_port, eDescp );
#endif
#else
		gt64240mii_txUnicast( eDescp->hdr.dest_port, eDescp );
#endif		
#else
		if (link == LUC_BP_LINK_ID) {
#ifndef PROM_SIBYTE_BC				
			dp83820_txUnicast( eDescp->hdr.dest_port, eDescp );        
#endif			
		}	
		else
			bmc12500Eth_txUnicast( eDescp->hdr.dest_port, eDescp );       
#endif
#else
#if	defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
		if (isCheetah()) {			/* load balancing for cheeta */
#if 0
            if (bmc12500Eth_txLink_edesc(eDescp->hdr.dest_port, eDescp )
                != EEE_FORWARDED_PACKET) {
                pStats->discards++;
                luc_simQueueXmtCmpl( eDescp , 0);
            }
#else
			bmc12500Eth_txUnicast( eDescp->hdr.dest_port, eDescp );
#endif /* 0*/
		}
		else
			bmc12500Eth_txUnicast( eDescp->hdr.dest_port, eDescp );        
#endif
#endif
/*
		eee_forwardPacket( eDescp );
*/

	pStats->pktCnt++;
	pStats->byteCnt += pDesc->pktSize;

#ifdef PKT_USE_TIME_STAMP
    if (pDesc->pRefDesc)
        pDesc = pDesc->pRefDesc;

    pkt_setTimeStamp(pDesc->pTime, PKT_TIME_IDX_LUC_TX_CMP);
#endif

	return NFX_OK;
}


int32
luc_AddTxHdr(eee_desc_t   *eDescp, pkt_desc_t *pDesc) 
{
	struct ip		*pIpHdr = 0;
	luc_txPktHdr_t	*lucPktHdrPtr;
	eee_descBuf_t   *bd;
	int				ePktSize;
	uchar8			hdrOffset = 0;
	ushort16		ipOffset;

	ePktSize = (eDescp->hdr.control & EDESC_TOTAL_LEN);
	bd = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
	    &(eDescp->bd[0]) : &(eDescp->bd[1]);

	/* move buf to 32 bit aligment */
	hdrOffset = ((address_t)bd->buf & 0x3);

	/* sanity chk, can be removed later, there
	 *  there is enough space to add hdr 
	 */
	if (((address_t)bd->buf & 0x0ff) < 
		(sizeof(luc_txPktHdr_t) + hdrOffset) ||
		(hdrOffset & 0x1)) {	/* hdrOffset must be even */
		return NFX_ERR;
	}

	/* lucPktHdrPtr should pt to 32 bit aligned address,
	 * sizeof(luc_txPktHdr_t) == 20
	 */
	lucPktHdrPtr = 
		(luc_txPktHdr_t*)(bd->buf - hdrOffset - sizeof(luc_txPktHdr_t));
	bzero(lucPktHdrPtr, (sizeof(luc_txPktHdr_t) + hdrOffset));

	/* cannot use pDesc directly, because it doesn't have all the info */
	pIpHdr = (struct ip*)(bd->buf + (pDesc->pIpHdr  - pDesc->pMac));	

	/* no chksums for ip from ssc or arp */

	/* setup iphdr chksum flags */
	if ((pDesc->flags & PKT_IP_CSUM_ENA)) {

		lucPktHdrPtr->luctx_ctrl |= LUC_TXPKT_CTRL_IPCSUM;
		
		lucPktHdrPtr->luctx_l3Offset = pDesc->pIpHdr  - pDesc->pMac;
		lucPktHdrPtr->luctx_ipCsumOffset = lucPktHdrPtr->luctx_l3Offset + 10;
		lucPktHdrPtr->luctx_l4Offset = pDesc->pTplHdr - pDesc->pMac;
		pIpHdr->ip_sum = 0;		/* should not calculate csum, change later */
	}

	/* setup tcp/udp chksum */
	if (pDesc->flags & (PKT_UDP_CSUM_ENA | PKT_TCP_CSUM_ENA)) {

		/* setup l4 offset */
		lucPktHdrPtr->luctx_l4Offset = pDesc->pTplHdr - pDesc->pMac;

		/* calculate partial or whole chksum */
		lucPktHdrPtr->luctx_ctrl |= LUC_TXPKT_CTRL_TPLCSUM;

		ipOffset = htons(pIpHdr->ip_off);

		/* setup pseudo hdr for non-fragments or 
		 * the first fragment. (last one sent to luc) 
		 */
		if (!(ipOffset & IP_OFFMASK)) {

			lucPktHdrPtr->luctx_proto = pDesc->tplProt & 0xff;
			memcpy4(&(lucPktHdrPtr->luctx_srcAddr), 
				   (char*)&(pIpHdr->ip_src.s_addr), 4);
			memcpy4(&(lucPktHdrPtr->luctx_dstAddr), 
				   (char*)&(pIpHdr->ip_dst.s_addr), 4);
			
			/* setup udp/tcp len */
			if (!(ipOffset & (IP_MF | IP_RF)))	/* whole pkt */
				lucPktHdrPtr->luctx_tplLen   = 
					htons((ePktSize - lucPktHdrPtr->luctx_l4Offset));
			else	{			/* first fragment,it must be udp*/ 
				if (lucPktHdrPtr->luctx_proto != TPL_PROT_UDP)
					return NFX_ERR;

				/* retrive total len from udp hdr */
				lucPktHdrPtr->luctx_tplLen = 
					*(ushort16*)(bd->buf + lucPktHdrPtr->luctx_l4Offset + 4);

				/* setup fragments bits */
				lucPktHdrPtr->luctx_lFrag = 1;
				lucPktHdrPtr->luctx_frag = 1;
			}
			
			/* setup udp/tcp cksum offset */
			if (pDesc->tplProt == TPL_PROT_TCP) 
				lucPktHdrPtr->luctx_tplCsumOffset = 
					lucPktHdrPtr->luctx_l4Offset + 16;
			else
				lucPktHdrPtr->luctx_tplCsumOffset = 
					lucPktHdrPtr->luctx_l4Offset + 6;

			/* set chksum field to 0, should not calculate chksum */
			*(ushort16*)(bd->buf + lucPktHdrPtr->luctx_tplCsumOffset) = 0;
			
		}
		else {					/* must be non-first fragments */
			lucPktHdrPtr->luctx_frag = 1;
		}
	}
	
	/* setup linkId */
	lucPktHdrPtr->luctx_linkId = (pDesc->link & 0x3);

	/* set hdrOffset */
	lucPktHdrPtr->luctx_hdrOffset = hdrOffset;
	
	
	/* update buf length */
    if ((bd->len < LUC_ETH_MIN) && (ePktSize < LUC_ETH_MIN)) {
		/* set 0 to padding to ensure 
		 * cksum correctness
		 */
		bzero((bd->buf + bd->len), (LUC_ETH_MIN - bd->len));
        bd->len = LUC_ETH_MIN;
	}
	
	
	/* move bd buf ahead to luc header */
	bd->len += sizeof(luc_txPktHdr_t) + hdrOffset;
	bd->buf -= (sizeof(luc_txPktHdr_t) + hdrOffset);

	/* update eee pkt size */
	if (ePktSize < LUC_ETH_MIN)
		ePktSize = LUC_ETH_MIN;

	if (ePktSize > LUC_ETH_MAX)
		printf("luc tx: bad pkt size %d\n", ePktSize);
	

	ePktSize += sizeof(luc_txPktHdr_t) + hdrOffset; 

	eDescp->hdr.control &= ~EDESC_TOTAL_LEN;
    eDescp->hdr.control |= (ePktSize & EDESC_TOTAL_LEN);

#if 0
	printf("%s: pkt fwd, desc %lx, buf %lx, hdrOffset %d, linkid %d\n", 
		   __FUNCTION__, (address_t)eDescp, (address_t)bd->buf, 
		   lucPktHdrPtr->luctx_hdrOffset, lucPktHdrPtr->luctx_linkId);
	
	printf("frag %d, fst_frag %d, ctrl %x, proto %d, tplLen %d, src %x, dest %x\n",
		   lucPktHdrPtr->luctx_frag, lucPktHdrPtr->luctx_lFrag, 
		   lucPktHdrPtr->luctx_ctrl,
		   lucPktHdrPtr->luctx_proto, ntohs(lucPktHdrPtr->luctx_tplLen), 
		   lucPktHdrPtr->luctx_srcAddr, lucPktHdrPtr->luctx_dstAddr);

	printf("pkt size %d, offsets l3/l4/tplCksum/ipCksum %d/%d/%d/%d\n",
		   ePktSize, lucPktHdrPtr->luctx_l3Offset,
		   lucPktHdrPtr->luctx_l4Offset, lucPktHdrPtr->luctx_tplCsumOffset,
		   lucPktHdrPtr->luctx_ipCsumOffset);
#endif /* 0 */

	/* convert luxrx_field to big endian */
	lucPktHdrPtr->luctx_data = htons(lucPktHdrPtr->luctx_data);

#if 0	
	bsd_ShowPkt(bd->buf, 96);
#endif /* 0 */

	return NFX_OK;
}

