/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-utils.c
 *
 * RCS:  $Id: luc-utils.c,v 1.28 2002/11/20 18:03:56 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains LUC utilities routines.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/01/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/cm-api.h>
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#include "luc-pkthdr.h"


int32 luc_traceFlag = 0;
char  print_buf[128];


static void luc_showLinkStats( int32 link)
{
	luc_stats_t	*pRxStats;
	luc_stats_t	*pTxStats;

	pTxStats = &pLucCfg->linkCfg[link].txStats;
	pRxStats = &pLucCfg->linkCfg[link].rxStats;
	
    printf("\n");
    printf("luc alloc cnt=%d\n", pLucCfg->allocCnt);
    printf("\n");
    printf("\n");
    printf("Link %d statistics            RX            TX\n", link);
	printf("-----------------------------------------------------\n");
    printf("  MCPU Error   :    %12u  %12u\n", 
					pRxStats->mcpuErr, pTxStats->mcpuErr);
    printf("  No EEE Desc  :    %12u  %12u\n", 
					pRxStats->noEeeDesc, pTxStats->noEeeDesc);
    printf("  No Pkt Desc  :    %12u  %12u\n", 
					pRxStats->noPktDesc, pTxStats->noPktDesc);
    printf("  No Buffer    :    %12u  %12u\n", 
					pRxStats->noBuffErr, pTxStats->noBuffErr);
    printf("  Bad Pkt Desc :    %12u  %12u\n", 
					pRxStats->badPktDesc, pTxStats->badPktDesc);
    printf("  MCast Count  :    %12u  %12u\n", 
					pRxStats->mcastCnt, pTxStats->mcastCnt);
    printf("  Packets Count:    %12u  %12u\n", 
					pRxStats->pktCnt, pTxStats->pktCnt);
    printf("  Bytes Count  :    %12u  %12u\n", 
					pRxStats->byteCnt, pTxStats->byteCnt);
    printf("  Pkt Discarded:    %12u  %12u\n", 
					pRxStats->discards, pTxStats->discards);
    printf("  Slow Path Pkt:    %12u  %12u\n", 
					pRxStats->slowPktCnt, pTxStats->slowPktCnt);
    printf("  Fast Path Pkt:    %12u  %12u\n", 
					pRxStats->fastPktCnt, pTxStats->fastPktCnt);
    printf("  GMC Pkt Count:    %12u  %12u\n", 
					pRxStats->gmcPktCnt, pTxStats->gmcPktCnt);
    printf("  GMC Pkt Free :    %12u  %12u\n", 
					pRxStats->gmcPktFree, pTxStats->gmcPktFree);
    printf("  Pkt Cod Bound:    %12u  %12u\n", 
					pRxStats->cntCodBound, pTxStats->cntCodBound);	
	printf("  Luc Verfy Err:    %12u  %12u\n", 
					pRxStats->verifyErr, pTxStats->verifyErr);
	printf("  Luc Gen Err  :    %12u  %12u\n", 
					pRxStats->hwErr, pTxStats->hwErr);
	printf("  Luc No Class :    %12u  %12u\n", 
					pRxStats->hwBadClass, pTxStats->hwBadClass);
	printf("  Luc Bad Link :    %12u  %12u\n", 
					pRxStats->linkErr, pTxStats->linkErr);
	printf("  IP  Ver Err  :    %12u  %12u\n", 
					pRxStats->hwBadIpVer, pTxStats->hwBadIpVer);
    printf("  IP  Csum Err :    %12u  %12u\n", 
					pRxStats->hwBadIpCsum, pTxStats->hwBadIpCsum);
    printf("  TPL Csm Err  :    %12u  %12u\n", 
					pRxStats->hwBadTplCsum, pTxStats->hwBadTplCsum);
    printf("  IP Hlen Err  :    %12u  %12u\n", 
					pRxStats->hwBadIpHlen, pTxStats->hwBadIpHlen);
    printf("  TCP Hlen Err :    %12u  %12u\n", 
					pRxStats->hwBadTcpHlen, pTxStats->hwBadTcpHlen);
	printf("  Reass. CkSum :    %12u  %12u\n",
					 pRxStats->reaChkSumErr, pTxStats->reaChkSumErr);
	printf("\n");
}

luc_class_stats_t	lucClassStats[LUC_RXPKT_MAXCLASS];
void luc_showStats( int32 link )
{
	int i;
	
	for (i=0; i< lucMaxLinkNum; i++)
		luc_showLinkStats(i);	/* show link i Stats */


	pkt_showStats( );

	/* show pkts stats by class */
	{
		int i;
		
		printf(" pkts stats by classification \n");
		for (i=0; i<LUC_RXPKT_MAXCLASS; i++) {
			if (lucClassStats[i].pktCnt != 0) {
				char	*name = 0;
				
				switch (i) {
				case LUC_RXPKT_CLASS_FULL:
					name = "Full Class Pkts";
					break;
					
				case LUC_RXPKT_CLASS_MAC:
					name = "Mac Frames";
					break;
					
				case LUC_RXPKT_CLASS_NOPROT:
					name = "Unknown Frames";
					break;
					
				case LUC_RXPKT_CLASS_ARP:
					name = "Arp Frames";
					break;
					
				case LUC_RXPKT_CLASS_ICMP:
					name = "Icmp Pkts";
					break;

				case LUC_RXPKT_CLASS_FRAG:
					name = "Frag IP Pkts";
					break;
					
				case LUC_RXPKT_CLASS_IPNOPROT:
					name = "Unknown IP Pkts";
					break;

				default:
					name = "Don't care";
				}
				
				printf("%20s, class %d, pkt %u, bytes %u\n",
					   name, i, lucClassStats[i].pktCnt, lucClassStats[i].byteCnt);
			}
			
		}

		bzero(lucClassStats, sizeof(lucClassStats));
	}
	
}

void luc_showDebug( )
{
#ifndef PROM
    nfs_displayMntInfo();		/* Show Mount Stats */
    nfs_displayStat();			/* Show NFS Stats */
#endif
    tpl_showStats();			/* Show TPL Stats */
    luc_showLinkStats( 0);   	/* Show LUC link 0 Stats */
    luc_showLinkStats( 1);    	/* Show LUC link 1 Stats */
    pkt_showStats( );			/* Show PKT Stats */
	pkt_showInUse( );			/* Show PKT In use owners/stats */
}

#ifdef NOTUSED
int32 luc_showTCR( void )
{
	int32		i;
	luc_ring_t 	*pRing;
	luc_txCmplDesc_t	*pTxCmp;

	pRing  = &pLucCfg->tcr;
	pTxCmp = pRing->startAddr;

	printf( "LUC TCR: 0x%08x \n", pRing);
	printf("--------------------------------------\n");
	printf( "Start=0x%08x  End=0x%08x  Read=0x%08x  Write=0x%08x\n",
			pRing->startAddr, pRing->endAddr,
			pRing->rdAddr, pRing->wrAddr); 


	for (i=0; i < LUC_MAX_DESC_TCR; i++)
	{
		printf( "STATUS   : 0x%08x\n", pTxCmp->status);
		printf( "PKT DESC : 0x%08x\n", pTxCmp->pDesc);
		printf( "\n");
		pTxCmp++;
	}
	return 0;

}
#endif


void luc_showPkt( byte *pkt, int32 len)
{
    int32 i, j, remain;
    char c;

	remain = len;
    for (i=0; i < len; i++)
    {
        if (i !=0 && !(i % 16))
        {
            printf("\t");
            for (j=0; j <16; j++)
            {
                c = print_buf[j];
                printf("%c",(((c >= ' ') && (c < '~')) ? c:'.'));

            }
            printf("\n");
			remain -= 16;
        }

        printf("%02x ", pkt[i]);
        print_buf[i%16] = pkt[i];

    }

	if (remain)
	{
		for (i=0; i < (16 - remain); i++)
		{
	        printf("   ");
		}

		printf("\t");
	    for (j=0; j < remain; j++)
	    {
	        c = print_buf[j];
        	printf("%c",(((c >= ' ') && (c < '~')) ? c:'.'));
        }
    	printf("\n");
	}
    printf("\n");

}

void luc_trace( char *msg, int32 type, void *pDesc, byte *pData, int32 datalen)
{
	if (!luc_traceFlag)
		return;

	printf("-----------LUC Trace: %s : Start------------\n",msg);
	switch (type)
	{
		case 	LUC_TRACE_PKT:
				if (luc_traceFlag & LUC_TRACE_PKT)
				{
					if (pDesc)
						pkt_showDesc( pDesc );

					if ((pData) && (datalen))
						luc_showPkt( pData, datalen);
				}		
				break;

		case	LUC_TRACE_TCD:
		case	LUC_TRACE_RCD:
		case	LUC_TRACE_FBD:
		default:
				break;
	}
	printf("-----------LUC Trace: %s : End--------------\n",msg);
}

int32 pkt_xmtCmp(int32 status, pkt_desc_t *pDesc )
{
	return 0;
}

void luc_panic(void)
{
	*((char *)0) = 1;
}


static char *dashLine = "--------------------------------------------------";

void
luc_showCodEntry( int32 count, nfx_cod_entry_t *pCod )
{
	int32	i;

	printf("%s\n", dashLine);
	printf("NumOfCod=%d\n", count);
	for (i=0; i < count; i++)
	{
		printf("CodEntry[%d]:  cod=0x%llx, offset=%d len=%d\n",
				i, pCod->cod, pCod->offset, pCod->length);
		pCod++;
	}
	printf("%s\n", dashLine);
}

void
luc_showFragCb( int32 fcnt, luc_fragCb_t *fcb )
{
    int32   i;

#ifdef LUC_DBG_COD
	printf("%s\n", dashLine);
    for (i=0; i < fcnt; i++)
    {
        printf("fcb %d: cod=0x%llx ipOff=0x%x codLen=%d codOff=%d\n",
                i, fcb->fcbCodEntry[0].cod, fcb->ipOffset,
                fcb->fcbCodEntry[0].codLength, 
			    fcb->fcbCodEntry[0].codOffset);
        fcb++;
    }
	printf("%s\n", dashLine);
#endif
}
