/*
 *-----------------------------------------------------------------
 *
 * Name:	malloc-ui.c
 *
 * RCS:  $Id: malloc-ui.c,v 1.1.1.1 2002/10/30 21:56:38 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	UI functions for malloc module
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/4/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/malloc-api.h>
#include <exports/timer-api.h>
#include "malloc.h"

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showStats()
{
	uint32	i;

    if(mmBase.stackSize > 0)
    {
        printf("\n");
        printf("Malloc Stacks:\n");
        for (i=0; i<MM_NUM_STACK_SIZES; i++) {
            printf("   num %4d byte buffers           = %d\n",
                   mm_stack_buf_size[i], mm_stack_num_bufs[i]);
        }
        for (i=0; i<MM_NUM_STACK_SIZES; i++) {
            printf("   num %4d byte buffers avail     = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].avail);
        }
        for (i=0; i<MM_NUM_STACK_SIZES; i++) {
            printf("   num %4d byte buffers allocated = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].allocated);
            /*
             * freed = freed - num_bufs, since at init time we FREE all bufs to
             * add to stack
             */
            printf("   num %4d byte buffers freed     = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].freed - mm_stack_num_bufs[i]);
            printf("   num %4d byte buffers unavail   = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].unavail);
        }
    }
    
	printf("\n");

	printf("Malloc Lists:\n");
	printf("   Number of %4d byte buffers = %d\n", MM_BUFSIZE, mm_num_bufs);
	printf("   Number of free buffers      = %d\n", mm_num_free);
	printf("   Largest contiguous buffer   = %d\n", mm_max_contig);
	printf("\n");

	printf("Malloc Errors:\n");
	printf("   malloc request too big  = %d\n", mm_stats.too_big);
	printf("   no contiguous buffer    = %d\n", mm_stats.no_contig);
	printf("   not enough free buffers = %d\n",
											mm_stats.not_enough_free_buffers);
	printf("\n");

	printf("Free space avail is %lld bytes (of %lld total)\n",
		eee_ramTotalHeapAvail(), eee_ramTotalHeapSize());
	printf("\n");

}	/* malloc_showStats() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showFreeList()
{
	mm_free_node_t	* ptr;

	printf("free list head node at %p\n", &mm_free_list);
	printf("    Snext = %p\n", mm_free_list.Snext);
	printf("    Sprev = %p\n", mm_free_list.Sprev);
	printf("    Anext = %p\n", mm_free_list.Anext);
	printf("    Aprev = %p\n", mm_free_list.Aprev);

	printf("\nFREE SIZE LIST:\n");
	ptr = mm_free_list.Snext;
	while (ptr != &(mm_free_list)) {
		printf("    Node: %p\n", ptr);
		printf("        size  = 0x%x\n", ptr->size);
		printf("        bdesc = %p\n", ptr->addr);
		printf("        Snext = %p\n", ptr->Snext);
		printf("        Sprev = %p\n", ptr->Sprev);
		ptr = ptr->Snext;
	}

	printf("\nFREE ADDR LIST:\n");
	ptr = mm_free_list.Anext;
	while (ptr != &(mm_free_list)) {
		printf("    Node: %p\n", ptr);
		printf("        size  = 0x%x\n", ptr->size);
		printf("        bdesc = %p\n", ptr->addr);
		printf("        Anext = %p\n", ptr->Anext);
		printf("        Aprev = %p\n", ptr->Aprev);
		ptr = ptr->Anext;
	}

	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showAllocList()
{
	mm_alloc_node_t	* ptr;

	printf("alloc list head node at %p\n", &mm_alloc_list);
	printf("    next = %p\n", mm_alloc_list.next);

	printf("\nALLOC LIST:\n");
	ptr = mm_alloc_list.next;
	while (ptr) {
		printf("    Node: %p\n", ptr);
		printf("        size  = 0x%x\n", ptr->size);
		printf("        bdesc = %p\n", ptr->addr);
		printf("        next  = %p\n", ptr->next);
		ptr = ptr->next;
	}

	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showFreeNodeStack()
{
	uint32			i;
	mm_free_node_t	** ptr;

	printf("Num free nodes available = %d\n", mm_free_node_stack.avail);
	ptr = mm_free_node_stack.top;
	for (i=0; i<mm_free_node_stack.avail; i++, ptr++)
		printf("    %p\n", *ptr);
	printf("\n");
}

/*-----------------------------------------------------------------

 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showAllocNodeStack()
{
	uint32			i;
	mm_alloc_node_t	** ptr;

	printf("Num alloc nodes available = %d\n", mm_alloc_node_stack.avail);
	ptr = mm_alloc_node_stack.top;
	for (i=0; i<mm_alloc_node_stack.avail; i++, ptr++)
		printf("    %p\n", *ptr);
	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showAllocRanges()
{
	uint32	i, j, inc;

	i = 0;
	j = 0;

	for (inc = 0x20 ; j < 0x200; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x40 ; j < 0x400; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x80 ; j < 0x1000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x100 ; j < 0x2000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x100 ; j < 0x4000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x400 ; j < 0x8000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x800 ; j < 0x10000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x4000 ; j < 0x80000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	if (mm_stats.malloc_size[i] != 0)
		printf("       > 512K   : %d\n", mm_stats.malloc_size[i]);
}


void
malloc_clearHistogram()
{
#ifdef MEM_HISTOGRAM
	bzero(mem_histogram, sizeof(mem_histogram));
#endif  /* MEM_HISTOGRAM */
	bzero(mem_histogram_cont, sizeof(mem_histogram_cont));
}


/*
 * Print the histogram info
 *
 * with
 * mode:  boolean: 0: regular, sort by size, 1: sort by usage (see type)
 * output: boolean: 0: normal, 1: short notation
 * type:  int:     0: max usage, 1: curr usage, 2: total usage 
 * nbr:   if mode == 0 print only if value of given 'type' is bigger than 'nbr'
 * nbr:   if mode == 1 print only first 'nbr' of lines
 */

void
malloc_showHistogram (int mode, int output, int type, int nbr)
{
	cache_show();
	pool_show();
}


int
malloc_test(int argc, char **argv)
{
	if (!strcmp("cleanup", argv[1])) {
		cleanup_caches();
	}
	return NFX_OK;
}


typedef struct _malloc_owner {
	void	* ra;
	uint32	len;
} mm_owner_t;

#define MM_OWNER_MAX	0x2000
mm_owner_t	mm_owner[MM_OWNER_MAX];
uint32		mm_num_ra;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	returns index of RA in mm_owner[] or MM_OWNER_MAX if the RA
 *				is not presently in array
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int32
malloc_getOwnerIndex(
	void	* ra
)
{
	uint32	i;

	for (i=0; i< mm_num_ra; i++) {
		if (mm_owner[i].ra == ra)
			return i;
	}

	return MM_OWNER_MAX;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	returns index into mm_owner[] of new RA, or -1 if array is full
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int32
malloc_addNewOwner(
	void	* ra
)
{
	if (mm_num_ra == MM_OWNER_MAX)
		return -1;

	mm_owner[mm_num_ra].ra = ra;
	mm_owner[mm_num_ra].len = 0;

	mm_num_ra++;

	return (mm_num_ra-1);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	Increments total length allocated by the RA associated with
 *				the given index
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
malloc_incOwnerLen(
	uint32	index,
	uint32	len
)
{
	mm_owner[index].len += len;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
malloc_addOwner(
	void	* ra,
	uint32	len
)
{
	int32	oi;

	if ((oi = malloc_getOwnerIndex(ra)) == MM_OWNER_MAX) {
		if ((oi = malloc_addNewOwner(ra)) == -1) {
			printf("Could not add new malloc owner, ra = %p, len = 0x%x\n",
																ra, len);
			return;
		}
	}
	malloc_incOwnerLen(oi, len);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int
mm_owner_cmp(
	mm_owner_t	* p1,
	mm_owner_t	* p2
)
{
	if (p1->len < p2->len)
		return -1;
	if (p1->len == p2->len)
		return 0;
	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
malloc_showOwner()
{
	mm_alloc_node_t	* anp;
	uint32			i;

#ifndef MM_VALIDATE
	printf("owner information not available if compiled without MM_VALIDATE\n");
	return;
#endif

    if (mmBase.flexSize > 0)
    {
        mm_flex_desc_t  *mmFlex = mmBase.flex;
        mm_flex_t       *flex;
        mm_flex_head    *head;
#ifdef MM_USE_FLEX_TAIL
        mm_flex_tail    *tail;
#endif
        void		    *ra;
        uint32		     len;
        uint32           nbr = 0 ;

        flex = mmFlex->start;

        bzero(mm_owner, MM_OWNER_MAX * sizeof(mm_owner_t));
        mm_num_ra = 0;

        NFX_SPIN_LOCK(&mmBase.flexSpin);

        for (i = 0 ; i < (MM_FLEX_MAX / MM_FLEX_MODULO); i++)
        {
            nbr = 0;
            LIST_FOREACH(head, &(flex[i].useList), entry)
                {
                    ASSERT(head->free_ra == 0);
                    ASSERT(head->malloc_ra);
#ifdef MM_USE_FLEX_TAIL
                    tail = (mm_flex_tail*)(((uchar8*)(head + 1)) + head->length);
                    ASSERT(tail->sig == MM_TAIL_SIG);
                    ASSERT(head->sig == MM_HEAD_SIG);
#endif
                    nbr++;

                    ra = head->malloc_ra;
                    len = head->size;       // the real num_bytes of malloc req
                    // len = head->length;     // the size aligned to the next 64 byte boundary
                    // printf("ra = 0x%x, len = %d\n", ra, len);
                    malloc_addOwner(ra, len);
                }
            ASSERT(nbr == flex[i].histogram[MM_HIST_CUR]);
        }
        
        NFX_SPIN_UNLOCK(&mmBase.flexSpin);

        // printf("calling qsort\n");
        qsort(mm_owner, mm_num_ra, sizeof(mm_owner_t), mm_owner_cmp);
        // printf("back from qsort\n");

        printf("\nOwner list in flex memory\n\n");
        for (i=0; i<mm_num_ra; i++)
            printf("RA : %p, len = %d\n", mm_owner[i].ra, mm_owner[i].len);
    }
    
    if(mmBase.stackSize > 0)
    {
        bzero(mm_owner, MM_OWNER_MAX * sizeof(mm_owner_t));
        mm_num_ra = 0;

        NFX_SPIN_LOCK(&mmBase.stackSpin);

        //printf("Stacks:\n");
        for (i=0; i<MM_NUM_STACK_SIZES; i++) 
        {
            uchar8		* bdesc;
            mm_head_t	* head;
            void		* ra;
            uint32		len;

            //printf("    0x%x bytes:\n", mm_stack_buf_size[i]);
            for (bdesc = mm_stack_buf_base[i]; bdesc < mm_stack_buf_end[i];
                 bdesc += mm_stack_bdesc_size[i]) 
            {
                head = (mm_head_t *)bdesc;
                if ((head->free_ra == 0) && (head->malloc_ra)) 
                {
                    ra = head->malloc_ra;
                    len = head->length;
                    // printf("ra = 0x%x, len = %d\n", ra, len);
                    malloc_addOwner(ra, len);
                }
            }
        }

        NFX_SPIN_UNLOCK(&mmBase.stackSpin);

        // printf("calling qsort\n");
        qsort(mm_owner, mm_num_ra, sizeof(mm_owner_t), mm_owner_cmp);
        // printf("back from qsort\n");

        printf("\nOwner list in fixed size stacks memory\n\n");
        for (i=0; i<mm_num_ra; i++)
            printf("RA : %p, len = %d\n", mm_owner[i].ra, mm_owner[i].len);
	}

	bzero(mm_owner, MM_OWNER_MAX * sizeof(mm_owner_t));
    mm_num_ra = 0;

    NFX_SPIN_LOCK(&mmBase.contSpin);

	//printf("List:\n");
	anp = mm_alloc_list.next;
	while (anp) {
		mm_bdesc_t	* bdesc;
		void		* ra;
		uint32		len;

		bdesc = anp->addr;
		ra = bdesc->head.malloc_ra;
		len = bdesc->head.length;
		//printf("ra = 0x%x, len = %d\n", ra, len);
		malloc_addOwner(ra, len);
		anp = anp->next;
	}

    NFX_SPIN_UNLOCK(&mmBase.contSpin);

//  for (i=0; i<mm_num_ra; i++)
//		printf("RA : 0x%x, len = %d\n", mm_owner[i].ra, mm_owner[i].len);

//	printf("calling qsort\n");
	qsort(mm_owner, mm_num_ra, sizeof(mm_owner_t), mm_owner_cmp);
//	printf("back from qsort\n");

	printf("\nOwner list in continious memory\n\n");
	for (i=0; i<mm_num_ra; i++)
		printf("RA : %p, len = %d\n", mm_owner[i].ra, mm_owner[i].len);
}







