/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-bcm5600.c,v 1.5 2002/07/27 01:05:32 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */

#if defined (DIAG_R7000)
#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include <exports/hwMap.h>
#include <exports/gt64240mii.h>
#include <exports/timer-api.h>
#include <exports/bcm5600-ll.h>
#include "mdk-mem.h"

extern uint32 nfxMyCPUId;

/* Global definations */
extern uint32 mdk_gt_mii_test_port;
extern gt64240mii_cfg_t *gt64240mii;
uint32 mac_lpbk, phy_5228_lpbk, phy_8353_lpbk;
uint32 port0_added_hash = 0;
uint32 testMode;
extern uint32 fc_conflict;

extern uint32	time_handle_port0;
extern uint32	time_handle_port1;
extern uint32	time_handle_port2;

void
bcm5600_pkg_init()
{
	tp->resultCnt = 1;
	tp->param[TP_LBPKT_CNT] = 10000;
#ifdef DIAG_R7000
	if(!time_handle_port0)
		time_handle_port0 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port1)
		time_handle_port1 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port2)
		time_handle_port2 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#endif	
}

/* LOOPBACK TESTS */
/* Loopback test from BCM5600's 8353 phy to control/data plane */

void
bcm5600_reg_test(
	mdk_test_params_t *tp
)
{
}



int32
bcm5600_8353_phy_lb_port0(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 port;
	uint32 phy_id;
	uint32 data;
    uint32 mac1, mac2;

	if(isFC() && (nfxMyCPUId != 0)) { 
			tp->testresult = RET_SKIP_TEST;
			return(tp->testresult);
	}

	if(isFC() && (nfxMyCPUId == 0)) { 
		phy_id = 0x0a;
	}
	mdk_gt_mii_test_port = port = 0;

	if(!port0_added_hash) {
			mac1 = 0x0808;
			mac2 = 0x02000100; 
			gt64240_addHashTableEntry(port, mac1, mac2, 1, 0);
			delay_us(10000);
			port0_added_hash = 1;
	}	
	reg_addr = 0x0; /* Control register */
	data = 0x5140; /* bit 14 control register */
	gt_mii_writeSMI(phy_id, reg_addr, data);
	delay_us(10000);
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	phy_8353_lpbk = 1;
	testMode = GT_LB_TEST_PHY;
	mdk_gt_mii_diagLBTest(port,testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}	

int32
disable_phy_8353(
	mdk_test_params_t	* tp
)
{
	gt_reset_loopback(0);
	tp->testresult = RET_PASSED;
	return(tp->testresult);
}		



#if 0
int32
bcm5600_8353_phy_lb_port2(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 port;
	uint32 phy_id;
	uint32 data, tmp;

	if(isFC() && (nfxMyCPUId != 0)) { 
			tp->testresult = RET_SKIP_TEST;
			return;
	}

	if(isFC() && (nfxMyCPUId == 0)) { 
		/* this does not work, therefore use 0xa */
		/* phy_id = 0x0b; */
		phy_id = 0x0a;
	}
	mdk_gt_mii_test_port = port = 2;
	reg_addr = 0x0; /* Control register */
	data = 0x5140; /* bit 14 control register */
	gt_mii_writeSMI(phy_id, reg_addr, data);
	delay_us(1000);
    gt_mii_readSMI(phy_id, reg_addr, &tmp); 
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	phy_8353_lpbk = 1;
	testMode = GT_LB_TEST_PHY;
	mdk_gt_mii_diagLBTest(port,testMode);
	tp->testresult = RET_SKIPPED;
	return;
}	
#endif

int32
bcm5600_int_lb_port0(
	mdk_test_params_t	* tp
)
{
	/* This test is run from the GT64240 to the 5600
	 * and is looped back at the mac of the 5600
	 */ 
	uint32 gt_port, bcm_port_lb,enable, bcm_port;
    uint32 mac1, mac2;

	mdk_gt_mii_test_port = gt_port = 0;
	enable = 1;
	if((isFC()) && (nfxMyCPUId == 0)) {
		bcm_port_lb = 19;
		bcm_port = 23;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 19, enable);		
	}		

	if(!port0_added_hash) {
			mac1 = 0x0808;
			mac2 = 0x02000100; 
			gt64240_addHashTableEntry(gt_port, mac1, mac2, 1, 0);
			delay_us(1000);
			port0_added_hash = 1;
	}

	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[gt_port].rx_trace = 0;
			gt64240mii->port[gt_port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[gt_port].rx_trace = 1;
			gt64240mii->port[gt_port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[gt_port].rx_trace = 0;
			gt64240mii->port[gt_port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[gt_port].rx_trace = 1;
			gt64240mii->port[gt_port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	mac_lpbk = 1;
	testMode = GT_LB_TEST_MAC;
	mdk_gt_mii_diagLBTest(gt_port,testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}	


int32
bcm5600_int_lb_port2(
	mdk_test_params_t	* tp
)
{
	/* This test is run from the GT64240 to the 5600
	 * and is looped back at the mac of the 5600
	 */ 
	uint32 gt_port,bcm_port_lb, enable, bcm_port;

	mdk_gt_mii_test_port = gt_port = 2;
	enable = 1;
	if((isFC()) && (nfxMyCPUId == 0)) {
		bcm_port_lb = 23;
		bcm_port = 19;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 23, enable);		
	}		
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[gt_port].rx_trace = 0;
			gt64240mii->port[gt_port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[gt_port].rx_trace = 1;
			gt64240mii->port[gt_port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[gt_port].rx_trace = 0;
			gt64240mii->port[gt_port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[gt_port].rx_trace = 1;
			gt64240mii->port[gt_port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	mac_lpbk = 1;
	testMode = GT_LB_TEST_MAC;
	mdk_gt_mii_diagLBTest(gt_port, testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}	

int32
bcm5600_setMacLpbk0(
	mdk_test_params_t	* tp
)
{
	uint32 gt_port, bcm_port_lb,enable, bcm_port;
    uint32 mac1, mac2;

	mdk_gt_mii_test_port = gt_port = 0;
	enable = 1;
	if((isFC()) && (nfxMyCPUId == 0)) {
		bcm_port_lb = 19;
		bcm_port = 23;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 19, enable);		
		bcm_port_lb = 18;
		bcm_port = 22;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 18, enable);		
		bcm_port_lb = 17;
		bcm_port = 21;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 17, enable);		
		bcm_port_lb = 16;
		bcm_port = 20;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 16, enable);		
		tp->testresult = RET_PASSED;
	}		
	else
		tp->testresult = RET_SKIP_TEST;
	return(tp->testresult);
}

int32
bcm5600_setMacLpbk2(
	mdk_test_params_t	* tp
)
{
	uint32 gt_port,bcm_port_lb, enable, bcm_port;

	mdk_gt_mii_test_port = gt_port = 2;
	enable = 1;
	if((isFC()) && (nfxMyCPUId == 0)) {
		bcm_port_lb = 23;
		bcm_port = 19;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 23, enable);		
		bcm_port_lb = 22;
		bcm_port = 18;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 22, enable);		
		bcm_port_lb = 21;
		bcm_port = 17;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 21, enable);		
		bcm_port_lb = 20;
		bcm_port = 16;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 20, enable);		
		tp->testresult = RET_PASSED;
	}		
	else
		tp->testresult = RET_SKIP_TEST;
	return(tp->testresult);
}

int32
disable_mac_lb0(
	mdk_test_params_t	* tp
)
{
	uint32 bcm_port_lb,disable;

	disable = 0; /* disable */
	if((isFC())  && (nfxMyCPUId == 0)) {
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 19, disable);		
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 18, disable);		
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 17, disable);		
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, 16, disable);		
	mac_lpbk = 0;
	tp->testresult = RET_PASSED;
	}		
	else
		tp->testresult = RET_SKIP_TEST;
	return(tp->testresult);
}

int32
disable_mac_lb2(
	mdk_test_params_t	* tp
)
{
	uint32 bcm_port_lb,disable;

	disable = 0; /* disable */
	if((isFC()) && (nfxMyCPUId == 0)) 
	{
			MAC_LOOPBACK_SET(&mac_fe_5600, 0, 23, disable);		
			MAC_LOOPBACK_SET(&mac_fe_5600, 0, 22, disable);		
			MAC_LOOPBACK_SET(&mac_fe_5600, 0, 21, disable);		
			MAC_LOOPBACK_SET(&mac_fe_5600, 0, 20, disable);		
	mac_lpbk = 0;
	tp->testresult = RET_PASSED;
	}		
	else
		tp->testresult = RET_SKIP_TEST;
	return(tp->testresult);
}


int32
bcm5600_pkg_err(
	mdk_log_entry_t	* err
)
{
	switch (err->testresult) {
		case ERR_ADDR_FAIL:
				break;
		case ERR_TIMEOUT:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nLoopback Test Timeout\n");
			break;
		default:
			printf("Unknown error, testresult = 0x%x\n", err->testresult);
			break;
	}

	return 0;
}

#endif
