/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-lsi8104.c,v 1.2 2002/08/28 22:21:38 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	lipika	
 *
 * Date Created:	8/20/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"
#if defined (DIAG_SIBYTE)
#include <exports/timer-api.h>
#include "../../sm-lsi8104/lsi8104.h"

extern uint32	time_handle_dp83820;
extern uint32	time_handle_port0;
extern uint32	time_handle_port1;
extern uint32	time_handle_port2;
extern uint32	time_handle_port3;
extern uint32	time_handle_port4;
extern uint32	time_handle_port5;


int32 sb_phy_lb_test(mdk_test_params_t	* tp, uint32 port, uint32 testMode);

/******************** FUNCTIONS ***********************************/

int32
lsi8104_pkg_init()
{
	tp->resultCnt = 1;
	tp->param[TP_LBPKT_CNT] = 10000;
	tp->param[TP_PKT_TRACE] = 0;
	if(!time_handle_port0)
		time_handle_port0 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port1)
		time_handle_port1 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port2)
		time_handle_port2 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port3)
		time_handle_port3 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port4)
		time_handle_port4 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port5)
		time_handle_port5 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
}

/* MAIN TEST FUNCTIONS */

int32 
sb_lsiMac_Regtest(mdk_test_params_t *tp)
{
	int i,j, regval;
	int num_devices = 0;

	if (isTXRX())
		num_devices = LSI_NUM_DEVICES_TXRX;
	else if (isFP())
		num_devices = LSI_NUM_DEVICES_FP;

	if (num_devices > 0) {

		for (i = 0; i < num_devices; i++) 
			lsi_gmac_write(i, 0, i+1);

		for (i = 0; i < num_devices; i++) {
			regval = lsi_gmac_read(i, 0);
			if (regval != i+1) {
				tp->testresult = ERR_DATA_FAIL;
				tp->result[0].addr[i+j] = 0;
				tp->result[0].exptd[i+j] = i+1;
				tp->result[0].rcvd[i+j] = regval;	
				j++;
			}	
			else
				tp->testresult = RET_PASSED;
		}
		for (i = 0; i < num_devices; i++) 
			lsi_gmac_write(i, 0, 0);

		for (i = 0; i < num_devices; i++) {
			regval = lsi_gmac_read(i, 0);
			if (regval != 0) {	
				tp->testresult = ERR_DATA_FAIL;
				tp->result[0].addr[i+j] = 0;
				tp->result[0].exptd[i+j] = 0;
				tp->result[0].rcvd[i+j] = regval;	
				j++;
			}		
			else
				tp->testresult = RET_PASSED;
		}
	}
	else
		printf("No devices found!\n");

	return(tp->testresult);
}		

int32
sb_lsi_mac_linkUp_test_port0(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_status_t status;

	status.wd = lsi_gmac_read(0, LSI_GMAC_STATUS);
	delay_us(10000);
	if (status.bits.link)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;

	tp->devName = "LSI Port 0";

	return(tp->testresult);
}

int32
sb_lsi_mac_linkUp_test_port1(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_status_t status;

	status.wd = lsi_gmac_read(1, LSI_GMAC_STATUS);
	delay_us(10000);
	if (status.bits.link)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;

	tp->devName = "LSI Port 1";

	return(tp->testresult);
}

int32
sb_lsi_mac_linkUp_test_port2(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_status_t status;

	status.wd = lsi_gmac_read(2, LSI_GMAC_STATUS);
	delay_us(10000);
	if (status.bits.link)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;

	tp->devName = "LSI Port 2";

	return(tp->testresult);
}

int32
sb_lsi_mac_linkUp_test_port3(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_status_t status;

	status.wd = lsi_gmac_read(3, LSI_GMAC_STATUS);
	delay_us(10000);
	if (status.bits.link)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;

	tp->devName = "LSI Port 3";

	return(tp->testresult);
}

int32
sb_lsi_mac_linkUp_test_port4(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_status_t status;

	if(isTXRX())
		return(tp->testresult = RET_SKIP_TEST);

	status.wd = lsi_gmac_read(4, LSI_GMAC_STATUS);
	delay_us(10000);
	if (status.bits.link)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;

	tp->devName = "LSI Port 4";

	return(tp->testresult);
}

int32
sb_lsi_mac_linkUp_test_port5(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_status_t status;

	if(isTXRX())
		return(tp->testresult = RET_SKIP_TEST);

	status.wd = lsi_gmac_read(5, LSI_GMAC_STATUS);
	delay_us(10000);
	if (status.bits.link)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;

	tp->devName = "LSI Port 5";

	return(tp->testresult);
}

/* LOOPBACKS */
int32
sb_lsi_mac_lb_test_port0(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;
	lsi_gmac_conf4_t conf;
	uint32 port,unit;

	unit = port = 0;

	mdk_lsi_checkSync(port);

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	delay_us(10000);
	conf.bits.loopback = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
	delay_us(10000);

	testMode = TEST_MAC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_lsi_mac_lb_test_port1(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;
	lsi_gmac_conf4_t conf;
	uint32 port,unit;

	unit = port = 1;

	mdk_lsi_checkSync(port);

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	delay_us(10000);
	conf.bits.loopback = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
	delay_us(10000);

	testMode = TEST_MAC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_lsi_mac_lb_test_port2(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;
	lsi_gmac_conf4_t conf;
	uint32 port,unit;

	unit = port = 2;

	mdk_lsi_checkSync(port);

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	delay_us(10000);
	conf.bits.loopback = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
	delay_us(10000);

	testMode = TEST_MAC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_lsi_mac_lb_test_port3(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;
	lsi_gmac_conf4_t conf;
	uint32 port,unit;

	unit = port = 3;

	mdk_lsi_checkSync(port);

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	delay_us(10000);
	conf.bits.loopback = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
	delay_us(10000);

	testMode = TEST_MAC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_lsi_mac_lb_test_port4(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;
	lsi_gmac_conf4_t conf;
	uint32 port,unit;

	if(isTXRX())
		return(tp->testresult = RET_SKIP_TEST);	

	unit = port = 4;

	mdk_lsi_checkSync(port);

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	delay_us(10000);
	conf.bits.loopback = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
	delay_us(10000);

	testMode = TEST_MAC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_lsi_mac_lb_test_port5(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;
	lsi_gmac_conf4_t conf;
	uint32 port,unit;

	if(isTXRX())
		return(tp->testresult = RET_SKIP_TEST);	

	unit = port = 5;

	mdk_lsi_checkSync(port);

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	delay_us(10000);
	conf.bits.loopback = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
	delay_us(10000);

	testMode = TEST_MAC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_reset_lsi_mac_lpbk(
	mdk_test_params_t	* tp
)
{
	lsi_gmac_conf4_t conf;
	int num_devices = 0;
	uint32 i=0;

	if (isTXRX())
		num_devices = LSI_NUM_DEVICES_TXRX;
	else if (isFP())
		num_devices = LSI_NUM_DEVICES_FP;

	if (num_devices > 0) {
		for(i=0;i<num_devices;i++) {
			conf.wd = lsi_gmac_read(i, LSI_GMAC_CONFIG4);
			delay_us(10000);
			conf.bits.loopback = 0;
			lsi_gmac_write(i, LSI_GMAC_CONFIG4, conf.wd);
			delay_us(10000);
		}	
	}
	return(tp->testresult = RET_PASSED);
}

void
mdk_lsi_checkSync(uint32 port)
{
	lsi_gmac_status_t status;
	lsi_gmac_conf4_t conf;
	uint32 	cnt = 0;
	
	status.wd = lsi_gmac_read(port, LSI_GMAC_STATUS);
	delay_us(10000);
	while (!status.bits.rxsync) {
		lsi_gmac_init(port);
		delay_us(30000);
		status.wd = lsi_gmac_read(port, LSI_GMAC_STATUS);
		delay_us(10000);
		if(cnt > 5)
			break;	
		cnt++;
	}	

	return;
}		

int32
lsi8104_pkg_err(
	mdk_log_entry_t	* err
)
{

	uint32 i;
	uint32 num1 = 0;
	uint32 num2 = 0;

	switch (err->testresult) {
		case ERR_DATA_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("LUC Register Test Failure : \n");
				printf("-------------------------------\n");
			        printf("addr	expected	got \n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1 - 1);i++) {
				printf("0x%x	 0x%x		0x%x\n",
					err->result[0].addr[i], err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}
			break;	
		case ERR_TIMEOUT:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("Loopback test timeout\n");
			}	
				break;
		case ERR_UNEXPECTED_RESULT:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nResult not within expected limits.\n");
			break;
		case ERR_PKTDROP:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nSome pkts have been dropped.\n");
			break;
		case ERR_LPBKDATA:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nData corruption.\n");
			break;
		default:
				printf("Unknown error, testresult = 0x%x\n", err->testresult);
				break;
	}

	return 0;
}
#endif
