/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-pci.c,v 1.4 2002/07/11 23:27:54 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"
#if defined (DIAG_R7000) || defined (DIAG_SIBYTE)
#include "mdk-dp83820.h"
#endif

#ifdef DIAG_SIBYTE
reginfo_t txrx[] = {
	{"r",0x00, 0x0001166D,0x00000000,0},
	{"w",0x04, 0x02a00156,0x002a0156,0},
	{"r",0x08, 0x06000001,0x00000000,0},
	{"r",0x00, 0x00002300,0x00000000,0}, 
	{NULL,NULL,NULL,NULL,NULL}
	};

reginfo_t fp[] = {
	{"r",0x00, 0x0002166D,0x00000000,0},
	{"r",0x08, 0x06000001,0x00000000,0},
	{"r",0x00, 0x00002300,0x00000000,0}, 
	{NULL,NULL,NULL,NULL,NULL}
	};

#define TXRX_REG_MAX	3
#define FP_REG_MAX	2

#endif

reginfo_t dp[] = {
	{"r",0x00, 0x0022100B,0x00000000,0},
	{"r",0x08, 0x02000000,0x00000000,0},
	{NULL,NULL,NULL,NULL,NULL}
	};

#define DP_REG_MAX	2


#ifdef DIAG_R7000
reginfo_t qlogic[] = {
	{"r",0x00, 0x00001077,0x00000000,0}, 
	{"r",0x02, 0x00002300,0x00000000,0}, 
	{NULL,NULL,NULL,NULL,NULL}
};	


reginfo_t bcm5600[] = {
	{"r",0x00, 0x560014e4,0x00000000,0}, /* Vendor ID: Check the expected values */
	{"r",0x04, 0x02a00002,0x00000000,0}, /* Command/Status */ 
	{NULL,NULL,NULL,NULL,NULL}
};

#define BC5600_REG_MAX  2
#define QLOGIC_REG_MAX	0

#define BCM5600_PCI_BASE_ADDR	0x70000000
#define BCM5600_BUS_ID			8
#define BCM5600_DEV_ID			8
#endif



int32
pci_pkg_init()
{
	tp->resultCnt = 1;

}

#ifdef DIAG_SIBYTE
int32
mpci_txrx(
	mdk_test_params_t       * tp
)
{
	
	reginfo_t *regp;
	uint32 rc;
	
	if(isFP())
		return(tp->testresult = RET_SKIP_FP);

	if(isTXRX()) {
		if (tp->param[TP_VERBOSE] == 1) {
		printf("\nTXRX SIBYTE:  PCI Read/Write\n");
		printf("=========================================================\n");
		printf("Register	Result	Expected	Received\n");
		}
		
		regp =  txrx;
		rc =  pci_reg_test(regp, TXRX_REG_MAX, 0, 0);
	}	
	return(rc);
}

int32
mpci_fp(
	mdk_test_params_t       * tp
)
{
	reginfo_t *regp;
	uint32 rc;
	
	if(isTXRX())
		return(tp->testresult =  RET_SKIP_FP);

	if(isFP()) {
		if (tp->param[TP_VERBOSE] == 1) {
		printf("\nFP SIBYTE:  PCI Read/Write\n");
		printf("=========================================================\n");
		printf("Register	Result	Expected	Received\n");
		}
		regp =  fp;
		rc = pci_reg_test(regp, FP_REG_MAX, 0, 1);
	}
	return (rc);
}
#endif


int32
mpci_gmac(
	mdk_test_params_t       * tp
)
{
	reginfo_t *regp;
	uint32 rc;
	
	if(isFP())
		return(tp->testresult = RET_SKIP_FP);

	if (tp->param[TP_VERBOSE] == 1) {
	printf("\nDP83820:  PCI Read/Write\n");
	printf("=================================\n");
	printf(" Register	Result	Expected	Received\n");
	}
	regp =  dp;
	rc = pci_reg_test(regp, DP_REG_MAX, DP83820_BUSID, DP83820_DEVID);

	return(rc);
}

int32
mpci_qlogic(
	mdk_test_params_t       * tp
)
{
	uint32 rc;
#if defined (DIAG_R7000)
	reginfo_t *regp;
	
	if (tp->param[TP_VERBOSE] == 1)
		//printf("Not supported");	

	return (tp->testresult = RET_SKIP_TEST);

	if (tp->param[TP_VERBOSE] == 1) {
	printf("\nQlogic ISP2310:  PCI Read/Write\n");
	printf("=================================\n");
	printf(" Register	Result	Expected	Received\n");
	}

	regp =  qlogic;
	rc = pci_reg_test(regp, QLOGIC_REG_MAX, 0, 4);
	return(rc);
#endif	

}


int32
mpci_bc5600(
	mdk_test_params_t       * tp
)
{
#if defined (DIAG_R7000)
	reginfo_t *regp;
	uint32 rc;
	
	if((isFC()) && (nfxMyCPUId))
		return(tp->testresult = RET_SKIP_FP);

	if (tp->param[TP_VERBOSE] == 1) {
	printf("\nBroadcom 5600:  PCI Read/Write\n");
	printf("=================================\n");
	printf(" Register	Result	Expected	Received\n");
	}

	regp =  bcm5600;
	rc = pci_reg_test(regp, BC5600_REG_MAX, BCM5600_BUS_ID, BCM5600_DEV_ID);
	return(rc) ;
#endif

}

int32
pci_reg_test(reginfo_t *pciReg, uint32 reg_max,uint32 busId, uint32 devId)
{
	uint32 i, j=0;
	uint32 sdata,rdata,data;
	uchar8 * test;
		
	test = "r";

	for(i=0;i<reg_max;i++){	
		if(!(strcmp((pciReg[i].rw),test))) {
			pci_cfg_read(busId,devId,0,pciReg[i].reg,&rdata);
			if(rdata == pciReg[i].expected)
				pciReg[i].res = 1;
			else {	
				pciReg[i].res = 0;	
				tp->result[0].addr[j] = pciReg[i].reg;
				tp->result[0].exptd[j] = pciReg[i].expected ;
				tp->result[0].rcvd[j] = rdata;	
				tp->testresult = ERR_DATA_FAIL;
				j++;
			}
		}
		else {
			pci_cfg_read(busId,devId,0,pciReg[i].reg,&sdata);
			data = sdata;
			sdata |= pciReg[i].wbits;
			pci_cfg_write(busId,devId,0,pciReg[i].reg,sdata);
			pci_cfg_read(busId,devId,0,pciReg[i].reg,&rdata);
			if(rdata == pciReg[i].expected)
				pciReg[i].res = 1;
			else	
			{
				pciReg[i].res = 0;	
				tp->result[0].addr[j] = pciReg[i].reg;
				tp->result[0].exptd[j] = pciReg[i].expected ;
				tp->result[0].rcvd[j] = rdata;	
				tp->testresult = ERR_DATA_FAIL;
				j++;
			}	
			/* store original back */
			pci_cfg_write(busId,devId,0,pciReg[i].reg,data);
		}
		if (tp->param[TP_VERBOSE] == 1) {
			printf("%08x	%s	%08x	%08x\n",pciReg[i].reg, 
				(pciReg[i].res == 1)? "PASS":"FAIL" , 
					pciReg[i].expected,rdata);}
	}
	if(tp->testresult != ERR_DATA_FAIL)
		tp->testresult = RET_PASSED;

	return(tp->testresult);
}			

int32
pci_pkg_err(
	mdk_log_entry_t	* err
)
{
	uint32 num1 = 0;
	uint32 i;
	switch (err->testresult) {
		case ERR_DATA_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("PCI Register Test Failure : \n");
				printf("-------------------------------\n");
			        printf("addr\t expected\t got \n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1);i++) {
				printf("0x%x\t 0x%x\t 0x%x\n",
					err->result[0].addr[i], err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}
				break;
		case ERR_ADDR_FAIL:
				break;
		default:
				printf("Unknown error, testresult = 0x%x\n", err->testresult);
				break;
	}

	return 0;
}
