/* vi:ts=4
 *-----------------------------------------------------------------
 * 
 * Name:        mnt-decode.c
 *
 * RCS:  $Id: mnt-decode.c,v 1.11 2002/08/10 03:00:51 agile Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *  Contain NFS decode functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        5/28/01 
 *
 *-----------------------------------------------------------------
 */
#ifdef LINUX_TEST
#include <stdio.h>              /*REMOVE*/
#endif	//LINUX_TEST
#include <nfx-incl.h>
#include <exports/nfs-api.h>

#define NFS_DECODE_FILENAME     1
#define NFS_DECODE_PATHNAME     2
#define NFS_DECODE_EXPORTINFO   3


#define NFS_DECODE_DIRCOUNT 1
#define NFS_DECODE_MAXCOUNT 2

static char *mnt_procs[] =	 {         
	"\tProc 0 = (NULL)\n",
	"\tProc 1 = (Add MOUNT Entry)\n",
	"\tProc 2 = (Return MOUNT Entries)\n",
	"\tProc 3 = (Remove MOUNT Entry)\n",
	"\tProc 4 = (Remove All MOUNT Entries)\n",
	"\tProc 5 = (Return Export List)\n",
	"\tProc 6 = (Return Export List)\n",
	"\tProc 7 = (POSIX Pathconf Information)\n",
};

void
mnt_rxDecodeAddMnt(rpcCallHdr_t *rpcCallp, char *msgp)
{
    int32       length;

    /*
     *Decode pathname
     */
    length = nfs_decodeName(msgp, 
				NFS_DECODE_PATHNAME, NFS_DECODE_RX);



}

int32
mnt_rxDecodePkt (pkt_desc_t *descp)
{
	uint32			rpcHdrSize;	
	rpcCallHdr_t	*rpcCallp;
	char			*msgp;

	if (descp == NULL)
	{
		printf("\tMNT Rx: Bad RPC header/pkt!\n");
		return (NFX_ERR);
	}
	rpcCallp = (rpcCallHdr_t *)descp->pHdr;
	if (rpcCallp == NULL)
	{
		printf("\tMNT Rx: Bad RPC header/pkt!\n");
		return (NFX_ERR);
	}	
	if (mnt_tplRcvCheckPkt(rpcCallp) != NFX_OK)
	{
		printf("\tMNT Rx: Bad RPC header/pkt!\n");
		return (NFX_ERR);
	}

	printf("\n\t----- RPC Header -----\n");
	//printf("\tXID: %d \n", rpcCallp->rpc_xid);
	printf("\tXID: 0x%08x\n", rpcCallp->rpc_xid);
	printf("\tDirection: %s\n", "Call");
	printf("\tRPC Version: %d \n", rpcCallp->rpc_rpcVer);
	printf("\tProgram number: %d (%s)\n",
			rpcCallp->rpc_prog, nfs_decodeGetProgNumStr(rpcCallp->rpc_prog ));
	printf("\tProgram Version: %s\n", (rpcCallp->progVer == MNT_VER1)? "V1" : 
						(rpcCallp->progVer == MNT_VER2)? "V2": 
						(rpcCallp->progVer == MNT_VER3)? "V3": "Invalid");
	printf("\tProcedure number: %d \n", rpcCallp->rpc_proc);

	nfs_rxDecodeAuthSysHostName(rpcCallp);

	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;

	printf("%s",mnt_procs[rpcCallp->rpc_proc]);
	printf("\t----- MOUNT packet -----\n");
	switch (rpcCallp->rpc_proc)
	{
		case MNT_PROC_NULL:

			break;
		case MNT_PROC_MNT:
			mnt_rxDecodeAddMnt(rpcCallp, msgp);
			break;
		case MNT_PROC_DUMP:

			break;
		case MNT_PROC_UMNT:

			break;
		case MNT_PROC_UMNTALL:

			break;

		case MNT_PROC_EXPORT:

			break;
		case MNT_PROC_EXPORTALL:

			break;
		case MNT_PROC_PATHCONF:

			break;

	}
	return (NFX_OK);
}

void
mnt_xmtDecodeAddMount(
		mnt_request_t 	*mntReqp,		/*request control block*/
		char 			*msgp			/*Reply message pointer after RCP reply hdr*/
)
{
	char                    *destMsgp;
	int32                   length;
	uint32					version;

	version = mntReqp->rpcCallHdrp->progVer;
	destMsgp = msgp;
	length = 0;

	/*
	 *Decode object filehandle
	 */
	length = nfs_decodeFhandle(destMsgp, version);
	destMsgp += length;

	/*
	 *TODO: Which Authentication Flavors do we support?
	 */
	length = nfs_decodeAuthFlavor (destMsgp);

}

void
mnt_xmtDecodeShowMount(
		mnt_request_t 	*mntReqp,		/*request control block*/
		char 			*msgp			/*Reply message pointer after RCP reply hdr*/
)
{
	char                    *destMsgp;
	int32                   length;
//	uint32                  version;
	uint32                  presentFlag;

	destMsgp = msgp;
	length = 0;

	length = nfs_decodePresentFlag (destMsgp, &presentFlag);
	destMsgp += length;

	while (presentFlag)
	{
		/*
		 *decode hostname string
		 */
		length = nfs_decodeName(destMsgp, 
						NFS_DECODE_HOSTNAME, NFS_DECODE_XMT);
		if (length == NFX_ERR)
		{
			printf("%s: Failure to decode filename!\n",
            __FUNCTION__);
			return (NFX_ERR);
		}

		destMsgp += length;

		/*
		 *decode directory pathname
		 */
		length = nfs_decodeName(destMsgp, 
						NFS_DECODE_PATHNAME, NFS_DECODE_XMT);
		if (length == NFX_ERR)
		{
			printf("%s: Failure to decode filename!\n",
            __FUNCTION__);
			return (NFX_ERR);
		}
		destMsgp += length;


		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		destMsgp += length;

	}

}
void
mnt_xmtDecodeExportList(
		mnt_request_t *mntReqp,		/*request control block*/
		char *msgp					/*Reply message pointer after RCP reply hdr*/
)
{
	char                    *destMsgp;
	int32                   length;
//	uint32                  nfs_version;
	uint32                  presentFlag;

	destMsgp = msgp;
	length = 0;

	length = nfs_decodePresentFlag (destMsgp, &presentFlag);
	destMsgp += length;

	while (presentFlag)
	{
		/*
		 *Decode pathname
		 */
		length = nfs_decodeName(destMsgp, 
					NFS_DECODE_PATHNAME, NFS_DECODE_XMT);
		if (length == NFX_ERR)
		{
        	printf("%s: Failure to decode filename!\n",
            __FUNCTION__);
        	return (NFX_ERR);
		}

		destMsgp += length;

		/*
		 *Decode Export Info
		 */
		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		destMsgp += length;
		if (presentFlag)
		{
			length = nfs_decodeName(destMsgp,
                    NFS_DECODE_EXPORTINFO, NFS_DECODE_XMT);
			if (length == NFX_ERR)
			{
        		printf("%s: Failure to decode filename!\n",
            			__FUNCTION__);
        		return (NFX_ERR);
			}

			destMsgp += length;
		}

	}
}

void
mnt_xmtDecodePkt(mnt_request_t *mntReqp)
{
	pkt_desc_t              *tplXmitPktDescp;
	rpcReplyHdr_t   		*rpcReplyHdrp;
	char					*msgp;
	uint32					replyStatus;


	printf("\t----- RPC Header -----\n");

	tplXmitPktDescp = (pkt_desc_t *)mntReqp->pktDescp;
	msgp = (char *)tplXmitPktDescp->pHdr;
	rpcReplyHdrp = (rpcReplyHdr_t *)tplXmitPktDescp->pHdr;
	printf("\tXID: 0x%08x \n", rpcReplyHdrp->rpc_xid);
	printf("\tDirection: %s\n", (rpcReplyHdrp->rpc_direction == RPC_CALL)? "Call" : "Reply");

	
	if ((mntReqp->rpcCallHdrp->rpc_proc <MNT_MIN_PROC) || 
				(mntReqp->rpcCallHdrp->rpc_proc > MNT_MAX_PROC) )
		return;

	printf("%s",mnt_procs[mntReqp->rpcCallHdrp->rpc_proc]);

	msgp += sizeof(rpcReplyHdr_t);

	switch (mntReqp->rpcCallHdrp->rpc_proc)
	{
		case MNT_PROC_NULL:


			break;

		case MNT_PROC_MNT:
			/*
	 		 *Decode Status
	 		 */
			replyStatus = *(uint32 *)msgp;
			printf("\tStatus: %d - %s\n", replyStatus, nfs_getStatus(replyStatus));
			msgp += 4;	
						
			mnt_xmtDecodeAddMount(mntReqp, msgp);
			break;

		case MNT_PROC_DUMP:
			mnt_xmtDecodeShowMount(mntReqp, msgp);
			break;

		case MNT_PROC_UMNT:


			break;

		case MNT_PROC_UMNTALL:


			break;

		case MNT_PROC_EXPORT:
			/*Return list of export files systems and their
		 	 *export information
			 */
			mnt_xmtDecodeExportList(mntReqp, msgp);
			break;

		case MNT_PROC_EXPORTALL:

			break;

		case MNT_PROC_PATHCONF:

			break;

	}




}
