/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-main.c
 *
 * RCS:  $Id: ndmp-main.c,v 1.56 2002/12/04 22:12:53 nitinj Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */

#include <stdlib.h>
#include <sys/time.h>
#include <sys/select.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <errno.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/uio.h>

#include <nfx-types.h>
#include <exports/eee-api.h>
#include <exports/anpssc-api.h>
#include <exports/elog-api.h>
#include <exports/scsi-api.h>
#include <nfx-incl.h>
#include <exports/event-api.h>
#include "ndmp.h"
#include "ndmp-ssc.h"
#include "ndmp-debug.h"
#include "ndmp-fs-api.h"

#ifdef SSC_mips
#include <neteee/eee.h>
#endif

int		agileFd;	/* val returned when we register w/ regAgileMsg */
int		agileMsgBufSiz;
int		ndmp_tot_sessions;
int		ndmp_enabled = 1;
int		nxt_sessId;
int		ndmp_hi_ver = NDMP_VER_3;		//highest NDMP version we support, default = 3
int 	ndmp_ssc_par_msgStats[MAX_SSC_PAR_STAT + MAX_SSC_CHLD_STAT];
static pid_t    pid;

extern ndmpAgileMsgEnt_t ndmpAgileMsgTbl;

extern ndmpMsgEnt_t ndmpMsgTbl[];
extern ndmpMsgEnt_t ndmpv2_MsgTbl[];

int ndmp_procSession (ndmpSess_t *sess);
int zip_remove_prev_files(struct tm *tm_todfil);
int ndmp_procAgileRcvMsg(ndmpSess_t *sess, char8 *rcvbuf, int in_len, ndmp_src_t *src);
int wr_xdr_rec_stream(char  *cl_Handle, char *buf, int len);
int rd_xdr_rec_stream(char *cl_Handle, char *buf, int maxlen);
void free_xdr_rec_stream(ndmpSess_t  *sess,  ndmpMsgEnt_t *msgent);
void ndmp_cpuEvent(nfx_eventMsg_t *evt);
int ndmp_ParentProcUIMsg(void);
int ndmp_sendNotifyConn (ndmpSess_t *sess);
void ndmp_InitMain(int *serverFd);
int my_system(char *command);
#ifdef NFX_SMP
void ndmp_sendCloseSessToNNIM(int32 sessId, ndmpSess_t * sess, int32 reply_required);
void FinishClosing(ndmpSess_t *sess);
#else
void ndmp_sendCloseSessToNNIM(int32 sessId);
#endif
int add_dates_to_lst ( struct tm *tm_cfil, int *i);

static void ndmp_storeLogFromTmp(int idx);

sessId_entry_t 	sessId_Tbl[NDMP_MAX_SESS];
tfil_t *filhd = NULL;

//Define below allows 1 session vs. multiple sessions and allows GDB
//forking and GDB gets funny.
//#define NDMP_GDB 1


/*-----------------------------------------------------------------
 * Name:  ndmp_CreateSess
 *
 * Description:	Creates a new NDMP session
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
ndmpSess_t *
ndmp_CreateSess(int	clientFd, int32 idx, int32 sessId, struct sockaddr_in *from)
{
	ndmpSess_t			*sess;
    char                logFile[1024];

#ifndef SSC_mips 
	struct protoent 	*prot;
#else 
	int					sock;
    struct sockaddr_eee seee;
    socklen_t len;
#endif

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_CreateSess\n"));

    NDMP_SSC_ALLOC(sess, ndmpSess_t, sizeof(ndmpSess_t));
	if (sess == 0)
	{
		NDMP_DBG_PRINTF(D_INFO, ("system call (malloc): %s.", strerror(errno)));
		E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
				("system call (malloc): %s.", strerror(errno)));
		return(0);
	}
	bzero((void *) sess, sizeof(ndmpSess_t));
	sess->clientFd      = clientFd;
	memcpy(&sess->host_ipaddr, from, sizeof(struct sockaddr_in));
	sess->sess_idx     	= idx;			/* sess Id tbl index */
	sess->sessId        = sessId;
	sess->xmtSeq	    = 0;
	sess->authed    	= FALSE;
	sess->conn_refused  = FALSE;		/* Whether we refuse client right away */
	sess->conn_closed  	= FALSE;
	memset(&(sess->msgRcvd), 0, sizeof(CtrlMsg_t));
	sess->protoVer      = -1;	/* not negotiated yet */
    sess->partial_bkup  = FALSE;
    sess->logFp         = NULL;

    if (ndmp_trace_msg)
    {
        bzero(logFile, sizeof(char)*1024);
        sprintf(logFile, "/tmp/ndmp_msg_%s_%d.log",
                            inet_ntoa(sess->host_ipaddr.sin_addr), sessId);
        sess->logFp = fopen(logFile, "w");
        if (sess->logFp == NULL)
        {
            E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
                  ("Unable to open NDMP log file. Cannot store messages.\n"));
        }
    }

    /* Create socket for passing messages */

#ifndef SSC_mips

    setprotoent(0);
    prot = getprotobyname ("udp");
    endprotoent();
    if (!prot) 
	{
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_CreateSess: getprotobyname failed"));
        NDMP_SSC_FREE(sess);
        return(NULL);
    }
    sess->Agilesock = socket(PF_INET, SOCK_DGRAM, prot->p_proto);

#else

    seee.seee_family = AF_EEE;
    seee.seee_bplane = 0;
    seee.seee_slot = SSC_SLOT;
    seee.seee_cpu = NFX_SSC_UNIX;
    seee.seee_app = 0;
    if ((sock = socket(PF_EEE, SOCK_DGRAM, 0)) >= 0) {
        if (bind(sock, (struct sockaddr *)&seee, sizeof(seee)) < 0) {
            close(sock);
            sock = -1;
        }
		setSocketOpt(sock);
    }
    sess->Agilesock = sock;
    len = sizeof(seee);
    getsockname(sock, (struct sockaddr *)&seee, &len);
    NDMP_DBG_PRINTF(D_INFO, ("seee.seee_app : %d\n", seee.seee_app));
    NDMP_DBG_PRINTF(D_INFO, ("Session [%d] : sess->Agilesock [%d]\n", 
				sess->sess_idx, sess->Agilesock));
    NDMP_DBG_PRINTF(D_INFO, ("App Name : %s\n", eee_getAppNameFromId(seee.seee_app)));

#endif /* SSC_mips */

    if (sess->Agilesock < 0) 
	{
        printf("ndmp_CreateSess: socket failed");
        NDMP_SSC_FREE(sess);
        return(NULL);
    }

	/* Send an agile msg to parent to report myself
     * so parent can fwd UI msgs to me later on.
	 */
	ndmp_ChildRpt2Parent(sess);

	/* Initialize a record-oriented XDR *xdr_cmd stream object 
	 * /usr/include/rpc/xdr.h
	 * XDR pseudo records for TCP
	 */
	xdrrec_create(&sess->xdr_cmd, 
					0, 0, 					// sendsz & recvsz buffer size.  0 = use default bufsiz
					(char *)sess, 					// client handle
					rd_xdr_rec_stream, 
					wr_xdr_rec_stream);			// called when stream's input/output is full/empty
	

	return((ndmpSess_t *)sess);
} /* ndmp_CreateSess */


/*-----------------------------------------------------------------
 * Name:  ndmp_sendCloseSessToNNIM
 *
 * Description:	Send CLOSE_SESSION request to FP-NIM
 * 	
 * Created by:  Nitin Jain
 *
 * Date Created: 10/15/02
 *
 *-----------------------------------------------------------------
 */
#ifdef NFX_SMP
void
ndmp_sendCloseSessToNNIM(int32 sessId, ndmpSess_t * sess, int32 reply_required)
#else
void
ndmp_sendCloseSessToNNIM(int32 sessId)
#endif
{
	ndmp_NFXReq_t       ndmp_req;

	int32				sock;

    /* Send message to NNIM to remove Mover and Data 
     * state machines there 
     */  
    memset((void*)&ndmp_req, 0, sizeof(ndmp_req));
    ndmp_req.hdr.mid        = NDMP_MID_CLOSE_SESSION;
    ndmp_req.hdr.sessId     = sessId;
#ifdef NFX_SMP
	ndmp_req.userInfo		= reply_required? 1: 0;
	ndmp_req.hdr.seqId     = sess->xmt2fp_seqId;
    sess->xmt2fp_seqId ++;
    ndmp_req.hdr.idx        = sess->sess_idx;
#else
	sock = -1;
#endif
    if (sessId == NDMP_SESSID_ALL)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendCloseSessToNNIM:  send msg to NNIM to CLOSE SESSION for sessId All\n"));
#ifdef NFX_SMP
		sock = -1;
#endif
    }
    else
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendCloseSessToNNIM:  send msg to NNIM to CLOSE SESSION for sessId %d.\n", sessId));
#ifdef NFX_SMP
		sock = sess->Agilesock;
#endif
    }

#ifdef NFX_SMP
	if (sendAgileMsg (sock, &ndmp_req, sizeof(ndmp_req),
#else
    if (sendAgileMsg (-1, &ndmp_req, sizeof(ndmp_req),
#endif
                      "ndmp",       /* NDMP on the N-NIM */
                      NFX_NFP_FP2, 1, MGMTPLANE, /* cpu, slot, plane */
                      NULL, NULL, 0) < 0)
    {
        if (sessId != NDMP_SESSID_ALL) 
        {
            NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendCloseSessToNNIM:  Agile message send to NNIM to CLOSE SESSION failed, sessId %d\n", sessId));
        }
        else
        {
            NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendCloseSessToNNIM:  Agile message send to NNIM to CLOSE SESSION failed, sessId All\n"));
        }
    }

    return;
}

#ifdef NFX_SMP
/*-----------------------------------------------------------------
 * Name:  FinishClosing
 *
 * Description:	Close the NDMP session
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
void
FinishClosing(ndmpSess_t   *sess)
{

	if (sess->clientFd >= 0)
	{
		/* Do only if we close connection due to reasons 
		 * other than because we are refusing connection
	 	 * to client.
		 */
		if (!sess->conn_refused)
		{
			/* Close tape or SCSI device if either is opened */
			ndmp_closeTapeDev(sess);
			ndmp_closeScsiDev(sess);

			/* Dealloc resources */
			/* resource allocated for CONFIG_FS_INFO req. */
			if (sess->fsInfoLst)				
                NDMP_SSC_FREE(sess->fsInfoLst);
			if (sess->vol_lst)
                NDMP_SSC_FREE(sess->vol_lst);

			free_env_on_SSC(&sess->Serv.Data);
		}

		/* Send an agile msg to parent to report my stats
     	 * so parent can accumulate & show to user if need.
	 	 */
		ndmp_ChildSndStats2Parent(sess);

        /* Close log file */
        if (sess->logFp != NULL)
        {
            fclose(sess->logFp);
            sess->logFp = NULL;
        }

		/* Close client's socket */
		(void)close(sess->clientFd);
		sess->clientFd = -1;

		/* Close our internal Agile's socket */
		(void)close(sess->Agilesock);
		sess->Agilesock = -1;

		xdr_destroy(&sess->xdr_cmd);

        if (sess->saveList)
            NDMP_SSC_FREE(sess->saveList);

       sess->saveList = sess->nextblobPtr = NULL;
	}
	if (!sess->conn_closed)	
	{
		sess->conn_closed = TRUE;
	}

	return;
}

/*-----------------------------------------------------------------
 * Name:  ndmp_CloseSess
 *
 * Description:	Close the NDMP session
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
void
ndmp_CloseSess(ndmpSess_t	*sess)
{
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_CloseSess\n"));

	if ((sess->clientFd >= 0) && (!sess->conn_refused) &&
			(sess->authed))
	{
		ndmp_sendCloseSessToNNIM(sess->sessId, sess, 1);
		return;
	}
	FinishClosing(sess);

} /* ndmp_CloseSess */

int32 ndmp_procCloseSessRsp(ndmpSess_t   *sess)
{
	printf("ndmp_procCloseSessRsp:  sessid %d \n", sess->sessId);

	FinishClosing(sess);
	return NFX_OK;
}
#else
/*-----------------------------------------------------------------
 * Name:  ndmp_CloseSess
 *
 * Description:	Close the NDMP session
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
void
ndmp_CloseSess(ndmpSess_t	*sess)
{
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_CloseSess\n"));

	if (sess->clientFd >= 0)
	{
		/* Do only if we close connection due to reasons 
		 * other than because we are refusing connection
	 	 * to client.
		 */
		if (!sess->conn_refused)
		{
			/* Close tape or SCSI device if either is opened */
			ndmp_closeTapeDev(sess);
			ndmp_closeScsiDev(sess);

			/* Dealloc resources */
			/* resource allocated for CONFIG_FS_INFO req. */
			if (sess->fsInfoLst)				
                NDMP_SSC_FREE(sess->fsInfoLst);
			if (sess->vol_lst)
                NDMP_SSC_FREE(sess->vol_lst);

			free_env_on_SSC(&sess->Serv.Data);

			if (sess->authed)
                ndmp_sendCloseSessToNNIM(sess->sessId);
		}
		/* Send an agile msg to parent to report my stats
     	 * so parent can accumulate & show to user if need.
	 	 */
		ndmp_ChildSndStats2Parent(sess);

        /* Close log file */
        if (sess->logFp != NULL)
        {
            fclose(sess->logFp);
            sess->logFp = NULL;
        }

		/* Close client's socket */
		(void)close(sess->clientFd);
		sess->clientFd = -1;

		/* Close our internal Agile's socket */
		(void)close(sess->Agilesock);
		sess->Agilesock = -1;

		xdr_destroy(&sess->xdr_cmd);

        if (sess->saveList)
            NDMP_SSC_FREE(sess->saveList);

       sess->saveList = sess->nextblobPtr = NULL;
	}

	if (!sess->conn_closed)	
	{
		sess->conn_closed = TRUE;
	}
} /* ndmp_CloseSess */
#endif
/*-----------------------------------------------------------------
 * Name:  		my_system
 *
 * Description:	Used to issue a system call.  Similar to
 *				the 'system' call but that one did not
 *				allow process w/ suid so man page suggest
 *				us do it the long way.
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        10/09/02
 *
 *-----------------------------------------------------------------
 */
int my_system(char *command)
{
    int pid, status;
    extern char **environ;

    if (command == 0)
        return 1;
    pid = fork();
    if (pid == -1)
        return -1;
    if (pid == 0) {
        char *argv[4];
        char *shell = getenv("SHELL");
        if (shell == NULL)
            shell = "/bin/sh";
        argv[0] = "sh";
        argv[1] = "-c";
        argv[2] = command;
        argv[3] = 0;
        execve(shell, argv, environ);
        exit(127);
    }
    do {
        if (waitpid(pid, &status, 0) == -1) {
            if (errno != EINTR)
                return -1;
        } else
            return status;
    } while(1);
}
/*-----------------------------------------------------------------
 * Name:  		add_dates_to_lst
 *
 * Description:	Add all dates of ndmp trace files in the /var/agile dir
 *				to our list in chronological order.  We'll use this list 
*				to compare later to decide which trace files to keep.
 *
 * Created by:  Anh Vu
 *
 * Date Created:        10/11/02
 *
 *-----------------------------------------------------------------
 */
int
add_dates_to_lst ( struct tm *tm_cfil, int *i )
{
	tfil_t *ent, *cur, *prev;
	int yr, mon, day, insert;


   	NDMP_SSC_ALLOC(ent, tfil_t, sizeof(tfil_t));
	bzero(ent, sizeof(tfil_t));
	memcpy(&ent->time, tm_cfil, sizeof(struct tm));

	yr = ent->time.tm_year;
	mon = ent->time.tm_mon;
	day = ent->time.tm_mday;
	NDMP_DBG_PRINTF(D_INFO, ("adding:  date:  %02d-%02d-%02d\n", 
					ent->time.tm_mon, ent->time.tm_mday, ent->time.tm_year));

	if (filhd == NULL)
	{
		filhd = ent;
		insert = TRUE;
	} else {
		for (prev = NULL, cur = filhd; cur != NULL; cur = cur->nxt)
		{
			/* If equal, go to next */
			if ((yr == cur->time.tm_year) && (mon == cur->time.tm_mon) 
							&& (day == cur->time.tm_mday))
			{
				break;
			/* If less than, insert in front */
			} else if ((yr < cur->time.tm_year) || (mon < cur->time.tm_mon) || (day < cur->time.tm_mday))
			{
				ent->nxt = cur;
				if (prev == NULL)
					filhd = ent;
				else 
					prev->nxt = ent;
				insert = TRUE;
				break;
			/* If cur is the last, insert after cur */
			} else if (cur->nxt == NULL)
			{
				cur->nxt = ent;
				insert = TRUE;
				break;
			/* If > cur and less than the next, insert in middle */
			} else if ((yr < cur->nxt->time.tm_year) || 
					(mon < cur->nxt->time.tm_mon) || (day < cur->nxt->time.tm_mday))
			{
				ent->nxt = cur->nxt;
				cur->nxt = ent;
				insert = TRUE;
				break;
			/* Try next node */
			} else
				prev = cur;
		}
	}
	if (insert == TRUE)
		(*i)++;
	else 
		NDMP_SSC_FREE(ent);

	for (cur=filhd; (cur !=NULL) ;cur=cur->nxt)
	{
	     NDMP_DBG_PRINTF(D_INFO, ("list:  date:  %02d-%02d-%02d\n", 
					 cur->time.tm_mon, cur->time.tm_mday, cur->time.tm_year));
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:  		zip_remove_prev_files
 *
 * Description:	Zip all unzipped ndmp_msg log files in /var/agile whose
 *  			date is not today's since the today's one is continuously
 *  			being updated. 
 * 				Files that we want to zip are log files w/ name 
 * 				such as "ndmp_msg_10.2.19.10_10052002.log"
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int 
zip_remove_prev_files(struct tm *tm)
{
	DIR *dirptr;
	struct dirent *dir_ent;
	struct stat status;
	struct tm *tm_cfil;  //current file 
	struct tm tm_todfil;
	char *ptr;
	//pid_t pid;
	char command[200]; //yrstr[5], monstr[3], daystr[3];
	tfil_t *cur, *tmp;
	int i;		//total num of ndmp trace log file dates in this dir
	int yr, mon, day, found;
	char buf[1024];

	if ((dirptr = opendir(ndmpLogDir)) == NULL)
	{
       	E_LOG (class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
       			("zip_remove_prev_files: Unable to open dir %s, rcv error %s\n", 
				 ndmpLogDir, strerror(errno)));
		return NFX_ERR;
	}	
	memcpy(&tm_todfil, tm, sizeof(struct tm));
	i = 0;
	filhd = NULL;
	while ((dir_ent = readdir(dirptr)) != NULL)
	{
		/* Find only files that start w/ "ndmp_msg_" and almost end w/ ".log" */
		if ( strncmp(dir_ent->d_name, "ndmp_msg_", 9) == NULL)
		{
		   /* Is the file zipped already?  Yes, ignore.  No, then if 
			* file is not today's, zip it.   First find file ending with 
		    * '.log'
			*/
			if ((ptr = strstr(dir_ent->d_name, ".log")) != NULL)
			{
				bzero(&status, sizeof (struct stat));
				bzero(buf, sizeof(buf));
				sprintf(buf, "%s%s", ndmpLogDir, dir_ent->d_name);
				if (stat(buf, &status) != 0)
				{
				    NDMP_DBG_PRINTF(D_INFO, ("Stat error %d, file %s\n", errno, buf));	
				    continue;
				}

				/* Get modified time of this unzipped file */
				tm_cfil = localtime((const time_t *) &status.st_mtime);

				/* put all dates into the list so we can remove 
				 * unwanted ones later. 
				 */
			    NDMP_DBG_PRINTF(D_INFO, ("file %s, date %02d-%02d-%02d\n", dir_ent->d_name,
					tm_cfil->tm_mon, tm_cfil->tm_mday, tm_cfil->tm_year));

				add_dates_to_lst(tm_cfil, &i); 
			}
		}	
	}
	/* Remove unwanted dates */
	if ( i > ndmp_trace_files )
	{
		for (cur=filhd; (cur !=NULL) && (i > ndmp_trace_files) ; i--)
		{
			tmp = cur;
			cur = cur->nxt;
           	NDMP_SSC_FREE(tmp);
			filhd = cur;
		}
	}
    NDMP_DBG_PRINTF(D_INFO, ("Dates remained\n"));
	for (cur=filhd; (cur !=NULL) ;cur=cur->nxt)
	{
	     NDMP_DBG_PRINTF(D_INFO, ("date:  %02d-%02d-%02d\n", 
				 cur->time.tm_mon, cur->time.tm_mday, cur->time.tm_year));
	}
	rewinddir(dirptr);
	while ((dir_ent = readdir(dirptr)) != NULL)
	{
		if ( strncmp(dir_ent->d_name, "ndmp_msg_", 9) == NULL)
		{
			if ((ptr = strstr(dir_ent->d_name, ".log")) != NULL)
			{
				bzero(&status, sizeof (struct stat));
				bzero(buf, sizeof(buf));
				sprintf(buf, "%s%s", ndmpLogDir, dir_ent->d_name);
				if (stat(buf, &status) != 0)
				{
					NDMP_DBG_PRINTF(D_INFO, ("Stat error %d, file %s\n", errno, buf));	
				    continue;
				}
				tm_cfil = localtime((const time_t *) &status.st_mtime);
				/* if file is not zipped, was it open prior to today? 
				 * i.e., see if file ends w/ '.gz'
				 */	
				yr = tm_cfil->tm_year; mon = tm_cfil->tm_mon; day = tm_cfil->tm_mday;

			     NDMP_DBG_PRINTF(D_INFO, ("file %s, date %02d-%02d-%02d\n", dir_ent->d_name,
					tm_cfil->tm_mon, tm_cfil->tm_mday, tm_cfil->tm_year));

				for (found = FALSE, cur=filhd; (cur !=NULL) ; cur=cur->nxt)
				{
					/* If file has dates that we want to keep */
					if ((yr == cur->time.tm_year) && (mon == cur->time.tm_mon) 
							&& (day == cur->time.tm_mday))
					{
						/* mark that we want to keep this one */
						found = TRUE;
						/* zip if not zipped already */
						if (*(ptr+4)==NULL)
						{
							if ( (tm_cfil->tm_year != tm_todfil.tm_year) ||
								(tm_cfil->tm_mon != tm_todfil.tm_mon) ||
								(tm_cfil->tm_mday != tm_todfil.tm_mday) )
							{
								printf("Zipping %s%s\n", ndmpLogDir, dir_ent->d_name);
								sprintf(command, "/usr/bin/gzip %s%s", ndmpLogDir, dir_ent->d_name);
								my_system(command);
							}
						}
						break;
					} 
				}
				/* If file's date does not fall into the list
				 * of dates we like to retain, delete it 
				 */
				if (found == FALSE)
				{
					printf("Removing %s%s\n", ndmpLogDir, dir_ent->d_name);
					sprintf(command, "/bin/rm %s%s", ndmpLogDir, dir_ent->d_name);
					my_system(command);
				}
			}
		}
	}
	closedir(dirptr);
	/* Free our list */
	for (cur=filhd; (cur !=NULL) ; )
	{
		tmp = cur;	
		cur=cur->nxt;
		NDMP_SSC_FREE(tmp);
	}
	return NFX_OK;
}

// static void ndmp_storeLogFromTmp(pid_t pid)
static void ndmp_storeLogFromTmp(int idx)
{
    int tmpFd, logFd, cnt;
    char tmpName[1024], logName[1024];
    char *buf;
    time_t t;
    struct tm *tm;
    // int32 i;

    // for (i = 0; i < NDMP_MAX_SESS; i++)
    // {
        // if (sessId_Tbl[i].pid == pid)
        // {
            bzero(tmpName, sizeof(char)*1024);
            bzero(logName, sizeof(char)*1024);
            time(&t);
            tm = localtime((const time_t *)&t);
            sprintf(logName, "%sndmp_msg_%s_%02d%02d%04d.log",
                           ndmpLogDir, inet_ntoa(sessId_Tbl[idx].from.sin_addr),
                           tm->tm_mon+1, tm->tm_mday, tm->tm_year+1900);
            sprintf(tmpName, "/tmp/ndmp_msg_%s_%d.log",
             inet_ntoa(sessId_Tbl[idx].from.sin_addr), sessId_Tbl[idx].sessId); 
            tmpFd = open(tmpName, O_RDONLY);
            if (tmpFd == -1)
                return;
            logFd = open(logName, O_CREAT|O_WRONLY|O_APPEND, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
            if (logFd == -1)
            {
                close (tmpFd);
                return;
            }
			if (zip_remove_prev_files(tm) != NFX_OK)
				printf("Unable to zip previous ndmp log files in dir %s\n", ndmpLogDir);

            NDMP_SSC_ALLOC(buf, char, 8192);
            bzero(buf, 8192);
            while ((cnt = read(tmpFd, buf, 8192)) > 0)
            {
                write(logFd, buf, cnt);
                bzero(buf, 8192);
            }

            NDMP_SSC_FREE(buf);
            close(logFd);
            close(tmpFd);
            unlink(tmpName); // Unlink the tmp name
            // break;
        // }
    // }

    return;
}

#ifndef NDMP_GDB		// define flag to do gdb
/*-----------------------------------------------------------------
 * Name:  	Signal handler routine. 
 *
 * Description:  
 * 
 * Created by:  Anh Vu
 *
 * Date Created:        1/17/02
 *
 *-----------------------------------------------------------------
 */
static void clean_me_up(int sig)
{
    int status, rc;
	int i;

    /* Do not use any non-re-entrant fucntions */

	switch (sig) {
		case SIGINT:
		case SIGTERM:

		  if (pid)		// if parent
		  {
			signal(SIGCHLD, SIG_IGN);  /* if we get a sigchild, ignore it */

			/* If i'm the parent, send sigterm to all my kids */
			for (i = 0; i < NDMP_MAX_SESS; i++)
    		{
        		if (sessId_Tbl[i].in_use)
					kill(sessId_Tbl[i].pid, SIGTERM);			
    		}
		  }
		  exit(1);

        case SIGCHLD:
			/* Catch all the children who terminated and caused this signal */
			while( (rc = waitpid(-1, &status, WNOHANG)) > 0)
			{
                // ndmp_storeLogFromTmp(rc);
				ndmpMarkFreeSessId(rc);
#if 0
				ndmpFreeSessId(rc, -1);	
				if (ndmp_tot_sessions > 0)
					ndmp_tot_sessions--;
#endif
			}
			break;
    }
}
#endif

/*-----------------------------------------------------------------
 * Name:    ndmp_cpuEvent
 *
 * Description: Handle the card events 
 * 
 * Created by:  Nitin Jain
 *
 * Date Created: 07/31/02
 *
 *-----------------------------------------------------------------
 */

void ndmp_cpuEvent(nfx_eventMsg_t *evt)
{
    int32 i = 0, rc;
    ndmp_NFXDelSessReq_t req;
    char rspbuf[sizeof(ndmp_NFXShowChildRsp_t)+10];
    ndmp_NFXDelSessRsp_t *rsp = (ndmp_NFXDelSessRsp_t *)rspbuf;
    struct timeval to;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_cpuEvent: Begin\n"));
    if (evt->param.cpuEvent.state == EVENT_STATE_DOWN)
    {
        bzero(&req, sizeof(ndmp_NFXDelSessReq_t));
        switch (evt->param.cpuEvent.slot)
        {
            case 1:
                /*
                 * FP is down. Close connections to all the clients.
                 */
                NDMP_DBG_PRINTF(D_INFO, ("ndmp_cpuEvent: FP-down event\n"));
                strcpy(req.reason, "FP card down");
                break;

            case 2:
                /*
                 * FC is down. Close connections to all the clients.
                 */
                NDMP_DBG_PRINTF(D_INFO, ("ndmp_cpuEvent: SP-down event\n"));
                return;

            case 0:
                /*
                 * This should never happen on SSC
                 */
                NDMP_DBG_PRINTF(D_INFO, ("ndmp_cpuEvent: SSC-down event!!\n"));
                return;
        }

        req.mid = NDMP_MID_UI_DEL_SESS;
        for (i = 0; i < NDMP_MAX_SESS; i++)
        {
            if (sessId_Tbl[i].in_use == TRUE)
            {
                req.sessId = sessId_Tbl[i].sessId;
                to.tv_sec = 30;
                to.tv_usec = 0; 
                rc = sendAgileMsg(-1,&req,sizeof(ndmp_NFXDelSessReq_t),
                                  sessId_Tbl[i].app_name, 0, 0, 0, &to,
                                  rspbuf, sizeof(rspbuf));
                if ((rc <= 0) || (rsp->stat == NFX_ERR))
                {
                    NDMP_DBG_PRINTF(D_INFO, ("ndmp_cpuEvent: sendMsg failed."));
					NDMP_DBG_PRINTF(D_INFO, (" Kill process %d.", sessId_Tbl[i].pid));
                    NDMP_DBG_PRINTF(D_INFO, (" May have to close tape manually.\n"));
                }
                else
                    NDMP_DBG_PRINTF(D_INFO, ("Sess %d is deleted.\n",
                                                 sessId_Tbl[i].sessId));
            }
        }
    }
    NDMP_DBG_PRINTF(D_INFO, ("ndmp_cpuEvent: End\n"));
    return;
}

/*-----------------------------------------------------------------
 * Name:    ndmp_handleNfxEvent
 *
 * Description: Handle NFX event
 * 
 * Created by:  Nitin Jain
 *
 * Date Created: 07/31/02
 *
 *-----------------------------------------------------------------
 */

int32 ndmp_handleNfxEvent(char *rcv, ndmp_src_t *src)
{
    nfx_eventMsg_t *evt = (nfx_eventMsg_t *)rcv;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_handleNfxEvent: Begin\n"));

    switch (evt->eventType)
    {
        case NFX_EVENT_MKFS:
            ndmp_cleanDumpDate((char *)evt);
            break;

        case NFX_EVENT_VOLUME:
            {
                switch (evt->param.volEvent.event)
                {
                    case EVENT_VOL_DELETE:
                    case EVENT_VOL_ONLINE:
                        /*
                         * We may have aborted a backup due to any number
                         * of reasons. When we get the vol online event
                         * go and sync with the FS all the snapshots that
                         * are relevant for us.
                         */
                        ndmp_cleanDumpDate((char *)evt);
                        break;
                    default:
                        NDMP_DBG_PRINTF(D_INFO, ("ndmp_handleNfxEvent: Unsupported event\n"));
                }
            }
            break;

        case NFX_EVENT_CPU:
            ndmp_cpuEvent(evt);
            break;

        default:
           /* We don't handle any other events. */
           break;
    }

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_handleNfxEvent: End\n"));
    return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:  ndmp_ParentProcUIMsg
 *
 * Description:	
 *
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_ParentProcUIMsg(void)
{
	ndmpParentUIMsgEnt_t	*msgent;
	int32			rc = NFX_OK, in_len;
	char			rcvbuf[1024];
	char8			fromapp[1024];
	uint32 			fromcpu = 0, fromslot = 0;
	uint32          *msgTyp; 
	ndmp_src_t		src;

	/* Wait for Agile message from modules from N-NIM or within SSC */	
	bzero(rcvbuf, 1024);
    bzero(fromapp, 1024);
   	in_len = recvAgileMsg (agileFd, rcvbuf, 
					sizeof(rcvbuf), NULL, fromapp, &fromcpu, &fromslot, 0);

   	if (in_len <= 0) 
	{  
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_ParentProcUIMsg:  Rcv error %s", strerror(errno)));
       	E_LOG (class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
       			("ndmp_ParentProcUIMsg: rcv error %s", strerror(errno)));
       	return(-1);
   	}

	msgTyp = (uint32 *)rcvbuf;

	if ((msgent = ndmpGetParentUIMsgEnt(*msgTyp)) == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_ParentProcUIMsg:  received unknown msg type %d.\n", *msgTyp));
		return NFX_ERR;
	}

   	NDMP_DBG_PRINTF(D_INFO, ("ndmp_ParentProcUIMsg:  Rcv %d.\n", *msgTyp));

	ndmp_ssc_par_msgStats[MAX_SSC_CHLD_STAT + (msgent->stat_idx)]++;

	strcpy(src.app, fromapp);			
	src.cpu = fromcpu;			
	src.slot = fromslot;			

	/* Call message functions to process it.  */
	if ((rc = (msgent->msgfunc_ui)(rcvbuf, &src)) != NFX_OK)
	{
    	NDMP_DBG_PRINTF(D_INFO, ("Error processing msg --%s--.\n", msgent->msg_name));
        E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
                                ("Error processing msg --%s--.", msgent->msg_name));
	
		return rc;
	}
	return NFX_OK;
}
	
/*-----------------------------------------------------------------
 * Name:  ndmp_InitMain
 *
 * Description:	
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
void 
ndmp_InitMain(int32 *serverFd)
{
	int					on = 1;
	struct sockaddr_in	sin;

#ifndef NDMP_GDB		// define flag to do gdb
	/* Block all but the signals we'll be using */
	/* Set up our termination handler */
    signal (SIGTERM, clean_me_up);
    signal (SIGINT, clean_me_up);
	
	/* Also, let's keep track of all terminating children */
    signal (SIGCHLD, clean_me_up);
#endif

	/* Register ourselves to receive the messages for NDMP, used for UI cmds.  */
    if ((agileFd = regAgileApp ("ssc_ndmp")) < 0)
	{
        	NDMP_DBG_PRINTF(D_INFO, ("error registering with APN "
					"for NDMP msgs %s", strerror(errno)));
        	E_LOG (class_1, crit_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
					("error registering with APN for NDMP msgs %s", strerror(errno)));
        	exit(1);
    }
    NDMP_DBG_PRINTF(D_INFO, ("AgileFd %d\n", agileFd));

	/* Prepare to accept client's connection to NDMP_SVR_PORT */
	sin.sin_family 		= AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port 		= htons(NDMP_SVR_PORT);

	if ((*serverFd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		NDMP_DBG_PRINTF(D_INFO, ("Socket system call err: %s.", strerror(errno)));
        E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
					("Socket system call err: %s.", strerror(errno)));
		exit(1); 
	}
		
	(void) setsockopt(*serverFd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on));

#ifdef SSC_mips
	setSocketOpt(*serverFd);
#endif

	if (bind(*serverFd, (struct sockaddr *)&sin, sizeof(sin)) < 0)
	{
		NDMP_DBG_PRINTF(D_INFO, ("Bind system call err: %s.", strerror(errno)));
        E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
				("Bind system call err: %s.", strerror(errno)));
		close(*serverFd);
		exit(1); 
	}
		
	if (listen(*serverFd, 5) < 0)
	{
		NDMP_DBG_PRINTF(D_INFO, ("Listen system call err: %s.", strerror(errno)));
        E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
					("Listen system call err: %s.", strerror(errno)));
		close(*serverFd);
		exit(1); 
	}
		
	NDMP_DBG_PRINTF(D_INFO, ("ndmp running\n"));

	ndmp_tot_sessions = 0;

	return;
}
/*-----------------------------------------------------------------
 * Name:  main
 *
 * Description:	 Creates a socket to accept client's connections.
 * 	As a client connects, a child process is created to service 
 * 	this client.  The parent process continues to listen for more 
 *	clients connecting to NDMP port advertized.  We support up
 *	to NDMP_MAX_SESS sessions.
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int main(int argc, char *argv[])
	
{
	int					serverFd, rc, maxfd;
	int 				idx, newSessId = -1, i;
	fd_set              rset1, readmask;


	/* Initialize our session id entry table */
	ndmp_InitSessIdTbl();

	ndmp_InitMain(&serverFd);

    ndmp_readRegistryEntry();

	/* Parent NDMP server uses 2 sockets:
	 *  1) serverFd - Used to accept connection from NDMP clients
	 *  2) agileFd  - Used to accept UI cmds.  Children NDMP servers
	 *   			  will report themselves to the parent to this
	 *				  port.  Parent will then use the fromapp from
	 *				  the child's message.
	 */

	FD_ZERO(&rset1);
	FD_SET(serverFd, &rset1); 		
	FD_SET(agileFd, &rset1); 

	kill (getppid(), SIGUSR1);

    /*
     * Send this indication to FP. So that any sessions that are
     * in process should be aborted. Doing so we will make sure that
     * if NDMP was restarted for any reason there would be no sessions
     * remaining from the previous instance.
     */
#ifdef NFX_SMP
    //ndmp_sendCloseSessToNNIM(NDMP_SESSID_ALL, sess, 0);
#else
    ndmp_sendCloseSessToNNIM(NDMP_SESSID_ALL);
#endif

	for (;;)
	{
		struct sockaddr_in	from;
		int					from_len = sizeof(from);
		int					clientFd;

#ifdef NDMP_GDB		// define flag to do gdb
		ndmpSess_t			*newSess;		// test as parent only
#endif	
		readmask = rset1;

        /*
         * In case the child died during the backup/restore session
         * Send an indication to FP to abort ongoing session.
         */
		//For now, come back later when SMP NDMP is working
        for (i = 0; i < NDMP_MAX_SESS; i++)
        {
            if (sessId_Tbl[i].sessDel)
            {
                //NDMP_DBG_PRINTF(D_INFO, ("main : Clearing session on NNIM\n"));
#ifdef NFX_SMP
                //ndmp_sendCloseSessToNNIM(sessId_Tbl[i].sessId, sess, 0);
#else
                ndmp_sendCloseSessToNNIM(sessId_Tbl[i].sessId);
#endif
                /* Call close tag twice, if we have a MC open, we would
                   need to close the MC and Tape */
                ndmp_procCloseCTagReq(sessId_Tbl[i].sessId, "tape");
                ndmp_procCloseCTagReq(sessId_Tbl[i].sessId, "mc");
                ndmp_storeLogFromTmp(i);
                bzero(&sessId_Tbl[i], sizeof(sessId_entry_t));
                sessId_Tbl[i].pid = -1;
                sessId_Tbl[i].sessId = -1;
		        if (ndmp_tot_sessions > 0)
			        ndmp_tot_sessions--;
                break;
            }
        }
		/* Pick the larger of the 2 sockets */
    	maxfd = ((serverFd > agileFd)? serverFd: agileFd) + 1; 

		rc = select(maxfd, &readmask, NULL, NULL, NULL);

		if (rc < 0)
		{
            NDMP_DBG_PRINTF(D_DBG, ("Error number [%d] : %s\n", 
				errno, (errno == EINTR)?"EINTR" :
					(errno == EBADF)?"EBADF" :
					(errno == EINVAL)?"EINVAL" :
					(errno == ENOMEM)?"ENOMEM" :"Other error"));
			continue;
		}

		/* Check UI cmds to process */
		if (FD_ISSET(agileFd, &readmask)) 
		{
			ndmp_ParentProcUIMsg();
		}

		/* Accept new clients only when we get new client who is 
		 * trying to connect, otherwise we'll be blocked on 
		 * the 'accept' call and will not accept UI cmds. 
		 */
		if (FD_ISSET(serverFd, &readmask)) 
		{
			if ((clientFd = accept(serverFd, (struct sockaddr *)&from,
					 &from_len)) < 0)
			{
				NDMP_DBG_PRINTF(D_INFO, ("Accept system call err: %s.", 
						strerror(errno)));
       			E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 
					0, 0, 0,
					("Accept system call err: %s.", strerror(errno)));
				continue;
			}

			NDMP_DBG_PRINTF(D_INFO, ("Received connection from %s, clientFd %d\n",
		  	inet_ntoa(from.sin_addr), clientFd));
	
#ifndef NDMP_GDB
			/* Keep tracks of how many sessions created */
			if (ndmp_tot_sessions >= NDMP_MAX_SESS) 
			{
				NDMP_DBG_PRINTF(D_INFO, ("Exceeded max number of sessions allowed: %d\n", NDMP_MAX_SESS)); 
				NDMP_DBG_PRINTF(D_INFO, ("Close clientFd %d\n", clientFd)); 
					(void)close(clientFd);
				continue;
			}

			idx = ndmpGetFreeSessId(&newSessId);
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_tot_sessions: %d\n", ndmp_tot_sessions)); 

			if ((idx == -1) || ((pid = fork()) < 0))
			{
				NDMP_DBG_PRINTF(D_INFO, ("Fork system call err: %s.", strerror(errno)));
           		E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("Fork system call err: %s.", strerror(errno)));
				(void)close(clientFd);
				if (pid < 0)
					ndmpFreeSessId(-1, idx);
				continue;
			}

			if (pid == 0)
			{
				/* Child process takes care of new client connection. */
				ndmpSess_t	*newSess;
			
				(void) close(serverFd); 
				(void) close(agileFd);

				NDMP_DBG_PRINTF(D_INFO, ("Child process pid = 	%d, clientFd %d\n", getpid(), clientFd));
#endif
				if ((newSess = ndmp_CreateSess(clientFd, idx, newSessId, &from)) == 0)
				{
					(void)close(clientFd);
					exit(1);
				}
			
				rc = ndmp_procSession(newSess);
	    		ndmp_CloseSess(newSess);
                NDMP_SSC_FREE(newSess);

#ifndef NDMP_GDB
				exit(0);
			} else {
				
				/* update our SessId_Tbl w/ child's process id for this sessid */
				ndmpSetSessId(idx, newSessId, pid, &from);
				ndmp_tot_sessions++;
#endif
				/* Parent process close client's connection and 
				 * accept new ones 
				 */
				(void)close(clientFd); 
				NDMP_DBG_PRINTF(D_INFO, ("forked %d\n", pid));
#ifndef NDMP_GDB
			}
#endif 
		} /* if FD_ISSET */ 

	} // for
	exit(0);
}

/*-----------------------------------------------------------------
 * Name:  ndmpXmtMsg
 *
 * Description:	 Send a request or reply message
 *   conn	- ptr to connection.
 *   message - message code.
 *	 msgtyp - request /reply
 *   err         - error code to place in header.
 *   padded_data   - padded message body
 *   data_len   - of padded data
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmpXmtMsg(		ndmpSess_t		*sess,
				ndmp_HdrMsgTyp	msgtyp,
				ndmp_Msg 		msgid,
				uint32			rspSeq,
				ndmp_Err		err,
				void*			data)
{
	ndmp_Hdr		hdr;
	ndmpMsgEnt_t	*msgent;
	struct timeval	time;
	ndmpMsgEnt_t	*tbl;

	NDMP_DBG_PRINTF(D_INFO, ("ndmpXmtMsg: message:0x%x\n", msgid));

	/* Find message entry in table  */
	tbl = (sess->protoVer == NDMP_VER_2)?  ndmpv2_MsgTbl: ndmpMsgTbl;	
	if ((msgent = ndmpGetMsgEnt(tbl, msgid)) == 0)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmpXmtMsg:   message 0x%x: not supported.\n", msgid));
		E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
				("ndmpXmtMsg:   message 0x%x: not supported.\n", msgid));
		return(-1);
	}
	gettimeofday(&time, 0);

	hdr.xmtSeq		= ++(sess->xmtSeq);	/* our own sequence no. */
	hdr.time_stamp	= time.tv_sec;		/* current time */
	hdr.msgtyp		= msgtyp;			/* request/response */
	hdr.msgId    	= msgid;			/* what message we want to send */
	hdr.rspSeq		= (msgtyp == NDMP_MSG_REQ)? 0 : rspSeq;
	hdr.error		= (msgtyp == NDMP_MSG_REQ)? 0 : err;	/* err decoding or recving msg */


	/* Encode the header */
    sess->xdr_cmd.x_op = XDR_ENCODE;
    if (!xdr_ndmp_Hdr(&sess->xdr_cmd, &hdr))
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmpXmtMsg:  Err encoding request hdr for -- %s --\n", msgent->msg_name));
		E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
					("ndmpXmtMsg:  Err encoding request hdr for -- %s --\n", msgent->msg_name));
        xdrrec_endofrecord(&sess->xdr_cmd, 1);
        return(-1);
    }

    /* Print the header */
    if (ndmp_trace_msg)
    {
        ndmp_print_hdr(&hdr, msgent->msg_name, XMT_MSG, sess);
        fflush(sess->logFp);
    }

	/* If things are cool and we have a message body */
    if (err == NDMP_NO_ERR && data)
    {
		/* xdr encode the body for request message */
		if ((msgtyp == NDMP_MSG_REQ) && msgent->reqfunc)
		{
        	if (!(*msgent->reqfunc)(&sess->xdr_cmd, data))
        	{
				NDMP_DBG_PRINTF(D_INFO, ("ndmpXmtMsg:  Err encoding Request msg -- %s --\n", msgent->msg_name));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("ndmpXmtMsg:  Err encoding Request msg -- %s --\n", msgent->msg_name));
            	xdrrec_endofrecord(&sess->xdr_cmd, 1);
            	return(-1);
			}
            if (ndmp_trace_msg && msgent->printReq)
            {
                (msgent->printReq)((char *)data, sess);
                fflush(sess->logFp);
            }
		} else if ((msgtyp == NDMP_MSG_RSP) && msgent->rspfunc)
		{
			/* else xdr encode body of a response message */
        	if (!(*msgent->rspfunc)(&sess->xdr_cmd, data))
        	{
				NDMP_DBG_PRINTF(D_INFO, ("ndmpXmtMsg:  Err encoding Response msg -- %s --\n", msgent->msg_name));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("ndmpXmtMsg:  Err encoding Response msg -- %s --\n", msgent->msg_name));
            	xdrrec_endofrecord(&sess->xdr_cmd, 1);
            	return(-1);
			}
            if (ndmp_trace_msg && msgent->printRsp)
            {
                (msgent->printRsp)((char *)data, sess);
                fflush(sess->logFp);
            }
        }
    }
	/* Pad the record for XDR and send buffer now */
   	xdrrec_endofrecord(&sess->xdr_cmd, 1);
	return NFX_OK;
} /* ndmpXmtMsg */

/*-----------------------------------------------------------------
 * Name:  ndmp_sendNotifyConn
 *
 * Description:	 Send a Notify Connect message to client
 *				 to either accept his request to connect or
 *				 refuse it.  We can refuse due to max session
 *				 exceeded or because ndmp is disabled.
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        3/14/02
 *
 *-----------------------------------------------------------------
 */
int
ndmp_sendNotifyConn (ndmpSess_t *sess)
{
	ndmp_notify_conn_req 	req;


	/* Keep tracks of how many sessions created */
	if ((!ndmp_enabled) || (ndmp_tot_sessions >= NDMP_MAX_SESS))
	{
		if (ndmp_tot_sessions >= NDMP_MAX_SESS)
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendNotifyConn:  Refuse "
				"connection -  Max sessions exceeded %d.\n", NDMP_MAX_SESS));
		} else {
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendNotifyConn:  Refuse "
				"connection - Ndmp is disabled\n"));
		}	
		sess->conn_refused = TRUE;
		sess->conn_closed = TRUE;
    } 

	/* Send NOTIFY_CONNECTED since the client successfully connected to us */

	memset(&req, 0, sizeof(ndmp_notify_conn_req));
	req.reason = (sess->conn_refused)?
				NDMP_REFUSED: NDMP_CONNECTED;

	req.protoVer = ndmp_hi_ver;		//Advertise highest NDMP version that we support
	req.text_reason = "";

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendNotifyConn: sending NOTIFY_CONNECTED, "
				"[%d]sessId %d\n", sess->sess_idx, sess->sessId));
	ndmpXmtMsg(sess, NDMP_MSG_REQ, NDMP_NOTIFY_CONNECTED, 0, NDMP_NO_ERR, &req);


	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:  ndmp_procSession
 *
 * Description:		Process msgs for each session (a forked child). 
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_procSession (ndmpSess_t *sess)
{
	ndmpMsgEnt_t			*msgent;
	int			 			stat, in_len, maxfdp1, err;
	char8					*AgileMsgbuf;
	char8					fromapp[1024];
	uint32 					fromcpu, fromslot;
	fd_set					rset, rdmask;
	ndmpMsgEnt_t			*tbl;


	/* Read message from client */
    NDMP_DBG_PRINTF(D_INFO, ("------------------------------------------\n"));
    NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession: [%d]SessId -----> %d.  Begin\n",
			sess->sess_idx, sess->sessId));

	ndmp_sendNotifyConn(sess);

	agileMsgBufSiz = STDBUFSIZ;
	FD_ZERO(&rset);
	FD_SET(sess->Agilesock, &rset);
	FD_SET(sess->clientFd, &rset);

	while (!sess->conn_closed) 
	{
		err = NDMP_NO_ERR;

		rdmask = rset;

		/* Pick the larger of the 2 sockets 
		 * 	clientFd:  used to communicate w/ NDMP client.
		 * 	Agilesock: used to communicate w/ FP card and for
		 * 			  UI process to send us UI cmds.
		 */
		maxfdp1 = ((sess->clientFd > sess->Agilesock)? 
					sess->clientFd: sess->Agilesock) + 1;
		select(maxfdp1, &rdmask, NULL, NULL, NULL);
		
#ifdef NFX_SMP
		if ( !sess->tcp_disc && !sess->conn_await_close &&
#else
		if ( !sess->conn_await_close &&
#endif
				((FD_ISSET(sess->clientFd, &rdmask)) || 
				(xdrrec_eof(&sess->xdr_cmd) == FALSE)) )
		{
			NDMP_DBG_PRINTF(D_INFO, ("----------------\n"));
    		NDMP_DBG_PRINTF(D_DBG, ("ndmp_procSession: fd_isset on ndmp client\n"));

			/* Receive data & extract header */
			sess->xdr_cmd.x_op = XDR_DECODE;
			xdrrec_skiprecord(&sess->xdr_cmd);
			if (!xdr_ndmp_Hdr(&sess->xdr_cmd, &(sess->msgRcvd.hdr)))
			{
				NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  [%d]Sessid %d.  "
					"Connection is closed OR error decoding NDMP hdr.\n", 
					sess->sess_idx, sess->sessId));
				//err = NDMP_XDR_DECODE_ERR; //if Legato sends
					//us extra 4 bytes, we don't want to reply
					//with bogus write reply.	
				//goto errReply;
#ifdef NFX_SMP
				goto getAgileMsg;
#else
				continue;
#endif
			}

			/* Protocol version has not been established yet, use v3 
			 * msg table.
			 */
			tbl = ((sess->protoVer == -1) || (sess->protoVer == NDMP_VER_3))?  
							ndmpMsgTbl : ndmpv2_MsgTbl;

			if ((msgent = ndmpGetMsgEnt(tbl, sess->msgRcvd.hdr.msgId)) == NULL)
			{
					NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  [%d]Sessid %d."
						"  msgid %d NOT SUPPORTED.\n", sess->sess_idx, 
						sess->sessId,  sess->msgRcvd.hdr.msgId));		
					err = NDMP_NOT_SUPPORTED_ERR;
					goto errReply;
			}
			sess->cl_msgent = msgent;


            /* We got the header. */
            if (ndmp_trace_msg)
            {
                ndmp_print_hdr(&sess->msgRcvd.hdr, msgent->msg_name,
                               RCV_MSG, sess);
                fflush(sess->logFp);
            }

			/* If this message required authentication and the connection
		 	 * has not been authenticated, reply w/ err.
		 	 */
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  [%d]Sessid %d.  Rcvd -- %s "
					"0x%x-- msg from CLIENT %s\n", sess->sess_idx, sess->sessId, 
					msgent->msg_name, sess->msgRcvd.hdr.msgId,
					inet_ntoa(sess->host_ipaddr.sin_addr)));
			if (msgent->authReqd && !sess->authed)		
			{
				NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  Sessid %d.  "
					"Received -- %s -- msg while connection is un-authorized.\n",
                  	sess->sessId, msgent->msg_name));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("ndmp_procSession:  Received -- %s -- msg while connection is un-authorized.\n",
			                  msgent->msg_name));
            	err = NDMP_NOT_AUTHORIZED_ERR;
				goto errReply;
			}

			if (!msgent->msgfunc_cl)
			{
				NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  [%d]Sessid %d.  "
					"Request -- %s -- not supported\n", sess->sess_idx, 
					sess->sessId, msgent->msg_name));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("ndmp_procSession:  Request -- %s -- not supported", 
						msgent->msg_name));
            	err = NDMP_NOT_SUPPORTED_ERR;
				goto errReply;
			}


			/* Decode request message content.  We won't be getting 
			 * responses from client. 
			 */
			if (msgent->reqfunc)
			{
				memset(sess->msgRcvd.content, 0, NDMP_MAX_REQ_SIZ);
				if (!(msgent->reqfunc)(&sess->xdr_cmd, sess->msgRcvd.content))
				{
					NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  Err "
						"decoding msg %s content.\n", msgent->msg_name));
					E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("ndmp_procSession:  Err decoding msg %s content\n", 
						msgent->msg_name));
				}	
			}

            /* Print the received message */
            if (ndmp_trace_msg && msgent->printReq)
            {
                (msgent->printReq)(sess->msgRcvd.content, sess);
                fflush(sess->logFp);
            }

			/* Call message functions to process it.
			 * If we have error w/in msgfunc, we already sent response
			 * w/ error code to the client already. 
			 */
			stat = (msgent->msgfunc_cl)(sess);

			/* Free the memory malloced by XDR libraries for the body of the 
			 * request.  Note that the XDR strucs includes pointers
 			 * for various fields, so XDR has to malloc these when decoding.
			 * msgRcvd.content is already fixed memory.  We don't need to free
			 * it.
			 */
			free_xdr_rec_stream(sess, msgent);

			if (stat == NFX_OK)
			{
				continue;
			};

errReply:
			/* Dont' send reply to client if he's alread disconnected from server */
			if (!sess->conn_closed)
			{
				/* send reply to client w/ error */
				if (err)
					ndmpXmtMsg(sess, NDMP_MSG_RSP, sess->msgRcvd.hdr.msgId, 
						sess->msgRcvd.hdr.xmtSeq, err, 0);

			}
		}
getAgileMsg:
		if (FD_ISSET(sess->Agilesock, &rdmask) && !sess->conn_closed)
		{		
			ndmp_src_t	src;

			/* Wait for Agile message from modules from N-NIM or within SSC */	
            NDMP_SSC_ALLOC(AgileMsgbuf, char, agileMsgBufSiz);
            /* Check for AgileMsgbug NULL !!! */
			memset(AgileMsgbuf, 0, agileMsgBufSiz);
        	in_len = recvAgileMsg (sess->Agilesock, AgileMsgbuf, 
					agileMsgBufSiz, NULL, fromapp, &fromcpu, &fromslot, 0);

        	if (in_len <= 0) 
			{  
            	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procSession:  [%d]SessId %d - "
						"Rcv error %s", sess->sess_idx, sess->sessId, strerror(errno)));
            	E_LOG (class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
            			("ndmp_procSession: rcv error %s", strerror(errno)));
            	return(-1);
        	}
			strcpy(src.app, fromapp);;			
			src.cpu = fromcpu;			
			src.slot = fromslot;			

			ndmp_procAgileRcvMsg(sess, AgileMsgbuf, in_len, &src);
            NDMP_SSC_FREE(AgileMsgbuf);
		} 
	}
	return NFX_OK;

} /* ndmp_procSession */

/*-----------------------------------------------------------------
 * Name:  ndmp_procAgileRcvMsg
 *
 * Description:	Process Agile message received from NDMP server on
 *  			FP card, UI, tape-driver, or registery manager processes.
 *
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_procAgileRcvMsg(ndmpSess_t *sess, char8 *rcvbuf, int in_len, ndmp_src_t *src)
{
	ndmpAgileMsgEnt_t			*msgent;
	uint32 			*msgTyp = (uint32 *)rcvbuf;
	int32			rc = NFX_OK;



	sess->in_len = in_len;

	if ((msgent = ndmpGetAgileMsgEnt(*msgTyp)) == NULL)
	{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_procAgileRcvMsg:  received unknown msg "
				"type %d for [%d]sessid %d.\n", *msgTyp, sess->sess_idx, sess->sessId));
		return NFX_ERR;
	}

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procAgileRcvMsg:  Proc Id %d - [%d]SessId %d"
			" -- received %s --\n", getpid(), sess->sess_idx, 
			sess->sessId, msgent->msg_name));

	sess->ndmp_ssc_chld_msgStats[msgent->stat_idx_rcvd]++;
	/* Call message functions to process internal messages.  */
	if ((rc = (msgent->msgfunc_int)(sess, rcvbuf, src)) != NFX_OK)
	{
    	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procAgileRcvMsg:  Error processing msg --%s--.\n", msgent->msg_name));
        E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
                                ("ndmp_procAgileRcvMsg:  Error processing msg --%s--.\n", msgent->msg_name));
	
		return rc;
	}
	return NFX_OK;
}
/*-----------------------------------------------------------------
 * Name:  rd_xdr_rec_stream
 * 
 * Description: Read the xdr stream
 * 
 * Created by:  Anh Vu
 * 
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
rd_xdr_rec_stream(char *cl_Handle,
                  char *buf, 
                  int maxlen)       // maxlen to read
{   
    ndmpSess_t  *sess = (ndmpSess_t *)cl_Handle;
    int         readlen;
#if 0
	int			i;
    char        *ptr;
#endif

    NDMP_DBG_PRINTF(D_DBG, ("rd_xdr_rec_stream:  Begin, maxlen = %d\n", maxlen));
    readlen = read(sess->clientFd, buf, maxlen);
    if (readlen <= 0)
    {
        /* Problem w/ read?  Must be connection closing. */  
        NDMP_DBG_PRINTF(D_INFO, ("rd_xdr_rec_stream:  problem w/ rd?  Must be connection's closed.\n"));
		if (sess->wait_tape_open_rsp)
		{
			NDMP_DBG_PRINTF(D_INFO, ("rd_xdr_rec_stream:  Set conn_await_close TRUE\n"));
			sess->conn_await_close = TRUE;
		} else {
#ifdef NFX_SMP
			NDMP_DBG_PRINTF(D_INFO, ("rd_xdr_rec_stream:  Set tcp_disc TRUE\n"));
        	sess->tcp_disc = TRUE;
#else
			NDMP_DBG_PRINTF(D_INFO, ("rd_xdr_rec_stream:  Set conn_closed TRUE\n"));
        	sess->conn_closed = TRUE;
#endif
		}
        return(-1);
    }

    NDMP_DBG_PRINTF(D_INFO, ("rd_xdr_rec_stream:  len read = %d, buf:\n", readlen));
#if 0
    ptr = buf; 
    for (i=0;i < readlen; i++)
    {
        if ((i % 16) == 0)
            NDMP_DBG_PRINTF(D_DBG, ("\n[%04x]: ", i));
        NDMP_DBG_PRINTF(D_DBG, ("%02x ", (uchar8)ptr[i]));
    }
    NDMP_DBG_PRINTF(D_DBG, ("\n"));
#endif
    return(readlen);
}
/*-----------------------------------------------------------------
 * Name:  wr_xdr_rec_stream
 *
 * Description: Writes the xdr stream
 *
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
wr_xdr_rec_stream(char  *cl_Handle,
             char    *buf,
             int     len)       // length to write
{
    ndmpSess_t      *sess = (ndmpSess_t *)cl_Handle;
    register int    n;
    register int    cnt;

#if 0
    int             i;
    char            *ptr;

    ptr = buf;
    for (i=0;i < len; i++)
    {
        if ((i % 16) == 0)
            NDMP_DBG_PRINTF(D_DBG, ("\n[%04x]: ", i));
        NDMP_DBG_PRINTF(D_DBG, ("%02x ", (uchar8)ptr[i]));
    }
    NDMP_DBG_PRINTF(D_DBG, ("\n"));
#endif

    NDMP_DBG_PRINTF(D_DBG, ("wr_xdr_rec_stream:  Begin, len %d %s\n", len, buf));

    for(cnt = len; cnt > 0; cnt -= n, buf += n)
    {
        if ((n = write(sess->clientFd, buf, cnt)) < 0)
        {
            NDMP_DBG_PRINTF(D_INFO, ("wr_xdr_rec_stream:  Connection's closed, write failed.\n"));
			if (sess->wait_tape_open_rsp)
			{
				NDMP_DBG_PRINTF(D_INFO, ("wr_xdr_rec_stream:  Set conn_await_close TRUE\n"));
				sess->conn_await_close = TRUE;
			} else {
				NDMP_DBG_PRINTF(D_INFO, ("wr_xdr_rec_stream:  Set conn_closed TRUE\n"));
        		sess->tcp_disc = TRUE;
			}
            return (-1);
        }
        NDMP_DBG_PRINTF(D_INFO, ("wr_xdr_rec_stream:  %d chars written\n", n));
    }
    NDMP_DBG_PRINTF(D_INFO, ("wr_xdr_rec_stream:  len written %d \n", len));
    return(len);
}

/*-----------------------------------------------------------------
 * Name:  free_xdr_rec_stream
 *
 * Description:     Frees the XDR stream.
 *
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
void
free_xdr_rec_stream(ndmpSess_t  *sess,  ndmpMsgEnt_t *msgent)
{

    //NDMP_DBG_PRINTF(D_INFO, ("free_xdr_rec_stream:  Begin, sessId %d, Request msg %s\n", sess->sessId, msgent->msg_name));

    /* Free the memory malloced by XDR libraries when we receive the msgs */
    sess->xdr_cmd.x_op = XDR_FREE;

	if (msgent->reqfunc)
    {
		(msgent->reqfunc)(&sess->xdr_cmd, sess->msgRcvd.content);	
	}
    return;
}   

