/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: nb-cmd.c,v 1.1.1.1 2002/04/23 17:52:14 laurentb Exp $
 *
 * Name			: nb-cmd.c
 *                              
 * Description	: Contain NetBIOS UI command functions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 11/29/2001
 *
 *-----------------------------------------------------------------
 */

#include "nb-api.h"
#include "nb-ns.h"
#include "nb-test.h"

extern int32 cifs_cmdHelpCmd (char *pCmd);

#ifdef FS_STANDALONE
int32 strequ(char8 * str1, char8 * str2);
#endif //FS_STANDALONE

/*-----------------------------------------------------------------
 * Name			: nb_cmdUnitTest
 *
 * Description	: NetBIOS unit test commands
 *
 * Created by	: Xiaobing Zhang
 * 
 * Date Created	: 11/29/2001
 *-----------------------------------------------------------------
 */
int32 nb_cmdUnitTest( int32 argc, char *argv[] )
{
	if (argc < 3)
	{
		cifs_cmdHelpCmd("nb");
		return 0;
	}

	/* Name service */
	if (strequ(argv[2], "oreqque1"))	
	{
		printf("NetBIOS outgoing name query request\n");
		nbtest_oQueryReq(1);
	}
	if (strequ(argv[2], "oreqque2"))	
	{
		printf("NetBIOS outgoing name query request\n");
		nbtest_oQueryReq(2);
	}
	if (strequ(argv[2], "oreqque3"))	
	{
		printf("NetBIOS outgoing name query request\n");
		nbtest_oQueryReq(3);
	}
	else if (strequ(argv[2], "oreqrel"))
	{
		printf("NetBIOS outgoing name release request\n");
		nbtest_oReleaseReq();
	}
	else if (strequ(argv[2], "orspreg"))
	{
		printf("NetBIOS outgoing name registration response\n");
		nbtest_oRegRsp();
	}
	else if (strequ(argv[2], "orspque"))
	{
		printf("NetBIOS outgoing name query response\n");
		nbtest_oQueryRsp();
	}
	else if (strequ(argv[2], "orsprel"))
	{
		printf("NetBIOS outgoing name release response\n");
		nbtest_oReleaseRsp();
	}
	else if (strequ(argv[2], "ireqreg"))
	{
		printf("NetBIOS incoming name registration request\n");
		nbtest_iRegReq();
	}
	else if (strequ(argv[2], "ireqque"))
	{
		printf("NetBIOS incoming name query request\n");
		nbtest_iQueryReq();
	}
	else if (strequ(argv[2], "ireqrel"))
	{
		printf("NetBIOS incoming name release request\n");
		nbtest_iReleaseReq();
	}
	else if (strequ(argv[2], "irspreg"))
	{
		printf("NetBIOS incoming name registration response\n");
		nbtest_iRegRsp();
	}
	else if (strequ(argv[2], "irspref"))
	{
		printf("NetBIOS incoming name registration response to refresh request\n");
		nbtest_iRefRsp();
	}
	else if (strequ(argv[2], "irspque"))
	{
		printf("NetBIOS incoming name query response\n");
		nbtest_iQueryRsp();
	}
	else if (strequ(argv[2], "irsprel"))
	{
		printf("NetBIOS incoming name release response\n");
		nbtest_iReleaseRsp();
	}
	else if (strequ(argv[2], "irspwack"))
	{
		printf("NetBIOS incoming WACK response\n");
		nbtest_iWackRsp();
	}

	/* Session service */
	else if (strequ(argv[2], "ireqss"))
	{
		printf("NetBIOS incoming session request\n");
		nbtest_iSsnReq();
	}
	else if (strequ(argv[2], "odata"))
	{
		printf("NetBIOS outgoing user data\n");
		nbtest_oSsnData();
	}
	else if (strequ(argv[2], "idata"))
	{
		printf("NetBIOS incoming user data\n");
		nbtest_iSsnData();
	}
	else if (strequ(argv[2], "ika"))
	{
		printf("NetBIOS incoming session keep alive\n");
		nbtest_iKeepAlive();
	}
	else if (strequ(argv[2], "endss"))
	{
		printf("NetBIOS end session\n");
		nbtest_endss();
	}

	/* Datagram service */
	else if (strequ(argv[2], "odgm"))
	{
		printf("NetBIOS outgoing datagram\n");
		nbtest_odgm();
	}

	return 0;
}

