/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: nb-ds.h,v 1.1.1.1 2002/04/23 17:52:14 laurentb Exp $
 *
 * Name			: nb-ds.h
 *
 * Description	: NetBIOS datagram service definitions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 12/05/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _NB_DS_H
#define _NB_DS_H

/*-----------------------------
 * Datagram service definitions
 *-----------------------------
 */
#define NB_DS_UDP_PORT				138

/* Datagram message type */
#define NB_DS_DGM_TYPE_DIR_UNIQUE		0x10 // Direct_unique datagram
#define NB_DS_DGM_TYPE_DIR_GROUP		0x11 // Direct_group datagram
#define NB_DS_DGM_TYPE_BROADCAST		0x13 // Broadcast datagram
#define NB_DS_DGM_TYPE_ERR				0x13 // Datagram error
#define NB_DS_DGM_TYPE_QUERY_REQ		0x14 // Datagram query request
#define NB_DS_DGM_TYPE_POS_QUERY_RSP	0x15 // Datagram positive query response
#define NB_DS_DGM_TYPE_NEG_QUERY_RSP	0x16 // Datagram negative query response

/* Datagram source end-node type */
#define NB_DS_NODE_TYPE_B			0x00
#define NB_DS_NODE_TYPE_P			0x01
#define NB_DS_NODE_TYPE_M			0x02
#define NB_DS_NODE_TYPE_H			0x02
#define NB_DS_NODE_TYPE_NBDD		0x03

/* Datagram error */
#define NB_DS_ERR_DEST_NAME_NOT_PRESENT		0x82
#define NB_DS_ERR_INVALID_SRC_NAME_FORMAT	0x83
#define NB_DS_ERR_INVALID_DEST_NAME_FORMAT	0x84

/*-----------------------------------------
 * Datagram service related data structures 
 *-----------------------------------------
 */
/* Flags */
typedef struct nb_dsFlags
{
	uint16			nodeType;
	boolean			first;
	boolean			more;
} nb_dsFlags_t;

/* Datagram header */
typedef struct nb_dsHeader
{
	uint16			dgmID;			// Datagram ID
	nb_dsFlags_t	flags;			// Flags
	uint32			srcIP;			// Source IP address
	uint16			srcPort;		// Source port
	uint16			dgmLen;			// Datagram length, including names and data
	uint16			offset;			// Packet offset
} nb_dsHeader_t;

/* Datagram packet */
typedef struct nb_dsPacket
{
	nb_dsHeader_t	header;
	nb_name_t		srcName;		// Source name
	nb_name_t		destName;		// Destination name
	uint32			dataLen;		// Length of user data
	/*
	char			userData[NB_S_MAX_IP_DATAGRAM];
	*/
	char			*userData;
									// User data
} nb_dsPacket_t;

/*--------------------
 * Function prototypes
 *--------------------
 */
nb_dsPacket_t *nb_dsCreateDgmPkt(nb_srvCB_t *nbSrvCbP);

nb_dsPacket_t *nb_dsCreateAnncePkt(nb_srvCB_t *nbSrvCbP,
								   char *destNameStr,
								   uint32 userDataLen);

void nb_dsUpdateXid(nb_dsPacket_t *dsPktP, uchar8 *dsXmtBufP);

void nb_dsUpdateNodeType(nb_dsPacket_t *dsPktP,
						 uchar8 *dsXmtBufP,
						 uint16 nodeType);

int32 nb_dsXmtPktReq(nb_srvCB_t *nbSrvCbP,
					 nb_name_t destName,
					 pkt_desc_t *msgBufferP,
					 uint32 smbMsgLen);

#endif /* _NB_DS_H */
