/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-attr.h
 *
 * RCS:  $Id: nfs-attr.h,v 1.2 2002/10/04 01:29:25 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Contains function prototype for NFS attribute routines.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 10/01/02
 *
 *-----------------------------------------------------------------
 */
#ifndef	_NFS_ATTR_H
#define	_NFS_ATTR_H

int32 
nfs_extractCharVal (char 	*nfsMsgp,    
                	uchar8 	*value );

int32 
nfs_extractShortVal(char 	 *nfsMsgp,
                	ushort16 *value );

int32 
nfs_extractLongVal( char 		*nfsMsgp,
                	uint32 		*value  );

int32 
nfs_extractLongLongVal( char 	*nfsMsgp, 
                		uint64 	*value  );

int32
nfs_extractFhandle( int32 		nfs_version,
            		char 		*nfsMsgp,
            		fs_Handle 	*fhandlep );

int32
nfs_extractFhandle( int32 		nfs_version,   
            		char 		*nfsMsgp,
            		fs_Handle 	*fhandlep );

int32   /*length = number of bytes of the the object */
nfs_extractString ( char 	 *sourceBufp,   /*source message buffer*/
        			char 	 *destNamep,    /*return name string*/
        			ushort16 *nameLen,  	/*return name length in bytes*/
        			ushort16 maxLen     	/*maximum len allowed*/
				   );

int32   /* lenth of filehandle in bytes */
nfs_storeFhandle(int32      nfs_version,    /*NFS version*/
                 fs_Handle  *fhandlep,      /*Source file handle*/
                 char       *destbufp);      /*destination buffer */

int32   /* lenth of filehandle in bytes */
nfs_storeV3Fhandle( fs_Handle   *fhandlep,      /*Source file handle*/
                    char        *destbufp);     /*destination buffer */

int32   /* lenth of filehandle in bytes */
nfs_storeV2Fhandle( fs_Handle   *fhandlep,      /*Source file handle*/
                    char        *destbufp);     /*destination buffer */

int32 
nfs_storeByteValToBuf(uchar8 *destBuf, uchar8  byteVal);

int32 
nfs_storeShortValToBuf(char *destBuf, ushort16 shortVal);

int32 
nfs_storeLongValToBuf(char *destBuf, uint32 longVal);

int32 
nfs_storeLongLongValToBuf(char *destBuf, uint64 longLongVal);

int32
nfs_storeString( char 	*DestBufp,   /*dst buffer to store input string*/
    			char 	*string,     /*string*/
    			uint32 	strLen,      /*string length in bytes*/
    			uint32 	maxLen );    /*maximum string length allowed*/

int32
nfs_storeIpAddr(char 	*destBufp,	/* dest buffer to store */
                uint32 	ipAddr);

int32 /* length = number of bytes of the field */
nfs_storeAuthFlavor(char *msgp);


int32   
nfs_extractSetattr( int32 		nfs_version,
            		char 		*nfsMsgp,
            		fs_Attr 	*fsAttrp,
            		uint32 		*atimeType,
            		uint32 		*mtimeType );
int32 
nfs_extractReaddirCookie(uint32 	nfs_version,    
        				 char 		*nfs_msgp,     
        				 uint64 	*cookie,      
        				 uint64 	*cookieVerf); 

int32   
nfs_extractDevNum(char *nfs_msgp, fs_DevNum *devNump);

int32 
nfs_storeDirEntry(uint32 		nfs_version, 
				  fs_DirEntry	*fsDirEntryp,	
				  char 			*destMsgp);

int32   
nfs_storeDirEntryPlus( fs_DirEntryPlus  *fsDirEntryPlusp,
					   char 			*destMsgp );

int32 
nfs_storePreOpAttributes( fs_Attr 	*fs_attrp,	
						  char 		*destBuff,		
						  uint32 	attrAvail );

int32 
nfs_storeV3Attributes(uint32 	reqType,     	
					  uchar8 	attrAvailable,
					  fs_Attr 	*fs_attrp,	
					  char 		*destMsgp);

int32
nfs_storeV2Attributes(uint32 	reqType,     	
					  uchar8 	attrAvailable,
					  fs_Attr 	*fs_attrp,	
					  char 		*destBuff);

int32  
nfs_storeAttributes(uint32 		reqType,     
					uchar8 		attrAvailable,
					fs_Attr 	*fs_attrp,
					char 		*destBuff,
					int32 		nfs_version);

int32
nfs_storeFieldPresentFLag(char *msgp, uint32 presentFlag);

int32
nfs_storeStatInfo(char *destMsgp, fs_Stat *fsStatp, uint32 nfs_version);

int32
nfs_storePathconf(char *destMsgp, fs_Pathconf *pathconfp);

int32
nfs_storeFsinfo(char *destMsgp, fs_Fsinfo *fsInfop);


#endif	/* _NFS_ATTR_H */
