/* vi:ts=4
 *-----------------------------------------------------------------
 * 
 * Name:        nfs-decode.c
 *
 * RCS:  $Id: nfs-decode.c,v 1.29 2002/09/13 00:19:24 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *  Contain NFS decode & encode functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        5/28/01 
 *
 *-----------------------------------------------------------------
 */
#ifdef	NFP_TXRX
#include <nfx-incl.h>
#include <exports/nfs-api.h>

#define	NFS_DBG_ON

char                hostName[FS_NAME_MAX];

//static char *line[] = {"\t---------------------------------------------------------------------\n"};

//static char *rpcHdr = {"\n\t----- RPC Header -----\n"};
static char *nfs_v2Procs[] = 	{	"\tProc 0 = (NULL)\n", 
					"\tProc 1 = (GETATTR-Get file attributes)\n",
					"\tProc 2 = (SETATTR-Set file attributes)\n",
					"\tProc 3 = (Root)\n",
					"\tProc 4 = (LOOKUP-Look up file name)\n",
					"\tProc 5 = (READLINK-Read from symbolic link)\n",
					"\tProc 6 = (READ-Read from a file)\n",
					"\tProc 7 = (WriteCache)\n",
					"\tProc 8 = (WRITE-Write to a file)\n",
					"\tProc 9 = (CREATE-Create file)\n",
					"\tProc 10 = (REMOVE-Remove file)\n",
					"\tProc 11 = (RENAME-Rename file)\n",
					"\tProc 12 = (LINK-Hard link)\n",
					"\tProc 13 = (SYMLINK-Make a symbolic link)\n",
					"\tProc 14 = (MKDIR-Create a directory)\n",
					"\tProc 15 = (RMDIR-Remove a directory)\n",
					"\tProc 16 = (READDIR-Read from directory)\n",
					"\tProc 17 = (STATFS-Get Filesystem Attributes)\n",
				};
static char *nfs_v3Procs[] = 	{	"\tProc 0 = (NULL)\n", 
					"\tProc 1 = (GETATTR-Get file attributes)\n",
					"\tProc 2 = (SETATTR-Set file attributes)\n",
					"\tProc 3 = (LOOKUP-Look up file name)\n",
					"\tProc 4 = (ACCESS-Check access permission)\n",
					"\tProc 5 = (READLINK-Read from symbolic link)\n",
					"\tProc 6 = (READ-read from file\n",
					"\tProc 7 = (WRITE-write to file)\n",
					"\tProc 8 = (CREATE-Create a file)\n",
					"\tProc 9 = (MKDIR-Create a directory)\n",
					"\tProc 10 = (SYMLINK-Make a symbolic link)\n",
					"\tProc 11 = (MKNOD-Make a special file)\n",
					"\tProc 12 = (REMOVE-Remove  a file)\n",
					"\tProc 13 = (RMDIR-Remove a directory)\n",
					"\tProc 14 = (RENAME-Rename a file or directory)\n",
					"\tProc 15 = (LINK-Create a hard link to an object)\n",
					"\tProc 16 = (READDIR-Read from directory)\n",
					"\tProc 17 = (READDIRPLUS-Read from directory - plus)\n",
					"\tProc 18 = (FSSTAT-Get filesystem statistics)\n",
					"\tProc 19 = (FSINFO-Get static filesystem information)\n",
					"\tProc 20 = (PATHCONF-Get POSIX information)\n",
					"\tProc 21 = (COMMIT-Commit to stable storage)\n",
				};


static char *noValueStr[] = {"No value"};

static char *authTypes[] = 	{"\tAUTH_NONE\n", 		//0: AUTH_NONE
					"\tAUTH_SYS\n",					//1: AUTH_SYS or AUTH_UNIX
					"\tAUTH_SHORT\n",				//2: AUTH_SHORT
					"\tAUTH_DH\n",					//3: Diffie-Hellman exchange of DES key
					"\tAUTH_KERB4\n",				//4: Kerberos
					"\tAUTH_RSA\n",					//5: 
					"\tAUTH_GSS\n",					//6: Gerneric Security services API
				};

#define	MAX_LINE_SIZE	128

char8	printBuf[MAX_LINE_SIZE];

char fhandleBuf[] = "\tFile handle = \t";

/*-----------------------------------------------------------------
 * Name:    nfs_getFtypeStr	 
 *
 * Description:
 * Returns File Type string. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
char *nfs_getFtypeStr(char faType)
{
	switch (faType)
	{
		case FS_FTYPE_NON:
			return ("(Invalid)");
		case  FS_FTYPE_REG:
			return ("(Regular)");
		case FS_FTYPE_DIR:
			return ("(Directory)");
		case FS_FTYPE_BLK:
			return ("(Block device)");
		case FS_FTYPE_CHR:
			return ("(Character device)");
		case FS_FTYPE_LNK:
			return ("(Symbolic link)");
		case FS_FTYPE_SOCK:
			return ("(Socket)");
		case FS_FTYPE_FIFO:
			return ("(Named pipe)");
	}
	return ("(Invalid)");
}

/*-----------------------------------------------------------------
 * Name:   	nfs_decodeGetTimeTypeStr 
 *
 * Description:
 * Returns time type string.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
char *nfs_decodeGetTimeTypeStr(uint32 timeType)
{

	switch (timeType)
	{
		case NFS_SATTR_DONT_CHANGE:
			return ("Don't change");	
		case NFS_SATTR_SERVER_TIME:
			return ("Set to server time");
		case NFS_SATTR_CLIENT_TIME:
			return ("Set to client time");

	}
	return ("Invalid time type");
}

/*-----------------------------------------------------------------
 * Name:   	nfs_txDecodeString 
 *
 * Description:
 * Decodes an outgoing string.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*length = number of bytes of the the object */
nfs_txDecodeString (char *sourceBufp,    /*source message buffer*/
	char *destNamep,    /*return name string*/
	ushort16 *nameLen)  /*return name length in bytes*/
{   
	char    *currMsgp;
	uint32  *msgp;
	uint32  *destbufp;
	uint32  strLen;
	int32   i;
	int32   iteration;
	uint32  fieldLen;
	uint32  padLen;

	if ( (sourceBufp == NULL) || (destNamep == NULL) )
		return (NFX_ERR);

	msgp = (uint32 *)sourceBufp;
	currMsgp = sourceBufp;

	fieldLen = padLen = strLen = 0;
	strLen = *msgp++;
	*nameLen = strLen;
	currMsgp += sizeof(uint32);
	fieldLen += sizeof(uint32);
	//printf("nfs_txDecodeString 1: strLen: %d\n", strLen);
	if ( ( strLen < 0) ||
		(strLen > FS_PATH_MAX) )
	{
		printf("%s: Invalid string length (%d)\n", 
						__FUNCTION__, strLen);
		return (NFX_ERR);
	}
	destbufp = (uint32 *)destNamep;

	iteration = strLen >> 2;
	/*
     *Need account for padding bytes if 
	 *string length is not divisible by 4.
     *XDR primitive type is represented in 
	 *integral number of 4 byte XDR unit.
     */
	if (strLen & 0x03)
		iteration++;

	for (i = 0;  i < iteration; i++)
	{
		*destbufp++ = ntohl(*msgp++);
	}
	fieldLen += (iteration << 2);

	return (fieldLen);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_decodeGetProgNumStr 
 *
 * Description:
 * Returns RPC Program Number string.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
char *nfs_decodeGetProgNumStr(uint32 rpcProgNum )
{
	switch (rpcProgNum)
	{
		case NFS_PROG:
			return ("NFS");
		case MNT_PROG:
			return ("MOUNT");
		case NLM_PROG:
			return ("NLM");
		case NSM_PROG:
			return ("NSM");

	}
	return ("Invalid");
}

/*-----------------------------------------------------------------
 * Name:   	nfs_decodePresentFlag 
 *
 * Description:
 * Decode TRUE field present flag. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 
nfs_decodePresentFlag (char *nfs_msgp, uint32 *presentFlag)
{
	uint32	*msgp;
		
	printf("\tValue follows = \t");
	msgp = (uint32 *)nfs_msgp;
	*presentFlag = *msgp;
	
	printf("%s\n", (*presentFlag == 0)? "No": "Yes");

	return (sizeof(uint32));
}

/*-----------------------------------------------------------------
 * Name:   	nfs_txExtractLongVal 
 *
 * Description:
 * Extracts a 32-bits value from message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_txExtractLongVal(char *nfsMsgp,    /*source message buffer*/
	uint32 *value               /*return value*/
)
{
#ifdef	NOTUSED
	*value = (*(uint32 *)nfsMsgp);
	return (sizeof(uint32));
#endif
	char	*dest = (char *)value;
	char	*src = (char *)nfsMsgp;

	memcpy(dest, src, sizeof(uint32));
	return (sizeof(uint32));
}

/*-----------------------------------------------------------------
 * Name:    nfs_txExtractLongLongVal
 *
 * Description:
 * Retrieves a long (64 bits) value information from message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_txExtractLongLongVal(char *nfsMsgp, /*source message buffer*/
                uint64 *value               /*return value*/
)
{
	int32           *srcMsg32p;
	int32           *destMsg32p;


	srcMsg32p = (int32 *)nfsMsgp;
	destMsg32p = (int32 *)value;
	destMsg32p[0] = srcMsg32p[1];
	destMsg32p[1] = srcMsg32p[0];

#ifdef	NOTUSED
	char *dest = (char*)nfsMsgp;
	char *src = (char*)value;

	dest[0] = src [4];
	dest[1] = src [5];
	dest[2] = src [6];
	dest[3] = src [7];
	dest[4] = src [0];
	dest[5] = src [1];
	dest[6] = src [2];
	dest[7] = src [3];
#endif
	return (sizeof(uint64));
}  

/*-----------------------------------------------------------------
 * Name:   	nfs_decodeFileId 
 *
 * Description:
 * Decodes fileid field of a READDIR reply. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 
nfs_decodeFileId(char *nfs_msgp, uint32 nfs_version)
{
	uint32	fileId32;
	uint64	fileId64;
	int32	length;

	/*
	 *Decode fileid
	 */
	printf("\tfileid = \t");
	if (nfs_version == NFS_VER2)
	{
		length = nfs_txExtractLongVal(nfs_msgp, &fileId32);	
		printf("%d\n", fileId32);
		return (length);
	}
	else
	{
        length = nfs_txExtractLongLongVal(nfs_msgp, &fileId64);
        printf("%lld\n", fileId64);
		return (length);
	}
}

/*-----------------------------------------------------------------
 * Name:   	nfs_decodeCookie 
 * 
 * Description:
 * Decodes cookie field of a READDIR reply.
 * 
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 * 
 *-----------------------------------------------------------------
 */
int32 
nfs_decodeCookie(char *nfs_msgp, uint32 nfs_version)
{
	uint64		cookie64;
	uint32		cookie32;
	int32		length;

	/*
	 *Decode cookie
	 */
	printf("\tCookie = \t");
	if (nfs_version == NFS_VER2)
	{
		length = nfs_txExtractLongVal(nfs_msgp, &cookie32);	
		printf("%d\t\n", cookie32);
		return (length);
	}
	else
	{
		length= nfs_txExtractLongLongVal(nfs_msgp, &cookie64);
		printf("%lld\n", cookie64);
		return (length);
	}
}

/*-----------------------------------------------------------------
 * Name:   	nfs_decodeDirEntry 
 *
 * Description:
 * Decodes fields of a READDIR reply.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodeDirEntry(char *nfs_msgp, uint32 reqType, uint32 nfs_version)
{

//	uint64			*fileIdp;
	char			*destMsgp;
	uint32			fieldLen;
	int32			length;
	uint32			presentFlag;

	fieldLen = 0;
	destMsgp = nfs_msgp;
	length = nfs_decodeFileId(destMsgp, nfs_version);
	fieldLen += length;
	destMsgp += length;
	/*
	 *Decode name
	 */	
	length = nfs_decodeName(destMsgp, 
					NFS_DECODE_FILENAME, NFS_DECODE_XMT);
	if (length == NFX_ERR)
	{
		printf("%s: Failure to decode filename!\n",
			__FUNCTION__);
		return (NFX_ERR);
	}
	fieldLen += length;
	destMsgp += length;

	/*
	 *Decode Cookie
	 */
	length =   nfs_decodeCookie(destMsgp, nfs_version);
	fieldLen += length;
	destMsgp += length;
	if (reqType == NFS_READDIRPLUS)
	{
		/*
		 *For Readdirplus: decode attributes & filehandle
		 */
		length = nfs_decodeAttributes(destMsgp, NFS_VER3, reqType);
		fieldLen += length;
		destMsgp += length;

		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		destMsgp += length;
		fieldLen += length;

		length = nfs_decodeFhandle(destMsgp,NFS_VER3);
		fieldLen += length;
		destMsgp += length;
	}
	printf("\n");

	return (fieldLen);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_decodeAuthFlavor 
 *
 * Description:
 * Decodes Authentication flavor field of a READDIR reply.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodeAuthFlavor (char *destMsgp)
{
//	uint32		*msgp;
	uint32		authFlavor;
//	int32		length;

	printf("\tAuthentication Flavors = \t");
	authFlavor = (*(uint32 *)destMsgp);
	switch (authFlavor)
	{
		case NFS_RPC_AUTH_NONE:
			printf("NULL\n");	
			break;
		case NFS_PRC_AUTH_SYS:
			printf("AUTH_UNIX\n");
			break;
		case NFS_RPC_AUTH_SHORT:
			printf("AUTH_SHORT\n");
			break;
		case NFS_RPC_AUTH_DH:
			printf("AUTH_DH\n");
			break;
		case NFS_RPC_AUTH_KERB4:
			printf("AUTH_KERB4\n");
			break;
	}

	return (8);

}

/*-----------------------------------------------------------------
 * Name:      	nfs_decodeFhandle 
 *
 * Description:
 *      Decode outgoing filehandle from buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodeFhandle(char *nfs_msgp, 	/*Pointer to filehandle*/
			int32 nfs_version)	/*NFS version*/
{
//	char		*bufp;
	uint32		*msgp;
	int32		iteration;
	int32		count;
	int32		i;
//	uint64		volId, inode;
//	int32		length;
	fs_Handle	*fhandlep;

	printf("\tFile handle = \t");
	msgp = (uint32 *)nfs_msgp;

	switch (nfs_version)
	{
		case NFS_VER3:
		{
			uint32		fHandleLen;
			uint32		fieldLen;
	
			/*
			 *Get filehandle length (byte count)
			 */	
			msgp++;
			fHandleLen = sizeof(fs_Handle);
			fhandlep = (fs_Handle       *)msgp;
			iteration = fHandleLen >> 2; 
			count = 0;
			for (i = 0; i < iteration; i++, msgp++)
			{
				printf("%08x ", *msgp);
				count++;
				if (count == 4)
				{
					printf("\n\t\t\t");
					count = 0;
				}
			}
			printf("\n");
			//msgp = (char *)fhandlep;
			//printf("\t\t\t(VolId: %lld ", fhandlep->volId.whole_thing);
			//printf("Inode: %lld) \n", fhandlep->inum);
			fieldLen = fHandleLen + sizeof (int32);
			return (fieldLen);
		}
		case NFS_VER2:
		default:
			iteration = NFS_V2_FHANDLE_LEN >> 2;
			count = 0;
			fhandlep = (fs_Handle       *)msgp;
			for (i = 0; i < iteration; i++, msgp++)
			{
				printf("%08x ", *msgp);
				count++;
				if (count == 4)
				{
					printf("\n\t\t\t");
					count = 0;
				}

			}
			printf("\n");
			//printf("\t\t\t(VolId: %lld ", fhandlep->volId.whole_thing);
			//printf("Inode: %lld) \n", fhandlep->inum);
			return (NFS_V2_FHANDLE_LEN);

	}
	return (NFX_ERR);
}
/*-----------------------------------------------------------------
 * Name:         nfs_RxDecodeFhandle
 *
 * Description:
 *      Decode received filehandle from input buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_rxDecodeFhandle(char *nfs_msgp, 	/*Pointer to filehandle*/
			int32 nfs_version)	/*NFS version*/
{
//	char		*bufp;
	uint32		*msgp;
	int32		iteration;
	int32		count;
	int32		i;
//	uint64		volId, inode;
//	int32		length;
	fs_Handle	*fhandlep;
	fs_Handle	fileHandle;
	int32		ourFhandleLen;

	printf("\tFile handle = \t");
	msgp = (uint32 *)nfs_msgp;
	ourFhandleLen = sizeof(fs_Handle);
	switch (nfs_version)
	{
		case NFS_VER3:
		{
			uint32		fHandleLen;
			uint32		fieldLen;
	
			/*
			 *Get filehandle length (byte count)
			 */	
			msgp++;
			fHandleLen = sizeof(fs_Handle);
			fhandlep = (fs_Handle       *)msgp;
			memcpy(&fileHandle, fhandlep, ourFhandleLen);
			xdr_swap(&fileHandle, ourFhandleLen, 0);
			fhandlep = &fileHandle;
			iteration = fHandleLen >> 2; 
			count = 0;
			for (i = 0; i < iteration; i++, msgp++)
			{
				printf("%08x ", *msgp);
				count++;
				if (count == 4)
				{
					printf("\n\t\t\t");
					count = 0;
				}
			}
			printf("\n");
			//printf("\t\t\t(VolId: %lld ", 
			//			fhandlep->volId.whole_thing);
			//printf("Inode: %lld) \n", fhandlep->inum);
			fieldLen = fHandleLen + sizeof (int32);
			return (fieldLen);
		}
		case NFS_VER2:
		default:
			memcpy(&fileHandle, nfs_msgp, ourFhandleLen);
			xdr_swap(&fileHandle, ourFhandleLen, 0);	
			iteration = NFS_V2_FHANDLE_LEN >> 2;
			count = 0;
			for (i = 0; i < iteration; i++, msgp++)
			{
				printf("%08x ", *msgp);
				count++;
				if (count == 4)
				{
					printf("\n\t\t\t");
					count = 0;
				}

			}
			printf("\n");
			//printf("\t\t\t(VolId: %lld ", 
			//				fileHandle.volId.whole_thing);
			//printf("Inode: %lld) \n", fileHandle.inum);
			return (NFS_V2_FHANDLE_LEN);

	}
	return (NFX_ERR);
}

/*-----------------------------------------------------------------
 * Name:      	nfs_decodeString 
 *
 * Description:
 *      Decode string name from input buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodeString(char *nfs_msgp)
{
	int32   	length;
	char    	name[FS_PATH_MAX];
	ushort16	nameLen;

	memset (name, 0, FS_PATH_MAX);
	/*
	 *Extract the filename
	 */
	length = nfs_extractString(nfs_msgp, name, &nameLen, FS_PATH_MAX);
	if (length < 0)
	{
		printf("Invalid name\n");
		return (NFX_ERR);
	}
	printf("\t%s\n", name);

	return (length);
}



/*-----------------------------------------------------------------
 * Name:       nfs_decodeName
 *
 * Description:
 *      Decode string name from input buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodeName(char *nfs_msgp, uint32 type, int32 direction)
{
	int32   	length;
	char    	name[FS_PATH_MAX];
	ushort16	nameLen;
	switch (type)
	{
		case NFS_DECODE_FILENAME:
			printf("\tFile Name = ");
			break;
		case NFS_DECODE_PATHNAME:
			printf("\n\tPath Name = ");
			break;
		case NFS_DECODE_EXPORTINFO:
			printf("\tExportinfo = ");
			break;
		case NFS_DECODE_HOSTNAME:
			printf("\tHost Name = ");
			break;
		default:
			printf("\tName = ");
			break;

	}
	memset (name, 0, FS_PATH_MAX);
	/*
	 *Extract name
	 */
	if (direction == NFS_DECODE_RX)
		length = nfs_extractString(nfs_msgp, name, &nameLen, FS_PATH_MAX);
	else
		length = nfs_txDecodeString(nfs_msgp, name, &nameLen);
	if (length < 0)
	{
		printf("Invalid name\n");
		return (NFX_ERR);
	}
	printf("\t%s\n", name);

	return (length);
}

/*-----------------------------------------------------------------
 * Name:      nfs_decodeCreateMode 
 *
 * Description:
 *      Decode Create file mode (Unchecked, Guarded, or Exclusive) 
 *	from input buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *field length = number of byte counts
	 	 */
nfs_decodeCreateMode(char *nfs_msgp, uint32 *mode)
{
	int32	length;

	printf("\tMethod = ");
	length = nfs_extractLongVal(nfs_msgp, mode);	
  
	switch (*mode)
	{
		case FS_CREATE_UNCHECKED:
			printf("Unchecked\n");
			break;
		case FS_CREATE_EXCLUSIVE:
			printf("Exclusive\n");
			break;
		case FS_CREATE_GUARDED:
			printf("Guarded\n");
			break;
	}
	return (length);

}

/*-----------------------------------------------------------------
 * Name:     nfs_decodeEof 
 *
 * Description:
 *  Decodes end of file (eof) flag. 
 *
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 * 
 *-----------------------------------------------------------------
 */
int32
nfs_decodeEof(char *nfs_msgp)
{
	uint32		eof;
	int32		length;

	printf("\n\tEOF = ");
	length = nfs_txExtractLongVal(nfs_msgp, &eof);	
	printf("%s\n", (eof == 0)? "False": "True");	

	return(length);
}

/*-----------------------------------------------------------------
 * Name:    	nfs_decodeReaddirCookie 
 *
 * Description:
 *  Decodes Readdir cookie field.   
 *
 * Created by:  Joseph Vu
 *  
 * Date Created:        5/28/01
 *  
 *-----------------------------------------------------------------
 */
int32
nfs_decodeReaddirCookie(char *nfs_msgp, uint32 nfs_version)
{
	uint64	cookie;
	uint64	cookieVerif;
	int32	length;

	printf("\n\tCookie = ");
	length  = nfs_extractReaddirCookie(nfs_version, nfs_msgp, 
								&cookie, &cookieVerif);
	printf("%lld\n", cookie);
	if (nfs_version == NFS_VER3)
	{
		printf("\n\tVerifier = ");
		printf("%lld\n", cookieVerif);
	}
	return (length);
}

/*-----------------------------------------------------------------
 * Name:       nfs_docodeReaddirCount 
 *
 * Description:
 *  Decodes Readdir Count field.                       
 *
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 * 
 *-----------------------------------------------------------------
 */
int32
nfs_docodeReaddirCount(char *nfs_msgp, uint32 countType)
{
	uint32		byteCount;
	int32		length;

	if (countType == NFS_DECODE_DIRCOUNT)
		printf("\n\tDirectory Count = ");
	else		
		printf("\n\tMax Count = ");
	length = nfs_extractLongVal(nfs_msgp, &byteCount);
	printf("%d\n", byteCount);

	return (sizeof(uint32));
}

/*-----------------------------------------------------------------
 * Name:      nfs_decodeFileBeginOffset 
 *
 * Description:
 *  Decodes NFS Read "Begin Offset" field. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeFileBeginOffset(char *nfs_msgp)
{
	uint32	offset32;

	printf("\n\tBegin Offset = ");
	nfs_extractLongVal(nfs_msgp, &offset32);
	printf("%d\n", offset32);
	return (sizeof(uint32));
}
/*-----------------------------------------------------------------
 * Name:     nfs_decodeFileOffset 
 *
 * Description:
 *  Decodes NFS Read "file Offset" field.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeFileOffset(char *nfs_msgp, uint32 nfs_version)
{
	uint32	offset32;
	uint64	offset64;
	
	printf("\n\tOffset = ");

	if (nfs_version == NFS_VER2)
	{
		nfs_extractLongVal(nfs_msgp, &offset32);
		printf("%d\n", offset32);
		return (sizeof(uint32));
	}
	else
	{
		nfs_extractLongLongVal(nfs_msgp, &offset64);
		printf("%lld\n", offset64);
		return (sizeof(uint64));
	}
	
}

/*-----------------------------------------------------------------
 * Name:    nfs_decodeVerifier 
 *
 * Description:
 *  Decodes NFS "verifier" field.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeVerifier(char *nfs_msgp, int32 direction)
{
	uint64		verifier;

	printf("\n\tVerifier = ");
	if (direction == NFS_DECODE_RX)
		nfs_extractLongLongVal(nfs_msgp, &verifier);
	else
		nfs_txExtractLongLongVal(nfs_msgp, &verifier);
	printf("%lld\n", verifier);
	return (sizeof(uint64));

}


/*-----------------------------------------------------------------
 * Name:    nfs_decodeCountField 
 *
 * Description:
 *  Decodes NFS "count" field.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeCountField(char *nfs_msgp, int32 direction)
{
	uint32	count;

	printf("\n\tCount = ");
	if (direction == NFS_DECODE_RX)
		nfs_extractLongVal(nfs_msgp, &count);
	else
		nfs_txExtractLongVal(nfs_msgp, &count);
	printf("%d\n", count);
	return (sizeof(uint32));

}


/*-----------------------------------------------------------------
 * Name:   	nfs_decodeStableHowField 
 *
 * Description:
 *  Decodes NFS WRITE "Stable How" field.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeStableHowField(char *nfs_msgp)
{
	uint32 	stableHow;

	printf("\n\tStable = ");
	
	nfs_extractLongVal(nfs_msgp, &stableHow);

	switch (stableHow)
	{
		case NFS_WRITE_UNSTABLE:
			printf("%s \n", "Unstable");
			break;
		case NFS_WRITE_DATASYNC:
			printf("%s \n", "Data Sync");
			break;
		case NFS_WRITE_FILESYNC:
			printf("%s \n", "File Sync");
			break;
	}

	return (sizeof(uint32));
}


void nfs_decodeDevNum(char *nfs_msgp)
{
	fs_DevNum	*devnump;

	devnump = (fs_DevNum *)nfs_msgp;

	printf("\n\tDevnum.major = %d", devnump->major);
	printf("\n\tDevnum.minor = %d\n", devnump->minor);
}

/*-----------------------------------------------------------------
 * Name:       nfs_decodeFsinfo 
 *
 * Description:
 *      Decode NFS FSINFO items.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeFsinfo(char *nfs_msgp)
{
	uint64		value64;
	uint32		value32;
	int32		length;

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tRead request maximum size(rtmax) = %d\n",value32);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tRead request preferred size(rtpref) = %d\n",value32);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tRead request suggested size(rtmult) = %d\n",value32);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tWRITE request maximum size (wtmax)= %d\n",value32);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tWRITE request preferred size (wtpref) = %d\n",value32);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tWrite request suggest size (wtmult) = %d\n",value32);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tREADDIR request preferred size (dtpref) = %d\n",value32);

	length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
	nfs_msgp += length;
	printf("\n\tmaxfilesize = %lld\n",value64);

	length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
	nfs_msgp += length;
	printf("\n\tTime_delta = %lld\n",value64);

	length = nfs_txExtractLongVal(nfs_msgp, &value32);
	nfs_msgp += length;
	printf("\n\tProperties= %d\n",value32);

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:      	nfs_decodeStatInfo 
 *
 * Description:
 *      Decode NFS STATFS items.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeStatInfo(char *nfs_msgp, uint32 nfs_version)
{
	uint64		value64;
	uint32		value32;
	int32		length;


	switch (nfs_version)
	{
		case NFS_VER2:
		{
			length = nfs_txExtractLongVal(nfs_msgp, &value32);
			nfs_msgp += length;
			printf("\n\tOptimum transfer size (bytes) = %d\n",value32);

			length = nfs_txExtractLongVal(nfs_msgp, &value32);
			nfs_msgp += length;
			printf("\n\tBlock size (bsize) = %d\n",value32);

			length = nfs_txExtractLongVal(nfs_msgp, &value32);
			nfs_msgp += length;
			printf("\n\tTotal bsize blocks = %d\n",value32);

			length = nfs_txExtractLongVal(nfs_msgp, &value32);
			nfs_msgp += length;
			printf("\n\tFree bsize blocks = %d\n",value32);

			length = nfs_txExtractLongVal(nfs_msgp, &value32);
			nfs_msgp += length;
			printf("\n\tAvailable bsize blocks = %d\n",value32);
			break;
		}
		case NFS_VER3:
		{
			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;
			printf("\n\tTotal space (bytes) = %lld\n",value64);

			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;
			printf("\n\tAvailable space = %lld\n",value64);

			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;
			printf("\n\tAmount of free space = %lld\n",value64);

			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;
			printf("\n\tTotal file slots = %lld\n",value64);

			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;

			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;
			printf("\n\tTotal Free file slots = %lld\n",value64);

			length = nfs_txExtractLongLongVal(nfs_msgp, &value64);
			nfs_msgp += length;
			printf("\n\tAvailable free file slots = %lld\n",value64);

			length = nfs_txExtractLongVal(nfs_msgp, &value32);
			nfs_msgp += length;
			printf("\n\tInvariant time = %d\n",value32);
						
			break;
		}

	}
	return (NFX_OK);
}
/*-----------------------------------------------------------------
 * Name:       nfs_decodeAccessBits 
 *
 * Description:
 *      Decode access bits from input buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *length = num of bytes of length field
	 	 */
nfs_decodeAccessBits(char *nfs_msgp, int32 direction)
{
	int32		accessbits;
	int32		length;

	printf("\n\tAccess bits = ");

	if (direction == NFS_DECODE_RX)
		length = nfs_extractLongVal(nfs_msgp, &accessbits);
	else
		length = nfs_txExtractLongVal(nfs_msgp, &accessbits);

	nfs_msgp += length;

	printf("%08x\n", accessbits);

	return (sizeof(uint32));
	
}

/*-----------------------------------------------------------------
 * Name:      nfs_decodePreOpAttributes 
 *
 * Description:
 *      Decode pre-operation atrributes from input buffer.  
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodePreOpAttributes(char *msgp)
{
	nfs_preOpAttr_t	*preOpAttrp;
	char			*destMsgp;
	int32			length;
	uint32			presentFlag;
	uint64			faSize;	

	destMsgp = msgp;
	length = nfs_decodePresentFlag (destMsgp, &presentFlag);
	destMsgp += length;
	if (!presentFlag)
		/*
		 *No pre-op attributes present
		 */
		return (length);
	printf("\n\tPre-operation attributes: \n");
	preOpAttrp = (nfs_preOpAttr_t *)destMsgp;
	nfs_txExtractLongLongVal((char *)&preOpAttrp->faSize,
					&faSize); 
	printf("\tFile size: %lld\n", faSize);	
	printf("\tModification time: %d.%d\n",preOpAttrp->faMtime.secs,
			preOpAttrp->faMtime.nsecs);
	printf("\tAttribute modification time: %d.%d\n", 
			preOpAttrp->faCtime.secs, preOpAttrp->faCtime.nsecs);
	length += sizeof(nfs_preOpAttr_t);	
	return (length);
}

/*-----------------------------------------------------------------
 * Name:     nfs_decodeAttributes 
 *
 * Description:
 *      Decode post-operation atrributes from input buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *length = number of bytes of the attributes field
	 	 */
nfs_decodeAttributes(char *msgp, int32 nfs_version, uint32 reqType)
{
	int32		*msg32p;
//	nfs_fattr_t	*fattrp;
	char		*destMsgp;
	int32		length;
	uint32		presentFlag;

	length = 0;
	destMsgp = msgp;
	if ( ((nfs_version == NFS_VER2) && (reqType == NFS_READDIR)) ||
		( (nfs_version == NFS_VER3) && (reqType != NFS_GETATTR) ) )
	{
		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		if (!presentFlag)
			/*
			 *return if no attributes is available
	 		 */
			return (length);
		destMsgp += length;
	}	
	printf("\n\tPost-operation attributes: \n");
#ifdef	NOTUSED
	fattrp = (nfs_fattr_t *)destMsgp;
			
	printf("\tFile Type: %d %s\n", 
			fattrp->faType, nfs_getFtypeStr(fattrp->faType));
	printf("\tMode: %03o\n", fattrp->faMode);
	printf("\tLink count = %d\n", fattrp->faLink);
	printf("\tUID = %d, GID: %d\n", 
			fattrp->faUid, fattrp->faGid);
#endif
	msg32p = (int32 *)destMsgp;
	printf("\tFile Type: %d %s\n", 
			*msg32p, nfs_getFtypeStr(*msg32p));
	msg32p++;
	printf("\tMode: %03o\n", *msg32p++);
	printf("\tLink count = %d\n", *msg32p++);
	//printf("\tUID = %d, GID: %d\n",
	//			*msg32p++, *msg32p++);
	printf("\tUID = %d\n", *msg32p++);
	printf("\tGID = %d\n", *msg32p++);

	length += 20;
	switch (nfs_version)
	{
		case	NFS_VER2:
		{
			nfs_v2SpecificFattr_t		*v2SpecificFattrp;

			//v2SpecificFattrp = &fattrp->version.v2Fattr;
			v2SpecificFattrp = (nfs_v2SpecificFattr_t *)msg32p;
			printf("\tFile size: %d, Block size: %d, Device no.: %d, No. of blocks: %d\n",
				v2SpecificFattrp->faSize, v2SpecificFattrp->faBlocksize,
				v2SpecificFattrp->faRdev, v2SpecificFattrp->faBlocks);
			printf("\tFile system id: %d\n", v2SpecificFattrp->faFsid);
			printf("\tFile id: %d\n", v2SpecificFattrp->faFileid);
			printf("\tAccess time: %d.%d\n", 
					v2SpecificFattrp->faAtime.secs,
					v2SpecificFattrp->faAtime.usecs);
			printf("\tModification time: %d.%d\n", 
					v2SpecificFattrp->faMtime.secs, 
					v2SpecificFattrp->faMtime.usecs);
			printf("\tAttribute modification time: %d.%d\n", 
					v2SpecificFattrp->faCtime.secs,
					v2SpecificFattrp->faCtime.usecs);
			length += sizeof(nfs_v2SpecificFattr_t);
			break;
		}
		case	NFS_VER3:
		{
			nfs_v3SpecificFattr_t		*v3SpecificFattrp;
			uint64						faSize;	
			uint64						faUsed;
			uint64						faFsid, faFileid;

			//v3SpecificFattrp = &fattrp->version.v3Fattr;
			v3SpecificFattrp = (nfs_v3SpecificFattr_t *)msg32p;
			nfs_txExtractLongLongVal((char *)&v3SpecificFattrp->faSize,
					&faSize); 
			printf("\tFile size: %lld\n",faSize);
			nfs_txExtractLongLongVal((char *)&v3SpecificFattrp->faUsed,
					&faUsed); 
			printf("\tDisk space used: %lld\n", faUsed);
			printf("\tSpecial: Major: %d, Minor: %d\n",
				v3SpecificFattrp->faRdev.major, 
				v3SpecificFattrp->faRdev.minor);
			nfs_txExtractLongLongVal((char *)&v3SpecificFattrp->faFsid,
					&faFsid); 
			nfs_txExtractLongLongVal((char *)&v3SpecificFattrp->faFileid,
					&faFileid); 
			printf("\tFile system id: %lld\n", faFsid);
			printf("\tFile id: %lld\n",faFileid);
			printf("\tAccess time: %d.%d\n", 
				v3SpecificFattrp->faAtime.secs, 
				v3SpecificFattrp->faAtime.nsecs);
			printf("\tModification time: %d.%d\n", 
				v3SpecificFattrp->faAtime.secs,
				v3SpecificFattrp->faAtime.nsecs);
			printf("\tAttribute modification time: %d.%d\n", 
				v3SpecificFattrp->faCtime.secs,
				v3SpecificFattrp->faCtime.nsecs);
			length += sizeof(nfs_v3SpecificFattr_t);
			break;	
		}
	} 

	return (length);
}

/*-----------------------------------------------------------------
 * Name:      nfs_decodeSetAttr 
 *
 * Description:
 *      Decode received Setattr field.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_decodeSetAttr(char *nfs_msgp, int32 nfs_version, uint32 procNum)
{
	uint32		atimeType;
	uint32		mtimeType;
	int32		length;
	fs_Attr		fsAttr;
	fs_Attr		*fsAttrp;
	uint32		*msgp;

	length = 0;
	printf("\tSet Attributes:\n");
	fsAttrp = &fsAttr;
	length = nfs_extractSetattr(nfs_version, 
					nfs_msgp, fsAttrp, 
					&atimeType, &mtimeType);
	printf("\tMode: ");
	if (fsAttrp->mode == -1)
		printf("%s\n", noValueStr[0]);
	else
		printf("%03o\n",fsAttrp->mode);
	
	printf("\tUID: ");
	if (fsAttrp->uid == -1)
		printf("%s\n", noValueStr[0]);
	else
		printf("%d\n", fsAttrp->uid);

	printf("\tGID: ");
	if (fsAttrp->gid == -1)
		printf("%s\n", noValueStr[0]);
	else
		printf("%d\n",fsAttrp->gid);

	printf("\tFile size: ");
	if (fsAttrp->size == -1)
		printf("%s\n", noValueStr[0]);
	else
		printf("%lld\n",fsAttrp->size);				

	switch (nfs_version)
	{
		case NFS_VER2:
		{
			printf("\tAccess time: %d.%d\n", 
				fsAttrp->atime.secs, fsAttrp->atime.nsecs);
 
			printf("\tModification time: %d.%d\n", 
				fsAttrp->mtime.secs, fsAttrp->atime.nsecs);
			return (sizeof (nfs_v2Sattr_t));	
		}			

		case NFS_VER3:
		{
			printf("\tAccess time type: %s\n", 
					nfs_decodeGetTimeTypeStr(atimeType));
			if (atimeType == NFS_SATTR_CLIENT_TIME)
			{
				printf("\tAccess time: %d.%d\n", 
					fsAttrp->atime.secs, fsAttrp->atime.nsecs);
			}
 
			printf("\tModification time type: %s\n", 
						nfs_decodeGetTimeTypeStr(mtimeType)); 
			if (mtimeType == NFS_SATTR_CLIENT_TIME)
			{
				printf("\tModification time: %d.%d\n", 
						fsAttrp->mtime.secs, fsAttrp->mtime.nsecs);
			}
		
			if (procNum != NFS_V3_PROC_SETATTR)
				return (length);	
			/*	
			 *Decode guard field
			 */
			printf("\tGuard: ");
			msgp = (uint32 *)(nfs_msgp + length);
			length += sizeof(uint32);
			if (*msgp)
			{
				printf("\tAttributes change time: %d.%d\n", 
					fsAttrp->ctime.secs, fsAttrp->ctime.nsecs);
				length += sizeof(fs_Time);
			}
			else
			{
				printf("%s\n", noValueStr[0]);
			}
			return (sizeof(nfs_v3Sattr_t));	
		}
	}
	return (NFX_ERR);
}

/*-----------------------------------------------------------------
 * Name:      nfs_decodeSetAttr
 *
 * Description:
 *      Decode status field.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
char *
nfs_getStatus(uint32 replyStatus)
{
	switch (replyStatus)
	{

        case NFS_OK:
		return ("(OK)");
        case NFSERR_PERM:		
		return ("(Not Owner. No permission)"); 
        case NFSERR_NOENT:
		return ("(No such file in directory)");
        case NFSERR_IO:
		return("(Hard error occured-like disk error)");
        case NFSERR_NXIO:
		return ("(No such device or address)");
        case NFSERR_ACCES:
		return("(Permission denied)");
        case NFSERR_EXIST: 
		return ("(File already exist)");
        case NFSERR_XDEV:
		return ("(Version 3 only. Attempt to do cross-device hard link)");
        case NFSERR_NODEV:
		return ("(No such device)");
        case NFSERR_NOTDIR:
		return ("(Is not directory)");
        case NFSERR_ISDIR:
		return ("(is a directory)");
        case NFSERR_INVAL:
		return ("(Version 3 only.  Invalid argument)");
        case NFSERR_FBIG:
		return ("(File too large)");
        case NFSERR_NOSPC:
		return ("(No space left)");
        case NFSERR_ROFS:
		return ("Read-only file system");
        case NFSERR_MLINK:
		return ("Version 3 only.  Too many hard links");
        case NFSERR_NAMETOL:
		return ("Filename was too long");
        case NFSERR_NOTEMPTY:
		return("Remove a dir that was not empty");
        case NFSERR_DQUOT:
		return ("Resource (quota) hard limit exceeded");
        case NFSERR_STALE:
		return ("Invalid filehandle");
        case NFSERR_REMOTE:
		return("Version 3 only.Too many levels of remote in path");
        case NFSERR_BADHANDLE:
		return("Illegal NFS filehandle");
        case NFSERR_NOT_SYNC:
		return("Update synch mismatch was detected during a SETATTR");
        case NFSERR_BAD_COOKIE:
		return("READDIR/READDIRPLUS cookie is stale");
        case NFSERR_NOTSUPP:
		return("Operation is not supported");
        case NFSERR_TOOSMALL:
		return ("Buffer or request is too small");
        case NFSERR_SERVERFAULT:
		return("A generic error-default error");
        case NFSERR_BADTYPE:
		return("Object type not supported");


	}

	return ("Invalid Status");

}

/*-----------------------------------------------------------------
 * Name:     nfs_rxDecodeFsinfo 
 *
 * Description:
 *      Decode received FSINFO.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeFsinfo(rpcCallHdr_t *rpcCallp, char *nfs_msgp)
{
	/*
	 *Decode filehandle
	 */
	nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);

}

/*-----------------------------------------------------------------
 * Name:    	nfs_rxDecodeRead 
 *
 * Description:
 *      Decode received READ procedure. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeRead(rpcCallHdr_t *rpcCallp, char *nfs_msgp)
{
	int32	length;

	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	if (rpcCallp->rpc_proc == NFS_V3_PROC_PATHCONF)
		return;
	length = nfs_decodeFileOffset(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	length = nfs_decodeCountField(nfs_msgp, NFS_DECODE_RX);
	nfs_msgp += length;

}
/*-----------------------------------------------------------------
 * Name:        nfs_rxDecodeWrite
 *
 * Description:
 *      Decode received WRITE procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeWrite(rpcCallHdr_t *rpcCallp, char *nfs_msgp)
{
	int32	length;

	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	if (rpcCallp->progVer == NFS_VER2)
	{
		length = nfs_decodeFileBeginOffset(nfs_msgp);
		nfs_msgp += length;
	}
	length = nfs_decodeFileOffset(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	length = nfs_decodeCountField(nfs_msgp, NFS_DECODE_RX);
	nfs_msgp += length;

	if (rpcCallp->progVer == NFS_VER3)
	{
		length = nfs_decodeStableHowField(nfs_msgp);
	}
}
/*-----------------------------------------------------------------
 * Name:        nfs_rxDecodeWrite
 *
 * Description:
 *      Decode received STATFS/FSSTAT procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeStat(rpcCallHdr_t *rpcCallp, char *nfs_msgp)
{
	/*
	 *Decode filehandle
	 */
	nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);

}

/*-----------------------------------------------------------------
 * Name:       nfs_rxDecodeGetAttr 
 *
 * Description:
 *      Decode received NFS Getattr Call procedure. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeGetAttr(rpcCallHdr_t *rpcCallp, char *nfs_msgp)
{

	/*
	 *Decode filehandle
	 */
	nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);

}

/*-----------------------------------------------------------------
 * Name:      nfs_rxDecodeSetAttr 
 *
 * Description:
 * Decode received NFS Setattr Call procedure.     
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeSetAttr(rpcCallHdr_t *rpcCallp, char *nfs_msgp)
{
	int32			length;

	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);

	nfs_msgp += length;

	/*
	 *Decode SetAttr
	 */
	length = nfs_decodeSetAttr(nfs_msgp, rpcCallp->progVer, 
							rpcCallp->rpc_proc);

}

/*-----------------------------------------------------------------
 * Name:     	nfs_rxDecodeLookup 
 *
 * Description:
 *      Decode reveived NFS Lookup Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeLookup(rpcCallHdr_t *rpcCallp, char *msgp)
{
        int32                   length;
	char			*nfs_msgp;

	nfs_msgp = msgp;

	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);

	nfs_msgp += length;
	/*
	 *Decode filename
	 */
	length = nfs_decodeName(nfs_msgp, NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: Failure to decode filename!\n", __FUNCTION__);
	}
}

/*-----------------------------------------------------------------
 * Name:       nfs_rxDecodeReadlink 
 *
 * Description:
 *      Decode received NFS Readlink Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeReadlink(rpcCallHdr_t *rpcCallp, char *msgp)
{
	char                    *nfs_msgp;

	nfs_msgp = msgp;
	/*
	 *Decode filehandle
	 */
	nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);

}
/*-----------------------------------------------------------------
 * Name:      	nfs_rxDecodeSymlink 
 *
 * Description:
 *      Decode received NFS Symlink Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeSymlink (rpcCallHdr_t *rpcCallp, char *msgp)
{
	char			*nfs_msgp;
	int32			length;
	uint32			nfs_version;

	nfs_msgp = msgp; 
	nfs_version = rpcCallp->progVer;
	length = 0;
	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, nfs_version);
	nfs_msgp += length;

	/*
	 *Decode filename
	 */
	length = nfs_decodeName(nfs_msgp, NFS_DECODE_FILENAME, 
							NFS_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: decode filename failed!\n", __FUNCTION__);
		return;
	}
	nfs_msgp += length;

	switch (nfs_version)
	{
		case NFS_VER2:
		
			/*
			 *decode symbolic link string data
			 */
			length = nfs_decodeName(nfs_msgp, 
								NFS_DECODE_PATHNAME, NFS_DECODE_RX);
			if (length == NFX_ERR)
			{
				printf("%s: decode symlink string data failed!\n",
							   	__FUNCTION__);
				return;
			}
			nfs_msgp += length;
			/*
			 *decode set attributes
			 */
			length = nfs_decodeSetAttr(nfs_msgp, nfs_version, 
									rpcCallp->rpc_proc);
			break;

		case NFS_VER3:
			/*
			 *decode set attributes
			 */
			length = nfs_decodeSetAttr(nfs_msgp, nfs_version, 
									rpcCallp->rpc_proc);
			nfs_msgp += length;

			/*
			 *decode symbolic link string data
			 */
			nfs_decodeName(nfs_msgp, 
						NFS_DECODE_PATHNAME, NFS_DECODE_RX);
			if (length == NFX_ERR)
			{
				printf("%s: decode symlink string data failed!\n",
							   	__FUNCTION__);
				return;
			}
			break;
	}

}
/*-----------------------------------------------------------------
 * Name:        nfs_rxDecodeAccess
 *
 * Description:
 *      Decode received NFS Access Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeAccess(rpcCallHdr_t *rpcCallp, char *msgp)
{
	char			*nfs_msgp;
	int32			length;

	nfs_msgp = msgp;

	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;
	/*
	 *Decode access bits
	 */
	nfs_decodeAccessBits(nfs_msgp, NFS_DECODE_RX);
	return;
}

/*-----------------------------------------------------------------
 * Name:      nfs_rxDecodeCreate 
 *
 * Description:
 *      Decode received NFS Create Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_rxDecodeCreate(rpcCallHdr_t *rpcCallp, char *msgp)
{

	char			*nfs_msgp;
	int32			length;
	uint32			nfs_version;

	nfs_msgp = msgp;
	nfs_version = rpcCallp->progVer;

	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, nfs_version);
	nfs_msgp += length;
	/*
	 *Decode filename
	 */
	length = nfs_decodeName(nfs_msgp, 
					NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: decode filename failed!\n", __FUNCTION__);
		return;
	}
	nfs_msgp += length;

	switch (nfs_version)
	{
		case NFS_VER2:
			/*
			 *decode set attributes
			 */
			length = nfs_decodeSetAttr(nfs_msgp, nfs_version, 
									rpcCallp->rpc_proc);
			break;
		case NFS_VER3:
		{
			uint32		mode;

			length = nfs_decodeCreateMode(nfs_msgp, &mode);
			nfs_msgp += length;
			switch (mode)
			{

				case FS_CREATE_UNCHECKED:
				case FS_CREATE_GUARDED:
					length = nfs_decodeSetAttr(nfs_msgp, 
									nfs_version,
									rpcCallp->rpc_proc);
					break;
				case FS_CREATE_EXCLUSIVE:
					nfs_decodeVerifier(nfs_msgp, NFS_DECODE_RX);
					break;
			}
		}

	}
	return;

}

/*-----------------------------------------------------------------
 * Name:      nfs_rxDecodeMkdir
 *
 * Description:
 *      Decode received NFS MKDIR Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeMkdir (rpcCallHdr_t *rpcCallp, char *msgp)
{
	char                    *nfs_msgp;
	int32                   length;
	uint32					nfs_version;

	nfs_msgp = msgp;
	nfs_version = rpcCallp->progVer;
	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, nfs_version);
	nfs_msgp += length;

	/*
	 *Decode filename
	 */
	length = nfs_decodeName(nfs_msgp, 
					NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: decode filename failed!\n", __FUNCTION__);
		return;
	}
	nfs_msgp += length;

	/*
	 *decode set attributes
	 */
	nfs_decodeSetAttr(nfs_msgp, nfs_version, rpcCallp->rpc_proc);

}

/*-----------------------------------------------------------------
 * Name:     	nfs_rxDecodeRemove 
 *
 * Description:
 *      Decode received NFS Remove/RMDIR Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeRemove(rpcCallHdr_t *rpcCallp, char *msgp)
{
	char                    *nfs_msgp;
	int32					length;

	nfs_msgp = msgp;
	
	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	/*
	 *Decode filename
	 */
	length = nfs_decodeName(nfs_msgp, 
					NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
		printf("%s: decode filename failed!\n", __FUNCTION__);
	
	return;
}

/*-----------------------------------------------------------------
 * Name:       	nfs_rxDecodeRename 
 *
 * Description:
 *      Decode received NFS Rename Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeRename(rpcCallHdr_t *rpcCallp, char *msgp)
{
	char                    *nfs_msgp;
	int32					length;

	nfs_msgp = msgp;
	
	/*
	 *Decode fromdir filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	/*
	 *Decode fromname
	 */
	length = nfs_decodeName(nfs_msgp, 
						NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: decode fromName failed!\n", __FUNCTION__);
		return;
	}
	nfs_msgp += length;

	/*
	 *Decode todir filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, rpcCallp->progVer);
	nfs_msgp += length;

	/*
	 *Decode toname
	 */
	length = nfs_decodeName(nfs_msgp, 
				NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: decode toName failed!\n", __FUNCTION__);
		return;
	}
	nfs_msgp += length;

	return;
}

/*-----------------------------------------------------------------
 * Name:       	nfs_rxDecodeReaddir( 
 *
 * Description:
 * Decodes received NFS READDIR Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeReaddir(rpcCallHdr_t *rpcCallp, char *msgp)
{
	char			*nfs_msgp;
	int32			length;
	uint32			nfs_version;

	nfs_msgp = msgp;
	nfs_version = rpcCallp->progVer;	
	/*
	 *Decode filehandle
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, nfs_version);
	nfs_msgp += length;

	/*
	 *Decode cookie/verifier field
	 */
	length = nfs_decodeReaddirCookie(nfs_msgp, nfs_version);
	nfs_msgp += length;

	if ((nfs_version == NFS_VER3) && 
		(rpcCallp->rpc_proc == NFS_V3_PROC_READDIRPLUS ) )
	{
		length = nfs_docodeReaddirCount(nfs_msgp,NFS_DECODE_DIRCOUNT);
		nfs_msgp += length;
	}
	length = nfs_docodeReaddirCount(nfs_msgp,NFS_DECODE_MAXCOUNT);

}

/*-----------------------------------------------------------------
 * Name:	nfs_rxDecodeHardlink 
 *
 * Description:
 * Decodes received NFS LINK Call procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_rxDecodeHardlink(rpcCallHdr_t *rpcCallp, char *msgp)
{
	char                    *nfs_msgp;
	int32                   length;
	uint32                  nfs_version;


	nfs_msgp = msgp;
	nfs_version = rpcCallp->progVer;
	/*
	 *Decode filehandle for the existing filesystem object
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, nfs_version);
	nfs_msgp += length;

	/*
	 *Decode filehandle for the directory in 
	 *which the link is to be created
	 */
	length = nfs_rxDecodeFhandle(nfs_msgp, nfs_version);
	nfs_msgp += length;

	/*
	 *Decode fromname
	 */
	length = nfs_decodeName(nfs_msgp, 
					NFS_DECODE_FILENAME, NFS_DECODE_RX);
	if (length == NFX_ERR)
		printf("%s: decode fromname failed!\n", __FUNCTION__);	

	return;
}


/*-----------------------------------------------------------------
 * Name:      	nfs_rxDecodeV2Pkt 
 *
 * Description:
 *      Decode NFS Version 2 received packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_rxDecodeV2Pkt(pkt_desc_t *descp)
{
	char                    *msgp;
	rpcCallHdr_t			*rpcCallp;
	uint32                  rpcHdrSize;
//	int32					length;

	rpcCallp = (rpcCallHdr_t *)descp->pHdr;
	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;


	if ((rpcCallp->rpc_proc < NFS_MIN_PROC) || 
		(rpcCallp->rpc_proc > NFS_MAX_V2PROC) )
		return (NFX_ERR);
	printf("%s",nfs_v2Procs[rpcCallp->rpc_proc]);

	switch (rpcCallp->rpc_proc)
	{
		case NFS_V2_PROC_NULL:		
			/*Do nothing*/
			break;
		case NFS_V2_PROC_GETATTR:	
			/*Get attributes for file/dir*/
			nfs_rxDecodeGetAttr(rpcCallp, msgp);
			break;			
		case NFS_V2_PROC_SETATTR:	
			/*Set attributes for file/dir*/
			nfs_rxDecodeSetAttr(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_ROOT:		
			/* Obsolete */
			break;
		case NFS_V2P_PROC_LOOKUP:	
			/*lookup attributes of a file*/
			nfs_rxDecodeLookup(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_READLINK:	
			/*read a soft link*/
			nfs_rxDecodeReadlink(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_READ:		
			/*read data from a file*/
			nfs_rxDecodeRead(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_WRITECACHE:	
			/* Obsolete */
			break;
		case NFS_V2_PROC_WRITE:		
			/*write data to a file*/
			nfs_rxDecodeWrite(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_CREATE:	
			/*create a new file*/
			nfs_rxDecodeCreate(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_REMOVE:	
			/*remove a file*/
		case NFS_V2_PROC_RMDIR:		
			/*remove a directory*/
			nfs_rxDecodeRemove(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_RENAME:	
			/*rename a file/directory*/
			nfs_rxDecodeRename(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_LINK:		
			/*create a hard link to a file*/
			nfs_rxDecodeHardlink(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_SYMLINK:	
			/*create a soft link*/
			nfs_rxDecodeSymlink(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_MKDIR:		
			/*create a new directory*/
			nfs_rxDecodeMkdir (rpcCallp, msgp);

			break;
		case NFS_V2_PROC_READDIR:	
			/*read a directory contents*/
			nfs_rxDecodeReaddir(rpcCallp, msgp);
			break;
		case NFS_V2_PROC_STATFS:	
			/*get filesystem statistic*/
			break;
	}


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfs_rxDecodeV3Pkt
 *
 * Description:
 *      Decode NFS Version 3 received packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_rxDecodeV3Pkt(pkt_desc_t *descp)
{
	char					*msgp;
	rpcCallHdr_t			*rpcCallp;
	uint32					rpcHdrSize;
//	int32					length;


	rpcCallp = (rpcCallHdr_t *)descp->pHdr;
	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;

	if ((rpcCallp->rpc_proc < NFS_MIN_PROC) || 
		(rpcCallp->rpc_proc > NFS_MAX_V3PROC)  )
		return (NFX_ERR);
	printf("%s",nfs_v3Procs[rpcCallp->rpc_proc]);

	switch (rpcCallp->rpc_proc)
	{

		case NFS_V3_PROC_NULL:		
			/*Do nothing*/
			break;
		case NFS_V3_PROC_GETATTR:	
			/*Get attributes for file/dir*/
			nfs_rxDecodeGetAttr(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_SETATTR:	
			/*Set attributes for file/dir*/
			nfs_rxDecodeSetAttr(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_LOOKUP:	
			/*lookup attributes of a file*/
			nfs_rxDecodeLookup(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_ACCESS:	
			/*check access permission on object*/
			nfs_rxDecodeAccess(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_READLINK:	
			/*read a soft link*/
			nfs_rxDecodeReadlink(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_CREATE:	
			/*create a new file*/
			nfs_rxDecodeCreate(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_MKDIR:		
			/*create a new directory*/
			nfs_rxDecodeMkdir (rpcCallp, msgp);
			break;
		case NFS_V3_PROC_SYMLINK:	
			/*create a soft link*/
			nfs_rxDecodeSymlink(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_MKNOD:		
			/*create a device file*/
			break;		
		case NFS_V3_PROC_REMOVE:	
			/*remove a file*/
		case NFS_V3_PROC_RMDIR:		 
			/*remove a directory*/
			nfs_rxDecodeRemove(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_RENAME:	
			/*rename a file/directory*/
			nfs_rxDecodeRename(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_LINK:		
			/*create a hard link to a file*/
			nfs_rxDecodeHardlink(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_READDIR:	
			/*read a directory contents*/
			nfs_rxDecodeReaddir(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_READDIRPLUS:	
			/*read a directory's contents & attributes*/
			nfs_rxDecodeReaddir(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_FSSTAT:	
			/*get filesystem statistic*/
			nfs_rxDecodeStat(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_FSINFO:	
			/*get statistic filesystem information*/
			nfs_rxDecodeFsinfo(rpcCallp, msgp);
			break;
		case NFS_V3_PROC_PATHCONF:	
			/*get POSIX information*/
		case NFS_V3_PROC_COMMIT:	
			/*commit write data to stable storage*/
		case NFS_V3_PROC_READ:		
			/*read data from a file*/
			nfs_rxDecodeRead(rpcCallp, msgp);

			break;
		case NFS_V3_PROC_WRITE:		
			/*write data to a file*/
			nfs_rxDecodeWrite(rpcCallp, msgp);
			break;

	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:       	nfs_rxDecodeAuthSysHostName 
 *
 * Description:
 *      Decode AUTHSYS HostName.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_rxDecodeAuthSysHostName(rpcCallHdr_t        *rpcCallp)
{
	char		*msgp;
	ushort16	nameLen;
	int32		length;
	uint32		authType;

	msgp = (char *)rpcCallp;

	/*
	 *Skip XID, Call Type, RPC version, Program number, 
	 *Progam version and Procedure number
	 */	
	msgp += 24;


	authType =  *((uint32 *)msgp);
	printf("\tAuthType: ");

	if ( (ntohl(authType) <  RPC_AUTH_NONE) ||
		 ( ntohl(authType) > RPCSEC_GSS) )
		printf("%s\n", "Invalid");
	
	printf("%s",authTypes[ntohl(authType)]);			
	if (ntohl(authType) != PRC_AUTH_SYS)
		return (NFX_OK);			

	/*
	 *For AuthSys: Skip AuthType, AuthLen and Stamp fields
	 */
	msgp += 12;
	memset(hostName, 0, sizeof(hostName));
	length = nfs_extractString(msgp, &hostName[0], &nameLen, FS_NAME_MAX);

	printf("\tHostname: ");
	printf("%s\n", hostName);

	return (NFX_OK);
}
/*-----------------------------------------------------------------
 * Name:       nfs_rxDecodePkt 
 *
 * Description:
 *     	Decode NFS received packet. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_rxDecodePkt(pkt_desc_t *descp)
{
//	char                    *nfs_msgp;
	rpcCallHdr_t		*rpcCallp;

	if (descp == NULL)
	{
		printf("\tNFS Rx: Bad RPC header/pkt!\n");
		return (NFX_ERR);
	}
	rpcCallp = (rpcCallHdr_t *)descp->pHdr;
	if (rpcCallp == NULL) 
	{
		printf("\tNFS Rx: Bad RPC header/pkt!\n");
		return (NFX_ERR);
	}		
	if (nfs_tplRcvCheckNFSPkt(rpcCallp) != NFX_OK)
	{
		printf("\tNFS Rx: Bad RPC header/pkt!\n");
		return (NFX_ERR);
	}

	printf("\n\t----- RPC Header -----\n");

//	printf("\tXID: %d \n", rpcCallp->rpc_xid);
	printf("\tXID: 0x%08x\n", rpcCallp->rpc_xid);
	printf("\tDirection: %s\n", "Call");
	printf("\tRPC Version: %d \n", rpcCallp->rpc_rpcVer);
	printf("\tProgram number: %d (%s)\n", 
		rpcCallp->rpc_prog, nfs_decodeGetProgNumStr(rpcCallp->rpc_prog ));
	printf("\tProgram Version: %s\n", (rpcCallp->progVer == NFS_VER2)? "V2" : "V3");
	printf("\tProcedure number: %d \n", rpcCallp->rpc_proc);

	nfs_rxDecodeAuthSysHostName(rpcCallp);

	printf("\t----- NFS packet -----\n");

	switch (rpcCallp->progVer)
	{
		case NFS_VER2:
			nfs_rxDecodeV2Pkt(descp);	
			break;
		case NFS_VER3:
			nfs_rxDecodeV3Pkt(descp);
			break;
	}

	return (NFX_OK);

}


/*-----------------------------------------------------------------
 * Name:       nfs_xmtDecodeGetAttrResp
 *
 * Description:
 *      Decode NFS GetAttr reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeGetAttrResp(
	nfs_request_t *nfsReqp, 	/*reqCB*/
	char *msgp			/*Reply message pointer after RCP reply hdr*/
)
{

	nfs_decodeAttributes(msgp, nfsReqp->rpcCallHdrp->progVer, 
						nfsReqp->reqType);
}

/*-----------------------------------------------------------------
 * Name:      	nfs_xmtDecodeSetAttr 
 *
 * Description:
 *      Decode NFS SetAttr reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeSetAttr(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	uint32		nfs_version;
	int32		length;
	char		*destMsgp;


	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	destMsgp = msgp;

	if (nfs_version == NFS_VER3)
	{
		/*
		 *Decode object pre-op attributes
		 */
		length = nfs_decodePreOpAttributes(destMsgp);
		destMsgp += length;
	}	

	/*
	 *Decode object post-op attributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);

}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodeLookup 
 *
 * Description:
 *      Decode NFS Lookup reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeLookup(
	nfs_request_t *nfsReqp, 	/*reqCB*/
	char *msgp,	/*Reply message pointer after RCP reply hdr*/
	int32 status
)
{
	int32		length;
	char		*destMsgp;
	uint32		nfs_version;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	destMsgp = msgp;

	if (status == NFS_OK)
	{
		/*
	 	 *Decode lookup filehandle
	 	 */
		length = nfs_decodeFhandle(destMsgp, nfs_version);
		destMsgp += length;

		/*
	 	 *Decode Lookup attributes
	 	 */
		length= nfs_decodeAttributes(destMsgp, 
				nfs_version, nfsReqp->reqType);
		destMsgp += length;
	}
	if (nfs_version == NFS_VER2)
		return;
	/*
	 *Decode the parent attributes
	 */
	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);

}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodeSymlink 
 *
 * Description:
 *      Decode NFS Symlink reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeSymlink(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	char		*destMsgp;
	uint32		nfs_version;
	int32		length;
	int32		presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	/*
	 *If NFS Version 2, return
	 */
	if (nfs_version == NFS_VER2)
		return;
	destMsgp = msgp;

	/*
	 *Decode object filehandle
	 */
	length = nfs_decodePresentFlag (destMsgp, &presentFlag);
	destMsgp += length;
	length = nfs_decodeFhandle(destMsgp, nfs_version);
	destMsgp += length;

	/*
	 *Decode Object post-op atrributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;

	/*
	 *Decode directory pre-op file attributes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;
	/*
	 *Decode directory post-op file attributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
									nfsReqp->reqType);

}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodeReadlink 
 *
 * Description:
 * Decode NFS Readlink outgoing reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeReadlink(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	char            *destMsgp;
	uint32          nfs_version;
	int32           length;

	destMsgp = msgp;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;

	if (nfs_version == NFS_VER3)
	{
		/*
		 *decode post-op attributes for the symbolic link
		 */
		length = nfs_decodeAttributes(destMsgp, nfs_version, 
									nfsReqp->reqType);
		destMsgp += length;
	}

	/*
	 *Decode symbolic link data
	 */
	length = nfs_decodeName(destMsgp, 
					NFS_DECODE_PATHNAME, NFS_DECODE_XMT);
	if (length == NFX_ERR)
		printf("%s: decode symlink data failed!\n", __FUNCTION__);
	return;
}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodeAccess 
 *
 * Description:
 * Decode NFS Access (Check Access Permission) reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeAccess(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	char		*nfs_msgp;
	int32		length;

	nfs_msgp = msgp;

	/*
	 *Decode post-op attributes
	 */
	length = nfs_decodeAttributes(nfs_msgp, 
				nfsReqp->rpcCallHdrp->progVer, nfsReqp->reqType);
	nfs_msgp += length;
	/*
	 *Decode access bits
	 */
	nfs_decodeAccessBits(nfs_msgp, NFS_DECODE_XMT);
	return;
}

/*-----------------------------------------------------------------
 * Name:       	 nfs_xmtDecodeCreate 
 *
 * Description:
 * Decode NFS CREATE reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeCreate(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	char			*destMsgp;
	int32			length;
	uint32			nfs_version;
	int32			presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	destMsgp = msgp;
	length = 0;

	if (nfs_version == NFS_VER3)
	{
		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		destMsgp += length;
	}

	/*
	 *Decode object filehandle
	 */
	length = nfs_decodeFhandle(destMsgp, nfs_version);
	destMsgp += length;

	/*
	 *Decode object post-op attributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
			nfsReqp->reqType);
	destMsgp += length;

	if (nfs_version == NFS_VER2)
		return;

	/*
	 *Decode pre-op & post-op attributes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;

	length = nfs_decodeAttributes(destMsgp, nfs_version, 
			nfsReqp->reqType);


}

/*-----------------------------------------------------------------
 * Name: 	nfs_xmtDecodeMkdir 
 *
 * Description:
 * Decode NFS MKDIR reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_xmtDecodeMkdir(
	nfs_request_t *nfsReqp,	/*reqCB*/
	char *msgp	/*Reply message pointer after RCP reply hdr*/
)
{
	return (nfs_xmtDecodeCreate(nfsReqp, msgp));

}

/*-----------------------------------------------------------------
 * Name:   	nfs_xmtDecodeReaddir 
 * 
 * Description:
 * Decode NFS READDIR reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeReaddir(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	
	char                    *destMsgp;
	int32                   length;
	uint32					nfs_version;
	uint32					presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	destMsgp = msgp;
	length = 0;

	if (nfs_version == NFS_VER3)
	{
		/*
		 *Decode attribute
		 */
		length = nfs_decodeAttributes(destMsgp, nfs_version, 
									nfsReqp->reqType);
		destMsgp += length;
		length = nfs_decodeVerifier(destMsgp, NFS_DECODE_XMT);
		destMsgp += length;
	}

	length = nfs_decodePresentFlag (destMsgp, &presentFlag);
	destMsgp += length;

	while (presentFlag)
	{
		length = nfs_decodeDirEntry(destMsgp, 
						nfsReqp->reqType, nfs_version);
		if (length == NFX_ERR)
		{
			printf("%s: nfs_decodeDirEntry() failed!\n", __FUNCTION__);
			return;
		}
		destMsgp += length;
		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		destMsgp += length;
	}
	length = nfs_decodeEof(destMsgp);

}

void
nfs_xmtDecodeReaddirPlus(
	nfs_request_t *nfsReqp,         /*reqCB*/
	char *msgp      /*Reply message pointer after RCP reply hdr*/
)
{
	char                    *destMsgp;
	int32                   length;
	uint32					nfs_version;
	uint32					presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	destMsgp = msgp;
	length = 0;

	/*
	 *Decode post-op directory attributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;

	length = nfs_decodeVerifier(destMsgp, NFS_DECODE_XMT);
	destMsgp += length;

	length = nfs_decodePresentFlag (destMsgp, &presentFlag);
	destMsgp += length;

	while (presentFlag)
	{
		length = nfs_decodeDirEntry(destMsgp, 
						nfsReqp->reqType, nfs_version);
		if (length == NFX_ERR)
		{
			printf("%s: nfs_decodeDirEntry() failed!\n", __FUNCTION__);
			return;
		}
		destMsgp += length;
		length = nfs_decodePresentFlag (destMsgp, &presentFlag);
		destMsgp += length;
	}
	length = nfs_decodeEof(destMsgp);

}

/*-----------------------------------------------------------------
 * Name:   	nfs_xmtDecodeRename 
 *
 * Description:
 * Decode NFS RENAME reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeRename(
	nfs_request_t *nfsReqp,	/*reqCB*/
	char *msgp 		/*Reply message pointer after RCP reply hdr*/
)
{
	char		*destMsgp;
	int32		length;
	uint32		nfs_version;
//	uint32		presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	if (nfs_version == NFS_VER2)
		return;
	destMsgp = msgp;
	length = 0;

	/*
	 *Decode pre-op fromdir attrbutes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;

	length= nfs_decodeAttributes(destMsgp, nfs_version, 
							nfsReqp->reqType);
	destMsgp += length;

	if (nfsReqp->rpcCallHdrp->rpc_proc != NFS_V3_PROC_RENAME)
		return;
	
	/*
	 *Decode pre-op todir attrbutes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;

	/*
	 *Decode post-op todir attrbutes
	 */
	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	
}

/*-----------------------------------------------------------------
 * Name:   	nfs_xmtDecodeHardlink 
 *
 * Description:
 * Decode NFS Hard link reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeHardlink(
	nfs_request_t *nfsReqp,         /*request control block (reqCB)*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	char		*destMsgp;
	uint32		nfs_version;
	int32		length;
//	int32		presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;
	/*
	 *If NFS Version 2, return
	 */
	if (nfs_version == NFS_VER2)
		return;
	destMsgp = msgp;

	/*
	 *Decode Object post-op atrributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;

	/*
	 *Decode directory pre-op file attributes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;
	/*
	 *Decode directory post-op file attributes
	 */
	length = nfs_decodeAttributes(destMsgp, nfs_version, 
									nfsReqp->reqType);

}

/*-----------------------------------------------------------------
 * Name:   	nfs_xmtDecodeFsinfo 
 *
 * Description:
 * Decode NFS FSINFO reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeFsinfo(
	nfs_request_t *nfsReqp,	/*request control block (reqCB)*/
	char *msgp	/*Reply message pointer after RCP reply hdr*/
)
{
	char		*destMsgp;
	int32		length;
	uint32		nfs_version;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;

	destMsgp = msgp;
	length = 0;

	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;
	length = nfs_decodeFsinfo(destMsgp);
}

/*-----------------------------------------------------------------
 * Name:   nfs_xmtDecodeFsstat 
 *
 * Description:
 * Decode NFS FSSTAT reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeFsstat(
	nfs_request_t *nfsReqp,         /*request control block (reqCB)*/
	char *msgp	/*Reply message pointer after RCP reply hdr*/
)
{
	char            *destMsgp;
	int32           length;
	uint32			nfs_version;
//	uint32			presentFlag;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;

	destMsgp = msgp;
	length = 0;

	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;

	length = nfs_decodeStatInfo(destMsgp, nfs_version);
}

/*-----------------------------------------------------------------
 * Name:  	nfs_xmtDecodeRead 
 *
 * Description:
 * Decode NFS READ reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeRead (
	nfs_request_t *nfsReqp,         /*request control block (reqCB)*/
	char *msgp		/*Reply message pointer after RCP reply hdr*/
)
{
	char            *destMsgp;
	int32           length;
	uint32			nfs_version;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;

	destMsgp = msgp;
	length = 0;


	/*
	 *Decode post-op file attributes
	 */
	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;

	if (nfs_version == NFS_VER3)
	{
		/*
	 	 *Decode count field
	 	 */
		length = nfs_decodeCountField(destMsgp, NFS_DECODE_XMT);
		destMsgp += length;

		/*
		 *Decode EOF flag
		 */
		length = nfs_decodeEof(destMsgp);
		destMsgp += length;

	}
	
}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodeWrite 
 *
 * Description:
 *      Decode NFS outgoing reply write packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeWrite(
	nfs_request_t *nfsReqp,	/*request control block (reqCB)*/
	char *msgp	/*Reply message pointer after RCP reply hdr*/
)
{
	char            *destMsgp;
	int32           length;
	uint32			nfs_version;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;

	destMsgp = msgp;
	length = 0;

	if (nfs_version == NFS_VER2)
	{
		length= nfs_decodeAttributes(destMsgp, nfs_version, 
									nfsReqp->reqType);
		return;
	}

	/*
	 *Decode pre-op file attributes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;

	/*
	 *Decode post-op file attributes
	 */
	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);
	destMsgp += length;

	/*
	 *Decode count field
	 */
	length = nfs_decodeCountField(destMsgp, NFS_DECODE_XMT);
	destMsgp += length;

	/*
	 *Decode stable field
	 */
	length = nfs_decodeStableHowField(destMsgp);
	destMsgp += length;

	length = nfs_decodeVerifier(destMsgp, NFS_DECODE_XMT);
}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodeCommit 
 *
 * Description:
 *      Decode NFS outgoing reply commit packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeCommit(
	nfs_request_t *nfsReqp,	/*request control block (reqCB)*/
	char *msgp,	/*Reply message pointer after RCP reply hdr*/
	int32 status
)
{
	char            *destMsgp;
	int32           length;
	uint32			nfs_version;

	nfs_version = nfsReqp->rpcCallHdrp->progVer;

	destMsgp = msgp;
	length = 0;

	/*
	 *Decode pre-op file attributes
	 */
	length = nfs_decodePreOpAttributes(destMsgp);
	destMsgp += length;

	/*
	 *Decode post-op file attributes
	 */
	length= nfs_decodeAttributes(destMsgp, nfs_version, 
								nfsReqp->reqType);

	if  ((nfsReqp->rpcCallHdrp->rpc_proc == NFS_V3_PROC_PATHCONF) ||
			(status != NFX_OK) )
		return;
	destMsgp += length;

	length = nfs_decodeVerifier(destMsgp, NFS_DECODE_XMT);
}

/*-----------------------------------------------------------------
 * Name:        nfs_xmtDecodePkt
 *
 * Description:
 *      Decode NFS version 2 outgoing reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeV2Pkt(nfs_request_t *nfsReqp)
{
	pkt_desc_t		*tplXmitPktDescp;
	char			*msgp;
	uint32			*destMsgp;
	uint32			replyStatus;

	if ((nfsReqp->rpcCallHdrp->rpc_proc < NFS_MIN_PROC) || 
				(nfsReqp->rpcCallHdrp->rpc_proc > NFS_MAX_V2PROC) )
		return;
	printf("%s",nfs_v2Procs[nfsReqp->rpcCallHdrp->rpc_proc]);

	tplXmitPktDescp = (pkt_desc_t *)nfsReqp->pktDescp;
	msgp = (char *)tplXmitPktDescp->pHdr;

	msgp += sizeof(rpcReplyHdr_t);

	/*
	 *Decode status
	 */
	destMsgp = (uint32 *)msgp;
	
	replyStatus = *destMsgp;		
	printf("\tStatus: %d %s\n", 
			replyStatus, nfs_getStatus(replyStatus));

	msgp += sizeof(uint32);

	switch (nfsReqp->rpcCallHdrp->rpc_proc)
	{
		case NFS_V2_PROC_NULL:		
			/*Do nothing*/
			break;
		case NFS_V2_PROC_GETATTR:	
			/*Get attributes for file/dir*/
			nfs_xmtDecodeGetAttrResp(nfsReqp, msgp);
			break;			
		case NFS_V2_PROC_SETATTR:	
			/*Set attributes for file/dir*/
			nfs_xmtDecodeSetAttr(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_ROOT:		
			/* Obsolete */
			break;
		case NFS_V2P_PROC_LOOKUP:	
			/*lookup attributes of a file*/
			if ( (replyStatus != NFS_OK) && 
				(nfsReqp->rpcCallHdrp->progVer == NFS_VER2) )
				return;
			nfs_xmtDecodeLookup(nfsReqp, msgp, replyStatus);
			break;
		case NFS_V2_PROC_READLINK:	
			/*read a soft link*/
			nfs_xmtDecodeReadlink(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_READ:		
			/*read data from a file*/
			nfs_xmtDecodeRead(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_WRITECACHE:	
			/* Obsolete */
		case NFS_V2_PROC_WRITE:		
			/*write data to a file*/
			nfs_xmtDecodeWrite(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_CREATE:	
			/*create a new file*/
			nfs_xmtDecodeCreate(nfsReqp, msgp);	
			break;
		case NFS_V2_PROC_REMOVE:	
			/*remove a file*/
		case NFS_V2_PROC_RMDIR:		
			/*remove a directory*/
			/*
			nfs_rxDecodeRemove(nfsReqp);
			*/
			break;
		case NFS_V2_PROC_RENAME:	
			/*rename a file/directory*/
			nfs_xmtDecodeRename(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_LINK:		
			/*create a hard link to a file*/
			nfs_xmtDecodeHardlink(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_SYMLINK:
			/*create a soft link*/
			nfs_xmtDecodeSymlink(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_MKDIR:		
			/*create a new directory*/
			/*
			nfs_rxDecodeMkdir (nfsReqp);
			*/
			nfs_xmtDecodeMkdir(nfsReqp, msgp);
			break;
		case NFS_V2_PROC_READDIR:	
			/*read a directory contents*/
			nfs_xmtDecodeReaddir(nfsReqp, msgp);
		case NFS_V2_PROC_STATFS:	
			/*get filesystem statistic*/
			break;
	}


}

/*-----------------------------------------------------------------
 * Name:        nfs_xmtDecodePkt
 *
 * Description:
 *      Decode NFS version 3 outgoing reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodeV3Pkt(nfs_request_t *nfsReqp)
{
	pkt_desc_t		*tplXmitPktDescp;
	char			*msgp;
	uint32			*destMsgp;
	uint32			replyStatus;

	if ((nfsReqp->rpcCallHdrp->rpc_proc < NFS_MIN_PROC) || 
			(nfsReqp->rpcCallHdrp->rpc_proc > NFS_MAX_V3PROC)  )
		return;
        printf("%s",nfs_v3Procs[nfsReqp->rpcCallHdrp->rpc_proc]);

	tplXmitPktDescp = (pkt_desc_t *)nfsReqp->pktDescp;
	msgp = (char *)tplXmitPktDescp->pHdr;

	msgp += sizeof(rpcReplyHdr_t);
	destMsgp = (uint32 *)msgp;

	/*
	 *Decode status
	 */
	replyStatus = *destMsgp;
	printf("\tStatus: %d - %s\n", 
				replyStatus, nfs_getStatus(replyStatus));
	msgp += sizeof(uint32);

	switch (nfsReqp->rpcCallHdrp->rpc_proc)
	{
		case NFS_V3_PROC_NULL:		
			/*Do nothing*/
			break;
		case NFS_V3_PROC_GETATTR:	
			/*Get attributes for file/dir*/
			nfs_xmtDecodeGetAttrResp(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_SETATTR:	
			/*Set attributes for file/dir*/
			nfs_xmtDecodeSetAttr(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_LOOKUP:	
			/*lookup attributes of a file*/
			nfs_xmtDecodeLookup(nfsReqp, msgp, replyStatus);
			break;
		case NFS_V3_PROC_ACCESS:	
			/*check access permission on object*/
			nfs_xmtDecodeAccess(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_READLINK:	
			/*read a soft link*/
			nfs_xmtDecodeReadlink(nfsReqp, msgp);	
			break;
		case NFS_V3_PROC_READ:		
			/*read data from a file*/
			nfs_xmtDecodeRead(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_WRITE:		
			/*write data to a file*/
			nfs_xmtDecodeWrite(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_CREATE:	
			/*create a new file*/
			nfs_xmtDecodeCreate(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_MKNOD:		
			/*create a device file*/
		case NFS_V3_PROC_MKDIR:		
			/*create a new directory*/
			nfs_xmtDecodeMkdir(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_SYMLINK:	
			/*create a soft link*/
			nfs_xmtDecodeSymlink(nfsReqp, msgp);
			break;
			
		case NFS_V3_PROC_REMOVE:	
			/*remove a file*/
		case NFS_V3_PROC_RMDIR:		 
			/*remove a directory*/
		case NFS_V3_PROC_RENAME:	
			/*rename a file/directory*/
			nfs_xmtDecodeRename(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_LINK:		
			/*create a hard link to a file*/
			nfs_xmtDecodeHardlink(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_READDIR:	
			/*read a directory contents*/
			nfs_xmtDecodeReaddir(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_READDIRPLUS:	
			/*read a directory's contents & attributes*/
			nfs_xmtDecodeReaddirPlus(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_FSSTAT:	
			/*get filesystem statistic*/
			nfs_xmtDecodeFsstat(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_FSINFO:	
			/*get statistic filesystem information*/
			nfs_xmtDecodeFsinfo(nfsReqp, msgp);
			break;
		case NFS_V3_PROC_PATHCONF:	
			/*get POSIX information*/
		case NFS_V3_PROC_COMMIT:	
			/*commit write data to stable storage*/
			nfs_xmtDecodeCommit(nfsReqp, msgp, replyStatus);
			break;
	}

}

/*-----------------------------------------------------------------
 * Name:       	nfs_xmtDecodePkt 
 *
 * Description:
 *      Decode NFS outgoing reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_xmtDecodePkt(nfs_request_t *nfsReqp)
{
	rpcReplyHdr_t	*rpcReplyHdrp;
	pkt_desc_t		*tplXmitPktDescp;
	char			*msgp;
	
	printf("\t----- RPC Header -----\n");

	tplXmitPktDescp = (pkt_desc_t *)nfsReqp->pktDescp;
	msgp = (char *)tplXmitPktDescp->pHdr;
	rpcReplyHdrp = (rpcReplyHdr_t *)tplXmitPktDescp->pHdr;
	printf("\tXID: 0x%08x\n", rpcReplyHdrp->rpc_xid);
	printf("\tDirection: %s\n", (rpcReplyHdrp->rpc_direction == RPC_CALL)? "Call" : "Reply");	

#ifdef	NOTUSED
	/*
	 *TODO: Reply status, Accepted Status
	 */
	switch (rpcReplyHdrp->rpc_status)
	{
		case MSG_ACCEPTED:
		{
			

			break;
		}
		case MSG_DENIED:

		case AUTH_ERROR:
	}
#endif

	printf("\t----- NFS packet -----\n");

	switch (nfsReqp->rpcCallHdrp->progVer)
	{
		case NFS_VER2:
			nfs_xmtDecodeV2Pkt(nfsReqp);
			break;
		case NFS_VER3:
			nfs_xmtDecodeV3Pkt(nfsReqp);
			break;
	}

}

#endif	//NFP_TXRX
