/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-msg.c
 *
 * RCS:  $Id: nfs-msg.c,v 1.95 2002/12/03 22:38:54 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *      Contain NFS and File System interface functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef  NFP_TXRX

#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/nfs-api.h>
#include <exports/mem-api.h>
#include <sys/machine-dep.h>
#include "nfsv2-msg.h"
#include "nfsv3-msg.h"

extern void sibyte_cacheInvalidate(address_t _sa, uint32 _nbytes);

void nfs_msgXmtCmplErr( fs_MsgReadResp *fsMsgp );

/*
#define	NFS_DBG_ON
*/

/*
#define	NFS_USE_SHMEM
*/

//#define	NFS_DBG_PRINTF		if (trace_enable) printf
#define	NFS_DBG_PRINTF		

extern uint32  trace_enable;

extern void mnt_getMountdInfoResp(eee_desc_t *eDescp );

uint64	boottime = 0;			/*TODO:  Get Boot time*/

/*
 *NFS V2 MSG procedures
 */
int32 (*nfsrv2_msgResp[FS_REQ_NFS_MAX]) 
				(int32 			status,
				 eee_descBuf_t	*bd, 
				 char 			*txMsgp, 
				 int32 			*txMsgLen, 
				 nfs_request_t  *nfsRep)   =
{
		nfsv2_msgErrResp,			/*0: NULL-Error*/
        nfsv2_msgGetAttrResp,		/*1: GETATTR*/
        nfsv2_msgSetAttrResp,		/*2: SETATTR*/
        nfsv2_msgLookupResp,		/*3: LOOKUP*/
        nfsv2_msgErrResp,			/*4: NFS V3 ACCESS-Error*/
        nfsv2_msgReadlinkResp,		/*5: READLINK*/
        nfsv2_msgReadResp,			/*6: READ*/
        nfsv2_msgWriteResp,			/*7: WRITE*/
        nfsv2_msgCreateResp,		/*8: CREATE*/
        nfsv2_msgMkdirResp,			/*9: MKDIR*/
        nfsv2_msgSymlinkResp,		/*10: SYMLINK*/
        nfsv2_msgErrResp,			/*11: NFS V3 MKNOD-Error*/
        nfsv2_msgRemoveResp,		/*12: REMOVE*/
        nfsv2_msgRmdirResp,			/*13: RMDIR*/
        nfsv2_msgRenameResp,		/*14: RENAME*/
        nfsv2_msgHardLinkResp,		/*15:  HARDLINK*/
        nfsv2_msgReaddirResp,		/*16:  READDIR*/
        nfsv2_msgErrResp,			/*17:  READDIRPLUS-Error*/
        nfsv2_msgStatfsResp,		/*18:  STATFS*/
        nfsv2_msgErrResp,			/*19: NFS V3 FSINFO-Error*/
        nfsv2_msgErrResp,			/*20: NFS V3 PATHCONF-Error*/
        nfsv2_msgErrResp,			/*21: NFS V3 COMMIT-Error*/
};

/*
 *NFS V3 MSG  procedures
 */
int32 (*nfsrv3_msgResp[FS_REQ_NFS_MAX]) 
				(int32 			status, 
				 eee_descBuf_t 	*bd, 
				 char 			*txMsgp, 
				 int32 			*txMsgLen, 
				 nfs_request_t 	*nfsRep)     =
{
		nfsv3_msgErrResp,			/*0: NULLr*/
        nfsv3_msgGetAttrResp,		/*1: GETATTR*/
        nfsv3_msgSetAttrResp,		/*2: SETATTR*/
        nfsv3_msgLookupResp,		/*3: LOOKUP*/
        nfsv3_msgAccessResp,		/*4: ACCESS*/
        nfsv3_msgReadlinkResp,		/*5: READLINK*/
        nfsv3_msgReadResp,			/*6: READ*/
        nfsv3_msgWriteResp,			/*7: WRITE*/
        nfsv3_msgCreateResp,		/*8: CREATE*/
        nfsv3_msgMkdirResp,			/*9: MKDIR*/
        nfsv3_msgSymlinkResp,		/*10: SYMLINK*/
        nfsv3_msgMknodResp,			/*11: MKNOD*/
        nfsv3_msgRemoveResp,		/*12: REMOVE*/
        nfsv3_msgRmdirResp,			/*13: RMDIR*/
        nfsv3_msgRenameResp,		/*14: RENAME*/
        nfsv3_msgHardLinkResp,		/*15:  HARDLINK*/
        nfsv3_msgReaddirResp,		/*16:  READDIR*/
        nfsv3_msgReaddirplusResp,	/*17:  READDIRPLUS-Error*/
        nfsv3_msgStatfsResp,		/*18:  STATFS*/
        nfsv3_msgFSInfoResp,		/*19: FSINFO*/
        nfsv3_msgPathconfResp,		/*20: PATHCONF*/
        nfsv3_msgCommitResp,		/*21: COMMIT*/
};

extern pkt_desc_t *nfs_tplAllocPkt(
		nfp_clientCBPtr_t clientCBp, 
		nfs_request_t 		*nfsReqp,
		int32				usrDataLen);

/*---------------------------------------------------------------
 * Name: 	nfs_msgFindAttrType 
 *  
 * Description:
 * Finds the input attribute type.  
 * Created by:  Joseph Vu   
 *  
 * Date Created:        5/28/01
 *
 *----------------------------------------------------------------- */
fs_Attr *nfs_msgFindAttrType(fs_Attr *attrp, int32 numAttrs, 
						int32 searchAttrType)
{
	int32		attrIdx;
	fs_Attr 	*currAttrp;

	if ((numAttrs > NFS_MAX_NUM_OF_ATTR) ||
		(attrp == NULL) )
		return (NULL);

	attrIdx = 0;
	currAttrp = attrp;
	while (attrIdx < numAttrs)
	{
		if (currAttrp->attrType == searchAttrType)	
			return (currAttrp);
		attrIdx += 1;
		currAttrp += 1;
	}

	return (NULL);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_msgXmtComplete 
 *
 * Description:
 *     	Send transmit complete message to file system. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_msgXmtComplete(nfs_request_t *nfsReqp, pkt_desc_t *descp)
{
	fs_MsgTransmitComplete		*fsXmitCompletep;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Enter\n", __FUNCTION__);
#endif

	/*
	 *Allocate buffer to send request to file system
	 */
    nfsReqp->fileMsgp =  eee_allocateBuffer(EEE_SHARED_POOL,
                                            EEE_BUF_LARGE,
                                            NFX_COS_HIGH);
    if (nfsReqp->fileMsgp == NULL)
		goto nfs_msgXmtCompleteErr;

	fsXmitCompletep = (fs_MsgTransmitComplete  *)nfsReqp->fileMsgp;

	/*
	 *Fill in File System request header message 
	 */
	nfs_msgBuildFSHdr(&fsXmitCompletep->hdr, FS_REQ_TX_COMPL, 0, nfsReqp);

	/*
	 *Fill in filehandle of file to be read, file offset 
	 *and number of bytes to read
	 */
	fsXmitCompletep->fileHandle = nfsReqp->fileHandle;
	fsXmitCompletep->fileOffset = nfsReqp->readXmitCmpInd.fileOffset;
	fsXmitCompletep->bytesRead  = nfsReqp->readXmitCmpInd.bytesRead;
	fsXmitCompletep->fsPrivate  = nfsReqp->readXmitCmpInd.fsPrivate;

	nfsReqp->fileMsgLen = sizeof(fs_MsgTransmitComplete);

	/*
	 *Send message to file system
	 */
	nfs_msgSendToFs(nfsReqp);

	/*
	 *De-alloc buffer
	 */
nfs_msgXmtCompleteErr:

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Exit\n", __FUNCTION__);
#endif

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name: nfs_msgXmtPkt 
 *
 * Description:
 *      Transmit a NFS reply pkt to the client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_msgXmtPkt(pkt_desc_t *tplXmitPktDescp, nfs_request_t *nfsReqp)
{
	int32 				ret;
	nfp_clientCBPtr_t	clientCBp;	

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter: pktDescp: %p nfsReqp: %p\n", 
			__FUNCTION__, tplXmitPktDescp, nfsReqp);
#endif

	if ((tplXmitPktDescp == NULL) || (nfsReqp == NULL))
	{
		nfs_globalCfgp->nfs_debugPktCounters.tpl_badDesc++;
		return (NFX_ERR);
	}

#ifdef PKT_USE_TIME_STAMP
	/* Shaun 06/06/02
	 * Fix pkt time stamp mem. leak
	 * due to retransmission.
	 */
	if (nfsReqp->pTime)
	{
	    tplXmitPktDescp->pTime = nfsReqp->pTime;
		nfsReqp->pTime = NULL;
	}
#endif

	/*
	 *Save our request in appInfo for callback 
	 *from TPL complete Ind.
	 */
	tplXmitPktDescp->appInfo = nfsReqp;
	nfsReqp->pktDescp = (void *)tplXmitPktDescp;
	clientCBp = nfsReqp->clientCBp;
	nfsReqp->reqState = NFS_REQ_REPLIED;

	if (trace_enable != 0)
     	nfs_xmtDecodePkt(nfsReqp);

	/* 
	 * Perform Host to Net conversion.
	 *
	 * 01/04/02 Shaun
	 * Don't swap RPC/NFS headers for rexmit pkts 
     * since it had already been swapped.
	 */
	if (nfsReqp->reqState != NFS_REQ_DONE)
	    xdr_swap(tplXmitPktDescp->pHdr, tplXmitPktDescp->hdrSize, 1);
	nfs_globalCfgp->nfs_debugPktCounters.tpl_xmtPktSend++;
	ret = tpl_xmtPkt (nfsReqp->tplConnp, tplXmitPktDescp);

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Exit\n", __FUNCTION__);
#endif
	return (ret);
}

/*-----------------------------------------------------------------
 * Name: nfs_msgSendToFS
 * Description:
 *     	Send a NFS control message to the file system for processing. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK: No error 
	 	 *NFX_ERR: error occurred
	 	 */
nfs_msgSendToFs(nfs_request_t *nfsReqp)
{
	eee_desc_t			*eDescp;
	volatile eee_desc_t *vdesc;
    uchar8              *fpbuf, *fpbuf_c;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter\n", __FUNCTION__);
#endif

    vdesc = eDescp = eee_allocateDesc(EEE_FP_POOL, NFX_COS_HIGH);
 
	if (eDescp == NULL)
		return (NFX_ERR);

    fpbuf = eee_allocateBuffer(EEE_FP_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
    if (fpbuf == NULL)
        return (NFX_ERR);

	NFX_ASSERT(((address_t)nfsReqp->fileMsgp & (8 - 1)) == 0);
	uint64 *srcbuf = nfsReqp->fileMsgp;
	uint64 *srcend = srcbuf + (nfsReqp->fileMsgLen + 8 - 1) / 8;
	volatile uint64 *dstbuf = (uint64 *)fpbuf;
	while (srcbuf < srcend) {
		uint64 dword1 = srcbuf[0];
		uint64 dword2 = srcbuf[1];
		uint64 dword3 = srcbuf[2];
		uint64 dword4 = srcbuf[3];
		dstbuf[0] = dword1;
		dstbuf[1] = dword2;
		dstbuf[2] = dword3;
		dstbuf[3] = dword4;
		dstbuf += 4;
		srcbuf += 4;
	}

	volatile uint64 *desc_dwords = (uint64 *)eDescp;
	vdesc->hdr.control = ((1 << EDESC_BUFCNT_SHIFT) | nfsReqp->fileMsgLen);
	vdesc->hdr.offset = 0;
	vdesc->hdr.dest_port = nfs_globalCfgp->eee_efsPortId;
	vdesc->hdr.src_port = nfs_globalCfgp->eee_nfsPortId;

	union {
		struct {
			ushort16 control;
			ushort16 len;
			uint32 res_;
		} u;
		uint64 control_len;
	} qq;

	qq.u.control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
	qq.u.len = nfsReqp->fileMsgLen;

	desc_dwords[6] = qq.control_len;
	desc_dwords[7] = (uint64)fpbuf;

    eee_deallocateBuffer(nfsReqp->fileMsgp);

	eee_forwardLocalPktUni(nfs_globalCfgp->eee_efsPortId, eDescp);

	nfsReqp->reqState = NFS_REQ_SEND_TO_FS;
	nfsReqp->timeSendToFsInSecs = nfs_utilGetTimeOfDayInSecs();

	nfsReqp->fileMsgp = NULL;
	nfsReqp->fileMsgLen = 0;		
	nfs_globalCfgp->nfs_debugPktCounters.nfs_eeeRamDeAlloc1++;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Exit\n", __FUNCTION__);
#endif

	return (NFX_OK);

}


/*
 *------------------------------------------------------------------------------
 * Function Name:     	nfs_msgRpcReply 
 *
 * Description:   
 *  Build a RPC reply header on the response message to the client.
 *
 * Created by:          Joseph Vu
 *
 * Created:             5/28/2000
 *
 *
 *------------------------------------------------------------------------------
 */
int32	/*Return length in bytes of RPC reply header*/
nfs_msgRpcReply( char *msgp,
		uint32	reqXid,
		int32 replyStatus,
		int32 acceptStatus
)
{
	int32			msgLen;
	rpcReplyHdr_t	*rpcReplyp;

#ifdef NOTUSED
	uint32		*rpcReplyp;

	rpcReplyp = (uint32 *)msgp;

	msgLen = 0;
	*rpcReplyp++ = reqXid;
	*rpcReplyp++ = RPC_REPLY;
	*rpcReplyp++ = replyStatus;
	/*---------------------------------------------------------------------
	 *Return AUTH_NONE for now.
	 *Todo:  Handle RPC mismatch & authentication error case later
	 *This probably OK for now since RPC version is alway 2 
	 *and currently we do not handle authentication
	 *---------------------------------------------------------------------
	 */
	*rpcReplyp++ = NFS_RPC_AUTH_NONE;
	*rpcReplyp++ = 0;					/*length*/	
	*rpcReplyp++ = acceptStatus;
#endif

	rpcReplyp = (rpcReplyHdr_t *)msgp;

	rpcReplyp->rpc_xid       = reqXid;
	rpcReplyp->rpc_direction = RPC_REPLY;
	rpcReplyp->rpc_status    = replyStatus;
	/*---------------------------------------------------------------------
	 *Return AUTH_NONE for now.
	 *Todo:  Handle RPC mismatch & authentication error case later
	 *This probably OK for now since RPC version is alway 2 
	 *and currently we do not handle authentication
	 *---------------------------------------------------------------------
	 */
	rpcReplyp->rp_u.rpu_rok.rok_auth.authtype    = NFS_RPC_AUTH_NONE;
	rpcReplyp->rp_u.rpu_rok.rok_auth.authlen     = 0;
	rpcReplyp->rp_u.rpu_rok.rok_status           = acceptStatus;

	msgLen = 24;		/*Todo:  Fix this*/

	return (msgLen);
}
/*
 *-----------------------------------------------------------------------
 * Function Name:     	nfs_msgSendRespFromCache	
 *
 * Description:        
 *	
 *	This function is call to send a response to a client for
 *	request information that we found in cache.  This is done
 *	to avoid sending a request to the file system.    
 *
 * Created by:          Joseph Vu
 *
 * Created:             5/28/2000
 *
 *
 *------------------------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK or
	 	 *NFX_ERR
	 	 */
nfs_msgSendRespFromCache(nfs_request_t *nfsReqp)
{
	nfs_pktCounters_t		*txPktStatp;
	rpcCallHdr_t            *rpcCallHdrp;
	nfs_fileInfo_t          *nfs_fileInfop;
	nfp_clientCBPtr_t       clientCBp;
	nfp_fileCBPtr_t         fileCBp;
	pkt_desc_t				*tplXmitPktDescp;
	char                    *txMsgp;
	int32                   txMsgLen;
	int32					length;
	int32					nfs_version;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter\n", __FUNCTION__);
#endif

	fileCBp = nfsReqp->fileCBp;

	nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;

	rpcCallHdrp = nfsReqp->rpcCallHdrp;
	nfs_version = rpcCallHdrp->progVer;

	clientCBp = nfsReqp->clientCBp;
	//connId = nfsReqp->tplConnp;

	tplXmitPktDescp = (pkt_desc_t *)nfs_tplAllocPkt(clientCBp, 
													nfsReqp,
													0 );
	if (tplXmitPktDescp == NULL)
	{
		nfs_globalCfgp->nfs_debugPktCounters.tpl_noPktRes++;
		nfs_msgQueReplyMsg(clientCBp, NFS_TX_CACHE_MSG_TYPE,
							(void *)nfsReqp,0,0);
		return (NFX_ERR);
	}
	
	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;

	/*
	 *Start with RPC reply header
	 */
	txMsgLen += nfs_msgRpcReply(txMsgp, rpcCallHdrp->rpc_xid,
                                RPC_REPLY_MSG_ACCEPTED, RPC_REPLY_SUCCESS);
	txMsgp += txMsgLen;
	/*
	 *Set Status
	 */
	*(uint32 *)txMsgp = NFS_OK;
	txMsgp += 4;
	txMsgLen += 4;
        
	txPktStatp = &nfs_globalCfgp->nfs_xmtPktCounters;

	switch (nfsReqp->reqType)
	{
		case NFS_GETATTR:
			/*      
			 *Store attributes for the object
			 */
			length = nfs_storeAttributes(nfsReqp->reqType, 
						NFS_ATTR_AVAIL,
						&nfs_fileInfop->fs_attr, txMsgp, nfs_version); 

			/*
			 *Update message buffer pointer & length
			 *Increment transmit statistic counter
			 */
			txMsgp += length;
			txMsgLen += length;
			txPktStatp->getAttr++;
			break;

		case NFS_ACCESS:
			/*      
			 *Store attributes for the object
			 */
			length = nfs_storeAttributes(nfsReqp->reqType, 
						NFS_ATTR_AVAIL,
						&nfs_fileInfop->fs_attr, txMsgp, nfs_version); 
			/*
			 *Update message buffer pointer & length
			 */
			txMsgp += length;
			txMsgLen += length;
			/*
			 *Store bitmask of access permission
			 *Increment transmit statistic counter
			 */ 
			*(uint32 *)txMsgp =  nfs_fileInfop->nfs_accessPerm;
			txMsgp += 4;
			txMsgLen += 4;
			txPktStatp->access++;
			break;
		case NFS_READLINK:
			/*      
			 *For NFS Version 3, store attributes for the object
			 */
			if (nfs_version == NFS_VER3)
			{
				length = nfs_storeAttributes(nfsReqp->reqType, 
						NFS_ATTR_AVAIL,
						&nfs_fileInfop->fs_attr, txMsgp, nfs_version); 
				/*
			 	 *Update message buffer pointer & length
			 	 */
				txMsgp += length;
				txMsgLen += length;
			}
			/*
			 *Store pathname
			 */
			length = nfs_storeString(txMsgp, nfs_fileInfop->nfs_namep, 
						nfs_fileInfop->nfs_nameLen, FS_PATH_MAX);
			txMsgLen += length;
			txPktStatp->readLink++;

			break;
		case NFS_LOOKUP:
		{
			nfp_fileCBPtr_t		lookupFileCBp;
			fs_Handle			*fhandlep;
			nfs_fileInfo_t 		*lookupFileInfop;
			int32				length;

			/*
			 *Store filehandle of the object corresponding to name
			 */
			
			lookupFileCBp = nfsReqp->toFileCBp;
			fhandlep = &lookupFileCBp->nfp_fileHandle;
			length = nfs_storeFhandle(nfs_version, fhandlep, txMsgp);
			/*
			 *Update message buffer pointer & length
			 */
			txMsgp += length;
			txMsgLen += length;

			/*
			 *Store attributes of the object corresponding to name
			 *Increment packet statistic counter
			 */			
			lookupFileInfop = &lookupFileCBp->protocol.nfp_nfsFileInfo;	
			length = nfs_storeAttributes(nfsReqp->reqType, 
					NFS_ATTR_AVAIL,
					&lookupFileInfop->fs_attr, txMsgp, nfs_version); 

			txPktStatp->lookUp++;

			/*
			 *Update message buffer pointer & length
			 */
			txMsgp += length;
			txMsgLen += length;
			
			if (nfs_version == NFS_VER2)
				break;
			/*
			 *Store attributes of the directory 
			 */
			length = nfs_storeAttributes(nfsReqp->reqType, 
						NFS_ATTR_AVAIL,
						&nfs_fileInfop->fs_attr, txMsgp, nfs_version); 

			txMsgLen += length;
			
			break;
		}
	}
	/*
	 *Set packet desc hdr size
	 */
	tplXmitPktDescp->hdrSize = txMsgLen;
#ifdef	NOTUSED
	if (nfsReqp->reqState > NFS_REQ_INPROG) 
		return (NFX_OK);
#endif	//NOTUSED
	if (nfs_msgXmtPkt(tplXmitPktDescp, nfsReqp) != NFX_OK)
	{
		//Error: Set reqState to DONE so that
		//nfs_reqTimeoutHandler() will free nfsReqp
		nfsReqp->pktDescp = (void *)tplXmitPktDescp;
		nfsReqp->reqState = NFS_REQ_DONE;
	}
#ifdef NFS_DBG_PRINTF_ON
	printf("%s Exit\n", __FUNCTION__);
#endif

	return (NFX_OK);
}

/* Shaun 09/10/2002
 *
 * Macro to invalidate & uncache FS msg 
 * Part of performance tunning.
 *
 */
#define NFS_UNCACHE_FS_MSG( bd )								\
{																\
	if (cacheit) {												\
	sibyte_cacheInvalidate((address_t)bd->buf, bd->len);		\
	bd->buf = (char *) cached_to_uncached((address_t) bd->buf);	\
	}															\
}																\

/*
 *----------------------------------------------------------------
 * Function :	nfs_msgRcv 
 *
 * Description: Function registered with the EEE layer to receive
 * 				response messages  from File System (fp-cpu)
 *
 * Created by:  Joseph Vu 
 *
 * Created:     5/28/2000
 * 
 *
 *---------------------------------------------------------------
 */
void nfs_msgRcv( eee_desc_t *eDescp )
{
	nfs_pktCounters_t	*txPktStatp;
	rpcCallHdr_t		*rpcCallHdrp;
	eee_descBuf_t		*bd;
	fs_MsgRespHdr		*fsMsgRespHdrp;
	nfs_request_t 		*nfsReqp;
	pkt_desc_t			*tplXmitPktDescp;
	nfp_clientCBPtr_t   clientCBp;
	char				*txMsgp;
	int32				status;
	int32				txMsgLen;
	int32				nfs_version;
	int32 				usrDataSize;
	int32				result;
	int32				msgLen;
	uint32				cacheit;


#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Enter: eDescp: %p\n",__FUNCTION__, eDescp);
#endif

	if (eDescp == NULL)
	{
		return;
	}

	bd = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
				&eDescp->bd[0] : &eDescp->bd[1];

	if (bd->buf == NULL)
	{
		eee_freePacket(eDescp);
		return;
	}

	/*
	 * Shaun 9/10/02
	 *
	 * Convert read of FS Response over 
	 * Hyper Transport into cached reads.
 	 * Part of performance tunning.
	 */
	if (((address_t)(bd->buf) & 0xF000000000) == 0x4000000000) 
	{
		bd->buf = (char *) uncached_to_cached((address_t) bd->buf);
		cacheit = 1;
	}
	else
		cacheit =0;

	fsMsgRespHdrp = (fs_MsgRespHdr *) (bd->buf);
	msgLen  = (eDescp->hdr.control & EDESC_TOTAL_LEN );


#ifdef NFS_DBG_PRINTF_ON
	printf("%s: msg Type: %d Len: %d\n", 
			__FUNCTION__, fsMsgRespHdrp->requestType, msgLen);
#endif

	/*
	 * Increment messages response count from the file system
	 * Need to count both Mount & NFS.
	 */
	nfs_globalCfgp->nfs_totalNumOfRespRcvdFromFs++;

	/*
	 * Send Get Root response to MOUNT.
	 */
	if (fsMsgRespHdrp->requestType == FS_REQ_GET_ROOT)
	{
		NFS_UNCACHE_FS_MSG( bd )
		mnt_msgFpRcv(eDescp);
		return;
	}

	/*
	 * Receive Invalidate Cache Request from FP.
	 */
	if (fsMsgRespHdrp->requestType == FS_REQ_CACHE_INV)
	{
		nfs_cacheFlush( FS_CACHE_INV_TYPE_ALL );

		NFS_UNCACHE_FS_MSG( bd )
    	eee_freePacket(eDescp);
		return;
	}

	/*
	 * Make sure FS Response are in range.
	 */
	if ((fsMsgRespHdrp->requestType < FS_REQ_NULL) ||
        (fsMsgRespHdrp->requestType >= FS_REQ_NFS_MAX)) 
	{
		nfs_globalCfgp->nfs_debugPktCounters.nfs_rxInvalidResp++;

		NFS_UNCACHE_FS_MSG( bd )
    	eee_freePacket(eDescp);
		return;
	}	


	nfsReqp = (nfs_request_t *)fsMsgRespHdrp->clientPrivate;

    if (nfp_validateNfsReqCB(nfsReqp) != NFX_OK )
	{

		/* Shaun 09/20/02  Bug# 2181
		 * 
		 * Send tx cmp ind to FS for Read
		 * for nfsReq that were timed-out 
		 * due to long FS response.
		 */
	    if (fsMsgRespHdrp->requestType == FS_REQ_READ)
		{
			nfs_msgXmtCmplErr( (fs_MsgReadResp *) fsMsgRespHdrp );
		}

		//Invalid NFS request
		nfs_globalCfgp->nfs_debugPktCounters.nfs_rxInvalidResp++;

		NFS_UNCACHE_FS_MSG( bd )
    	eee_freePacket(eDescp);
		return;
	}


	//Validate request received from the file system
    if ( (nfsReqp->rpcCallHdrp == NULL) ||
		 (nfsReqp->rpcCallHdrp->rpc_xid != fsMsgRespHdrp->clientId) )
    {
		nfs_globalCfgp->nfs_debugPktCounters.nfs_dropFsResp++;
        nfsReqp->reqState = NFS_REQ_DONE;

		/* Shaun 09/20/02  Bug# 2181
		 * 
		 * Send tx cmp ind to FS for Read
		 * for nfsReq that were timed-out 
		 * due to long FS response.
		 */
	    if (fsMsgRespHdrp->requestType == FS_REQ_READ)
		{
			nfs_msgXmtCmplErr( (fs_MsgReadResp *) fsMsgRespHdrp );
		}

		NFS_UNCACHE_FS_MSG( bd )
    	eee_freePacket(eDescp);
		return;
    }

	/* Shaun 04/19/02 #564
	 *
	 * If Volume is unmounted or offline
	 * (due to FP problem) DO NOT return
	 * I/O Error to Client. 
	 * Instead, drop the response so that 
	 * client gets an RPC timeout.
	 */
	rpcCallHdrp = nfsReqp->rpcCallHdrp;
	nfs_version = rpcCallHdrp->progVer;
	if ((fsMsgRespHdrp->fsStatus == FS_ERR_NOT_MOUNTED) ||
        (fsMsgRespHdrp->fsStatus == FS_ERR_NO_RESOURCE))
	{
		nfs_globalCfgp->nfs_debugPktCounters.nfs_fsResourceErr++;
		//Set SNMP counters
		if (nfs_version == NFS_VER2)
			nfs_globalCfgp->nfs_v2XmtPktCounters.nfs_fsResourceErr++;
		else
			nfs_globalCfgp->nfs_v3XmtPktCounters.nfs_fsResourceErr++;

		/* Shaun 11/01/02 #2470, #2674
		 * 
		 * Set reqState to DONE 
		 * Free Req CB to remove it from 
		 * non-idempotent cache.
		 */ 
        nfsReqp->reqState = NFS_REQ_DONE;
        nfs_msgFreeAll(nfsReqp, 0);

		/*
		 * Uncache fs buffer.
		 */ 
		NFS_UNCACHE_FS_MSG( bd )

		/*
		 * Free eee descriptor.
		 */ 
    	eee_freePacket(eDescp);

		return;
	}

	if ((fsMsgRespHdrp->fsStatus < FS_ERR_NONE) || 
		(fsMsgRespHdrp->fsStatus > FS_ERR_SERVERFAULT) )   
    {
        //printf ("Unknown fsStatus field %d (msg type %d)\n", 
        //        fsMsgRespHdrp->fsStatus, fsMsgRespHdrp->requestType);
		if (nfs_version >= NFS_VER3)
			status = NFSERR_SERVERFAULT;
		else
			status = NFSERR_ACCES;
	}
    else
    {
		status = fsMsgRespHdrp->fsStatus;
	}
#ifdef NFS_DBG_PRINTF_ON
	printf("%s: FS Status: %d NFS Status: %d\n", 
			__FUNCTION__, fsMsgRespHdrp->fsStatus, status);
#endif

	clientCBp  = (nfp_clientCBPtr_t)nfsReqp->clientCBp;
	usrDataSize = 0;

#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp( nfsReqp->pTime, PKT_TIME_IDX_FS_RSP);
#endif

	if (fsMsgRespHdrp->requestType == FS_REQ_READ)
	{
			fs_MsgReadResp		*fsReadRespMsgp;
			int32				pad;
		
			fsReadRespMsgp = (fs_MsgReadResp *)bd->buf;	
			pad = (fsReadRespMsgp->bytesRead & 0x03);
			if (pad)
				pad = 4 - pad;

			usrDataSize = pad + fsReadRespMsgp->bytesRead;	
	}
	else if ( (fsMsgRespHdrp->requestType == FS_REQ_READDIR) ||
			 (fsMsgRespHdrp->requestType == FS_REQ_READDIRPLUS) )
	{
		//Set size for READDIR or READDIRPLUS
		usrDataSize = msgLen;
	}
	

	/*
	 *Allocate packet Desc for transmission
	 */
	tplXmitPktDescp = (pkt_desc_t *)nfs_tplAllocPkt(clientCBp, 
													nfsReqp,
													usrDataSize);
	if (tplXmitPktDescp == NULL)
	{
		NFS_UNCACHE_FS_MSG( bd )

		nfs_globalCfgp->nfs_debugPktCounters.tpl_noPktRes++;

		nfs_msgQueReplyMsg(clientCBp, NFS_TX_REG_MSG_TYPE,
						(void *)eDescp,0,0);

		return;
	}


	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;
	/*
	 *Build RPC reply message to send to the client
	 */	
	txMsgLen += nfs_msgRpcReply(txMsgp, rpcCallHdrp->rpc_xid,
				RPC_REPLY_MSG_ACCEPTED, RPC_REPLY_SUCCESS);
	txMsgp += txMsgLen;

	txPktStatp = &nfs_globalCfgp->nfs_xmtPktCounters;

	nfsReqp->pktDescp = (void *)tplXmitPktDescp;
	nfsReqp->edescp = eDescp;

	switch (nfs_version)
	{
		case 	NFS_VER2:
				result = (*(nfsrv2_msgResp[fsMsgRespHdrp->requestType]))
								(status, bd, txMsgp, &txMsgLen, nfsReqp);
				NFS_UNCACHE_FS_MSG( bd )
				break;

		case 	NFS_VER3:
				result = (*(nfsrv3_msgResp[fsMsgRespHdrp->requestType]))
								(status, bd, txMsgp, &txMsgLen, nfsReqp);
				NFS_UNCACHE_FS_MSG( bd )
				break;
		default:
				//Invalid NFS version
				nfsReqp->reqState = NFS_REQ_DONE;

				NFS_UNCACHE_FS_MSG( bd )
				eee_freePacket(eDescp);
				return;
	}


#ifdef NFS_USE_TIME_STAMP
	pkt_setTxByteCount( txMsgLen, nfsReqp->reqType );
	pkt_setTxByteCount( usrDataSize, nfsReqp->reqType );
#endif

	/*
	 *Set packet desc hdr size
	 */
	tplXmitPktDescp->hdrSize = txMsgLen;

	if (nfs_msgXmtPkt(tplXmitPktDescp, nfsReqp) != NFX_OK)
	{
		//Error:  set reqState to DONE so that
		//nfs_reqTimeoutHandler() will free nfsReqp
		nfsReqp->pktDescp = (void *)tplXmitPktDescp;
		nfsReqp->reqState = NFS_REQ_DONE;
	}
    eee_freePacket(eDescp);

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Exit\n",__FUNCTION__);
#endif

}

/*-----------------------------------------------------------------
 * Name: nfs_msgBuildFSHdr 
 *
 * Description:
 *     	Build a  file system request message header for message that 
 *	will be sent to file system.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nfs_msgBuildFSHdr(fs_MsgReqHdr *fsMsgReqHdrp,	/*File System message req hdr*/
		uint32 reqType, 						/*Req message type*/
		uint32 attrFlag, 						/*request attributes flag*/
		nfs_request_t *nfsReqp					/*reqCB*/
)
{
	int32				i;
	uint32				nfs_version;
	rpcCallHdr_t		*rpcCallHdrp;
	rpc_authSysInfo_t	*authSysInfop;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Enter\n", __FUNCTION__);
#endif

	rpcCallHdrp  = nfsReqp->rpcCallHdrp;
	authSysInfop = &rpcCallHdrp->authFlavor.authSys;
	nfs_version  = rpcCallHdrp->progVer;

	/*
	 *Filled in File System (FS) Req header msg
	 */
	fsMsgReqHdrp->requestType = reqType;
	fsMsgReqHdrp->attrType = attrFlag;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: authSysInfop %d UID: %d GID: %d\n",
	__FUNCTION__, authSysInfop, authSysInfop->userId,  authSysInfop->groupId);	
#endif

	fsMsgReqHdrp->uid = authSysInfop->userId;
	fsMsgReqHdrp->gid = authSysInfop->groupId;

	fsMsgReqHdrp->numGids = authSysInfop->auxGidCount;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: No GIDs: %d\n", __FUNCTION__, fsMsgReqHdrp->numGids);
#endif

	for (i = 0; i < authSysInfop->auxGidCount; i++)
	{
		fsMsgReqHdrp->gids[i] = authSysInfop->auxGids[i];
#ifdef NFS_DBG_PRINTF_ON
		printf("\tGid[%d]: %d\n", i, fsMsgReqHdrp->gids[i]);
#endif
	}
	
	fsMsgReqHdrp->proto = (nfs_version == NFS_VER2)? FS_PROTO_NFS_V2:
			(nfs_version == NFS_VER3)? FS_PROTO_NFS_V3: FS_PROTO_NFS_V4;
	fsMsgReqHdrp->clientPrivate = nfsReqp;
	fsMsgReqHdrp->clientId = (uint32)nfsReqp->rpcCallHdrp->rpc_xid;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: nfsReqp: %p Xid: %d UID: %d GID: %d\n", 
		__FUNCTION__, 
		nfsReqp, 
		fsMsgReqHdrp->clientId, 
		fsMsgReqHdrp->uid, 
		fsMsgReqHdrp->gid);

	printf("%s: Exit\n", __FUNCTION__);
#endif

	return;
}


/*
 *--------------------------------------------------------------------------
 * Function Name:     	nfs_msgFreeAll	
 *
 * Description:        
 * This function called after received a transmit 
 * complete indicdation from 
 * Transport Layer (TPL) to de-allocate all previous 
 * allocated memory for this request. 
 *
 * Created by:          Joseph Vu
 *
 * Created:             5/28/2000
 *
 *
 *---------------------------------------------------------------------------
 */
void
nfs_msgFreeAll(nfs_request_t *nfsReqp,  int32 freePktDesc)
{
	pkt_desc_t 					*pDesc;
	nfp_clientCBPtr_t			clientCBp;
	nfp_requestCBPtr_t			reqCBp;	
	int32						ret;
	void						*ra;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter:  nfsReqp: %p pktDescp: %p\n", 
		__FUNCTION__, nfsReqp, nfsReqp->pktDescp);
#endif

	ra = (void *)get_ra();
	if (nfsReqp->reqState != NFS_REQ_DONE)
	{
		panic("%s: ra: %p\n", __FUNCTION__, ra);
	}

#ifdef PKT_USE_TIME_STAMP
    pkt_freeTimeStamp(nfsReqp->pTime);
	nfsReqp->pTime = NULL;
#endif

	clientCBp = (nfp_clientCBPtr_t)nfsReqp->clientCBp;

	pDesc = (pkt_desc_t *)nfsReqp->pktDescp;
	if (nfsReqp->reqType == NFS_READ) 
	{
		if (nfsReqp->readXmitCmpInd.bytesRead != 0)
		{
			nfs_msgXmtComplete(nfsReqp, pDesc);	
			nfs_globalCfgp->nfs_debugPktCounters.nfs_readXmtCmpInd++;
			nfs_globalCfgp->nfs_debugPktCounters.nfs_readXmtCmpInd2++;
		}
	}

	/* Shaun 08/10/22
	 *
	 * Peformance tuning:
	 * Delete ReqCb from hash table.
	 */
	if (nfsReqp->rpcCallHdrp != NULL)
	{
    	if (nfs_isReqNonIdempotent( nfsReqp->rpcCallHdrp->progVer,
                            	nfsReqp->rpcCallHdrp->rpc_proc) == TRUE)
    	{
        	nfs_hashDeleteReqCb(nfsReqp->rpcCallHdrp->rpc_xid);
    	}
	}

	/*
	 *Free TPL request packet
	 */
	if ( (nfsReqp->pktDescp != NULL) && freePktDesc)
	{
		/*
		 * Shaun 04/23/02
		 *
		 * Fix for #678.
		 * Connection can be removed for NFS NULL procedures.
		 * Free PKT & let TPL layer count it as bad conn err.
		 */

#ifdef NOTUSED /* Shaun #678 */
		if ( (tpl_validateConnCb( nfsReqp->tplConnp ) == NFX_OK) &&
			(pDesc && (pDesc->flags & PKT_DESC_INUSE)) )
#endif
		{

#ifdef NFS_DBG_PRINTF_ON
			printf("%s:  Free TPL pkt desc: %08x\n", 
				__FUNCTION__, nfsReqp->pktDescp);
#endif
		
			ret = TPL_FREE_PACKET(nfsReqp->tplConnp, nfsReqp->pktDescp);
			if (ret != NFX_OK)
			{
				/*
		 	 	 *Handle error
		 	 	 */	
#ifdef NFS_DBG_PRINTF_ON
				printf("%s: tpl_freePkt failed (%d)!\n", __FUNCTION__, ret);
#endif

			}
			nfs_globalCfgp->nfs_debugPktCounters.tpl_xmtPktFree++;
			nfsReqp->pktDescp = NULL;

		}
	}

	if (nfs_delReqCBFromList(nfsReqp) != NFX_OK)
	{
		/*
		 *TODO:  Handle error
		 */
#ifdef NFS_DBG_PRINTF_ON
		printf("%s: nfs_delReqCBFromList failed!\n", __FUNCTION__);
#endif
	}

	/* 
	 * Shaun 03/14/02
	 * Call eee_ramDealloc() AFTER nfs_delReqCBFromList()
	 * which uses rpcCallHdrp.   Set rpcCallHdrp to NULL .
	 */
	if (nfsReqp->rpcCallHdrp != NULL)
	{
//		if (eee_ramDealloc(nfsReqp->rpcCallHdrp) != NFX_OK)
//			NFS_DBG_PRINTF("nfs_msgFreeAll 4: eee_ramDealloc failed!\n");
		nfsReqp->rpcCallHdrp = NULL;
		nfs_globalCfgp->nfs_debugPktCounters.nfs_eeeRamDeAlloc3++;		
	}

	/*
	 *De-alloc request control block
	 */
	nfsReqp->reqState = 0;
	reqCBp = (nfp_requestCBPtr_t) nfsReqp->reqCBp; 

#ifdef NFS_DBG_PRINTF_ON
	printf("%s : deallocReqCB: %p\n", __FUNCTION__, nfsReqp->reqCBp);
#endif

	if (nfp_deallocReqCB(reqCBp) != NFX_OK)
	{
		/*
		 *TODO:  Handle error
		 */
#ifdef NFS_DBG_PRINTF_ON
		printf("%s : nfp_deallocReqCB failed!\n", __FUNCTION__);
#endif
	}
	nfs_globalCfgp->nfs_debugPktCounters.nfs_freeReqCB++;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s:  Exit\n", __FUNCTION__);
#endif

}

/*
 *----------------------------------------------------------------------
 * Function Name:      	nfs_msgSendErrResp 
 *
 * Description:
 *	This function is call when an error occurred during processing of the
 *	client's request.  This function is call to send an error response 
 *      back to the client.
 *
 * Created by:          Joseph Vu
 *
 * Created:             5/28/2000
 *
 *-----------------------------------------------------------------------
 */
int32   /*
         *NFX_OK or
         *NFX_ERR
         */
nfs_msgSendErrResp (nfs_request_t *nfsReqp, uint32 reqXid, int32 status)
{
	nfp_clientCBPtr_t 		clientCBp;
	pkt_desc_t              *tplXmitPktDescp;
	char                    *txMsgp;
	int32                   txMsgLen;
	int32                   length;
	int32					nfs_version;
	int32					nfs_proc;
	int32					i;
	int32					numOfAttrs;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s:  Enter\n", __FUNCTION__);
#endif

	clientCBp = nfsReqp->clientCBp;
	if (nfp_validateClientCB(clientCBp) != NFX_OK)	
	{
		//panic ("nfs_msgSendErrResp: Bad client control block!\n");
	    return (NFX_ERR);

	}
	tplXmitPktDescp = (pkt_desc_t *)nfs_tplAllocPkt(clientCBp, 
													nfsReqp,
													0);
	if (tplXmitPktDescp == NULL)
	{
		nfs_globalCfgp->nfs_debugPktCounters.tpl_noPktRes++;
		nfs_msgQueReplyMsg(clientCBp, NFS_TX_ERR_MSG_TYPE,
						(void *)nfsReqp,reqXid,status);
		return (NFX_ERR);
	}


	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;
	/*
	 *Start with RPC reply header
	 */
	txMsgLen += nfs_msgRpcReply(txMsgp, reqXid,
			RPC_REPLY_MSG_ACCEPTED, RPC_REPLY_SUCCESS);
	txMsgp += txMsgLen;
	/*
	 *Set Status
	 */
	//msgp = (uint32 *)txMsgp;
	//*msgp++ = status;
	*(uint32 *)txMsgp = status;
	txMsgp += 4;
	txMsgLen += 4;

	if (nfsReqp->rpcCallHdrp)
	{		
		//Set attributes not available flag
		nfs_version = nfsReqp->rpcCallHdrp->progVer;
		nfs_proc = nfsReqp->rpcCallHdrp->rpc_proc;	
    	if ( ( (nfs_version == NFS_VER3) &&
        	(nfs_proc && (nfs_proc != NFS_V3_PROC_GETATTR)) ) ||
        	((nfs_version == NFS_VER2) &&
        	(nfs_proc && (nfs_proc == NFS_V2_PROC_READDIR)) ) )
    	{
			numOfAttrs = nfs_getNumAttrFields(nfs_proc);
			//Set flag to indicate attributes
			//is not available	

			for (i=0; (i < numOfAttrs && i < 4); i++)
			{
				length = nfs_storeFieldPresentFLag(txMsgp, 
							NFS_ATTR_NOT_AVAIL);
        		txMsgp += length;
				txMsgLen += length;
			};
    	}
	}
	/*
	 *Set packet desc hdr size
	 */
	tplXmitPktDescp->hdrSize = txMsgLen;
	if (nfs_msgXmtPkt(tplXmitPktDescp, nfsReqp) != NFX_OK)
	{
		//Error:  set reqState to DONE so that
		//nfs_reqTimeoutHandler() will free nfsReqp
        nfsReqp->pktDescp = (void *)tplXmitPktDescp;
        nfsReqp->reqState = NFS_REQ_DONE;
    }

#ifdef NFS_DBG_PRINTF_ON
	printf("%s:  Exit\n", __FUNCTION__);
#endif
	
	return (NFX_OK);
}

/*
 *----------------------------------------------------------------------
 * Function Name:      nfs_msgQueReplyMsg 
 *
 * Description:
 *	Queue up current reply since we cannot process 
 *  it at this time (most likely, we
 *  NFS server cannot allocate packet from TPL).
 *	We will retry later.
 *
 * Created by:          Joseph Vu
 *
 * Created:             10/28/2001
 *
 *----------------------------------------------------------------------- */
int32
nfs_msgQueReplyMsg(nfp_clientCBPtr_t       clientCBp,
	uint32 msgType, void *info1p, void *info2p,
	void *info3p)
{

	nfp_replyCBPtr_t	replyEntryp;
	nfs_reply_t         *nfs_replyp;

	nfs_globalCfgp->nfs_debugPktCounters.tpl_noPktRes++;
	replyEntryp = (nfp_replyCBPtr_t)nfp_allocReplyCB();
	if (replyEntryp == NULL)
	{
		return (NFX_ERR);
	}
	nfs_replyp = &replyEntryp->protocol.nfs_reply;	
	nfs_replyp->tx_msgType = msgType;
	nfs_replyp->info1p = info1p;
	nfs_replyp->info2p = info2p;
	nfs_replyp->info3p = info3p;
	if (nfs_addCBToList(&clientCBp->nfp_replyList, (void *)replyEntryp,
		NFP_REPLY_NODE) != NFX_OK)
	{
		return (NFX_ERR);
	}
	return (NFX_OK);
}

/*
 *----------------------------------------------------------------------
 * Function Name:     	nfs_msgTxQueReply 
 *
 * Description:
 *	Retry function for sending reply back to the client.
 *	
 * Created by:          Joseph Vu
 *
 * Created:             10/28/2001
 *
 *----------------------------------------------------------------------- */
int32 nfs_msgTxQueReply (nfp_replyCBPtr_t    replyEntryp)
{
	nfs_reply_t			*nfs_replyp;
	eee_desc_t          *eDescp;
	nfs_request_t       *nfsReqp;
	uint32              reqXid;
	int32               status;

	if (nfp_validateReplyCB(replyEntryp) != NFX_OK)
		return (NFX_ERR);
	
	nfs_replyp = &replyEntryp->protocol.nfs_reply;
	switch (nfs_replyp->tx_msgType)
	{
		case NFS_TX_REG_MSG_TYPE:
			eDescp = nfs_replyp->info1p;
			//printf("nfs_msgTxQueReply 3: eDescp: %p\n", eDescp);
			nfs_msgRcv( eDescp );
        	break;
        case NFS_TX_CACHE_MSG_TYPE:
			nfsReqp = nfs_replyp->info1p;
		    //printf("nfs_msgTxQueReply 4: nfsReqp: %p\n", nfsReqp);
			nfs_msgSendRespFromCache(nfsReqp);
			break;

		case NFS_TX_ERR_MSG_TYPE:
			nfsReqp = nfs_replyp->info1p;
			reqXid = (uint32) nfs_replyp->info2p;
			status = (uint32) nfs_replyp->info1p;
			//printf("nfs_msgTxQueReply 5: nfsReqp: %p\n", nfsReqp);
			if (nfs_msgSendErrResp (nfsReqp, reqXid, status) != NFX_OK)
			{	
				nfsReqp->reqState = NFS_REQ_DONE;
				nfs_msgFreeAll(nfsReqp, 0);			
			}					
			break;
		case NFS_TX_SUNACL_MSG_TYPE:
			nfsReqp = nfs_replyp->info1p;
			if (nfp_validateReqCB(nfsReqp) != NFX_OK)
				break;
			nfs_procRxNfsAcl(nfsReqp);
			break;
	}

	return (NFX_OK);
}
/*
 *----------------------------------------------------------------------
 * Function Name:      nfs_msgTxTimerHandler 
 *
 * Description:
 *	Timeout function for handling of retrying of sending back
 *	reply to client.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             10/28/2001
 *
 *----------------------------------------------------------------------- */
int32 nfs_msgTxTimerHandler(int32 tmr, 
		void *info1, void *info2, 
		void *info3, void *info4)
{
	int32					count;
	int32					iterations;
	int32					i;
	nfp_replyCBPtr_t		replyEntryp;
	nfp_clientCBPtr_t		clientCBp;
	nfp_nodePtr_t       	tailp;

	count = 0;
	for (clientCBp = 
		(nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep;
		((count < NFP_MAX_CLIENT) && (clientCBp != NULL));
		count++, clientCBp++)
	{
		if (!(clientCBp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
		iterations = clientCBp->nfp_replyList.nfp_reqCnt;
		for (i = 0; (i < iterations); i++)
		{
			tailp = clientCBp->nfp_replyList.nfp_tailp;
			if (nfp_validateNode(tailp) != NFX_OK)
				return (NFX_ERR);	
			replyEntryp = 
				(nfp_replyCBPtr_t )tailp->nfp_entryCBp;
			nfs_msgTxQueReply (replyEntryp);
			nfs_delNode(&clientCBp->nfp_replyList,
						clientCBp->nfp_replyList.nfp_tailp);
			nfp_deallocReplyCB(replyEntryp);
		}

	}
	return (NFX_OK);

}


/*
 *----------------------------------------------------------------------
 * Function Name:	nfs_msgXmtCmplErr
 *
 * Description:		Send back transmit complete ind. to File System
 *					for Read Response when original request has been
 *					timed-out due to long response from File System.
 *
 *----------------------------------------------------------------------
 */
void 
nfs_msgXmtCmplErr( fs_MsgReadResp *fsMsgp )
{
	nfs_request_t			*nfsReqp;
    nfp_requestCBPtr_t      reqCBp;


#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Enter\n", __FUNCTION__);
#endif

	/*
	 * Allocate dummy nfs req CB.
	 */
    reqCBp = nfp_allocReqCB ();
    if (reqCBp == NULL)
    {
#ifdef NFS_DBG_PRINTF_ON
		printf("%s: Exit\n", __FUNCTION__);
#endif
        return;
    }
    nfs_globalCfgp->nfs_debugPktCounters.nfs_allocReqCB++;


	nfsReqp              = &reqCBp->protocol.nfs_req; 
	/*
	 * Clear nfsReqCB contents.
	 */
	memset(nfsReqp, 0, sizeof(nfs_request_t));

 	nfsReqp->reqCBp      = (void *)reqCBp;
 	nfsReqp->rpcCallHdrp = &nfsReqp->rpcHdr;

	/*
	 * Thats all FS needs for READ xmt complete ind.
	 */
	nfsReqp->readXmitCmpInd.fsPrivate = fsMsgp->fsPrivate;
	
	/*
	 * Tell FS to release the Read CODs.
	 */
	nfs_msgXmtComplete(nfsReqp, NULL);

	/*
	 * Free the nfsReq.
	 */
	nfsReqp->reqState = NFS_REQ_DONE;
	nfs_msgFreeAll(nfsReqp, 0);

#ifdef NFS_DBG_PRINTF_ON
	printf("%s: Exit\n", __FUNCTION__);
#endif

}

#endif  //NFP_TXRX
