/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-tpl.c
 *
 * RCS:  $Id: nfs-tpl.c,v 1.67 2002/10/04 01:29:25 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain NFS/Transport Layer Protocol (TPL) interface functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/nfs-api.h>
#include "nfsv3-proc.h"
#include "nfsv2-proc.h"
#include "nfs-hash.h"

#define	NFS_DBG_ON

uint32	trace_enable	= 0;		
int32	nfs_debug = 0;

uint32	nfs_debugCmpPktInd = 0;		//For debug only

uint32	nfs_tmpDebug1 = 0;
uint32	nfs_tmpDebug2 = 0;

extern	uint32 nfs_bootTimeInSecs;
/*
 *Externs
 */
nfs_request_t *nfs_findReqCB(
        nfp_clientCBPtr_t clientCBp,  /*Client control block*/
        uint32 xid,             /*request transaction ID*/
        uint32 procNum);         /*request procedure number*/

/*
 *NFS Server 2 procedures
 */
int32 (*nfsrv2_procs[NFS_MAX_V2PROC]) (nfs_request_t	*nfsReqp) =
{
        nfsv2_procRxNullReq,		/*0: NULL*/
        nfsv2_procRxGetAttrReq,		/*1: GETATTR*/
        nfsv2_procRxSetAttrReq,		/*2: SETATTR*/
        nfsv2_procRxNoOpReq,		/*3: ROOT: Obsolete*/
        nfsv2_procRxLookupReq,		/*4: LOOKUP*/
        nfsv2_procRxReadLinkReq,	/*5: READLINK*/
        nfsv2_procRxReadReq,		/*6: READ*/
        nfsv2_procRxNoOpReq,		/*7: WRITECACHE: Not Used*/
        nfsv2_procRxWriteReq,		/*8: WRITE*/
        nfsv2_procRxCreateReq,		/*9: CREATE*/
        nfsv2_procRxRemoveReq,		/*10: REMOVE*/
        nfsv2_procRxRenameReq,		/*11: RENAME*/
        nfsv2_procRxLinkReq,		/*12: LINK*/
        nfsv2_procRxSymlinkReq,		/*13: SYMLINK*/
        nfsv2_procRxMkdirReq,		/*14:  MKDIR*/
        nfsv2_procRxRmdirReq,		/*15:  RMDIR*/
        nfsv2_procRxReaddirReq,		/*16:  READDIR*/
        nfsv2_procRxStatfsReq,		/*17:  STATFS*/
};

/*
 *NFS Server 3 procedures
 */
int32 (*nfsrv3_procs[NFS_MAX_V3PROC]) (nfs_request_t    *nfsReqp) = 
{
        nfsv3_procRxNullReq,		/*0: NULL*/
        nfsv3_procRxGetAttrReq,		/*1: GETATTR*/
        nfsv3_procRxSetAttrReq,		/*2: SETATTR*/
        nfsv3_procRxLookupReq,		/*3: LOOKUP*/
        nfsv3_procRxAccessReq,		/*4: ACCESS*/
        nfsv3_procRxReadLinkReq,	/*5: READLINK*/
        nfsv3_procRxReadReq,		/*6: READ*/
        nfsv3_procRxWriteReq,		/*7: WRITE*/
        nfsv3_procRxCreateReq,		/*8: CREATE*/
        nfsv3_procRxMkdirReq,		/*9: MKDIR*/
        nfsv3_procRxSymlinkReq,		/*10: SYMLINK*/
        nfsv3_procRxMkNodReq,		/*11:  MKNOD*/
        nfsv3_procRxRemoveReq,		/*12:  REMOVE*/
        nfsv3_procRxRmdirReq,		/*13:  RMDIR*/
        nfsv3_procRxRenameReq,		/*14:  RENAME*/
        nfsv3_procRxLinkReq,		/*15:  LINK*/
        nfsv3_procRxReaddirReq,		/*16:  READDIR*/
        nfsv3_procRxReaddirPlusReq,	/*17:  READDIRPLUS*/
        nfsv3_procRxStatfsReq,		/*18:  FSSTAT*/ 
        nfsv3_procRxFsinfoReq,		/*19:  FSINFO*/
        nfsv3_procRxPathConfReq,	/*20:  PATHCONF*/
        nfsv3_procRxCommitReq		/*21:  COMMIT*/
};

/*
 * NFS V2 wait for re-xmt request flag, TRUE for nonidempotent operations.
 */
boolean	nfsv2_waitForRexmt[NFS_MAX_V2PROC] = {
		FALSE,		/*0: NULL*/
		FALSE,		/*1: GETATTR*/
		FALSE,		/*2: SETATTR*/
		FALSE,		/*3: ROOT: Obsolete*/
		FALSE,		/*4: LOOKUP*/
		FALSE,		/*5: READLINK*/
		FALSE,		/*6: READ*/
		FALSE,		/*7: WRITECACHE: Not Used*/
		FALSE,		/*8: WRITE*/
		TRUE,		/*9: CREATE*/
		TRUE,		/*10: REMOVE*/
		TRUE,		/*11: RENAME*/
		TRUE,		/*12: LINK*/
		TRUE,		/*13: SYMLINK*/
		TRUE,		/*14: MKDIR*/
		TRUE,		/*15: RMDIR*/
		FALSE,		/*16: READDIR*/
		FALSE		/*17: STATFS*/
};

/*
 * NFS V3 wait for re-xmt request flag, TRUE for nonidempotent operations.
 */
boolean	nfsv3_waitForRexmt[NFS_MAX_V3PROC] = {
		FALSE,		/*0: NULL*/
		FALSE,		/*1: GETATTR*/
		FALSE,		/*2: SETATTR*/
		FALSE,		/*3: LOOKUP*/
		FALSE,		/*4: ACCESS*/
		FALSE,		/*5: READLINK*/
		FALSE,		/*6: READ*/
		FALSE,		/*7: WRITE*/
		TRUE,		/*8: CREATE*/
		TRUE,		/*9: MKDIR*/
		TRUE,		/*10: SYMLINK*/
		TRUE,		/*11: MKNOD*/
		TRUE,		/*12: REMOVE*/
		TRUE,		/*13: RMDIR*/
		TRUE,		/*14: RENAME*/
		TRUE,		/*15: LINK*/
		FALSE,		/*16: READDIR*/
		FALSE,		/*17: READDIRPLUS*/
		FALSE,		/*18: FSSTAT*/
		FALSE,		/*19: FSINFO*/
		FALSE,		/*20: PATHCONF*/
		FALSE		/*21: COMMIT*/
};

/*
 * NFS request keep alive count for nonidempotent operations.
 * The time interval between two retransmitted requests is different. 
 * Each additional retransmission doubles the previous interval.
 * SunOS has longer interval (first re-xmt is 1.25sec) than 
 * Linux (first re-xmt is 0.7sec).
 */
uint32	nfs_reqKeepAliveCount[NFS_MAX_REXMT_REQ] = {
		15,		/*Before 1st rexmt req is received. 1.3sec + 0.2sec*/
		27,		/*Before 2nd rexmt req is received. 2.5sec + 0.2sec*/
		52,		/*Before 3rd rexmt req is received. 5.0sec + 0.2sec*/
		102,	/*Before 4th rexmt req is received. 10sec + 0.2sec*/
		202		/*Before 5th rexmt req is received. 20sec + 0.2sec*/
};


int32
nfs_isReqNonIdempotent( int32 ver, int32 rpc_proc )
{
    if (ver == NFS_VER3)
        return (nfsv3_waitForRexmt[rpc_proc] );
    else
        return (nfsv2_waitForRexmt[rpc_proc] );
}

/*-----------------------------------------------------------------
 * Name:      	nfs_tplAddConnInd	
 *
 * Description:
 *	Add connection indication from the transport layer to
 *	adds a new connection. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_tplAddConnInd( tpl_bindCb_t	*pBind, 	/*ptr to Bind CB */
				   void 		*connIdp, 	/*ptr to newly added connection*/
				   void 		**appHdlpp,	/*ptr to correspond app. handle*/
				   uint32 		raddr,		/*remote IP address*/
				   ushort16 	rport		/*remote port*/
)
{
	nfp_clientCBPtr_t	clientCBp;	
	uint32				connId;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter\n", __FUNCTION__);
#endif

#if defined(NFX_SMP) && defined(NFP_TXRX) && defined(SMP_TXRX_VERIFY)
	if (CM_IS_TXRX_NCPU)
		panic("%s: called in wrong cpu \n", __FUNCTION__);
#endif /* NFX_SMP */

	connId = (uint32)connIdp;

	/*
	 *Check if client control block exist 
	 */
	clientCBp = nfp_getClientCBbyAddr(raddr);
	if (clientCBp == NULL)
	{   
		/*
	 	 *Allocate NFP Client Control Block (CCB)
	 	 */
		clientCBp = nfp_allocClientCB();
		if (clientCBp == NULL)
			return (TPL_ERR_NO_RSRC);
    }
	
	/*
	 *Filled in handle upon return to TPL
	 */
	*appHdlpp = clientCBp;

	/* 
	 * incr conn count in client 
	 */
	clientCBp->nfp_connCnt++;		
	clientCBp->nfp_connId2 = connId;
	clientCBp->nfp_ipAddr  = raddr;
	clientCBp->proto       = NFP_PROTO_NFS;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Exit\n", __FUNCTION__);
#endif

	return (NFX_OK);	
}

/*-----------------------------------------------------------------
 * Name:	nfs_tplDelConnInd	
 *
 * Description: 
 * 	Delete connection indication from the transport layer to delete
 *	an existing connection.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_tplDelConnInd (void *appHdlp, void *connIdp)
{
	nfp_clientCBPtr_t	clientCBp;	

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter\n", __FUNCTION__);
#endif

#if defined(NFX_SMP) && defined(NFP_TXRX) && defined(SMP_TXRX_VERIFY)
	if (CM_IS_TXRX_NCPU)
		panic("%s: called in wrong cpu \n", __FUNCTION__);
#endif /* NFX_SMP */

	clientCBp = (nfp_clientCBPtr_t)appHdlp;
	if ( (clientCBp == NULL) || (connIdp == NULL) )
		return (TPL_ERR_BAD_HDL);

	/*
     * decr. conn count in client
	 */
    clientCBp->nfp_connCnt--;

    /* 
	 * Delete client if all connections
     * assoc. with clients are deleted.
     */
    if (clientCBp->nfp_connCnt <= 0)
		nfp_delClientCB (clientCBp);

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Exit\n", __FUNCTION__);
#endif

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:       	nfs_processRpcAuth
 *
 * Description:  processed rpc auth.
 *
 *-----------------------------------------------------------------
 */
int32
nfs_processRpcAuth(rpcCallHdr_t *pktRpcCallHdrp, 
                   rpcCallHdr_t *newRpcCallp)
{
    char            *msgp;
    uint32          length;
    uint32          fieldLen;
 
    newRpcCallp->rpc_xid = pktRpcCallHdrp->rpc_xid;
    newRpcCallp->rpc_direction = pktRpcCallHdrp->rpc_direction;
    newRpcCallp->rpc_rpcVer = pktRpcCallHdrp->rpc_rpcVer;
    newRpcCallp->rpc_prog = pktRpcCallHdrp->rpc_prog;
    newRpcCallp->progVer = pktRpcCallHdrp->progVer;
    newRpcCallp->rpc_proc = pktRpcCallHdrp->rpc_proc;

    length = 24;
    msgp = (char *)pktRpcCallHdrp + length;
    
    fieldLen = rpc_extractAuthSysInfo(msgp, &newRpcCallp->authFlavor.authSys);
    if (fieldLen == NFX_ERR)
    {
        return -1;
    }   
    length += fieldLen;

    return (length);

}

/*-----------------------------------------------------------------
 * Name:       	nfs_tplRcvProcessPkt 
 *
 * Description:
 *	Process packet received from the transport layer.
 * Created by:  Joseph Vu
 *
 * Date Create: 5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK or NFX_ERROR
	 	 */
nfs_tplRcvProcessPkt(
		nfp_clientCBPtr_t       clientCBp,
		rpcCallHdr_t			*pktRpcCallHdrp,
		pkt_desc_t *descp		/*start of message header*/
)
{
	int32                   rpcHdrSize;
	int32                   error;
	nfp_requestCBPtr_t      reqCBp;
	nfs_request_t           *nfsReqp;
	int32					nfs_outstandingReqs;
	int32					nfs_outstandingReqsSndToFs;

	if (nfs_globalCfgp->nfs_maxReqsThreshold > 0)
	{	
		//Check NFS receive request queue threshold
		//or outstanding request between NFS server
		//the file system
		//Drop received NFS request if system is too 
		//busy	
		nfs_outstandingReqs = 
			(NFP_MAX_REQUEST - nfp_globalCfgp->reqCbCfgList.freeEntriesCnt);

		nfs_outstandingReqsSndToFs = 
			(nfs_globalCfgp->nfs_totalNumOfReqXmtToFs - 
			nfs_globalCfgp->nfs_totalNumOfRespRcvdFromFs);

		if ( (nfs_outstandingReqs > nfs_globalCfgp->nfs_maxReqsThreshold) ||
			(nfs_outstandingReqsSndToFs > nfs_globalCfgp->nfs_maxReqsThreshold) )
		{
			nfs_globalCfgp->nfs_debugPktCounters.nfs_maxReqsDropPktErr++;
			return (NFX_ERR);
		}

	}
	error = NFX_OK;

	/*
	 *Allocate request control block
	 */
	reqCBp = nfp_allocReqCB ();
	if (reqCBp == NULL)
	{
		return (NFX_ERR);
	}
	nfs_globalCfgp->nfs_debugPktCounters.nfs_allocReqCB++;
	nfsReqp = &reqCBp->protocol.nfs_req;

	rpcHdrSize =  nfs_processRpcAuth(pktRpcCallHdrp, &nfsReqp->rpcHdr);
	if (rpcHdrSize <= 0 )	
	{
		nfp_deallocReqCB( reqCBp );
		return (NFX_ERR);
	}

	nfsReqp->rpcCallHdrp = &nfsReqp->rpcHdr;	
	nfsReqp->clientCBp = (void *)clientCBp;
	nfsReqp->reqState = NFS_REQ_INPROG; 
	nfsReqp->reqCBp = (void *)reqCBp;
	nfsReqp->tplConnp = descp->pTplConn;

	nfsReqp->rxPktDesc= descp;

	clientCBp->nfp_localIpAddr = descp->laddr;
#ifdef	NOTUSED
	if (descp->dataSize != 0)
	{
		nfsReqp->dataSize = descp->dataSize;
		nfsReqp->cod = descp->cod;
	}
#endif	//NOTUSED
	/*
	 *Save procedure start message pointer and message length in bytes
	 */
	nfsReqp->procStart = descp->pHdr + rpcHdrSize;
	nfsReqp->procLen = descp->hdrSize - rpcHdrSize;

	/*
	 * Set keep alive counter
	 */
	nfsReqp->reXmtCounter = 0;
	nfsReqp->keepAliveCounter = 0;

	/* Shaun 08/10/02
	 *
	 * Performance tuning:
	 * Add req for Non-idempotent req only.
	 * Use new hash function.
	 */
    if (nfs_isReqNonIdempotent( pktRpcCallHdrp->progVer,
                             	pktRpcCallHdrp->rpc_proc) == TRUE)
    {
        nfsReqp->keepAliveCounter = nfs_reqKeepAliveCount[0];

        nfs_hashInsertReqCb(pktRpcCallHdrp->rpc_xid, nfsReqp );
    }

    /*
     *Add ReqCB to the Global Config Block
     */
    if (nfs_addCBToList(nfs_globalCfgp->pReqCbList,
                        (void *)nfsReqp,
                        NFP_REQCB_NODE)  != NFX_OK)
    {
        return (NFX_ERR);
    }

	/*
	 *Handle special program number 100227 (NFS ACL) 
	 *send by the Sun Solaris client
	 */
	if (pktRpcCallHdrp->rpc_prog == RPC_NFS_ACL_PROG_NUM)
	{
		nfs_procRxNfsAcl(nfsReqp);
		return (NFX_OK);
	}

	/*
	 *Check if client has correct permission
	 */
	if (pktRpcCallHdrp->rpc_proc)	//Don't do checking for NULL  Proc
	{
		error = nfs_checkClientPermission(nfsReqp);
		if (error != NFS_OK)
		{
			switch (error)
			{
				case NFS_FILESYS_NOT_READY:
					//Volume or filesystem is not ready,
					//wait for callback from SSC MOUNTD
					nfsReqp->reqFlag |= NFS_REQ_FILESYS_NOT_READY;
					return (error); 

				default: 
				{
					if (nfs_globalCfgp->nfs_BootUpTimeCnt > NFS_BOOTUP_TIME_CNT)
					{
						nfs_tmpDebug1++;
						goto nfs_tplRcvProcessPktErr;	
					}
					nfs_tmpDebug2++;
					nfsReqp->reqState = NFS_REQ_DONE;
					nfs_msgFreeAll(nfsReqp, 0);			
					return (error); 
				}
			}
		}
	}

#ifdef PKT_USE_TIME_STAMP
	nfsReqp->pTime = descp->pTime;
	descp->pTime = NULL;
#endif

	/*
	 *Call NFS procedure
	 */
	if (pktRpcCallHdrp->progVer == NFS_VER3)
	{
		error = (*(nfsrv3_procs[pktRpcCallHdrp->rpc_proc]))(nfsReqp);
	}
	else
		error = (*(nfsrv2_procs[pktRpcCallHdrp->rpc_proc]))(nfsReqp);

#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp(nfsReqp->pTime, PKT_TIME_IDX_FS_REQ);
#endif

	if (error != NFS_OK)
		goto nfs_tplRcvProcessPktErr;
	

	(descp->tplProt == TPL_PROT_UDP)?
		nfs_globalCfgp->nfs_rxPktCounters.totalUdpCalls++ :
		nfs_globalCfgp->nfs_rxPktCounters.totalTcpCalls++;
	return (NFS_OK);
			

nfs_tplRcvProcessPktErr:
#ifdef PKT_USE_TIME_STAMP
    /* does not go to fp, set time stamp now */
pkt_setTimeStamp(nfsReqp->pTime, PKT_TIME_IDX_FS_RSP);
#endif
	if (nfs_msgSendErrResp(nfsReqp, pktRpcCallHdrp->rpc_xid, error) != NFX_OK)
	{
		nfsReqp->reqState = NFS_REQ_DONE;
		nfs_msgFreeAll(nfsReqp, 0);			
	}

	return (error);
}

/*-----------------------------------------------------------------
 * Name:       	nfs_tplRcvCheckNFSPkt 
 *
 * Description:
 *     	Check received NFS packet. 
 *
 * Created by:  Joseph Vu
 *
 * Date Create: 5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_tplRcvCheckNFSPkt(rpcCallHdr_t *rpcCallp)
{

	/*
	 *Check RPC version number
	 */
	if (rpcCallp->rpc_rpcVer != RPC_VER2)
	{
		return (NFS_INT_RPC_MISMATCH);
	}

	/*
	 *Check program version number
	 */
	if ((rpcCallp->progVer <  NFS_VER2) ||
		(rpcCallp->progVer > NFS_VER3) )
	{
		return (NFS_INT_RPC_PROG_MISMATCH);
	}

	/*
	 *Check procedure number
	 */
	if ( ((rpcCallp->rpc_prog == NFS_VER3) && 
		((rpcCallp->rpc_proc < NFS_MIN_PROC) ||
		(rpcCallp->rpc_proc > NFS_MAX_V3PROC))) ||

		((rpcCallp->rpc_prog == NFS_VER2) &&
		((rpcCallp->rpc_proc < NFS_MIN_PROC) ||
		(rpcCallp->rpc_proc > NFS_MAX_V2PROC)))
 		)
	{
		return (NFS_INT_RPC_PROC_UNAVAIL);
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nfs_tplRcvPktInd 
 *
 * Description: 
 *	Receive packet indication from the transport layer for processing.
 *
 * Created by:  Joseph Vu
 *
 * Date Create:	5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_tplRcvPktInd(void *appHdlp, pkt_desc_t *descp)
{
	int32				error;
	int32				rc;
	rpcCallHdr_t		*rpcCallp;
	nfp_clientCBPtr_t	clientCBp;	
	nfs_request_t		*nfsReqp;

#if defined(NFX_SMP) && defined(NFP_TXRX) && defined(SMP_TXRX_VERIFY)
	if (CM_IS_TXRX_NCPU)
		panic("%s: called in wrong cpu \n", __FUNCTION__);
#endif /* NFX_SMP */
	
	if ((appHdlp == NULL) || (descp == NULL) ||
		(descp->pHdr == NULL) )
		return (TPL_FREE_PKT);


    if (!(descp->flags & PKT_DESC_INUSE))
    {
        panic("%s: bad pkt=%p flags=%x\n",
            __FUNCTION__,descp, descp->flags);
    }

#ifdef PKT_USE_TIME_STAMP
    pkt_setTimeStamp(descp->pTime, PKT_TIME_IDX_NFS_RX);
#endif

	error = NFX_OK;
	//Update access time for UDP connection
	if (descp->tplProt == TPL_PROT_UDP)
	{
		int32				aTime;

		aTime = nfs_utilGetTimeOfDayInSecs();
		tpl_connUpdateActivityTime(descp->pTplConn, aTime);
	}

	/*
	 * Perform Net to Host conversion.
	 */
	xdr_swap(descp->pHdr,(sizeof(rpcCallHdr_t)-sizeof(rpc_authSysInfo_t)), 0);

	if (trace_enable != 0)
		nfs_rxDecodePkt(descp);

	rpcCallp = (rpcCallHdr_t *)descp->pHdr;

	clientCBp = (nfp_clientCBPtr_t)appHdlp;

	if (nfp_validateClientCB(clientCBp) != NFX_OK)
	{
		panic("nfs_tplRcvPktInd-Error:  bad client control block!\n");
		return (TPL_FREE_PKT);
	} 

	rc = nfs_tplRcvCheckNFSPkt(rpcCallp);
	if (rc != NFX_OK)
	{
		error = mnt_mapToNFSError(rc);
		nfs_globalCfgp->nfs_debugPktCounters.nfs_invalidTplPktRcvd++;
		goto nfs_rcvPktIndErr;
	}	
	/*
	 *Check Duplicate request cache if this this a re-xmit req
	 *Duplicate request cache handle re-tranmission request from the client.
	 *reqState: 
	 *	NFS_REQ_INPROG:		Request is in progress and has not been replied to, 
	 *						we ignore the request.
	 *  NFS_REQ_SEND_TO_FS	Request has been sent to file system and awaiting 
	 *						for a response
	 *	NFS_REQ_REPLIED:	Request has been replied to the client recently, 
	 *						then it is likely that
	 *						the retransmitted request and its reply crossed in 
	 *						transit, we ignore the request.
	 *	NFS_REQ_DONE:		We have completed this request.  Send cached reply 
	 *						to the client.
	 *	NFS_REQ_REXMIT:		retransmitted the request.
	 *
  	 */

	nfsReqp = NULL;
	if (nfs_isReqNonIdempotent(rpcCallp->progVer, rpcCallp->rpc_proc) == TRUE)
	{
		nfsReqp = nfs_hashLookupReqCb( rpcCallp->rpc_xid );
	}

	if (nfsReqp != NULL)
	{
		/*
 	 	 *Found request, check state of the request 
 	 	 */
		if (nfsReqp->reqState == NFS_REQ_DONE)
		{
			uint32	kaCount;

			/*
		 	 *reqState must be done.
 	 	 	 *Return reply to client
 	 	 	*/
			//since COD info are not valid anymore,
			//Do not retransmit
			//For NFS V2/V3 Read is procedure num 6
			if (rpcCallp->rpc_proc !=  NFS_V3_PROC_READ)
			{
				nfs_msgXmtPkt((pkt_desc_t *)nfsReqp->pktDescp,
							nfsReqp);
				nfsReqp->reqState =  NFS_REQ_REXMIT;

				//Reset keep alive counter
				nfsReqp->reXmtCounter++;
				if (nfsReqp->reXmtCounter >= NFS_MAX_REXMT_REQ)
				{
					//Reach the max number of re-xmt
					//Ready to delete the request control block
					kaCount = 0;
				}
				else
				{
					if (rpcCallp->progVer == NFS_VER3)
					{
						if (nfsv3_waitForRexmt[rpcCallp->rpc_proc] == TRUE)
						{
							kaCount = nfs_reqKeepAliveCount[nfsReqp->reXmtCounter];
						}
					}
					else
					{
						if (nfsv3_waitForRexmt[rpcCallp->rpc_proc] == TRUE)
						{
							kaCount = nfs_reqKeepAliveCount[nfsReqp->reXmtCounter];
						}
					}
				}
				nfsReqp->keepAliveCounter = kaCount;
			}
			nfs_updateXmtStatCounter(nfsReqp);
		}
		/*
		 *request in progress or has been replied recently.  Ignore request.
		 */
		return (TPL_FREE_PKT);
	}


	error = nfs_tplRcvProcessPkt(clientCBp, rpcCallp, descp);
	if (error == NFS_OK)
		return (TPL_FREE_PKT);



nfs_rcvPktIndErr:

	/*
	 *Handle error.  Return a error reply &
	 *free all alloc memory
	 *Call function nfs_msgSendErrResp() to send a error reply
	 *pkt to client
	 */
	if (error == NFS_FILESYS_NOT_READY)
	{
		//Wait If volume or filesystem is not ready 
		return (TPL_HOLD_PKT);
	}
	(descp->tplProt == TPL_PROT_UDP)?
		nfs_globalCfgp->nfs_rxPktCounters.totalBadUdpCalls++ :
		nfs_globalCfgp->nfs_rxPktCounters.totalBadTcpCalls++;
	return (TPL_FREE_PKT);
}

/*-----------------------------------------------------------------
 * Name: nfs_tplXmtCmpInd       
 *
 * Description: 
 *	Transmit packet complete indication from the transport layer.
 *	 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_tplXmtCmpInd (void *appHdlp, pkt_desc_t *descp)
{
//	nfp_clientCBPtr_t		clientCBp;
	nfs_request_t 			*nfsReqp;

#if defined(NFX_SMP) && defined(NFP_TXRX) && defined(SMP_TXRX_VERIFY)
	if (CM_IS_TXRX_NCPU)
		panic("%s: called in wrong cpu \n", __FUNCTION__);
#endif /* NFX_SMP */

	nfs_globalCfgp->nfs_debugPktCounters.tpl_xmtPktCmpInd++;
	if (descp == NULL)
	{
		panic("nfs_tplXmtCmpInd: bad appHdlp/descp!\n");
	}

	/*
	 *Retrieve our NFS request from descp appInfo 
	 */
	nfsReqp = descp->appInfo;
	if (nfp_validateNfsReqCB(nfsReqp) != NFX_OK )
	{
		/*
		 *ERROR: No corresponding reqCB with the packet descriptor
		 */	
		//panic("nfs_tplXmtCmpInd 2: Bad nfs RequestCB(nfsReqp)!\n");
		
		//return (TPL_FREE_PKT);
	}

#ifdef PKT_USE_TIME_STAMP
	pkt_setNfsProcTime( descp->pTime, nfsReqp->reqType);
#endif



	/* Shaun 08/10/22
	 *
	 * Peformance tuning:
	 * For Idempotent requests, free assoc. resource.
	 * Read Xmt Cmp Ind will be sent in nfs_msgFreeAll.
	 */
    if ((nfsReqp->rpcCallHdrp) &&
        (nfs_isReqNonIdempotent(nfsReqp->rpcCallHdrp->progVer,
                            nfsReqp->rpcCallHdrp->rpc_proc) != TRUE))
    {
        if (nfsReqp->rpcCallHdrp->rpc_proc == 0)
        {
            if (descp->tplProt == TPL_PROT_UDP)
                tpl_delConnReq(descp->pTplConn);
        }

        nfsReqp->reqState = NFS_REQ_DONE;
        nfs_msgFreeAll( nfsReqp, 1);

        return (TPL_HOLD_PKT);
    }

#ifdef NOTUSED
	/*
	 *If request was a read, we need to transmit complete to file system so it
	 *can free the buffer bak to the system.
	 */
	if ((nfsReqp->reqType == NFS_READ) && 
		(nfsReqp->reqState != NFS_REQ_REXMIT))
	{

		if (nfsReqp->readXmitCompInd.bytesRead != 0)
		{
	    	nfs_msgXmtComplete(nfsReqp, descp);	
			nfs_globalCfgp->nfs_debugPktCounters.nfs_readXmtCmpInd++;
		}
	}
	else
	{
		/* 
		 * Shaun 03/25/02
		 *
		 * Fix for #273. Workaround for #314
		 * Remove tpl connection associated 
		 * with NFS NULL Procedure.
		 */
		if ((nfsReqp->rpcCallHdrp) && 
			(nfsReqp->rpcCallHdrp->rpc_proc == 0))
		{
			if (descp->tplProt == TPL_PROT_UDP)
				tpl_delConnReq(descp->pTplConn);
		}
	}
#endif

	/*
	 *Set reqState to Done.  We will hold the pkt for now
	 *for the duplicate request cache.	This pkt will be free later. 
	 */
	nfsReqp->reqState = NFS_REQ_DONE;

	return (TPL_HOLD_PKT);

}


/*-----------------------------------------------------------------
 * Name: nfs_tplBindReq
 *
 * Description:
 * Binds NFS to Transport Layer (UDP/TCP).  
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_tplBindReq(uint32 ipaddr)
{
	int32			bindOpt;
	ushort16		protocol;
	tpl_bindCb_t	bindCB;
	tpl_bindCb_t	*bindCBp;
	tpl_bindCb_t	*tplRspBindp;

#if defined(NFX_SMP) && defined(NFP_TXRX) && defined(SMP_TXRX_VERIFY)
	if (CM_IS_TXRX_NCPU)
		panic("%s: called in wrong cpu \n", __FUNCTION__);
#endif /* NFX_SMP */

	protocol = TPL_PROT_UDP;
	while (1)
	{
		
		memset(&bindCB, 0, sizeof(tpl_bindCb_t));
		/*
		 * Bind NFS server on UDP or TCP.
		 * fill in all fields in tpl_bindCB_t
		 */
		bindOpt = TPL_BIND_SERVER | TPL_BIND_USE_RPC;
		bindCBp = &bindCB;
		bindCBp->laddr         = ipaddr;	
		bindCBp->lport         = htons(NFS_PORT);
		bindCBp->tplProt       = protocol;	
		bindCBp->ul_addConnInd = nfs_tplAddConnInd;
		bindCBp->ul_delConnInd = nfs_tplDelConnInd;
		bindCBp->ul_rcvPktInd  = nfs_tplRcvPktInd;
		bindCBp->ul_xmtCmpInd  = nfs_tplXmtCmpInd;

		if (tpl_bindReq(bindCBp, &tplRspBindp, bindOpt) != NFX_OK)
		{
			/*
			 *TODO: handle error
			 */
			return (NFX_ERR);

		}
		if (protocol == TPL_PROT_TCP)
			break;

		protocol = TPL_PROT_TCP;
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nfs_tplProcessNfsReq 
 *
 * Description:
 * Process a  NFS procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nfs_tplProcessNfsReq(nfs_request_t *nfsReqp)
{
	int32			error;
	pkt_desc_t      *descp;

	if (nfp_validateNfsReqCB(nfsReqp) != NFX_OK )
	{
		//panic ("nfs_tplProcessNfsReq: Bad NFS request control block!\n");
		//TODO: Increment debug counter
		return (NFX_ERR);
	}
	
	/*
	 * Shaun 03/15/02
	 * nfsReq may have already been 
	 * deleted (rpcCallHdrp is NULL).
	 * Just return.
	 */
	descp = (pkt_desc_t      *)nfsReqp->rxPktDesc;
	if (nfsReqp->rpcCallHdrp == NULL)
	{
		nfsReqp->reqState = NFS_REQ_DONE;
		nfs_msgFreeAll(nfsReqp, TRUE);
		if (descp)
		{
			PKT_FREE_PACKET(descp);
			nfsReqp->rxPktDesc = NULL;			
		}
        return (NFX_ERR);
	}
    if (!(descp->flags & PKT_DESC_INUSE))
    {
		nfs_globalCfgp->nfs_debugPktCounters.tpl_badDesc++;
		goto nfs_tplProcessNfsReqErr;
    }

	/*
	 *Call NFS procedure
	 */
	if (nfsReqp->rpcCallHdrp->progVer == NFS_VER3)
		error = (*(nfsrv3_procs[nfsReqp->rpcCallHdrp->rpc_proc]))(nfsReqp);
	else
		error = (*(nfsrv2_procs[nfsReqp->rpcCallHdrp->rpc_proc]))(nfsReqp);
	if (error != NFS_OK)
		goto nfs_tplProcessNfsReqErr;
	if (descp)
	{
		PKT_FREE_PACKET(descp);
		nfsReqp->rxPktDesc = NULL;			
	}
	return (NFS_OK);


nfs_tplProcessNfsReqErr:
	if (nfs_msgSendErrResp(nfsReqp, nfsReqp->rpcCallHdrp->rpc_xid, error) != NFX_OK)
	{
		nfsReqp->reqState = NFS_REQ_DONE;
		nfs_msgFreeAll(nfsReqp, TRUE);
	}
	if (descp)
	{
		PKT_FREE_PACKET(descp);
		nfsReqp->rxPktDesc = NULL;			
	}
	return (NFX_ERR);
}
#endif  //NFP_TXRX
