/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	  nfsv2-msg.h
 *
 * RCS:  $Id: nfsv2-msg.h,v 1.1 2002/09/23 23:30:07 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Contains function prototypes for routines that
 *				handle NFS V2 response from File System. 
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFSV2_MSG_H
#define _NFSV2_MSG_H

/*-----------------------------------------------------------------
 * Name:		nfsv2_msgGetAttrResp 
 *
 * Description: Map file system Getattr response message to a NFS 
 *				reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *NFX_OK or NFX_ERR
         */
nfsv2_msgGetAttrResp( int32 		status, 	/*operation result status*/
					  eee_descBuf_t *bd,		/*file system resp. msg*/
					  char 			*txMsgp,	/*Pointer to tx msg buffer*/
					  int32 		*txMsgLen,  /*current length of message*/
					  nfs_request_t *nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgLookupResp	
 *
 * Description: Map File System Lookup response to a NFS V2 reply. 
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgLookupResp(int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen, 	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/ 

/*-----------------------------------------------------------------
 * Name: 		nfsv2_msgSetAttrResp 
 *
 * Description: Map File System SetAttr response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgSetAttrResp(int32 			status,		/*operation result status*/
					 eee_descBuf_t	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Pointer to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:		nfsv2_msgSymlinkResp 
 *
 * Description: Map File System Symlink response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgSymlinkResp(int32 			status,		/*operation result status*/
					 eee_descBuf_t	*bd,		/*file system resp. msg*/
					 char 			*txMsgp, 	/*pointer to tx Msg buffer*/
					 int32 			*txMsgLen, 	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:      	nfsv2_msgHardLinkResp 
 *
 * Description: Map File System Link response to a NFS V2 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgHardLinkResp(int32			status,		/*operation result status*/
					  eee_descBuf_t *bd,      	/*file system resp. msg*/
					  char 			*txMsgp,	/*pointer to tx msg buffer*/
					  int32 		*txMsgLen,	/*current length of message*/
					  nfs_request_t *nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name: 		nfsv2_msgReadlinkResp 
 *
 * Description: Map File System Readlink response to a NFS V2 reply. 
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK or NFX_ERR
	 	 */
nfsv2_msgReadlinkResp(int32 		status,		/*operation result status*/
					  eee_descBuf_t *bd,		/*file system resp. msg*/
					  char 			*txMsgp, 	/*Pointer to tx msg buffer*/
					  int32 		*txMsgLen, 	/*length of msg buffer*/
					  nfs_request_t *nfsReqp );	/*req control block*/

/*-----------------------------------------------------------------
 * Name:        nfsv2_msgCreateResp
 *
 * Description: Map File System Create response to a NFS V2 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *NFS_OK or NFX_ERR
         */
nfsv2_msgCreateResp(int32 			status,		/*Operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Poiner to tx msg buffer*/
					int32 			*txMsgLen,	/*length of the message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:		nfsv2_msgRemoveResp 
 *
 * Description: Map File System Remove response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgRemoveResp(int32 			status,		/*Operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:      	nfsv2_msgRmdirResp 
 *
 * Description: Map File System RMDIR response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgRmdirResp( int32 			status,		/*Operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t	*nfsReqp );	/*reqCB*/
       

/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgMkdirResp 
 *
 * Description: Map File System Mkdir response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgMkdirResp(int32 			status,		/*operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen, 	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:        nfsv2_msgReaddirResp
 *
 * Description: Map File System Readdir response to a NFS V2 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32	/*
		 *NFX_OK  or NFX_ERR
	 	 */
nfsv2_msgReaddirResp(int32 			status,		/*Operation result status*/
					 eee_descBuf_t 	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Pointer to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:      	nfsv2_msgRenameResp	
 *
 * Description: Map File System Rename response to a NFS V2 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgRenameResp(int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/


/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgStatfsResp 
 *
 * Description: Map the file system control message to NFS V2 STATFS 
 *
 * Created by:  Joseph Vu
 *
 * Date Created: 5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgStatfsResp (int32 			status,		/*operation result status*/
					 eee_descBuf_t 	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Ptr. to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgWriteResp 
 *
 * Description: Map File System Write response to a NFS V2 Write .
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgWriteResp (int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:	nfsv2_msgReadResp 
 *  
 * Description: Map File System Read response to a NFS V2 Read procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgReadResp(int32 			status,		/*operation result status*/
				  eee_descBuf_t 	*bd,		/*file system resp. msg*/
				  char 				*txMsgp,	/*Pointer to tx msg buffer*/
				  int32 			*txMsgLen,	/*current length of message*/
				  nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:nfsv2_msgErrResp
 *
 * Description:
 *  Received an invalid message type received from the file system,
 *  Set request state done to clean up request and return.
 *
 *-----------------------------------------------------------------
 */

int32
nfsv2_msgErrResp( int32 		status,		/*operation result status*/
				  eee_descBuf_t *bd,		/*file system resp. msg*/
				  char 			*txMsgp,	/*Pointer to tx msg buffer*/
				  int32 		*txMsgLen,	/*current length of message*/
				  nfs_request_t *nfsReqp );	/*request control block*/
#endif /* _NFSV2_MSG_H */
