/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nlm-debug.c
 *
 * RCS:  $Id: nlm-trace.c,v 1.2 2002/09/19 02:39:42 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *  Contain Network Lock Manager (NLM) trace functions 
 *  and debug utility functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/nfs-api.h>

#define NLM_STR					"NLM"
#define	NLM_BAD_RPC_HDR_ERR_STR	"[%s]Bad RPC Hdr\n"
#define	NLM_BAD_RPC_MISMATCH_ERR_STR "[%s]RPC mistmatch\n"
#define	NLM_BAD_RPC_PROG_MISMATCH_ERR_STR "[%s]RPC Prog mistmatch\n"
#define	NLM_BAD_RPC_PROG_UNAVAIL_ERR_STR "[%s]RPC Proc Unvail\n"

#define     NLM_DECODE_RX           1   
#define     NLM_DECODE_XMT          2  

static char *nlm_procs[] =   {
    "\tProc 0 = (NLM_NULL)\n",
    "\tProc 1 = (NLM_TEST)\n",
    "\tProc 2 = (NLM_LOCK)\n",
    "\tProc 3 = (NLM_CANCEL)\n",
    "\tProc 4 = (NLM_UNLOCK)\n",
    "\tProc 5 = (NLM_GRANTED)\n",
    "\tProc 6 = (NLM_TEST_MSG)\n",
    "\tProc 7 = (NLM_LOCK_MSG)\n",
    "\tProc 8 = (NLM_CANCEL_MSG)\n",
    "\tProc 9 = (NLM_UNLOCK_MSG)\n",
    "\tProc 10 = (NLM_GRANTED_MSG)\n",
    "\tProc 11 = (NLM_TEST_RES)\n",
    "\tProc 12 = (NLM_LOCK_RES)\n",
    "\tProc 13 = (NLM_CANCEL_RES)\n",
    "\tProc 14 = (NLM_UNLOCK_RES)\n",
    "\tProc 15 = (NLM_GRANTED_RES)\n",
    "\tProc 16 = (NOT_ASSIGNED)\n",
    "\tProc 17 = (NOT_ASSIGNED)\n",
    "\tProc 18 = (NOT_ASSIGNED)\n",
    "\tProc 19 = (NOT_ASSIGNED)\n",
    "\tProc 20 = (NLM_SHARE)\n",
    "\tProc 21 = (NLM_UNSHARE)\n",
    "\tProc 22 = (NLM_NM_LOCK)\n",
    "\tProc 23 = (NLM_FREE_ALL)\n",
};

/*
 *Externs
 */
int32 
nfs_txExtractLongVal(char *nfsMsgp,
	uint32 *value              
);

int32  
nfs_txDecodeString (char *sourceBufp,    
	char *destNamep,  
	ushort16 *nameLen);

int32 
nfs_rxDecodeFhandle(char *nfs_msgp,
	int32 nfs_version);

uint32	
rpc_getHdrSize(rpcCallHdr_t *pktRpcCallHdrp);

int32 nlm_tplRcvCheckPkt(rpcCallHdr_t *rpcCallp);

int32 nfs_txExtractLongLongVal(char *nfsMsgp, 
	uint64 *value
);

/*-----------------------------------------------------------------
 * Name:  nlm_getStatus 		 
 *
 * Description:
 *		Returns string of reply status.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
char *nlm_getStatus(int32 replyStatus)
{
	switch (replyStatus)
	{
		case NLM_STAT_GRANTED:
			return ("(NLM_GRANTED)");
		case NLM_ERR_DENIED:
			return ("(NLM_DENIED)");
		case NLM_ERR_DENIED_NOLOCKS:
			return ("(NLM_DENIED_NOLOCKS)");
		case NLM_ERR_BLOCKED:
			return ("(NLM_BLOCKED)");
		case NLM_ERR_DENIED_GRACE_PERIOD:
			return ("(NLM_DENIED_GRACE_PERIOD)");
		case NLM_ERR_DEADLCK:
			return ("(NLM_DEADLOCK)");
		case NLM_ERR_ROFS:
			return ("(Read-only File System)");
		case NLM_ERR_STALE_FH:
			return ("(NLM_STALE_FH)");
		case NLM_ERR_FBIG:
			return ("(Offset/length exceeds range supportted by ther server)");
		case NLM_FAILED:
			return ("(Generic: unknown error)");
	}

	return ("(Invalid error reply Status)");
	
}

/*-----------------------------------------------------------------
 * Name: nfs_decodeSvid	 
 *
 * Description:
 *		Decode/Encode Svid field.
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeSvid(char *msgp, int32 direction)
{
	uint32 		svid;

	printf("\n\tSvid = ");
	if (direction == NLM_DECODE_RX)
		nfs_extractLongVal(msgp, &svid);
	else
		nfs_txExtractLongVal (msgp, &svid);
	printf("%d\n", svid);

	return (sizeof(uint32));
}

/*-----------------------------------------------------------------
 * Name: nfs_decodeLockOwnerIdent	 
 *
 * Description:
 *		Decode/Encode lock owner Id field.
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfs_decodeLockOwnerIdent(char *msgp, int32 direction)
{
	int32		length;
	ushort16	bufLen;
	static char	buf[NLM_MAX_NAME];
	
	//decode identity of the lock owner
	memset(buf, 0, sizeof(buf));
	bufLen = 0;
	if (direction == NLM_DECODE_RX)
		length =  nfs_extractString(msgp, buf, 
					(ushort16 *)&bufLen, 
					NLM_MAX_NAME);
	else
		length = nfs_txDecodeString (msgp, buf, &bufLen); 
	if (length == NFX_ERR)
	{
		printf("%s: Failure to decode lock owner identity!\n",
						__FUNCTION__);
		return (NFX_ERR);
	}
	printf("\tlock owner = \t%s\n", buf);
	return (length);

}

/*-----------------------------------------------------------------
 * Name:nlm_decodeFileOffset	 
 *
 * Description:
 *		Decode/Encode lock file offset field.
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_decodeFileOffset(char *msgp, int32 nfs_version, int32 direction)
{
	uint32		offset32;
	uint64		offset64;

	printf("\n\tOffset = ");

	if (nfs_version == NFS_VER2)
	{
		if (direction == NFS_DECODE_RX)
			nfs_extractLongVal(msgp, &offset32);
		else
			nfs_txExtractLongVal(msgp, &offset32);
	
		printf("%d\n", offset32);
		return (sizeof(uint32));
	}
	else
	{
		if (direction == NFS_DECODE_RX)
			nfs_extractLongLongVal(msgp, &offset64);
		else
			nfs_txExtractLongLongVal(msgp, &offset64);
		printf("%lld\n", offset64);
		return (sizeof(uint64));
	}

}

/*-----------------------------------------------------------------
 * Name:nlm_decodeCountField		
 *
 * Description:
 *		Decode/Encode lock file count or length field.
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32 
nlm_decodeCountField(char *msgp, int32 nfs_version, int32 direction)
{

	uint32		len32;
	uint64		len64;

	printf("\n\tCount = ");

	if (nfs_version == NFS_VER2)
	{
		if (direction == NFS_DECODE_RX)
			nfs_extractLongVal(msgp, &len32);
		else
			nfs_txExtractLongVal(msgp, &len32);
	
		printf("%d\n", len32);
		return (sizeof(uint32));
	}
	else
	{
		if (direction == NFS_DECODE_RX)
			nfs_extractLongLongVal(msgp, &len64);
		else
			nfs_txExtractLongLongVal(msgp, &len64);
		printf("%lld\n", len64);
		return (sizeof(uint64));
	}
}

/*-----------------------------------------------------------------
 * Name:   	nlm_decodeRpcHdr 
 *
 * Description:
 *		Print the RPC header contents.
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
void
nlm_decodeRpcHdr(rpcCallHdr_t *rpcCallp)
{
	printf("\n\t----- RPC Header -----\n");

	printf("\tXID: 0x%08x\n", rpcCallp->rpc_xid);
	printf("\tDirection: %s\n", (rpcCallp->rpc_direction == 0)? "Call": "Reply");	
	printf("\tRPC Version: %d \n", rpcCallp->rpc_rpcVer);
	printf("\tProgram number: %d (%s)\n",
			rpcCallp->rpc_prog, nfs_decodeGetProgNumStr(rpcCallp->rpc_prog ));

	printf("\tProgram Version: %s\n", (rpcCallp->progVer == NLM_VER1)? "V1" :
                        (rpcCallp->progVer == NLM_VER3)? "V3":
                        (rpcCallp->progVer == NLM_VER4)? "V4": "Invalid");
    printf("\tProcedure number: %d \n", rpcCallp->rpc_proc);

	nfs_rxDecodeAuthSysHostName(rpcCallp);

	
}

/*-----------------------------------------------------------------
 * Name:  nlm_decodeRpcHdrErr 	
 *
 * Description:
 *		Print the RPC header decode type error.
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
void nlm_decodeRpcHdrErr(int32 error)
{
	switch (error)
	{
		case NFS_INT_RPC_MISMATCH:
			printf(NLM_BAD_RPC_MISMATCH_ERR_STR,NLM_STR);
			break;
		case NFS_INT_RPC_PROG_MISMATCH:
			printf(NLM_BAD_RPC_PROG_MISMATCH_ERR_STR,NLM_STR);
			break;
		case NFS_INT_RPC_PROC_UNAVAIL:
			printf(NLM_BAD_RPC_PROG_UNAVAIL_ERR_STR,NLM_STR);
			break;
	}
	return;
}
/*-----------------------------------------------------------------
 * Name:nlm_decodeCookieObj  	 
 *
 * Description:
 *		Decode NLM cookie field.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_decodeCookieObj(char *msgp, int32 direction)
{
	uint32		*msg32p;
	int32		length;
	int32		iteration;
	uint32		cookieLen;
	char		cookie[NLM_MAX_NAME];
	uint32		totalLen = 0;
	int32		count, i;

	printf("\tCookie = \t");

	msg32p = (uint32 *)msgp;
	memset(cookie, 0, sizeof(cookie));

	if (direction == NLM_DECODE_RX)
	{
		length = nfs_extractLongVal(msgp, &cookieLen);
	}
	else
	{
		//length = nfs_txDecodeString(msgp, cookie, &cookieLen);
		//length = nfs_txExtractLongVal(msgp, &cookieLen);
		length = 4;				
   		cookieLen = (*(uint32 *)msgp);
	}

	if ( (cookieLen < 0) || (cookieLen > NLM_MAX_NAME ) )
		return (NFX_ERR);

	count = 0;
	iteration = (cookieLen >> 2) + 1;
	for (i = 0; i < iteration; i++, msg32p++)
	{
		printf("%08x ", *msg32p);
		count++;
		if (count == 4)
		{
			printf("\n\t\t\t");
			count = 0;
		}
	}
	printf("\n");
	totalLen += length + cookieLen;

	
	return (totalLen);
}

/*-----------------------------------------------------------------
 * Name:nlm_decodeExclusiveObj		
 *
 * Description:
 *		Decode NLM Exclusive field.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_decodeExclusiveObj(char *msgp, int32 direction)
{
	uint32		exclusiveFlag = 0;
	int32		length;

	if (direction == NLM_DECODE_RX)
		length = nfs_extractLongVal(msgp, &exclusiveFlag);
	else
		length = nfs_txExtractLongVal(msgp, &exclusiveFlag);

	printf("\tExclusive = \t%s\n",
		(exclusiveFlag != 0)? "True": "False");
	
	return (length);

}

/*-----------------------------------------------------------------
 * Name:	nlm_decodeReclaimObj		
 *
 * Description:
 *		Decode NLM ReclaiReclaimm field.	
 *		True if the client is attempting to restablish a lock
 *		during the server's grace period following a crash.
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_decodeReclaimObj(char *msgp)
{
	uint32		reclaim = 0;
	int32		length;


	length = nfs_extractLongVal(msgp, &reclaim);
	printf("\tReclaim = \t%s\n",
		(reclaim != 0)? "True": "False");
	
	return (length);
}

/*-----------------------------------------------------------------
 * Name:nlm_decodeStateObj			
 *
 * Description:
 *	Decode NLM state field.	
 *	The current state of the client's Status Monitor state number.
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_decodeStateObj(char *msgp)
{
	uint32		state = 0;
	int32		length;

	length = nfs_extractLongVal(msgp, &state);
	printf("\tState = \t%s\n",
		(state != 0)? "True": "False");
	
	return (length);
}

/*-----------------------------------------------------------------
 * Name:	nlm_decodeBlockObj		
 *
 * Description:
 *	Decode NLM Block field.
 *	A LOCK request cannot be granted if a conflicting lock
 *  is already held.  If block is set to true the client indicates
 *	that it expects the call to block on the server until the request
 *	can be granted.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_decodeBlockObj(char *msgp)
{
	uint32		block = 0;
	int32		length;

	length = nfs_extractLongVal(msgp, &block);
	printf("\tBlock = \t%s\n",
		(block != 0)? "True": "False");
	
	return (length);
}


/*-----------------------------------------------------------------
 * Name:nlm_decodeAlockObj		
 *
 * Description:
 *		Decode NLM alock object.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_decodeAlockObj(char *msgp,
					int32 nlm_version
)
{
	int32		totalLen;
	int32		nfs_version;
	int32		length;
	ushort16	bufLen;
	static char	buf[NLM_MAX_NAME];

	memset(buf, 0, sizeof(buf));
	bufLen = 0;
	totalLen = 0;

	//decode caller name
	length =  nfs_extractString(msgp, buf, 
					(ushort16 *)&bufLen, 
					NLM_MAX_NAME);
	if (length == NFX_ERR)
	{
		printf("%s: Failure to decode caller_name!\n",__FUNCTION__);
		return (NFX_ERR);
	}
	printf("\tCaller = \t%s\n", buf);
	

	//update message pointer
	totalLen += length;
	msgp += length;

    //Map NLM version to comparable
    //NFS version
	//nfs_version = nlm_utilMapToCompNFSVer(nlm_version);
	NLM_MAP_TO_COMP_NFS_VER(nfs_version, nlm_version);

	//decode filehanle
	length = nfs_rxDecodeFhandle(msgp, nfs_version);
	if (length == NFX_ERR)
	{
		printf("%s: Failure to decode fileHandle!\n",__FUNCTION__);
		return (NFX_ERR);
	}

	//update message pointer
	msgp += length;
	totalLen += length;

	//decode identity of the lock owner
	length = nfs_decodeLockOwnerIdent(msgp, NLM_DECODE_RX);
	if (length == NFX_ERR)
	{
		printf("%s: Failure to decode lock owner!\n",__FUNCTION__);
		return (NFX_ERR);
	}
	//update message pointer
	msgp += length;
	totalLen += length;

	//decode Svid
	length = nfs_decodeSvid(msgp, NLM_DECODE_RX);
	//update message pointer
	msgp += length;
	totalLen += length;
	
	//decode offset of the locked region
	length = nlm_decodeFileOffset(msgp, nfs_version, NLM_DECODE_RX);
	//update message pointer
	msgp += length;
	totalLen += length;

	//decode len or extent of the locked region
	length = nlm_decodeCountField(msgp, nfs_version, NFS_DECODE_RX);
	totalLen += length;

	return (totalLen);
}

/*-----------------------------------------------------------------
 * Name:   nlm_rxDecodeTestProc 
 *
 * Description:
 *		Decode NLM received TEST packet.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_rxDecodeTestProc(rpcCallHdr_t *rpcCallp)
{
	uint32			rpcHdrSize;
	char			*msgp;
	int32			length;

	
	//Skip the RPC Header
	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;

	//Decode Rx Cookie
	length = nlm_decodeCookieObj(msgp, NLM_DECODE_RX);
	if (length == NFX_ERR)
		return (NFX_ERR);			

	//update msg field
	msgp += length;

	//Decode Exclusive field
	length = nlm_decodeExclusiveObj(msgp, NLM_DECODE_RX);
	//update msg field
	msgp += length;

	//Decode alock field
	length = nlm_decodeAlockObj(msgp, rpcCallp->progVer);
	if (length == NFX_ERR)
		return (NFX_ERR);

	return (NFX_OK);	
}

/*-----------------------------------------------------------------
 * Name:   nlm_rxDecodeLockProc 
 *
 * Description:
 *		Decode NLM received LOCK packet.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_rxDecodeLockProc(rpcCallHdr_t *rpcCallp)
{
	int32			length;
	uint32			rpcHdrSize;
	char			*msgp;

	
	//Skip the RPC Header
	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;

	//Decode Rx Cookie
	length = nlm_decodeCookieObj(msgp, NLM_DECODE_RX);
	if (length == NFX_ERR)
		return (NFX_ERR);			
	//update msg field
	msgp += length;

	//Decode Block field
	length = nlm_decodeBlockObj(msgp);
	//update msg field
	msgp += length;

	//Decode Exclusive field
	length = nlm_decodeExclusiveObj(msgp, NLM_DECODE_RX);
	//update msg field
	msgp += length;

	//Decode alock field
	length = nlm_decodeAlockObj(msgp, rpcCallp->progVer);
	if (length == NFX_ERR)
		return (NFX_ERR);
	//update msg field
	msgp += length;

	//Decode reclaim field
	length = nlm_decodeReclaimObj(msgp);
	//update msg field
	msgp += length;

	//Decode state field
	nlm_decodeStateObj(msgp);	

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   nlm_rxDecodeCancelProc 
 *
 * Description:
 *		Decode NLM received CANCEL packet.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_rxDecodeCancelProc(rpcCallHdr_t *rpcCallp)
{
	int32			length;
	uint32			rpcHdrSize;
	char			*msgp;

	//Skip the RPC Header
	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;

	//Decode Rx Cookie
	length = nlm_decodeCookieObj(msgp, NLM_DECODE_RX);
	if (length == NFX_ERR)
		return (NFX_ERR);			
	//update msg field
	msgp += length;

	//Decode Block field
	length = nlm_decodeBlockObj(msgp);
	//update msg field
	msgp += length;

	//Decode Exclusive field
	length = nlm_decodeExclusiveObj(msgp, NLM_DECODE_RX);
	//update msg field
	msgp += length;

	//Decode alock field
	length = nlm_decodeAlockObj(msgp, rpcCallp->progVer);
	if (length == NFX_ERR)
		return (NFX_ERR);

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:  nlm_rxDecodeUnlockProc	 
 *
 * Description:
 *		Decode NLM received Unlock packet.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_rxDecodeUnlockProc(rpcCallHdr_t *rpcCallp)
{
	uint32			rpcHdrSize;
	char			*msgp;
	int32			length;

	//Skip the RPC Header
	rpcHdrSize = rpc_getHdrSize(rpcCallp);
	msgp = (char *)rpcCallp;
	msgp += rpcHdrSize;

	//Decode Rx Cookie
	length = nlm_decodeCookieObj(msgp, NLM_DECODE_RX);
	if (length == NFX_ERR)
		return (NFX_ERR);			
	//update msg field
	msgp += length;

	//Decode alock field
	length = nlm_decodeAlockObj(msgp, rpcCallp->progVer);
	if (length == NFX_ERR)
		return (NFX_ERR);

	return (NFX_OK);

}

int32 nlm_rxDecodeRpcCallProc(rpcCallHdr_t	*rpcCallp)
{
	int32			rc;

	rc = nlm_tplRcvCheckPkt(rpcCallp);
	if (rc != NFX_OK)
	{
		nlm_decodeRpcHdrErr(rc);
		return (NFX_ERR);
	}

	//print RPC Header
	nlm_decodeRpcHdr(rpcCallp);	

	rc = NFX_OK;
	printf("%s",nlm_procs[rpcCallp->rpc_proc]);	

	printf("\t----- NLM Call Packet -----\n");

	switch (rpcCallp->rpc_proc)
	{
		case NLM_NULL:

			break;

		case NLM_TEST:
			rc = nlm_rxDecodeTestProc(rpcCallp);	
			if (rc != NFX_OK)
				printf("Rx Decode: cannot decode NLM_TEST packet (xid: %08x)\n",
						rpcCallp->rpc_xid);						
			
			break;

		case NLM_LOCK:
			rc = nlm_rxDecodeLockProc(rpcCallp);	
			if (rc != NFX_OK)
				printf("Rx Decode: cannot decode NLM_LOCK packet (xid: %08x)\n",
						rpcCallp->rpc_xid);						

			break;

		case NLM_CANCEL:
			rc = nlm_rxDecodeCancelProc(rpcCallp);	
			if (rc != NFX_OK)
				printf("Rx Decode: cannot decode NLM_CANCEL packet (xid: %08x)\n",
						rpcCallp->rpc_xid);						

			break;

		case NLM_UNLOCK:
			rc = nlm_rxDecodeUnlockProc(rpcCallp);	
			if (rc != NFX_OK)
				printf("Rx Decode: cannot decode NLM_CANCEL packet (xid: %08x)\n",
						rpcCallp->rpc_xid);						

			break;

		case NLM_GRANTED:

			break;

		case NLM_TEST_MSG:

			break;

		case NLM_LOCK_MSG:


			break;

		case NLM_CANCEL_MSG:

			break;

		case NLM_UNLOCK_MSG:

			break;

		case NLM_GRANTED_MSG:

			break;

		case NLM_TEST_RES:

			break;

		case NLM_LOCK_RES:

			break;

		case NLM_CANCEL_RES:

			break;

		case NLM_UNLOCK_RES:

			break;

		case NLM_GRANTED_RES:

			break;

		case NLM_SHARE:

			break;

		case NLM_UNSHARE:

			break;
		case NLM_NM_LOCK:

			break;


		case NLM_FREE_ALL:

			break;

		default:
			printf("Rx: Unsupported NLM procedure number %d\n", rpcCallp->rpc_proc);	
			rc = NFX_ERR;
			break;			

	}

	return (NFX_OK);
}

int32 nlm_rxDecodeRpcReplyProc(char *rpcHdrp)
{
	rpcReplyHdr_t		*rpcReplyHdrp;

	rpcReplyHdrp = (rpcReplyHdr_t *)rpcHdrp;		

	printf("\t----- NLM Reply Packet -----\n");

	printf("\n\t----- RPC Header -----\n");

	printf("\tXID: 0x%08x\n", rpcReplyHdrp->rpc_xid);
	printf("\tDirection: %s\n", "Reply");	

	//Reply Status
	switch (rpcReplyHdrp->rpc_status)
	{
		case RPC_REPLY_MSG_ACCEPTED:
		{
			printf("\tReply Status: %s\n",  "MSG_ACCEPTED");
			break;
		}
		case RPC_REPLY_MSG_DENIED:
			printf("\tReply Status: %s\n",  "MSG_DENIED");
			break;

	}

	return (NFX_OK);	
}

/*-----------------------------------------------------------------
 * Name:   nlm_rxDecodePkt 
 *
 * Description:
 *		Decode NLM received packet.	
 *	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_rxDecodePkt(pkt_desc_t *descp)
{
	int32			rc;
	rpcCallHdr_t	*rpcCallp;


	if (descp == NULL)
	{
		printf(NLM_BAD_RPC_HDR_ERR_STR,NLM_STR);
		return (NFX_ERR);
	}

	rpcCallp = (rpcCallHdr_t *)descp->pHdr;

	
	if (rpcCallp == NULL)
	{
		printf(NLM_BAD_RPC_HDR_ERR_STR,NLM_STR);
		return (NFX_ERR);
	}

	if (rpcCallp->rpc_direction == RPC_CALL)
	{
		rc = nlm_rxDecodeRpcCallProc(rpcCallp);
	}
	else
	{
		rc = nlm_rxDecodeRpcReplyProc(descp->pHdr);
	}
	return (rc);
}

void nlm_decodeRpcReplyHdr(rpcReplyHdr_t *rpcReplyHdrp)
{
	if (rpcReplyHdrp == NULL)
		return;
	printf("\tXID: 0x%08x\n", rpcReplyHdrp->rpc_xid);
	printf("\tDirection: %s\n", 
		(rpcReplyHdrp->rpc_direction == RPC_CALL)? "Call" : "Reply");	
	//Reply Status
	switch (rpcReplyHdrp->rpc_status)
	{
		case RPC_REPLY_MSG_ACCEPTED:
		{
			printf("\tReply Status: %s\n",  "MSG_ACCEPTED");
			break;
		}
		case RPC_REPLY_MSG_DENIED:
			printf("\tReply Status: %s\n",  "MSG_DENIED");
			break;

	}

	return;

}

/*-----------------------------------------------------------------
 * Name:     nlm_xmtDecodeTest  	
 *
 * Description:
 *      Decode NLM_TEST  reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nlm_xmtDecodeTest(
	nlm_request_t *nlmReqp, 
	char *msgp,
	int32 replyStatus)
{
	int32		length;
	int32		nfs_version;
	int32		nlm_version;

	//For NLM_TEST, client only expect NLM_DENIED error return
	if (replyStatus != NLM_ERR_DENIED)
	{
		printf("%s: Invalid error code (%d) return in reply!\n",
				__FUNCTION__, replyStatus);
		return (NFX_ERR);
	}
	nlm_version = nlmReqp->rpcCallHdrp->progVer;

   //Map NLM version to comparable
    //NFS version

	//nfs_version = nlm_utilMapToCompNFSVer(nlm_version);
	NLM_MAP_TO_COMP_NFS_VER(nfs_version, nlm_version);

	//status: must be NLM_DENIED, 
	//Get exclusive field
	
	length = nlm_decodeCookieObj(msgp, NLM_DECODE_XMT);
	if (length == NFX_ERR)
		return (NFX_ERR);			
	msgp += length;

	//Get svid field
	length = nfs_decodeSvid(msgp, NLM_DECODE_XMT);
	msgp += length;

	//Get host identify (oh) field
	length = nfs_decodeLockOwnerIdent(msgp, NLM_DECODE_XMT);
	if (length == NFX_ERR)
		return (NFX_ERR);			
	msgp += length;

	//Get file Offset field
	length = nlm_decodeFileOffset(msgp, nfs_version, NLM_DECODE_XMT);
	msgp += length;

	//Get len or count field
	length = nlm_decodeCountField(msgp, nfs_version, NLM_DECODE_XMT);

	return (NFX_OK);		
}


/*-----------------------------------------------------------------
 * Name:       	nlm_xmtDecodePkt 
 *
 * Description:
 *      Decode NLM outgoing reply packet.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void
nlm_xmtDecodePkt(nlm_request_t *nlmReqp)
{
	rpcCallHdr_t		*rpcCallHdrp;
	pkt_desc_t			*tplXmitPktDescp;
	rpcReplyHdr_t		*rpcReplyHdrp;
	char				*msgp;
	int32				length;
	uint32				replyStatus;

	if ( (nlmReqp == NULL) ||
		 (nlmReqp->rpcCallHdrp == NULL) )
		return;

	printf("\t----- RPC Header -----\n");

	rpcCallHdrp = nlmReqp->rpcCallHdrp;

	if (( rpcCallHdrp->rpc_proc < NLM_MIN_PROC) ||
	 	(rpcCallHdrp->rpc_proc > NLM_MAX_PROC )  )
		printf("%s: Bad procedure number!\n", __FUNCTION__);

	tplXmitPktDescp = (pkt_desc_t *)nlmReqp->txPktDescp;
	msgp = (char *)tplXmitPktDescp->pHdr;
	rpcReplyHdrp = (rpcReplyHdr_t *)tplXmitPktDescp->pHdr;
	nlm_decodeRpcReplyHdr(rpcReplyHdrp);

	//No arguments for NULL procedure, just return 
	if (rpcCallHdrp->rpc_proc == NLM_NULL)
		return;

	printf("\t----- NLM Reply packet -----\n");

	//Decode reply cookie object
	
	length = nlm_decodeCookieObj(msgp, NLM_DECODE_XMT);
	if (length == NFX_ERR)
		return;
	msgp += length;

	//Get reply status
	replyStatus = *(uint32 *)msgp;
	printf("\tStatus: %d - %s\n", 
			replyStatus, nlm_getStatus(replyStatus));
	msgp += sizeof(uint32);

	switch 	(rpcCallHdrp->rpc_proc)
	{
		case NLM_TEST:
			if (replyStatus != NLM_STAT_GRANTED)
				nlm_xmtDecodeTest(nlmReqp, msgp, replyStatus);
			break;
		case NLM_LOCK:
			break;
		case NLM_CANCEL:
			break;
		case NLM_UNLOCK:
			break;
		case NLM_GRANTED:
			//nlm_xmtDecodeGranted(nlmReqp, msgp);
			//TODO
			break;
	}

	return;
}
