/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:		nsm-cmd.c
 *                              
 * RCS:  $Id: nsm-cmd.c,v 1.3 2002/10/18 02:52:43 johnh Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: Contain NSM UI command functions on TXRX only.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
#ifdef	NFP_TXRX
#include "nsm.h"

#ifdef NSM_UNIT_TEST
int32 nsm_cmdTest(int32 argc, char *argv[]);
#endif //NSM_UNIT_TEST
int32 nsm_cmdShow(int32 argc, char *argv[]);
int32 nsm_cmdTrace(int32 argc, char *argv[]);
int32 nsm_cmdHelpAll(void);
int32 nsm_cmdHelpCmd (char *pCmd);
int32 nsm_cmdClear(int32 argc, char *argv[]);

typedef struct
{
	int32	(*cmdFunc)(int32 argc, char *argv[]);
	char	*cmd;
	char	*helpTxt;
} nsm_cmdTbl_t;


nsm_cmdTbl_t nsm_cmdTable[ ] =
{
#ifdef NSM_UNIT_TEST
	{nsm_cmdTest,	"test",		"nsm test init|mon|unmon|notify"},
#endif //NSM_UNIT_TEST
	{nsm_cmdShow,	"show",		"nsm show stats"},
	{nsm_cmdTrace,	"trace",	"nsm trace on|off"},
	{nsm_cmdClear,	"clear",	"nsm clear stats"}
};

#define NSM_CMD_MAX	sizeof(nsm_cmdTable)/sizeof(nsm_cmdTbl_t)


/*-----------------------------------------------------------------
 * Name:		nsm_cmdHelp
 *
 * Description: Display NSM help-texts.
 *
 *              nsm help [show | trace]
 *
 * Created by:	Xiaobing Zhang 
 *
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdHelp(int32 argc, char *argv[])
{
	if (argc < 3)
		return (nsm_cmdHelpAll());
	else
		return (nsm_cmdHelpCmd(argv[2]));
} // nsm_cmdHelp()

/*-----------------------------------------------------------------
 * Name:		nsm_cmdHelpAll
 *
 * Description: Display help-texts for all NSM commands.
 *
 * Created by:	Xiaobing Zhang 
 *
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdHelpAll(void)
{
	int32	i;

	printf("Usage:");
	for (i=0; i < NSM_CMD_MAX; i++)
	{
		printf("\t%s\n", nsm_cmdTable[i].helpTxt);
	}
	return (0);
} // nsm_cmdHelpAll()

/*-----------------------------------------------------------------
 * Name:		nsm_cmdHelpCmd
 *
 * Description: Display help-texts for an individual NSM command.
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdHelpCmd(char *pCmd)
{
	int32	i;

	for (i=0; i < NSM_CMD_MAX; i++)
	{
		if (strequ(pCmd, nsm_cmdTable[i].cmd))
		{
			printf("Usage:");
			printf("\t%s\n", nsm_cmdTable[i].helpTxt);
			return 0;
		}
	}
	return (nsm_cmdHelpAll());
} // nsm_cmdHelpCmd()

/*-----------------------------------------------------------------
 * Name:		nsm_cmdShow
 *
 * Description: Dispatch for nsm show commands.
 *
 *              nsm show stats
 * 
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdShow(int32 argc, char *argv[])
{
	if (argc < 3)
	{
		nsm_cmdHelpCmd("show");
		return (0);
	}

	if ( !(strncmp(argv[2], "stats", 4)))
	{
		nsm_showStats();
		return (NFX_OK);
	}
	return (nsm_cmdHelpCmd("show"));
} // nsm_cmdShow()

/*-----------------------------------------------------------------
 * Name:		nsm_cmdTrace
 *
 * Description: Enables or disables NSM tracing.
 *
 *              nsm trace <on|off>
 * 
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdTrace(int32 argc, char *argv[])
{
	if (argc < 3)
	{
		nsm_cmdHelpCmd("trace");
		return 0;
	}
	if (strequ(argv[2], "on"))	
	{
		nsm_traceEnable = 1;
		printf("NSM trace has been turned on.\n");
	}
	else if (strequ(argv[2], "off"))
	{
		nsm_traceEnable = 0;
		printf("NSM trace has been turned off\n");
	}
	else
		nsm_cmdHelpCmd("trace");

	return 0;
} // nsm_cmdTrace()

/*-----------------------------------------------------------------
 * Name:		nsm_cmdClear
 *
 * Description: Clear the stats and debug counters.
 *
 *              nsm clear stats
 * 
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdClear(int32 argc, char *argv[])
{
	if (argc < 3)
	{
		nsm_cmdHelpCmd("clear");
		return (0);
	}

	if ( !(strncmp(argv[2], "stats", 4)))
	{
		nsm_clearStats();
		return (NFX_OK);
	}
	return (nsm_cmdHelpCmd("clear"));
} // nsm_cmdClear()

#ifdef NSM_UNIT_TEST
/*-----------------------------------------------------------------
 * Name:		nsm_cmdTest
 *
 * Description: NSM unit test.
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	08/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmdTest(int32 argc, char *argv[])
{
	if (argc < 3)
	{
		nsm_cmdHelpCmd("test");
		return 0;
	}
	if (strequ(argv[2], "init"))
	{
		printf("Unit test inilization.\n");
		nsmtest_init();
	}
	if (strequ(argv[2], "mon"))
	{
		printf("Unit test monitor request.\n");
		nsmtest_mon();
	}
	if (strequ(argv[2], "unmon"))
	{
		printf("Unit test unmonitor request.\n");
		nsmtest_unmon();
	}
	if (strequ(argv[2], "notify"))
	{
		printf("Unit test notify request.\n");
		nsmtest_notify();
	}
	else
		nsm_cmdHelpCmd("test");

	return 0;
} // nsm_cmdTest()
#endif //NSM_UNIT_TEST

/*-----------------------------------------------------------------
 * Name: nsm_cmd
 *
 * Description: Main dispatch for NSM commands.  Called from
 *              command table.
 *
 *				nsm show stats
 *				nsm trace on|off
 *
 * Created by : Xiaobing Zhang
 *
 * Date Created:	09/01/2002
 *-----------------------------------------------------------------
 */
int32 nsm_cmd(int32 argc, char *argv[])
{
	int32	i;

	if (argc < 2)
	{
		return (nsm_cmdHelp(argc, argv));
	}

	for (i=0; i < NSM_CMD_MAX; i++)
	{
		if (strequ(argv[1], nsm_cmdTable[i].cmd))
		{
			return ((*nsm_cmdTable[i].cmdFunc)(argc, argv));
		}
	}
	return (nsm_cmdHelp(argc, argv));
} // nsm_cmd()

#endif	//NFP_TXRX
