/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:			nsm-thread.c
 *
 * RCS:  $Id: nsm-thread.c,v 1.6 2002/10/24 21:12:45 isabellec Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contains NSM processing functions running in a thread.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include "nsm.h"

extern int32
nfs_addCBToList(nfp_list_t *nodeListp, void *CtrlBlkp, uint32 nodeType);

/********************************
 * NSM server recovery processing
 ********************************
 */

#if 0
/*-----------------------------------------------------------------
 * Name:		nsm_getVolIdList
 *
 * Description: Get volume Id list from EVM.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_getVolIdList(evm_MsgVolList_t	*evmVolIdListp,
				 eee_desc_t			*evmVolIdList_eDescp)
{
	int32				rc;
	evm_volId_t			id;
	esm_event_t			evt;

	evmVolIdList_eDescp = NULL;
	evmVolIdListp = NULL;
	memset(&evt, 0, sizeof(esm_event_t));

	//---------------------------------------
	// Send request to get all the volume IDs
	//---------------------------------------
	id.whole_thing = -1;
	rc = nsm_getVolCfgReq(id);
	if (rc != NFX_OK)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_getVolIdList: " 
								"Failed to send message to EVM"));
		return (NFX_ERR);
	}

	//-------------------------------
	// Wait for the response from EVM
	//-------------------------------
	thread_yield(&evt, NSM_TIMEOUT_EVM);

	//------------------
	// Received response
	//------------------
	switch (evt.event)
	{
		case EVM_VOL_LIST:
			// Got volume IDs
			evmVolIdListp = (evm_MsgVolList_t *)(evt.p2.u.p);
			evmVolIdList_eDescp = (eee_desc_t *)(evt.p1.u.p);
			if ((evmVolIdListp == NULL) || (evmVolIdList_eDescp == NULL))
			{
				eee_freePacket(evmVolIdList_eDescp);
				rc = NFX_ERR;
			}
			else
				rc = NFX_OK;
			break;

		case THREAD_EVENT_TIMEOUT:
			NSM_PRINTF(LOG_INFO, ("nsm_getVolIdList: Thread timeout"));
			rc = NFX_ERR;
			break;

		default:
			NSM_PRINTF(LOG_INFO, ("nsm_getVolIdList: "
									"Invalid message type %d", evt.event));
			//eee_freePacket((eee_desc_t *)(evt.p1.u.p));
			rc = NFX_ERR;
			break;
	}

	return (rc);
} // nsm_getVolIdList()
#endif

/*-----------------------------------------------------------------
 * Name:		nsm_getVolCfg
 *
 * Description: Get volume config from EVM.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_getVolCfg(	evm_volId_t			id,
				evm_MsgVolCreate_t	**evmVolCfgp,
				eee_desc_t			**evmVolCfg_eDescp)
{
	int32				rc;
	esm_event_t			evt;

	*evmVolCfg_eDescp = NULL;
	*evmVolCfgp = NULL;
	memset(&evt, 0, sizeof(esm_event_t));

	//----------------------------------
	// Send request to get volume config 
	//----------------------------------
	rc = nsm_msgGetVolCfgReq(id);
	if (rc != NFX_OK)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_getVolCfg: " 
								"Failed to send message to EVM"));
		return (NFX_ERR);
	}

	//-------------------------------
	// Wait for the response from EVM
	//-------------------------------
	thread_yield(&evt, NSM_TIMEOUT_EVM);

	//------------------
	// Received response
	//------------------
	switch (evt.event)
	{
		case EVM_VOL_CONFIG:
			// Got volume config 
			*evmVolCfgp = (evm_MsgVolCreate_t *)(evt.p2.u.p);
			*evmVolCfg_eDescp = (eee_desc_t *)(evt.p1.u.p);
			if ((*evmVolCfgp == NULL) || (*evmVolCfg_eDescp == NULL))
			{
				eee_freePacket(*evmVolCfg_eDescp);
				rc = NFX_ERR;
			}
			else
				rc = NFX_OK;
			break;

		case THREAD_EVENT_TIMEOUT:
			NSM_PRINTF(LOG_INFO, ("nsm_getVolCfg: Thread timeout"));
			rc = NFX_ERR;
			break;

		default:
			NSM_PRINTF(LOG_INFO, ("nsm_getVolCfg: "
									"Invalid message type %d", evt.event));
			//eee_freePacket((eee_desc_t *)(evt.p1.u.p));
			rc = NFX_ERR;
			break;
	}

	return (rc);
} // nsm_getVolCfg()

/*-----------------------------------------------------------------
 * Name:		nsm_getVolNameList
 *
 * Description: Get volume Id and name list from EVM.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_getVolNameList(evm_MsgVolNameList_t	**evmVolNameListp,
					eee_desc_t			**evmVolNameList_eDescp)
{
	int32				rc;
	esm_event_t			evt;

	*evmVolNameList_eDescp = NULL;
	*evmVolNameListp = NULL;
	memset(&evt, 0, sizeof(esm_event_t));

	//-------------------------------------------------------
	// Send request to get all the volume IDs and their names
	//-------------------------------------------------------
	rc = nsm_msgGetVolNameListReq();
	if (rc != NFX_OK)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_getVolNameList: " 
								"Failed to send message to EVM"));
		return (NFX_ERR);
	}

	//-------------------------------
	// Wait for the response from EVM
	//-------------------------------
	thread_yield(&evt, NSM_TIMEOUT_EVM);

	//------------------
	// Received response
	//------------------
	switch (evt.event)
	{
		case EVM_VOL_NAME_LIST:
			// Got volume IDs and names
			*evmVolNameListp = (evm_MsgVolNameList_t *)(evt.p2.u.p);
			*evmVolNameList_eDescp = (eee_desc_t *)(evt.p1.u.p);
			if ((*evmVolNameListp == NULL) || (*evmVolNameList_eDescp == NULL))
			{
				eee_freePacket(*evmVolNameList_eDescp);
				rc = NFX_ERR;
			}
			else
			{
				NSM_PRINTF(LOG_DEBUG, ("nsm_getVolNameList: "
					"(*evmVolNameListp)->num = %d", (*evmVolNameListp)->num));
				rc = NFX_OK;
			}
			break;

		case THREAD_EVENT_TIMEOUT:
			NSM_PRINTF(LOG_INFO, ("nsm_getVolNameList: Thread timeout"));
			rc = NFX_ERR;
			break;

		default:
			NSM_PRINTF(LOG_INFO, ("nsm_getVolNameList: "
									"Invalid message type %d", evt.event));
			//eee_freePacket((eee_desc_t *)(evt.p1.u.p));
			rc = NFX_ERR;
			break;
	}

	return (rc);
} // nsm_getVolNameList()

/*-----------------------------------------------------------------
 * Name:		nsm_getVolName
 *
 * Description: Get volume name.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/09/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_getVolName(evm_volId_t volId, char **volName)
{
	int32				rc;
	evm_MsgVolCreate_t	*evmVolCfgp;
	eee_desc_t			*evmVolCfg_eDescp;

	rc = nsm_getVolCfg(volId, &evmVolCfgp, &evmVolCfg_eDescp);
	if (rc != NFX_OK)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_getVolName: "
							"Failed to get config for volume %llx from EVM",
							volId.whole_thing));
		return (rc);
	}

	strcpy(*volName, evmVolCfgp->config.name);

	// Free packet descriptor
	eee_freePacket(evmVolCfg_eDescp);

	return (NFX_OK);
} // nsm_getVolName()

/*-----------------------------------------------------------------
 * Name:		nsm_waitForFsReady
 *
 * Description: Check if filesystem is ready to be used.
 *              The file system is ready to be used if the state
 *              is ONLINE.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/07/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_waitForFsReady(evm_volId_t volId)
{
	int32				rc;
	int32				numOfTries = NSM_MAX_RETRIES_FS_UP + 1;
	esm_event_t			evt;
	fs_MsgViewFsResp	*fsViewFsResp;
	fs_FSInfo			*fsInfop;
	boolean				tryAgain = FALSE;

	memset(&evt, 0, sizeof(esm_event_t));

	while(numOfTries > 0)
	{
		numOfTries--;

		//--------------------------------------------
		// Send request to FS to get file system state 
		//--------------------------------------------
		rc = nsm_msgViewFsReq(volId);
		if (rc != NFX_OK)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_waitForFsReady: " 
									"Failed to send message to EFS"));
			break;
		}

		//-------------------------------
		// Wait for the response from FS
		//-------------------------------
		thread_yield(&evt, NSM_TIMEOUT_EFS);

		//------------------
		// Received response
		//------------------
		switch (evt.event)
		{
			case FS_REQ_VIEWFS:
				// Got file system info
				fsViewFsResp = (fs_MsgViewFsResp *)(evt.p2.u.p);
				if (fsViewFsResp->nbrFs != 1)
				{
					NSM_PRINTF(LOG_ERR, ("nsm_waitForFsReady: "
							"Get file system state for volume 0x%llx failed!",
							 volId.whole_thing));
				}
				else
				{
					fsInfop = &(fsViewFsResp->u.fsInfo[0]);
					if ((fsInfop->state == FS_VOL_STATE_ONLINE) ||
						(fsInfop->state == FS_VOL_STATE_ONLINE_RDONLY) ||
						(fsInfop->state == FS_VOL_STATE_BACKUP_RESTORE))
					{
						NSM_PRINTF(LOG_DEBUG, ("nsm_waitForFsReady: "
								"volId 0x%llx is ready", volId.whole_thing));
						eee_freePacket((eee_desc_t *)(evt.p1.u.p));
						return (NFX_OK);
					}
					else
						tryAgain = TRUE;
				}
				eee_freePacket((eee_desc_t *)(evt.p1.u.p));
				break;

			case THREAD_EVENT_TIMEOUT:
				NSM_PRINTF(LOG_INFO, ("nsm_waitForFsReady: Thread timeout"));
				break;

			default:
				NSM_PRINTF(LOG_INFO, ("nsm_waitForFsReady: "
									"Invalid message type %d", evt.event));
				//eee_freePacket((eee_desc_t *)(evt.p1.u.p));
				break;
		}

		if (tryAgain == TRUE)
			thread_sleep(NULL, NSM_TIMEOUT_WAIT_FS_UP);
		else
			break;
	} // while()

	return (NFX_ERR);
} // nsm_waitForFsReady()

/*-----------------------------------------------------------------
 * Name:		nsm_getClientPort
 *
 * Description: Send portmap GETPORT message to get client's NSM port.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_getClientPort(nsm_clientCB_t *clientCBp, nsm_requestCB_t *reqCBp)
{
	int32			i, rc;
	esm_event_t		evt;
	nsm_txCmdInfo_t	*txCmdInfop;
	boolean			firstTime;

	memset(&evt, 0, sizeof(esm_event_t));

	txCmdInfop = &reqCBp->txCmdInfo;
	txCmdInfop->cmdXid = nlm_getNextCmdXid();	//Assign Xid for getport command
	txCmdInfop->laddr = 0;		//TODO: Don't care?
	txCmdInfop->lport = ntohs(NSM_PMAP_PORT);
	txCmdInfop->raddr = clientCBp->hostIpAddr;
	txCmdInfop->rport = RPC_PORTMAPPER_WELLKNOWN_PORT;
	txCmdInfop->pBsdSock = clientCBp->pmap_pBsdSock;

	//------------------------------------------------------
	// Send Portmap GETPORT message to get client's NSM port
	//------------------------------------------------------
	firstTime = TRUE;
	for (i = 0; i < NSM_MAX_RETRIES_REQ_MSG + 1; i++)
	{
		NSM_PRINTF(LOG_DEBUG, ("nsm_getClientPort: "
					"Retry counter (i) = %d, firstTime = %d", i, firstTime));

		rc = nsm_procTxGetPortReq(reqCBp);	// Retry several times
		if (rc == NFX_ERR)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_getClientPort: "
									"Failed to send portmap request"));
			return (NFX_ERR);
		}
		if (firstTime == FALSE)
		{
			nsm_globalCfgp->nsm_debugCounters.nsm_pktGetPortRetries++;
		}

		thread_yield(&evt, NSM_TIMEOUT_PMAP);

		//------------------
		// Received response
		//------------------
		switch(evt.event)
		{
			case NSM_MSG_PMAP_RESP:
				clientCBp->hostPort = evt.p1.u.w;
				NSM_PRINTF(LOG_DEBUG, ("nsm_getClientPort: "
								"Thread %d received NSM port# %d for client %s\n",
								clientCBp->notifyThreadId,
								clientCBp->hostPort,
								clientCBp->hostName));
				return (NFX_OK);
				break;

			case THREAD_EVENT_TIMEOUT:
				NSM_PRINTF(LOG_INFO, ("nsm_getClientPort: Thread %d timeout",
										clientCBp->notifyThreadId));
				firstTime = FALSE;
				break;

			default:
				NSM_PRINTF(LOG_INFO, ("nsm_getClientPort: "
						"Thread %d received invalid event %d for client %s\n",
						clientCBp->notifyThreadId, evt.event, clientCBp->hostName));
				firstTime = FALSE;
				break;
		}
	}

	return (NFX_ERR);
} // nsm_getClientPort()

/*-----------------------------------------------------------------
 * Name:		nsm_sendNotifyMsg
 *
 * Description: Send SM_NOTIFY message to the given client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/07/2002
 *
 *-----------------------------------------------------------------
 */
void
nsm_sendNotifyMsg(esm_handle_t handle, void *userInfo)
{
	int32			rc, count;
	esm_event_t		evt;
	nsm_clientCB_t	*clientCBp;
	nsm_requestCB_t	*reqCBp;
	nsm_txCmdInfo_t	*txCmdInfop;
	boolean			firstTime, succeed = FALSE;

	memset(&evt, 0, sizeof(esm_event_t));

	clientCBp = (nsm_clientCB_t *)userInfo;
	if (clientCBp->notifyThreadId != handle)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_sendNotifyMsg: Incorrect thread Id."
							"handle = %d, clientCBp->notifyThreadId = %d",
							handle, clientCBp->notifyThreadId));
		return;
	}

	NSM_PRINTF(LOG_DEBUG, ("nsm_sendNotifyMsg: "
				"clientCBp = %p, clientCBp->hostName = %s, notifyThreadId = %d",
				clientCBp, clientCBp->hostName, clientCBp->notifyThreadId));

	// Allocate a requeset CB
	reqCBp = nsm_allocReqCB(clientCBp);
	if (reqCBp == NULL)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_sendNotifyMsg: "
								"Failed to allocate a requestCB"));
		nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
		goto nsm_sendNotifyMsgDone;
	}

	clientCBp->notifyReqp = reqCBp;

	//----------------------
	// Get client's NSM port
	//----------------------
	rc = nsm_getClientPort(clientCBp, reqCBp);
	if (rc == NFX_ERR)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_sendNotifyMsg: "
								"Failed to get NSM port for client %s",
								clientCBp->hostName));
		nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
		goto nsm_sendNotifyMsgDone;
	}

	//-----------------------
	// Send SM_NOTIFY message
	//-----------------------
	txCmdInfop = &reqCBp->txCmdInfo;
	txCmdInfop->cmdXid = nlm_getNextCmdXid();	//Assign Xid for getport command
	txCmdInfop->laddr = 0;		//TODO: Don't care?
	txCmdInfop->lport = NSM_PORT;
	txCmdInfop->raddr = clientCBp->hostIpAddr;
	txCmdInfop->rport = clientCBp->hostPort;
	txCmdInfop->pBsdSock = clientCBp->nsm_pBsdSock;

	firstTime = TRUE;

	// Retry several times
	for (count = 0; count < NSM_MAX_RETRIES_REQ_MSG + 1; count++)
	{
		rc = nsm_procTxNotifyReq(reqCBp);
		if (rc == NFX_ERR)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_sendNotifyMsg: "
									"Failed to send SM_NOTIFY request"));
			nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
			goto nsm_sendNotifyMsgDone;
		}

		if (firstTime == FALSE)
		{
			nsm_globalCfgp->nsm_debugCounters.nsm_pktNotifyRetries++;
		}

		thread_yield(&evt, NSM_TIMEOUT_NOTIFY);

		//------------------
		// Received response
		//------------------
		switch(evt.event)
		{
			case NSM_MSG_NOTIFY_RESP:
				succeed = TRUE;
				NSM_PRINTF(LOG_DEBUG, ("nsm_sendNotifyMsg: "
					"Thread %d received SM_NOTIFY response from client %s\n",
					clientCBp->notifyThreadId, clientCBp->hostName));
				goto nsm_sendNotifyMsgDone;
				break;

			case THREAD_EVENT_TIMEOUT:
				NSM_PRINTF(LOG_INFO, ("nsm_sendNotifyMsg: Thread %d timeout",
										clientCBp->notifyThreadId));
				firstTime = FALSE;
				break;

			default:
				NSM_PRINTF(LOG_INFO, ("nsm_sendNotifyMsg: "
						"Thread %d received invalid event %d for client %s\n",
						clientCBp->notifyThreadId, evt.event, clientCBp->hostName));
				firstTime = FALSE;
				break;
		}
		PKT_FREE_PACKET(reqCBp->txPktDescp);
	}

nsm_sendNotifyMsgDone:

	if (succeed == FALSE)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_sendNotifyMsg: "
			"Failed to send SM_NOTIFY request to client %s", clientCBp->hostName));
	}

	// Change clientCB status
	clientCBp->status = NSM_CLIENT_STATUS_UNMON;

	// Free notifyReqCB
	reqCBp->reqState = NSM_REQ_DONE;
	nsm_delReqCB(reqCBp, FALSE);
	clientCBp->notifyReqp = NULL;

	//Clear thread Id
	clientCBp->notifyThreadId = 0;

	return;				// Exit the thread
} // nsm_sendNotifyMsg()
	
/*-----------------------------------------------------------------
 * Name:		nsm_sendNotifyToOneClient
 *
 * Description: send SM_NOTIFY message to the given client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/07/2002
 *
 *-----------------------------------------------------------------
 */
void
nsm_sendNotifyToOneClient(nsm_fileRecord_t *record)
{
	nsm_clientCB_t		*clientCBp;

	//-----------------------------------
	// Search for corresponding client CB
	//-----------------------------------
	clientCBp = nsm_getClientCB(record->ipAddr);
	if (clientCBp != NULL)
	{
		if (clientCBp->nameLen == 0)
		{
			// Host name is not stored in clientCB yet
			// This also implies host is not monitored
			clientCBp->nameLen = record->nameLen;
			strcpy(clientCBp->hostName, record->hostName);
		}
		else if (strcmp(clientCBp->hostName, record->hostName) != 0)
		{
			// Verify host name
			// There is conflict
			NSM_PRINTF(LOG_WARNING, ("nsm_sendNotifyToOneClient: "
								"Two hosts have same IP address")); 
		}

		if (clientCBp->status != 0)
		{
			// SM_NOTIFY had already been sent
			NSM_PRINTF(LOG_DEBUG, ("nsm_sendNotifyToOneClient: "
						"Notify has been sent to client %s", clientCBp->hostName));
			return;
		}
	}
	else
	{
		//----------------------------------
		// Add a new client CB to hash table
		//----------------------------------
		clientCBp = nsm_addClientCB(record->ipAddr,
									record->nameLen,
									record->hostName);
		if (clientCBp == NULL)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_sendNotifyToOneClient: "
						"Couldn't send SM_NOTIFY to host %s because of "
						"insufficient memory", record->hostName));
			nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
			return;
		}

		// Mark the status to be in recovery stage
		clientCBp->status = NSM_CLIENT_STATUS_RECOV;
	}

	//-----------------------------------
	// Start a thread to send the message
	//-----------------------------------
	clientCBp->notifyThreadId = thread_create((void *)clientCBp);
	thread_start(&(clientCBp->notifyThreadId), nsm_sendNotifyMsg);
	NSM_PRINTF(LOG_DEBUG, ("nsm_sendNotifyToOneClient: "
						   	"clientCBp->hostName = %s, notifyThreadId = %d",
							clientCBp->hostName, clientCBp->notifyThreadId));

	return;
} // nsm_sendNotifyToOneClient()

/*-----------------------------------------------------------------
 * Name:		nsm_sendNotify
 *
 * Description: Get client list stored on a specific volume and 
 *              send SM_NOTIFY message to the clients.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/05/2002
 *
 *-----------------------------------------------------------------
 */
void
nsm_sendNotify(char *fullPathName)
{
	int32				fd, rc;
	uint32				totalCount;
	nsm_fileRecord_t	record;
	boolean				fileExists = TRUE;

	//---------------------
	// Open lock hosts file
	//---------------------
	fd = nsm_fileOpen(fullPathName, NSM_FILE_OPENMODE_READ, &totalCount);
	if (fd == NFX_ERR)
	{
		NSM_PRINTF(LOG_INFO, ("nsm_sendNotify: "
								"File %s doesn't exist", fullPathName));
		fileExists = FALSE;
		goto nsm_sendNotifyDone;
	}

	//--------------------------------------------------
	// Get name and IP address of every host in the file
	//--------------------------------------------------
	while(nsm_fileGetNextRecord(fd, &record) == NFX_OK)
	{
		NSM_PRINTF(LOG_DEBUG, ("nsm_sendNotify: "
				"record.ipAddr = 0x%x, record.nameLen = %d, record.hostName = %s",
				record.ipAddr, record.nameLen, record.hostName));

		// Found the record. Send notity to this client.
		nsm_sendNotifyToOneClient(&record);
	}

nsm_sendNotifyDone:
	if (fileExists == TRUE)
	{
		// Remove the file
		rc = file_unlink(fullPathName);
		if (rc == NFX_ERR)
		{
			NSM_PRINTF(LOG_INFO, ("nsm_sendNotify: "
									"Remove file (%s) failed", fullPathName));
		}
	}

	return;
} // nsm_sendNotify()

#ifdef NSM_UNIT_TEST
extern evm_volId_t     nsmtest_volId1, nsmtest_volId2;
extern char            nsmtest_volName1[VOL_MAX_NAME + 1];
extern char            nsmtest_volName2[VOL_MAX_NAME + 1];
#endif // NSM_UNIT_TEST

/*-----------------------------------------------------------------
 * Name:		nsm_srvrRecover
 *
 * Description: Get client list stored on file system volumes and 
 *              send SM_NOTIFY message to clients after Status
 *              Monitor restarts following a sever crash or reboot. 
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
void
nsm_srvrRecover()
{
	int32					rc;
	eee_desc_t				*evmVolNameList_eDescp;
	evm_MsgVolNameList_t	*evmVolNameListp;
	uint32					num;
	char					fullPathName[NSM_MAX_NAME + 1];
	nsm_volumeCB_t			*volCBp;

	// Wait for a while untill all the volumes are up.
	printf("nsm_srvrRecover: Sleep %d seconds. Wait for all volumes up...\n",
					(NSM_TIMEOUT_WAIT_VOLS_UP/1000));

	thread_sleep(NULL, NSM_TIMEOUT_WAIT_VOLS_UP);	//Sleep for 30 seconds

	//---------------------------------------------------
	// Set Status Monitor state number as time of the day
	//---------------------------------------------------
	nsm_globalCfgp->state = nfs_utilGetTimeOfDayInSecs();

	//-------------------------------------------------------
	// Send request to get all the volume IDs and their names
	//-------------------------------------------------------
	rc = nsm_getVolNameList(&evmVolNameListp, &evmVolNameList_eDescp);
	if (rc != NFX_OK)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_srvrRecover: " 
								"Failed to get volume list from EVM"));
		return;
	}

	//------------------------
	// Go through every volume 
	//------------------------
	for (num = 0; num < evmVolNameListp->num; num++)
	{
		NSM_PRINTF(LOG_DEBUG, ("nsm_srvrRecover: volName = %s, volId =0x%llx",
								evmVolNameListp->s[num].name,
								evmVolNameListp->s[num].id.whole_thing)); 
#ifdef NSM_UNIT_TEST
	if (num == 0)
	{
		nsmtest_volId1.whole_thing = evmVolNameListp->s[num].id.whole_thing;
		strcpy(nsmtest_volName1, evmVolNameListp->s[num].name);
	}
	if (num == 1)
	{
		nsmtest_volId2.whole_thing = evmVolNameListp->s[num].id.whole_thing;
		strcpy(nsmtest_volName2, evmVolNameListp->s[num].name);
	}
#endif // NSM_UNIT_TEST

		// Alloc a new volumeCB, then add it to volume hash table
		volCBp = nsm_addVolumeCB(evmVolNameListp->s[num].id,
								 evmVolNameListp->s[num].name);
		if (volCBp == NULL)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_srvrRecover: Insufficient memory"));
			continue;
		}

		//------------------------------------------------
		// Wait untill the file system is ready or timeout
		//------------------------------------------------
		rc = nsm_waitForFsReady(evmVolNameListp->s[num].id);
		if (rc == NFX_ERR)
		{
			// The file system is not up. Ignore this one.
			continue;
		}

		//------------------------------------------------------------
		// The file system is mounted. Send SM_NOTIFY to all the hosts 
		// that had locks on files on this volume before reboot.
		//------------------------------------------------------------

		// Get full path of lock hosts file on the volume
		// The format is volName:/filename
		strcpy(fullPathName, evmVolNameListp->s[num].name);
		strcat(fullPathName, ":/");
		strcat(fullPathName, NSM_LOCK_FILE_NAME);

		// To send notify
		nsm_sendNotify(fullPathName);
	}

	// Free packet descriptor
	eee_freePacket(evmVolNameList_eDescp);

	return;
} // nsm_srvrRecover

/*-----------------------------------------------------------------
 * Name:		nsm_mon
 *
 * Description: This function processes monitor request.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_mon(nsm_event_t *nsmEvtp)
{
	int32			rc;
	evm_volId_t     volId;
	uint32			hostIpAddr;
	char			*hostName;
	nsm_clientCB_t	*clientCBp;
	nfp_node_t		*nfpNodep;
	nsm_volumeCB_t	*volCBp;
	char			volName[VOL_MAX_NAME + 1];
	char			*volNamep;

	volId.whole_thing = nsmEvtp->volId.whole_thing;
	hostIpAddr = nsmEvtp->ipAddr;
	hostName = nsmEvtp->hostName;

	NSM_PRINTF(LOG_DEBUG, ("nsm_mon: "
							"volId = 0x%llx, hostName = %s, hostIpAddr = 0x%x",
							volId.whole_thing, hostName, hostIpAddr));

	//-----------------------------------
	// Search for corresponding client CB
	//-----------------------------------
	clientCBp = nsm_getClientCB(hostIpAddr);
	if (clientCBp != NULL)
	{
		//Found the client CB

		NSM_PRINTF(LOG_DEBUG, ("nsm_mon: Found clientCB"));

		if (clientCBp->nameLen == 0)
		{
			// Host name is not stored in clientCB yet
			// This also implies host is not monitored
			clientCBp->nameLen = strlen(hostName);
			strcpy(clientCBp->hostName, hostName);
		}
		else if (strcmp(clientCBp->hostName, hostName) != 0)
		{
			// Verify host name
			// There is conflict. Host changed IP address?
			NSM_PRINTF(LOG_ERR, ("nsm_mon: "
								"Two hosts have same IP address")); 
			//TODO: What to do with this situation? 
			//      Replace the old name with the new name?
		}

		if (clientCBp->status == NSM_CLIENT_STATUS_RECOV)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_mon: Server is in recovery state"));
			return (NFX_ERR);
		}
	}
	else
	{
		NSM_PRINTF(LOG_DEBUG, ("nsm_mon: clientCB not found"));

		//----------------------------------
		// Add a new client CB to hash table
		//----------------------------------
		clientCBp = nsm_addClientCB(hostIpAddr, strlen(hostName), hostName);
		if (clientCBp == NULL)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_mon: "
						"Couldn't add a new clientCB to hash table"));
			nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
			return (NFX_ERR);
		}
	}
	clientCBp->nlm_clientCBp = nsmEvtp->nlm_clientCBp;

	//------------------------------------------------------
	// Check if volume is in the volume list of the clientCB
	//------------------------------------------------------
	nfpNodep = nsm_findVolumeCBNode(&clientCBp->volumeList, volId);
	if (nfpNodep != NULL)
	{
		NSM_PRINTF(LOG_DEBUG, ("nsm_mon: Found volumeCBNode"));

		// The client has already been monitored on this volume
		return (NFX_OK);
	}
	else
	{
		NSM_PRINTF(LOG_DEBUG, ("nsm_mon: volumeCBNode not found"));

		//----------------------------------
		// Add the volume to the volume list
		//----------------------------------

		// Search the volumeCB given volId in the hash table
		volCBp = nsm_getVolumeCB(volId);
		if (volCBp == NULL)
		{
			NSM_PRINTF(LOG_DEBUG, ("nsm_mon: volumeCB not found"));

			// volume CB is not found in the hash table.
			// This is a new volume. Add it to the hash table.

			// Get volume name
			volNamep = volName;
			rc = nsm_getVolName(volId, &volNamep);
			if (rc == NFX_ERR)
			{
				NSM_PRINTF(LOG_ERR, ("nsm_mon: "
								"Couldn't get name of volume 0x%llx",
								volId.whole_thing));
				nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
				return (NFX_ERR);

			}

			NSM_PRINTF(LOG_DEBUG, ("nsm_mon: volName = %s", volName));

			// Add the volume CB to hash table
			volCBp = nsm_addVolumeCB(volId, volName);
			if (volCBp == NULL)
			{
				NSM_PRINTF(LOG_ERR, ("nsm_mon: "
								"Couldn't add a new volume CB to hash table "
								"for volume 0x%llx", volId.whole_thing));
				nsm_globalCfgp->nsm_statCounters.nsm_errSrvFault++;
				return (NFX_ERR);
			}
		}
		else
		{
			strcpy(volName, volCBp->volName);
		}
		NSM_PRINTF(LOG_DEBUG, ("nsm_mon: volName = %s", volName));

		// Now, add the volume to the volume list for this client
		nfs_addCBToList(&clientCBp->volumeList, volCBp, NFP_GENERIC_NODE);
	}

	//--------------------------------------------
	// Update lock hosts file.
	// Add the new host to the file on the volume.
	//--------------------------------------------
	rc = nsm_addHostToFile(volName, hostIpAddr, hostName);
	if (rc == NFX_ERR)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_mon: Update lock hosts file failed"));

		// Remove volume CB node from volume list
		nfpNodep = nsm_findVolumeCBNode(&clientCBp->volumeList, volId);
		nfs_delNode(&clientCBp->volumeList, nfpNodep);
		return (NFX_ERR);
	}

	// Update clientCB status
	clientCBp->status = NSM_CLIENT_STATUS_MON;

	return (NFX_OK);
} // nsm_mon()

/*-----------------------------------------------------------------
 * Name:		nsm_unmon
 *
 * Description: This function processes unmonitor request.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/16/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_unmon(nsm_event_t *nsmEvtp)
{
	int32			rc;
	evm_volId_t     volId;
	uint32			hostIpAddr;
	char			*hostName;
	nsm_clientCB_t	*clientCBp;
	nfp_node_t		*nfpNodep;
	nsm_volumeCB_t	*volCBp;
	char			volName[VOL_MAX_NAME + 1];

	volId.whole_thing = nsmEvtp->volId.whole_thing;
	hostIpAddr = nsmEvtp->ipAddr;
	hostName = nsmEvtp->hostName;

	NSM_PRINTF(LOG_DEBUG, ("nsm_unmon: hostName = %s, hostIpAddr = %d",
							hostName, hostIpAddr));

	//-----------------------------------
	// Search for corresponding client CB
	//-----------------------------------
	clientCBp = nsm_getClientCB(hostIpAddr);
	if (clientCBp != NULL)
	{
		// Found the client CB
		// Verify host name
		if (strcmp(clientCBp->hostName, hostName) != 0)
		{
			// There is conflict. Host changed IP address?
			NSM_PRINTF(LOG_ERR, ("nsm_unmon: "
								"Two hosts have same IP address"));
			//TODO: What to do with this situation? 
			//      Replace the old name with the new name?
		}
	}
	else
	{
		NSM_PRINTF(LOG_ERR, ("nsm_unmon: clientCB not found"));
		return (NFX_ERR);
	}

	//------------------------------------------------------
	// Check if volume is in the volume list of the clientCB
	//------------------------------------------------------
	nfpNodep = nsm_findVolumeCBNode(&clientCBp->volumeList, volId);
	if (nfpNodep == NULL)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_unmon: "
							"The client has no locks on files on this volume"));
		return (NFX_ERR);
	}
	volCBp = (nsm_volumeCB_t *)nfpNodep->nfp_entryCBp;
	strcpy(volName, volCBp->volName);

	NSM_PRINTF(LOG_DEBUG, ("nsm_unmon: volName = %s", volName));

	// Now, remove the volume from the volume list
	nfs_delNode(&clientCBp->volumeList, nfpNodep);

	//--------------------------------------------
	// Update lock hosts file.
	// Remove the host from the file on the volume.
	//--------------------------------------------
	rc = nsm_removeHostFromFile(volName, hostIpAddr, hostName);
	if (rc == NFX_ERR)
	{
		NSM_PRINTF(LOG_ERR, ("nsm_unmon: Update lock hosts file failed"));
		return (NFX_ERR);
	}

	if (clientCBp->volumeList.nfp_reqCnt == 0)
	{
		// Client is no longer monitored. Update the state.
		clientCBp->status = NSM_CLIENT_STATUS_UNMON;
	}

	return (NFX_OK);
} // nsm_unmon()

/*-----------------------------------------------------------------
 * Name:		nsm_notify
 *
 * Description: This function processes notify request.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/16/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_notify(nsm_event_t *nsmEvtp)
{
	int32				rc;
	nsm_clientCB_t		*clientCBp;
	nfp_list_t			*volListp;
	nfp_node_t			*nodep, *prevp;
	nsm_volumeCB_t		*volCBp;

	clientCBp = nsmEvtp->nsm_clientCBp;

	volListp = &clientCBp->volumeList;
	nodep = volListp->nfp_headp;
	while(nodep)
	{
		volCBp = (nsm_volumeCB_t *)nodep->nfp_entryCBp;

		// Update lock hosts file.
		// Remove the host from the file on the volume.
		rc = nsm_removeHostFromFile(volCBp->volName,
									clientCBp->hostIpAddr,
									clientCBp->hostName);
		if (rc == NFX_ERR)
		{
			NSM_PRINTF(LOG_ERR, ("nsm_notify: Update lock hosts file failed"));
		}

		prevp = nodep;
		nodep = nodep->nfp_nextp;

		// Remove the volume from the volume list
		nfs_delNode(volListp, prevp);
	}

	// Client is no longer monitored. Update the state.
	clientCBp->status = NSM_CLIENT_STATUS_UNMON;

	return (NFX_OK);
} // nsm_notify()


/**************************
 * Main processing function
 **************************
 */

/*-----------------------------------------------------------------
 * Name:		nsm_process
 *
 * Description: This function runs in a thread to handle all
 *              NSM processing.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
void
nsm_process(esm_handle_t handle, void *userInfo)
{
	int32				rc;
	esm_event_t			evt;
	nsm_event_t			*nsmEvtp;
	int32				status;

	//----------------------
	// Recovery after reboot
	//----------------------
	nsm_srvrRecover();

	NSM_PRINTF(LOG_DEBUG, ("nsm_process: Server recovery is done"));

	//----------------------------------------------------------------
	// Wait for events to process SM_MON, SM_UNMON, SM_NOTIFY requests
	//----------------------------------------------------------------
	while(1)
	{
		thread_yield(&evt, 0);

		nsm_globalCfgp->nsm_debugCounters.nsm_evtToThreadRcvd++;

		if ((evt.event <  NSM_MSG_MON) || (evt.event >= NSM_MSG_MAX))
		{
			// Received wrong event.
			NSM_PRINTF(LOG_ERR, ("nsm_process: "
							"Received wrong event %d\n", evt.event));
			continue;
		}

		nsmEvtp = (nsm_event_t *)(evt.p1.u.p);

		//------------------------------------
		// Received events we are waiting for.
		// Do the corresponding processing.
		//------------------------------------
		switch (evt.event)
		{
			case NSM_MSG_MON:
				{
					NSM_PRINTF(LOG_DEBUG, ("nsm_process: "
											"Recieved NSM_MSG_MON"));
					rc = nsm_mon(nsmEvtp);
					if (rc == NFX_OK)
					{
						// Agree to monitor
						nsm_globalCfgp->nsm_statCounters.nsm_succ++;
						status = NFX_OK;
					}
					else
					{
						// Cannot monitor
						nsm_globalCfgp->nsm_statCounters.nsm_fail++;
						status = NFX_ERR;
					}

					// To tell NLM about the monitor result
					rc = nsm_msgFromThread(	nsmEvtp->volId,
											nsmEvtp->nlm_clientCBp,
											NULL,
											status,
											NSM_MSG_MON_RESP);
				}
				break;

			case NSM_MSG_UNMON:
				{
					NSM_PRINTF(LOG_DEBUG, ("nsm_process: "
											"Recieved NSM_MSG_UNMON"));
					rc = nsm_unmon(nsmEvtp);
					if (rc == NFX_OK)
					{
						// Agree to unmonitor
						status = NFX_OK;
					}
					else
					{
						// Cannot unmonitor
						status = NFX_ERR;
					}

					// To tell NLM about the unmonitor result
					rc = nsm_msgFromThread(	nsmEvtp->volId,
											nsmEvtp->nlm_clientCBp,
											NULL,
											status,
											NSM_MSG_UNMON_RESP);
				}
				break;

			case NSM_MSG_NOTIFY:
				{
					NSM_PRINTF(LOG_DEBUG, ("nsm_process: "
											"Recieved NSM_MSG_NOTIFY"));
					rc = nsm_notify(nsmEvtp);

					// To send SM_NOTIFY response to client
					rc = nsm_msgFromThread(	nsmEvtp->volId,
											NULL,
											nsmEvtp->nsm_requestCBp,
											0,
											NSM_MSG_NOTIFY_RESP);

					if (rc != NFX_OK)
					{
						// Remove nsm_requestCB
						nsmEvtp->nsm_requestCBp->reqState = NSM_REQ_DONE;
						nsm_delReqCB(nsmEvtp->nsm_requestCBp, FALSE);
					}
				}
				break;
			default:
				NSM_PRINTF(LOG_ERR, ("nsm_process: "
								"Received wrong event %d", evt.event));
				break;
		}
		NSM_SHMEM_DEALLOC(nsmEvtp);
	}
} // nsm_process()

#endif	//NFP_TXRX
