/*
 *-----------------------------------------------------------------
 *
 * Name:	pci-init.c
 *
 * RCS:  $Id: pci-init.c,v 1.10 2002/05/02 18:34:05 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: PCI initialization routines
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/gt64240r.h>
#include <exports/hwMap.h>
#include <exports/pci-api.h>
#include "pci.h"

pci_api_func_t	pci_func;

void
pci_init()
{

#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#ifdef SIBYTE
	pci_func.cfg_read = pci_sb1250_cfg_read;
	pci_func.cfg_write = pci_sb1250_cfg_write;
	pci_func.check_status = pci_sb1250_cfg_check_status;
	pci_func.clear_status = pci_sb1250_cfg_clear_status;
#else
	pci_func.cfg_read = pci_gt_cfg_read;
	pci_func.cfg_write = pci_gt_cfg_write;
	pci_func.check_status = pci_gt_cfg_check_status;
	pci_func.clear_status = pci_gt_cfg_clear_status;
#endif
#endif	/* EVAL_BOARD || AGILE_HARDWARE */

#ifdef PROM
#ifdef SIBYTE
	pci_nfp_init();
#else
#ifndef R9000
	while (!(*(volatile uint32 *)PHYS_TO_K1(HW_GALILEO_BASE + PCI_0MODE) &
																0x80000000));
	pci_cfg_write(0, 0, 0, 4, 0x2a00146);
	while (!(*(volatile uint32 *)PHYS_TO_K1(HW_GALILEO_BASE + PCI_1MODE) &
																0x80000000));
	pci_cfg_write(8, 0, 0, 4, 0x2a00146);

#ifdef AGILE_HARDWARE
	if (isSSC()) {
		/*
		 * Set up Basis part for Microdrives
		 */
		pci_cfg_write(0, 8, 0, 4, 0x3);
		pci_cfg_write(0, 8, 0, 0x10, 0x00000001);

		/*
		 * Set up Toshiba part
		 */
		pci_cfg_write(0, 7, 0, 0x4, 0x02a00146);

		/*
		 * Set up 5600 #1 (PCI_0)
		 */
		pci_cfg_write(0, 6, 0, 4, 0x2a00142);
		pci_cfg_write(0, 6, 0, 0x10, 0xa6000000);

		/*
		 * Set up 5600 #2 (PCI_0)
		 */
		pci_cfg_write(0, 9, 0, 4, 0x2a00142);
		pci_cfg_write(0, 9, 0, 0x10, 0xa6800000);

		/*
		 * Set up 5632 #1 (PCI_1)
		 */
		pci_cfg_write(8, 6, 0, 4, 0x2a00142);
		pci_cfg_write(8, 6, 0, 0x10, 0x18000000);

		/*
		 * Set up 5632 #2 (PCI_1)
		 */
		pci_cfg_write(8, 7, 0, 4, 0x2a00142);
		pci_cfg_write(8, 7, 0, 0x10, 0x19000000);

		/*
		 * Set up 5632 #3 (PCI_1)
		 */
		pci_cfg_write(8, 8, 0, 4, 0x2a00142);
		pci_cfg_write(8, 8, 0, 0x10, 0x1a000000);

		/*
		 * Set up Hercules (PCI_1)
		 */
		pci_cfg_write(8, 9, 0, 4, 0x2a00142);
		pci_cfg_write(8, 9, 0, 0x10, 0x1b000000);

		/*
		 * Set up GT register access from PCI
		 */
		pci_cfg_write(0, 0, 0, 0x20, 0xa4000000);

		/*
		 * Set up FLASH ROM access from Toshiba
		 * Hack for PR# 3367 - to allow Toshiba
		 * to read SSC prom ver string stored on
		 * the big flash.
		 */
		pci_cfg_write(0, 0, 1, 0x1c, 0xb3000000);
		pci_cfg_write(0, 0, 1, 0x20, 0xb3800000);


		/*
		 * Make necessary changes to PCI addressing to work around
		 * Toshiba bug using its TLB entries.  New memory map allows
		 * Toshiba to access PCI spaces within Kseg1.
		 */

		/*
		 * PCI to CS3 address
		 */
		pci_cfg_write(0, 0, 1, 0x1c, 0xa0400000);

		/*
		 * PCI to BOOTCS address
		 */
		pci_cfg_write(0, 0, 1, 0x20, 0xa0800000);

		/*
		 * P2P Mem0 BAR
		 */
		pci_cfg_write(0, 0, 2, 0x10, 0xa2000000);

		/*
		 * PCI to SCS0 BAR
		 */
		pci_cfg_write(0, 0, 0, 0x10, 0xa7000000);

		/*
		 * PCI to CS3 window = 4 MB (0xa0400000 - 0xa0800000)
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
											PCI_0CS_3_BANK_SIZE) = 0x003ff000;

		/*
		 * PCI to BOOTCS window = 4 MB (0xa0800000 - 0xa0c00000)
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
										PCI_0CS_BOOT_BANK_SIZE) = 0x003ff000;

		/*
		 * PCI0 to PCI1 P2P window = 32 MB (0xa2000000 - 0xa4000000)
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
										PCI_0P2P_MEM0_BAR_SIZE) = 0x01fff000;

		/*
		 * PCI to SCS0 window = 16 MB (0xa7000000 - 0xa8000000)
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
											PCI_0SCS_0_BANK_SIZE) = 0x00fff000;

		/*
		 * PCI to CS3 Remap from 0xa0400000 to 0x1f600000
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
								PCI_0CS_3_BASE_ADDRESS_REMAP) = 0x1f600000;

		/*
		 * PCI to BOOTCS Remap from 0xa0800000 to 0x1fc00000
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
								PCI_0CS_BOOTCS_BASE_ADDRESS_REMAP) = 0x1fc00000;

		/*
		 * PCI0 to PCI1 P2P Remap from 0xa2000000 to 0x18000000
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
							PCI_0P2P_MEM0_BASE_ADDRESS_REMAP_LOW) = 0x18000000;

		/*
		 * PCI0 to SCS0 Remap from 0xa7000000 to 0x0F000000
		 */
		*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE +
									PCI_0SCS_0_BASE_ADDRESS_REMAP) = 0x0f000000;

	}
	else if (isFC()) {
		/*
		 * Set up Qlogic
		 */

		/* Set up National MAC
		 *
		 */
	}
#endif

#endif
#endif

#endif

}	/* pci_init() */

