
/*
 * Copyright (c) 1998 Manuel Bouyer.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*-
 * Copyright (c) 1998 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Charles M. Hannum, by Onno van der Linden and by Manuel Bouyer.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


#include <nfx-types.h>
#include <nfx-defs.h>
#include <exports/libc.h>

#include "queue.h"

#include "pcmciavar.h"
#include "disklabel.h"
#include "disk.h"
#include "diskio.h"

#include "exec_elf.h"

#include "wdcreg.h"

extern void pcic_wait_fifo(void);
// Diag new function.  allows files to be read  multiple times from CF
extern void oclose( int fd);
//#define DEBUG

#if defined(DEBUG)
#define debug(x) { if (disk_debug_param & DEBUG_LOAD) { printf x; }}
#define debug_cmd(x) { if (disk_debug_param & DEBUG_CMD) { printf x; }}
#define debug_read(x) { if (disk_debug_param & DEBUG_READ) { printf x; }}
#else
#define debug(x)
#define debug_cmd(x)
#define debug_read(x)
#endif

#define assert NFX_ASSERT

/* XXX */
#define swap16(x) ((((x) & 0xff00) >> 8) | ((x & 0xff) << 8))

struct disksc diskscs[2];


// int disk_debug_param = DEBUG_CMD | DEBUG_READ | DEBUG_PCMCIA | DEBUG_LOAD;
//int disk_debug_param = DEBUG_CMD | DEBUG_READ | DEBUG_PCMCIA;
int disk_debug_param = 0;


/*
 * wdc_alloc_sc - allocate disk structure
 */
struct disksc *
wdc_alloc_sc(int disk_no)
{
	assert(0 <= disk_no && disk_no < 2);
	return &diskscs[disk_no];
}


/*
 * Disk Controller register definitions.
 */
#define _WDC_REGMASK 7
#define _WDC_AUX 8
#define _WDC_RDONLY  16
#define _WDC_WRONLY  32
enum wdc_regs { 		
	wdr_error = _WDC_RDONLY | 1,
	wdr_precomp = _WDC_WRONLY | 1,
	wdr_features = _WDC_WRONLY | 1,
	wdr_seccnt = 2,
	wdr_ireason = 2,
	wdr_sector = 3,
	wdr_cyl_lo = 4,
	wdr_cyl_hi = 5,
	wdr_sdh = 6,
	wdr_status = _WDC_RDONLY | 7,
	wdr_command = _WDC_WRONLY | 7,
	wdr_altsts = _WDC_RDONLY | _WDC_AUX,
	wdr_ctlr = _WDC_WRONLY | _WDC_AUX
};



/*
 * wdc_pcmcia_init - initialize the disk structure for a disk card in
 * a pcmcia socket.
 */
int
wdc_pcmcia_init(struct disksc *sc, struct pcmcia_socket *psock)
{
	int rc = 0;
	struct pcmcia_function *pf;
	struct pcmcia_config_entry *cfe;
	int cnt = 0;

	if (!sc->opened) {
		debug(("wdc_pcmcia_init: sock %s\n", pcmcia_sockname(psock)));

		pf = SIMPLEQ_FIRST(&psock->pf_head);
		cfe = SIMPLEQ_FIRST(&pf->cfe_head);
		while (cfe && cfe->num_iospace != 1) {
			cfe = SIMPLEQ_NEXT(cfe, cfe_list);
			++cnt;
		}
		if (!cfe)
			printf("wdc_pcmcia_init: no applicable function\n");

		pcmcia_enable_socket(pf->socket);
		pcmcia_function_init(pf, cfe);
		pcmcia_function_enable(pf);
		pcmcia_map_io(pf->socket);
	
		if (psock->niowindows == 1 && psock->iomap[0].size == 16) {
			sc->cmdreg = psock->iomap[0].addr;
			sc->ctlreg = psock->iomap[0].addr + 14;
			rc = 1;
		}
	}
	else {
		rc = 1;
	}

	return rc;
}


/*
 * wdc_valid_cmdreg - return true if the register is valid
 */
static int
wdc_valid_cmdreg(int reg)
{
	return 1 <= reg && reg <= wdr_status;
}


/*
 * wdc_read_cmdreg - read the value from the cmd register
 */
uint8
wdc_read_cmdreg(struct disksc *sc, int reg)
{
	assert(wdc_valid_cmdreg(reg));
	return *(uint8 *)(sc->cmdreg + reg);
}


/*
 * wdc_write_cmdreg - write to a cmd register
 */
void
wdc_write_cmdreg(struct disksc *sc, int reg, uint8 val)
{
	assert(wdc_valid_cmdreg(reg));
	*(uint8 *)(sc->cmdreg + reg) = val;
}


/*
 * wdc_read_ctlreg - read from control register
 */
static uint8
wdc_read_ctlreg(struct disksc *sc)
{
	return *(uint8 *)(sc->ctlreg);
}


/*
 * wdc_write_ctlreg - write to control register
 */
static void
wdc_write_ctlreg(struct disksc *sc, uint8 val)
{
	*(uint8 *)sc->ctlreg = val;
}


/*
 * wdc_read_reg - read from a disk controller register
 */
static uint8
wdc_read_reg(struct disksc *sc, enum wdc_regs reg)
{
	assert(!(reg & _WDC_WRONLY));

	if (!(reg & _WDC_AUX)) {
		return wdc_read_cmdreg(sc, reg & _WDC_REGMASK);
	}
	else {
		return wdc_read_ctlreg(sc);
	}
}


/*
 * wdc_write_reg - write to a disk controller register
 */
static void
wdc_write_reg(struct disksc *sc, enum wdc_regs reg, uint8 val)
{
	assert(!(reg & _WDC_RDONLY));

	debug_cmd(("wdc_write_reg: r=%02x v=%02x\n", reg & _WDC_REGMASK, val));

	pcic_wait_fifo();
	if (!(reg & _WDC_AUX)) {
		wdc_write_cmdreg(sc, reg & _WDC_REGMASK, val);
	}
	else {
		wdc_write_ctlreg(sc, val);
	}
	pcic_wait_fifo();
}


static void
wdc_select_channel(struct disksc *sc)
{
	assert(sc->channel == 0 || sc->channel == 1);
	wdc_write_reg(sc, wdr_sdh, WDSD_IBM | (sc->channel << 4));
}


static void
wdc_input_bytes(struct disksc *sc, char *buf, int cnt)
{
	assert(((unsigned)cnt & 0x1) == 0);
	assert(((unsigned)buf & 0x1) == 0);

	if (buf) {
		char *end = buf + cnt;
		char *bufptr;

		for (bufptr = buf; bufptr < end; bufptr += 2) {
			*(ushort16 *)bufptr = *(ushort16 *)sc->cmdreg;
		}
	}
	else {
		int i;
		volatile ushort16 sink;
		for (i = 0; i < cnt; i += 2) {
			sink = *(ushort16 *)sc->cmdreg;
		}
	}
}


static void
wdc_output_bytes(struct disksc *sc, char *buf, int cnt)
{
	assert(((unsigned)cnt & 0x1) == 0);
	assert(((unsigned)buf & 0x1) == 0);

	if (buf) {
		char *end = buf + cnt;
		char *bufptr;

		for (bufptr = buf; bufptr < end; bufptr += 2) {
			*(ushort16 *)sc->cmdreg = *(ushort16 *)bufptr;
		}
	}
	else {
		int i;
		for (i = 0; i < cnt; i += 2) {
			*(ushort16 *)sc->cmdreg = 0;
		}
	}
}


#define WDCDELAY 100			/* microseconds */
#define WDCNDELAY_RST (WDC_RESET_WAIT * 1000 / WDCDELAY)
#define WDC_RESET_WAIT 31000

/*
 * wdc_waitreset - wait for the disk to come out of reset. Return true
 * if it did not timeout.
 */
static int
wdc_waitreset(struct disksc *sc)
{
	int i;
	int ready = 0;
	uint8 status;

	for (i = 0; i < WDCNDELAY_RST && !ready; ++i) {
		wdc_write_reg(sc, wdr_sdh, WDSD_IBM);
		delay_us(10);
		status = wdc_read_reg(sc, wdr_status);
		delay_us(WDCDELAY);
		ready = !(status & WDCS_BSY);
	}

	debug(("wdc_waitreset: device%sready i=%d\n", ready ? " " : " not ", i));

	return ready;
}


/*
 * wdc_reset - issue a reset command to the device
 */
static void
wdc_reset(struct disksc *sc)
{
	debug(("wdc_reset:\n"));
	
	wdc_write_reg(sc, wdr_ctlr, WDCTL_RST | WDCTL_4BIT);
	delay_us(10);
	wdc_write_reg(sc, wdr_ctlr, WDCTL_4BIT);
	delay_us(2000);
}


/*
 * wdc_detect_drive - return true if the device is present. Resets the
 * device.
 */
static int
wdc_detect_drive(struct disksc *sc)
{
	debug(("wdc_detect_drive:\n"));
	
	wdc_reset(sc);
	return wdc_waitreset(sc);
}


/*
 * wdc_read_status - read the disk controller status
 */
static uint8
wdc_read_status(struct disksc *sc)
{
	wdc_write_reg(sc, wdr_sdh, WDSD_IBM);
	delay_us(10);
	return wdc_read_reg(sc, wdr_status);
}


static int
wdc_status_ok(uint8 status, int mask, int bits)
{
	return (status & WDCS_BSY) == 0 && (status & mask) == bits;
}


/*
 * wdc_wait_for_status - wait for a mask in the status register and !bsy
 */
static int
wdc_wait_for_status(struct disksc *sc, int mask, int bits, int timeout)
{
	uint8 status = WDCS_BSY;
	int time = 0;
	int rc = 0;

	debug_cmd(("wdc_wait_for_status: mask=%02x bits=%02x timeout=%d\n",
		   mask, bits, timeout));
	
	while (!wdc_status_ok(status, mask, bits) && time < timeout) {
		status = wdc_read_reg(sc, wdr_status);
		delay_us(WDCDELAY);
		++time;
	}

	if (time >= timeout) {
		printf("wdc_wait_for_status: timed out: status=%02x\n", status);
	}
	else {
		assert(wdc_status_ok(status, mask, bits));
		rc = 1;
	}

	if (status & WDCS_ERR) {
		sc->error = wdc_read_reg(sc, wdr_error);
		debug(("wdc_wait_for_status: error %x\n", sc->error));
	}

	sc->status = status;

	return rc;
}


/*
 * disk_read_block - read a block blkno into buffer buf
 */
int
disk_read_block(struct disksc *sc, int blkno, char *buf, int bcount)
{
	struct wdc_command cmd;
	int rc;

	debug_read(("disk_read_block: blk=%d cnt=%d\n", blkno, bcount));
	
	memset(&cmd, 0, sizeof(cmd));
	cmd.r_sector = (blkno >> 0) & 0xff;
	cmd.r_cyl = (blkno >> 8) & 0xffff;
	cmd.r_head = (blkno >> 24) & 0x0f;
	cmd.r_head |= WDSD_LBA;
	cmd.r_command = WDCC_READ;
	cmd.r_count = bcount / 512;
	cmd.data = buf;
	cmd.bcount = bcount;
	cmd.timeout = 1000;
	cmd.r_st_bmask = WDCS_DRDY;
	cmd.r_st_pmask = WDCS_DRDY;
	cmd.flags |= AT_READ;

	rc = wdc_exec_command(sc, &cmd);

	debug_read(("disk_read_block: return %d\n", rc));

	return rc;
}


/*
 * Compute checksum for disk label.
 */
static unsigned
dkcksum(struct disklabel *lp)
{
	register ushort16 *start, *end;
	register ushort16 sum = 0;

	start = (ushort16 *)lp;
	end = (ushort16 *)&lp->d_partitions[lp->d_npartitions];
	while (start < end)
		sum ^= *start++;
	return (sum);
}



/*
 * getdisklabel - read the label from the buffer containing the disk block
 */
char *
getdisklabel(char *buf, struct disklabel *lp)
{
	struct disklabel *dlp, *elp;
	char *msg = (char *)0;

	elp = (struct disklabel *)(buf + DEV_BSIZE - sizeof(*dlp));
	for (dlp = (struct disklabel *)buf; dlp <= elp;
	    dlp = (struct disklabel *)((char *)dlp + sizeof(long))) {
		if (dlp->d_magic != DISKMAGIC || dlp->d_magic2 != DISKMAGIC) {
			if (msg == (char *)0)
				msg = "no disk label";
		} else if (dlp->d_npartitions > MAXPARTITIONS ||
			   dkcksum(dlp) != 0)
			msg = "disk label corrupted";
		else {
			*lp = *dlp;
			msg = (char *)0;
			break;
		}
	}
	return (msg);
}


static void
disk_printf_label(struct disklabel *lp)
{
	int i;
	
	debug(("read label ok: lp->d_secsize = %d lp->d_secpercyl = %d\n",
		   lp->d_secsize, lp->d_secpercyl));
	debug(("lp->d_npartitions = %d\n", lp->d_npartitions));
	debug(("nsectors=%d ntracks=%d\n", lp->d_nsectors, lp->d_ntracks));
	for (i = 0; i < MAXPARTITIONS; ++i) {
		struct partition *pp = &lp->d_partitions[i];
		debug(("partition %d: size=%d off=%d cpg=%d\n",
			   i, pp->p_size, pp->p_offset, pp->p_cpg));
	}
}

				 
/*
 * disk_read_label - read the disk label
 */
static int
disk_read_label(struct disksc *sc)
{
	struct disklabel *lp = &sc->label;
	char label_buf[1024];
	char *msg;
	int rc = 0;

 	lp->d_secsize = DEV_BSIZE;
	lp->d_secpercyl = 1;
	lp->d_npartitions = MAXPARTITIONS;
	lp->d_partitions[0].p_offset = 0;
	lp->d_partitions[0].p_size = 0x7fff0000;

	disk_read_block(sc, LABELSECTOR, label_buf, 512);

	msg = getdisklabel(label_buf, lp);

	if (msg)
		printf("%s\n", msg);
	else {
#if defined(DEBUG)
		disk_printf_label(lp);
#endif
		rc = 1;
	}

	return rc;
}



/*
 * disk_recalibrate - issue recalibrate command
 */
static int
disk_recalibrate(struct disksc *sc)
{
	struct wdc_command cmd;
	int rc;

	debug(("disk_recalibrate: "));

	memset(&cmd, 0, sizeof(cmd));
	cmd.r_command = WDCC_RECAL;
	cmd.r_st_bmask = WDCS_DRDY;
	cmd.r_st_pmask = WDCS_DRDY;
	cmd.timeout = 1000000;
	rc = wdc_exec_command(sc, &cmd);
	debug(("wdc_command returned %d\n", rc));

	return rc;
}


/*
 * wdc_probe - probe for the device. Return true if the disk is present
 */
int
wdc_probe(struct disksc *sc)
{
	uint8 status, status1;
	int rc = 0;
	struct ataparams params;

	if (1 || !sc->opened) {
//
// if (pcmcia_socket_carddetect(struct pcmcia_socket *sc)
//
//
		debug(("wdc_probe: before reset "));
// printf(" -wdc_probe: before reset\n");
		wdc_write_reg(sc, wdr_sdh, WDSD_IBM);
		delay_us(1000);
		status = wdc_read_reg(sc, wdr_status);
		debug(("st=%x\n", status));
		wdc_write_reg(sc, wdr_sdh, WDSD_IBM | 0x10);
		delay_us(1000);
		status1 = wdc_read_reg(sc, wdr_status);
		debug(("st1=%x\n", status1));

// printf("  - status %x status1 %x \n",status,status1);
		if ((status & 0x7f) != 0x7f) {
	 	/* Not an empty channel */
// printf(" - call wdc_detect_drive \n");
			rc = wdc_detect_drive(sc);
		}
		status = wdc_read_status(sc);
		wdc_write_reg(sc, wdr_sdh, WDSD_IBM | 0x10);
		status1 = wdc_read_reg(sc, wdr_status);
		debug(("wdc_probe: after reset: st=%02x st1=%02x ",
			   status, status1));
// printf(" -wdc_probe: after reset: st=%02x st1=%02x ",status, status1);

		debug(("sc=0x%x sn=0x%x cl=0x%x ch=0x%x\n",
			   wdc_read_reg(sc, wdr_seccnt),
			   wdc_read_reg(sc, wdr_sector),
			   wdc_read_reg(sc, wdr_cyl_lo),
			   wdc_read_reg(sc, wdr_cyl_hi)));

		wdc_write_reg(sc, wdr_sdh, WDSD_IBM);
// printf(" - in wdc_probe calling  wdc_wait_for_status sc = %X \n",(uint32)sc);
// printf(" - sc->channel = %X sc->state = %X sc->opened =%X\n",sc->channel,sc->state,sc->opened);

		wdc_wait_for_status(sc, WDCS_DRDY, WDCS_DRDY, 1000);

		rc = rc && ata_get_params(sc, 0, &params) &&
			disk_recalibrate(sc) &&
			disk_read_label(sc);

		if (rc) {
			sc->opened = 1;
			printf("disk model: %s\n", params.atap_model);
			printf("disk geometry: cylinders=%d heads=%d sectors=%d\n",
				   params.atap_cylinders,
				   params.atap_heads, params.atap_sectors);

		}
	}
	else
		rc = 1;
	
	return rc;
}


/*
 * wdccommand - write the command to the disk controller
 */
static void
wdccommand(struct disksc *sc, struct wdc_command *command)
{
	debug_cmd(("wdccommand: cmd=0x%x cyl=%d h=%d sec=%d cnt=%d precomp=%d\n",
		   command->r_command, command->r_cyl, command->r_head,
		   command->r_sector, command->r_count, command->r_precomp));

	wdc_write_reg(sc, wdr_sdh, WDSD_IBM | (sc->channel << 4) |
				  command->r_head);
	/* Load parameters. wdr_features(ATA/ATAPI) = wdr_precomp(ST506) */
	wdc_write_reg(sc, wdr_precomp, command->r_precomp);
	wdc_write_reg(sc, wdr_cyl_lo, command->r_cyl);
	wdc_write_reg(sc, wdr_cyl_hi, command->r_cyl >> 8);
	wdc_write_reg(sc, wdr_sector, command->r_sector);
	wdc_write_reg(sc, wdr_seccnt, command->r_count);

	/* Send command. */
	wdc_write_reg(sc, wdr_command, command->r_command);
}


/*
 * wdc_wait_command_ready - wait for the controller to become nonbusy
 */
static int
wdc_wait_command_ready(struct disksc *sc, struct wdc_command *command)
{
	int rc = 1;
	
//printf(" - in wdc_wait_command_ready calling wdc_wait_for_status\n");

	if (command->r_command != ATAPI_SOFT_RESET) {
		rc = wdc_wait_for_status(sc, command->r_st_bmask | WDCS_DRQ,
								 command->r_st_bmask, command->timeout);
	}
	else
		delay_us(10 * 1000);

	return rc;
}


/*
 * wdc_read_regs - read the disk registers state
 */
static void
wdc_read_regs(struct disksc *sc, struct wdc_command *command)
{
	command->r_head = wdc_read_reg(sc, wdr_sdh);
	command->r_cyl = wdc_read_reg(sc, wdr_cyl_hi) << 8;
	command->r_cyl |= wdc_read_reg(sc, wdr_cyl_lo);
	command->r_sector = wdc_read_reg(sc, wdr_sector);
	command->r_count = wdc_read_reg(sc, wdr_seccnt);
	command->r_error = wdc_read_reg(sc, wdr_error);
	command->r_precomp = command->r_error;
}


/*
 * wdccommand_done - process the command status
 */
static int
wdccommand_done(struct disksc *sc, struct wdc_command *command)
{
	int rc;
	
	debug_cmd(("wdccommand_done: \n"));

	if (sc->status & WDCS_DWF)
		command->flags |= AT_DF;
	if (sc->status & WDCS_ERR) {
		command->flags |= AT_ERROR;
		command->r_error = sc->error;
	}

	rc = (command->flags & (AT_ERROR | AT_DF)) == 0;
	
	if ((command->flags & AT_READREG) && rc) {
		wdc_read_regs(sc, command);
	}

	return rc;
}


/*
 * wdccommand_intr - wait for the command to complete and process the
 * status
 */
static int
wdccommand_intr(struct disksc *sc, struct wdc_command *command)
{
	int rc = 0;

	debug_cmd(("wdccommand_intr: \n"));

	if (wdc_wait_for_status(sc, command->r_st_pmask, command->r_st_pmask,
							command->timeout)) {
		if (command->flags & AT_READ) {
			wdc_input_bytes(sc, command->data, command->bcount);
		}
		else if (command->flags & AT_WRITE) {
			wdc_output_bytes(sc, command->data, command->bcount);
		}
		rc = wdccommand_done(sc, command);
	}
	else {
		printf("wdccommand: timed out\n");
	}

	return rc;
}


/*
 * wdc_exec_commadn - execute a command. Returns non-zero if successful.
 */
int
wdc_exec_command(struct disksc *sc, struct wdc_command *command)
{
	int rc = 0;

	debug_cmd(("wdc_exec_command: \n"));

	wdc_select_channel(sc);
	if (wdc_wait_command_ready(sc, command)) {
		wdccommand(sc, command);
		delay_us(10);
		rc = wdccommand_intr(sc, command);
	}

	return rc;
}


/*
 * wdc_make_command -
 */
static void
wdc_make_command(struct wdc_command *command, unsigned r_command,
				 unsigned bmask, unsigned pmask, unsigned flags, char *buf,
				 int count, int timeout)
{
	memset(command, 0, sizeof(*command));
	command->r_command = r_command;
	command->r_st_bmask = bmask;
	command->r_st_pmask = pmask;
	command->flags = flags;
	command->data = buf;
	command->bcount = count;
	command->timeout = timeout;
}


/* Get the disk's parameters */
int
ata_get_params(struct disksc *sc, int flags, struct ataparams *prms)
{
	char tb[ATAPARAMS_SIZE];
	struct wdc_command wdc_c;

	int i;
	ushort16 *p;
	int rc = 0;

	debug(("ata_get_params:\n"));

	bzero(tb, sizeof(tb));
	bzero(prms, sizeof(struct ataparams));
	bzero(&wdc_c, sizeof(struct wdc_command));
	wdc_make_command(&wdc_c, WDCC_IDENTIFY, WDCS_DRDY, WDCS_DRQ,
					 AT_READ | flags, tb, ATAPARAMS_SIZE, 1000);
	if ((rc = wdc_exec_command(sc, &wdc_c))) {
		memcpy(prms, tb, sizeof(struct ataparams));
		for (i = 0; i < sizeof(prms->atap_model); i += 2) {
			p = (ushort16 *)(prms->atap_model + i);
			*p = swap16(*p);
		}
		for (i = 0; i < sizeof(prms->atap_serial); i += 2) {
			p = (ushort16 *)(prms->atap_serial + i);
			*p = swap16(*p);
		}
		for (i = 0; i < sizeof(prms->atap_revision); i += 2) {
			p = (ushort16 *)(prms->atap_revision + i);
			*p = swap16(*p);
		}
	}

	debug(("ata_get_params: return %d\n", rc));

	return rc;
}


/*
 * ata_set_mode - set the disk mode
 */
int
ata_set_mode(struct disksc *sc, uchar8 mode, uchar8 flags)
{
	struct wdc_command wdc_c;
	
	debug(("ata_set_mode:\n"));
	
	memset(&wdc_c, 0, sizeof(wdc_c));
	wdc_c.r_command = SET_FEATURES;
	wdc_c.r_st_bmask = 0;
	wdc_c.r_st_pmask = 0;
	wdc_c.r_precomp = WDSF_SET_MODE;
	wdc_c.r_count = mode;
	wdc_c.flags = AT_READ | flags;
	wdc_c.timeout = 1000; /* 1s */
	return wdc_exec_command(sc, &wdc_c);
}



extern void set_entry_addr(uint32 addr);
extern int getopt(int, char **, char*);
extern int get_rsa(uint32 *, char *);


#ifdef  PROM_R9000
extern void mips_flush_dcache ();
#endif

void
do_disk_load(int argc, char **argv)
{
	extern int optind;
	extern char *optarg;
	uint32 load_address = 0;
	char *filename;
	int c;

	optind = 0;
	while ((c = getopt(argc, argv, "a:")) != -1) {
		switch (c) {
		case 'a':
			if (!get_rsa(&load_address, optarg)) {
				printf("bad load address\n");
				return;
			}
			break;
		default:
			printf("bad option list\n");
			return;
		}
	}

	if (optind == argc) {
		printf("File name not specified, defaulting to /dev/wd0a/bsd\n");
		filename = "/dev/wd0a/bsd";
	}
	else {
		filename = argv[optind];
	}

	if (load_address != 0) {
		printf("Loading file %s at %x...\n", filename, load_address);
	}
	else {
		printf("Loading file %s...\n", filename);
	}

	disk_load_file(filename, load_address);

#ifdef  PROM_R9000
        // flush the loaded data from cache.
        mips_flush_dcache();
#endif

	return;
}


int
disk_load_file(char *fname, uint32 load_address)
{
	int fd, i;
	Elf32_Ehdr eh;
	Elf32_Phdr *ph;
	char *errs = 0;
	char buf[4096];

	do_pcmcia_init();

	if ((fd = oopen(fname, 0)) < 0) {
		errs="open err: %s\n";
		goto err;
	}

	/* read the elf header */
	if(oread(fd, (char *)&eh, sizeof(eh)) != sizeof(eh)) {
		goto serr;
	}

	if (IS_ELF(eh)) {
		ph = (Elf32_Phdr *) buf;
		olseek(fd, eh.e_phoff, 0);
		if(oread(fd, (char *)ph, 4096) != 4096) {
			goto serr;
		}

		for(i = 0; i < eh.e_phnum; i++, ph++) {
			if(ph->p_type == PT_LOAD) {
				olseek(fd, ph->p_offset, 0);
				printf("0x%x:0x%x, ",ph->p_paddr, ph->p_filesz);
				if(oread(fd, (char *)ph->p_paddr, ph->p_filesz)
				   !=  ph->p_filesz) {
					goto serr;
				}
				if(ph->p_memsz > ph->p_filesz) {
					printf("Zero 0x%x:0x%x, ",
						   ph->p_paddr + ph->p_filesz,
						   ph->p_memsz - ph->p_filesz);
					bzero((char *)ph->p_paddr + ph->p_filesz,
						  ph->p_memsz - ph->p_filesz);
				}
			}
		}
		printf("start at 0x%x\n", eh.e_entry);
		set_entry_addr(eh.e_entry);
///////////////
//Diags
                oclose(fd);
//////////////
		return 0;
	serr:
		errs = "%s sz err\n";
	err:
		printf(errs, fname);
		return -1;
	}
	else {
		int len;

		len = olseek(fd, 0, 2);
		if (len == -1) {
			errs = "seek end err: %s\n";
			goto err;
		}
	
		if (olseek(fd, 0, 0) < 0) {
			errs = "seek start err: %s\n";
			goto errx;
		}

		if (oread(fd, (char *)load_address, len) < 0) {
			errs = "read err: %s\n";
			goto errx;
		}

		printf("Loaded %s at %x length %d\n", fname, load_address, len);
///////////////
//Diags
                oclose(fd);
//////////////
		return 0;
	errx:
		printf(errs, fname);
		return -1;
	}
}

/*
 * do_disk_init - load file from the disk. Tries /dev/wd0a/bsd and
 * /dev/wd1a/bsd.
 */
void
do_disk_init()
{
	int i;
	uint32 entry_point = -1;

	debug(("do_disk_init\n"));

	for (i = 0; i < 2 && entry_point == -1; ++i) {
		char filename[256];
		sprintf(filename, "/dev/wd%ca/bsd", '0' + i);
		entry_point = loadelffile(filename);
	}

	if (entry_point != -1) {
		printf("loaded file at %x\n", entry_point);
		set_entry_addr(entry_point);
	}
	else {
		printf("boot failed\n");
	}
}


/*
 * do_disk_toshiba_init - load toshiba file from disk
 */
void
do_disk_toshiba_init()
{
	int i;
	char filename[256];
	int result = -1;
	
	for (i = 0; i < 2 && result == -1; ++i) {

		sprintf(filename, "/dev/wd%ca/ssc-mgmt.bin", '0' + i);
		result = loadfile(filename, 0xb0008000);
	}

	if (result != -1) {
		printf("Loaded file %s at 0x%x\n", filename, 0xb0008000);
	}
	else {
		printf("Load failed\n");
	}
}



/*
 * loadelffile - load an ELF file from the disk
 */
int
loadelffile(char *fname)
{
	int fd, i;
	Elf32_Ehdr eh;
	Elf32_Phdr *ph;
	char *errs = 0;
	char buf[4096];

	if ((fd = oopen(fname, 0)) < 0) {
		errs="open err: %s\n";
		goto err;
	}

	/* read the elf header */
	if(oread(fd, (char *)&eh, sizeof(eh)) != sizeof(eh)) {
		goto serr;
	}

	ph = (Elf32_Phdr *) buf;
	olseek(fd, eh.e_phoff, 0);
	if(oread(fd, (char *)ph, 4096) != 4096) {
		goto serr;
	}

	for(i = 0; i < eh.e_phnum; i++, ph++) {
		if(ph->p_type == PT_LOAD) {
			olseek(fd, ph->p_offset, 0);
			printf("0x%x:0x%x, ",ph->p_paddr, ph->p_filesz);
			if(oread(fd, (char *)ph->p_paddr, ph->p_filesz) !=  ph->p_filesz) {
				goto serr;
			}
			if(ph->p_memsz > ph->p_filesz) {
				printf("Zero 0x%x:0x%x, ",
					ph->p_paddr + ph->p_filesz,
					ph->p_memsz - ph->p_filesz);
				bzero((char *)ph->p_paddr + ph->p_filesz,
					ph->p_memsz - ph->p_filesz);
			}
		}
	}
	printf("start at 0x%x\n", eh.e_entry);
	return(eh.e_entry);
serr:
	errs = "%s sz err\n";
err:
	printf(errs, fname);
	return (-1);
}


/*
 * loadfile - load a binary file from disk at address 
 */
int
loadfile(char *fname, uint32 address)
{
	int fd;
	int len;
	char *errs;

	if ((fd = oopen(fname, 0)) < 0) {
		errs = "open err: %s\n";
		goto err;
	}

	len = olseek(fd, 0, 2);
	if (len == -1) {
		errs = "seek end err: %s\n";
		goto err;
	}
	printf("Opened %s len %d\n", fname, len);
	
	if (olseek(fd, 0, 0) < 0) {
		errs = "seek start err: %s\n";
		goto err;
	}

	if (oread(fd, (char *)address, len) < 0) {
		errs = "read err: %s\n";
		goto err;
	}

	return 0;
 err:
	printf(errs, fname);
	return -1;
}
