#define IPM_LINK_MAX    4
/* testing function */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h> 
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <nfx-types.h>
#include "exports/ipmd-lib.h"


#define TMPSIZE			800

#define INTF_ENA		1
#define INTF_DIS		2
#define IP_ADD			3
#define IP_DEL			4
#define RT_ADD			5
#define RT_DEL			6
#define ARP_ADD			7
#define ARP_DEL			8
#define IPMD_RESTART	9
#define IPMD_SHUT		10
#define RT_SHOW			11
#define ARP_SHOW		12
#define INTF_SHOW		13
#define IP_SHOW			14
#define SHM_SHOW		15
#define SHM_CHCK		16
#define UDPCONN_SHOW	17
#define IFSTAT_SHOW		18
#define IPSTAT_SHOW		19
#define UDPSTAT_SHOW	20
#define TCPCONN_SHOW	21

struct cmdent 
{
	char	*cmd;
	int		pcount;
	int		index;
	char	*help;
}  
cmd_table [] = {  
	{"enaIntf", 3, INTF_ENA, "cmd enaIntf <interface>"},
	{"disIntf", 3, INTF_DIS, "cmd disIntf <interface>"},
	{"addIp",   5, IP_ADD,	 "cmd addIp <intf> <ip> <maks>"},
	{"delIp",   4, IP_DEL,	 "cmd delIp <intf> <ip>"},
	{"addRt",   5, RT_ADD,	 "cmd addRt <dest> <mask> <gw>"},
	{"delRt",   5, RT_DEL,	 "cmd delRt <dest> <mask> <gw>"},	
	{"addArp",  3, ARP_ADD,	 "cmd addArp <ip>"}, /* mac default */
	{"delArp",  3, ARP_DEL,	 "cmd delArp <ip>"},
	{"rstIpmd", 2, IPMD_RESTART, "cmd rstIpmd"},
	{"shutIpmd",2, IPMD_SHUT, "cmd shutIpmd"},
	{"showRt",  2, RT_SHOW,  "cmd showRt"},
	{"showArp", 2, ARP_SHOW, "cmd showArp"},
	{"showIntf",2, INTF_SHOW,"cmd showIntf"},
	{"showIp",	2, IP_SHOW,	 "cmd showIp"},
	{"showShm", 2, SHM_SHOW, "cmd showShm"},
	{"chckShm", 2, SHM_CHCK, "cmd chckShm"},
	{"showUdpConn", 2, UDPCONN_SHOW, "cmd showUdpConn"},
	{"showIfStat",  2, IFSTAT_SHOW,  "cmd showIfStat"},	
	{"showIpStat",  2, IPSTAT_SHOW,  "cmd showIpStat"},
	{"showUdpStat", 2, UDPSTAT_SHOW, "cmd showUdpStat"},
	{"showTcpConn", 2, TCPCONN_SHOW, "cmd showTcpConn"},
	{"last", 0, 0, NULL}
};

 
/* make sure tmpbuf is 32 bits aligned */
uint32	tmpbuf[TMPSIZE];
char	def_mac[] = {1, 2, 3, 4, 5, 6};


int
main(int argc, char **argv)
{
  int index;
  int i, count = 0, need = 0;
  ipm_rt_t		*ipm_rtp;
  ipm_arp_t		*ipm_arpp;
  ipm_if_t		*ipm_ifp;
  ipm_ip_t		*ipm_ipp;
  struct cmdent	*cmd;
  

  /* find out what cmd */
  for (cmd = cmd_table; ; cmd++) {
	  if (strcmp(cmd->cmd, "last") == 0) 
	  {
		  /* no cmd found */
		  printf("invalid cmd %s\n", argv[1]);
		  return 0;
	  }

	  if (strcmp(cmd->cmd, argv[1]) == 0) {
		  /* cmd found */
		  if (argc != cmd->pcount) {
			  printf("incorrect format: %s\n", cmd->help);
			  return 0;
		  }
		  index = cmd->index;
		  break;
	  }
	  
  }
			  

  switch(index)
	{
	case INTF_ENA:
		ipmd_EnableIntf(argv[2], "ea", 0);
		break;

	case INTF_DIS:
		ipmd_DisableIntf(argv[2], "ea", 0);
		break;

	case IP_ADD:
	  ipmd_AddIp(argv[2], inet_addr(argv[3]), inet_addr(argv[4]), "ea", 0);
	  break;

	case IP_DEL:
		ipmd_DelIp(argv[2], inet_addr(argv[3]), "ea", 0);
		break;
	  
	case RT_ADD:
		ipmd_AddRt(inet_addr(argv[2]), 
				   inet_addr(argv[3]), 
				   inet_addr(argv[4]),
				   "ea", 0);	  
		break;

	case RT_DEL:
		ipmd_DelRt(inet_addr(argv[2]), 
				   inet_addr(argv[3]), 
				   inet_addr(argv[4]),
				   "ea", 0);	  
		break;

	case ARP_ADD:
		ipmd_AddArp(inet_addr(argv[2]), def_mac, "ea", 0);
		break;

	case ARP_DEL:
		ipmd_DelArp(inet_addr(argv[2]), "ea", 0);
		break;

	case IPMD_RESTART:
		ipmd_Restart("ea");
		break;

	case IPMD_SHUT:
		ipmd_ShutDown("ea");
		break;

	case RT_SHOW:
		/* test to get system route table */  
		if (ipmd_GetRtTbl((ipm_rt_t*)tmpbuf, TMPSIZE*sizeof(uint32), &count, &need) 
			== IPM_GET_OK) {
			printf("print system route table\n");
			for (i=0,ipm_rtp = (ipm_rt_t*)tmpbuf; i<count; i++, ipm_rtp++) {
				printf("ip %x; mask %x; nexthop %x; flag %x; intf %s\n", 
					   ipm_rtp->rt_addr, ipm_rtp->rt_mask, ipm_rtp->rt_nhop,
					   ipm_rtp->rt_flags, ipm_rtp->rt_ifname);
			} 
		} else
			printf ("err getting rt table\n");
		break;
		
	case ARP_SHOW:
		if (ipmd_GetArpTbl((ipm_arp_t*)tmpbuf, TMPSIZE*sizeof(uint32), &count, &need) 
			== IPM_GET_OK) {
			printf("print system arp table\n");
			for (i=0,ipm_arpp = (ipm_arp_t*)tmpbuf; i<count; i++, ipm_arpp++) {
				printf("ip %x; mac %x; age %x; intf %s\n", 
					   ipm_arpp->arp_addr, *(uint32*)ipm_arpp->arp_mac, 
					   ipm_arpp->arp_age, ipm_arpp->arp_ifname);
			} 
		} else
			printf("err geting arp table\n");
		break;
		
	case INTF_SHOW:
	  if (ipmd_GetIfByName(NULL, (ipm_if_t*)tmpbuf, TMPSIZE*sizeof(uint32), &count, &need)
		  == IPM_GET_OK) {
		  printf("print system interface table\n");
		  for (i=0,ipm_ifp = (ipm_if_t*)tmpbuf; i<count; i++, ipm_ifp++) {
			  printf("intf %s; slot %d; cpu %d; link %d; flags %x; mtu %d, mac %x:%x:%x:%x:%x:%x\n",
					 ipm_ifp->if_name, ipm_ifp->if_slot, ipm_ifp->if_cpu, 
					 ipm_ifp->if_link, ipm_ifp->if_flags, ipm_ifp->if_mtusize,
					 ipm_ifp->if_mac[0]&0xff, ipm_ifp->if_mac[1]&0xff, ipm_ifp->if_mac[2]&0xff,
					 ipm_ifp->if_mac[3]&0xff, ipm_ifp->if_mac[4]&0xff, ipm_ifp->if_mac[5]&0xff);

			  
		  } 		  
	  }
	  else
		  printf("err getting intf table\n");
	  break;

	case IP_SHOW:
		if (ipmd_GetIpByName(NULL, (ipm_ip_t*)tmpbuf, TMPSIZE*sizeof(uint32), &count, &need)
			== IPM_GET_OK) {
			printf("print system IP table\n");
			for (i=0,ipm_ipp = (ipm_ip_t*)tmpbuf; i<count; i++, ipm_ipp++) {
				printf("ip %x; mask %x; intf %s\n", ipm_ipp->ip_addr, ipm_ipp->ip_mask,
					   ipm_ipp->ip_ifname);
				  
			} 		  
		}
		else
			printf("err getting Ip table\n");
		break;
		
	case SHM_SHOW:
		if (ipmd_ShowShmem() != NFX_OK)
			printf("showShmem failed\n");
		break;
		
	case SHM_CHCK:
		ipmd_ValidateShmem();
		break;
		
	case UDPCONN_SHOW:
#if defined(SSC_x86) || defined(SSC_mips)
		/* SSC */
		if (ipmd_GetUdpConnSSC((ipm_conn_t*)tmpbuf, 
								 TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("udp conn table ssc  get failed\n");
		else {
			ipm_conn_t		*ipm_connp= (ipm_conn_t*)tmpbuf;
			printf("ssc table\n");
			for (i=0; i < count; i++) {
				printf("laddr:%x; lport:%d; faddr:%x; fport:%x\n",
					   ipm_connp->conn_lAddr, ntohs(ipm_connp->conn_lPort),
					   ipm_connp->conn_rAddr, ntohs(ipm_connp->conn_rPort));
				  
				ipm_connp++;
			}
		}
#endif 
		/* TXRX */
		if (ipmd_GetUdpConnTable((ipm_conn_t*)tmpbuf, 
								 TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("udp conn table get failed\n");
		else {
			ipm_conn_t		*ipm_connp= (ipm_conn_t*)tmpbuf;
			printf("\n txrx table\n");
			for (i=0; i < count; i++) {
				printf("laddr:%x; lport:%d; faddr:%x; fport:%d\n",
					   ipm_connp->conn_lAddr, ntohs(ipm_connp->conn_lPort),
					   ipm_connp->conn_rAddr, ntohs(ipm_connp->conn_rPort));
				  
				ipm_connp++;
			}
		}
		break;

	case TCPCONN_SHOW:
#if defined(SSC_x86) || defined(SSC_mips)
		/* SSC */
		if (ipmd_GetTcpConnSSC((ipm_conn_t*)tmpbuf, 
								 TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("tcp conn table ssc  get failed\n");
		else {
			ipm_conn_t		*ipm_connp= (ipm_conn_t*)tmpbuf;
			printf("ssc table\n");
			for (i=0; i < count; i++) {
				printf("laddr:%x; lport:%d; faddr:%x; fport:%x; state:%d\n",
					   ipm_connp->conn_lAddr, ntohs(ipm_connp->conn_lPort),
					   ipm_connp->conn_rAddr, ntohs(ipm_connp->conn_rPort),
					   ipm_connp->conn_state);
				  
				ipm_connp++;
			}
		}
#endif /* SSC */

		/* TXRX */
		if (ipmd_GetTcpConnTable((ipm_conn_t*)tmpbuf, 
								 TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("tcp conn table get failed\n");
		else {
			ipm_conn_t		*ipm_connp= (ipm_conn_t*)tmpbuf;
			printf("\n txrx table\n");
			for (i=0; i < count; i++) {
				printf("laddr:%x; lport:%d; faddr:%x; fport:%d; state: %d\n",
					   ipm_connp->conn_lAddr, ntohs(ipm_connp->conn_lPort),
					   ipm_connp->conn_rAddr, ntohs(ipm_connp->conn_rPort),
					   ipm_connp->conn_state);
				  
				ipm_connp++;
			}
		}
		
		{
			ipm_conn_t	conn;
			ipm_conn_t	next;
			ipm_conn_t	*connp = NULL;
			
			printf("using getnext to get tcp table\n");
			while (ipmd_GetTcpNxtConn(connp, &next) != IPM_GET_ERR) {
				memcpy(&conn, &next, sizeof(ipm_conn_t));
				connp = &conn;
				printf("laddr:%x; lport:%d; faddr:%x; fport:%d; state: %d\n",
					   connp->conn_lAddr, ntohs(connp->conn_lPort),
					   connp->conn_rAddr, ntohs(connp->conn_rPort),
					   connp->conn_state);
			}
		}
		break;

	case IFSTAT_SHOW:
		if (ipmd_GetIfStats(NULL, (ipm_ifstat_t*)tmpbuf, 
							TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("ifstat get failded\n");
		else {
			ipm_ifstat_t *ipm_ifstatp = (ipm_ifstat_t*)tmpbuf;
			for (i=0; i < count; i++) {
				printf("if:%s; speed:%x; hdrlen:%d; addrlen:%d\n",
					   ipm_ifstatp->ifi_ifname, ipm_ifstatp->ifi_speed, 
					   ipm_ifstatp->ifi_hdrlen, ipm_ifstatp->ifi_addrlen);
				
				ipm_ifstatp++;
			}
		}
		break;
		
	case IPSTAT_SHOW:
		if (ipmd_GetIpStats(-1, (ipm_ipstat_t*)tmpbuf,
							TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("ipstat get failded\n");
		else {
			ipm_ipstat_t	*ipm_ipstatp = (ipm_ipstat_t*)tmpbuf;
			for (i=0; i<count; i++) {
				printf("card:%d; ttl:%d; in_ip:%d; out_ip:%d\n",
					   ipm_ipstatp->ips_card, ipm_ipstatp->ips_defaultTTL,
					   ipm_ipstatp->ips_total, ipm_ipstatp->ips_localout);
				ipm_ipstatp++;
			}
		}
		break;
		
	case UDPSTAT_SHOW:
		if (ipmd_GetUdpStats(-1, (ipm_udpstat_t*)tmpbuf,
							 TMPSIZE*sizeof(uint32), &count, &need) != IPM_GET_OK)
			printf("udp stat get failed\n");
		else {
			ipm_udpstat_t	*ipm_udpstatp = (ipm_udpstat_t*)tmpbuf;
			for (i=0; i<count; i++) {
				printf("card:%d; in_udp:%d; out_udp:%d\n",ipm_udpstatp->udps_card,
					   ipm_udpstatp->udps_ipackets, ipm_udpstatp->udps_opackets);
				ipm_udpstatp++;
			}
		}
		break;

	default:
		printf("cmd %s unrecoginized\n", cmd->cmd);
	}
  return 1;
}
