/*
 *------------------------------------------------------------------------------
 *
 * Name:	raid-sm.c
 *
 * RCS:		$Id: raid-sm.c,v 1.23 2002/10/23 18:37:03 nitinb Exp $
 *
 * Copyright (C) 2001, 2002 Agile Storage Inc.
 *
 * Description:	RAID config manager device state machines.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 4/17/01
 *
 *------------------------------------------------------------------------------
 */

#include "raid-common.h"
#include "raid-xyratex.h"
#include "../sm-scsi/scsi-dev-cmds.h"

void raid_do_sdm_open(raid_dev_t *, void *);
void raid_do_sdm_info(raid_dev_t *, void *);
void raid_do_sdm_open_failed(raid_dev_t *, void *);
void raid_do_open_reply_busy(raid_dev_t *, void *);
void raid_do_open_reply_success_after_open(raid_dev_t *, void *);
void raid_do_open_reply_success_after_info(raid_dev_t *, void *);
void raid_do_close_reply_not_open(raid_dev_t *, void *);
void raid_save_close_and_do_sdm_close(raid_dev_t *, void *);
void raid_do_hold_close(raid_dev_t *, void *);
void raid_do_message_req(raid_dev_t *, void *);
void raid_do_message_req_reply_not_open(raid_dev_t *, void *);
void raid_do_message_req_reply_busy(raid_dev_t *, void *);
void raid_do_sdm_relay_done_success(raid_dev_t *, void *);
void raid_do_sdm_relay_done_failed(raid_dev_t *, void *);
void raid_do_send_message_req_reply(raid_dev_t *, void *);
void raid_do_message_req_reply_success_and_close(raid_dev_t *, void *);
void raid_do_message_req_reply_failed_and_close(raid_dev_t *, void *);
void raid_do_message_req_reply_io_error(raid_dev_t *, void *);
void raid_do_close_reply_success(raid_dev_t *, void *);
void raid_do_short_close_reply_success(raid_dev_t *, void *);
void raid_do_sdm_close(raid_dev_t *, void *);

raid_sm_t raid_dev_sm[RAID_STATE_COUNT] = {
{
	RAID_STATE_CLOSED,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_SDM_OPEN_PENDING,			raid_do_sdm_open },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_CLOSED,				raid_do_close_reply_not_open },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_CLOSED,				raid_do_message_req_reply_not_open },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_TERMINAL,				NULL },
	}
},
{
	RAID_STATE_SDM_OPEN_PENDING,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_SDM_OPEN_PENDING,			raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_OPEN,				raid_do_open_reply_success_after_open },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_CLOSED,				raid_do_sdm_open_failed },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_SDM_OPEN_PENDING,                    raid_do_sdm_info },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_OPEN,				raid_do_open_reply_success_after_info },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_CLOSED,				raid_do_sdm_open_failed },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_INVALID,				NULL }, /* XXX */
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_SDM_OPEN_PENDING,			raid_do_message_req_reply_not_open },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_INVALID,				NULL },	/* XXX */
	}
},
{
	RAID_STATE_OPEN,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_OPEN,				raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_SDM_CLOSE_PENDING,			raid_save_close_and_do_sdm_close },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_OPEN_ACTIVE,				raid_do_message_req },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },
	}
},
{
	RAID_STATE_OPEN_ACTIVE,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_OPEN_ACTIVE,				raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_HOLDING_CLOSE,			raid_do_hold_close },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_OPEN_ACTIVE,				raid_do_message_req_reply_busy },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_OPEN,				raid_do_send_message_req_reply }, 
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_OPEN_ACTIVE,				raid_do_sdm_relay_done_success },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_OPEN,				raid_do_sdm_relay_done_failed },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_OPEN_ACTIVE,				raid_do_sdm_relay_done_success },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_DESTROY_PENDING,			NULL },
	}
},
{
	RAID_STATE_HOLDING_CLOSE, 
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_HOLDING_CLOSE,			raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_HOLDING_CLOSE,			raid_do_close_reply_not_open },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_HOLDING_CLOSE,			raid_do_message_req_reply_not_open },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_SDM_CLOSE_PENDING,			raid_do_message_req_reply_success_and_close},
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_HOLDING_CLOSE,			raid_do_sdm_relay_done_success },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_SDM_CLOSE_PENDING,			raid_do_message_req_reply_failed_and_close },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_HOLDING_CLOSE,			raid_do_sdm_relay_done_success },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_INVALID,				NULL },	/* XXX */
	}
},
{
	RAID_STATE_SDM_CLOSE_PENDING,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_SDM_CLOSE_PENDING,			raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_SDM_CLOSE_PENDING,			raid_do_close_reply_not_open },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_CLOSED,				raid_do_close_reply_success },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_CLOSED,				raid_do_close_reply_success },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_SDM_CLOSE_PENDING,			raid_do_message_req_reply_not_open },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_INVALID,				NULL },	/* XXX */
	}
},
{
	RAID_STATE_DESTROY_PENDING,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_DESTROY_PENDING,			raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_DESTROY_PENDING,			raid_do_close_reply_success },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_DESTROY_PENDING,			raid_do_message_req_reply_not_open },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	raid_do_message_req_reply_io_error },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	raid_do_message_req_reply_io_error },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	raid_do_message_req_reply_io_error },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_DESTROY_PENDING,			NULL },
	}
},
{
	RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,
	{
		{ RAID_EVENT_OPEN_REQ,			RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	raid_do_open_reply_busy },
		{ RAID_EVENT_SDM_OPEN_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_OPEN_ALREADY_OPEN,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_INFO_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_CLOSE_REQ,			RAID_STATE_TERMINAL,				raid_do_short_close_reply_success },
		{ RAID_EVENT_SDM_CLOSE_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_CLOSE_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_MESSAGE_REQ,		RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	raid_do_message_req_reply_busy },
		{ RAID_EVENT_RESPONSE_REQ,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_DONE,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_FAILED,		RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_SDM_RELAY_SCSI_ERROR,	RAID_STATE_INVALID,				NULL },
		{ RAID_EVENT_DEVICE_REMOVED,		RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },
	}
}};

void 
raid_do_sdm_open(raid_dev_t *dev, void *cookie) 
{
	sdm_openReqMsg_t *	sdm_open_req;

	RAID_DBG(2, ("raid_do_sdm_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the RAID_OPEN context */
	VERIFY(cookie);
	VERIFY(dev->open_req_context == NULL);
	dev->open_req_context = (raid_msg_context_t *)cookie;

	/* Send SDM open message */
	sdm_open_req = (sdm_openReqMsg_t *)malloc(sizeof(sdm_openReqMsg_t));

	sdm_open_req->hdr.mid = SDM_MID_DEV_OPEN_REQ;
	sdm_open_req->hdr.chandle = (uint32)dev;
	sdm_open_req->id = dev->id;
	sdm_open_req->slot = 0;
	sdm_open_req->flags = 0;

	raid_send_msg(sdm_open_req, sizeof(sdm_openReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_open_req;
}

void 
raid_do_sdm_info(raid_dev_t *dev, void *cookie)
{
	sdm_devInfoReqMsg_t *	sdm_info_req;

	RAID_DBG(2, ("raid_do_sdm_info: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* The cookie is the SDM OPEN reponse */
	VERIFY(cookie);
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Send SDM info message */
	sdm_info_req = (sdm_devInfoReqMsg_t *)malloc(sizeof(sdm_devInfoReqMsg_t));

	sdm_info_req->hdr.mid = SDM_MID_DEV_INFO_REQ;
	sdm_info_req->hdr.chandle = (uint32)dev;
	sdm_info_req->id = dev->id;

	raid_send_msg(sdm_info_req, sizeof(sdm_devInfoReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_info_req;
}


void 
raid_do_sdm_open_failed(raid_dev_t *dev, void *cookie)
{
	raid_openRspMsg_t	raid_open_rsp;
	
	RAID_DBG(2, ("raid_do_sdm_open_failed: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SDM open or info response */
	VERIFY(cookie);

	/* Reply to raid open message -- no such device */
	VERIFY(dev->open_req_context);
	raid_open_rsp.header.mid = RAID_MID_OPEN_RSP;
	raid_open_rsp.header.cookie = dev->open_req_context->req_msg->cookie;
	raid_open_rsp.rCode = RAID_RCODE_NO_DEVICE;
	raid_send_msg(&raid_open_rsp, sizeof(raid_openRspMsg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the raid open/info context */
	VERIFY(dev->open_req_context);
	raid_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void 
raid_do_open_reply_busy(raid_dev_t *dev, void *cookie)
{
	raid_openRspMsg_t	raid_open_rsp;
	raid_msg_context_t *	context;

	RAID_DBG(2, ("raid_do_open_reply_busy: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is raid open message context. */
	VERIFY(cookie);
	context = cookie;

	/* Reply to current raid open message -- device is busy */
	raid_open_rsp.header.mid = RAID_MID_OPEN_RSP;
	raid_open_rsp.header.cookie = context->req_msg->cookie;
	raid_open_rsp.rCode = RAID_RCODE_BUSY;
	raid_send_msg(&raid_open_rsp, sizeof(raid_openRspMsg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	raid_free_context(context);
}

void 
raid_do_open_reply_success_after_open(raid_dev_t *dev, void *cookie) 
{
	sdm_openRspMsg_t *	sdm_open_rsp;
	raid_openRspMsg_t	raid_open_rsp;

	RAID_DBG(2, ("raid_do_open_reply_success_after_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM open response */
	VERIFY(cookie);
	sdm_open_rsp = cookie;
	VERIFY(sdm_open_rsp->hdr.mid == SDM_MID_DEV_OPEN_RSP);
	dev->open_handle = sdm_open_rsp->handle;
	dev->sdm_close_on_close = 1;

	/* Reply to raid open message */
	VERIFY(dev->open_req_context);
	raid_open_rsp.header.mid = RAID_MID_OPEN_RSP;
	raid_open_rsp.header.cookie = dev->open_req_context->req_msg->cookie;
	raid_open_rsp.rCode = RAID_RCODE_SUCCESS;
	raid_open_rsp.scsi_id = dev->id;
	raid_open_rsp.wwn = dev->wwn;
	raid_open_rsp.lun = dev->lun;
	raid_send_msg(&raid_open_rsp, sizeof(raid_openRspMsg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the raid open context */
	VERIFY(dev->open_req_context);
	raid_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void 
raid_do_open_reply_success_after_info(raid_dev_t *dev, void *cookie) 
{
	sdm_devInfoRspMsg_t *	sdm_info_rsp;
	raid_openRspMsg_t	raid_open_rsp;

	RAID_DBG(2, ("raid_do_open_reply_success_after_info: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM info response */
	/* Cookie is the SDM info response */
	VERIFY(cookie);
	sdm_info_rsp = cookie;
	VERIFY(sdm_info_rsp->hdr.mid == SDM_MID_DEV_INFO_RSP);
	dev->open_handle = sdm_info_rsp->openHandle;
	dev->sdm_close_on_close = 0;

	/* Reply to raid open message */
	VERIFY(dev->open_req_context);
	raid_open_rsp.header.mid = RAID_MID_OPEN_RSP;
	raid_open_rsp.header.cookie = dev->open_req_context->req_msg->cookie;
	raid_open_rsp.rCode = RAID_RCODE_SUCCESS;
	raid_open_rsp.scsi_id = dev->id;
	raid_open_rsp.wwn = dev->wwn;
	raid_open_rsp.lun = dev->lun;
	raid_send_msg(&raid_open_rsp, sizeof(raid_openRspMsg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the raid open context */
	VERIFY(dev->open_req_context);
	raid_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void 
raid_do_close_reply_not_open(raid_dev_t *dev, void *cookie) 
{
	raid_closeRspMsg_t	resp;
	raid_msg_context_t *	context;

	RAID_DBG(2, ("raid_do_close_not_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the close context */
	VERIFY(cookie);
	context = cookie;

	bzero(&resp, sizeof(raid_closeRspMsg_t));
	resp.header.mid = RAID_MID_CLOSE_RSP;
	resp.header.cookie = context->req_msg->cookie;
	resp.rCode = RAID_RCODE_NOT_OPEN;
	raid_send_msg(&resp, sizeof(raid_closeRspMsg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	raid_free_context(context);
}

void 
raid_save_close_and_do_sdm_close(raid_dev_t *dev, void *cookie)
{
	RAID_DBG(2, ("raid_save_close_and_do_sdm_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the RAID_CLOSE context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (raid_msg_context_t *)cookie;


	/* Do we *need* to SDM close on a close? */
	if (dev->sdm_close_on_close) 
		raid_do_sdm_close(dev, NULL);
	else {
		/* Skip SDM close -- fake out a successfull SDM close completion */
		raid_sm_run(dev, NULL, RAID_EVENT_SDM_CLOSE_DONE);
	}
}

void 
raid_do_hold_close(raid_dev_t *dev, void *cookie)
{
	RAID_DBG(2, ("raid_do_hold_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the RAID_CLOSE context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (raid_msg_context_t *)cookie;
}

void 
raid_do_message_req(raid_dev_t *dev, void *cookie)
{
	raid_cmd_transformer_t * cmd_t;
	sdm_relayCdbReqMsg_t *	 scsi_req = NULL;
	int			 scsi_req_len = 0;
	raid_msg_context_t *	 context;
	int32			 rc;
	raid_genericRspMsg_t *	 rsp;

	RAID_DBG(2, ("raid_do_message_req: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is message context */
        VERIFY(cookie);
        VERIFY(dev->message_req_context == NULL);
        dev->message_req_context = cookie;
	
	cmd_t = dev->dev_cmd_transformer;
	while (cmd_t->req_mid != dev->message_req_context->req_msg->mid && 
	       cmd_t->req_mid != 0)
		++cmd_t;
	VERIFY(cmd_t->req_mid != 0);
	VERIFY(cmd_t->constructor);
	rc = (*cmd_t->constructor)(dev, dev->message_req_context->req_msg, &scsi_req, &scsi_req_len);
	if (rc !=  NFX_OK) {
		RAID_DBG(2, ("raid_do_message_req: request failed: rCode %d\n", rc));
		context = dev->message_req_context;
		rsp = (raid_genericRspMsg_t *)malloc(sizeof(raid_genericRspMsg_t));
		bzero(rsp, sizeof(raid_genericRspMsg_t));
		rsp->header.mid = (uint32)context->req_msg->mid + 1;
		rsp->header.cookie = context->req_msg->cookie;
		rsp->rCode = rc;
		context->rsp_msg = (raid_msgHdr_t *)rsp;
		context->rsp_msg_len = sizeof(raid_genericRspMsg_t);
		raid_sm_run(dev, NULL, RAID_EVENT_RESPONSE_REQ);
	}
	else {
		VERIFY(scsi_req);
		raid_send_msg(scsi_req, scsi_req_len, "sdm", 0, 0);

		/* Stash the message */
		VERIFY(dev->pending_req_context == NULL);
		dev->pending_req_context = scsi_req;
	}
}

void 
raid_do_message_req_reply_not_open(raid_dev_t *dev, void *cookie)
{
	raid_cmd_transformer_t * cmd_t;
	raid_msg_context_t *	 context;
	raid_genericRspMsg_t 	 resp;

	RAID_DBG(2, ("raid_do_message_req_reply_not_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is raid message context */
	VERIFY(cookie);
	context = cookie;

	cmd_t = dev->dev_cmd_transformer;
	while (cmd_t->req_mid != context->req_msg->mid && 
	       cmd_t->req_mid != 0)
		++cmd_t;
	VERIFY(cmd_t->req_mid != 0);

	bzero(&resp, sizeof(raid_genericRspMsg_t));
	resp.header.mid = cmd_t->rsp_mid;
	resp.header.cookie = context->req_msg->cookie;
	resp.rCode = RAID_RCODE_NOT_OPEN;
	raid_send_msg(&resp, sizeof(raid_genericRspMsg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);

	/* Free the raid message context */
	raid_free_context(context);
}

void 
raid_do_message_req_reply_busy(raid_dev_t *dev, void *cookie)
{
	raid_cmd_transformer_t * cmd_t;
	raid_msg_context_t *	 context;
	raid_genericRspMsg_t 	 resp;

	RAID_DBG(2, ("raid_do_message_req_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is raid message context */
	VERIFY(cookie);
	context = cookie;

	cmd_t = dev->dev_cmd_transformer;
	while (cmd_t->req_mid != context->req_msg->mid && 
	       cmd_t->req_mid != 0)
		++cmd_t;
	VERIFY(cmd_t->req_mid != 0);

	bzero(&resp, sizeof(raid_genericRspMsg_t));
	resp.header.mid = cmd_t->rsp_mid;
	resp.header.cookie = context->req_msg->cookie;
	resp.rCode = RAID_RCODE_BUSY;
	raid_send_msg(&resp, sizeof(raid_genericRspMsg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);

	/* Free the raid message context */
	raid_free_context(context);
}

void 
raid_do_message_req_reply_io_error(raid_dev_t *dev, void *cookie) 
{
	void *			 scsi_resp;
	raid_cmd_transformer_t * cmd_t;
	raid_msg_context_t *	 context;
	raid_genericRspMsg_t 	 resp;

	RAID_DBG(2, ("raid_do_message_req_reply_io_error: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SCSI relay response */
	VERIFY(cookie);
	scsi_resp = cookie;

	VERIFY(dev->message_req_context);
	context = dev->message_req_context;

	cmd_t = dev->dev_cmd_transformer;
        while (cmd_t->req_mid != context->req_msg->mid &&
               cmd_t->req_mid != 0)
                ++cmd_t;
        VERIFY(cmd_t->req_mid != 0);

	if (cmd_t->aborter)
		(*cmd_t->aborter)(dev, scsi_resp);

        bzero(&resp, sizeof(raid_genericRspMsg_t));
        resp.header.mid = cmd_t->rsp_mid;
		resp.header.cookie = context->req_msg->cookie;
        resp.rCode = RAID_RCODE_IO_ERROR;
        raid_send_msg(&resp, sizeof(raid_genericRspMsg_t),
                      context->req_src_app,
                      context->req_src_cpu,
                      context->req_src_slot);

	/* Free the raid message context */
	raid_free_context(dev->message_req_context);
	dev->message_req_context = NULL;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;
}

void 
raid_do_sdm_relay_done_success(raid_dev_t *dev, void *cookie)
{
	sdm_relayCdbRspMsg_t *	 scsi_resp;
	scsi_sense_t *		 sense;
	byte			 scsi_status;
	byte			 scsi_sense_key;
	raid_cmd_transformer_t * cmd_t;
	raid_msg_context_t *	 context;
	sdm_relayCdbReqMsg_t *	 scsi_req = NULL;
	int			 scsi_req_len = 0;
	raid_genericRspMsg_t * 	 resp = NULL;
	int32			 resp_len = 0;
	void*			old_pending_req_context;

	RAID_DBG(2, ("raid_do_sdm_relay_done_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SCSI relay response */
	VERIFY(cookie);
	scsi_resp = cookie;

	VERIFY(dev->message_req_context);
	context = dev->message_req_context;

	cmd_t = dev->dev_cmd_transformer;
        while (cmd_t->req_mid != context->req_msg->mid &&
               cmd_t->req_mid != 0)
                ++cmd_t;
        VERIFY(cmd_t->req_mid != 0);

	scsi_status = scsi_resp->status & SD_DEV_STATUS_MASK;
	if (scsi_status == SCSI_DEV_STATUS_CHECK) {
		sense = (scsi_sense_t *)scsi_resp->senseData;
		scsi_sense_key = sense->senseKey;
	}
	if (scsi_status != SCSI_DEV_STATUS_GOOD &&
	    (scsi_status != SCSI_DEV_STATUS_CHECK || scsi_sense_key != SCSI_SENSE_KEY_VENDOR_SPECIFIC)) {
		VERIFY(cmd_t->aborter);
		(*cmd_t->aborter)(dev, scsi_resp);
		resp_len = sizeof(raid_genericRspMsg_t);
		resp = (raid_genericRspMsg_t *)malloc(resp_len);
		bzero(resp, sizeof(raid_genericRspMsg_t));
		resp->header.mid = cmd_t->rsp_mid;
		resp->header.cookie = context->req_msg->cookie;
		resp->rCode = RAID_RCODE_IO_ERROR;
	}
	else {
		VERIFY(cmd_t->deconstructor);
		(*cmd_t->deconstructor)(dev, scsi_resp, &scsi_req, &scsi_req_len, (raid_msgHdr_t **)&resp, &resp_len);
		
		/* Another SCSI command? */
		if (scsi_req) {
			raid_send_msg(scsi_req, scsi_req_len, "sdm", 0, 0);

			/* Free the old SCSI message context */
			VERIFY(dev->pending_req_context);
			free(dev->pending_req_context);
			dev->pending_req_context = NULL;

			/* and stash the new SCSI message */
			VERIFY(dev->pending_req_context == NULL);
			dev->pending_req_context = scsi_req;
			
			return;
		}
		
		VERIFY(resp);
		resp->header.mid = cmd_t->rsp_mid;
		resp->header.cookie = context->req_msg->cookie;
	}

	context->rsp_msg = (raid_msgHdr_t *)resp;
	context->rsp_msg_len = resp_len;
	old_pending_req_context = dev->pending_req_context;
	raid_sm_run(dev, NULL, RAID_EVENT_RESPONSE_REQ);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);

	/* It may be that raid_sm_run() might have sent another req. and
	 * thereby changed the pending req. context. If such is the case
	 * then we don't want to free the new pending req. context
	 */
	if(dev->pending_req_context == old_pending_req_context) {
		free(dev->pending_req_context);
		dev->pending_req_context = NULL;
	}
}

void 
raid_do_sdm_relay_done_failed(raid_dev_t *dev, void *cookie)
{
	void *			 scsi_resp;
	raid_cmd_transformer_t * cmd_t;
	raid_msg_context_t *	 context;
	raid_genericRspMsg_t 	 resp;

	RAID_DBG(2, ("raid_do_sdm_relay_done_failed: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SCSI relay response */
	VERIFY(cookie);
	scsi_resp = cookie;

	VERIFY(dev->message_req_context);
	context = dev->message_req_context;

	cmd_t = dev->dev_cmd_transformer;
        while (cmd_t->req_mid != context->req_msg->mid &&
               cmd_t->req_mid != 0)
                ++cmd_t;
        VERIFY(cmd_t->req_mid != 0);

	if (cmd_t->aborter)
		(*cmd_t->aborter)(dev, scsi_resp);

        bzero(&resp, sizeof(raid_genericRspMsg_t));
        resp.header.mid = cmd_t->rsp_mid;
		resp.header.cookie = context->req_msg->cookie;
        resp.rCode = RAID_RCODE_SDM_ERROR;
        raid_send_msg(&resp, sizeof(raid_genericRspMsg_t),
                      context->req_src_app,
                      context->req_src_cpu,
                      context->req_src_slot);

	/* Free the raid message context */
	raid_free_context(dev->message_req_context);
	dev->message_req_context = NULL;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;
}

void 
raid_do_send_message_req_reply(raid_dev_t *dev, void * cookie)
{
	raid_msg_context_t *	 context;
	raid_msgHdr_t*			msg;

	RAID_DBG(2, ("raid_do_send_message_req_reply: dev 0x%p, cookie 0x%p\n", dev, cookie));

	VERIFY(dev->message_req_context);
	context = dev->message_req_context;

	raid_send_msg(context->rsp_msg, context->rsp_msg_len,
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);

	/* if a lun create or delete req., then sdm needs to be informed about it on close */
	msg = (context->req_msg);
	if(msg->mid == RAID_MID_LUN_DELETE_REQ ||
	   msg->mid == RAID_MID_LUN_CREATE_REQ ||
	   msg->mid == RAID_MID_LUN_CREATE_SIMPLE_REQ ||
	   msg->mid == RAID_MID_CTRL_CLRCFG_REQ) {
			dev->rediscOnClose = 1;
	}

	raid_free_context(dev->message_req_context);
	dev->message_req_context = NULL;
}

void 
raid_do_message_req_reply_success_and_close(raid_dev_t *dev, void *cookie)
{
	RAID_DBG(2, ("raid_do_message_req_reply_success_and_close: dev 0x%p, cookie 0x%p\n", 
		     dev, cookie));

	/* 
	 * Cookie is the SCSI relay reply; pass it along.
	 * Send message reply 
	 */
	raid_do_send_message_req_reply(dev, cookie);

	/* Start close state machine */
	VERIFY(dev->close_req_context);
	cookie = dev->close_req_context;

	/* Do we *need* to SDM close on a close? */
	if (dev->sdm_close_on_close) 
		raid_do_sdm_close(dev, cookie);
	else {
		/* Skip SDM close -- fake out a successfull SDM close completion */
		raid_sm_run(dev, NULL, RAID_EVENT_SDM_CLOSE_DONE);
	}
}

void 
raid_do_message_req_reply_failed_and_close(raid_dev_t *dev, void *cookie)
{
	RAID_DBG(2, ("raid_do_message_req_reply_failed_and_close: dev 0x%p, cookie 0x%p\n", 
		     dev, cookie));

	/* 
	 * Cookie is the SCSI relay reply; pass it along.
	 * Send message reply 
	 */
	raid_do_message_req_reply_io_error(dev, cookie);

	/* Start close state machine */
	VERIFY(dev->close_req_context);
	cookie = dev->close_req_context;
	dev->close_req_context = NULL;

	/* Do we *need* to SDM close on a close? */
	if (dev->sdm_close_on_close) 
		raid_do_sdm_close(dev, cookie);
	else {
		/* Skip SDM close -- fake out a successfull SDM close completion */
		raid_sm_run(dev, NULL, RAID_EVENT_SDM_CLOSE_DONE);
	}
}

void
raid_do_short_close_reply_success(raid_dev_t *dev, void *cookie)
{
	raid_closeRspMsg_t	raid_close_rsp;

	RAID_DBG(2, ("raid_do_short_close_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	VERIFY(cookie);
	dev->close_req_context = (raid_msg_context_t *)cookie;
	dev->open_handle = -1;
	
	/* Reply to raid close message */
	raid_close_rsp.header.mid = RAID_MID_CLOSE_RSP;
	raid_close_rsp.header.cookie = dev->close_req_context->req_msg->cookie;
	raid_close_rsp.rCode = RAID_RCODE_SUCCESS;
	raid_send_msg(&raid_close_rsp, sizeof(raid_closeRspMsg_t), 
		      dev->close_req_context->req_src_app,
		      dev->close_req_context->req_src_cpu,
		      dev->close_req_context->req_src_slot);

	/* Free the raid close context */
	raid_free_context(dev->close_req_context);
	dev->close_req_context = NULL;
}

void 
raid_do_close_reply_success(raid_dev_t *dev, void *cookie) 
{
	raid_closeRspMsg_t	raid_close_rsp;
	
	RAID_DBG(2, ("raid_do_close_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SDM close response, or NULL if SDM close was skipped */
	if (cookie) {
		VERIFY(dev->pending_req_context);
		free(dev->pending_req_context);
		dev->pending_req_context = NULL;
	}
	dev->open_handle = -1;

	/* Reply to raid close message */
	VERIFY(dev->close_req_context);
	raid_close_rsp.header.mid = RAID_MID_CLOSE_RSP;
	raid_close_rsp.header.cookie = dev->close_req_context->req_msg->cookie;
	raid_close_rsp.rCode = RAID_RCODE_SUCCESS;
	raid_send_msg(&raid_close_rsp, sizeof(raid_closeRspMsg_t), 
		      dev->close_req_context->req_src_app,
		      dev->close_req_context->req_src_cpu,
		      dev->close_req_context->req_src_slot);

	/* Free the raid close context */
	VERIFY(dev->close_req_context);
	raid_free_context(dev->close_req_context);
	dev->close_req_context = NULL;

	/* ask sdm to do rediscovery if we had done some lun modifications previously */
	if(dev->rediscOnClose) {
		sdm_rediscReqMsg_t rediscMsg;
		int	msg_len = sizeof(rediscMsg);

		dev->rediscOnClose = 0;
		rediscMsg.hdr.mid = SDM_MID_REDISCOVER_REQ;
		rediscMsg.wwn = dev->wwn;
			
		raid_send_msg(&rediscMsg, msg_len, "sdm", 0, 0);
	}
}

void 
raid_do_sdm_close(raid_dev_t *dev, void *cookie)
{
	sdm_closeReqMsg_t *	sdm_close_req;

	RAID_DBG(2, ("raid_do_sdm_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	if (cookie) {
		VERIFY(dev->pending_req_context);
		free(dev->pending_req_context);
		dev->pending_req_context = NULL;
	}

	/* Send SDM close message */
	sdm_close_req = (sdm_closeReqMsg_t *)malloc(sizeof(sdm_closeReqMsg_t));

	sdm_close_req->hdr.mid = SDM_MID_DEV_CLOSE_REQ;
	sdm_close_req->hdr.chandle = (uint32)dev;
	sdm_close_req->handle = dev->open_handle;

	raid_send_msg(sdm_close_req, sizeof(sdm_closeReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_close_req;
}

void 
raid_sm_create(raid_dev_t *dev) 
{
	RAID_DBG(2, ("raid_sm_create: dev 0x%p\n", dev));

	dev->dev_sm = raid_dev_sm;
	dev->state = RAID_STATE_INITIAL;
}

void 
raid_sm_destroy(raid_dev_t *dev) 
{
	RAID_DBG(2, ("raid_sm_destroy: dev 0x%p\n", dev));

	VERIFY(dev->open_req_context == NULL);
	VERIFY(dev->close_req_context == NULL);
	VERIFY(dev->pending_req_context == NULL);
	VERIFY(dev->message_req_context == NULL);
	raid_device_destroy(dev);
}

void 
raid_sm_run(raid_dev_t *dev, void *cookie, raid_dev_event_t event) 
{
	raid_sm_t *	rs;

	RAID_DBG(2, ("raid_sm_run: dev 0x%p, cookie 0x%p, current_state %d, event %d\n", 
		     dev, cookie, dev->state, event));	

	rs = &dev->dev_sm[dev->state];
	VERIFY(rs->current_state == dev->state);
	VERIFY(rs->events[event].event == event);

	/* Move to next state */
	if (rs->events[event].next_state == RAID_STATE_INVALID)
		panic("Invalid transition in raid_sm_run: current_state %d, dev 0x%p, cookie 0x%p, event %d\n",
		      dev->state, dev, cookie, event);
	else
	if (rs->events[event].next_state == RAID_STATE_TERMINAL) {
		RAID_DBG(2, ("raid_sm_run: Reached terminal state: current_state %d, dev 0x%p, cookie 0x%p, event %d\n", 
			     dev->state, dev, cookie, event)); 
		/* Execute transition callout before destroying device */
		if (rs->events[event].callout)
			(*rs->events[event].callout)(dev, cookie);
		raid_sm_destroy(dev);
	}
	else {
		RAID_DBG(2, ("raid_sm_run: Next state %d.\n", rs->events[event].next_state));
		dev->state = rs->events[event].next_state;
		/* Execute transition callout function */
		if (rs->events[event].callout)
			(*rs->events[event].callout)(dev, cookie);
	}
}

raid_msg_context_t *    
raid_save_context(raid_msgHdr_t *msg, char *app, uint32 cpu, uint32 slot)
{
	raid_msg_context_t *context = NULL;
	uint32		    len;

	context = (raid_msg_context_t *)malloc(sizeof(raid_msg_context_t));
	bzero(context, sizeof(raid_msg_context_t));
	strcpy(context->req_src_app, app);
	context->req_src_cpu = cpu;
	context->req_src_slot = slot;

	switch(msg->mid) {
	case RAID_MID_OPEN_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_openReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_openReqMsg_t));
		break;

	case RAID_MID_CLOSE_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_closeReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_closeReqMsg_t));
		break;

	case RAID_MID_CTRL_INFO_REQ:
        case RAID_MID_CTRL_GETPARAM_REQ:
	case RAID_MID_CTRL_GETSTATE_REQ:
	case RAID_MID_PHYDEV_LIST_REQ:
	case RAID_MID_LOGDEV_LIST_REQ:
	case RAID_MID_CTRL_CLRCFG_REQ:
	case RAID_MID_CTRL_RESET_REQ:
	case RAID_MID_CTRL_FLUCACHE_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_msgHdr_t));
		bcopy(msg, context->req_msg, sizeof(raid_msgHdr_t));
		break;

	case RAID_MID_PHYDEV_INFO_REQ:		
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_phyDevInfoReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_phyDevInfoReqMsg_t));
		break;

	case RAID_MID_PHYDEV_STATS_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_phyDevStatsReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_phyDevStatsReqMsg_t));
		break;

	case RAID_MID_LOGDEV_INFO_REQ:		
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_logDevInfoReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_logDevInfoReqMsg_t));
		break;

	case RAID_MID_CTRL_CFGDATA_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_ctrlConfigReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_ctrlConfigReqMsg_t));
		break;
		
	case RAID_MID_LUN_CREATE_REQ:
		len = sizeof(raid_createLunReqMsg_t) + 
			((raid_createLunReqMsg_t *)msg)->lundef.numOfPhyDevices * sizeof(raid_phydev_id_t);
		context->req_msg = (raid_msgHdr_t *)malloc(len);
		bcopy(msg, context->req_msg, len);
		break;

	case RAID_MID_LUN_CREATE_SIMPLE_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_createSimpleLunReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_createSimpleLunReqMsg_t));
		break;

	case RAID_MID_LUN_DELETE_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_deleteLunReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_deleteLunReqMsg_t));
		break;

	case RAID_MID_LUN_INIT_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_initLunReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_initLunReqMsg_t));
		break;

	case RAID_MID_LUN_CHECK_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_checkLunReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_checkLunReqMsg_t));
		break;

	case RAID_MID_LUN_CACHE_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_cacheLunReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_cacheLunReqMsg_t));
		break;

	case RAID_MID_LUN_BPLIST_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_bpListReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_bpListReqMsg_t));
		break;

	case RAID_MID_LOGDEV_PHYLIST_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_logDevPhyListReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_logDevPhyListReqMsg_t));
		break;

	case RAID_MID_CTRL_SETPARAM_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_setCtrlParamReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_setCtrlParamReqMsg_t));
		break;

	case RAID_MID_PHY_PASSTHRU_REQ:
		len = sizeof(raid_phyPassThruReqMsg_t);
		if(((raid_phyPassThruReqMsg_t *)msg)->dataOut)
			len += ((raid_phyPassThruReqMsg_t *)msg)->dataLen - 1;
		context->req_msg = (raid_msgHdr_t *)malloc(len);
		bcopy(msg, context->req_msg, len);
		break;

	case RAID_MID_UPDATE_DRV_FW_REQ:
		context->req_msg = (raid_msgHdr_t *)malloc(sizeof(raid_updateDrvFwReqMsg_t));
		bcopy(msg, context->req_msg, sizeof(raid_updateDrvFwReqMsg_t));
		break;

	default:
		VERIFY(0);
		;
	}
	return(context);
}

void
raid_free_context(raid_msg_context_t *context)
{
	if (context->req_msg)
		free(context->req_msg);
	if (context->rsp_msg)
		free(context->rsp_msg);
	free(context);
}

void 
raid_close_all_devices(void)
{
	raid_dev_t *    dev;
	raid_dev_t *    next;

	if ((dev = raid_devs)) {
		do {
			next = dev->next;
			if (dev->state != RAID_STATE_CLOSED) {
				RAID_DBG(2, ("raid_close_all_devices: Closing dev 0x%p\n", dev));
				raid_do_sdm_close(dev, NULL);
			}
			else
				RAID_DBG(2, ("raid_close_all_devices: No need to close dev 0x%p\n", dev));
			dev = next;
		} while (dev != raid_devs);
	}
}
