/*  
 *=============================================================================
 * File Name   :    sb1250-dm.h
 *
 *  Description:    
 *
 *  Created    :    20 December 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 *	RCS        : $Id: sb1250-dm.h,v 1.6 2002/10/22 01:30:40 jong Exp $
 *=============================================================================
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/mem-api.h>
#include <exports/timer-api.h>
#include <exports/cm-api.h>
#include "sb1250-dm-regs.h"
#include <exports/hwMap.h>
#include "sb1250-dm-api.h"

#ifndef SB1250_DM_H_INCLUDED
#define SB1250_DM_H_INCLUDED

#define SB1250_DM_Q_LEN			0x4000

//--------------------------------------------------
// SB1250 Descriptor Format
//--------------------------------------------------
typedef struct _sb1250dm_desc {
	uint64		dw1;
	uint64		dw2;
} sb1250dm_desc_t;

//--------------------------------------------------
// Definitions for first double word of descriptor.
//--------------------------------------------------
#define SB1250DM_DESC_DST_ADDR		0x000000FFFFFFFFFFLL
#define SB1250DM_DESC_UNC_DST		0x0000010000000000LL
#define SB1250DM_DESC_UNC_SRC		0x0000020000000000LL
#define SB1250DM_DESC_INTERRUPT		0x0000040000000000LL
#define SB1250DM_DESC_THROTTLE		0x0000080000000000LL
#define SB1250DM_DESC_DIR_DST		0x0000300000000000LL
#define SB1250DM_DESC_DIR_SRC		0x0000C00000000000LL
#define SB1250DM_DESC_ZERO_MEM		0x0001000000000000LL
#define SB1250DM_DESC_PREFETCH		0x0002000000000000LL
#define SB1250DM_DESC_L2C_DST		0x0004000000000000LL
#define SB1250DM_DESC_L2C_SRC		0x0008000000000000LL

#ifdef PROM
#define SB1250DM_DESC_SET_DST_ADDR(_d_, _pbuf_)			\
{														\
	uint64 _buf_ = 0xe008000000;						\
    (_buf_) &= SB1250DM_DESC_DST_ADDR;                  \
    (_buf_) |= (SB1250DM_DESC_INTERRUPT|SB1250DM_DESC_UNC_DST);	            \
    ((_d_)->dw1) = (_buf_);								\
}	
#else
#define SB1250DM_DESC_SET_DST_ADDR(_d_, _pbuf_)			\
{														\
	uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);			    \
    (_buf_) &= SB1250DM_DESC_DST_ADDR;                  \
    (_buf_) |= (SB1250DM_DESC_INTERRUPT|SB1250DM_DESC_UNC_DST);	            \
    ((_d_)->dw1) = (_buf_);								\
}	
#endif

#define SB1250DM_DESC_SET_DST_ADDR_LOCAL(_d_, _pbuf_)			\
{														\
	uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);			    \
    (_buf_) &= SB1250DM_DESC_DST_ADDR;                  \
    (_buf_) |= (SB1250DM_DESC_INTERRUPT);               \
    ((_d_)->dw1) = (_buf_);								\
}	
// (_buf_) |= (SB1250DM_DESC_UNC_DST|SB1250DM_DESC_UNC_SRC); 

#define SB1250DM_DESC_GET_DST_ADDR(_d_, _addr_)			\
{                                                 	    \
    uint64 _pbuf_;                                      \
    _pbuf_ = (((_d_)->dw1) & SB1250DM_DESC_DST_ADDR);	\
    _addr_ = PHYS_TO_VIRT(_pbuf_);					    \
}

//--------------------------------------------------
// Definitions for second double word of descriptor.
//--------------------------------------------------
#define SB1250DM_DESC_SRC_ADDR		0x000000FFFFFFFFFFLL
#define SB1250DM_DESC_LEN			0x0FFFFF0000000000LL

#define SB1250DM_DESC_LEN_SHIFT		40

#define SB1250DM_DESC_SET_SRC_ADDR(_d_, _pbuf_)			\
{														\
    uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);				\
    (_buf_) &= SB1250DM_DESC_SRC_ADDR;                  \
    ((_d_)->dw2) = (_buf_);		                        \
}	


#define SB1250DM_DESC_GET_SRC_ADDR(_d_, _addr_)			\
{                                                 	    \
    uint64 _pbuf_;                                      \
    _pbuf_ = (((_d_)->dw2) & SB1250DM_DESC_SRC_ADDR);	\
    _addr_ = PHYS_TO_VIRT(_pbuf_);					    \
}

#define SB1250DM_DESC_SET_LEN(_d_, _sz_)				                 \
{														                 \
    uint64 _sz1_ = _sz_;												 \
    ((_d_)->dw2) &= ~SB1250DM_DESC_LEN;  				                 \
    ((_d_)->dw2) |= ((_sz1_)<<SB1250DM_DESC_LEN_SHIFT)&SB1250DM_DESC_LEN; \
}

#define SB1250DM_DESC_GET_LEN(_d_)	                                     \
  ((((_d_)->dw2) & SB1250DM_DESC_LEN) >> SB1250DM_DESC_LEN_SHIFT) 


typedef struct _sb1250dm_qCtrl {
	sb1250dm_desc_t	*qStart;
	sb1250dm_desc_t	*qEnd;
	sb1250dm_desc_t	*qtxCur;
	sb1250dm_desc_t	*qclnCur;
	uint32			qd;
	uint32			bufCnt;
	uint32			byteCnt;
	uint32			qFullCnt;
} sb1250dm_qCtrl_t;


typedef struct _sb1250dm_test_t {
	uint32			running;
	uint32			byteCnt;
	uint32			numLoopsToDo;
	uint32			numLoopsDone;
	uchar8			* srcAddr;
	uchar8			* destAddr;
	eeeRTC_t		totalTime;
	eeeRTC_t		startTime;
} sb1250dm_test_t;

typedef struct _sb1250dm_cfg {
	uint32				inUse;
	uint32				chnlId;			
	int                	(*callback)(char *, int, void *);
    void                ** cb_cookie;
	sb1250dm_regs_t	    regs;
	sb1250dm_qCtrl_t	dmqc;
	sb1250dm_test_t		test;
} sb1250dmCfg_t;

/*
 * External Variable Declarations
 */
extern sb1250dmCfg_t *sb1250dmCfg[SB1250DM_CHNL_CNT];

/*
 * Function Prototypes
 */
int    sb1250dm_writeReg(volatile uint64 *, uint64);
uint64 sb1250dm_readReg(volatile uint64 *);
void   sb1250dm_txCleanPoll(void *, uint32);
void   sb1250dm_rxAppPkt(eee_desc_t *);

#endif // SB1250_MOV_H_INCLUDED 
