/*
 *  Copyright (c) 2001
 *  Broadcom Corporation, Irvine, CA
 *
 *  BROADCOM PROPRIETARY AND CONFIDENTIAL
 *  
 *  This software is furnished under a Broadcom license agreement
 *  and may only be used in accordance with and subject to the terms 
 *  and conditions of such license. 
 *
 * Author: Mark Vandevoorde
 */


#ifndef SBPROF_H
#define SBPROF_H

// Code must be -D_PROFILING_ or most functions are replaced by empty macros.

#ifdef _PROFILING_

// adjust these declarations to get 64-, 32-, and 8-bit types
typedef unsigned long long u_int64_t;
typedef unsigned int u_int32_t;
typedef unsigned char u_int8_t;

#define MAX_SAMPLE_BYTES (24*1024*1024)
#define MAX_TBSAMPLE_BYTES (12*1024*1024)

/* Discard any buffered samples and start sampling specified events.
   Vectors level 5 interrupts to sbprof_cpu_intr() and enable level 5
   interrupts. */
void sbprof_start();

/* Stop sampling */
void sbprof_stop();

/***************************************************************************
 * Routines for gathering ZBbus profiles using trace buffer
 ***************************************************************************/

/* Requires: No subsequent use of SCD performance counters
	     or trace buffer.
   Effect:   Starts gathering random ZBbus profiles using trace buffer and
             SCD perf counter 1.
   Modifies: SCD perf counters.  I.e., counters 0, 2, & 3 may be written at
             any time by the zbbus profiling code.
 */
void sbprof_zbprof_start();

/* Effect: Stops collection of ZBbus profiles */
void sbprof_zbprof_stop();


#else /* _PROFILING_ */

#define sbprof_start(x)
#define sbprof_stop()
#define sbprof_zbprof_start()
#define sbprof_zbprof_stop()

#endif /* _PROFILING_ */

/***************************************************************************
 * Routines for using 40-bit SCD cycle counter
 *
 * Client responsible for either handling interrupts or making sure
 * the cycles counter never saturates, e.g., by doing
 * zclk_timer_init(0) at least every 2^40 - 1 ZCLKs.
 ***************************************************************************/

/* Configures SCD counter 0 to count ZCLKs starting from val;
   Configures SCD counters1,2,3 to count nothing.
   Initializes each counter to "val"
   Must not be called while gathering ZBbus profiles.

unsigned long long val; */
#define zclk_timer_init(val) \
  __asm__ __volatile__ (".set push;" \
                        ".set mips64;" \
                        "la   $8, 0xb00204c0;" /* SCD perf_cnt_cfg */ \
                        "sd   %0, 0x10($8);"   /* write val to counter0 */ \
                        "sd   %0, 0x18($8);"   /* write val to counter1 */ \
                        "sd   %0, 0x20($8);"   /* write val to counter2 */ \
                        "sd   %0, 0x28($8);"   /* write val to counter3 */ \
                        "sd   %1, 0($8);"      /* config counter0 for zclks*/ \
                        ".set pop" \
			: /* no outputs */ \
                                                     /* enable, counter0 */ \
			: /* inputs */ "r"(val), "r" ((1ULL << 33) | 0x32313001ULL) \
			: /* modifies */ "$8" )


/* Reads SCD counter 0 and puts result in value
   unsigned long long val; */
#define zclk_get(val) \
  __asm__ __volatile__ (".set push;" \
                        ".set mips64;" \
                        "la   $8, 0xb00204c0;" /* SCD perf_cnt_cfg */ \
                        "ld   %0, 0x10($8);"   /* write val to counter0 */ \
                        ".set pop" \
			: /* outputs */ "=r"(val) \
			: /* inputs */ \
			: /* modifies */ "$8" )

/* Returns the cpu clock speed in hertz */
unsigned long long cpu_hz();

#endif
