/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: ecc.c,v 1.4 2002/11/07 01:17:17 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/26/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <sys/mips.h>
#include <sys/sb1250/sb1250_regs.h>
#include <sys/sb1250/sb1250_int.h>
#include <sys/sb1250/sb1250_mc.h>
#include <exports/except-api.h>
#include "../sm-except/except.h"

extern uint64 dmfc0_cerr_i(void);
extern uint64 dmfc0_cerr_d(void);
extern uint64 dmfc0_berr_dpa(void);
extern uint64 dmfc0_cerr_dpa(void);
extern void   dmtc0_berr_dpa(uint64);
extern void   dmtc0_cerr_dpa(uint64);
extern void   dmtc0_cerr_i(uint64);
extern void   dmtc0_cerr_d(uint64);
extern void   signextend(address_t *);

uint32 singleBitErrCauseException = 0;
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/26/02
 *
 *-----------------------------------------------------------------
 */

void
ecc_set_test(uint32 mem_channel, int set, address_t addr)
{
    volatile uint64 * mctestecc_p;
    volatile uint32 * test_mem = (volatile uint32 *)addr;

    printf("ecc_set_test, set = 0x%x, addr = %p\n", set, (void *)addr);

	mctestecc_p = (volatile uint64 *)
            PHYS_TO_K1(A_MC_REGISTER((address_t)mem_channel, R_MC_TEST_ECC));

    printf("mctestecc_p = %p\n", (void *)mctestecc_p);
    signextend((address_t *)&mctestecc_p);
    signextend((address_t *)&test_mem);
    *mctestecc_p = (0xff & set);    /* 1=invert 0=none*/
    *test_mem = 0xdeadbeef;
    *mctestecc_p = 0;               /* 1=invert 0=none*/
}

void
ecc_set_test_data(uint32 mem_channel, uint64 set, address_t addr)
{
    volatile uint64 * mctestdata_p;
    volatile uint64 * test_mem = (volatile uint64 *)addr;

    printf("ecc_set_test_data, set = 0x%llx, addr = %p\n", set, (void *)addr);

	mctestdata_p = (volatile uint64 *)
            PHYS_TO_K1(A_MC_REGISTER((address_t)mem_channel, R_MC_TEST_DATA));
    printf("mctestdata_p = %p\n", (void *)mctestdata_p);
    signextend((address_t *)&mctestdata_p);
    signextend((address_t *)&test_mem);
    *mctestdata_p = set;            /* 1=invert 0=none*/
    *test_mem = 0xa5a5a5a5a5a5a5a5LL;
    *mctestdata_p = 0;              /* 1=invert 0=none*/
}

void
ecc_enable(uint32 mem_channel)
{
	volatile uint64	* mccfg_p;
    volatile uint64 scd_berr_st;

    /* clear any previous errors */
    scd_berr_st = *((volatile uint64 *)PHYS_TO_K1(A_SCD_BUS_ERR_STATUS));
    dmtc0_berr_dpa(0);
    dmtc0_cerr_dpa(0);

	mccfg_p = (volatile uint64 *)
            PHYS_TO_K1(A_MC_REGISTER((address_t)mem_channel, R_MC_CONFIG));
    signextend((address_t *)&mccfg_p);

    *mccfg_p &= ~M_MC_ECC_DISABLE;
}

void
ecc_disable(uint32  mem_channel)
{
	volatile uint64	* mccfg_p;

	mccfg_p = (volatile uint64 *)
            PHYS_TO_K1(A_MC_REGISTER((address_t)mem_channel, R_MC_CONFIG));
    signextend((address_t *)&mccfg_p);

    *mccfg_p |= M_MC_ECC_DISABLE;
}

void
ecc_view(uint32 cpu)
{
    uint32 sr;
    uint32 cause;

    volatile uint64   status_src;
    volatile uint64   berr_dpa;
    volatile uint64   cerr_dpa;
    volatile uint64   errctl;
    volatile uint64   cerr_d;
    volatile uint64    cerr_i;
    volatile uint64   * imsk_p;
    volatile uint64   * imap_p;
    volatile uint64   * istsrc_p;
    volatile uint64   * istmap_p;
    volatile uint64   scd_berr_st;
    volatile uint64   * l2_ecc_addr;
    volatile uint64   berr_l2_cnts;
    volatile uint64   berr_mie_cnts;
    volatile uint64   ** berr_data;

    sr = _mips_mfc0(C0_SR);
    cause = _mips_mfc0(C0_CAUSE);

    istsrc_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_SOURCE_STATUS));
    istmap_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_STATUS_BASE));
    imsk_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_MASK));
    imap_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_MAP_BASE));
    signextend((address_t *)&istsrc_p);
    signextend((address_t *)&istmap_p);
    signextend((address_t *)&imsk_p);
    signextend((address_t *)&imap_p);

    status_src = *istsrc_p;
    scd_berr_st = *((volatile uint64 *)PHYS_TO_K1(A_DBG_BUS_ERR_STATUS));

    /* get cache error info
     * ErrCtrl  = reg 26 , 0
     * CacheErr-I = reg 27 , 0
     * CacheErr-D = reg 27 , 1
     * CacheErr-DPA = reg 27 , 3
     */
    errctl = _mips_mfc0(C0_ECC);
    cerr_i   = dmfc0_cerr_i();
    cerr_d   = dmfc0_cerr_d();
    cerr_dpa = dmfc0_cerr_dpa();

    /* and few other err counters and info */
    l2_ecc_addr = *((volatile uint64 **)PHYS_TO_K1(A_L2_EEC_ADDRESS));
    berr_l2_cnts = *((volatile uint64 *)PHYS_TO_K1(A_BUS_L2_ERRORS));
    berr_mie_cnts = *((volatile uint64 *)PHYS_TO_K1(A_BUS_MEM_IO_ERRORS));
    berr_data = ((volatile uint64 **)PHYS_TO_K1(A_BUS_ERR_DATA_0));

    printf("SR = 0x%x CAUSE = 0x%x IE+IMASK=0x%x\n", sr, cause, SR_IE | SR_IMASK);
    printf("Interrupt Mask       = 0x%016llx\n", *imsk_p);
    printf("Interrupt Status Src = 0x%016llx\n", status_src);
    printf("Interrupt Status Map[0]=%llx [1]=%llx\n" 
           "                    [2]=%llx [3]=%llx\n" 
           "                    [4]=%llx [5]=%llx\n" 
           "                    [6]=%llx [7]=%llx\n",
            istmap_p[0], istmap_p[1],
            istmap_p[2], istmap_p[3],
            istmap_p[4], istmap_p[5],
            istmap_p[6], istmap_p[7]);
    printf("Intr BAD ECC Map = %llx\n", imap_p[K_INT_BAD_ECC]);
    printf("Intr IO BERR Map = %llx\n", imap_p[K_INT_IO_BUS]);
    printf("ECC Error Info\n");
    printf("    L2 EccAddr = %p\n", l2_ecc_addr);
    printf("    data   [0] = %p [1] = %p\n", berr_data[0], berr_data[1]);
    printf("           [2] = %p [3] = %p\n", berr_data[2], berr_data[3]);
    printf("    CERR CTL   = 0x%016llx\n", errctl);
    printf("    CERR_I     = 0x%016llx\n", cerr_i);
    printf("    CERR_D     = 0x%016llx\n", cerr_d);
    printf("    CERR_DPA   = 0x%016llx\n", cerr_dpa);
    printf("    BERR_DPA   = 0x%016llx\n", berr_dpa);
    printf("    L2_ErrCnt  = 0x%08llx UCTagE=%d CTagE=%d UCDataE=%d CDataE=%d\n",
            berr_l2_cnts,
            (int)((berr_l2_cnts>>24) & 0xff),
            (int)((berr_l2_cnts>>16) & 0xff),
            (int)((berr_l2_cnts>>8)  & 0xff),
            (int)((berr_l2_cnts>>0)  & 0xff));
    printf("    MemB_ErrCnt= 0x%08llx BusErr=%d UCMemE=%d CMemE=%d\n",
            berr_mie_cnts,
            (int)((berr_mie_cnts>>16) & 0xff),
            (int)((berr_mie_cnts>> 8) & 0xff),
            (int)((berr_mie_cnts>> 0) & 0xff));
    printf("    BErr_St    = %08llx %s dcode=%d rid=%d tid=%d\n",
            scd_berr_st,
            (scd_berr_st & 0x40000000)?"MultiErr":"",
            (int)((scd_berr_st>>22)&0x7),
            (int)((scd_berr_st>>18)&0xf),
            (int)((scd_berr_st>> 8)&0x3ff));
}
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	John Hahn
 *
 * Date Created:	8/23/02
 *
 *-----------------------------------------------------------------
 */
void
ecc_bus_error_process(
	uint32	    sr,
	uint32	    cause,
	address_t	epc,
	volatile uint64	* istsrc_p,
    volatile uint64   status_src
)
{
    volatile uint64   mask;
    volatile uint64   berr_dpa;
    volatile uint64   cerr_dpa;
    volatile uint64   errctl;
    volatile uint64   cerr_d;
    volatile uint64   cerr_i;
    volatile uint64   scd_berr_st;
    volatile uint64 * l2_ecc_addr;
    volatile uint64   berr_l2_cnts;
    volatile uint64   berr_mie_cnts;
    volatile uint64 ** berr_data;
	volatile uint64	* imsk_p;

    volatile uint64   io_intst;
    volatile uint64   io_errpa;

    imsk_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER(0, R_IMR_INTERRUPT_MASK));
    signextend((address_t *)&imsk_p);

    mask = *imsk_p;

    /*
     * read and clear the bus err status to clear the intr
     */
    scd_berr_st = (*(volatile uint64 *)PHYS_TO_K1(A_SCD_BUS_ERR_STATUS));

    /*
     * read and clear the io int err status
     */
    io_intst = (*(volatile uint64 *)PHYS_TO_K1(A_IO_INTERRUPT_STATUS));

    /* possible bus error address */
    io_errpa   = (0xffff & (*(volatile uint64 *)PHYS_TO_K1(A_IO_INTERRUPT_ADDR1)));
    io_errpa <<= 16;
    io_errpa  |= (0xffff & (*(volatile uint64 *)PHYS_TO_K1(A_IO_INTERRUPT_ADDR0)));

    /* and few other err counters and info */
    l2_ecc_addr = *((volatile uint64 **)PHYS_TO_K1(A_L2_EEC_ADDRESS));
    berr_l2_cnts = *((volatile uint64  *)PHYS_TO_K1(A_BUS_L2_ERRORS));
    berr_mie_cnts = *((volatile uint64  *)PHYS_TO_K1(A_BUS_MEM_IO_ERRORS));
    berr_data = ((volatile uint64 **)PHYS_TO_K1(A_BUS_ERR_DATA_0));

    /* get cache error info
     * ErrCtrl  = reg 26 , 0
     * CacheErr-I = reg 27 , 0
     * CacheErr-D = reg 27 , 1
     * CacheErr-DPA = reg 27 , 3
     */
    errctl = _mips_mfc0(C0_ECC);
    cerr_i   = dmfc0_cerr_i();
    cerr_d   = dmfc0_cerr_d();
    cerr_dpa = dmfc0_cerr_dpa();

    /* get Bus Error DPA CP0 reg 26 , 1 */
    berr_dpa = dmfc0_berr_dpa();

    /* After reading them .. clear them */
    dmtc0_berr_dpa(0);
    dmtc0_cerr_dpa(0);

    *imsk_p |= (M_INT_BAD_ECC | M_INT_IO_BUS);
    ecc_disable(0);
    ecc_disable(1);

    printf("\nError Exception");
    if (cause & CAUSE_IP2) printf(" BUS");
    if (cause & CAUSE_IP3) printf(" ECC");
    printf("\nInterrupt Mask       = 0x%016llx\n", mask);
    printf("Interrupt Status Src = 0x%016llx\n", status_src);
    printf ("CachErr:  %p\n", (void *)EXC_DBGREG[R_ERREPC]);
    printf ("BadVaddr: %p\n", (void *)EXC_DBGREG[R_BADVA]);
    printf("    L2 EccAddr = %p\n", l2_ecc_addr);
    printf("    data   [0] = %p [1] = %p\n", berr_data[0], berr_data[1]);
    printf("           [2] = %p [3] = %p\n", berr_data[2], berr_data[3]);
    printf("    CERR CTL   = 0x%016llx\n", errctl);
    printf("    CERR_I     = 0x%016llx\n", cerr_i);
    printf("    CERR_D     = 0x%016llx\n", cerr_d);
    printf("    CERR_DPA   = 0x%016llx\n", cerr_dpa);
    printf("    BERR_DPA   = 0x%016llx\n", berr_dpa);
    printf("    IO Err St  = 0x%08llx", io_intst);
    if (io_intst & 0x0200) printf(" parity");
    if (io_intst & 0x0400) printf(" timeo");
    if (io_intst & 0x0800) printf(" iladr");
    if (io_intst & 0x1000) printf(" mulcs");
    if (io_intst & 0x2000) printf(" cache");
    if (io_intst & 0x4000) printf(" coherr");
    printf(" cs=%d\n", ((uint32)io_intst & 0xff));
    printf("    IO Err addr= 0x%08llx\n", io_errpa);
    printf("    L2_ErrCnt  = 0x%08llx UCTagE=%d CTagE=%d UCDataE=%d CDataE=%d\n",
            berr_l2_cnts,
            (int)((berr_l2_cnts>>24) & 0xff),
            (int)((berr_l2_cnts>>16) & 0xff),
            (int)((berr_l2_cnts>>8)  & 0xff),
            (int)((berr_l2_cnts>>0)  & 0xff));
    printf("    MemB_ErrCnt= 0x%08llx BusErr=%d UCMemE=%d CMemE=%d\n",
            berr_mie_cnts,
            (int)((berr_mie_cnts>>16) & 0xff),
            (int)((berr_mie_cnts>> 8) & 0xff),
            (int)((berr_mie_cnts>> 0) & 0xff));
    printf("    BErr_St    = %08llx %s dcode=%d rid=%d tid=%d\n",
            scd_berr_st,
            (scd_berr_st & 0x40000000)?"MultiErr":"",
            (int)((scd_berr_st>>22)&0x7),
            (int)((scd_berr_st>>18)&0xf),
            (int)((scd_berr_st>> 8)&0x3ff));
}

int
ecc_bus_error_check(
    uint32      sr,
    uint32      cause,
    address_t   epc
)
{
	volatile uint64	* istsrc_p;
    volatile uint64   src_status;

    istsrc_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER(0, R_IMR_INTERRUPT_SOURCE_STATUS));
    signextend((address_t *)&istsrc_p);

    src_status = *istsrc_p;
    if ((cause & (CAUSE_IP3 | CAUSE_IP2)) ||
        (src_status & (M_INT_BAD_ECC | M_INT_IO_BUS)) ) {
        ecc_bus_error_process(sr, cause, epc, istsrc_p, *istsrc_p);
        return 1;
    }
    return 0;
}

void
ecc_error_exc(
	uint32	    sr,
	uint32	    cause,
	address_t	epc
)
{
	volatile uint64	* istsrc_p;

    istsrc_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER(0, R_IMR_INTERRUPT_SOURCE_STATUS));
    signextend((address_t *)&istsrc_p);

    ecc_bus_error_process(sr, cause, epc, istsrc_p, *istsrc_p);
}

void
ecc_timer_test(
	uint32	    sr,
	uint32	    cause,
	address_t	epc
)
{
	volatile uint64	* imsk_p;
    imsk_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER(0, R_IMR_INTERRUPT_MASK));
    signextend((address_t *)&imsk_p);
	*imsk_p |= M_INT_TIMER_0;
    printf("\nTimer0 Error Exception, SR = 0x%x, CAUSE = 0x%x EPC = %p\n\n",
           sr, cause, (void *)epc);
    ecc_view(0);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * disable ECC ints
 * disable ECC dram-config  & clear ECC errs & Bus watcher stats
 * enable  ECC dram-config
 *
 * clear isr related regsiters
 * map the ECC intr
 * register ECC isr
 * enable ECC ints
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/26/02
 *
 *-----------------------------------------------------------------
 */
void
ecc_exception_start(uint32 cpu)
{
    volatile uint64	* imsk_p;
    volatile uint64	* imap_p;
    volatile uint64 * scd_berr_st_p;
    volatile uint64 * berr_l2_cnts_p;
    volatile uint64 * berr_mie_cnts_p;

    uint32 sr;

    scd_berr_st_p = ((volatile uint64 *)PHYS_TO_K1(A_SCD_BUS_ERR_STATUS));
    berr_l2_cnts_p = ((volatile uint64 *)PHYS_TO_K1(A_BUS_L2_ERRORS));
    berr_mie_cnts_p = ((volatile uint64 *)PHYS_TO_K1(A_BUS_MEM_IO_ERRORS));

    signextend((address_t *)&scd_berr_st_p);
    signextend((address_t *)&berr_l2_cnts_p);
    signextend((address_t *)&berr_mie_cnts_p);

    imsk_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_MASK));
    imap_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_MAP_BASE));
    signextend((address_t *)&imsk_p);
    signextend((address_t *)&imap_p);

    *imsk_p |= (M_INT_COR_ECC | M_INT_BAD_ECC | M_INT_IO_BUS);      /* disable ecc intr */

    /* clear intr and counters */
    *scd_berr_st_p;
    *berr_l2_cnts_p = 0;
    *berr_mie_cnts_p = 0;

    printf("ECC exception handler Initialized\n");
    intr_registerHandler(K_INT_MAP_I1+2, ecc_error_exc);
    intr_registerHandler(K_INT_MAP_I0+2, ecc_error_exc);
    imap_p[K_INT_BAD_ECC] = K_INT_MAP_I1;        /* maps to IM3 */
    imap_p[K_INT_IO_BUS]  = K_INT_MAP_I0;        /* maps to IM2 */

    if( ! singleBitErrCauseException) {
        *imsk_p &= ~(M_INT_BAD_ECC | M_INT_IO_BUS);     /* enable ecc intr */
    }
    else {
        imap_p[K_INT_COR_ECC] = K_INT_MAP_I1;        /* maps COR_ECC & BAD_ECC to IM3  ??*/
        *imsk_p &= ~(M_INT_COR_ECC | M_INT_BAD_ECC | M_INT_IO_BUS);     /* enable ecc intr */
        printf(" ECC Exception on Single Bit Errors Enabled\n\n");
    }
    printf(" enabling SR_IE\n");
    sr = _mips_mfc0(C0_SR);
    sr |= SR_IE;
    _mips_mtc0(C0_SR,sr);
}

void
ecc_exception_stop(uint32 cpu)
{
    volatile uint64	* imsk_p;
    uint32 sr;

    imsk_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_MASK));
    signextend((address_t *)&imsk_p);
    if( ! singleBitErrCauseException) {
        *imsk_p |= (M_INT_BAD_ECC | M_INT_IO_BUS);     /* disable ecc interrupt */
    } else {
        *imsk_p |= (M_INT_COR_ECC | M_INT_BAD_ECC | M_INT_IO_BUS);     /* disable ecc interrupt */
    }

    ecc_disable(0);
    ecc_disable(1);
    printf("ECC exception stop\n");

    printf(" disabling SR_IE\n");
    sr = _mips_mfc0(C0_SR);
    sr &= ~SR_IE;
    _mips_mtc0(C0_SR,sr);
}

void
ecc_exception_disable_intr(uint32 cpu)
{
    volatile uint64	* imsk_p;

    imsk_p = (volatile uint64 *)
                    PHYS_TO_K1(A_IMR_REGISTER((address_t)cpu, R_IMR_INTERRUPT_MASK));
    signextend((address_t *)&imsk_p);
    if( ! singleBitErrCauseException) {
        *imsk_p |= (M_INT_BAD_ECC | M_INT_IO_BUS);     /* disable ecc interrupt */
    } else {
        *imsk_p |= (M_INT_COR_ECC | M_INT_BAD_ECC | M_INT_IO_BUS);     /* disable ecc interrupt */
    }
    printf("Disable ECC Interrupt exception\n");
}

void
ecc_exception_init(void)
{
    ecc_enable(0);
    ecc_enable(1);
    ecc_exception_start(0);
}
