/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-txrx.c
 *
 * RCS:  $Id: sbm-txrx.c,v 1.27 2002/11/07 01:16:54 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description:	"rx driver" for packets received by GMC
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifndef SBM_REV2
#error Compiled without SBM_REV2
#endif

#ifdef NFP_TXRX

#include <exports/eee-api.h>
#include <exports/sbm-api.h>
#include "sbm.h"
#include <exports/scsi-api.h>
#include <exports/malloc-api.h>
#ifndef LINUX_TEST
#include <exports/sb1250-dm-api.h>
#endif
#include <exports/dp-proxy.h>

uint32 sbm_evm_dest_port;
uint32 sbm_scsit_dest_port; 

extern int32 sbm_verifyPktInTXRXPool(eee_descPtr_t edesc);


int32 sbm_tx_to_luc_use_datamover = 0;

/*------------------------------------------------------------------*
 *
 * TX functions
 *
 *------------------------------------------------------------------*/
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_copySacDataIntoFlatBuf(
	eee_descPtr_t	edesc,
	int32			cod_len,
	int32			cod_offset,
	char			* bptr
)
{
	eee_descBuf_t	* bd;
	int32			len;
	uint32			num_bufs;

	while (1) {
		num_bufs = EEE_GET_NUM_BUFS(edesc);

		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);

		while ((num_bufs) && (cod_offset)) {
			if (cod_offset >= bd->len) {
				cod_offset -= bd->len;
				bd++;
				num_bufs--;
			}
			else
				break;
		}
		if (num_bufs == 0)
			edesc = edesc->hdr.next;
		else if (cod_offset < bd->len)
			break;
		else
			edesc = edesc->hdr.next;
	}

	while (cod_len) {
		if (cod_len + cod_offset > bd->len)
			len = bd->len - cod_offset;
		else
			len = cod_len;
		memcpy(bptr, bd->buf+cod_offset, len);
		sbm_stats.bytes_copied_tx += len;
		sbm_stats.bytes_copied_tx_total += len;
		cod_offset = 0;
		cod_len -= len;
		bptr += len;
		bd++;
		num_bufs--;
		if (cod_len) {
			if (num_bufs == 0) {
				edesc = edesc->hdr.next;
				num_bufs = EEE_GET_NUM_BUFS(edesc);

				if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
					bd = &(edesc->bd[0]);
				else
					bd = &(edesc->bd[1]);
			}
		}
	}
}

#ifndef LINUX_TEST
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/27/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_datamoverSacDataIntoFlatBuf(
	eee_descPtr_t	edesc,
	int32			cod_len,
	int32			cod_offset,
	char			* bptr,
    void            * cb_cookie
)
{
	eee_descBuf_t	* bd;
	int32			len;
	uint32			num_bufs;

	while (1) {
		num_bufs = EEE_GET_NUM_BUFS(edesc);

		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);

		while ((num_bufs) && (cod_offset)) {
			if (cod_offset >= bd->len) {
				cod_offset -= bd->len;
				bd++;
				num_bufs--;
			}
			else
				break;
		}
		if (num_bufs == 0)
			edesc = edesc->hdr.next;
		else if (cod_offset < bd->len)
			break;
		else
			edesc = edesc->hdr.next;
	}

	while (cod_len) {
		if (cod_len + cod_offset > bd->len)
			len = bd->len - cod_offset;
		else
			len = cod_len;
        if (len == cod_len) {
            sb1250dm_movData_buf(SB1250DM_CHNL_SAC_TO_LUC,
                                    len,
                                    bd->buf+cod_offset,
                                    bptr,
                                    cb_cookie);
        }
        else {
            sb1250dm_movData_buf(SB1250DM_CHNL_SAC_TO_LUC,
                                    len,
                                    bd->buf+cod_offset,
                                    bptr,
                                    (void *)0);
        }
		cod_offset = 0;
		cod_len -= len;
		bptr += len;
		bd++;
		num_bufs--;
		if (cod_len) {
			if (num_bufs == 0) {
				edesc = edesc->hdr.next;
				num_bufs = EEE_GET_NUM_BUFS(edesc);

				if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
					bd = &(edesc->bd[0]);
				else
					bd = &(edesc->bd[1]);
			}
		}
	}
}   /* sbm_datamoverSacDataIntoFlatBuf() */
#endif

/*-----------------------------------------------------------------
 * Name:	sbm_sendDataToLuc
 *
 * Description:	copies data into buffer provided by the LUC tx function.
 *				assumes the buffer is large enough to hold the length of
 *				data requested.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_sendDataToLuc(
	nfx_cod_t	cod,
	int32		cod_len,
	int32		cod_offset,
	int32		pad_size,
	char		* newbuf,
    void        * cb_cookie)
{
	eee_descPtr_t	edesc;
#ifdef USE_SAC_CHECKSUM
	uint32			calc;
#endif


	SBM_TX_DEBUG(("sbm_sendDataToLuc, 0x%llx, 0x%x, 0x%x, 0x%x, %p: ",
							cod, cod_len, cod_offset, pad_size, newbuf));
	if (sbm_verifyIsCODInSac(cod, 1)) {
		edesc = sbm_getDescFromCOD(cod);

		if ((cod_len+cod_offset-pad_size) > EEE_GET_PKT_LEN(edesc)) {
			printf("Invalid cod len, req = 0x%x, off = 0x%x, stored = 0x%x\n",
								cod_len, cod_offset, EEE_GET_PKT_LEN(edesc));
			return SBM_ERR_BAD_LEN;
		}

#ifdef USE_SAC_CHECKSUM
		if (edesc->hdr.attr != (calc = sbm_verifyCalcPktChecksum(edesc))) {
			SBM_TX_DEBUG(("chksum failed, stored = 0x%x, calc = 0x%x\n",
														edesc->hdr.attr, calc));
			sbm_stats.chksum_verify_error++;
			sbm_stats.chksum_verify_error_total++;
			return SBM_ERR_CHKSUM_FAILED;
		}
#endif
#ifndef LINUX_TEST
        if (sbm_tx_to_luc_use_datamover)
    		sbm_datamoverSacDataIntoFlatBuf(edesc,
                                            cod_len,
                                            cod_offset,
                                            newbuf,
                                            cb_cookie);
        else {
			extern int32 luc_xmtGmcDataReqCB(uchar8 * buf,
												uint32 len, void *pDesc);

    		sbm_copySacDataIntoFlatBuf(edesc, cod_len, cod_offset, newbuf);
            luc_xmtGmcDataReqCB(NULL, 0, cb_cookie);
        }
#else
    		sbm_copySacDataIntoFlatBuf(edesc, cod_len, cod_offset, newbuf);
#endif

		sbm_stats.cods_tx_luc++;
		sbm_stats.cods_tx_luc_total++;

		SBM_TX_DEBUG(("ok\n"));
	}
	else {
		SBM_TX_DEBUG(("cod not in SAC\n"));
		panic("sbm_sendDataToLuc, COD not in SAC, cod = 0x%llx\n", cod);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:	sbm_sac_tx
 *
 * Description:	Function that prepares the users data to be transmitted to
 *				either the SNIM or the requesting SRC (via the command message,
 *					CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM or
 *					CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SRC
 *				This is called by the sbm_app function which received the
 *				CMD_TO_MCPU message.  A flat buffer is being created, which
 *				will then be either:
 *					proxied to the FP memory before being sent on the
 *																data plane, or
 *					delivered to the requesting src app.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_sac_tx_snim(
	nfx_cod_t	cod,
	int32		cod_len,
	int32		cod_offset,
	char		* newbuf)
{
	eee_descPtr_t	edesc;
#ifdef USE_SAC_CHECKSUM
	uint32			calc;
#endif

	SBM_TX_DEBUG(("sbm_sac_tx_snim, 0x%llx, 0x%x, 0x%x, %p: ",
											cod, cod_len, cod_offset, newbuf));
	if (sbm_verifyIsCODInSac(cod, 1)) {
		edesc = sbm_getDescFromCOD(cod);

		if (cod_len > EEE_GET_PKT_LEN(edesc)) {
			printf("Invalid cod len, req = 0x%x, off = 0x%x, stored = 0x%x\n",
								cod_len, cod_offset, EEE_GET_PKT_LEN(edesc));
			return SBM_ERR_BAD_LEN;
		}

#ifdef USE_SAC_CHECKSUM
		if (edesc->hdr.attr != (calc = sbm_verifyCalcPktChecksum(edesc))) {
			SBM_TX_DEBUG(("chksum failed, stored = 0x%x, calc = 0x%x\n",
														edesc->hdr.attr, calc));
			sbm_stats.chksum_verify_error++;
			sbm_stats.chksum_verify_error_total++;
			return SBM_ERR_CHKSUM_FAILED;
		}
#endif

		sbm_copySacDataIntoFlatBuf(edesc, cod_len, cod_offset, newbuf);

		sbm_stats.cods_tx_snim++;
		sbm_stats.cods_tx_snim_total++;

		SBM_TX_DEBUG(("ok\n"));
	}
	else {
		SBM_TX_DEBUG(("cod not in SAC\n"));
		panic("sbm_sac_tx_snim, COD not in SAC, cod = 0x%llx\n", cod);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_sac_tx_src(
	nfx_cod_t	cod,
	int32		cod_len,
	int32		cod_offset,
	char		* newbuf)
{
	eee_descPtr_t	edesc;
#ifdef USE_SAC_CHECKSUM
	uint32			calc;
#endif

	SBM_TX_DEBUG(("sbm_sac_tx_src, 0x%llx, 0x%x, 0x%x, %p: ",
											cod, cod_len, cod_offset, newbuf));
	if (sbm_verifyIsCODInSac(cod, 1)) {
		edesc = sbm_getDescFromCOD(cod);

		if (cod_len > EEE_GET_PKT_LEN(edesc)) {
			printf("Invalid cod len, req = 0x%x, off = 0x%x, stored = 0x%x\n",
								cod_len, cod_offset, EEE_GET_PKT_LEN(edesc));
			return SBM_ERR_BAD_LEN;
		}

#ifdef USE_SAC_CHECKSUM
		if (edesc->hdr.attr != (calc = sbm_verifyCalcPktChecksum(edesc))) {
			SBM_TX_DEBUG(("chksum failed, stored = 0x%x, calc = 0x%x\n",
														edesc->hdr.attr, calc));
			sbm_stats.chksum_verify_error++;
			sbm_stats.chksum_verify_error_total++;
			return SBM_ERR_CHKSUM_FAILED;
		}
#endif

		sbm_copySacDataIntoFlatBuf(edesc, cod_len, cod_offset, newbuf);

		sbm_stats.cods_tx_src++;
		sbm_stats.cods_tx_src_total++;

		SBM_TX_DEBUG(("ok\n"));
	}
	else {
		SBM_TX_DEBUG(("cod not in SAC\n"));
		panic("sbm_sac_tx_src, COD not in SAC, cod = 0x%llx\n", cod);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/22/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_txCodToSnimWithDataMover(
	nfx_cod_t	cod,
	uint32		len,
	uint32		offset,
	uchar8		* msg,
	uint32		msglen,
	uint32		dp,
	uint32		sp
)
{
	eee_descPtr_t	edesc;

//	printf("sbm_txCodToSnimWithDataMover, 0x%016llx, 0x%x, 0x%x, %p, 0x%x\n",
//												cod, len, offset, msg, msglen);

	edesc = sbm_getDescFromCOD(cod);
	
	dp_proxy_tx_sac_data(edesc, len, offset, msg, msglen, dp, sp);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_txCodToSnimWithDataMoverCB(
	uchar8	* buf,
	uint32	len,
    void    * cb_cookie
)
{
	/*
	 * if this was the alloc'd space for the data proxy rx descriptor, then
	 * it must be dealloc'd.  We'll assume for now that the data size wasn't
	 * <= 32 or we would have a memory leak.  It should always be 8192.
	 */
	if (cb_cookie)
		eee_ramDealloc(buf);

    return NFX_OK;
}

/*------------------------------------------------------------------*
 *
 * RX functions
 *
 *------------------------------------------------------------------*/
/*-----------------------------------------------------------------
 * Name:	sbm_sendCODToSac
 *
 * Description:	function called by LUC rx routine to integrate the new cod
 *				into the SAC.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_sendCODToSac(
	nfx_cod_t	cod
)
{
	eee_descPtr_t	edesc;

	SBM_RX_DEBUG(("sbm_sendCODToSac, 0x%llx: ", cod));

	if (!sbm_verifyIsCODInSac(cod, 0)) {

		edesc = sbm_getDescFromCOD(cod);

		if (!sbm_verifyIsPktOk(edesc))
			panic("Error in pkt from luc, edesc = %p\n", edesc);

#ifdef USE_SAC_CHECKSUM
		edesc->hdr.attr = sbm_verifyCalcPktChecksum(edesc);
#endif
		sbm_verifyAssignPktAttribute(edesc, EEE_ATTR_SAC_COD_LUC);
		sbm_verifyAssignPktValidFreeAttr(edesc);

		sbm_stats.len_verify_skip++;
		sbm_stats.len_verify_skip_total++;

		sbm_stats.cods_rx_luc++;
		sbm_stats.cods_rx_luc_total++;
		sbm_stats.cods_rx++;
		sbm_stats.cods_rx_total++;
		sbm_stats.cods_present++;

		sbm_stats.num_bufs += sbm_getNumBufsInPkt(edesc);
		sbm_stats.tot_bytes += EEE_GET_PKT_LEN(edesc);

		SBM_RX_DEBUG(("ok\n"));
	}
	else {
		SBM_RX_DEBUG(("cod already in sac\n"));
		panic("cod, 0x%llx, already in SAC\n", cod);
	}
}

/*-----------------------------------------------------------------
 * Name:	sbm_sac_rx
 *
 * Description: registered receive application for user data sent to SAC from
 *				SNIM.
 *				It is assumed that the buffers and descriptors passed in here
 *				are from the TXRX pool.  If not, we'll panic.
 *				This function should strip the gmc_app_hdr and scsi_read_resp
 *				headers, 'store' the COD, and send the scsi_read_resp message
 *				to EVM.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_sac_rx(
	eee_descPtr_t	edesc
)
{

	sbm_rx_add_to_sac(edesc, sbm_evm_dest_port);
}

/*-----------------------------------------------------------------
 * Name:    sbm_sac_scsit_rx
 *
 * Description: registered receive application for user data sent to SAC from
 *              SNIM.
 *              It is assumed that the buffers and descriptors passed in here
 *              are from the TXRX pool.  If not, we'll panic.
 *              This function should strip the gmc_app_hdr and scsi_read_resp
 *              headers, 'store' the COD, and send the scsi_read_resp message
 *              to EVM.
 *
 * Created by:  Rick Lund
 *
 * Date Created:    8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_sac_scsit_rx(
    eee_descPtr_t   edesc
)
{
    sbm_rx_add_to_sac(edesc, sbm_scsit_dest_port);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_rx_copy_to_txrx(
	eee_descPtr_t	edesc,
	eee_descPtr_t	* new_edesc,
	uint32			nbufs
)
{
	uint32			i;
	uchar8			* new_buf[EEE_MAX_BUFS];
	eee_descBuf_t	* bd, * new_bd;

	if (edesc->hdr.next)
		panic("unsupported chain in copy_to_txrx\n");

	*new_edesc = eee_allocateDesc(EEE_TXRX_POOL, NFX_COS_HIGH);
	if (!*new_edesc)
		return NFX_ERR;

	for (i=0; i<nbufs; i++) {
		new_buf[i] = eee_allocateBuffer(EEE_TXRX_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
		if (!new_buf[i]) {
			sbm_stats.no_buf_error++;
			sbm_stats.no_buf_error_total++;
			return NFX_ERR;
		}
	}

	memcpy(*new_edesc, edesc, sizeof(eee_descHdr_t));
	sbm_stats.bytes_copied_rx += sizeof(eee_descHdr_t);
	sbm_stats.bytes_copied_rx_total += sizeof(eee_descHdr_t);

	if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) {
		bd = &(edesc->bd[0]);
		new_bd = &((*new_edesc)->bd[0]);
	}
	else {
		bd = &(edesc->bd[1]);
		new_bd = &((*new_edesc)->bd[1]);
	}

	for (i=0; i<nbufs; i++) {
		memcpy(new_buf[i], bd->buf, bd->len);
		sbm_stats.bytes_copied_rx += bd->len;
		sbm_stats.bytes_copied_rx_total += bd->len;
		new_bd->buf = new_buf[i];
		new_bd->len = bd->len;
		new_bd->control = bd->control;
		bd++;
		new_bd++;
	}

	eee_freePacket(edesc);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_rx_add_to_sac(
	eee_descPtr_t	edesc,
	uint32			msg_port
)
{
	uint32			hdr_len, cod_len, num_bufs, edesc_pkt_len, sp;
	eee_descBuf_t	* bd;
	nfx_cod_t		cod;
	scsi_readRspMsg_t	* scsi_msg;
	eee_descPtr_t	fp_desc;
	uchar8			* fp_buf;
	eee_descPtr_t	tmp;
	int32			toFreeCod = 0;

	SBM_RX_DEBUG(("sbm_sac_rx, edesc = %p: ", edesc));

	num_bufs = EEE_GET_NUM_BUFS(edesc);

	if (!sbm_verifyPktInTXRXPool(edesc)) {
		SBM_RX_DEBUG(("Pkt not in TXRX pool\n"));
		if (sbm_rx_copy_to_txrx(edesc, &tmp, num_bufs) != NFX_OK) {
			printf("Error from sbm_rx_copy_to_txrx\n");
			return;
		}
		edesc = tmp;
	}

//	if (num_bufs != 1) {
//		SBM_RX_DEBUG(("Pkt not in single buffer\n"));
//		panic("Pkt not in single buffer\n");
//	}

	if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
		bd = &(edesc->bd[0]);
	else
		bd = &(edesc->bd[1]);

	scsi_msg = (scsi_readRspMsg_t *)((uchar8 *)(bd->buf)+sizeof(gmc_app_hdr_t));

	hdr_len = sizeof(gmc_app_hdr_t) + sizeof(scsi_readRspMsg_t);
	edesc_pkt_len = EEE_GET_PKT_LEN(edesc);

	if (edesc_pkt_len < hdr_len) {
		SBM_RX_DEBUG(("bad len, edesc_pkt_len = %x, hdr_len = %x\n",
													edesc_pkt_len, hdr_len));
		printf("sbm_rx_add_to_sac: bad len, edesc_pkt_len = %x, hdr_len = %x\n",
													edesc_pkt_len, hdr_len);
		eee_freePacket(edesc);
		return;
	}
	cod_len = edesc_pkt_len - hdr_len;

	/*
	 * verify length and checksum with
	 * those set by SNIM in the gmc_app_hdr
	 */
	if (scsi_msg->dataLen) {
		if (scsi_msg->dataLen != cod_len) {
			SBM_RX_DEBUG(("len error, gmc->len = 0x%x, ed->len = 0x%x\n",
												scsi_msg->dataLen, cod_len));
			sbm_stats.len_verify_error++;
			sbm_stats.len_verify_error_total++;
			/*
			 * If data length received is not what's expected, replace dataLen
			 * is scsi msg (which will be delivered to EVM), and free COD.
			 */
			scsi_msg->dataLen = cod_len;
			toFreeCod = 1;
		}
		else {
			sbm_stats.len_verify_ok++;
			sbm_stats.len_verify_ok_total++;
		}
	}
	else {
		sbm_stats.len_verify_skip++;
		sbm_stats.len_verify_skip_total++;
	}

	EEE_SET_PKT_LEN(edesc, cod_len);
	bd->buf += hdr_len;
	bd->len -= hdr_len;

	SBM_RX_DEBUG(("len = 0x%x, ", cod_len));

	sp = edesc->hdr.src_port;
	cod = sbm_getCODFromDesc(edesc, EEE_GET_SLOT_NUM(sp), EEE_GET_DEST_CPU(sp));

	SBM_RX_DEBUG(("cod = 0x%llx, ", cod));

#ifdef USE_SAC_CHECKSUM
	if (scsi_msg->chksum) {
		uint32		chksum;
		chksum = sbm_verifyCalcPktChecksum(edesc);
		if (scsi_msg->chksum != chksum) {
			SBM_RX_DEBUG(("chksum error, gmc->chksum = 0x%x, calc = 0x%x\n",
												scsi_msg->chksum, chksum));
			sbm_stats.chksum_verify_error++;
			sbm_stats.chksum_verify_error_total++;
			eee_freePacket(edesc);
			return;
		}
		else {
			sbm_stats.chksum_verify_ok++;
			sbm_stats.chksum_verify_ok_total++;
			edesc->hdr.attr = chksum;
		}
	}
	else {
		sbm_stats.chksum_verify_skip++;
		sbm_stats.chksum_verify_skip_total++;
	}
#endif

	if (!toFreeCod) {
	/*
	 * COD verifies OK, set attributes
	 */
	sbm_verifyAssignPktAttribute(edesc, EEE_ATTR_SAC_COD_SNIM);
	sbm_verifyAssignPktValidFreeAttr(edesc);

	sbm_stats.cods_rx_snim++;
	sbm_stats.cods_rx_snim_total++;
	sbm_stats.cods_rx++;
	sbm_stats.cods_rx_total++;
	sbm_stats.cods_present++;

	sbm_stats.num_bufs += sbm_getNumBufsInPkt(edesc);
	sbm_stats.tot_bytes += EEE_GET_PKT_LEN(edesc);
	}

	/*
	 * Send SCSI Read Rsp message to FP
	 */
	scsi_msg->codHandle = cod;

	fp_buf = eee_allocateBuffer(EEE_FP_POOL, EEE_BUF_SMALL, NFX_COS_HIGH);
	fp_desc = eee_allocateDesc(EEE_FP_POOL, NFX_COS_HIGH);

	memcpy(fp_buf, scsi_msg, sizeof(scsi_readRspMsg_t));
	fp_desc->hdr.control = (1 << EDESC_BUFCNT_SHIFT) | sizeof(scsi_readRspMsg_t);
	fp_desc->hdr.offset = 0;
	fp_desc->hdr.dest_port = msg_port;
	fp_desc->hdr.src_port = edesc->hdr.src_port;
	fp_desc->hdr.attr = 0;
	fp_desc->hdr.next = 0;
	bd = &(fp_desc->bd[1]);
	bd->buf = fp_buf;
	bd->len = sizeof(scsi_readRspMsg_t);
	bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_SMALL;

	eee_forwardPacket(fp_desc);

	if (toFreeCod)
		eee_freePacket(edesc);

	SBM_RX_DEBUG(("ok\n"));

}	/* sbm_sac_rx */

#endif	/* NFP_TXRX */

