/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-verify.c
 *
 * RCS:  $Id: sbm-verify.c,v 1.13 2002/10/01 00:42:03 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:	Verification routines for:
 *					eee buffer and descriptor attributes,
 *					COD checksumming
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifndef SBM_REV2
#error Compiled without SBM_REV2
#endif

#include <exports/eee-api.h>
#include <exports/sbm-api.h>
#ifndef LINUX_TEST
#include <exports/pte-api.h>
#endif
#include "sbm.h"

#define SBM_ATTR_MIN	EEE_ATTR_SAC_COD_FREE_OK
#define SBM_ATTR_MAX	EEE_ATTR_SAC_COD_LUC

#ifdef NFP_TXRX

#ifdef EEE_ATTR
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_verifyAssignPktAttribute(
	eee_descPtr_t	edesc,
	int32			attr
)
{
	eee_attrAssignPkt(edesc, attr);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_verifyAssignPktValidFreeAttr(
	eee_descPtr_t	edesc
)
{
	eee_attrSetPktValidFreeAttr(edesc, EEE_ATTR_SAC_COD_FREE_OK, 0, 0, 0);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_verifyAssignDescAttribute(
	eee_descPtr_t	edesc,
	int32			attr
)
{
	eee_attrAssignDesc(edesc, attr);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_verifyAssignBufAttribute(
	uchar8			* buf,
	int32			attr
)
{
	eee_attrAssignBuf(buf, attr);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsEdescInSac(
	eee_descPtr_t	edesc,
    int32           pref
)
{
	byte	last_attr;

	last_attr = eee_attrGetDescLastAttr(edesc);

	if ((last_attr >= SBM_ATTR_MIN) && (last_attr <= SBM_ATTR_MAX))
		return 1;
	else
		return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsBufInSac(
	void	* buf,
    int32   pref
)
{
	byte	last_attr;

	last_attr = eee_attrGetBufLastAttr(buf);

	if ((last_attr >= SBM_ATTR_MIN) && (last_attr <= SBM_ATTR_MAX))
		return 1;
	else
		return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsPktInSac(
	eee_descPtr_t	edesc,
    int32           pref
)
{
	uint32			num_bufs, i;
	eee_descBuf_t	* bd;

	if (!sbm_verifyPktInTXRXPool(edesc))
		return 0;
			
	if (EEE_GET_PKT_LEN(edesc) == 0)		/* are we a middle, chained desc? */
		return 0;

	while (edesc) {

		num_bufs = EEE_GET_NUM_BUFS(edesc);

		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);

		for (i=0; i<num_bufs; i++) {
			if (!sbm_verifyIsBufInSac(bd->buf, pref))
				return 0;
			bd++;
		}

		if (!sbm_verifyIsEdescInSac(edesc, pref))
			return 0;

		edesc = edesc->hdr.next;
	}

	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsCODInSac(
	nfx_cod_t	cod,
    int32       pref
)
{
	eee_descPtr_t	edesc;

	edesc = sbm_getDescFromCOD(cod);

	return (sbm_verifyIsPktInSac(edesc, pref));

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsEdescOKToFree(
	eee_descPtr_t	edesc,
    int32           pref
)
{
	byte	last_attr;

	last_attr = eee_attrGetDescLastAttr(edesc);

	if (last_attr == EEE_ATTR_SAC_COD_FREE_OK)
		return 1;
	else
		return 0;

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsBufOKToFree(
	void	* buf,
    int32   pref
)
{
	byte	last_attr;

	last_attr = eee_attrGetDescLastAttr(buf);

	if (last_attr == EEE_ATTR_SAC_COD_FREE_OK)
		return 1;
	else
		return 0;

}
#endif	/* EEE_ATTR */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyPktInTXRXPool(
	eee_descPtr_t	edesc
)
{
	uint32			num_bufs, i;
	eee_descBuf_t	* bd;

	while (edesc) {
#ifndef LINUX_TEST
		if (((address_t)edesc < get_memory_virt_base(MMU_SM_TXRX_REGION)) ||
				((address_t)edesc >= get_memory_virt_end(MMU_SM_TXRX_REGION)))
			return 0;
#else
		if (((address_t)edesc < 0x30000000) || ((address_t)edesc >= 0x31000000))
			return 0;
#endif
		num_bufs = EEE_GET_NUM_BUFS(edesc);

		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);

#ifndef LINUX_TEST
		for (i=0; i<num_bufs; i++) {
			if (((address_t)(bd->buf) < get_memory_virt_base(MMU_SM_TXRX_REGION)) ||
				((address_t)(bd->buf) >= get_memory_virt_end(MMU_SM_TXRX_REGION)))
				return 0;
			bd++;
		}
#else
		for (i=0; i<num_bufs; i++) {
			if (((address_t)(bd->buf) < 0x30000000) ||
										((address_t)(bd->buf) >= 0x31000000))
				return 0;
			bd++;
		}
#endif

		edesc = edesc->hdr.next;
	}

	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsPktOk(
	eee_descPtr_t	edesc
)
{
	uint32	pkt_len, tot_len, i, nbufs;
	eee_descBuf_t	* bd;
    eee_descPtr_t   pkt_edesc;

    pkt_edesc = edesc;

	pkt_len = EEE_GET_PKT_LEN(edesc);
	tot_len = 0;

	while (edesc) {
		nbufs = EEE_GET_NUM_BUFS(edesc);
		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);

		for (i=0; i<nbufs; i++,bd++) {
			tot_len += bd->len;
			if (!(bd->control & EDESC_BD_CMD_IN_USE)) {
                sbm_testDisplayPkt(pkt_edesc);
				return 0;
            }
		}
		edesc = edesc->hdr.next;
	}

	if (tot_len != pkt_len) {
		printf("sbm_verifyIsEEEPktOk, len error, pkt_len = 0x%x, calc_len = 0x%x\n", pkt_len, tot_len);
        sbm_testDisplayPkt(pkt_edesc);
		return 0;
	}

	return 1;
}

#ifdef USE_SAC_CHECKSUM
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
uint32
sbm_verifyCalcPktChecksum(
	eee_descPtr_t	edesc
)
{
	uint32			num_bufs, i;
	eee_descBuf_t	* bd;
	uint32			chksum = 0;

	while (edesc) {

		num_bufs = EEE_GET_NUM_BUFS(edesc);

		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);

		for (i=0; i<num_bufs; i++) {
			chksum = uint32_checksum(bd->buf, bd->len, chksum);
			bd++;
		}

		edesc = edesc->hdr.next;
	}

	return chksum;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
uint32
sbm_verifyCalcFlatBufferChecksum(
	uchar8	* ptr,
	uint32	len
)
{
	return (uint32_checksum(ptr, len, 0));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32
sbm_verifyIsChecksumValid(
	eee_descPtr_t	edesc
)
{
	if (edesc->hdr.attr == sbm_verifyCalcPktChecksum(edesc))
		return 1;
	else
		return 0;
}
#endif	/* USE_SAC_CHECKSUM */

#endif	/* NFP_TXRX */

