/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm.h
 *
 * RCS:  $Id: sbm.h,v 1.15 2002/11/07 01:16:54 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description:	local include file for SAC Buffer Management
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _SBM_H
#define _SBM_H

#include <exports/eee-api.h>
#include <exports/elog-api.h>
#include "sbm-error.h"

// hardcoded applid for gmc 
// Todo: clean this up
#define SBM_TRACE(_par_) E_LOG(class_1, LOG_DEBUG, 4, 0, 0, 0, _par_)

#if 0
#define SBM_INIT_DEBUG(_x)		printf _x
#define SBM_RX_DEBUG(_x)		printf _x
#define SBM_TX_DEBUG(_x)		printf _x
#define SBM_VERIFY_DEBUG(_x)	printf _x
#define SBM_MGMT_DEBUG(_x)		printf _x
#else
#define SBM_INIT_DEBUG(_x)
#define SBM_RX_DEBUG(_x)
#define SBM_TX_DEBUG(_x)
#define SBM_VERIFY_DEBUG(_x)
#define SBM_MGMT_DEBUG(_x)
#endif

#ifndef LINUX_TEST
#ifdef NFP_TXRX
#define SBM_REV2_USE_DM
#endif
#endif

#ifdef NFP_TXRX
/*------------------------------------------------------------------*/
/*
 * Statistics and Debugging counters
 */
typedef struct sbm_stats {

	uint64		cods_present;

	uint64		tot_bytes;
	uint64		num_bufs;

	uint64		cods_rx;
	uint64		cods_created;
	uint64		cods_freed;
	uint64		cods_merged;
	uint64		cods_split;
	uint64		cods_concat;
	uint64		cods_rx_total;
	uint64		cods_created_total;
	uint64		cods_freed_total;
	uint64		cods_merged_total;
	uint64		cods_split_total;
	uint64		cods_concat_total;

	uint64		cods_rx_luc;
	uint64		cods_rx_snim;
	uint64		cods_rx_luc_total;
	uint64		cods_rx_snim_total;

	uint64		bytes_copied_mgmt;
	uint64		bytes_copied_mgmt_total;
	uint64		bytes_copied_tx;
	uint64		bytes_copied_tx_total;
	uint64		bytes_copied_rx;
	uint64		bytes_copied_rx_total;
	uint64		bytes_copied_collapse;
	uint64		bytes_copied_collapse_total;
	uint64		desc_collapse;
	uint64		desc_collapse_total;

	uint64		cods_tx_luc;
	uint64		cods_tx_snim;
	uint64		cods_tx_src;
	uint64		cods_tx_luc_total;
	uint64		cods_tx_snim_total;
	uint64		cods_tx_src_total;

	uint64		desc_avail;
	uint64		buf_small_avail;
	uint64		buf_large_avail;
	uint64		buf_ultra_avail;

	uint64		no_desc_error;
	uint64		no_buf_error;
	uint64		no_desc_error_total;
	uint64		no_buf_error_total;

	uint64		empty_rq_desc;
	uint64		empty_rq_buf_ultra;
	uint64		empty_rq_desc_total;
	uint64		empty_rq_buf_ultra_total;

	uint64		len_verify_ok;
	uint64		len_verify_skip;
	uint64		len_verify_error;
	uint64		len_verify_ok_total;
	uint64		len_verify_skip_total;
	uint64		len_verify_error_total;
#ifdef USE_SAC_CHECKSUM
	uint64		chksum_verify_ok;
	uint64		chksum_verify_skip;
	uint64		chksum_verify_error;
	uint64		chksum_verify_ok_total;
	uint64		chksum_verify_skip_total;
	uint64		chksum_verify_error_total;
#endif
} sbm_stats_t;

extern sbm_stats_t		sbm_stats;
#endif	/* NFP_TXRX */

/*------------------------------------------------------------------*/

extern int32 sbm_init();

extern void sbm_rx_add_to_sac(eee_descPtr_t edesc, uint32 msg_port);

extern void sbm_test_rx(eee_descPtr_t edesc);

extern void sbm_testDisplayPkt( eee_descPtr_t   edesc);

#ifdef NFP_TXRX
extern void sbm_sac_rx(eee_descPtr_t edesc);
extern void sbm_processReceivePackets(eee_descPtr_t edesc);
extern void sbm_processMcastReceivePackets(eee_descPtr_t edesc);

extern int32 sbm_sac_tx_snim(
    nfx_cod_t   cod,
    int32       cod_len,
    int32       cod_offset,
    char        * newbuf
);

extern int32 sbm_sac_tx_src(
    nfx_cod_t   cod,
    int32       cod_len,
    int32       cod_offset,
    char        * newbuf
);

#define SBM_UPDATE_NUM_BUFS		sbm_sac_info->num_buf_avail = \
												sbm_stats->buf_ultra_avail;

#ifdef USE_SAC_CHECKSUM
extern uint32 sbm_checksum_cod(nfx_cod_t COD);
extern uint32 sbm_checksum_msg(uchar8 * ptr, uint32 len);
#endif

extern eee_descPtr_t sbm_getDescFromCOD(nfx_cod_t cod);
extern nfx_cod_t sbm_getCODFromDesc(eee_descPtr_t	edesc,
									uint32			slot,
									uint32			cpu);

extern uint32 sbm_getNumBufsInPkt(eee_descPtr_t	edesc);
extern void sbm_mgmt_update_sac_info_struct(void * cb, uint32 tref);
extern int32 sbm_txCodToSnimWithDataMoverCB(
    uchar8  * buf,
    uint32  len,
    void    * cb_cookie
);

#endif	/* NFP_TXRX */

#ifdef LINUX_TEST
extern void sbm_testSaveToFile(uchar8 * buf, uint32 len, uchar8 * fname);
#else
extern int32 sbm_txCodToSnimWithDataMover(
    nfx_cod_t   cod,
    uint32      len,
    uint32      offset,
    uchar8      * msg,
    uint32      msglen,
    uint32      dp,
    uint32      sp
);

#endif

#ifdef NFP_TXRX
#ifdef EEE_ATTR
extern void sbm_verifyAssignPktAttribute( eee_descPtr_t edesc, int32 attr);
extern void sbm_verifyAssignPktValidFreeAttr( eee_descPtr_t   edesc);
extern void sbm_verifyAssignDescAttribute( eee_descPtr_t   edesc, int32 attr);
extern void sbm_verifyAssignBufAttribute( uchar8 * buf, int32 attr);

extern int32 sbm_verifyIsEdescInSac( eee_descPtr_t   edesc, int32 pref);
extern int32 sbm_verifyIsBufInSac( void    * buf, int32 pref);
extern int32 sbm_verifyIsPktInSac( eee_descPtr_t   edesc, int32 pref);
extern int32 sbm_verifyIsCODInSac( nfx_cod_t   cod, int32 pref);
extern int32 sbm_verifyIsEdescOKToFree( eee_descPtr_t   edesc, int32 pref);
extern int32 sbm_verifyIsBufOKToFree( void    * buf, int32 pref);
extern int32 sbm_verifyIsPktOk( eee_descPtr_t   edesc);
#else
#define sbm_verifyAssignPktAttribute( edesc, attr);
#define sbm_verifyAssignPktValidFreeAttr( edesc);
#define sbm_verifyAssignDescAttribute( edesc, attr);
#define sbm_verifyAssignBufAttribute( edesc, attr);
#define sbm_verifyIsEdescInSac(  edesc,  pref) (pref)
#define sbm_verifyIsBufInSac(  buf,  pref) (pref)
#define sbm_verifyIsPktInSac(  edesc,  pref) (pref)
#define sbm_verifyIsCODInSac(  cod,  pref) (pref)
#define sbm_verifyIsEdescOKToFree( edesc,  pref) (pref)
#define sbm_verifyIsBufOKToFree( buf,  pref) (pref)
extern int32 sbm_verifyIsPktOk( eee_descPtr_t   edesc);
#endif	/* EEE_ATTR */
#endif	/* NFP_TXRX */

#endif	/* _SBM_H */

