/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-app-rx.c
 *
 * RCS:  $Id: sbm-app-rx.c,v 1.26 2002/08/20 20:23:22 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	SAC Buffer Management application interface
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef SBM_REV2
#error Compiled with SBM_REV2
#endif

#ifdef NFP_TXRX

#include <exports/sbm-api.h>
#include <exports/cm-api.h>
#include <exports/debug.h>
#ifdef USE_SAC_CHECKSUM
#include <exports/scsi-api.h>
#endif
#include "sbm.h"

/*-----------------------------------------------------------------
 * Name:	sbm_processReceivePackets(edesc)
 *
 * Description:	Process messages sent to MCPU via EEE layer
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_processReceivePackets(
	eee_descPtr_t	edesc
)
{
	eee_descBufPtr_t	bd;
	uchar8				* msg;
	ushort16			len;
	uint32				cmd;
	uint32				control;
    uint32              dest_port;
	uint32				i;

	SBM_TRACE(("sbm_processReceivePackets, 0x%x\n", edesc));

	bd = &(edesc->bd[0]);
	if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
		bd++;

	cmd = *(uint32 *)(bd->buf);
	len = bd->len;

	switch (cmd) {
		case CMD_TO_MCPU_SEND_CONTROL_READ_REQ:
			{
				sbm_sendControlPkt_t	* msg;
				uchar8					* newbuf;
				int32					rc;

				SBM_TRACE(("SEND_CONTROL_READ_REQ\n"));

				msg = (sbm_sendControlPkt_t *)bd->buf;

#if 0
				rc = dp_proxy_tx_flat_buffer(&(msg->msg),
											msg->msg_len,
											msg->dest_port,
											msg->source_port);
#else
				newbuf = (uchar8 *)eee_ramAlloc(msg->msg_len);
				if (newbuf == NULL) {
					printf("could not allocate memory for READ_REQ message\n");
					return;
				}
				memcpy(newbuf, &(msg->msg), msg->msg_len);

				rc = dp_proxy_tx_flat_buffer(newbuf,
											msg->msg_len,
											msg->dest_port,
											msg->source_port);
#endif

				if (rc != NFX_OK)
					printf("dp_proxy_tx_flat_buffer error, 0x%x\n", rc);

				break;
			}

		case CMD_TO_MCPU_SEND_DATA_TO_LUC:
		case CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM:
		case CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SRC:
			{
				sbm_sendControlAndSACData_t	* msg;
				uint32						cod_len;
				uint32						bdl_index;
				uint32						first_bdl_offset;
				int32						num_tbd;
				uint32						tot_len;
				uchar8						* buf, * newbuf;
				uchar8						* sac_block;
				sbm_bdlEntry_t				* bdl;
				uint32						len, offset;
				int32						rc;
				uint32						sp, dp;

				SBM_TRACE(("SEND_DATA_TO_LUC\n"));

				msg = (sbm_sendControlAndSACData_t *)bd->buf;

				cod_len = msg->length;

				SBM_TRACE(("MCPU: send to luc cod 0x%llx len %d offset %d\n",
											msg->COD, cod_len, msg->offset));

				num_tbd = sbm_GetNumBD(	msg->COD,
										cod_len,
										msg->offset,
										&bdl_index,
										&first_bdl_offset);

				if (num_tbd < 0) {
					ASSERT(FALSE);
					break;
				}

				tot_len = msg->msg_len + msg->length;
				
				buf = newbuf = (uchar8 *)eee_ramAlloc(tot_len);
				if (buf == NULL) {
					printf("could not allocate memory for DATA_TO_LUC message\n");
					return;
				}
				memcpy(buf, &(msg->msg), msg->msg_len);
				buf += msg->msg_len;

				offset = first_bdl_offset;
				while (cod_len) {
					sac_block = &(sbm_sac[bdl_index*sbm.sac_block_size]);
					bdl = sbm_bdl + bdl_index;
					if (cod_len > bdl->len-offset)
						len = bdl->len-offset;
					else
						len = cod_len;
					memcpy(buf, sac_block+offset+bdl->offset, len);
					offset = 0;
					cod_len -= len;
					buf += len;
					bdl_index = bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
				}

				if (cmd == CMD_TO_MCPU_SEND_DATA_TO_LUC) {
					sp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
													nfxMyCPUId,
													0,
													EEE_USER_DATA,
													NFX_COS_HIGH,
													PORT_BEST_FIT);
					rc = eee_sendMessage(newbuf,
										tot_len,
										msg->dest_port,
										sp);
                    eee_ramDealloc(newbuf);
				}
				else if (cmd == CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM) {
#ifdef USE_SAC_CHECKSUM
					scsi_writeReqMsg_t	* wp;
					wp = (scsi_writeReqMsg_t *)newbuf;
					buf = newbuf + msg->msg_len;
					wp->checkSum = (uint64)sbm_checksum_msg(buf, cod_len);
#endif
					rc = dp_proxy_tx_flat_buffer(newbuf,
												tot_len,
												msg->dest_port,
												msg->source_port);
				}
				else if (cmd == CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SRC) {
					dp = edesc->hdr.src_port;
                    if (EEE_GET_SLOT_NUM(dp) == NFX_SLOTTYPE_FC) {
					    rc = dp_proxy_tx_flat_buffer(newbuf,
												    tot_len,
												    dp,
												    msg->source_port);
                    } else {
					    rc = eee_sendMessage(newbuf,
										    tot_len,
										    dp,
										    msg->source_port);
                        eee_ramDealloc(newbuf);
                    }
				}

				if (rc != NFX_OK)
					printf("DATA_TO_DISK error, sendMessage returned 0x%x\n", rc);

				break;
			}

#if 0
		case CMD_TO_MCPU_RETRIEVE_SAC_DATA:
			{
				sbm_retrieveSACData_t		* msg;
				nfx_cod_t					cod;
				uint32						length, offset;
				uint32						cl_cnt, bytes;
				int32						num_tbd;
				uchar8						* data_ptr;

				SBM_TRACE(("RETRIEVE_SAC_DATA\n"));

				msg = (sbm_retrieveSACData_t *)bd->buf;
				qe = sbm.metaq.head;
				req = (sbm_txqRetrieveSACData_t *)&(qe->retrieve_req);

				/*
				 * How many BD entires are needed for this COD/length/offset
				 */
				cod = msg->COD;
				length = msg->length;
				offset = msg->offset;
				num_tbd = sbm_GetNumBD(cod, length, offset);
				if (num_tbd < 0) {
					ASSERT(FALSE);
					break;
				}

				/*
				 * How many cache lines are needed for this request
				 */
				bytes = sizeof(sbm_txqRetrieveSACData_t);
				bytes += (num_tbd * 8);
				bytes += 31;
				bytes &= ~31;
				cl_cnt = bytes / 32;

				/*
				 * Is there room in the queue for the message?
				 */
				if (cl_cnt > sbm.metaq.num_avail_cl) {
					printf("No space on TxQ for command\n");
					sbm_stats->no_metaq_space++;
					break;
				}

				if (cl_cnt >= 0x100) {
					printf("Message too large, cl_cnt = 0x%x\n", cl_cnt);
					sbm_stats->msg_too_large++;
					break;
				}

				/*
				 * Write BD entries to TxQ
				 */
				data_ptr = (uchar8 *)(req+1);
				if (data_ptr == (uchar8 *)sbm.metaq.end)
					data_ptr = (uchar8 *)sbm.metaq.start;

				sbm_CopyBD(	data_ptr,
							cod,
							length,
							offset,
							sbm.metaq.end,
							sbm.metaq.start);

				req->PCI_mem_addr = msg->dest_addr;
				req->ACK_addr = 0;					/* FIX THIS */
				req->TID = msg->TID;
				req->control = TXQ_CMD_RETRIEVE_SAC_DATA | TXQ_GMC_OWN |
									(cl_cnt << TXQ_CACHELINE_SHIFT) | num_tbd;

				/*
				 * Increment control structure pointer
				 */
				sbm.metaq.num_avail_cl -= cl_cnt;

				while (cl_cnt--) {
					if (++qe >= sbm.metaq.end)
						qe = sbm.metaq.start;
				}
				sbm.metaq.head = qe;
				break;
			}
#endif

		case CMD_TO_MCPU_MANIPULATE_FREE:
			{
				sbm_freeCOD_t		* msg;
				nfx_cod_t			COD;

				SBM_TRACE(("MANIPULATE_FREE\n"));

				msg = (sbm_freeCOD_t *)bd->buf;
				COD = msg->COD;

				if (sbm_freeCOD(COD) != NFX_OK)
					printf("sbm_free: Invalid COD handle\n");

				break;
			}

		case CMD_TO_MCPU_MANIPULATE_MERGE:
			{
				sbm_mergeCOD_t		* msg;
				nfx_cod_t			src_cod, dest_cod;
				uint32				src_len, src_off, dest_off;
				uint32				TID;

				SBM_TRACE(("MANIPULATE_MERGE\n"));

				msg = (sbm_mergeCOD_t *)bd->buf;
				src_cod = msg->src_COD;
				src_len = msg->src_len;
				src_off = msg->src_offset;
				dest_cod = msg->dest_COD;
				dest_off = msg->dest_offset;
				TID = msg->TID;

				if (sbm_mergeCOD(	src_cod,
									src_len,
									src_off,
									dest_cod,
									dest_off,
									TID) != NFX_OK)
					printf("sbm_merge: Invalid COD handle\n");

				break;
			}

		case CMD_TO_MCPU_MANIPULATE_SPLIT:
			{
				sbm_splitCOD_t		* msg;
				nfx_cod_t			src_cod, dest_cod;
				uint32				src_len, src_off, dest_off;

				SBM_TRACE(("MANIPULATE_SPLIT\n"));

				msg = (sbm_splitCOD_t *)bd->buf;
				src_cod = msg->src_COD;
				src_off = msg->src_offset;
				dest_cod = msg->dest_COD;

				if (sbm_splitCOD(	src_cod,
									src_off,
									dest_cod) != NFX_OK)
					printf("Invalid split COD command\n");

				break;
			}

		case CMD_TO_MCPU_MANIPULATE_CONCAT:
			{
				sbm_concatCOD_t		* msg;
				nfx_cod_t			src_cod[10], dest_cod, *p;
				uint32				num_src_cod, i;

				SBM_TRACE(("MANIPULATE_CONCAT\n"));

				msg = (sbm_concatCOD_t *)bd->buf;

				num_src_cod = msg->num_src_CODs;
				dest_cod = msg->dest_COD;
				if (num_src_cod >= 10) {
					printf("Invalid number of CODs to concatenante\n");
					break;
				}
				p = &(msg->src_COD);
				for (i=0; i<num_src_cod; i++) {
					src_cod[i] = *p++;
				}

				if (sbm_concatCOD(	num_src_cod,
									dest_cod,
									src_cod) != NFX_OK)
					printf("Invalid concatenate COD command\n");

				break;
			}

		case CMD_TO_MCPU_MANIPULATE_COPY:
			{
				sbm_copyCOD_t		* msg;
				nfx_cod_t			src_cod, dest_cod;
				uint32				src_len, src_off, dest_off;

				SBM_TRACE(("MANIPULATE_COPY\n"));

				msg = (sbm_copyCOD_t *)bd->buf;
				src_cod = msg->src_COD;
				src_len = msg->src_len;
				src_off = msg->src_offset;
				dest_cod = msg->dest_COD;
				dest_off = msg->dest_offset;

				if (sbm_copyCOD(	src_cod,
									src_len,
									src_off,
									dest_cod,
									dest_off) != NFX_OK)
					printf("sbm_copy: Invalid COD handle\n");

				break;
			}

		case CMD_TO_MCPU_MANIPULATE_CREATE:
			{
				sbm_createCOD_t		* msg;
				nfx_cod_t			cod;
				uint32				len, fill;

				SBM_TRACE(("MANIPULATE_CREATE\n"));

				msg = (sbm_createCOD_t *)bd->buf;
				cod = msg->COD;
				len = msg->len;
				fill = msg->fill;

				if (sbm_createCOD(	cod,
									len,
									fill) != NFX_OK) {
					panic("Could not create COD 0x%llx\n", cod);
				}

				break;
			}

#if 0
		case CMD_TO_MCPU_MANIPULATE_XOR:
			{
				sbm_xorCOD_t			* msg;
				sbm_copyqQueueEntry_t	* qe;
				sbm_txqXorSACBlock_t	* xor_msg;

				msg = (sbm_xorCOD_t *)bd->buf;
				qe = sbm.copyq.head;
				xor_msg = (sbm_txqXorSACBlock_t *)&(qe->xorqe);

				if (xor_msg->control & TXQ_GMC_OWN) {
					panic("No space on TxQ for command\n");
					break;
				}

				control = TXQ_CMD_XOR_SAC_BLOCKS;

				xor_msg->num_bytes =	0;
				xor_msg->ACK_addr =	0;
				xor_msg->TID =	0;

				xor_msg->control = control | TXQ_GMC_OWN;
				if (++qe >= sbm.copyq.end)
					qe = sbm.copyq.start;
				sbm.copyq.head = qe;
				break;
			}
#endif

		case CMD_TO_MCPU_ADDRESS_OF_SAC_INFO_STRUCT:
			{
				sbm_infoAddrMsg_t	* msg;

				msg = (sbm_infoAddrMsg_t *)(bd->buf);

				sbm_sac_info = msg->addr;
//				printf("sbm_sac_info structure at %p\n", sbm_sac_info);

				sbm_sac_info->total_memory_size = 0;
				sbm_sac_info->total_sac_blocks = sbm.num_sac_blocks;
				sbm_sac_info->num_sac_blocks_avail = sbm_stats->num_sac_blocks_avail;

				break;
			}

		default:
			panic("Invalid Message to MCPU, edesc at 0x%x, cmd = 0x%x\n",
															edesc, cmd);
	}

	eee_freePacket(edesc);

}	/* sbm_processReceivePackets() */


/*-----------------------------------------------------------------
 * Name:	sbm_processMcastReceivePackets(edesc)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_processMcastReceivePackets(
	eee_descPtr_t	edesc
)
{
	SBM_TRACE(("sbm_processMcastReceivePackets, 0x%x\n", edesc));

}	/* sbm_processMcastReceivePackets() */


/*-----------------------------------------------------------------
 * Name:		sbm_sendDataToLuc
 *
 * Description: Copy user data specified by COD, length & offset 
 *				to buffer.
 *
 * Created by:	
 *
 * Date Created:	
 *
 *-----------------------------------------------------------------
 */
void
sbm_sendDataToLuc(nfx_cod_t 	cod, 
				  int32 		cod_len, 
				  int32 		cod_offset, 
				  char 			*newbuf)
{

uint32                      bdl_index;
uint32                      first_bdl_offset;
int32                       num_tbd;
uchar8                      * buf ;
uchar8                      * sac_block;
sbm_bdlEntry_t              * bdl;
uint32                      len, offset;



//  SBM_TRACE(("MCPU: send to luc cod 0x%llx len %d offset %d\n",
//  			msg->COD, cod_len, msg->offset));

    num_tbd = sbm_GetNumBD( cod,
                            cod_len,
                            cod_offset,
                            &bdl_index,
                            &first_bdl_offset);

    if (num_tbd < 0) {
        ASSERT(FALSE);
    }

	buf  = newbuf;

    offset = first_bdl_offset;
    while (cod_len) {
        sac_block = &(sbm_sac[bdl_index*sbm.sac_block_size]);
        bdl = sbm_bdl + bdl_index;
        if (cod_len > bdl->len-offset)
            len = bdl->len-offset;
        else
            len = cod_len;
        memcpy(buf, sac_block+offset+bdl->offset, len);
        offset = 0;
        cod_len -= len;
        buf += len;
        bdl_index = bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
    }


	return;
}

#endif	/* NFP_TXRX */

