/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-debug.c
 *
 * RCS:  $Id: sbm-debug.c,v 1.20 2002/08/20 20:23:22 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Contains routines for displaying and debugging the
 *				SAC buffer management structures.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/23/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef SBM_REV2
#error Compiled with SBM_REV2
#endif

#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/malloc-api.h>
#include <exports/cod-api.h>
#include <exports/sbm-api.h>
#include "sbm.h"

#ifdef LINUX_TEST
#include <fcntl.h>
#endif


#ifdef NFP_TXRX

#include <exports/hash-api.h>
#include "sbm.h"
#include "../sm-hash/hash.h"

/*-----------------------------------------------------------------
 * Name:	sbm_debugHashTableDisplay()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/23/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_debugHashTableDisplay()
{
	ht_t			* ptr;
	uint32			i, j;
	ht_item_t		* np, * t;
	sbm_hash_node_t * p;

	ptr = (ht_t *)sbm.sbm_hash;

	printf(" index            COD handle         COD handle         COD handle\n");
	printf("                   HMT index          HMT index          HMT index\n");

	printf("-------   ------------------  ------------------  ------------------\n");

	for (i=0; i<ptr->num_buckets; i++) {
		printf("%7x:  ", i);
		for (np = ptr->head[i]; np != NULL; np=t) {
			for (j = 0, t = np; ((t != NULL) && (j < 3)); t = t->next, j++) {
				p = t->data;
				printf("0x%016llx  ", p->COD);
			}
			printf("\n");
			printf("          ");
			for (j = 0, t = np; ((t != NULL) && (j < 3)); t = t->next, j++) {
				p = t->data;
				printf("        0x%08x  ", p->hmt_index);
			}
			if (t != NULL)
				printf("\n          ");
		}
		printf("\n\n");
	}
	printf("-------   ------------------  ------------------  ------------------\n");

}	/* sbm_debugHashTableDisplay() */

/*-----------------------------------------------------------------
 * Name:	sbm_debugHMTDisplay()
 *
 * Description:	Displays Handle Mapping Table structure
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/23/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_debugHMTDisplay(uint32 index)
{
	sbm_hmtEntry_t	* hmt = sbm_hmt;
	uint32			i;

#ifdef USE_SAC_CHECKSUM
	printf("  HMT     BDL index    COD length    checksum\n");
	printf("-------   ----------   ----------   ----------\n");
#else
	printf("  HMT     BDL index    COD length\n");
	printf("-------   ----------   ----------\n");
#endif
	if (index == 0xFFFFFFFF) {
		for (i = 0; i < sbm.num_sac_blocks; i++) {
#ifdef USE_SAC_CHECKSUM
			printf("%7x:  0x%08x   0x%08x   0x%08x\n",
							i, hmt[i].bdl_index, hmt[i].COD_len, hmt[i].chksum);
#else
			printf("%7x:  0x%08x   0x%08x\n",
								i, hmt[i].bdl_index, hmt[i].COD_len);
#endif
		}
	}
	else {
		if (index < sbm.num_sac_blocks) {
#ifdef USE_SAC_CHECKSUM
			printf("%7x:  0x%08x   0x%08x   0x%08x\n",
			index, hmt[index].bdl_index, hmt[index].COD_len, hmt[index].chksum);
#else
			printf("%7x:  0x%08x   0x%08x\n",
							index, hmt[index].bdl_index, hmt[index].COD_len);
#endif
			
		}
		else
			printf("Invalid index, 0x%x\n", index);
	}
#ifdef USE_SAC_CHECKSUM
	printf("-------   ----------   ----------   ----------\n");
#else
	printf("-------   ----------   ----------\n");
#endif

}	/* sbm_debugHMTDisplay() */

/*-----------------------------------------------------------------
 * Name:	sbm_debugBDLDisplay()
 *
 * Description:	Displays the BDL Table
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/23/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_debugBDLDisplay(uint32 index)
{
	sbm_bdlEntry_t	* bdl = sbm_bdl;
	uint32			i;

	printf("  BDL     length  offset   next_index\n");
	printf("-------   ------  ------   ----------\n");
	if (index == 0xFFFFFFFF) {
		for (i = 0; i < sbm.num_sac_blocks; i++) {
			printf("%7x:  0x%04x  0x%04x   0x%08x\n",
						i, bdl[i].len, bdl[i].offset, bdl[i].next_bdl_index);
		}
	}
	else {
		if (index < sbm.num_sac_blocks) {
			printf("%7x:  0x%04x  0x%04x   0x%08x\n",
									index, bdl[index].len, bdl[index].offset,
													bdl[index].next_bdl_index);
		}
		else
			printf("Invalid index, 0x%x\n", index);
	}
	printf("-------   ------  ------   ----------\n");

}	/* sbm_debugBDLDisplay() */

/*-----------------------------------------------------------------
 * Name:	sbm_debugIFQDisplay()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void sbm_debugIFQDisplay()
{
	uint32	* p;

	printf("SBM IFQ\n");
	printf("start = %p, end = %p, head = %p\n",
									sbm_ifq->start, sbm_ifq->end, sbm_ifq->head);
	for (p = (uint32 *)sbm_ifq->start; p < (uint32 *)sbm_ifq->end; p++) {
        if ((uint32)p & 0x1F)
            printf("             0x%08x", *p);
        else
            printf("%p : 0x%08x", p, *p);
		if (p == sbm_ifq->head)
			printf(" <-- head\n");
		else
			printf("\n");
	}

}	/* sbm_debugIFQDisplay() */

/*-----------------------------------------------------------------
 * Name:	sbm_debugFBQDisplay()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void sbm_debugFBQDisplay()
{
	uint32	* p;

	printf("SBM FBQ\n");
	printf("start = %p, end = %p, head = %p\n",
									sbm_fbq->start, sbm_fbq->end, sbm_fbq->head);
	for (p = (uint32 *)sbm_fbq->start; p < (uint32 *)sbm_fbq->end; p++) {
        if ((uint32)p & 0x1F)
            printf("             0x%08x", *p);
        else
            printf("%p : 0x%08x", p, *p);
		if (p == sbm_fbq->head)
			printf(" <-- head\n");
		else
			printf("\n");
	}

}	/* sbm_debugFBQDisplay() */

/*-----------------------------------------------------------------
 * Name:	sbm_debugCODDisplay()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/28/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_debugCODDisplay(
	nfx_cod_t	cod
)
{
	uint32			hmt_index, bdl_index;
	sbm_hmtEntry_t	* hmt;
	sbm_bdlEntry_t	* bdl;
	sbm_hash_node_t	* rptr;

	rptr = (sbm_hash_node_t *)sbm_doesCODExist(cod);
	if (!rptr) {
		printf("Invalid COD, 0x%llx\n", cod);
		return;
	}

	hmt_index = rptr->hmt_index;
	hmt = sbm_hmt + hmt_index;

	printf("COD 0x%llx : ", cod);
#ifdef USE_SAC_CHECKSUM
	printf("hmt index  : %d, COD_len = %d, chksum = 0x%x\n", hmt_index, hmt->COD_len, hmt->chksum);
#else
	printf("hmt index  : %d, COD_len = %d\n", hmt_index, hmt->COD_len);
#endif
	bdl_index = hmt->bdl_index & ~HMT_ENTRY_IN_USE;
	while (1) {
		bdl = sbm_bdl + bdl_index;
		printf("   bdl %d : len = 0x%x, offset = 0x%x\n", bdl_index, 
													bdl->len, bdl->offset);
		bdl_index = bdl->next_bdl_index;

		if (bdl_index == BDL_EOC)
			break;
		else
			bdl_index &= ~BDL_ENTRY_IN_USE;
	}

}	/* sbm_debugCODDisplay() */

/*-----------------------------------------------------------------
 * Name:	sbm_debugSummaryDisplay()
 *
 * Description:
 *
 * Created by:	Jan Vanhoutte
 *
 * Date Created:	1/4/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_debugSummaryDisplay(
	void
)
{
	uint32			hmt_index, bdl_index, index, inuse, bdl_eoc, bdl_free;
	uint32	        * p;
	sbm_ifqEntry_t	* ifqptr;
	sbm_fbqEntry_t	* fbqptr;
	sbm_hmtEntry_t	* hmt = sbm.hmt;
	sbm_bdlEntry_t	* bdl = sbm.bdl;
	sbm_hash_node_t	* rptr;

	// printf ("num_sac_blocks      %d\n", sbm.num_sac_blocks);

	inuse = 0;
	for (index = 0; index < sbm.num_sac_blocks; index++) {
		if (hmt[index].bdl_index & HMT_ENTRY_IN_USE)
			inuse++;
	}
	printf("HMT:  inuse   %8d\n", inuse);
	printf("      free    %8d\n", sbm_stats->num_free_hmt_indicies);
	printf("      total   %8d\n", sbm.num_sac_blocks);   
	printf("COD:  present %8d\n", sbm_stats->num_cods_in_sac);
	printf("      rcvd    %8d\n", sbm_stats->num_cods_received);
	printf("      created %8d\n", sbm_stats->num_cods_created);
	printf("      copied  %8d\n", sbm_stats->num_cods_copied);
	printf("      freed   %8d\n", sbm_stats->num_cods_freed);

	inuse = bdl_eoc = bdl_free = 0;
	for (index = 0; index < sbm.num_sac_blocks; index++) {
		if (bdl[index].next_bdl_index & BDL_ENTRY_IN_USE)
			inuse++;
		if (bdl[index].next_bdl_index == BDL_EOC)
			bdl_eoc++;
		if (bdl[index].next_bdl_index == BDL_ENTRY_FREE)
			bdl_free++;
	}
	printf("BDL:  inuse   %8d\n", inuse);
	printf("      eoc     %8d\n", bdl_eoc);
	printf("      free    %8d\n", bdl_free);
	printf("      total   %8d\n", sbm.num_sac_blocks);
	printf("SAC:  inuse   %8d\n", sbm_stats->num_sac_blocks_in_use);
	printf("      avail   %8d\n", sbm_stats->num_sac_blocks_avail);

	printf("\nno free IFQ entry  %8d\n", sbm_stats->no_free_ifq);
	printf("no free FBQ entry  %8d\n", sbm_stats->no_free_fbq);

	printf("\nlen verify ok      %8d\n", sbm_stats->len_verify_ok);
	printf("len verify error   %8d\n", sbm_stats->len_verify_error);
	printf("len verify skip    %8d\n", sbm_stats->len_verify_skip);
#ifdef USE_SAC_CHECKSUM
	printf("crc verify ok      %8d\n", sbm_stats->chksum_verify_ok);
	printf("crc verify error   %8d\n", sbm_stats->chksum_verify_error);
	printf("crc verify skip    %8d\n", sbm_stats->chksum_verify_skip);
#endif

}	/* sbm_debugSummaryDisplay() */


#endif	/* NFP_TXRX */

#if 0
/*-----------------------------------------------------------------
 * Name:	sbm_test_rcv()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/10/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_test_rcv(
	eee_descPtr_t	edesc
)
{
	eee_descBuf_t	* bptr;
	uint32			* iptr;

	printf("sbm_test_rcv, edesc = %p\n", edesc);

	bptr = &(edesc->bd[0]);
	if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
		bptr++;

	iptr = (uint32 *)(bptr->buf);
	printf("cmd = 0x%x\n", *iptr++);
	printf("dp  = 0x%x\n", *iptr++);
	printf("sp  = 0x%x\n", *iptr++);
	printf("TID = 0x%x\n", *iptr++);
	printf("len = 0x%x\n", *iptr++);

	printf("message = %s\n", iptr);

}	/* sbm_test_rcv() */
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
sacify_ack_rcv(
	eee_descPtr_t	edesc
)
{
	sbm_ack_msg_t	* msg;

	msg = (sbm_ack_msg_t *)edesc->bd[1].buf;

	switch (msg->mid) {
		case SBM_MSG_COD_RCV:
			if (msg->err)
				printf("ERROR IN \n");
			printf("SAC STORED COD 0x%llx, len = %d\n",
									msg->msg.cod_rcv.cod, msg->msg.cod_rcv.len);
			break;

		case SBM_MSG_MERGE_ACK:
			if (msg->err)
				printf("ERROR IN \n");
			printf("Merge ACK, TID = 0x%x\n", msg->msg.mgmt_ack.TID);
			break;

		case SBM_MSG_SPLIT_ACK:
			if (msg->err)
				printf("ERROR IN \n");
			printf("Split ACK, TID = 0x%x\n", msg->msg.mgmt_ack.TID);
			break;

		case SBM_MSG_CONCAT_ACK:
			if (msg->err)
				printf("ERROR IN \n");
			printf("Concat ACK, TID = 0x%x\n", msg->msg.mgmt_ack.TID);
			break;

		case SBM_MSG_XOR_ACK:
			if (msg->err)
				printf("ERROR IN \n");
			printf("XOR ACK, TID = 0x%x\n", msg->msg.mgmt_ack.TID);
			break;
		default:
	}

	eee_freePacket(edesc);

}	/* sacify_ack_rcv() */

#ifdef LINUX_TEST
/*-----------------------------------------------------------------
 * Name:	sbm_debugSacify()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/30/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_debugSacify(
	uchar8	* fname,
	int32	gran
)
{
	uchar8	* buf;
	uint32	dp, sp;
	int32	len;
	int32	sacify_fd;
	uint32	sac_app_id, ack_app_id;
	enet_hdr_t		* enet_hdr;
	agile_hdr_t		* agile_hdr;
	gmc_app_hdr_t	* gmc_app_hdr;
	uint32			hdr_offset;
	nfx_cod_t		cod;
	int32			rc;

#ifdef FIX_HDR_STUFF
	hdr_offset = sizeof(gmc_app_hdr_t);
#else
	hdr_offset = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) +
														sizeof(gmc_app_hdr_t);
#endif
	if ((buf = eee_ramAlloc(gran + hdr_offset)) == 0) {
		printf("sacify, unable to malloc buffer, %d\n", gran);
		return;
	}

	if ((sacify_fd = open(fname, O_RDONLY)) < 0) {
		printf("Error opening file %s\n", fname);
		return;
	}

	sac_app_id = eee_getApplicationId("sac");
	ack_app_id = eee_getApplicationId("sacify_ack");

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							sac_app_id,
							EEE_USER_DATA,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							ack_app_id,
							EEE_USER_DATA,
							NFX_COS_HIGH,
							0);

	while ((len = read(sacify_fd, buf+hdr_offset, gran)) > 0) {
#ifdef FIX_HDR_STUFF
		gmc_app_hdr = (gmc_app_hdr_t *)buf;
#else
		enet_hdr = (enet_hdr_t *)buf;
		agile_hdr = (agile_hdr_t *)(buf + sizeof(enet_hdr_t));
		gmc_app_hdr = (gmc_app_hdr_t *)
							(buf + sizeof(enet_hdr_t) + sizeof(agile_hdr_t));
#endif
		cod = COD_GET_NEXT();
printf("debugSacify, cod = 0x%llx\n", cod);
		gmc_app_hdr->control = ack_app_id;
		gmc_app_hdr->id.cod = cod;
#ifdef USE_SAC_CHECKSUM
		gmc_app_hdr->cod_len = len;
		gmc_app_hdr->chksum = (uint64)sbm_checksum_msg(buf+hdr_offset, len);
printf("checksum = 0x%llx\n", gmc_app_hdr->chksum);
#endif

		printf("buf = %p, ehdr = %p, agile_hdr = %p, gmc_hdr = %p\n",
								buf, enet_hdr, agile_hdr, gmc_app_hdr);
printf("sending message, length = 0x%x\n", len+hdr_offset);
		rc = eee_sendMessage(buf, len+hdr_offset, dp, sp);
		if (rc != NFX_OK)
			printf("sbm_debugSacify error, rc = 0x%x\n", rc);
	}

	eee_ramDealloc(buf);

	close(sacify_fd);

}	/* sbm_debugSacify() */
#endif	/* LINUX_TEST */

/*-----------------------------------------------------------------
 * Name:	gmc_test_rcv()
 *
 * Description:	registered application for testing, which will get
 *				delivered the data plane packets coming from the MCPU/GMC
 *				TxQ's and copyQ's.
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
gmc_test_rcv(
	eee_descPtr_t	edesc
)
{
	uint32			buf_cnt, tot_len, i, bd_len, data_len, hdr_off;
	eee_descBuf_t	* bd;
	uchar8			* bd_buf;
	uchar8			* file_data, * dptr;
	int32			fd, n;
	uchar8			fname[80];
	gmc_app_hdr_t	* gmc_hdr;

#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
	hdr_off = 0;
#else
	hdr_off = sizeof(gmc_app_hdr_t);
#endif
#else
	hdr_off = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + sizeof(gmc_app_hdr_t);
#endif

	if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
		bd = &(edesc->bd[0]);
	else
		bd = &(edesc->bd[1]);

	tot_len = edesc->hdr.control & EDESC_TOTAL_LEN;
	data_len = tot_len - hdr_off;

	file_data = (uchar8 *)eee_ramAlloc(data_len);
	dptr = file_data;

printf("gmc_test_rcv, malloc buffer at %p\n", file_data);
	buf_cnt = (edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;

	for (i=0; i<buf_cnt; i++, bd++) {
		bd_len = bd->len;
		bd_buf = bd->buf;
#ifndef NO_GMC_APP_NO_TX
		if (i==0) {
#ifdef FIX_HDR_STUFF
			gmc_hdr = (gmc_app_hdr_t *)bd_buf;
#else
			gmc_hdr = (gmc_app_hdr_t *)(bd_buf +
									sizeof(enet_hdr_t) + sizeof(agile_hdr_t));
#endif
			bd_buf += hdr_off;
			bd_len -= hdr_off;
		}
#endif
		memcpy(dptr, bd_buf, bd_len);
		dptr += bd_len;
	}

#ifdef NO_GMC_APP_NO_TX
	printf("first control word = 0x%x\n", *(uint32 *)file_data);

	memcpy(fname, file_data, 4);
	fname[4] = '\0';
#else
	printf("TID = 0x%x\n", gmc_hdr->id.tid);

	memcpy(fname, &(gmc_hdr->id.tid), 4);
	fname[4] = '\0';
#endif

	eee_freePacket(edesc);

	printf("gmc_test_rcv : got data, desc = %p, saving to file %s\n",
																edesc, fname);

#ifdef NO_GMC_APP_NO_TX
	if (*(uchar8 *)file_data == 'a') {
#else
	if (gmc_hdr->id.tid == 0x65656565) {
#endif
#ifdef LINUX_TEST
		/*
		 * send sac data
		 */
		if ((fd = open(fname, O_WRONLY | O_CREAT, 0666)) < 0) {
			printf("Unable to open file %s for writing\n", fname);
			return;
		}

		n = write(fd, file_data, data_len);
		if (n != data_len)
			printf("error writing file %s\n", fname);

		close(fd);
#endif
	}
	else {
		/*
		 * read request
		 */
		printf("Received Read Request message : ");
		for (i=0, dptr = file_data; i<data_len; i++)
			printf("%c", *dptr++);
		printf("\n\n");
	}

	eee_ramDealloc(file_data);

}	/* gmc_test_rcv() */
