/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-mgmt.c
 *
 * RCS:  $Id: sbm-mgmt.c,v 1.25 2002/08/20 20:23:22 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	COD Manipulation commands
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef SBM_REV2
#error Compiled with SBM_REV2
#endif

#ifdef NFP_TXRX

#include <exports/sbm-api.h>
#include <exports/debug.h>
#include "sbm.h"

/*-----------------------------------------------------------------
 * Name:	sbm_freeCOD(nfx_cod_t COD)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */

int32
sbm_freeCOD(
	nfx_cod_t	COD
)
{	
	sbm_hash_node_t		data;
	sbm_hash_node_t		* rptr;
	sbm_hmtEntry_t		* hmt;
	sbm_bdlEntry_t		* bdl;
	uint32				hmt_index;
	uint32				bdl_index, next_bdl_index;
	uint32				COD_low;	/* for hashing */
	uint32				index;

	SBM_TRACE(("sbm_freeCOD: 0x%llx\n", COD));

	data.COD = COD;

	COD_low = COD;
	if ((rptr = (sbm_hash_node_t *)
					hash_delete(sbm.sbm_hash, COD_low, &data)) != NULL) {
		hmt_index = rptr->hmt_index;
		sbm_hashNodeStackPush(rptr);

		hmt = sbm.hmt + hmt_index;

		bdl_index = hmt->bdl_index;
		if (!(bdl_index & HMT_ENTRY_IN_USE))
			panic("Error, HMT index not in use, 0x%x\n", hmt);

		index = bdl_index & ~HMT_ENTRY_IN_USE;

		do {
			bdl = (sbm.bdl + index);
			next_bdl_index = bdl->next_bdl_index;
			if (!(next_bdl_index & BDL_ENTRY_IN_USE))
				panic("Error, BDL index 0x%x not in use\n", next_bdl_index);
			sbm_freeBDLIndex(index);
			bdl_index = next_bdl_index;
			index = bdl_index & ~BDL_ENTRY_IN_USE;
		} while (bdl_index != BDL_EOC);

		sbm_freeHMTIndex(hmt_index);

		SBM_TRACE(("freed COD\n"));
	}
	else {
		printf("Error, COD (0x%lx) not in tables\n", COD);
		return SBM_ERR_INVALID_COD;
	}

	sbm_stats->num_cods_freed++;
	sbm_stats->num_cods_in_sac--;

	SBM_UPDATE_NUM_BLOCKS;

	return NFX_OK;

}	/* sbm_freeCOD() */

/*-----------------------------------------------------------------
 * Name:	sbm_mergeCOD()
 *
 * Description:	Merge a src COD into a dest COD
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/25/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_mergeCOD(
	nfx_cod_t	src_cod,
	uint32		src_len,
	uint32		src_off,
	nfx_cod_t	dest_cod,
	uint32		dest_off,
	uint32		TID
)
{
	sbm_hash_node_t			* rptr;
	uint32					cl_cnt;
	uint32					src_hmt_index, dest_hmt_index;
	sbm_hmtEntry_t			* src_hmt, * dest_hmt;
	uint32					start_src_index, start_dest_index;
	uint32					last_src_index, last_dest_index;
	sbm_bdlEntry_t			* start_src_bdl, * start_dest_bdl;
	sbm_bdlEntry_t			* last_src_bdl, * last_dest_bdl;
	uint32					src_off_left, dest_off_left;
	uint32					src_len_left, overlap_left;
	uint32					num_src_bdl;
	uint32					to_free_bdl[32];
	uint32					to_free_hmt;
	uint32					to_free_bdl_num = 0;
	uint32					dont_free = 0;
	uint32					i;
	sbm_hash_node_t			data;
	uint32					COD_low;

	SBM_TRACE(("sbm_mergeCOD, src_cod = 0x%llx, src_len = %d, src_off = %d\n",
												src_cod, src_len, src_off));
	SBM_TRACE(("              dest_cod = 0x%llx, dest_off = %d\n",
														dest_cod, dest_off));
/*
printf("src COD: ");
sbm_debugCODDisplay(src_cod);
printf("dest COD: ");
sbm_debugCODDisplay(dest_cod);
*/

	/*
	 * Verify src and dest CODs exist
	 */
	if (!(rptr = (sbm_hash_node_t *)sbm_doesCODExist(src_cod))) {
		printf("cod, 0x%llx, does not exist in hash table\n", src_cod);
		ASSERT(FALSE);
		return -1;
	}
	src_hmt_index = rptr->hmt_index;
	src_hmt = sbm.hmt + src_hmt_index;

#ifdef USE_SAC_CHECKSUM
	{
		uint64	chksum;
		if ((chksum = (uint64)sbm_checksum_cod(src_cod)) != src_hmt->chksum)
			panic("mergeCOD chksum error, src_cod = 0x%llx, hmt->chksum = 0x%x, calc = 0x%x\n", src_cod, src_hmt->chksum, chksum);
	}
#endif

	if (!(rptr = (sbm_hash_node_t *)sbm_doesCODExist(dest_cod))) {
		printf("cod, 0x%llx, does not exist in hash table\n", dest_cod);
		ASSERT(FALSE);
		return -1;
	}
	dest_hmt_index = rptr->hmt_index;
	dest_hmt = sbm.hmt + dest_hmt_index;

#ifdef USE_SAC_CHECKSUM
	{
		uint64	chksum;
		if ((chksum = (uint64)sbm_checksum_cod(dest_cod)) != dest_hmt->chksum)
			panic("mergeCOD chksum error, dest_cod = 0x%llx, hmt->chksum = 0x%x, calc = 0x%x\n", dest_cod, dest_hmt->chksum, chksum);
	}
#endif

	/*
	 * Verify src length and offset
	 */
	if ((src_len+src_off) > src_hmt->COD_len) {
		printf("len + off, %d, > cod len, %d\n",
											src_len+src_off, src_hmt->COD_len);
		ASSERT(FALSE);
		return -1;
	}

	if (dest_off > dest_hmt->COD_len) {
		printf("dest offset, %d, > dest_len, %d\n",
												dest_off, dest_hmt->COD_len);
		ASSERT(FALSE);
		return -1;
	}

	/*
	 * Calculate the following:
	 *		start_src_index	: first BDL that contains bytes to merge
	 *		start_dest_index: first BDL of the merged bytes destination
	 *							(or last BDL in list, if dest_off == COD_len)
	 *		last_src_index	: last BDL that contains bytes to merge
	 *		last_dest_index	: last BDL of the merged bytes destination
	 *							(or last BDL in list)
	 *		src_off_left	: number of data bytes offset into the
	 *							start_src_index
	 *		dest_off_left	: number of data bytes offset into the
	 *							start_dest_index
	 *		src_len_left	: number of data bytes left in the last_src_index
	 *							that need to be merged into destination
	 *		overlap_left	: number of data bytes in the last_dest_index
	 *							that should be overwritten with merged data.
	 */

	/*
	 * Find start bdl's for src and dest offsets
	 */
	src_off_left = src_off;
	start_src_index = src_hmt->bdl_index & ~HMT_ENTRY_IN_USE;
	while (1) {
		start_src_bdl = sbm.bdl + start_src_index;
		if (src_off_left >= start_src_bdl->len) {
			to_free_bdl[to_free_bdl_num++] = start_src_index;
			src_off_left -= start_src_bdl->len;
			start_src_index = start_src_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
		}
		else
			break;
	}

	dest_off_left = dest_off;
	start_dest_index = dest_hmt->bdl_index;
	start_dest_bdl = sbm.bdl + (start_dest_index & ~HMT_ENTRY_IN_USE);
	while (start_dest_bdl->next_bdl_index != BDL_EOC) {
		if (dest_off_left >= start_dest_bdl->len) {
			dest_off_left -= start_dest_bdl->len;
			start_dest_index = start_dest_bdl->next_bdl_index;
			start_dest_bdl = sbm.bdl +
										(start_dest_index & ~BDL_ENTRY_IN_USE);
		}
		else
			break;
	}
	start_dest_index &= ~BDL_ENTRY_IN_USE;

	/*
	 * Find last bdl's depending on src_len
	 */
	num_src_bdl = 1;
	if (src_len + src_off_left <= start_src_bdl->len) {
		last_src_bdl = start_src_bdl;
		last_src_index = start_src_index;
		src_len_left = src_len;
	}
	else {
		src_len_left = src_len - (start_src_bdl->len - src_off_left);
		last_src_index = start_src_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
		while (1) {
			num_src_bdl++;
			last_src_bdl = sbm.bdl + last_src_index;
			if (src_len_left > last_src_bdl->len) {
				src_len_left -= last_src_bdl->len;
				last_src_index = last_src_bdl->next_bdl_index &
															~BDL_ENTRY_IN_USE;
			}
			else
				break;
		}
	}

	if (src_len + dest_off_left <= start_dest_bdl->len) {
		last_dest_bdl = start_dest_bdl;
		last_dest_index = start_dest_index;
		overlap_left = src_len;
	}
	else {
		overlap_left = src_len + dest_off_left;
		last_dest_index = start_dest_index;
		last_dest_bdl = sbm.bdl + (last_dest_index & ~BDL_ENTRY_IN_USE);
		while (last_dest_bdl->next_bdl_index != BDL_EOC) {
			if (overlap_left > last_dest_bdl->len) {
				overlap_left -= last_dest_bdl->len;
				last_dest_index = last_dest_bdl->next_bdl_index;
				last_dest_bdl = sbm.bdl +
										(last_dest_index & ~BDL_ENTRY_IN_USE);
			}
			else
				break;
		}
	}
	last_dest_index &= ~BDL_ENTRY_IN_USE;

	/*
	 * Get rest of src BDL indicies to free
	 */
	i = last_src_bdl->next_bdl_index;
	while (i != BDL_EOC) {
		uint32	index;
		sbm_bdlEntry_t	* bdl;

		index = i & ~BDL_ENTRY_IN_USE;
		to_free_bdl[to_free_bdl_num++] = index;
		bdl = sbm.bdl + index;
		i = bdl->next_bdl_index;
	}

	/*
	printf("start_src_index %d start_dest_index %d\n",
			start_src_index, start_dest_index);
	printf("last_src_index %d last_dest_index %d\n",
			last_src_index, last_dest_index);
	printf("src_off_left %d dest_off_left %d\n",
			src_off_left, dest_off_left);
	printf("src_len_left %d overlap_left %d\n",
			src_len_left, overlap_left);
	*/

	/*
	 * There are 4 cases for merging (checked in this order) :
	 *		1. (dest_offset == dest_hmt->COD_len) : this is really a
	 *			concantenate request
	 *		2. (dest_offset + src_len >= dest_hmt->COD_len) : this is also
	 *			a concatenante, but we need to truncate part of the dest
	 *			COD first.
	 *		3. the src_len spans multiple dest BDLs.  We can use pointer
	 *			manipulation only in this case.
	 *		4. the src_len bytes need to be merged into a single dest BDL.
	 *			In this case, the src bytes need to be copied into the dest
	 *			BDL block by the GMC.  We'll queue copy commands and the
	 *			command will be complete, after the GMC returns the last
	 *			ACK packet.
	 */
#ifdef SBM_USE_TEMPORARY_COPY
	if (dest_off == dest_hmt->COD_len) {
		uint32			bytes_needed, new_buf_cnt, i, j;
		uint32			new_buf[10], new_size[10];
		sbm_bdlEntry_t	* dest_bdl;

		bytes_needed = src_len;
		new_buf_cnt = (bytes_needed/sbm.sac_block_size);
		if (bytes_needed % sbm.sac_block_size)
			new_buf_cnt++;
		for (i=0; i<new_buf_cnt; i++) {
			new_buf[i] = sbm_getBDLIndex();
			if (new_buf[i] == 0xffffffff) {
				for (j=0 ;j<i; j++)
					sbm_ungetBDLIndex(new_buf[j]);
				printf("SBM ERROR, no free buffers available\n");
				return;
			}
			else {
				if (bytes_needed >= sbm.sac_block_size) {
					new_size[i] = sbm.sac_block_size;
					bytes_needed -= sbm.sac_block_size;
				}
				else
					new_size[i] = bytes_needed;
			}
		}
		start_dest_bdl->next_bdl_index = new_buf[0] | BDL_ENTRY_IN_USE;
		for (i=0; i<new_buf_cnt; i++) {
			dest_bdl = &(sbm.bdl[new_buf[i]]);
			dest_bdl->len = new_size[i];
			dest_bdl->offset = 0;
			if (i < new_buf_cnt-1)
				dest_bdl->next_bdl_index = new_buf[i+1] | BDL_ENTRY_IN_USE;
		}
		dest_bdl->next_bdl_index = BDL_EOC;
		dest_hmt->COD_len += src_len;

	}

#else

	if (dest_off == dest_hmt->COD_len) {
		src_hmt->bdl_index = start_src_index | HMT_ENTRY_IN_USE;
		src_hmt->COD_len = src_len;
		start_src_bdl->offset = start_src_bdl->offset + src_off_left;
		start_src_bdl->len = start_src_bdl->len - src_off_left;
		last_src_bdl->len = src_len_left;
		last_src_bdl->next_bdl_index = BDL_EOC;

		/*
		 * Concatenante
		 */
		dest_hmt->COD_len += src_len;
		last_dest_bdl->next_bdl_index = start_src_index | BDL_ENTRY_IN_USE;
	}
	else

#endif

#ifdef SBM_USE_TEMPORARY_COPY
	if (dest_off + src_len > dest_hmt->COD_len) {
		uint32			bytes_needed, new_buf_cnt, i, j;
		uint32			new_buf[10], new_size[10];
		sbm_bdlEntry_t	* dest_bdl;

		bytes_needed = src_len;
		new_buf_cnt = (bytes_needed/sbm.sac_block_size);
		if (bytes_needed % sbm.sac_block_size)
			new_buf_cnt++;
		for (i=0; i<new_buf_cnt; i++) {
			new_buf[i] = sbm_getBDLIndex();
			if (new_buf[i] == 0xffffffff) {
				for (j=0 ;j<i; j++)
					sbm_ungetBDLIndex(new_buf[j]);
				printf("SBM ERROR, no free buffers available\n");
				return;
			}
			else {
				if (bytes_needed >= sbm.sac_block_size) {
					new_size[i] = sbm.sac_block_size;
					bytes_needed -= sbm.sac_block_size;
				}
				else
					new_size[i] = bytes_needed;
			}
		}
		start_dest_bdl->len = dest_off_left;
		start_dest_bdl->next_bdl_index = new_buf[0] | BDL_ENTRY_IN_USE;
		for (i=0; i<new_buf_cnt; i++) {
			dest_bdl = &(sbm.bdl[new_buf[i]]);
			dest_bdl->len = new_size[i];
			dest_bdl->offset = 0;
			if (i < new_buf_cnt-1)
				dest_bdl->next_bdl_index = new_buf[i+1] | BDL_ENTRY_IN_USE;
		}
		dest_bdl->next_bdl_index = BDL_EOC;
		dest_hmt->COD_len = dest_off + src_len;

	}

	{
		uint32	i;
		uint32	len_left, copy_len;
		uint32	src_index, dest_index;
		uchar8	*src_ptr, *dest_ptr;
		sbm_bdlEntry_t			* src_bdl, * dest_bdl;
		uint32	src_bytes_left, dest_bytes_left;
		uint32	src_offset, dest_offset;

		len_left = src_len;
		src_index = start_src_index;
		dest_index = start_dest_index;

		src_offset = src_off_left;
		dest_offset = dest_off_left;
		src_bytes_left = start_src_bdl->len - src_off_left;
		dest_bytes_left = start_dest_bdl->len - dest_off_left;

		while (len_left) {
			src_bdl = sbm.bdl + src_index;
			dest_bdl = sbm.bdl + dest_index;
			src_bytes_left = src_bdl->len - src_offset;
			dest_bytes_left = dest_bdl->len - dest_offset;

			src_ptr = (uchar8 *)sbm_getSACBlockAddr(src_index) +
												src_bdl->offset + src_offset;
			dest_ptr = (uchar8 *)sbm_getSACBlockAddr(dest_index) +
												dest_bdl->offset + dest_offset;
			copy_len = MIN(len_left, src_bytes_left);
			copy_len = MIN(copy_len, dest_bytes_left);

			memcpy(dest_ptr, src_ptr, copy_len);
			dest_ptr += copy_len;
			src_ptr += copy_len;

			if ((copy_len == src_bytes_left) && (copy_len == dest_bytes_left)) {
				src_offset = 0;
				dest_offset = 0;
				src_index = src_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
				dest_index = dest_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
			}
			else if (copy_len == src_bytes_left) {
				src_index = src_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
				dest_offset += copy_len;
				src_offset = 0;
			}
			else if (copy_len == dest_bytes_left) {
				dest_index = dest_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
				src_offset += copy_len;
				dest_offset = 0;
			}
			len_left -= copy_len;
		}
		}

#else

	else if (dest_off + src_len >= dest_hmt->COD_len) {
		/*
		 * Get dest BDL indicies to free
		 */
		i = start_dest_bdl->next_bdl_index;
		while (i != BDL_EOC) {
			uint32	index;
			sbm_bdlEntry_t	* bdl;

			index = i & ~BDL_ENTRY_IN_USE;
			to_free_bdl[to_free_bdl_num++] = index;
			bdl = sbm.bdl + index;
			i = bdl->next_bdl_index;
		}

		dest_hmt->COD_len = dest_off;
		start_dest_bdl->next_bdl_index = BDL_EOC;
		start_dest_bdl->len = dest_off_left;

		src_hmt->bdl_index = start_src_index | HMT_ENTRY_IN_USE;
		src_hmt->COD_len = src_len;
		start_src_bdl->offset = start_src_bdl->offset + src_off_left;
		start_src_bdl->len = start_src_bdl->len - src_off_left;
		last_src_bdl->len = src_len_left;
		last_src_bdl->next_bdl_index = BDL_EOC;

		/*
		 * Concatenante
		 */
		dest_hmt->COD_len += src_len;
		last_dest_bdl->next_bdl_index = start_src_index | BDL_ENTRY_IN_USE;
	}
	else if (start_dest_index != last_dest_index) {
		/*
		 * Get dest BDL indicies to free
		 */
		i = start_dest_index;
		while (i != last_dest_index) {
			sbm_bdlEntry_t	* bdl;

			bdl = sbm.bdl + i;
			i = bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
			if (i != last_dest_index)
			{
				to_free_bdl[to_free_bdl_num++] = i;
			}
		}

		start_dest_bdl->len = dest_off_left;
		start_dest_bdl->next_bdl_index = start_src_index | BDL_ENTRY_IN_USE;
		start_src_bdl->offset = start_src_bdl->offset + src_off_left;
		start_src_bdl->len = start_src_bdl->len - src_off_left;
		last_src_bdl->len = src_len_left;
		last_src_bdl->next_bdl_index = last_dest_index | BDL_ENTRY_IN_USE;
		last_dest_bdl->len = last_dest_bdl->len - overlap_left;
		last_dest_bdl->offset = last_dest_bdl->offset + overlap_left;

	}
	else {

		if ((dest_off_left > 0) &&
					(start_dest_bdl->len == overlap_left + dest_off_left)) {

			last_src_bdl->next_bdl_index = start_dest_bdl->next_bdl_index;
			start_dest_bdl->len = dest_off_left;
			start_dest_bdl->next_bdl_index = start_src_index | BDL_ENTRY_IN_USE;
			start_src_bdl->offset = start_src_bdl->offset + src_off_left;
			start_src_bdl->len = start_src_bdl->len - src_off_left;
			last_src_bdl->len = src_len_left;
		}
		else {
			uint32	remain_len, num_bytes;
			uint32	d_off, s_off;
			uint32	src_index;
			sbm_bdlEntry_t	* src_bdl;

#if 0
			dont_free = 1;
#endif

			/*
			 * Is there room in the queue for the message?
			 */
			cl_cnt = num_src_bdl;
			if (cl_cnt > sbm.copyq.num_avail_cl) {
				printf("No space on CopyQ for command\n");
				sbm_stats->no_copyq_space++;
				ASSERT(FALSE);
				return -1;
			}

			/*
			 * Copy source COD bytes into dest COD
			 */
			qe = sbm.copyq.head;

			remain_len = src_len;
			src_bdl = sbm.bdl + start_src_index;
			d_off = dest_off_left + start_dest_bdl->offset;
			src_index = start_src_index;
			s_off = src_off_left + start_src_bdl->offset;
			if (remain_len >= src_bdl->len) {
				num_bytes = src_bdl->len - src_off_left;
				remain_len -= (src_bdl->len - src_off_left);
			}
			else {
				num_bytes = remain_len;
				remain_len = 0;
			}
			while (num_src_bdl--) {
				/*
				 * Write new entry into copyq
				 */
SBM_TRACE(("copy cmd, src index = %d, off = %d, dest index = %d, off = %d, len = %d\n", src_index, s_off, start_dest_index, d_off, num_bytes));
				req = (sbm_txqCopySACData_t *)&(qe->copyqe);
				req->src_SAC_block_index = src_index;
				req->src_offset = s_off;
				req->dest_SAC_block_index = start_dest_index;
				req->dest_offset = d_off;
				req->num_bytes = num_bytes;
				req->ACK_addr = 0;
				req->TID = TID;
				req->control = TXQ_CMD_COPY_SAC_DATA | TXQ_GMC_OWN |
												(1 << TXQ_CACHELINE_SHIFT);

to_free_bdl[to_free_bdl_num++] = src_index;

				d_off += num_bytes;
				src_index = src_bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
				src_bdl = sbm.bdl + src_index;
				s_off = src_bdl->offset;
				if (remain_len >= src_bdl->len) {
					num_bytes = src_bdl->len;
					remain_len -= src_bdl->len;
				}
				else {
					num_bytes = remain_len;
					remain_len = 0;
				}

				/*
				 * Advance control pointers
				 */
				if (++qe >= sbm.copyq.end)
					qe = sbm.copyq.start;
			}
			if (remain_len)
				panic("remain_len = %d\n", remain_len);

			sbm.copyq.head = qe;
			sbm.copyq.num_avail_cl -= cl_cnt;

sbm_checkGMCQueues(0, 0);
gmc_process_copyq();

		}
	}

#endif

#ifdef SBM_USE_TEMPORARY_COPY
	sbm_freeCOD(src_cod);

#ifdef USE_SAC_CHECKSUM
	dest_hmt->chksum = (uint64)sbm_checksum_cod(dest_cod);
#endif

#else
	if (!dont_free) {
		/*
		 * Free unneeded BDL entries and src HMT index
		 */
		for (; to_free_bdl_num; to_free_bdl_num--) {
SBM_TRACE(("free FBQ Entry %d\n", to_free_bdl[to_free_bdl_num-1]));
			sbm_freeBDLIndexto_free_bdl[to_free_bdl_num-1]);
		}
SBM_TRACE(("free IFQ Entry %d\n", src_hmt_index));
		sbm_freeHMTIndex(src_hmt_index);

		data.COD = src_cod;

		COD_low = src_cod;
		if ((rptr = (sbm_hash_node_t *)
					hash_delete(sbm.sbm_hash, COD_low, &data)) != NULL) {
			sbm_hashNodeStackPush(rptr);
		}
		else
			panic("Cannot remove entry from hash table\n");
	}
#endif

/*
printf("New COD: ");
sbm_debugCODDisplay(dest_cod);
*/

	return NFX_OK;

}	/* sbm_mergeCOD() */

/*-----------------------------------------------------------------
 * Name:	sbm_splitCOD()
 *
 * Description:	Split a src COD into a src COD and a dest COD
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/25/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_splitCOD(
	nfx_cod_t	src_cod,
	uint32		src_off,
	nfx_cod_t	dest_cod
)
{
	sbm_hash_node_t			data;
	sbm_hash_node_t			* rptr;
	uint32					COD_low;
	uint32					src_hmt_index;
	sbm_hmtEntry_t			* hmt, * new_hmt;
	uint32					new_hmt_index;
	sbm_hash_node_t			* bptr;
	sbm_bdlEntry_t			* bdl;
	uint32					offset_left;
	uint32					remain_left, copy_len, keep_len;
	uint32					new_bdl_index, next_bdl_index;
	uchar8					* src, * dest;
	uint32					src_block, dest_block, src_offset, dest_offset, len;
	uint32					i;

	SBM_TRACE(("sbm_splitCOD, src_cod = 0x%llx, offset = %d, dest_cod = 0x%llx\n",
											src_cod, src_off, dest_cod));

	/*
	 * Verify src COD exists and dest COD doesn't
	 */
	data.COD = src_cod;
	COD_low = src_cod;
	if ((rptr = (sbm_hash_node_t *)hash_lookup(sbm.sbm_hash, COD_low, &data))
																	!= NULL) {
		src_hmt_index = rptr->hmt_index;
		SBM_TRACE(("src COD hmt index = %d\n", src_hmt_index));
	}
	else {
		printf("src_cod, 0x%llx, doesn't exist in hash table\n", src_cod);
		ASSERT(FALSE);
		return -1;
	}

	data.COD = dest_cod;
	COD_low = dest_cod;
	if ((rptr = (sbm_hash_node_t *)hash_lookup(sbm.sbm_hash, COD_low, &data))
																	!= NULL) {
		printf("dest_cod, 0x%llx, already exists in hash table\n", dest_cod);
		ASSERT(FALSE);
		return -1;
	}

	/*
	 * Verify offset in source COD
	 */
	hmt = sbm.hmt + src_hmt_index;
	if ((src_off == 0) || (src_off >= hmt->COD_len)) {
		printf("src_offset, %d, >= COD length, %d\n", src_off, hmt->COD_len);
		ASSERT(FALSE);
		return -1;
	}

#ifdef USE_SAC_CHECKSUM
	{
		uint64	chksum;
		if ((chksum = (uint64)sbm_checksum_cod(src_cod)) != hmt->chksum)
			panic("splitCOD chksum error, src_cod = 0x%llx, hmt->chksum = 0x%x, calc = 0x%x\n", src_cod, hmt->chksum, chksum);
	}
#endif

	/*
	 * Split up source COD
	 */
	bdl = sbm.bdl + (hmt->bdl_index & ~HMT_ENTRY_IN_USE);
	offset_left = src_off;
	remain_left = hmt->COD_len;
	while (1) {
		if (bdl->len == offset_left) {
			new_hmt_index = sbm_getHMTIndex();
			if (new_hmt_index == 0xFFFFFFFF) {
				printf("Could not allocate free HMT index\n");
				ASSERT(FALSE);
				return -1;
			}
			new_hmt = sbm.hmt + new_hmt_index;
			new_hmt->bdl_index = bdl->next_bdl_index;
			new_hmt->COD_len = hmt->COD_len - src_off;
			bdl->next_bdl_index = BDL_EOC;
			hmt->COD_len = src_off;
			bptr = sbm_hashNodeStackPop();
			bptr->hmt_index = new_hmt_index;
			bptr->COD = dest_cod;
			COD_low = dest_cod;
			if (hash_insert(sbm_ptr->sbm_hash, (void *)COD_low,
															bptr) != NFX_OK) {
				printf("Error in Local Hash Insert\n");
				sbm_hashNodeStackPush(bptr);
			}
			sbm_stats->num_cods_in_sac++;
#ifdef USE_SAC_CHECKSUM
			hmt->chksum = (uint64)sbm_checksum_cod(src_cod);
			new_hmt->chksum = (uint64)sbm_checksum_cod(dest_cod);
#endif
			break;
		}
		else if (bdl->len < offset_left) {
			offset_left -= bdl->len;
			remain_left -= bdl->len;
			bdl = sbm.bdl + (bdl->next_bdl_index & ~BDL_ENTRY_IN_USE);
		}
		else {
			//printf("splitCOD, misaligned case\n");

			/*
			 * Allocate new HMT index and new BDL index
			 */
			new_hmt_index = sbm_getHMTIndex();
			if (new_hmt_index == 0xFFFFFFFF) {
				printf("Could not allocate HMT index\n");
				ASSERT(FALSE);
				return -1;
			}

			new_bdl_index = sbm_getBDLIndex();
			if (new_bdl_index == 0xFFFFFFFF) {
				printf("Could not allocate HMT index\n");
				sbm_ungetHMTIndex(new_hmt_index);
				ASSERT(FALSE);
				return -1;
			}

			copy_len = bdl->len - offset_left;
			keep_len = offset_left;

			/*
			 * Copy bytes from src sac block into new sac block
			 */
			src_block = bdl - sbm.bdl;
			src_offset = bdl->offset + keep_len;
			dest_block = new_bdl_index;
			dest_offset = 0;
			len = copy_len;

			if (src_offset + len > sbm.sac_block_size)
				panic("GMC : offset + len > size (%d + %d > %d)\n",
										src_offset, len, sbm.sac_block_size);
			src = sbm_sac + (src_block * sbm.sac_block_size);
			dest = sbm_sac + (dest_block * sbm.sac_block_size);

			SBM_TRACE(("copy from sac %p (off %d) to index %p (off %d), len = %d\n",
									src, src_offset, dest, dest_offset, len));
			for (i=0; i<len; i++)
				*(dest + dest_offset + i) = *(src + src_offset + i);

			/*
			 * Update src COD entry
			 */
			hmt->COD_len = hmt->COD_len - remain_left + keep_len;
			next_bdl_index = bdl->next_bdl_index;
			bdl->next_bdl_index = BDL_EOC;
			bdl->len = keep_len;
#ifdef USE_SAC_CHECKSUM
			hmt->chksum = (uint64)sbm_checksum_cod(src_cod);
#endif

			/*
			 * Create new COD, and add to hash table
			 */
			hmt = sbm.hmt + new_hmt_index;
			hmt->COD_len = remain_left - keep_len;
			hmt->bdl_index = new_bdl_index | HMT_ENTRY_IN_USE;
			bdl = sbm.bdl + new_bdl_index;
			bdl->len = copy_len;
			bdl->offset = 0;
			bdl->next_bdl_index = next_bdl_index | BDL_ENTRY_IN_USE;
			if (sbm_insertCOD(dest_cod, new_hmt_index) != NFX_OK) {
				panic("Error inserting COD into hash table\n");
			}

#ifdef USE_SAC_CHECKSUM
			hmt->chksum = (uint64)sbm_checksum_cod(dest_cod);
#endif

			sbm_stats->num_cods_copied++;
			sbm_stats->num_cods_in_sac++;

			break;
		}
	}

	return NFX_OK;

}	/* sbm_splitCOD() */

/*-----------------------------------------------------------------
 * Name:	sbm_concatCOD()
 *
 * Description:	concatenate multiple src CODs into a single dest COD
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/25/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_concatCOD(
	uint32		num_src,
	nfx_cod_t	dest_cod,
	nfx_cod_t	* src_cod
)
{
	uint32				i;
	uint32				tot_len;
	sbm_hash_node_t		data;
	sbm_hash_node_t		* bptr;
	sbm_bdlEntry_t		* bdl;
	sbm_hmtEntry_t		* hmt;
	uint32				COD_low;
	uint32				hmt_index, index;

	SBM_TRACE(("sbm_concatCOD, dest_cod = 0x%016llx, src_cods = 0x%016llx\n",
														dest_cod, src_cod[0]));
	for (i=1; i<num_src; i++) {
		SBM_TRACE(("%57c0x%016llx\n", ' ', src_cod[i]));
	}

	/*
	 * Verify src CODs exist and dest COD doesn't
	 */
	for (i=0; i<num_src; i++) {
		data.COD = src_cod[i];
		COD_low = src_cod[i];
		if ((hash_lookup(sbm.sbm_hash, COD_low, &data)) == NULL) {
			printf("src_cod, 0x%llx, doesn't exist in hash table\n",
																src_cod[i]);
			ASSERT(FALSE);
			return -1;
		}
	}

	data.COD = dest_cod;
	COD_low = dest_cod;
	if (hash_lookup(sbm.sbm_hash, COD_low, &data) != NULL) {
		printf("dest_cod, 0x%llx, already exists in hash table\n", dest_cod);
		ASSERT(FALSE);
		return -1;
	}

	/*
	 * Concatenante src CODs
	 */
	data.COD = src_cod[0];
	COD_low = src_cod[0];
	bptr = (sbm_hash_node_t *)hash_lookup(sbm.sbm_hash, COD_low, &data);
	hmt_index = bptr->hmt_index;
	hmt = sbm.hmt + hmt_index;
	tot_len = hmt->COD_len;
	bdl = sbm.bdl + (hmt->bdl_index & ~HMT_ENTRY_IN_USE);
	while (bdl->next_bdl_index != BDL_EOC)
		bdl = sbm.bdl + bdl->next_bdl_index;

#ifdef USE_SAC_CHECKSUM
		{
			uint64	chksum;
			if ((chksum = (uint64)sbm_checksum_cod(src_cod[0])) != hmt->chksum)
				panic("concatCOD chksum error, src_cod = 0x%llx, hmt->chksum = 0x%x, calc = 0x%x\n", src_cod[0], hmt->chksum, chksum);
		}
#endif

	for (i=1; i<num_src; i++) {
		data.COD = src_cod[i];
		COD_low = src_cod[i];
		bptr = (sbm_hash_node_t *)hash_lookup(sbm.sbm_hash, COD_low, &data);
		index = bptr->hmt_index;
		hmt = sbm.hmt + index;
#ifdef USE_SAC_CHECKSUM
		{
			uint64	chksum;
			if ((chksum = (uint64)sbm_checksum_cod(src_cod[i])) != hmt->chksum)
				panic("concatCOD chksum error, src_cod = 0x%llx, hmt->chksum = 0x%x, calc = 0x%x\n", src_cod[i], hmt->chksum, chksum);
		}
#endif
		tot_len += hmt->COD_len;
		bdl->next_bdl_index = hmt->bdl_index;
		while (bdl->next_bdl_index != BDL_EOC)
			bdl = sbm.bdl + bdl->next_bdl_index;
	}
	hmt = sbm.hmt + hmt_index;
	hmt->COD_len = tot_len;

	/*
	 * Add new COD to hash table
	 */
	bptr = sbm_hashNodeStackPop();
	bptr->COD = dest_cod;
	bptr->hmt_index = hmt_index;

	COD_low = dest_cod;
	if (hash_insert(sbm.sbm_hash, COD_low, bptr) != NFX_OK) {
		sbm_hashNodeStackPush(bptr);
		panic("concatenate COD error, hash insert failed\n");
	}

#ifdef USE_SAC_CHECKSUM
	hmt->chksum = (uint64)sbm_checksum_cod(dest_cod);
#endif

	/*
	 * Release src CODs
	 */
	for (i=0; i<num_src; i++) {
		data.COD = src_cod[i];
		COD_low = src_cod[i];
		if ((bptr = (sbm_hash_node_t *)
						hash_delete(sbm.sbm_hash, COD_low, &data)) == NULL) {
			panic("error in hash_delete, cod 0x%llx\n", src_cod[i]);
		}
		else {
			index = bptr->hmt_index;
			if (i != 0)
				sbm_freeHMTIndex(index);
		}
	}
	sbm_stats->num_cods_freed+=num_src-1;
	sbm_stats->num_cods_in_sac-=num_src-1;

	return NFX_OK;

}	/* sbm_concatCOD() */

/*-----------------------------------------------------------------
 * Name:	sbm_copyCOD()
 *
 * Description:	Copy a src COD into a dest COD
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/25/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_copyCOD(
	nfx_cod_t	src_cod,
	uint32		src_len,
	uint32		src_off,
	nfx_cod_t	dest_cod,
	uint32		dest_off
)
{
	sbm_hash_node_t			* bptr;
	uint32					hmt_index, bdl_index, dest_bdl_index;
	sbm_hmtEntry_t			* hmt;
	sbm_bdlEntry_t			* bdl;
	uint32					src_block_offset;
	uchar8					* src, * dest;
	uint32					src_block, dest_block, src_offset, dest_offset, len;
	uint32					i;


	SBM_TRACE(("sbm_copyCOD, src_cod = 0x%llx, src_len = %d, src_off = %d\n",
												src_cod, src_len, src_off));
	SBM_TRACE(("             dest_cod = 0x%llx, dest_off = %d\n",
														dest_cod, dest_off));

	/*
	 * Verify src COD exist and dest COD doesn't
	 */
	if (!(bptr = (sbm_hash_node_t *)sbm_doesCODExist(src_cod))) {
		printf("cod, 0x%llx, does not exist in hash table\n", src_cod);
		ASSERT(FALSE);
		return -1;
	}
	if (sbm_doesCODExist(dest_cod)) {
		printf("cod, 0x%llx, already exists in hash table\n", dest_cod);
		ASSERT(FALSE);
		return -1;
	}

	/*
	 * Verify src length and offset
	 */
	hmt_index = bptr->hmt_index;
	hmt = sbm.hmt + hmt_index;
	if (hmt->COD_len < (src_len, + src_off)) {
		printf("len + off, %d, > cod len, %d\n", src_len+src_off, hmt->COD_len);
		ASSERT(FALSE);
		return -1;
	}

#ifdef USE_SAC_CHECKSUM
	{
		uint64 chksum;
		if ((chksum = (uint64)sbm_checksum_cod(src_cod)) != hmt->chksum) {
			panic("copyCOD (0x%llx), checksum error, hmt->chksum = 0x%x, calc = 0x%x\n", src_cod, hmt->chksum, chksum);
		}
	}
#endif

	/*
	 * Find sac block from which to copy bytes
	 */
	bdl_index = hmt->bdl_index & ~HMT_ENTRY_IN_USE;
	bdl = sbm.bdl + bdl_index;
	while (1) {
		if (src_off >= bdl->len) {
			src_off -= bdl->len;
			bdl_index = bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
			bdl = sbm.bdl + bdl_index;
		}
		else {
			src_block_offset = bdl->offset + src_off;
			break;
		}
	}

	/*
	 * Verify bytes are all in same sac block
	 */
	if (bdl->len-src_off < src_len) {
		printf("Source bytes are not from a single SAC block\n");
		printf("block length = %d, offset = %d, req length = %d\n",
												bdl->len, src_off, src_len);
		ASSERT(FALSE);
		return -1;
	}

	if (dest_off + src_len > sbm.sac_block_size) {
		printf("Dest offset + source len > block size (%d + %d > %d)\n",
										dest_off, src_len, sbm.sac_block_size);
		ASSERT(FALSE);
		return -1;
	}

	/*
	 * create new COD
	 */
	if (sbm_createCOD(dest_cod, sbm.sac_block_size, 0) != NFX_OK) {
		printf("error creating new cod\n");
		ASSERT(FALSE);
		return -1;
	}

	bptr = (sbm_hash_node_t *)sbm_doesCODExist(dest_cod);
	hmt_index = bptr->hmt_index;
	hmt = sbm.hmt + hmt_index;
	dest_bdl_index = hmt->bdl_index & ~HMT_ENTRY_IN_USE;
	hmt->COD_len = src_len;
	bdl = sbm.bdl + dest_bdl_index;
	bdl->len = src_len;
	bdl->offset = dest_off;


	src_block = bdl_index;
	src_offset = src_block_offset;
	dest_block = dest_bdl_index;
	dest_offset = dest_off;
	len = src_len;

	if (src_offset + len > sbm.sac_block_size)
		panic("GMC : offset + len > size (%d + %d > %d)\n",
										src_offset, len, sbm.sac_block_size);
	src = sbm_sac + (src_block * sbm.sac_block_size);
	dest = sbm_sac + (dest_block * sbm.sac_block_size);

	SBM_TRACE(("copy from sac %p (off %d) to index %p (off %d), len = %d\n",
									src, src_offset, dest, dest_offset, len));
	for (i=0; i<len; i++)
		*(dest + dest_offset + i) = *(src + src_offset + i);

#ifdef USE_SAC_CHECKSUM
	hmt->chksum = (uint64)sbm_checksum_cod(dest_cod);
#endif

	sbm_stats->num_cods_copied++;

	return NFX_OK;

}	/* sbm_copyCOD() */

#define CREATE_BDL_MAX		32
/*-----------------------------------------------------------------
 * Name:	sbm_createCOD()
 *
 * Description:	Create a new COD and initialize it to 'fill' word
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/26/01
 *
 *-----------------------------------------------------------------
 */
sbm_createCOD(
	nfx_cod_t	cod,
	uint32		len,
	uint32		fill
)
{
	uint32			hmt_index;
	sbm_bdlEntry_t	* bdl;
	sbm_hmtEntry_t	* hmt;
	uint32			num_blocks, i, j;
	uint32			bdl_index[CREATE_BDL_MAX];

	SBM_TRACE(("sbm_createCOD: 0x%llx len %d fill %d\n", cod, len, fill));

	/*
	 * Verify COD doesn't exist
 	 */
	if (sbm_doesCODExist(cod)) {
		printf("cod, 0x%llx, already exists in hash table\n", cod);
		ASSERT(FALSE);
		return -1;
	}

	/*
	 * Create COD
	 */
	hmt_index = sbm_getHMTIndex();
	if (hmt_index == 0xFFFFFFFF) {
		printf("Could not allocate HMT index\n");
		ASSERT(FALSE);
		return -1;
	}

	num_blocks = (len + (sbm.sac_block_size-1)) / sbm.sac_block_size;
	if (num_blocks > CREATE_BDL_MAX)
		panic("createCOD, too many block, %d\n", num_blocks);
/*
	bdl_index = (uint32 * )malloc(num_blocks * sizeof(uint32));
*/

	for (i=0; i<num_blocks; i++) {
		bdl_index[i] = sbm_getBDLIndex();
		if (bdl_index[i] == 0xFFFFFFFF) {
			for (j=0; j<i; j++) {
				sbm_ungetBDLIndex(bdl_index[j]);
			}
			sbm_ungetHMTIndex(hmt_index);
			ASSERT(FALSE);
			return -1;
		}
	}

	hmt = sbm.hmt + hmt_index;
	hmt->COD_len = len;
	hmt->bdl_index = bdl_index[0] | HMT_ENTRY_IN_USE;
	for (i=0; i<num_blocks-1; i++) {
		bdl = sbm.bdl + bdl_index[i];
		bdl->len = ((len > sbm.sac_block_size) ? sbm.sac_block_size : len);
		bdl->offset = 0;
		bdl->next_bdl_index = bdl_index[i+1] | BDL_ENTRY_IN_USE;
		len -= bdl->len;
	}
	bdl = sbm.bdl + bdl_index[i];
	bdl->len = len;
	bdl->offset = 0;
	bdl->next_bdl_index = BDL_EOC;

	/*
	 * Add COD to hash table
	 */
	if (sbm_insertCOD(cod, hmt_index) != NFX_OK)
		panic("Error inserting COD into hash table\n");

	/*
	 * Initialize COD with 'fill' word
	 */
	for (i=0; i<num_blocks; i++) {
		sbm_fill_sac_block(bdl_index[i], fill);
	}

#ifdef USE_SAC_CHECKSUM
	hmt->chksum = (uint64)sbm_checksum_cod(cod);
#endif

    /* printf("sbm_create: create cod 0x%llx\n", cod); */

	sbm_stats->num_cods_created++;
	sbm_stats->num_cods_in_sac++;

	return NFX_OK;

}	/* sbm_createCOD() */

#endif	/* NFP_TXRX */

