/*  FileName: scsi-err.c
 *
 *  Description: Contains functions for printing and logging SCSI module
 *				 errors.
 *
 *	Created:	22 February	2001
 *
 *  Author:		Frank Salzmann
 *
 *  Copyright (c)  Agile Storage, Inc. 2001
 */			 

#include "scsi.h"

static char *scsi_err_msg[] = {
  "SCSI Ok",									  /* SCSI_OK				*/
  "SCSI Err: ",									  /* SCSI_ERR_GENERIC		*/
  "SCSI Err: Could not allocate memory for: ", 	  /* SCSI_ERR_MALLOC       	*/
  "SCSI Err: Device Number Out Of Range: ",    	  /* SCSI_ERR_DEVNUM_RANGE 	*/
  "SCSI Err: Device State Machine Not Created: ", /* SCSI_ERR_SM_CREATE	   	*/
  "SCSI Err: Out of request descriptors: ", 	  /* SCSI_ERR_SD_ALLOC 		*/
};

int
scsi_err( int errCode, char *msg ) 
{
	uint32 msgType;
	uint32 msgLevel; 

	msgLevel = (errCode & 0xFFFF0000) >> 16;
	msgType = errCode & 0x0000FFFF;

	if( scsiErrLevelCtrl[msgLevel] == SCSI_ERR_LEVEL_ENABLED )
	{
		printf("%s%s\n", scsi_err_msg[msgType], msg);
	}
	return 0;
}

int
scsi_debug( char *msg, uint32 level )
{
	if( scsiDebugLevelCtrl[level] == 1 )
	{
		printf("%s\n", msg);
	}
	return 0;
}
