/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: read.c
 *
 * RCS:  
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Sends test read data to a device.
 *
 * Created by:	Frank Salzmann
 *
 * Date Created: July 28, 2001
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "rw_api.h"

static void print_usage()
{
	fprintf(stderr, "Usage:\n");
	fprintf(stderr, " %s [-s] [-p] [-k] [-f OutputFile] <wwn> <lun> <LBA> <block_count>\n",self);
	fprintf(stderr, "     -s = interpret as FS Superblock\n");
	fprintf(stderr, "     -p = physical block addressing\n");
	fprintf(stderr, "     -k = kick open write gate\n");
}

int main(int argc, char *argv[])
{
	int			exit_code = 0;
	uint64      wwn;
	uint32      lun;
	uint32 		sdmHandle;
	uint32		lba;
	uint32		block_count;
	int			c;
	uint32		flags = 0;

	self = argv[0];						// How we were invoked
	if (rindex(self,'/')) {
		strcpy(self, rindex(self,'/') + 1);	// Discard path
	}

	/* <wwn> <lun> <LBA> <block_count> */
	if (argc < 5) {		// Make sure there's a command string at least
		print_usage();
		exit(1);
	}

	/* get options */
	while((c = getopt(argc, argv, "pskf:")) != -1) {
		switch(c) {
			case 's': 
					flags |= RW_FLAG_READ_SUPER;
					break;
			case 'p':
					flags |= RW_FLAG_PHY_OP;
					break;
		    case 'k':
					flags |= RW_FLAG_KICK_WR_GATE;
					break;
			case 'f':
					flags |= RW_FLAG_OUTPUT_FILE;
					outputFile = (char *)malloc(256);
					if(!outputFile) {
							fprintf(stderr, "malloc failed: %s\n", strerror(errno));
							exit(errno);
					}
					strncpy(outputFile, optarg, 255);
					break;
			default:
					print_usage();
					exit(1);
		}
	}
	if(flags & RW_FLAG_READ_SUPER) argc--;
	if(flags & RW_FLAG_PHY_OP) argc--;
	if(flags & RW_FLAG_KICK_WR_GATE) argc--;
	if(flags & RW_FLAG_OUTPUT_FILE) argc -= 2;

	if(argc != 5) {
			print_usage();
			exit(1);
	}

	argv = argv + (optind - 1);
	wwn = strtoq(argv[1], NULL, 0);
	lun = strtol(argv[2], NULL, 0);
	lba = strtol(argv[3], NULL, 0);
	block_count = strtol(argv[4], NULL, 0);
	if (block_count % 2) {
		printf("block count must be a multiple of 2\n");
		exit(1);
	}

	if(block_count > 128) {
		printf("Block size > 128 not supported\n");
		exit(1);
	}
	
	sdmHandle = sdm_ui_getSdmHandle(wwn, lun);
	if (sdmHandle == 0)
	exit(1);

	if(flags & RW_FLAG_READ_SUPER)
		exit_code = disk_read_super(sdmHandle, lba);
	else
		exit_code = disk_read(sdmHandle, lba, block_count, flags);

	exit (exit_code);
}
