#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#undef NULL
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/scsi-api.h>
#include "../sm-scsi-test/scsi-test.h"

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"

#define FILE_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)

#define SDM_USE_DB_FILE 0

char devDbFile[256];

eDB_t *
sdm_dbInit( char *dbFile, char *name )
{
#if SDM_USE_DB_FILE 
	int fd;
#endif
	eDB_t *db;
	eDB_mrec_t *mrec;

	db = (eDB_t *)malloc(sizeof(eDB_t));
	if( db == (eDB_t *)NULL ) {
		//printf("Error: Cannot allocate database control memory.\n");
		return(0);
	}

#if SDM_USE_DB_FILE 

	strcpy(devDbFile, dbFile);

	if( (fd = open(dbFile, O_RDWR, FILE_MODE)) < 0 ) {

		//printf("Creating New database......\n");
		fd = open(dbFile, O_RDWR | O_CREAT, FILE_MODE);
		if( fd < 0 ) {
			//printf("Error: Unable to open device database: %s\n", dbFile);
			return(0);
		}

		mrec = (eDB_mrec_t *)malloc(sizeof(eDB_mrec_t));
		if( mrec == (eDB_mrec_t *)NULL ) {
			//printf("Error: Cannot allocate database master record.\n");
			return(0);
		}

		/* Write out control field to database file */
		memset(mrec, 0, sizeof(eDB_mrec_t));
		strcpy(mrec->dbName, name);
		write(fd, mrec, sizeof(eDB_mrec_t));
		db->mrec  = mrec;
		db->fdData = fd;

	}
	else {
		//printf("Database Open....\n");
		db->fdData = fd;
		sdm_dbImport(db);
	}
	sdm_dbClose(db);
#else
	mrec = (eDB_mrec_t *)malloc(sizeof(eDB_mrec_t));
	if( mrec == (eDB_mrec_t *)NULL ) {
		//printf("Error: Cannot allocate database master record.\n");
		return(0);
	}
	memset(mrec, 0, sizeof(eDB_mrec_t));
	strcpy(mrec->dbName, name);
	db->mrec  = mrec;
#endif // SDM_USE_DB_FILE

	return db;
} /* sdm_dbInit */

int
sdm_dbClose(eDB_t *db)
{
#if SDM_USE_DB_FILE
	close(db->fdData);
#endif
	return(0);
} /* sdm_dbClose */

int
sdm_dbOpen(eDB_t *db)
{
	int rc;
#if SDM_USE_DB_FILE
	int fd;

	if( (fd = open(devDbFile, O_RDWR, FILE_MODE)) < 0 ) {
		rc = SDM_FAIL;
	}
	else {
		rc = SDM_OK;
	} 
#endif
	return(rc);
} /* sdm_dbClose */

int
sdm_dbImport(eDB_t *db)
{
#if SDM_USE_DB_FILE
	int        i;
	int		   recCnt;
	eDB_mrec_t *mrec;
	sdm_devDBRec_t *devRec;

	mrec = (eDB_mrec_t *)malloc(sizeof(eDB_mrec_t));
	if( mrec == (eDB_mrec_t *)NULL ) {
		//printf("Error: Cannot allocate database master record.\n");
		return(-1);
	}
	db->mrec = mrec;

	//printf("Reading database master record.....\n");
	read(db->fdData, mrec, sizeof(eDB_mrec_t));

	//printf("Database contains, %d, device records.\n", mrec->recCount);

	//printf("Importing database records......\n");	
	recCnt = db->mrec->recCount;
	db->mrec->recCount = 0;
	db->drec_head = 0;	
	db->drec_tail = 0;	
	for(i=0; i < recCnt; i++) {

		devRec = (sdm_devDBRec_t *)malloc(sizeof(sdm_devDBRec_t));
		if( !devRec ) {
			//printf("Error: Cannot allocate database record.\n");
			return(-1);
		}
		read(db->fdData, devRec, sizeof(sdm_devDBRec_t)); 
		//printf("Importing Device: WWN: %016llx\n", devRec->id);
		sdm_dbAddRec(db, devRec->id, devRec, sizeof(sdm_devDBRec_t),0);
	}
#endif

	return 0;
} /* sdm_dbImportRecs */

int
sdm_dbWriteRec( eDB_t *db, eDB_drec_t *drec )
{
#if SDM_USE_DB_FILE
	if( sdm_dbOpen(db) == SDM_FAIL ) {
		//printf("sdm_dbWriteRec: Error: Cannot open device database.\n");
		return(0);
	}

	// Write out control field to database file 
	lseek(db->fdData, 0, SEEK_SET);
	write(db->fdData, db->mrec, sizeof(eDB_mrec_t));

	// Find EOF and write in the record. 
	lseek(db->fdData, 0, SEEK_END);
	write(db->fdData, drec->dobject, sizeof(sdm_devDBRec_t));

	// Close database file
	sdm_dbClose(db);
#endif
	return 0;
} /* sdm_dbWriteRec */

int
sdm_dbAddRec(eDB_t *db, uint64 key, void *dobj, uint32 dobj_size, int32 fsave)
{
	eDB_drec_t *drec;

	drec = (eDB_drec_t *)malloc(sizeof(eDB_drec_t));
	if( !drec ) {
		//printf("Error: Cannot allocate database record.\n");
		return(-1);
	}

	drec->key = key;
	drec->dobject = dobj;
	drec->dobject_size = dobj_size;

	if( db->mrec->recCount == 0 ) {
		db->drec_head = drec;	
		db->drec_tail = drec;	
		drec->next = 0;
		db->mrec->recCount++;
	}	
	else  {
		db->drec_tail->next = drec;
		db->drec_tail = drec;
		drec->next = 0;
		db->mrec->recCount++;
	}

	// Don't write record to file if it is being added due
	// to the import process.
	if( fsave )
		sdm_dbWriteRec( db, drec );

	return 0;
} /* sdm_dbAddRec */

int
sdm_dbRemRec(eDB_t *db, uint64 key)
{
	int i;
	eDB_drec_t *drecPrev;
	eDB_drec_t *drec;

	drec = db->drec_head;
	drecPrev = 0;
	for(i=0; (i < db->mrec->recCount) && (drec); i++) {
		// Look for the data record.
		if(!drec)
			return 0;

		if( drec->key == key ) {

			// Unlink the data record.
			if( drec == db->drec_head ) {
				db->drec_head = drec->next;
			}

			if( drec == db->drec_tail ) {
				db->drec_tail = drecPrev;
			}

			if( drecPrev )
				drecPrev->next = drec->next;

			if( drec )
				free( drec );

			db->mrec->recCount--;
			return 0;
		}
		drecPrev = drec;
		drec = drec->next;
	}	
	return 0;
} /* sdm_dbRemRec */

void *
sdm_dbFindRec(eDB_t *db, uint64 key)
{
	int i;
	eDB_drec_t *drec;

	drec = db->drec_head;
	for(i=0; (i < db->mrec->recCount) && (drec); i++) {
		if( drec->key == key )
			return drec->dobject;	
		drec = drec->next;
	}	
	return 0;
} /* sdm_dbFindRec */

void *
sdm_dbGetNextRec(eDB_t *db, uint64 key)
{
	int i;
	eDB_drec_t *drec;

	drec = db->drec_head;
	if( key == 0  && drec )
		return drec->dobject;

	for(i=0; (i < db->mrec->recCount) && (drec); i++) {
		if( drec->key == key ) 
			if( drec->next )
				return drec->next->dobject;	
		drec = drec->next;
	}	
	return 0;
} // sdm_dbGetNextRec

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_dbGetNumEntries
 *
 * Description: 	Returns number of entries in the SDM device database.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 11, 2001
 *
 * Returns:         Num Entries
 *
 *------------------------------------------------------------------------------
 */
int
sdm_dbGetNumEntries( eDB_t *db, uint32 type, uint32 port, char *vendor )
{
	int i, count=0, found, j;
	eDB_drec_t *drec;
	sdm_devDBRec_t *dev;

	// if( type == SDM_DEVLIST_ALL ) { 
	// 	count = db->mrec->recCount;
	// }
	// else {
		drec = db->drec_head;	
		for( i=0; i < db->mrec->recCount; i++ ) {
			dev = (sdm_devDBRec_t *)drec->dobject;

            if (port != SDM_DEVLIST_PORTALL)
            {
                found = 0;
                for(j=0; j < dev->pathCnt; j++) {
                    if( SCSI_PATH_GET_PORT(dev->pathList[j].path) == port )
                    {
                        found = 1;
                        break;
                    }                        
                }
                if (!found)
                {
                    drec = drec->next;
                    continue; // If not found continue the the next rec.
                } 
            }
            
            if (vendor && strcasecmp(vendor, dev->st.vendor))
            {
                drec = drec->next;
                continue; // If not found continue the the next rec.
            }

	        if( type == SDM_DEVLIST_ALL ) { 
	            ++count;
	        }
			else
            if((type == SDM_DEVLIST_ALL_DISK) && (dev->devType == SCSI_DEVTYPE_DISK)) {
					count++;
			}
			else
			if((dev->devType == type) && !(dev->state == SDM_DEV_STATE_INIT) && 
				!(dev->state == SDM_DEV_STATE_INIT_OPENED) &&
				!(dev->state == SDM_DEV_STATE_INIT_OPENING)) {
				count++;
			}

			drec = drec->next;
		}
	// }
	return count;
} /* sdm_dbGetNumEntries */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_dbGetDevList
 *
 * Description: 	Returns list of entries in the SDM device database.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 11, 2001
 *
 * Returns:         Num Entries
 *
 *------------------------------------------------------------------------------
 */
int
sdm_dbGetDevList( eDB_t *db, scsi_id_t *list, uint32 type,
                  uint32 port, char *vendor )
{
	int i, recCount, j, found;
	eDB_drec_t *drec;
	sdm_devDBRec_t *dev;
	
	drec = db->drec_head;	
	recCount = db->mrec->recCount;
	for( i=0; i < recCount; i++ ) {
		dev = (sdm_devDBRec_t *)drec->dobject;

        if (port != SDM_DEVLIST_PORTALL)
        {
            found = 0;
            for(j=0; j < dev->pathCnt; j++) {
                if( SCSI_PATH_GET_PORT(dev->pathList[j].path) == port )
                {
                    found = 1;
                    break;
                }
            }
            if (!found)
            {
                drec = drec->next;
                continue; // If not found continue the the next rec.
            }
        }

        if (vendor && strcasecmp(vendor, dev->st.vendor))
        {
            drec = drec->next;
            continue; // If not found continue the the next rec.
        }

		if( type == SDM_DEVLIST_ALL ) {
			*list = dev->id; 
			list++;
		}
		else
		if((type == SDM_DEVLIST_ALL_DISK) && (dev->devType == SCSI_DEVTYPE_DISK)) {
			*list = dev->id; 
			list++;
		}
		else
		if((dev->devType == type) && !(dev->state == SDM_DEV_STATE_INIT) &&
			!(dev->state == SDM_DEV_STATE_INIT_OPENED) &&
			!(dev->state == SDM_DEV_STATE_INIT_OPENING)) {
			*list = dev->id; 
			list++;
		}

		drec = drec->next;
	}
	return(0);
} /* sdm_dbGetDevList */


//================= sdm_devDBRec specific functions ======================
sdm_devDBRec_t *
sdm_dbFindWwnLun(eDB_t *db, uint64 wwn, uint32 lun)
{
	int i;
	eDB_drec_t *drec;
	sdm_devDBRec_t *devRec;

	drec = db->drec_head;
	for(i=0; (i < db->mrec->recCount) && (drec); i++) {
		devRec = (sdm_devDBRec_t *)drec->dobject;
		if( devRec->wwn == wwn ) {
			if( devRec->lun == lun )
				return devRec;	
		}
		drec = drec->next;
	}	
	return 0;
} // sdm_dbFindWwnLun


sdm_devDBRec_t *
sdm_dbFindDevname(eDB_t *db, char *name)
{
	int 			len, i;
	eDB_drec_t		*drec;
	sdm_devDBRec_t	*devRec;

	len = strlen(name);
	drec = db->drec_head;
	for(i=0; i < db->mrec->recCount && drec; i++) {
		devRec = (sdm_devDBRec_t *)drec->dobject;
		if(!strncmp(devRec->st.name, name, len))
			return devRec;
		drec = drec->next;
	}

	return NULL;
}

sdm_devDBRec_t *
sdm_dbFindId(eDB_t *db, scsi_id_t id)
{
	int				i;
	eDB_drec_t		*drec;
	sdm_devDBRec_t	*devRec;

	drec = db->drec_head;
	for(i=0; i < db->mrec->recCount && drec; i++) {
		devRec = (sdm_devDBRec_t *)drec->dobject;
		if(devRec->id == id)
			return devRec;
		drec = drec->next;
	}

	return NULL;
}
