//------------------------------------------------------------------------------
//  FileName   : sdm-init-disk.c
//
//  Description: Contains state tables and initialization functions for
//               disk and raid devices. 
//
//	Created    : 28 March 2002
//
//  Author     : Frank Salzmann
//
//  RCS        : $Id: sdm-init-disk.c,v 1.20 2002/11/09 00:37:35 nitinb Exp $
//
//  Copyright (c)  Agile Storage, Inc. 2002
//------------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#undef NULL

#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/eee-api.h>  
#include <exports/scsi-api.h>
#include <exports/elog-api.h>

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"
#include "sdm-init-disk-mylex.h"
#include "sdm-msg.h"
#include "../sm-scsi/scsi-dev-cmds.h"

//-----------------------------------------------------------------------
//-- DISKS and RAID Arrays
//-----------------------------------------------------------------------

//-- Generic Disk Device Init State Table
int sdm_initDiskGeneric(sdm_dir_t *, char *);

sdm_dist_t sdm_idGenericST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV, SDM_DIE_RCAP_REQ, sdm_initDiskGeneric}, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ, SDM_DIE_RCAP_RSP, sdm_sendRCapCmd    },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP, SDM_DIE_SN_REQ  , sdm_procRCap       },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq      },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP  , SDM_DIE_FIN     , sdm_procSnRsp      },
	{SDM_DIS_INIT  , SDM_DIE_FIN     , NULL            , sdm_initFinish     },
	{SDM_DIS_INIT  , SDM_DIE_TO      , NULL            , sdm_initTimeout    },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP , NULL            , sdm_verifyInqRsp   },
	{0,0,0}
};

//-- Hitachi Intialization State Table
int sdm_initHitachi(sdm_dir_t *, char *);
int sdm_procHitachiRaidRsp( sdm_dir_t *, char *);
int sdm_getHitachiModelNum( sdm_dir_t *, char *);
int sdm_initFinishHitachi(sdm_dir_t *, char *);

sdm_dist_t sdm_idHitachiST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV, SDM_DIE_RLUN_REQ, sdm_initHitachi       },
	{SDM_DIS_INIT  , SDM_DIE_INIT_LUN, SDM_DIE_RCAP_REQ, sdm_initHitachi       }, 
	{SDM_DIS_INIT  , SDM_DIE_RLUN_REQ, SDM_DIE_RLUN_RSP, sdm_sendRLunCmd       },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_RSP, SDM_DIE_RCAP_REQ, sdm_procRLun          }, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ, SDM_DIE_RCAP_RSP, sdm_sendRCapCmd       },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP, SDM_DIE_SN_REQ  , sdm_procRCap          },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq         },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP  , SDM_DIE_FIN     , sdm_procSnRsp         },
	{SDM_DIS_INIT  , SDM_DIE_FIN     , NULL            , sdm_initFinishHitachi },
	{SDM_DIS_INIT  , SDM_DIE_TO      , NULL            , sdm_initTimeout       },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP , NULL            , sdm_verifyInqRsp      },
	{0,0,0}
};

//-- EMC Intialization State Table
int sdm_initEMC(sdm_dir_t *, char *);
int sdm_procEMCRaidRsp( sdm_dir_t *, char *);
int sdm_initFinishEMC(sdm_dir_t *, char *);

sdm_dist_t sdm_idEMCST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV, SDM_DIE_RLUN_REQ, sdm_initEMC           },
	{SDM_DIS_INIT  , SDM_DIE_INIT_LUN, SDM_DIE_RCAP_REQ, sdm_initEMC           }, 
	{SDM_DIS_INIT  , SDM_DIE_RLUN_REQ, SDM_DIE_RLUN_RSP, sdm_sendRLunCmd       },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_RSP, SDM_DIE_RCAP_REQ, sdm_procRLun          }, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ, SDM_DIE_RCAP_RSP, sdm_sendRCapCmd       },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP, SDM_DIE_SN_REQ  , sdm_procRCap          },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq         },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP  , SDM_DIE_FIN     , sdm_procSnRsp         },
	{SDM_DIS_INIT  , SDM_DIE_FIN     , NULL            , sdm_initFinishEMC     },
	{SDM_DIS_INIT  , SDM_DIE_TO      , NULL            , sdm_initTimeout       },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP , NULL            , sdm_verifyInqRsp      },
	{0,0,0}
};

//-- Mylex Intialization State Table
int sdm_initMylex(sdm_dir_t *, char *);
int sdm_procMylexDACARMSnRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_procMylexFFx2SnRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_sendMylexRaidReq( sdm_dir_t *, void * ) ;
int sdm_procMylexRaidRsp( sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_initFinishMylex(sdm_dir_t *, char *);

sdm_dist_t sdm_idMylexDACARMST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV, SDM_DIE_RLUN_REQ, sdm_initMylex           },
	{SDM_DIS_INIT  , SDM_DIE_INIT_LUN, SDM_DIE_RCAP_REQ, sdm_initMylex			 }, 
	{SDM_DIS_INIT  , SDM_DIE_RLUN_REQ, SDM_DIE_RLUN_RSP, sdm_sendRLunCmd	     },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_RSP, SDM_DIE_RCAP_REQ, sdm_procRLun			 }, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ, SDM_DIE_RCAP_RSP, sdm_sendRCapCmd	     },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP, SDM_DIE_SN_REQ  , sdm_procRCap			 },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq			 },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP  , SDM_DIE_RAID_REQ, sdm_procMylexDACARMSnRsp},
	{SDM_DIS_INIT  , SDM_DIE_RAID_REQ, SDM_DIE_RAID_RSP, sdm_sendMylexRaidReq    },
	{SDM_DIS_INIT  , SDM_DIE_RAID_RSP, SDM_DIE_FIN     , sdm_procMylexRaidRsp    },
	{SDM_DIS_INIT  , SDM_DIE_FIN     , NULL            , sdm_initFinishMylex	 },
	{SDM_DIS_INIT  , SDM_DIE_TO      , NULL            , sdm_initTimeout		 },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP , NULL            , sdm_verifyInqRsp		 },
	{0,0,0}
};

sdm_dist_t sdm_idMylexFFx2ST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV, SDM_DIE_RLUN_REQ, sdm_initMylex           },
	{SDM_DIS_INIT  , SDM_DIE_INIT_LUN, SDM_DIE_RCAP_REQ, sdm_initMylex		     }, 
	{SDM_DIS_INIT  , SDM_DIE_RLUN_REQ, SDM_DIE_RLUN_RSP, sdm_sendRLunCmd	     },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_RSP, SDM_DIE_RCAP_REQ, sdm_procRLun			 }, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ, SDM_DIE_RCAP_RSP, sdm_sendRCapCmd	     },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP, SDM_DIE_SN_REQ  , sdm_procRCap			 },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq			 },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP  , SDM_DIE_RAID_REQ, sdm_procMylexFFx2SnRsp  },
	{SDM_DIS_INIT  , SDM_DIE_RAID_REQ, SDM_DIE_RAID_RSP, sdm_sendMylexRaidReq    },
	{SDM_DIS_INIT  , SDM_DIE_RAID_RSP, SDM_DIE_FIN     , sdm_procMylexRaidRsp    },
	{SDM_DIS_INIT  , SDM_DIE_FIN     , NULL            , sdm_initFinishMylex	 },
	{SDM_DIS_INIT  , SDM_DIE_TO      , NULL            , sdm_initTimeout		 },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP , NULL            , sdm_verifyInqRsp		 },
	{0,0,0}
};

//-- IBM Profibre Intialization State Table
int sdm_procProfibreSnRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;

sdm_dist_t sdm_idProfibreST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV, SDM_DIE_RLUN_REQ, sdm_initDiskGeneric  },
	{SDM_DIS_INIT  , SDM_DIE_INIT_LUN, SDM_DIE_RCAP_REQ, sdm_initDiskGeneric  },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_REQ, SDM_DIE_RLUN_RSP, sdm_sendRLunCmd      },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_RSP, SDM_DIE_RCAP_REQ, sdm_procRLun         }, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ, SDM_DIE_RCAP_RSP, sdm_sendRCapCmd      },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP, SDM_DIE_SN_REQ  , sdm_procRCap         },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq        },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP  , SDM_DIE_FIN     , sdm_procProfibreSnRsp},
	{SDM_DIS_INIT  , SDM_DIE_FIN     , NULL            , sdm_initFinish       },
	{SDM_DIS_INIT  , SDM_DIE_TO      , NULL            , sdm_initTimeout      },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP , NULL            , sdm_verifyInqRsp     },
	{0,0,0}
};

//-- LSI INF-01-00 Intialization State Table
int sdm_initDiskLsi(sdm_dir_t *, char *);
int sdm_sendLsiRaidReq( sdm_dir_t *, void * ) ;
int sdm_procLsiRaidRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_sendLsiLunidReq( sdm_dir_t *, void * ) ;
int sdm_procLsiLunidRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_sendLsiAccessReq( sdm_dir_t *, void * ) ;
int sdm_procLsiAccessRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_sendLsiIsSnapReq( sdm_dir_t *, void * ) ;
int sdm_procLsiIsSnapRsp(sdm_dir_t *, scsi_relayRspMsg_t * ) ;
int sdm_sendLsiRHwReq(sdm_dir_t *, void * );
int sdm_procLsiRHwRsp(sdm_dir_t *, scsi_relayRspMsg_t * );
int sdm_setLsiModelNum(sdm_dir_t *, char *);
int sdm_initFinishLsi(sdm_dir_t *, char *);
sdm_dist_t sdm_idLsiST[] = {
	{SDM_DIS_INIT  , SDM_DIE_INIT_DEV  , SDM_DIE_RLUN_REQ  , sdm_initDiskLsi      },
	{SDM_DIS_INIT  , SDM_DIE_INIT_LUN  , SDM_DIE_RCAP_REQ  , sdm_initDiskLsi      },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_REQ  , SDM_DIE_RLUN_RSP  , sdm_sendRLunCmd      },
	{SDM_DIS_INIT  , SDM_DIE_RLUN_RSP  , SDM_DIE_RCAP_REQ  , sdm_procRLun         }, 
	{SDM_DIS_INIT  , SDM_DIE_RCAP_REQ  , SDM_DIE_RCAP_RSP  , sdm_sendRCapCmd      },
	{SDM_DIS_INIT  , SDM_DIE_RCAP_RSP  , SDM_DIE_ACCESS_REQ, sdm_procRCap         },
	{SDM_DIS_INIT  , SDM_DIE_ACCESS_REQ, SDM_DIE_ACCESS_RSP, sdm_sendLsiAccessReq },
	{SDM_DIS_INIT  , SDM_DIE_ACCESS_RSP, SDM_DIE_SN_REQ    , sdm_procLsiAccessRsp },
	{SDM_DIS_INIT  , SDM_DIE_SN_REQ    , SDM_DIE_SN_RSP    , sdm_sendSnReq        },
	{SDM_DIS_INIT  , SDM_DIE_SN_RSP    , SDM_DIE_RAID_REQ  , sdm_procSnRsp        },
	{SDM_DIS_INIT  , SDM_DIE_RAID_REQ  , SDM_DIE_RAID_RSP  , sdm_sendLsiRaidReq   },
	{SDM_DIS_INIT  , SDM_DIE_RAID_RSP  , SDM_DIE_LUNID_REQ , sdm_procLsiRaidRsp   },
	{SDM_DIS_INIT  , SDM_DIE_LUNID_REQ , SDM_DIE_LUNID_RSP , sdm_sendLsiLunidReq  },
	{SDM_DIS_INIT  , SDM_DIE_LUNID_RSP , SDM_DIE_RHW_REQ   , sdm_procLsiLunidRsp  },
	{SDM_DIS_INIT  , SDM_DIE_RHW_REQ   , SDM_DIE_RHW_RSP   , sdm_sendLsiRHwReq    },
	{SDM_DIS_INIT  , SDM_DIE_RHW_RSP   , SDM_DIE_FIN       , sdm_procLsiRHwRsp    },
	{SDM_DIS_INIT  , SDM_DIE_FIN       , NULL              , sdm_initFinishLsi    },
	{SDM_DIS_INIT  , SDM_DIE_TO        , NULL              , sdm_initTimeout      },
	{SDM_DIS_VERIFY, SDM_DIE_INQ_RSP   , NULL              , sdm_verifyInqRsp     },
	{0,0,0}
};

//-- Device Initialization Database Setup.
//-- For each and every vendor/device that we want to support 
//-- and entry needs to be in this table.
sdm_devSupport_t sdm_diskList[] = {
	{"NETAPP" , "X231_ST336704FC", "*", sdm_idGenericST     },
	{"NETAPP" , "*"              , "*",	sdm_idGenericST		},  // Any NETAPP Model
	{"HITACHI", "OPEN-3"         , "*", sdm_idHitachiST		},
	{"HITACHI", "*"              , "*", sdm_idHitachiST		},  // Any Hitachi Model
	{"EMC"    , "SYMMETRIX"      , "*", sdm_idEMCST			},  // EMC Symmetrix
	{"MYLEX"  , "DACARMRB"       , "7770", sdm_idMylexDACARMST },
	{"MYLEX"  , "DACARMRB"       , "5840", sdm_idMylexFFx2ST },
	{"MYLEX"  , "DACARMRB"       , "7840", sdm_idMylexFFx2ST },
	{"MYLEX"  , "FFx2"           , "5840", sdm_idMylexFFx2ST   },
	{"MYLEX"  , "FFx2"           , "7800", sdm_idMylexFFx2ST   },
	{"MYLEX"  , "FFx2"           , "7840", sdm_idMylexFFx2ST   },
	{"MYLEX"  , "*"              , "*", sdm_idMylexDACARMST },  // Mylex: all
	{"IBM"    , "ProFibre-4000R" , "*", sdm_idProfibreST    },  // IBM Profibre array
	{"LSI"    , "INF-01-00"      , "*", sdm_idLsiST         },  // LSI array: 2700
	{"LSI"    , "*"              , "*", sdm_idLsiST         },  // LSI array: all
	{"*"      , "*"              , "*", sdm_idGenericST     },  // Disk Catch All
	{0,0,"",0}
};

//------------------------------------------------------------------------------
// Function Name:   sdm_initDiskGeneric
// 
// Description:     
//
// Data Created:    March 16, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initDiskGeneric(sdm_dir_t *dir, char *inqRsp)
{
	//printf("sdm_initDiskGeneric\n");

	//-- Create scsi id
	dir->id = sdm_initGetScsiId(dir->wwn, dir->lun, dir->st.vendor);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initDiskGeneric


//------------------------------------------------------------------------------
// Function Name:   sdm_initHitachi
// 
// Description:     
//
// Data Created:    March 16, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initHitachi(sdm_dir_t *dir, char *inqRsp)
{
	uint32        ldevId;
	unsigned char ldev[5];

	//-- Extract the Hitachi vendor specific logical device id.
    ldev[0] = inqRsp[44];
    ldev[1] = inqRsp[45];
    ldev[2] = inqRsp[46];
    ldev[3] = inqRsp[47];
    ldev[4] = '\0';
    ldevId = strtoq(ldev, NULL, 16);

	//-- Create scsi id
	//dir->id   = sdm_initGetScsiId(dir->wwn, ldevId, dir->vendor);
	SDM_INIT_ID_SET_WWN( dir->id, dir->wwn);
	SDM_INIT_ID_SET_LUN( dir->id, dir->lun);
	
	//-- Save LDEV
	dir->v.hitachi.ldevId = ldevId;

	//-- Get RAID level from Inq response.
	sdm_procHitachiRaidRsp(dir, inqRsp);

	//-- Get chassis model number from Inq Response.
	sdm_getHitachiModelNum(dir, inqRsp);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initHitachi

//------------------------------------------------------------------------------
// Function Name:   sdm_procHitachiRaidRsp
// 
// Description:     
//
// Data Created:    April 30, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procHitachiRaidRsp(sdm_dir_t *dir, char *inqRsp ) 
{ 
	dir->raidLevel = inqRsp[98];

	return SDM_OK;
} // sdm_procHitachiRaidRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_getHitachiModelNum
// 
// Description:     
//
// Data Created:    November 1, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_getHitachiModelNum(sdm_dir_t *dir, char *inqRsp ) 
{ 
	char *sn;

	sn = &inqRsp[0x24];

    //-- Figure out the HDS model number. Determined by looking at the first
    //-- four bytes of the serial number.
    //-- "0400" = HDS9960
    //-- "0401" = HDS9910
    //-- "????" = HDSxxxx, this is what it will look like if we can't tell.
    if( strncmp("0400", sn, 4) == 0 )
            strcpy(dir->st.model, "HDS9960");
    else
    if( strncmp("0401", sn, 4) == 0 )
            strcpy(dir->st.model, "HDS9910");
    else
            strcpy(dir->st.model, "HDSxxxx");

	return SDM_OK;
} // sdm_getHitachiModelNum


//------------------------------------------------------------------------------
// Function Name:   sdm_initFinishHitachi
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initFinishHitachi(sdm_dir_t *dir, char *notused) 
{ 
	//printf("sdm_initFinishHitachi\n");

	//-- Create the device hitachi device name string.
	sdm_createDevName(dir->st.name, dir->st.vendor, dir->st.sn, dir->v.hitachi.ldevId);

	//-- Create the real scsi id.
	dir->id   = sdm_initGetScsiId(dir->wwn, dir->v.hitachi.ldevId, dir->st.vendor);

	//-- Call SCSI application to add device.
	//-- We will then create the SDM data base record
	//-- when a successful reply is received from the SCSI app.
	sdm_sendScsiAddMsg(dir);

	return SDM_OK;
} // sdm_initFinishHitachi


//------------------------------------------------------------------------------
// Function Name:   sdm_initEMC
// 
// Description:     
//
// Data Created:    March 16, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initEMC(sdm_dir_t *dir, char *inqRsp)
{

#if 0	
	{
		int i;

		printf("sdm_initEMC: wwn 0x%llx lun %d\n", dir->wwn, dir->lun);
		for (i = 0; i < 128; i += 8) {
			printf("0x%02x: %02x %02x %02x %02x %02x %02x %02x %02x \n", 
				   i, 
				   inqRsp[i+0], inqRsp[i+1], inqRsp[i+2], inqRsp[i+3], 
				   inqRsp[i+4], inqRsp[i+5], inqRsp[i+6], inqRsp[i+7]);
		}
	}
#endif

	//-- Create scsi id
	SDM_INIT_ID_SET_WWN( dir->id, dir->wwn);
	SDM_INIT_ID_SET_LUN( dir->id, dir->lun);
	
	//-- Get RAID level from Inq response.
	sdm_procEMCRaidRsp(dir, inqRsp);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initHitachi

//------------------------------------------------------------------------------
// Function Name:   sdm_procEMCRaidRsp
// 
// Description:     
//
// Data Created:    April 30, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procEMCRaidRsp(sdm_dir_t *dir, char *inqRsp ) 
{ 
	dir->raidLevel = 3;			/* XXX hardcode to 3 for now */

	return SDM_OK;
} // sdm_procEMCRaidRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_initFinishEMC
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initFinishEMC(sdm_dir_t *dir, char *notused) 
{ 
	//printf("sdm_initFinishEMC\n");

	//-- Create the device EMC device name string.
	// sdm_createDevName(dir->name, dir->vendor, dir->sn, dir->v.hitachi.ldevId);
	sdm_createDevName(dir->st.name, dir->st.vendor, dir->st.sn, dir->lun);
	
	//-- Create the real scsi id.
	// dir->id   = sdm_initGetScsiId(dir->wwn, dir->v.hitachi.ldevId, dir->vendor);
	dir->id   = sdm_initGetScsiId(dir->wwn, dir->lun, dir->st.vendor);

	//-- Call SCSI application to add device.
	//-- We will then create the SDM data base record
	//-- when a successful reply is received from the SCSI app.
	sdm_sendScsiAddMsg(dir);

	return SDM_OK;
} // sdm_initFinishEMC


//------------------------------------------------------------------------------
// Function Name:   sdm_initMylex
// 
// Description:     
//
// Data Created:    March 26, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initMylex(sdm_dir_t *dir, char *inqRsp)
{
	//printf("sdm_initMylex\n");

	//-- Create scsi id
	dir->id = sdm_initGetScsiId(dir->wwn, dir->lun, dir->st.vendor);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initMylex

//------------------------------------------------------------------------------
// Function Name:   sdm_procMylexDACARMSnRsp
// 
// Description:     
//
// Data Created:    March 27, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procMylexDACARMSnRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	int i, j;
	uchar8 *buf;
	scsi_sense_t *sense;
	int fgoodRsp = 0;

	//printf("sdm_procMylexDACARMSnRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						//-- The Mylex device didn't like the request
						//-- This could be because this is a "pseudo" device.
						fgoodRsp = 0;
						break;
				}
			}
			else {
				//-- The Mylex device didn't like the request
				//-- This could be because this is a "pseudo" device.
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		//-- Get pointer to buffer containing the inquiry data.
		buf = i_msg->data;
		
		//-- Parse Unit Serial Number Page.
		memset(dir->st.sn, 0, sizeof(dir->st.sn));
		/* If LUN uninitialized, used the WWN instead */
		if (buf[8] == ' ' && buf[9] == ' ') {
			uint64_t wwn = dir->wwn;
			for (j = 0; j < 16; ++j) {
				if (j < 4)
					dir->st.sn[j] = 'X';
				else {
					dir->st.sn[j] = (wwn & 0xF000000000000000LL) >> 60;
					dir->st.sn[j] = (dir->st.sn[j] >= 0 && dir->st.sn[j] <= 9) ? dir->st.sn[j] + '0' : dir->st.sn[j] - 0xa + 'a';
				}
				wwn = wwn << 4;
			}
		}
		else {
			for(i=8, j=0; i < 16 && j < 32; i++,j+=2) {
				dir->st.sn[j] = (buf[i] >> 4) & 0x0F;
				dir->st.sn[j+1] = (buf[i] & 0x0F);
				dir->st.sn[j] = (dir->st.sn[j] >= 0 && dir->st.sn[j] <= 9) ? dir->st.sn[j] + '0' : dir->st.sn[j] - 0xa + 'a';
				dir->st.sn[j+1] = (dir->st.sn[j+1] >= 0 && dir->st.sn[j+1] <= 9) ? dir->st.sn[j+1] + '0' : dir->st.sn[j+1] - 0xa + 'a';
			}
		}

		//printf("SNLen: %d, SN: %s\n", slen, dir->sn);
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procMylexDACARMSnRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_procMylexFFx2SnRsp
// 
// Description:     
//
// Data Created:    March 27, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procMylexFFx2SnRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	int i, j;
	uint32 fgoodRsp = 0;
	uchar8 *buf;
	scsi_sense_t *sense;

	//printf("sdm_procMylexFFx2SnRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		//-- Get pointer to buffer containing the inquiry data.
		buf = i_msg->data;

		//-- Parse Unit Serial Number Page.
		memset(dir->st.sn, 0, sizeof(dir->st.sn));
		/* If LUN uninitialized, used the WWN instead */
		if (buf[12] == ' ' && buf[13] == ' ') {
			uint64_t wwn = dir->wwn;
			for (j = 0; j < 16; ++j) {
				if (j < 4)
					dir->st.sn[j] = 'X';
				else {
					dir->st.sn[j] = (wwn & 0xF000000000000000LL) >> 60;
					dir->st.sn[j] = (dir->st.sn[j] >= 0 && dir->st.sn[j] <= 9) ? dir->st.sn[j] + '0' : dir->st.sn[j] - 0xa + 'a';
				}
				wwn = wwn << 4;
			}
		}
		else {
			for (i = 12, j = 0; i < 28 && j < 32; i++, j++)
				dir->st.sn[j] = buf[i];
		}
		//printf("SNLen: %d, SN: %s\n", slen, dir->sn);
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procMylexFFx2SnRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_sendMylexRaidReq
// 
// Description:     Sends out FSI MDACIOCTL_GETLOGDEVINFOVALID IOCTL in order to
//                  acquire raid level of device.
//
// Data Created:    June 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------

int 
sdm_sendMylexRaidReq(sdm_dir_t *dir, void *notused)
{
	uint32 mlen, allocLen, destCpu, destSlot, tid; 
	scsi_relayReqMsg_t *scsi_o_msg;

	//printf("sdm_sendMylexRaidReq\n");

	//-- Allocate an SDM transaction Id
	tid = sdm_allocRelayTransaction( NULL,
									 NULL,
									 sdm_initDevProcMsg,
									 (int)dir);

	//-- Allocate memory for the message.
	mlen = (sizeof(scsi_relayReqMsg_t) - 1 + 16);
	scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
	memset(scsi_o_msg, 0, mlen);

	//-- Setup the FSI MDACIOCTL_GETLOGDEVINFOVALID IOCTL CDB.
	allocLen = sizeof(FSI_GetLogDevInfoValid_t);
	scsi_o_msg->cdb[0] = MDACMD_IOCTL;
	scsi_o_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
    scsi_o_msg->cdb[3] = (dir->lun & 0xFF00) >> 8;
    scsi_o_msg->cdb[4] = (dir->lun & 0x00FF);
    scsi_o_msg->cdb[7] = (allocLen & 0xFF00) >> 8;
    scsi_o_msg->cdb[8] = (allocLen & 0x00FF);

	//-- Setup the SCSI msg header.
	scsi_o_msg->hdr.mid     = SCSI_MID_RELAY_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = dir->path;
	scsi_o_msg->lun         = 0;
	scsi_o_msg->dataLen     = 0;
	scsi_o_msg->dataInLen   = allocLen;
	scsi_o_msg->retryCnt    = 1;
	scsi_o_msg->timeOutCnt  = 60;
	scsi_o_msg->maxTimeToLive = 60;

	//-- Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
	sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	free(scsi_o_msg);

	return SDM_OK;
}

//------------------------------------------------------------------------------
// Function Name:   sdm_procMylexRaidRsp
// 
// Description:     
//
// Data Created:    May 02, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procMylexRaidRsp(sdm_dir_t *dir, scsi_relayRspMsg_t * i_msg)
{ 
	uint32 fgoodRsp = 0;
	FSI_GetLogDevInfoValid_t *di;
	scsi_sense_t *sense;

	//printf("sdm_procMylexRaidRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NO_SENSE:
					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

				    case SCSI_SENSE_KEY_VENDOR_SPECIFIC:
					default:
						//-- The Mylex device didn't like the request
						//-- This could be because this is a "pseudo" device.
						fgoodRsp = 0;
						break;
				}

			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		//-- Get pointer to buffer containing Logical device info block
		di = (FSI_GetLogDevInfoValid_t *)i_msg->data;
		dir->raidLevel = di->RaidLevel;
		//printf("RAID level: %d\n", dir->raidLevel);
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procMylexRaidRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_initFinishMylex
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initFinishMylex(sdm_dir_t *dir, char *notused) 
{ 
	uint32 idHash_lo;	
	//printf("sdm_initFinishMylex\n");

	//-- Create the device hitachi device name string.
	sdm_createDevName(dir->st.name, dir->st.vendor, dir->st.sn, dir->lun);

	//-- Create the real scsi id.
	//dir->id = sdm_initGetScsiId(dir->wwn, dir->lun, dir->vendor);
	
	idHash_lo = crc32_checksum( dir->st.name, strlen(dir->st.name), 0 );
	dir->id = sdm_initGetScsiId(idHash_lo, dir->lun, dir->st.vendor);

	//-- Call SCSI application to add device.
	//-- We will then create the SDM data base record
	//-- when a successful reply is received from the SCSI app.
	sdm_sendScsiAddMsg(dir);

	return SDM_OK;
} // sdm_initFinishMylex

//------------------------------------------------------------------------------
// Function Name:   sdm_procProfibreSnRsp
// 
// Description:     
//
// Data Created:    May 3, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procProfibreSnRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	int i, j;
	uint32 fgoodRsp = 0;
	uchar8 *buf;
	scsi_sense_t *sense;

	//printf("sdm_procProfibreSnRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		//-- Get pointer to buffer containing the inquiry data.
		buf = i_msg->data;
		
		//-- Parse Unit Serial Number Page.
		memset(dir->st.sn, 0, sizeof(dir->st.sn));
		for(i=4, j=0; i < 24 && j < 32; i++,j++) {
			dir->st.sn[j] = buf[i];
		}
		//printf("SNLen: %d, SN: %s\n", slen, dir->sn);
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);	
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procProfibreSnRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_initDiskLsi
// 
// Description:     
//
// Data Created:    September 11, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initDiskLsi(sdm_dir_t *dir, char *inqRsp)
{
	//printf("sdm_initLsi\n");

	//-- Create scsi id
	dir->id = sdm_initGetScsiId(dir->wwn, dir->lun, dir->st.vendor);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initDiskLsi

//------------------------------------------------------------------------------
// Function Name:   sdm_sendLsiRaidReq
// 
// Description:     
//
// Data Created:    September 11, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_sendLsiRaidReq( sdm_dir_t *dir, void *notused ) 
{
	uint32 mlen, destCpu, destSlot, tid; 
	scsi_relayReqMsg_t *scsi_o_msg;

	//printf("sdm_sendLsiRaidReq\n");

	//-- Allocate an SDM transaction Id
	tid = sdm_allocRelayTransaction( NULL,
									 NULL,
									 sdm_initDevProcMsg,
									 (int)dir);

	//-- Allocate memory for the message.
	mlen = (sizeof(scsi_relayReqMsg_t) - 1 + 16);
	scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
	memset(scsi_o_msg, 0, mlen);

	//-- Setup the Report Lun CDB.
	scsi_o_msg->cdb[0] = SCSI_CMD_MODE_SENSE_6;
	scsi_o_msg->cdb[2] = 0x2B;	// Page Code
    scsi_o_msg->cdb[4] = 255;

	//-- Setup the SCSI msg header.
	scsi_o_msg->hdr.mid     = SCSI_MID_RELAY_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = dir->path;
	scsi_o_msg->lun         = dir->lun;
	scsi_o_msg->dataLen     = 0;

	//-- Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
	sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	free(scsi_o_msg);

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			("SDM_INIT: Get LSI Raid Level:   0x%016llx:%d", 
			 dir->wwn, dir->lun));
	return SDM_OK;
} // sdm_sendLsiRaidReq

//------------------------------------------------------------------------------
// Function Name:   sdm_procLsiRaidRsp
// 
// Description:     
//
// Data Created:    September 11, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procLsiRaidRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	int i;
	uint32 fgoodRsp = 0;
	uchar8 *buf;
	scsi_sense_t *sense;

	//printf("sdm_procLsiRaidRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		//-- Get pointer to buffer containing the inquiry data.
		buf = i_msg->data;
	
		i = 4;		   // -- Skip past the mode sense header
		i += buf[3];  //-- skip past the response block desc 

		//-- Extract raid level from byte 3 of page data.
		dir->raidLevel = buf[i+3];

		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Raid Rsp, Ok:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);	

		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Raid Rsp, Err:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procLsiRaidRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_initFinishLsi
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initFinishLsi(sdm_dir_t *dir, char *notused) 
{ 
	uint32 idHash_hi;
	uint32 idHash_lo;

	//printf("sdm_initFinishLsi\n");

	//-- Create the lsi device name string.

	//printf("Array: %s, ", dir->v.lsi.arrayUsrLabel);
	//printf("Volume: %s\n", dir->v.lsi.volUsrLabel);
	//printf("Array Id: %s\n", dir->v.lsi.arrayId_str);

	strcat(dir->st.name, dir->st.vendor);
	strcat(dir->st.name,"_");
	if( strlen(dir->v.lsi.arrayUsrLabel) > 0 )
		strcat(dir->st.name,dir->v.lsi.arrayUsrLabel); 
	else
		strcat(dir->st.name, dir->v.lsi.arrayId_str);
	strcat(dir->st.name,"_");
	if( strlen(dir->v.lsi.volUsrLabel) > 0 )
		strcat(dir->st.name,dir->v.lsi.volUsrLabel); 
	else
		strcat(dir->st.name, dir->v.lsi.lunid_str);

	//-- Create the real scsi id.
	idHash_lo = crc32_checksum( dir->st.name, strlen(dir->st.name), 0 );
	idHash_hi = crc32_checksum( dir->v.lsi.lunid_str, strlen(dir->v.lsi.lunid_str), 0 );
	dir->id = sdm_initGetScsiId(idHash_lo, idHash_hi, dir->st.vendor);

	//-- Call SCSI application to add device.
	//-- We will then create the SDM data base record
	//-- when a successful reply is received from the SCSI app.
	sdm_sendScsiAddMsg(dir);

	return SDM_OK;
} // sdm_initFinishLsi

//------------------------------------------------------------------------------
// Function Name:   sdm_sendLsiLunidReq
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_sendLsiLunidReq( sdm_dir_t *dir, void *notused ) 
{
	uint32 mlen, destCpu, destSlot, tid; 
	scsi_relayReqMsg_t *scsi_o_msg;

	//printf("sdm_sendLsiLunidReq\n");

	//-- Allocate an SDM transaction Id
	tid = sdm_allocRelayTransaction( NULL,
									 NULL,
									 sdm_initDevProcMsg,
									 (int)dir);

	//-- Allocate memory for the message.
	mlen = (sizeof(scsi_relayReqMsg_t) - 1 + 16);
	scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
	memset(scsi_o_msg, 0, mlen);

	//-- Setup the Report Lun CDB.
	scsi_o_msg->cdb[0] = 0x12;  // SCSI_CMD_INQUIRY
	scsi_o_msg->cdb[1] = 0x1;	// Set EVPD Bit.
	scsi_o_msg->cdb[2] = 0xC8;	// Request Extended Device Id Page
    scsi_o_msg->cdb[4] = 255;

	//-- Setup the SCSI msg header.
	scsi_o_msg->hdr.mid     = SCSI_MID_RELAY_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = dir->path;
	scsi_o_msg->lun         = dir->lun;
	scsi_o_msg->dataLen     = 0;

	//-- Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
	sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	free(scsi_o_msg);

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			("SDM_INIT: Get Lun Id Page:   0x%016llx:%d", 
			 dir->wwn, dir->lun));
	return SDM_OK;
} // sdm_sendLsiLunidReq

//------------------------------------------------------------------------------
// Function Name:   sdm_procLsiLunidRsp
// 
// Description:     
//
// Data Created:    September 11, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procLsiLunidRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	int i, j;
	uint32 fgoodRsp = 0;
	scsi_sense_t *sense;
	sdm_lsiExtDevIdPg_t *devId;

	//printf("sdm_procLsiLunidRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {

		//-- Get pointer to buffer containing the data.
		devId = (sdm_lsiExtDevIdPg_t *)&i_msg->data;
		
		//-- Extract dev id string.
		memcpy(dir->v.lsi.lunid, &(devId->volId[0]), SDM_DIE_ID_LEN);

		//-- Create readable dev id string.
		for(i=0,j=0; i < SDM_DIE_ID_LEN; i++, j+=2)
			sprintf(&(dir->v.lsi.lunid_str[j]),"%02x", dir->v.lsi.lunid[i] & 0xFF);

		//-- Extract user volume label string.
		for(i=0, j=0; i < 0x3C; i++) {
			//printf("%c", devId->volUsrLabel[i]);
			if(devId->volUsrLabel[i] != '\0')
            {
                if (devId->volUsrLabel[i] != ' ')
				    dir->v.lsi.volUsrLabel[j++] = devId->volUsrLabel[i];
                else
				    dir->v.lsi.volUsrLabel[j++] = '-';
            }
		}

		//-- Extract user array label string.
		for(i=0, j=0; i < 0x3C; i++) {
			//printf("%c", devId->arrayUsrLabel[i]);
			if(devId->arrayUsrLabel[i] != '\0')
            {
                if (devId->arrayUsrLabel[i] != ' ')
				    dir->v.lsi.arrayUsrLabel[j++] = devId->arrayUsrLabel[i];
                else
				    dir->v.lsi.arrayUsrLabel[j++] = '-';
            }
		}

		//-- Extract array id string.
		memcpy(dir->v.lsi.arrayId, &(devId->arrayId[0]), SDM_DIE_ID_LEN);

		//-- Create readable array id string.
		for(i=0,j=0; i < SDM_DIE_ID_LEN; i++, j+=2)
			sprintf(&(dir->v.lsi.arrayId_str[j]),"%02x", dir->v.lsi.arrayId[i] & 0xFF);

		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Id Rsp, Ok:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);	
		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Id Rsp, Err:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procLsiLunidRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_sendLsiAccessReq
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_sendLsiAccessReq( sdm_dir_t *dir, void *notused ) 
{
	uint32 mlen, destCpu, destSlot, tid; 
	scsi_relayReqMsg_t *scsi_o_msg;

	//printf("sdm_sendLsiAccessReq\n");

	//-- Allocate an SDM transaction Id
	tid = sdm_allocRelayTransaction( NULL,
									 NULL,
									 sdm_initDevProcMsg,
									 (int)dir);

	//-- Allocate memory for the message.
	mlen = (sizeof(scsi_relayReqMsg_t) - 1 + 16);
	scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
	memset(scsi_o_msg, 0, mlen);

	//-- Setup the Report Lun CDB.
	scsi_o_msg->cdb[0] = 0x12;  // SCSI_CMD_INQUIRY
	scsi_o_msg->cdb[1] = 0x1;	// Set EVPD Bit.
	scsi_o_msg->cdb[2] = 0xC9;	// Request Volume Access Control VPD Page
    scsi_o_msg->cdb[4] = 255;

	//-- Setup the SCSI msg header.
	scsi_o_msg->hdr.mid     = SCSI_MID_RELAY_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = dir->path;
	scsi_o_msg->lun         = dir->lun;
	scsi_o_msg->dataLen     = 0;

	//-- Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
	sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	free(scsi_o_msg);

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			("SDM_INIT: Get Access Control Page:   0x%016llx:%d", 
			 dir->wwn, dir->lun));
	return SDM_OK;
} // sdm_sendLsiAccessReq

//------------------------------------------------------------------------------
// Function Name:   sdm_procLsiAccessRsp
// 
// Description:     
//
// Data Created:    September 11, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procLsiAccessRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	uint32 fgoodRsp = 0;
	scsi_sense_t *sense;
	//sdm_lsiDevIdPg_t *devId;

	//printf("sdm_procLsiAccessRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		uint32 cvp;
		//-- Get pointer to buffer containing the data.
		cvp = i_msg->data[8];
		
		//-- See if the bit is set to mark this controller
		//-- as the current volume path.
		if( !(cvp & 1) ) {

			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM_INIT: Proc LSI Access Control Rsp, No Access:   0x%016llx:%d", 
					dir->wwn, dir->lun));

			free(dir);		//-- Ignore the path for now. Later on 
			                //-- we will mark this as the secondary path.
			return SDM_OK;
		}
		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Access Control Rsp, Ok:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);	
		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Access Control Rsp, Err:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}


	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procLsiAccessRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_sendLsiIsSnapReq
// 
// Description:     
//
// Data Created:    March 18, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_sendLsiIsSnapReq( sdm_dir_t *dir, void *notused ) 
{
	uint32 mlen, destCpu, destSlot, tid; 
	scsi_relayReqMsg_t *scsi_o_msg;

	//printf("sdm_sendLsiIsSnapReq\n");

	//-- Allocate an SDM transaction Id
	tid = sdm_allocRelayTransaction( NULL,
									 NULL,
									 sdm_initDevProcMsg,
									 (int)dir);

	//-- Allocate memory for the message.
	mlen = (sizeof(scsi_relayReqMsg_t) - 1 + 16);
	scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
	memset(scsi_o_msg, 0, mlen);

	//-- Setup the Report Lun CDB.
	scsi_o_msg->cdb[0] = 0x12;  // SCSI_CMD_INQUIRY
	scsi_o_msg->cdb[1] = 0x1;	// Set EVPD Bit.
	scsi_o_msg->cdb[2] = 0xCA;	// Request Snapshot Base Id VPD Page
    scsi_o_msg->cdb[4] = 255;

	//-- Setup the SCSI msg header.
	scsi_o_msg->hdr.mid     = SCSI_MID_RELAY_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = dir->path;
	scsi_o_msg->lun         = dir->lun;
	scsi_o_msg->dataLen     = 0;

	//-- Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
	sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	free(scsi_o_msg);

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			("SDM_INIT: Get Is Snap Page:   0x%016llx:%d", 
			 dir->wwn, dir->lun));
	return SDM_OK;
} // sdm_sendLsiIsSnapReq

//------------------------------------------------------------------------------
// Function Name:   sdm_procLsiIsSnapRsp
// 
// Description:     
//
// Data Created:    September 11, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procLsiIsSnapRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{ 
	uint32 fgoodRsp = 0;
	scsi_sense_t *sense;

	//printf("sdm_procIsSnapRsp\n");

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		uint32 sn;
		//-- Get pointer to buffer containing the data.
		sn = i_msg->data[8];
		
		//-- Ignore this volume if it is a snapshot volume.
		if( (sn & 1) ) {
			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM_INIT: Proc LSI Is Snap Rsp, isSnapVol:   0x%016llx:%d", 
					dir->wwn, dir->lun));
			free(dir);		
			return SDM_OK;
		}

		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Is Snap Rsp, Ok:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);	
		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Is Snap Rsp, Err:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}


	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procLsiIsSnapRsp


//------------------------------------------------------------------------------
// Function Name:   sdm_sendLsiRHwReq
// 
// Description:     Sends req. for hardware version page
//
// Data Created:    Nov 6, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_sendLsiRHwReq(sdm_dir_t *dir, void *notused ) 
{
	uint32 mlen, destCpu, destSlot, tid; 
	scsi_relayReqMsg_t *scsi_o_msg;

	//-- Allocate an SDM transaction Id
	tid = sdm_allocRelayTransaction( NULL,
									 NULL,
									 sdm_initDevProcMsg,
									 (int)dir);

	//-- Allocate memory for the message.
	mlen = (sizeof(scsi_relayReqMsg_t) - 1 + 16);
	scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
	memset(scsi_o_msg, 0, mlen);

	//-- Setup the Report Lun CDB.
	scsi_o_msg->cdb[0] = 0x12;  // SCSI_CMD_INQUIRY
	scsi_o_msg->cdb[1] = 0x1;	// Set EVPD Bit.
	scsi_o_msg->cdb[2] = 0xC0;	// Request Hardware version page
    scsi_o_msg->cdb[4] = 255;

	//-- Setup the SCSI msg header.
	scsi_o_msg->hdr.mid     = SCSI_MID_RELAY_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = dir->path;
	scsi_o_msg->lun         = dir->lun;
	scsi_o_msg->dataLen     = 0;

	//-- Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
	sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	free(scsi_o_msg);

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			("SDM_INIT: Get Hardware Version Page:   0x%016llx:%d", 
			 dir->wwn, dir->lun));
	return SDM_OK;
} // sdm_sendLsiRHwReq


//------------------------------------------------------------------------------
// Function Name:   sdm_procLsiRHwRsp
// 
// Description:     Processes response to hardware version page request
//
// Data Created:    Nov 6, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_procLsiRHwRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg ) 
{
	uint32 fgoodRsp = 0;
	scsi_sense_t *sense;

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
			fgoodRsp = 1;
			break;

		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
		default:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
						fgoodRsp = 1;
						break;

					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
						//-- Fatal Error unable to access device 

					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						// Device does not support this command

					case SCSI_SENSE_KEY_NO_SENSE:
					default:
						fgoodRsp = 0;
						break;
				}
			}
			else {
				fgoodRsp = 0;
			}
	}

	if( fgoodRsp ) {
		char controllerStr[5];
		//-- Get pointer to buffer containing the data.
		memcpy((void *)controllerStr, &i_msg->data[154], 4);
		controllerStr[4] = 0;
		
		sdm_setLsiModelNum(dir, controllerStr);

		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Hardware Version Rsp, Ok:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}
	else {
		dir->control |= (SDM_DIE_CTRL_EVM_NO_NOTIFY | SDM_DIE_CTRL_INIT_ERROR);	
		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM_INIT: Proc LSI Hardware Version Rsp, Err:   0x%016llx:%d", 
				dir->wwn, dir->lun));
	}

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL); 

	return SDM_OK;
} // sdm_procLsiRHwRsp


//------------------------------------------------------------------------------
// Function Name:   sdm_setLsiModelNum
// 
// Description:   	Sets the LSI controller number  
//
// Data Created:    Nov 6, 2002
//
// Returns:         SDM_OK
//-------------------------------------------------------------------------------
int
sdm_setLsiModelNum(sdm_dir_t *dir, char *controllerStr)
{
	int controllerNum = atoi(controllerStr);

	switch(controllerNum) {

		case 2772:
				strcpy(dir->st.model, "E2400");
				break;
		case 4774:
				strcpy(dir->st.model, "E4400");
				break;
		case 4884:
				strcpy(dir->st.model, "E4600");
				break;
		default:
				controllerNum /= 1000;
				if(controllerNum)
					sprintf(dir->st.model, "E%dXXX", controllerNum);
				else
					strcpy(dir->st.model, "Unknown");
				break;
	}

	return SDM_OK;
} // sdm_setLsiModelNum
