/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: env.c,v 1.11 2002/11/21 01:48:40 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/18/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>
#include "seep.h"

#undef DEBUG_PRINT
#ifdef DEBUG_PRINT
#define DPRINT(_x)      printf _x
#else
#define DPRINT(_x)
#endif

void env_view_basic(uchar8 * key, uchar8 * val, uchar8 ** values);
int32 env_str_to_val_basic(int32 key, uchar8 * str,
                                        uchar8 ** values, uchar8 * val);
void env_val_to_str_basic(int32 key, uchar8 val,
                                        uchar8 ** values, uchar8 * str);
int32 env_str_to_val_loadopts(int32 key, uchar8 * str,
                                        uchar8 ** values, uchar8 * val);
void env_val_to_str_loadopts(int32 key, uchar8 val,
                                        uchar8 ** values, uchar8 * str);


seepenv_entry_t default_seepenv[NUM_ENV_VARS] = {
	{"autoload",  "on",  {"off", "on", 0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
	{"autoreboot","on",  {"off", "on", 0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
	{"load_src",  "disk", {"disk", "network", 0},	env_view_basic,
													env_str_to_val_basic,
													env_val_to_str_basic},
	{"load_opts", "mAt", {"m/s|A/a|d|t", 0},	env_view_basic,
												env_str_to_val_loadopts,
												env_val_to_str_loadopts},
	{"boot_dev",  "wd0a", {"wd0a", "wd1a", "sc1", 0},	env_view_basic,
												env_str_to_val_basic,
												env_val_to_str_basic},
	{"upgrade_prom", "yes",{"no", "yes", 0},		env_view_basic,
												env_str_to_val_basic,
												env_val_to_str_basic},
	{"env_reset", "no", {"no", "yes", 0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
#if defined(R9000)
	{"autolinkenable", "3",  {"0", "1", "2", "3", 0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
#else
    {"autolinkenable", "on",  {"off", "on", 0}, env_view_basic,
                                            env_str_to_val_basic,
                                            env_val_to_str_basic},
#endif
	{"hw_qt", "off",  {"off", "on",0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
	{"hw_ft", "off",  {"off", "on",0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
	{"hw_diag", "off",  {"off", "on",0},	env_view_basic,
											env_str_to_val_basic,
											env_val_to_str_basic},
	{NULL,	NULL, {0}, NULL, NULL, NULL},
};

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
int32
seepenv_is_valid()
{
    uchar8			val;

    NFX_SPIN_LOCK(&wait_seep_lock);
#ifdef SIBYTE_BC	
	seepRead(SEEP_PCB_LOCAL, &val, SEEP_ENV_START+ENV_KEY_RESET_ENV, 1);
#else	
	seepRead(SEEP_PCB, &val, SEEP_ENV_START+ENV_KEY_RESET_ENV, 1);
#endif	
    NFX_SPIN_UNLOCK(&wait_seep_lock);
	if (val == 0)
		return 1;
	else
		return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/17/02
 *
 *-----------------------------------------------------------------
 */
void
seepenv_init()
{
	if (!seepenv_is_valid())
		seepenv_reset();
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/18/02
 *
 *-----------------------------------------------------------------
 */
int32
seepenv_get(
	int32	key,
	uchar8	* str
)
{
	seepenv_entry_t	* entry;
	uchar8			val;
	int32			i;

    DPRINT((">seepenv_get: key=%d\n", key));
	if (key >= NUM_ENV_VARS)
		return -1;

    NFX_SPIN_LOCK(&wait_seep_lock);
#ifdef SIBYTE_BC	
    seepRead(SEEP_PCB_LOCAL, &val, SEEP_ENV_START+key, 1);
#else	
    seepRead(SEEP_PCB, &val, SEEP_ENV_START+key, 1);
#endif	
    NFX_SPIN_UNLOCK(&wait_seep_lock);

    entry = default_seepenv;
    for (i=0; i<key; i++, entry++);

    (*entry->val_to_str_func)(key, val, entry->values, str);

    DPRINT(("<seepenv_get: val=%d str=%s\n", val, str));
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/18/02
 *
 *-----------------------------------------------------------------
 */
int32
seepenv_set(
	int32	key,
	uchar8	* str
)
{
	seepenv_entry_t	* entry;
	uchar8			val;
	int32			i;

    DPRINT((">seepenv_set: key=%d str=%s\n", key, str));
	if (key >= NUM_ENV_VARS)
		return -1;

    entry = default_seepenv;
    for (i=0; i<key; i++, entry++);

    if ((*entry->str_to_val_func)(key, str, entry->values, &val) != NFX_OK) {
        printf("Error setting env variable, key = %d, str = %s\n", key, str);
        return -1;
    }

    NFX_SPIN_LOCK(&wait_seep_lock);
#ifdef SIBYTE_BC	
    seepWrite(SEEP_PCB_LOCAL, &val, SEEP_ENV_START+key, 1);
#else	
    seepWrite(SEEP_PCB, &val, SEEP_ENV_START+key, 1);
#endif	
    NFX_SPIN_UNLOCK(&wait_seep_lock);

    DPRINT(("<seepenv_set: val=%d\n", val));
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/18/02
 *
 *-----------------------------------------------------------------
 */
void
seepenv_show()
{
	seepenv_entry_t	* entry;
	int32			key;
	uchar8			str[ENV_MAX_STR_LEN];

	printf("\nCurrent Environment Settings:\n");

	for (key = 0, entry = default_seepenv; entry->key; entry++, key++) {

		bzero(str, ENV_MAX_STR_LEN);
		seepenv_get(key, str);

		(*entry->view_func)(entry->key, str, entry->values);
	}

	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/18/02
 *
 *-----------------------------------------------------------------
 */
void
seepenv_reset()
{
	seepenv_entry_t	* entry;
	int32			key;
	uchar8			val=0;
	int32			i=0;

	printf("\nResetting Default Environment Settings\n\n");
	for (key = 0, entry = default_seepenv; entry->key; entry++, key++)
		seepenv_set(key, entry->val);

	/* Initialise the rest of the env variables to 0 */	
#if defined (R9000) || defined (SIBYTE_BC)	
	for(i=SEEP_ENV_START+key;i<SEEP_NVRAM_START;i++) 
#else		
	for(i=SEEP_ENV_START+key;i<SEEP_CHASSIS_START;i++) 
#endif		
	{
	    NFX_SPIN_LOCK(&wait_seep_lock);
#ifdef SIBYTE_BC		
	    seepWrite(SEEP_PCB_LOCAL, &val, i, 1);
#else		
	    seepWrite(SEEP_PCB, &val, i, 1);
#endif		
	    NFX_SPIN_UNLOCK(&wait_seep_lock);
	}	

	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/17/02
 *
 *-----------------------------------------------------------------
 */
void
env_view_basic(
	uchar8	* key,
	uchar8	* val,
	uchar8	** values
)
{
	uchar8	* s;
	int32	i;

	printf("\t%-12s = %-8s   ( ", key, val);
	for (i=0;;i++) {
		s = values[i];
		if (s == NULL)
			break;
		printf("%s ", s);
	}
	printf(")\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
int32
env_str_to_val_basic(
	int32	key,
	uchar8	* str,
	uchar8	** values,
	uchar8	* val
)
{
	seepenv_entry_t	* entry;
	int32			i;

	if (key >= NUM_ENV_VARS)
		return -1;

	entry = default_seepenv;
	for (i=0; i<key; i++, entry++);

	for (i=0;;i++) {
		if (values[i] == 0)
			break;
		if (!strcmp(str, values[i])) {
			*val = i;
			return NFX_OK;
		}
	}

	return -1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
void
env_val_to_str_basic(
	int32	key,
	uchar8	val,
	uchar8	** values,
	uchar8	* str
)
{
	uchar8		i;

    /* find the max possible values for entry */
	for (i=0; values[i] != 0;i++);
    if (val >= i) {
        printf("bogus value in seepenv %d forcing to 0\n", val);
        val = 0;        // force val to 0
    }
	strcpy(str, values[val]);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
int32
env_str_to_val_loadopts(
	int32	key,
	uchar8	* str,
	uchar8	** values,
	uchar8	* val
)
{
	*val = 0;
	if (strchr(str, 'm')) {
		if (strchr(str, 's')) {
			printf("Invalid value, cannot set both 'm' and 's'\n");
			return -1;
		}
		*val |= ENV_VAL_LOAD_OPTS_MULTI;
	}
	if (strchr(str, 'A')) {
		if (strchr(str, 'a')) {
			printf("Invalid value, cannot set both 'A' and 'a'\n");
			return -1;
		}
		*val |= ENV_VAL_LOAD_OPTS_DONT_ASK;
	}
	if (strchr(str, 'd'))
		*val |= ENV_VAL_LOAD_OPTS_DEFAULT;
	if (strchr(str, 't'))
		*val |= ENV_VAL_LOAD_OPTS_CONSOLE;

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
void
env_val_to_str_loadopts(
	int32	key,
	uchar8	val,
	uchar8	** values,
	uchar8	* str
)
{
	str[0] = '\0';
	if (val & ENV_VAL_LOAD_OPTS_MULTI)
		strcat(str, "m");
	else
		strcat(str, "s");
	if (val & ENV_VAL_LOAD_OPTS_DONT_ASK)
		strcat(str, "A");
	else
		strcat(str, "a");
	if (val & ENV_VAL_LOAD_OPTS_DEFAULT)
		strcat(str, "d");
	if (val & ENV_VAL_LOAD_OPTS_CONSOLE)
		strcat(str, "t");

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
int32
seepenv_get_key(
	uchar8	* str,
	int32	* key
)
{
	seepenv_entry_t	* entry;
	int32			i;

	entry = default_seepenv;
	i = 0;
	while (entry->key) {
		if (!strcmp(entry->key, str)) {
			*key = i;
			return NFX_OK;
		}
		entry++;
		i++;
	}

	return -1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
int32
check_env_autoreboot_flag()
{
	uchar8	str[ENV_MAX_STR_LEN];

	if (seepenv_get(ENV_KEY_AUTOREBOOT, str) == NFX_OK) {
		if (!strcmp(str, "off"))
			return 0;
	}

	return 1;
}


/* The following 3 functions have been implemented to jump directly to the
 * diag partition in the big flash if the hw_qt/hw_ft/hw_diag env fields
 * are set to "on".
 */

/*-----------------------------------------------------------------
 * Name: get_hw_qt_info
 *
 * Description:
 *
 * Created by:	lipika
 *
 * Date Created:	4/15/03
 *
 *-----------------------------------------------------------------
 */
int32
get_hw_qt_info()
{
	uchar8	str[ENV_MAX_STR_LEN];
	uint32 ret=0;

	if (seepenv_get(ENV_KEY_HW_QT_OPTION, str) == NFX_OK) {
		if (!strcmp(str, "on"))
			ret = 1;
		else 
			ret = 0;
	}

	return(ret);
}	

/*-----------------------------------------------------------------
 * Name: get_hw_ft_info
 *
 * Description:
 *
 * Created by:	lipika
 *
 * Date Created:	4/15/03
 *
 *-----------------------------------------------------------------
 */
int32
get_hw_ft_info()
{
	uchar8	str[ENV_MAX_STR_LEN];
	uint32 ret=0;

	if (seepenv_get(ENV_KEY_HW_FT_OPTION, str) == NFX_OK) {
		if (!strcmp(str, "on"))
			ret = 1;
		else 
			ret = 0;
	}

	return(ret);
}	

/*-----------------------------------------------------------------
 * Name: get_hw_diag_info
 *
 * Description:
 *
 * Created by:	lipika
 *
 * Date Created:	4/15/03
 *
 *-----------------------------------------------------------------
 */
int32
get_hw_diag_info()
{
	uchar8	str[ENV_MAX_STR_LEN];
	uint32 ret=0;

	if (seepenv_get(ENV_KEY_HW_DIAG_OPTION, str) == NFX_OK) {
		if (!strcmp(str, "on"))
			ret = 1;
		else 
			ret = 0;
	}

	return(ret);
}	

