/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: ibutton-api.c,v 1.4 2002/12/10 23:08:07 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hw-api.h>
#include <exports/bsp-api.h>
#include <exports/ibutton-api.h>
#include <exports/timer-api.h>
#include <exports/seep-api.h>
#include <sys/bmfpga.h>
#include "seep.h"
#include "../sm-chassis/cm.h"


void seep_startTimer(int32 timer);
int32 system_uptime_timer;
void chassis_timeout(int32 tmr_handle, void * usr_param0, void * usr_param1, void * usr_param2, void * usr_param3);

#define IBUTTON_TIMER_FREQ	60*MSEC_PER_SEC /* Polling freq for noting system state */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
static ibutton_t *
ibutton_base(
	uint32	which_ibutton
)
{
	if (which_ibutton == IBUTTON_BACKPLANE) {
	    if(isSSCmgmt())	
		return (ibutton_t *)(HW_BMFPGA_BASE + IBUTTON_BACKPLANE_OFFSET);
	    else
		return (ibutton_t *)PHYS_TO_K1(HW_BMFPGA_BASE + IBUTTON_BACKPLANE_OFFSET);
	}	
	if (which_ibutton == IBUTTON_PS0) {
	    if(isSSCmgmt())	
		return (ibutton_t *)(HW_BMFPGA_BASE + IBUTTON_PS0_OFFSET);
	    else
		return (ibutton_t *)PHYS_TO_K1(HW_BMFPGA_BASE + IBUTTON_PS0_OFFSET);
	}	
	if (which_ibutton == IBUTTON_PS1) {
	    if(isSSCmgmt())	
		return (ibutton_t *)(HW_BMFPGA_BASE + IBUTTON_PS1_OFFSET);
	    else
		return (ibutton_t *)PHYS_TO_K1(HW_BMFPGA_BASE + IBUTTON_PS1_OFFSET);
	}
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */

uint32
ibutton_detect(
	uint32	which_ibutton
)
{
	volatile ibutton_t	* ib = (volatile ibutton_t *)ibutton_base(which_ibutton);
	uint32	intr;

//printf("    reset/detect\n");
#if 0
/* old boards don't support ibutton anyway */
	if (get_busclock() == 100)
		ib->clock_divisor = IBUTTON_CLOCK_DIVISOR_CLOCK_ENABLE |
							IBUTTON_CLOCK_DIVISOR_DIV2 |
							IBUTTON_CLOCK_DIVISOR_PRE0;
	else
#endif
		ib->clock_divisor = IBUTTON_CLOCK_DIVISOR_CLOCK_ENABLE |
							IBUTTON_CLOCK_DIVISOR_DIV1 |
							IBUTTON_CLOCK_DIVISOR_DIV0 |
							IBUTTON_CLOCK_DIVISOR_PRE1 |
							IBUTTON_CLOCK_DIVISOR_PRE0;
	ib->command = IBUTTON_COMMAND_RESET;

	delay_us(2000);		/* wait for 2 ms */

	intr = ib->intr;
	return (intr == 0xd);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
uchar8
ibutton_rx_byte(
	uint32	which_ibutton
)
{
	volatile ibutton_t	* ib = (volatile ibutton_t *)ibutton_base(which_ibutton);
	uchar8	val;

	while (!(ib->intr & IBUTTON_INTR_TRANSMIT_BUF_EMPTY));

	ib->buffer = 0xFF;

	while (!(ib->intr & IBUTTON_INTR_RX_BUF_FULL));

	val = ib->buffer;

//printf("    rx byte: 0x%x\n", val);

	return val;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
void
ibutton_tx_byte(
	uint32	which_ibutton,
	uchar8	data
)
{
	volatile ibutton_t	* ib = (volatile ibutton_t *)ibutton_base(which_ibutton);
	uchar8	val;

//printf("    tx byte: 0x%x\n", data);

	while (!(ib->intr & IBUTTON_INTR_TRANSMIT_BUF_EMPTY));

	ib->buffer = data;

	while (!(ib->intr & IBUTTON_INTR_RX_BUF_FULL));
	
	val = ib->buffer;

	if (val != data)
		printf("ibutton_tx_byte echo error, wrote 0x%x, read 0x%x\n", data, val);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
uchar8
ibutton_read(
	uint32	which_ibutton,
	uint32	offset
)
{
	uint32	offset_upper = offset >> 8;
	uint32	offset_lower = offset & 0xFF;
	uchar8	val;

	ibutton_detect(which_ibutton);

	ibutton_tx_byte(which_ibutton, 0xCC);
	ibutton_tx_byte(which_ibutton, 0xF0);
	ibutton_tx_byte(which_ibutton, offset_lower);
	ibutton_tx_byte(which_ibutton, offset_upper);
	val = ibutton_rx_byte(which_ibutton);

	ibutton_detect(which_ibutton);

//printf("ibutton_read, offset = %d, val = %d\n", offset, val);

	return (val);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
void
ibutton_write(
	uint32	which_ibutton,
	uint32	offset,
	uchar8	data
)
{
	uint32	offset_upper = offset >> 8;
	uint32	offset_lower = offset & 0xFF;
	uchar8	off_low, off_up, auth;
//printf("ibutton_write, offset = %d, data = %d\n", offset, data);
	ibutton_detect(which_ibutton);

	ibutton_tx_byte(which_ibutton, 0xCC);
	ibutton_tx_byte(which_ibutton, 0x0F);
	ibutton_tx_byte(which_ibutton, offset_lower);
	ibutton_tx_byte(which_ibutton, offset_upper);
	ibutton_tx_byte(which_ibutton, data);

	ibutton_detect(which_ibutton);

	ibutton_tx_byte(which_ibutton, 0xCC);
	ibutton_tx_byte(which_ibutton, 0xAA);
	off_low = ibutton_rx_byte(which_ibutton);
	off_up = ibutton_rx_byte(which_ibutton);
	auth = ibutton_rx_byte(which_ibutton);

	ibutton_detect(which_ibutton);

	ibutton_tx_byte(which_ibutton, 0xCC);
	ibutton_tx_byte(which_ibutton, 0x55);
	ibutton_tx_byte(which_ibutton, offset_lower);
	ibutton_tx_byte(which_ibutton, offset_upper);
	ibutton_tx_byte(which_ibutton, auth);

	delay_us(5000);		/* wait 5 ms */
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
void
ibutton_read_rom(
	uint32	which_ibutton,
	uchar8	* rom		/* needs to be array of 8 chars */
)
{
	uint32	i;

	ibutton_detect(which_ibutton);
	ibutton_tx_byte(which_ibutton, 0x33);
	for (i=0; i<8; i++)
		rom[i] = ibutton_rx_byte(which_ibutton);

	ibutton_detect(which_ibutton);
}

/*-----------------------------------------------------------------
 * Name:	ibutton
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
int32
ibutton(
	uint32	op,
	uint32	which_ibutton,
	uint32	offset,
	uint32	len,
	uchar8	* buf
)
{
	uint32	i;
#ifndef SSC_MGMT
	uchar8	save[512];
	uint32	err = 0;
#endif

	switch (op) {
		case IBUTTON_READ:
			for (i=0; i<len; i++)
				buf[i] = ibutton_read(which_ibutton, offset + i);

			break;

		case IBUTTON_WRITE:
			for (i=0; i<len; i++)
				ibutton_write(which_ibutton, offset+i, buf[i]);

			break;

		case IBUTTON_DETECT:
			return (ibutton_detect(which_ibutton));

		case IBUTTON_READ_ROM:
			ibutton_read_rom(which_ibutton, buf);

			break;

		case IBUTTON_TEST:
#ifndef SSC_MGMT
			srandom(_mips_mfc0(C0_COUNT));
			for (i=0; i<512; i++) {
				buf[i] = random() & 0xFF;
			}
			printf("save current state ... ");
			for (i=0; i<512; i++)
				save[i] = ibutton_read(which_ibutton, i);
			printf("write random data ... ");
			for (i=0; i<512; i++)
				ibutton_write(which_ibutton, i, buf[i]);
			printf("verify ... ");
			for (i=0; i<512; i++) {
				uchar8	val;
				val = ibutton_read(which_ibutton, i);
				if (val != buf[i])
					printf("ibutton test error, wrote 0x%x, read 0x%x\n",
																buf[i], val);
			}
			printf("restore ... ");
			for (i=0; i<512; i++)
				ibutton_write(which_ibutton, i, save[i]);
			printf("verify ... ");
			for (i=0; i<512; i++) {
				uchar8	val;
				val = ibutton_read(which_ibutton, i);
				if (val != save[i]) {
					printf("ibutton restore error, wrote 0x%x, read 0x%x\n",
																save[i], val);
					err = 1;
				}
			}

			if (!err)
				printf("\n\nibutton test passed\n");
			else
				printf("\n\nIBUTTON TEST FAILED\n");

#else
			printf("skipped on SSC_MGMT\n");
#endif
			break;

		default:
			printf("bad ibutton op, %d\n", op);
	}

	return 0;

}	/* ibutton() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/02
 *
 *-----------------------------------------------------------------
 */
int
do_ibutton(
	int		argc,
	char	* argv[]
)
{
	uint32	offset;
	uint32	i, len;
	uchar8	buf[512], data;
	uint32	which_ibutton;

	if (strncmp(argv[2], "backplane", 1) == 0)
		which_ibutton = IBUTTON_BACKPLANE;
	else if (strncmp(argv[2], "ps0", 3) == 0)
		which_ibutton = IBUTTON_PS0;
	else if (strncmp(argv[2], "ps1", 3) == 0)
		which_ibutton = IBUTTON_PS1;
	else {
		printf("ibutton choices : backplane, ps0, ps1\n");
		return 0;
	}

	if (strncmp(argv[1], "readrom", 7) == 0) {
		ibutton(IBUTTON_READ_ROM, which_ibutton, 0, 0, buf);
		printf("rom[0] = 0x%08x\n", buf[0]);
		printf("rom[1] = 0x%08x\n", buf[1]);
		printf("rom[2] = 0x%08x\n", buf[2]);
		printf("rom[3] = 0x%08x\n", buf[3]);
		printf("rom[4] = 0x%08x\n", buf[4]);
		printf("rom[5] = 0x%08x\n", buf[5]);
		printf("rom[6] = 0x%08x\n", buf[6]);
		printf("rom[7] = 0x%08x\n", buf[7]);
	}
	else if (strncmp(argv[1], "read", 4) == 0) {
		if (argc != 5)
			return -1;

		offset = atox(argv[3]);
		len = atox(argv[4]);

		if (offset + len > 512) {
			printf("%d + %d > 512\n", offset, len);
			return 1;
		}

		ibutton(IBUTTON_READ, which_ibutton, offset, len, buf);

		for (i=0; i<len; i++)
			printf("%d: read data 0x%x\n", i, buf[i]);
	}
	else if (strncmp(argv[1], "write", 5) == 0) {
		if (argc < 6)
			return -1;

		offset = atox(argv[3]);
		len = atox(argv[4]);
		data = atox(argv[5]);

		if (offset + len > 512) {
			printf("%d + %d > 512\n", offset, len);
			return 1;
		}

		for (i=0; i<len; i++)
			buf[i] = data;

		ibutton(IBUTTON_WRITE, which_ibutton, offset, len, buf);
	}
	else if (strncmp(argv[1], "detect", 5) == 0) {
		if (ibutton(IBUTTON_DETECT, which_ibutton, 0, 0, buf))
			printf("%s ibutton is present\n", argv[2]);
		else
			printf("%s ibutton is not present\n", argv[2]);
	}
	else if (strncmp(argv[1], "test", 4) == 0) {
		ibutton(IBUTTON_TEST, which_ibutton, 0, 0, buf);
	}
	else
		return -1;

	return 0;

}

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:	chassis_trackReboots()
 *
 * Description: Increments the counters on the chassis seep.
 *              On the SSC_MGMT.
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
chassis_trackReboots(uint32 slot)
{
	uint32 noFPreboots=0;
	uint32 noFCreboots=0;
	
	if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0)) {
	    if(slot == NFX_SLOTTYPE_NFP){
       	    seepRead(SEEP_CHASSIS, (uchar8 *)&noFPreboots, 
		    SEEP_CHASSIS_FPREBOOT_OFFSET, SEEP_CHASSIS_REBOOTNUM_LEN-1);
            noFPreboots ++;
	        seepWrite(SEEP_CHASSIS, (uchar8 *)&noFPreboots, 
	 	              SEEP_CHASSIS_FPREBOOT_OFFSET, SEEP_CHASSIS_REBOOTNUM_LEN-1);
	    }
	    if(slot == NFX_SLOTTYPE_FC){
           	seepRead(SEEP_CHASSIS, (uchar8 *)&noFCreboots, 
		    SEEP_CHASSIS_FPREBOOT_OFFSET+2, SEEP_CHASSIS_REBOOTNUM_LEN-1);
		    noFCreboots ++;
            seepWrite(SEEP_CHASSIS, (uchar8 *)&noFCreboots, 
		    		SEEP_CHASSIS_FPREBOOT_OFFSET+2, SEEP_CHASSIS_REBOOTNUM_LEN-1);
	    }
	}	
}		

/*-----------------------------------------------------------------
 * Name:	chassis_trackCpuState()
 *
 * Description: Tracks the cpu state.
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
uint32
chassis_trackCpuState()
{
	uint32 i=0,j=0;
	uint32 allUp=0,fcUp=0, txrxUp=0, fpUp=0;
	
	/* We declare it a system if atleast both txrx, both fp and 1 fc comes up.
	 * If system is up, then start the system uptime timer
	 */

	for (i=0; i<CM_NUM_SLOTS; i++) {
		cm_slot_t *cm_slot = &cm.slot[i+1];
		if(cm_slot->present) {
			for (j=0; j<cm_slot->num_cpus; j++) {
				cm_cpu_t *cpu = &cm_slot->cpu[j];
				if (cpu->state == CM_SLICE_STATE_RUNTIME) { 
					if ((i+1) == NFX_SLOTTYPE_NFP) {
						if((j==0) || (j==1))
							txrxUp++;
						if((j==2) || (j==3))
							fpUp++;
					}		
					if ((i+1) == NFX_SLOTTYPE_FC)
						fcUp ++;
				}		
			}	
		}		
	}	
        /* at least one FC should be working */
	if((txrxUp == 2) && (fpUp == 2) && (fcUp))
            allUp = txrxUp + fpUp + fcUp;

	return(allUp);
}

/*-----------------------------------------------------------------
 * Name:	chassis_trackSystemTime()
 *
 * Description: pass an argument to differentiate between restart
 *              of the timer and the first instance that timer is started
 *              On SSC-MGMT.
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
chassis_trackSystemTime(uint32 i)
{
	static uint32 systemUptimeClock_started = 0;
	uint32 allUp=0;
	uint32 rct=0;
	
	// Is there any change in the CPU states ?
	allUp = chassis_trackCpuState();

	// If the function is called from a timeout, then check if all cpus are up
	// to decide if we should restart timer.
	// If function is called from sm-chassis/cm-intr.c,then it is a cpu UP/DOWN event that
	// is triggering the timer, so check if we have already satisfied the minimum requiremt
	// and if we have already started the timer.
	// We should not start multiple instances of the timer with UP/DOWN events.
	
	if(i) { // Timer called from timeout : restart timer
		// check to see if we have already started the timer 
		// check cpu state
		if((allUp) && (systemUptimeClock_started)) {
			rct = eee_timerStart(system_uptime_timer, (void *)chassis_timeout,0,0,0,0);
			if(rct != NFX_OK)
				printf("Could not start SYSTEM_UPTIME_TIMER %d Error %08x \n",system_uptime_timer, rct);
			}	
		else if((allUp == 0) && (systemUptimeClock_started)) {
			// Stop the timer because a cpu has gone down
			rct = eee_timerStop(system_uptime_timer);
			if(rct)
				printf("chassis_trackSystemTime: failed to stop system_uptime_timer.\n");
			systemUptimeClock_started = 0;
		}		
	}	
	else {
		// this would be the first time we start the timer initiated from cpu up/down event
		if((allUp) && (!systemUptimeClock_started)) {
			 // check against starting multiple instances of timer 
			rct = eee_timerStart(system_uptime_timer, (void *)chassis_timeout,0,0,0,0);
			if(rct != NFX_OK)
				printf("Could not start SYSTEM_UPTIME_TIMER %d Error %08x \n",system_uptime_timer, rct);
			systemUptimeClock_started = 1;
		}	
		else if((allUp == 0) && (systemUptimeClock_started)) {
			// Stop the timer because a cpu has gone down
			rct = eee_timerStop(system_uptime_timer);
			if(rct)
				printf("chassis_trackSystemTime: failed to stop system_uptime_timer.\n");
			systemUptimeClock_started = 0;
		}		
	}	
}

/*-----------------------------------------------------------------
 * Name:	systemUptimeTimer()
 *
 * Description: Creates the timers that keep track of system uptime
 *              SSC-MGMT.
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
systemUptimeTimer()
{
	uint32 systemUptime=0;
	uint32 cumSystemUptime=0;

	/* Create the timer for the first time that board is powered up */
	system_uptime_timer = eee_timerCreate(IBUTTON_TIMER_FREQ, EEE_TIMER_ONCE); //5 min timer
	if(system_uptime_timer == TIMER_ERR_NO_TIMER_AVAILABLE)
		printf("ERROR: Could not create system uptime timer for this card\n");
	
	/* The PCB uptime is cleared to 0 if a graceful shutdown is done
	 * and the cumulative uptime incremented.
	 * If a brutal system down is carried out, then the cumulative system uptime
	 * on the chassis seep needs to be incremented by the PCB seep value
	 * and then cleared to 0.
	 */ 
        NFX_SPIN_LOCK(&wait_seep_lock);
        seepRead(SEEP_PCB, (uchar8 *)&systemUptime, 0x7b, SEEP_CHASSIS_UPTIME_LEN-1); //system uptime
        NFX_SPIN_UNLOCK(&wait_seep_lock);

		if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0)) {
            if(systemUptime) {
                seepRead(SEEP_CHASSIS, (uchar8 *)&cumSystemUptime, 
				          SEEP_CHASSIS_CUMSYSUPTIME_OFFSET, SEEP_CHASSIS_UPTIME_LEN-1);
                cumSystemUptime += systemUptime;
                seepWrite(SEEP_CHASSIS, (uchar8 *)&cumSystemUptime, 
				          SEEP_CHASSIS_CUMSYSUPTIME_OFFSET, SEEP_CHASSIS_UPTIME_LEN-1);
            }		
		    systemUptime = 0;
            seepWrite(SEEP_PCB, (uchar8 *)&systemUptime, 0x7b, SEEP_CHASSIS_UPTIME_LEN-1);
		}	
}

/*-----------------------------------------------------------------
 * Name:	chassis_timeout()
 *
 * Description: generic timeout function for SSC_MGMT
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
chassis_timeout(int32 tmr_handle,
         void * usr_param0,
         void * usr_param1,
         void * usr_param2,
         void * usr_param3)
{
	uint32 systemUptime=0;

       	NFX_SPIN_LOCK(&wait_seep_lock);
        seepRead(SEEP_PCB, (uchar8 *)&systemUptime, 0x7b, SEEP_CHASSIS_UPTIME_LEN-1); //system uptime
        NFX_SPIN_UNLOCK(&wait_seep_lock);

	/* Increment the timer field and restart timer */
	if(tmr_handle == system_uptime_timer){
	    systemUptime += IBUTTON_TIMER_FREQ/(60*MSEC_PER_SEC);  // minute timer
        NFX_SPIN_LOCK(&wait_seep_lock);
        seepWrite(SEEP_PCB, (uchar8 *)&systemUptime, 0x7b, SEEP_CHASSIS_UPTIME_LEN-1);
        NFX_SPIN_UNLOCK(&wait_seep_lock);
	}		

	chassis_trackSystemTime(1); /* Indicates who calls the trackSystemTime function */
}
#endif
