/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	seep-ui.c
 *
 * RCS:  $Id: seep-ui.c,v 1.25 2002/12/10 23:08:08 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	11/28/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/cm-api.h>
#include <exports/ibutton-api.h>
#ifndef SSC_MGMT
#include <exports/i2c-api.h>
#endif
#include <exports/seep-api.h>
#include <exports/hw-api.h>
#include "seep.h"

#ifdef NFX_SMP
extern nfx_spinlock_t wait_seep_lock;
#endif
#ifdef SSC_MGMT
extern int32 i2c(
    uint32  op,
    uchar8  i2c_addr,
    uchar8  byte_addr,
    uint32  len,
    uchar8  * buf
);
#endif

/*-----------------------------------------------------------------
 * Name:	do_seep_show()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	11/28/01
 *
 *-----------------------------------------------------------------
 */
void
do_seep_show(
	int32	which_seep
)
{
		volatile uint32 cumUptime_mins = 0;
		volatile uint32 boardUptime_mins = 0;
		volatile uint32 years=0, days=0, hours=0, mins=0;
		volatile uint32 noReboots=0;
		uint32 i=0;

#ifdef SIBYTE_BC		
	if ((which_seep == SEEP_PCB) || (which_seep == SEEP_PCB_LOCAL))
#else			
	if (which_seep == SEEP_PCB)
#endif			
	{
		if (isTXRX() || isSSC() || isFC() || isFP()) {
			seepPCB_t	* seep = (seepPCB_t *)seep_struct[SEEP_PCB];
#ifdef SIBYTE_BC			
			if (which_seep == SEEP_PCB_LOCAL)
				   	seep = (seepPCB_t *)seep_struct[SEEP_PCB_LOCAL];
#endif
			/*read in seep info struct from seep*/
			NFX_SPIN_LOCK(&wait_seep_lock);
			seepRead(which_seep, (void *)seep, 0, sizeof(seepPCB_t));
			NFX_SPIN_UNLOCK(&wait_seep_lock);

			for(i=0;i<SEEP_PCB_UPTIME_LEN-1;i++) {
				cumUptime_mins |= ((seep->cumulative_uptime[i] & 0xff) << 8*i);
				boardUptime_mins |= seep->uptime_since_reboot[i] & 0xff << 8*i;
			}

			printf("PCB SEEP Info (sig=%s)\n", seep->signature);
			printf("\tModel Number:          %s\n", seep->model_num);
#if defined(R9000) || defined(SIBYTE_BC)
			printf("\tSystem Serial Number:  %s\n", seep->serial_num);
			printf("\tSystem Revision:       %s\n", seep->revision);
			printf("\tBoard Serial Number:   %s\n", seep->board_serial_num);
			printf("\tBoard Revision:        %s\n", seep->board_revision);
#else
			printf("\tAssembly Number:       %s\n", seep->assembly_num);
			printf("\tSerial Number:         %s\n", seep->serial_num);
			printf("\tRevision:              %s\n", seep->revision);
#endif
			printf("\tDeviation:             %s\n", seep->deviation);
			printf("\tResult of ICT:         %s\n", seep->result_ict);
			printf("\tDate of ICT:           %s\n", seep->date_ict);
#if !defined(R9000) && !defined(SIBYTE_BC)
			printf("\tResult of QT:          %s\n", seep->result_qt);
#endif
			printf("\tResult of FT:          %s\n", seep->result_ft);
#if !defined(R9000) && !defined(SIBYTE_BC)
			printf("\tResult of DVT:         %s\n", seep->result_dvt);
			printf("\tResult of ESS:         %s\n", seep->result_ess);
			printf("\tDate of ESS:           %s\n", seep->date_ess);
#endif
			printf("\tIP addr:               %s\n", seep->ip_addr);
			printf("\tIP mask:               %s\n", seep->ip_mask);
#ifdef R9000			
			printf("\tMAC addr:              %s\n", seep->mac_addr);
            {
                printf("\tTFTP server:           %d.%d.%d.%d\n", 
                       seep->uptime_since_reboot[0],
                       seep->uptime_since_reboot[1],
                       seep->uptime_since_reboot[2],
                       seep->uptime_since_reboot[3]);
            }
#endif
			/* Display the minutes stored in the cumulative uptime in years/days/hours/mins */
			years = cumUptime_mins/(60*24*365);	
			days = (cumUptime_mins/(60*24)) - years*365;
			hours = (cumUptime_mins/60) - days*24 - years*24*365;
			mins = cumUptime_mins - hours*60 - days*60*24 - years*60*24*365;
			if(years)
			printf("\tCumulative Uptime:     %d Years %d Days %d Hours %d Mins\n",
							years, days, hours, mins);
			else
			printf("\tCumulative Uptime:     %d Days %d Hours %d Mins\n",
							days, hours, mins);

#ifndef R9000
			/* Display the minutes stored in the board uptime in years/ days/hours/mins */
			years = boardUptime_mins/(60*24*365);	
			days = (boardUptime_mins/(60*24)) - years*365;
			hours = (boardUptime_mins/60) - days*24 - years*24*365;
			mins = boardUptime_mins - hours*60 - days*60*24 - years*60*24*365;
			if(years)
			printf("\tBoard Uptime:          %d Years %d Days %d Hours %d Mins\n",
							years, days, hours, mins);
			else
			printf("\tBoard Uptime:          %d Days %d Hours %d Mins\n",
							days, hours, mins);
#endif

			/* Convert the no of reboots into num */
			for(i=0;i<SEEP_PCB_REBOOTNUM_LEN-1;i++) {
				noReboots |= seep->no_of_reboots[i] & 0xff << 8*i;
			}
			printf("\tNo of Reboots:         %d\n", noReboots);
		}
		else {
			printf("PCB SEEP not available from this CPU\n");
		}
	}
#if !defined (R9000) && !defined (SIBYTE_BC)	
	else if (which_seep == SEEP_CHASSIS) {
		if ((isSSC()) || (isSSCmgmt())) {

			seepPCB_t	* seep_pcb = (seepPCB_t *)seep_struct[SEEP_PCB];
			seepChassis_t	* seep; 
		    volatile uint32 noFPReboots=0, noFCReboots=0;

			/* To support old boards which do not have ibuttons and are
			 * confined to a smaller space for chassis seep
			 */
			if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0)) {
			    seep = (seepChassis_t *)seep_struct[SEEP_CHASSIS];
			    memset((void *)seep,0,sizeof(seepChassis_t));
			}	
			else {
			    (seepOldChassis_t *)seep = (seepOldChassis_t *)seep_struct[SEEP_CHASSIS];
			    memset((void *)seep,0,sizeof(seepOldChassis_t));
			}
			memset((void *)seep_pcb,0,sizeof(seepPCB_t));

			if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0)) {
				ibutton(IBUTTON_READ, IBUTTON_BACKPLANE, 0, sizeof(seepChassis_t),(void *)seep);
			}	
			else {
			    seepRead(SEEP_CHASSIS, (void *)seep, 0, sizeof(seepOldChassis_t));
			}		

			NFX_SPIN_INIT(&wait_seep_lock, SPINLOCK_LEVEL_UNKNOWN);

			NFX_SPIN_LOCK(&wait_seep_lock);
			seepRead(SEEP_PCB, (void *)seep_pcb, 0, sizeof(seepPCB_t));
			NFX_SPIN_UNLOCK(&wait_seep_lock);

			printf("Chassis SEEP Info (sig=%s)\n", seep->signature);
			printf("\tSystem Model Number:       %s\n", seep->system_model_num);
			printf("\tChassis Model Number:      %s\n", seep->chassis_model_num);
			printf("\tSerial Number:      %s\n", seep->serial_num);
			printf("\tRevision:           %s\n", seep->revision);
			printf("\tDeviation:          %s\n", seep->deviation);
			printf("\tMAC addr:           %s\n", seep->mac_addr);

			    /* Convert the no of reboots into num */
			for(i=0;i<SEEP_CHASSIS_REBOOTNUM_LEN-1;i++)
			   	noReboots |= seep_pcb->no_of_reboots[i] & 0xff << 8*i;
			printf("\tSSC reboots:        %d\n", noReboots);

			if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0))
			{
			    for(i=0;i<SEEP_CHASSIS_REBOOTNUM_LEN-1;i++)
			    	noFPReboots |= seep->no_of_FP_reboots[i] & 0xff << 8*i;
			    printf("\tFP reboots:         %d\n", noFPReboots);
				
			    for(i=0;i<SEEP_CHASSIS_REBOOTNUM_LEN-1;i++)
			    	noFCReboots |= seep->no_of_FC_reboots[i] & 0xff << 8*i;
			    printf("\tFC reboots:         %d\n", noFCReboots);

			    for(i=0;i<SEEP_CHASSIS_UPTIME_LEN-1;i++)
			    	boardUptime_mins |= seep_pcb->system_uptime[i] & 0xff << 8*i;
			    for(i=0;i<SEEP_CHASSIS_UPTIME_LEN-1;i++)
			    	cumUptime_mins |= ((seep->cumulative_system_uptime[i] & 0xff) << 8*i);

			    cumUptime_mins += boardUptime_mins; 
			    /* Display the minutes stored in the cumulative uptime in years/days/hours/mins */
			    years = cumUptime_mins/(60*24*365);	
			    days = (cumUptime_mins/(60*24)) - years*365;
			    hours = (cumUptime_mins/60) - days*24 - years*24*365;
			    mins = cumUptime_mins - hours*60 - days*60*24 - years*60*24*365;
			    if(years)
			    	printf("\tCumulative System Uptime:  %d Years %d Days %d Hours %d Mins\n",
			    					years, days, hours, mins);
			    else
			    	printf("\tCumulative System Uptime:  %d Days %d Hours %d Mins\n",
			    				days, hours, mins);

			    years = boardUptime_mins/(60*24*365);	
			    days = (boardUptime_mins/(60*24)) - years*365;
			    hours = (boardUptime_mins/60) - days*24 - years*24*365;
			    mins = boardUptime_mins - hours*60 - days*60*24 - years*60*24*365;
			    if(years)
			    	printf("\tSystem Uptime:  %d Years %d Days %d Hours %d Mins\n",
			    				years, days, hours, mins);
			    else
			    	printf("\tSystem Uptime:  %d Days %d Hours %d Mins\n",
			    				days, hours, mins);
			}	
		}
		else {
			printf("Chassis SEEP only available from SSC\n");
		}
	}
	else if (which_seep == SEEP_PS0) {
		if (isSSC()) {
			seepPowerSupply_t	* seep = (seepPowerSupply_t *)seep_struct[SEEP_PS0];

			printf("PS0 SEEP Info (sig=%s)\n", seep->signature);
			printf("\tModel Number:       %s\n", seep->model_num);
			printf("\tSerial Number:      %s\n", seep->serial_num);
			printf("\tRevision:           %s\n", seep->revision);
			printf("\tDeviation:          %s\n", seep->deviation);
		}
		else {
			printf("PS0 SEEP only available from SSC\n");
		}
	}
	else if (which_seep == SEEP_PS1) {
		if (isSSC()) {
			seepPowerSupply_t	* seep = (seepPowerSupply_t *)seep_struct[SEEP_PS1];

			printf("PS1 SEEP Info (sig=%s)\n", seep->signature);
			printf("\tModel Number:       %s\n", seep->model_num);
			printf("\tSerial Number:      %s\n", seep->serial_num);
			printf("\tRevision:           %s\n", seep->revision);
			printf("\tDeviation:          %s\n", seep->deviation);
		}
		else {
			printf("PS1 SEEP only available from SSC\n");
		}
	}
#endif

}	/* do_seep_show() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
void
do_seep_erase(
	int32	which_seep
)
{
	uchar8	seep[SEEP_TOTAL_LEN];

#ifdef SIBYTE_BC	
	if ((which_seep == SEEP_PCB) || (which_seep == SEEP_PCB_LOCAL))
#else			
	if (which_seep == SEEP_PCB) 
#endif			
	{
		if (isTXRX() || isSSC() || isFC() || isFP()) {
			bzero(seep, SEEP_TOTAL_LEN);

			NFX_SPIN_LOCK(&wait_seep_lock);
			seepWrite(which_seep, seep, 0, SEEP_TOTAL_LEN);
			seepRead(which_seep, seep_struct[which_seep], 0, SEEP_TOTAL_LEN);
			NFX_SPIN_UNLOCK(&wait_seep_lock);
		}
		else
			printf("PCB SEEP not available from this CPU\n");
	}
	else
		printf("Invalid SEEP_ERASE command\n");

}	/* do_seep_erase() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_model(
	uchar8	* item
)
{
	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	if (strlen(item) >= SEEP_CHASSIS_MODEL_LEN) {
		printf("invalid length: %s\n",item);
		return 0;
	}

	if (strlen(item) > 0)
		return 1;

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_assm(
	uchar8	* item
)
{
	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	if (strlen(item) != SEEP_PCB_ASSEMBLY_LEN-1) {
		printf("invalid length: %s\n",item);
		return 0;
	}

#if 0
	if ((isdigit(item[0])) &&
		(isdigit(item[1])) &&
		(isdigit(item[2])) &&
		(item[3] == '-') &&
		(isdigit(item[4])) &&
		(isdigit(item[5])) &&
		(isdigit(item[6])) &&
		(isdigit(item[7])) &&
		(item[8] == '-') &&
		(isdigit(item[9])) &&
		(isdigit(item[10])) &&
		(isdigit(item[11])) &&
		(isdigit(item[12])))
		return 1;
#else
	return 1;
#endif

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_serial(
	uchar8	* item
)
{
	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	if (strlen(item) != SEEP_PCB_SERIAL_LEN-1) {
		printf("invalid length: %s\n",item);
		return 0;
	}

#if 0
	if ((isdigit(item[0])) &&
		(isdigit(item[1])) &&
		(isdigit(item[2])) &&
		(isdigit(item[3])) &&
		(isdigit(item[4])) &&
		(isdigit(item[5])) &&
		(isdigit(item[6])) &&
		(isdigit(item[7])) &&
		(isdigit(item[8])) &&
		(isdigit(item[9])))
		return 1;
#else
	return 1;
#endif

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_rev(
	uchar8	* item,
    int maxlen
)
{
	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	if (strlen(item) >= maxlen) {
		printf("invalid length: %s\n",item);
		return 0;
	}

#if 0
	if ((isdigit(item[0])) &&
		(isdigit(item[1])) &&
		(item[2] == '.') &&
		(isdigit(item[3])) &&
		(isdigit(item[4])))
		return 1;

	if ((item[0] == 'A') &&
		(isdigit(item[1])) &&
		(isdigit(item[2])) &&
		(item[3] == '\0'))
		return 1;
#else
	return 1;
#endif

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_dev(
	uchar8	* item
)
{
	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	if (strlen(item) >= SEEP_PCB_DEVIATION_LEN) {
		printf("invalid length: %s\n",item);
		return 0;
	}

#if 0
	if ((item[0] == 'D') && 
		(isdigit(item[1])) &&
		(isdigit(item[2])) &&
		(isdigit(item[3])))
		return 1;
#else
	return 1;
#endif

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_ip(
	uchar8	* item
)
{
	uchar8	* p;

	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	p = item;

	if (strlen(item) >= SEEP_PCB_IP_ADDR_LEN) {
		printf("invalid length: %s\n",item);
		return 0;
	}

	if ((p = (uchar8 *)strchr(p, '.')))
		if ((p = (uchar8 *)strchr(p+1, '.')))
			if ((p = (uchar8 *)strchr(p+1, '.')))
				return 1;

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Tony
 *
 * Date Created:	5/22/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_misc_info(uchar8 *item)
{
	uchar8 *p;

	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	p = item;

	if (strlen(item) >= SEEP_PCB_MISC_INFO_LEN) {
		printf("invalid length: %s\n", item);
		return 0;
	}
	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Lipika
 *
 * Date Created:	3/03/03
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_diagbits_info(uchar8 *item)
{
	uchar8 *p;

	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	p = item;
	//item[0] -= '0';
	if (strlen(item) >= SEEP_PCB_DIAG_TEST_LEN) {
		printf("invalid length: %s\n", item);
		return 0;
	}
	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Lipika
 *
 * Date Created:	3/03/03
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_test_result_info(uchar8 *item)
{
	uchar8 *p;

	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	p = item;

	if (strlen(item) >= SEEP_PCB_DIAG_TEST_LEN) {
		printf("invalid length: %s\n", item);
		return 0;
	}
	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Lipika
 *
 * Date Created:	3/03/03
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_date(uchar8 *item)
{
	uchar8 *p;

	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	p = item;

	if (strlen(item) >= SEEP_PCB_DATE_LEN) {
		printf("invalid length: %s\n", item);
		return 0;
	}
	return 1;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/3/02
 *
 *-----------------------------------------------------------------
 */
int32
seep_get_mac_addr(
	uchar8	* item
)
{
	bzero(item, SEEP_MAX_STRING_LEN);

	get_line(item, 0);

	if (strlen(item) == 0)
		return 0;

	if (strlen(item) >= SEEP_CHASSIS_MAC_ADDR_LEN) {
		printf("invalid length: %s\n",item);
		return 0;
	}

	if ((item[0] == '0') && (item[1] == '0') &&
		(item[2] == '0') && (item[3] == '7') &&
		(item[4] == '3') && (item[5] == '4') &&
		(item[10] == '0') && (item[11] == '0'))
		return 1;

	printf("Invalid format\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:	do_seep_program()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	11/28/01
 *
 *-----------------------------------------------------------------
 */
void
do_seep_program(
	int32	which_seep
)
{
	uchar8 item[SEEP_MAX_STRING_LEN];

#ifdef SIBYTE_BC	
	if ((which_seep == SEEP_PCB) || (which_seep == SEEP_PCB_LOCAL)) 
#else			
	if (which_seep == SEEP_PCB)
#endif			
	{
		seepPCB_t	* seep = (seepPCB_t *)seep_struct[SEEP_PCB];
#ifdef SIBYTE_BC
		/* For Bobcat, the TXRX reads the Seep info from the Board seep
		 * via memory mapped GT registers.
		 * Thus we need to differentiate between the Board seep (SEEP_PCB)
		 * and the local seep (SEEP_PCB_LOCAL). SEEP_PCB is not valid on the
		 * FP, and SEEP_PCB_LOCAL is same as SEEP_PCB on the SSC/FC.  
		 */ 
	    if((which_seep == SEEP_PCB) && isFP()) {
			which_seep = SEEP_PCB_LOCAL;	
		    seep = (seepPCB_t *)seep_struct[SEEP_PCB_LOCAL];
		}	
		else if (which_seep == SEEP_PCB_LOCAL)
		    seep = (seepPCB_t *)seep_struct[SEEP_PCB_LOCAL];
#endif	


		if (isTXRX() || isSSC() || isFC() || isFP()) 
		{
			do_seep_show(which_seep);

			printf("\n========= Enter New Fields For PCB SEEP =========\n");

			printf("\tModel Number [%s]: ", seep->model_num);
			if (seep_get_model(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->model_num, item, SEEP_CHASSIS_MODEL_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#if !defined(R9000) && !defined(SIBYTE_BC)
			printf("\tAssembly Number (NNN-NNNN-NNNN) [%s]: ",
														seep->assembly_num);
			if (seep_get_assm(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->assembly_num, item, SEEP_PCB_ASSEMBLY_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#endif
#if defined(R9000) || defined(SIBYTE_BC)
			printf("\tSystem Serial Number (NNNNNNNNNN) [%s]: ", seep->serial_num);
			if (seep_get_serial(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->serial_num, item, SEEP_CHASSIS_SERIAL_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tSystem Revision Number (NN.NN or ANN) [%s]: ", seep->revision);
			if (seep_get_rev(item, SEEP_CHASSIS_REVISION_LEN) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->revision, item, SEEP_CHASSIS_REVISION_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tBoard Serial Number (NNNNNNNNNN) [%s]: ", seep->board_serial_num);
			if (seep_get_serial(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->board_serial_num, item, SEEP_PCB_SERIAL_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tBoard Revision Number (NN.NN or ANN) [%s]: ", seep->board_revision);
			if (seep_get_rev(item, SEEP_PCB_REVISION_LEN) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->board_revision, item, SEEP_PCB_REVISION_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#else
			printf("\tSerial Number (NNNNNNNNNN) [%s]: ", seep->serial_num);
			if (seep_get_serial(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->serial_num, item, SEEP_PCB_SERIAL_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tRevision Number (NN.NN or ANN) [%s]: ", seep->revision);
			if (seep_get_rev(item, SEEP_PCB_REVISION_LEN) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->revision, item, SEEP_PCB_REVISION_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#endif
			printf("\tDeviation Number (DNNN) [%s]: ", seep->deviation);
			if (seep_get_dev(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->deviation, item, SEEP_PCB_DEVIATION_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tResult for ICT[%s]: ", seep->result_ict);
			if (seep_get_test_result_info(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->result_ict, item, SEEP_PCB_DIAG_TEST_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tDate of ICT Test[%s]: ", seep->date_ict);
			if (seep_get_date(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->date_ict, item, SEEP_PCB_DATE_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#if !defined(R9000) && !defined(SIBYTE_BC)
			printf("\tResult for Quick Test[%s]: ", seep->result_qt);
			if (seep_get_test_result_info(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->result_qt, item, SEEP_PCB_DIAG_TEST_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#endif
			printf("\tResult for Functional Test[%s]: ", seep->result_ft);
			if (seep_get_test_result_info(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->result_ft, item, SEEP_PCB_DIAG_TEST_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#if !defined(R9000) && !defined(SIBYTE_BC)
			printf("\tResult for DVT[%s]: ", seep->result_dvt);
			if (seep_get_test_result_info(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->result_dvt, item, SEEP_PCB_DIAG_TEST_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tResult for ESS[%s]: ", seep->result_ess);
			if (seep_get_test_result_info(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->result_ess, item, SEEP_PCB_DIAG_TEST_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tDate of ESS Test[%s]: ", seep->date_ess);
			if (seep_get_date(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->date_ess, item, SEEP_PCB_DATE_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#endif
			printf("\tIP addr [%s]: ", seep->ip_addr);
			if (seep_get_ip(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->ip_addr, item, SEEP_PCB_IP_ADDR_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
			printf("\tIP mask [%s]: ", seep->ip_mask);
			if (seep_get_ip(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->ip_mask, item, SEEP_PCB_IP_MASK_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
#ifdef R9000			
			printf("\tMac Address (000734xxxx00) [%s]: ", seep->mac_addr);
			if (seep_get_mac_addr(item) == 1)
				strncpy(seep->mac_addr, item, SEEP_CHASSIS_MAC_ADDR_LEN);
            printf("\tTFTP addr [%d.%d.%d.%d]: ", 
                   seep->uptime_since_reboot[0],
                   seep->uptime_since_reboot[1],
                   seep->uptime_since_reboot[2],
                   seep->uptime_since_reboot[3]);
            if (seep_get_ip(item) == 1) {
                int i1, i2, i3, i4;
                sscanf(item, "%d.%d.%d.%d", &i1, &i2, &i3, &i4);
                seep->uptime_since_reboot[0] = i1;
                seep->uptime_since_reboot[1] = i2;
                seep->uptime_since_reboot[2] = i3;
                seep->uptime_since_reboot[3] = i4;
            }
#endif
			NFX_SPIN_LOCK(&wait_seep_lock);			
			strcpy(seep->signature,SEEP_SIG_PCB);
			seepWrite(which_seep, seep_struct[which_seep], 0, sizeof(seepPCB_t));
			NFX_SPIN_UNLOCK(&wait_seep_lock);			
		}
		else
			printf("PCB SEEP not available from this CPU\n");
	}
#if !defined (SIBYTE_BC) && !defined (R9000)	
	else if (which_seep == SEEP_CHASSIS) {
		if (isSSC()) {
			seepChassis_t	* seep = (seepChassis_t *)seep_struct[SEEP_CHASSIS];

			do_seep_show(which_seep);

			printf("\n========= Enter New Fields For Chassis SEEP =========\n");

			printf("\tSystem Model Number [%s]: ", seep->system_model_num);
			if (seep_get_model(item) == 1)
				strncpy(seep->system_model_num, item, SEEP_CHASSIS_MODEL_LEN);

			printf("\tChassis Model Number [%s]: ", seep->chassis_model_num);
			if (seep_get_model(item) == 1)
				strncpy(seep->chassis_model_num, item, SEEP_CHASSIS_MODEL_LEN);

			printf("\tSerial Number (NNNNNNNNNN) [%s]: ", seep->serial_num);
			if (seep_get_serial(item) == 1)
				strncpy(seep->serial_num, item, SEEP_CHASSIS_SERIAL_LEN);

			printf("\tRevision Number (NN.NN or ANN) [%s]: ", seep->revision);
			if (seep_get_rev(item, SEEP_CHASSIS_REVISION_LEN) == 1)
				strncpy(seep->revision, item, SEEP_CHASSIS_REVISION_LEN);

			printf("\tDeviation Number (DNNN) [%s]: ", seep->deviation);
			if (seep_get_dev(item) == 1)
				strncpy(seep->deviation, item, SEEP_CHASSIS_DEVIATION_LEN);

			printf("\tMac Address (000734xxxx00) [%s]: ", seep->mac_addr);
			if (seep_get_mac_addr(item) == 1)
				strncpy(seep->mac_addr, item, SEEP_CHASSIS_MAC_ADDR_LEN);

			strcpy(seep->signature,SEEP_SIG_CHASSIS);

			if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0))
				seepWrite(which_seep, seep_struct[SEEP_CHASSIS],
													0, sizeof(seepChassis_t));
			else
				seepWrite(which_seep, seep_struct[SEEP_CHASSIS],
									SEEP_CHASSIS_START, sizeof(seepChassis_t));

			do_seep_show(which_seep);
		}
		else
			printf("Chassis SEEP only available from SSC\n");
	}
#endif

}	/* do_seep_program() */

/*-----------------------------------------------------------------
 * Name:	do_seep_program_one_item()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	03/25/02
 *
 *-----------------------------------------------------------------
 */
void
do_seep_program_one_item(uchar8 * category)
{

		uchar8 item[SEEP_MAX_STRING_LEN];
		uint32	write_to_seep = 1;

		seepPCB_t	* seep = (seepPCB_t *)seep_struct[SEEP_PCB];
		if (!strcmp(category, "model")) {
			printf("\tModel Number [%s]: ", seep->model_num);
			if (seep_get_model(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->model_num, item, SEEP_CHASSIS_MODEL_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}
#if !defined(R9000) && !defined(SIBYTE_BC)
		else if (!strcmp(category, "assembly")) { 
			printf("\tAssembly Number (NNN-NNNN-NNNN) [%s]: ",
														seep->assembly_num);
			if (seep_get_assm(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->assembly_num, item, SEEP_PCB_ASSEMBLY_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}
#endif
		else if (!strcmp(category, "serial")) {
			printf("\tSerial Number (NNNNNNNNNN) [%s]: ", seep->serial_num);
			if (seep_get_serial(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->serial_num, item, SEEP_PCB_SERIAL_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}
		else if (!strcmp(category, "revision")) {
			printf("\tRevision Number (NN.NN or ANN) [%s]: ", seep->revision);
			if (seep_get_rev(item, SEEP_PCB_REVISION_LEN) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->revision, item, SEEP_PCB_REVISION_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}
		else if (!strcmp(category, "deviation"))  {
			printf("\tDeviation Number (DNNN) [%s]: ", seep->deviation);
			if (seep_get_dev(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->deviation, item, SEEP_PCB_DEVIATION_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}
		else if (!strcmp(category, "ip")) {
			printf("\tIP addr [%s]: ", seep->ip_addr);
			if (seep_get_ip(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->ip_addr, item, SEEP_PCB_IP_ADDR_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}
		/*
		else if (!strcmp(category, "misc")) {
			printf("\tMisc Info [%s]: ", seep->misc_info);
			if (seep_get_misc_info(item) == 1) {
				NFX_SPIN_LOCK(&wait_seep_lock);			
				strncpy(seep->misc_info, item, SEEP_PCB_MISC_INFO_LEN);
				NFX_SPIN_UNLOCK(&wait_seep_lock);			
			}
		}*/
		else
			write_to_seep = 0;

		if (write_to_seep) {
			NFX_SPIN_LOCK(&wait_seep_lock);			
			seepWrite(SEEP_PCB, seep_struct[SEEP_PCB], 0, SEEP_TOTAL_LEN);
			NFX_SPIN_UNLOCK(&wait_seep_lock);			
		}	
}

/*-----------------------------------------------------------------
 * Name:	do_seep()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	11/28/01
 *
 *-----------------------------------------------------------------
 */
int32
do_seep(
	int32	argc,
	char8	* argv[]
)
{
	int32	which_seep=0;

	if (argc == 2) {
		which_seep = SEEP_PCB;
	}	
	else {
		if (!strcmp(argv[2], "pcb"))
			which_seep = SEEP_PCB;
#if !defined (PROM_R9000) || !defined (SIBYTE_BC)		
		else if ((!strcmp(argv[2], "chassis")) && (isSSC()))
			which_seep = SEEP_CHASSIS;
		else if ((!strcmp(argv[2], "ps0")) && (isSSC()))
			which_seep = SEEP_PS0;
		else if ((!strcmp(argv[2], "ps1")) && (isSSC()))
			which_seep = SEEP_PS1;
#endif

#ifdef SIBYTE_BC		
		else if ((!strcmp(argv[2], "local")) && ((isTXRX()) || (isFP())))
			which_seep = SEEP_PCB_LOCAL;
#endif		
		else {
			printf("Invalid SEEP, %s\n", argv[2]);
			return -1;
		}
	}

	if (!strcmp(argv[1],"view"))
		do_seep_show(which_seep);
	else if (!strcmp(argv[1],"program")) {
#ifdef SIBYTE_BC		
		/* Do not allow seep to be programmed from TXRX/FP */	
		do_seep_program(SEEP_PCB_LOCAL);
#else			
		do_seep_program(which_seep);
#endif		
	}
	else if (!strcmp(argv[1], "erase"))
		do_seep_erase(which_seep);
	else if (!strcmp(argv[1], "quick")) {
		if (argc == 3) {
			/* quick program one item only*/
			do_seep_program_one_item(argv[2]);
		}
    }
	else if (!strcmp(argv[1], "dump")) {
        int32 i2c_addr;
#ifdef PROM
        char * seep_buf = (char *) PHYS_TO_K0(0x03000200);
#else
        char * seep_buf = (char *) PHYS_TO_K0(0x00700200);
#endif

        i2c_addr = atox(argv[3]);

		NFX_SPIN_LOCK(&wait_seep_lock);			
        i2c(I2C_READ, 0, i2c_addr, 0, SEEP_TOTAL_LEN, seep_buf);
		NFX_SPIN_UNLOCK(&wait_seep_lock);			

        printf("i2c read: data dumped to %p\n", seep_buf);
    }
	else if (!strcmp(argv[1], "fill")) {
        int32 i2c_addr;
#ifdef PROM
        char * seep_buf = (char *) PHYS_TO_K0(0x03000000);
#else
        char * seep_buf = (char *) PHYS_TO_K0(0x00700200);
#endif

        i2c_addr = atox(argv[3]);

		NFX_SPIN_LOCK(&wait_seep_lock);			
        i2c(I2C_WRITE, 0, i2c_addr, 0, SEEP_TOTAL_LEN, seep_buf);
		NFX_SPIN_UNLOCK(&wait_seep_lock);			

        printf("i2c write: data filled from %p\n", seep_buf);
    }
	else {
		return -1;
		printf("Invalid option, %s\n", argv[1]);
	}

	return 0;
}

