#if !defined(LINUX_TEST) && !defined(EVAL_BOARD) && !defined(SSC_MGMT) && !defined(SIBYTE)

#if defined(__OpenBSD__)
#include <sys/types.h>
#include <sys/tty.h>
#include <sys/param.h>
#include <sys/mbuf.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <machine/autoconf.h>
#include <machine/cpu.h>
#include <mips64/dev/clockvar.h>
#include <mips64/archtype.h>
#include <sys/conf.h>
#include <sys/proc.h>
#endif


#include <nfx-types.h>
#include <nfx-defs.h>
#include <exports/eee-error.h>
#include "gt64240-sdma.h"
#include "gt64240-uart-regs.h"
#include "gt64240-uart.h"
#include "gt64240-brg.h"
#include <exports/serio-api.h>

//#define DEBUG_UART
#if defined(DEBUG_UART)
#define debug(x) printf x
#else
#define debug(x)
#endif


extern int bus_clock;


extern int entering_debugger;
#define u_assert(x) { if (!entering_debugger && !(x)) {entering_debugger = 1; assert((x)); } }

/* Macros to clear/set/test flags. */
#define	SET(t, f)	(t) |= (f)
#define	CLR(t, f)	(t) &= ~(f)
#define	ISSET(t, f)	((t) & (f))

#if !defined(GT_MPSC_GOOD_REGS)
uint32 gtu_mainregs[3] = { 0x3fffffff, 0x0, 0x0 };
#endif

static int32 gtu_init(struct gtuart *sc, uint32 unit);

#if defined(__OpenBSD__)

extern void print_psr(void);

cdev_decl(gtuart);

#include <dev/cons.h>

int gtuartopens = 0;

static int gtuart_match(struct device *, void *, void *);
static void gtuart_attach(struct device *, struct device *, void *);
static int gtu_intr(void *arg);
static void gtu_start(struct tty *tp);
static int gtu_param(struct tty *tp, struct termios *t);

struct cfattach gtuart_ca = {
	sizeof(struct gtuart), gtuart_match, gtuart_attach
};

struct cfdriver gtuart_cd = {
	0, "gtuart", DV_TTY, 0, 0
};


static int gtuartmajor;
static struct gtuart gtuconsole;
int gtuart_initialized;
struct gtuart *gtuconsole_ptr;


static int
gtu_base2unit(uint32 addr)
{
	int rc = -1;

	switch (addr) {
	case GT_MPSC_0:
		rc = 0;
		break;
	case GT_MPSC_1:
		rc = 1;
		break;
	}

	return rc;
}


static int
gtuart_match(struct device *parent, void *cfdata, void *aux)
{
	struct confargs *ca = (struct confargs *)aux;

	return (strcmp(ca->ca_name, gtuart_cd.cd_name) == 0);
}


static void
gtu_copysoftc(struct gtuart *sc, struct gtuart *cp)
{
	cp->dma = sc->dma;
#if !defined(GT_MPSC_GOOD_REGS)
	memcpy(cp->regs, sc->regs, sizeof(cp->regs));
	cp->unit = sc->unit;
#endif			   
	gtuart_initialized = 1;
}


static void
gtuart_attach(struct device *parent, struct device *self, void *aux)
{
	struct gtuart *sc = (struct gtuart *)self;
	intrmask_t s;

	printf("\n");

	s = splimp();
	if (sc->dev.dv_unit == gtu_base2unit(CONADDR)) {
		/* The console is already initialized, copy the structure */
		gtu_copysoftc(&gtuconsole, sc);
		gtuconsole_ptr = sc;
		printf("%s: console\n", sc->dev.dv_xname);
	}
	else
		gtu_init(sc, sc->dev.dv_unit);
	galileo_establish_intr(0, GT_SDMA_INTR, 0, IPL_TTY, gtu_intr, sc,
						   "gtuartintr");
	splx(s);
}


struct gtuart *
gtu_sc(int unit)
{
	struct gtuart *sc = 0;

	if (gtuart_initialized) {
		u_assert(unit < gtuart_cd.cd_ndevs);
		sc = gtuart_cd.cd_devs[unit];
	}
	else if (gtu_base2unit(CONADDR) == unit)
		sc = &gtuconsole;
	
	return sc;
}


void
gtuartcnprobe(struct consdev *cp)
{
	if (CONADDR == GT_MPSC_0 || CONADDR == GT_MPSC_1) {
		for (gtuartmajor = 0; gtuartmajor < nchrdev; gtuartmajor++)
			if (cdevsw[gtuartmajor].d_open == gtuartopen)
				break;
		cp->cn_dev = makedev(gtuartmajor, gtu_base2unit(CONADDR));
		cp->cn_pri = CN_REMOTE;
	}
	else
		cp->cn_pri = CN_DEAD;
}


void
gtuartcninit(struct consdev *cp)
{
	gtu_init(&gtuconsole, gtu_base2unit(CONADDR));
}

extern int gotchars;

int 
gtuartcngetc(dev_t dev)
{
//	printf("before splhigh: ");
//	print_psr();

	{
	int c;
	int s = splhigh();

	struct gtuart *sc = gtu_sc(minor(dev));

//	printf("after splhigh: ");
//	print_psr();

	while (!gtu_rxrdy(sc));
	c = gtu_rxchar(sc);
	splx(s);

//	printf("after splx: ");
//	print_psr();

	return c;
	}
}


void
gtuartcnputc(dev_t dev, int c)
{
	int s = splhigh();
	struct gtuart *sc = gtu_sc(minor(dev));
	while (!gtu_txrdy(sc));
	gtsdma_tx_nointerrupt(&sc->dma, c);
	splx(s);
}


void
gtuartcnpollc(dev_t dev, int on)
{
}


void gtuart_print_status()
{
	gtsdma_print_status(&gtuconsole_ptr->dma);
}


#else
struct gtuart uarts[GT_MPSC_UNITS];

struct gtuart *
gtu_sc(int unit)
{
	return &uarts[unit];
}

#endif



void
gtbrg_init(uint32 unit)
{
	gtbrg_write_reg(unit, GT_BRG_BCR, GT_BRG_BCR_EN | GT_BRG_BCR_BCLK);
	gtbrg_write_reg(unit, GT_BRG_BTR, 0);
}


int
gtbrg_setcdv(uint32 unit, int baud)
{
	uint32 cdv;

#if 0
	cdv = BCLK_IN_RATE / (16 * baud) - 1;
	gtbrg_write_reg(unit, GT_BRG_BCR, cdv | GT_BRG_BCR_EN | GT_BRG_BCR_BCLK);
#else
#if 1
	cdv = (bus_clock/(2*baud*16))-1;
#else
	cdv = (66000000/(2*baud*16))-1;
#endif
	gtbrg_write_reg(unit, GT_BRG_BCR, cdv | GT_BRG_BCR_EN | GT_BRG_BCR_TCLK);
#endif

	return NFX_OK;
}



static int
gtu_unit2mrrshift(uint32 unit)
{
	return unit * GT_MPSC_MRR_MR1_OFFSET;
}


static int
gtu_unit2crrshift(uint32 unit)
{
	return unit * GT_MPSC_RCRR_CRR1_OFFSET;
}


static void
gtu_setbrg(uint32 unit, uint32 reg, uint32 brg)
{
	uint32 v;
	int shift;
	
	u_assert(reg == GT_MPSC_RCRR || reg == GT_MPSC_TCRR);

	v = gtu_read_mainreg(reg);
	shift = gtu_unit2crrshift(unit);
	v &= ~(0xf << shift);
	v |= (brg << shift);
	gtu_write_mainreg(reg, v);
}

static void
gtu_abort_receive(struct gtuart *sc)
{
	int i;
	
	/* Write abort receive command and wait for the bit to clear */
	gtu_write_reg(sc->unit, GT_MPSC_CR, GT_MPSC_CR_RA);
#if defined(GT_MPSC_GOOD_REGS)
	while (gtu_read_reg(sc->unit, GT_MPSC_CR) & GT_MPSC_CR_RA);
#else	
	for (i = 0; i < 1000; ++i)
		gtu_read_reg(sc->unit, GT_MPSC_CR);
#endif	
}


static void
gtu_connect(struct gtuart *sc)
{
	uint32 v;

	v = gtu_read_mainreg(GT_MPSC_MRR);
	v &= ~(0x7 << gtu_unit2mrrshift(sc->unit));
	v |= (GT_MPSC_MRR_MR_SERx << gtu_unit2mrrshift(sc->unit));
	gtu_write_mainreg(GT_MPSC_MRR, v);
}


static void
gtu_set_databits(struct gtuart *sc, int bits)
{
	uint32 v;

	v = gtu_read_reg(sc->unit, GT_MPSC_PCR);
	v &= GT_MPSC_PCR_CL_MASK << GT_MPSC_PCR_CL_OFFSET;
	v |= bits;
	gtu_write_reg(sc->unit, GT_MPSC_PCR, v);
}


static int32
gtu_init(struct gtuart *sc, uint32 unit)
{
	u_assert(gtu_valid_unit(unit));

	sc->unit = unit;

	/* Setup port multiplexor register */
//	GT_REG_WRITE(0xf010, 0x1102);

	/* Set the port into UART mode, enable receiver and transmitter */
	gtu_write_reg(unit, GT_MPSC_MMCRL,
				  GT_MPSC_MMCRL_MODE_UART | GT_MPSC_MMCRL_ER
				  | GT_MPSC_MMCRL_ET | GT_MPSC_MMCRL_NLM);

	/* Connect this UART port to BRG<unit> */
	if (0) {
		gtu_setbrg(unit, GT_MPSC_RCRR, GT_MPSC_RCRR_BRG0);
		gtu_setbrg(unit, GT_MPSC_TCRR, GT_MPSC_TCRR_BRG0);

		/* Initialize BRG0 */
		gtbrg_init(0);
	}
	
	/* Stop any pending rx */
	if (0)
		gtu_abort_receive(sc);

	/* initialize SDMA */
	gtsdma_init(&sc->dma, unit);

	/* Set default baud rate */
#if 0
	gtu_baud(sc, TTY_DEFAULT_RATE);
#else
	gtu_baud(sc, 57600);
#endif
	
    /* Connect to serial port */
	gtu_connect(sc);

	/* allow byte-aligned buffers, 16 Tx clock divider, 16 rx clock divider,
	 16 byte preamble, 0xf preamble pattern */
	gtu_write_reg(unit, GT_MPSC_MMCRH,
				  GT_MPSC_MMCRH_RDW | (2 << GT_MPSC_MMCRH_TCDV_OFFSET) |
				  (2 << GT_MPSC_MMCRH_RCDV_OFFSET) |
				  (6 << GT_MPSC_MMCRH_TPL_OFFSET) |
				  (0xf << GT_MPSC_MMCRH_TPPT_OFFSET));

	gtu_write_reg(unit, GT_MPSC_UBSR, 0x003000ab);
	gtu_write_reg(unit, GT_MPSC_MIR, 0);

	gtu_write_reg(unit, GT_MPSC_CFR, 0);
	gtu_write_reg(unit, GT_MPSC_CHR5, 0);
	gtu_write_reg(unit, GT_MPSC_CHR6, 0);
	gtu_write_reg(unit, GT_MPSC_CHR7, 0);
	gtu_write_reg(unit, GT_MPSC_CHR8, 0);

	/* Set 8 data bits, 1 stop bit */
	gtu_set_databits(sc, GT_MPSC_PCR_CL_8);

	gtu_write_reg(unit, GT_MPSC_CR, GT_MPSC_CR_EH);

	/* Enable interrupts */
	gtu_write_intreg(unit, GT_MPSC_MASK, 0x0);
	gtu_write_intreg(1, GT_MPSC_MASK, 0x0);

	return NFX_OK;
}


#if !defined(__OpenBSD__)

int32 gt64240_console(uint32 op, uint32 chan, int32 data)
{
	int rc = -1;
	struct gtuart *sc;
	
	if (gtu_valid_unit(chan) && (sc = &uarts[chan])) {
		switch (op) {
		case OP_INIT:
			rc = gtu_init(sc, chan);
			break;
		case OP_BAUD:
			rc = gtu_baud(sc, data);
			break;
		case OP_TXRDY:
			rc = gtu_txrdy(sc);
			break;
		case OP_RXRDY:
			rc = gtu_rxrdy(sc);
			break;
		case OP_TX:
			rc = gtu_txchar(sc, data);
			break;
		case OP_RX:
			rc = gtu_rxchar(sc);
			break;
		}
	}

	return rc;
}

#endif

int
gtu_baud(struct gtuart *sc, int baud)
{
	return gtbrg_setcdv(sc->unit, baud);
}


int
gtu_txrdy(struct gtuart *sc)
{
	gtsdma_intr(&sc->dma);
	return sc->dma.tx.free != 0;
}


int
gtu_rxrdy(struct gtuart *sc)
{
	gtsdma_intr(&sc->dma);
	return sc->dma.rx.last != sc->dma.rx.first;
}


int
gtu_txchar(struct gtuart *sc, int data)
{
	char c = data;

	return gtsdma_tx(&sc->dma, &c, 1);
}


int
gtu_rxchar(struct gtuart *sc)
{
	char c;

	gtsdma_rx(&sc->dma, &c, 1);
	return c;
}


#if defined(__OpenBSD__)

void
gtu_loopback_mode(struct gtuart *sc)
{
	gtu_write_reg(sc->unit, GT_MPSC_MMCRL,
				  gtu_read_reg(sc->unit, GT_MPSC_MMCRL) | GT_MPSC_MMCRL_LPBK);
}


void
gtu_normal_mode(struct gtuart *sc)
{
	gtu_write_reg(sc->unit, GT_MPSC_MMCRL,
				  gtu_read_reg(sc->unit, GT_MPSC_MMCRL) & ~GT_MPSC_MMCRL_LPBK);
}


static int
do_open(struct gtuart *sc, dev_t dev)
{
	struct tty *tp;
	int error;

	int s = spltty();
	if (!sc->tty) {
		tp = sc->tty = ttymalloc();
		tty_attach(tp);
	}
	else
		tp = sc->tty;
	splx(s);
	
	tp->t_oproc = gtu_start;
	tp->t_param = gtu_param;
	tp->t_dev = dev;
	if (!ISSET(tp->t_state, TS_ISOPEN)) {
		SET(tp->t_state, TS_WOPEN);
		ttychars(tp);
		tp->t_iflag = TTYDEF_IFLAG;
		tp->t_oflag = TTYDEF_OFLAG;
		tp->t_lflag = TTYDEF_LFLAG;
		tp->t_cflag = TTYDEF_CFLAG;
		tp->t_ispeed = tp->t_ospeed = 57600;
		SET(tp->t_cflag, CLOCAL);
			
		s = spltty();
		/* Force gtu_param to initialize baud rate */
		sc->initialize = 1;
		debug(("do_open: call gtu_param: speed=%d\n",
			   tp->t_termios.c_ispeed));
		gtu_param(tp, &tp->t_termios);
		ttsetwater(tp);
		if (++gtuartopens)
			timeout(gtu_softintr, NULL, 1);
		splx(s);
	}

	error = (*linesw[tp->t_line].l_open)(dev, tp);
	return error;
}

int
gtuartopen(dev_t dev, int flag, int mode, struct proc *p)
{
	int rc = ENXIO;
	int unit = minor(dev);
	
	debug(("gtuartopen: %d\n", unit));
	
	if (unit < gtuart_cd.cd_ndevs)
		rc = do_open(gtu_sc(unit), dev);

	debug(("gtuartopen: rc=%d\n", rc));
	
	return rc;
}


int
gtuartclose(dev_t dev, int flag, int mode, struct proc *p)
{
	int unit = minor(dev);
	struct gtuart *sc = gtu_sc(unit);
	struct tty *tp = sc->tty;
	int rc = 0;
	
	debug(("gtuartclose: %d\n", unit));

	if (ISSET(tp->t_state, TS_ISOPEN)) {
		(*linesw[tp->t_line].l_close)(tp, flag);
		ttyclose(tp);
		if (--gtuartopens == 0)
			untimeout(gtu_softintr, NULL);
	}

	return rc;
}


int
gtuartread(dev_t dev, struct uio *uio, int flag)
{
	struct tty *tp = gtuarttty(dev);
	return ((*linesw[tp->t_line].l_read)(tp, uio, flag));
}


int
gtuartwrite(dev_t dev, struct uio *uio, int flag)
{
	struct tty *tp = gtuarttty(dev);
	return ((*linesw[tp->t_line].l_write)(tp, uio, flag));
}


static int
do_ioctl(struct gtuart *sc, u_long cmd, caddr_t data, int flag, struct proc *p)
{
	struct tty *tp = sc->tty;
	int error;

	debug(("do_ioctl: call linesw_ioctl\n"));
	error = (*linesw[tp->t_line].l_ioctl)(tp, cmd, data, flag, p);
	if (error < 0) {
		debug(("do_ioctl: call ttioctl\n"));
		error = ttioctl(tp, cmd, data, flag, p);
		if (error < 0) {
			debug(("do_ioctl : error=%d\n", error));
			error = 0;
			switch (cmd) {
			case TIOCSBRK:
				break;
			case TIOCCBRK:
				break;
			case TIOCSDTR:
				break;
			case TIOCCDTR:
				break;
			case TIOCMSET:
				break;
			case TIOCMBIS:
				break;
			case TIOCMBIC:
				break;
			case TIOCMGET:
				break;
			case TIOCGFLAGS:
				break;
			case TIOCSFLAGS:
				break;
			default:
				error = ENOTTY;
				break;
			}
		}
	}

	return error;
}


int
gtuartioctl(dev_t dev, u_long cmd, caddr_t data, int flag,
			struct proc *p)
{
	int unit = minor(dev);
	int rc = ENXIO;

	if (unit < gtuart_cd.cd_ndevs)
		rc = do_ioctl(gtu_sc(unit), cmd, data, flag, p);

	return rc;
}


int
gtuartstop(struct tty *tp, int flag)
{
	int s;

	s = spltty();
	if (ISSET(tp->t_state, TS_BUSY))
		if (!ISSET(tp->t_state, TS_TTSTOP))
			SET(tp->t_state, TS_FLUSH);
	splx(s);
	return 0;
}


struct tty *
gtuarttty(dev_t dev)
{
	int unit = minor(dev);
	struct tty *tp = 0;
	if (unit < gtuart_cd.cd_ndevs)
		tp = gtu_sc(unit)->tty;
	return tp;
}


static void
gtu_sleep(struct gtuart *sc)
{
	gtsdma_txabort(&sc->dma);
	gtsdma_rxabort(&sc->dma);
	gtu_abort_receive(sc);
}


static void
gtu_start(struct tty *tp)
{
	struct gtuart *sc = gtu_sc(minor(tp->t_dev));
	int s = spltty();

	if (!ISSET(tp->t_state, TS_BUSY)) {
		if (!ISSET(tp->t_state, TS_TIMEOUT|TS_TTSTOP)) {
			if (tp->t_outq.c_cc <= tp->t_lowat) {
				if (ISSET(tp->t_state, TS_ASLEEP)) {
					CLR(tp->t_state, TS_ASLEEP);
					debug(("gtu_start: call wakeup\n"));
					wakeup(&tp->t_outq);
				}
				if (tp->t_outq.c_cc != 0) {
					debug(("gtu_start: call selwakeup\n"));
					selwakeup(&tp->t_wsel);
				}
			}

			if (sc->dma.tx.free > 1) {	
				char buf[GTSDMA_NTX];
				int n;
				
				n = q_to_b(&tp->t_outq, buf, sc->dma.tx.free - 1);
				if (n == sc->dma.tx.free - 1)
					SET(tp->t_state, TS_BUSY);
				u_assert(n <= sc->dma.tx.free - 1);
				debug(("gtu_start: call tx: n=%d\n", n));
				if (n != 0) {
					gtsdma_tx(&sc->dma, buf, n);
				}
			}
			else {
				SET(tp->t_state, TS_BUSY);
			}
		}
		else {
			debug(("gtu_start: stop or timeout\n"));
		}
	}
	else {
		debug(("gtu_start: busy\n"));
	}
	splx(s);
}


static void
gtu_set_stopbit(struct gtuart *sc, int x)
{
	assert(x == 0 || x == GT_MPSC_PCR_SBL);
	gtu_write_reg(sc->unit, GT_MPSC_PCR,
				  (gtu_read_reg(sc->unit, GT_MPSC_PCR) &
				   ~GT_MPSC_PCR_SBL) | x);
}


static int
gtu_termios_parity(struct termios *t)
{
	int v;
	
	if (ISSET(t->c_cflag, PARENB)) {
		if (ISSET(t->c_cflag, PARODD)) {
			v = GT_MPSC_CR_TPM_ODD | GT_MPSC_CR_RPM_ODD;
		}
		else {
			v = GT_MPSC_CR_TPM_EVEN | GT_MPSC_CR_RPM_EVEN;
		}
	}
	else {
		v = GT_MPSC_CR_TPM_LOW | GT_MPSC_CR_RPM_LOW;
	}

	return v;
}


static void
gtu_set_parity(struct gtuart *sc, uint32 parity)
{
	gtu_write_reg(sc->unit, GT_MPSC_CR, parity);
}


static int
gtu_param_validargs(struct tty *tp, struct termios *t)
{
	return t->c_ispeed == 0 || (t->c_ispeed == t->c_ospeed);
}


static void
gtu_setttyflags(struct gtuart *sc, struct tty *tp, struct termios *t)
{
	int oldcflag;

	tp->t_ispeed = t->c_ispeed;
	tp->t_ospeed = t->c_ospeed;
	oldcflag = tp->t_cflag;
	tp->t_cflag = t->c_cflag;

	if (ISSET(oldcflag, MDMBUF) != ISSET(tp->t_cflag, MDMBUF) &&
		(*linesw[tp->t_line].l_modem)(tp, 0) == 0) {
		debug(("gtu_setttyflags: sleep\n"));
		gtu_sleep(sc);
	}
}


static void
gtu_do_setparam(struct gtuart *sc, struct tty *tp, struct termios *t)
{
	int s = spltty();
		
	switch (ISSET(t->c_cflag, CSIZE)) {
	case CS5:
		gtu_set_databits(sc, GT_MPSC_PCR_CL_5);
		break;
	case CS6:
		gtu_set_databits(sc, GT_MPSC_PCR_CL_6);
		break;
	case CS7:
		gtu_set_databits(sc, GT_MPSC_PCR_CL_7);
		break;
	case CS8:
		gtu_set_databits(sc, GT_MPSC_PCR_CL_8);
		break;
	}

	gtu_set_parity(sc, gtu_termios_parity(t));
	gtu_set_stopbit(sc, ISSET(t->c_cflag, CSTOPB) ?
					GT_MPSC_PCR_SBL : 0);

	if (sc->initialize || (tp->t_ispeed != t->c_ispeed)) {
		sc->initialize = 0;
		if (tp->t_ispeed != 0) {
			debug(("gtu_do_setparam: speed=%d\n", t->c_ispeed));
			gtu_baud(sc, t->c_ispeed);
		}
		else {
			debug(("gtu_do_setparam: sleep\n"));
			gtu_sleep(sc);
		}
	}

	gtu_setttyflags(sc, tp, t);

	splx(s);

	gtu_start(tp);
}


static int
gtu_param(struct tty *tp, struct termios *t)
{
	int rc = EINVAL;
 
	if (gtu_param_validargs(tp, t)) {
		debug(("gtu_param: call gtu_do_setparam\n"));
		gtu_do_setparam(gtu_sc(minor(tp->t_dev)), tp, t);
		rc = 0;
	}
	
	return rc;
}


static int
gtu_intr(void *arg)
{
	struct gtuart *sc = arg;
	struct tty *tp = sc->tty;
	int again = 1;

//	debug(("gtu_intr:\n"));

	while (again) {
		gtsdma_intr(&sc->dma);

		again = 0;
		if ((tp = sc->tty)) {
			debug(("gtu_intr:\n"));
			
			if (gtu_rxrdy(sc)) {
				debug(("gtu_intr: gtu_rxrdy\n"));
				gtsdma_check_debugger(&sc->dma);
				//setsofttty();
			}
			
			if (gtu_txrdy(sc) && sc->dma.tx.free > 1 &&
				ISSET(tp->t_state, TS_BUSY)) {
				CLR(tp->t_state, TS_BUSY | TS_FLUSH);
//				wakeup(&tp->t_outq);
				(*linesw[tp->t_line].l_start)(tp);
				again = 1;
			}
		}
	}

	return 1;
}


void
gtu_softintr(void *arg)
{
	int unit;
	
	for (unit = 0; unit < gtuart_cd.cd_ndevs; unit++) {
		struct gtuart *sc = gtu_sc(unit);
		if (sc) {
			struct tty *tp = sc->tty;
			assert(tp);
			while (gtu_rxrdy(sc)) {
				char c;
				gtsdma_rx(&sc->dma, &c, 1);
				debug(("gtu_softintr: got char %d\n", c));
				(*linesw[tp->t_line].l_rint)(c, tp);
			}
		}
	}

	timeout(gtu_softintr, NULL, 1);
}


#endif
#endif
