/*
 *-----------------------------------------------------------------
 *
 * Name:	spinlock-api.c
 *
 * RCS:  $Id: spinlock-api.c,v 1.12 2002/12/09 23:54:18 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/19/01
 *
 *-----------------------------------------------------------------
 */

#if defined(NFX_SMP)
#if defined(NFX_SPINLOCK_DEBUG)

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/spinlock-api.h>
#include <exports/cm-api.h>
#include "spinlock.h"
#include <exports/queue-api.h>
#include "../sm-esm/esm-api.h"
#include "../sm-esm/esm.h"

uint32	spinlock_level[SPINLOCK_LEVEL_MAX]	CPU_PRIVATE;
uint32	spinlock_level_max CPU_PRIVATE;

extern uint64  trylock_failed;
extern uint64  locks_acquired_count;
extern uint32  locks_currently_held;

#define SPINLOCK_MAGIC 0xcaceacec

#define SPINLOCK_STATIC_MAX		64

nfx_spinlock_t	* spinlock_static_lock[SPINLOCK_STATIC_MAX];
uint32			spinlock_static_num = 0;

void
spinlock_print(nfx_spinlock_t *lockp)
{
	printf("lock at %p: lock=%d cpu=%d level=%d owner=%p magic=%x\n",
		   lockp, lockp->lock, lockp->cpu_num, lockp->lock_level,
		   (void *)lockp->owner, lockp->sl_magic);
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Max
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
void
spinlock_error(nfx_spinlock_t *lockp, char *format, ...)
{
	va_list		ap;
	char str[256];
	va_start(ap, format);
	spinlock_print(lockp);
	vsprintf(str, format, ap);
	va_end(ap);
	panic(str);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/07/02
 *
 *-----------------------------------------------------------------
 */
void
spinlock_init_spinlocks()
{
	uint32	i;

	bzero(spinlock_level, SPINLOCK_LEVEL_MAX * sizeof(uint32));
	spinlock_level_max = 0;

	trylock_failed = 0;
	locks_acquired_count = 0;
	locks_currently_held = 0;

	for (i=0; i<SPINLOCK_DEBUG_NUM; i++)
		NFX_SPIN_INIT_DEBUG(&spinlock_debug[i], SPINLOCK_LEVEL_DEBUG);
}

/*-----------------------------------------------------------------
 * Name:	spinlock_init(lockp, level)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/19/01
 *
 *-----------------------------------------------------------------
 */
int32
spinlock_init(
	nfx_spinlock_t	* lockp,
	ushort16		level
)
{
	extern uchar8	* _ftext, * _end;

//	printf("spinlock_init, lockp = %p, level = %d\n", lockp, level);

	lockp->lock = 0;
	lockp->accumulated_wait_time = 0;
	lockp->lock_level = level;
	lockp->owner = 0;
	lockp->cpu_num = -1;
	lockp->num_got = 0;
	lockp->num_waited_for = 0;
	lockp->sl_magic = SPINLOCK_MAGIC;

	if (((uchar8 *)lockp >= (uchar8 *)&_ftext) &&
										((uchar8 *)lockp < (uchar8 *)&_end))
		spinlock_static_lock[spinlock_static_num++] = lockp;

	return 0;

}	/* spinlock_init() */

int32
spinlock_init_debug(
	nfx_spinlock_t	* lockp,
	ushort16		level
)
{
	lockp->lock = 0;
	lockp->accumulated_wait_time = 0;
	lockp->lock_level = level;
	lockp->owner = 0;
	lockp->cpu_num = -1;
	lockp->num_got = 0;
	lockp->num_waited_for = 0;
	lockp->sl_magic = SPINLOCK_MAGIC;

	return 0;
}	/* spinlock_init_debug() */


static void
add_thread_spinlock(nfx_spinlock_t *lockp)
{
	if (esm_currInst && (esm_currInst->flags & ESM_CAN_YIELD)) {
		LIST_INSERT_HEAD(&esm_currInst->sm_spinlocksHeld, lockp, sl_entry);
	}
}


static void
remove_thread_spinlock(nfx_spinlock_t *lockp)
{
	if (esm_currInst && (esm_currInst->flags & ESM_CAN_YIELD)) {
		LIST_REMOVE(lockp, sl_entry);
	}
}


/*-----------------------------------------------------------------
 * Name:	spinlock_getlock(lockp)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/19/01
 *
 *-----------------------------------------------------------------
 */
void
spinlock_getlock(
	nfx_spinlock_t	* lockp
)
{
	int32		start_time, end_time;
	address_t	ra = get_ra();

	if (!(lockp->lock_level && lockp->sl_magic == SPINLOCK_MAGIC))
		spinlock_error(lockp,
					   "lock at %p not initialized, ra = %p\n", lockp, ra);

//	printf("spinlock_getlock, lockp = %p\n", lockp);

	if (lockp->lock_level < spinlock_level_max)
		spinlock_error(lockp,
					   "SPINLOCK Error: level %d, current level %d, ra = %p\n",
					   lockp->lock_level, spinlock_level_max, ra);

	if (lockp->cpu_num == nfxMyCoreId) {
		spinlock_error(lockp,
		   "deadlock in spinlock_getlock: lk=%p owner=%p ra=%p cpu=%d\n",
					   lockp, lockp->owner, ra, nfxMyCoreId);
	}

	if (SPINLOCK_TRYLOCK(lockp)) {
	}
	else {
		start_time = _mips_mfc0(C0_COUNT);
		SPINLOCK_LOCK(lockp);
		end_time = _mips_mfc0(C0_COUNT);
		lockp->accumulated_wait_time += (end_time - start_time);
		lockp->num_waited_for++;
	}

	lockp->num_got++;
	lockp->owner = ra;
	lockp->cpu_num = nfxMyCoreId;
	lockp->lock_start_time = _mips_mfc0(C0_COUNT);

	spinlock_level[lockp->lock_level]++;
	if (lockp->lock_level > spinlock_level_max)
		spinlock_level_max = lockp->lock_level;

	locks_acquired_count++;
	locks_currently_held++;

	add_thread_spinlock(lockp);

}	/* spinlock_getlock() */

/*-----------------------------------------------------------------
 * Name:	spinlock_unlock(lockp)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/19/01
 *
 *-----------------------------------------------------------------
 */
int32
spinlock_unlock(
	nfx_spinlock_t	* lockp
)
{
	int32		end_time, time_held;
	int32		i;
	address_t	ra = get_ra();

//	printf("spinlock_unlock, lockp = %p\n", lockp);

	end_time = _mips_mfc0(C0_COUNT);
	time_held = end_time - lockp->lock_start_time;

	if (!(lockp->lock_level && lockp->sl_magic == SPINLOCK_MAGIC))
		spinlock_error(lockp, "lock at %p not initialized, ra = %p\n",
					   lockp, ra);

	if (lockp->cpu_num != nfxMyCoreId) {
		spinlock_error(lockp,
					   "lock at %p not owned by cpu %d: owner=%p ra=%p\n",
					   lockp, nfxMyCoreId, lockp->owner, ra);
	}

	spinlock_level[lockp->lock_level]--;
	if (lockp->lock_level == spinlock_level_max) {
		if (spinlock_level[lockp->lock_level] == 0) {
			spinlock_level_max = 0;
			for (i=lockp->lock_level; i > 0; i--) {
				if (spinlock_level[i]) {
					spinlock_level_max = i;
					break;
				}
			}
		}
	}

	if (time_held > lockp->max_time_held)
		lockp->max_time_held = time_held;

	lockp->owner = ra;
	lockp->cpu_num = -1;

	remove_thread_spinlock(lockp);

	SPINLOCK_UNLOCK(lockp);

	locks_currently_held--;

	return 1;

}	/* spinlock_unlock() */

/*-----------------------------------------------------------------
 * Name:	spinlock_trylock(lockp)
 *
 * Description:	Returns 1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/07/02
 *
 *-----------------------------------------------------------------
 */
int32
spinlock_trylock(
	nfx_spinlock_t	* lockp
)
{
	address_t	ra = get_ra();

	if (!(lockp->lock_level && lockp->sl_magic == SPINLOCK_MAGIC))
		spinlock_error(lockp,
					   "lock at %p not initialized, ra = %p\n", lockp, ra);
			
	if (lockp->lock_level < spinlock_level_max)
		spinlock_error(lockp,
					   "SPINLOCK Error: level %d, current level %d, ra = %p\n",
					   lockp->lock_level, spinlock_level_max, ra);

	if (SPINLOCK_TRYLOCK(lockp)) {
		lockp->owner = ra;
		lockp->cpu_num = nfxMyCoreId;
		lockp->lock_start_time = _mips_mfc0(C0_COUNT);
		lockp->num_got++;

		spinlock_level[lockp->lock_level]++;
		if (lockp->lock_level > spinlock_level_max)
			spinlock_level_max = lockp->lock_level;

		locks_acquired_count++;
		locks_currently_held++;

		add_thread_spinlock(lockp);

		return 1;
	}

	trylock_failed++;

	return 0;

}	/* spinlock_trylock() */


/*-----------------------------------------------------------------
 * Name :	spinlock_owned()
 *
 * Description: Return true if spinlock is locked by this cpu.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	11/22/02
 *
 *-----------------------------------------------------------------
 */
int
spinlock_owned(nfx_spinlock_t *lockp)
{
	address_t	ra = get_ra();

	if (!(lockp->lock_level && lockp->sl_magic == SPINLOCK_MAGIC))
		spinlock_error(lockp,
					   "lock at %p not initialized, ra = %p\n", lockp, ra);

	return lockp->lock != 0 && lockp->cpu_num == nfxMyCoreId;
	
}

#else

void
spinlock_init_spinlocks(void)
{
}

#endif /* NFX_SPINLOCK_DEBUG */

#endif	/* NFX_SMP */

