/*
 *-----------------------------------------------------------------
 *
 * Name:	spinlock-api.h
 *
 * RCS:  $Id: spinlock-api.h,v 1.9 2002/12/09 23:54:18 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/19/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _SPINLOCK_API_H
#define _SPINLOCK_API_H

#define SPINLOCK_LEVEL_MAX			200

#define SPINLOCK_LEVEL_ESM			1
#define SPINLOCK_LEVEL_WAIT_TIMER	1
#define SPINLOCK_LEVEL_STATS		1
#define SPINLOCK_LEVEL_MALLOC		1
#define SPINLOCK_LEVEL_FILE			1
#define SPINLOCK_LEVEL_ATOMIC		(SPINLOCK_LEVEL_MAX-1)
#define SPINLOCK_LEVEL_DEBUG		1
#define SPINLOCK_LEVEL_UNKNOWN		1

#ifdef LANGUAGE_C

#ifdef NFX_SMP

void spinlock_print(nfx_spinlock_t *lockp);

int32 spinlock_init(nfx_spinlock_t	* lockp, ushort16 level);
int32 spinlock_init_debug(nfx_spinlock_t	* lockp, ushort16 level);
void spinlock_getlock(nfx_spinlock_t	* lockp);
int32 spinlock_unlock(nfx_spinlock_t	* lockp);
int32 spinlock_trylock(nfx_spinlock_t	* lockp);
int	spinlock_owned(nfx_spinlock_t		* lockp);
void atomic_increment(void * valptr, uint32 size, uint32 step);
void atomic_decrement(void * valptr, uint32 size, uint32 step);

#include <exports/spinlock-inlines.h>
//extern void SPINLOCK_LOCK(nfx_spinlock_t	* lockp);
//extern void SPINLOCK_UNLOCK(nfx_spinlock_t	* lockp);
//extern int32 SPINLOCK_TRYLOCK(nfx_spinlock_t	* lockp);
extern void ATOMIC_INC_INT(uint32 * iptr, uint32 step);
extern void ATOMIC_INC_DWORD(uint64 * iptr, uint32 step);
extern void ATOMIC_DEC_INT(uint32 * iptr, uint32 step);
extern void ATOMIC_DEC_DWORD(uint64 * iptr, uint32 step);
extern uint32 ATOMIC_OR_INT(uint32 * iptr, uint32 val);
extern uint32 ATOMIC_AND_INT(uint32 * iptr, uint32 val);
extern uint32 ATOMIC_OR_DWORD(uint64 * iptr, uint64 val);
extern uint32 ATOMIC_AND_DWORD(uint64 * iptr, uint64 val);

#if defined(NFX_SPINLOCK_DEBUG)
#define ASSERT_LOCK_OWNED(lock) NFX_ASSERT(spinlock_owned(lock))
#define ASSERT_LOCK_NOT_OWNED(lock) NFX_ASSERT(!spinlock_owned(lock))
#define NFX_SPIN_INIT_DEBUG(lockp, level)	spinlock_init_debug(lockp, level)
#else
#define ASSERT_LOCK_OWNED(lock)
#define ASSERT_LOCK_NOT_OWNED(lock)
#define NFX_SPIN_INIT_DEBUG(lockp, level)	(lockp)=0
#endif

#else

#define ATOMIC_INC_INT(iptr,step)	*(uint32 *)(iptr) = *(uint32 *)(iptr)+(step)
#define ATOMIC_INC_DWORD(iptr,step) *(uint64 *)(iptr) = *(uint64 *)(iptr)+(step)
#define ATOMIC_DEC_INT(iptr,step)	*(uint32 *)(iptr) = *(uint32 *)(iptr)-(step)
#define ATOMIC_DEC_DWORD(iptr,step) *(uint64 *)(iptr) = *(uint64 *)(iptr)-(step)
#define ATOMIC_OR_INT(iptr, val) ({uint32 _t; _t = *(uint32 *)(iptr); *(uint32 *)(iptr) = _t | (val); _t;})
#define ATOMIC_AND_INT(iptr, val) ({uint32 _t; _t = *(uint32 *)(iptr); *(uint32 *)(iptr) = _t & (val); _t;})
#define ATOMIC_OR_DWORD(iptr, val) ({uint64 _t; _t = *(uint64 *)(iptr); *(uint64 *)(iptr) = _t | (val); _t;})
#define ATOMIC_AND_DWORD(iptr, val) ({uint64 _t; _t = *(uint64 *)(iptr); *(uint64 *)(iptr) = _t & (val); _t;})

#define ASSERT_LOCK_OWNED(lock)
#define ASSERT_LOCK_NOT_OWNED(lock)
#define NFX_SPIN_INIT_DEBUG(lockp, level)

static inline void
atomic_increment(
	void * valptr,
	uint32	size,
	uint32	step
)
{
	if (size == 4)
		ATOMIC_INC_INT(valptr, step);
	else if (size == 8)
		ATOMIC_INC_DWORD(valptr, step);
}

static inline void
atomic_decrement(
	void * valptr,
	uint32	size,
	uint32	step
)
{
	if (size == 4)
		ATOMIC_DEC_INT(valptr, step);
	else if (size == 8)
		ATOMIC_DEC_DWORD(valptr, step);
}

#endif	/* NFX_SMP */

#endif	/* LANGUAGE_C */

#endif	/* _SPINLOCK_API_H */

