/*
 *-----------------------------------------------------------------
 *
 * Name:	spinlock-inlines.h
 *
 * RCS:  $Id$
 *
 * Copyright (C) 2003, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/15/03
 *
 *-----------------------------------------------------------------
 */

#include <sys/mips.h>

#ifdef SIBYTE

/*-----------------------------------------------------------------
 * Name:	SPINLOCK_LOCK(_lockp)
 *
 * Description:	Acquire a spinlock.
 *
 *				_lockp is of nfx_spinlock_t type.
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
#define SPINLOCK_LOCK(_lockp) {				\
	__asm__ __volatile__					\
	(	".set noreorder;"					\
		"sync;"								\
		"ll		$8,0(%0);"					\
		"bne	$8,$0,.-4;"					\
		"li		$8,1;"						\
		"sc		$8,0(%0);"					\
		"beq	$8,$0,.-16;"				\
		"nop;"								\
		".set reorder;"						\
		: /* no outputs */					\
		: /* inputs */ "r" (_lockp)			\
		: /* modifies */ "$8", "memory"		\
	);										\
}

/*-----------------------------------------------------------------
 * Name:	SPINLOCK_UNLOCK(_lockp)
 *
 * Description: Release a spinlock.
 *
 *				_lockp is of nfx_spinlock_t type.
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
#define SPINLOCK_UNLOCK(_lockp) {			\
	__asm__ __volatile__					\
	(	".set noreorder;"					\
		"sync;"								\
		"sw		$0,0(%0);"					\
		".set reorder;"						\
		: /* no outputs */					\
		: /* inputs */ "r" (_lockp)			\
		: /* modifies */ "memory"			\
	);										\
}

/*-----------------------------------------------------------------
 * Name: SPINLOCK_TRYLOCK(_lockp)
 *
 * Description: Try to acquire a spinlock.  If spinlock successfully acquired,
 *				return 1.  If fails to get spinlock, immediately
 *				return 0 (don't spin, waiting for lock).
 *
 *				_lockp is of nfx_spinlock_t type.
 *
 *				Returns 1 is spinlock is acquired, 0 if not acquired.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
#define SPINLOCK_TRYLOCK(_lockp)			\
({											\
	uint32	_r;								\
	__asm__ __volatile__					\
	(	".set noreorder;"					\
		"sync;"								\
		"ll		%0,0(%1);"					\
		"bne	%0,$0,.+16;"				\
		"move	%0,$0;"						\
		"li		%0,1;"						\
		"sc		%0,0(%1);"					\
		".set reorder;"						\
		: /* outputs */ "=r" (_r)			\
		: /* inputs */ "r" (_lockp)			\
		: /* modifies */ "memory"			\
	);										\
	_r;										\
})

#endif	/* SIBYTE */

