/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-msg.c
 *
 * RCS:		$Id: tape-msg.c,v 1.33 2002/11/07 23:20:57 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver message processing code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 10/2/01
 *
 *------------------------------------------------------------------------------
 */

#include "tape-common.h"
#include "tape-api.h"
#include "tape-msg.h"
#include "tape-device.h"
#include "../sm-scsi/scsi-dev-cmds.h"
#include <sys/socket.h>
#include <exports/opt-api.h>

int		tape_rxif = -1;
struct timeval	tape_receive_poll_to;

void tape_receive_msg(tape_msg_hdr_t *, int, char *, uint32, uint32);
void tape_proc_get_devlist(tape_get_devlist_req_msg_t *, char *, uint32, uint32);
void tape_proc_open(tape_open_req_msg_t *, char *, uint32, uint32);
void tape_proc_get_handle(tape_get_handle_req_msg_t *, char *, uint32, uint32);
void tape_proc_close(tape_close_req_msg_t *, char *, uint32, uint32);
void tape_proc_close_tag(tape_close_req_msg_t *, char *, uint32, uint32);
void tape_proc_get_state(tape_get_state_req_msg_t *, char *, uint32, uint32);
void tape_proc_space(tape_space_req_msg_t *, char *, uint32, uint32);
void tape_proc_space_rsp_success(tape_msg_context_t *, sdm_relayCdbRspMsg_t *);
void tape_proc_space_rsp_failed(tape_msg_context_t *, uint32);
void tape_proc_write_fms(tape_write_fm_req_msg_t *, char *, uint32, uint32);
void tape_proc_write_fms_rsp_success(tape_msg_context_t *, sdm_relayCdbRspMsg_t *);
void tape_proc_write_fms_rsp_failed(tape_msg_context_t *, uint32);
void tape_proc_load(tape_load_req_msg_t *, char *, uint32, uint32);
void tape_proc_load_rsp_success(tape_msg_context_t *, sdm_relayCdbRspMsg_t *);
void tape_proc_load_rsp_failed(tape_msg_context_t *, uint32);
void tape_proc_read(tape_read_req_msg_t *, char *, uint32, uint32);
void tape_proc_read_rsp_success(tape_msg_context_t *, sdm_readDataRspMsg_t *);
void tape_proc_read_rsp_failed(tape_msg_context_t *, uint32);
void tape_proc_write(tape_write_req_msg_t *, char *, uint32, uint32);
void tape_proc_write_rsp_success(tape_msg_context_t *, sdm_writeDataRspMsg_t *);
void tape_proc_write_rsp_failed(tape_msg_context_t *, uint32);
void tape_proc_passthru(tape_passthru_req_msg_t *, char *, uint32, uint32);
void tape_proc_passthru_rsp_success(tape_msg_context_t *, sdm_relayCdbRspMsg_t *);
void tape_proc_passthru_rsp_failed(tape_msg_context_t *, uint32);
void tape_proc_set_debug_level(tape_set_debug_level_req_msg_t *, char *, uint32, uint32);

void mc_proc_get_devlist(mc_get_devlist_req_msg_t *, char *, uint32, uint32);
void mc_proc_open(mc_open_req_msg_t *, char *, uint32, uint32);
void mc_proc_get_handle(mc_get_handle_req_msg_t *, char *, uint32, uint32);
void mc_proc_close(mc_close_req_msg_t *, char *, uint32, uint32);
void mc_proc_passthru(mc_passthru_req_msg_t *, char *, uint32, uint32);
void mc_proc_passthru_rsp_success(tape_msg_context_t *, sdm_relayCdbRspMsg_t *);
void mc_proc_passthru_rsp_failed(tape_msg_context_t *, uint32);

void tape_proc_sdm_open_rsp(sdm_openRspMsg_t *);
void tape_proc_sdm_info_rsp(sdm_devInfoRspMsg_t *);
void tape_proc_sdm_relay_rsp(sdm_relayCdbRspMsg_t *);
void tape_proc_sdm_read_rsp(sdm_readDataRspMsg_t *);
void tape_proc_sdm_write_rsp(sdm_writeDataRspMsg_t *);
void tape_proc_sdm_dev_status_rsp(sdm_devStatusRspMsg_t *);
void tape_proc_sdm_close_rsp(sdm_closeRspMsg_t *);
void tape_proc_sdm_err_rsp(sdm_errRspMsg_t *);

void
tape_msg_init(void)
{
	tape_rxif = regAgileApp("tape-driver");
	if (tape_rxif < 0 ) {
		perror("tape-driver failed to register for messages");
		exit(1);
	}

#if 0
	{
	    int rc;
	    int rcvbuf = 0, rcvbuf_len = sizeof(rcvbuf);
	    int sndbuf = 0, sndbuf_len = sizeof(sndbuf);

	    rc = getsockopt(tape_rxif, SOL_SOCKET, SO_RCVBUF, &rcvbuf, &rcvbuf_len);
	    printf("SO_RCVBUF: rcvbuf %d\n", rcvbuf);

	    rc = getsockopt(tape_rxif, SOL_SOCKET, SO_SNDBUF, &sndbuf, &sndbuf_len);
	    printf("SO_SNDBUF: sndbuf %d\n", sndbuf);
	}
#endif

	tape_receive_poll_to.tv_sec = 1;
	tape_receive_poll_to.tv_usec = 0;
}

void
tape_receive_poll(void)
{
	uint32		src_cpu, src_slot, src_plane ;
	char		src_app[32];
	static char	recv_msg[65536*4];
	int		recv_bytes;

	recv_bytes = recvAgileMsg(tape_rxif, 
				  recv_msg, 
				  sizeof(recv_msg), 
				  &tape_receive_poll_to, 
				  src_app, &src_cpu, &src_slot, &src_plane);
	if (recv_bytes < 0) {
		TP_DBG(2, ("tape_receive_poll: msg receive error\n"));
	}
	else 
	if (recv_bytes) {
		TP_DBG(2, ("tape_receive_poll: received msg: "
			   "%d bytes (mid 0x%x) from app %s, cpu %d, slot %d\n",
			   recv_bytes, *((uint32 *)(recv_msg)), src_app, src_cpu, src_slot));
		tape_receive_msg((tape_msg_hdr_t *)recv_msg, recv_bytes, src_app, src_cpu, src_slot);
	}
}

void 
tape_receive_msg(tape_msg_hdr_t *recv_msg, int recv_len, char *app, uint32 cpu, uint32 slot)
{
	tape_msg_hdr_t	resp;

	switch(recv_msg->mid) {
	case TAPE_MID_GET_DEVLIST_REQ:
		if (recv_len != sizeof(tape_get_devlist_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid DEVLIST_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_get_devlist((tape_get_devlist_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_OPEN_REQ:
		if (recv_len != sizeof(tape_open_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid OPEN_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_open((tape_open_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_GET_HANDLE_REQ:
		if (recv_len != sizeof(tape_get_handle_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid GET_HANDLE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_get_handle((tape_get_handle_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_CLOSE_REQ:
		if (recv_len != sizeof(tape_close_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid CLOSE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_close((tape_close_req_msg_t *)recv_msg, app, cpu, slot);
		break;

    case TAPE_MID_ID_CLOSE_REQ:
        if (recv_len != sizeof(tape_close_req_msg_t)) {
            TP_DBG(2, ("tape_receive_msg: Invalid CLOSE_REQ message length %d\n", recv_len));
            goto error_response;
        }
        tape_proc_close_tag((tape_close_req_msg_t *)recv_msg, app, cpu, slot);
        break;

	case TAPE_MID_GET_STATE_REQ:
		if (recv_len != sizeof(tape_get_state_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid GET_STATE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_get_state((tape_get_state_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_SPACE_REQ:
		if (recv_len != sizeof(tape_space_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SPACE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_space((tape_space_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_WRITE_FM_REQ:
		if (recv_len != sizeof(tape_write_fm_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid WRITE_FM_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_write_fms((tape_write_fm_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_LOAD_REQ:
		if (recv_len != sizeof(tape_load_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid LOAD_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_load((tape_load_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_READ_REQ:
		if (recv_len != sizeof(tape_read_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid READ_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_read((tape_read_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_WRITE_REQ:
		if (recv_len != sizeof(tape_write_req_msg_t)-1 + ((tape_write_req_msg_t *)recv_msg)->bytecount) {
			TP_DBG(2, ("tape_receive_msg: Invalid WRITE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_write((tape_write_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_PASSTHRU_REQ:
		if (((((tape_passthru_req_msg_t *)recv_msg)->flags & TAPE_PASSTHRU_FLAGS_DATA_OUT) &&
		     recv_len != sizeof(tape_passthru_req_msg_t) + ((tape_passthru_req_msg_t *)recv_msg)->data_len) ||
		    (!(((tape_passthru_req_msg_t *)recv_msg)->flags & TAPE_PASSTHRU_FLAGS_DATA_OUT) &&
		     recv_len != sizeof(tape_passthru_req_msg_t))) {
			TP_DBG(2, ("tape_receive_msg: Invalid PASSTHRU_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_passthru((tape_passthru_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case TAPE_MID_SET_DEBUG_LEVEL_REQ:
		if (recv_len != sizeof(tape_set_debug_level_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SET_DEBUG_LEVEL_REQ message length %d\n", recv_len));
			goto error_response;
		}
		tape_proc_set_debug_level((tape_set_debug_level_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case MC_MID_GET_DEVLIST_REQ:
		if (recv_len != sizeof(mc_get_devlist_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid DEVLIST_REQ message length %d\n", recv_len));
			goto error_response;
		}
		mc_proc_get_devlist((mc_get_devlist_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case MC_MID_OPEN_REQ:
		if (recv_len != sizeof(mc_open_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid OPEN_REQ message length %d\n", recv_len));
			goto error_response;
		}
		mc_proc_open((mc_open_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case MC_MID_GET_HANDLE_REQ:
		if (recv_len != sizeof(mc_get_handle_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid GET_HANDLE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		mc_proc_get_handle((mc_get_handle_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case MC_MID_CLOSE_REQ:
		if (recv_len != sizeof(mc_close_req_msg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid CLOSE_REQ message length %d\n", recv_len));
			goto error_response;
		}
		mc_proc_close((mc_close_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case MC_MID_PASSTHRU_REQ:
		if (((((mc_passthru_req_msg_t *)recv_msg)->flags & MC_PASSTHRU_FLAGS_DATA_OUT) &&
		     recv_len != sizeof(mc_passthru_req_msg_t) + ((mc_passthru_req_msg_t *)recv_msg)->data_len) ||
		    (!(((mc_passthru_req_msg_t *)recv_msg)->flags & MC_PASSTHRU_FLAGS_DATA_OUT) &&
		     recv_len != sizeof(mc_passthru_req_msg_t))) {
			TP_DBG(2, ("tape_receive_msg: Invalid PASSTHRU_REQ message length %d\n", recv_len));
			goto error_response;
		}
		mc_proc_passthru((mc_passthru_req_msg_t *)recv_msg, app, cpu, slot);
		break;

	case SDM_MID_DEV_OPEN_RSP:
		if (recv_len != sizeof(sdm_openRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_DEV_OPEN_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_open_rsp((sdm_openRspMsg_t *)recv_msg);
		break;

	case SDM_MID_DEV_INFO_RSP:
		if (recv_len != sizeof(sdm_devInfoRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_DEV_INFO_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_info_rsp((sdm_devInfoRspMsg_t *)recv_msg);
		break;

	case SDM_MID_RELAY_CDB_RSP:
		if (recv_len < sizeof(sdm_relayCdbRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_RELAY_CDB_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_relay_rsp((sdm_relayCdbRspMsg_t *)recv_msg);
		break;

	case SDM_MID_READ_RSP:
		if (recv_len < sizeof(sdm_readDataRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_READ_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_read_rsp((sdm_readDataRspMsg_t *)recv_msg);
		break;

	case SDM_MID_WRITE_RSP:
		if (recv_len != sizeof(sdm_writeDataRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_WRITE_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_write_rsp((sdm_writeDataRspMsg_t *)recv_msg);
		break;

	case SDM_MID_DEV_STATUS_RSP:
		if (recv_len != sizeof(sdm_devStatusRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_DEV_STATUS_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_dev_status_rsp((sdm_devStatusRspMsg_t *)recv_msg);
		break;

	case SDM_MID_DEV_CLOSE_RSP:
		if (recv_len != sizeof(sdm_closeRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_DEV_CLOSE_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_close_rsp((sdm_closeRspMsg_t *)recv_msg);
		break;

	case SDM_MID_ERR_RSP:
		if (recv_len != sizeof(sdm_errRspMsg_t)) {
			TP_DBG(2, ("tape_receive_msg: Invalid SDM_MID_ERR_RSP message length %d -- dropping\n", recv_len));
			goto drop_message;
		}
		tape_proc_sdm_err_rsp((sdm_errRspMsg_t *)recv_msg);
		break;
		
	default:
		TP_DBG(2, ("tape_receive_msg: Invalid mid 0x%x\n", recv_msg->mid));
	}

	return;

 error_response:
	resp.mid = TAPE_MID_ERROR_RSP;
	resp.cookie = recv_msg->cookie;
	tape_send_msg(&resp, sizeof(tape_msg_hdr_t), app, cpu, slot);
	return;

 drop_message:
	return;
}

void 
tape_proc_get_devlist(tape_get_devlist_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_get_devlist_rsp_msg_t *	resp;
	tape_dev_info_t *		dev_info;
	int				alloced_size = 0;
	int				used_size = 0;
	tape_dev_t *			dev;
	tape_dev_t *			next;
	int				dev_index;

	TP_DBG(2, ("tape_proc_get_devlist: app %s, cpu %d, slot %d\n",
		   app, cpu, slot));
	alloced_size = TAPE_MAX_PHYSICAL * 16 * sizeof(tape_dev_info_t);
	resp = (tape_get_devlist_rsp_msg_t *)malloc(alloced_size);
	resp->hdr.mid = TAPE_MID_GET_DEVLIST_RSP;
	resp->hdr.cookie = msg->hdr.cookie;
	resp->ecode = TAPE_ECODE_SUCCESS;
	resp->dev_count = 0;

	dev_info = resp->info;
	used_size = sizeof(tape_get_devlist_rsp_msg_t) - sizeof(tape_dev_info_t);

	dev_index = 0;
	if ((dev = tape_devs)) {
		do {
			next = dev->next;
			if (dev->mc == 0) {
				int rewind, unload, density;

				for (rewind = 0; rewind < 2; ++rewind) { 
					for (unload = 0; unload < 2; ++unload) { 
						for (density = 0; density < TAPE_DENSITY_COUNT; ++density) { 
							if (dev->alias < 0) {
								sprintf(dev_info->devicename, "%016llx_%s_%s_%s",
									dev->id, 
									rewind ? "R" : "NR",
									unload ? "U" : "NU",
									dev->cap->densities[density].dens_name);
							}
							else {
								sprintf(dev_info->devicename, "%s%s%d%s",
									rewind ? "R" : "NR",
									unload ? "U" : "NU",
									dev->alias,
									dev->cap->densities[density].alias_dens_name);
							}

							sprintf(dev_info->physical_devicename, "%016llx_%s_%s_%s",
								dev->id, 
								rewind ? "R" : "NR",
								unload ? "U" : "NU",
								dev->cap->densities[density].dens_name);

							strcpy(dev_info->scsi_vendor_id, dev->vendor_name);
							strcpy(dev_info->scsi_product_id, dev->model_name);
							dev_info->scsi_id = dev->id;
							dev_info->attributes = 
								(rewind ? TAPE_ATTRIB_REWIND_ON_CLOSE : 0) | 
								(unload ? TAPE_ATTRIB_UNLOAD_ON_CLOSE : 0) | 
								((density << TAPE_ATTRIB_DENSITY_SHIFT) & TAPE_ATTRIB_DENSITY_MASK);
							++resp->dev_count;
							++dev_info;
							used_size += sizeof(tape_dev_info_t);
						}
					}
				}
			}
			dev = next;
		} while (dev != tape_devs);
	}

	tape_send_msg(resp, used_size, app, cpu, slot);
	return;
}

void 
tape_proc_open(tape_open_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_open_rsp_msg_t resp;
	int		    rc;
	scsi_id_t	    id;
	uint32		    attributes;
	tape_dev_t *	    dev;
	tape_msg_context_t *context;

	TP_DBG(2, ("tape_proc_open: devicename %s, app %s, cpu %d, slot %d\n",
		   msg->devicename, app, cpu, slot));

	rc = tape_parse_devicename(msg->devicename, &id, &attributes);
	if (rc != NFX_OK) {
		/* Reject open -- invalid devicename */
		goto open_failed_no_device;
	}

	TP_DBG(2, ("tape_proc_open: parse succeeded : id 0x%016llx, attributes 0x%08x\n", id, attributes));
	dev = tape_device_lookup_by_id(id);
	if (dev == NULL) {
		/* Reject open -- no such physical device */
		goto open_failed_no_device;
	}
	
	if (dev->mc) {
		/* Reject open -- wrong device type */
		goto open_failed_wrong_device_type;
	}

	VERIFY(dev->id == id);
	context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
	context->private = attributes;
	tape_sm_run(dev, context, TAPE_EVENT_OPEN_REQ);

	return;

 open_failed_no_device:
	TP_DBG(2, ("tape_proc_open: open failed: No device %s\n", msg->devicename));
	resp.hdr.mid = TAPE_MID_OPEN_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(tape_open_rsp_msg_t), app, cpu, slot);
	return;

 open_failed_wrong_device_type:
	TP_DBG(2, ("tape_proc_open: open failed: Device %s is a TAPE device.\n", msg->devicename));
	resp.hdr.mid = TAPE_MID_OPEN_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(tape_open_rsp_msg_t), app, cpu, slot);
	return;
}

void tape_proc_get_handle(tape_get_handle_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_get_handle_rsp_msg_t resp;
	int			  rc;
	scsi_id_t		  id;
	uint32			  attributes;
	tape_dev_t *		  dev;
	int			  rewind, unload, density;

	TP_DBG(2, ("tape_proc_get_handle: devicename %s, app %s, cpu %d, slot %d\n",
		   msg->devicename, app, cpu, slot));

	rc = tape_parse_devicename(msg->devicename, &id, &attributes);
	if (rc != NFX_OK) {
		/* Reject open -- invalid devicename */
		goto get_handle_failed_no_device;
	}

	TP_DBG(2, ("tape_proc_get_handle: parse succeeded : id 0x%016llx, attributes 0x%08x\n", id, attributes));
	dev = tape_device_lookup_by_id(id);
	if (dev == NULL) {
		/* Reject open -- no such physical device */
		goto get_handle_failed_no_device;
	}
	
	VERIFY(dev->id == id);

	bzero(&resp, sizeof(tape_get_handle_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_GET_HANDLE_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	if (dev->state == TAPE_STATE_CLOSED || dev->open_handle <= 0)
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
	else
	if (dev->state != TAPE_STATE_OPEN)
		resp.ecode = TAPE_ECODE_DEVICE_BUSY;
	else {
		resp.ecode = TAPE_ECODE_SUCCESS;
		resp.handle = dev->open_handle;
		resp.scsi_id = dev->id;
		resp.wwn = dev->wwn;
		resp.lun = dev->lun;
		rewind = dev->open_attributes & TAPE_ATTRIB_REWIND_ON_CLOSE;
		unload = dev->open_attributes & TAPE_ATTRIB_UNLOAD_ON_CLOSE;
		density = (dev->open_attributes & TAPE_ATTRIB_DENSITY_MASK) >> TAPE_ATTRIB_DENSITY_SHIFT;
		if (dev->alias < 0) {
			sprintf(resp.devicename, "%016llx_%s_%s_%s",
				dev->id, 
				rewind ? "R" : "NR",
				unload ? "U" : "NU",
				dev->cap->densities[density].dens_name);
		}
		else {
			sprintf(resp.devicename, "%s%s%d%s",
				rewind ? "R" : "NR",
				unload ? "U" : "NU",
				dev->alias,
				dev->cap->densities[density].alias_dens_name);
		}
		sprintf(resp.physical_devicename, "%016llx_%s_%s_%s",
			dev->id, 
			rewind ? "R" : "NR",
			unload ? "U" : "NU",
			dev->cap->densities[density].dens_name);
	}
	tape_send_msg(&resp, sizeof(tape_get_handle_rsp_msg_t), app, cpu, slot);
	return;

 get_handle_failed_no_device:
	TP_DBG(2, ("tape_proc_get_handle: failed: No device %s\n", msg->devicename));
	bzero(&resp, sizeof(tape_get_handle_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_GET_HANDLE_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(tape_get_handle_rsp_msg_t), app, cpu, slot);
	return;
}

void 
tape_proc_close(tape_close_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_close_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	scsi_id_t		id;
	uint32			attributes;
	int			rc;

	TP_DBG(2, ("tape_proc_close: devicename %s, app %s, cpu %d, slot %d\n",
		   msg->devicename, app, cpu, slot));

	rc = tape_parse_devicename(msg->devicename, &id, &attributes);
	if (rc != NFX_OK) {
		/* Reject close -- invalid devicename */
		goto close_failed_no_device;
	}

	TP_DBG(2, ("tape_proc_close: parse succeeded : id 0x%016llx, attributes 0x%08x\n", id, attributes));
	dev = tape_device_lookup_by_id(id);
	if (dev == NULL) {
		/* Reject open -- no such physical device */
		goto close_failed_no_device;
	}

	context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
	if (msg->client_tag == dev->open_client_tag)
		tape_sm_run(dev, context, TAPE_EVENT_CLOSE_REQ);
	else
		tape_sm_run(dev, context, TAPE_EVENT_SERF_CLOSE_REQ);
	return;

 close_failed_no_device:
	TP_DBG(2, ("tape_proc_close: close failed: No device %s\n", msg->devicename));
	resp.hdr.mid = TAPE_MID_CLOSE_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(tape_close_rsp_msg_t), app, cpu, slot);
	return;
}

void
tape_proc_close_tag(tape_close_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
    tape_close_rsp_msg_t    resp;
    tape_dev_t *        dev;
    tape_msg_context_t *    context;

    TP_DBG(2, ("tape_proc_close_tag: client_tag %d, app %s, cpu %d, slot %d\n",
           msg->client_tag, app, cpu, slot));

    dev = tape_device_lookup_by_client_tag(msg->client_tag, msg->devicename);
    if (dev == NULL) {
        /* Reject open -- no such physical device */
        goto close_failed_no_device;
    }

    context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
    if (dev->mc)
        tape_sm_run(dev, context, MC_EVENT_CLOSE_REQ);
    else
        tape_sm_run(dev, context, TAPE_EVENT_CLOSE_REQ);
    return;

 close_failed_no_device:
    TP_DBG(2, ("tape_proc_close_tag: close : Device %s not open\n", msg->devicename));
    resp.hdr.mid = TAPE_MID_CLOSE_RSP;
    resp.hdr.cookie = msg->hdr.cookie;
    resp.ecode = TAPE_ECODE_NO_DEVICE;
    tape_send_msg(&resp, sizeof(tape_close_rsp_msg_t), app, cpu, slot);
    return;
}

void
tape_proc_get_state(tape_get_state_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_get_state_rsp_msg_t	resp;
	tape_dev_t *			dev;
	tape_msg_context_t *		context;

	TP_DBG(2, ("tape_proc_get_state: handle 0x%x, app %s, cpu %d, slot %d\n",
		   msg->handle, app, cpu, slot));

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		tape_sm_run(dev, context, TAPE_EVENT_GET_STATE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_get_state: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_GET_STATE_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_get_state_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_space(tape_space_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_space_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("tape_proc_space: handle 0x%x, opcode 0x%x, count %d, app %s, cpu %d, slot %d\n",
		   msg->handle, msg->opcode, msg->count, app, cpu, slot));

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)tape_proc_space_rsp_success;
		context->callback_failed = (void *)tape_proc_space_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_space: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_SPACE_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_space_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_space_rsp_success(tape_msg_context_t *context, sdm_relayCdbRspMsg_t *scsi_resp)
{
	tape_space_rsp_msg_t	resp;
	scsi_sense_t *		sense;

	TP_DBG(2, ("tape_proc_space_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	bzero(&resp, sizeof(tape_space_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_SPACE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;

	if ((scsi_resp->status & SD_DEV_STATUS_MASK) == 0) {
		/* Success */
		resp.ecode = TAPE_ECODE_SUCCESS;
	}
	else 
	if ((scsi_resp->status & SD_DEV_STATUS_MASK) != SCSI_DEV_STATUS_CHECK) { 
		/* General I/O error */
		TP_DBG(2, ("tape_proc_space_rsp_success: request failed: SCSI status 0x%x\n", scsi_resp->status & SD_DEV_STATUS_MASK));
		resp.ecode = TAPE_ECODE_IO_ERROR;
	}
	else {
		/* SCSI check condition */
		TP_DBG(2, ("tape_proc_space_rsp_success: SCSI check condition.\n"));
		VERIFY(scsi_resp->status & SD_DEV_SENSE_VALID);
		sense = (scsi_sense_t *)scsi_resp->senseData;
		switch (sense->senseKey & SCSI_SENSE_KEY_MASK) {
		case SCSI_SENSE_KEY_NO_SENSE:
			/* Parse ILI, EOM and FM bits */
			if (sense->senseKey & SCSI_SENSE_FILEMARK_BIT) {
				if (sense->respCode & SCSI_SENSE_VALID_BIT)
					resp.residue = ((uint32)sense->info[0] << 24) | 
						((uint32)sense->info[1] << 16) | 
						((uint32)sense->info[2] << 8) | 
						((uint32)sense->info[3]);
				resp.ecode = TAPE_ECODE_FM_ENCOUNTERED;
			}
			else
			if (sense->senseKey & SCSI_SENSE_EOM_BIT) {
				if (sense->respCode & SCSI_SENSE_VALID_BIT)
					resp.residue = ((uint32)sense->info[0] << 24) | 
						((uint32)sense->info[1] << 16) | 
						((uint32)sense->info[2] << 8) | 
						((uint32)sense->info[3]);
				resp.ecode = TAPE_ECODE_EOM_ENCOUNTERED;
			}
			break;

		case SCSI_SENSE_KEY_MEDIUM_ERROR:
			resp.ecode = TAPE_ECODE_MEDIUM_ERR;
			break;
		case SCSI_SENSE_KEY_BLANK_CHECK:
			resp.ecode = TAPE_ECODE_EOD_ENCOUNTERED;
			break;
		default:
			resp.ecode = TAPE_ECODE_IO_ERROR;
		}
	}

	tape_send_msg(&resp, sizeof(tape_space_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_space_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	tape_space_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_space_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(tape_space_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_SPACE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(tape_space_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_write_fms(tape_write_fm_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_write_fm_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("tape_proc_write_fms: handle 0x%x, count %d, app %s, cpu %d, slot %d\n",
		   msg->handle, msg->count, app, cpu, slot));

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)tape_proc_write_fms_rsp_success;
		context->callback_failed = (void *)tape_proc_write_fms_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_write_fms: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_WRITE_FM_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_space_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_write_fms_rsp_success(tape_msg_context_t *context, sdm_relayCdbRspMsg_t *scsi_resp)
{
	tape_write_fm_rsp_msg_t	resp;
	scsi_sense_t *		sense;

	TP_DBG(2, ("tape_proc_write_fms_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	bzero(&resp, sizeof(tape_write_fm_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_WRITE_FM_RSP;
	resp.hdr.cookie = context->req_msg->cookie;

	if ((scsi_resp->status & SD_DEV_STATUS_MASK) == 0) {
		/* Success */
		resp.ecode = TAPE_ECODE_SUCCESS;
	}
	else 
	if ((scsi_resp->status & SD_DEV_STATUS_MASK) != SCSI_DEV_STATUS_CHECK) { 
		/* General I/O error */
		TP_DBG(2, ("tape_proc_write_fms_rsp_success: request failed: SCSI status 0x%x\n", scsi_resp->status & SD_DEV_STATUS_MASK));
		resp.ecode = TAPE_ECODE_IO_ERROR;
	}
	else {
		/* SCSI check condition */
		TP_DBG(2, ("tape_proc_write_fms_rsp_success: SCSI check condition.\n"));
		VERIFY(scsi_resp->status & SD_DEV_SENSE_VALID);
		sense = (scsi_sense_t *)scsi_resp->senseData;
		switch (sense->senseKey & SCSI_SENSE_KEY_MASK) {
		case SCSI_SENSE_KEY_NO_SENSE:
			resp.ecode = TAPE_ECODE_SUCCESS;
			break;
		case SCSI_SENSE_KEY_MEDIUM_ERROR:
			resp.ecode = TAPE_ECODE_MEDIUM_ERR;
			break;
		case SCSI_SENSE_KEY_BLANK_CHECK:
			resp.ecode = TAPE_ECODE_EOD_ENCOUNTERED;
			break;
		default:
			resp.ecode = TAPE_ECODE_IO_ERROR;
		}
	}

	tape_send_msg(&resp, sizeof(tape_write_fm_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_write_fms_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	tape_write_fm_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_write_fms_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(tape_write_fm_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_WRITE_FM_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(tape_write_fm_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_load(tape_load_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_load_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("tape_proc_load: handle 0x%x, load_or_unload %d, app %s, cpu %d, slot %d\n",
		   msg->handle, msg->load_or_unload, app, cpu, slot));

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)tape_proc_load_rsp_success;
		context->callback_failed = (void *)tape_proc_load_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_load: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_LOAD_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_load_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_load_rsp_success(tape_msg_context_t *context, sdm_relayCdbRspMsg_t *scsi_resp)
{
	tape_load_rsp_msg_t	resp;
	scsi_sense_t *		sense;

	TP_DBG(2, ("tape_proc_load_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	bzero(&resp, sizeof(tape_load_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_LOAD_RSP;
	resp.hdr.cookie = context->req_msg->cookie;

	if ((scsi_resp->status & SD_DEV_STATUS_MASK) == 0) {
		/* Success */
		resp.ecode = TAPE_ECODE_SUCCESS;
	}
	else 
	if ((scsi_resp->status & SD_DEV_STATUS_MASK) != SCSI_DEV_STATUS_CHECK) { 
		/* General I/O error */
		TP_DBG(2, ("tape_proc_load_rsp_success: request failed: SCSI status 0x%x\n", scsi_resp->status & SD_DEV_STATUS_MASK));
		resp.ecode = TAPE_ECODE_IO_ERROR;
	}
	else {
		/* SCSI check condition */
		TP_DBG(2, ("tape_proc_load_rsp_success: SCSI check condition.\n"));
		VERIFY(scsi_resp->status & SD_DEV_SENSE_VALID);
		sense = (scsi_sense_t *)scsi_resp->senseData;
		switch (sense->senseKey & SCSI_SENSE_KEY_MASK) {
		case SCSI_SENSE_KEY_NO_SENSE:
			resp.ecode = TAPE_ECODE_SUCCESS;
			break;
		case SCSI_SENSE_KEY_MEDIUM_ERROR:
			resp.ecode = TAPE_ECODE_MEDIUM_ERR;
			break;
		case SCSI_SENSE_KEY_BLANK_CHECK:
			resp.ecode = TAPE_ECODE_EOD_ENCOUNTERED;
			break;
		default:
			resp.ecode = TAPE_ECODE_IO_ERROR;
		}
	}

	tape_send_msg(&resp, sizeof(tape_load_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_load_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	tape_load_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_load_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(tape_load_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_LOAD_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(tape_load_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_read(tape_read_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_read_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("tape_proc_read: handle 0x%x, bytecount %d, app %s, cpu %d, slot %d\n",
		   msg->handle, msg->bytecount, app, cpu, slot));

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)tape_proc_read_rsp_success;
		context->callback_failed = (void *)tape_proc_read_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_read: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_READ_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_read_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_read_rsp_success(tape_msg_context_t *context, sdm_readDataRspMsg_t *scsi_resp)
{
	tape_read_rsp_msg_t *	resp = NULL;
	int			resp_len = 0;
	tape_read_req_msg_t *	req = (tape_read_req_msg_t *)context->req_msg;

	TP_DBG(2, ("tape_proc_read_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	/* hack hack hack -- compute actual length of message, since struct is not packed */
	resp_len = (int)(&(resp->data));

	resp_len += scsi_resp->dataLen;
	resp_len = (resp_len < sizeof(tape_read_rsp_msg_t)) ? sizeof(tape_read_rsp_msg_t) : resp_len;
	resp = malloc(resp_len);
	bzero(resp, sizeof(tape_read_rsp_msg_t));
	resp->hdr.mid = TAPE_MID_READ_RSP;
	resp->hdr.cookie = context->req_msg->cookie;
	resp->bytecount = scsi_resp->dataLen;
	bcopy(scsi_resp->data, resp->data, scsi_resp->dataLen);

	switch(scsi_resp->eCode) {
	case SDM_OK:
		resp->ecode = TAPE_ECODE_SUCCESS;
		break;

	case SCSI_MSG_ECODE_MEDIUM_ERROR:
		resp->ecode = TAPE_ECODE_MEDIUM_ERR;
		break;

	case SCSI_MSG_ECODE_ILI:
		resp->ecode = TAPE_ECODE_ILI;
		resp->residue = req->bytecount - resp->bytecount;
		/* XXX note we don't currently deal with -ve residues */
		break;

	case SCSI_MSG_ECODE_FM_ENCOUNTERED:
		resp->ecode = TAPE_ECODE_FM_ENCOUNTERED;
		break;

	case SCSI_MSG_ECODE_END_OF_MEDIUM:
		resp->ecode = TAPE_ECODE_EOM_ENCOUNTERED;
		break;

	default:
		/* General I/O error */
		TP_DBG(2, ("tape_proc_read_rsp_success: request failed: SCSI eCode 0x%x\n", 
			   scsi_resp->eCode));
		resp->ecode = TAPE_ECODE_IO_ERROR;		
	}

	tape_send_msg(resp, resp_len,
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);

	free(resp);
}

void 
tape_proc_read_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	tape_read_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_read_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(tape_read_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_READ_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(tape_read_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_write(tape_write_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_write_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("tape_proc_write: handle 0x%x, bytecount %d, app %s, cpu %d, slot %d\n",
		   msg->handle, msg->bytecount, app, cpu, slot));

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)tape_proc_write_rsp_success;
		context->callback_failed = (void *)tape_proc_write_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_write: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_WRITE_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_write_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_write_rsp_success(tape_msg_context_t *context, sdm_writeDataRspMsg_t *scsi_resp)
{
	tape_write_rsp_msg_t resp;

	TP_DBG(2, ("tape_proc_write_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	bzero(&resp, sizeof(tape_write_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_WRITE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;

	switch(scsi_resp->eCode) {
	case SDM_OK:
		resp.ecode = TAPE_ECODE_SUCCESS;
		break;

	case SCSI_MSG_ECODE_MEDIUM_ERROR:
		resp.ecode = TAPE_ECODE_MEDIUM_ERR;
		break;

	case SCSI_MSG_ECODE_ILI:
		resp.ecode = TAPE_ECODE_ILI;
		break;

	case SCSI_MSG_ECODE_FM_ENCOUNTERED:
		resp.ecode = TAPE_ECODE_FM_ENCOUNTERED;
		break;

	case SCSI_MSG_ECODE_END_OF_MEDIUM:
		resp.ecode = TAPE_ECODE_EOM_ENCOUNTERED;
		break;

	default:
		/* General I/O error */
		TP_DBG(2, ("tape_proc_write_rsp_success: request failed: SCSI eCode 0x%x\n", 
			   scsi_resp->eCode));
		resp.ecode = TAPE_ECODE_IO_ERROR;		
	}

	tape_send_msg(&resp, sizeof(tape_write_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_write_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	tape_write_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_write_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(tape_write_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_WRITE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(tape_write_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_passthru(tape_passthru_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_passthru_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("tape_proc_passthru: handle 0x%x, app %s, cpu %d, slot %d\n",
		   msg->handle, app, cpu, slot));
#ifdef DEBUG
	{
		int i;

		TP_DBG(2, ("tape_proc_passthru: CDB: "));
		for (i = 0; i < 16; ++i)
			TP_DBG(2, ("%02x ", msg->cdb[i]));
		TP_DBG(2, ("\n"));
	}
#endif

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)tape_proc_passthru_rsp_success;
		context->callback_failed = (void *)tape_proc_passthru_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("tape_proc_passthru: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = TAPE_MID_PASSTHRU_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(tape_passthru_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
tape_proc_passthru_rsp_success(tape_msg_context_t *context, sdm_relayCdbRspMsg_t *scsi_resp)
{
	tape_passthru_rsp_msg_t *	resp = NULL;
	int				resp_len = 0;

	TP_DBG(2, ("tape_proc_passthru_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	/* hack hack hack -- compute actual length of message, since struct is not packed */
	resp_len = (int)(&(resp->data));

	resp_len += scsi_resp->dataLen;
	resp_len = (resp_len < sizeof(tape_passthru_rsp_msg_t)) ? sizeof(tape_passthru_rsp_msg_t) : resp_len;
	resp = malloc(resp_len);

	bzero(resp, sizeof(tape_passthru_rsp_msg_t));
	resp->hdr.mid = TAPE_MID_PASSTHRU_RSP;
	resp->hdr.cookie = context->req_msg->cookie;

	/* Set SCSI status */
	resp->scsi_status = scsi_resp->status & SD_DEV_STATUS_MASK;

	/* Copy sense, if necessary */
	if (resp->scsi_status == SCSI_DEV_STATUS_CHECK)
		bcopy(scsi_resp->senseData, resp->sense_data, sizeof(scsi_resp->senseData));

	/* Copy data in, if necessary */
	if (((tape_passthru_req_msg_t *)context->req_msg)->flags & TAPE_PASSTHRU_FLAGS_DATA_IN) {
		resp->data_len = scsi_resp->dataLen;
		bcopy(scsi_resp->data, resp->data, scsi_resp->dataLen);
	}
	else {
		/* Assume all data-out data transmitted */
		resp->data_len = ((tape_passthru_req_msg_t *)context->req_msg)->data_len;
	}

	resp->ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(resp, resp_len,
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);

	free(resp);
}

void 
tape_proc_passthru_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	tape_passthru_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_passthru_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(tape_passthru_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_PASSTHRU_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(tape_passthru_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_set_debug_level(tape_set_debug_level_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_get_state_rsp_msg_t	resp;

	TP_DBG(2, ("tape_proc_set_debug_level: debug_level 0x%x, app %s, cpu %d, slot %d\n",
		   msg->debug_level, app, cpu, slot));

	tape_dbg_level = msg->debug_level;

	resp.hdr.mid = TAPE_MID_SET_DEBUG_LEVEL_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	tape_send_msg(&resp, sizeof(tape_set_debug_level_rsp_msg_t), app, cpu, slot);
}

void
mc_proc_get_devlist(mc_get_devlist_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	mc_get_devlist_rsp_msg_t *	resp;
	mc_dev_info_t *			dev_info;
	int				alloced_size = 0;
	int				used_size = 0;
	tape_dev_t *			dev;
	tape_dev_t *			next;

	TP_DBG(2, ("mc_proc_get_devlist: app %s, cpu %d, slot %d\n",
		   app, cpu, slot));
	alloced_size = 1024 * 16;
	resp = (mc_get_devlist_rsp_msg_t *)malloc(alloced_size);
	resp->hdr.mid = MC_MID_GET_DEVLIST_RSP;
	resp->hdr.cookie = msg->hdr.cookie;
	resp->ecode = TAPE_ECODE_SUCCESS;
	resp->dev_count = 0;

	dev_info = resp->info;
	used_size = sizeof(mc_get_devlist_rsp_msg_t) - sizeof(mc_dev_info_t);

	if ((dev = tape_devs)) {
		do {
			next = dev->next;
			if (dev->mc == 1) {
				if (dev->alias < 0)
					sprintf(dev_info->devicename, "%016llx", dev->id);
				else
					sprintf(dev_info->devicename, "mc%d", dev->alias);
				sprintf(dev_info->physical_devicename, "%016llx", dev->id);
				strcpy(dev_info->scsi_vendor_id, dev->vendor_name);
				strcpy(dev_info->scsi_product_id, dev->model_name);
				dev_info->scsi_id = dev->id;
				
				++resp->dev_count;
				++dev_info;
				used_size += sizeof(mc_dev_info_t);
			}
			dev = next;
		} while (dev != tape_devs);
	}
	
	tape_send_msg(resp, used_size, app, cpu, slot);
	return;
}

void 
mc_proc_open(mc_open_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	mc_open_rsp_msg_t   resp;
	int		    rc;
	scsi_id_t	    id;
	tape_dev_t *	    dev;
	tape_msg_context_t *context;

	TP_DBG(2, ("mc_proc_open: devicename %s, app %s, cpu %d, slot %d\n",
		   msg->devicename, app, cpu, slot));

	rc = mc_parse_devicename(msg->devicename, &id);
	if (rc != NFX_OK) {
		/* Reject open -- invalid devicename */
		goto open_failed_no_device;
	}

	TP_DBG(2, ("mc_proc_open: parse succeeded : id 0x%016llx\n", id));
	dev = tape_device_lookup_by_id(id);
	if (dev == NULL) {
		/* Reject open -- no such physical device */
		goto open_failed_no_device;
	}
	
	if (!dev->mc) {
		/* Reject open -- wrong device type */
		goto open_failed_wrong_device_type;
	}

	VERIFY(dev->id == id);
	context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
	tape_sm_run(dev, context, TAPE_EVENT_OPEN_REQ);

	return;

 open_failed_no_device:
	TP_DBG(2, ("mc_proc_open: open failed: No device %s\n", msg->devicename));
	resp.hdr.mid = MC_MID_OPEN_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(mc_open_rsp_msg_t), app, cpu, slot);
	return;

 open_failed_wrong_device_type:
	TP_DBG(2, ("mc_proc_open: open failed: Device %s is an MC device.\n", msg->devicename));
	resp.hdr.mid = MC_MID_OPEN_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(mc_open_rsp_msg_t), app, cpu, slot);
	return;
}

void
mc_proc_get_handle(mc_get_handle_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	mc_get_handle_rsp_msg_t   resp;
	int			  rc;
	scsi_id_t		  id;
	tape_dev_t *		  dev;

	TP_DBG(2, ("mc_proc_get_handle: devicename %s, app %s, cpu %d, slot %d\n",
		   msg->devicename, app, cpu, slot));

	rc = mc_parse_devicename(msg->devicename, &id);
	if (rc != NFX_OK) {
		/* Reject open -- invalid devicename */
		goto get_handle_failed_no_device;
	}

	TP_DBG(2, ("mc_proc_get_handle: parse succeeded : id 0x%016llx\n", id));
	dev = tape_device_lookup_by_id(id);
	if (dev == NULL) {
		/* Reject open -- no such physical device */
		goto get_handle_failed_no_device;
	}
	
	VERIFY(dev->id == id);

	bzero(&resp, sizeof(mc_get_handle_rsp_msg_t));
	resp.hdr.mid = MC_MID_GET_HANDLE_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	if (dev->state == TAPE_STATE_CLOSED || dev->open_handle <= 0)
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
	else
	if (dev->state != TAPE_STATE_OPEN)
		resp.ecode = TAPE_ECODE_DEVICE_BUSY;
	else {
		resp.hdr.mid = MC_MID_GET_HANDLE_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_SUCCESS;
		resp.handle = dev->open_handle;
		resp.scsi_id = dev->id;
		resp.wwn = dev->wwn;
		resp.lun = dev->lun;
		if (dev->alias < 0)
			sprintf(resp.devicename, "%016llx", dev->id);
		else
			sprintf(resp.devicename, "scsi%d", dev->alias);
		sprintf(resp.physical_devicename, "%016llx", dev->id);
	}
	tape_send_msg(&resp, sizeof(mc_get_handle_rsp_msg_t), app, cpu, slot);
	return;

 get_handle_failed_no_device:
	TP_DBG(2, ("mc_proc_get_handle: open failed: No device %s\n", msg->devicename));
	bzero(&resp, sizeof(tape_get_handle_rsp_msg_t));
	resp.hdr.mid = MC_MID_GET_HANDLE_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(mc_get_handle_rsp_msg_t), app, cpu, slot);
	return;
}

void 
mc_proc_close(mc_close_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	mc_close_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	scsi_id_t		id;
	int			rc;

	TP_DBG(2, ("mc_proc_close: devicename %s, app %s, cpu %d, slot %d\n",
		   msg->devicename, app, cpu, slot));

	rc = mc_parse_devicename(msg->devicename, &id);
	if (rc != NFX_OK) {
		/* Reject close -- invalid devicename */
		goto close_failed_no_device;
	}

	TP_DBG(2, ("mc_proc_close: parse succeeded : id 0x%016llx\n", id));
	dev = tape_device_lookup_by_id(id);
	if (dev == NULL) {
		/* Reject open -- no such physical device */
		goto close_failed_no_device;
	}

	context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
	if (msg->client_tag == dev->open_client_tag)
		tape_sm_run(dev, context, MC_EVENT_CLOSE_REQ);
	else
		tape_sm_run(dev, context, MC_EVENT_SERF_CLOSE_REQ);

	return;

 close_failed_no_device:
	TP_DBG(2, ("mc_proc_close: close failed: No device %s\n", msg->devicename));
	resp.hdr.mid = MC_MID_CLOSE_RSP;
	resp.hdr.cookie = msg->hdr.cookie;
	resp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&resp, sizeof(tape_close_rsp_msg_t), app, cpu, slot);
	return;
}

void 
mc_proc_passthru(mc_passthru_req_msg_t *msg, char *app, uint32 cpu, uint32 slot)
{
	mc_passthru_rsp_msg_t	resp;
	tape_dev_t *		dev;
	tape_msg_context_t *	context;
	
	TP_DBG(2, ("mc_proc_passthru: handle 0x%x, app %s, cpu %d, slot %d\n",
		   msg->handle, app, cpu, slot));

#ifdef DEBUG
	{
		int i;

		TP_DBG(2, ("mc_proc_passthru: CDB: "));
		for (i = 0; i < 16; ++i)
			TP_DBG(2, ("%02x ", msg->cdb[i]));
		TP_DBG(2, ("\n"));
	}
#endif

	dev = tape_device_lookup_by_handle(msg->handle);
	if (dev) {
		VERIFY(dev->open_handle == msg->handle);
		context = tape_save_context((tape_msg_hdr_t *)msg, app, cpu, slot);
		context->callback_success = (void *)mc_proc_passthru_rsp_success;
		context->callback_failed = (void *)mc_proc_passthru_rsp_failed;
		tape_sm_run(dev, context, TAPE_EVENT_MESSAGE_REQ);
		return;
	}
	else {
		/* Reject request -- invalid handle */
		TP_DBG(2, ("mc_proc_passthru: request failed: Invalid handle 0x%x\n", msg->handle));
		resp.hdr.mid = MC_MID_PASSTHRU_RSP;
		resp.hdr.cookie = msg->hdr.cookie;
		resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
		tape_send_msg(&resp, sizeof(mc_passthru_rsp_msg_t), app, cpu, slot);
		return;
	}
}

void 
mc_proc_passthru_rsp_success(tape_msg_context_t *context, sdm_relayCdbRspMsg_t *scsi_resp)
{
	mc_passthru_rsp_msg_t *	resp = NULL;
	int			resp_len = 0;

	TP_DBG(2, ("mc_proc_passthru_rsp_success: context 0x%p, scsi_resp 0x%p\n",
		   context, scsi_resp));

	/* hack hack hack -- compute actual length of message, since struct is not packed */
	resp_len = (int)(&(resp->data));

	resp_len += scsi_resp->dataLen;
	resp_len = (resp_len < sizeof(mc_passthru_rsp_msg_t)) ? sizeof(mc_passthru_rsp_msg_t) : resp_len;
	resp = malloc(resp_len);

	bzero(resp, sizeof(mc_passthru_rsp_msg_t));
	resp->hdr.mid = MC_MID_PASSTHRU_RSP;
	resp->hdr.cookie = context->req_msg->cookie;

	/* Set SCSI status */
	resp->scsi_status = scsi_resp->status & SD_DEV_STATUS_MASK;

	/* Copy sense, if necessary */
	if (resp->scsi_status == SCSI_DEV_STATUS_CHECK)
		bcopy(scsi_resp->senseData, resp->sense_data, sizeof(scsi_resp->senseData));

	/* Copy data in, if necessary */
	if (((mc_passthru_req_msg_t *)context->req_msg)->flags & MC_PASSTHRU_FLAGS_DATA_IN) {
		resp->data_len = scsi_resp->dataLen;
		bcopy(scsi_resp->data, resp->data, scsi_resp->dataLen);
	}
	else {
		/* Assume all data-out data transmitted */
		resp->data_len = ((mc_passthru_req_msg_t *)context->req_msg)->data_len;
	}

	resp->ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(resp, resp_len,
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);

	free(resp);
}

void 
mc_proc_passthru_rsp_failed(tape_msg_context_t *context, uint32 ecode)
{
	mc_passthru_rsp_msg_t	resp;

	TP_DBG(2, ("mc_proc_passthru_rsp_failed: context 0x%p, ecode 0x%x\n",
		   context, ecode));

	bzero(&resp, sizeof(mc_passthru_rsp_msg_t));
	resp.hdr.mid = MC_MID_PASSTHRU_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = ecode;

	tape_send_msg(&resp, sizeof(mc_passthru_rsp_msg_t), 
		      context->req_src_app, 
		      context->req_src_cpu, 
		      context->req_src_slot);
}

void 
tape_proc_sdm_open_rsp(sdm_openRspMsg_t *msg)
{
	tape_dev_t *	dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_open_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_open_rsp: rCode 0x%08x\n", msg->rCode));

	if (msg->rCode == SDM_OK)
		tape_sm_run(dev, msg, (dev->mc ? MC_EVENT_SDM_OPEN_DONE : TAPE_EVENT_SDM_OPEN_DONE));
	else
	if (msg->rCode == SDM_TO_MANY_OPENS)
		tape_sm_run(dev, msg, (dev->mc ? TAPE_EVENT_SDM_OPEN_ALREADY_OPEN : TAPE_EVENT_SDM_OPEN_ALREADY_OPEN));
	else
		tape_sm_run(dev, msg, (dev->mc ? MC_EVENT_SDM_OPEN_FAILED : TAPE_EVENT_SDM_OPEN_FAILED));
}

void 
tape_proc_sdm_info_rsp(sdm_devInfoRspMsg_t *msg)
{
	tape_dev_t *	dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_info_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_info_rsp: rc 0x%08x\n", msg->rc));

	if (msg->rc == SDM_OK)
		tape_sm_run(dev, msg, (dev->mc ? MC_EVENT_SDM_INFO_DONE : TAPE_EVENT_SDM_INFO_DONE));
	else
		tape_sm_run(dev, msg, (dev->mc ? MC_EVENT_SDM_INFO_FAILED : TAPE_EVENT_SDM_INFO_FAILED));
}

void 
tape_proc_sdm_relay_rsp(sdm_relayCdbRspMsg_t *msg)
{
	tape_dev_t *		dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_relay_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	if (msg->eCode == SDM_FAIL || (msg->status & SD_DRV_STATUS_MASK)) {
		TP_DBG(2, ("tape_proc_sdm_relay_rsp: Failed: eCode 0x%x, driver status 0x%08x\n", 
			   msg->eCode, msg->status));
		tape_sm_run(dev, msg, TAPE_EVENT_SDM_RELAY_FAILED);
	}
	else 
	if ((msg->status & SD_DEV_STATUS_MASK) == 0) {
		TP_DBG(2, ("tape_proc_sdm_relay_rsp: Success.\n"));
		tape_sm_run(dev, msg, TAPE_EVENT_SDM_RELAY_DONE);
	}
	else {
		TP_DBG(2, ("tape_proc_sdm_relay_rsp: device status 0x%08x\n", msg->status));
#ifdef DEBUG
		if (msg->status & SD_DEV_SENSE_VALID) {
			if (tape_dbg_level >= 2) {
				int i;

				for (i = 0; i < 32; ++i) {
					if ((i % 8) == 0) {
						if (i > 0)
							printf("\n");
						printf("tape_proc_sdm_relay_rsp: sense ");
					}
					printf("%02x ", msg->senseData[i]);
				}
				printf("\n");
			}
		}
#endif /* DEBUG */
		tape_sm_run(dev, msg, TAPE_EVENT_SDM_RELAY_SCSI_ERROR);
	}
}

void 
tape_proc_sdm_read_rsp(sdm_readDataRspMsg_t *msg)
{
	tape_dev_t *		dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_read_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_read_rsp: eCode 0x%08x, dataLen %d\n", msg->eCode, msg->dataLen));

	tape_sm_run(dev, msg, TAPE_EVENT_SDM_READ_DONE);

	/* XXX need to complete checking for error completions */
}

void 
tape_proc_sdm_write_rsp(sdm_writeDataRspMsg_t *msg)
{
	tape_dev_t *		dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_write_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_write_rsp: eCode 0x%08x\n", msg->eCode));

	tape_sm_run(dev, msg, TAPE_EVENT_SDM_WRITE_DONE);

	/* XXX need to complete checking for error completions */
}

void 
tape_proc_sdm_dev_status_rsp(sdm_devStatusRspMsg_t *msg)
{
	tape_dev_t *		dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_dev_status_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_dev_status_rsp: eCode 0x%08x\n", msg->eCode));

	tape_sm_run(dev, msg, TAPE_EVENT_SDM_DEV_STATUS_DONE);
}

void 
tape_proc_sdm_close_rsp(sdm_closeRspMsg_t *msg)
{
	tape_dev_t *	dev;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_close_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_close_rsp: rCode 0x%08x\n", msg->rCode));

	if (msg->rCode == SDM_OK)
		tape_sm_run(dev, msg, (dev->mc ? MC_EVENT_SDM_CLOSE_DONE : TAPE_EVENT_SDM_CLOSE_DONE));
	else
		tape_sm_run(dev, msg, (dev->mc ? MC_EVENT_SDM_CLOSE_DONE : TAPE_EVENT_SDM_CLOSE_FAILED));
}

void 
tape_proc_sdm_err_rsp(sdm_errRspMsg_t *msg)
{
	tape_dev_t *	dev;
	sdm_msgHdr_t *	req;

	dev = tape_chandle_to_device(msg->hdr.chandle);
	if (dev == NULL) {
		TP_DBG(0, ("tape_proc_sdm_err_rsp: unrecognized chandle 0x%x: dropping message.\n", 
			   msg->hdr.chandle));
		return;
	}

	TP_DBG(2, ("tape_proc_sdm_err_rsp: eCode 0x%08x\n", msg->eCode));

	req = dev->pending_req_context;
	if (req == NULL) {
		TP_DBG(0, ("tape_proc_sdm_err_rsp: No request context -- dropping\n"));
		return;
	}

	switch (req->mid) {
	case SDM_MID_WRITE_REQ:
		tape_sm_run(dev, msg, TAPE_EVENT_SDM_WRITE_FAILED);
		break;

	case SDM_MID_READ_REQ:
		tape_sm_run(dev, msg, TAPE_EVENT_SDM_READ_FAILED);
		break;

	case SDM_MID_RELAY_CDB_REQ:
		tape_sm_run(dev, msg, TAPE_EVENT_SDM_RELAY_FAILED);
		break;

    case SDM_MID_DEV_STATUS_REQ:
        tape_sm_run(dev, msg, TAPE_EVENT_SDM_DEV_STATUS_FAILED);
        break;

    case SDM_MID_DEV_OPEN_REQ:
        tape_sm_run(dev, msg, TAPE_EVENT_SDM_OPEN_FAILED);
        break;

    case SDM_MID_DEV_CLOSE_REQ:
        tape_sm_run(dev, msg, TAPE_EVENT_SDM_CLOSE_FAILED);
        break;

	default: 
		VERIFY(0);
	}
}

void
tape_send_msg(void *send_msg, int send_len, char *app, uint32 cpu, uint32 slot)
{
	int rc;

	TP_DBG(2, ("tape_send_msg: sending msg: "
		   "%d bytes to app %s, cpu %d, slot %d\n",
		   send_len, app, cpu, slot));
	
	rc = sendAgileMsg(tape_rxif,
			  send_msg, send_len, 
			  app, cpu, slot, 0, 
			  NULL, 
			  NULL, 0);
}

int
tape_get_alias(scsi_id_t scsi_id, int32 *alias)
{

#define OPTIONS_TAPE_ALIAS "options.tape.alias"
#define MAX_ALIASES        128
#define REGM_TIMEOUT	   5

	struct timeval		to;
	opt_msgHdr_t *		hdr;
	opt_getOptionReq_t *	get_req;
	opt_setOptionReq_t *	set_req;
	int32			req_len;
	char *			req_buf[1024];
	opt_getOptionRsp_t *	get_rsp;
	opt_setOptionRsp_t *	set_rsp;
	int32			rsp_len;
	int32			rsp_alloc_len = 1024;
	char *			*rsp_buf = NULL;
	int32			i;
	int			entry_count;
	int			alias_mask[MAX_ALIASES];
	char			tmp_option[128];

	to.tv_sec = REGM_TIMEOUT;
	to.tv_usec = 0;

	bzero(alias_mask, sizeof(alias_mask));

	hdr = (opt_msgHdr_t *)req_buf;
	get_req = (opt_getOptionReq_t *)(hdr + 1);

	hdr->msgType = OPT_GET_OPTION_REQ;
	hdr->userInfo = 0;

	sprintf(get_req->option, "%s.*", OPTIONS_TAPE_ALIAS);

	req_len = sizeof(opt_msgHdr_t) + sizeof(opt_getOptionReq_t);
	rsp_buf = malloc(rsp_alloc_len);
	while (1) {
		rsp_len = sendAgileMsg(-1,
				       req_buf, req_len,
				       "regm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				       &to,
				       rsp_buf, rsp_alloc_len);
		if (rsp_len < 0) {
			TP_DBG(0, ("tape_get_alias: sendAgileMsg failed\n"));
			free(rsp_buf);
			return(NFX_ERR);
		}
		if (rsp_len == 0) {
			TP_DBG(0, ("tape_get_alias: sendAgileMsg to registryMgr timed out\n"));			
			free(rsp_buf);
			return(NFX_ERR);
		}
		hdr = (opt_msgHdr_t *)rsp_buf;
		if (hdr->msgType != OPT_GET_OPTION_RSP) {
			TP_DBG(0, ("tape_get_alias: Invalid MID 0x%x (expected 0x%x)\n", hdr->msgType, OPT_GET_OPTION_RSP));
			free(rsp_buf);
			return(NFX_ERR);
		}
		get_rsp = (opt_getOptionRsp_t *)(hdr + 1);
		if (rsp_len < (sizeof(opt_msgHdr_t) + get_rsp->nbEntry * sizeof(opt_entry_t))) {
			free(rsp_buf);
			rsp_alloc_len *= 2;
			rsp_buf = malloc(rsp_alloc_len);
		}
		else
			break;
	}

	entry_count = get_rsp->nbEntry;
	for (i = 0; i < entry_count; ++i) {
		TP_DBG(2, ("tape_get_alias: Entry %d: option '%s' = '%s'\n",
			   i, get_rsp->entry[i].option, get_rsp->entry[i].value));
		*alias = atol(get_rsp->entry[i].value);
		alias_mask[*alias] = 1;
		sprintf(tmp_option, "%s.%016llx", OPTIONS_TAPE_ALIAS, scsi_id);
		if (strcmp(get_rsp->entry[i].option, tmp_option) == 0)
			break;
	}
	
	if (i < entry_count) {
		TP_DBG(2, ("tape_get_alias: Alias for %016llx found -- %d\n", scsi_id, *alias));
	}
	else {
		*alias = -1;
		for (i = 0; i < MAX_ALIASES; ++i) {
			if (alias_mask[i] == 0) {
				*alias = i;
				TP_DBG(2, ("tape_get_alias: Alias for %016llx not found -- assigning %d\n", scsi_id, *alias));
				break;
			}
		}

		if (i == MAX_ALIASES) {
			TP_DBG(0, ("tape_get_alias: MAX_ALIASES exceeded\n"));
			free(rsp_buf);
			return(NFX_ERR);
		}

		hdr = (opt_msgHdr_t *)req_buf;
		set_req = (opt_setOptionReq_t *)(hdr + 1);

		hdr->msgType = OPT_SET_OPTION_REQ;
		hdr->userInfo = 0;

		set_req->del = 0;
		sprintf(set_req->entry.option, "%s.%016llx", OPTIONS_TAPE_ALIAS, scsi_id);
		sprintf(set_req->entry.value, "%d", *alias);

		req_len = sizeof(opt_msgHdr_t) + sizeof(opt_setOptionReq_t);
		rsp_len = sendAgileMsg(-1,
				       req_buf, req_len,
				       "regm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				       &to,
				       rsp_buf, rsp_alloc_len);
		if (rsp_len < 0) {
			TP_DBG(0, ("tape_get_alias: sendAgileMsg failed\n"));
			free(rsp_buf);
			return(NFX_ERR);
		}
		if (rsp_len == 0) {
			TP_DBG(0, ("tape_get_alias: sendAgileMsg to registryMgr timed out\n"));			
			free(rsp_buf);
			return(NFX_ERR);
		}

		hdr = (opt_msgHdr_t *)rsp_buf;
		if (hdr->msgType != OPT_SET_OPTION_RSP) {
			TP_DBG(0, ("tape_get_alias: Invalid MID 0x%x (expected 0x%x)\n", hdr->msgType, OPT_SET_OPTION_RSP));
			free(rsp_buf);
			return(NFX_ERR);
		}
		set_rsp = (opt_setOptionRsp_t *)(hdr + 1);
		if (set_rsp->result != NFX_OK) {
			TP_DBG(0, ("OPT_SET_OPTION_REQ failed\n"));
			free(rsp_buf);
			return(NFX_ERR);
		}
	}
		
	free(rsp_buf);
	if (*alias >= 0)
		return(NFX_OK);
	else
		return(NFX_ERR);
}

