/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tftp-api.h
 *
 * RCS:  $Id: tftp-api.h,v 1.1.1.1 2001/10/12 03:52:49 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains TFTP protocol API definitions & prototypes.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 09/15/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _TFTP_API_H
#define _TFTP_API_H


#define TFTP_ERRCODE_UNDEFINED			0
#define TFTP_ERRCODE_FILE_NOT_FOUND		1
#define TFTP_ERRCODE_ACCESS				2
#define TFTP_ERRCODE_DISK_FULL			3
#define TFTP_ERRCODE_ILLEGAL_OP			4
#define TFTP_ERRCODE_UNKNOW_TID			5
#define TFTP_ERRCODE_FILE_EXISTS		6
#define TFTP_ERRCODE_NO_SUCH_USER		7
#define TFTP_ERRCODE_MASK				7

#define TFTP_ERR_BASE				0x100000
#define TFTP_ERR_NOT_INIT			(TFTP_ERR_BASE-1)
#define TFTP_ERR_NO_RAM_RSRC		(TFTP_ERR_NOT_INIT-1)
#define TFTP_ERR_NO_TIMER			(TFTP_ERR_NO_RAM_RSRC-1)
#define TFTP_ERR_NO_FUNC			(TFTP_ERR_NO_TIMER-1)
#define TFTP_ERR_BIND_FAIL			(TFTP_ERR_NO_FUNC-1)

#define TFTP_EVT_TIMEOUT			0xFD	/* TFTP timeout event */
#define TFTP_EVT_ERROR				0xFE	/* TFTP error event */
#define TFTP_EVT_EOF				0xFF	/* TFTP EOF event */

#define TFTP_MODE_BIN				0		/* binary mode xfer */
#define TFTP_MODE_ASCII				1		/* ascii mode xfer */

/*-----------------------------------------------------------------
 * Name:        tftp_init
 *
 * Description: Initialize TFTP module related data structure.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/15/01
 *-----------------------------------------------------------------
 */
void tftp_init( void );

/*-----------------------------------------------------------------
 * Name:        tftp_open
 *
 * Description: Open an TFTP connection for file xfer.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/15/01
 *-----------------------------------------------------------------
 */
void *	/* NULL if failed, or ptr to TftpCB */
tftp_open(char    *filename,						/* filename */
		  uint32  srvrIp,							/* tftp IP address */
          int32   (*ul_dataInd)(char  *pData, 		/* RxData callback */
								int32  size ), 
          int32   (*ul_eventInd)(void *tftpCb, 		/* Event callback */
								int32 event) );	

/*-----------------------------------------------------------------
 * Name:        tftp_close
 *
 * Description: Close & release TFTP related resource.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/15/01
 *-----------------------------------------------------------------
 */
void tftp_close( void *tftpCb );


#endif /* _TFTP_API_H */
