/*
 *-----------------------------------------------------------------
 *
 * Name:	timer-ui.c
 *
 * RCS:  $Id: timer-ui.c,v 1.6 2002/11/22 21:11:30 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	UI interface functions for timer module
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/23/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/queue-api.h>
#include <exports/timer-api.h>
#include "timer.h"

/*-----------------------------------------------------------------
 * Name:	timer_showStats()
 *
 * Description:	Displays general timer info
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/29/01
 *
 *-----------------------------------------------------------------
 */
void
timer_showStats()
{
	uint32			i;
	eeeRTC_t		exp_time;
	timer_block_t	* tmr;

	eee_rtc(&exp_time);
	printf("Current Timer Tic = %lld\n", eee_rtcToMS(&exp_time));

	NFX_SPIN_LOCK(&timer_spin);
	printf("hndl     dur   exp_tic    type     owner        p1        p2        p3       p4\n");
	printf("----  ------  --------  ------  --------  --------  --------  --------  --------\n");
	TAILQ_FOREACH(tmr, &timers, tb_entry) {
		printf("%4d  %6lld  %8lld  %s  %p  %p  %p  %p  %p\n",
			   tmr->tmr_handle, eee_rtcToMS(&tmr->duration),
			   eee_rtcToMS(&tmr->tb_expiry_time),
			   (char *)((tmr->type == EEE_TIMER_ONCE) ? "  ONCE" : "REPEAT"), 
			   (void *)tmr->owner,
			   tmr->usr_p1, tmr->usr_p2,
			   tmr->usr_p3, tmr->usr_p4);
	}
	printf("----  ------  --------  ------  --------  --------  --------  --------  --------\n");
	printf("\n\nTimers allocated but not running:\n");
	for (i=0; i<TIMER_NUM_TIMER_BLOCKS; i++) {
		if ((eee_timerGetAllocState(i) == TIMER_STATE_ALLOC) && 
				(eee_timerGetRunningState(i) == TIMER_STATE_STOPPED)) {
			tmr = &(tmr_block[i]);
			printf("    %2d (%p)\n", i, (void *)tmr->owner);
		}
	}
	NFX_SPIN_UNLOCK(&timer_spin);
	printf("\n");

}	/* timer_showStats() */

