/*
 *-----------------------------------------------------------------
 *
 * Name:	watchdog.c
 *
 * RCS:  $Id: watchdog.c,v 1.8 2002/05/13 18:33:44 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

uint32	watchdog_state CPU_PRIVATE;
uint32	watchdog_expiry CPU_PRIVATE;
uint32	watchdog_state_save CPU_PRIVATE;
uint32	watchdog_expiry_save CPU_PRIVATE;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_init()
{
	wd_init();
	watchdog_enable(8);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_kick()
{
	wd_kick();
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_enable(
	int32	nsec
)
{
	watchdog_expiry = nsec;

	wd_enable(watchdog_expiry);

	watchdog_state = 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_disable()
{
	wd_disable();

	watchdog_state = 0;
}


/*-----------------------------------------------------------------
 * Name :	watchdog_stop()
 *
 * Description: Temporary stop the  watchdog, saving the state so it can be
 * restarted  with watchdog_restart()
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	05/10/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_stop(void)
{
	watchdog_state_save = watchdog_state;
	if (watchdog_state_save) {
		watchdog_expiry_save = watchdog_expiry;
		watchdog_disable();
	}
}


/*-----------------------------------------------------------------
 * Name :	watchdog_restart()
 *
 * Description: Restart the watchdog timer stopped by watchdog_stop
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	05/10/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_restart(void)
{
	if (watchdog_state_save)
		watchdog_enable(watchdog_expiry_save);
}



/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
watchdog_view()
{
	printf("Watchdog timer is %s, ",
							(watchdog_state == 1) ? "RUNNING" : "STOPPED");
	printf("expiry = %d sec\n", watchdog_expiry);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
int32
watchdog_cmd(
	int32	argc,
	uchar8	* argv[]
)
{
	if (argc < 2)
		return -1;

	if (!strncmp(argv[1], "dis", 3)) {
		watchdog_disable();
		printf("Watchdog timer disabled\n");
	}
	else if (!strncmp(argv[1], "ena", 3)) {
		if (argc < 3)
			return -1;
		watchdog_enable(atoi(argv[2]));
		printf("\nWatchdog timer enabled, %d sec\n", atoi(argv[2]));
	}
	else if (!strncmp(argv[1], "view", 4))
		watchdog_view();
	else
		return -1;

	return 0;
}

#ifdef PROM
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
int32
get_wd_state()
{
	return watchdog_state;
}
#endif
