/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tpl-ipc.c
 *
 * RCS:  $Id: 
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * IPC queues between transport layer txrx and application txrx.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/22/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/hw-api.h>
#include <exports/malloc-api.h>
#include <exports/pkt-api.h>
#include <exports/tpl-api.h>
#include <exports/tpl.h>

#if defined(NFX_SMP)

#define TPL_IPC_ALLOC		eee_ramAlloc // allocateBlock
//#define TPL_IPC_FREE		eee_ramDealloc

#define TPL_IPC_PRINTF(a)	// printf a

typedef struct tpl_q_const 
{
	uint32	qEntSize;
	uint32	qLength;
} tpl_q_const_t;

static  tpl_q_const_t		tpl_ipc_qConst[TPL_IPC_Q_CNT] = {
	{sizeof(void*), PKT_MAX_DESC_CNT},	/* TPL_IPC_PKTQ_TX */
	{sizeof(void*), PKT_MAX_DESC_CNT},	/* TPL_IPC_PKTQ_TXCOMP */
	{sizeof(void*), 2},					/* TPL_IPC_PKTQ_ALLOC_S */
	{sizeof(void*), 2048},				/* TPL_IPC_PKTQ_ALLOC_L */
	{sizeof(void*), 512},				/* TPL_IPC_PKTQ_ALLOC_U */
	{sizeof(void*), PKT_MAX_DESC_CNT},	/* TPL_IPC_PKTQ_RX */
	{sizeof(void*), 4096},				/* TPL_IPC_EVNTQ_NTOA */
	{sizeof(void*), 4096},				/* TPL_IPC_EVNTQ_ATON */
	{sizeof(void*), PKT_MAX_DESC_CNT},	/* TPL_IPC_PKTQ_FREE */
};


static void		tpl_ipcPoll(void* cb, uint32 time_ref);
static int32	tpl_ipcEvntProc();
static int32	tpl_ipcAcpuEvntProc();
static int32	tpl_ipcReplenishPkt();
static int32	tpl_ipcTxPktProc();
static int32	tpl_ipcFreePktProc();
static int32	tpl_ipcAcpuTxPktComplProc();
static int32	tpl_ipcAcpuRxPktProc();

static int32	tpl_ipcEvntBindReq(tpl_evntMsg_t *evntMsg);
static int32	tpl_ipcEvntUnbindReq(tpl_evntMsg_t *evntMsg);
static int32	tpl_ipcEvntCloseReq(tpl_evntMsg_t *evntMsg);
static int32	tpl_ipcEvntAddConnAck(tpl_evntMsg_t	*evntMsg);
static int32	tpl_ipcEvntDelConnAck(tpl_evntMsg_t	*evntMsg);
static int32	tpl_ipcAcpuEvntBindReqAck(tpl_evntMsg_t	*evntMsg);
static int32	tpl_ipcAcpuEvntUnbindReqAck(tpl_evntMsg_t	*evntMsg);
static int32	tpl_ipcAcpuEvntCloseReqAck(tpl_evntMsg_t	*evntMsg);
static int32	tpl_ipcAcpuEvntAddConn(tpl_evntMsg_t	*evntMsg);
static int32	tpl_ipcAcpuEvntDelConn(tpl_evntMsg_t	*evntMsg);

typedef int32  (*tpl_evnt_fn_t)(tpl_evntMsg_t	*msg);
static tpl_evnt_fn_t	tpl_evnt_aproc[TPL_EVNT_CNT] = 
{
	NULL,						/* TPL_EVNT_BIND_REQ */
	tpl_ipcAcpuEvntBindReqAck,	/* TPL_EVNT_BIND_REQ_ACK */
	NULL,						/* TPL_EVNT_UNBIND_REQ */
	tpl_ipcAcpuEvntUnbindReqAck,/* TPL_EVNT_UNBIND_REQ_ACK */
	NULL,						/* TPL_EVNT_CLOSE_REQ */
	tpl_ipcAcpuEvntCloseReqAck,	/* TPL_EVNT_CLOSE_REQ_ACK */
	tpl_ipcAcpuEvntAddConn,		/* TPL_EVNT_ADD_CONN */
	NULL,						/* TPL_EVNT_ADD_CONN_ACK */
	tpl_ipcAcpuEvntDelConn,		/* TPL_EVNT_DEL_CONN */
	NULL						/* TPL__EVNT_DEL_CONN_ACK */ 
};

static tpl_evnt_fn_t	tpl_evnt_nproc[TPL_EVNT_CNT] = 
{
	tpl_ipcEvntBindReq,			/* TPL_EVNT_BIND_REQ */
	NULL,						/* TPL_EVNT_BIND_REQ_ACK */
	tpl_ipcEvntUnbindReq,		/* TPL_EVNT_UNBIND_REQ */
	NULL,						/* TPL_EVNT_UNBIND_REQ_ACK */
	tpl_ipcEvntCloseReq,		/* TPL_EVNT_CLOSE_REQ */
	NULL,						/* TPL_EVNT_CLOSE_REQ_ACK */
	NULL,						/* TPL_EVNT_ADD_CONN */
	tpl_ipcEvntAddConnAck,		/* TPL_EVNT_ADD_CONN_ACK */
	NULL,						/* TPL_EVNT_DEL_CONN */
	tpl_ipcEvntDelConnAck		/* TPL_EVNT_DEL_CONN_ACK */
};

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcInit
 *
 * Description: initiate ipc queues between transport and application
 *				txrx. 
 *
 * Created by : Su Wang
 *
 * Date Created: 11/26/2002
 *
 *-----------------------------------------------------------------
 */
void
tpl_ipcInit()
{
	int			i, rc;
	pkt_desc_t	*pDesc;
	
	
	/* initialization */
	/* allocate ipc queues in shared space */
	for (i=0; i < TPL_IPC_Q_CNT; i++) {

		printf("allocate queue size (i=%d) %d\n", i,
			   tpl_ipc_qConst[i].qEntSize*tpl_ipc_qConst[i].qLength);
		
		pTplCfg->tplIpcQ[i].tpl_ipcqStartPtr = 
			(address_t*)TPL_IPC_ALLOC(tpl_ipc_qConst[i].qEntSize*
										  tpl_ipc_qConst[i].qLength);
			
		pTplCfg->tplIpcQ[i].tpl_ipcqEndPtr = pTplCfg->tplIpcQ[i].tpl_ipcqStartPtr +
				tpl_ipc_qConst[i].qLength - 1;
		
		pTplCfg->tplIpcQ[i].tpl_ipcqRdPtr = pTplCfg->tplIpcQ[i].tpl_ipcqWrPtr =
			pTplCfg->tplIpcQ[i].tpl_ipcqStartPtr;
		
		pTplCfg->tplIpcQ[i].tpl_ipcqTotalFwd =  pTplCfg->tplIpcQ[i].tpl_ipcqTotalRcv
			= pTplCfg->tplIpcQ[i].tpl_ipcqDropCnt = 0;
	}


	/* replenish queues */
	while (!TPL_IPCQ_FULL(&(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_U]))) {
		pDesc = PKT_ALLOC_PKT_BYTYPE(0,EEE_BUF_ULTRA);
		TPL_IPCQ_WRITE(&(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_U]), pDesc);
	}

	while (!TPL_IPCQ_FULL(&(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_L]))) {
		pDesc = PKT_ALLOC_PKT_BYTYPE(0,EEE_BUF_LARGE);
		TPL_IPCQ_WRITE(&(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_L]), pDesc);
	}


	/* register with eee polling */
    rc = eee_registerPollingFunc(EEE_HIGH_PRIORITY, tpl_ipcPoll, 0 );
    if (rc < 0)
        panic("%s: tpl ipc poll regsiter failed %d\n", __FUNCTION__, rc);
 
	return;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcPoll
 *
 * Description: main polling loop
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static void 
tpl_ipcPoll(void* cb, uint32 time_ref)
{

	if (CM_IS_TXRX_NCPU) {
		tpl_ipcTxPktProc();
		tpl_ipcFreePktProc();
	    tpl_ipcReplenishPkt();
		tpl_ipcEvntProc();
	}
	else {
				/* ACPU */
		tpl_ipcAcpuTxPktComplProc();
		tpl_ipcAcpuRxPktProc();
		tpl_ipcAcpuEvntProc();
	}
	

	return;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcReplenishPkt
 *
 * Description: replenish TPL_IPC_PKTQ_ALLOC_L and TPL_IPC_PKTQ_ALLOC_U
 *				NCPU func.
 *
 * Note: what if NCPU is not fast enough in allocating pkts, can happen,
 * need to run some stress test.
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcReplenishPkt()
{
	pkt_desc_t	*pDesc;
	tpl_ipcq_t		*myQueuePtr;
	
	/*  TPL_IPC_PKTQ_ALLOC_L queue */
	myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_L]);
	while (!(TPL_IPCQ_FULL(myQueuePtr))) {
		pDesc = PKT_ALLOC_PKT_BYTYPE(0,EEE_BUF_LARGE);
		TPL_IPCQ_WRITE(myQueuePtr, pDesc);
	}
	

	/*  TPL_IPC_PKTQ_ALLOC_U queue */
	myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_U]);
	while (!(TPL_IPCQ_FULL(myQueuePtr))) {
		pDesc = PKT_ALLOC_PKT_BYTYPE(0,EEE_BUF_ULTRA);
		TPL_IPCQ_WRITE(myQueuePtr, pDesc);
	}	

	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name: 		tpl_ipcTxPktProc
 *
 * Description: process pkts in TPL_IPC_PKTQ_TX queue, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcTxPktProc()
{
	tpl_ipcq_t		*myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_TX]);
	pkt_desc_t		*pDesc;

	while(!(TPL_IPCQ_EMPTY(myQueuePtr))) {
		TPL_IPCQ_READ(myQueuePtr, pDesc);
		tpl_xmtPktCore(pDesc->pTplConn, pDesc);
	}

	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name: 		tpl_ipcFreePktProc
 *
 * Description: process pkts in TPL_IPC_PKTQ_FREE queue, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcFreePktProc()
{
	tpl_ipcq_t		*myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_FREE]);
	pkt_desc_t		*pDesc;

	while(!(TPL_IPCQ_EMPTY(myQueuePtr))) {
		TPL_IPCQ_READ(myQueuePtr, pDesc);
		PKT_FREE_PACKET(pDesc);
	}

	return NFX_OK;	
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuTxPktComplProc
 *
 * Description: process pkts in TPL_IPC_PKTQ_TXCOMPL queue, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuTxPktComplProc()
{
	tpl_ipcq_t		*myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_TXCOMP]);
	pkt_desc_t		*pDesc;
	tpl_bindCb_t	*pBind;
	tpl_connCb_t	*pConn;

	while (!(TPL_IPCQ_EMPTY(myQueuePtr))) {
		TPL_IPCQ_READ(myQueuePtr, pDesc);
		
		pConn = (tpl_connCb_t*)pDesc->pTplConn;
		pBind = pConn->pBindCb;

		if ((pBind->ul_xmtCmpInd)(pConn->pAppHdl, pDesc) == TPL_FREE_PKT) 
			TPL_FREE_PACKET( pConn, pDesc );
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuRxPktProc
 *
 * Description: process pkts in TPL_IPC_PKTQ_RX queue, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuRxPktProc()
{
	tpl_ipcq_t		*myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_RX]);
	pkt_desc_t		*pDesc;
	tpl_bindCb_t	*pBind;
	tpl_connCb_t	*pConn;

	while (!(TPL_IPCQ_EMPTY(myQueuePtr))) {
		TPL_IPCQ_READ(myQueuePtr, pDesc);
		
		pConn = (tpl_connCb_t*)pDesc->pTplConn;
		pBind = pConn->pBindCb;

		if (TPL_IPC_GET_CONNSTATE(pConn) < TPL_CONN_IPCSTATE_ADDED) {
			
			/* application not ready to receive, queue it first */
			if (pConn->ipcRxPkt) {
				PKT_FREE_PACKET(pConn->ipcRxPkt);
			}
			
			pConn->ipcRxPkt = pDesc;
			continue;
		}
		
			
		if ((pBind->ul_rcvPktInd)(pConn->pAppHdl, pDesc) == TPL_FREE_PKT) {
			PKT_FREE_PACKET(pDesc);
		}
	}

	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name: 		tpl_ipcEvntProc
 *
 * Description: evnt processing function on NCPU
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32  
tpl_ipcEvntProc() 
{
	tpl_ipcq_t		*myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_ATON]);
	tpl_evntMsg_t	*myEvntMsgPtr;

	while (!TPL_IPCQ_EMPTY(myQueuePtr)) {
		TPL_IPCQ_READ(myQueuePtr, myEvntMsgPtr);

		TPL_IPC_PRINTF(("%s: msg is %d, func is %lx\n", __FUNCTION__,
			   myEvntMsgPtr->evnt_msg, 	
			   (address_t)tpl_evnt_nproc[myEvntMsgPtr->evnt_msg]));
		
		if (tpl_evnt_nproc[myEvntMsgPtr->evnt_msg])
			(*tpl_evnt_nproc[myEvntMsgPtr->evnt_msg])(myEvntMsgPtr);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuEvntProc
 *
 * Description: evnt processing function on ACPU
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32  
tpl_ipcAcpuEvntProc() 
{
	tpl_ipcq_t		*myQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_NTOA]);
	tpl_evntMsg_t	*myEvntMsgPtr;

	while (!TPL_IPCQ_EMPTY(myQueuePtr)) {
		TPL_IPCQ_READ(myQueuePtr, myEvntMsgPtr);
		
		if (tpl_evnt_aproc[myEvntMsgPtr->evnt_msg])
			(*tpl_evnt_aproc[myEvntMsgPtr->evnt_msg])(myEvntMsgPtr);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcEvntBindReq
 *
 * Description: process bind request, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcEvntBindReq(tpl_evntMsg_t *evntMsg)
{
	tpl_bindCb_t	*bindReq, *bindRsp = NULL;
	int32			rc;
	int32			bindOpt;
	tpl_ipcq_t		*evntQNtoAPtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_NTOA]);


	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));

	bindReq = (tpl_bindCb_t*)(evntMsg + 1);
	bindOpt = bindReq->bindFlags;
	
	rc = tpl_bindReqCore(bindReq, &bindRsp, bindOpt);
	
	/* copy result bck to evntMsg */
	evntMsg->evnt_msg = TPL_EVNT_BIND_REQ_ACK;
	evntMsg->evnt_rc = rc;
	evntMsg->evnt_data = bindRsp;
	

	/* put response to TPL_IPC_EVNTQ_NTOA */
	if (TPL_IPCQ_FULL(evntQNtoAPtr)) {
		evntQNtoAPtr->tpl_ipcqDropCnt ++;
		return NFX_ERR;
	}
	
	TPL_IPCQ_WRITE(evntQNtoAPtr, evntMsg);
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcEvntUnbindReq
 *
 * Description: process unbind request, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcEvntUnbindReq(tpl_evntMsg_t *evntMsg)
{
	tpl_bindCb_t	*bindReq;
	int32			rc;
	tpl_ipcq_t		*evntQNtoAPtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_NTOA]);
	
	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));
	bindReq = (tpl_bindCb_t*)(evntMsg->evnt_data);
	
	rc = tpl_unbindReqCore(bindReq);
	
	/* copy result bck to evntMsg */
	evntMsg->evnt_msg = TPL_EVNT_UNBIND_REQ_ACK;
	evntMsg->evnt_rc = rc;

	/* put response to TPL_IPC_EVNTQ_NTOA */
	if (TPL_IPCQ_FULL(evntQNtoAPtr)) {
		evntQNtoAPtr->tpl_ipcqDropCnt ++;
		return NFX_ERR;
	}
	
	TPL_IPCQ_WRITE(evntQNtoAPtr, evntMsg);
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcEvntCloseReq
 *
 * Description: process close request, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcEvntCloseReq(tpl_evntMsg_t *evntMsg)
{
	tpl_connCb_t	*pConn;
	tpl_ipcq_t		*evntQNtoAPtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_NTOA]);


	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));

	/* since tpl_delConnReq() always return NFX_OK, acknowledge request
	 * first. This is to avoid race condition, where TPL_EVNT_DEL_CONN 
	 * will come before TPL_EVNT_CLOSE_REQ_ACK for udp connections
	 */
	evntMsg->evnt_msg = TPL_EVNT_CLOSE_REQ_ACK;
	evntMsg->evnt_rc = NFX_OK;
	
	/* put response to TPL_IPC_EVNTQ_NTOA */
	if (TPL_IPCQ_FULL(evntQNtoAPtr)) {
		evntQNtoAPtr->tpl_ipcqDropCnt ++;
		return NFX_ERR;
	}
	
	TPL_IPCQ_WRITE(evntQNtoAPtr, evntMsg);

	pConn = (tpl_connCb_t*)evntMsg->evnt_data;
	tpl_delConnReqCore(pConn);
	
	return NFX_ERR;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcEvntAddConnAck
 *
 * Description: process add conn ack, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcEvntAddConnAck(tpl_evntMsg_t	*evntMsg)
{
	tpl_connCb_t	*pConn;
	int32			rc;
	pkt_desc_t		*pDesc;
	

	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));

	pConn = (tpl_connCb_t*)evntMsg->evnt_data;
	rc    = evntMsg->evnt_rc;

	/* application don't like this connection
	 * close it
	 */
	if (rc != NFX_OK) 
		tpl_delConnReqCore(pConn);

	/* free msg that initiated from NCPU */
	TPL_EVNTMSG_FREE(evntMsg);
	
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcEvntDelConnAck
 *
 * Description: process del conn ack, NCPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcEvntDelConnAck(tpl_evntMsg_t	*evntMsg)
{
	tpl_connCb_t	*pConn;
	int32			rc;

	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));

	pConn = (tpl_connCb_t*)evntMsg->evnt_data;
	rc    = evntMsg->evnt_rc;
	
	/* nothin to be done here */

	/* free msg that initiated from NCPU */
	TPL_EVNTMSG_FREE(evntMsg);
	
	return NFX_OK;	
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuEvntBindReqAck
 *
 * Description: process bind request ack, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuEvntBindReqAck(tpl_evntMsg_t	*evntMsg)
{
	tpl_bindCb_t	*pBind;
	tpl_bindCb_t	*pBindReq;
	int32			rc;
	
	pBind	= evntMsg->evnt_data;
	pBindReq= (tpl_bindCb_t*)(evntMsg + 1);
	rc		= evntMsg->evnt_rc;

	TPL_IPC_PRINTF(("%s: msg %d, rc =%d \n", __FUNCTION__, evntMsg->evnt_msg, rc));
	
	/* let application know, bind result */
	if (pBindReq->ul_bindReqAck)
		(*pBindReq->ul_bindReqAck)(pBindReq->pBsdSock, rc, pBind);
	
	/* free msg that initiated from ACPU */
	TPL_EVNTMSG_FREE(evntMsg);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuEvntUnbindReqAck
 *
 * Description: process unbind request ack, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuEvntUnbindReqAck(tpl_evntMsg_t	*evntMsg)
{
	tpl_bindCb_t	*pBind, *pBindReq;
	int32			rc;
	
	pBind	= evntMsg->evnt_data;
	pBindReq= (tpl_bindCb_t*)(evntMsg + 1);
	rc		= evntMsg->evnt_rc;
	
	TPL_IPC_PRINTF(("%s: msg %d, rc =%d \n", __FUNCTION__, evntMsg->evnt_msg, rc));
	/* application don't need to know unbind ack */
	// (*pBind->ul_UnbindInd)(rc, pBind);
	
	/* free msg that initiated from ACPU */
	TPL_EVNTMSG_FREE(evntMsg);

	return NFX_OK;	
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuEvntCloseReqAck
 *
 * Description: process close conn request ack, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuEvntCloseReqAck(tpl_evntMsg_t	*evntMsg)
{
	tpl_connCb_t	*pConn;
	int32			rc;

	pConn = (tpl_connCb_t*)evntMsg->evnt_data;
	rc		= evntMsg->evnt_rc;

	TPL_IPC_PRINTF(("%s: msg %d, rc =%d \n", __FUNCTION__, evntMsg->evnt_msg, rc));

	/* application don't need to know, because it
	 * always return NFX_OK
	 */

	/* free msg that initiated from ACPU */
	TPL_EVNTMSG_FREE(evntMsg);

	return NFX_OK;	
	
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuEvntAddConn
 *
 * Description: process add conn ind, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuEvntAddConn(tpl_evntMsg_t	*evntMsg)
{
	tpl_connCb_t	*pConn;
	tpl_bindCb_t	*pBind;
	uint32			rc;
	pkt_desc_t		*pDesc;
	tpl_ipcq_t		*evntQAtoNPtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_ATON]);

	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));
	pConn = (tpl_connCb_t*)evntMsg->evnt_data;
	
	pBind = pConn->pBindCb;
	
	rc = (*pBind->ul_addConnInd)
		(pBind, pConn, &pConn->pAppHdl, pConn->raddr, pConn->rport);

	if (rc == NFX_OK)
		TPL_IPC_SET_CONNSTATE(pConn, TPL_CONN_IPCSTATE_ADDED);

	/* copy result bck to evntMsg */
	evntMsg->evnt_msg = TPL_EVNT_ADD_CONN_ACK;
	evntMsg->evnt_rc = rc;

	/* is there any queue pkt ? */
	pDesc = pConn->ipcRxPkt;
	pConn->ipcRxPkt = NULL;
	if (pDesc) {
		if (rc == NFX_OK) {
			if ((pBind->ul_rcvPktInd)(pConn->pAppHdl, pDesc) == TPL_FREE_PKT) {
				PKT_FREE_PACKET(pDesc);
			}
		}
		else
			PKT_FREE_PACKET(pDesc);
	}

	/* put response to TPL_IPC_EVNTQ_ATON */
	if (TPL_IPCQ_FULL(evntQAtoNPtr)) {
		evntQAtoNPtr->tpl_ipcqDropCnt ++;
		return NFX_ERR;
	}
	
	TPL_IPCQ_WRITE(evntQAtoNPtr, evntMsg);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcAcpuEvntDelConn
 *
 * Description: process delete conn ind, ACPU func.
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/02/2002
 *
 *-----------------------------------------------------------------
 */
static int32
tpl_ipcAcpuEvntDelConn(tpl_evntMsg_t	*evntMsg)
{
	tpl_connCb_t	*pConn;
	tpl_bindCb_t	*pBind;
	uint32			rc;
	tpl_ipcq_t		*evntQAtoNPtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_ATON]);

	TPL_IPC_PRINTF(("%s: msg %d\n", __FUNCTION__, evntMsg->evnt_msg));
	pConn = (tpl_connCb_t*)evntMsg->evnt_data;
	
	pBind = pConn->pBindCb;
	
	rc = (*pBind->ul_delConnInd)
		(pConn->pAppHdl, pConn);
	
	/* copy result bck to evntMsg */
	evntMsg->evnt_msg = TPL_EVNT_DEL_CONN_ACK;
	evntMsg->evnt_rc = rc;

	/* put response to TPL_IPC_EVNTQ_NTOA */
	if (TPL_IPCQ_FULL(evntQAtoNPtr)) {
		evntQAtoNPtr->tpl_ipcqDropCnt ++;
		return NFX_ERR;
	}
	
	TPL_IPCQ_WRITE(evntQAtoNPtr, evntMsg);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: 		tpl_ipcSendEvntMsg
 *
 * Description: initiate evnt msg between NCPU and ACPU
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/04/2002
 *
 *-----------------------------------------------------------------
 */
int32
tpl_ipcSendEvntMsg(uint32 msgType, void* data, int txrxNcpu)
{
	int				msgSize;
	tpl_evntMsg_t	*evntMsgPtr;
	tpl_ipcq_t		*evntQueuePtr;
	int32			rc = NFX_OK;

	
	/* determine msg size */
	if (msgType == TPL_EVNT_BIND_REQ) 
		msgSize = sizeof(tpl_evntMsg_t) + sizeof(tpl_bindCb_t);
	else if (msgType == TPL_EVNT_DEL_CONN)
		msgSize = sizeof(tpl_evntMsg_t) + sizeof(tpl_connCb_t);
	else
		msgSize = sizeof(tpl_evntMsg_t);

	TPL_IPC_PRINTF(("%s: msg %d, queue %s\n", __FUNCTION__, 
					msgType, txrxNcpu? "NtoA":"AtoN"));
	

	/* allocate evntMsg */
	TPL_EVNTMSG_ALLOC(evntMsgPtr, msgSize);
	evntMsgPtr->evnt_msg	= msgType;
	evntMsgPtr->evnt_rc		= NFX_OK;
	evntMsgPtr->evnt_data	= data;

	switch (msgType) {
	  case TPL_EVNT_BIND_REQ:
		/* copy bind  request into msg */
		memcpy((char*)(evntMsgPtr + 1), (char*)data, sizeof(tpl_bindCb_t));
		
		/* reset data ptr to bind req */
		evntMsgPtr->evnt_data = (evntMsgPtr + 1);
		break;
		
	case TPL_EVNT_DEL_CONN:
		/* need to copy pConn, becasue ACPU need function ptr from
		 * pConn->pBind->function
		 */
		memcpy((char*)(evntMsgPtr + 1), (char*)data, sizeof(tpl_connCb_t));
		evntMsgPtr->evnt_data = (evntMsgPtr + 1);
		break;
	}
	
	/* put msg on evnt queue */
	if (txrxNcpu) 
		evntQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_NTOA]);
	else 
		evntQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_EVNTQ_ATON]);	
	

	if (TPL_IPCQ_FULL(evntQueuePtr)) {
		printf("evnt queue is full\n");
		evntQueuePtr->tpl_ipcqDropCnt++;
		goto out;
	}
	TPL_IPCQ_WRITE(evntQueuePtr, evntMsgPtr);
	
out:
	/* successfull msgs are freed upon acknowledements */
	if (rc != NFX_OK) 
		TPL_EVNTMSG_FREE(evntMsgPtr);
		
	return rc;
}

/*-----------------------------------------------------------------
 * Name: 		pkt_AcpuAllocPkt
 *
 * Description: ACPU alloc pkts from TPL_IPC_PKTQ_ALLOC_X
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/04/2002
 *
 *-----------------------------------------------------------------
 */
pkt_desc_t *
pkt_AcpuAllocPkt(ushort16  pktUsr, int type, char *filename, int32 linenum)
{
	tpl_ipcq_t	*pktQueuePtr;
	pkt_desc_t	*pDesc;

	if (type == EEE_BUF_ULTRA)
		pktQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_U]);
	else
		pktQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_ALLOC_L]);
	
	if (TPL_IPCQ_EMPTY(pktQueuePtr))
		return NULL;
	
	TPL_IPCQ_READ(pktQueuePtr, pDesc);
	
	return pDesc;
}

/*-----------------------------------------------------------------
 * Name: 		pkt_AcpuFreePkt
 *
 * Description: ACPU free pkts to TPL_IPC_PKTQ_FREE
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/04/2002
 *
 *-----------------------------------------------------------------
 */
int32
pkt_AcpuFreePkt(pkt_desc_t *pDesc, char *filename, int32 linenum)
{
	tpl_ipcq_t	*pktFreeQueuePtr = &(pTplCfg->tplIpcQ[TPL_IPC_PKTQ_FREE]);

	if (TPL_IPCQ_FULL(pktFreeQueuePtr))
		panic("%s: ipc pkt free queue full\n", __FUNCTION__);
	
	TPL_IPCQ_WRITE(pktFreeQueuePtr, pDesc);
	return NFX_OK;	
}

static char *tpl_ipcq_name_str[] = {
	"TX",
	"TXCOMP",
	"ALLOC SMALL",
	"ALLOC LARGE",
	"ALLOC ULTRA",
	"RX",
	"EVNTQ NTOA",
	"EVNTQ ATON",
	"FREE"
};


/*-----------------------------------------------------------------
 * Name: 		tpl_ipcShowQueue
 *
 * Description: show IPC queue
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 12/09/2002
 *
 *-----------------------------------------------------------------
 */
void
tpl_ipcShowQueues()
{
	int i;

	printf("event message allocated: ntoa %d, aton %d\n\n",
		   pTplCfg->NtoAMsgCnt, pTplCfg->AtoNMsgCnt);
	
	printf("tpl ipc queues\n");
	printf("----------------------------------------------------------------\n");
	printf("Name        Start       End         RdPtr       WrPtr       Drop        Rcv        Fwd         HWater\n");
	printf("----------------------------------------------------------------\n");
	
	for (i=0; i<TPL_IPC_Q_CNT; i++) {
		
		printf("%-12s%-12x%-12x%-12x%-12x%-12d%-12d%-12d%-12d\n",
			   tpl_ipcq_name_str[i], 
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqStartPtr,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqEndPtr,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqRdPtr,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqWrPtr,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqDropCnt,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqTotalRcv,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqTotalFwd,
			   (uint32)pTplCfg->tplIpcQ[i].tpl_ipcqHWater);
	}
	
}

#endif /* NFX_SMP */
