/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tpl-ipc.h
 *
 * RCS:  $Id: 
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * IPC queues between transport layer txrx and application txrx.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/22/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _TPL_IPC_H
#define _TPL_IPC_H

/* txrx IPC evnet definitions */
typedef struct {
#define TPL_EVNT_BIND_REQ		0  
#define TPL_EVNT_BIND_REQ_ACK   1
#define TPL_EVNT_UNBIND_REQ		2
#define TPL_EVNT_UNBIND_REQ_ACK 3
#define TPL_EVNT_CLOSE_REQ		4
#define TPL_EVNT_CLOSE_REQ_ACK  5
#define TPL_EVNT_ADD_CONN		6
#define TPL_EVNT_ADD_CONN_ACK   7
#define TPL_EVNT_DEL_CONN		8
#define TPL_EVNT_DEL_CONN_ACK	9
#define TPL_EVNT_CNT			10  /* maxinum num. of evnts */
	int32	evnt_msg;
	int32   evnt_rc;
	void	*evnt_data;
} tpl_evntMsg_t;


/* common queue structure */
typedef struct tpl_ipcq {
	address_t	*tpl_ipcqStartPtr;  /* start of Q allocation */
	address_t	*tpl_ipcqEndPtr;    /* end of Q allocation */ 
	address_t	*tpl_ipcqRdPtr;     /* start of Q entries */
	address_t	*tpl_ipcqWrPtr;		/* empty entry immediately after end of Q.*/
	uint32		tpl_ipcqDropCnt;	/* entries dropped because Q is full */
	uint32		tpl_ipcqHWater;		/* higher water mark for entries in queue */
	uint32		tpl_ipcqTotalRcv;   /* total num. of entries rcv by queue */
	uint32		tpl_ipcqTotalFwd;   /* total num. of entries forward by queue */
} tpl_ipcq_t;

/* ipc queue definitions */
#define TPL_IPC_PKTQ_TX			0
#define TPL_IPC_PKTQ_TXCOMP		1
#define TPL_IPC_PKTQ_ALLOC_S	2
#define TPL_IPC_PKTQ_ALLOC_L	3
#define TPL_IPC_PKTQ_ALLOC_U	4
#define TPL_IPC_PKTQ_RX			5
#define TPL_IPC_EVNTQ_NTOA		6
#define TPL_IPC_EVNTQ_ATON		7
#define TPL_IPC_PKTQ_FREE		8

#define TPL_IPC_Q_CNT			9

#define TPL_EVNTMSG_ALLOC(_ptr, _msgSize)	\
{											\
	_ptr = (void*)eee_ramAlloc(_msgSize);	\
	if (CM_IS_TXRX_NCPU)					\
		pTplCfg->NtoAMsgCnt ++;				\
	else									\
		pTplCfg->AtoNMsgCnt ++;				\
}

#define TPL_EVNTMSG_FREE(_ptr)				\
{											\
	eee_ramDealloc(_ptr);					\
	if (CM_IS_TXRX_NCPU)					\
		pTplCfg->NtoAMsgCnt --;				\
	else									\
		pTplCfg->AtoNMsgCnt --;				\
}

#define TPL_IPCQ_EMPTY(_qCommon)        \
    ((_qCommon)->tpl_ipcqRdPtr == (_qCommon)->tpl_ipcqWrPtr)

#define TPL_IPCQ_FULL(_qCommon)										  \
	((((_qCommon)->tpl_ipcqRdPtr == (_qCommon)->tpl_ipcqStartPtr) &&  \
		((_qCommon)->tpl_ipcqWrPtr == (_qCommon)->tpl_ipcqEndPtr)) || \
	(((_qCommon)->tpl_ipcqWrPtr) + 1 == (_qCommon)->tpl_ipcqRdPtr))
 
#define TPL_IPCQ_WRITE(_qCommon, _entry)								\
{																		\
	uint32	_diff;														\
	*((_qCommon)->tpl_ipcqWrPtr) = (address_t)_entry;					\
	if ((_qCommon)->tpl_ipcqWrPtr == (_qCommon)->tpl_ipcqEndPtr)		\
		(_qCommon)->tpl_ipcqWrPtr = (_qCommon)->tpl_ipcqStartPtr;		\
	else																\
		(_qCommon)->tpl_ipcqWrPtr = (_qCommon)->tpl_ipcqWrPtr + 1;		\
																		\
	(_qCommon)->tpl_ipcqTotalRcv ++;									\
	_diff = (uint32)((int32)(_qCommon)->tpl_ipcqTotalRcv -			\
					 (int32)(_qCommon)->tpl_ipcqTotalFwd);			\
	if ((_qCommon)->tpl_ipcqHWater < _diff)								\
		(_qCommon)->tpl_ipcqHWater = _diff;								\
}

#define TPL_IPCQ_READ(_qCommon, _entry)									\
{																		\
	_entry = (void*)(*((_qCommon)->tpl_ipcqRdPtr));						\
	if ((_qCommon)->tpl_ipcqRdPtr == (_qCommon)->tpl_ipcqEndPtr)		\
		(_qCommon)->tpl_ipcqRdPtr = (_qCommon)->tpl_ipcqStartPtr;		\
	else																\
		(_qCommon)->tpl_ipcqRdPtr = (_qCommon)->tpl_ipcqRdPtr + 1;		\
																		\
	(_qCommon)->tpl_ipcqTotalFwd ++;									\
}

int32	
tpl_ipcSendEvntMsg(uint32 msgType, void* data, int txrxNcpu);
pkt_desc_t *	
pkt_AcpuAllocPkt(ushort16  pktUsr, int type, char *filename, int32 linenum);
int32	
pkt_AcpuFreePkt(pkt_desc_t *pDesc, char *filename, int32 linenum);
void
tpl_ipcShowQueues(void);
void
tpl_ipcInit(void);
#endif /* _TPL_IPC_H */



	

