

#ifndef __TSSCCOMM_H
#define __TSSCCOMM_H

/*
 *-----------------------------------------------------------------
 *
 * Name:	tssccomm.h
 *
 * RCS:  $Id: tssccomm.h,v 1.4 2002/04/22 18:06:45 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Communication between Toshiba and SSC
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	01/31/2002
 *
 *-----------------------------------------------------------------
 */

#define TSC_MESSAGE_SIZE 1516
#define TSC_RING_LENGTH 100

#define TSC_MESSAGE_FLAG_RECEIVER_OWNS 0x1
#define TSC_MESSAGE_FLAG_FIRST 0x2
#define TSC_MESSAGE_FLAG_LAST 0x4


/* Message header format */
typedef struct tsc_message_header
{
	uint32 flags;				/* Flags, combination of TSC_MESSAGE_FLAG_* */
	uint32 bytecount;			/* Number of bytes in this message */
	uint32 next_index;			/* Index of the next message in the ring */
	int data_offset;			/* Offset to message data from header */
} tsc_message_header_t;


#define TSC_MESSAGE_DATA_SIZE (TSC_MESSAGE_SIZE - sizeof(tsc_message_header_t))


/* Message - header followed by data */
typedef struct tsc_message
{
	tsc_message_header_t header;
	char data[TSC_MESSAGE_DATA_SIZE];
} tsc_message_t;


/* The driver structure */
typedef struct tsc_sc
{
#if defined(__OpenBSD__)
	struct ifnet sc_if;			/* Our interface */
#endif
	tsc_message_t *first_tx_message; /* first message in transmit
										 ring*/
	tsc_message_t *tx_message; /* Next after last message transmitted */
	tsc_message_t *first_rx_message; /* First message in receive ring */
	tsc_message_t *rx_message;	/* Message we will receive next */
} tsc_sc_t;


#if defined(__OpenBSD__)
typedef struct mbuf data_descriptor_t;
#else
typedef eee_desc_t data_descriptor_t;
#endif


#if !defined(__OpenBSD__)
#define assert(x) NFX_ASSERT((x))
#endif


data_descriptor_t *tsc_make_data_descriptor(tsc_sc_t *sc,
										   tsc_message_t *message);
char *tsc_message_data(tsc_message_t *message);
tsc_message_t *tsc_last_rx_message(tsc_sc_t *sc);
void tsc_make_tx_message(tsc_message_t *message, uint32 flags,
						 uint32 bytecount, data_descriptor_t *data,
						 uint32 offset);
int tsc_receiver_owns(tsc_message_t *message);
void tsc_receiver_cleanup(tsc_message_t *message);
int tsc_is_first_message(tsc_message_t *message);
int tsc_is_last_message(tsc_message_t *message);
int tsc_next_message_index(tsc_message_t *message);
int tsc_message_length(tsc_message_t *message);
tsc_message_t *tsc_next_tx_message(struct tsc_sc *sc, tsc_message_t *message);
tsc_message_t *tsc_last_rx_message(struct tsc_sc *sc);
void tsc_set_last_rx_message(struct tsc_sc *sc, tsc_message_t *message);
tsc_message_t *tsc_next_rx_message(struct tsc_sc *sc, tsc_message_t *message);
int tsc_calculate_packet_length(struct tsc_sc *sc, tsc_message_t *start_message);
void tsc_rx_cleanup_packet(struct tsc_sc *sc, tsc_message_t *start_message);
tsc_message_t *tsc_last_tx_message(struct tsc_sc *sc);
void tsc_set_last_tx_message(struct tsc_sc *sc, tsc_message_t *message);
int tsc_check_buffers_availability(struct tsc_sc *sc, int length);
int tsc_rxpacket_ready(struct tsc_sc *sc);
void tsc_copy_from_packet(char *buf, struct tsc_sc *sc, tsc_message_t **start_message_ptr, int *offset_ptr, int len);
void tsc_copy_message_data(char *dest, data_descriptor_t *data,
						   uint32 offset, uint32 bytecount);
void tsc_set_message_flags(tsc_message_t *message, uint32 flags);
const uint32 tsc_first_message_flags(void);
const uint32 tsc_first_and_last_message_flags(void);
data_descriptor_t *tsc_do_rxmessage(struct tsc_sc *sc);
int tsc_txput_packet(struct tsc_sc *sc, data_descriptor_t *data);
int tsc_data_length(data_descriptor_t *data);
void tsc_txstart(struct tsc_sc *sc);
void tsc_init(void);
void tsc_init_tx_ring(tsc_sc_t *sc, tsc_message_t *start, int len);
void tsc_reinit_tx_ring(tsc_sc_t *sc, tsc_message_t *start,
						tsc_message_t *last, int len);
void tsc_init_rx_ring(tsc_sc_t *sc, tsc_message_t *start, int len);
void tsc_reinit_rx_ring(tsc_sc_t *sc, tsc_message_t *start,
						tsc_message_t *last, int len);
void tsc_print_message(tsc_message_t *message);
void tsc_print_txqueue(tsc_sc_t *sc);
void tsc_print_rxqueue(tsc_sc_t *sc);
tsc_message_t *tsc_ssc_tx_ring_addr(void);
tsc_message_t *tsc_ssc_rx_ring_addr(void);
void tsc_init_tx_message(tsc_message_t *message, int idx);
int tsc_check_ring(tsc_message_t *start, int len);

#endif


