/*
 *-----------------------------------------------------------------
 *
 * Name:	cli.c
 *
 * RCS:  $Id: cli.c,v 1.3 2002/11/23 19:04:52 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	gets commands from command line and calls appropriate
 *				function.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include "pmon-ui.h"

int32	repeating_cmd CPU_PRIVATE;
char8	prnbuf[LINESZ + 8] CPU_PRIVATE;		/* commonly used print buffer */
char8	line[LINESZ + 1] CPU_PRIVATE;		/* input line */

/*-----------------------------------------------------------------
 * Name:	expand()
 *
 * Description:	expand environment variables.
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
uchar8 * expand(
	uchar8	* cmdline
)
{
	char8	*ip, *op, *v;
	char8	var[256];
	static uchar8	expline[LINESZ + 8];
#if 0
	extern char *getenv ();
#endif

	if (!strchr (cmdline, '$'))
	return cmdline;

	ip = cmdline;
	op = expline;
	while (*ip) {
		if (op >= (char8 *)&expline[sizeof(expline) - 1]) {
			printf ("Line too long after expansion\n");
			return (0);
		}

		if (*ip != '$') {
			*op++ = *ip++;
			continue;
		}

		ip++;
		if (*ip == '$') {
			*op++ = '$';
			continue;
		}

		/* get variable name */
		v = var;
		if (*ip == '{') {
			/* allow ${xxx} */
			ip++;
			while (*ip && *ip != '}') {
				*v++ = *ip++;
			}
			if (*ip && *ip != '}') {
				printf ("Variable syntax\n");
				return (0);
			}
			ip++;
		}
		else {
			/* look for $[A-Za-z0-9]* */
			while (isalpha(*ip) || isdigit(*ip))
				*v++ = *ip++;
		}

		*v = 0;
#if 0
		if (!(v = getenv (var))) {
			printf ("Undefined variable %s\n", var);
			return (0);
		}
#endif

		if (op + strlen(v) >= (char8 *)&expline[sizeof(expline) - 1]) {
			printf ("Line too long after expansion\n");
			return (0);
		}

		while (*v)
			*op++ = *v++;
	}

	*op = '\0';

	return (expline);

}	/* expand() */

/*-----------------------------------------------------------------
 * Name:	do_cmd()
 *
 * Description:	execute a command
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
do_cmd(
	uchar8	* p
)
{
	char8		*av[MAX_AC];			/* argument holder */
	int32		ac;						/* # of arguments */
	char8		*cmdlist[20];
	int32		i, nc, j, c;
#if 0	
	int32 rptcmd;
	char *t, tmp[11];
#endif	

	repeating_cmd = 0;
#if 0
	if (!*p || strempty (p)) {	/* blank */
		rptcmd = matchenv ("rptcmd");
		if (rptcmd) {		/* repeat requested */
			repeating_cmd = 1;
			t = gethistn (histno - 1);
			if (rptcmd == 1)
				strcpy (p, t);	/* all cmds */
			else if (rptcmd == 2) {	/* trace only */
				if (wordsz (t) > 10)
					return;
				getword (tmp, t);
				if (strequ (tmp, "t") || strequ (tmp, "to"))
					strcpy (p, tmp);
				else
					return;
			} else {
				printf ("bad rptcmd value [%s]\n", getenv ("rptcmd"));
				return;
			}
		} else
			return;
	}
#endif

	if (!(p = expand (p)))
		return;

	nc = 0;
	cmdlist[nc++] = p;
	for (; *p;) {
		c = *p;
		if (c == '\'' || c == '"') {
			p++;
			while (*p && *p != c)
				++p;
			if (*p)
				p++;
		} else if (c == ';') {
			*p++ = 0;
			cmdlist[nc++] = p;
		} else
			p++;
	}

	for (j = 0; j < nc; j++) {
		ac = argvize (av, cmdlist[j]);
		if (ac > 0) {
			for (i = 0; CmdTable[i].name != 0; i++)
				if (strequ (CmdTable[i].name, av[0]))
					break;
			if (CmdTable[i].name != 0) {
				int stat = -1;

				if( repeating_cmd && !CmdTable[i].repeat ) {
					/*
					 * This command isn't supposed to repeat.  Just return.
					 */

					repeating_cmd = 0;
					return;
				}

				if (ac < CmdTable[i].minac)
					printf ("%s: not enough arguments\n", CmdTable[i].name);
				else if (ac > CmdTable[i].maxac)
					printf ("%s: too many arguments\n", CmdTable[i].name);
				else
					stat = (CmdTable[i].func) (ac, av);

				if (stat < 0)
					printf ("usage: %s %s\n", CmdTable[i].name,
															CmdTable[i].opts);
				if (stat != 0)
					break;	/* skip commands following ';' */
			} else {
				printf ("%s: Command not found.\n", av[0]);
				break;
			}
		}
	}

}	/* do_cmd() */


