/*
 *-----------------------------------------------------------------
 *
 * Name:	hist.c
 *
 * RCS:  $Id: hist.c,v 1.7 2002/11/21 21:08:32 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
#if 0
#include "mips.h"
#include "pmon.h"
#include "stdio.h"
#include "termio.h"
#include "string.h"
#endif

#include <nfx-incl.h>
#include "pmon-ui.h"
#include <exports/serio-api.h>
#include <exports/malloc-api.h>

#ifdef LINUX_TEST

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <termios.h>

fd_set	rfds;

#endif	/* LINUX_TEST */

#define PREV	CNTRL('P')
#define NEXT	CNTRL('N')
#define FORW	CNTRL('F')
#define BACK	CNTRL('B')
#define BEGIN	CNTRL('A')
#define END	CNTRL('E')
#define DELETE	CNTRL('D')
#define DELLINE	CNTRL('K')
#define MARK	CNTRL(' ')
#define KILL	CNTRL('W')

#define HISTMAX 40
uchar8	* hist[HISTMAX + 1] CPU_PRIVATE;	/* array of old commands */

int32	histi CPU_PRIVATE;		/* index to hist[], indicates next slot */
int32	histo CPU_PRIVATE;		/* index to hist[], indicates oldest entry */
int32	histr CPU_PRIVATE;		/* index to hist[], used while walking back with PREV */
int32	histno CPU_PRIVATE;		/* current history number */
int32	column CPU_PRIVATE;		/* current column number */

char8	* gethistn(), * gethists();

void addchar();
void backspace();
void right();
void left();
void putstr();
void addhist();

/*-----------------------------------------------------------------
 * Name:	histinit()
 *
 * Description:	initialize history mechanism
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
histinit()
{
	histno = 1;

	memset(hist, 0, (HISTMAX+1)*4);

}	/* histinit() */

/*-----------------------------------------------------------------
 * Name:	get_line()
 *
 * Description:	get a line from stdin using lineedit and history
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int
get_line (
	uchar8	*p,
	int32	usehist
)
{
	int32	c, oc, i, n, mark;

#ifdef LINUX_TEST
    struct timeval  tv;
    int32           rc;
#endif  /* LINUX_TEST */


#if 0
	struct termio	tbuf;

	ioctl (STDIN, SETNCNE, &tbuf);
#endif

	mark = column = 0;
	histr = histi;

	if (*p) {
		putstr (p);
		left (column);
	}

	for (;;) {

#ifdef LINUX_TEST
#ifndef LINUX_POLL_NONBLOCKING_FD
        FD_ZERO(&rfds);
        FD_SET(0, &rfds);

        tv.tv_sec = 0;
        tv.tv_usec = 0;

        if ((rc = select(1, &rfds, NULL, NULL, &tv)) <= 0) {
            eee_poll(1);
            if (FD_ISSET(0, &rfds))
                printf("ERROR: rc = 0, but ISSET != 0\n");
        }
        else {
            if (!(FD_ISSET(0, &rfds)))
                printf("ERROR: rc = 0x%x, but ISSET = 0\n", rc);
            while (c = getchar()) {
#else
		while (c = getchar()) {
#endif
#else
		c = getchar ();
#endif
		if (c == PREV && usehist) {
			right (p, strlen (p) - column);
			backspace (column);
			if (histr != histo)
				histr = decmod (histr, HISTMAX);
			strcpy (p, hist[histr]);
			putstr (p);
			mark = 0;
		} else if (c == NEXT && usehist) {
			right (p, strlen (p) - column);
			backspace (column);
			if (histr != histi)
				histr = incmod (histr, HISTMAX);
			strcpy (p, hist[histr]);
			putstr (p);
			mark = 0;
		} else if (c == FORW)
			right (p, 1);
		else if (c == BACK)
			left (1);
		else if (c == BEGIN)
			left (column);
		else if (c == END)
			right (p, strlen (&p[column]));
		else if (c == MARK)
			mark = column;
		else if (c == DELETE) {
			oc = column;
			strdchr (&p[column]);
			putstr (&p[column]);
			putstr (" ");
			left (strlen (&p[oc]) + 1);
			if (mark > oc)
				mark--;
		} else if (c == DELLINE) {
			n = strlen (&p[column]);
			p[column] = '\0';
			oc = column;
			for (i = n; i > 0; i--)
				putstr (" ");
			left (n);
			if (mark > oc)
				mark = oc;
		} else if (c == KILL) {
			if (column > mark) {
				n = column - mark;
				left (n);
			} else if (column < mark) {
				n = mark - column;
			} else
				continue;
			for (i = n; i > 0; i--)
				strdchr (&p[column]);
			oc = column;
			putstr (&p[column]);
			for (i = n; i > 0; i--)
				putstr (" ");
			left (strlen (&p[oc]) + n);
			mark = column;
		} else if ((c == '\b') || (c == 0x7F)) {
			left (1);
			oc = column;
			strdchr (&p[column]);
			putstr (&p[column]);
			putstr (" ");
			left (strlen (&p[oc]) + 1);
			if (mark >= oc)
				mark--;
		} else if (c == '\n' || c == '\r') {
			putstr ("\n");
			break;
		} else if (isprint (c)) {
			addchar (p, c);
		} else {
			putchar (CNTRL ('G'));
		}
	}
#ifdef LINUX_TEST
	break;
	}
#ifndef LINUX_POLL_NONBLOCKING_FD
	}
#else
#endif
#endif	/* LINUX_TEST */

#if 0
	ioctl (STDIN, TCSETAW, &tbuf);
#endif

	return 0;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int	
get_input(
	char8	* p
)
{

	*p = '\0';
	return get_line (p, 1);

}	/* get_input() */

/*-----------------------------------------------------------------
 * Name:	get_cmd()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int
get_cmd (
	char8	* p
)
{
	char8	* q, buf[20], * t;
	uint32	v;
	int32	i;

#if 1
	get_input(p);
#else
	*p = '\0';
	get_line (p, 1);
#endif

	/* do C-shell history subs */
	if ((q = findbang (p))) {
		getword (buf, q);
		strdchr (buf);		/* delete the bang */
		if (!strempty (buf)) {
			for (i = strlen (buf) + 1; i > 0; i--)
				strdchr (q);
			if (strequ (buf, "!")) {	/* repeat last cmd */
				stristr (q, hist[decmod (histi, HISTMAX)]);
				printf ("%s\n", p);
			} else if (atob (&v, buf, 10)) {	/* hist# */
				if ((t = gethistn (v))) {
					stristr (q, t);
					printf ("%s\n", p);
				} else
					printf ("%s: Event not found.\n", buf);
			} else {		/* hist string */
				if ((t = gethists (buf))) {
					stristr (q, t);
					printf ("%s\n", p);
				} else
					printf ("%s: Event not found.\n", buf);
			}
		}
	}

	addhist (p);

	return 0;
}	/* get_cmd() */

/*************************************************************
*  static addchar(p,c) add char to line, updates 'column'
*/
void
addchar (
	char8	* p,
	int32	c
)
{
	int32	oc;

	if (strlen (p) > LINESZ - 40)
		return;
	oc = column;
	strichr (&p[column], c);
	putstr (&p[column]);
	left (strlen (&p[oc]) - 1);

}	/* addchar() */

/*-----------------------------------------------------------------
 * Name:	do_history()
 *
 * Description:	'hi' command
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int32
do_history (
	int32	ac,
	char8	*av[]
)
{
	int32	i, n, l;
	uint32	siz;

#if 0
	if (ac == 2) {
		if (!get_rsa (&siz, av[1]))
			return (-1);
	} else {
		siz = moresz;
	}
#else
	siz = moresz;
#endif

#if 0
	ioctl (STDIN, CBREAK, NULL);
#endif
	l = siz;
	i = decmod (histi, HISTMAX);
	n = histno - 1;
	while (i != decmod (histo, HISTMAX)) {
		sprintf (prnbuf, "%5d  %s", n, hist[i]);
		if (more (prnbuf, &l, (ac > 1) ? 0 : siz))
			break;
		i = decmod (i, HISTMAX);
		n--;
	}
	return (0);

}	/* dohi() */

/*-----------------------------------------------------------------
 * Name:	backspace()
 *
 * Description:	distructive backspace, updates 'column'
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
backspace (
	int32	n
)
{
	for (; n > 0 && column > 0; n--) {
		printf ("\b \b");
		column--;
	}

}	/* backspace() */

/*-----------------------------------------------------------------
 * Name:	right()
 *
 * Description:	move the cursor n places right, updates global 'column'
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
right(
	char8	* p,
	int32	n
)
{
	int32	len;

	len = strlen (p);
	for (; n > 0 && column < len; n--) {
		putchar (p[column]);
		column++;
	}

}	/* right() */

/*-----------------------------------------------------------------
 * Name:	left()
 *
 * Description:	move the cursor n placees left, updating global 'column'
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
left(
	int32	n
)
{
	for (; n > 0 && column > 0; n--) {
		putchar ('\b');
		column--;
	}

}	/* left() */

/*-----------------------------------------------------------------
 * Name:	gethistn()
 *
 * Description:	retrurn ptr to hist entry n
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
char8 *
gethistn(
	int32	n
)
{
	int32	m, d;

	if (n < (histno - HISTMAX) || n > histno)
		return (0);

	m = histi;
	d = histno - n;
	for (; d > 0; d--)
		m = decmod (m, HISTMAX);

	return (hist[m]);

}	/* gethistn() */

/*-----------------------------------------------------------------
 * Name:	gethists()
 *
 * Description:	use strbequ to return ptr to matching hist string
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
char8 *
gethists(
	char8	* p
)
{
	int32	i;

	i = decmod (histi, HISTMAX);
	while (i != decmod (histo, HISTMAX)) {
		if (hist[i] == 0)
			break;
		if (strbequ (hist[i], p))
			return (hist[i]);
		i = decmod (i, HISTMAX);
	}

	return (0);

}	/* gethists() */

/*-----------------------------------------------------------------
 * Name:	addhist()
 *
 * Description:	add command line to history
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
addhist(
	char8	* p
)
{
	int32	i;

	if (strempty (p))
		return;

	i = decmod (histi, HISTMAX);

	if (hist[i] != 0) {
		if (strequ (p, hist[i]))
			return;
	}

	if (incmod (histi, HISTMAX) == histo) {
		if (hist[histo] != 0)
			eee_ramDealloc (hist[histo]);
		hist[histo] = 0;
		histo = incmod (histo, HISTMAX);
	}

	hist[histi] = (uchar8 *)eee_ramAlloc (strlen (p) + 1);
	if (hist[histi] == 0) {
		printf ("addhist: out of memory\n");
		return;
	}

	strcpy (hist[histi], p);
	histi = incmod (histi, HISTMAX);
	histno++;

}	/* addhist() */

/*-----------------------------------------------------------------
 * Name:	putstr()
 *
 * Description:	print string and update global 'column'
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
void
putstr(
	char8	* p
)
{
	int32	i;

	for (i = 0; p[i] != 0; i++) {
#ifndef LINUX_TEST
		putchar (p[i]);
#endif
		if (p[i] == '\n')
			column = 0;
		else
			column++;
	}

}	/* putstr() */

/*-----------------------------------------------------------------
 * Name:	findbang()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
char8 *
findbang(
	char8	* p
)
{
	int32	quote;

	for (; *p; p++) {
		if (*p == '\'' || *p == '"') {
			quote = *p;
			p++;
			while (*p && *p != quote)
				p++;
		} else if (*p == '!')
			return (p);
	}

	return (0);

}	/* findbang() */

