/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: ccm-sync.c,v 1.7 2002/10/04 02:17:05 gregs Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Greg Seibert
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <string.h>
#include <stdlib.h>

#include "nfx-defs.h"
#include "nfx-types.h"

#include "exports/elog-api.h"
#include "exports/anpssc-api.h"
#include "exports/ncm-api.h"
#include "exports/ccm-api.h"
#include "ccm-loc.h"

extern void
ask_node_for_update(char *nName)
{
	uint64 buf[256];
	ncmMsg_t *ncm_msg = (ncmMsg_t *)buf;
	ccmLclCfgRq_t *cfg_rq = (ccmLclCfgRq_t *)(ncm_msg->appMsg);
	int sz = sizeof (ncmMsg_t) + sizeof (ccmLclCfgRq_t) - 1;

	ncm_msg->msgType = NCM_SEND;
	strcpy (ncm_msg->appName, "ccm");
	ncm_msg->rqResp = 0;
	cfg_rq->hdr.msgType = ccmLclCfgRq;
	cfg_rq->hdr.clientPrivate = 0;
		
	strcpy (ncm_msg->nodeName, nName);

	sendAgileMsg (-1, ncm_msg, sz,
	              "ncm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
	               NULL, NULL, 0);
}

static char *mySCC = NULL;

extern void
sync_SCC (char **cfg)
{
	uint64 buf[1024];
	ncmMsg_t *ncm_msg = (ncmMsg_t *)buf;
	ccmCfgSync_t *sync_this = (ccmCfgSync_t *)(ncm_msg->appMsg);
	int sz;

	if (!we_have_a_scc) {
		return;
	}

#if defined (VERBOSE_DEBUG)
	E_LOG (class_2, debug_s, myappid, we_have_a_scc, 0, 0,
	       ("sync_SCC %s=%s", cfg[0], cfg[1]));
#endif

	sz = sizeof (ncmMsg_t) + sizeof (ccmCfgSync_t) + strlen (cfg[0]);

	if (cfg[1]) {	// We're not telling the SCC to delete this object
		sz += strlen (cfg[1]) + 1; // Remember the '\0'
	}

	ncm_msg->msgType = NCM_SEND;
	strcpy (ncm_msg->appName, "ccm");
	strcpy (ncm_msg->nodeName, mySCC);
	ncm_msg->rqResp = 0;

	sync_this->hdr.msgType = ccmCfgSync;
	sync_this->hdr.clientPrivate = 0;
	strcpy (sync_this->cfg_strings, cfg[0]);
	if (cfg[1]) {
		strcpy (sync_this->cfg_strings + strlen(cfg[0]) + 1, cfg[1]);
	}
	else {
		*(sync_this->cfg_strings + strlen(cfg[0]) + 1) = '\0';
	}

	sendAgileMsg (-1, ncm_msg, sz,
	              "ncm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
	               NULL, NULL, 0);
}

static char *clust_cfg_objs[] = {	// Stuff the SCC gets
"cluster.nfs.*",
"cluster.lv.*",
"cluster.ip.*",
"cluster.route.*",
"cluster.ntp.mode",
"slave.*",
NULL };

extern void
sync_up_SCC (char *sccName)
{
	char **these_objs;
	char **this;
	char **one;
/*
	First off, let's keep track of who our SCC is.
 */
	if (mySCC) {
		free (mySCC);
	}
	mySCC = strdup (sccName);

	E_LOG (class_2, debug_s, myappid, 0, 0, 0, ("sync up %s", mySCC));
/*
	For all the listed cluster configuration objects, send them over
	to the SCC
 */
	for (these_objs = clust_cfg_objs; *these_objs; these_objs++) {
		this = get_cfg_val (*these_objs);
		if (this) {
			for (one = this; *one; one += 2) {
				sync_SCC (one);
			}
			free_cfg_val (this);
		}
	}
}
