/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: System Down agent
 *
 * RCS:  $Id: down.c,v 1.2 2002/03/27 01:24:16 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: This program notifies ccmd when the system is going down
 *              and waits for response. It will be run from /etc/rc.shutdown
 *              blocking the system shutdown until all the file system can be
 *              cleanly unmounted. It will then send (a) message(s) to the
 *              chassis manager to reboot the embedded card(s).
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 22 Mar 2002
 *
 *-----------------------------------------------------------------
 */
#include <unistd.h>

#include "nfx-incl.h"
#include "exports/ncm-api.h"
#include "exports/evm-api.h"
#include "exports/ccm-api.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"

#define DROP_DEAD_WAIT		120		// Only wait this long for ccmd response

extern int
main(int argc, char *argv[])
{
	long long buf[1024];
	ncmMsg_t *ncm_msg = (ncmMsg_t *)buf;
	ccmShutdown_Msg_t *shutit = (ccmShutdown_Msg_t *)ncm_msg->appMsg;
	int rc;
	struct timeval dd_to;
	int sz;
/*
	First, tell ccmd to shutdown everything
 */
	ncm_msg->msgType = NCM_SEND;
	strcpy (ncm_msg->appName, "ccm");
	strcpy (ncm_msg->nodeName, "pcc");
	ncm_msg->rqResp = 1;

	memset (shutit, 0, sizeof(ccmShutdown_Msg_t));
	shutit->hdr.msgType = ccmShutDown;
	gethostname (shutit->fromNode, sizeof(shutit->fromNode));

	dd_to.tv_sec = DROP_DEAD_WAIT;
	dd_to.tv_usec = 0;

	sz = sizeof (ccmShutdown_Msg_t) + (int)((char *)shutit - (char *)ncm_msg);

	rc = sendAgileMsg (-1, ncm_msg, sz,
	                   "ncm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &dd_to,
						buf, sizeof(buf));

	if ((rc < sz) || !(shutit->done)) {
		e_log (class_1, emerg_s, 0, 0, 0, 0, __LINE__, __FILE__,
		       "ccmd failed to shutdown properly\n");
		sleep (10);		// Wait for ems/syslog to get message out.
	}
/*
	Now, tell the local chassis manager to reset the embedded cards
 */

// TODO - Eventually this api should be in cm-api.h

	exit (0);
}
