/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: dumplic.c
 *
 * RCS:  $Id: dumplic.c,v 1.6 2002/08/28 22:10:16 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Dump the license database
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 4 Jan 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include <db.h>
#include <unistd.h>
#include <time.h>

#include "licman-api.h"
#include "licman-names.h"
#include "licrec.h"

#define SYSTEM_NAME "Agile Storage SF2000"

extern int
main ()
{
	DB *licDb;
	DBT keyThang;
	DBT dataThang;

	u_int32_t topPart;
	u_int32_t midPart;
	u_int64_t botPart;
	int prodId;
	key_rec_t lic;
	int sw_rev;
	char date[20];

	licDb = dbopen (LICDBFILE, O_RDONLY, 0, DB_BTREE,
	                NULL);

	if (!licDb) {
		fprintf (stderr, "No software licenses installed\n");
		return 1;
	}

	fprintf (stdout, "%-20s              Hardware Serial No. %u\n", 
	         SYSTEM_NAME, (unsigned int)gethostid());
	fprintf (stdout, "------------------------------------------------------------------\n");
	fprintf (stdout, "Software Component  Release       Install Date  Key\n");
	fprintf (stdout, "------------------------------------------------------------------\n");

	while (!licDb->seq (licDb, &keyThang, &dataThang, R_NEXT)) {
		if (dataThang.size != sizeof (lic)) {
			fprintf (stderr, "Error at database key %d\n", 
			                  *(int *)keyThang.data);
		}
		else {
			memcpy (&prodId, keyThang.data, sizeof(prodId));
			if ((prodId <= 0) || (prodId >= LIC_INVALID)) {
				fprintf (stderr, "Invalid key %d\n", prodId);
			}
			else {
				memcpy (&lic, dataThang.data, sizeof(lic));
				botPart = lic.lic_key;
				topPart = (u_int32_t)(botPart >> 56);
				midPart = (u_int32_t)(botPart >> 40) & 0xffff;
				botPart &= 0xffffffffff;
				sw_rev = (botPart >> 12) & 0xff;

				strftime (date, sizeof(date), "%d-%b-%Y", 
				          gmtime(&lic.install_time));

				fprintf (stdout, "%-20s   %d %20s   %02x-%04x-%010llx\n", 
				         lic_names[prodId], sw_rev, date,
				         topPart, midPart, botPart);
			}
		}
	}

	licDb->close (licDb);
	return 0;
}
