/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	lnm-tx.c
 *
 * RCS:  $Id: lnm-tx.c,v 1.6 2002/05/18 01:37:49 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Prepare and send LNM cluster control messages
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	11/05/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#include <stdio.h>      // fprintf(), stderr
#include <stdlib.h>     // exit()
#include <string.h>     // memset()
#include <unistd.h>     // close()

#include <sys/time.h>
#include <sys/socket.h>
#include <signal.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <errno.h>

#include "exports/elog-api.h"
#include "exports/lnm-api.h"
#include "exports/opt-api.h"
#include "lnm-local.h"

/*------------------------------------------------------------------------------
 * Prototypes of local functions.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

extern 	char				myClusterName[LNM_MAX_CLUSTER_NAME];
extern	int					clusterSockPort;
extern	cluster_nodeInfo_t	*myClusterNode;
extern  cluster_nodeInfo_t 	*myClusterPCC;
extern  cluster_nodeInfo_t 	*myClusterSCC;
extern	cluster_nodeInfo_t	clusterNodeInfoArray[MAX_NODE_PER_CLUSTER];
extern	int					clusterDebugLevel;


/*------------------------------------------------------------------------------
 * Name         : cluster_sendInitHello()
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 12/22/01
 *
 *------------------------------------------------------------------------------
 */
void cluster_sendInitHello()
{
	cluster_controlMsg_t ccMsg;
	int i;

	printf("Entered cluster_sendInitHello\n");

	/*
	 * Specify the message type and fill in the 
	 * message specific information. Generic message 
	 * info will be filled in cluster_txControlMsg
	 */
	ccMsg.msgType = CLUSTER_INIT_HELLO_MSG;
	ccMsg.msg.initHelloMsg.nodeState = myClusterNode->nodeState;
	ccMsg.msg.initHelloMsg.preferredPCCMode = myClusterNode->preferredPCCMode;

	for (i = 0; i < 2; i++)
	{
		if (myClusterNode->nodeIpa[i])
		{
			cluster_txControlMsg(&ccMsg, sizeof(cluster_controlMsg_t), 
			myClusterNode->nodeBroadcast[i]);
		}
	}

	return;
}

/*------------------------------------------------------------------------------
 * Name         : cluster_sendNodeHello()
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 12/22/01
 *
 *------------------------------------------------------------------------------
 */
void cluster_sendNodeHello(uint32 *dstIpa)
{
	cluster_controlMsg_t ccMsg;
	int i;

	if (clusterDebugLevel == 0)
		E_LOG(class_7, debug_s, myAppId, 0, 0, 0,
		      ("Entered cluster_sendNodeHello\n"));

	/*
	 * Specify the message type and fill in the 
	 * message specific information. Generic message 
	 * info will be filled in cluster_txControlMsg
	 */
	if (dstIpa) {
		ccMsg.msgType = CLUSTER_NODE_HELLO_MSG;
		ccMsg.msg.nodeHelloMsg.nodeState = myClusterNode->nodeState;
		ccMsg.msg.nodeHelloMsg.CCMode = myClusterNode->CCMode;
		ccMsg.msg.nodeHelloMsg.preferredPCCMode = 
	                            myClusterNode->preferredPCCMode;
		strncpy (ccMsg.msg.nodeHelloMsg.nodeName, 
		         myClusterNode->nodeName, LNM_MAX_NODE_NAME);
		for (i = 0; i < 2; i++)
		{
			if (dstIpa[i])
			{
				cluster_txControlMsg(&ccMsg, sizeof(cluster_controlMsg_t), 
				                     dstIpa[i]);
			}
		}
	}

	return;
}

/*------------------------------------------------------------------------------
 * Name         : cluster_sendAssignSCCReq()
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 12/22/01
 *
 *------------------------------------------------------------------------------
 */
void cluster_sendAssignSCCReq(cluster_nodeInfo_t	*sccCandidateNode)
{
	int i;
	cluster_controlMsg_t ccMsg;

	if (clusterDebugLevel == 0)
		E_LOG(class_7, debug_s, myAppId, 0, 0, 0,
		      ("Entered cluster_sendAssignSCCReq\n"));

	/*
	 * Specify the message type and fill in the 
	 * message specific information. Generic message 
	 * info will be filled in cluster_txControlMsg
	 */
	ccMsg.msgType = CLUSTER_ASSIGN_SCC_REQ;

	/*
	 * You are yourself the PCC
	 */
	ccMsg.msg.assignSCCReq.pccChassisId = myClusterNode->nodeChassisId;
	ccMsg.msg.assignSCCReq.pccIpa[0] = myClusterNode->nodeIpa[0];
	ccMsg.msg.assignSCCReq.pccIpa[1] = myClusterNode->nodeIpa[1];
	ccMsg.msg.assignSCCReq.preferredPCCMode = myClusterNode->preferredPCCMode;

	if (myClusterSCC)
	{
		ccMsg.msg.assignSCCReq.sccChassisId = myClusterSCC->nodeChassisId;
		ccMsg.msg.assignSCCReq.sccIpa[0] = myClusterSCC->nodeIpa[0];
		ccMsg.msg.assignSCCReq.sccIpa[1] = myClusterSCC->nodeIpa[1];
	}
	else if (sccCandidateNode)
	{
		ccMsg.msg.assignSCCReq.sccChassisId = sccCandidateNode->nodeChassisId;
		ccMsg.msg.assignSCCReq.sccIpa[0] = sccCandidateNode->nodeIpa[0];
		ccMsg.msg.assignSCCReq.sccIpa[1] = sccCandidateNode->nodeIpa[1];
	}

	/*
	 * TBD
	 */
	ccMsg.msg.assignSCCReq.clusterState = myClusterNode->clusterState;
	ccMsg.msg.assignSCCReq.unmanagedOrphanAdoptMode = 
								myClusterNode->unmanagedOrphanAdoptMode;
	ccMsg.msg.assignSCCReq.nodeTimeoutForcedTakeoverMode = 
								myClusterNode->nodeTimeoutForcedTakeoverMode;

	for (i = 0; i < 2; i++)
	{
		if (ccMsg.msg.assignSCCReq.sccIpa[i])
		{
			cluster_txControlMsg(&ccMsg, sizeof(cluster_controlMsg_t), 
									ccMsg.msg.assignSCCReq.sccIpa[i]);
		}
	}
	return;
}

/*------------------------------------------------------------------------------
 * Name         : cluster_sendFindPCCReq()
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 12/22/01
 *
 *------------------------------------------------------------------------------
 */
void cluster_sendFindPCCReq(void)
{
	cluster_controlMsg_t ccMsg;
	int i;

	if (clusterDebugLevel == 0)
		E_LOG(class_7, debug_s, myAppId, 0, 0, 0,
		      ("Entered cluster_sendFindPCCReq\n"));

	/*
	 * Specify the message type and fill in the 
	 * message specific information. Generic message 
	 * info will be filled in cluster_txControlMsg
	 */
	ccMsg.msgType = CLUSTER_FIND_PCC_REQ;
	ccMsg.msg.findPCCReq.nodeState = myClusterNode->nodeState;
	ccMsg.msg.findPCCReq.preferredPCCMode = myClusterNode->preferredPCCMode;
	strcpy (ccMsg.msg.findPCCReq.myName, myClusterNode->nodeName);

	for (i = 0; i < 2; i++)
	{
		if (myClusterNode->nodeIpa[i])
		{
			cluster_txControlMsg(&ccMsg, sizeof(cluster_controlMsg_t), 
			myClusterNode->nodeBroadcast[i]);
		}
	}

	return;
}

/*------------------------------------------------------------------------------
 * Name         : cluster_txControlMsg()
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 12/10/01
 *
 *------------------------------------------------------------------------------
 */
void cluster_txControlMsg(cluster_controlMsg_t* msg, int msgLen, uint32 dstIpa)
{
	int         rc;
	struct sockaddr_in dstAddr;
	extern int	clusterSockFd;

	/*
	 * Fill in the common fields of message format
	 */
	strcpy(msg->clusterName, myClusterName);
	msg->srcChassisId = myClusterNode->nodeChassisId;
	msg->srcIpa[0] = myClusterNode->nodeIpa[0];
	msg->srcIpa[1] = myClusterNode->nodeIpa[1];
	msg->dstIpa = dstIpa;

#ifdef SIMULATE_TWO_NODES
	/*
	 * If a send is called send first message
	 * to other node (same IP diff socket)
	 * If it was a broadcast we will send
	 * a copy to ourselves
	 */

	if (dstIpa != 0)
	{
		int i;
		for (i = 0; i < MAX_NODE_PER_CLUSTER; i++)
		{
			if (clusterNodeInfoArray[i].nodeIpa == dstIpa)
				break;
		}

		if (i >= MAX_NODE_PER_CLUSTER)
		{
			printf("No node with IP addr 0x%x\n", dstIpa);
			return;
		}
		
		bzero(&(dstAddr), sizeof(dstAddr));	
		dstAddr.sin_family = AF_INET;
		dstAddr.sin_port = htons(clusterSockPort + 
									clusterNodeInfoArray[i].nodePortOffset);
		dstAddr.sin_addr.s_addr = dstIpa - 
									clusterNodeInfoArray[i].nodePortOffset;
	}

	if (dstIpa == 0)
	{
		/*
		 * Simulated broadcast for a 2 node environment
		 * send packet to both the valid ports 
		 */
		bzero(&(dstAddr), sizeof(dstAddr));	
		dstAddr.sin_family = AF_INET;
		if (myClusterNode->nodePortOffset)
		{
			/*
			 * We are using port offsets since this is 
			 * 2 nodes simulated on the on same machine
			 */
			dstAddr.sin_port = htons(6001);
		}
		else
		{
			dstAddr.sin_port = htons(clusterSockPort);
		}
		dstAddr.sin_addr.s_addr = dstIpa;

		/*
	 	 * Send unicast message for first node 
	 	 */
		if ((rc=sendto(clusterSockFd, msg, msgLen, 0,   
					(struct sockaddr *)&dstAddr,   
							sizeof(struct sockaddr))) != msgLen) 
		{
			E_LOG(class_1, err_s, myAppId, 0, 0, __LINE__,
			      ("Error sending 1st unicast packet: %s", strerror(errno)));
			return;
		}

		if (myClusterNode->nodePortOffset == 0)
		{
			/*
			 * We have each node simulated on a separate
			 * physical machine so we do not need to 
			 * send to other port
			 */
			 return;
		}

		/*
		 * We are using port offsets since this is 
		 * 2 nodes simulated on the on same machine
		 * To simulate broadcast send message to the
		 * other port also
		 */

		dstAddr.sin_port = htons(6002);

		/*
	 	 * Send unicast message for second node 
	 	 */
		if ((rc=sendto(clusterSockFd, msg, msgLen, 0,   
					(struct sockaddr *)&dstAddr,   
							sizeof(struct sockaddr))) != msgLen) 
		{
			E_LOG(class_1, err_s, myAppId, 0, 0, __LINE__,
			      ("Error sending 2nd unicast packet: %s", strerror(errno)));
			return;
		}
	}
#else

	bzero(&(dstAddr), sizeof(dstAddr));	
	dstAddr.sin_len = INET_ADDRSTRLEN;
	dstAddr.sin_family = AF_INET;
	dstAddr.sin_port = htons(clusterSockPort);	
	dstAddr.sin_addr.s_addr = dstIpa;

	/*
	 * Send the message
	 */
	if ((rc=sendto(clusterSockFd, msg, msgLen, 0,
				(struct sockaddr *)&dstAddr,   
						sizeof(struct sockaddr))) != msgLen) 
	{
		E_LOG(class_1, err_s, myAppId, 0, 0, __LINE__,
		      ("Error sending message: %s", strerror(errno)));
		return;
	}

#ifdef SIM_BCAST
	if (dstIpa == 0)
	{
		/*
	 	 * Send the message to other node
	 	 */
		dstAddr.sin_addr.s_addr = 0x5600000a;
		
		if ((rc=sendto(clusterSockFd, msg, msgLen, 0,   
					(struct sockaddr *)&dstAddr,   
							sizeof(struct sockaddr))) != msgLen) 
		{
			E_LOG(class_1, err_s, myAppId, 0, 0, __LINE__,
			      ("Error sending message to other node: %s", strerror(errno)));
			return;
		}
	}
#endif
#endif

}

