/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_am.c,v 1.6 2002/09/28 00:11:41 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:    Pankaj Gupta
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/evm-api.h"
#include "exports/fs-api.h"
#include "exports/agent-api.h"

#define AM_CLI_MSG_LEN 10000

/*
 * Constants.
 */

// Wait for Agent to respond (60s)
static struct timeval waitAmTo = {60, 0};

/*
 * Data structures definition
 */

typedef struct
{
    char    buf[10000];  // optimize!!!
    uint32  len;
} am_msg_t;

/*
 * Extern functions.
 */

void install_am (priv_t myPriv, enum node_type node);
static void display_am_show_summary(am_msg_cmd_t *amRsp);
static void display_am_show_detail(am_srcConfig_t *srcCfgPtr);

/*
 *------------------------------------------------------------------------------
 * Name         : isValidName
 *
 * Description  : 
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

static int32 isValidName(char *str)
{
	char curChar;
    int32  len = 0;
    int32  idx = 0;

	/*
	 * A am name must have len > 0, cannot start with 
	 * non alpha-numeric character and can only have 
	 * a-z, A-Z or 0-9
	 */
	
	if (str == NULL)
		return 0;

    len = strlen(str);

	if (len >= AM_MAX_NAME)
		return 0;

    for (idx=0; idx < len; idx++)
	{
		curChar = str[idx];

		if ((idx == 0) && !isalnum(curChar))
			return 0;

		if (!isalnum(str[idx]) && (curChar != '-')
				&& (curChar != '_'))
		{
			return 0;
		}
	}

    return 1;
}

/*
 *------------------------------------------------------------------------------
 * Name         : has_digits
 *
 * Description  : 
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

static int32 has_digits(char *str)
{
    int32  cnt;

    for (cnt=0; cnt < strlen(str); cnt++)
        if (!isdigit(str[cnt]))
            return 0;
    return 1;
}

/*
 *------------------------------------------------------------------------------
 * Name         : display_am_show_summary
 *
 * Description  : Show summary records
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

void display_am_show_summary(am_msg_cmd_t *amRsp)
{
	int32 idx;
	am_srcConfig_t *srcCfgPtr;
	am_tgtConfig_t *tgtCfgPtr;
	struct in_addr in;

	printf("Name           Source IP      Source Vol     Target IP      Target Vol     State (Adm/Op)\n");
	printf("-----------------------------------------------------------------------------------------\n");

	for(idx = 0; idx < amRsp->numBlk; idx++)
	{
		srcCfgPtr = &amRsp->srcCfg[idx];

		in.s_addr = srcCfgPtr->srcAddr;
		printf("%-15s%-15s%-15s", srcCfgPtr->amName, inet_ntoa(in),
									srcCfgPtr->srcFsysName);

		in.s_addr = srcCfgPtr->tgtAddr;
		printf("%-15s%-15s%s/%s\n", inet_ntoa(in), srcCfgPtr->tgtFsysName, 
				srcCfgPtr->adminState ? "Enabled":"Disabled",
				srcCfgPtr->operState ? "Active":"Idle");
	}
	printf("\n");

	printf("Allowed hosts\n");
	printf("-------------\n");
	tgtCfgPtr = &amRsp->tgtCfg;
	for (idx = 0; idx < MAX_TGT_HOST_ENTRY; idx++)
	{
		if (tgtCfgPtr->allowedHost[idx] == 0)
			continue;

		in.s_addr = tgtCfgPtr->allowedHost[idx];
		printf("%s\n", inet_ntoa(in));
	}
	printf("\n");
}

/*
 *------------------------------------------------------------------------------
 * Name         : display_am_show_detail
 *
 * Description  : Show records in detail
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

void display_am_show_detail(am_srcConfig_t *srcCfgPtr)
{
	int32 idx;
	struct in_addr in;

	printf("\n");
	printf("Name:          %s\n", srcCfgPtr->amName);
	printf("Admin State:   %s\n", srcCfgPtr->adminState ? "Enabled":"Disabled");
	printf("Oper State:    %s\n", srcCfgPtr->operState ? "In Progress":"Idle");


	printf("Started:       %d\n", srcCfgPtr->numMirrorStarted);
	printf("Completed:     %d\n", srcCfgPtr->numMirrorCompleted);
	printf("Aborted:       %d\n", srcCfgPtr->numMirrorAborted);

	in.s_addr = srcCfgPtr->srcAddr;
	printf("Source IP:     %s 0x%x\n", inet_ntoa(in), srcCfgPtr->srcAddr);
	printf("Source Vol:    %s 0x%llx\n", srcCfgPtr->srcFsysName,
						srcCfgPtr->srcVolId.whole_thing);

	in.s_addr = srcCfgPtr->tgtAddr;
	printf("Target IP:     %s 0x%x\n", inet_ntoa(in), srcCfgPtr->tgtAddr);
	printf("Target Vol:    %s 0x%llx\n", srcCfgPtr->tgtFsysName,
						srcCfgPtr->tgtVolId.whole_thing);


	printf("Interval:      ");
	if (srcCfgPtr->minBitMap)
	{
		printf("min  : ");
		for (idx = 0; idx <= 59; idx++)
		{
			if (GET_BIT(srcCfgPtr->minBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("min  : *\n");
	}
	
	if (srcCfgPtr->hourBitMap)
	{
		printf("               hour : ");
		for (idx = 0; idx <= 23; idx++)
		{
			if (GET_BIT(srcCfgPtr->hourBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               hour : *\n");
	}

	if (srcCfgPtr->dateBitMap)
	{
		printf("               date : ");
		for (idx = 1; idx <= 31; idx++)
		{
			if (GET_BIT(srcCfgPtr->dateBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               date : *\n");
	}

	if (srcCfgPtr->monthBitMap)
	{
		printf("               month: ");
		for (idx = 1; idx <= 12; idx++)
		{
			if (GET_BIT(srcCfgPtr->monthBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               month: *\n");
	}

	if (srcCfgPtr->dayBitMap)
	{
		printf("               day  : ");
		for (idx = 0; idx <= 7; idx++)
		{
			if (GET_BIT(srcCfgPtr->dayBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               day  : *\n");
	}

	printf("Max Data Rate: %d\n", srcCfgPtr->maxDataRate);
	printf("\n");
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_create
 *
 * Description  : Create a mirror
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

#define AM_CREATE_CMD_OPTARGS "  [-s STATE] [-r RATE] [-m MINUTE] [-h HOUR] [-d DATE] [-M MONTH] [-D DAY]\n"

DEFUN(am_create,
      am_create_cmd,
      "mirror create MIRRORNAME -a SRCIPA -f SRCFSYS -A TGTIPA -F TGTFSYS [OPT1 ARG1] [OPT2 ARG2] [OPT3 ARG3] [OPT4 ARG4] [OPT5 ARG5] [OPT6 ARG6] [OPT7 ARG7]",
      "Mirror commands\n"
      "Create a new mirror\n"
      "Name of the new mirror\n"
	  "Specify source IP address\n"
	  "Specify source IP address\n"
	  "Specify source file system name\n"
	  "Specify source file system name\n"
	  "Specify target IP address\n"
	  "Specify target IP address\n"
	  "Specify target file system name\n"
	  "Specify target file system name\n"
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
	  AM_CREATE_CMD_OPTARGS
      AM_CREATE_CMD_OPTARGS)
{
	char 			c;
	char			*token;
	int32			readVal;
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;

	am_srcConfig_t	*srcCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_CREATE_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (!isValidName(argv[0]))
	{
		printf ("Invalid mirror name\n");
		return CMD_ERR_FAILED;
	}
    strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);

	srcCfgPtr->adminState = 1;
	srcCfgPtr->maxDataRate = 0; 	// Default data rate TBD

	srcCfgPtr->srcAddr = inet_addr(argv[1]);
	if (!isValidName(argv[2]))
	{
		printf ("Invalid source file system name %s\n", argv[2]);
		return CMD_ERR_FAILED;
	}
	strncpy(srcCfgPtr->srcFsysName, argv[2], VOL_MAX_NAME);

	srcCfgPtr->tgtAddr = inet_addr(argv[3]);
	if (!isValidName(argv[4]))
	{
		printf ("Invalid target file system name %s\n", argv[4]);
		return CMD_ERR_FAILED;
	}
	strncpy(srcCfgPtr->tgtFsysName, argv[4], VOL_MAX_NAME);

    optind = 5;
    while ((c = getopt (argc, argv, "s:r:m:h:d:M:D:")) != -1)
    {
        switch (c)
        {
            case 'r':
                if (!has_digits(optarg))
                {
                    printf ("Max data rate not a number\n");
                    return CMD_ERR_FAILED;
                }
				readVal = strtol(optarg, NULL, 0);
				if ((readVal == LONG_MIN) || (readVal == LONG_MAX) ||
					(readVal < 0))
				{
					printf ("Invalid number range\n");
                    return CMD_ERR_FAILED;
				}
                srcCfgPtr->maxDataRate = readVal;
                printf("srcCfgPtr->maxDataRate = %d\n", srcCfgPtr->maxDataRate);
            	break;

            case 's':
                if (strncasecmp(optarg, "enable", strlen(optarg)) == 0)
                    srcCfgPtr->adminState = 1;
                else if (strncasecmp(optarg, "disable", strlen(optarg)) == 0)
                    srcCfgPtr->adminState = 0;
				else
                {
                    printf ("Invalid adminState specified\n");
                    return (CMD_ERR_FAILED);
                }
                printf("srcCfgPtr->adminState = %d\n", srcCfgPtr->adminState);
            	break;

			case 'm':
				// read minute interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 0 || readVal > 59)
					{
						printf ("Invalid minute value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(srcCfgPtr->minBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 0 || readVal > 59)
						{
							printf ("Invalid minute value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(srcCfgPtr->minBitMap, readVal);
					}
				}
                printf("srcCfgPtr->minBitMap = 0x%llx\n", srcCfgPtr->minBitMap);
				break;

			case 'h':
				// read hour interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 0 || readVal > 23)
					{
						printf ("Invalid hour value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(srcCfgPtr->hourBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 0 || readVal > 23)
						{
							printf ("Invalid hour value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(srcCfgPtr->hourBitMap, readVal);
					}
				}
                printf("srcCfgPtr->hourBitMap = 0x%x\n", srcCfgPtr->hourBitMap);
				break;

			case 'd':
				// read date interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 1 || readVal > 31)
					{
						printf ("Invalid date value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(srcCfgPtr->dateBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 1 || readVal > 31)
						{
							printf ("Invalid date value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(srcCfgPtr->dateBitMap, readVal);
					}
				}
                printf("srcCfgPtr->dateBitMap = 0x%x\n", srcCfgPtr->dateBitMap);
				break;

			case 'M':
				// read month interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 1 || readVal > 12)
					{
						printf ("Invalid month value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(srcCfgPtr->monthBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 1 || readVal > 12)
						{
							printf ("Invalid month value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(srcCfgPtr->monthBitMap, readVal);
					}
				}
                printf("srcCfgPtr->monthBitMap = 0x%x\n", srcCfgPtr->monthBitMap);
				break;

			case 'D':
				// read day interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 0 || readVal > 7)
					{
						printf ("Invalid day value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(srcCfgPtr->dayBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 0 || readVal > 7)
						{
							printf ("Invalid day value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(srcCfgPtr->dayBitMap, readVal);
					}
				}
                printf("srcCfgPtr->dayBitMap = 0x%x\n", srcCfgPtr->dayBitMap);
				break;

            case '?':
                if (isprint (optopt))
                    fprintf (stderr, "Unknown option `-%c'.\n", optopt);
                else
                    fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
            	return CMD_ERR_FAILED;

            default:
                fprintf (stderr, "Unknown option `-%c'.\n", optopt);
            	return CMD_ERR_FAILED;

        } // switch
    } // while

	if (optind != argc)
	{
		printf("Syntax error\n");
		return CMD_ERR_FAILED;
	}

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_CREATE_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);
		

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_delete
 *
 * Description  : Delete the mirror
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_delete,
      am_delete_cmd,
      "mirror delete MIRRORNAME",
      "Mirror commands\n"
      "Delete the specified mirror\n"
      "Name of an existing mirror\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_srcConfig_t	*srcCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_DELETE_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (!isValidName(argv[0]))
	{
		printf ("Invalid mirror name\n");
		return CMD_ERR_FAILED;
	}

    strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_DELETE_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_enable
 *
 * Description  : Enable a mirror config
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_enable,
      am_enable_cmd,
      "mirror enable MIRRORNAME",
      "Mirror commands\n"
      "Enable the specified mirror\n"
      "Name of an existing mirror\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_srcConfig_t	*srcCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_ENABLE_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (!isValidName(argv[0]))
	{
		printf ("Invalid mirror name\n");
		return CMD_ERR_FAILED;
	}
    strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_ENABLE_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_disable
 *
 * Description  : Disable a mirror config
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_disable,
      am_disable_cmd,
      "mirror disable MIRRORNAME",
      "Mirror commands\n"
      "Disable the specified mirror\n"
      "Name of an existing mirror\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_srcConfig_t	*srcCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_DISABLE_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (!isValidName(argv[0]))
	{
		printf ("Invalid mirror name\n");
		return CMD_ERR_FAILED;
	}
    strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_DISABLE_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_start
 *
 * Description  : Start a mirror operation
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_start,
      am_start_cmd,
      "mirror start MIRRORNAME",
      "Mirror commands\n"
      "Start mirroring for specified mirror\n"
      "Name of an existing mirror\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_srcConfig_t	*srcCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_USER_START_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (!isValidName(argv[0]))
	{
		printf ("Invalid mirror name\n");
		return CMD_ERR_FAILED;
	}
    strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_USER_START_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_stop
 *
 * Description  : Stop a mirror operation
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_stop,
      am_stop_cmd,
      "mirror stop MIRRORNAME",
      "Mirror commands\n"
      "Stop current mirror operation if any in progress\n"
      "Name of an existing mirror\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_srcConfig_t	*srcCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_USER_STOP_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (!isValidName(argv[0]))
	{
		printf ("Invalid mirror name\n");
		return CMD_ERR_FAILED;
	}
    strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_USER_STOP_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_host_add
 *
 * Description  : Add a host from list of allowed nodes
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_host_add,
      am_host_add_cmd,
      "mirror host add -a IPADDR",
      "Mirror commands\n"
      "Host which is allowed to mirror to this node\n"
      "Add a host from allowed node list\n"
	  "IP address of the host\n"
	  "IP address of the host\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_tgtConfig_t	*tgtCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_ADD_HOST_REQ;
	tgtCfgPtr = &amReq.tgtCfg;
	tgtCfgPtr->allowedHost[0] = inet_addr(argv[0]);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_ADD_HOST_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_host_delete
 *
 * Description  : Remove a host from list of allowed nodes
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_host_delete,
      am_host_delete_cmd,
      "mirror host delete -a IPADDR",
      "Mirror commands\n"
      "Host which is allowed to mirror to this node\n"
      "Delete a host from allowed node list\n"
	  "IP address of the host\n"
	  "IP address of the host\n")
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_tgtConfig_t	*tgtCfgPtr;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_DELETE_HOST_REQ;
	tgtCfgPtr = &amReq.tgtCfg;
	tgtCfgPtr->allowedHost[0] = inet_addr(argv[0]);

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp.msgType != AM_DELETE_HOST_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	if (amRsp.result != NFX_OK)
        fprintf(stderr, "%s", amRsp.output);

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : am_show
 *
 * Description  : Show the confiured mirrors and state
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(am_show,
      am_show_cmd,
      "mirror show [MIRRORNAME]",
      "Mirror commands\n"
      "Show list and summary of all configured mirrors\n"
      "Name of an existing mirror\n")
{
	int32			cnt;
	am_msg_cmd_t	amReq;
	am_srcConfig_t	*srcCfgPtr;
    am_msg_cmd_t    *amRsp;
	int32			rspLen;

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_SHOW_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;

	if (argc == 1)
	{
		if (!isValidName(argv[0]))
		{
			printf ("Invalid mirror name\n");
			return CMD_ERR_FAILED;
		}
    	strncpy(srcCfgPtr->amName, argv[0], AM_MAX_NAME);
		rspLen = sizeof(am_msg_cmd_t);
	}
	else
	{
		rspLen = sizeof(am_msg_cmd_t) + 
					(MAX_SRC_MIRROR_ENTRY - 1)*sizeof(am_srcConfig_t); 
	}

	amRsp = malloc(rspLen);
	if (amRsp == NULL)
		perror("No memory for amRsp\n");

    /*
     * Send message to AM.
     */
    cnt = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, amRsp, rspLen);

    /*
     * Receive response from AM.
     */
    if (cnt < sizeof(am_msg_cmd_t))
    {
        fprintf(stderr, "No response from - am -\n");
    }
    else if (amRsp->msgType != AM_SHOW_MIRROR_RSP)
    {
        fprintf(stderr, "Incorrect message type from - am -\n");
    }

	/*
	 * Display detail or summary as requested
	 */
	if (amRsp->result != NFX_OK)
	{
        fprintf(stderr, "%s", amRsp->output);
	}
	else
	{
		if ((amRsp->numBlk > 0) || (amRsp->tgtCfg.allowedHost[0]))
		{
			if (argc == 1)
			{
				display_am_show_detail(&amRsp->srcCfg[0]);
			}
			else
			{
				display_am_show_summary(amRsp);
			}
		}
	}

	free(amRsp);
    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : install_am
 *
 * Description  : Install the UI commands
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

void install_am (priv_t myPriv, enum node_type node)
{
    switch (myPriv)
    {
        case diagPriv:
        case fullPriv:
             install_element(node, &am_delete_cmd);
             install_element(node, &am_start_cmd);
             install_element(node, &am_stop_cmd);

        case changePriv:
             install_element(node, &am_enable_cmd);
             install_element(node, &am_disable_cmd);
        case addPriv:
             install_element(node, &am_create_cmd);
             install_element(node, &am_host_add_cmd);
             install_element(node, &am_host_delete_cmd);
        case viewPriv:
             install_element(node, &am_show_cmd);
    }
    return;
}
