/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_arp.c,v 1.12 2002/11/22 02:51:24 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Greg Seibert
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#if defined(__OpenBSD__)
#include <netinet/if_ether.h>
#endif /* SSC_mips */
#include "exports/ipmd-lib.h"

extern	uint64		result_buf[];
/*-----------------------------------------------------------------
 * Name: arp_add
 *
 * Description: add arp
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/29/02
 *
 *-----------------------------------------------------------------
 */
DEFUN	(arp_add,
		 ena_arp_add_cmd,
		 "arp add A.B.C.D -a MAC-ADDR [-n NODE]",
		 "Arp commands\n"
		 "Add stataic arp entry\n"
		 "Arp IP  address\n"
		 "Associated mac address\n"
		 "Mac format XX:XX:XX:XX:XX:XX\n"
		 "Which node\n"
		 "Node name\n")
{
	char	tmp_mac[6];
	char	*mac_ptr = 0;
	char	node[64];
	int		opt_num = 0;	/* num. of options */
	int		my_argc = 2;
	int		c;

	/* Shaun 11/14/02 
	 * Fixed #2735
	 * Moved mac address before -n option checked
	 */
	if ((mac_ptr = (char*)ether_aton(argv[1])) == NULL) {
		printf("Incorrect mac address %s\n", argv[1]);
		return CMD_ERR_FAILED;
	}
			
	strcpy(node, "local");
	optind = 0;
	while ((c = getopt(argc, argv, "n:")) != -1) {
		switch (c) {

		case 'n':
			strcpy(node, optarg);
			opt_num++;
			break;
		}
	}
	
	/* check parameter number matches */
	my_argc += 2* opt_num;
	if (my_argc != argc) {
		printf("incorrect number of parameters, argc %d, in_arg %d\n",
			   argc, my_argc);
		return  CMD_ERR_FAILED;
	}

	memcpy(tmp_mac, mac_ptr, 6);
	
	if (ipmd_AddArpCluster(inet_addr(argv[0]),
					tmp_mac, NULL, 0, node) != NFX_OK)
		return CMD_ERR_FAILED;
	else
		return CMD_SUCCESS;
}

/*-----------------------------------------------------------------
 * Name: arp_rm
 *
 * Description: remove arp
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/29/02
 *
 *-----------------------------------------------------------------
 */
DEFUN	(arp_rm,
		 ena_arp_rm_cmd,
		 "arp delete A.B.C.D [-n NODE]",
		 "Arp commands\n"
		 "Remove an arp entry\n"
		 "Arp IP  address\n"
		 "Which node\n"
		 "Node name\n")
{
	char	node[64];
	int		opt_num = 0;	/* num. of options */
	int		my_argc = 1;
	int		c;
	
	strcpy(node, "local");
	optind = 0;
	while ((c = getopt(argc, argv, "n:")) != -1) {
		switch (c) {
		case 'n':
			strcpy(node, optarg);
			opt_num++;
			break;
		}
	}

	/* check parameter number matches */
	my_argc += 2* opt_num;
	if (my_argc != argc) {
		printf("incorrect number of parameters, argc %d, in_arg %d\n",
			   argc, my_argc);
		return  CMD_ERR_FAILED;
	}

	if (ipmd_DelArpCluster(inet_addr(argv[0]), NULL, 0, node) != NFX_OK)
		return CMD_ERR_FAILED;
	else
		return CMD_SUCCESS;
}

/*-----------------------------------------------------------------
 * Name: arp_show
 *
 * Description: show ssc arp table.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/29/02
 *
 *-----------------------------------------------------------------
 */
DEFUN   (arp_show,
         vi_arp_show_cmd,
		 "arp show  [-a A.B.C.D] [-n NODE]",
		 "Arp commands\n"
		 "Show arp entries\n"
		 "Show single entry\n"
		 "Entry IP\n"
		 "Which node\n"
		 "Node name\n")
{
	int			count=0, need=0;
	ipm_arp_t	*ipm_arpp;
	char		ip[20], mac[20];
	uint32		ip_spec = 0;
	int			opt_num = 0;	/* num. of options */
	int			my_argc = 0;
	char		node[64];
	int			i;

	strcpy(node, "local");

	/* copy option ourselves, 
	 * getopt() does not work
	 */
	for (i=0; (i+2) <=argc; i+=2) { /* argc must be even */
		
		if (strcasecmp(argv[i], "-n") == 0) {
			strcpy(node, argv[i+1]);
			opt_num ++;
		}

		if (strcasecmp(argv[i], "-a") == 0) {
			if (!inet_aton(argv[i+1], (struct in_addr*)&ip_spec)) {
        		printf("incorrect IP address %s\n", argv[i+1]);
            	return  CMD_ERR_FAILED;
			}
			opt_num ++;
		}
		
	}

	/* check parameter number matches */
	my_argc += 2* opt_num;
	if (my_argc != argc) {
		printf("incorrect number of parameters, argc %d, in_arg %d\n",
			   argc, my_argc);
		return  CMD_ERR_FAILED;
	}

	if (ipmd_GetArpTblCluster((ipm_arp_t*)result_buf, IPM_SSC_BUFSIZE*sizeof(uint64), 
							  &count, &need, node) != IPM_GET_OK)
		return CMD_ERR_FAILED;
	
	printf("ARP Table\n");
	printf("---------------------------------------------------------\n");
	printf("dest              mac                 age               interface\n");
	printf("---------------------------------------------------------\n");
	for (ipm_arpp = (ipm_arp_t*)result_buf; count >0; count--, ipm_arpp++) {

		/* not the entry that is looked for */
		if (ip_spec && (ip_spec != ipm_arpp->arp_addr))
			continue;

		strcpy(ip, inet_ntoa(*((struct in_addr*)&ipm_arpp->arp_addr)));
		strcpy(mac, ether_ntoa((struct ether_addr*)ipm_arpp->arp_mac));
#if defined(LINK_AGG)
		if (ipm_arpp->arp_vifName[0] != '\0')
			printf("%-18s%-20s%-18d%s\n", 
				   ip, mac, ipm_arpp->arp_age, 
				   ipm_arpp->arp_vifName);
		else
#endif /* LINK_AGG */
			printf("%-18s%-20s%-18d%s\n", 
				   ip, mac, ipm_arpp->arp_age,  ipm_arpp->arp_ifname);

	}
	return CMD_SUCCESS;
}

void
install_arp (priv_t myPriv, enum node_type node)
{
	switch (myPriv) {
	case diagPriv:
	case fullPriv:
		install_element(node, &ena_arp_rm_cmd);
	case changePriv:
	case addPriv:
		install_element(node, &ena_arp_add_cmd);

	case viewPriv:
		install_element(node, &vi_arp_show_cmd);	
	}	
}

