/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_autosupport.c,v 1.5 2002/09/18 23:43:14 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:    Pankaj Gupta
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"
#include "exports/event-api.h"
#include "../sm-event/event-as.h"
#include "exports/opt-api.h"

char *eventTypeStr [] = {
                        "Node",
						"PCC",
						"SCC",
						"Card",
						"CPU",
						"Volume",
						"IP i/f",
						"Port",
						"NFS",
						"Route",
						"Quota",
						"Mkfs"
						};


// Wait for eventd to respond (10)
static struct timeval waitEventdTo = {10, 0};

/*
 * Extern functions.
 */

extern void install_autosupport (priv_t myPriv, enum node_type node);



/*
 *       shyhpeiy@agilestorage.com, a@b.com
 *       ^       ^^           ^^  ^ ^^^^^
 * state 0       12           34  0 01234
 */
static int
isValidEmailAddr(char *buf)
{
int i, state, newstart;
char c;
char tmpbuf[MAX_VALUE_LEN];

	i = 0;
	state = 0;
	newstart = 0;
	while (i < strlen(buf))
	{
		c = buf[i];
		if ( ((c >= 'a') && (c <= 'z')) ||
			 ((c >= 'A') && (c <= 'Z')) ||
			 ((c >= '0') && (c <= '9')) ||
			 (c == '-') ||
			 ((state == 0) && ((c == '.') || (c == '_')))
		   )
		{
			i++;
			// if state is 1, 3, 5, 7, 9,... then after
			// getting a valid character, we should get
			// into state 2, 4, 6, 8, 10,...
			if (state % 2)
				state++;
			continue;
		}
		else if ((state == 0) && (c == '@'))
		{
			i++;
			state = 1;
			continue;
		}
		else if (state && !(state % 2) && (c == '.'))
		{
			i++;
			state++;
			continue;
		}
		else if ((c == ' ') || (c == ','))
		{
			i++;
			/* must be 2,4,6,8,.. to be valid email addr */
			if ((state % 2) == 0)
			{
				// Prev email addr is OK. Need to check next one...
				state = 0;
				newstart = i;
				continue;
			}
			else
			{
				strncpy(tmpbuf, &buf[newstart], i - newstart - 1);
				printf("Invalid email address: %s\n", tmpbuf);
				return 0;
			}
		}
		else
		{
			printf("Invalid character %c in email address\n", c);
			return 0;
		}
	}

	if ((state % 2) == 0)	/* must be 2,4,6,8,.. to be valid email addr */
		return 1;
	else
	{
		strcpy(tmpbuf, &buf[newstart]);
		printf("Invalid email address: %s\n", tmpbuf);
		return 0;
	}
}

/*
 *------------------------------------------------------------------------------
 * Name         : has_digits
 *
 * Description  : 
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

static int32 has_digits(char *str)
{
    int32  cnt;

    for (cnt=0; cnt < strlen(str); cnt++)
        if (!isdigit(str[cnt]))
            return 0;
    return 1;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_state
 *
 * Description  : Modify autosupport state for the NFX
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_state,
      autosupport_state_cmd,
      "autosupport state MODE",
      "autosupport command\n"
	  "modify autosupport state\n"
	  "enable or disable\n")
{
	int rc;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_SET_REQ;
	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_ENABLE);
	if (strncmp ("enable", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(eventCfgReq.entry.value, "on");
	}
	else if (strncmp ("disable", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(eventCfgReq.entry.value, "off");
	}
	else
	{
		printf("Invalid autosupport state. Choose {enable|disable}\n");
		return CMD_ERR_FAILED;
	}

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rc != sizeof(event_configMsg_t))
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_to
 *
 * Description  : Define who to send autosupport mail to
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_to,
      autosupport_to_cmd,
      "autosupport email to EMAIL",
      "autosupport command\n"
      "set autosupport email address\n"
	  "who to send autosupport email to\n"
	  "email address\n")
{
	int rc;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

	if (strlen(argv[0]) >= sizeof(eventCfgReq.entry.value))
	{
		printf("Email address too long. Should be less than %d characters\n",
				sizeof(eventCfgReq.entry.value));
		return CMD_ERR_FAILED;
	}

	if (!isValidEmailAddr(argv[0]))
	{
		printf("Err 1\n");
		return CMD_ERR_FAILED;
	}

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_SET_REQ;
	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_TO);

	strncpy(eventCfgReq.entry.value, argv[0], MAX_VALUE_LEN);

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rc != sizeof(event_configMsg_t))
	{
		printf("Err 2 rc = %d sizeof(event_configMsg_t) = %d\n",
						rc, sizeof(event_configMsg_t));
		return CMD_ERR_FAILED;
	}

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_noteto
 *
 * Description  : Define who to put in noteto field for autosupport mail
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_noteto,
      autosupport_noteto_cmd,
      "autosupport email noteto EMAIL",
      "autosupport command\n"
      "autosupport email option\n"
	  "who to put in noteto field for autosupport email\n"
	  "email address\n")
{
	int rc;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

	if (strlen(argv[0]) >= sizeof(eventCfgReq.entry.value))
	{
		printf("Email address too long. Should be less than %d characters\n",
				sizeof(eventCfgReq.entry.value));
		return CMD_ERR_FAILED;
	}

	if (!isValidEmailAddr(argv[0]))
		return CMD_ERR_FAILED;

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_SET_REQ;
	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_NOTETO);

	strncpy(eventCfgReq.entry.value, argv[0], MAX_VALUE_LEN);

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rc != sizeof(event_configMsg_t))
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_show_config
 *
 * Description  : Show current autosupport configuration
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_show_config,
      autosupport_show_config_cmd,
      "autosupport show config",
      "autosupport command\n"
      "show autosupport related info\n"
      "show autosupport configuration for this node\n")
{
	int 				rsplen;
	int32 				idx;
	as_config_t			asCfg;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_VIEW_REQ;

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &asCfg, sizeof(as_config_t));

	if (rsplen != sizeof(as_config_t))
		return CMD_ERR_FAILED;

	printf("Auto Support configuration\n");
	printf("--------------------------\n");
	printf("Auto Support State: %s\n", asCfg.enable ? "Enabled" : "Disabled");
	printf("Auto Support TO address: %s\n", asCfg.to);
	printf("Auto Support NOTETO address: %s\n", asCfg.noteto);

	printf("Auto Support Schedule:\n");
	if (asCfg.minBitMap)
	{
		printf("               min  : ");
		for (idx = 0; idx <= 59; idx++)
		{
			if (GET_BIT(asCfg.minBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               min  : *\n");
	}
	
	if (asCfg.hourBitMap)
	{
		printf("               hour : ");
		for (idx = 0; idx <= 23; idx++)
		{
			if (GET_BIT(asCfg.hourBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               hour : *\n");
	}

	if (asCfg.dateBitMap)
	{
		printf("               date : ");
		for (idx = 1; idx <= 31; idx++)
		{
			if (GET_BIT(asCfg.dateBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               date : *\n");
	}

	if (asCfg.monthBitMap)
	{
		printf("               month: ");
		for (idx = 1; idx <= 12; idx++)
		{
			if (GET_BIT(asCfg.monthBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               month: *\n");
	}

	if (asCfg.dayBitMap)
	{
		printf("               day  : ");
		for (idx = 0; idx <= 7; idx++)
		{
			if (GET_BIT(asCfg.dayBitMap, idx))
				printf("%d ", idx);
		}
		printf("\n");
	}
	else
	{
		printf("               day  : *\n");
	}
	printf("\n");

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_schedule
 *
 * Description  : Set schedule for auto support report generation
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

#define AS_SCHEDULE_CMD_OPTARGS "  [-m MINUTE] [-h HOUR] [-d DATE] [-M MONTH] [-D DAY]\n"

DEFUN(autosupport_schedule,
      autosupport_schedule_cmd,
      "autosupport (schedule|interval) [OPT1 ARG1] [OPT2 ARG2] [OPT3 ARG3] [OPT4 ARG4] [OPT5 ARG5]",
      "autosupport command\n"
      "schedule for auto support report generation\n"
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
	  AS_SCHEDULE_CMD_OPTARGS
      AS_SCHEDULE_CMD_OPTARGS)
{
	char 				c;
	char				*token;
	int32				readVal;
	int32				rc;
	event_configMsg_t	eventCfgReq;
    event_configMsg_t	eventCfgRsp;

	uint64				minBitMap = 0;      // frequency of mirroring
	uint32				hourBitMap = 0;
	uint32				dateBitMap = 0;
	uint32				monthBitMap = 0;
	uint32				dayBitMap = 0;

	bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
	bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_SET_REQ;
	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_SCHEDULE);

    optind = 0;
    while ((c = getopt (argc, argv, "m:h:d:M:D:")) != -1)
    {
        switch (c)
        {
			case 'm':
				// read minute interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 0 || readVal > 59)
					{
						printf ("Invalid minute value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(minBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 0 || readVal > 59)
						{
							printf ("Invalid minute value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(minBitMap, readVal);
					}
				}
                //printf("minBitMap = 0x%llx\n", minBitMap);
				break;

			case 'h':
				// read hour interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 0 || readVal > 23)
					{
						printf ("Invalid hour value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(hourBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 0 || readVal > 23)
						{
							printf ("Invalid hour value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(hourBitMap, readVal);
					}
				}
                //printf("hourBitMap = 0x%x\n", hourBitMap);
				break;

			case 'd':
				// read date interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 1 || readVal > 31)
					{
						printf ("Invalid date value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(dateBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 1 || readVal > 31)
						{
							printf ("Invalid date value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(dateBitMap, readVal);
					}
				}
                //printf("dateBitMap = 0x%x\n", dateBitMap);
				break;

			case 'M':
				// read month interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 1 || readVal > 12)
					{
						printf ("Invalid month value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(monthBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 1 || readVal > 12)
						{
							printf ("Invalid month value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(monthBitMap, readVal);
					}
				}
                //printf("monthBitMap = 0x%x\n", monthBitMap);
				break;

			case 'D':
				// read day interval 
				token = strtok(optarg, ",");
				if (token && has_digits(token))
				{
					readVal = strtoq(token, NULL, 0);
					if (readVal < 0 || readVal > 7)
					{
						printf ("Invalid day value\n");
                    	return (CMD_ERR_FAILED);
					}
					SET_BIT(dayBitMap, readVal);
				}
				while (token) {
					token = strtok(NULL, ",");
					if (token && has_digits(token))
					{
						readVal = strtoq(token, NULL, 0);
						if (readVal < 0 || readVal > 7)
						{
							printf ("Invalid day value\n");
                    		return (CMD_ERR_FAILED);
						}
						SET_BIT(dayBitMap, readVal);
					}
				}
                //printf("dayBitMap = 0x%x\n", dayBitMap);
				break;

            case '?':
                if (isprint (optopt))
                    fprintf (stderr, "Unknown option `-%c'.\n", optopt);
                else
                    fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
            	return CMD_ERR_FAILED;

            default:
                fprintf (stderr, "Unknown option `-%c'.\n", optopt);
            	return CMD_ERR_FAILED;

        } // switch
    } // while

	if (optind != argc)
	{
		printf("Syntax error optind %d argc %d\n", optind, argc);
		return CMD_ERR_FAILED;
	}

	/*
	 * Pack the option value string
	 */
	sprintf(eventCfgReq.entry.value, "0x%llx:0x%x:0x%x:0x%x:0x%x",
			minBitMap, hourBitMap, dateBitMap, monthBitMap, dayBitMap);

    /*
     * Send the request to eventd daemon on SSC
     */
    rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
                   "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                    &waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rc != sizeof(event_configMsg_t))
		return CMD_ERR_FAILED;

    return(CMD_SUCCESS);
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_show_stats
 *
 * Description  : Show event stats
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_show_stats,
      autosupport_show_stats_cmd,
      "autosupport show statistics",
      "autosupport command\n"
      "show autosupport related info\n"
      "show autosupport event statistics for this node\n")
{
	int					idx;
	int 				rsplen;
	event_configMsg_t   eventCfgReq;
	event_stats_t		eventStatRsp;

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventStatRsp, sizeof(event_stats_t));

	eventCfgReq.msgType = NFX_EVENT_AS_STATS_VIEW_REQ;

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &eventStatRsp, sizeof(event_stats_t));

	if (rsplen != sizeof(event_stats_t))
		return CMD_ERR_FAILED;

	printf("Event Type        Count       Last Event Time          \n");
	printf("-------------------------------------------------------\n");

	for (idx = 0; idx < NFX_EVENT_MAX; idx++)
	{
		printf
		(
			"%-14s    %-4d        %s", 
			eventTypeStr[idx],
			eventStatRsp.eventRcvd[idx], 
			eventStatRsp.eventRcvd[idx] ?  
				ctime((const time_t*)&eventStatRsp.lastEventTime[idx]):"None\n"
		);
	}

	printf("-------------------------------------------------------\n");

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_clear_stats
 *
 * Description  : Show event stats
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_clear_stats,
      autosupport_clear_stats_cmd,
      "autosupport clear statistics",
      "autosupport command\n"
      "clear autosupport related info\n"
      "clear autosupport event statistics for this node\n")
{
	int 				rsplen;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_STATS_CLEAR_REQ;

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rsplen != sizeof(event_configMsg_t))
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : autosupport_generate_report
 *
 * Description  : Generate instant autosupport report
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(autosupport_generate_report,
      autosupport_generate_report_cmd,
      "autosupport generate report",
      "autosupport command\n"
      "command to generate instant autosupport report\n"
      "generate instant autosupport report\n")
{
	int 				rsplen;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

    bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
    bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_GENERATE_REPORT;

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rsplen != sizeof(event_configMsg_t))
		return CMD_ERR_FAILED;

	if (eventCfgRsp.status != NFX_EVENT_STATUS_OK)
	{
		printf("Verify autosupport mode. Report not generated.\n");
	}

	return CMD_SUCCESS;
}

void
install_autosupport (priv_t myPriv, enum node_type node)
{
    switch (myPriv)
    {
        case diagPriv:
        case fullPriv:
        case addPriv:
        case changePriv:
             install_element(node, &autosupport_state_cmd);
             install_element(node, &autosupport_to_cmd);
             install_element(node, &autosupport_noteto_cmd);
             install_element(node, &autosupport_clear_stats_cmd);
             install_element(node, &autosupport_generate_report_cmd);
             install_element(node, &autosupport_schedule_cmd);

        case viewPriv:
             install_element(node, &autosupport_show_config_cmd);
             install_element(node, &autosupport_show_stats_cmd);
    }
    return;
}
