/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_fsutil.c,v 1.2 2002/04/12 18:18:46 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rahul Khona
 *
 * Date Created: March 12, 2002
 *
 *-----------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "exports/evm-api.h"
#include "exports/fs-api.h"

#include "exports/anpssc-api.h"

#include "exports/nfxnis-api.h"

#include "cmd_fsutil.h"

#define DEFAULT_NETWORK "0.0.0.0"
#define PASSWD_DELIM ':'
int
getUserId(char *name, uid_t *id)
{
    char    *passwd;
    char    *delim, *uidp;
    int   uid;
    passwd = nfxnis_userinfo(DEFAULT_NETWORK, name);
    if (passwd == NULL) {
        return -1;
    }

    /*
     * Parse the passwd entry
     */
    delim = strchr(passwd, PASSWD_DELIM);
    uidp = delim = strchr(delim + 1, PASSWD_DELIM);
    delim = strchr(delim + 1, PASSWD_DELIM);
    *delim = '\0';
    uid = atoi(uidp);
    *delim = PASSWD_DELIM;
    free(passwd);
    *id = uid;
    return 0;
}

/*
 * Given a name, get the corresponding group id
 */

int
getGroupId(char *name, gid_t *id)
{
    char    *group;
    char    *delim, *gidp;
    int   gid;
    group = nfxnis_groupinfo(DEFAULT_NETWORK, name);
    if (group == NULL) {
        return -1;
    }

    /*
     * Parse the passwd entry
     */
    delim = strchr(group, PASSWD_DELIM);
    gidp = delim = strchr(group + 1, PASSWD_DELIM);
    delim = strchr(group + 1, PASSWD_DELIM);
    *delim = '\0';
    gid = atoi(gidp);
    *delim = PASSWD_DELIM;
    free(group);
    *id = gid;
    return 0;
}

void *
alloc_msg(int type, uint32 size)
{
    fs_MsgReqHdr *hdr;
    void *msg;

    msg = (fs_MsgReqHdr *)malloc(size);
    bzero(msg, size);

    hdr = msg;
    hdr->requestType = type;
    hdr->len = size;

    return msg;
}

/*
 * Borrowed from fs
 */

#define EVM_RESP_MAX 1024
static char evmRespBuf[EVM_RESP_MAX];
int
send_evm_message(void *msg, int msgLen)
{
    struct timeval      timeout = { 10, 0};
    int                 rc;

    rc = sendAgileMsg(  -1,     // system assigned tmp socket
                        msg,
                        msgLen,
                        "evm",
                        NFX_SSC_UNIX,
                        0,  // TODO -replace with correct cm api call for SSC slot
                        MGMTPLANE,
                        &timeout,
                        evmRespBuf,
                        EVM_RESP_MAX);
    if (rc <= 0) {
        perror("sendAgileMessage");
        fprintf(stderr, "Failed to communicate with EVM\n");
        return NFX_ERR;
    }
    return NFX_OK;
}
/*
 * Borrowed from fs
 */

int
get_volid(char *id_str, evm_volId_t *volId)
{

    evm_MsgGetId_t      msg;
    evm_MsgStat_t       *respMsg;
    int                 rc;

    msg.hdr.msgType = EVM_VOL_GET_ID;
    strncpy(msg.name, id_str, VOL_MAX_NAME);

    rc = send_evm_message(&msg, sizeof msg);
    if (rc != NFX_OK) {
        fprintf(stderr, "Failed to obtain volume ID\n");
        return 0;
    }
    respMsg = (evm_MsgStat_t *) evmRespBuf;
    if (respMsg->stat != NFX_OK)
    {
        fprintf(stderr, "EVM returned error status (%d)\n", respMsg->stat);
        return FALSE;
    }
    *volId = respMsg->payload.lVolId;
    //printf("returning volid %lld\n", volId->whole_thing);
    return 1;
}
