/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_port.c,v 1.5 2002/08/15 16:58:10 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Greg Seibert
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

#define STORAGE_CMD "/usr/local/agile/bin/storage"

DEFUN  (port_disable,
        port_disable_cmd,
        "port disable PORTIDENTIFIER",
		"FibreChannel Port Command\n"
        "disable FibreChannel Port.\n"
        "Port Identifier of FibreChannel port (e.g. sp.2 or sp2.2)\n")
{
	execute_command (STORAGE_CMD, 5, "control", "port", "disable", argv[0], NULL);
    return CMD_SUCCESS;
}

DEFUN  (port_enable,
        port_enable_cmd,
        "port enable PORTIDENTIFIER",
		"FibreChannel Port Command\n"
        "enable FibreChannel Port.\n"
		"Port Identifier of FibreChannel port (e.g. sp.2 or sp2.2)\n")
{
	execute_command (STORAGE_CMD, 5, "control", "port", "enable", argv[0], NULL);
    return CMD_SUCCESS;
}

DEFUN  (port_reset,
        port_reset_cmd,
        "port reset PORTIDENTIFIER",
		"FibreChannel Port Command\n"
        "reset FibreChannel Port.\n"
		"Port Identifier of FibreChannel port (e.g. sp.2 or sp2.2)\n")
{
	execute_command (STORAGE_CMD, 5, "control", "port", "reset", argv[0], NULL);
    return CMD_SUCCESS;
}

DEFUN  (port_show_detail,
        port_show_detail_cmd,
        "port show detail PORTIDENTIFIER",
		"FibreChannel Port Command\n"
		"Display information about FibreChannel port.\n"
		"Display detailed information about FibreChannel port.\n"
        "Port Identifier of FibreChannel port (e.g. sp.2 or sp2.2)\n")
{
	execute_command (STORAGE_CMD, 5, "view", "port", "detail", argv[0], NULL);
    return CMD_SUCCESS;
}

DEFUN  (port_show_stats,
        port_show_stats_cmd,
        "port show stats PORTIDENTIFIER",
		"FibreChannel Port Command\n"
		"Display information about FibreChannel port.\n"
		"Display detailed statistics about FibreChannel port.\n"
        "Port Identifier of FibreChannel port (e.g. sp.2 or sp2.2)\n")
{
	execute_command (STORAGE_CMD, 5, "view", "port", "stats", argv[0], NULL);
    return CMD_SUCCESS;
}

DEFUN  (port_show_topology,
        port_show_topology_cmd,
        "port show topology PORTIDENTIFIER",
		"FibreChannel Port Command\n"
		"Display information about FibreChannel port.\n"
		"Display detailed FibreChannel fabric topology information from given FibreChannel port.\n"
        "Port Identifier of FibreChannel port (e.g. sp.2 or sp2.2)\n")
{
	execute_command (STORAGE_CMD, 5, "view", "port", "topology", argv[0], NULL);
    return CMD_SUCCESS;
}


void
install_port (priv_t myPriv, enum node_type node)
{
	switch (myPriv)
    {
        case diagPriv:
        case fullPriv:
			install_element (node, &port_disable_cmd);	
			install_element (node, &port_enable_cmd);	
			install_element (node, &port_reset_cmd);	
        case changePriv:
        case addPriv:
        case viewPriv:
			install_element (node, &port_show_detail_cmd);	
			install_element (node, &port_show_stats_cmd); 
			install_element (node, &port_show_topology_cmd); 
    }
    return;
}
