/* 
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Raid Configuration commands
 *
 * Created by:	Nitin Bahadur
 *
 * Date Created: Sep. 4, 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

#define	RAID_CMD		"Raid Command\n"
#define	RAID_DEV_CMD	"Device Manipulation Commands.\n"
#define	RAID_PHY_CMD	"Physical Device Information.\n"
#define	RAID_CFG_CMD	"Configuration commands.\n"
#define	RAID_LUN_CMD	"LUN commands.\n"

DEFUN   (raid_devlist,
         raid_devlist_cmd,
         "raid devlist",
		 RAID_CMD
         "List Raid devices.\n")
{
    execute_command ("raid-ui", 1, "devlist");
    return CMD_SUCCESS;
} 

DEFUN   (raid_open,
         raid_open_cmd,
         "raid dev open DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Open a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "open", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_close,
         raid_close_cmd,
         "raid dev close DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Close a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "close", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_ctrl_reset,
         raid_ctrl_reset_cmd,
         "raid dev reset DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Reset a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "ctrl_reset", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_ctrl_state,
         raid_ctrl_state_cmd,
         "raid dev state DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Get state of a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "ctrl_state", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_ctrl_flush_cache,
         raid_ctrl_flush_cache_cmd,
         "raid dev flush_cache DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Flushes cache of a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "ctrl_flush_cache", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_ctrl_info,
         raid_ctrl_info_cmd,
         "raid dev info DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Get information about a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "ctrl_info", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_ctrl_parm,
         raid_ctrl_parm_cmd,
         "raid dev parm_get DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Get raid device parmeters.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "ctrl_parm", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_ctrl_parm_set,
         raid_ctrl_parm_set_cmd,
         "raid dev parm_set_default DEVNAME",
		 RAID_CMD
		 RAID_DEV_CMD
         "Set default parameters for raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "ctrl_parm_set", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_physdev,
         raid_physdev_cmd,
         "raid info phys_info DEVNAME",
		 RAID_CMD
		 RAID_PHY_CMD
         "Get info on physical devices of raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "physdev", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_physdev_stats,
         raid_physdev_stats_cmd,
         "raid info phys_stats DEVNAME",
		 RAID_CMD
		 RAID_PHY_CMD
         "Get statistics for raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "physdev_stats", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_config_clr,
         raid_config_clr_cmd,
         "raid config clr DEVNAME",
		 RAID_CMD
		 RAID_CFG_CMD
         "Clears configuration of a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "config_clr", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_config_save,
         raid_config_save_cmd,
         "raid config save DEVNAME FILEPATH",
		 RAID_CMD
		 RAID_CFG_CMD
         "Saves configuration of a raid device.\n"
		 "Raid device name.\n"
		 "File Path.\n")
{
    execute_command ("raid-ui", 3, "config_save", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_config_load,
         raid_config_load_cmd,
         "raid config load DEVNAME FILEPATH",
		 RAID_CMD
		 RAID_CFG_CMD
         "Loads configuration of a raid device.\n"
		 "Raid device name.\n"
		 "File Path.\n")
{
    execute_command ("raid-ui", 3, "config_load", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_list,
         raid_lun_list_cmd,
         "raid lun list DEVNAME",
		 RAID_CMD
		 RAID_LUN_CMD
         "Gets list of LUNs for a raid device.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "lun_list", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_bg_status,
         raid_lun_bg_status_cmd,
         "raid lun bg_status DEVNAME",
		 RAID_CMD
		 RAID_LUN_CMD
         "Raid device background status.\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 2, "lun_bg_status", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_create_simple,
         raid_lun_create_simple_cmd,
         "raid lun create_simple DEVNAME LUN-COUNT RAID_TYPE SIZE-IN-MB STRIPESIZE-IN-KB",
		 RAID_CMD
		 RAID_LUN_CMD
         "Create a simple LUN for a raid device.\n"
		 "Raid device name.\n"
		 "LUN-count.\n"
		 "Raid Type.\n"
		 "Size in MB.\n"
		 "Stripesize in KB.\n")
{
    execute_command ("raid-ui", 6, "lun_create_simple", argv[0], argv[1], argv[2], argv[3], argv[4]);
    return CMD_SUCCESS;
} 

DEFUN	(raid_lun_create,
		 raid_lun_create_cmd,
		 "raid lun create DEVNAME RAID-TYPE SIZE-IN-MB STRIPESIZE-IN-KB CID/TID .",
		 RAID_CMD
		 RAID_LUN_CMD
		 "Create a LUN for a raid device.\n"
		 "Raid device name.\n"
		 "Raid Tyoe.\n"
		 "Size in MB.\n"
		 "Stripesize in KB.\n"
		 "CID/TID pair.\n"
		 "CID/TID pairs.\n")
{
	execute_vararg_command("raid-ui", argc, "lun_create", argv);
	return CMD_SUCCESS;
}

DEFUN   (raid_lun_delete,
         raid_lun_delete_cmd,
         "raid lun delete DEVNAME LUN-id",
		 RAID_CMD
		 RAID_LUN_CMD
         "Deletes a LUN from a raid device.\n"
		 "Raid device name.\n"
		 "LUN id\n.")
{
    execute_command ("raid-ui", 3, "lun_delete", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_init,
         raid_lun_init_cmd,
         "raid lun init DEVNAME ( LUN-id | ALL | ALLNOT )",
		 RAID_CMD
		 RAID_LUN_CMD
         "Initialize raid device LUN(s).\n"
		 "Raid device name.\n")
{
    execute_command ("raid-ui", 3, "lun_init", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_check,
         raid_lun_check_cmd,
         "raid lun check DEVNAME LUN-id",
		 RAID_CMD
		 RAID_LUN_CMD
         "Perform check on raid device LUN.\n"
		 "Raid device name.\n"
		 "LUN id\n.")
{
    execute_command ("raid-ui", 3, "lun_check", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_physlist,
         raid_lun_physlist_cmd,
         "raid lun physlist DEVNAME LUN-id",
		 RAID_CMD
		 RAID_LUN_CMD
         "Get physical storage info for raid device LUN.\n"
		 "Raid device name.\n"
		 "LUN id\n.")
{
    execute_command ("raid-ui", 3, "lun_physlist", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_lun_cache,
         raid_lun_cache_cmd,
         "raid lun cache DEVNAME LUN-id ReadCache WriteCache",
		 RAID_CMD
		 RAID_LUN_CMD
         "Sets the read and write caching parameters for a lun cache.\n"
		 "Raid device name.\n"
		 "LUN id\n."
		 "ON | OFF.\n"
		 "ON | OFF.\n")
{
    execute_command ("raid-ui", 5, "lun_cache", argv[0], argv[1], argv[2], argv[3]);
    return CMD_SUCCESS;
} 

DEFUN   (raid_dbg_level,
         raid_dbg_level_cmd,
         "raid set_debug Value",
		 RAID_CMD
         "Sets the debug level for raid debugging.\n"
		 "Raid debugging level.\n")
{
    execute_command ("raid-ui", 2, "dbg_level", argv[0]);
    return CMD_SUCCESS;
} 

void
install_raid(priv_t myPriv, enum node_type node)
{
	switch (myPriv)
    {
        case diagPriv:
			install_element (node, &raid_dbg_level_cmd);
        case fullPriv:
			install_element (node, &raid_config_clr_cmd);	
			install_element (node, &raid_config_load_cmd);	
			install_element (node, &raid_ctrl_parm_set_cmd);	
			install_element (node, &raid_lun_create_simple_cmd);	
        case changePriv:
			install_element (node, &raid_config_save_cmd);	
			install_element (node, &raid_devlist_cmd);	
			install_element (node, &raid_open_cmd);	
			install_element (node, &raid_close_cmd);	
			install_element (node, &raid_ctrl_reset_cmd);	
			install_element (node, &raid_ctrl_state_cmd);
			install_element (node, &raid_ctrl_flush_cache_cmd);	
			install_element (node, &raid_ctrl_info_cmd);	
			install_element (node, &raid_ctrl_parm_cmd);	
			install_element (node, &raid_physdev_cmd);	
			install_element (node, &raid_physdev_stats_cmd);	
			install_element (node, &raid_lun_list_cmd);	
			install_element (node, &raid_lun_bg_status_cmd);	
			install_element (node, &raid_lun_create_cmd); 
			install_element (node, &raid_lun_delete_cmd);	
			install_element (node, &raid_lun_init_cmd);	
			install_element (node, &raid_lun_check_cmd);	
			install_element (node, &raid_lun_physlist_cmd);	
			install_element (node, &raid_lun_cache_cmd);	
        case addPriv:
        case viewPriv:
    }
    return;
}
