/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: nfxsh.h
 *
 * RCS:  $Id: nfxsh.h,v 1.26 2002/10/29 02:17:30 syen Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: All the definitions worth defining (global) for the NFX
 *              CLI.
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 23 Oct 2001
 *
 *-----------------------------------------------------------------
 */
#if !defined(__nfxsh_h)
#define __nfxsh_h

#include <sys/types.h>

#include "nfx-types.h"

// Where helper executables live

#define AGILE_BIN "/usr/local/agile/bin"

// The admin privledges and their group names. Keep these three synchronized

#define NUMPRIVS 5

typedef enum { viewPriv = 0, 
               addPriv, 
               changePriv, 
               fullPriv, 
               diagPriv } priv_t;

extern char *groupNames[NUMPRIVS];


// Function prototypes for global methods

extern void become_root(void);
extern void unbecome_root(void);

extern void install_admin (priv_t myPriv, enum node_type node);
extern void install_arp (priv_t myPriv, enum node_type node);
extern void install_cluster (priv_t myPriv, enum node_type node);
extern void install_fileset (priv_t myPriv, enum node_type node);
extern void install_interface (priv_t myPriv, enum node_type node);
extern void install_vif (priv_t myPriv, enum node_type node);
extern void install_license (priv_t myPriv, enum node_type node);
extern void install_lun (priv_t myPriv, enum node_type node);
extern void install_nameservice (priv_t myPriv, enum node_type node);
extern void install_ndmp (priv_t myPriv, enum node_type node);
extern void install_nfs (priv_t myPriv, enum node_type node);
extern void install_port (priv_t myPriv, enum node_type node);
extern void install_quota (priv_t myPriv, enum node_type node);
extern void install_rcon (priv_t myPriv, enum node_type node);
extern void install_route (priv_t myPriv, enum node_type node);
extern void install_snapmirror (priv_t myPriv, enum node_type node);
extern void install_snapshot (priv_t myPriv, enum node_type node);
extern void install_snmp (priv_t myPriv, enum node_type node);
extern void install_spool (priv_t myPriv, enum node_type node);
extern void install_system (priv_t myPriv, enum node_type node);
extern void install_vol (priv_t myPriv, enum node_type node);
extern void install_lvol (priv_t myPriv, enum node_type node);
extern void install_help (priv_t myPriv, enum node_type node);
extern void install_raid (priv_t myPriv, enum node_type node);
extern void install_scsi (priv_t myPriv, enum node_type node);
extern void install_tape (priv_t myPriv, enum node_type node);
extern void install_disk (priv_t myPriv, enum node_type node);
extern void install_am (priv_t myPriv, enum node_type node);
extern void install_elog (priv_t myPriv, enum node_type node);
extern void install_av (priv_t myPriv, enum node_type node);
extern void install_autosupport (priv_t myPriv, enum node_type node);
extern void install_stats(priv_t myPriv, enum node_type node);

extern int do_upgrade(int argc, char **argv);
extern int do_show_version(int argc, char **argv);
extern int do_patch(int argc, char **argv);

extern int do_copy_init(int argc, char **argv);
extern int do_copy_all(int argc, char **argv);
extern int do_copy_config(int argc, char **argv);
extern int do_prom_upgrade(int argc, char **argv);
extern int do_restore_config(int argc, char **argv);
extern int do_reset_config(int argc, char **argv);
extern void shutdown_all(void);
extern void reboot_all(void);

// IP stuff helper

extern int32 net_atoipmask(char *, uint32 *, uint32 *);

extern in_addr_t nfxsh_nis_hostAddr(char *exportAddr, char *hostName);

// John's feel-good thumb twiddler

extern pid_t start_twiddle (void);
extern void stop_twiddle (pid_t twid);

// And Max's daemon weapon

extern void kill_daemon (char *pid_file);

extern int node_has_services (char *nodeName);
extern void remove_cluster_services (void);
extern void warn_need_reboot (void);

// Upgrade stuff used elsewhere

extern int ensure_dir_exists(const char *path, int mode);
extern int get_our_addr(in_addr_t them, char *iface, in_addr_t *us);



#endif
