/*
 *-----------------------------------------------------------------
 *
 * Name: Hardware System Health Monitor 
 *
 * RCS:  $Id:
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Connects to specifed CPUs using rcon and collects
 * 							hardware statistics.
 *
 * Created by:	Christa Porter
 *
 * Date Created: 20 Sep 2002
 *
 *-----------------------------------------------------------------
 */

#include "sysHealth.h"

static void sig_handler(int);

int main(int argc, char *argv[]) {

	struct sigaction nint_action;

	nint_action.sa_handler = sig_handler;
	nint_action.sa_flags = 0;
	sigemptyset(&nint_action.sa_mask);
	if(sigaction(SIGSEGV, &nint_action, NULL) == -1)
		fprintf(stderr, "Can't handle SIGSEGV signal\n");
	if(sigaction(SIGBUS, &nint_action, NULL) == -1)
		fprintf(stderr, "Can't handle SIGBUS signal\n");
		
	if (argc < 2) {
		printf ("Usage %s <card(fp, ssc, sp or all)>\n", argv[0]);
		exit (1);
	}
	if(!strcmp(argv[1], "fp")) {
		sysHealthFP();
		sysHealthSum(1, 0, 0);
	} else if(!strcmp(argv[1], "ssc")) {
		sysHealthSSC();
		sysHealthSum(0, 1, 0);
	} else if(!strcmp(argv[1], "sp")) {
		sysHealthSP();
		sysHealthSum(0, 0, 1);
	} else if(!strcmp(argv[1], "all")) {
		sysHealthFP();
		sysHealthSSC();
		sysHealthSP();
		sysHealthSum(1, 1, 1);
	} else { 
		printf ("Usage %s card(fp, ssc, sp or all)\n", argv[0]);
		exit (1);
	}

	exit(0);
}

static void sig_handler(int signo) {
	shutdown_rcon_session();
	fprintf(stderr, "Signal handled by sig_handler\n");
	exit(1);
}

