#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"
#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "unistd.h"

#include "../sm-gcov/ssc-stdio.h"

#undef DEBUG
#if defined(DEBUG)
#define debug(x) printf x
#else
#define debug(x)
#endif

int myappid;
char *myname = "stdio";
int sock;

static void
mmm()
{
}

size_t buflen = 200 * 1024;

int
main(int argc, char **argv)
{
	char *buf = malloc(buflen);
	myappid = eee_getApplicationId(myname);

	sock = regAgileApp(myname);

	if (sock < 0) {
		E_LOG(class_1, err_s, myappid, 0, 0, 0,
			  ("can not register application: %m\n"));
		exit(1);
	}

	signal(SIGHUP, mmm);

#if 0
	kill(getppid(), SIGUSR1);
#endif

	while (1) {
		uint32 cpu, slot, plane;
		char appName[256];
		int rv = recvAgileMsg(sock, buf, buflen, NULL,
							  appName, &cpu, &slot, &plane);
		struct ssc_stdio_msg_hdr *hdr = (struct ssc_stdio_msg_hdr *)buf;
		if (rv > 0) {
			switch (hdr->type) {
			case SSC_STDIO_OPEN:
			{
				struct ssc_stdio_open_msg *msg =
					(struct ssc_stdio_open_msg *)buf;
				struct ssc_stdio_open_rsp *rsp =
					(struct ssc_stdio_open_rsp *)buf;
				char *name = buf + sizeof(*msg);
				char *mode = buf + sizeof(*msg) + msg->namelen + 1;
				FILE *f;
				debug(("open: name = %s mode = %s\n", name, mode));
				f = fopen(name, mode);
				rsp->fd = (uint32_t)f;
				debug(("got fd=0x%x\n", rsp->fd));
				sendAgileMsg(sock, rsp, sizeof(*rsp),
							 appName, cpu, slot, plane, NULL, NULL, NULL);
				break;
			}
			case SSC_STDIO_READ:
			{
				struct ssc_stdio_read_msg *msg =
					(struct ssc_stdio_read_msg *)buf;
				struct ssc_stdio_read_rsp *rsp =
					(struct ssc_stdio_read_rsp *)buf;
				uint32_t nelm = msg->nelm;
				uint32_t sz = msg->sz;
				FILE *f = (FILE *)msg->fd;
				debug(("read: fd = 0x%x sz = %d nelm = %d\n",
					   (uint32_t)f, sz, nelm));
				rsp->nelm = fread(buf + sizeof(*rsp), sz, nelm, f);
				sendAgileMsg(sock, rsp, sizeof(*rsp) + sz * rsp->nelm,
							 appName, cpu, slot, plane, NULL, NULL, NULL);
				break;
			}
			case SSC_STDIO_WRITE:
			{
				struct ssc_stdio_write_msg *msg =
					(struct ssc_stdio_write_msg *)buf;
				struct ssc_stdio_write_rsp *rsp =
					(struct ssc_stdio_write_rsp *)buf;
				uint32_t nelm = msg->nelm;
				uint32_t sz = msg->sz;
				FILE *f = (FILE *)msg->fd;

				debug(("write: fd=0x%x sz = %d nelm = %d\n",
					   (uint32_t)f, sz, nelm));
				rsp->nelm = fwrite(buf + sizeof(*msg), sz, nelm, f);
				sendAgileMsg(sock, rsp, sizeof(*rsp),
							 appName, cpu, slot, plane, NULL, NULL, NULL);
				break;
			}
			case SSC_STDIO_SEEK:
			{
				struct ssc_stdio_seek_msg *msg =
					(struct ssc_stdio_seek_msg *)buf;
				struct ssc_stdio_seek_rsp *rsp =
					(struct ssc_stdio_seek_rsp *)buf;
				uint32_t off = msg->offset;
				uint32_t whence = msg->whence;
				FILE *f = (FILE *)msg->fd;
				rsp->offset = fseek(f, off, whence);
				sendAgileMsg(sock, rsp, sizeof(*rsp),
							 appName, cpu, slot, plane, NULL, NULL, NULL);
				break;
			}
			case SSC_STDIO_CLOSE:
			{
				struct ssc_stdio_close_msg *msg =
					(struct ssc_stdio_close_msg *)buf;
				struct ssc_stdio_close_rsp *rsp =
					(struct ssc_stdio_close_rsp *)buf;
				FILE *f = (FILE *)msg->fd;
				debug(("close\n"));
				fclose(f);
				sendAgileMsg(sock, rsp, sizeof(*rsp),
							 appName, cpu, slot, plane, NULL, NULL, NULL);
				break;
			}
			}
		}
		else {
			E_LOG(class_1, err_s, myappid, 0, 0, 0,
				  ("recvAgileMsg failed: %m\n"));
			exit(1);
		}
	}

	return 0;
}
