/*
 *------------------------------------------------------------------------------
 *
 * Name:	raid-xyratex.c
 *
 * RCS:		$Id: raid-xyratex.c,v 1.29 2002/10/23 18:37:03 nitinb Exp $
 *
 * Copyright (C) 2001, 2002 Agile Storage Inc.
 *
 * Description:	RAID config manager Xyratex specific code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 4/19/01
 *
 *------------------------------------------------------------------------------
 */

#include "raid-common.h"
#include "raid-xyratex.h"
#include "../sm-scsi/scsi-dev-cmds.h"
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>

extern int raid_xyratex_setup_cmd_cdb(raid_phyPassThruReqMsg_t *, 
			   		sdm_relayCdbReqMsg_t *);

char *	 FSI_HardwareIDToString(byte);
char *	 FSI_devTypeToString(byte);
char *	 FSI_cachelineSizeToString(byte);
uint32	 FSI_cachelineSizeToKB(byte);
char *	 FSI_stripeSizeToString(byte);
uint32	 FSI_stripeSizeToKB(byte);
uint32   FSI_KBTostripeSize(int32);

static int
raid_xyratex_emit_lun_config(FILE *f, char *config, int config_len)
{
	FSI_phys_dev_def_t *	pdd;
	FSI_used_dev_def_t *	udd;
	FSI_logical_dev_def_t * ldd;
	int			i;

	fprintf(f, "###############################################\n");
	fprintf(f, "# Logical device configuration.                \n");
	fprintf(f, "###############################################\n");

	while (config_len) {
		switch (*config) {
		case 0x0F:
			pdd = (FSI_phys_dev_def_t *)config;
			fprintf(f, "<PDD>\n");
#if 0
			fprintf(f, "\trdn=0x%04x\n",			pdd->rdn);
			fprintf(f, "\tcontroller_num=%d\n",		pdd->controller_num);
			fprintf(f, "\tchannel_num=%d\n",		pdd->channel_num);
			fprintf(f, "\ttarget_id=%d\n",			pdd->target_id);
			fprintf(f, "\tdev_size=0x%08x\n",		pdd->dev_size);
			fprintf(f, "\tdev_start_addr=0x%08x\n",		pdd->dev_start_addr);
#else
			fprintf(f, "\trdn=0x%04x ",			pdd->rdn);
			fprintf(f, "controller_num=%d ",		pdd->controller_num);
			fprintf(f, "channel_num=%d ",			pdd->channel_num);
			fprintf(f, "target_id=%d ",			pdd->target_id);
			fprintf(f, "dev_size=0x%08x ",			pdd->dev_size);
			fprintf(f, "dev_start_addr=0x%08x\n",		pdd->dev_start_addr);
#endif
			fprintf(f, "</PDD>\n");
			config += sizeof(FSI_phys_dev_def_t);
			config_len -= sizeof(FSI_phys_dev_def_t);
			break;

		default:
			ldd = (FSI_logical_dev_def_t *)config;
			fprintf(f, "<LDD>\n");
#if 0
			fprintf(f, "\trdn=0x%04x\n",			ldd->rdn);
			fprintf(f, "\tdev_type=0x%02x\n",		ldd->dev_type);
			fprintf(f, "\tdev_size=0x%08x\n",		ldd->dev_size);
			fprintf(f, "\tstripe_size=%s\n",		FSI_stripeSizeToString(ldd->stripe_size));
			fprintf(f, "\tcacheline_size=%s\n",		FSI_cachelineSizeToString(ldd->cacheline_size));
			fprintf(f, "\trw_control=0x%02x\n",		ldd->rw_control);
			fprintf(f, "\tudd_count=%d\n",			ldd->dev_used);
			fprintf(f, "\tudd_depth=0x%08x\n",		ldd->used_devsize);
#else
			fprintf(f, "\trdn=0x%04x ",			ldd->rdn);
			fprintf(f, "dev_type=0x%02x ",			ldd->dev_type);
			fprintf(f, "dev_size=0x%08x ",			ldd->dev_size);
			fprintf(f, "stripe_size=%s ",			FSI_stripeSizeToString(ldd->stripe_size));
			fprintf(f, "cacheline_size=%s ",		FSI_cachelineSizeToString(ldd->cacheline_size));
			fprintf(f, "rw_control=0x%02x ",		ldd->rw_control);
			fprintf(f, "udd_count=%d ",			ldd->dev_used);
			fprintf(f, "udd_depth=0x%08x\n",		ldd->used_devsize);
#endif
			config += sizeof(FSI_logical_dev_def_t);
			config_len -= sizeof(FSI_logical_dev_def_t);

			for (i = 0; i < ldd->dev_used; ++i) {
				udd = (FSI_used_dev_def_t *)config;
				fprintf(f, "\t<UDD>\n");
#if 0
				fprintf(f, "\t\trdn=0x%04x\n",			udd->rdn);
				fprintf(f, "\t\tdev_start_addr=0x%08x\n",	udd->dev_start_addr);
#else
				fprintf(f, "\t\trdn=0x%04x ",			udd->rdn);
				fprintf(f, "dev_start_addr=0x%08x\n",		udd->dev_start_addr);
#endif
				fprintf(f, "\t</UDD>\n");
				config += sizeof(FSI_used_dev_def_t);
				config_len -= sizeof(FSI_used_dev_def_t);
			}
			fprintf(f, "</LDD>\n");
			fprintf(f, "\n");
		}
	}
	
	return(NFX_OK);
}

static int
raid_xyratex_emit_ctrl_config(FILE *f, raid_ctrlParam_t *p)
{
	fprintf(f, "###############################################\n");
	fprintf(f, "# Controller parameter configuration.          \n");
	fprintf(f, "###############################################\n");

	fprintf(f, "<CPARMS>\n");

	fprintf(f, "\treadAhead=%d\n",		p->readAhead);
	fprintf(f, "\treasRestrict=%d\n",	p->reasRestrict);
	fprintf(f, "\ttrueVerify=%d\n",	p->trueVerify);
	fprintf(f, "\twriteVerify=%d\n",	p->writeVerify);
	fprintf(f, "\tbgInit=%d\n",		p->bgInit);
	fprintf(f, "\tsuperReadAhead=%d\n",	p->superReadAhead);
	fprintf(f, "\tcoalOptimal=%d\n",	p->coalOptimal);
	fprintf(f, "\tautoRebuild=%d\n",	p->autoRebuild);
	fprintf(f, "\toperFault=%d\n",		p->operFault);
	fprintf(f, "\trebuildRate=%d\n",	p->rebuildRate);
	fprintf(f, "\tinitRate=%d\n",		p->initRate);
	fprintf(f, "\tconsistencyCheckRate=%d\n",p->consistencyCheckRate);
	fprintf(f, "\tstartupMode=%d\n",	p->startupMode);
	fprintf(f, "\tstartupNumDev=%d\n",	p->startupNumDev);
	fprintf(f, "\tstartupDelay=%d\n",	p->startupDelay);
	fprintf(f, "\tscsiStartupDelay=%d\n",	p->scsiStartupDelay);
	fprintf(f, "\tvendorTUR=%d\n",		p->vendorTUR);
	fprintf(f, "\tdisCCInvLun=%d\n",	p->disCCInvLun);
	fprintf(f, "\tnoPause=%d\n",		p->noPause);
	fprintf(f, "\tqueueFullBusy=%d\n",	p->queueFullBusy);
	fprintf(f, "\tdisBusyFailback=%d\n",	p->disBusyFailback);
	fprintf(f, "\tnodeNameRet=%d\n",	p->nodeNameRet);
	fprintf(f, "\tsafteofups=%d\n",	p->safteofups);
	fprintf(f, "\ttopology=%d\n",		p->topology);
	fprintf(f, "\tresetProp=%d\n",		p->resetProp);
	fprintf(f, "\trs232PortType=%d\n",	p->rs232PortType);
	fprintf(f, "\trs232BaudRate=%d\n",	p->rs232BaudRate);
	fprintf(f, "\tsmartLrgTrans=%d\n",	p->smartLrgTrans);
	fprintf(f, "\tframeSize=%d\n",		p->frameSize);
	fprintf(f, "\tpciLatCtrl=%d\n",	p->pciLatCtrl);
	fprintf(f, "\tautoFailback=%d\n",	p->autoFailback);
	fprintf(f, "\tforceSimplex=%d\n",	p->forceSimplex);
	fprintf(f, "\tconserveCache=%d\n",	p->conserveCache);
	fprintf(f, "\tnoRSTCOM=%d\n",		p->noRSTCOM);
	fprintf(f, "\tqueueLimit=%d\n",	p->queueLimit);
	fprintf(f, "\tctrl0Port0=%d\n",	p->ctrl0Port0);
	fprintf(f, "\tctrl0Port1=%d\n",	p->ctrl0Port1);
	fprintf(f, "\tctrl1Port0=%d\n",	p->ctrl1Port0);
	fprintf(f, "\tctrl1Port1=%d\n",	p->ctrl1Port1);
	fprintf(f, "\tctrl0Port0LoopId=%d\n",	p->ctrl0Port0LoopId);
	fprintf(f, "\tctrl0Port1LoopId=%d\n",	p->ctrl0Port1LoopId);
	fprintf(f, "\tctrl1Port0LoopId=%d\n",	p->ctrl1Port0LoopId);
	fprintf(f, "\tctrl1Port1LoopId=%d\n",	p->ctrl1Port1LoopId);
	fprintf(f, "\tdebugDump=%d\n",		p->debugDump);
	fprintf(f, "\trofInterval=%d\n",	p->rofInterval);
	fprintf(f, "\trofReboot=%d\n",		p->rofReboot); 

	fprintf(f, "</CPARMS>\n");

	return(NFX_OK);
}

#define MAX_LINE_LENGTH		256
static int
raid_xyratex_parse_config_file(FILE *f, raid_ctrlParam_t *cparms, char *config, int *in_config_len)
{
	char				line[MAX_LINE_LENGTH];
	char *				variable;
	uint32				value;
	char *				tok;
	char *				p;
	int				parsing_pdd = 0;
	int				parsing_ldd = 0;
	int				parsing_udd = 0;
	int				parsing_cparms = 0;
	FSI_phys_dev_def_t *		pdd;
	FSI_used_dev_def_t *		udd;
	FSI_logical_dev_def_t *		ldd;
	int				out_config_len = 0;

	while (fgets(line, MAX_LINE_LENGTH, f)) {
		p = line;
		while ((tok = strtok_r(p, " \t", &p)) != NULL) {
			if (*tok == '#')
				goto END_OF_LINE;

			if (strstr(tok, "<PDD>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: <PDD>\n"));
				pdd = (FSI_phys_dev_def_t *)config;
				bzero(pdd, sizeof(FSI_phys_dev_def_t));
				config += sizeof(FSI_phys_dev_def_t);
				out_config_len += sizeof(FSI_phys_dev_def_t);
				parsing_pdd = 1;
			} 
			else
			if (strstr(tok, "</PDD>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: </PDD>\n"));
				pdd->dev_type = 0x0F;
				pdd->dev_state = RAID_PHYDEVSTATE_OPTIMAL;
				pdd = NULL;
				parsing_pdd = 0;
			} 
			else
			if (strstr(tok, "<LDD>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: <LDD>\n"));
				ldd = (FSI_logical_dev_def_t *)config;
				bzero(ldd, sizeof(FSI_logical_dev_def_t));
				config += sizeof(FSI_logical_dev_def_t);
				out_config_len += sizeof(FSI_logical_dev_def_t);
				parsing_ldd = 1;
			} 
			else
			if (strstr(tok, "</LDD>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: </LDD>\n"));
				ldd->dev_state = RAID_LOGDEVSTATE_OPTIMAL;
				ldd = NULL;
				parsing_ldd = 0;
			} 
			else
			if (strstr(tok, "<UDD>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: <UDD>\n"));
				udd = (FSI_used_dev_def_t *)config;
				bzero(udd, sizeof(FSI_used_dev_def_t));
				config += sizeof(FSI_used_dev_def_t);
				out_config_len += sizeof(FSI_used_dev_def_t);
				parsing_udd = 1;
			} 
			else
			if (strstr(tok, "</UDD>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: </UDD>\n"));
				udd->dev_state = RAID_LOGDEVSTATE_OPTIMAL;
				udd = NULL;
				parsing_udd = 0;
			} 
			else
			if (strstr(tok, "<CPARMS>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: <CPARMS>\n"));
				bzero(cparms, sizeof(raid_ctrlParam_t));
				parsing_cparms = 1;
			} 
			else
			if (strstr(tok, "</CPARMS>")) {
				RAID_DBG(2, ("raid_xyratex_parse_config_file: </CPARMS>\n"));
				parsing_cparms = 0;
			} 
			else
			if (*tok != '\0' && *tok != '\n' && *tok != '\t' && *tok != ' ') {
				if ((variable = strtok_r(tok, "=", &tok)) == NULL) {
					printf("variable = %s, tok = %s\n", variable, tok);
					return(NFX_ERR);
				}
				value = strtol(tok, &tok, 0);
				if (*tok != '\0' && *tok != '\n' && *tok != '\t' && *tok != ' ') {
					printf("end char = %d\n", *tok);
					printf("variable = %s, value = %d, tok = %s\n", variable, value, tok); 
					return(NFX_ERR);
				}
				RAID_DBG(2, ("raid_xyratex_parse_config_file: variable '%s, value 0x%x\n", variable, value));
				if (parsing_pdd) {
					if (strstr(variable, "rdn"))		pdd->rdn = value; else
					if (strstr(variable, "controller_num")) pdd->controller_num = value; else
					if (strstr(variable, "channel_num"))	pdd->channel_num = value; else
					if (strstr(variable, "target_id"))	pdd->target_id = value; else
					if (strstr(variable, "dev_size"))	pdd->dev_size = value; else
					if (strstr(variable, "dev_start_addr")) pdd->dev_start_addr = value; 
					else {
						RAID_DBG(2, ("raid_xyratex_parse_config_file: "
							     "Invalid variable '%s' while parsing PDD.\n", 
							     variable));
						return(NFX_ERR);
					}
				}
				else
				if (parsing_ldd && !parsing_udd) {
					if (strstr(variable, "rdn"))		ldd->rdn = value; else
					if (strstr(variable, "dev_type"))	ldd->dev_type = value; else
					if (strstr(variable, "dev_size"))	ldd->dev_size = value; else
					if (strstr(variable, "stripe_size"))	ldd->stripe_size = FSI_KBTostripeSize(value); else
					if (strstr(variable, "cacheline_size")) ldd->cacheline_size = FSI_KBTostripeSize(value); else
					if (strstr(variable, "rw_control"))	ldd->rw_control = value; else
					if (strstr(variable, "udd_count"))	ldd->dev_used = value; else
					if (strstr(variable, "udd_depth"))	ldd->used_devsize = value; 
					else {
						RAID_DBG(2, ("raid_xyratex_parse_config_file: "
							     "Invalid variable '%s' while parsing LDD.\n", 
							     variable));
						return(NFX_ERR);
					}
				}
				else
				if (parsing_udd) {
					if (strstr(variable, "rdn"))		udd->rdn = value; else
					if (strstr(variable, "dev_start_addr")) udd->dev_start_addr = value; 
					else {
						RAID_DBG(2, ("raid_xyratex_parse_config_file: "
							     "Invalid variable '%s' while parsing UDD.\n", 
							     variable));
						return(NFX_ERR);
					}
				}
				else
				if (parsing_cparms) {
					if (strstr(variable, "readAhead"))		cparms->readAhead = value; else
					if (strstr(variable, "reasRestrict"))		cparms->reasRestrict = value; else
					if (strstr(variable, "trueVerify"))		cparms->trueVerify = value; else
					if (strstr(variable, "writeVerify"))		cparms->writeVerify = value; else
					if (strstr(variable, "bgInit"))			cparms->bgInit = value; else
					if (strstr(variable, "superReadAhead"))		cparms->superReadAhead = value; else
					if (strstr(variable, "coalOptimal"))		cparms->coalOptimal = value; else
					if (strstr(variable, "autoRebuild"))		cparms->autoRebuild = value; else
					if (strstr(variable, "operFault"))		cparms->operFault = value; else
					if (strstr(variable, "rebuildRate"))		cparms->rebuildRate = value; else
					if (strstr(variable, "initRate"))		cparms->initRate = value; else
					if (strstr(variable, "consistencyCheckRate"))	cparms->consistencyCheckRate = value; else
					if (strstr(variable, "startupMode"))		cparms->startupMode = value; else
					if (strstr(variable, "startupNumDev"))		cparms->startupNumDev = value; else
					if (strstr(variable, "startupDelay"))		cparms->startupDelay = value; else
					if (strstr(variable, "scsiStartupDelay"))	cparms->scsiStartupDelay = value; else
					if (strstr(variable, "vendorTUR"))		cparms->vendorTUR = value; else
					if (strstr(variable, "disCCInvLun"))		cparms->disCCInvLun = value; else
					if (strstr(variable, "noPause"))		cparms->noPause = value; else
					if (strstr(variable, "queueFullBusy"))		cparms->queueFullBusy = value; else
					if (strstr(variable, "disBusyFailback"))	cparms->disBusyFailback = value; else
					if (strstr(variable, "nodeNameRet"))		cparms->nodeNameRet = value; else
					if (strstr(variable, "safteofups"))		cparms->safteofups = value; else
					if (strstr(variable, "topology"))		cparms->topology = value; else
					if (strstr(variable, "resetProp"))		cparms->resetProp = value; else
					if (strstr(variable, "rs232PortType"))		cparms->rs232PortType = value; else
					if (strstr(variable, "rs232BaudRate"))		cparms->rs232BaudRate = value; else
					if (strstr(variable, "smartLrgTrans"))		cparms->smartLrgTrans = value; else
					if (strstr(variable, "frameSize"))		cparms->frameSize = value; else
					if (strstr(variable, "pciLatCtrl"))		cparms->pciLatCtrl = value; else
					if (strstr(variable, "autoFailback"))		cparms->autoFailback = value; else
					if (strstr(variable, "forceSimplex"))		cparms->forceSimplex = value; else
					if (strstr(variable, "conserveCache"))		cparms->conserveCache = value; else
					if (strstr(variable, "noRSTCOM"))		cparms->noRSTCOM = value; else
					if (strstr(variable, "queueLimit"))		cparms->queueLimit = value; else
					if (strstr(variable, "ctrl0Port0"))		cparms->ctrl0Port0 = value; else
					if (strstr(variable, "ctrl0Port1"))		cparms->ctrl0Port1 = value; else
					if (strstr(variable, "ctrl1Port0"))		cparms->ctrl1Port0 = value; else
					if (strstr(variable, "ctrl1Port1"))		cparms->ctrl1Port1 = value; else
					if (strstr(variable, "ctrl0Port0LoopId"))	cparms->ctrl0Port0LoopId = value; else
					if (strstr(variable, "ctrl0Port1LoopId"))	cparms->ctrl0Port1LoopId = value; else
					if (strstr(variable, "ctrl1Port0LoopId"))	cparms->ctrl1Port0LoopId = value; else
					if (strstr(variable, "ctrl1Port1LoopId"))	cparms->ctrl1Port1LoopId = value; else
					if (strstr(variable, "debugDump"))		cparms->debugDump = value; else
					if (strstr(variable, "rofInterval"))		cparms->rofInterval = value; else
					if (strstr(variable, "rofReboot"))		cparms->rofReboot = value;
					else {
						RAID_DBG(2, ("raid_xyratex_parse_config_file: "
							     "Invalid variable '%s' while parsing CPARMS.\n", 
							     variable));
						return(NFX_ERR);
					}
				}
			}
		}
	END_OF_LINE:
	}
	*in_config_len = out_config_len;
	return(NFX_OK);
}

static int32
raid_xyratex_ctrl_reset_constructor(raid_dev_t *		dev, 
				    raid_msgHdr_t *		msg_hdr, 
				    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				    int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_ctrl_reset_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_RESET_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_RESETDEVICE;
	req_msg->cdb[6] = MDADEV_PHYSICALCONTROLLER;

	RAID_DBG(2, ("raid_xyratex_ctrl_reset_constructor: dev 0x%p: MDACIOCTL_RESETDEVICE\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_ctrl_reset_deconstructor(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				      sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				      int32 *			sdm_relay_req_len,
				      raid_msgHdr_t **		rsp_msg, 
				      int32 *			rsp_msg_len)
{
	raid_ctrlResetRspMsg_t *	rsp;

	RAID_DBG(2, ("raid_xyratex_ctrl_reset_deconstructor: dev 0x%p\n", dev)); 

	rsp = malloc(sizeof(raid_ctrlResetRspMsg_t));
	bzero(rsp, sizeof(raid_ctrlResetRspMsg_t));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_ctrlResetRspMsg_t);
		return(NFX_OK);
	}

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = sizeof(raid_getCtrlParamRspMsg_t);
	
	return(NFX_OK);
}

static int32
raid_xyratex_ctrl_flush_constructor(raid_dev_t *		dev, 
				    raid_msgHdr_t *		msg_hdr, 
				    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				    int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_ctrl_reset_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_FLUCACHE_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_FLUSHDEVICEDATA;
	req_msg->cdb[6] = MDADEV_PHYSICALCONTROLLER;

	RAID_DBG(2, ("raid_xyratex_ctrl_flush_constructor: dev 0x%p: MDACIOCTL_FLUSHDEVICEDATA\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_ctrl_flush_deconstructor(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				      sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				      int32 *			sdm_relay_req_len,
				      raid_msgHdr_t **		rsp_msg, 
				      int32 *			rsp_msg_len)
{
	raid_flushCtrlCacheRspMsg_t *	rsp;

	RAID_DBG(2, ("raid_xyratex_ctrl_flush_deconstructor: dev 0x%p\n", dev));

	rsp = malloc(sizeof(raid_flushCtrlCacheRspMsg_t));
	bzero(rsp, sizeof(raid_flushCtrlCacheRspMsg_t));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_flushCtrlCacheRspMsg_t);
		return(NFX_OK);
	}

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = sizeof(raid_flushCtrlCacheRspMsg_t);
	
	return(NFX_OK);
}

static int32
raid_xyratex_get_ctrl_param_constructor(raid_dev_t *		dev, 
					raid_msgHdr_t *		msg_hdr, 
					sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_param_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_GETPARAM_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerParms_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERPARAMETER;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_param_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERPARAMETER\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_ctrl_param_deconstructor(raid_dev_t *			dev, 
					  sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					  sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					  int32 *			sdm_relay_req_len,
					  raid_msgHdr_t **		rsp_msg, 
					  int32 *			rsp_msg_len)
{
	FSI_ControllerParms_t *		cp;
	raid_getCtrlParamRspMsg_t *	rsp;
	raid_ctrlParam_t *		p;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_param_deconstructor: dev 0x%p\n", dev));

	rsp = malloc(sizeof(raid_getCtrlParamRspMsg_t));
	bzero(rsp, sizeof(raid_getCtrlParamRspMsg_t));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_getCtrlParamRspMsg_t);
		return(NFX_OK);
	}

	cp = (FSI_ControllerParms_t *)sdm_relay_rsp->data;
	p = &rsp->params;

	p->readAhead = (cp->Byte0 & FSI_CPARMS_B0_RDAHEN) ? 1 : 0;
	p->reasRestrict = (cp->Byte0 & FSI_CPARMS_B0_REASS1S) ? 1 : 0;
	p->trueVerify = (cp->Byte0 & FSI_CPARMS_B0_TRUVRFY) ? 1 : 0;
	p->writeVerify = (cp->Byte0 & FSI_CPARMS_B0_DWTVRFY) ? 1 : 0;
	p->bgInit = (cp->Byte0 & FSI_CPARMS_B0_BG_INIT) ? 1 : 0;
	p->superReadAhead = (cp->Byte1 & FSI_CPARMS_B1_SUPERREADAHEAD) ? 1 : 0;
	p->coalOptimal = (cp->Byte2 & FSI_CPARMS_B2_DEVICE_COMBING) ? 1 : 0;
	p->autoRebuild = (cp->Byte4 & FSI_CPARMS_B4_V_ARM) ? 1 : 0;
	p->operFault = (cp->Byte4 & FSI_CPARMS_B4_V_OFM) ? 1 : 0;
	p->rebuildRate = cp->RebuildRate;
	p->initRate = cp->BGIRate;
	p->consistencyCheckRate = cp->ConsistencyCheckRate;
	p->startupMode = cp->StartUpMode;
	p->startupNumDev = cp->SpinDevs;
	p->startupDelay = cp->SpinDelay1;
	p->scsiStartupDelay = cp->SpinDelay2;
	p->vendorTUR = (cp->Byte12 & FSI_CPARMS_B12_VUTURSNS) ? 1 : 0;
	p->disCCInvLun = (cp->Byte12 & FSI_CPARMS_B12_DCCFIL) ? 1 : 0;
	p->noPause = (cp->Byte12 & FSI_CPARMS_B12_NOPAUSE) ? 1 : 0;
	p->queueFullBusy = (cp->Byte12 & FSI_CPARMS_B12_DISQFULL) ? 1 : 0;
	p->disBusyFailback = (cp->Byte12 & FSI_CPARMS_B12_DISBUSY) ? 1 : 0;
	p->nodeNameRet = (cp->Byte13 & FSI_CPARMS_B13_FAILOVERNODENAME) ? 1 : 0;
	p->safteofups = (cp->Byte13 & FSI_CPARMS_B13_DISABLE_UPS) ? 1 : 0;
	p->topology = (cp->Byte13 & FSI_CPARMS_B13_FTOPO_MASK) >> FSI_CPARMS_B13_FTOPO_SHIFT;
	p->resetProp = (cp->ResetStyle & FSI_CPARMS_B14_RESET_PROPAGATE_RESET) ? 1 : 0;
	p->rs232PortType = cp->SerialControl;
	p->rs232BaudRate = 300 << cp->SerialBaud;
	p->smartLrgTrans = (cp->Byte19 & FSI_CPARMS_B19_SMARTLARGEHOSTXFERS) ? 1 : 0;
	switch (cp->Byte19 & FSI_CPARMS_B19_FRAME_SIZE_MASK) {
	case FSI_CPARMS_B19_FRAME_SIZE_2048: 
		p->frameSize = 2048; 
		break;
	case FSI_CPARMS_B19_FRAME_SIZE_512: 
		p->frameSize = 512;
		break;
	case FSI_CPARMS_B19_FRAME_SIZE_1024: 
		p->frameSize = 1024;
		break;
	default:
		p->frameSize = -1;
	}
	p->pciLatCtrl = (cp->Byte19 & FSI_CPARMS_B19_PCI_LATENCY_MASK) >> FSI_CPARMS_B19_PCI_LATENCY_SHIFT;
	p->autoFailback = (cp->Byte20 & FSI_CPARMS_B20_MS_AUTOREST) ? 1 : 0;
	p->forceSimplex = (cp->AAbits & FSI_CPARMS_B21_MS_AA_FSIM) ? 1 : 0;
	p->conserveCache = (cp->AAbits & FSI_CPARMS_B21_MS_AA_CCASH) ? 1 : 0;
	p->noRSTCOM = (cp->SimplexNoReset & FSI_CPARMS_B22_MS_FLG_SIMPLEX_NO_RSTCOM) ? 1 : 0;
	p->queueLimit = cp->QueueLimit;
	
	p->ctrl0Port0 = cp->HardLoopID[0][0] & 0x80 ? 1 : 0;
	p->ctrl0Port1 = cp->HardLoopID[0][1] & 0x80 ? 1 : 0;
	p->ctrl1Port0 = cp->HardLoopID[1][0] & 0x80 ? 1 : 0;
	p->ctrl1Port1 = cp->HardLoopID[1][1] & 0x80 ? 1 : 0;
	p->ctrl0Port0LoopId = cp->HardLoopID[0][0] & 0x7f;
	p->ctrl0Port1LoopId = cp->HardLoopID[0][1] & 0x7f;
	p->ctrl1Port0LoopId = cp->HardLoopID[1][0] & 0x7f;
	p->ctrl1Port1LoopId = cp->HardLoopID[1][1] & 0x7f;

	p->debugDump = cp->DebugDumpEnable;
	p->rofInterval = (cp->ROFControl & FSI_CPARMS_B66_ROF_REARM_INTERVAL_MASK) >> FSI_CPARMS_B66_ROF_REARM_INTERVAL_SHIFT;
	p->rofReboot = (cp->ROFControl & FSI_CPARMS_B66_ROF_REBOOT_COUNT_MASK) >> FSI_CPARMS_B66_ROF_REBOOT_COUNT_SHIFT;

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = sizeof(raid_getCtrlParamRspMsg_t);
	
	return(NFX_OK);
}

static int32
raid_xyratex_get_ctrl_state_constructor(raid_dev_t *		dev, 
					raid_msgHdr_t *		msg_hdr, 
					sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_state_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_GETSTATE_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = 20;
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETDUALCONTROLLERSTATUS;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_state_constructor: dev 0x%p: MDACIOCTL_GETDUALCONTROLLERSTATUS\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_ctrl_state_deconstructor(raid_dev_t *			dev, 
					  sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					  sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					  int32 *			sdm_relay_req_len,
					  raid_msgHdr_t **		rsp_msg, 
					  int32 *			rsp_msg_len)
{
	raid_getCtrlStateRspMsg_t *	rsp;
	uint32				master_slave_state;
	uint32				master_slave_add_info;
	uint32				partner_status;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_state_deconstructor: dev 0x%p\n", dev));

	rsp = malloc(sizeof(raid_getCtrlStateRspMsg_t));
	bzero(rsp, sizeof(raid_getCtrlStateRspMsg_t));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_getCtrlStateRspMsg_t);
		return(NFX_OK);
	}

	memcpy(&master_slave_state, &sdm_relay_rsp->data[4], 4);
	memcpy(&master_slave_add_info, &sdm_relay_rsp->data[8], 4);
	memcpy(&partner_status, &sdm_relay_rsp->data[12], 4);

	switch (master_slave_state) {
	case 0x00: strcpy(rsp->masterSlaveState, "Disabled or in Simplex mode");					break;
	case 0x10: strcpy(rsp->masterSlaveState, "Default Master is Master");						break;
	case 0x11: strcpy(rsp->masterSlaveState, "Default Master received first ping.");				break;
	case 0x12: strcpy(rsp->masterSlaveState, "Master in negotiation.");						break;
	case 0x13: strcpy(rsp->masterSlaveState, "Negotiation finished, waiting for a ping.");				break;
	case 0x14: strcpy(rsp->masterSlaveState, "Negotiation finished, received ping.");				break;
	case 0x15: strcpy(rsp->masterSlaveState, "Controller-Controller nexus established.");				break;
	case 0x16: strcpy(rsp->masterSlaveState, "Insufficient memory.");						break;
	case 0x17: strcpy(rsp->masterSlaveState, "Waiting for debounce ping - Negotiation with partner not complete."); break;
	case 0x20: strcpy(rsp->masterSlaveState, "Controller in slot 1 failed.");					break;
	case 0x21: strcpy(rsp->masterSlaveState, "Controller in slot 1 is ejected.");					break;
	case 0x22: strcpy(rsp->masterSlaveState, "Controller in slot 1 removal is detected.");				break;
	case 0x23: strcpy(rsp->masterSlaveState, "Controller in slot 1 is inserted.");					break;
	case 0x24: strcpy(rsp->masterSlaveState, "Command controller in slot 1 is inserted.");				break;
	case 0x25: strcpy(rsp->masterSlaveState, "Command controller in slot 1 to pause.");				break;
	case 0x26: strcpy(rsp->masterSlaveState, "Paused controller in slot 1  ready to resume.");			break;
	case 0x27: strcpy(rsp->masterSlaveState, "Paused controller in slot 1 failed.");				break;
	case 0x80: strcpy(rsp->masterSlaveState, "Controller in slot 0 failed.");					break;
	case 0x81: strcpy(rsp->masterSlaveState, "Controller in slot 0 is ejected.");					break;
	case 0x82: strcpy(rsp->masterSlaveState, "Controller in slot 0 removal detected.");				break;
	case 0x83: strcpy(rsp->masterSlaveState, "Controller in slot 0 controller is inserted.");			break;
	case 0x84: strcpy(rsp->masterSlaveState, "Command controller in slot 0 is inserted.");				break;
	case 0x85: strcpy(rsp->masterSlaveState, "Relinquish control of controller in slot 1");				break;

	default:  sprintf(rsp->masterSlaveState, "Undocumented result code: 0x%02x", master_slave_state);		break;
	} 

	switch (master_slave_add_info) {
	case 0x0000: strcpy(rsp->masterSlaveAdditionInfo, "No additional info available.");				break;
	case 0x0100: strcpy(rsp->masterSlaveAdditionInfo, "No RA, Insufficient Memory.");				break;
	case 0x0101: strcpy(rsp->masterSlaveAdditionInfo, "No Ping from Slave.");					break;
	case 0x0102: strcpy(rsp->masterSlaveAdditionInfo, "Slave Failed Diagnostics.");					break;
	case 0x0103: strcpy(rsp->masterSlaveAdditionInfo, "Negotiation with slave failed.");				break;
	case 0x0105: strcpy(rsp->masterSlaveAdditionInfo, "Configuration Sync Failed.");				break;
	case 0x0106: strcpy(rsp->masterSlaveAdditionInfo, "Firmware download to slave failed.");			break;
	case 0x0107: strcpy(rsp->masterSlaveAdditionInfo, "No first Ping from Slave.");					break;
	case 0x0108: strcpy(rsp->masterSlaveAdditionInfo, "Update Mode Pages Failed.");					break;
	case 0x0110: strcpy(rsp->masterSlaveAdditionInfo, "Write back synchronization failed on channel 0");		break;
	case 0x0111: strcpy(rsp->masterSlaveAdditionInfo, "Write back synchronization failed on channel 1");		break;
	case 0x0112: strcpy(rsp->masterSlaveAdditionInfo, "Write back synchronization failed on channel 2");		break;
	case 0x0113: strcpy(rsp->masterSlaveAdditionInfo, "Write back synchronization failed on channel 3");		break;
	case 0x0114: strcpy(rsp->masterSlaveAdditionInfo, "Write back synchronization failed on channel 4");		break;
	case 0x0115: strcpy(rsp->masterSlaveAdditionInfo, "Write back synchronization failed on channel 5");		break;
	case 0x0200: strcpy(rsp->masterSlaveAdditionInfo, "Master did not negotiate");					break;
	case 0x0201: strcpy(rsp->masterSlaveAdditionInfo, "No Ping from Master");					break;
	case 0x0202: strcpy(rsp->masterSlaveAdditionInfo, "Started as Preemptive Master");				break;
	case 0x0203: strcpy(rsp->masterSlaveAdditionInfo, "Host Channel Failed");					break;
	case 0x0204: strcpy(rsp->masterSlaveAdditionInfo, "Relinquish Failed");						break;
	default:    sprintf(rsp->masterSlaveAdditionInfo, "Undocumented result code: 0x%02x", master_slave_add_info);	break;
	}

	switch (partner_status) {
	case 0x0000: strcpy(rsp->parterState, "Controller is running in simplex mode.");				break;
	case 0x0100: strcpy(rsp->parterState, "Booting, no nexus yet.");						break;
	case 0x0101: strcpy(rsp->parterState, "Partner is replacement controller.");					break;
	case 0x0200: strcpy(rsp->parterState, "Partner is active.");							break;
	case 0x0201: strcpy(rsp->parterState, "Partner is survivor.");							break;
	case 0x0300: strcpy(rsp->parterState, "Ping time-out.");							break;
	case 0x0301: strcpy(rsp->parterState, "Negotiation - get \"chunk\" failure.");					break;
	case 0x0302: strcpy(rsp->parterState, "Negotiation - SCSI comms failed, cables are wrong or firmware versions are different."); break;
	case 0x0303: strcpy(rsp->parterState, "Negotiation - host ID mismatch.");					break;
	case 0x0304: strcpy(rsp->parterState, "Negotiation - SLIP/DIFFL/FBR mismatch.");				break;
	case 0x0305: strcpy(rsp->parterState, "Negotiation - disk channels available mismatch.");			break;
	case 0x0306: strcpy(rsp->parterState, "Negotiation - host channels available mismatch.");			break;
	case 0x0307: strcpy(rsp->parterState, "Negotiation - firmware version mismatch.");				break;
	case 0x0308: strcpy(rsp->parterState, "Negotiation - firmware header type mismatch.");				break;
	case 0x0309: strcpy(rsp->parterState, "Negotiation - memory size mismatch.");					break;
	case 0x030a: strcpy(rsp->parterState, "Negotiation - memory read of partner failed.");				break;
	case 0x030b: strcpy(rsp->parterState, "Negotiation - Cache memory size mismatch.");				break;
	case 0x030c: strcpy(rsp->parterState, "Kill Partner command received.");					break;
	case 0x030d: strcpy(rsp->parterState, "Partner failed during failback TID handover.");				break;
	case 0x030e: strcpy(rsp->parterState, "Partner did not enter nexus after negotiation completed.");              break;
	case 0x030f: strcpy(rsp->parterState, "Partner failed for unknown reason.");					break;
	case 0x0310: strcpy(rsp->parterState, "Write back synchronization failed on channel 0.");			break;
	case 0x0311: strcpy(rsp->parterState, "Write back synchronization failed on channel 1.");			break;
	case 0x0312: strcpy(rsp->parterState, "Write back synchronization failed on channel 2.");			break;
	case 0x0313: strcpy(rsp->parterState, "Write back synchronization failed on channel 3.");			break;
	case 0x0314: strcpy(rsp->parterState, "Write back synchronization failed on channel 4.");			break;
	case 0x0315: strcpy(rsp->parterState, "Write back synchronization failed on channel 5.");			break;
	case 0x0316: strcpy(rsp->parterState, "Negotiation - firmware build mismatch.");				break;
	case 0x0317: strcpy(rsp->parterState, "Negotiation - device channel cables are crossed.");			break;
	case 0x0318: strcpy(rsp->parterState, "Negotiation - attempting to get survivor state from partner.");          break;
	case 0x0319: strcpy(rsp->parterState, "Negotiation - could not negotiate autoboot.");				break;
	case 0x031F: strcpy(rsp->parterState, "Reserved - No room for channel expansion.");				break;
	case 0x0320: strcpy(rsp->parterState, "Hot pull of parner detected while nexus active.");			break;
	case 0x0321: strcpy(rsp->parterState, "Partner absent at boot.");						break;
	case 0x0322: strcpy(rsp->parterState, "BBU Power failed before failover finished.");				break;
	case 0x0323: strcpy(rsp->parterState, "BBU Power failed before relinquish finished.");				break;
	case 0x0324: strcpy(rsp->parterState, "Killed due to cable failure.");						break;
	case 0x0325: strcpy(rsp->parterState, "'Mylex' not found in partner's FW header.");				break;
	case 0x0326: strcpy(rsp->parterState, "Could not get DRVCMD for sending version to partner.");			break;
	case 0x0340: strcpy(rsp->parterState, "Controller-Controller locking had timeout error.");			break;
	case 0x0341: strcpy(rsp->parterState, "Controller-controller locking had unrecoverable SCSI error.");		break;
	case 0x0342: strcpy(rsp->parterState, "Kill partner in order to do migrate recovery.");				break;
	case 0x0343: strcpy(rsp->parterState, "General CTOC message failure.");						break;
	case 0x0344: strcpy(rsp->parterState, "Kill flush failed.");							break;
	case 0x0400: strcpy(rsp->parterState, "Partner is removed.");							break;
	case 0x0500: strcpy(rsp->parterState, "Partner is inserted.");							break;
	}

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = sizeof(raid_getCtrlStateRspMsg_t);
	
	return(NFX_OK);
}

typedef struct {
	int32				state;
} raid_xyratex_set_ctrl_param_sm_t;

static int32 
raid_xyratex_set_ctrl_param_aborter(raid_dev_t *		dev, 
				    sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_set_ctrl_param_sm_t *st = dev->private_context;	

	RAID_DBG(2, ("raid_xyratex_set_ctrl_param_aborter: dev 0x%p\n", dev));

	if (st) {
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32
raid_xyratex_set_ctrl_param_constructor(raid_dev_t *		dev, 
					raid_msgHdr_t *		msg_hdr, 
					sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_set_ctrl_param_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_SETPARAM_REQ);

	dev->private_context = malloc(sizeof(raid_xyratex_set_ctrl_param_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_set_ctrl_param_sm_t));

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerParms_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERPARAMETER;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_param_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERPARAMETER\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_set_ctrl_param_deconstructor(raid_dev_t *			dev, 
					  sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					  sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					  int32 *			sdm_relay_req_len,
					  raid_msgHdr_t **		rsp_msg, 
					  int32 *			rsp_msg_len)
{
	raid_xyratex_set_ctrl_param_sm_t * st = dev->private_context;
	raid_setCtrlParamRspMsg_t *	   rsp;
	sdm_relayCdbReqMsg_t *		   req_msg;
	uint32				   req_msg_len;
	FSI_ControllerParms_t *		   cp;
	raid_ctrlParam_t *		   p;

	RAID_DBG(2, ("raid_xyratex_set_ctrl_param_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_setCtrlParamRspMsg_t));
			bzero(rsp, sizeof(raid_setCtrlParamRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_setCtrlParamRspMsg_t);
			raid_xyratex_set_ctrl_param_aborter(dev, NULL);
			return(NFX_OK);
		}

		req_msg_len = sizeof(sdm_relayCdbReqMsg_t) + sizeof(FSI_ControllerParms_t);
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(req_msg_len);
		VERIFY(req_msg);
		bzero(req_msg, req_msg_len);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataLen = sizeof(FSI_ControllerParms_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_SETCONTROLLERPARAMETER;
		req_msg->cdb[7] = req_msg->dataLen >> 8;
		req_msg->cdb[8] = req_msg->dataLen & 0xFF;

		memcpy(req_msg->data, sdm_relay_rsp->data, sizeof(FSI_ControllerParms_t));

		cp = (FSI_ControllerParms_t *)req_msg->data;
		p = &((raid_setCtrlParamReqMsg_t *)dev->message_req_context->req_msg)->params;

		if (p->readAhead)
			cp->Byte0 |= FSI_CPARMS_B0_RDAHEN;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_RDAHEN;

		if (p->reasRestrict)
			cp->Byte0 |= FSI_CPARMS_B0_REASS1S;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_REASS1S;

		if (p->trueVerify)
			cp->Byte0 |= FSI_CPARMS_B0_TRUVRFY;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_TRUVRFY;			

		if (p->writeVerify)
			cp->Byte0 |= FSI_CPARMS_B0_DWTVRFY;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_DWTVRFY;

		if (p->bgInit)
			cp->Byte0 |= FSI_CPARMS_B0_BG_INIT;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_BG_INIT;

		if (p->superReadAhead)
			cp->Byte1 |= FSI_CPARMS_B1_SUPERREADAHEAD;
		else
			cp->Byte1 &= ~FSI_CPARMS_B1_SUPERREADAHEAD;

		if (p->coalOptimal)
			cp->Byte2 |= FSI_CPARMS_B2_DEVICE_COMBING;
		else 
			cp->Byte2 &= ~FSI_CPARMS_B2_DEVICE_COMBING;

		if (p->autoRebuild)
			cp->Byte4 |= FSI_CPARMS_B4_V_ARM;
		else
			cp->Byte4 &= ~FSI_CPARMS_B4_V_ARM;

		if (p->operFault)
			cp->Byte4 |= FSI_CPARMS_B4_V_OFM;
		else 
			cp->Byte4 &= ~FSI_CPARMS_B4_V_OFM;

		cp->RebuildRate = p->rebuildRate;
		cp->BGIRate = p->initRate;
		cp->ConsistencyCheckRate = p->consistencyCheckRate;
		cp->StartUpMode = p->startupMode;
		cp->SpinDevs = p->startupNumDev;
		cp->SpinDelay1 = p->startupDelay;
		cp->SpinDelay2 = p->scsiStartupDelay;

		if (p->vendorTUR) 
			cp->Byte12 |= FSI_CPARMS_B12_VUTURSNS;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_VUTURSNS;

		if (p->disCCInvLun)
			cp->Byte12 |= FSI_CPARMS_B12_DCCFIL;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_DCCFIL;

		if (p->noPause)
			cp->Byte12 |= FSI_CPARMS_B12_NOPAUSE;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_NOPAUSE;

		if (p->queueFullBusy)
			cp->Byte12 |= FSI_CPARMS_B12_DISQFULL;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_DISQFULL; 

		if (p->disBusyFailback)
			cp->Byte12 |= FSI_CPARMS_B12_DISBUSY;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_DISBUSY;

		if (p->nodeNameRet)
			cp->Byte13 |= FSI_CPARMS_B13_FAILOVERNODENAME;
		else
			cp->Byte13 &= ~FSI_CPARMS_B13_FAILOVERNODENAME;

		if (p->safteofups)
			cp->Byte13 |= FSI_CPARMS_B13_DISABLE_UPS;
		else
			cp->Byte13 &= ~FSI_CPARMS_B13_DISABLE_UPS;

		cp->Byte13 &= ~FSI_CPARMS_B13_FTOPO_MASK;
		cp->Byte13 |= (p->topology << FSI_CPARMS_B13_FTOPO_SHIFT) & FSI_CPARMS_B13_FTOPO_MASK;

		if (p->resetProp)
			cp->ResetStyle |= FSI_CPARMS_B14_RESET_PROPAGATE_RESET;
		else
			cp->ResetStyle &= ~FSI_CPARMS_B14_RESET_PROPAGATE_RESET;

		cp->SerialControl = p->rs232PortType;

		if (p->rs232BaudRate < 4800)
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_2400;
		else
		if (p->rs232BaudRate >= 4800 && p->rs232BaudRate < 9600)
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_4800;
		else
		if (p->rs232BaudRate >= 9600 && p->rs232BaudRate < 19200)
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_9600;
		else
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_19200;

		if (p->smartLrgTrans)
			cp->Byte19 |= FSI_CPARMS_B19_SMARTLARGEHOSTXFERS;
		else
			cp->Byte19 &= ~FSI_CPARMS_B19_SMARTLARGEHOSTXFERS;

		if (p->frameSize < 1024)
			cp->Byte19 |= FSI_CPARMS_B19_FRAME_SIZE_512;
		else
		if (p->frameSize >= 1024 && p->frameSize < 2048)
			cp->Byte19 |= FSI_CPARMS_B19_FRAME_SIZE_1024;
		else
			cp->Byte19 |= FSI_CPARMS_B19_FRAME_SIZE_2048;

		cp->Byte19 &= ~FSI_CPARMS_B19_PCI_LATENCY_MASK;
		cp->Byte19 |= (p->pciLatCtrl << FSI_CPARMS_B19_PCI_LATENCY_SHIFT) & FSI_CPARMS_B19_PCI_LATENCY_MASK;

		if (p->autoFailback)
			cp->Byte20 |= FSI_CPARMS_B20_MS_AUTOREST;
		else
			cp->Byte20 &= ~FSI_CPARMS_B20_MS_AUTOREST;

		if (p->forceSimplex)
			cp->AAbits |= FSI_CPARMS_B21_MS_AA_FSIM;
		else
			cp->AAbits &= ~FSI_CPARMS_B21_MS_AA_FSIM;

		if (p->conserveCache)
			cp->AAbits |= FSI_CPARMS_B21_MS_AA_CCASH;
		else
			cp->AAbits &= ~FSI_CPARMS_B21_MS_AA_CCASH;

		if (p->noRSTCOM)
			cp->SimplexNoReset |= FSI_CPARMS_B22_MS_FLG_SIMPLEX_NO_RSTCOM;
		else
			cp->SimplexNoReset &= ~FSI_CPARMS_B22_MS_FLG_SIMPLEX_NO_RSTCOM;

		cp->QueueLimit = p->queueLimit;
	
		/* ctrl0Port0 */
		/* ctrl0Port1 */
		/* ctrl1Port0 */
		/* ctrl1Port1 */

		cp->DebugDumpEnable = p->debugDump;

		cp->ROFControl &= ~FSI_CPARMS_B66_ROF_REARM_INTERVAL_MASK;
		cp->ROFControl |= (p->rofInterval << FSI_CPARMS_B66_ROF_REARM_INTERVAL_SHIFT) & FSI_CPARMS_B66_ROF_REARM_INTERVAL_MASK;
		cp->ROFControl &= ~FSI_CPARMS_B66_ROF_REBOOT_COUNT_MASK;
		cp->ROFControl |= (p->rofReboot << FSI_CPARMS_B66_ROF_REBOOT_COUNT_SHIFT) & FSI_CPARMS_B66_ROF_REBOOT_COUNT_MASK;
		
		RAID_DBG(2, ("raid_xyratex_set_ctrl_param_constructor: dev 0x%p: MDACIOCTL_SETCONTROLLERPARAMETER\n", dev));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = req_msg_len;
		st->state = 1;
		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_setCtrlParamRspMsg_t));
			bzero(rsp, sizeof(raid_setCtrlParamRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_setCtrlParamRspMsg_t);
			raid_xyratex_set_ctrl_param_aborter(dev, NULL);
			return(NFX_OK);
		}

		rsp = malloc(sizeof(raid_setCtrlParamRspMsg_t));
		bzero(rsp, sizeof(raid_setCtrlParamRspMsg_t));
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_setCtrlParamRspMsg_t);
		raid_xyratex_set_ctrl_param_aborter(dev, NULL);
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

static int32
raid_xyratex_get_ctrl_info_constructor(raid_dev_t *		dev, 
				       raid_msgHdr_t *		msg_hdr, 
				       sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				       int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_info_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_INFO_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_info_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_ctrl_info_deconstructor(raid_dev_t *		dev, 
					 sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					 sdm_relayCdbReqMsg_t **sdm_relay_req, 
					 int32 *	        sdm_relay_req_len,
					 raid_msgHdr_t **	rsp_msg, 
					 int32 *		rsp_msg_len)
{
	FSI_ControllerInfo_t *	ci;
	raid_ctrlInfoRspMsg_t *	rsp;

	RAID_DBG(2, ("raid_xyratex_get_ctrl_info_deconstructor: dev 0x%p\n", dev));

	rsp = malloc(sizeof(raid_ctrlInfoRspMsg_t));
	bzero(rsp, sizeof(raid_ctrlInfoRspMsg_t));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_ctrlInfoRspMsg_t);
		return(NFX_OK);
	}

	ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;
	sprintf(rsp->ctrlType, "%s", FSI_HardwareIDToString(ci->ControllerType));
	strncpy(rsp->ctrlName, ci->ControllerName, 16);
	strncpy(rsp->serialNumber, ci->SerialNumber, 16);
	strncpy(rsp->vendorName, ci->VendorName, 16);
	sprintf(rsp->hardwareVer, "%d", ci->HardwareRev);
	sprintf(rsp->firmwareVer, "%d.%03d", ci->MajorFirmwareRev, ci->MinorFirmwareRev);
	rsp->cacheInMB = ci->CacheSizeMB[1] * 256 + ci->CacheSizeMB[0];
	rsp->memoryInMB = ci->DRAMSizeMB[1] * 256 + ci->DRAMSizeMB[0];
	rsp->bbuPresent = ci->OtherPCOInfo & BBU_PRESENT_MASK;
	rsp->numDriveChannels = ci->nPhysicalChannels;
	rsp->maxDrivesPerChannel = ci->MaxTargetsPerCh[0];
	memcpy(&rsp->ctrlWwn, ci->Slot0WWN, 8);
	SWAP8(&rsp->ctrlWwn);
	memcpy(&rsp->dualCtrlWwn, ci->Slot1WWN, 8);
	SWAP8(&rsp->dualCtrlWwn);
	rsp->numLogicalDevices = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];
	rsp->numPhysicalDevices = ci->nPDPresent[1] * 256 + ci->nPDPresent[0];

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = sizeof(raid_ctrlInfoRspMsg_t);
	
	return(NFX_OK);
}

typedef struct {
	int32				state;
	int32				bus_number;
	int32				bus_count;
	int32				target_number;
	int32				target_count;
	raid_phyDevListRspMsg_t *	msg_rsp; 
	int32				msg_rsp_len;
	int32				dev_index;
	int32				dev_count;
} raid_xyratex_get_physdev_list_sm_t;

static int32 
raid_xyratex_get_physdev_list_aborter(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_get_physdev_list_sm_t * st = dev->private_context;

	RAID_DBG(2, ("raid_xyratex_get_physdev_list_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->msg_rsp)
			free(st->msg_rsp);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32
raid_xyratex_get_physdev_list_constructor(raid_dev_t *			dev, 
					  raid_msgHdr_t *		msg_hdr, 
					  sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					  int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_get_physdev_list_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_PHYDEV_LIST_REQ);

	dev->private_context = malloc(sizeof(raid_xyratex_get_physdev_list_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_get_physdev_list_sm_t));

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_physdev_list_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n", dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_physdev_list_deconstructor(raid_dev_t *		dev, 
					    sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					    int32 *			sdm_relay_req_len,
					    raid_msgHdr_t **		rsp_msg, 
					    int32 *			rsp_msg_len)
{

	raid_xyratex_get_physdev_list_sm_t * st = dev->private_context;
	FSI_ControllerInfo_t *		     ci;
	FSI_GetPhysDevInfoValid_t *	     di;
	sdm_relayCdbReqMsg_t *		     req_msg;

	RAID_DBG(2, ("raid_xyratex_get_physdev_list_deconstructor: dev 0x%p, "
		     "state %d, bus_number %d, target_number %d, dev_index %d\n", 
		     dev, st->state, st->bus_number, st->target_number, st->dev_index));

	switch(st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			*rsp_msg_len = sizeof(raid_phyDevListRspMsg_t);
			*rsp_msg = malloc(*rsp_msg_len);
			bzero(*rsp_msg, *rsp_msg_len);
			((raid_phyDevListRspMsg_t *)(*rsp_msg))->rCode = RAID_RCODE_IO_ERROR;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;
		st->bus_count = ci->nPhysicalChannels;
		st->target_count = ci->MaxTargetsPerCh[0];
		st->dev_count = ci->nPDPresent[1] * 256 + ci->nPDPresent[0];
		st->msg_rsp_len = sizeof(raid_phyDevListRspMsg_t) + st->dev_count * sizeof(raid_phydev_id_t);
		st->msg_rsp = (raid_phyDevListRspMsg_t *)malloc(st->msg_rsp_len);
		bzero(st->msg_rsp, st->msg_rsp_len);
		st->msg_rsp->numEntries = st->dev_count;

		if (st->dev_count == 0) {
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
		req_msg->cdb[3] = st->bus_number;
		req_msg->cdb[4] = st->target_number;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_get_physdev_list_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
			     dev, st->bus_number, st->target_number));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			st->msg_rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		di = (FSI_GetPhysDevInfoValid_t *)sdm_relay_rsp->data;

		st->bus_number = di->ch;
		st->target_number = di->id;

		/*
		 * Byte 8 of the Inquiry data is the Vendor ID. If
		 * this was not set, device is not there or is invalid
		 * Byte 0 of the Inquiry data is the Device type. c7
		 * is a Mylex controller, so don't include this 
		 */
		if (di->InquiryData[0] != 0xc7 && di->InquiryData[8] != 0x00) {
			st->msg_rsp->idList[st->dev_index].channelNumber = di->ch;
			st->msg_rsp->idList[st->dev_index].targetId = di->id;
			++st->dev_index;
		}
		
		if (st->dev_index < st->dev_count) {

			++st->target_number;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
			req_msg->cdb[3] = st->bus_number;
			req_msg->cdb[4] = st->target_number;
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

			RAID_DBG(2, ("raid_xyratex_get_physdev_list_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
				     dev, st->bus_number, st->target_number));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
		}
		else {
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
		}
		break;

	default:
		VERIFY(0);
	}

	return(NFX_OK);
}

typedef struct {
	int32				state;
	raid_phyDevInfoRspMsg_t *	msg_rsp;
	uint32				msg_rsp_len;
} raid_xyratex_get_physdev_info_sm_t;

static int32 
raid_xyratex_get_physdev_info_aborter(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_get_physdev_info_sm_t * st = dev->private_context;

	RAID_DBG(2, ("raid_xyratex_get_physdev_info_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->msg_rsp)
			free(st->msg_rsp);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_get_physdev_info_constructor(raid_dev_t *			dev, 
					  raid_msgHdr_t *		msg_hdr, 
					  sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					  int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	 req_msg;

	RAID_DBG(2, ("raid_xyratex_get_physdev_info_constructor: dev 0x%p: channelNumber/targetId %d/%d\n", 
		     dev, 
		     ((raid_phyDevInfoReqMsg_t *)msg_hdr)->id.channelNumber, 
		     ((raid_phyDevInfoReqMsg_t *)msg_hdr)->id.targetId));

	VERIFY(msg_hdr->mid == RAID_MID_PHYDEV_INFO_REQ);

	dev->private_context = malloc(sizeof(raid_xyratex_get_physdev_info_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_get_physdev_info_sm_t));

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);	

	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
	req_msg->cdb[3] = ((raid_phyDevInfoReqMsg_t *)msg_hdr)->id.channelNumber;
	req_msg->cdb[4] = ((raid_phyDevInfoReqMsg_t *)msg_hdr)->id.targetId;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_physdev_info_constructor: dev 0x%p: "
		     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
		     dev, 
		     ((raid_phyDevInfoReqMsg_t *)msg_hdr)->id.channelNumber,
		     ((raid_phyDevInfoReqMsg_t *)msg_hdr)->id.targetId));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_physdev_info_deconstructor(raid_dev_t *		dev, 
					    sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					    int32 *			sdm_relay_req_len,
					    raid_msgHdr_t **		rsp_msg, 
					    int32 *			rsp_msg_len)
{
	FSI_GetPhysDevInfoValid_t *		di;
	FSI_freeSpace_t *			fs;
	raid_phyDevInfoReqMsg_t *		info_req_msg;
	raid_phyDevInfoRspMsg_t *		rsp;
	raid_xyratex_get_physdev_info_sm_t *	st = dev->private_context;
	sdm_relayCdbReqMsg_t *			req_msg;
	uint32					i, size;
	int						free_size_found;

	RAID_DBG(2, ("raid_xyratex_get_physdev_info_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			*rsp_msg_len = sizeof(raid_phyDevInfoRspMsg_t);
			*rsp_msg = malloc(*rsp_msg_len);
			bzero(*rsp_msg, *rsp_msg_len);
			((raid_phyDevInfoRspMsg_t *)(*rsp_msg))->rCode = RAID_RCODE_IO_ERROR;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		info_req_msg = (raid_phyDevInfoReqMsg_t *)dev->message_req_context->req_msg;
		di = (FSI_GetPhysDevInfoValid_t *)sdm_relay_rsp->data;

		st->msg_rsp_len = sizeof(raid_phyDevInfoRspMsg_t);
		st->msg_rsp = rsp = malloc(st->msg_rsp_len);
		bzero(st->msg_rsp, st->msg_rsp_len);

		/* Did we get the requested physical device ? */
		if (info_req_msg->id.channelNumber != di->ch ||
		    info_req_msg->id.targetId != di->id) {
			rsp->rCode = RAID_RCODE_INVALID_PHYSICAL_DEVICE;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		rsp->id.channelNumber = di->ch;
		rsp->id.targetId = di->id;
		strncpy(rsp->vendorName, &di->InquiryData[8], 8);
		strncpy(rsp->modelName, &di->InquiryData[16], 16);
		strncpy(rsp->serialNumber, di->SerialNumber, 16);
		strncpy(rsp->microcodeVer, &di->InquiryData[32], 4);
		memcpy(&rsp->wwn, di->MacAddress, 8);
		SWAP8(&rsp->wwn);

		/* enclosure/row/column */

		memcpy(&rsp->physicalSizeInMB, &di->OrgDevSize, 4);
		rsp->physicalSizeInMB /= RAID_SECTORS_PER_MB;
		memcpy(&rsp->usableSizeInMB, &di->UseableDevSize, 4);
		rsp->usableSizeInMB /= RAID_SECTORS_PER_MB;

		rsp->sectorSizeInByte = di->BlockSizeInBytes[1] * 256 + di->BlockSizeInBytes[0];
		rsp->deviceState = di->PDState;

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = 8;

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_XLATEPHYSDEVTORAIDDEV;
		req_msg->cdb[3] = info_req_msg->id.channelNumber;
		req_msg->cdb[4] = info_req_msg->id.targetId;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_get_physdev_info_deconstructor: dev 0x%p: "
			     "MDACIOCTL_XLATEPHYSDEVTORAIDDEV bus %d, target %d\n", 
			     dev, 
			     info_req_msg->id.channelNumber,
			     info_req_msg->id.targetId));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;

	case 1:		
		rsp = st->msg_rsp;
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp->rdn = 0xFFFF;
			if (rsp->deviceState == RAID_PHYDEVSTATE_UNCONFIGURED)
				rsp->freeSizeInMB = rsp->usableSizeInMB;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}
		
		rsp->rdn = sdm_relay_rsp->data[1] * 256 + sdm_relay_rsp->data[0];

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_freeSpace_t);

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETFREESPACELIST;
		req_msg->cdb[3] = (rsp->rdn & 0xFF00) >> 8;
		req_msg->cdb[4] = (rsp->rdn & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_get_physdev_info_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETFREESPACELIST rdn 0x%x\n", 
			     dev, rsp->rdn));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			
		st->state = 2;

		break;

	case 2: 
		rsp = st->msg_rsp;
		free_size_found = 0;
		fs = (FSI_freeSpace_t *)sdm_relay_rsp->data;
		for (i = 0; i < fs->NumEntries; ++i) {
			memcpy(&size, fs->DeviceSpace[i].Size, 4);
			if (fs->DeviceSpace[i].RDN == rsp->rdn) {
				rsp->freeSizeInMB = size;
				rsp->freeSizeInMB /= RAID_SECTORS_PER_MB;
				free_size_found = 1;
			}
		}
		if (rsp->freeSizeInMB == 0 && rsp->deviceState == RAID_PHYDEVSTATE_UNCONFIGURED)
			rsp->freeSizeInMB = rsp->usableSizeInMB;
		else if(!free_size_found) {
			rsp->freeSizeInMB = rsp->usableSizeInMB;
		}
		
		*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
		*rsp_msg_len = st->msg_rsp_len;
		free(st);
		dev->private_context = NULL;

		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

typedef struct {
	int32				state;
	raid_phyDevStatsRspMsg_t *	msg_rsp;
	uint32				msg_rsp_len;
} raid_xyratex_get_physdev_stats_sm_t;

static int32 
raid_xyratex_get_physdev_stats_aborter(raid_dev_t *		dev, 
				       sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_get_physdev_stats_sm_t *st = dev->private_context;	

	RAID_DBG(2, ("raid_xyratex_get_physdev_stats_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->msg_rsp)
			free(st->msg_rsp);
		free(st);
		dev->private_context = NULL;
	}
	return(NFX_OK);
}

static int32 
raid_xyratex_get_physdev_stats_constructor(raid_dev_t *			dev, 
					   raid_msgHdr_t *		msg_hdr, 
					   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					   int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	 req_msg;

	RAID_DBG(2, ("raid_xyratex_get_physdev_stats_constructor: dev 0x%p: channelNumber/targetId %d/%d\n", 
		     dev, 
		     ((raid_phyDevStatsReqMsg_t *)msg_hdr)->id.channelNumber, 
		     ((raid_phyDevStatsReqMsg_t *)msg_hdr)->id.targetId));

	VERIFY(msg_hdr->mid == RAID_MID_PHYDEV_STATS_REQ);

	dev->private_context = malloc(sizeof(raid_xyratex_get_physdev_stats_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_get_physdev_stats_sm_t));

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);	

	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
	req_msg->cdb[3] = ((raid_phyDevStatsReqMsg_t *)msg_hdr)->id.channelNumber;
	req_msg->cdb[4] = ((raid_phyDevStatsReqMsg_t *)msg_hdr)->id.targetId;
	req_msg->cdb[5] = ((raid_phyDevStatsReqMsg_t *)msg_hdr)->reset_on_read ? 1 : 0;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_physdev_stats_constructor: dev 0x%p: "
		     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
		     dev, 
		     ((raid_phyDevStatsReqMsg_t *)msg_hdr)->id.channelNumber,
		     ((raid_phyDevStatsReqMsg_t *)msg_hdr)->id.targetId));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_physdev_stats_deconstructor(raid_dev_t *		dev, 
					     sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					     int32 *			sdm_relay_req_len,
					     raid_msgHdr_t **		rsp_msg, 
					     int32 *			rsp_msg_len)
{
	FSI_GetPhysDevInfoValid_t *		di;
	FSI_GetPhysDevStats_t *			ds;
	raid_phyDevStatsReqMsg_t *		stats_req_msg;
	raid_phyDevStatsRspMsg_t *		rsp;
	int32					rsp_len;
	raid_xyratex_get_physdev_stats_sm_t *	st = dev->private_context;
	sdm_relayCdbReqMsg_t *			req_msg;

	RAID_DBG(2, ("raid_xyratex_get_physdev_info_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp_len = sizeof(raid_phyDevStatsRspMsg_t);
			rsp = malloc(rsp_len);
			bzero(rsp, rsp_len);
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_get_physdev_stats_aborter(dev, NULL);
			*rsp_msg_len = rsp_len;
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}

		stats_req_msg = (raid_phyDevStatsReqMsg_t *)dev->message_req_context->req_msg;
		di = (FSI_GetPhysDevInfoValid_t *)sdm_relay_rsp->data;

		st->msg_rsp_len = sizeof(raid_phyDevStatsRspMsg_t);
		st->msg_rsp = rsp = malloc(st->msg_rsp_len);
		bzero(st->msg_rsp, st->msg_rsp_len);

		/* Did we get the requested physical device ? */
		if (stats_req_msg->id.channelNumber != di->ch ||
		    stats_req_msg->id.targetId != di->id) {
			rsp->rCode = RAID_RCODE_INVALID_PHYSICAL_DEVICE;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			st->msg_rsp = NULL;
			raid_xyratex_get_physdev_stats_aborter(dev, NULL);
			return(NFX_OK);
		}

		rsp->id.channelNumber = di->ch;
		rsp->id.targetId = di->id;

		rsp->numSoftErrors = di->SoftErrors;
		rsp->numHardErrors = di->HardErrors;
		rsp->numParityErrors = di->ParityErrors;
		rsp->numMiscErrors = di->MiscErrors;


		/* if device is unconfigured, then return here */
		if (di->PDState == RAID_PHYDEVSTATE_UNCONFIGURED) {
			rsp->rCode = RAID_RCODE_PHYSDEV_UNCONFIGURED;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			st->msg_rsp = NULL;
			raid_xyratex_get_physdev_stats_aborter(dev, NULL);
			return(NFX_OK);
		}
		
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetPhysDevStats_t);

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVSTATISTICS;
		req_msg->cdb[3] = stats_req_msg->id.channelNumber;
		req_msg->cdb[4] = stats_req_msg->id.targetId;
		req_msg->cdb[6] = stats_req_msg->reset_on_read ? 1 : 0;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_get_physdev_stats_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETPHYSDEVSTATISTICS bus %d, target %d\n", 
			     dev, 
			     stats_req_msg->id.channelNumber,
			     stats_req_msg->id.targetId));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;

	case 1:		
		rsp = st->msg_rsp;
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			st->msg_rsp = NULL;
			raid_xyratex_get_physdev_stats_aborter(dev, NULL);
			return(NFX_OK);
		}

		stats_req_msg = (raid_phyDevStatsReqMsg_t *)dev->message_req_context->req_msg;
		ds = (FSI_GetPhysDevStats_t *)sdm_relay_rsp->data;

		memcpy(&rsp->numMsecSinceStart, ds->MilliSecondsSinceStart, 4);
		memcpy(&rsp->numReads, ds->NumberOfReads, 4);
		memcpy(&rsp->numWrites, ds->NumberOfWrites, 4);
		memcpy(&rsp->dataReadsInKB, ds->BytesRead, 4);
		memcpy(&rsp->dataWritesInKB, ds->BytesWritten, 4);
		rsp->dataReadsInKB /= 1024;
		rsp->dataWritesInKB /= 1024;

		*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
		*rsp_msg_len = st->msg_rsp_len;
		st->msg_rsp = NULL;
		raid_xyratex_get_physdev_stats_aborter(dev, NULL);		
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

typedef struct {
	int32			state;
	FILE *			open_f;
	char *			c_parms_buf;
	raid_ctrlParam_t *	c_parms;
	char *			config_buf;
	char *			config;
	int			config_len;
} raid_xyratex_ctrl_config_sm_t;

static int32 
raid_xyratex_ctrl_config_aborter(raid_dev_t *		dev, 
				 sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_ctrl_config_sm_t *st = dev->private_context;	

	RAID_DBG(2, ("raid_xyratex_ctrl_config_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->open_f)
			fclose(st->open_f);
		if (st->c_parms_buf)
			free(st->c_parms_buf);
		if(st->config_buf)
			free(st->config_buf);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_ctrl_config_constructor(raid_dev_t *		dev, 
				     raid_msgHdr_t *		msg_hdr, 
				     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				     int32 *			sdm_relay_req_len)
{
	raid_ctrlConfigReqMsg_t *	config_req_msg = (raid_ctrlConfigReqMsg_t *)msg_hdr;
	raid_xyratex_ctrl_config_sm_t * st;
	sdm_relayCdbReqMsg_t *		req_msg;
	int				rc;
	
	RAID_DBG(2, ("raid_xyratex_ctrl_config_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_CFGDATA_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_ctrl_config_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_ctrl_config_sm_t));

	if (config_req_msg->type == RAID_CONFIG_SAVE) {
		st->open_f = fopen(config_req_msg->fileName, "w");
		VERIFY(st->open_f);
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = 65535;
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETGROUPCONFINFO;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_ctrl_config_constructor: dev 0x%p: MDACIOCTL_GETGROUPCONFINFO\n",
			     dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 10;
	}
	else {
		st->open_f = fopen(config_req_msg->fileName, "r");
		VERIFY(st->open_f);
		st->c_parms_buf = malloc(sizeof(sdm_relayCdbReqMsg_t) + sizeof(raid_ctrlParam_t));
		bzero(st->c_parms_buf, sizeof(sdm_relayCdbReqMsg_t) + sizeof(raid_ctrlParam_t)); 
		st->c_parms = (raid_ctrlParam_t *)(((sdm_relayCdbReqMsg_t *)st->c_parms_buf)->data);
		st->config_len = 65536;
		st->config_buf = malloc(st->config_len);
		bzero(st->config_buf, st->config_len);
		st->config = (char *)(((sdm_relayCdbReqMsg_t *)st->config_buf)->data);
		st->config_len -= sizeof(sdm_relayCdbReqMsg_t);

		rc = raid_xyratex_parse_config_file(st->open_f, (raid_ctrlParam_t *)st->c_parms, st->config, &st->config_len);	
		if (rc != NFX_OK)
			return(RAID_RCODE_PARSE_ERROR);
		
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_CLEARCONF;
		req_msg->cdb[5] = 0x01;

		RAID_DBG(2, ("raid_xyratex_ctrl_config_constructor: dev 0x%p: MDACIOCTL_CLEARCONF\n",
			     dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 0;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_ctrl_config_deconstructor(raid_dev_t *			dev, 
				       sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				       sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				       int32 *			sdm_relay_req_len,
				       raid_msgHdr_t **		rsp_msg, 
				       int32 *			rsp_msg_len)
{
	raid_xyratex_ctrl_config_sm_t * st = dev->private_context;
	sdm_relayCdbReqMsg_t *		req_msg;
	uint32				req_msg_len;
	raid_ctrlConfigRspMsg_t *	rsp;
	FSI_ControllerParms_t *		cp;

	RAID_DBG(2, ("raid_xyratex_ctrl_config_deconstructor: dev 0x%p\n", dev));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
			bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_ctrl_config_aborter(dev, NULL);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)st->config_buf;
		req_msg_len = sizeof(sdm_relayCdbReqMsg_t) + st->config_len;
		st->config_buf = NULL;	/* So aborter doesn't free it */
		
		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataLen = st->config_len;

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_CREATENEWCONF;
		req_msg->cdb[7] = req_msg->dataLen >> 8;
		req_msg->cdb[8] = req_msg->dataLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_ctrl_config_deconstructor: dev 0x%p: MDACIOCTL_CREATENEWCONF\n", 
			     dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = req_msg_len;
			
		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
			bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_ctrl_config_aborter(dev, NULL);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_ControllerParms_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERPARAMETER;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_ctrl_config_deconstructor: dev 0x%p: MDACIOCTL_GETCONTROLLERPARAMETER\n", dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 2;
		break;

	case 2:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
			bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_ctrl_config_aborter(dev, NULL);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
			return(NFX_OK);
		}

		req_msg_len = sizeof(sdm_relayCdbReqMsg_t) + sizeof(FSI_ControllerParms_t);
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(req_msg_len);
		VERIFY(req_msg);
		bzero(req_msg, req_msg_len);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataLen = sizeof(FSI_ControllerParms_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_SETCONTROLLERPARAMETER;
		req_msg->cdb[7] = req_msg->dataLen >> 8;
		req_msg->cdb[8] = req_msg->dataLen & 0xFF;

		memcpy(req_msg->data, sdm_relay_rsp->data, sizeof(FSI_ControllerParms_t));

		cp = (FSI_ControllerParms_t *)req_msg->data;

		if (st->c_parms->readAhead)
			cp->Byte0 |= FSI_CPARMS_B0_RDAHEN;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_RDAHEN;

		if (st->c_parms->reasRestrict)
			cp->Byte0 |= FSI_CPARMS_B0_REASS1S;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_REASS1S;

		if (st->c_parms->trueVerify)
			cp->Byte0 |= FSI_CPARMS_B0_TRUVRFY;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_TRUVRFY;			

		if (st->c_parms->writeVerify)
			cp->Byte0 |= FSI_CPARMS_B0_DWTVRFY;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_DWTVRFY;

		if (st->c_parms->bgInit)
			cp->Byte0 |= FSI_CPARMS_B0_BG_INIT;
		else
			cp->Byte0 &= ~FSI_CPARMS_B0_BG_INIT;

		if (st->c_parms->superReadAhead)
			cp->Byte1 |= FSI_CPARMS_B1_SUPERREADAHEAD;
		else
			cp->Byte1 &= ~FSI_CPARMS_B1_SUPERREADAHEAD;

		if (st->c_parms->coalOptimal)
			cp->Byte2 |= FSI_CPARMS_B2_DEVICE_COMBING;
		else 
			cp->Byte2 &= ~FSI_CPARMS_B2_DEVICE_COMBING;

		if (st->c_parms->autoRebuild)
			cp->Byte4 |= FSI_CPARMS_B4_V_ARM;
		else
			cp->Byte4 &= ~FSI_CPARMS_B4_V_ARM;

		if (st->c_parms->operFault)
			cp->Byte4 |= FSI_CPARMS_B4_V_OFM;
		else 
			cp->Byte4 &= ~FSI_CPARMS_B4_V_OFM;

		cp->RebuildRate = st->c_parms->rebuildRate;
		cp->BGIRate = st->c_parms->initRate;
		cp->ConsistencyCheckRate = st->c_parms->consistencyCheckRate;
		cp->StartUpMode = st->c_parms->startupMode;
		cp->SpinDevs = st->c_parms->startupNumDev;
		cp->SpinDelay1 = st->c_parms->startupDelay;
		cp->SpinDelay2 = st->c_parms->scsiStartupDelay;

		if (st->c_parms->vendorTUR) 
			cp->Byte12 |= FSI_CPARMS_B12_VUTURSNS;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_VUTURSNS;

		if (st->c_parms->disCCInvLun)
			cp->Byte12 |= FSI_CPARMS_B12_DCCFIL;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_DCCFIL;

		if (st->c_parms->noPause)
			cp->Byte12 |= FSI_CPARMS_B12_NOPAUSE;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_NOPAUSE;

		if (st->c_parms->queueFullBusy)
			cp->Byte12 |= FSI_CPARMS_B12_DISQFULL;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_DISQFULL; 

		if (st->c_parms->disBusyFailback)
			cp->Byte12 |= FSI_CPARMS_B12_DISBUSY;
		else
			cp->Byte12 &= ~FSI_CPARMS_B12_DISBUSY;

		if (st->c_parms->nodeNameRet)
			cp->Byte13 |= FSI_CPARMS_B13_FAILOVERNODENAME;
		else
			cp->Byte13 &= ~FSI_CPARMS_B13_FAILOVERNODENAME;

		if (st->c_parms->safteofups)
			cp->Byte13 |= FSI_CPARMS_B13_DISABLE_UPS;
		else
			cp->Byte13 &= ~FSI_CPARMS_B13_DISABLE_UPS;

		cp->Byte13 &= ~FSI_CPARMS_B13_FTOPO_MASK;
		cp->Byte13 |= (st->c_parms->topology << FSI_CPARMS_B13_FTOPO_SHIFT) & FSI_CPARMS_B13_FTOPO_MASK;

		if (st->c_parms->resetProp)
			cp->ResetStyle |= FSI_CPARMS_B14_RESET_PROPAGATE_RESET;
		else
			cp->ResetStyle &= ~FSI_CPARMS_B14_RESET_PROPAGATE_RESET;

		cp->SerialControl = st->c_parms->rs232PortType;

		if (st->c_parms->rs232BaudRate < 4800)
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_2400;
		else
		if (st->c_parms->rs232BaudRate >= 4800 && st->c_parms->rs232BaudRate < 9600)
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_4800;
		else
		if (st->c_parms->rs232BaudRate >= 9600 && st->c_parms->rs232BaudRate < 19200)
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_9600;
		else
			cp->SerialBaud = FSI_CPARMS_B16_SERIAL_BAUD_19200;

		if (st->c_parms->smartLrgTrans)
			cp->Byte19 |= FSI_CPARMS_B19_SMARTLARGEHOSTXFERS;
		else
			cp->Byte19 &= ~FSI_CPARMS_B19_SMARTLARGEHOSTXFERS;

		if (st->c_parms->frameSize < 1024)
			cp->Byte19 |= FSI_CPARMS_B19_FRAME_SIZE_512;
		else
		if (st->c_parms->frameSize >= 1024 && st->c_parms->frameSize < 2048)
			cp->Byte19 |= FSI_CPARMS_B19_FRAME_SIZE_1024;
		else
			cp->Byte19 |= FSI_CPARMS_B19_FRAME_SIZE_2048;

		cp->Byte19 &= ~FSI_CPARMS_B19_PCI_LATENCY_MASK;
		cp->Byte19 |= (st->c_parms->pciLatCtrl << FSI_CPARMS_B19_PCI_LATENCY_SHIFT) & FSI_CPARMS_B19_PCI_LATENCY_MASK;

		if (st->c_parms->autoFailback)
			cp->Byte20 |= FSI_CPARMS_B20_MS_AUTOREST;
		else
			cp->Byte20 &= ~FSI_CPARMS_B20_MS_AUTOREST;

		if (st->c_parms->forceSimplex)
			cp->AAbits |= FSI_CPARMS_B21_MS_AA_FSIM;
		else
			cp->AAbits &= ~FSI_CPARMS_B21_MS_AA_FSIM;

		if (st->c_parms->conserveCache)
			cp->AAbits |= FSI_CPARMS_B21_MS_AA_CCASH;
		else
			cp->AAbits &= FSI_CPARMS_B21_MS_AA_CCASH;

		if (st->c_parms->noRSTCOM)
			cp->SimplexNoReset |= FSI_CPARMS_B22_MS_FLG_SIMPLEX_NO_RSTCOM;
		else
			cp->SimplexNoReset &= ~FSI_CPARMS_B22_MS_FLG_SIMPLEX_NO_RSTCOM;

		cp->QueueLimit = st->c_parms->queueLimit;
	
		/* ctrl0Port0 */
		/* ctrl0Port1 */
		/* ctrl1Port0 */
		/* ctrl1Port1 */

		cp->DebugDumpEnable = st->c_parms->debugDump;

		cp->ROFControl &= ~FSI_CPARMS_B66_ROF_REARM_INTERVAL_MASK;
		cp->ROFControl |= 
			(st->c_parms->rofInterval << FSI_CPARMS_B66_ROF_REARM_INTERVAL_SHIFT) & FSI_CPARMS_B66_ROF_REARM_INTERVAL_MASK;
		cp->ROFControl &= ~FSI_CPARMS_B66_ROF_REBOOT_COUNT_MASK;
		cp->ROFControl |= 
			(st->c_parms->rofReboot << FSI_CPARMS_B66_ROF_REBOOT_COUNT_SHIFT) & FSI_CPARMS_B66_ROF_REBOOT_COUNT_MASK;

		RAID_DBG(2, ("raid_xyratex_ctrl_config_deconstructor: dev 0x%p: MDACIOCTL_SETCONTROLLERPARAMETER\n", dev));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = req_msg_len;
		st->state = 3;
		break;

	case 3:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
			bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_ctrl_config_aborter(dev, NULL);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
			return(NFX_OK);
		}
		rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
		VERIFY(rsp);
		bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
		raid_xyratex_ctrl_config_aborter(dev, NULL);		
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
		break;

	case 10:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
			bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_ctrl_config_aborter(dev, NULL);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
			return(NFX_OK);
		}

		raid_xyratex_emit_lun_config(st->open_f, sdm_relay_rsp->data, sdm_relay_rsp->dataLen);

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_ControllerParms_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERPARAMETER;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_ctrl_config_deconstructor: dev 0x%p: MDACIOCTL_GETCONTROLLERPARAMETER\n", dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 11;
		break;

	case 11:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
			bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_ctrl_config_aborter(dev, NULL);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
			return(NFX_OK);
		}

		cp = (FSI_ControllerParms_t *)sdm_relay_rsp->data;
		st->c_parms = malloc(sizeof(raid_ctrlParam_t));
		bzero(st->c_parms, sizeof(raid_ctrlParam_t));

		st->c_parms->readAhead = (cp->Byte0 & FSI_CPARMS_B0_RDAHEN) ? 1 : 0;
		st->c_parms->reasRestrict = (cp->Byte0 & FSI_CPARMS_B0_REASS1S) ? 1 : 0;
		st->c_parms->trueVerify = (cp->Byte0 & FSI_CPARMS_B0_TRUVRFY) ? 1 : 0;
		st->c_parms->writeVerify = (cp->Byte0 & FSI_CPARMS_B0_DWTVRFY) ? 1 : 0;
		st->c_parms->bgInit = (cp->Byte0 & FSI_CPARMS_B0_BG_INIT) ? 1 : 0;
		st->c_parms->superReadAhead = (cp->Byte1 & FSI_CPARMS_B1_SUPERREADAHEAD) ? 1 : 0;
		st->c_parms->coalOptimal = (cp->Byte2 & FSI_CPARMS_B2_DEVICE_COMBING) ? 1 : 0;
		st->c_parms->autoRebuild = (cp->Byte4 & FSI_CPARMS_B4_V_ARM) ? 1 : 0;
		st->c_parms->operFault = (cp->Byte4 & FSI_CPARMS_B4_V_OFM) ? 1 : 0;
		st->c_parms->rebuildRate = cp->RebuildRate;
		st->c_parms->initRate = cp->BGIRate;
		st->c_parms->consistencyCheckRate = cp->ConsistencyCheckRate;
		st->c_parms->startupMode = cp->StartUpMode;
		st->c_parms->startupNumDev = cp->SpinDevs;
		st->c_parms->startupDelay = cp->SpinDelay1;
		st->c_parms->scsiStartupDelay = cp->SpinDelay2;
		st->c_parms->vendorTUR = (cp->Byte12 & FSI_CPARMS_B12_VUTURSNS) ? 1 : 0;
		st->c_parms->disCCInvLun = (cp->Byte12 & FSI_CPARMS_B12_DCCFIL) ? 1 : 0;
		st->c_parms->noPause = (cp->Byte12 & FSI_CPARMS_B12_NOPAUSE) ? 1 : 0;
		st->c_parms->queueFullBusy = (cp->Byte12 & FSI_CPARMS_B12_DISQFULL) ? 1 : 0;
		st->c_parms->disBusyFailback = (cp->Byte12 & FSI_CPARMS_B12_DISBUSY) ? 1 : 0;
		st->c_parms->nodeNameRet = (cp->Byte13 & FSI_CPARMS_B13_FAILOVERNODENAME) ? 1 : 0;
		st->c_parms->safteofups = (cp->Byte13 & FSI_CPARMS_B13_DISABLE_UPS) ? 1 : 0;
		st->c_parms->topology = (cp->Byte13 & FSI_CPARMS_B13_FTOPO_MASK) >> FSI_CPARMS_B13_FTOPO_SHIFT;
		st->c_parms->resetProp = (cp->ResetStyle & FSI_CPARMS_B14_RESET_PROPAGATE_RESET) ? 1 : 0;
		st->c_parms->rs232PortType = cp->SerialControl;
		st->c_parms->rs232BaudRate = 300 << cp->SerialBaud;
		st->c_parms->smartLrgTrans = (cp->Byte19 & FSI_CPARMS_B19_SMARTLARGEHOSTXFERS) ? 1 : 0;
		switch (cp->Byte19 & FSI_CPARMS_B19_FRAME_SIZE_MASK) {
		case FSI_CPARMS_B19_FRAME_SIZE_2048: 
			st->c_parms->frameSize = 2048; 
			break;
		case FSI_CPARMS_B19_FRAME_SIZE_512: 
			st->c_parms->frameSize = 512;
			break;
		case FSI_CPARMS_B19_FRAME_SIZE_1024: 
			st->c_parms->frameSize = 1024;
			break;
		default:
			st->c_parms->frameSize = -1;
		}
		st->c_parms->pciLatCtrl = (cp->Byte19 & FSI_CPARMS_B19_PCI_LATENCY_MASK) >> FSI_CPARMS_B19_PCI_LATENCY_SHIFT;
		st->c_parms->autoFailback = (cp->Byte20 & FSI_CPARMS_B20_MS_AUTOREST) ? 1 : 0;
		st->c_parms->forceSimplex = (cp->AAbits & FSI_CPARMS_B21_MS_AA_FSIM) ? 1 : 0;
		st->c_parms->conserveCache = (cp->AAbits & FSI_CPARMS_B21_MS_AA_CCASH) ? 1 : 0;
		st->c_parms->noRSTCOM = (cp->SimplexNoReset & FSI_CPARMS_B22_MS_FLG_SIMPLEX_NO_RSTCOM) ? 1 : 0;
		st->c_parms->queueLimit = cp->QueueLimit;
	
		st->c_parms->ctrl0Port0 = cp->HardLoopID[0][0] & 0x80 ? 1 : 0;
		st->c_parms->ctrl0Port1 = cp->HardLoopID[0][1] & 0x80 ? 1 : 0;
		st->c_parms->ctrl1Port0 = cp->HardLoopID[1][0] & 0x80 ? 1 : 0;
		st->c_parms->ctrl1Port1 = cp->HardLoopID[1][1] & 0x80 ? 1 : 0;
		st->c_parms->ctrl0Port0LoopId = cp->HardLoopID[0][0] & 0x7f;
		st->c_parms->ctrl0Port1LoopId = cp->HardLoopID[0][1] & 0x7f;
		st->c_parms->ctrl1Port0LoopId = cp->HardLoopID[1][0] & 0x7f;
		st->c_parms->ctrl1Port1LoopId = cp->HardLoopID[1][1] & 0x7f;

		st->c_parms->debugDump = cp->DebugDumpEnable;
		st->c_parms->rofInterval = 
			(cp->ROFControl & FSI_CPARMS_B66_ROF_REARM_INTERVAL_MASK) >> FSI_CPARMS_B66_ROF_REARM_INTERVAL_SHIFT;
		st->c_parms->rofReboot = 
			(cp->ROFControl & FSI_CPARMS_B66_ROF_REBOOT_COUNT_MASK) >> FSI_CPARMS_B66_ROF_REBOOT_COUNT_SHIFT;

		raid_xyratex_emit_ctrl_config(st->open_f, st->c_parms);
		
		rsp = malloc(sizeof(raid_ctrlConfigRspMsg_t));
		VERIFY(rsp);
		bzero(rsp, sizeof(raid_ctrlConfigRspMsg_t));
		raid_xyratex_ctrl_config_aborter(dev, NULL);		
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_ctrlConfigRspMsg_t);
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);

}

static int32 
raid_xyratex_config_clear_aborter(raid_dev_t *		dev, 
				  sdm_relayCdbRspMsg_t *sdm_relay_rsp)
{
	RAID_DBG(2, ("raid_xyratex_config_clear_aborter: dev 0x%p\n", dev));

	return(NFX_OK);
}

static int32 
raid_xyratex_config_clear_constructor(raid_dev_t *		dev, 
				      raid_msgHdr_t *		msg_hdr, 
				      sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				      int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	 req_msg;

	RAID_DBG(2, ("raid_xyratex_config_clear_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_CTRL_CLRCFG_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_CLEARCONF;
	req_msg->cdb[5] = 0x01;

	RAID_DBG(2, ("raid_xyratex_config_clear_constructor: dev 0x%p: MDACIOCTL_CLEARCONF\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_config_clear_deconstructor(raid_dev_t *		dev, 
					sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					int32 *			sdm_relay_req_len,
					raid_msgHdr_t **	rsp_msg, 
					int32 *			rsp_msg_len)
{
	raid_ctrlClrCfgRspMsg_t *	rsp;
	
	RAID_DBG(2, ("raid_xyratex_config_clear_deconstructor: dev 0x%p\n", dev));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp = malloc(sizeof(raid_ctrlClrCfgRspMsg_t));
		bzero(rsp, sizeof(raid_ctrlClrCfgRspMsg_t));
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_ctrlClrCfgRspMsg_t);
		return(NFX_OK);
	}

	rsp = malloc(sizeof(raid_ctrlClrCfgRspMsg_t));
	VERIFY(rsp);
	bzero(rsp, sizeof(raid_ctrlClrCfgRspMsg_t));

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = sizeof(raid_ctrlClrCfgRspMsg_t);

	return(NFX_OK);
}

typedef struct {
	int32				state;
	raid_logDevListRspMsg_t *	msg_rsp; 
	int32				msg_rsp_len;
	int32				logdev;
	int32				dev_index;
	int32				dev_count;
} raid_xyratex_get_logdev_list_sm_t;

static int32 
raid_xyratex_get_logdev_list_aborter(raid_dev_t *		dev, 
				     sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_get_logdev_list_sm_t *st = dev->private_context;

	RAID_DBG(2, ("raid_xyratex_get_logdev_list_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->msg_rsp)
			free(st->msg_rsp);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32
raid_xyratex_get_logdev_list_constructor(raid_dev_t *			dev, 
					 raid_msgHdr_t *		msg_hdr, 
					 sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					 int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *	req_msg;

	RAID_DBG(2, ("raid_xyratex_get_logdev_list_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_LOGDEV_LIST_REQ);

	dev->private_context = malloc(sizeof(raid_xyratex_get_logdev_list_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_get_logdev_list_sm_t));

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_logdev_list_constructor: dev 0x%p: "
		     "MDACIOCTL_GETCONTROLLERINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_logdev_list_deconstructor(raid_dev_t *			dev, 
					   sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					   int32 *			sdm_relay_req_len,
					   raid_msgHdr_t **		rsp_msg, 
					   int32 *			rsp_msg_len)
{
	FSI_ControllerInfo_t *		ci;
	FSI_GetLogDevInfoValid_t *	di;
	sdm_relayCdbReqMsg_t *		req_msg;

	raid_xyratex_get_logdev_list_sm_t *st = dev->private_context;

	RAID_DBG(2, ("raid_xyratex_get_logdev_list_deconstructor: dev 0x%p, "
		     "state %d, logdev 0x%x, dev_index %d\n", 
		     dev, st->state, st->logdev, st->dev_index));

	switch(st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			*rsp_msg_len = sizeof(raid_logDevListRspMsg_t);
			*rsp_msg = malloc(*rsp_msg_len);
			bzero(*rsp_msg, *rsp_msg_len);
			((raid_logDevListRspMsg_t *)(*rsp_msg))->rCode = RAID_RCODE_IO_ERROR;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;
		st->dev_count = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];
		st->msg_rsp_len = sizeof(raid_logDevListRspMsg_t) + st->dev_count * sizeof(raid_logdev_id_t);
		st->msg_rsp = (raid_logDevListRspMsg_t *)malloc(st->msg_rsp_len);
		bzero(st->msg_rsp, st->msg_rsp_len);
		st->msg_rsp->numEntries = st->dev_count;

		if (st->dev_count == 0) {
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
		req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
		req_msg->cdb[4] = (st->logdev & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_get_logdev_list_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
			     dev, st->logdev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			st->msg_rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
		}

		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		st->logdev = di->LDN[1] * 256 + di->LDN[0];

		st->msg_rsp->idList[st->dev_index] = st->logdev;
		++st->dev_index;

		if (st->dev_index < st->dev_count) {

			++st->logdev;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
			req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev & 0x00FF);
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

			RAID_DBG(2, ("raid_xyratex_get_logdev_list_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
				     dev, st->logdev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			st->state = 1;
		}
		else {
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			free(st);
			dev->private_context = NULL;
		}
		break;

	default:
		VERIFY(0);
	}

	return(NFX_OK);
}

typedef struct {
	int32				state;
	raid_logDevInfoRspMsg_t *	msg_rsp;
	uint32				msg_rsp_len;
} raid_xyratex_get_logdev_info_sm_t;

static int32 
raid_xyratex_get_logdev_info_aborter(raid_dev_t *		dev, 
				     sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_get_logdev_info_sm_t *st = dev->private_context; 

	RAID_DBG(2, ("raid_xyratex_get_logdev_info_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->msg_rsp)
			free(st->msg_rsp);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_get_logdev_info_constructor(raid_dev_t *			dev, 
					 raid_msgHdr_t *		msg_hdr, 
					 sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					 int32 *			sdm_relay_req_len)
{
	sdm_relayCdbReqMsg_t *			req_msg;
	raid_xyratex_get_logdev_info_sm_t *	st;

	RAID_DBG(2, ("raid_xyratex_get_logdev_info_constructor: dev 0x%p: id 0x%x\n", 
		     dev, 
		     ((raid_logDevInfoReqMsg_t *)msg_hdr)->id));

	VERIFY(msg_hdr->mid == RAID_MID_LOGDEV_INFO_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_get_logdev_info_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_get_logdev_info_sm_t));

	st->msg_rsp_len = sizeof(raid_logDevInfoRspMsg_t);
	st->msg_rsp = malloc(st->msg_rsp_len);
	bzero(st->msg_rsp, st->msg_rsp_len);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
	req_msg->cdb[3] = (((raid_logDevInfoReqMsg_t *)msg_hdr)->id & 0xFF00) >> 8;
	req_msg->cdb[4] = (((raid_logDevInfoReqMsg_t *)msg_hdr)->id & 0x00FF);
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_get_logdev_list_constructor: dev 0x%p: "
		     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
		     dev, ((raid_logDevInfoReqMsg_t *)msg_hdr)->id));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_get_logdev_info_deconstructor(raid_dev_t *			dev, 
					   sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					   int32 *			sdm_relay_req_len,
					   raid_msgHdr_t **		rsp_msg, 
					   int32 *			rsp_msg_len)
{
	FSI_GetLogDevInfoValid_t *		di;
	FSI_LogDevParms_t *			dp;
	raid_xyratex_get_logdev_info_sm_t *	st = dev->private_context;
	raid_logDevInfoReqMsg_t *		info_req_msg;
	raid_logDevInfoRspMsg_t *		rsp = st->msg_rsp;

	sdm_relayCdbReqMsg_t *			req_msg;

	RAID_DBG(2, ("raid_xyratex_get_logdev_info_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg_len = st->msg_rsp_len;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			st->msg_rsp = NULL;
			raid_xyratex_get_logdev_info_aborter(dev, NULL);
			return(NFX_OK);
		}

		info_req_msg = (raid_logDevInfoReqMsg_t *)dev->message_req_context->req_msg;
		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		/* Did we get the requested logical device ? */
		if (info_req_msg->id != di->LDN[1] * 256 + di->LDN[0]) {
			rsp->rCode = RAID_RCODE_INVALID_LUN;
			*rsp_msg_len = st->msg_rsp_len;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			st->msg_rsp = NULL;
			raid_xyratex_get_logdev_info_aborter(dev, NULL);
			return(NFX_OK);
		}

		rsp->id = di->LDN[1] * 256 + di->LDN[0];
		rsp->raidLevel = di->RaidLevel;
		memcpy(&rsp->physicalSizeInMB, &di->OrgDeviceSize, 4);
		rsp->physicalSizeInMB /= RAID_SECTORS_PER_MB;
		memcpy(&rsp->actualSizeInMB, &di->DeviceSize, 4); 
		rsp->actualSizeInMB /= RAID_SECTORS_PER_MB;
		rsp->stripeSizeInKB = FSI_stripeSizeToKB(di->StripeSize);
		rsp->cacheLineSizeInKB = FSI_cachelineSizeToKB(di->CacheLineSize);
		rsp->deviceState = di->State;
		rsp->hasBgTask = (di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK) != 0;

		rsp->isInitialized = (di->LDControl & FSI_LDCONTROL_INITIALIZED) ? 1 : 0;

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_LogDevParms_t);

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVPARAMETER;
		req_msg->cdb[3] = (info_req_msg->id & 0xFF00) >> 8;
		req_msg->cdb[4] = (info_req_msg->id & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("rbraid_xyratex_get_logdev_info_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVPARAMETER, logdev 0x%x\n", 
			     dev, info_req_msg->id));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg_len = st->msg_rsp_len;
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			st->msg_rsp = NULL;
			raid_xyratex_get_logdev_info_aborter(dev, NULL);
			return(NFX_OK);
		}

		dp = (FSI_LogDevParms_t *)sdm_relay_rsp->data;

		rsp->writeCacheEnabled = 
			(((dp->rw_control & FSI_RWCONTROL_WRITE_CACHE_MASK) == FSI_RWCONTROL_WRITE_CACHE_ENABLED) ||
			 ((dp->rw_control & FSI_RWCONTROL_WRITE_CACHE_MASK) == FSI_RWCONTROL_INT_WRITE_CACHE_ENABLED));
		rsp->readCacheEnabled = 
			(((dp->rw_control & FSI_RWCONTROL_READ_CACHE_MASK) == FSI_RWCONTROL_READ_CACHE_ENABLED) ||
			 ((dp->rw_control & FSI_RWCONTROL_READ_CACHE_MASK) == FSI_RWCONTROL_READ_AHEAD_ENABLED) ||
			 ((dp->rw_control & FSI_RWCONTROL_READ_CACHE_MASK) == FSI_RWCONTROL_INT_READ_AHEAD_ENABLED));
		
		*rsp_msg_len = st->msg_rsp_len;
		*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
		st->msg_rsp = NULL;
		raid_xyratex_get_logdev_info_aborter(dev, NULL);
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

typedef struct {
	uint32			dev_size;
	byte			channel_num;
	byte			target_id;
	ushort16		rdn;
	uint32			free_space;
} raid_xyratex_phys_dev_def_t;

typedef struct {
	int32				state;
	int32				logdev_count;
	uint32				highest_pdd_id;
	uint32				existing_stripe_size;
	int32				physdev_count;
	int32				physdev_index;
	int32				bus_count;
	int32				target_count;
	int32				bus_id;
	int32				target_id;
	raid_xyratex_phys_dev_def_t *	pdd;
} raid_xyratex_lun_create_sm_t;

static int32 
raid_xyratex_lun_create_aborter(raid_dev_t *		dev, 
			       sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_lun_create_sm_t *st = dev->private_context;	
	
	RAID_DBG(2, ("raid_xyratex_lun_create_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->pdd)
			free(st->pdd);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_create_simple_constructor(raid_dev_t *			dev, 
					   raid_msgHdr_t *		msg_hdr, 
					   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					   int32 *			sdm_relay_req_len)
{
	raid_createSimpleLunReqMsg_t *req = (raid_createSimpleLunReqMsg_t *)msg_hdr;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_create_simple_constructor: dev 0x%p, count %d, type %d\n", 
		     dev, 
		     req->lunCount,
		     req->raidLevel));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_CREATE_SIMPLE_REQ);

	dev->private_context = malloc(sizeof(raid_xyratex_lun_create_sm_t));
	bzero(dev->private_context, sizeof(raid_xyratex_lun_create_sm_t));

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_CLEARCONF;
	req_msg->cdb[5] = 0x01;

	RAID_DBG(2, ("raid_xyratex_lun_create_simple_constructor: dev 0x%p: MDACIOCTL_CLEARCONF\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_create_simple_deconstructor(raid_dev_t *		dev, 
					     sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
					     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
					     int32 *			sdm_relay_req_len,
					     raid_msgHdr_t **		rsp_msg, 
					     int32 *			rsp_msg_len)
{
	raid_xyratex_lun_create_sm_t *st = dev->private_context;
	raid_createSimpleLunReqMsg_t *req = (raid_createSimpleLunReqMsg_t *)dev->message_req_context->req_msg;
	raid_createSimpleLunRspMsg_t *rsp;
	FSI_ControllerInfo_t *	      ci;
	FSI_GetPhysDevInfoValid_t *   di;
	FSI_phys_dev_def_t *	      pdd;
	FSI_used_dev_def_t *	      udd;
	FSI_logical_dev_def_t *	      ldd;
	int32			      pdd_count, udd_count, ldd_count;
	sdm_relayCdbReqMsg_t *	      req_msg;
	int			      req_msg_len;
	char *			      p;
	int			      i, j;
	uint32			      dataout_len;
	uint32			      max_depth_per_lun, depth_per_lun;
	uint32			      max_lun_size, lun_size;
	uint32			      sectors_in_stripe;

	RAID_DBG(2, ("raid_xyratex_lun_create_simple_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createSimpleLunRspMsg_t));
			bzero(rsp, sizeof(raid_createSimpleLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createSimpleLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_create_simple_deconstructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n", dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createSimpleLunRspMsg_t));
			bzero(rsp, sizeof(raid_createSimpleLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createSimpleLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;
		st->physdev_count = ci->nPDPresent[1] * 256 + ci->nPDPresent[0];
		st->pdd = malloc(sizeof(raid_xyratex_phys_dev_def_t) * st->physdev_count); 
		bzero(st->pdd, sizeof(raid_xyratex_phys_dev_def_t) * st->physdev_count);
		st->bus_count = ci->nPhysicalChannels;
		st->target_count = ci->MaxTargetsPerCh[0];

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
		req_msg->cdb[3] = st->bus_id;
		req_msg->cdb[4] = st->target_id;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_create_simple_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
			     dev, st->bus_id, st->target_id));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 2;
		break;

	case 2: 
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createSimpleLunRspMsg_t));
			bzero(rsp, sizeof(raid_createSimpleLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createSimpleLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		di = (FSI_GetPhysDevInfoValid_t *)sdm_relay_rsp->data;

		st->bus_id = di->ch;
		st->target_id = di->id;

		/*
		 * Byte 8 of the Inquiry data is the Vendor ID. If
		 * this was not set, device is not there or is invalid
		 * Byte 0 of the Inquiry data is the Device type. c7
		 * is a Mylex controller, so don't include this 
		 */
		if (di->InquiryData[0] != 0xc7 && di->InquiryData[8] != 0x00) {
			st->pdd[st->physdev_index].channel_num = di->ch;
			st->pdd[st->physdev_index].target_id = di->id;
			memcpy(&st->pdd[st->physdev_index].dev_size, &di->UseableDevSize, 4);
			++st->physdev_index;
		}
		
		if (st->physdev_index < st->physdev_count) {

			++st->target_id;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
			req_msg->cdb[3] = st->bus_id;
			req_msg->cdb[4] = st->target_id;
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

			RAID_DBG(2, ("raid_xyratex_lun_create_simple_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
				     dev, st->bus_id, st->target_id));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			st->state = 2;
			break;
		}

		/* Construct the config */

		/* Step 1: Check that all physical drives are the same size */

		for (i = 0; i < st->physdev_count; ++i) {
			if (st->pdd[i].dev_size != st->pdd[0].dev_size) {
				rsp = malloc(sizeof(raid_createSimpleLunRspMsg_t));
				bzero(rsp, sizeof(raid_createSimpleLunRspMsg_t));
				rsp->rCode = RAID_RCODE_INVALID_PARAMETERS;
				*rsp_msg = (raid_msgHdr_t *)rsp; 
				*rsp_msg_len = sizeof(raid_createSimpleLunRspMsg_t);
				raid_xyratex_lun_create_aborter(dev, NULL);
				return(NFX_OK);
			}
		}

		/* Step 2: Construct PDDs, LDDs and UDDs */

		pdd_count = st->physdev_count;
		ldd_count = req->lunCount;
		udd_count = st->physdev_count;
		dataout_len = pdd_count * sizeof(FSI_phys_dev_def_t) + 
			ldd_count * sizeof(FSI_logical_dev_def_t) + 
			ldd_count * udd_count * sizeof(FSI_used_dev_def_t);
		req_msg = NULL;
		req_msg_len = (int)(&(req_msg->data));
		req_msg_len += dataout_len;
			
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(req_msg_len);
		VERIFY(req_msg);
		bzero(req_msg, req_msg_len);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataLen = dataout_len;

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_CREATENEWCONF;
		req_msg->cdb[7] = req_msg->dataLen >> 8;
		req_msg->cdb[8] = req_msg->dataLen & 0xFF;

		/* Construct the PDDs */
		p = req_msg->data;
		for (i = 0; i < st->physdev_count; ++i) {
			pdd = (FSI_phys_dev_def_t *)p;
			pdd->dev_type = 0x0F;
			pdd->dev_state = RAID_PHYDEVSTATE_OPTIMAL;
			pdd->rdn = 0xc000 + i;
			pdd->dev_start_addr = 0;
			pdd->dev_size = st->pdd[i].dev_size;
			pdd->channel_num = st->pdd[i].channel_num;
			pdd->target_id = st->pdd[i].target_id;

			RAID_DBG(2, ("<PDD>\n"));
			RAID_DBG(2, ("\trdn=0x%04x ",		pdd->rdn));
			RAID_DBG(2, ("controller_num=%d ",	pdd->controller_num));
			RAID_DBG(2, ("channel_num=%d ",		pdd->channel_num));
			RAID_DBG(2, ("target_id=%d ",		pdd->target_id));
			RAID_DBG(2, ("dev_size=0x%08x ",	pdd->dev_size));
			RAID_DBG(2, ("dev_start_addr=0x%08x\n",	pdd->dev_start_addr));
			RAID_DBG(2, ("</PDD>\n"));

			p += sizeof(FSI_phys_dev_def_t);
		}

		/* Construct LDDs and UDDs */
		max_depth_per_lun = st->pdd[0].dev_size / req->lunCount;
		max_lun_size = (req->raidLevel == 0 ? udd_count : udd_count - 1) * max_depth_per_lun;
		lun_size = req->sizeInMB * RAID_SECTORS_PER_MB;
		if (max_lun_size > lun_size)
			depth_per_lun = lun_size / (req->raidLevel == 0 ? udd_count : udd_count - 1);
		else
			depth_per_lun = max_depth_per_lun;

		/* Adjust depth to be a whole multiple of stripes */
		sectors_in_stripe = 1 << (FSI_KBTostripeSize(req->stripeSizeInKB) - 1);	
		if (depth_per_lun % sectors_in_stripe) {
			depth_per_lun &= ~(sectors_in_stripe - 1);
			depth_per_lun += sectors_in_stripe;
		}
		lun_size = (req->raidLevel == 0 ? udd_count : udd_count - 1) * depth_per_lun;

		for (i = 0; i < ldd_count; ++i) {
			ldd = (FSI_logical_dev_def_t *)p;
			ldd->dev_type = req->raidLevel;
			ldd->dev_state = RAID_LOGDEVSTATE_OPTIMAL;
			ldd->rdn = i;
			ldd->dev_size = lun_size;
			ldd->dev_used = udd_count;
			ldd->stripe_size = FSI_KBTostripeSize(req->stripeSizeInKB);
			ldd->cacheline_size = FSI_KBTostripeSize(req->stripeSizeInKB);
			ldd->rw_control = FSI_RWCONTROL_WRITE_CACHE_ENABLED;
			ldd->used_devsize = depth_per_lun;

			RAID_DBG(2, ("<LDD>\n"));
			RAID_DBG(2, ("\trdn=0x%04x ",		ldd->rdn));
			RAID_DBG(2, ("dev_type=0x%02x ",	ldd->dev_type));
			RAID_DBG(2, ("dev_size=0x%08x ",	ldd->dev_size));
			RAID_DBG(2, ("stripe_size=%s ",		FSI_stripeSizeToString(ldd->stripe_size)));
			RAID_DBG(2, ("cacheline_size=%s ",	FSI_cachelineSizeToString(ldd->cacheline_size)));
			RAID_DBG(2, ("rw_control=0x%02x ",	ldd->rw_control));
			RAID_DBG(2, ("udd_count=%d ",		ldd->dev_used));
			RAID_DBG(2, ("udd_depth=0x%08x\n",	ldd->used_devsize));

			p = (char *)&ldd->udds[0];
				
			for (j = 0; j < udd_count; ++j) {
				udd = (FSI_used_dev_def_t *)p;
				udd->dev_state = RAID_LOGDEVSTATE_OPTIMAL;
				udd->rdn = 0xc000 + j;
				udd->dev_start_addr = i * depth_per_lun;

				RAID_DBG(2, ("\t<UDD>\n"));
				RAID_DBG(2, ("\t\trdn=0x%04x ",			udd->rdn));
				RAID_DBG(2, ("dev_start_addr=0x%08x\n",		udd->dev_start_addr));
				RAID_DBG(2, ("\t</UDD>\n"));

				p += sizeof(FSI_used_dev_def_t);
			}

			RAID_DBG(2, ("</LDD>\n"));
			RAID_DBG(2, ("\n"));
		}
		RAID_DBG(2, ("raid_xyratex_lun_create_simple_deconstructor: dev 0x%p: MDACIOCTL_CREATENEWCONF\n", 
			     dev));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = req_msg_len;
			
		st->state = 3;
		break;

	case 3:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createSimpleLunRspMsg_t));
			bzero(rsp, sizeof(raid_createSimpleLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createSimpleLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		rsp = malloc(sizeof(raid_createSimpleLunRspMsg_t));
		VERIFY(rsp);
		bzero(rsp, sizeof(raid_createSimpleLunRspMsg_t));

		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_createSimpleLunRspMsg_t);

		free(st);
		dev->private_context = NULL;
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

static int32 
raid_xyratex_lun_create_constructor(raid_dev_t *		dev, 
				    raid_msgHdr_t *		msg_hdr, 
				    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				    int32 *			sdm_relay_req_len)
{
	raid_createLunReqMsg_t *      req = (raid_createLunReqMsg_t *)msg_hdr;
	raid_xyratex_lun_create_sm_t *st;
	sdm_relayCdbReqMsg_t *	      req_msg;
	int			      i;

	RAID_DBG(2, ("raid_xyratex_lun_create_constructor: dev 0x%p\n", 
		     dev));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_CREATE_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_lun_create_sm_t));
	bzero(st, sizeof(raid_xyratex_lun_create_sm_t));
	st->physdev_count = req->lundef.numOfPhyDevices;
	st->pdd = malloc(st->physdev_count * sizeof(raid_xyratex_phys_dev_def_t));
	bzero(st->pdd, st->physdev_count * sizeof(raid_xyratex_phys_dev_def_t));

	for (i = 0; i < st->physdev_count; ++i) {
		st->pdd[i].channel_num = req->lundef.phyDevList[i].channelNumber;
		st->pdd[i].target_id = req->lundef.phyDevList[i].targetId;
	}
		
	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);

        req_msg->cdb[0] = MDACMD_IOCTL;
        req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
        req_msg->cdb[7] = req_msg->dataInLen >> 8;
        req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_create_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_create_deconstructor(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				      sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				      int32 *			sdm_relay_req_len,
				      raid_msgHdr_t **		rsp_msg, 
				      int32 *			rsp_msg_len)
{
	raid_xyratex_lun_create_sm_t *st = dev->private_context;
	raid_createLunReqMsg_t *      req = (raid_createLunReqMsg_t *)dev->message_req_context->req_msg;
	raid_createLunRspMsg_t *      rsp;
	FSI_ControllerInfo_t *	      ci;
	FSI_GetPhysDevInfoValid_t *   di;
	FSI_phys_dev_def_t *	      pdd;
	FSI_used_dev_def_t *	      udd;
	FSI_logical_dev_def_t *	      ldd;
	int32			      pdd_count, udd_count, ldd_count;
	sdm_relayCdbReqMsg_t *	      req_msg;
	int			      req_msg_len;
	char *			      p;
	int			      i, j;
	uint32			      dataout_len, data_len;
	uint32			      lun_depth, lun_size, sectors_in_stripe;
	FSI_freeSpace_t *	      fs;
	int							free_size_found;

	RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p, state %d\n", dev, st->state));

 next_state:

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;

		st->logdev_count = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];

		if (st->logdev_count == RAID_MAX_LUNS) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_TOO_MANY_LUNS;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}
		else
		if (st->logdev_count > 0) {
			st->state = 1;
			goto next_state;
		}
		else {
			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_CLEARCONF;
			req_msg->cdb[5] = 0x01;

			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: MDACIOCTL_CLEARCONF\n",
				     dev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			st->state = 1;
		}
		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
		req_msg->cdb[3] = st->pdd[st->physdev_index].channel_num;
		req_msg->cdb[4] = st->pdd[st->physdev_index].target_id;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
			     dev, st->pdd[st->physdev_index].channel_num, st->pdd[st->physdev_index].target_id));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 2;
		break;

	case 2: 
		di = (FSI_GetPhysDevInfoValid_t *)sdm_relay_rsp->data;
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK ||
		    di->ch != st->pdd[st->physdev_index].channel_num ||
		    di->id != st->pdd[st->physdev_index].target_id) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_INVALID_PHYSICAL_DEVICE;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		memcpy(&st->pdd[st->physdev_index].dev_size, &di->UseableDevSize, 4);

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = 8;

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_XLATEPHYSDEVTORAIDDEV;
		req_msg->cdb[3] = st->pdd[st->physdev_index].channel_num;
		req_msg->cdb[4] = st->pdd[st->physdev_index].target_id;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: "
			     "MDACIOCTL_XLATEPHYSDEVTORAIDDEV bus %d, target %d\n", 
			     dev, 
			     st->pdd[st->physdev_index].channel_num,
			     st->pdd[st->physdev_index].target_id));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 3;
		break;

	case 3:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			st->pdd[st->physdev_index].rdn = 0xFFFF;
			st->pdd[st->physdev_index].free_space = st->pdd[st->physdev_index].dev_size;
		}
		else
			st->pdd[st->physdev_index].rdn = sdm_relay_rsp->data[1] * 256 + sdm_relay_rsp->data[0];

		++st->physdev_index;
		
		if (st->physdev_index < st->physdev_count) {
			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
			req_msg->cdb[3] = st->pdd[st->physdev_index].channel_num;
			req_msg->cdb[4] = st->pdd[st->physdev_index].target_id;
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETPHYSDEVINFOVALID bus %d, target %d\n", 
				     dev, st->pdd[st->physdev_index].channel_num, st->pdd[st->physdev_index].target_id));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			st->state = 2;
			break;
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_freeSpace_t);

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETFREESPACELIST;
		req_msg->cdb[3] = 0;
		req_msg->cdb[4] = 0;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: MDACIOCTL_GETFREESPACELIST\n",
			     dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			
		st->state = 4;
		break;

	case 4:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_GOOD) {
			fs = (FSI_freeSpace_t *)sdm_relay_rsp->data;
			for (j = 0; j < st->physdev_count; ++j) {
				free_size_found = 0;
				for (i = 0; i < fs->NumEntries; ++i) 
					if (st->pdd[j].rdn == fs->DeviceSpace[i].RDN) {
						memcpy(&st->pdd[j].free_space, fs->DeviceSpace[i].Size, 4);
						free_size_found = 1;
					}
				if(!free_size_found) 
					st->pdd[j].free_space = st->pdd[j].dev_size;
			}
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = 65535;
		
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETGROUPCONFINFO;
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: MDACIOCTL_GETGROUPCONFINFO\n",
			     dev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 5;
		break;

	case 5:
		st->highest_pdd_id = 0;
		st->existing_stripe_size = 0;
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_GOOD) {
			p = sdm_relay_rsp->data;
			data_len = sdm_relay_rsp->dataLen;

			/*
			 * Scan the current config for the following:
			 * (1) The highest assigned PDD device number.
			 * (2) Stripe size of existing LUNs -- should be the same for all.
			 */
			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: Existing config\n"));
			while (data_len) {
				if (*p == 0x0F) {
					pdd = (FSI_phys_dev_def_t *)p;
					if (st->highest_pdd_id < pdd->rdn)
						st->highest_pdd_id = pdd->rdn;
					RAID_DBG(2, ("PDD %04x: CID/TID %02d/%02d: size %d, start LBA 0x%x\n",
						     pdd->rdn, 
						     pdd->channel_num, pdd->target_id, 
						     pdd->dev_size, pdd->dev_start_addr));
					p += sizeof(FSI_phys_dev_def_t);
					data_len -= sizeof(FSI_phys_dev_def_t);
				}
				else {
					ldd = (FSI_logical_dev_def_t *)p;
					RAID_DBG(2, ("LDD %04x (%s):\n", ldd->rdn, FSI_devTypeToString(ldd->dev_type)));
					RAID_DBG(2, ("   size 0x%x, depth 0x%x, stripe_size %s, cacheline_size %s\n",
						     ldd->dev_size, 
						     ldd->used_devsize,
						     FSI_stripeSizeToString(ldd->stripe_size),
						     FSI_cachelineSizeToString(ldd->cacheline_size)));
					if (st->existing_stripe_size == 0)
						st->existing_stripe_size = ldd->stripe_size;
					p += sizeof(FSI_logical_dev_def_t);
					data_len -= sizeof(FSI_logical_dev_def_t);
					
					for (i = 0; i < ldd->dev_used; ++i) {
						udd = (FSI_used_dev_def_t *)p;
						RAID_DBG(2, ("   UDD[%d] %04x: start LBA 0x%x\n", i, udd->rdn, udd->dev_start_addr));
						p += sizeof(FSI_used_dev_def_t);
						data_len -= sizeof(FSI_used_dev_def_t);
					}
				}
			}
		}

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: logdev_count %d, highest_pdd_id 0x%x, existing_stripe_size %s\n",
			     st->logdev_count, st->highest_pdd_id, FSI_stripeSizeToString(st->existing_stripe_size)));
		for (i = 0; i < st->physdev_count; ++i) {
			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: [%d] RDN 0x%x CID/TID %d/%d size 0x%x, freespace 0x%x\n", 
				     i, 
				     st->pdd[i].rdn, 
				     st->pdd[i].channel_num,
				     st->pdd[i].target_id,
				     st->pdd[i].dev_size,
				     st->pdd[i].free_space));
		}

		/* Check that requested stripe size matches that of existing LUNs */
		if (st->existing_stripe_size && 
		    FSI_KBTostripeSize(req->lundef.stripeSizeInKB) != st->existing_stripe_size) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_INVALID_PARAMETERS;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		sectors_in_stripe = 1 << (FSI_KBTostripeSize(req->lundef.stripeSizeInKB) - 1);
		lun_size = req->lundef.sizeInMB * RAID_SECTORS_PER_MB;
		lun_depth = lun_size / (req->lundef.raidLevel == 0 ? st->physdev_count : st->physdev_count - 1);
		if (lun_depth % sectors_in_stripe) {
			lun_depth &= ~(sectors_in_stripe - 1);
			lun_depth += sectors_in_stripe;
		}
		lun_size = lun_depth * (req->lundef.raidLevel == 0 ? st->physdev_count : st->physdev_count - 1);

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: sectors_in_stripe %d, lun_size 0x%x, lun_depth 0x%x\n", 
			     sectors_in_stripe, lun_size, lun_depth));

		/* Check that required physical devices has sufficient space */
		for (i = 0; i < st->physdev_count; ++i) {
			if (st->pdd[i].free_space < lun_depth) {
				rsp = malloc(sizeof(raid_createLunRspMsg_t));
				bzero(rsp, sizeof(raid_createLunRspMsg_t));
				rsp->rCode = RAID_RCODE_INSUFFICIENT_SPACE;
				*rsp_msg = (raid_msgHdr_t *)rsp; 
				*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
				raid_xyratex_lun_create_aborter(dev, NULL);
				return(NFX_OK);
			}
		}

		/* Construct the config */
		pdd_count = 0;
		for (i = 0; i < st->physdev_count; ++i) {
			if (st->pdd[i].rdn == 0xFFFF)
				++pdd_count;
		}
		ldd_count = 1;
		udd_count = st->physdev_count;

		dataout_len = pdd_count * sizeof(FSI_phys_dev_def_t) + 
			ldd_count * sizeof(FSI_logical_dev_def_t) + 
			ldd_count * udd_count * sizeof(FSI_used_dev_def_t);

		req_msg = NULL;
		req_msg_len = (int)(&(req_msg->data));
		req_msg_len += dataout_len;
			
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(req_msg_len);
		VERIFY(req_msg);
		bzero(req_msg, req_msg_len);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataLen = dataout_len;

		if (st->logdev_count == 0) {
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_CREATENEWCONF;
			req_msg->cdb[7] = req_msg->dataLen >> 8;
			req_msg->cdb[8] = req_msg->dataLen & 0xFF;
			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: MDACIOCTL_CREATENEWCONF\n", 
				     dev));
		}
		else {
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_ADDNEWCONF;
			req_msg->cdb[3] = 0; /* (st->logdev_count - 1) >> 8; */
			req_msg->cdb[4] = 0; /* (st->logdev_count - 1) & 0xFF; */
			req_msg->cdb[7] = req_msg->dataLen >> 8;
			req_msg->cdb[8] = req_msg->dataLen & 0xFF;
			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: MDACIOCTL_ADDNEWCONF\n", 
				     dev));
		}

		/* Construct the needed PDDs */
		if (st->highest_pdd_id == 0)
			st->highest_pdd_id = 0xc000;
		else
			++st->highest_pdd_id;
		p = req_msg->data;
		for (i = 0; i < st->physdev_count; ++i) {
			if (st->pdd[i].rdn == 0xFFFF) {
				pdd = (FSI_phys_dev_def_t *)p;
				pdd->dev_type = 0x0F;
				pdd->dev_state = RAID_PHYDEVSTATE_OPTIMAL;
				pdd->rdn = st->pdd[i].rdn = st->highest_pdd_id++;
				pdd->dev_start_addr = 0;
				pdd->dev_size = st->pdd[i].dev_size;
				pdd->channel_num = st->pdd[i].channel_num;
				pdd->target_id = st->pdd[i].target_id;
				p += sizeof(FSI_phys_dev_def_t);
			}
		}

		/* Construct LDDs and UDDs */
		ldd = (FSI_logical_dev_def_t *)p;
		ldd->dev_type = req->lundef.raidLevel;
		ldd->dev_state = RAID_LOGDEVSTATE_OPTIMAL;
		ldd->rdn = st->logdev_count;
		ldd->dev_size = lun_size;
		ldd->dev_used = st->physdev_count;
		ldd->stripe_size = FSI_KBTostripeSize(req->lundef.stripeSizeInKB);
		ldd->cacheline_size = FSI_KBTostripeSize(req->lundef.stripeSizeInKB);
		ldd->rw_control = req->lundef.cacheEnabled ? FSI_RWCONTROL_WRITE_CACHE_ENABLED : 0;
		ldd->used_devsize = lun_depth;
		p = (char *)&ldd->udds[0];
				
		for (i = 0; i < udd_count; ++i) {
			udd = (FSI_used_dev_def_t *)p;
			udd->dev_state = RAID_LOGDEVSTATE_OPTIMAL;
			udd->rdn = st->pdd[i].rdn;
			udd->dev_start_addr = st->pdd[i].dev_size - st->pdd[i].free_space;
			if (udd->dev_start_addr % sectors_in_stripe) {
				udd->dev_start_addr &= ~(sectors_in_stripe - 1);
				udd->dev_start_addr += sectors_in_stripe;
			}
			p += sizeof(FSI_used_dev_def_t);
		}

		RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: New/added config\n"));
		p = req_msg->data;
		data_len = req_msg->dataLen;
			
		while (data_len) {
			if (*p == 0x0F) {
				pdd = (FSI_phys_dev_def_t *)p;
				RAID_DBG(2, ("PDD %04x: CID/TID %02d/%02d: size 0x%x, start LBA 0x%x\n",
					     pdd->rdn, 
					     pdd->channel_num, pdd->target_id, 
					     pdd->dev_size, pdd->dev_start_addr));
				p += sizeof(FSI_phys_dev_def_t);
				data_len -= sizeof(FSI_phys_dev_def_t);
			}
			else {
				ldd = (FSI_logical_dev_def_t *)p;
				RAID_DBG(2, ("LDD %04x (%s):\n", ldd->rdn, FSI_devTypeToString(ldd->dev_type)));
				RAID_DBG(2, ("   size 0x%x, depth 0x%x, stripe_size %s, cacheline_size %s\n",
					     ldd->dev_size, 
					     ldd->used_devsize,
					     FSI_stripeSizeToString(ldd->stripe_size),
					     FSI_cachelineSizeToString(ldd->cacheline_size)));
				p += sizeof(FSI_logical_dev_def_t);
				data_len -= sizeof(FSI_logical_dev_def_t);
					
				for (i = 0; i < ldd->dev_used; ++i) {
					udd = (FSI_used_dev_def_t *)p;
					RAID_DBG(2, ("   UDD[%d] %04x: start LBA 0x%x\n", i, udd->rdn, udd->dev_start_addr));
					p += sizeof(FSI_used_dev_def_t);
					data_len -= sizeof(FSI_used_dev_def_t);
				}
			}
		}

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = req_msg_len;
			
		st->state = 6;
		break;

	case 6:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}

		/* Should we auto-init ? */
		if (req->lundef.autoInit) {
			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_INITRAIDDEVSTART;
			req_msg->cdb[3] = (st->logdev_count & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev_count & 0x00FF);

			RAID_DBG(2, ("raid_xyratex_lun_create_deconstructor: dev 0x%p: MDACIOCTL_INITRAIDDEVSTART LUN id %d\n",
				     dev, st->logdev_count));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			st->state = 7;
		}
		else {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			VERIFY(rsp);
			bzero(rsp, sizeof(raid_createLunRspMsg_t));

			rsp->id = st->logdev_count;
			
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);

			raid_xyratex_lun_create_aborter(dev, NULL);
		}
		break;

	case 7:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_createLunRspMsg_t));
			bzero(rsp, sizeof(raid_createLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_createLunRspMsg_t);
			raid_xyratex_lun_create_aborter(dev, NULL);
			return(NFX_OK);
		}
		rsp = malloc(sizeof(raid_createLunRspMsg_t));
		VERIFY(rsp);
		bzero(rsp, sizeof(raid_createLunRspMsg_t));

		rsp->id = st->logdev_count;
			
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_createLunRspMsg_t);

		raid_xyratex_lun_create_aborter(dev, NULL);
		break;
		
	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

static int32 
raid_xyratex_lun_phylist_constructor(raid_dev_t *		dev, 
				     raid_msgHdr_t *		msg_hdr, 
				     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				     int32 *			sdm_relay_req_len)
{
	raid_logDevPhyListReqMsg_t *  req = (raid_logDevPhyListReqMsg_t *)msg_hdr;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_phylist_constructor: dev 0x%p, LUN ID 0x%x\n", 
		     dev, req->id));

	VERIFY(msg_hdr->mid == RAID_MID_LOGDEV_PHYLIST_REQ);

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = 65535;
		
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETGROUPCONFINFO;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_phylist_deconstructor: dev 0x%p: MDACIOCTL_GETGROUPCONFINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_phylist_deconstructor(raid_dev_t *		dev, 
				       sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				       sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				       int32 *			sdm_relay_req_len,
				       raid_msgHdr_t **		rsp_msg, 
				       int32 *			rsp_msg_len)
{
	raid_logDevPhyListReqMsg_t *  req = (raid_logDevPhyListReqMsg_t *)dev->message_req_context->req_msg;
	raid_logDevPhyListRspMsg_t *  rsp;
	int			      rsp_len;
	FSI_phys_dev_def_t *	      pdd;
	FSI_phys_dev_def_t *	      first_pdd = NULL;
	FSI_used_dev_def_t *	      udd;
	FSI_logical_dev_def_t *	      ldd;
	FSI_logical_dev_def_t *	      found_ldd = NULL;
	char *			      p;
	int			      i;
	int			      data_len;

	RAID_DBG(2, ("raid_xyratex_lun_phylist_deconstructor: dev 0x%p\n", dev));

	if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
		rsp = malloc(sizeof(raid_logDevPhyListRspMsg_t));
		bzero(rsp, sizeof(raid_logDevPhyListRspMsg_t));
		rsp->rCode = RAID_RCODE_IO_ERROR;
		*rsp_msg = (raid_msgHdr_t *)rsp; 
		*rsp_msg_len = sizeof(raid_logDevPhyListRspMsg_t);
		return(NFX_OK);
	}

	p = sdm_relay_rsp->data;
	data_len = sdm_relay_rsp->dataLen;
		
	while (data_len) {
		switch (*p) {
		case 0x0F:
			pdd = (FSI_phys_dev_def_t *)p;
			if (first_pdd == NULL)
				first_pdd = pdd;
			p += sizeof(FSI_phys_dev_def_t);
			data_len -= sizeof(FSI_phys_dev_def_t);
			break;

		default:
			ldd = (FSI_logical_dev_def_t *)p;
			if (ldd->rdn == req->id)
				found_ldd = ldd;
			p += sizeof(FSI_logical_dev_def_t);
			data_len -= sizeof(FSI_logical_dev_def_t);

			for (i = 0; i < ldd->dev_used; ++i) {
				udd = (FSI_used_dev_def_t *)p;
				p += sizeof(FSI_used_dev_def_t);
				data_len -= sizeof(FSI_used_dev_def_t);
			}
		}
	}
	if (found_ldd == NULL) {
		rsp = malloc(sizeof(raid_logDevPhyListRspMsg_t));
		bzero(rsp, sizeof(raid_logDevPhyListRspMsg_t));
		rsp->rCode = RAID_RCODE_INVALID_LUN;
		*rsp_msg = (raid_msgHdr_t *)rsp; 
		*rsp_msg_len = sizeof(raid_logDevPhyListRspMsg_t);
		return(NFX_OK);
	}

	rsp_len = sizeof(raid_logDevPhyListRspMsg_t) + found_ldd->dev_used * sizeof(raid_logDevPhyDrv_t);
	rsp = (raid_logDevPhyListRspMsg_t *)malloc(rsp_len);
	bzero(rsp, rsp_len);

	rsp->numDevices = found_ldd->dev_used;

	p = (char *)found_ldd;
	p += sizeof(FSI_logical_dev_def_t);
	for (i = 0; i < found_ldd->dev_used; ++i) {
		udd = (FSI_used_dev_def_t *)p;
		for (pdd = first_pdd; pdd->dev_type == 0x0f && pdd->rdn != udd->rdn; ++pdd)
			;
		VERIFY(pdd->rdn == udd->rdn);
		rsp->driveList[i].rdn = pdd->rdn;
		rsp->driveList[i].channelNumber = pdd->channel_num;
		rsp->driveList[i].targetId = pdd->target_id;
		rsp->driveList[i].startBlock = udd->dev_start_addr + pdd->dev_start_addr;
		rsp->driveList[i].usedBlocks = found_ldd->used_devsize;
		rsp->driveList[i].totalBlocks = pdd->dev_size;
		p += sizeof(FSI_used_dev_def_t);
	}

	*rsp_msg = (raid_msgHdr_t *)rsp;
	*rsp_msg_len = rsp_len;

	return(NFX_OK);
}

typedef struct {
	int32				state;
	int32				logdev_count;
	int32				logdev_index;
	int32				logdev;
	uint32				highest_ldd_id;
} raid_xyratex_lun_delete_sm_t;

static int32 
raid_xyratex_lun_delete_aborter(raid_dev_t *		dev, 
				sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{

	raid_xyratex_lun_delete_sm_t *st = dev->private_context;	
	
	RAID_DBG(2, ("raid_xyratex_lun_delete_aborter: dev 0x%p\n", dev));

	if (st) {
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_delete_constructor(raid_dev_t *		dev, 
				    raid_msgHdr_t *		msg_hdr, 
				    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				    int32 *			sdm_relay_req_len)
{
	raid_xyratex_lun_delete_sm_t *st;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_delete_constructor: dev 0x%p\n", 
		     dev));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_DELETE_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_lun_delete_sm_t));
	bzero(st, sizeof(raid_xyratex_lun_delete_sm_t));
		
	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);

        req_msg->cdb[0] = MDACMD_IOCTL;
        req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
        req_msg->cdb[7] = req_msg->dataInLen >> 8;
        req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_delete_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_delete_deconstructor(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				      sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				      int32 *			sdm_relay_req_len,
				      raid_msgHdr_t **		rsp_msg, 
				      int32 *			rsp_msg_len)
{
	raid_xyratex_lun_delete_sm_t *st = dev->private_context;
	raid_deleteLunReqMsg_t *      req = (raid_deleteLunReqMsg_t *)dev->message_req_context->req_msg;
	raid_deleteLunRspMsg_t *      rsp;
	FSI_ControllerInfo_t *	      ci;
	FSI_GetLogDevInfoValid_t *    di;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_delete_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_deleteLunRspMsg_t));
			bzero(rsp, sizeof(raid_deleteLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_deleteLunRspMsg_t);
			raid_xyratex_lun_delete_aborter(dev, NULL);
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;

		st->logdev_count = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
		req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
		req_msg->cdb[4] = (st->logdev & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_delete_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
			     dev, st->logdev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_deleteLunRspMsg_t));
			bzero(rsp, sizeof(raid_deleteLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_deleteLunRspMsg_t);
			raid_xyratex_lun_delete_aborter(dev, NULL);
			return(NFX_OK);
		}

		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		st->logdev = di->LDN[1] * 256 + di->LDN[0];
		if (st->logdev > st->highest_ldd_id)
			st->highest_ldd_id = st->logdev;
		++st->logdev_index;

		if (st->logdev_index < st->logdev_count) {

			++st->logdev;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
			req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev & 0x00FF);
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

			RAID_DBG(2, ("raid_xyratex_lun_delete_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
				     dev, st->logdev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			st->state = 1;
		}
		else {
			/* Check that requested LUN is the last one */
			if (req->id != st->highest_ldd_id) {
				rsp = malloc(sizeof(raid_deleteLunRspMsg_t));
				bzero(rsp, sizeof(raid_deleteLunRspMsg_t));
				rsp->rCode = RAID_RCODE_UNABLE_TO_DELETE_LUN;
				*rsp_msg = (raid_msgHdr_t *)rsp; 
				*rsp_msg_len = sizeof(raid_deleteLunRspMsg_t);
				raid_xyratex_lun_delete_aborter(dev, NULL);
				return(NFX_OK);
			}

			if (st->logdev_count == 1) {
				req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
				VERIFY(req_msg);
				bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

				req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
				req_msg->hdr.chandle = (uint32)dev;
				req_msg->handle = dev->open_handle;
				req_msg->retryCnt = 1;
				req_msg->timeOutCnt = 60;
				req_msg->maxTimeToLive = 60;
	
				req_msg->cdb[0] = MDACMD_IOCTL;
				req_msg->cdb[2] = MDACIOCTL_CLEARCONF;
				req_msg->cdb[5] = 0x01;

				RAID_DBG(2, ("raid_xyratex_lun_delete_deconstructor: dev 0x%p: MDACIOCTL_CLEARCONF\n",
					     dev));
				*sdm_relay_req = req_msg;
				*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			}
			else {
				req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
				VERIFY(req_msg);
				bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

				req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
				req_msg->hdr.chandle = (uint32)dev;
				req_msg->handle = dev->open_handle;
				req_msg->retryCnt = 1;
				req_msg->timeOutCnt = 60;
				req_msg->maxTimeToLive = 60;
	
				req_msg->cdb[0] = MDACMD_IOCTL;
				req_msg->cdb[2] = MDACIOCTL_DELETERAIDDEV;
				req_msg->cdb[3] = (req->id & 0xFF00) >> 8;
				req_msg->cdb[4] = (req->id & 0x00FF);

				RAID_DBG(2, ("raid_xyratex_lun_delete_deconstructor: dev 0x%p: MDACIOCTL_DELETERAIDDEV LUN id %d\n",
					     dev, req->id));
				*sdm_relay_req = req_msg;
				*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
				
			}
			st->state = 2;
		}
		break;

	case 2:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_deleteLunRspMsg_t));
			bzero(rsp, sizeof(raid_deleteLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_deleteLunRspMsg_t);
			raid_xyratex_lun_delete_aborter(dev, NULL);
			return(NFX_OK);
		}

		rsp = malloc(sizeof(raid_deleteLunRspMsg_t));
		VERIFY(rsp);
		bzero(rsp, sizeof(raid_deleteLunRspMsg_t));

		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_deleteLunRspMsg_t);

		raid_xyratex_lun_delete_aborter(dev, NULL);
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

typedef struct {
	int32				state;
	int32				logdev_count;
	int32				logdev_index;
	int32				logdev;
	int32				req_id_valid;
	int32				bg_started_count;
	raid_bpListRspMsg_t *		msg_rsp;
	int32				msg_rsp_len;
} raid_xyratex_lun_bg_op_sm_t;

static int32 
raid_xyratex_lun_bg_op_aborter(raid_dev_t *		dev, 
			       sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{

	raid_xyratex_lun_bg_op_sm_t *st = dev->private_context;	
	
	RAID_DBG(2, ("raid_xyratex_lun_bg_op_aborter: dev 0x%p\n", dev));

	if (st) {
		if (st->msg_rsp)
			free(st->msg_rsp);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_init_constructor(raid_dev_t *			dev, 
				  raid_msgHdr_t *		msg_hdr, 
				  sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				  int32 *			sdm_relay_req_len)
{
	raid_xyratex_lun_bg_op_sm_t *st;
	sdm_relayCdbReqMsg_t *       req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_init_constructor: dev 0x%p\n", 
		     dev));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_INIT_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_lun_bg_op_sm_t));
	bzero(st, sizeof(raid_xyratex_lun_bg_op_sm_t));
		
	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);

        req_msg->cdb[0] = MDACMD_IOCTL;
        req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
        req_msg->cdb[7] = req_msg->dataInLen >> 8;
        req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_init_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_init_deconstructor(raid_dev_t *		dev, 
				    sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				    int32 *			sdm_relay_req_len,
				    raid_msgHdr_t **		rsp_msg, 
				    int32 *			rsp_msg_len)
{
	raid_xyratex_lun_bg_op_sm_t * st = dev->private_context;
	raid_initLunReqMsg_t *        req = (raid_initLunReqMsg_t *)dev->message_req_context->req_msg;
	raid_initLunRspMsg_t *        rsp;
	FSI_ControllerInfo_t *	      ci;
	FSI_GetLogDevInfoValid_t *    di;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_init_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_initLunRspMsg_t));
			bzero(rsp, sizeof(raid_initLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;

		st->logdev_count = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
		req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
		req_msg->cdb[4] = (st->logdev & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_init_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
			     dev, st->logdev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_initLunRspMsg_t));
			bzero(rsp, sizeof(raid_initLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		st->logdev = di->LDN[1] * 256 + di->LDN[0];

		RAID_DBG(2, ("raid_xyratex_lun_init_deconstructor: "
			     "LDN 0x%x, LDControl 0x%x, OperationsStatus 0x%x\n", 
			     st->logdev, di->LDControl, di->OperationsStatus));

		if (req->type == RAID_INITTYPE_ONE && st->logdev == req->id)
			++st->req_id_valid;

		if (req->type == RAID_INITTYPE_ONE && 
		    st->logdev == req->id && 
		    (di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK)) {
			rsp = malloc(sizeof(raid_initLunRspMsg_t));
			bzero(rsp, sizeof(raid_initLunRspMsg_t));
			rsp->rCode = RAID_RCODE_LUN_IS_BUSY;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		if ((req->type == RAID_INITTYPE_ONE && 
		     st->logdev == req->id && 
		     !(di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK)) ||
		    (req->type == RAID_INITTYPE_ALL && 
		     !(di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK)) ||
		    (req->type == RAID_INITTYPE_ALLNOT && 
		     !(di->LDControl & FSI_LDCONTROL_INITIALIZED) && 
		     !(di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK))) {

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_INITRAIDDEVSTART;
			req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev & 0x00FF);

			RAID_DBG(2, ("raid_xyratex_lun_init_deconstructor: dev 0x%p: MDACIOCTL_INITRAIDDEVSTART LUN id %d\n",
				     dev, st->logdev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			++st->bg_started_count;

			st->state = 2;
		}
		else {
			++st->logdev_index;
			if (st->logdev_index < st->logdev_count) {

				++st->logdev;

				req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
				VERIFY(req_msg);
				bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

				req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
				req_msg->hdr.chandle = (uint32)dev;
				req_msg->handle = dev->open_handle;
				req_msg->retryCnt = 1;
				req_msg->timeOutCnt = 60;
				req_msg->maxTimeToLive = 60;
				req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
			
				req_msg->cdb[0] = MDACMD_IOCTL;
				req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
				req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
				req_msg->cdb[4] = (st->logdev & 0x00FF);
				req_msg->cdb[7] = req_msg->dataInLen >> 8;
				req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

				RAID_DBG(2, ("raid_xyratex_lun_init_deconstructor: dev 0x%p: "
					     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
					     dev, st->logdev));
				*sdm_relay_req = req_msg;
				*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			
				st->state = 1;
			}
			else 
			if (req->type == RAID_INITTYPE_ONE && st->bg_started_count == 0) {
				rsp = malloc(sizeof(raid_initLunRspMsg_t));
				bzero(rsp, sizeof(raid_initLunRspMsg_t));
				rsp->rCode = RAID_RCODE_INVALID_LUN;
				*rsp_msg = (raid_msgHdr_t *)rsp; 
				*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
				raid_xyratex_lun_bg_op_aborter(dev, NULL);
				return(NFX_OK);
			}
			else {
				rsp = malloc(sizeof(raid_initLunRspMsg_t));
				bzero(rsp, sizeof(raid_initLunRspMsg_t));
				rsp->init_started_count = st->bg_started_count;
				*rsp_msg = (raid_msgHdr_t *)rsp;
				*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
				raid_xyratex_lun_bg_op_aborter(dev, NULL);
				return(NFX_OK);
			}
		}
		break;

	case 2:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_initLunRspMsg_t));
			bzero(rsp, sizeof(raid_initLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		++st->logdev_index;
		if (st->logdev_index < st->logdev_count) {

			++st->logdev;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
			
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
			req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev & 0x00FF);
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

			RAID_DBG(2, ("raid_xyratex_lun_init_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
				     dev, st->logdev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			
			st->state = 1;
		}
		else 
		if (req->type == RAID_INITTYPE_ONE && st->bg_started_count == 0) {
			rsp = malloc(sizeof(raid_initLunRspMsg_t));
			bzero(rsp, sizeof(raid_initLunRspMsg_t));
			rsp->rCode = RAID_RCODE_INVALID_LUN;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}
		else {
			rsp = malloc(sizeof(raid_initLunRspMsg_t));
			bzero(rsp, sizeof(raid_initLunRspMsg_t));
			rsp->init_started_count = st->bg_started_count;
			*rsp_msg = (raid_msgHdr_t *)rsp;
			*rsp_msg_len = sizeof(raid_initLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}
		break;

	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

static int32 
raid_xyratex_lun_check_constructor(raid_dev_t *			dev, 
				   raid_msgHdr_t *		msg_hdr, 
				   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				   int32 *			sdm_relay_req_len)
{
	raid_xyratex_lun_bg_op_sm_t *st;
	sdm_relayCdbReqMsg_t *       req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_check_constructor: dev 0x%p\n", 
		     dev));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_CHECK_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_lun_bg_op_sm_t));
	bzero(st, sizeof(raid_xyratex_lun_bg_op_sm_t));
		
	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);

        req_msg->cdb[0] = MDACMD_IOCTL;
        req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
        req_msg->cdb[7] = req_msg->dataInLen >> 8;
        req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_check_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_check_deconstructor(raid_dev_t *		dev, 
				     sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				     int32 *			sdm_relay_req_len,
				     raid_msgHdr_t **		rsp_msg, 
				     int32 *			rsp_msg_len)
{
	raid_xyratex_lun_bg_op_sm_t * st = dev->private_context;
	raid_checkLunReqMsg_t *       req = (raid_checkLunReqMsg_t *)dev->message_req_context->req_msg;
	raid_checkLunRspMsg_t *       rsp;
	FSI_ControllerInfo_t *	      ci;
	FSI_GetLogDevInfoValid_t *    di;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_check_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_checkLunRspMsg_t));
			bzero(rsp, sizeof(raid_checkLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_checkLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;

		st->logdev_count = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
		req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
		req_msg->cdb[4] = (st->logdev & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_check_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
			     dev, st->logdev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_checkLunRspMsg_t));
			bzero(rsp, sizeof(raid_checkLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_checkLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		st->logdev = di->LDN[1] * 256 + di->LDN[0];

		RAID_DBG(2, ("raid_xyratex_lun_check_deconstructor: "
			     "LDN 0x%x, LDControl 0x%x, OperationsStatus 0x%x\n", 
			     st->logdev, di->LDControl, di->OperationsStatus));

		if (st->logdev == req->id)
			++st->req_id_valid;

		if ((st->logdev == req->id && 
		     (di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK)) ||
		    (di->OperationsStatus & (FSI_OPSTATUS_CCHECKACTIVE | FSI_OPSTATUS_REBUILDACTIVE | FSI_OPSTATUS_MDCONSISTACTIVE))) {
			rsp = malloc(sizeof(raid_checkLunRspMsg_t));
			bzero(rsp, sizeof(raid_checkLunRspMsg_t));
			rsp->rCode = RAID_RCODE_LUN_IS_BUSY;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_checkLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		++st->logdev_index;
		if (st->logdev_index < st->logdev_count) {

			++st->logdev;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
			
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
			req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev & 0x00FF);
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;
			
			RAID_DBG(2, ("raid_xyratex_lun_check_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
				     dev, st->logdev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			
			st->state = 1;
		}
		else {
			if (st->req_id_valid == 0) {
				rsp = malloc(sizeof(raid_checkLunRspMsg_t));
				bzero(rsp, sizeof(raid_checkLunRspMsg_t));
				rsp->rCode = RAID_RCODE_INVALID_LUN;
				*rsp_msg = (raid_msgHdr_t *)rsp; 
				*rsp_msg_len = sizeof(raid_checkLunRspMsg_t);
				raid_xyratex_lun_bg_op_aborter(dev, NULL);
				return(NFX_OK);
			}
			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
	
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_CONSISTENCYCHECKSTART;
			req_msg->cdb[3] = (req->id & 0xFF00) >> 8;
			req_msg->cdb[4] = (req->id & 0x00FF);
			req_msg->cdb[5] = (req->type == RAID_CHKTYPE_CHKRES) ? 0x01 : 0x00;

			RAID_DBG(2, ("raid_xyratex_lun_check_deconstructor: dev 0x%p: MDACIOCTL_CONSISTENCYCHECKSTART LUN id %d, type %d\n",
				     dev, st->logdev, req->type));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

			++st->bg_started_count;

			st->state = 2;
		}
		break;

	case 2:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_checkLunRspMsg_t));
			bzero(rsp, sizeof(raid_checkLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_checkLunRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		rsp = malloc(sizeof(raid_checkLunRspMsg_t));
		bzero(rsp, sizeof(raid_checkLunRspMsg_t));
		*rsp_msg = (raid_msgHdr_t *)rsp;
		*rsp_msg_len = sizeof(raid_checkLunRspMsg_t);
		raid_xyratex_lun_bg_op_aborter(dev, NULL);
		return(NFX_OK);
		
	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

static int32 
raid_xyratex_lun_bplist_constructor(raid_dev_t *		dev, 
				    raid_msgHdr_t *		msg_hdr, 
				    sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				    int32 *			sdm_relay_req_len)
{
	raid_xyratex_lun_bg_op_sm_t *st;
	sdm_relayCdbReqMsg_t *       req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_bplist_constructor: dev 0x%p\n", 
		     dev));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_BPLIST_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_lun_bg_op_sm_t));
	bzero(st, sizeof(raid_xyratex_lun_bg_op_sm_t));
		
	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_ControllerInfo_t);

        req_msg->cdb[0] = MDACMD_IOCTL;
        req_msg->cdb[2] = MDACIOCTL_GETCONTROLLERINFO;
        req_msg->cdb[7] = req_msg->dataInLen >> 8;
        req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_bplist_constructor: dev 0x%p: MDACIOCTL_GETCONTROLLERINFO\n",
		     dev));
	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_bplist_deconstructor(raid_dev_t *		dev, 
				      sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				      sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				      int32 *			sdm_relay_req_len,
				      raid_msgHdr_t **		rsp_msg, 
				      int32 *			rsp_msg_len)
{
	raid_xyratex_lun_bg_op_sm_t * st = dev->private_context;
	raid_bpListRspMsg_t *         rsp;
	FSI_ControllerInfo_t *	      ci;
	FSI_GetLogDevInfoValid_t *    di;
	sdm_relayCdbReqMsg_t *	      req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_bplist_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_bpListRspMsg_t));
			bzero(rsp, sizeof(raid_bpListRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_bpListRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		ci = (FSI_ControllerInfo_t *)sdm_relay_rsp->data;

		st->logdev_count = ci->nLDPresent[1] * 256 + ci->nLDPresent[0];

		st->msg_rsp_len = sizeof(raid_bpListRspMsg_t) + st->logdev_count * sizeof(raid_backProc_t);
		st->msg_rsp = (raid_bpListRspMsg_t *)malloc(st->msg_rsp_len);
		bzero(st->msg_rsp, st->msg_rsp_len);
		st->msg_rsp->numEntries = st->logdev_count;

		if (st->logdev_count == 0) {
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
		req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
		req_msg->cdb[4] = (st->logdev & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_bplist_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
			     dev, st->logdev));
		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;
		
	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_bpListRspMsg_t));
			bzero(rsp, sizeof(raid_bpListRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			*rsp_msg = (raid_msgHdr_t *)rsp; 
			*rsp_msg_len = sizeof(raid_bpListRspMsg_t);
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}

		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		st->logdev = di->LDN[1] * 256 + di->LDN[0];

		RAID_DBG(2, ("raid_xyratex_lun_bplist_deconstructor: "
			     "LDN 0x%x, LDControl 0x%x, OperationsStatus 0x%x\n", 
			     st->logdev, di->LDControl, di->OperationsStatus));

		st->msg_rsp->bpList[st->logdev_index].id = st->logdev;
		st->msg_rsp->bpList[st->logdev_index].checkProgress = -1;
		st->msg_rsp->bpList[st->logdev_index].initProgress = -1;
		st->msg_rsp->bpList[st->logdev_index].rebuildProgress = -1;
		st->msg_rsp->bpList[st->logdev_index].expandProgress = -1;
		
		if (di->OperationsStatus & FSI_OPSTATUS_ACTIVE_MASK) {
			uint32 devsize;
			uint32 blockno;

			memcpy(&devsize, di->DeviceSize, 4);
			
			if (di->OperationsStatus & FSI_OPSTATUS_CCHECKACTIVE) {
				memcpy(&blockno, di->ConsistencyCheckBlockNumber, 4);
				st->msg_rsp->bpList[st->logdev_index].checkProgress = 
					blockno / (devsize / 100);
			}
			if (di->OperationsStatus & FSI_OPSTATUS_INITACTIVE) {
				memcpy(&blockno, di->LDInitializationBlockNumber, 4);
				st->msg_rsp->bpList[st->logdev_index].initProgress = 
					blockno / (devsize / 100);
			}
			if (di->OperationsStatus & FSI_OPSTATUS_REBUILDACTIVE) {
				memcpy(&blockno, di->RebuildBlockNumber, 4);
				st->msg_rsp->bpList[st->logdev_index].rebuildProgress = 
					blockno / (devsize / 100);
			}
			if (di->OperationsStatus & FSI_OPSTATUS_MIGRATIONACTIVE) {
				memcpy(&blockno, di->DataMigrationBlockNumber, 4);
				st->msg_rsp->bpList[st->logdev_index].expandProgress = 
					blockno / (devsize / 100);
			}
		}

		++st->logdev_index;
		if (st->logdev_index < st->logdev_count) {

			++st->logdev;

			req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
			VERIFY(req_msg);
			bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

			req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
			req_msg->hdr.chandle = (uint32)dev;
			req_msg->handle = dev->open_handle;
			req_msg->retryCnt = 1;
			req_msg->timeOutCnt = 60;
			req_msg->maxTimeToLive = 60;
			req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
			
			req_msg->cdb[0] = MDACMD_IOCTL;
			req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
			req_msg->cdb[3] = (st->logdev & 0xFF00) >> 8;
			req_msg->cdb[4] = (st->logdev & 0x00FF);
			req_msg->cdb[7] = req_msg->dataInLen >> 8;
			req_msg->cdb[8] = req_msg->dataInLen & 0xFF;
			
			RAID_DBG(2, ("raid_xyratex_lun_check_deconstructor: dev 0x%p: "
				     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
				     dev, st->logdev));
			*sdm_relay_req = req_msg;
			*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);
			
			st->state = 1;
		}
		else {
			*rsp_msg = (raid_msgHdr_t *)st->msg_rsp;
			*rsp_msg_len = st->msg_rsp_len;
			st->msg_rsp = NULL;
			raid_xyratex_lun_bg_op_aborter(dev, NULL);
			return(NFX_OK);
		}
		break;
		
	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

typedef struct {
	int32				state;
} raid_xyratex_lun_cache_sm_t;

static int32 
raid_xyratex_lun_cache_aborter(raid_dev_t *		dev, 
			       sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_lun_cache_sm_t *st = dev->private_context;	

	RAID_DBG(2, ("raid_xyratex_lun_cache_aborter: dev 0x%p\n", dev));

	if (st) {
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_cache_constructor(raid_dev_t *			dev, 
				   raid_msgHdr_t *		msg_hdr, 
				   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				   int32 *			sdm_relay_req_len)
{
	raid_xyratex_lun_cache_sm_t *st;
	sdm_relayCdbReqMsg_t *       req_msg;

	RAID_DBG(2, ("raid_xyratex_lun_cache_constructor: dev 0x%p\n", 
		     dev));

	VERIFY(msg_hdr->mid == RAID_MID_LUN_CACHE_REQ);

	st = dev->private_context = malloc(sizeof(raid_xyratex_lun_cache_sm_t));
	bzero(st, sizeof(raid_xyratex_lun_cache_sm_t));
		
	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_GetLogDevInfoValid_t);
	
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETLOGDEVINFOVALID;
	req_msg->cdb[3] = (((raid_cacheLunReqMsg_t *)msg_hdr)->id & 0xFF00) >> 8;
	req_msg->cdb[4] = (((raid_cacheLunReqMsg_t *)msg_hdr)->id & 0x00FF);
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	RAID_DBG(2, ("raid_xyratex_lun_cache_constructor: dev 0x%p: "
		     "MDACIOCTL_GETLOGDEVINFOVALID logdev 0x%x\n",
		     dev, ((raid_cacheLunReqMsg_t *)msg_hdr)->id));

	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}

static int32 
raid_xyratex_lun_cache_deconstructor(raid_dev_t *		dev, 
				     sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				     int32 *			sdm_relay_req_len,
				     raid_msgHdr_t **		rsp_msg, 
				     int32 *			rsp_msg_len)
{
	raid_xyratex_lun_cache_sm_t * st = dev->private_context;
	raid_cacheLunReqMsg_t *       lun_cache_req_msg = (raid_cacheLunReqMsg_t *)dev->message_req_context->req_msg;
	FSI_GetLogDevInfoValid_t *    di;
	FSI_LogDevParms_t *	      dp;
	raid_cacheLunRspMsg_t *       rsp;
	sdm_relayCdbReqMsg_t *	      req_msg;
	uint32			      req_msg_len;

	RAID_DBG(2, ("raid_xyratex_lun_cache_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_cacheLunRspMsg_t));
			bzero(rsp, sizeof(raid_cacheLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_lun_cache_aborter(dev, NULL);
			*rsp_msg_len = sizeof(raid_cacheLunRspMsg_t);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}
		di = (FSI_GetLogDevInfoValid_t *)sdm_relay_rsp->data;

		/* Did we get the requested logical device ? */
		if (lun_cache_req_msg->id != di->LDN[1] * 256 + di->LDN[0]) {
			rsp = malloc(sizeof(raid_cacheLunRspMsg_t));
			bzero(rsp, sizeof(raid_cacheLunRspMsg_t));
			rsp->rCode = RAID_RCODE_INVALID_LUN;
			raid_xyratex_lun_cache_aborter(dev, NULL);
			*rsp_msg_len = sizeof(raid_cacheLunRspMsg_t);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
		VERIFY(req_msg);
		bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataInLen = sizeof(FSI_LogDevParms_t);

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_GETLOGDEVPARAMETER;
		req_msg->cdb[3] = (lun_cache_req_msg->id & 0xFF00) >> 8;
		req_msg->cdb[4] = (lun_cache_req_msg->id & 0x00FF);
		req_msg->cdb[7] = req_msg->dataInLen >> 8;
		req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

		RAID_DBG(2, ("raid_xyratex_lun_cache_deconstructor: dev 0x%p: "
			     "MDACIOCTL_GETLOGDEVPARAMETER, logdev 0x%x\n", 
			     dev, lun_cache_req_msg->id));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

		st->state = 1;
		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_cacheLunRspMsg_t));
			bzero(rsp, sizeof(raid_cacheLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_lun_cache_aborter(dev, NULL);
			*rsp_msg_len = sizeof(raid_cacheLunRspMsg_t);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}

		req_msg_len = sizeof(sdm_relayCdbReqMsg_t) + sizeof(FSI_LogDevParms_t);
		req_msg = (sdm_relayCdbReqMsg_t *)malloc(req_msg_len);
		VERIFY(req_msg);
		bzero(req_msg, req_msg_len);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;
		req_msg->dataLen = sizeof(FSI_LogDevParms_t);
	
		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_SETLOGDEVPARAMETER;
		req_msg->cdb[3] = (lun_cache_req_msg->id & 0xFF00) >> 8;
		req_msg->cdb[4] = (lun_cache_req_msg->id & 0x00FF);
		req_msg->cdb[7] = req_msg->dataLen >> 8;
		req_msg->cdb[8] = req_msg->dataLen & 0xFF;

		memcpy(req_msg->data, sdm_relay_rsp->data, sizeof(FSI_LogDevParms_t));

		dp = (FSI_LogDevParms_t *)req_msg->data;

		dp->rw_control &= ~FSI_RWCONTROL_READ_CACHE_MASK;
		if (lun_cache_req_msg->rc_enabled)
			dp->rw_control |= FSI_RWCONTROL_READ_CACHE_ENABLED;
		else
			dp->rw_control |= FSI_RWCONTROL_READ_CACHE_DISABLED;
			
		dp->rw_control &= ~FSI_RWCONTROL_WRITE_CACHE_MASK;
		if (lun_cache_req_msg->wc_enabled)
			dp->rw_control |= FSI_RWCONTROL_WRITE_CACHE_ENABLED;
		else
			dp->rw_control |= FSI_RWCONTROL_WRITE_CACHE_DISABLED;

		RAID_DBG(2, ("raid_xyratex_lun_cache_deconstructor: dev 0x%p: "
			     "MDACIOCTL_SETLOGDEVPARAMETER logdev 0x%x\n",
			     dev, lun_cache_req_msg->id));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = req_msg_len;
		
		st->state = 2;
		break;

	case 2:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_cacheLunRspMsg_t));
			bzero(rsp, sizeof(raid_cacheLunRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_lun_cache_aborter(dev, NULL);
			*rsp_msg_len = sizeof(raid_cacheLunRspMsg_t);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}

		rsp = malloc(sizeof(raid_cacheLunRspMsg_t));
		bzero(rsp, sizeof(raid_cacheLunRspMsg_t));
		raid_xyratex_lun_cache_aborter(dev, NULL);
		*rsp_msg_len = sizeof(raid_cacheLunRspMsg_t);
		*rsp_msg = (raid_msgHdr_t *)rsp;
		break;
		
	default:
		VERIFY(0);
	}
	
	return(NFX_OK);
}

typedef struct {
	int32	state;
} raid_xyratex_phy_passthru_sm_t;

static int32 
raid_xyratex_phy_passthru_aborter(raid_dev_t *		dev, 
			       sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_phy_passthru_sm_t *st = (raid_xyratex_phy_passthru_sm_t *)dev->private_context;	

	RAID_DBG(2, ("raid_xyratex_phy_passthru_aborter: dev 0x%p\n", dev));

	if (st) {
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_phy_passthru_constructor(raid_dev_t *		dev, 
				   raid_msgHdr_t *		msg_hdr, 
				   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				   int32 *			sdm_relay_req_len)
{
	raid_xyratex_phy_passthru_sm_t	*st;
	sdm_relayCdbReqMsg_t		*req_msg;
	raid_phyPassThruReqMsg_t	*raid_msg = (raid_phyPassThruReqMsg_t *)msg_hdr;
	int			 	mlen;
	int			 	rc;

	RAID_DBG(2, ("raid_xyratex_phy_passthru_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_PHY_PASSTHRU_REQ);

	// create state 
	st = dev->private_context = malloc(sizeof(raid_xyratex_phy_passthru_sm_t));
	VERIFY(st);
	bzero(st, sizeof(raid_xyratex_phy_passthru_sm_t));

	// allocate memory for relay req.
	mlen = sizeof(sdm_relayCdbReqMsg_t) + RAID_PASSTHRU_CDB_ALLOC_LEN - 1;
	req_msg = malloc(mlen);
	VERIFY(req_msg);
	bzero(req_msg, mlen);

	// set up relay cmd
	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;

	// set up the pass through command cdb
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_PASS_THRU_CDB;
	req_msg->cdb[3] = (req_msg->hdr.chandle >> 24) & 0xFF;
	req_msg->cdb[4] = (req_msg->hdr.chandle >> 16) & 0xFF;
	req_msg->cdb[5] = (req_msg->hdr.chandle >> 8) & 0xFF;
	req_msg->cdb[6] = req_msg->hdr.chandle & 0xFF;
	req_msg->cdb[7] = (RAID_PASSTHRU_CDB_ALLOC_LEN >> 8)  & 0xFF;
	req_msg->cdb[8] = RAID_PASSTHRU_CDB_ALLOC_LEN  & 0xFF;

	// fill in the CDB for the actual command
	rc = raid_xyratex_setup_cmd_cdb(raid_msg, req_msg);
	if(rc) return(RAID_RCODE_INVALID_PARAMETERS);

	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = mlen;

	return(NFX_OK);
}

static int32 
raid_xyratex_phy_passthru_deconstructor(raid_dev_t *		dev, 
				     sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				     int32 *			sdm_relay_req_len,
				     raid_msgHdr_t **		rsp_msg, 
				     int32 *			rsp_msg_len)
{
	raid_xyratex_phy_passthru_sm_t	*st = (raid_xyratex_phy_passthru_sm_t *)dev->private_context;
	raid_phyPassThruReqMsg_t	*raid_msg = (raid_phyPassThruReqMsg_t *)dev->message_req_context->req_msg;
	raid_phyPassThruRspMsg_t	*rsp;
	sdm_relayCdbReqMsg_t		*req_msg;
	int				mlen;

	RAID_DBG(2, ("raid_xyratex_phy_passthru_deconstructor: dev 0x%p, state %d\n", dev, st->state));

	switch (st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_phyPassThruRspMsg_t));
			bzero(rsp, sizeof(raid_phyPassThruRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_phy_passthru_aborter(dev, NULL);
			*rsp_msg_len = sizeof(raid_phyPassThruRspMsg_t);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}

		// set up the pass thru initiate command
		if(raid_msg->dataOut)
			mlen = sizeof(sdm_relayCdbReqMsg_t) + raid_msg->dataLen;
		else
			mlen = sizeof(sdm_relayCdbReqMsg_t);

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(mlen);
		VERIFY(req_msg);
		bzero(req_msg, mlen);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_PASS_THRU_INITIATE;
		req_msg->cdb[3] = (req_msg->hdr.chandle >> 24) & 0xFF;
		req_msg->cdb[4] = (req_msg->hdr.chandle >> 16) & 0xFF;
		req_msg->cdb[5] = (req_msg->hdr.chandle >> 8) & 0xFF;
		req_msg->cdb[6] = req_msg->hdr.chandle & 0xFF;
		req_msg->cdb[7] = (raid_msg->dataLen >> 8) & 0xFF;
		req_msg->cdb[8] = raid_msg->dataLen & 0xFF;

		if(raid_msg->dataOut) {
			req_msg->dataLen = raid_msg->dataLen;
			memcpy((void *)req_msg->data, raid_msg->data, raid_msg->dataLen);
		}
		else
			req_msg->dataInLen = raid_msg->dataLen;

		RAID_DBG(2, ("raid_xyratex_phy_passthru_deconstructor: dev 0x%p: "
			     "MDACIOCTL_PASSTHRU_INITIATE cmd : %d\n", 
			     dev, raid_msg->cmd));

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = mlen;

		st->state = 1;

		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rsp = malloc(sizeof(raid_phyPassThruRspMsg_t));
			bzero(rsp, sizeof(raid_phyPassThruRspMsg_t));
			rsp->rCode = RAID_RCODE_IO_ERROR;
			raid_xyratex_phy_passthru_aborter(dev, NULL);
			*rsp_msg_len = sizeof(raid_phyPassThruRspMsg_t);
			*rsp_msg = (raid_msgHdr_t *)rsp;
			return(NFX_OK);
		}

		*rsp_msg_len = sizeof(raid_phyPassThruRspMsg_t) + sdm_relay_rsp->dataLen - 1;
		rsp = (raid_phyPassThruRspMsg_t *)malloc((*rsp_msg_len));
		VERIFY(rsp);
		bzero(rsp, (*rsp_msg_len));

		rsp->rCode = RAID_RCODE_SUCCESS;
		rsp->dataLen = sdm_relay_rsp->dataLen;
		if(rsp->dataLen)
			memcpy((void *)rsp->data, sdm_relay_rsp->data, sdm_relay_rsp->dataLen);

		*rsp_msg = (raid_msgHdr_t *)rsp;

		raid_xyratex_phy_passthru_aborter(dev, NULL);

		break;
	default:
		VERIFY(0);
	}

	return(NFX_OK);
}

#define RAID_PASSTHRU_CHUNK_SIZE		0x4000

typedef struct {
	int32	state;
	uint32	dataLen;
	uint32	dataOffset;
	uchar8	bufferId;
	uchar8	diskVendor;
	FILE	*fp;
} raid_xyratex_upd_drv_fw_sm_t;

static int32
raid_xyratex_setup_upd_drv_fw_relay(raid_dev_t *dev,
									raid_updateDrvFwReqMsg_t *raid_msg,
									sdm_relayCdbReqMsg_t **sdm_relay_req,
									int32 *sdm_relay_req_len, 
									raid_xyratex_upd_drv_fw_sm_t *st)
{
	sdm_relayCdbReqMsg_t		*req_msg;
	int			 	mlen;
	uint32			dataOut;

	RAID_DBG(2, ("raid_xyratex_setup_upd_drv_fw_relay: dataOffset %d\n",
				st->dataOffset));	

	// allocate memory for relay req.
	mlen = sizeof(sdm_relayCdbReqMsg_t) + RAID_PASSTHRU_CDB_ALLOC_LEN - 1;
	req_msg = malloc(mlen);
	VERIFY(req_msg);
	bzero(req_msg, mlen);

	dataOut = st->dataLen - st->dataOffset;
	dataOut = (dataOut > RAID_PASSTHRU_CHUNK_SIZE) ? RAID_PASSTHRU_CHUNK_SIZE : dataOut;

	// set up relay cmd
	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;

	// set up the pass through command cdb
	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_PASS_THRU_CDB;
	req_msg->cdb[3] = (req_msg->hdr.chandle >> 24) & 0xFF;
	req_msg->cdb[4] = (req_msg->hdr.chandle >> 16) & 0xFF;
	req_msg->cdb[5] = (req_msg->hdr.chandle >> 8) & 0xFF;
	req_msg->cdb[6] = req_msg->hdr.chandle & 0xFF;
	req_msg->cdb[7] = (RAID_PASSTHRU_CDB_ALLOC_LEN >> 8)  & 0xFF;
	req_msg->cdb[8] = RAID_PASSTHRU_CDB_ALLOC_LEN  & 0xFF;

	req_msg->dataLen = RAID_PASSTHRU_CDB_ALLOC_LEN;
	req_msg->data[0] = raid_msg->channelNumber;
	req_msg->data[1] = raid_msg->targetId;
	req_msg->data[2] = 10;		// 10 byte CDB
	req_msg->data[3] = 2;		// Controller -> Drive
	req_msg->data[4] = 1;		// allocation len type (1 == bytes)

	req_msg->data[16] = 0x3B;	// Write Buffer

	switch(st->diskVendor) {
		case DISK_VENDOR_SEAGATE:
			req_msg->data[17] = 0x07;	// Download and Save
			req_msg->data[18] = st->bufferId;	// Buffer ID
			req_msg->data[19] = (st->dataOffset >> 16) & 0xFF;	// bytes already sent
			req_msg->data[20] = (st->dataOffset >> 8) & 0xFF;
			req_msg->data[21] = st->dataOffset & 0xFF;
			req_msg->data[22] = 0;
			break;

		case DISK_VENDOR_IBM:
			req_msg->data[17] = 0x05;	// Download and Save
			req_msg->data[18] = st->bufferId;	// Buffer ID
			break;

		case DISK_VENDOR_HITACHI:
			req_msg->data[17] = 0x05;	// Download and Save
			break;

		default:
			free(req_msg);
			return RAID_RCODE_DISK_UPGRADE_NOTSUPPORTED;
	}
	req_msg->data[23] = (dataOut >> 8) & 0xFF;
	req_msg->data[24] = dataOut & 0xFF;

	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = mlen;

	return (NFX_OK);
}

static int32 
raid_xyratex_upd_drv_fw_aborter(raid_dev_t *		dev, 
			       sdm_relayCdbRspMsg_t *	sdm_relay_rsp)
{
	raid_xyratex_upd_drv_fw_sm_t *st = (raid_xyratex_upd_drv_fw_sm_t *)dev->private_context;	

	RAID_DBG(2, ("raid_xyratex_phy_passthru_aborter: dev 0x%p\n", dev));

	if (st) {
		if(st->fp) fclose(st->fp);
		free(st);
		dev->private_context = NULL;
	}

	return(NFX_OK);
}

static int32 
raid_xyratex_upd_drv_fw_constructor(raid_dev_t *		dev, 
				   raid_msgHdr_t *		msg_hdr, 
				   sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				   int32 *			sdm_relay_req_len)
{
	raid_xyratex_upd_drv_fw_sm_t	*st;
	raid_updateDrvFwReqMsg_t		*raid_msg = (raid_updateDrvFwReqMsg_t *)msg_hdr;
	struct stat						statBuf;
	sdm_relayCdbReqMsg_t			*req_msg;

	RAID_DBG(2, ("raid_xyratex_upd_drv_fw_constructor: dev 0x%p\n", dev));

	VERIFY(msg_hdr->mid == RAID_MID_UPDATE_DRV_FW_REQ);

	if(stat(raid_msg->filePath, &statBuf) != 0) 
		return(RAID_RCODE_CANNOT_OPEN_FILE);

	// create state 
	st = dev->private_context = malloc(sizeof(raid_xyratex_upd_drv_fw_sm_t));
	VERIFY(st);
	bzero(st, sizeof(raid_xyratex_upd_drv_fw_sm_t));
	st->dataLen = statBuf.st_size;

	if((st->fp = fopen(raid_msg->filePath, "rb")) == NULL) {
		free(st);
		return(RAID_RCODE_CANNOT_OPEN_FILE);
	}

	req_msg = (sdm_relayCdbReqMsg_t *)malloc(sizeof(sdm_relayCdbReqMsg_t));
	VERIFY(req_msg);
	bzero(req_msg, sizeof(sdm_relayCdbReqMsg_t));

	req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
	req_msg->hdr.chandle = (uint32)dev;
	req_msg->handle = dev->open_handle;
	req_msg->retryCnt = 1;
	req_msg->timeOutCnt = 60;
	req_msg->maxTimeToLive = 60;
	req_msg->dataInLen = sizeof(FSI_GetPhysDevInfoValid_t);	

	req_msg->cdb[0] = MDACMD_IOCTL;
	req_msg->cdb[2] = MDACIOCTL_GETPHYSDEVINFOVALID;
	req_msg->cdb[3] = raid_msg->channelNumber;
	req_msg->cdb[4] = raid_msg->targetId;
	req_msg->cdb[7] = req_msg->dataInLen >> 8;
	req_msg->cdb[8] = req_msg->dataInLen & 0xFF;

	*sdm_relay_req = req_msg;
	*sdm_relay_req_len = sizeof(sdm_relayCdbReqMsg_t);

	return(NFX_OK);
}


static int32 
raid_xyratex_upd_drv_fw_deconstructor(raid_dev_t *		dev, 
				     sdm_relayCdbRspMsg_t *	sdm_relay_rsp, 
				     sdm_relayCdbReqMsg_t **	sdm_relay_req, 
				     int32 *			sdm_relay_req_len,
				     raid_msgHdr_t **		rsp_msg, 
				     int32 *			rsp_msg_len)
{
	raid_xyratex_upd_drv_fw_sm_t	*st = (raid_xyratex_upd_drv_fw_sm_t *)dev->private_context;
	raid_updateDrvFwReqMsg_t	*raid_msg = (raid_updateDrvFwReqMsg_t *)dev->message_req_context->req_msg;
	raid_updateDrvFwRspMsg_t	*rsp;
	sdm_relayCdbReqMsg_t		*req_msg;
	int							mlen;
	uint32						dataOut;
	FSI_GetPhysDevInfoValid_t 	*di;
	int							rc = 0;
	char						vendor[32];

	
	RAID_DBG(2, ("raid_xyratex_upd_drv_fw_deconstructor: dev 0x%p, state %d\n", dev, st->state));
	
	switch(st->state) {
	case 0:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rc = RAID_RCODE_IO_ERROR;
			goto updateDrvFw_error;
		}
		di = (FSI_GetPhysDevInfoValid_t *)sdm_relay_rsp->data;

		// check if we got the correct physical device
		if(di->ch != raid_msg->channelNumber || di->id != raid_msg->targetId) {
			rc = RAID_RCODE_INVALID_PHYSICAL_DEVICE;
			goto updateDrvFw_error;
		}
		
		strncpy(vendor, &di->InquiryData[8], 8);
		if(!strncmp(vendor, "SEAGATE", 7))
			st->diskVendor = DISK_VENDOR_SEAGATE;
		else if(!strncmp(vendor, "IBM", 3))
			st->diskVendor = DISK_VENDOR_IBM;
		else if(!strncmp(vendor, "HITACHI", 7))
			st->diskVendor = DISK_VENDOR_HITACHI;
		else
			st->diskVendor = DISK_VENDOR_UNKNOWN;
						
		rc = raid_xyratex_setup_upd_drv_fw_relay(dev, raid_msg, sdm_relay_req, sdm_relay_req_len, st);
		if(rc) 
			goto updateDrvFw_error;

		st->state = 1;

		break;

	case 1:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rc = RAID_RCODE_IO_ERROR;
			goto updateDrvFw_error;
		}

		// set up the pass thru initiate command
		dataOut = st->dataLen - st->dataOffset;
		dataOut = (dataOut > RAID_PASSTHRU_CHUNK_SIZE) ? RAID_PASSTHRU_CHUNK_SIZE : dataOut;
		mlen = sizeof(sdm_relayCdbReqMsg_t) + dataOut;

		req_msg = (sdm_relayCdbReqMsg_t *)malloc(mlen);
		VERIFY(req_msg);
		bzero(req_msg, mlen);

		req_msg->hdr.mid = SDM_MID_RELAY_CDB_REQ;
		req_msg->hdr.chandle = (uint32)dev;
		req_msg->handle = dev->open_handle;
		req_msg->retryCnt = 1;
		req_msg->timeOutCnt = 60;
		req_msg->maxTimeToLive = 60;

		req_msg->cdb[0] = MDACMD_IOCTL;
		req_msg->cdb[2] = MDACIOCTL_PASS_THRU_INITIATE;
		req_msg->cdb[3] = (req_msg->hdr.chandle >> 24) & 0xFF;
		req_msg->cdb[4] = (req_msg->hdr.chandle >> 16) & 0xFF;
		req_msg->cdb[5] = (req_msg->hdr.chandle >> 8) & 0xFF;
		req_msg->cdb[6] = req_msg->hdr.chandle & 0xFF;
		req_msg->cdb[7] = (dataOut >> 8) & 0xFF;
		req_msg->cdb[8] = dataOut & 0xFF;

		req_msg->dataLen = dataOut;
		fread((void *)req_msg->data, 1, dataOut, st->fp);

		RAID_DBG(2, ("raid_xyratex_upd_drv_fw_deconstructor: dev 0x%p: "
			     "MDACIOCTL_PASSTHRU_INITIATE dataOffset %d totalLen %d\n", 
			     dev, st->dataOffset, st->dataLen));

		st->dataOffset += dataOut;
		st->bufferId++;

		*sdm_relay_req = req_msg;
		*sdm_relay_req_len = mlen;

		if(st->dataOffset < st->dataLen)
			st->state = 2;
		else
			st->state = 3;
		
		break;

	case 2:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			rc = RAID_RCODE_IO_ERROR;
			goto updateDrvFw_error;
		}

		raid_xyratex_setup_upd_drv_fw_relay(dev, raid_msg, sdm_relay_req, sdm_relay_req_len, st);

		st->state = 1;

		break;
		
	case 3:
		if ((sdm_relay_rsp->status & SD_DEV_STATUS_MASK) == SCSI_DEV_STATUS_CHECK) {
			scsi_sense_t	*sense = (scsi_sense_t *)sdm_relay_rsp->senseData;
			if(!(sense->asc == 0x80 && sense->ascQ == 0xb)) {
				rc = RAID_RCODE_IO_ERROR;
updateDrvFw_error:
				rsp = malloc(sizeof(raid_updateDrvFwRspMsg_t));
				bzero(rsp, sizeof(raid_updateDrvFwRspMsg_t));
				rsp->rCode = rc;
				raid_xyratex_upd_drv_fw_aborter(dev, NULL);
				*rsp_msg_len = sizeof(raid_updateDrvFwRspMsg_t);
				*rsp_msg = (raid_msgHdr_t *)rsp;
				return(NFX_OK);
			}
		}

		*rsp_msg_len = sizeof(raid_updateDrvFwRspMsg_t);
		rsp = (raid_updateDrvFwRspMsg_t *)malloc(*rsp_msg_len);
		VERIFY(rsp);
		bzero(rsp, (*rsp_msg_len));

		rsp->rCode = RAID_RCODE_SUCCESS;
		*rsp_msg = (raid_msgHdr_t *)rsp;

		raid_xyratex_upd_drv_fw_aborter(dev, NULL);

		break;

	default:
		VERIFY(0);
	}
	
	return (NFX_OK);
}

raid_cmd_transformer_t xyratex_transformer[] = {
	{
		RAID_MID_CTRL_RESET_REQ, RAID_MID_CTRL_RESET_RSP,
		raid_xyratex_ctrl_reset_constructor,
		raid_xyratex_ctrl_reset_deconstructor,
		NULL
	},
	{
		RAID_MID_CTRL_FLUCACHE_REQ, RAID_MID_CTRL_FLUCACHE_RSP,
		raid_xyratex_ctrl_flush_constructor,
		raid_xyratex_ctrl_flush_deconstructor,
		NULL
	},
	{
		RAID_MID_CTRL_GETSTATE_REQ, RAID_MID_CTRL_GETSTATE_RSP,
		raid_xyratex_get_ctrl_state_constructor,
		raid_xyratex_get_ctrl_state_deconstructor,
		NULL
	},
	{
		RAID_MID_CTRL_GETPARAM_REQ, RAID_MID_CTRL_GETPARAM_RSP,
		raid_xyratex_get_ctrl_param_constructor,
		raid_xyratex_get_ctrl_param_deconstructor,
		NULL
	},
	{
		RAID_MID_CTRL_SETPARAM_REQ, RAID_MID_CTRL_SETPARAM_RSP,
		raid_xyratex_set_ctrl_param_constructor,
		raid_xyratex_set_ctrl_param_deconstructor,
		raid_xyratex_set_ctrl_param_aborter
	},
	{
		RAID_MID_CTRL_INFO_REQ, RAID_MID_CTRL_INFO_RSP,
		raid_xyratex_get_ctrl_info_constructor,
		raid_xyratex_get_ctrl_info_deconstructor,
		NULL
	},
	{
		RAID_MID_PHYDEV_LIST_REQ, RAID_MID_PHYDEV_LIST_RSP,
		raid_xyratex_get_physdev_list_constructor,
		raid_xyratex_get_physdev_list_deconstructor,
		raid_xyratex_get_physdev_list_aborter
	},
	{
		RAID_MID_PHYDEV_INFO_REQ, RAID_MID_PHYDEV_INFO_RSP,
		raid_xyratex_get_physdev_info_constructor,
		raid_xyratex_get_physdev_info_deconstructor,
		raid_xyratex_get_physdev_info_aborter
	},
	{
		RAID_MID_PHYDEV_STATS_REQ, RAID_MID_PHYDEV_STATS_RSP,
		raid_xyratex_get_physdev_stats_constructor,
		raid_xyratex_get_physdev_stats_deconstructor,
		raid_xyratex_get_physdev_stats_aborter
	},
	{
		RAID_MID_CTRL_CFGDATA_REQ, RAID_MID_CTRL_CFGDATA_RSP,
		raid_xyratex_ctrl_config_constructor,
		raid_xyratex_ctrl_config_deconstructor,
		raid_xyratex_ctrl_config_aborter
	},		
	{
		RAID_MID_CTRL_CLRCFG_REQ, RAID_MID_CTRL_CLRCFG_RSP,
		raid_xyratex_config_clear_constructor,
		raid_xyratex_config_clear_deconstructor,
		raid_xyratex_config_clear_aborter
	},		
	{
		RAID_MID_LOGDEV_LIST_REQ, RAID_MID_LOGDEV_LIST_RSP,
		raid_xyratex_get_logdev_list_constructor,
		raid_xyratex_get_logdev_list_deconstructor,
		raid_xyratex_get_logdev_list_aborter
	},
	{
		RAID_MID_LOGDEV_INFO_REQ, RAID_MID_LOGDEV_INFO_RSP,
		raid_xyratex_get_logdev_info_constructor,
		raid_xyratex_get_logdev_info_deconstructor,
		raid_xyratex_get_logdev_info_aborter
	},
	{
		RAID_MID_LUN_CREATE_SIMPLE_REQ, RAID_MID_LUN_CREATE_SIMPLE_RSP,
		raid_xyratex_lun_create_simple_constructor,
		raid_xyratex_lun_create_simple_deconstructor,
		raid_xyratex_lun_create_aborter
	},
	{
		RAID_MID_LUN_CREATE_REQ, RAID_MID_LUN_CREATE_RSP,
		raid_xyratex_lun_create_constructor,
		raid_xyratex_lun_create_deconstructor,
		raid_xyratex_lun_create_aborter
	},
	{
		RAID_MID_LOGDEV_PHYLIST_REQ, RAID_MID_LOGDEV_PHYLIST_RSP,
		raid_xyratex_lun_phylist_constructor,
		raid_xyratex_lun_phylist_deconstructor,
		NULL
	},
	{
		RAID_MID_LUN_DELETE_REQ, RAID_MID_LUN_DELETE_RSP,
		raid_xyratex_lun_delete_constructor,
		raid_xyratex_lun_delete_deconstructor,
		raid_xyratex_lun_delete_aborter
	},
	{
		RAID_MID_LUN_INIT_REQ, RAID_MID_LUN_INIT_RSP,
		raid_xyratex_lun_init_constructor,
		raid_xyratex_lun_init_deconstructor,
		raid_xyratex_lun_bg_op_aborter
	},
	{
		RAID_MID_LUN_CHECK_REQ, RAID_MID_LUN_CHECK_RSP,
		raid_xyratex_lun_check_constructor,
		raid_xyratex_lun_check_deconstructor,
		raid_xyratex_lun_bg_op_aborter
	},
	{
		RAID_MID_LUN_BPLIST_REQ, RAID_MID_LUN_BPLIST_RSP,
		raid_xyratex_lun_bplist_constructor,
		raid_xyratex_lun_bplist_deconstructor,
		raid_xyratex_lun_bg_op_aborter
	},
	{
		RAID_MID_LUN_CACHE_REQ, RAID_MID_LUN_CACHE_RSP,
		raid_xyratex_lun_cache_constructor,
		raid_xyratex_lun_cache_deconstructor,
		raid_xyratex_lun_cache_aborter
	},
	{
		RAID_MID_PHY_PASSTHRU_REQ, RAID_MID_PHY_PASSTHRU_RSP,
		raid_xyratex_phy_passthru_constructor,
		raid_xyratex_phy_passthru_deconstructor,
		raid_xyratex_phy_passthru_aborter
	},
	{
		RAID_MID_UPDATE_DRV_FW_REQ, RAID_MID_UPDATE_DRV_FW_RSP,
		raid_xyratex_upd_drv_fw_constructor,
		raid_xyratex_upd_drv_fw_deconstructor,
		raid_xyratex_upd_drv_fw_aborter
	},
	{
		0, 0,
		NULL, 
		NULL,
		NULL
	}
};
	
	
char *
FSI_HardwareIDToString(byte controllerType)
{
	switch (controllerType) {
	case    FSI_CT_DAC960FL:	return "DAC960FL";
	case    FSI_CT_DAC960FF:	return "DAC960FF";
	case    FSI_CT_METEOR_FL:	return "Meteor_FL";
	case    FSI_CT_METEOR_FF:	return "Meteor_FF";
	case    FSI_CT_FFx:		return "FFx";
	case    FSI_CT_FFx2:		return "FFx2";
	case    FSI_CT_FF2:		return "FF2";
	default:			return "Unknown";
	}
}

char *
FSI_devTypeToString(byte dev_type)
{
	switch (dev_type)
	{
	case 0x0:		return("RAID-0 (striping)");
	case 0x1:		return("RAID-1 (mirroring)");
	case 0x3:		return("RAID-3 Right asymmetric parity");
	case 0x5:		return("RAID-5 Right asymmetric parity");
	case 0x6:		return("RAID-0+1");
	case 0x7:		return("JBOD");
	case 0x8:		return("New Mylex SPAN");
	case 0x9:		return("RAID-3 (Fixed parity)");
	case 0xb:		return("RAID-3 Left asymmetric parity");
	case 0xc:		return("Spanned device");
	case 0xd:		return("RAID-5 Left asymmetric parity");
	case 0xe:		return("Concatentation");
	case 0xf:		return("Physical device");
	default:		return("unknown");
	} 
}

char *
FSI_cachelineSizeToString(byte cacheline_size)
{
	switch (cacheline_size)
	{
	case 0:		return("none");
	case 1:		return("0.5");
	case 2:		return("1");
	case 3:		return("2");
	case 4:		return("4");
	case 5:		return("8");
	case 6:		return("16");
	case 7:		return("32");
	case 8:		return("64");
	default:	return("unknown");
	} 
}

uint32
FSI_cachelineSizeToKB(byte cacheline_size)
{
	switch (cacheline_size)
	{
	case 0:		return(0);
	case 1:		return(0);
	case 2:		return(1);
	case 3:		return(2);
	case 4:		return(4);
	case 5:		return(8);
	case 6:		return(16);
	case 7:		return(32);
	case 8:		return(64);
	default:	return(-1);
	} 
}


char *
FSI_stripeSizeToString(byte stripe_size)
{
	switch (stripe_size)
	{
	case 0:		return("none");
	case 1:		return("0.5");
	case 2:		return("1");
	case 3:		return("2");
	case 4:		return("4");
	case 5:		return("8");
	case 6:		return("16");
	case 7:		return("32");
	case 8:		return("64");
	default:	return("unknown");
	} 
}

uint32
FSI_stripeSizeToKB(byte stripe_size)
{
	switch (stripe_size)
	{
	case 0:		return(0);
	case 1:		return(0);
	case 2:		return(1);
	case 3:		return(2);
	case 4:		return(4);
	case 5:		return(8);
	case 6:		return(16);
	case 7:		return(32);
	case 8:		return(64);
	default:	return(-1);
	} 
}

uint32
FSI_KBTostripeSize(int32 KB)
{
	if (KB >= 0 && KB < 1)
		return(1);
	else
	if (KB >= 1 && KB < 2)
		return(2);
	else
	if (KB >= 2 && KB < 4)
		return(3);
	else
	if (KB >= 4 && KB < 8)
		return(4);
	else
	if (KB >= 8 && KB < 16)
		return(5);
	else
	if (KB >= 16 && KB < 32)
		return(6);
	else
	if (KB >= 32 && KB < 64)
		return(7);
	else
	if (KB >= 64 && KB < 128)
		return(8);
	else
		return(-1);
}
